/**
 * ElectoralAuthorityData.java
 *
 * Revision: $Revision$  (Last Modified: $Date$)
 * Last Modified by: [$Author$]
 * 
 * Copyright (c) 2004 Smartmatic Corp.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of
 * Smartmatic Corp. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Corp.
 */
package com.smartmatic.waffle.domain.authority;

import java.io.Serializable;
import java.util.Set;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;
import javax.persistence.SequenceGenerator;
import javax.persistence.Table;
import javax.persistence.UniqueConstraint;

import com.smartmatic.waffle.domain.bulk.CustomCodeAnnotation;
import com.smartmatic.waffle.domain.demographics.RegionData;
import com.smartmatic.waffle.domain.eligibleoption.ImageWrapper;
import com.smartmatic.waffle.domain.general.GeneralInfo;
import com.smartmatic.waffle.domain.user.UserData;
import com.smartmatic.waffle.lang.annotations.basic.ValidatorListener;
import com.smartmatic.waffle.persistence.metadata.MultilingualPropertyAnnotation;
import com.thoughtworks.xstream.annotations.XStreamOmitField;

/**
 * JavaBean that represents an Electoral Authority
 * 
 * Is the entity charge of direction, supervision and control of all acts
 * related to the electoral process.
 */
@Entity
@Table(name = "electoral_authority", uniqueConstraints = { @UniqueConstraint(columnNames = "authority_custom_code") })
@ValidatorListener(name = "com.smartmatic.waffle.validator.authority.ElectoralAuthoriryValidator")
public class ElectoralAuthorityData extends GeneralInfo implements Serializable {

	/** The Constant serialVersionUID. */
	private static final long serialVersionUID = -4981168187341895803L;

	/**
	 * Possible values of the status of a Electoral Authority.
	 */

	public enum Status {

		/**
		 * Specify election authority status ACTIVE that match
		 * ELECTION_AUTHORITY.STATUS
		 */

		ACTIVE,

		/**
		 * Specify election authority status INACTIVE that match
		 * ELECTION_AUTHORITY.STATUS
		 */

		INACTIVE
	}

	/**
	 * Type of Electoral Authority supported by the system.
	 */

	public enum Type {

		/**
		 * Master authority. Any Electoral authority whose type is
		 * MASTER_AUTHORITY implies that is allowed to see all information about
		 * all contest adjudicated by any other Electoral Authority
		 */

		MASTER_AUTHORITY,

		/**
		 * Default authority. An Electoral Authority whose type is AUTHORITY is
		 * only allowed to see all information linked to all contests
		 * adjudicated by itself.
		 */

		AUTHORITY
	}

	/** Electoral Authority city. */

	private String city;

	/** Electoral Authority description. */

	private String description;

	/** Electoral Authority address. */

	private String address;

	/** Electoral Authority status. */

	private Status status;

	/** Electoral Authority Type. @see #Type */

	private Type type;

	/**
	 * The region where the Electoral Authority is.
	 */
	private RegionData region;

	/**
	 * Field that contains the asymmetric keys that each voting machine uses to
	 * encrypt/sign its information. @author Ithamar Torres
	 * (itorres@smartmatic.com)
	 */
	private ImageWrapper asymmetricKeys;

	/**
	 * Field that contains the asymmetric key alias that each voting machine uses
	 * to encrypt/sign its information. @author Carlos Lameda
	 * (carlos.lameda@smartmatic.com)
	 */
	private String alias;

	/**
	 * Field that contains the asymmetric password that each voting machine uses
	 * to encrypt/sign its information. @author Carlos Lameda
	 * (carlos.lameda@smartmatic.com)
	 */
	private String protectionKey;

	/**
	 * Field that contains a SymmetricKey which will be inside pblock.
	 */

	@XStreamOmitField
	private String symmetricKey;

	/**
	 * Field that contains an alphanumeric Key gave to the authority of table.
	 */

	@XStreamOmitField
	private String authorityKey;

	/**
	 * Field that contains the username gave to the authority of table.
	 */

	@XStreamOmitField
	private String authorityUsername;

	/** The users. */
	private Set<UserData> users;

	/**
	 * Gets the users.
	 * 
	 * @return the users
	 */
	@OneToMany(mappedBy = "electoralAuthority")
	public Set<UserData> getUsers() {
		return this.users;
	}

	/**
	 * Sets the users.
	 * 
	 * @param users
	 *            the new users
	 */
	public void setUsers(Set<UserData> users) {
		this.users = users;
	}

	/**
	 * Getter of <code>code</code> property.
	 * 
	 * @return <code>code</code> property value.
	 * @see #code
	 */
	@Override
	@Id
	@Column(name = "authority_code")
	@GeneratedValue(strategy = GenerationType.AUTO, generator = "s_electoral_authority")
	@SequenceGenerator(name = "s_electoral_authority", sequenceName = "s_electoral_authority", allocationSize = 1, initialValue = 1)
	public long getCode() {
		return this.code;
	}

	/**
	 * Getter of <code>customCode</code> property.
	 * 
	 * @return <code>customCode</code> property value.
	 * @see #customCode
	 */
	@Override
	@CustomCodeAnnotation
	@Column(name = "authority_custom_code", nullable = false, unique = true, length = 255)
	public String getCustomCode() {
		return this.customCode;
	}

	/**
	 * Getter of <code>customCode</code> property.
	 * 
	 * @return <code>customCode</code> property value.
	 * @see #customCode
	 */
	@Override
	@MultilingualPropertyAnnotation
	@Column(name = "authority_name", unique = true, length = 160)
	public String getName() {
		return this.name;
	}

	/**
	 * Getter of {@link #city} field.
	 * 
	 * @return {@link #city} value.
	 */
	@Column(name = "city", length = 160)
	public String getCity() {
		return this.city;
	}

	/**
	 * Setter of {@link #city} field.
	 * 
	 * @param city
	 *            The String to set.
	 */
	public void setCity(String city) {
		this.city = city;
	}

	/**
	 * Getter of {@link #description} field.
	 * 
	 * @return {@link #description} value.
	 */
	@Column(name = "description", length = 2080)
	public String getDescription() {
		return this.description;
	}

	/**
	 * Setter of {@link #description} field.
	 * 
	 * @param description
	 *            The String to set.
	 */
	public void setDescription(String description) {
		this.description = description;
	}

	/**
	 * Getter of {@link #address} field.
	 * 
	 * @return {@link #address} value.
	 */
	@Column(name = "authority_address", length = 500)
	@MultilingualPropertyAnnotation
	public String getAddress() {
		return this.address;
	}

	/**
	 * Setter of {@link #address} field.
	 * 
	 * @param address
	 *            The String to set.
	 */
	public void setAddress(String address) {
		this.address = address;
	}

	/**
	 * Getter of {@link #status} field.
	 * 
	 * @return {@link #status} value.
	 */
	@Column(name = "status")
	public Status getStatus() {
		return this.status;
	}

	/**
	 * Setter of {@link #status} field.
	 * 
	 * @param status
	 *            The Status to set.
	 */
	public void setStatus(Status status) {
		this.status = status;
	}

	/**
	 * Getter of {@link #type} field.
	 * 
	 * @return {@link #type} value.
	 */
	@Column(name = "type")
	public Type getType() {
		return this.type;
	}

	/**
	 * Setter of {@link #type} field.
	 * 
	 * @param type
	 *            The Type to set.
	 */
	public void setType(Type type) {
		this.type = type;
	}

	/**
	 * Getter of {@link #region} field.
	 * 
	 * @return {@link #region} value.
	 */
	@ManyToOne(optional = true, targetEntity = RegionData.class)
	@JoinColumn(name = "region_code", nullable = true, updatable = true)
	public RegionData getRegion() {
		return this.region;
	}

	/**
	 * Setter of {@link #region} field.
	 * 
	 * @param region
	 *            The RegionData to set.
	 */
	public void setRegion(RegionData region) {
		this.region = region;
	}

	/**
	 * Getter of {@link #asymmetricKeys} field.
	 * 
	 * @return {@link #asymmetricKeys} value.
	 */

	@Column(name = "asymmetric_key")
	public ImageWrapper getAsymmetricKeys() {
		return this.asymmetricKeys;
	}

	/**
	 * Setter of {@link #asymmetricKeys} field.
	 * 
	 * @param asymmetricKeys
	 *            The Blob to set.
	 */
	public void setAsymmetricKeys(ImageWrapper asymmetricKeys) {
		this.asymmetricKeys = asymmetricKeys;
	}

	/**
	 * Getter of {@link #alias} field.
	 * 
	 * @return {@link #alias} value.
	 */
	@Column(name = "alias", length = 255)
	public String getAlias() {
		return this.alias;
	}

	/**
	 * Setter of {@link #alias} field.
	 * 
	 * @param alias
	 *            The String to set.
	 */
	public void setAlias(String alias) {
		this.alias = alias;
	}

	/**
	 * Getter of {@link #protectionKey} field.
	 * 
	 * @return {@link #protectionKey} value.
	 */
	@Column(name = "protection_key", length = 255)
	public String getProtectionKey() {
		return this.protectionKey;
	}

	/**
	 * Setter of {@link #protectionKey} field.
	 * 
	 * @param protectionKey
	 *            The String to set.
	 */
	public void setProtectionKey(String protectionKey) {
		this.protectionKey = protectionKey;
	}

	/**
	 * Getter of {@link #symmetricKey} field.
	 * 
	 * @return {@link #symmetricKey} value.
	 */
	@Column(name = "symmetric_key")
	public String getSymmetricKey() {
		return this.symmetricKey;
	}

	/**
	 * Setter of {@link #symmetricKey} field.
	 * 
	 * @param symmetricKey
	 *            The String to set.
	 */
	public void setSymmetricKey(String symmetricKey) {
		this.symmetricKey = symmetricKey;
	}

	/**
	 * Getter of {@link #authorityKey} field.
	 * 
	 * @return {@link #authorityKey} value.
	 */
	@Column(name = "authority_key")
	public String getAuthorityKey() {
		return this.authorityKey;
	}

	/**
	 * Setter of {@link #authorityKey} field.
	 * 
	 * @param authorityKey
	 *            The String to set.
	 */
	public void setAuthorityKey(String authorityKey) {
		this.authorityKey = authorityKey;
	}

	/**
	 * Getter of {@link #AuthorityUsername} field.
	 * 
	 * @return {@link #AuthorityUsername} value.
	 */
	@Column(name = "ea_username")
	public String getAuthorityUsername() {
		return this.authorityUsername;
	}

	/**
	 * Setter of {@link #AuthorityUsername} field.
	 * 
	 * @param authorityUserName
	 *            the new field that contains the username gave to the authority
	 *            of table
	 */
	public void setAuthorityUsername(String authorityUserName) {
		this.authorityUsername = authorityUserName;
	}
}
