/**
 * BulkLoadException.java
 *
 * Revision: $Revision$  (Last Modified: $Date$)
 * Last Modified by: [$Author$]
 * 
 * Copyright (c) 2004 Smartmatic Corp.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of
 * Smartmatic Corp. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Corp.
 */

package com.smartmatic.waffle.ecm.bulk.service.lang;

import com.smartmatic.waffle.lang.BaseException;
import com.smartmatic.waffle.lang.IExceptionInfo;

/**
 * Exception class that represents an abnormal situation in the bulk data
 * loading module.
 * 
 * @author Ithamar Torres (itorres@smartmatic.com)
 */

public class BulkLoadException extends BaseException {

	/**
	 * Holds the version of the class. It is used by the serialization engine to
	 * determine when a class is equivalent to another
	 * 
	 * @author Ithamar Torres (itorres@smartmatic.com)
	 */

	private static final long serialVersionUID = -336529124029419196L;

	/**
	 * Holds further details about the exception such as database constraint
	 * name violated, missing file in the resources, etc.
	 */
	private String exceptionDetail;

	/**
	 * Enum that contains all the possible reasons because the
	 * JNICommunicationException can be thrown.
	 * 
	 * @author Ithamar Torres (itorres@smartmatic.com)
	 */

	public enum ExceptionInfo implements IExceptionInfo {

		/**
		 * This enum instance indicates that the bulk data loading module could
		 * not find a class required to perform its operations. Basically this
		 * occurs when a domain class is not accesible from the bulk data
		 * loading module. It is important to remember that most of the bulk
		 * data loading are implemented using reflection so there is no a strong
		 * reference between this module and those that are really used.
		 */

		CLASS_NOT_FOUND,

		/**
		 * This enum instance indicates that the bulk data loading module could
		 * not parse the xml received as the input to the module. This exception
		 * is thrown from the xml parser so basically is common when this module
		 * receives an invalid xml file.
		 */

		XML_PARSE_EXCEPTION,

		/**
		 * This enum instance indicates that the bulk data loading module could
		 * not construct the object instance that it was required to store. This
		 * happen when the bulk data loading does not have enough permission to
		 * invoke a setter/getter required by this module or when occurs a
		 * exception inside a methos invoked by the bulk data loading module.
		 */

		OBJECT_CONSTRUCION_PROBLEM,

		/**
		 * This enum instance indicates that the bulk data loading module could
		 * not construct a enum instance from the information that is in the xml
		 * file.
		 */

		INCORRECT_ENUM_REFERENCE,

		/**
		 * This enum instance indicates that the bulk data loading system could
		 * not find in the database a object that is being referenced by the
		 * data received as parameter. Basically this occurs because there exist
		 * a problem in the data received. This mostly is a user error.
		 */

		OBJECT_REFERENCE_NOT_FOUND,

		/**
		 * This enum instance indicates that the bulk data loading system could
		 * not find the annotation in a class to check which is the id or the
		 * custom code of the class. This is a major problem because a class
		 * without any of those annotations can't be referenced by any other
		 * class in the bulk data loading module [The bulk data loading module
		 * requires to know the custom code and the id of all the domain classes
		 * used]
		 */

		OBJECT_REFERENCE_WITHOUT_CUSTOM_CODE_OR_ID,

		/**
		 * This enum instance indicates that the bulk data loading system could
		 * not find the custom code used to load a object from the database. It
		 * usually occurs when a user is performing an update and it does not
		 * provide the custom code field used to load the object from the
		 * database.
		 */

		OBJECT_UPDATE_WITHOUT_CUSTOM_CODE_OR_ID,

		/**
		 * This enum instance indicates that the bulk data loading could not
		 * perform a object insert/update because there was a binary field
		 * referenced in the xml file and the field that match the xml data in
		 * the class is not a binary wrapper.
		 */

		BINARY_FIELD_WITHOUT_WRAPPER,

		/**
		 * This enum instance indicates that the bulk data loading could not
		 * locate a file inside the zip file. This usually occurs because the
		 * user has some mistakes in the xml file or because a file is not found
		 * inside the zip file
		 */

		BINARY_FIELD_NOT_FOUND,

		/**
		 * Enum that indicates that the system could not create a zip file from
		 * the received input stream received as parameter.
		 */

		PROBLEM_CREATING_ZIP_FILE,

		/**
		 * This enum instance indicates that the user provided a object that
		 * references a binary field and the user does not provided the zip to
		 * look for the binary field.
		 */

		BINARY_FIELD_WITHOUT_PROVIDED_ZIP,

		/**
		 * This enum instance indicates that the user provided a file name that
		 * references a voice file and the user does not provided that file in
		 * the voices zip.
		 */

		VOICE_ZIP_WITHOUT_REQUIRED_FILE,

		/**
		 * This problem indicates that there was a problem with a XSD file.
		 * Mostly this occurs because the xsd its not found or the configuration
		 * file does not have the correct information about the location of the
		 * XSD.
		 */

		XML_SCHEMA_NOT_VALID,

		/**
		 * This problem indicates that the file received in the bulk data
		 * loading module is not a valid one. It does not comply with the
		 * minimum requirements to process the xml file.
		 */

		XML_NOT_VALID_BASE_XSD,

		/**
		 * This problem occurs when the xml received in the bulk data loading
		 * module does not comply with the restrictions expressed in the custom
		 * xsd configured to a specific class.
		 */

		XML_NOT_VALID_CUSTOM_XSD,

		/**
		 * This problem occurs when the field name was not the expected one.
		 */
		INCORRECT_FIELD_NAME,

		/**
		 * This problem occurs when something is wrong executing a database
		 * operation.
		 */
		DATABASE_OPERATION_PROBLEM,

		/**
		 * This problem occurs when there is a persistence problem.
		 */
		PERSISTENCE_PROBLEM,

		/**
		 * This problem occurs when the file to bulk upload is invalid.
		 */
		INVALID_FILE,

		/**
		 * This problem occurs when there is a constraint violation.
		 */
		CONSTRAINT_VIOLATION,

		/**
		 * This problem occurs when there is an unexpected error.
		 */
		UNEXPECTED_PROBLEM;

		/**
		 * <p>
		 * Each enumerator's resource bundle's key message.
		 * </p>
		 */

		private String message;

		/**
		 * Getter of <code>message</code> property.
		 * 
		 * @return <code>message</code> property value.
		 * @see #message
		 * 
		 * @author Edgar Trejo (edgar.trejo@smartmatic.com)
		 */

		@Override
		public String getMessage() {
			return this.message;
		}

		/**
		 * Constructor.
		 * 
		 * @author Edgar Trejo (edgar.trejo@smartmatic.com)
		 */

		private ExceptionInfo() {
			this.message = this.getClass().getName() + "." + this.name();
		}
	}

	/**
	 * Getter of <code>exceptionDetail</code> property.
	 * 
	 * @return <code>exceptionDetail</code> property value.
	 * @see #exceptionDetail
	 */

	public String getExceptionDetail() {
		return this.exceptionDetail;
	}

	/**
	 * Setter of <code>exceptionDetail</code> property.
	 * 
	 * @param exceptionDetail
	 *            The <code>exceptionDetail</code> to set.
	 * @see #exceptionDetail
	 */
	public void setExceptionDetail(String exceptionDetail) {
		this.exceptionDetail = exceptionDetail;
	}

	/**
	 * Instantiates a new bulk load exception.
	 * 
	 * @param exceptionInfo
	 *            the exception info
	 * @param exceptionDetail
	 *            the constraint name
	 */
	public BulkLoadException(IExceptionInfo exceptionInfo,
			String exceptionDetail) {
		super(exceptionInfo);
		this.setExceptionDetail(exceptionDetail);
	}

	/**
	 * Instantiates a new bulk load exception.
	 * 
	 * @param exceptionInfo
	 *            the exception info
	 * @param throwable
	 *            the throwable
	 * @param parameters
	 *            the parameters
	 */
	public BulkLoadException(IExceptionInfo exceptionInfo, Throwable throwable,
			Object... parameters) {
		super(exceptionInfo, throwable, parameters);
	}

	/**
	 * Instantiates a new bulk load exception.
	 * 
	 * @param exceptionInfo
	 *            the exception info
	 * @param throwable
	 *            the throwable
	 */
	public BulkLoadException(IExceptionInfo exceptionInfo, Throwable throwable) {
		super(exceptionInfo, throwable);
	}

	/**
	 * Instantiates a new bulk load exception.
	 * 
	 * @param exceptionInfo
	 *            the exception info
	 */
	public BulkLoadException(IExceptionInfo exceptionInfo) {
		super(exceptionInfo);
	}

	/**
	 * Instantiates a new bulk load exception.
	 * 
	 * @param exceptionInfo
	 *            the exception info
	 * @param parameters
	 *            the parameters
	 */
	public BulkLoadException(IExceptionInfo exceptionInfo, Object... parameters) {
		super(exceptionInfo, parameters);
	}

}
