/**
 * RegionBrowserHierarchyProvider.java
 *
 * Revision: $Revision$  (Last Modified: $Date$)
 * Last Modified by: [$Author$]
 * 
 * Copyright (c) 2004 Smartmatic Corp.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of
 * Smartmatic Corp. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Corp.
 */

package com.smartmatic.waffle.ecm.client.providers;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.ResourceBundle;
import java.util.Set;

import com.smartmatic.waffle.domain.demographics.CategoryData;
import com.smartmatic.waffle.domain.demographics.PollingPlaceData;
import com.smartmatic.waffle.domain.demographics.RegionData;
import com.smartmatic.waffle.domain.i18n.LanguageData;
import com.smartmatic.waffle.ecm.client.ECMServiceProvider;
import com.smartmatic.waffle.ecm.data.transfer.objects.BrowserHierarchyDTO;
import com.smartmatic.waffle.ecm.ui.controls.Browser.IBrowserHierarchyProvider;
import com.smartmatic.waffle.lang.BaseException;

/**
 * Class that provide the data source for the hierarchy widget.<br>
 * The data is about the region class.
 */
public class RegionBrowserHierarchyProvider implements
		IBrowserHierarchyProvider {

	/** The language. */
	private LanguageData language;

	/** The available regions. */
	private List<RegionData> availableRegions;

	/** The codes of available regions. */
	private List<Long> codesOfAvailableRegions;

	/** The show polling places. */
	private boolean showPollingPlaces;

	/**
	 * Setter of <code>showPollingPlaces</code> property.
	 * 
	 * @param showPollingPlaces
	 *            The <code>showPollingPlaces</code> to set.
	 * @see #showPollingPlaces
	 */
	public void setShowPollingPlaces(boolean showPollingPlaces) {
		this.showPollingPlaces = showPollingPlaces;
	}

	/**
	 * * Class constructor.
	 * 
	 * @param availableRegions
	 *            - custom codes list of available regions that could be used.
	 * @param showPollingPlaces
	 *            -Flag to indicate if the information about polling places must
	 *            be shown in the widget.
	 * @param language
	 *            - language in which the region name must be show
	 */
	public RegionBrowserHierarchyProvider(List<RegionData> availableRegions,
			boolean showPollingPlaces, LanguageData language) {
		this.availableRegions = availableRegions;
		this.codesOfAvailableRegions = new ArrayList<Long>();
		this.showPollingPlaces = showPollingPlaces;
		this.language = language;
		if (this.availableRegions != null) {
			for (RegionData currentRegion : availableRegions) {
				this.codesOfAvailableRegions.add(currentRegion.getCode());
			}
		}
	}

	/***
	 * {@inheritDoc}<br>
	 * This implementation search all regions belong to second level, and create
	 * the first page for the widget
	 * 
	 * @throws BaseException
	 */
	@Override
	public Set<BrowserHierarchyDTO> initialLoad() throws BaseException {
		Set<BrowserHierarchyDTO> list = new HashSet<BrowserHierarchyDTO>();
		BrowserHierarchyDTO dtoAux;
		List<RegionData> resgionList = ECMServiceProvider
				.getRegionsByCategoryLevel(CategoryData.Level.SECOND_LEVEL,
						this.codesOfAvailableRegions, this.language);

		for (RegionData current : resgionList) {
			dtoAux = new BrowserHierarchyDTO();
			dtoAux.setId(current);
			dtoAux.setTextToShow(current.getName() + " | "
					+ current.getCategory().getCategoryName());
			list.add(dtoAux);
		}
		return list;
	}

	/***
	 * {@inheritDoc}<br>
	 * This implementation search all children regions belong to the selected
	 * region.
	 * 
	 * @param selectedElement
	 *            the selected region.
	 * 
	 * @throws BaseException
	 */
	@Override
	public Set<BrowserHierarchyDTO> nextLevel(
			BrowserHierarchyDTO selectedElement) throws BaseException {
		Set<BrowserHierarchyDTO> list = new HashSet<BrowserHierarchyDTO>();
		BrowserHierarchyDTO dtoAux;
		ResourceBundle bundle = ResourceBundle
				.getBundle("com/smartmatic/waffle/ecm/client/resources/ECMApp");
		if (selectedElement.getId() instanceof RegionData) {
			RegionData selectedRegion = (RegionData) selectedElement.getId();
			if (selectedRegion.getCategory().isLastLevelCategory()) {
				if (this.showPollingPlaces) {
					List<PollingPlaceData> pollingPlaces = ECMServiceProvider
							.getPollingPlacesPerRegion(selectedRegion);
					for (PollingPlaceData current : pollingPlaces) {
						dtoAux = new BrowserHierarchyDTO();
						dtoAux.setId(current);
						dtoAux.setTextToShow(current.getName()
								+ " | "
								+ bundle.getString("RegionLevelNavigatorWidget.regionTable.header.pollingPlaceColumn"));
						list.add(dtoAux);
					}
				} else {
					return null;
				}
			} else {
				List<RegionData> resgionList = ECMServiceProvider
						.getChildrenRegions(selectedRegion,
								this.codesOfAvailableRegions, this.language);
				for (RegionData current : resgionList) {
					dtoAux = new BrowserHierarchyDTO();
					dtoAux.setId(current);
					dtoAux.setTextToShow(current.getName() + " | "
							+ current.getCategory().getCategoryName());
					list.add(dtoAux);
				}
			}
			return list;
		}
		return null;
	}

	/**
	 * *.
	 * 
	 * @param selectedElement
	 *            the selected element
	 * @return the sets the
	 * @throws BaseException
	 *             the base exception
	 * @inheritDoc <br>
	 *             No implemented yet.
	 */
	@Override
	public Set<BrowserHierarchyDTO> previousLevel(
			BrowserHierarchyDTO selectedElement) throws BaseException {
		return null;
	}

}
