package com.smartmatic.waffle.ecm.client.ui;

import java.io.File;
import java.io.IOException;
import java.security.cert.X509Certificate;
import java.util.ArrayList;
import java.util.List;
import java.util.zip.ZipFile;

import javax.swing.JScrollPane;
import javax.swing.JTree;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeModel;

import org.apache.commons.lang.StringUtils;
import org.dom4j.Document;
import org.dom4j.DocumentException;
import org.dom4j.DocumentHelper;
import org.dom4j.Element;

import com.smartmatic.waffle.ecm.bulk.service.lang.BulkLoadException;
import com.smartmatic.waffle.ecm.client.ECMClientSideConstants;
import com.smartmatic.waffle.ecm.client.util.SignatureException;
import com.smartmatic.waffle.ecm.client.util.SignatureUtils;
import com.smartmatic.waffle.ecm.data.transfer.objects.BulkErrorReportDTO;
import com.smartmatic.waffle.ecm.data.transfer.objects.BulkResultDTO;
import com.smartmatic.waffle.ecm.data.transfer.objects.UploadedFileDetailDTO;
import com.smartmatic.waffle.lang.BaseException;
import com.smartmatic.waffle.log.SMTTLogger;

public abstract class BulkUploadAbstractPanel extends javax.swing.JPanel {

	/**
	 * Method to update the status message while database records removal
	 * process is executing
	 * 
	 * @param bundleKey
	 *            bundle key used to find the translation of the message
	 */
	abstract void updateStatusMessage(String bundleKey);

	/**
	 * Sets the progress bar's current value
	 * 
	 * @param progress
	 *            value to update the progress bar
	 */
	abstract void updateProgressBar(int progress);

	/**
	 * Method called by some controllers to show error or information messages.
	 * 
	 * @param bundleKey
	 *            bundle key to find the translation message
	 * @param messageType
	 *            type of message to show
	 */
	abstract void showMessage(String bundleKey, int messageType);

	/**
	 * Updates the GUI components after database records removal process
	 * finished.
	 * 
	 * @param someErrorHasOccurred
	 *            true if some error occurred in the process to show different
	 *            component for each case
	 */
	abstract void finishDatabaseClearingProcess(boolean someErrorHasOccurred);

	/**
	 * Sets the progress bar's maximum value
	 * 
	 * @param progress
	 *            value to update the progress bar
	 */
	abstract void setProgressBarMaximumValue(int maxValue);

	/**
	 * Checks if the selected file is a resources file
	 * 
	 * @param fileToUpload
	 *            file to check
	 * 
	 * @return true if the file is a resources file, otherwise false
	 */
	protected boolean validateIfIsResourcesFile(File fileToUpload) {
		ZipFile file = null;
		try {
			file = new ZipFile(fileToUpload);
			return true;
		} catch (Exception e) {
			return false;
		} finally {
			if (file != null) {
				try {
					file.close();
				} catch (IOException e) {
					return false;
				}
			}
		}
	}

	/**
	 * Validates signature using selected files to upload.
	 * 
	 * @param selectedFiles
	 * @throws SignatureException
	 */
	protected void validateSignature(List<File> selectedFiles)
			throws SignatureException {

		File signatureFile = new File(selectedFiles.get(0).getParent(),
				"SignatureFile.xml");

		List<File> validatedFiles = new ArrayList<File>();

		boolean validSignature = SignatureUtils.validateSignatureFile(
				signatureFile, this.dataValidationCertificate, validatedFiles);

		if (!validSignature) {
			throw new SignatureException("Error validating Signature File");
		}

		if (!checkValidatedFilesVsSelectedFiles(validatedFiles, selectedFiles)) {
			throw new SignatureException(
					"Selected files do not correspond with validated ones");
		}
	}

	/**
	 * Checks if all files selected are contained in validated files
	 * 
	 * @param validated
	 *            List of files validated by the signature.
	 * @param selected
	 *            List of files selected to upload.
	 * @return True if all validated files are present on selected files.
	 */
	private boolean checkValidatedFilesVsSelectedFiles(List<File> validated,
			List<File> selected) {

		if (selected.size() != validated.size()) {
			return false;
		}

		for (File validatedFile : validated) {
			if (!selected.contains(validatedFile)) {
				return false;
			}
		}
		return true;
	}

	/**
	 * Method used to build the tree with the result of the bulk load process.
	 * 
	 * @param result
	 *            dto result with the process information.
	 * 
	 * @param uploadedFilesResultTree
	 *            jtree component which will show the result
	 * @param rootTreeNode
	 *            the root node of the jtree component.
	 * 
	 * @return true if there is not error or false otherwise.
	 */
	protected boolean buildResult(
			List<UploadedFileDetailDTO> fileUploadedDetailList,
			JTree uploadedFilesResultTree, DefaultMutableTreeNode rootTreeNode) {

		logger.info("Building Results: Bulk load process");

		boolean noErrorsFound = true;
		uploadedFilesResultTree.removeAll();
		rootTreeNode.removeAllChildren();
		DefaultTreeModel model = new DefaultTreeModel(rootTreeNode);

		uploadedFilesResultTree.setModel(model);

		int countFile = 0;
		int countChildsFile = 0;

		DefaultMutableTreeNode bulkUploadErrorMessage;
		DefaultMutableTreeNode bulkUploadErrorType;

		logger.info("Total uploaded Files: " + fileUploadedDetailList.size());

		for (UploadedFileDetailDTO fileUploadedDetail : fileUploadedDetailList) {
			DefaultMutableTreeNode fileNameNode = new DefaultMutableTreeNode(
					fileUploadedDetail.getFileName());
			model.insertNodeInto(fileNameNode, rootTreeNode, countFile);
			countFile++;
			DefaultMutableTreeNode bulkUploadResultDetail;
			countChildsFile = 0;

			BulkResultDTO resultUploadDTO = fileUploadedDetail
					.getBulkResultDTO();

			// nothing was uploaded
			if (resultUploadDTO.getIncorrectCount() == 0
					&& resultUploadDTO.getCorrectCount() == 0) {
				noErrorsFound = false;

				String elementsLoadedMessage = buildElementLoadedMessage(
						resultUploadDTO.getCorrectCount(),
						resultUploadDTO.getCorrectCount()
								+ resultUploadDTO.getIncorrectCount());

				String translatedName = fileUploadedDetail.getSmttFileType() != null ? fileUploadedDetail
						.getSmttFileType().getTranslatedName()
						: fileUploadedDetail.getEMLFileType()
								.getTranslatedName();

				bulkUploadResultDetail = new DefaultMutableTreeNode(
						translatedName
								+ " - "
								+ this.resourceBundle
										.getString("BulkUploadSmttFilesPanel.error.FILE_MUST_BE_CORRECTED")
								+ ". " + elementsLoadedMessage);

				for (BulkErrorReportDTO errorReport : resultUploadDTO
						.getErrorReportList()) {

					BaseException baseException = errorReport.getException();
					String errorMessage = null;

					if (baseException
							.getInfo()
							.equals(BulkLoadException.ExceptionInfo.CONSTRAINT_VIOLATION)) {
						BulkLoadException bulkLoadException = (BulkLoadException) baseException;

						errorMessage = this.resourceBundle
								.getString("BulkUploadSmttFilesPanel.error."
										+ baseException.getInfo().toString())
								+ ": " + bulkLoadException.getExceptionDetail();
					} else {
						errorMessage = this.resourceBundle
								.getString("BulkUploadSmttFilesPanel.error."
										+ baseException.getInfo().toString());
					}

					bulkUploadErrorType = new DefaultMutableTreeNode(
							errorMessage);
					model.insertNodeInto(bulkUploadErrorType,
							bulkUploadResultDetail, 0);

					bulkUploadErrorMessage = new DefaultMutableTreeNode(
							errorReport.getInputText());
					model.insertNodeInto(bulkUploadErrorMessage,
							bulkUploadErrorType, 0);
				}
			} else
			// everything was correctly uploaded
			if (resultUploadDTO.getCorrectCount() > 0
					&& resultUploadDTO.getIncorrectCount() == 0) {

				String elementsLoadedMessage = buildElementLoadedMessage(
						resultUploadDTO.getCorrectCount(),
						resultUploadDTO.getCorrectCount());

				String translatedName = fileUploadedDetail.getSmttFileType() != null ? fileUploadedDetail
						.getSmttFileType().getTranslatedName()
						: fileUploadedDetail.getEMLFileType()
								.getTranslatedName();

				bulkUploadResultDetail = new DefaultMutableTreeNode(
						translatedName + " - " + elementsLoadedMessage);
			}
			// there were upload errors
			else {
				logger.info("Upload errors on file: "
						+ fileUploadedDetail.getFileName());
				noErrorsFound = false;

				String elementsLoadedMessage = buildElementLoadedMessage(
						resultUploadDTO.getCorrectCount(),
						resultUploadDTO.getCorrectCount()
								+ resultUploadDTO.getIncorrectCount());

				String translatedName = fileUploadedDetail.getSmttFileType() != null ? fileUploadedDetail
						.getSmttFileType().getTranslatedName()
						: fileUploadedDetail.getEMLFileType()
								.getTranslatedName();

				bulkUploadResultDetail = new DefaultMutableTreeNode(
						translatedName
								+ " - "
								+ this.resourceBundle
										.getString("BulkUploadSmttFilesPanel.error.FILE_MUST_BE_CORRECTED")
								+ ". " + elementsLoadedMessage);

				for (BulkErrorReportDTO errorReport : resultUploadDTO
						.getErrorReportList()) {
					String errorMessage;
					try {
						BaseException baseException = errorReport
								.getException();

						if (baseException
								.getInfo()
								.equals(BulkLoadException.ExceptionInfo.CONSTRAINT_VIOLATION)) {
							BulkLoadException bulkLoadException = (BulkLoadException) baseException;

							errorMessage = this.resourceBundle
									.getString("BulkUploadSmttFilesPanel.error."
											+ baseException.getInfo()
													.toString())
									+ ": "
									+ bulkLoadException.getExceptionDetail();
						} else if (baseException
								.getInfo()
								.equals(BulkLoadException.ExceptionInfo.BINARY_FIELD_NOT_FOUND)) {

							BulkLoadException bulkLoadException = (BulkLoadException) baseException;

							errorMessage = this.resourceBundle
									.getString("BulkUploadSmttFilesPanel.error."
											+ bulkLoadException.getInfo()
													.toString());

							String customCode = getCustomCodeForError(errorReport
									.getInputText());

							String missingFileName = bulkLoadException
									.getExceptionDetail();

							errorMessage = StringUtils.replace(errorMessage,
									"${custom_code}", customCode);

							errorMessage = StringUtils.replace(errorMessage,
									"${missing_filename}", missingFileName);
						} else if (baseException
								.getInfo()
								.equals(BulkLoadException.ExceptionInfo.OBJECT_REFERENCE_NOT_FOUND)) {

							BulkLoadException bulkLoadException = (BulkLoadException) baseException;

							String customCode = bulkLoadException
									.getExceptionDetail();

							errorMessage = this.resourceBundle
									.getString("BulkUploadSmttFilesPanel.error."
											+ baseException.getInfo()
													.toString());

							errorMessage = StringUtils.replace(errorMessage,
									"${custom_code}", customCode);
						} else {
							errorMessage = this.resourceBundle
									.getString("BulkUploadSmttFilesPanel.error."
											+ baseException.getInfo()
													.toString());
						}
					} catch (Exception e) {
						logger.error(e, "Error processing BulkErrorReportDTO");
						errorMessage = errorReport.getException().getInfo()
								.toString();
					}

					bulkUploadErrorMessage = getMessageToShow(errorReport);

					bulkUploadErrorType = new DefaultMutableTreeNode(
							errorMessage);
					logger.info("Error: " + errorMessage);

					if (bulkUploadErrorMessage != null) {
						model.insertNodeInto(bulkUploadErrorMessage,
								bulkUploadResultDetail, 0);

						model.insertNodeInto(bulkUploadErrorType,
								bulkUploadErrorMessage, 0);
					} else {
						model.insertNodeInto(bulkUploadErrorType,
								bulkUploadResultDetail, 0);
					}

				}
			}
			model.insertNodeInto(bulkUploadResultDetail, fileNameNode,
					countChildsFile);
			countChildsFile++;
		}
		return noErrorsFound;
	}

	/**
	 * This method shows the result of the bulk upload in the window.
	 * 
	 * @param uploadedFilesDetailsList
	 *            list with the details of files uploaded
	 * 
	 * @return true if there are no errors, false otherwise
	 */
	protected boolean showResult(JTree uploadedFilesResultTree,
			DefaultMutableTreeNode rootTreeNode,
			JScrollPane paneWithFilesToUploadTable,
			JScrollPane paneWithFilesUploadedResult,
			List<UploadedFileDetailDTO> uploadedFilesDetailsList) {

		boolean bulkUploadResult = buildResult(uploadedFilesDetailsList,
				uploadedFilesResultTree, rootTreeNode);

		paneWithFilesToUploadTable.setVisible(false);
		paneWithFilesUploadedResult.setVisible(true);
		uploadedFilesResultTree.expandRow(0);

		updateUI();
		repaint();

		return bulkUploadResult;
	}

	/**
	 * Prepare GUI components for the execution of the database records removal
	 */
	protected void startDatabaseClearingProcess(
			BulkUploadAbstractPanel bulkUploadAbstractPanel,
			boolean deleteOnlyCodiData) {

		BulkUploadClearDatabaseController bulkUploadClearDatabaseController = new BulkUploadClearDatabaseController(
				bulkUploadAbstractPanel, deleteOnlyCodiData);
		bulkUploadClearDatabaseController.startWorker();
	}

	/**
	 * ResourceBundle used to find the translation for each languages in the
	 * language properties.
	 */
	protected java.util.ResourceBundle resourceBundle = java.util.ResourceBundle
			.getBundle("com/smartmatic/waffle/ecm/client/resources/ECMApp");

	/**
	 * Builds the message to show the user the elements loaded
	 * 
	 * @param itemNum
	 *            elements loaded
	 * @param total
	 *            total of elements to load
	 * 
	 * @return message builded to show the user the elements loaded
	 */
	private String buildElementLoadedMessage(int itemNum, int total) {
		String elementsLoadedMessage = this.resourceBundle
				.getString("BulkUploadSmttFilesPanel.message.elements.loaded");

		elementsLoadedMessage = StringUtils.replace(elementsLoadedMessage,
				"${item_num}", String.valueOf(itemNum));

		elementsLoadedMessage = StringUtils.replace(elementsLoadedMessage,
				"${total}", String.valueOf(total));

		return elementsLoadedMessage;
	}

	/**
	 * Builds the error-reporting message which will be shown in the tree.
	 * 
	 * @param errorReport
	 *            object that holds the error to report
	 * 
	 * @return the DefaultMutableTreeNode with the message ready to be showed
	 */
	private DefaultMutableTreeNode getMessageToShow(
			BulkErrorReportDTO errorReport) {

		String customCode = getCustomCodeForError(errorReport.getInputText());

		if (customCode != null) {
			BaseException baseException = errorReport.getException();

			if (baseException.getInfo().equals(
					BulkLoadException.ExceptionInfo.OBJECT_CONSTRUCION_PROBLEM)) {

				BulkLoadException bulkLoadException = (BulkLoadException) errorReport
						.getException();

				String errorMessage = this.resourceBundle
						.getString("BulkUploadSmttFilesPanel.row.error.customcode")
						+ " "
						+ this.resourceBundle
								.getString("BulkUploadSmttFilesPanel.row.error.invalid.column")
						+ ": " + bulkLoadException.getExceptionDetail();

				errorMessage = StringUtils.replace(errorMessage,
						"${custom_code}", customCode);

				return new DefaultMutableTreeNode(errorMessage);
			} else {
				String errorMessage = this.resourceBundle
						.getString("BulkUploadSmttFilesPanel.row.error.customcode");

				errorMessage = StringUtils.replace(errorMessage,
						"${custom_code}", customCode);

				return new DefaultMutableTreeNode(errorMessage);
			}
		} else {
			return null;
		}
	}

	/**
	 * Gets the custom code of the object failing in bulk upload.
	 * 
	 * @param inputText
	 *            the information regarding the error
	 * 
	 * @return the object custom code if is available otherwise null
	 */
	private String getCustomCodeForError(String inputText) {
		try {
			String xpath = ECMClientSideConstants.BULK_XPATH_QUERY;

			Document document = DocumentHelper.parseText(inputText);

			Element node = (Element) document.selectSingleNode(xpath);

			if (node != null) {
				return node.attributeValue("value");
			} else {
				return "";
			}

		} catch (DocumentException e) {
			return null;
		}
	}

	/**
	 * Reference to some file that has some problem executing bulk upload.
	 */
	protected File fileWithProblems;

	/**
	 * Data validation certificate
	 */
	protected X509Certificate dataValidationCertificate;

	/**
	 * Resource file reference
	 */
	protected File resourcesFileReference;

	/**
	 * For log messages purpose.
	 */
	private static SMTTLogger logger = SMTTLogger
			.getLogger(BulkUploadAbstractPanel.class);

	/**
	 * The object's serialization code.
	 */
	private static final long serialVersionUID = -6862399588755481595L;
}
