package com.smartmatic.waffle.ecm.client.ui;

import java.awt.Dialog.ModalExclusionType;

import net.sf.jasperreports.engine.JRException;
import net.sf.jasperreports.engine.JasperExportManager;
import net.sf.jasperreports.engine.JasperPrint;
import net.sf.jasperreports.view.JasperViewer;

import com.smartmatic.waffle.ecm.client.ui.common.AwtOperationType;
import com.smartmatic.waffle.ecm.client.ui.common.IAwtOperationListener;
import com.smartmatic.waffle.ecm.client.worker.ReportsGenerationWorker;
import com.smartmatic.waffle.ecm.core.printing.PrinterAssistant;
import com.smartmatic.waffle.log.SMTTLogger;

/**
 * Class used to manage the required tasks when performing a Screenshots
 * generation from the client application, this class will start the workers to
 * execute the server side methods.
 * 
 */
public class ReportGenerationController implements IAwtOperationListener {

	ReportGenerationPanel reportGenerationPanel;

	ReportsGenerationWorker reportGenerationWorker;

	/** The logger. */
	private static SMTTLogger logger = SMTTLogger
			.getLogger(ReportGenerationController.class);

	public ReportGenerationController(
			ReportGenerationPanel reportGenerationPanel) {
		this.reportGenerationPanel = reportGenerationPanel;
	}

	/**
	 * Method used to start the worker that performs the report generation
	 * process.
	 */
	public void startWorker() {

		if (this.reportGenerationPanel.getSelectedObject() != null
				&& !this.reportGenerationPanel.getSelectedObject().isEmpty()) {

			this.reportGenerationWorker = new ReportsGenerationWorker(
					this.reportGenerationPanel.getSelectedObject(),
					this.reportGenerationPanel.getSelectedReport(),
					this.reportGenerationPanel.isPrintReportWithoutPassword());

			this.reportGenerationWorker.addListener(this);

			Thread reportGenerationWorkerThread = new Thread(
					this.reportGenerationWorker, "reportGenerationWorkerThread");

			reportGenerationWorkerThread.start();
		} else {
			this.reportGenerationPanel
					.finishProcess("ReportGenerationPanelController.MustSelectAtLeastOneRegion");
		}

	}

	@Override
	public void awtOperation(AwtOperationType awtOperationType) {

		switch (awtOperationType) {
		case UPDATE_COMPONENTS:
			updateComponentsOperation();
			break;
		case REFRESH_LABEL:
			refreshLabelOperation();
			break;
		case UPDATE_PROGRESS_BAR:
			updateProgressBarOperation();
			break;
		case SHOW_ERROR_MESSAGE:
			showErrorMessageOperation();
			break;
		default:
			break;
		}
	}

	/**
	 * Method used to control the logic when an active worker fires the
	 * UPDATE_PROGRESS_BAR operation.
	 */
	private void updateProgressBarOperation() {

		this.reportGenerationPanel
				.setProgressBarValue(this.reportGenerationWorker.getProgress());

	}

	/**
	 * Method used to control the logic when an active worker fires the
	 * UPDATE_COMPONENTS operation.
	 */
	private void updateComponentsOperation() {

		if (this.reportGenerationWorker.isGenerationDone()) {
			for (JasperPrint report : this.reportGenerationWorker
					.getReportsResultList()) {

				if (Boolean.getBoolean("simulated_printing")) {

					JasperViewer jv = new JasperViewer(report, false);
					jv.setModalExclusionType(ModalExclusionType.APPLICATION_EXCLUDE);
					jv.setVisible(true);

				} else {
					if (this.reportGenerationPanel.checkPrinterPlugged()) {
						try {
							PrinterAssistant
									.sendDocumentToPrint(JasperExportManager
											.exportReportToPdf(report));
						} catch (JRException e) {
							logger.error(e, "Error exporting documents to pdf");
							showErrorMessageOperation();
						} catch (Exception e) {
							logger.error(e, "Error sending document to print");
							showErrorMessageOperation();
						}
					}
				}
			}

			this.reportGenerationPanel.refreshStatus("");
			this.reportGenerationPanel.blockButtons(false);
			this.reportGenerationPanel.blockTabs(false);
			this.reportGenerationPanel.finishProcess();

		} else {
			this.reportGenerationPanel.blockButtons(true);
			this.reportGenerationPanel.blockTabs(true);

		}
	}

	/**
	 * Method used to control the logic when an active worker fires the
	 * REFRESH_LABEL operation.
	 */
	private void refreshLabelOperation() {
		this.reportGenerationPanel
				.refreshStatus(this.reportGenerationPanel
						.getbundle()
						.getString(
								"ReportGenerationPanelController.progressBar.status.processing"));
	}

	/**
	 * Method used to control the logic when an active worker fires the
	 * SHOW_ERROR_MESSAGE operation.
	 */
	private void showErrorMessageOperation() {
		this.reportGenerationPanel
				.finishProcess("ReportGenerationPanelController.votingDevice.error.message");
	}
}
