package com.smartmatic.waffle.ecm.client.ui;

import java.util.Map;

import com.smartmatic.waffle.domain.i18n.LanguageData;
import com.smartmatic.waffle.ecm.client.ui.common.AwtOperationType;
import com.smartmatic.waffle.ecm.client.worker.ScreenshotsFileGenerationWorker;
import com.smartmatic.waffle.ecm.client.worker.ScreenshotsPrepareContestsWorker;
import com.smartmatic.waffle.ecm.client.worker.data.FileBackup;
import com.smartmatic.waffle.ecm.core.lang.PendriveException.PendriveExceptionInfo;
import com.smartmatic.waffle.ecm.data.transfer.type.USBGenerationType;
import com.smartmatic.waffle.ecm.ui.core.i18n.controller.LanguageController;
import com.smartmatic.waffle.log.SMTTLogger;

/**
 * Class used to manage the required tasks when performing a Screenshots
 * generation from the client application, this class will start the workers to
 * execute the server side methods.
 * 
 */
public class ScreenshotsGenerationController extends USBGenerationController {

	private static SMTTLogger logger = SMTTLogger
			.getLogger(ScreenshotsGenerationController.class);

	private ScreenshotsPrepareContestsWorker screenshotsPrepareContestsWorker;

	private ScreenshotsFileGenerationWorker screenshotsFileGenerationWorker;

	public ScreenshotsGenerationController(
			GUIGenerationInterface screenShotsGenerationPanel) {
		super(screenShotsGenerationPanel);
	}

	@Override
	public void awtOperation(AwtOperationType awtOperationType) {

		switch (awtOperationType) {
		case UPDATE_COMPONENTS:
			updateComponentsOperation();
			break;
		case REFRESH_LABEL:
			refreshLabelOperation();
			break;
		case UPDATE_PROGRESS_BAR:
			updateProgressBarOperation();
			break;
		case SHOW_ERROR_MESSAGE:
			showErrorMessageOperation();
			break;
		default:
			break;
		}
	}

	@Override
	protected void startPrepareContestsWorker() {
		LanguageData language = LanguageController.getInstance()
				.getSelectedLanguage();

		this.screenshotsPrepareContestsWorker = new ScreenshotsPrepareContestsWorker(
				getPanel().getSelectedObjects(), language, this.mountedPath);

		this.screenshotsPrepareContestsWorker.addListener(this);

		Thread screenShotsPrepareContestsWorkerThread = new Thread(
				this.screenshotsPrepareContestsWorker,
				"ScreenShotsPrepareContestWorkerThread");

		this.isPreparingContests = true;

		screenShotsPrepareContestsWorkerThread.start();
	}


    /**
     * {@inheritDoc}
     */
    @Override
    protected void startFileGenerationWorker() {
        ScreenshotsPrepareContestsWorker spcw;       // Worker Alias

        spcw = this.screenshotsPrepareContestsWorker;

        this.screenshotsFileGenerationWorker =
            new ScreenshotsFileGenerationWorker(
                this.mountedPath,
                spcw.getVotingCenterElectionMap(),
                spcw.getDeviceParameters()
            );

        this.screenshotsFileGenerationWorker.addListener(this);

        Thread screenshotsFileGenerationWorkerThread = new Thread(
            this.screenshotsFileGenerationWorker,
            "ScreenshotsFileGenerationWorkerThread"
        );

        screenshotsFileGenerationWorkerThread.start();
        this.isGeneratingFiles = true;
    }


	@Override
	protected USBGenerationType getUSBGenerationType() {
		return USBGenerationType.SCREENSHOT_USB;
	}

	@Override
	protected Map<String, FileBackup> getWrittenFiles() {
		return this.screenshotsFileGenerationWorker.getWrittenFiles();
	}

	@Override
	protected boolean isContestPreparationWorkerDone() {

		int toProcess = this.screenshotsPrepareContestsWorker
				.getContestsToProcess();
		int processed = this.screenshotsPrepareContestsWorker
				.getProcessedContests();

		return (toProcess == processed);
	}

	@Override
	protected boolean isFileGenerationDone() {
		return this.screenshotsFileGenerationWorker.isFileGenerationDone();
	}

	/**
	 * Method used to control the logic when an active worker fires the
	 * REFRESH_LABEL operation.
	 */
	private void refreshLabelOperation() {

		if (this.isPreparingContests) {

			getPanel().setPrepareContestIconInPendriveOkLabel();
			getPanel().setPendriveOkLabelVisible(true);

			int processed = this.screenshotsPrepareContestsWorker
					.getProcessedContests();
			int toProcess = this.screenshotsPrepareContestsWorker
					.getContestsToProcess();

			getPanel().setLabelStatusProcessingContests(processed, toProcess);
			getPanel().showLabelStatusProccess(true);
		}
	}

	/**
	 * Method used to control the logic when an active worker fires the
	 * UPDATE_PROGRESS_BAR operation.
	 */
	private void updateProgressBarOperation() {
		if (this.isGeneratingFiles) {
			getPanel().setProgressBarValue(
					this.screenshotsFileGenerationWorker
							.getVotingDevicesProcessed()
							* 100
							/ this.screenshotsFileGenerationWorker
									.getVotingDevicesToProcess());
		}
	}

	/**
	 * Method used to control the logic when an active worker fires the
	 * SHOW_ERROR_MESSAGE operation.
	 */
	private void showErrorMessageOperation() {
		String messageError = "ErrorMessage.GENERIC_ERROR";

		if (this.isCopyingOS) {
			PendriveExceptionInfo result = this.getCopyOSWorker().getResult();
			messageError = getCopyOsErrorMessage(result);
		} else if (this.isPreparingContests) {
			logger.error("Error preparing contest");
			messageError = "MasterUsbGenerationPanel."
					+ "prepare.master_drive.error";
		} else if (this.isGeneratingFiles) {
			logger.error("Error generating files");
			messageError = "MasterUsbGenerationPanel."
					+ "message.generating.pendrive.failure";
		} else if (this.isVerifyingFiles) {
			logger.error("Error verifiying files");
			messageError = "WritingInPenDriveInfo."
					+ "GENERATION_INTEGRITY_FAILURE";
		}

		resetOperationStatus();
		getPanel().showErrorMessage(messageError);
	}

	/**
	 * Gets error message key for a CopyOsWorker result
	 * 
	 * @param result
	 *            ExceptionInfo generated by worker
	 * @return The error message key
	 */
	private String getCopyOsErrorMessage(PendriveExceptionInfo result) {

		String messageError = "WritingInPenDriveInfo."
				+ "ERROR_WRITING_OS_IN_PENDRIVE.SCREENSHOT";
		if (result != null) {
			if (result == PendriveExceptionInfo.ERROR_EXECUTING_TAR_COMMAND) {
				messageError = "WritingInPenDriveInfo."
						+ "ERROR_WRITING_OS_IN_PENDRIVE.TAR_COMMAND";
			} else if (result == PendriveExceptionInfo.BAD_BLOCKS_DETECTED) {
				messageError = "WritingInPenDriveInfo."
						+ "ERROR_WRITING_OS_IN_PENDRIVE.FS_VERIFICATION";
			}
		}
		return messageError;
	}

}
