/**
 * FileVerificationUtils.java
 *
 * Revision: $Revision$  (Last Modified: $Date$)
 * Last Modified by: [$Author$]
 * 
 * Copyright (c) 2004 Smartmatic Corp.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of
 * Smartmatic Corp. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Corp.
 */

package com.smartmatic.waffle.ecm.client.util;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.Arrays;

import org.apache.commons.codec.digest.DigestUtils;
import org.apache.commons.io.FileUtils;
import org.apache.commons.lang.StringUtils;

import com.smartmatic.waffle.ecm.client.ECMClientSideConstants;
import com.smartmatic.waffle.ecm.client.lang.FileGenerationException;
import com.smartmatic.waffle.ecm.client.lang.FileGenerationException.FileGenerationExceptionInfo;
import com.smartmatic.waffle.ecm.client.worker.data.FileData;
import com.smartmatic.waffle.log.SMTTLogger;

/**
 * Executes utility functionalities for file verification.
 *
 */
public class FileVerificationUtils {

	/** Static SMTT logger instance. */
	private static SMTTLogger logger = SMTTLogger
			.getLogger(FileVerificationUtils.class);

	/**
	 * Verify if a file was written correctly from memory to disk and return the
	 * resulted hash
	 * 
	 * @param fileData
	 *            Information in memory of the file written on disk.
	 * @return resulted hash after validation.
	 * @throws FileGenerationException
	 *             if file is corrupted and it was not possible to recover.
	 * 
	 */
	public static byte[] verifyDiskFile(FileData fileData)
			throws FileGenerationException {

		byte[] diskHash = null;
		try {
			diskHash = getDiskFileHash(fileData.getFile());
		} catch (IOException e) {
			throw new FileGenerationException(e,
					FileGenerationExceptionInfo.WRITING_XML_FILE_FAILURE);
		}

		byte[] memoryHash = DigestUtils.sha256(fileData.getData());

		if (!Arrays.equals(diskHash, memoryHash)) {
			if (!recoverFile(fileData, memoryHash)) {
				throw new FileGenerationException(
						FileGenerationExceptionInfo.WRITING_XML_FILE_FAILURE);
			}
		}

		return memoryHash;
	}

	/**
	 * 
	 * Gets Hash given a file on disk
	 * 
	 * @param file
	 *            File to calculate hash.
	 * @return Sha256 digest
	 * @throws IOException
	 */
	public static byte[] getDiskFileHash(File file) throws IOException {

		InputStream diskDataStream = new FileInputStream(file);
		byte[] newHash = null;
		try {
			newHash = DigestUtils.sha256(diskDataStream);
		} finally {
			diskDataStream.close();
		}

		return newHash;
	}

	/**
	 * Recovers file using information form memory
	 * 
	 * @param fileData
	 *            Information in memory of the file written on disk.
	 * @param fileHash
	 *            calculated hash of the information in memory.
	 * @return True if recovery was successful.
	 */
	private static boolean recoverFile(FileData fileData, byte[] fileHash) {

		boolean result = false;
		int attempsToFileRecovery = getAttemptsToFileRecover();

		File fileToRecover = fileData.getFile();
		String fullPath = fileData.getFile().getAbsolutePath();
		byte[] data = fileData.getData();

		logger.info("Recovering file: " + fullPath);

		for (int i = 1; i <= attempsToFileRecovery; i++) {

			FileUtils.deleteQuietly(fileToRecover);

			if (fileToRecover.exists()) {
				logger.error("Not possible recover file: " + fullPath);
				break;
			}

			byte[] newHash = null;

			try {
				FileUtils.writeByteArrayToFile(fileToRecover, data);
				newHash = getDiskFileHash(fileToRecover);
			} catch (IOException e) {
				logger.error("Error recovering file, " + fullPath, e);
				continue;
			}

			if (Arrays.equals(newHash, fileHash)) {
				logger.info("File: " + fullPath + " recovered successfuly.");
				result = true;
				break;
			} else {
				logger.warn("File: " + fullPath
						+ " could not be recovered at attempt: " + i);
			}
		}

		return result;
	}

	/**
	 * Return the number of attempts for recover a damage file
	 * 
	 * @return the number of attempts
	 */
	private static int getAttemptsToFileRecover() {
		return ECMClientSideConstants.ATTEMPTS_TO_FILE_RECOVERY;
	}

	public static void deleteTemporaryPath() {

		if (ECMClientSideConstants.DELETE_GENERATION_TEMPORARY_FOLDER) {
			logger.info("Deleting generation temporary folder"
					+ ECMClientSideConstants.TEMPORARY_PATH_FOR_FILE_GENERATION);
			File generationTmpFolder = new File(
					ECMClientSideConstants.TEMPORARY_PATH_FOR_FILE_GENERATION);
			if (generationTmpFolder.exists()) {
				try {
					FileUtils.forceDelete(generationTmpFolder);
					logger.info("Temporary folder deleted");
				} catch (IOException e) {
					logger.error(e, StringUtils.defaultString(e.getMessage()));
				}
			}
		}
	}

}
