/**
 * PDFFilesSignatureRegister.java
 *
 * Revision: $Revision$  (Last Modified: $Date$)
 * Last Modified by: [$Author$]
 * 
 * Copyright (c) 2004 Smartmatic Corp.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of
 * Smartmatic Corp. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Corp.
 */
package com.smartmatic.waffle.ecm.client.util;

import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.IOException;

import org.apache.commons.codec.binary.Base64;
import org.dom4j.Document;
import org.dom4j.DocumentHelper;
import org.dom4j.Element;
import org.dom4j.io.OutputFormat;
import org.dom4j.io.XMLWriter;

import com.smartmatic.waffle.security.io.Base64Encoder;

/**
 * Creates the XML file of the CODI result file that will be signed.
 * 
 * @author Daniel Bastidas
 */
public class PDFFilesSignatureRegister {

	/** the CODI file data. */
	private byte[] data;

	/** The CODI file data signed in PKCS7 format. */
	private byte[] signedData;

	/** The CODI file. */
	private File codiFile;

	/** A bunch of CODI files. */
	private File[] list;

	/** The CODI file XML document that will be signed in PKCS7 format. */
	private Document document;

	/**
	 * Creates an instance of CODIFilesSignatureRegister class that create the
	 * XML document that will be signed in PKCS7 format.
	 * 
	 * @param dataArray
	 *            the CODI result file binary data
	 * @param codiFile
	 *            The CODI result file
	 * @param signedData
	 *            the CODI result file in PDF format and signed
	 */
	public PDFFilesSignatureRegister(byte[] dataArray, File codiFile,
			byte[] signedData) {
		this.data = null;
		if (dataArray != null) {
			this.data = dataArray.clone();
		}
		this.codiFile = codiFile;
		this.signedData = signedData.clone();
		this.document = DocumentHelper.createDocument();
		this.initPkcs7Format();
	}

	/**
	 * Creates an instance of CODIFilesSignatureRegister class that create the
	 * signature-register.xml file
	 * 
	 * @param data
	 *            the CODI file binary data
	 * @param files
	 *            the CODI files
	 */
	public PDFFilesSignatureRegister(byte[] data, File[] files) {
		this.signedData = null;
		this.list = null;
		if (data != null) {
			this.signedData = data.clone();
		}
		if (files != null) {
			this.list = files.clone();
		}
		this.document = DocumentHelper.createDocument();
		this.init();
	}

	/**
	 * Creates the signature-register.xml file
	 */
	private void init() {
		Element root = this.document.addElement("FinalData");
		for (File f : this.list) {
			Element signedPv = root.addElement("SignedPV");
			Element original = signedPv.addElement("OriginialFileName");
			original.setText(f.getName());
			Element docType = signedPv.addElement("DocType");
			int index = f.getName().indexOf(".");
			// Checks the document file extension
			if (index != -1) {
				docType.setText(f.getName().substring(index + 1));
			} else {
				docType.setText("");
			}

		}
		Element signature = root.addElement("signature");
		signature.setText(Base64Encoder.encode(this.signedData));
	}

	/**
	 * Creates the XML file of a result CODI file that will be signed in PKCS7
	 * format.
	 */
	private void initPkcs7Format() {
		Element root = this.document.addElement("FinalData");
		Element signedPv = root.addElement("SignedPV");
		Element original = signedPv.addElement("OriginialFileName");
		original.setText(this.codiFile.getName());
		Element docType = signedPv.addElement("DocType");
		docType.setText("PDF");

		Element data = signedPv.addElement("Data");
		data.setText(new String(Base64.encodeBase64(this.signedData, false)));
		Element formatF = root.addElement("FormatF");
		Element originialFileName = formatF.addElement("OriginialFileName");
		// retrieves the file name without the file extension
		originialFileName.setText(this.codiFile.getName().replaceAll(
				"\\.pdf|\\.PDF", ""));
		Element formatFData = formatF.addElement("Data");
		formatFData.setText(new String(Base64.encodeBase64(this.data, false)));
	}

	/**
	 * Retrieves the XML document as String.
	 * 
	 * @return the XML document as String
	 * @throws IOException
	 *             to report an input/output error
	 */
	public String asXML() throws IOException {
		return this.formatDocument(this.document, true);
	}

	/**
	 * Format the document.
	 * 
	 * @param document
	 *            the XML document to be formated
	 * @param prettyPrinted
	 *            Indicates if the document will be tabulated
	 * @return the String representation of the XML formated document
	 * @throws IOException
	 *             to report an input/output error
	 */
	private String formatDocument(Document document, boolean prettyPrinted)
			throws IOException {
		OutputFormat outputFormat = null;
		XMLWriter xmlWriter = null;
		ByteArrayOutputStream outputStream = null;
		String formatedDocument = null;
		try {
			if (prettyPrinted) {
				outputFormat = OutputFormat.createPrettyPrint();
			} else {
				outputFormat = OutputFormat.createCompactFormat();
			}
			outputFormat.setEncoding("UTF-8");
			outputStream = new ByteArrayOutputStream();
			xmlWriter = new XMLWriter(outputStream, outputFormat);
			try {
				xmlWriter.write(document);
			} finally {
				xmlWriter.close();
			}
			formatedDocument = outputStream.toString("UTF-8");
		} finally {
			xmlWriter.close();
			outputStream.close();
		}
		return formatedDocument;
	}
}
