/**
 * SignatureUtils.java
 *
 * Revision: $Revision$  (Last Modified: $Date$)
 * Last Modified by: [$Author$]
 * 
 * Copyright (c) 2004 Smartmatic Corp.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of
 * Smartmatic Corp. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Corp.
 */
package com.smartmatic.waffle.ecm.client.util;

import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.IOException;
import java.security.Provider;
import java.security.cert.X509Certificate;
import java.util.ArrayList;
import java.util.List;

import javax.xml.crypto.Data;
import javax.xml.crypto.OctetStreamData;
import javax.xml.crypto.URIDereferencer;
import javax.xml.crypto.URIReference;
import javax.xml.crypto.URIReferenceException;
import javax.xml.crypto.XMLCryptoContext;
import javax.xml.crypto.dsig.XMLSignature;
import javax.xml.crypto.dsig.XMLSignatureFactory;
import javax.xml.crypto.dsig.XMLValidateContext;
import javax.xml.crypto.dsig.dom.DOMValidateContext;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import org.apache.commons.io.FileUtils;

/**
 * Class used as utility to validate signature of the files
 */
public class SignatureUtils {

	/**
	 * Local references resolution for detached signature.
	 */
	public static class LocalURIDeferencer implements URIDereferencer {

		private File basePath;

		private List<File> referencedFiles = new ArrayList<File>();

		/**
		 * @param basePath
		 *            All references will be relative to the base path.
		 */
		public LocalURIDeferencer(File basePath) {
			this.basePath = basePath;
		}

		public List<File> getReferencedFiles() {
			return this.referencedFiles;
		}

		@Override
		public Data dereference(URIReference uriReference,
				XMLCryptoContext context) throws URIReferenceException {

			if (this.basePath == null || !this.basePath.exists()) {
				new URIReferenceException("Base path not found: "
						+ this.basePath);
			}

			File file = new File(this.basePath, uriReference.getURI());

			if (!file.exists()) {
				new URIReferenceException("Error, URI reference not found: "
						+ file.getPath());
			}

			try {
				byte[] fileArray = FileUtils.readFileToByteArray(file);
				ByteArrayInputStream inputStream = new ByteArrayInputStream(
						fileArray);
				OctetStreamData data = new javax.xml.crypto.OctetStreamData(
						inputStream);

				this.referencedFiles.add(file);
				return data;

			} catch (IOException e) {
				new URIReferenceException("Error reading reference file "
						+ file.getPath(), e);
			}
			return null;
		}
	}

	/**
	 * Validate a signature file using XMLDSig
	 * 
	 * @param signatureFile
	 *            Detached signature File to validate.
	 * @param xmlCert
	 *            Public key used to validate signature.
	 * @param validatedFiles
	 *            Output list of file resources validated in the signature.
	 * @return
	 * @throws SignatureException
	 */
	public static boolean validateSignatureFile(final File signatureFile,
			X509Certificate xmlCert, List<File> validatedFiles)
			throws SignatureException {

		org.w3c.dom.Document doc = getW3cDocument(signatureFile);
		XMLValidateContext valContext = new DOMValidateContext(
				xmlCert.getPublicKey(), doc.getDocumentElement());

		LocalURIDeferencer uriDeferencer = new LocalURIDeferencer(
				signatureFile.getParentFile());

		valContext.setURIDereferencer(uriDeferencer);

		XMLSignatureFactory factory = getXMLSignatureFactory();

		try {
			XMLSignature signature = factory.unmarshalXMLSignature(valContext);
			boolean result = signature.validate(valContext);

			if (validatedFiles != null) {
				validatedFiles.clear();
				validatedFiles.addAll(uriDeferencer.getReferencedFiles());
			}

			return result;
		} catch (Exception e) {
			throw new SignatureException("Error validating signature", e);
		}
	}

	/**
	 * Gets XML signature Factory
	 * 
	 * @return
	 * @throws SignatureException
	 */
	private static XMLSignatureFactory getXMLSignatureFactory()
			throws SignatureException {

		XMLSignatureFactory factory = null;

		String providerName = System.getProperty("jsr105Provider",
				"org.apache.jcp.xml.dsig.internal.dom.XMLDSigRI");

		try {
			Class<?> clazz = Class.forName(providerName);
			Provider provider = (Provider) clazz.newInstance();
			factory = XMLSignatureFactory.getInstance("DOM", provider);
		} catch (Exception e) {
			throw new SignatureException("Error building XMLSignatureFactory",
					e);
		}

		return factory;
	}

	/**
	 * Gets W3C document.
	 * 
	 * @param signatureFile
	 *            File to serialize.
	 * @return
	 * @throws SignatureException
	 */
	private static org.w3c.dom.Document getW3cDocument(File signatureFile)
			throws SignatureException {

		DocumentBuilderFactory dbFactory = DocumentBuilderFactory.newInstance();
		dbFactory.setNamespaceAware(true);

		try {
			DocumentBuilder dBuilder = dbFactory.newDocumentBuilder();
			org.w3c.dom.Document doc = dBuilder.parse(signatureFile);
			return doc;
		} catch (Exception e) {
			throw new SignatureException("Error loading Signature File", e);
		}
	}

}
