/**
 * BulkUploadEMLFilesWorker.java
 *
 * Revision: $Revision$  (Last Modified: $Date$)
 * Last Modified by: [$Author$]
 * 
 * Copyright (c) 2004 Smartmatic Corp.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of
 * Smartmatic Corp. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Corp.
 */
package com.smartmatic.waffle.ecm.client.worker;

import java.util.ArrayList;
import java.util.List;

import com.smartmatic.waffle.ecm.client.ECMServiceProvider;
import com.smartmatic.waffle.ecm.client.lang.ConnectionException;
import com.smartmatic.waffle.ecm.client.ui.common.AsynchronousOperation;
import com.smartmatic.waffle.ecm.client.ui.common.AwtOperationType;
import com.smartmatic.waffle.ecm.data.transfer.objects.BulkResultDTO;
import com.smartmatic.waffle.ecm.data.transfer.objects.FileToUploadDTO;
import com.smartmatic.waffle.ecm.data.transfer.objects.UploadedFileDetailDTO;
import com.smartmatic.waffle.ecm.data.transfer.type.EMLFileType;
import com.smartmatic.waffle.ecm.service.lang.BulkDataLoadException;
import com.smartmatic.waffle.log.SMTTLogger;

/**
 * Class that executes the main process to execute bulk upload of Smartmatic
 * files.
 */
public class BulkUploadEMLFilesWorker extends AsynchronousOperation {

	/**
	 * Static reference to log any info or error which occurred executing
	 * methods from this class.
	 */
	private static SMTTLogger logger = SMTTLogger
			.getLogger(BulkUploadEMLFilesWorker.class);

	/**
	 * Gets the list of codi files to upĺoad.
	 * 
	 * @return the list of codi files to upĺoad
	 */
	public List<FileToUploadDTO> getFilesToUpĺoadList() {
		return this.emlFilesToUpĺoadList;
	}

	/**
	 * BulkUploadEMLFilesWorker constructor.
	 * 
	 * @param emlFilesToUpĺoadList
	 *            Input list with files to load
	 * @param resourceFileContent
	 *            Hold the byte array with the resources file
	 */
	public BulkUploadEMLFilesWorker(List<FileToUploadDTO> emlFilesToUpĺoadList,
			byte[] resourceFileContent) {
		this.emlFilesToUpĺoadList = emlFilesToUpĺoadList;
		this.resourceFileContent = resourceFileContent;
		this.fileUploadedDetailList = new ArrayList<UploadedFileDetailDTO>();
	}

	/**
	 * Gets the number of processed files.
	 * 
	 * @return the number of processed files
	 */
	public int getNumberOfProcessedFiles() {
		return this.numberOfProcessedFiles;
	}

	/**
	 * Gets the file being processing.
	 * 
	 * @return the file being processing
	 */
	public FileToUploadDTO getFileBeingProcessing() {
		return this.fileBeingProcessing;
	}

	/**
	 * Gets the file uploaded detail list.
	 * 
	 * @return the file uploaded detail list
	 */
	public List<UploadedFileDetailDTO> getFileUploadedDetailList() {
		return this.fileUploadedDetailList;
	}

	/**
	 * Method that performs the bulk upload for codi files.
	 */
	public void startBulkUploadOperation() {
		try {
			for (FileToUploadDTO fileToUpload : this.emlFilesToUpĺoadList) {

				logger.info("Processing "
						+ fileToUpload.getFileToUploadDetailDTO().getFileInfo()
								.getName() + " file");

				UploadedFileDetailDTO fileUploadedDetailDTO = new UploadedFileDetailDTO();

				this.fileBeingProcessing = fileToUpload;

				fireAwtEvent(AwtOperationType.UPDATE_COMPONENTS);

				String fileToUploadName = fileToUpload
						.getFileToUploadDetailDTO().getFileInfo().getName();

				fileUploadedDetailDTO.setFileName(fileToUploadName);

				byte[] fileToUploadContent = fileToUpload
						.getFileToUploadDetailDTO().getFileContent();

				EMLFileType emlFileType = fileToUpload.getEMLFileType();

				fileUploadedDetailDTO.setEMLFileType(emlFileType);

				BulkResultDTO bulkResultDTO = uploadEMLFileIntoDataBase(
						fileToUploadName, fileToUploadContent, emlFileType,
						this.resourceFileContent);

				fileUploadedDetailDTO.setBulkResultDTO(bulkResultDTO);

				this.fileUploadedDetailList.add(fileUploadedDetailDTO);

				this.numberOfProcessedFiles++;
				fireAwtEvent(AwtOperationType.UPDATE_PROGRESS_BAR);
			}
			fireAwtEvent(AwtOperationType.SHOW_RESULT_MESSAGE);
		} catch (Exception e) {
			logger.error(e, "Error processing file: "
					+ this.fileBeingProcessing);
			fireAwtEvent(AwtOperationType.SHOW_ERROR_MESSAGE);
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.smartmatic.waffle.ecm.client.ui.common.AsynchronousOperation#
	 * performOperation()
	 */
	@Override
	protected void performOperation() {
		startBulkUploadOperation();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * com.smartmatic.waffle.ecm.client.ui.common.AsynchronousOperation#onCompleted
	 * ()
	 */
	@Override
	protected void onCompleted() {
	}

	/**
	 * Method that calls the process in server side to execute bulk upload of
	 * EML files
	 * 
	 * @param fileToUploadName
	 *            name of the file to upload
	 * @param fileToUploadContent
	 *            content of the file to upload
	 * @param fileType
	 *            type of file
	 * @param resourceFileContent
	 *            content of the resource file
	 * 
	 * @return dto with the result of bulk upload
	 * 
	 * @throws BulkDataLoadException
	 *             signal that an exception occurred executing bulk upload
	 * 
	 * @throws ConnectionException
	 *             Signal that an error occurred with the connection to the
	 *             server
	 */
	private BulkResultDTO uploadEMLFileIntoDataBase(String fileToUploadName,
			byte[] fileToUploadContent, EMLFileType emlFileType,
			byte[] resourceFileContent) throws BulkDataLoadException,
			ConnectionException {

		return ECMServiceProvider.uploadEMLFileIntoDataBase(fileToUploadName,
				fileToUploadContent, emlFileType, resourceFileContent);
	}

	/** Input list with files to load. */
	private List<FileToUploadDTO> emlFilesToUpĺoadList;

	/** Holds the byte array with the resources file. */
	private byte[] resourceFileContent;

	/** Holds the amount of processed files to show the process status. */
	private int numberOfProcessedFiles = 0;

	/**
	 * Holds the information about the file being processing in the moment to
	 * show the process status.
	 */
	private FileToUploadDTO fileBeingProcessing;

	/** Output list with the detail of elements uploaded. */
	private List<UploadedFileDetailDTO> fileUploadedDetailList;

}
