package com.smartmatic.waffle.ecm.client.worker;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import org.apache.commons.lang.StringUtils;

import com.smartmatic.waffle.ecm.client.ui.common.AsynchronousOperation;
import com.smartmatic.waffle.ecm.client.ui.common.AwtOperationType;
import com.smartmatic.waffle.localpersistence.SAESCommonDestinations;
import com.smartmatic.waffle.localpersistence.SAESLocalPersistenceManager;
import com.smartmatic.waffle.localpersistence.SAESLocalPersistenceManager.MountedDrive;
import com.smartmatic.waffle.log.SMTTLogger;

/**
 * Worker class used to detect the drives allowed to generate the Master USB.
 * 
 */
public class ConnectedDriveFinderWorker extends AsynchronousOperation {

	/** Logger. */
	private static SMTTLogger logger = SMTTLogger
			.getLogger(ConnectedDriveFinderWorker.class);

	/**
	 * List of devices allowed
	 */
	private List<SAESCommonDestinations> devicesAllowed;

	/**
	 * Boolean that holds if the current finder worker must be interrupted.
	 */
	private boolean disableWorker = false;

	private Set<String> uniqueDevices = new HashSet<String>();

	private Set<String> allowedExtensions;

	public ConnectedDriveFinderWorker() {
	}

	public ConnectedDriveFinderWorker(Set<String> allowedExtensions) {
		this.allowedExtensions = allowedExtensions;
	}

	/**
	 * Method used to set true the disableWorker variable.
	 */
	public void disableWorker() {
		this.disableWorker = true;
	}

	/**
	 * Return a list of devices allowed detected for the current worker.
	 * 
	 * @return a list of devices.
	 */
	public List<SAESCommonDestinations> getDevicesAllowed() {
		return this.devicesAllowed;
	}

	/**
	 * Performs the detect operation while the worker is enabled and the
	 * devicesAllow list is empty.
	 * 
	 * @throws Exception
	 *             Signal that an error has occurred detecting the drives.
	 */
	private void runFinder() throws Exception {

		this.devicesAllowed = new ArrayList<SAESCommonDestinations>();

		while (!this.disableWorker && this.devicesAllowed.size() == 0) {

			Thread.sleep(3000);

			checkMountedDevices();

			for (SAESCommonDestinations device : getDevices()) {

				MountedDrive drive = device.getDeviceInfo();

				if (drive != null
						&& drive.getMountedPath().getAbsolutePath()
								.contains("/media")
						&& !this.uniqueDevices.contains(drive.getDeviceName())
						&& filterExtensions(drive)) {
					this.devicesAllowed.add(device);
					this.uniqueDevices.add(drive.getDeviceName());
				}
			}
		}
	}

	/**
	 * If specified, this method will check if the mounted drive's file systems
	 * is allowed to be included in the drive detection.
	 * 
	 * @param drive
	 *            the drive to be checked
	 * @return true if the allowed file systems list contains the drive file
	 *         system, otherwise false.
	 */
	private boolean filterExtensions(MountedDrive drive) {
		if (this.allowedExtensions == null
				|| (this.allowedExtensions != null && this.allowedExtensions
						.isEmpty())) {
			return true;
		}

		return this.allowedExtensions.contains(drive.getFileSystem());
	}

	/**
	 * Performs a check of mounted devices
	 * 
	 * @throws Exception
	 *             Signal that an error has occurred checking the mounted
	 *             devices
	 */
	protected void checkMountedDevices() throws Exception {
		SAESLocalPersistenceManager.getInstance().checkMountedDevices();
	}

	/**
	 * Get a list of the founded devices
	 * 
	 * @return a list of devices
	 */
	protected List<SAESCommonDestinations> getDevices() {
		return SAESLocalPersistenceManager.getInstance().getStoreMedias();
	}

	@Override
	protected void performOperation() {

		try {
			logger.info("Finding connected drives...");
			runFinder();
		} catch (Exception e) {
			logger.error(e, StringUtils.defaultString(e.getMessage()));
			fireAwtEvent(AwtOperationType.SHOW_ERROR_MESSAGE);
		}

	}

	@Override
	protected void onCompleted() {
		fireAwtEvent(AwtOperationType.UPDATE_COMPONENTS);
	}

}
