/**
 * DecryptWorker.java
 *
 * Revision: $Revision$  (Last Modified: $Date$)
 * Last Modified by: [$Author$]
 * 
 * Copyright (c) 2004 Smartmatic Corp.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of
 * Smartmatic Corp. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Corp.
 */
package com.smartmatic.waffle.ecm.client.worker;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FilenameFilter;
import java.io.IOException;
import java.io.InputStream;
import java.util.Arrays;
import java.util.List;
import java.util.zip.InflaterOutputStream;

import javax.crypto.SecretKey;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.xpath.XPath;
import javax.xml.xpath.XPathConstants;
import javax.xml.xpath.XPathExpression;
import javax.xml.xpath.XPathFactory;

import org.apache.commons.codec.binary.Base64;
import org.apache.commons.io.FileUtils;
import org.apache.commons.io.FilenameUtils;
import org.apache.commons.lang.StringUtils;
import org.w3c.dom.Document;

import com.smartmatic.waffle.ecm.client.ECMClientSideConstants;
import com.smartmatic.waffle.ecm.client.lang.DecryptException;
import com.smartmatic.waffle.ecm.client.lang.DecryptException.DecryptExceptionInfo;
import com.smartmatic.waffle.ecm.client.ui.common.AsynchronousOperation;
import com.smartmatic.waffle.ecm.client.ui.common.AwtOperationType;
import com.smartmatic.waffle.ecm.core.lang.PendriveException;
import com.smartmatic.waffle.ecm.core.pendrive.util.PendriveUtil;
import com.smartmatic.waffle.log.SMTTLogger;
import com.smartmatic.waffle.security.asymmetric.EncryptDecryptUtil;
import com.smartmatic.waffle.security.asymmetric.PBEFileUtil;
import com.smartmatic.waffle.security.asymmetric.PBEFileUtil.CipheringAlgorithm;
import com.smartmatic.waffle.security.dto.CipherDTO;
import com.smartmatic.waffle.security.lang.CryptographicRuntimeException;
import com.smartmatic.waffle.security.lang.CryptographicStaticException;
import com.smartmatic.waffle.security.symmetric.SymmetricKeyManager;

/**
 * Worker class that executes the decipher process in a background thread.
 */
public class DecryptWorker extends AsynchronousOperation {

	/**
	 * XPath to extract rnd attribute from voting center nodes.
	 */
	private static final String VOTING_CENTER_RND_XPATH = "//VotingCenter[@folder-name='%s']/@%s";

	/** For logging proposes. */
	private static SMTTLogger logger = SMTTLogger
			.getLogger(DecryptWorker.class);

	/**
	 * Enum that holds the type of PBE that obtains the symmetric key to
	 * decipher xml files.
	 */
	enum PBEType {
		UP, SECRET;
	}

	/** Length of initialization vector for CBC ciphering. */
	protected Integer iVectorLength;

	/** The encryption algorithm which the files was encrypted. */
	protected CipheringAlgorithm cipheringAlgorithm;

	/** Indicates if the election file should be decompressed */
	protected boolean compressedElection;

	/** The voting device folder to decrypt. */
	private File votingDeviceFolderToDecrypt;

	/** The election file to decrypt. */
	private File electionFileToDecrypt;

	/** Passphrase used to encrypt the files. */
	private String passphrase;

	/**
	 * Holds the name of any file that produces an exception trying to decipher
	 */
	private String fileWithProblem;

	/**
	 * Holds the error bundle to show an explicit error message to the user.
	 */
	private String errorBundle;

	/**
	 * The application key to decrypt the voting center
	 */
	private String applicationKey;

	/**
	 * The key length.
	 */
	private int keyLength;

	/**
	 * The SAES folder.
	 */
	private File saesFolder;

	/**
	 * Instantiates a new DecryptWorker.
	 * 
	 * @param votingDeviceFolderToDecrypt
	 *            the voting device folder to decrypt
	 * 
	 * @param passphrase
	 *            the passphrase provided by the user to decrypt the files
	 * 
	 * @param applicationKey
	 *            the application key
	 * 
	 * @param keyLength
	 *            the key length
	 */
	public DecryptWorker(File votingDeviceFolderToDecrypt, String passphrase,
			String applicationKey, int keyLength) {

		this.votingDeviceFolderToDecrypt = votingDeviceFolderToDecrypt;

		this.applicationKey = applicationKey;

		this.passphrase = passphrase;

		this.electionFileToDecrypt = new File(
				this.votingDeviceFolderToDecrypt.getParentFile(),
				"election.xml");

		this.saesFolder = this.electionFileToDecrypt.getParentFile()
				.getParentFile();

		this.keyLength = keyLength;

		loadProperties();
	}

	/**
	 * Gets the reference of any file that produces an exception trying to
	 * decipher.
	 * 
	 * @return the reference of any file that produces an exception trying to
	 *         decipher
	 */
	public String getFileWithProblem() {
		return this.fileWithProblem;
	}

	/**
	 * Gets the error bundle to show an explicit error message to the user.
	 * 
	 * @return the error bundle to show an explicit error message to the user
	 */
	public String getErrorBundle() {
		return this.errorBundle;
	}

	/**
	 * Loads some needed properties from config-properties.xml
	 */
	protected void loadProperties() {

		this.compressedElection = ECMClientSideConstants.GENERATE_ELECTION_COMPRESS;

		String cipheringAlgorithm = ECMClientSideConstants.ENCRYPTION_ALGORITHM;

		if (cipheringAlgorithm.equals("CBC")) {
			this.cipheringAlgorithm = CipheringAlgorithm.CBC;
			this.iVectorLength = ECMClientSideConstants.LENGTH_KEY;
		} else if (cipheringAlgorithm.equals("ECB")) {
			this.cipheringAlgorithm = CipheringAlgorithm.ECB;
		}
	}

	/**
	 * Validates the selected folder, ie, checks that it is a proper
	 * polling-place folder containing the expected files.
	 * 
	 * @throws DecryptException
	 *             signals that the selected folder is not a valid folder
	 */
	protected boolean validateSelectedFolder() throws DecryptException {

		File votingDeviceSaesFolder = new File(
				this.votingDeviceFolderToDecrypt, "SAES");

		File votingDeviceFile = new File(votingDeviceSaesFolder,
				"voting-devices.xml");

		File resourcesFolder = new File(votingDeviceSaesFolder,
				"StringResources");

		if (!this.electionFileToDecrypt.exists()) {
			throw new DecryptException(
					DecryptExceptionInfo.ELECTION_FILE_NOT_FOUND);
		}

		if (!votingDeviceFile.exists()) {
			throw new DecryptException(
					DecryptExceptionInfo.VOTING_DEVICE_FILE_NOT_FOUND);
		}

		if (resourcesFolder.listFiles().length == 0) {
			throw new DecryptException(
					DecryptExceptionInfo.STRING_RESOURCES_FILE_NOT_FOUND);
		}

		return true;

	}

	/**
	 * Returns the data contained in a ciphered file.
	 * 
	 * @param pbeFile
	 *            password-based encryption file ciphered with a passphrase
	 * 
	 * @return the plain data
	 * 
	 * @throws DecryptException
	 *             signals that some problem occurs deciphering
	 */
	protected String getDataFromCipheredFile(File pbeFile)
			throws DecryptException {

		try {
			byte[] simpleFileData = null;

			String saltSeed = getSaltSeed();

			try {
				simpleFileData = PBEFileUtil.readDataFromCipheredFile(pbeFile,
						this.passphrase, this.cipheringAlgorithm,
						this.iVectorLength, saltSeed);
			} catch (CryptographicRuntimeException e) {
				return org.apache.commons.lang.StringUtils.EMPTY;
			}

			return org.apache.commons.codec.binary.StringUtils
					.newStringUtf8(Base64.encodeBase64(simpleFileData, true));

		} catch (Exception e) {
			this.fileWithProblem = pbeFile.getName();
			throw new DecryptException(e,
					DecryptExceptionInfo.PROBLEM_GETTING_SYMMETRIC_KEY);
		}
	}

	private String getSaltSeed() throws Exception {

		File votingCenterFile = new File(this.saesFolder, "voting-centers.xml");

		byte[] decipheredData = null;

		byte[] rawData = FileUtils.readFileToByteArray(votingCenterFile);

		SecretKey secretKey = SymmetricKeyManager.getSymmetricKey(Base64
				.decodeBase64(this.applicationKey));

		switch (this.cipheringAlgorithm) {
		case CBC:
			decipheredData = processCBCAlgorithm(secretKey, rawData,
					this.keyLength);

			return String.format("%s-%s",
					getAttributeFromXML(decipheredData, "code").toLowerCase(),
					getAttributeFromXML(decipheredData, "rnd"));

		case ECB:

			decipheredData = EncryptDecryptUtil.decryptData(rawData, secretKey);

			return String.format("%s-%s",
					getAttributeFromXML(decipheredData, "code"),
					getAttributeFromXML(decipheredData, "rnd"));
		}

		return null;
	}

	private String getAttributeFromXML(byte[] decipheredData, String attribute)
			throws Exception {

		InputStream xmlData = new ByteArrayInputStream(decipheredData);
		try {
			DocumentBuilderFactory factory = DocumentBuilderFactory
					.newInstance();
			DocumentBuilder builder = factory.newDocumentBuilder();
			Document doc = builder.parse(xmlData);
			XPathFactory xPathfactory = XPathFactory.newInstance();
			XPath xpath = xPathfactory.newXPath();
			XPathExpression expr = xpath.compile(String.format(
					DecryptWorker.VOTING_CENTER_RND_XPATH,
					this.votingDeviceFolderToDecrypt.getName(), attribute));
			return (String) expr.evaluate(doc, XPathConstants.STRING);
		} finally {
			xmlData.close();
		}
	}

	/**
	 * Returns the symmetric key from an specific ciphered file.
	 * 
	 * @param pbeType
	 *            represents the type of PBE to be opened (UP or SECRET)
	 * 
	 * @return the symmetric key
	 * 
	 * @throws DecryptException
	 *             signals that some problem occurs deciphering
	 */
	protected String getSpecificSymmetricKey(PBEType pbeType)
			throws DecryptException {

		File pbefile = null;
		String symmetricKey = null;

		switch (pbeType) {
		case UP: {

			pbefile = new File(this.votingDeviceFolderToDecrypt, "up-key");
			symmetricKey = getDataFromCipheredFile(pbefile);

			if (symmetricKey.isEmpty()) {
				pbefile = new File(this.votingDeviceFolderToDecrypt,
						"up-recount-key");
				symmetricKey = getDataFromCipheredFile(pbefile);

			}

			if (symmetricKey.isEmpty()) {
				throw new DecryptException(
						DecryptExceptionInfo.INVALID_PASSPHRASE);
			}
			return symmetricKey;
		}

		case SECRET:
			pbefile = new File(this.votingDeviceFolderToDecrypt, "secret-key");
			symmetricKey = getDataFromCipheredFile(pbefile);

			if (symmetricKey.isEmpty()) {
				pbefile = new File(this.votingDeviceFolderToDecrypt,
						"recount-key");
				symmetricKey = getDataFromCipheredFile(pbefile);
			}

			if (symmetricKey.isEmpty()) {
				throw new DecryptException(
						DecryptExceptionInfo.INVALID_PASSPHRASE);
			}
			return symmetricKey;
		}

		return org.apache.commons.lang.StringUtils.EMPTY;
	}

	/**
	 * Deciphers a file
	 * 
	 * @param selectedFile
	 *            cipher file to decipher
	 * 
	 * @param symmetricKey
	 *            symmetricKey key to decipher
	 * 
	 * @throws DecryptException
	 *             signals that some problem occurs deciphering a file
	 */
	protected void decipherFile(File selectedFile, String symmetricKey,
			int keyLength) throws DecryptException {

		try {
			logger.info("Decipher " + selectedFile.getName());

			byte[] decipheredData = null;

			byte[] key = Base64.decodeBase64(symmetricKey);

			SecretKey secretKey = SymmetricKeyManager.getSymmetricKey(key);

			byte[] rawData = FileUtils.readFileToByteArray(selectedFile);

			decipheredData = decipherData(keyLength, decipheredData, secretKey,
					rawData);

			decipheredData = processElectionFile(selectedFile, decipheredData);

			writeUncompressFile(selectedFile, decipheredData);

		} catch (Exception e) {
			this.fileWithProblem = selectedFile.getName();
			throw new DecryptException(e,
					DecryptExceptionInfo.PROBLEM_DECIPHERING_FILE);
		}
	}


    /**
     * Deciphers the given {@code file}, if and only if, it exists
     * and is a regular file.
     *
     * @param file
     *      the {@code File} to be deciphered
     * @param key
     *      {@code Base64 String} that represents the Symmetric key to be
     *      used while deciphering the {@code file}
     * @param keyLength
     *      {@code Key}'s length
     *
     * @throws DecryptException
     *      Signals that some problem occurs deciphering a file
     */
    protected void decipherFileIfExists(File file, String key, int keyLength)
    throws DecryptException {
        if ( file.isFile() ) {
            this.decipherFile(file, key, keyLength);
        }
    }


	/**
	 * Write the plain file.
	 * 
	 * @param selectedFile
	 *            The file processed.
	 * @param decipheredData
	 *            The deciphered data to write.
	 * @throws IOException
	 */
	private void writeUncompressFile(File selectedFile, byte[] decipheredData)
			throws IOException {
		String plainFileName = FilenameUtils
				.getBaseName(selectedFile.getName());

		File plainFile = new File(selectedFile.getParentFile(), plainFileName
				+ "-plain.xml");

		FileUtils.writeByteArrayToFile(plainFile, decipheredData);
	}

	/**
	 * Process the election.xml if it's necessary.
	 * 
	 * @param selectedFile
	 *            The selected file.
	 * @param decipheredData
	 *            The deciphered data.
	 * @return The uncompressed election.xml
	 * @throws IOException
	 */
	private byte[] processElectionFile(File selectedFile, byte[] decipheredData)
			throws IOException {
		if (checkFileCompress(selectedFile, "election.xml")) {
			decipheredData = uncompressFile(decipheredData);
		}
		return decipheredData;
	}

	/**
	 * Decipher the data with the respective cipher algorithm.
	 * 
	 * @param keyLength
	 *            The key length.
	 * @param decipheredData
	 *            The deciphered data.
	 * @param secretKey
	 *            The secret key.
	 * @param rawData
	 *            The data to decipher.
	 * @return The deciphered data.
	 * @throws CryptographicStaticException
	 * @throws CryptographicRuntimeException
	 */
	private byte[] decipherData(int keyLength, byte[] decipheredData,
			SecretKey secretKey, byte[] rawData)
			throws CryptographicStaticException, CryptographicRuntimeException {
		switch (this.cipheringAlgorithm) {
		case CBC:
			decipheredData = processCBCAlgorithm(secretKey, rawData, keyLength);
			break;

		case ECB:
			decipheredData = EncryptDecryptUtil.decryptData(rawData, secretKey);
			break;

		}
		return decipheredData;
	}

	/**
	 * Check if the selected file is compressed and is the needed file.
	 * 
	 * @param selectedFile
	 *            The selected file.
	 * @param fileToCheck
	 *            The file to check.
	 * @return
	 */
	private boolean checkFileCompress(File selectedFile, String fileToCheck) {
		return this.compressedElection
				&& selectedFile.getName().equalsIgnoreCase(fileToCheck);
	}

	/**
	 * Uncompress XML file.
	 * 
	 * @param decipheredData
	 *            Byte array with the file data compressed.
	 * @return Byte array with the file data uncompressed.
	 * @throws IOException
	 */
	private byte[] uncompressFile(byte[] decipheredData) throws IOException {
		logger.info("Decompressing xml file");

		ByteArrayOutputStream out = new ByteArrayOutputStream();

		InflaterOutputStream inflaterOut = new InflaterOutputStream(out);

		try {
			inflaterOut.write(decipheredData);
		} finally {
			inflaterOut.finish();
			inflaterOut.close();
		}

		decipheredData = out.toByteArray();
		return decipheredData;
	}

	/**
	 * Process to decipher data through CBC Cipher Algorithm.
	 * 
	 * @param secretKey
	 *            The secret key value.
	 * @param rawData
	 *            The data to decipher.
	 * @param keyLenght
	 *            The key length.
	 * @return The deciphered data
	 * @throws CryptographicStaticException
	 * @throws CryptographicRuntimeException
	 */
	private byte[] processCBCAlgorithm(SecretKey secretKey, byte[] rawData,
			int keyLenght) throws CryptographicStaticException,
			CryptographicRuntimeException {

		byte[] decipheredData;

		CipherDTO cipherDTO = EncryptDecryptUtil.prepareCipherDTO(
				this.iVectorLength, rawData, keyLenght);

		decipheredData = EncryptDecryptUtil.decryptData(
				cipherDTO.getCipheredData(), secretKey, cipherDTO.getiVector());
		return decipheredData;
	}


    /**
     * Decrypts the xml files in the voting device folder and it sub folders.
     *
     * @throws DecryptException
     *      signals that some problem occurs deciphering some file
     * @throws PendriveException
     *      signals that some problem occurs with media.
     */
    protected void decryptFiles() throws DecryptException, PendriveException {
        File devFolder;             // VotingDevice's Folder alias
        String upSymmetricKey;
        String votingDeviceSymmetricKey;

        devFolder      = this.votingDeviceFolderToDecrypt;
        upSymmetricKey = this.getSpecificSymmetricKey(PBEType.UP);

        this.decipherFile(
            this.electionFileToDecrypt, upSymmetricKey, this.keyLength
        );
        this.decipherFileIfExists(
            new File(devFolder.getParentFile(), "device-parameters.xml"),
            upSymmetricKey, this.keyLength
        );

        this.decryptGeneralAudioResources(upSymmetricKey);

        votingDeviceSymmetricKey = this.getSpecificSymmetricKey(PBEType.SECRET);

        this.decipherFilesRecursive(
            this.votingDeviceFolderToDecrypt, votingDeviceSymmetricKey,
            this.keyLength
        );

        PendriveUtil.sync();
        this.fireAwtEvent(AwtOperationType.SHOW_RESULT_MESSAGE);
    }


	/**
	 * Decrypts general audio resources files.
	 * 
	 * @param upSymmetricKey
	 * @throws DecryptException
	 *             signals that some problem occurs with media.
	 */
	private void decryptGeneralAudioResources(String upSymmetricKey)
			throws DecryptException {
		List<File> generalAudioResources = getGeneralAudioResources();

		for (File file : generalAudioResources) {
			decipherFile(file, upSymmetricKey, this.keyLength);
		}
	}

	/**
	 * Gets the general audio resources files.
	 * 
	 * @return the general audio resources files.
	 */
	private List<File> getGeneralAudioResources() {

		File parentDir = this.votingDeviceFolderToDecrypt.getParentFile();
		File[] files = parentDir.listFiles(new FilenameFilter() {

			@Override
			public boolean accept(File dir, String name) {

				return name.matches("^general-audio-resources.*\\.xml$");
			}
		});

		if (files.length == 0) {
			logger.warn("No general-audio-resources files detected to decrypt");
		}

		return Arrays.asList(files);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.smartmatic.waffle.ecm.client.ui.common.AsynchronousOperation#
	 * performOperation()
	 */
	@Override
	protected void performOperation() {
		try {
			if (validateSelectedFolder()) {
				decryptFiles();
			}
		} catch (DecryptException e) {
			if (e.getBundleKey().equals(
					"DecryptException.problem.getting.symmetric.key")
					|| e.getBundleKey().equals(
							"DecryptException.problem.deciphering.file")) {
				logger.error(
						e,
						StringUtils.defaultString(e.getMessage()
								+ this.fileWithProblem));
			} else {
				logger.error(e, StringUtils.defaultString(e.getMessage()));
			}
			this.errorBundle = e.getBundleKey();
			fireAwtEvent(AwtOperationType.SHOW_ERROR_MESSAGE);
		} catch (Exception e) {
			this.errorBundle = "DecryptException.unexpected.error";
			logger.error(e, StringUtils.defaultString(e.getMessage()));
			fireAwtEvent(AwtOperationType.SHOW_ERROR_MESSAGE);
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * com.smartmatic.waffle.ecm.client.ui.common.AsynchronousOperation#onCompleted
	 * ()
	 */
	@Override
	protected void onCompleted() {
	}

	/**
	 * Process all files and sub files
	 * 
	 * @param folderToProcess
	 *            Folder to search files to decipher
	 * 
	 * @param symmetricKey
	 *            symmetricKey key to decipher
	 * 
	 * 
	 * @throws DecryptException
	 *             signals that some problem ocurss deciphering some file
	 */
	private void decipherFilesRecursive(File folderToProcess,
			String symmetricKey, int keyLength) throws DecryptException {
		File[] files = folderToProcess.listFiles();

		for (int x = 0; x < files.length; x++) {
			if (files[x].isDirectory()) {
				decipherFilesRecursive(files[x], symmetricKey, keyLength);
			} else {
				if (files[x].getAbsolutePath().endsWith(".xml")
						&& !files[x].getAbsolutePath().endsWith("-plain.xml")) {
					decipherFile(files[x], symmetricKey, keyLength);
				}
			}
		}
	}
}
