package com.smartmatic.waffle.ecm.client.worker;

import java.io.BufferedReader;
import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.io.InputStream;
import java.security.KeyStore;
import java.security.cert.Certificate;
import java.util.AbstractMap;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.apache.commons.codec.binary.Base64;
import org.apache.commons.io.FileUtils;
import org.apache.commons.io.IOUtils;
import org.apache.commons.lang.StringUtils;
import org.apache.xml.security.signature.XMLSignature;
import org.dom4j.Document;
import org.dom4j.DocumentHelper;
import org.dom4j.Element;

import com.smartmatic.waffle.domain.BaseDataObject;
import com.smartmatic.waffle.domain.authority.ElectoralAuthorityData;
import com.smartmatic.waffle.domain.ballot.BallotCardData;
import com.smartmatic.waffle.domain.ballot.BallotData;
import com.smartmatic.waffle.domain.contest.ContestData;
import com.smartmatic.waffle.domain.contest.ContestGroupData;
import com.smartmatic.waffle.domain.demographics.DistrictData;
import com.smartmatic.waffle.domain.demographics.ElectionData;
import com.smartmatic.waffle.domain.demographics.PollingPlaceData;
import com.smartmatic.waffle.domain.demographics.ProgrammingUnitData;
import com.smartmatic.waffle.domain.demographics.RegionData;
import com.smartmatic.waffle.domain.demographics.VotingBoothData;
import com.smartmatic.waffle.domain.device.DeviceParameterData;
import com.smartmatic.waffle.domain.device.VotingCentersData;
import com.smartmatic.waffle.domain.device.VotingCentersData.VotingCenter;
import com.smartmatic.waffle.domain.device.VotingDeviceData;
import com.smartmatic.waffle.domain.device.VotingDevicesData;
import com.smartmatic.waffle.domain.eligibleoption.ImageWrapper;
import com.smartmatic.waffle.domain.eligibleoption.PartyData;
import com.smartmatic.waffle.domain.general.KeystoreData;
import com.smartmatic.waffle.domain.general.ReportTemplateData;
import com.smartmatic.waffle.domain.i18n.LanguageCollectionData;
import com.smartmatic.waffle.domain.i18n.LanguageData;
import com.smartmatic.waffle.domain.i18n.LanguageForVotingDeviceData;
import com.smartmatic.waffle.domain.multiLanguage.StringResourceData;
import com.smartmatic.waffle.domain.multiLanguage.StringResourcesData;
import com.smartmatic.waffle.domain.security.FileChecksumData;
import com.smartmatic.waffle.domain.xstream.converters.HibernateCollectionConverter;
import com.smartmatic.waffle.ecm.client.ECMClientSideConstants;
import com.smartmatic.waffle.ecm.client.ECMServiceProvider;
import com.smartmatic.waffle.ecm.client.lang.ConnectionException;
import com.smartmatic.waffle.ecm.client.lang.FileGenerationException;
import com.smartmatic.waffle.ecm.client.lang.FileGenerationException.FileGenerationExceptionInfo;
import com.smartmatic.waffle.ecm.client.ui.common.AsynchronousOperation;
import com.smartmatic.waffle.ecm.client.ui.common.AwtOperationType;
import com.smartmatic.waffle.ecm.client.ui.controller.SortedLanguage;
import com.smartmatic.waffle.ecm.client.ui.controller.SortedLanguageForVotingDevice;
import com.smartmatic.waffle.ecm.client.util.FileVerificationUtils;
import com.smartmatic.waffle.ecm.client.util.MasterFileGenerationUtils;
import com.smartmatic.waffle.ecm.client.worker.data.FileBackup;
import com.smartmatic.waffle.ecm.client.worker.data.FileData;
import com.smartmatic.waffle.ecm.core.lang.PendriveException;
import com.smartmatic.waffle.ecm.core.lang.PendriveException.PendriveExceptionInfo;
import com.smartmatic.waffle.ecm.core.pendrive.util.PendriveUtil;
import com.smartmatic.waffle.ecm.core.util.SAESZipUtil;
import com.smartmatic.waffle.ecm.core.util.TimestampUtil;
import com.smartmatic.waffle.ecm.data.transfer.objects.ElectionObjectDTO;
import com.smartmatic.waffle.ecm.data.transfer.objects.ProgrammingUnitElectionObjectDTO;
import com.smartmatic.waffle.ecm.data.transfer.objects.VotingDeviceDTO;
import com.smartmatic.waffle.ecm.service.impl.ECMServerCommonImpl;
import com.smartmatic.waffle.ecm.service.lang.ElectoralOrganizationOperationException;
import com.smartmatic.waffle.ecm.service.lang.ElectoralPhysicalLocationsRegistryException;
import com.smartmatic.waffle.ecm.service.lang.MasterUSBGenerationException;
import com.smartmatic.waffle.ecm.service.lang.UtilitiesException;
import com.smartmatic.waffle.ecm.service.lang.XmlGenerationException;
import com.smartmatic.waffle.log.SMTTLogger;
import com.smartmatic.waffle.security.asymmetric.AsymmetricKeyManager;
import com.smartmatic.waffle.security.lang.CryptographicStaticException;
import com.smartmatic.waffle.security.sign.SignXMLUtil;
import com.smartmatic.waffle.security.symmetric.SymmetricKeyManager;
import com.thoughtworks.xstream.XStream;
import com.thoughtworks.xstream.io.xml.DomDriver;
import com.thoughtworks.xstream.mapper.Mapper;

/**
 * Worker that perform the generation and writes the files for the master USB.
 */
public class MasterUSBFileGenerationWorker extends AsynchronousOperation {

	private static final String LIVE_BOOT_SHA256SUM_TXT_FILE_NAME = "live-boot-SHA256SUM.txt";

	/** Static SMTT logger instance. */
	private static SMTTLogger logger = SMTTLogger
			.getLogger(MasterUSBFileGenerationWorker.class);

	private String targetPath;

	private String mountedPath;

	private ProgrammingUnitElectionObjectDTO programmingUnitElectionObjectDto;

	private boolean generateUsingProgrammingUnits;

	private Map<ElectoralAuthorityData, Set<LanguageData>> electoralAuthorityLanguages;

	private List<VotingCenter> votingCenterList;

	private int votingDevicesToProcess;

	private int votingDevicesProcessed;

	private HashMap<String, FileBackup> writtenFilesMap;

	private boolean fileGenerationDone;

	private String applicationKey;

	/**
	 * Boolean that holds if the generation is being performed in test mode.
	 */
	protected boolean testMode;

	public MasterUSBFileGenerationWorker(String mountedPath,
			ProgrammingUnitElectionObjectDTO programmingUnitElectionObjectDto,
			boolean generateUsingProgrammingUnits, String applicationKey) {

		this.mountedPath = mountedPath;
		this.programmingUnitElectionObjectDto = programmingUnitElectionObjectDto;
		this.generateUsingProgrammingUnits = generateUsingProgrammingUnits;
		this.writtenFilesMap = new HashMap<String, FileBackup>();
		this.testMode = Boolean.getBoolean("test_mode");
		this.applicationKey = applicationKey;
	}

	/**
	 * Return the total number of voting devices to process
	 * 
	 * @return the total number of voting devices
	 */
	public int getVotingDevicesToProcess() {
		return this.votingDevicesToProcess;
	}

	/**
	 * Return the number of voting devices processed so far
	 * 
	 * @return the number of voting devices processed
	 */
	public int getVotingDevicesProcessed() {
		return this.votingDevicesProcessed;
	}

	/**
	 * Return the set of written files
	 * 
	 * @return the set of written files
	 */
	public Map<String, FileBackup> getWrittenFiles() {
		return this.writtenFilesMap;
	}

	/**
	 * Return the status of the current file generation
	 * 
	 * @return the file generation status
	 */
	public boolean isFileGenerationDone() {
		return this.fileGenerationDone;
	}

	/**
	 * Setter of the voting centers list property
	 * 
	 * @param votingCenterList
	 *            the voting center list to set
	 */
	protected void setVotingCentersList(List<VotingCenter> votingCenterList) {
		this.votingCenterList = votingCenterList;
	}

	/**
	 * Setter of the electoral authorities languages map, the key will be the
	 * authority language and the
	 * 
	 * @param electoralAuthorityLanguages
	 *            the electoral authorities languages map to set
	 */
	protected void setElectoralAuthorityLanguages(
			Map<ElectoralAuthorityData, Set<LanguageData>> electoralAuthorityLanguages) {
		this.electoralAuthorityLanguages = electoralAuthorityLanguages;
	}

	/**
	 * Getter of the voting centers list
	 * 
	 * @return the voting center list
	 */
	protected List<VotingCenter> getVotingCentersList() {
		return this.votingCenterList;
	}

	@Override
	protected void performOperation() {

		fireAwtEvent(AwtOperationType.UPDATE_COMPONENTS);
		try {
			FileVerificationUtils.deleteTemporaryPath();
			startFileGeneration();
		} catch (Exception e) {
			FileVerificationUtils.deleteTemporaryPath();
			logger.error(e, "Error on Master file generation");
			fireAwtEvent(AwtOperationType.SHOW_ERROR_MESSAGE);
		}
	}

	@Override
	protected void onCompleted() {
		fireAwtEvent(AwtOperationType.UPDATE_COMPONENTS);
		logger.info("File Generation Done");
	}

	/**
	 * Starts the process of the master USB file generation for the selected
	 * regions.
	 * 
	 * @throws ElectoralOrganizationOperationException
	 *             signal that an error has occurred retrieving electoral
	 *             information
	 * @throws ConnectionException
	 *             signal that an error has occurred with the server connection
	 * @throws ElectoralPhysicalLocationsRegistryException
	 *             signal that an error has occurred retrieving voting devices
	 *             information
	 * @throws MasterUSBGenerationException
	 *             signal that an error has occurred generating voting devices
	 *             data
	 * @throws UtilitiesException
	 *             signal that an error has occurred performing utilities tasks
	 * @throws FileGenerationException
	 *             signal that an error has occurred writing a file
	 * @throws IOException
	 *             signal that an error has occurred writing a file
	 * @throws CryptographicStaticException
	 *             signal that an error has occurred encrypting the file
	 */
	private void startFileGeneration() throws Exception {

		this.fileGenerationDone = false;

		logger.info("File generation started");

		this.targetPath = getTargetPath();

		this.electoralAuthorityLanguages = new HashMap<ElectoralAuthorityData, Set<LanguageData>>();

		Set<ProgrammingUnitData> programmingUnitSet = this.programmingUnitElectionObjectDto
				.getProgrammingUnitSet();

		Map<Long, ElectionObjectDTO> electionObjectMap = this.programmingUnitElectionObjectDto
				.getElectionObjectMap();

		for (ProgrammingUnitData programmingUnit : programmingUnitSet) {

			Set<FileChecksumData> checksums = null;
			checksums = getLiveBootChecksums();

			logger.info("Generating files for programming unit:"
					+ programmingUnit.getProgrammingUnitName());

			long programmingUnitGenerationTime = System.currentTimeMillis();

			this.votingCenterList = new ArrayList<VotingCenter>();

			ElectionObjectDTO electionObject = electionObjectMap
					.get(programmingUnit.getCode());

			List<RegionData> lastLevelRegionsList = electionObject
					.getLastLevelRegionToProcess();

			this.votingDevicesToProcess = getVotingDevicesTotal(lastLevelRegionsList);

			logger.info("Processing regions");

			ElectionData election = electionObject.getElectionDto()
					.getElection();

			election.setChecksums(checksums);

			generateCertificates(election);
			getRootCertFingerprint(election);

			File certToEncryptKS = writeCertToEncrypt(election);

			processLastLevelRegions(electionObject, lastLevelRegionsList,
					programmingUnit, certToEncryptKS);

			generateVotingCenters(election, this.votingCenterList);

			generateDefaultLanguages(election);

			logger.info("Generation completed for programming unit: "
					+ programmingUnit.getProgrammingUnitName());

			logger.info("Time processing programming unit: "
					+ programmingUnit.getProgrammingUnitName()
					+ " "
					+ TimestampUtil.prettyTimeDifference(
							programmingUnitGenerationTime,
							System.currentTimeMillis()));
		}

		generateInstallationFolder();

		generateSignatureFile();

		copyGeneratedFiles();

		mergeSaesFiles();

		this.fileGenerationDone = true;
	}

	private Set<FileChecksumData> getLiveBootChecksums()
			throws FileNotFoundException, IOException {

		Set<FileChecksumData> checksums = null;
		FileChecksumData checksumData;
		File file = new File(this.mountedPath,
				LIVE_BOOT_SHA256SUM_TXT_FILE_NAME);

		if (file.exists()) {

			checksums = new HashSet<FileChecksumData>();
			checksumData = new FileChecksumData();

			BufferedReader bufferReader = new BufferedReader(new FileReader(
					file));
			try {
				boolean registLine = false;
				String[] values;
				String line, base = null, hash = null, fileName = null;
				while ((line = bufferReader.readLine()) != null) {
					values = line.split(" ");
					if (values.length == 1 && !(values[0].trim()).isEmpty()) {
						base = values[0].trim();
					} else if (values.length == 3) {
						registLine = true;
						hash = values[0].trim();
						fileName = values[2].trim();
					}

					if (registLine) {
						checksumData = new FileChecksumData();
						checksumData.setBase(base);
						checksumData.setFile(fileName);
						checksumData.setHash(hash);
						checksums.add(checksumData);
						registLine = false;
					}
				}
			} finally {
				IOUtils.closeQuietly(bufferReader);
				FileUtils.forceDelete(file);
			}
		}
		return checksums;
	}

	/**
	 * Process the list of last level regions selected for the current master
	 * USB generation.
	 * 
	 * @param electionObject
	 *            the election object which contains data about the generation
	 * @param lastLevelRegions
	 *            the list of regions to process
	 * @param programmingUnit
	 *            the programming unit object
	 * @param certToEncryptKSFile
	 *            the certToEncrypt keystore to sign
	 * @throws ElectoralOrganizationOperationException
	 *             signal that an error has occurred retrieving electoral
	 *             authorities data
	 * @throws ConnectionException
	 *             signal that an error has occurred with the server connection
	 * @throws ElectoralPhysicalLocationsRegistryException
	 *             signal that an error has occurred retrieving voting devices
	 *             information
	 * @throws MasterUSBGenerationException
	 *             signal that an error has occurred generating voting devices
	 *             data
	 * @throws UtilitiesException
	 *             signal that an error has occurred performing utilities tasks
	 * @throws FileGenerationException
	 *             signal that an error has occurred writing a file
	 * @throws CryptographicStaticException
	 *             signal that an error has occurred encrypting the file
	 * @throws com.smartmatic.waffle.ecm.client.exception.ConnectionException
	 * @throws XmlGenerationException
	 */
	protected void processLastLevelRegions(ElectionObjectDTO electionObject,
			List<RegionData> lastLevelRegions,
			ProgrammingUnitData programmingUnit, File certToEncryptKSFile)
			throws ElectoralOrganizationOperationException,
			ConnectionException, ElectoralPhysicalLocationsRegistryException,
			MasterUSBGenerationException, UtilitiesException, IOException,
			FileGenerationException, CryptographicStaticException,
			PendriveException, XmlGenerationException,
			com.smartmatic.waffle.ecm.client.exception.ConnectionException {

		Map<Long, Set<LanguageData>> regionLanguageMap = electionObject
				.getRegionLanguageMap();

		List<File> voiceFiles = null;
		List<ContestGroupData> contestGroupForElection = new ArrayList<ContestGroupData>();
		RegionData regionWithParents = null;

		List<File> generalAudioResources = generateCommonAudioResources(
				electionObject.getElectionDto().getElection(), programmingUnit);

		for (RegionData region : lastLevelRegions) {
			if (!pendriveDisconected()) {
				throw new PendriveException(
						PendriveExceptionInfo.PENDRIVE_NOT_FOUND);
			}

			regionWithParents = ECMServiceProvider.getParentRegions(region);
			if (regionWithParents != null) {
				region.setAllParents(regionWithParents.getAllParents());
			}

			boolean audioVotingEnabled = false;
			if (region.getAllParents() != null) {
				for (RegionData regionParent : region.getAllParents()) {
					audioVotingEnabled = audioVotingEnabled
							|| regionParent.isAudioVotingSetting();

					if (audioVotingEnabled) {
						break;
					}
				}
			}

			logger.info("Generating files for region: "
					+ region.getRegionName());

			Set<LanguageData> regionLanguageSet = regionLanguageMap.get(region
					.getCode());

			setElectoralAuthorityLanguages(region, regionLanguageSet);

			if (audioVotingEnabled) {
				try {
					voiceFiles = downloadVoiceFiles(this.targetPath
							+ File.separator + "SAESLocal" + File.separator
							+ "SAES" + File.separator, regionLanguageSet);
				} catch (Exception e) {
					throw new FileGenerationException(
							FileGenerationExceptionInfo.WRITING_VOICE_FILE_FAILURE);
				}
			}

			processVotingDeviceList(region, regionLanguageSet, electionObject,
					programmingUnit, contestGroupForElection, voiceFiles,
					audioVotingEnabled, certToEncryptKSFile,
					generalAudioResources);

			logger.info("Generation completed for region: "
					+ region.getRegionName());
		}

		preparePartyPreview(electionObject.getElectionDto().getElection());

		generateElection(electionObject.getElectionDto().getElection(),
				programmingUnit);

        this.generateDeviceParamsFile(
            electionObject.getDeviceParameters(),
            programmingUnit,
            electionObject.getElectionDto().getElection()
        );
	}


    /**
     * Generates the "device-parameters" XML File.
     *
     * @param params
     *      persisted device parameters
     * @param progUnit
     *      associated {@code ProgrammingUnit}
     * @param election
     *      associated {@code Election}
     *
     * @throws FileGenerationException
     *      if the file cannot be generated
     */
    private void generateDeviceParamsFile(
        List<DeviceParameterData> params, ProgrammingUnitData progUnit,
        ElectionData election
    ) throws FileGenerationException {
        final String infoMsg = "Generating device-parameters.xml file";

        File xmlFile;         // File being generated
        FileData meta;        // File's meta information
        StringBuilder path;   // Relative path where the file should be created
        XStream xstream;      // XStream

        logger.info(infoMsg);

        path = new StringBuilder()
                       .append("SAES/")
                       .append( progUnit.getCustomCode().replace(" " , "_") )
                       .append("/device-parameters.xml");

        xmlFile = new File(this.targetPath, path.toString());
        xstream = new XStream(new DomDriver("UTF-8"));

        xstream.processAnnotations(DeviceParameterData.class);
        xstream.omitField(BaseDataObject.class, "version");
        xstream.alias("parameters", List.class);

        meta = MasterFileGenerationUtils.writeXML(
            xstream, params, xmlFile, progUnit.getProtectionKey(),
            election.getKeystore()
        );
        writeLastFileProcessed(meta);
    }


	/**
	 * Set the value configured for party preview to the display info ballot
	 * layout.
	 * 
	 * @param election
	 *            The active election.
	 */
	private void preparePartyPreview(ElectionData election) {
		if (election.getBallots() == null) {
			return;
		}
		for (BallotData ballot : election.getBallots()) {
			if (ballot.getBallotCards() == null) {
				return;
			}
			for (BallotCardData ballotCard : ballot.getBallotCards()) {
				if (ballotCard.getContest() == null) {
					return;
				}
				for (ContestData contest : ballotCard.getContest()) {
					if (contest.getBallotLayout() == null) {
						return;
					}
					contest.getBallotLayout().setPartyPreview(
							contest.isPartyPreview());
				}
			}
		}
	}

	/**
	 * Will process the voting devices data for the given region object, calling
	 * the methods to generate their required files.
	 * 
	 * @param region
	 *            the region to search its voting devices list
	 * @param regionLanguageSet
	 *            the set of languages that apply to the region
	 * @param electionObject
	 *            the election object reference
	 * @param programmingUnit
	 *            the programming unit that is being processed
	 * @param contestGroupForElection
	 *            the contest group list
	 * @param voiceFileToSign
	 *            The list with the voice files to be signed with the voting
	 *            device folder
	 * @param audioVotingEnabled
	 *            The audio voting flag
	 * @param certToEncryptKSFile
	 *            the certToEncrypt keystore to be signed with the voting device
	 *            folder
	 * @param generalAudioResources
	 *            The list with the general audio resource files to be signed
	 *            with the voting device folder
	 * @throws ConnectionException
	 *             signal that an error has occurred with the server connection
	 * @throws ElectoralPhysicalLocationsRegistryException
	 *             signal that an error has occurred retrieving voting devices
	 *             information
	 * @throws MasterUSBGenerationException
	 *             signal that an error has occurred generating voting devices
	 *             data
	 * @throws UtilitiesException
	 *             signal that an error has occurred performing utilities tasks
	 * @throws FileGenerationException
	 *             signal that an error has occurred writing a file
	 * @throws CryptographicStaticException
	 *             signal that an error has occurred encrypting the file
	 * @throws PendriveException
	 *             The USB device has been disconnected.
	 */
	protected void processVotingDeviceList(RegionData region,
			Set<LanguageData> regionLanguageSet,
			ElectionObjectDTO electionObject,
			ProgrammingUnitData programmingUnit,
			List<ContestGroupData> contestGroupForElection,
			List<File> voiceFileToSign, boolean audioVotingEnabled,
			File certToEncryptKSFile, List<File> generalAudioResources)
			throws ConnectionException,
			ElectoralPhysicalLocationsRegistryException,
			MasterUSBGenerationException, UtilitiesException,
			FileGenerationException, IOException, CryptographicStaticException,
			PendriveException {

		List<VotingDeviceData> votingDeviceList = ECMServiceProvider
				.getVotingDevicesByRegion(region);

		if (votingDeviceList.size() == 0) {
			logger.warn("The region " + region.getCode() + " "
					+ region.getRegionName()
					+ " has no associated voting device");
		} else {

			List<ReportTemplateData> reportTemplates = ECMServiceProvider
					.getReportsTemplatesByRegion(region.getCode());

			ElectionData activeElection = electionObject.getElectionDto()
					.getElection();

			ElectionData electionParameter = new ElectionData();
			electionParameter.setCustomCode(activeElection.getCustomCode());
			electionParameter.setCode(activeElection.getCode());

			electionParameter.setElectionOpeningDate(activeElection
					.getElectionOpeningDate());
			electionParameter.setElectionClosingDate(activeElection
					.getElectionClosingDate());
			electionParameter.setElectionName(activeElection.getElectionName());

			Map<String, Map<String, String>> resourcesLanguages = electionObject
					.getResourcesLanguages();

			Map<Long, List<ContestGroupData>> regionContestGroup = electionObject
					.getRegionContestGroup();

			List<ContestGroupData> contestGroupList = regionContestGroup
					.get(region.getCode());

			contestGroupForElection.addAll(contestGroupList);

			if (audioVotingEnabled) {
				setAudioResources(regionLanguageSet, electionObject,
						contestGroupList);
			}

			VotingCenter votingCenter = null;
			String pvTemplatesPath = "";

			for (VotingDeviceData votingDevice : votingDeviceList) {

				String randomSalt = ECMServerCommonImpl
						.generateRandomString(32);

				votingDevice.setLanguages(regionLanguageSet);

				VotingDeviceDTO dtoVotingDevice = ECMServiceProvider
						.generateVotingDeviceForRegion(votingDevice, region,
								contestGroupList, resourcesLanguages, false,
								electionParameter, audioVotingEnabled);

				VotingDevicesData votingDevicesData = dtoVotingDevice
						.getVotingDevicesData();

				votingDevicesData
						.setXmlnsEl("http://www.smartmatic.com/Saes/Voting/Election");

				List<LanguageForVotingDeviceData> votingDevicesLanguages = getVotingDevicesLanguages(regionLanguageSet);

				votingDevicesData.setLanguages(votingDevicesLanguages);

				String votingDeviceCustomCode = votingDevice.getCustomCode()
						.replace(" ", "_");

				if (this.generateUsingProgrammingUnits) {

					String programmingUnitCustomCode = programmingUnit
							.getCustomCode().replace(" ", "_");

					pvTemplatesPath = "SAES" + File.separator
							+ programmingUnitCustomCode + File.separator
							+ votingDeviceCustomCode + File.separator + "SAES";

					votingCenter = new VotingCenter(votingDevice.getName(),
							randomSalt, votingDeviceCustomCode,
							programmingUnitCustomCode);

					generatePVTemplates(reportTemplates, pvTemplatesPath);

					generateXML(votingDeviceCustomCode, programmingUnit,
							votingDevicesData, activeElection, randomSalt,
							voiceFileToSign, certToEncryptKSFile,
							generalAudioResources);

				} else {

					pvTemplatesPath = "SAES" + File.separator
							+ votingDeviceCustomCode + File.separator + "SAES";

					votingCenter = new VotingCenter(votingDevice.getName(),
							randomSalt, votingDeviceCustomCode, null);

					generatePVTemplates(reportTemplates, pvTemplatesPath);

					generateXML(votingDeviceCustomCode, null,
							votingDevicesData, activeElection, randomSalt,
							voiceFileToSign, certToEncryptKSFile,
							generalAudioResources);

				}

				this.votingCenterList.add(votingCenter);

				this.votingDevicesProcessed++;

				fireAwtEvent(AwtOperationType.UPDATE_PROGRESS_BAR);

				if (!pendriveDisconected()) {
					throw new PendriveException(
							PendriveExceptionInfo.PENDRIVE_NOT_FOUND);
				}

				logger.info("Generation Completed for voting device: "
						+ votingDevice.getName());
			}
		}
	}

	/**
	 * Generates all the required files for a voting device, including XML,
	 * resources and protection key files.
	 * 
	 * @param votingDeviceCustomCode
	 *            the custom code of the voting device to generate
	 * @param programmingUnit
	 *            the programming unit that is being generated
	 * @param votingDevices
	 *            the voting devices list to generate
	 * @param activeElection
	 *            the active election object
	 * @param randomSalt
	 *            the random salt
	 * @param voiceFileToSign
	 *            The list with the voice files to be signed with the voting
	 * @param audioVotingEnabled
	 *            The flag to set audio voting enable or disabled.
	 * @param certToEncryptKSFile
	 *            the certToEncrypt keystore to be signed with the voting device
	 *            folder
	 * @param generalAudioResources
	 *            The list with the general audio resource files to be signed
	 *            with the voting device folder
	 * @throws FileGenerationException
	 *             signal that an error has occurred writing a file
	 */
	protected void generateXML(String votingDeviceCustomCode,
			ProgrammingUnitData programmingUnit,
			VotingDevicesData votingDevices, ElectionData activeElection,
			String randomSalt, List<File> voiceFileToSign,
			File certToEncryptKSFile, List<File> generalAudioResources)
			throws IOException, FileGenerationException {

		Set<LanguageData> languages = votingDevices.getVotingDeviceData()
				.getLanguages();

		String votingDeviceProtectionKey = votingDevices.getVotingDeviceData()
				.getProtectionKey();

		File rootSAESDirectory = new File(this.targetPath, "SAES");

		File programmingUnitFolder;
		File votingDeviceFolder;
		String programmingUnitProtectionKey = null;

		if (programmingUnit != null) {
			programmingUnitFolder = new File(rootSAESDirectory, programmingUnit
					.getCustomCode().replace(" ", "_"));

			votingDeviceFolder = new File(programmingUnitFolder,
					votingDeviceCustomCode + File.separator + "SAES");

			programmingUnitProtectionKey = programmingUnit.getProtectionKey();
		} else {
			programmingUnitFolder = rootSAESDirectory;
			votingDeviceFolder = new File(rootSAESDirectory,
					votingDeviceCustomCode + File.separator + "SAES");
		}

		File resourcesFolder = new File(votingDeviceFolder, "StringResources");
		resourcesFolder.mkdirs();

		File totalizationKeyFile = new File(votingDeviceFolder, "totKey.crt");

		generateStringResources(resourcesFolder, languages,
				votingDeviceProtectionKey, activeElection);

		generateVotingDevice(votingDevices, votingDeviceFolder,
				votingDeviceProtectionKey, activeElection);

		generatePBFile(votingDeviceFolder, votingDevices,
				programmingUnitProtectionKey, randomSalt);

		for (ElectoralAuthorityData electoralAuthority : this.electoralAuthorityLanguages
				.keySet()) {

			writeTotalizationKeyFile(electoralAuthority, totalizationKeyFile);
		}

		File votingDeviceCertificate = new File(votingDeviceFolder,
				"machineKeys.p12");

		writeVotingDeviceCertificate(votingDevices.getVotingDeviceData()
				.getAsymmetricKeys(), votingDeviceProtectionKey,
				votingDeviceCertificate);

		List<File> partyImagesFiles = writePartyImages(activeElection
				.getPartyList());

		generateVotingDeviceSignatureFromCache(votingDeviceFolder,
				partyImagesFiles, voiceFileToSign, votingDeviceProtectionKey,
				activeElection, certToEncryptKSFile, generalAudioResources,
				programmingUnitFolder);

	}

	/**
	 * Set the audio resources from the map of eligible options by contest to
	 * each language given as parameter.
	 * 
	 * NOTE: No translations from eligible options are being used now for the
	 * audio resources.
	 * 
	 * @param languages
	 *            the set of language to set its audio resources
	 * @param electionObject
	 *            the election object to fetch eligible options texts.
	 * @param contestGroupList
	 *            the contest group to iterate over.
	 */
	private void setAudioResources(Set<LanguageData> languages,
			ElectionObjectDTO electionObject,
			List<ContestGroupData> contestGroupList) {

		Map<Long, Set<StringResourceData>> contestResources = electionObject
				.getElectionDto().getContestEligibleOptionResources();

		Set<Long> processedContests = new HashSet<Long>();
		for (ContestGroupData contestGroup : contestGroupList) {
			for (ContestData contest : contestGroup.getContests()) {
				if (!processedContests.contains(contest.getCode())) {
					Map<String, StringResourceData> resources = new HashMap<String, StringResourceData>();

					for (StringResourceData contestResource : contestResources
							.get(contest.getCode())) {
						resources
								.put(contestResource.getKey(), contestResource);
					}

					for (LanguageData language : languages) {
						if (language.getAudioStringResources() == null) {
							language.setAudioStringResources(new HashSet<StringResourceData>(
									resources.values()));
						} else {
							language.getAudioStringResources().addAll(
									resources.values());
						}
					}
				}
			}
		}
	}

	/**
	 * Method that return the relative path of a file given a base path
	 * 
	 * @param file
	 *            the file to get the relative path
	 * @param basePath
	 *            the base path
	 * @return the relative path
	 */
	private String getRelativePath(File file, String basePath) {

		String path = new File(basePath).toURI().relativize(file.toURI())
				.getPath();
		return path;
	}

	/**
	 * Check if a file with the same name was processed previously to remove it
	 * and add the last file processed.
	 * 
	 * @param file
	 *            The file data processed.
	 * @throws FileGenerationException
	 * @throws IOException
	 */
	private void writeLastFileProcessed(FileData fileData)
			throws FileGenerationException {

		if (fileData == null || fileData.getFile() == null
				|| fileData.getData() == null) {
			logger.warn("It was not possible to persist file for verification process");
		} else {
			byte[] hash = FileVerificationUtils.verifyDiskFile(fileData);
			String relativePath = getRelativePath(fileData.getFile(),
					this.targetPath);
			FileBackup backup = new FileBackup();
			backup.setFile(fileData.getFile());
			backup.setHash(hash);
			this.writtenFilesMap.put(relativePath, backup);
		}
	}

	/**
	 * Method that generates the signature file for the files included in the
	 * SAES folder of the master USB.
	 * 
	 * @throws ConnectionException
	 *             an error occurred with the server connection
	 * @throws ElectoralOrganizationOperationException
	 *             an error occurred retrieving the active election
	 */
	private void generateSignatureFile() throws ConnectionException,
			ElectoralOrganizationOperationException {

		ElectionData activeElection = ECMServiceProvider
				.retrieveActiveElection();

		List<File> fileListToSign = new ArrayList<File>();

		int level = 1;

		MasterFileGenerationUtils.getFilesToSign(this.targetPath + "/SAES",
				fileListToSign, level);

		Map<String, List<File>> filesByUriBase = new HashMap<String, List<File>>();

		filesByUriBase.put(this.targetPath + "/SAES/", fileListToSign);

		String signatureAlgorithm = XMLSignature.ALGO_ID_SIGNATURE_ECDSA_SHA256;

		File signatureXMLFile = new File(this.targetPath + File.separator
				+ "SAES", "Signature.xml");

		boolean encrypt = ECMClientSideConstants.GENERATE_XML_ENCRYPTED;

		String xml = "";

		try {
			xml = SignXMLUtil.signDetachedXML(activeElection.getKeystore()
					.getPkcs12().getBinaryData(), activeElection.getKeystore()
					.getAlias(), activeElection.getKeystore().getPassword(),
					filesByUriBase, signatureAlgorithm);

			FileData file = MasterFileGenerationUtils.writeSignatureXML(xml,
					signatureXMLFile, encrypt, this.applicationKey);

			writeLastFileProcessed(file);

		} catch (Exception e) {
			logger.error(e, StringUtils.defaultString(e.getMessage()));
		}

	}

	/**
	 * Generates the required data to write the default languages file.
	 * 
	 * @param election
	 *            the election which contains the languages and keystore data
	 * @throws FileGenerationException
	 *             signal that an error has occurred writing a file
	 */
	private void generateDefaultLanguages(ElectionData election)
			throws FileGenerationException {

		logger.info("Stating preparation to generate the default languages file");

		String xmlns = "http://www.smartmatic.com/Saes/Voting/Election";

		String xmlnsXs = "http://www.w3.org/2001/XMLSchema";

		XStream xstreamLanguages = new XStream(new DomDriver("UTF-8"));

		xstreamLanguages.omitField(BaseDataObject.class, "version");

		xstreamLanguages.processAnnotations(LanguageCollectionData.class);

		xstreamLanguages.setMode(XStream.NO_REFERENCES);

		List<LanguageData> languageList = election.getLanguages();

		Collections.sort(languageList, new SortedLanguage());

		LanguageCollectionData languageCollection = new LanguageCollectionData(
				xmlnsXs, xmlns, languageList);

		File rootSAESDirectory = new File(this.targetPath, "SAES");

		File defaultLanguagesFile = new File(rootSAESDirectory,
				"default-languages.xml");

		FileData file = MasterFileGenerationUtils.writeXML(xstreamLanguages,
				languageCollection, defaultLanguagesFile, this.applicationKey,
				election.getKeystore());

		writeLastFileProcessed(file);
	}

	/**
	 * Generate a secure keystore file (.p12) containing the certToEncript.
	 * 
	 * @param activeElection
	 *            The election instance containing the certToEncrypt
	 *            certificate.
	 * @return The generated .p12 keystore file.
	 */
	private File writeCertToEncrypt(ElectionData activeElection) {
		logger.info("Generating certificates");

		String path = this.targetPath + "/SAES/";
		File saesFolder = new File(path);
		File certToEncryptKSFile = null;

		try {
			activeElection.setCertToEncryptPassword(ECMServerCommonImpl
					.generateRandomString(32));

			FileData certToEncryptFile = MasterFileGenerationUtils
					.writeCertificateToEncrypt(saesFolder,
							activeElection.getPublickey());

			FileData certToEncryptKeyStore = MasterFileGenerationUtils
					.generateKeyStoreWithCertToEncrypt(path, certToEncryptFile,
							activeElection.getCertToEncryptPassword());

			certToEncryptKSFile = certToEncryptKeyStore.getFile();

			writeLastFileProcessed(certToEncryptKeyStore);

		} catch (Exception e) {
			logger.error(e, StringUtils.defaultString(e.getMessage()));
		}

		return certToEncryptKSFile;
	}

	/**
	 * Generates the required data to write the voting devices certificates.
	 * 
	 * @param election
	 *            the election to retrieve the security information
	 */
	private void generateCertificates(ElectionData election) {

		logger.info("Generating certificates");

		KeystoreData keystore = election.getKeystore();

		try {
			List<FileData> certificates = MasterFileGenerationUtils
					.writeCertificates(keystore.getPkcs12().getBinaryData(),
							keystore.getAlias(), keystore.getPassword(),
							this.targetPath);

			for (FileData file : certificates) {
				writeLastFileProcessed(file);
			}

		} catch (Exception e) {
			logger.error(e, StringUtils.defaultString(e.getMessage()));
		}
	}

	/**
	 * Calculate the fingerprint of root-ca.crt and set the value into the
	 * election object
	 * 
	 * @param election
	 *            the election which contains the data.
	 */
	private void getRootCertFingerprint(ElectionData election) {
		try {
			File certFile = new File(this.targetPath, "root-ca.crt");
			if (certFile.exists()) {
				election.setRootFp(MasterFileGenerationUtils
						.getFingerprint(certFile));
			}
		} catch (Exception e) {
			logger.error(e, StringUtils.defaultString(e.getMessage()));
		}
	}

	/**
	 * Generates the required data to write into the voting centers file.
	 * 
	 * @param election
	 *            the election which contains the keystore data
	 * @param votingCenterList
	 *            the voting centers list to write
	 * @throws FileGenerationException
	 *             signal that an error has occurred writing a file
	 */
	private void generateVotingCenters(ElectionData election,
			List<VotingCenter> votingCenterList) throws FileGenerationException {

		logger.info("Stating preparation to generate the voting center file");

		String xmlns = "http://www.smartmatic.com/Saes/Voting/VotingCenters";

		String xmlnsXs = "http://www.w3.org/2001/XMLSchema";

		VotingCentersData votingCenters = new VotingCentersData(xmlnsXs, xmlns,
				votingCenterList);

		XStream xstreamVotingCenter = new XStream(new DomDriver("UTF-8"));

		xstreamVotingCenter.processAnnotations(VotingCentersData.class);

		xstreamVotingCenter.setMode(XStream.NO_REFERENCES);

		File rootSAESDirectory = new File(this.targetPath, "SAES");

		File votingCenterFile = new File(rootSAESDirectory,
				"voting-centers.xml");

		FileData file = MasterFileGenerationUtils.writeXML(xstreamVotingCenter,
				votingCenters, votingCenterFile, this.applicationKey,
				election.getKeystore());

		writeLastFileProcessed(file);
	}

	/**
	 * Generates the required data to write into the election XML file.
	 * 
	 * @param election
	 *            the election which contains the data
	 * @param programmingUnit
	 *            the programming unit that is being generated
	 * @throws FileGenerationException
	 *             signal that an error has occurred writing a file
	 */
	private void generateElection(ElectionData election,
			ProgrammingUnitData programmingUnit) throws FileGenerationException {

		logger.info("Stating preparation to generate the election file");

		XStream xstreamElection = new XStream(new DomDriver("UTF-8"));

		xstreamElection.processAnnotations(DistrictData.RegionCodeData.class);

		xstreamElection.processAnnotations(PartyData.class);

		xstreamElection.omitField(BaseDataObject.class, "version");

		xstreamElection.setMode(XStream.NO_REFERENCES);

		xstreamElection.addDefaultImplementation(java.util.ArrayList.class,
				org.hibernate.collection.PersistentList.class);
		xstreamElection.addDefaultImplementation(java.util.HashMap.class,
				org.hibernate.collection.PersistentMap.class);
		xstreamElection.addDefaultImplementation(java.util.HashSet.class,
				org.hibernate.collection.PersistentSet.class);

		Mapper mapperElection = xstreamElection.getMapper();

		xstreamElection.registerConverter(new HibernateCollectionConverter(
				mapperElection));

		File rootSAESDirectory = new File(this.targetPath, "SAES");

		File programmingUnitFolder = new File(rootSAESDirectory,
				programmingUnit.getCustomCode().replace(" ", "_"));

		String programmingUnitProtectionKey = programmingUnit
				.getProtectionKey();

		File electionXMLFile = new File(programmingUnitFolder, "election.xml");

		FileData file = MasterFileGenerationUtils.writeXML(xstreamElection,
				election, electionXMLFile, programmingUnitProtectionKey,
				election.getKeystore());

		writeLastFileProcessed(file);
	}

	/**
	 * Generates the common audio resources XML files, which are loaded as
	 * resources in the bulk load step.
	 * 
	 * @param election
	 *            the current election object reference.
	 * @param programmingUnit
	 *            the programming unit object reference
	 * @return A list with all the general audio resouces.
	 * 
	 * @throws FileGenerationException
	 *             signal that an error has occurred writing the files
	 * @throws UtilitiesException
	 *             if an error occurs getting the resources list from the
	 *             database
	 * @throws ConnectionException
	 *             if an error occurs during the get resources server request
	 */
	private List<File> generateCommonAudioResources(ElectionData election,
			ProgrammingUnitData programmingUnit)
			throws FileGenerationException, UtilitiesException,
			ConnectionException {

		logger.info("Starting common audio resources generation");

		List<File> generalAudioResources = new ArrayList<File>();

		File rootSAESDirectory = new File(this.targetPath, "SAES");

		File programmingUnitFolder = new File(rootSAESDirectory,
				programmingUnit.getCustomCode().replace(" ", "_")
						+ "/StringResources");

		String programmingUnitProtectionKey = programmingUnit
				.getProtectionKey();

		List<ReportTemplateData> audioResources = ECMServiceProvider
				.getCommonAudioResources();

		for (ReportTemplateData audioResource : audioResources) {
			File audioResourceXML = new File(programmingUnitFolder,
					audioResource.getExportedName());

			String dataToWrite = new String(audioResource.getJrxml()
					.getBinaryData());

			FileData file = MasterFileGenerationUtils.writeXML(null,
					dataToWrite, audioResourceXML,
					programmingUnitProtectionKey, election.getKeystore());

			generalAudioResources.add(file.getFile());
			writeLastFileProcessed(file);
		}

		logger.info("Common audio resources generation finished successfuly");

		return generalAudioResources;
	}

	/**
	 * Generates the data for the voting device signature file.
	 * 
	 * @param votingDeviceFolder
	 *            the voting device folder which contains the voting device
	 *            files
	 * @param partyImagesFiles
	 *            the party images folder
	 * @param voiceFileToSign
	 *            The list with the voice files to be signed with the voting
	 * @param votingDeviceProtectionKey
	 *            the voting device protection key to encrypt
	 * @param activeElection
	 *            the current election data object
	 * @param certToEncryptKSFile
	 *            the certToEncrypt keystore to be signed with the voting device
	 *            folder
	 * @param generalAudioResources
	 *            The list with the general audio resource files to be signed
	 *            with the voting device folder
	 * @param programmingUnitFolder
	 *            the voting device folder which contains all the files
	 */
	private void generateVotingDeviceSignatureFromCache(
			File votingDeviceFolder, List<File> partyImagesFiles,
			List<File> voiceFileToSign, String votingDeviceProtectionKey,
			ElectionData activeElection, File certToEncryptKSFile,
			List<File> generalAudioResources, File programmingUnitFolder) {

		boolean encrypt = ECMClientSideConstants.GENERATE_XML_ENCRYPTED;

		byte[] electionPkcs12 = activeElection.getKeystore().getPkcs12()
				.getBinaryData();

		String alias = activeElection.getKeystore().getAlias();

		String passwordToSign = activeElection.getKeystore().getPassword();

		String imagesFolder = this.targetPath
				+ ECMClientSideConstants.PATH_PARTIES_IMAGES_FOLDER;

		File imagePartyDirectory = new File(imagesFolder);

		String xml = null;

		Map<String, List<Map.Entry<File, byte[]>>> filesByUriBase = new HashMap<String, List<Map.Entry<File, byte[]>>>();

		filesByUriBase.put(
				imagePartyDirectory.getParent() + File.separator,
				getFileListWithHash(imagePartyDirectory.getParent()
						+ File.separator, partyImagesFiles));

		if (voiceFileToSign != null) {
			String voiceFolder = this.targetPath
					+ ECMClientSideConstants.PATH_VOICES_FOLDER;

			File voiceDirectory = new File(voiceFolder);

			List<Map.Entry<File, byte[]>> filesToSign = filesByUriBase
					.get(voiceDirectory.getParent() + File.separator);

			if (filesToSign == null) {
				filesToSign = new ArrayList<Map.Entry<File, byte[]>>();
			}
			filesToSign.addAll(getFileListWithHash(voiceDirectory.getParent()
					+ File.separator, voiceFileToSign));

			filesByUriBase.put(voiceDirectory.getParent() + File.separator,
					filesToSign);
		}

		filesByUriBase.put(
				this.targetPath + File.separator + "SAES" + File.separator,
				getFileListWithHash(this.targetPath + File.separator + "SAES"
						+ File.separator, Arrays.asList(certToEncryptKSFile)));

		filesByUriBase.put(
				programmingUnitFolder + File.separator,
				getFileListWithHash(programmingUnitFolder + File.separator
						+ "StringResources", generalAudioResources));

		filesByUriBase.put(
				votingDeviceFolder + File.separator,
				getFileListWithHash(votingDeviceFolder + File.separator,
						MasterFileGenerationUtils
								.getFilesFromDirectory(votingDeviceFolder)));

		String signatureAlgorithm = XMLSignature.ALGO_ID_SIGNATURE_ECDSA_SHA256;
		File signatureXMLFile = new File(votingDeviceFolder, "Signature.xml");

		try {
			xml = SignXMLUtil.signDetachedXMLFromCache(electionPkcs12, alias,
					passwordToSign, filesByUriBase, signatureAlgorithm);

			FileData file = MasterFileGenerationUtils.writeSignatureXML(xml,
					signatureXMLFile, encrypt, votingDeviceProtectionKey);

			writeLastFileProcessed(file);
		} catch (Exception e) {
			logger.error(e, StringUtils.defaultString(e.getMessage()));
		}

	}

	/**
	 * Method to get the files with the respective hash code.
	 * 
	 * @param path
	 *            The path to the given files.
	 * @param filesToSign
	 *            The files to sign.
	 * @return Return a list of files with the respective hash code.
	 */
	private List<Map.Entry<File, byte[]>> getFileListWithHash(String path,
			List<File> filesToSign) {

		List<Map.Entry<File, byte[]>> files = new ArrayList<Map.Entry<File, byte[]>>();
		Map.Entry<File, byte[]> fileHash = null;
		for (File file : filesToSign) {

			FileBackup fileBackup = this.writtenFilesMap
					.get(file
							.getAbsolutePath()
							.substring(
									ECMClientSideConstants.TEMPORARY_PATH_FOR_FILE_GENERATION
											.length() + 1,
									file.getAbsolutePath().length()));

			fileHash = new AbstractMap.SimpleEntry<File, byte[]>(file, null);

			if (fileBackup != null) {
				fileHash.setValue(fileBackup.getHash());
			}

			files.add(fileHash);

		}

		return files;
	}

	/**
	 * Generates the data for the PB(password-based) files.
	 * 
	 * @param votingDeviceFolder
	 *            the voting device folder to store the generated files
	 * @param votingDevices
	 *            the voting devices data
	 * @param programmingUnitProtectionKey
	 *            the programming unit protection key
	 * @param randomSalt
	 *            the random salt
	 * @throws FileGenerationException
	 *             signal that an error has occurred writing a file
	 */
	private void generatePBFile(File votingDeviceFolder,
			VotingDevicesData votingDevices,
			String programmingUnitProtectionKey, String randomSalt)
			throws FileGenerationException {

		StringBuffer saltSeed = new StringBuffer(votingDevices
				.getVotingDeviceData().getName().toLowerCase());
		saltSeed.append("-");
		saltSeed.append(randomSalt);

		String votingDeviceProtectionKey = votingDevices.getVotingDeviceData()
				.getProtectionKey();

		String presidentKey = votingDevices.getVotingDeviceData()
				.getPresidentKey();

		String recounterKey = votingDevices.getVotingDeviceData()
				.getRecounterKey();

		File pbSecretKeyFile = new File(votingDeviceFolder.getParentFile(),
				"secret-key");

		FileData secretKeyFileData = MasterFileGenerationUtils
				.writeCipheredFile(pbSecretKeyFile, presidentKey,
						votingDeviceProtectionKey, saltSeed.toString(), true);

		writeLastFileProcessed(secretKeyFileData);

		File pbRecounterKeyFile = new File(votingDeviceFolder.getParentFile(),
				"recount-key");

		FileData recountKeyFileData = MasterFileGenerationUtils
				.writeCipheredFile(pbRecounterKeyFile, recounterKey,
						votingDeviceProtectionKey, saltSeed.toString(), true);

		writeLastFileProcessed(recountKeyFileData);

		if (this.generateUsingProgrammingUnits) {

			File pbUPKeyFile = new File(votingDeviceFolder.getParentFile(),
					"up-key");

			FileData upKeyFileData = MasterFileGenerationUtils
					.writeCipheredFile(pbUPKeyFile, presidentKey,
							programmingUnitProtectionKey, saltSeed.toString(),
							true);

			writeLastFileProcessed(upKeyFileData);

			File pbUPRecounterKeyFile = new File(
					votingDeviceFolder.getParentFile(), "up-recount-key");

			FileData upRecountKeyFileData = MasterFileGenerationUtils
					.writeCipheredFile(pbUPRecounterKeyFile, recounterKey,
							programmingUnitProtectionKey, saltSeed.toString(),
							true);

			writeLastFileProcessed(upRecountKeyFileData);
		}

	}

	/**
	 * Writes into the parties folder the set of required party images.
	 * 
	 * @param parties
	 *            the set of parties to write
	 * @return the list of files that were written
	 * @throws IOException
	 *             if an error has occurred writing a party image
	 * @throws FileGenerationException
	 */
	private List<File> writePartyImages(Set<PartyData> parties)
			throws IOException, FileGenerationException {

		List<File> imagesFiles = new ArrayList<File>();

		String imagesFolder = this.targetPath
				+ ECMClientSideConstants.PATH_PARTIES_IMAGES_FOLDER;

		File imagePartyDirectory = new File(imagesFolder);

		for (PartyData party : parties) {

			if (party.getImage() != null
					&& party.getImage().getBinaryData() != null) {

				File image = new File(imagePartyDirectory,
						party.getImageFileName());
				FileUtils.writeByteArrayToFile(image, party.getImage()
						.getBinaryData());

				imagesFiles.add(image);

				FileData fileData = new FileData();
				fileData.setFile(image);
				fileData.setData(party.getImage().getBinaryData());

				writeLastFileProcessed(fileData);

			}
		}

		return imagesFiles;
	}

	/**
	 * Writes the certificate of the election for the specified voting device
	 * 
	 * @param asymmetricKey
	 *            the certificate to write
	 * @param password
	 *            the password
	 * @param votingDeviceCertificate
	 *            the file which will contain the certificate
	 * @throws IOException
	 *             if an error has occurred writing the certificate
	 */
	private void writeVotingDeviceCertificate(ImageWrapper asymmetricKey,
			String password, File votingDeviceCertificate) throws IOException {

		final byte[] binaryData = asymmetricKey.getBinaryData();

		InputStream input = new ByteArrayInputStream(binaryData);

		try {
			KeyStore testKeyStore = null;

			testKeyStore = KeyStore
					.getInstance(AsymmetricKeyManager.KEYSTORE_PKCS12);

			testKeyStore.load(input, password.toCharArray());

			FileData file = new FileData();
			file.setFile(votingDeviceCertificate);
			file.setData(binaryData);

			FileUtils.writeByteArrayToFile(votingDeviceCertificate, binaryData);

			writeLastFileProcessed(file);

		} catch (Exception e) {
			logger.error(e, StringUtils.defaultString(e.getMessage()));
		} finally {
			input.close();
		}
	}

	/**
	 * Writes the totalization key used by the totalization device
	 * 
	 * @param electoralAuthority
	 *            the electoral authority to generate its key
	 * @param totalizationKeyFile
	 *            the totalization key file path to write
	 * @throws IOException
	 *             if an error has occurred writing the certificate
	 */
	private void writeTotalizationKeyFile(
			ElectoralAuthorityData electoralAuthority, File totalizationKeyFile)
			throws IOException {

		if (ECMClientSideConstants.GENERATE_TOTALIZATION_DEVICE) {

			final InputStream input = new ByteArrayInputStream(
					electoralAuthority.getAsymmetricKeys().getBinaryData());
			try {
				KeyStore keyStore = null;

				keyStore = KeyStore
						.getInstance(AsymmetricKeyManager.KEYSTORE_PKCS12);
				keyStore.load(input, electoralAuthority.getProtectionKey()
						.toCharArray());

				final Certificate[] certificateChain = keyStore
						.getCertificateChain(electoralAuthority.getAlias());

				final Certificate[] intermediateCAs = Arrays.copyOfRange(
						certificateChain, 0, certificateChain.length);

				FileData file = MasterFileGenerationUtils.writeCertificateFile(
						intermediateCAs, totalizationKeyFile);

				writeLastFileProcessed(file);

			} catch (Exception e) {
				logger.error(e, StringUtils.defaultString(e.getMessage()));
			} finally {
				input.close();
			}
		}
	}

	/**
	 * Generate the data required to write voting device XML file.
	 * 
	 * @param votingDevices
	 *            the voting devices data to write
	 * @param votingDeviceFolder
	 *            the folder of the voting device
	 * @param votingDeviceProtectionKey
	 *            the protection key used to cipher the file if required
	 * @param activeElection
	 *            the election which contains the security information
	 * @throws FileGenerationException
	 *             signal that an error has occurred writing a file
	 */
	private void generateVotingDevice(VotingDevicesData votingDevices,
			File votingDeviceFolder, String votingDeviceProtectionKey,
			ElectionData activeElection) throws FileGenerationException {

		logger.info("Stating preparation to generate the voting devices file");

		XStream xstreamVotingDevice = new XStream(new DomDriver("UTF-8"));

		xstreamVotingDevice.processAnnotations(VotingDevicesData.class);
		xstreamVotingDevice.processAnnotations(VotingBoothData.class);
		xstreamVotingDevice.processAnnotations(PollingPlaceData.class);

		xstreamVotingDevice.omitField(BaseDataObject.class, "version");

		xstreamVotingDevice.setMode(XStream.NO_REFERENCES);

		xstreamVotingDevice.addDefaultImplementation(java.util.ArrayList.class,
				org.hibernate.collection.PersistentList.class);
		xstreamVotingDevice.addDefaultImplementation(java.util.HashMap.class,
				org.hibernate.collection.PersistentMap.class);
		xstreamVotingDevice.addDefaultImplementation(java.util.HashSet.class,
				org.hibernate.collection.PersistentSet.class);

		Mapper mapperVotingDevice = xstreamVotingDevice.getMapper();

		xstreamVotingDevice.registerConverter(new HibernateCollectionConverter(
				mapperVotingDevice));

		File xmlFile = new File(votingDeviceFolder, "voting-devices.xml");

		FileData file = MasterFileGenerationUtils.writeXML(xstreamVotingDevice,
				votingDevices, xmlFile, votingDeviceProtectionKey,
				activeElection.getKeystore());

		writeLastFileProcessed(file);

	}

	/**
	 * Generates the data required to write the string resources file
	 * 
	 * @param resourcesFolder
	 *            the folder where the resources files should be stored
	 * @param languages
	 *            the languages to generate
	 * @param votingDeviceProtectionKey
	 *            the protection key to cipher the files if required
	 * @param activeElection
	 *            the election data which contains the election keystore
	 *            information
	 * @throws FileGenerationException
	 *             signal that an error has occurred writing a file
	 */
	private void generateStringResources(File resourcesFolder,
			Set<LanguageData> languages, String votingDeviceProtectionKey,
			ElectionData activeElection) throws FileGenerationException {

		logger.info("Stating preparation to generate the election resources file");

		XStream xstreamStringResources = new XStream(new DomDriver("UTF-8"));

		xstreamStringResources.processAnnotations(StringResourcesData.class);

		xstreamStringResources.setMode(XStream.NO_REFERENCES);

		for (LanguageData language : languages) {

			List<StringResourceData> list = new ArrayList<StringResourceData>();

			list.addAll(language.getStringResources().values());
			// add the district string resources (from the election level) to
			// the current string resources
			list.addAll(extractDistrictStringResourcesFromElectionDataByLanguage(
					activeElection, language.getCode()));

			StringResourcesData stringResources = new StringResourcesData();

			stringResources
					.setXmlns("http://www.smartmatic.com/Saes/Resources");

			stringResources.setXmlnsXs("http://www.w3.org/2001/XMLSchema");

			stringResources.setStringResources(list);

			File xmlFile = new File(resourcesFolder, "election-resources"
					+ language.getCustomCode() + ".xml");

			FileData file = MasterFileGenerationUtils.writeXML(
					xstreamStringResources, stringResources, xmlFile,
					votingDeviceProtectionKey, activeElection.getKeystore());

			writeLastFileProcessed(file);
		}

	}

	/**
	 * Helper method used to locate and extract the {@link StringResourceData
	 * district string resources} (for a particular language) from an
	 * {@link ElectionData election} object
	 * 
	 * @param electionData
	 *            the election object
	 * @param languageCode
	 *            the language's code used to filter the string resources
	 * @return a list with the district's string resources for the indicated
	 *         language
	 */
	private List<StringResourceData> extractDistrictStringResourcesFromElectionDataByLanguage(
			ElectionData electionData, long languageCode) {
		List<StringResourceData> result = new ArrayList<StringResourceData>();
		String districtKeyPrefix = DistrictData.class.getName();
		logger.debug("Locating string resources with key: " + districtKeyPrefix);
		for (LanguageData language : electionData.getLanguages()) {
			if (language.getCode() == languageCode) {
				for (Map.Entry<String, StringResourceData> entry : language
						.getStringResources().entrySet()) {
					if (entry.getKey().toLowerCase()
							.contains(districtKeyPrefix.toLowerCase())) {
						logger.debug("Extracting resource {Key: "
								+ entry.getValue().getKey() + ", Value: "
								+ entry.getValue().getValue() + "}");
						result.add(entry.getValue());
					}
				}
			}
		}
		return result;
	}

	/**
	 * Writes a set of PV templates into the given templates path
	 * 
	 * @param reportsTemplates
	 *            the set of templates to write
	 * @param pvTemplatesPath
	 *            the templates folder path where the templates should be
	 *            written
	 */
	private void generatePVTemplates(List<ReportTemplateData> reportsTemplates,
			String pvTemplatesPath) {

		if (ECMClientSideConstants.AVAILABLE_DOWNLOADING_PV_TEMPLATES
				&& pvTemplatesPath != null) {

			try {
				File pvTemplatesDir = new File(this.targetPath, pvTemplatesPath);

				pvTemplatesDir.mkdirs();

				for (ReportTemplateData current : reportsTemplates) {

					if (current.getJrxml().getBinaryData() != null) {
						File template = new File(pvTemplatesDir,
								current.getExportedName());

						FileUtils.writeByteArrayToFile(template, current
								.getJrxml().getBinaryData());

						FileData file = new FileData();

						file.setFile(template);

						file.setData(FileUtils.readFileToByteArray(template));

						writeLastFileProcessed(file);
					}
				}
			} catch (IOException e) {
				logger.error(e, StringUtils.defaultString(e.getMessage()));
				fireAwtEvent(AwtOperationType.SHOW_ERROR_MESSAGE);
			} catch (FileGenerationException e) {
				logger.error(e, "Error on file verification");
				fireAwtEvent(AwtOperationType.SHOW_ERROR_MESSAGE);
			}
		}

	}

	/**
	 * Set the languages for an electoral authority given a region
	 * 
	 * @param region
	 *            the region to search its electoral authorities
	 * @param languagesSet
	 *            the set of languages to set
	 * @throws ElectoralOrganizationOperationException
	 *             if an error occurred retrieving the electoral authorities for
	 *             a region
	 * @throws ConnectionException
	 *             if a server connection error occurred
	 */
	private void setElectoralAuthorityLanguages(RegionData region,
			Set<LanguageData> languagesSet)
			throws ElectoralOrganizationOperationException, ConnectionException {

		Map<ElectoralAuthorityData, Collection<ElectoralAuthorityData>> electoralAuthorityMap = ECMServiceProvider
				.getElectoralAuthoritiesByLastLevelRegion(region);

		Set<ElectoralAuthorityData> electoralAuthorityKeySet = electoralAuthorityMap
				.keySet();

		ElectoralAuthorityData electoralAuthorityKey = electoralAuthorityKeySet
				.iterator().next();

		if (electoralAuthorityKey == null) {
			logger.warn("The region " + region.getCode() + " "
					+ region.getRegionName()
					+ " has no associated Electoral Authority in any level");
		} else {
			Collection<ElectoralAuthorityData> electoralAuthorityCollection = electoralAuthorityMap
					.get(electoralAuthorityKey);

			for (ElectoralAuthorityData electoralAuthority : electoralAuthorityCollection) {

				this.electoralAuthorityLanguages.put(electoralAuthority,
						languagesSet);
			}
		}

	}

	/**
	 * Returns a list of language for voting device object required when
	 * generating the files for voting machines given a list of languages
	 * 
	 * @param languages
	 *            a list of languages
	 * @return the list of voting devices languages.
	 */
	private List<LanguageForVotingDeviceData> getVotingDevicesLanguages(
			Set<LanguageData> languages) {

		List<LanguageForVotingDeviceData> languageForVotingDeviceList = new ArrayList<LanguageForVotingDeviceData>();

		for (LanguageData language : languages) {

			LanguageForVotingDeviceData languageForVotingDevice = new LanguageForVotingDeviceData();

			languageForVotingDevice.setCode(language.getCode());
			languageForVotingDevice.setCustomCode(language.getCustomCode());
			languageForVotingDevice.setOfficialLanguage(language
					.getOfficialLanguage());
			languageForVotingDevice.setAudio(language.isAudio());
			languageForVotingDevice.setLanguageName(language.getLanguageName());
			languageForVotingDevice.setLanguageRFC(language.getLanguageRFC());
			languageForVotingDevice.setCountry(language.getCountry());
			languageForVotingDevice.setKey(' ');
			languageForVotingDevice.setMessage(language.getMessage());
			languageForVotingDevice.setPrecedence(language.getPrecedence());

			languageForVotingDeviceList.add(languageForVotingDevice);
		}

		if (languageForVotingDeviceList.size() > 1) {
			Collections.sort(languageForVotingDeviceList,
					new SortedLanguageForVotingDevice());
		}
		return languageForVotingDeviceList;
	}

	/**
	 * Generates the installation folder and its required files, if the
	 * generation of totalization device flag is active will generate the
	 * electoral authorities files as well.
	 */
	private void generateInstallationFolder() {

		String totalizationFilesPath = ECMClientSideConstants.TEMPORARY_PATH_FOR_FILE_GENERATION
				+ File.separator
				+ ECMClientSideConstants.INSTALLATION_FOLDER_PATH
				+ "/INSTALLATION_FOLDER";

		try {

			if (ECMClientSideConstants.GENERATE_TOTALIZATION_DEVICE) {

				String masterSymmetricKey = new String(
						Base64.encodeBase64(SymmetricKeyManager
								.generateSymmetricKey().getEncoded(), false));

				FileData file = MasterFileGenerationUtils.writeDatabaseDump(
						totalizationFilesPath, masterSymmetricKey);

				writeLastFileProcessed(file);

				generateElectoralAuthoritiesFiles(totalizationFilesPath,
						masterSymmetricKey);

			}

			downloadEMLFiles(totalizationFilesPath + File.separator
					+ ECMClientSideConstants.EML_FILES_DIR);

			writeFirstbootFolderInDrive();

		} catch (Exception e) {
			logger.error(e, StringUtils.defaultString(e.getMessage()));
		}

	}

	/**
	 * Generates the electoral authorities files required by the totalization
	 * device
	 * 
	 * @param totalizationFilesPath
	 *            the path of the totalization files
	 * @param masterSymmetricKey
	 *            the symmetric key to encrypt the files
	 * @throws ConnectionException
	 *             if a server connection error occurred getting electoral
	 *             information
	 * @throws ElectoralOrganizationOperationException
	 *             if a error occurred getting electoral information
	 * @throws FileGenerationException
	 *             signal that an error has occurred writing a file
	 */
	private void generateElectoralAuthoritiesFiles(
			String totalizationFilesPath, String masterSymmetricKey)
			throws ConnectionException,
			ElectoralOrganizationOperationException, FileGenerationException {

		for (ElectoralAuthorityData electoralAuthority : this.electoralAuthorityLanguages
				.keySet()) {

			logger.info("Generating files for electoral authority: "
					+ electoralAuthority.getName());

			String electoralAuthorityPath = totalizationFilesPath
					+ File.separator + "EA_" + electoralAuthority.getCode();

			File authorityFolder = new File(electoralAuthorityPath);

			authorityFolder.mkdirs();

			File pbLockFile = new File(electoralAuthorityPath + File.separator
					+ "pblock");

			FileData file = MasterFileGenerationUtils.writeCipheredFile(
					pbLockFile, electoralAuthority.getAuthorityKey(),
					masterSymmetricKey, electoralAuthority.getProtectionKey(),
					false);

			writeLastFileProcessed(file);

			writeElectoralAuthorityLanguages(electoralAuthorityPath,
					this.electoralAuthorityLanguages.get(electoralAuthority),
					masterSymmetricKey);

			writeElectoralAuthorityUserFile(totalizationFilesPath);

		}

		writeElectoralAuthorityUserFile(totalizationFilesPath);

	}

	/**
	 * Writes a user file for each of the electoral authorities
	 * 
	 * @param totalizationFilesPath
	 *            the path where the files should be stored
	 * @throws ConnectionException
	 *             a signal that an error has occurred with the server
	 *             connection
	 * @throws ElectoralOrganizationOperationException
	 *             a signal that an error has occurred retrieving electoral
	 *             authorities information
	 * @throws FileGenerationException
	 *             signal that an error has occurred writing a file
	 */
	private void writeElectoralAuthorityUserFile(String totalizationFilesPath)
			throws ConnectionException,
			ElectoralOrganizationOperationException, FileGenerationException {

		logger.info("Stating preparation to generate the user electoral authority file");

		Document document = DocumentHelper.createDocument();

		Element root = document.addElement("electoral-authorities");

		for (ElectoralAuthorityData electoralAuthority : this.electoralAuthorityLanguages
				.keySet()) {

			Element authority = root.addElement("electoral-authority");
			authority.addAttribute("code",
					String.valueOf(electoralAuthority.getCode()));
			authority.addAttribute("user",
					String.valueOf(electoralAuthority.getAuthorityUsername()));
		}

		File xmlPath = new File(totalizationFilesPath + File.separator
				+ "user-electoral-authority.ciph");

		String xml = document.asXML().toString();

		ElectionData election = ECMServiceProvider.retrieveActiveElection();

		FileData file = MasterFileGenerationUtils.writeXML(null, xml, xmlPath,
				this.applicationKey, election.getKeystore());

		writeLastFileProcessed(file);
	}

	/**
	 * Writes the content of the firstboot folder into the pen drive used for
	 * the generation
	 * 
	 * @throws IOException
	 *             if an error has occurred copying the files
	 */
	private void writeFirstbootFolderInDrive() throws IOException {

		String firstbootPath = ECMClientSideConstants.TEMPORARY_PATH_FOR_FILE_GENERATION
				+ File.separator
				+ ECMClientSideConstants.INSTALLATION_FOLDER_PATH;

		File firstbootFiles = new File(firstbootPath);

		File firstbootFolderInDrive = new File(this.mountedPath
				+ File.separator
				+ ECMClientSideConstants.INSTALLATION_FOLDER_PATH);

		if (firstbootFiles.exists()) {
			FileUtils.copyDirectory(firstbootFiles, firstbootFolderInDrive);
		}
	}

	/**
	 * Generates the required data to write the electoral authorities languages
	 * XML file
	 * 
	 * @param electoralAuthorityPath
	 *            the electoral authority path where the file should be stored
	 * @param languages
	 *            the languages set to write
	 * @param masterSymmetricKey
	 *            the symmetric key to encrypt
	 * @throws ConnectionException
	 *             signal that an error has occurred with the server connection
	 * @throws ElectoralOrganizationOperationException
	 *             a signal that an error has occurred retrieving electoral
	 *             authorities information
	 * @throws FileGenerationException
	 *             signal that an error has occurred writing a file
	 */
	private void writeElectoralAuthorityLanguages(
			String electoralAuthorityPath, Set<LanguageData> languages,
			String masterSymmetricKey) throws ConnectionException,
			ElectoralOrganizationOperationException, FileGenerationException {

		logger.info("Stating preparation to generate the default languages file");

		String xmlns = "http://www.smartmatic.com/Saes/Voting/Election";

		String xmlnsXs = "http://www.w3.org/2001/XMLSchema";

		XStream xstreamLanguages = new XStream(new DomDriver("UTF-8"));

		xstreamLanguages.omitField(BaseDataObject.class, "version");

		xstreamLanguages.processAnnotations(LanguageCollectionData.class);

		xstreamLanguages.setMode(XStream.NO_REFERENCES);

		List<LanguageData> languagesList = new ArrayList<LanguageData>();

		languagesList.addAll(languages);

		Collections.sort(languagesList, new SortedLanguage());

		LanguageCollectionData languageCollection = new LanguageCollectionData(
				xmlnsXs, xmlns, languagesList);

		File defaultLanguagesFile = new File(electoralAuthorityPath
				+ File.separator + "default-languages.ciph");

		ElectionData election = ECMServiceProvider.retrieveActiveElection();

		FileData file = MasterFileGenerationUtils.writeXML(xstreamLanguages,
				languageCollection, defaultLanguagesFile, masterSymmetricKey,
				election.getKeystore());

		writeLastFileProcessed(file);
	}

	/**
	 * Download the EML files from the server
	 * 
	 * @param emlFilesPath
	 *            the path were the EML files should be stored
	 * @throws ConnectionException
	 *             if a server connection error occurred
	 * @throws UtilitiesException
	 *             if an error occurred retrieving the EML files from the server
	 * @throws IOException
	 *             if an error occurred writing the files
	 */
	private void downloadEMLFiles(String emlFilesPath) throws Exception {

		logger.info("Downloading EML Files");

		Map<String, byte[]> emlFiles = ECMServiceProvider.getEMLFiles();

		if (emlFiles != null) {

			File emlFilesDir = new File(emlFilesPath);

			if (emlFilesDir.exists()) {
				FileUtils.forceDelete(emlFilesDir);
			}
			emlFilesDir.mkdirs();

			for (String emlFileName : emlFiles.keySet()) {

				File emlFile = new File(emlFilesPath, emlFileName);
				byte[] emlFileBytes = emlFiles.get(emlFileName);

				FileData file = new FileData();

				file.setFile(emlFile);
				file.setData(emlFileBytes);

				FileUtils.writeByteArrayToFile(emlFile, emlFileBytes);

				writeLastFileProcessed(file);
			}

			signEMLFiles(emlFilesDir);
			logger.info("EML Files Downloaded successfully");
		} else {
			logger.info("No EML Files Downloaded");
		}

	}

	/**
	 * Download the voice files from the server
	 * 
	 * @param voiceFilesPath
	 *            the path were the voice files should be stored
	 * @param regionLanguageSet
	 *            the languages to download the voices
	 * @return The list of voice files downloaded successfully to the memory.
	 * @throws ConnectionException
	 *             if a server connection error occurred
	 * @throws UtilitiesException
	 *             if an error occurred retrieving the voice files from the
	 *             server
	 * @throws IOException
	 *             if an error occurred writing the files
	 */
	private List<File> downloadVoiceFiles(String voiceFilesPath,
			Set<LanguageData> languageSet) throws Exception {

		logger.info("Downloading audio voice files");

		List<File> fileListToSign = null;
		byte[] voiceFile = ECMServiceProvider.getVoiceFiles();

		if (voiceFile != null) {

			File voiceFilesDirectory = new File(voiceFilesPath);

			if (!voiceFilesDirectory.exists()) {
				voiceFilesDirectory.mkdirs();
			}

			File voiceFileLocal = new File(voiceFilesPath,
					ECMClientSideConstants.VOICES_ZIP_NAME);

			FileUtils.writeByteArrayToFile(voiceFileLocal, voiceFile);

			List<String> desiredFiles = getDesiredVoiceFiles(languageSet);

			SAESZipUtil.unZipFile(voiceFileLocal, voiceFilesDirectory,
					desiredFiles);

			voiceFileLocal.delete();

			fileListToSign = MasterFileGenerationUtils
					.getFilesToSign(voiceFilesDirectory.toString()
							+ File.separator + "Voices");

			FileData fileData;
			for (File fileToSign : fileListToSign) {
				fileData = new FileData();

				fileData.setFile(fileToSign);
				fileData.setData(FileUtils.readFileToByteArray(fileToSign));

				writeLastFileProcessed(fileData);
			}

			logger.info("Audio voice files downloaded successfully");
		} else {
			logger.info("No audio voice files Downloaded");
		}

		return fileListToSign;

	}

	/**
	 * @param languageSet
	 * @return
	 */
	private List<String> getDesiredVoiceFiles(Set<LanguageData> languageSet) {
		List<String> desiredFiles = new ArrayList<String>();
		desiredFiles.add("Voices/common/");

		for (LanguageData language : languageSet) {
			if (language.getVoiceNameDAT() != null
					&& !language.getVoiceNameDAT().isEmpty()) {
				desiredFiles.add(language.getVoiceNameDAT());
			}
			if (language.getVoiceNameHDR() != null
					&& !language.getVoiceNameHDR().isEmpty()) {
				desiredFiles.add(language.getVoiceNameHDR());
			}
		}
		return desiredFiles;
	}

	/**
	 * Method to generate the detached signature of EML files.
	 * 
	 * @param emlFileFolder
	 *            directory where are the EML Files to sign.
	 * 
	 * @throws Exception
	 *             signals that a problem has occurred
	 */
	private void signEMLFiles(File emlFileFolder) throws Exception {

		File[] emlFiles = emlFileFolder.listFiles();

		boolean generateXMLencrypted = ECMClientSideConstants.GENERATE_INSTALLATION_FOLDER_ENCRYPTED;

		ElectionData activeElection;

		activeElection = ECMServiceProvider.retrieveActiveElection();

		byte[] electionPkcs12 = activeElection.getKeystore().getPkcs12()
				.getBinaryData();

		String electionAliasToSign = activeElection.getKeystore().getAlias();

		String electionPasswordToSign = activeElection.getKeystore()
				.getPassword();

		if (emlFiles != null && emlFiles.length != 0) {

			List<File> fileListToSign = new ArrayList<File>(
					Arrays.asList(emlFiles));

			Map<String, List<File>> filesByUriBase = new HashMap<String, List<File>>();

			filesByUriBase.put(emlFileFolder + "/", fileListToSign);

			String xml = SignXMLUtil
					.signDetachedXML(electionPkcs12, electionAliasToSign,
							electionPasswordToSign, filesByUriBase,
							XMLSignature.ALGO_ID_SIGNATURE_ECDSA_SHA256);

			File signatureXMLFile = new File(emlFileFolder, "Signature.xml");

			FileData file = MasterFileGenerationUtils
					.writeSignatureXML(xml, signatureXMLFile,
							generateXMLencrypted, this.applicationKey);

			writeLastFileProcessed(file);

		}
	}

	/**
	 * Return the total amount of voting devices to process
	 * 
	 * @param lastLevelRegionsList
	 *            the regions list to process
	 * @return the total of voting devices
	 * @throws ConnectionException
	 *             signal that an error has occurred with the server connection
	 * @throws ElectoralPhysicalLocationsRegistryException
	 *             signal that an error has occurred retrieving voting devices
	 *             information
	 */
	private int getVotingDevicesTotal(List<RegionData> lastLevelRegionsList)
			throws ConnectionException,
			ElectoralPhysicalLocationsRegistryException {

		int totalOfVotingDevices = 0;

		for (RegionData region : lastLevelRegionsList) {
			List<VotingDeviceData> votingDevices = ECMServiceProvider
					.getVotingDevicesByRegion(region);

			totalOfVotingDevices += votingDevices.size();
		}

		return totalOfVotingDevices;
	}

	/**
	 * The path to write the files generated for current invocation, if
	 * necessary to generate the files locally first, then it will return the
	 * temporary path otherwise the mounted path will be the target.
	 * 
	 * @return the target path
	 */
	private String getTargetPath() {

		if (ECMClientSideConstants.WRITE_FILES_IN_DRIVE) {
			return this.mountedPath;
		} else {
			return ECMClientSideConstants.TEMPORARY_PATH_FOR_FILE_GENERATION;
		}
	}

	/**
	 * Method that copy the generated files from the local device to the USB
	 * drive if the flag to write directly to the USB drive is disabled.
	 * 
	 * @throws Exception
	 */
	private void copyGeneratedFiles() throws Exception {

		if (!ECMClientSideConstants.WRITE_FILES_IN_DRIVE) {

			File SAESFolder = new File(this.mountedPath, "SAES");
			File SAESLocalFolder = new File(this.mountedPath, "SAESLocal");

			File tmpSAESFolder = new File(
					ECMClientSideConstants.TEMPORARY_PATH_FOR_FILE_GENERATION,
					"SAES");
			File tmpSAESLocalFolder = new File(
					ECMClientSideConstants.TEMPORARY_PATH_FOR_FILE_GENERATION,
					"SAESLocal");

			FileUtils.copyDirectory(SAESLocalFolder, tmpSAESLocalFolder);

			signSAESLocal(tmpSAESLocalFolder);

			FileUtils.forceDelete(SAESFolder);
			FileUtils.forceDelete(SAESLocalFolder);

			long copyTime = System.currentTimeMillis();

			logger.info("Copying SAES folder");
			FileUtils.copyDirectory(tmpSAESFolder, SAESFolder);

			logger.info("Copying SAESLocal folder");
			FileUtils.copyDirectory(tmpSAESLocalFolder, SAESLocalFolder);

			logger.info("SAES and SAESLocal copied successfully");
			logger.info("Time copying SAES and SAESLocal folder: "
					+ TimestampUtil.prettyTimeDifference(copyTime,
							System.currentTimeMillis()));
		}
	}

	/**
	 * Method to generate the detached signature of EML files.
	 * 
	 * @param emlFileFolder
	 *            directory where are the EML Files to sign.
	 * 
	 * @throws Exception
	 *             signals that a problem has occurred
	 */
	private void signSAESLocal(File saesLocalFolder) throws Exception {

		List<File> fileListToSign = getSaesLocalFilesToSing(saesLocalFolder);

		boolean generateXMLencrypted = ECMClientSideConstants.GENERATE_INSTALLATION_FOLDER_ENCRYPTED;

		ElectionData activeElection = ECMServiceProvider
				.retrieveActiveElection();

		byte[] electionPkcs12 = activeElection.getKeystore().getPkcs12()
				.getBinaryData();

		String electionAliasToSign = activeElection.getKeystore().getAlias();

		String electionPasswordToSign = activeElection.getKeystore()
				.getPassword();

		if (!fileListToSign.isEmpty()) {

			Map<String, List<File>> filesByUriBase = new HashMap<String, List<File>>();

			filesByUriBase.put(saesLocalFolder.getPath(), fileListToSign);

			String xml = SignXMLUtil
					.signDetachedXML(electionPkcs12, electionAliasToSign,
							electionPasswordToSign, filesByUriBase,
							XMLSignature.ALGO_ID_SIGNATURE_ECDSA_SHA256);

			File signatureXMLFile = new File(saesLocalFolder, "Signature.xml");

			FileData file = MasterFileGenerationUtils
					.writeSignatureXML(xml, signatureXMLFile,
							generateXMLencrypted, this.applicationKey);

			writeLastFileProcessed(file);

		}
	}

	/**
	 * Gets files to sign from SAESLocal.
	 * 
	 * @param saesLocalFolder
	 * @return
	 */
	private List<File> getSaesLocalFilesToSing(File saesLocalFolder) {

		List<File> filesToSign = new ArrayList<File>();

		for (File file : saesLocalFolder.listFiles()) {
			if (!file.getName().equals("SAES")) {
				if (file.isFile()) {
					filesToSign.add(file);
				} else if (file.isDirectory()) {
					filesToSign.addAll(getSaesLocalFilesToSing(file));
				}
			}
		}

		return filesToSign;
	}

	/**
	 * Perform the merge of SAES folders for the current master generation.
	 * 
	 */
	private void mergeSaesFiles() {
		if (!this.testMode) {
			try {
				logger.info("Starting to merge SAESLocal and SAESLocalUnsigned");
				PendriveUtil.mergeSAESLocalSAESLocalUnsigned(this.mountedPath,
						this.testMode);
				logger.info("SAESLocal and SAESLocalUnsigned merged successfully");
			} catch (PendriveException e) {
				logger.error(e, StringUtils.defaultString(e.getMessage()));
			}
		}
	}

	/**
	 * Check if exist the mounted path.
	 * 
	 * @return true if the mounted path exist.
	 */
	private boolean pendriveDisconected() {
		File mountedPathFile = new File(this.mountedPath);
		return mountedPathFile.exists();
	}
}