package com.smartmatic.waffle.ecm.client.worker;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import net.sf.jasperreports.engine.JasperPrint;

import org.apache.commons.lang.StringUtils;

import com.smartmatic.waffle.domain.demographics.PollingPlaceData;
import com.smartmatic.waffle.domain.demographics.RegionData;
import com.smartmatic.waffle.domain.device.VotingDeviceData;
import com.smartmatic.waffle.domain.i18n.LanguageData;
import com.smartmatic.waffle.ecm.client.ECMClientSideConstants;
import com.smartmatic.waffle.ecm.client.ECMServiceProvider;
import com.smartmatic.waffle.ecm.client.lang.ConnectionException;
import com.smartmatic.waffle.ecm.client.ui.common.AsynchronousOperation;
import com.smartmatic.waffle.ecm.client.ui.common.AwtOperationType;
import com.smartmatic.waffle.ecm.core.printing.PrinterAssistant;
import com.smartmatic.waffle.ecm.data.transfer.comparators.LanguageByLanguageRFCComparator;
import com.smartmatic.waffle.ecm.data.transfer.objects.BrowserHierarchyDTO;
import com.smartmatic.waffle.ecm.data.transfer.type.ReportGenerationType;
import com.smartmatic.waffle.ecm.service.lang.DemographyException;
import com.smartmatic.waffle.ecm.service.lang.ElectoralOrganizationOperationException;
import com.smartmatic.waffle.ecm.service.lang.ElectoralPhysicalLocationsRegistryException;
import com.smartmatic.waffle.ecm.service.lang.UtilitiesException;
import com.smartmatic.waffle.log.SMTTLogger;

/**
 * Worker class that prepares and calls the backend report generation methods,
 * after the generation is done a list of reports will be sent to the client.
 *
 */
public class ReportsGenerationWorker extends AsynchronousOperation {

	/** Logger. */
	private static SMTTLogger logger = SMTTLogger
			.getLogger(ReportsGenerationWorker.class);

	private Set<BrowserHierarchyDTO> selectedObjects;

	private ReportGenerationType reportType;

	private List<JasperPrint> reportsResultList;

	private boolean printReportWithoutPassword;

	private boolean generationDone;

	private int generationProgress;

	public ReportsGenerationWorker(Set<BrowserHierarchyDTO> selectedObjects,
			ReportGenerationType reportType, boolean printReportWithoutPassword) {

		this.selectedObjects = selectedObjects;
		this.reportType = reportType;
		this.printReportWithoutPassword = printReportWithoutPassword;

		this.reportsResultList = new ArrayList<JasperPrint>();
	}

	/**
	 * Return the status of the current generation
	 * 
	 * @return the generation process status
	 */
	public boolean isGenerationDone() {
		return this.generationDone;
	}

	/**
	 * Returns the reports list to print
	 * 
	 * @return the reports list to print
	 */
	public List<JasperPrint> getReportsResultList() {
		return this.reportsResultList;
	}

	/**
	 * The current generation progress
	 * 
	 * @return the generation progress
	 */
	public int getProgress() {
		return this.generationProgress;
	}

	@Override
	protected void performOperation() {

		try {

			logger.info("Starting report generation process");
			fireAwtEvent(AwtOperationType.UPDATE_COMPONENTS);
			startReportGeneration();

		} catch (Exception e) {
			logger.error(e, StringUtils.defaultString(e.getMessage()));
			fireAwtEvent(AwtOperationType.SHOW_ERROR_MESSAGE);
		}
	}

	@Override
	protected void onCompleted() {
		fireAwtEvent(AwtOperationType.UPDATE_COMPONENTS);
	}

	/**
	 * Starts the report generation process sending the last level region to the
	 * specific report type methods.
	 * 
	 * @throws DemographyException
	 *             if an error occurred getting the last level regions
	 * @throws ConnectionException
	 *             if an error occurred with the server connection
	 * @throws ElectoralPhysicalLocationsRegistryException
	 *             if an error occurred getting the voting devices information
	 * @throws UtilitiesException
	 *             if an error occurred generating the reports
	 * @throws ElectoralOrganizationOperationException
	 *             if an error occurred getting electoral authorities list
	 */
	protected void startReportGeneration() throws DemographyException,
			ConnectionException, ElectoralPhysicalLocationsRegistryException,
			UtilitiesException, ElectoralOrganizationOperationException {

		this.generationDone = false;

		fireAwtEvent(AwtOperationType.REFRESH_LABEL);

		Set<PollingPlaceData> pollingPlaces = getPollingPlaces(this.selectedObjects);

		List<RegionData> selectedRegions = getRegions(this.selectedObjects);

		if (selectedRegions.size() > 0) {
			List<RegionData> lastLevelRegions = ECMServiceProvider
					.getLastLevelRegionsLight(selectedRegions);

			for (RegionData region : lastLevelRegions) {
				try {
					pollingPlaces.addAll(ECMServiceProvider
							.getPollingPlacesPerRegion(region));
				} catch (Exception e) {
					logger.error("Error getting polling places for region"
							+ region.getName(), e);
				}
			}
		}

		generateReports(pollingPlaces, this.printReportWithoutPassword);

		this.generationDone = true;
	}

	/**
	 * Generate the reports for voting devices and recount machines, after the
	 * completion a reports result list is set to its further use.
	 * 
	 * @param pollingPlaces
	 *            the list of polling places to generate the reports
	 * @param printReportWithoutPassword
	 *            true if the reports with password should be printed, otherwise
	 *            false
	 * @throws ConnectionException
	 *             if a server connection error occurred
	 * @throws ElectoralPhysicalLocationsRegistryException
	 *             if an error occurred getting voting devices data
	 * @throws UtilitiesException
	 *             if an error occurred generating the reports
	 */
	protected void generateReports(Set<PollingPlaceData> pollingPlaces,
			boolean printReportWithoutPassword) throws ConnectionException,
			ElectoralPhysicalLocationsRegistryException, UtilitiesException {

		Map<LanguageData, List<VotingDeviceData>> languageVotingDevicesMap = getLanguageVotingDevicesMap(pollingPlaces);

		Map<LanguageData, List<JasperPrint>> languageJasperPrintsMap = getLanguageJasperPrintMap(
				languageVotingDevicesMap, printReportWithoutPassword);

		for (LanguageData language : languageJasperPrintsMap.keySet()) {

			logger.info("Generating reports for language: "
					+ language.getLanguageRFC());
			JasperPrint jasper = PrinterAssistant
					.mergeJaspers(languageJasperPrintsMap.get(language));
			this.reportsResultList.add(jasper);
		}

	}

	/**
	 * Generates a map with the language as key and the list of reports for a
	 * list of voting devices as the value
	 * 
	 * @param languageVotingDevicesMap
	 *            the map which contains the language and a list of voting
	 *            devices
	 * @param printReportWithoutPassword
	 *            true if the reports with password should be printed, otherwise
	 *            false
	 * @return the language and jasper print reports generated
	 * @throws UtilitiesException
	 *             if an error occurred generating the reports
	 * @throws ConnectionException
	 *             if a server connection error occurred
	 */
	private Map<LanguageData, List<JasperPrint>> getLanguageJasperPrintMap(
			Map<LanguageData, List<VotingDeviceData>> languageVotingDevicesMap,
			boolean printReportWithoutPassword) throws UtilitiesException,
			ConnectionException {

		Map<LanguageData, List<JasperPrint>> languageJasperPrint = new HashMap<LanguageData, List<JasperPrint>>();

		for (LanguageData language : languageVotingDevicesMap.keySet()) {

			List<JasperPrint> jasperPrintList = ECMServiceProvider
					.generateReports(language,
							languageVotingDevicesMap.get(language),
							isVotingDevicesReport(), printReportWithoutPassword);

			languageJasperPrint.put(language, jasperPrintList);
		}

		return languageJasperPrint;
	}

	/**
	 * Returns a map with the language and a list of voting devices for that
	 * language key
	 * 
	 * @param pollingPlaces
	 *            polling places set to get the map
	 * @return the map generated
	 * @throws ConnectionException
	 *             if a server connection error occurs
	 * @throws ElectoralPhysicalLocationsRegistryException
	 * @throws UtilitiesException
	 *             if an error occurs getting the languages
	 */
	private Map<LanguageData, List<VotingDeviceData>> getLanguageVotingDevicesMap(
			Set<PollingPlaceData> pollingPlaces) throws ConnectionException,
			ElectoralPhysicalLocationsRegistryException, UtilitiesException {

		this.generationProgress = 2;
		fireAwtEvent(AwtOperationType.UPDATE_PROGRESS_BAR);

		Map<LanguageData, List<VotingDeviceData>> languageVotingDevicesMap = new HashMap<LanguageData, List<VotingDeviceData>>();

		List<VotingDeviceData> pollingPlaceVotingDevices;
		List<LanguageData> regionParentLanguages;

		int totalToProcess = pollingPlaces.size() + 2;

		int pollingPlacesProcessed = 2;

		for (PollingPlaceData pollingPlace : pollingPlaces) {

			Map<LanguageData, List<VotingDeviceData>> languageVotingDevicesMapTmp;

			pollingPlaceVotingDevices = getVotingDevicesByPollingPlace(pollingPlace);

			regionParentLanguages = getRegionParentLanguages(pollingPlace
					.getRegion());

			if (generateMultiLanguageReports()) {
				languageVotingDevicesMapTmp = getMultiLanguageVotingDeviceMap(
						pollingPlaceVotingDevices, regionParentLanguages);
			} else {
				languageVotingDevicesMapTmp = getSingleLanguageVotingDeviceMap(
						regionParentLanguages, pollingPlaceVotingDevices);
			}

			for (LanguageData language : languageVotingDevicesMapTmp.keySet()) {
				if (languageVotingDevicesMap.containsKey(language)) {
					languageVotingDevicesMap.get(language).addAll(
							languageVotingDevicesMapTmp.get(language));
				} else {
					languageVotingDevicesMap.put(language,
							languageVotingDevicesMapTmp.get(language));
				}
			}

			this.generationProgress = (++pollingPlacesProcessed * 100)
					/ totalToProcess;

			fireAwtEvent(AwtOperationType.UPDATE_PROGRESS_BAR);
		}

		return languageVotingDevicesMap;
	}

	/**
	 * Generates a map with a single language as key and a list of voting
	 * devices as value
	 * 
	 * @param languages
	 *            the list of languages to generate the map
	 * @param votingDevices
	 *            the list of voting devices to set in each language
	 * @return the generated map
	 */
	private Map<LanguageData, List<VotingDeviceData>> getSingleLanguageVotingDeviceMap(
			List<LanguageData> languages, List<VotingDeviceData> votingDevices) {

		Map<LanguageData, List<VotingDeviceData>> votingDevicesByLanguage = new HashMap<LanguageData, List<VotingDeviceData>>();

		for (LanguageData indexlanguage : languages) {

			if (votingDevicesByLanguage.containsKey(indexlanguage)) {
				votingDevicesByLanguage.get(indexlanguage)
						.addAll(votingDevices);
			} else {
				votingDevicesByLanguage.put(indexlanguage, votingDevices);
			}
		}
		return votingDevicesByLanguage;

	}

	/**
	 * If the generation with multi languages is required this method will
	 * generate the map with the languages merged
	 * 
	 * @param votingDevicesList
	 *            the list of voting devices to set for each language
	 * @param languagesList
	 *            a list of languages to generate the map
	 * @return a map with the language as the key and a list of voting devices
	 *         as value
	 */
	private Map<LanguageData, List<VotingDeviceData>> getMultiLanguageVotingDeviceMap(
			List<VotingDeviceData> votingDevicesList,
			List<LanguageData> languagesList) {

		Map<LanguageData, List<VotingDeviceData>> languageVotingDevicesMap = new HashMap<LanguageData, List<VotingDeviceData>>();

		if (languagesList.size() > 1) {
			LanguageData languageRFCMerged = getLanguageMerged(languagesList);
			languageVotingDevicesMap.put(languageRFCMerged, votingDevicesList);
		} else {
			languageVotingDevicesMap.put(languagesList.get(0),
					votingDevicesList);
		}
		return languageVotingDevicesMap;
	}

	/**
	 * Merged and return a new language from a list of languages
	 * 
	 * @param languagesList
	 *            a language list to merge
	 * @return the new language merged
	 */
	private LanguageData getLanguageMerged(List<LanguageData> languagesList) {

		Collections.sort(languagesList, new LanguageByLanguageRFCComparator());

		StringBuilder languageRFC = new StringBuilder();
		StringBuilder languageCustomCode = new StringBuilder();

		for (LanguageData language : languagesList) {

			if ((language.getLanguageRFC() != null)
					&& (!languageRFC.toString().contains(
							language.getLanguageRFC()))) {

				if (languageRFC.length() != 0) {
					languageRFC.append("-");
					languageCustomCode.append("|");
				}
				languageRFC.append(language.getLanguageRFC());
				languageCustomCode.append(language.getCustomCode());
			}
		}
		LanguageData languageMerged = new LanguageData();
		languageMerged.setLanguageRFC(languageRFC.toString());
		languageMerged.setCode(1);
		languageMerged.setCustomCode(languageCustomCode.toString());

		return languageMerged;
	}

	/**
	 * Transforms and returns from a list of selected widget objects a list of
	 * regions
	 * 
	 * @param selectedObjects
	 *            the list of selected objects to transform
	 * @return a list of regions
	 */
	private List<RegionData> getRegions(Set<BrowserHierarchyDTO> selectedObjects) {

		List<RegionData> regions = new ArrayList<RegionData>();
		for (BrowserHierarchyDTO widgetObject : selectedObjects) {

			if (widgetObject.getId() instanceof RegionData) {
				regions.add((RegionData) widgetObject.getId());
			}
		}
		return regions;
	}

	/**
	 * Transforms and returns from a list of selected widget objects a set of
	 * polling places
	 * 
	 * @param selectedObjects
	 *            the list of selected objects to transform
	 * @return a set of polling places
	 */
	private Set<PollingPlaceData> getPollingPlaces(
			Set<BrowserHierarchyDTO> selectedObjects) {
		Set<PollingPlaceData> pollingPlaces = new HashSet<PollingPlaceData>();
		for (BrowserHierarchyDTO widgetObject : selectedObjects) {

			if (widgetObject.getId() instanceof PollingPlaceData) {
				pollingPlaces.add((PollingPlaceData) widgetObject.getId());
			}
		}
		return pollingPlaces;
	}

	/**
	 * Returns a list of voting devices by polling place
	 * 
	 * @param pollingPlace
	 *            the polling place to get its voting devices
	 * @return the list of voting devices
	 * @throws ConnectionException
	 *             if a server connection error has occurred
	 * @throws ElectoralPhysicalLocationsRegistryException
	 *             if an error occurs getting the voting devices
	 */
	private List<VotingDeviceData> getVotingDevicesByPollingPlace(
			PollingPlaceData pollingPlace) throws ConnectionException,
			ElectoralPhysicalLocationsRegistryException {
		return ECMServiceProvider.getVotingDevicesByPollingPlace(pollingPlace);
	}

	/**
	 * Returns a list of languages by region
	 * 
	 * @param region
	 *            the region to get languages
	 * @return the list of languages that belongs to the region
	 * @throws UtilitiesException
	 *             if an error occurred getting the languages
	 * @throws ConnectionException
	 *             if a server error has occurred
	 */
	private List<LanguageData> getRegionParentLanguages(RegionData region)
			throws UtilitiesException, ConnectionException {
		return ECMServiceProvider.getParentLanguages(region);
	}

	/**
	 * Returns true if reports with multi languages must be generated
	 * 
	 * @return true if multi languages report must be generated
	 */
	private boolean generateMultiLanguageReports() {
		return ECMClientSideConstants.REPORTS_WITH_MULTI_LANGUAGES;
	}

	/**
	 * True if the selected report if for voting devices, otherwise false
	 * 
	 * @return true is report type is for voting devices
	 */
	private boolean isVotingDevicesReport() {
		return (ReportGenerationType.VOTING_DEVICE_REPORT == this.reportType);
	}
}
