#!/bin/bash

# Filename: OS-install.sh
# Synopsis: Script to create a USB from an OS image tarball.
#           This script is mainly to install TRUSTY distribution.
#           It uses GRUB as bootloader.
# Author: Luis Dalo
# Version: 0.0.3
# Last Edited: 9 Nov 2017

set -e

###############
#  VARIABLES  #
###############

SCRIPTS_PATH=$(dirname $(readlink -f $0))
HELP="Usage (NON-INTERACTIVE 1): $0  [USB_DEVICE]  [PATH_TO_OS_TARBALL]  [PATH_TO_PREP_FILE]\n
where:\n
- USB_DEVICE: Device to be used. i.e. /dev/sdb\n
- PATH_TO_OS_TARBALL: Absolute path where to find the OS tarball.\n
- PATH_TO_PREP_FILE: Absolute path where the swap.prep file is located.\n\n
Usage (NON-INTERACTIVE 2): $0  [USB_DEVICE]  [PATH_TO_OS_TARBALL]\n
where:\n
- USB_DEVICE: Device to be used. i.e. /dev/sdb\n
- PATH_TO_OS_TARBALL: Absolute path where to find the OS tarball.\n\n
Usage (USER-INTERACTIVE): $0\n"


# Function: die
# @param1: Message: Message to display
# @param2: Exitcode: Exit code to exit the script
function die()
{
    echo -e ${1}
    exit ${2}
}

###################################
#  USER ENTRY OR NON-INTERACTIVE  #
###################################

if [ ${#} -eq 0 ]
then
    # User interactive
    echo -e "Enter device to be used and press [ENTER]:"
    read -e USB_DEVICE 

    echo -e "\nEnter absolute path to OS and press [ENTER]:"
    read -e PATH_TO_OS

    echo
    read -r -p "Do you wish to load a new swap.prep file? [y/N]: " PREP_LOAD_ANS
    case "${PREP_LOAD_ANS}" in
        [yY])
            echo -e "\nEnter path where swap.prep file is located and press [ENTER]:"
            read -e PATH_TO_PREP
            PREP_LOAD_FLAG=true
            ;;
        *)
            PREP_LOAD_FLAG=false
            ;;
    esac
elif [ ${#} -eq 3 ]
then
    USB_DEVICE=${1}
    PATH_TO_OS=${2}
    PATH_TO_PREP=${3}
    PREP_LOAD_FLAG=true
    format=true
elif [ ${#} -eq 2 ]
then
    USB_DEVICE=${1}
    PATH_TO_OS=${2}
    PREP_LOAD_FLAG=false
    format=true
else
    echo "E: Wrong usage."
    echo -e ${HELP}
    exit 1
fi

##############################
#  VALIDATING GRUB PACKAGES  #
##############################

if [ $(dpkg -l | grep grub-efi-amd64-bin | wc -l) -eq 0 ]
then
    echo "E: Package grub-efi-amd64-bin not installed."
    echo "Fix: sudo apt-get install grub-efi-amd64-bin"
    echo "Exiting..."
    exit 1
fi

if [ $(dpkg -l | grep grub-pc-bin | wc -l) -eq 0 ]
then
    echo "E: Package grub-pc-bin not installed."
    echo "Fix: sudo apt-get install grub-pc-bin"
    echo "Exiting..."
    exit 1
fi

##########################
#  VALIDATING ARGUMENTS  #
##########################

for item in ${USB_DEVICE} ${PATH_TO_OS}
do
    if [ -z ${item} ]
    then
        echo "E: Device path, or to OS tarball, cannot be empty."
        echo -e ${HELP}
        exit 1
    fi
done

if [ ! -e ${USB_DEVICE} ]
then
    echo "E: Not a valid device (${USB_DEVICE})."
    echo -e ${HELP}
    exit 1
fi

if [ ! -f ${PATH_TO_OS} ]
then
    echo "E: ${PATH_TO_OS} file not found."
    echo -e ${HELP}
    exit 1
fi

if [ "${USB_DEVICE}" == "/" ]
then
    echo "E: USB_DEVICE cannot be the root path."
    echo -e ${HELP}
    exit 1
fi

if ${PREP_LOAD_FLAG}
then
    if [ -z ${PATH_TO_PREP} ]
    then
       echo "E: PATH_TO_PREP_FILE cannot be empty"
       echo -e ${HELP}
       exit 1
    elif [ ! -d ${PATH_TO_PREP} ]
    then
        echo "E: ${PATH_TO_PREP} not a directory."
        echo -e ${HELP}
        exit 1
    elif [ ! -f ${PATH_TO_PREP}/swap.prep ]
    then
        echo "E: Path ${PATH_TO_PREP} does not contain a swap.prep file"
        echo -e ${HELP}
        exit 1
    fi
fi

###########
#  BEGIN  #
###########

# Normalizing the DEVICE
USB_DEVICE=$(echo ${USB_DEVICE} | sed 's/[0-9].*//')

echo "I: Device found"

####################
#  FORMATTING USB  #
####################

if [ "${format,,}" == "false" ]
then
    FORMAT_USB=n
elif [ "${format,,}" == "true" ]
then
    FORMAT_USB=y
else
    # User Interactive
    echo -n "Do you wish to format ${USB_DEVICE} (Y/n): "
    read FORMAT_USB
fi

if [[ "${FORMAT_USB,,}" == "y" || -z "${FORMAT_USB}" ]]
then
    echo "I: Formatting ${USB_DEVICE}"

    ${SCRIPTS_PATH}/smtt-create-partitions.sh ${USB_DEVICE} || RESULT=$?
    if [[ $RESULT -eq 2 ]]; then die "E:Error Creating filesystem" 3; fi
    if [[ $RESULT -eq 1 ]]; then die "E:Generic Error on smtt-create-partitions.sh" 1; fi

    echo "I: Mounting EFI Partition"
    EFI_MOUNT_POINT=/tmp/efi_mount_point
    mkdir -p ${EFI_MOUNT_POINT}
    mount ${USB_DEVICE}2 ${EFI_MOUNT_POINT}

    if [ ! -f /etc/init/smtt-start-devmon.conf ]
    then
    echo "I: Mounting Ext3 Partition"
       USB_PATH=$(mktemp -d)
       mkdir -p ${USB_PATH}
       mount ${USB_DEVICE}3 ${USB_PATH}
    fi

    echo "I: Done formatting and re-mounting"
else
    echo "I: No formatting requested"
fi
sleep 5

# The USB EXT3 partition should be mounted in a new point
USB_PATH=$(cat /proc/mounts | grep ${USB_DEVICE}3 | awk '{print $2}')

################
#  BOOTLOADER  #
################


if [ -d ${EFI_MOUNT_POINT} ]
then
    echo "I: Installing GRUB bootloader for UEFI"
    sudo grub-install --target=x86_64-efi --efi-directory=${EFI_MOUNT_POINT} --boot-directory=${USB_PATH}/boot --removable --recheck ${USB_DEVICE}
    sudo mkdir -p ${EFI_MOUNT_POINT}/EFI/ubuntu
    sudo cp ${EFI_MOUNT_POINT}/EFI/BOOT/BOOTX64.EFI ${EFI_MOUNT_POINT}/EFI/ubuntu/grubx64.efi
    umount ${EFI_MOUNT_POINT}
    rm -rf ${EFI_MOUNT_POINT}
fi

echo "I: Installing GRUB bootloader for BIOS"
sudo grub-install --target=i386-pc --boot-directory=${USB_PATH}/boot --recheck ${USB_DEVICE}
echo "I: Done GRUB bootloader"

#####################
#  OS INSTALLATION  #
#####################

echo "I: Initializing OS Installation"
# making a temporary directory
temp_dir=$(mktemp -d)

tar -xmf ${PATH_TO_OS} -C ${temp_dir} || die "E:Error unzipping files" 2

echo -n "I: Copying files............... "
cp -aR ${temp_dir}/OS/live ${USB_PATH}
find ${temp_dir}/OS -name SAES* -exec cp -aR {} ${USB_PATH} \;
sudo cp ${temp_dir}/OS/boot/grub/grub.cfg ${USB_PATH}/boot/grub/
sudo cp ${temp_dir}/OS/boot/grub/splash-* ${USB_PATH}/boot/grub/
sudo cp ${temp_dir}/OS/boot/grub/i386-pc/host_bridge.mod ${USB_PATH}/boot/grub/i386-pc/
sudo cp ${temp_dir}/OS/boot/grub/x86_64-efi/host_bridge.mod ${USB_PATH}/boot/grub/x86_64-efi/
sudo chown -R 1000.1000 ${USB_PATH}

echo "Done"

##########################################
#  STORING swap.prep FILE INTO SQUASHFS  #
##########################################

if ${PREP_LOAD_FLAG}
then
    echo "I: Initializing storing of the swap.prep file."
    ${SCRIPTS_PATH}/smtt-write-application-key.sh ${USB_PATH} ${PATH_TO_PREP}
fi

##########################
#  GENERATING CHECKSUMS  #
##########################

echo "I: Initializing CHECKSUMS"
${SCRIPTS_PATH}/smtt-make-checksums.sh ${USB_PATH}

##############
#  CLEAN UP  #
##############

rm -rf ${temp_dir}
sync
if [ ! -f /etc/init/smtt-start-devmon.conf ]
then
    echo "I: Unmounting Ext3 Partition"
    umount ${USB_PATH}
    rm -r ${USB_PATH}
fi
echo -e "\nI: Finished"

#########
#  END  #
#########
