/**
 * PendriveUtil.java
 *
 * Revision: $Revision$  (Last Modified: $Date$)
 * Last Modified by: [$Author$]
 * 
 * Copyright (c) 2004 Smartmatic Corp.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of
 * Smartmatic Corp. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Corp.
 */
package com.smartmatic.waffle.ecm.core.pendrive.util;

import java.util.ArrayList;
import java.util.List;
import java.util.regex.Pattern;

import org.apache.commons.io.IOUtils;
import org.apache.commons.lang.StringUtils;

import com.smartmatic.waffle.ecm.core.lang.PendriveException;
import com.smartmatic.waffle.ecm.core.lang.PendriveException.PendriveExceptionInfo;
import com.smartmatic.waffle.ecm.core.util.TimestampUtil;
import com.smartmatic.waffle.localpersistence.SAESCommonDestinations;
import com.smartmatic.waffle.localpersistence.SAESLocalPersistenceManager;
import com.smartmatic.waffle.log.SMTTLogger;

/**
 * Utility class used to do specific tasks related with USB stick.
 */
public class PendriveUtil {

	/** The logger. */
	private static SMTTLogger logger = SMTTLogger.getLogger(PendriveUtil.class);

	/** The process instance to wait for the command execution. */
	private static Process process;

	/**
	 * Checks if the pendrive's label follow this rule: 8 hex-digits, then 4, 4,
	 * 4, and then 12. Hyphen-separated
	 * 
	 * @param pendrivePath
	 *            pendrive path to media
	 * 
	 * @return true, if label is follows the rule otherwise false
	 * 
	 * @throws PendriveException
	 *             signals that some exception has occurred testing the device.
	 */
	public static boolean checkPendriveLabel(String pendrivePath)
			throws PendriveException {

		String[] checkPath = null;

		String[] checkLabel = null;

		try {
			checkPath = StringUtils.split(pendrivePath, '/');

			checkLabel = StringUtils.split(StringUtils.replace(
					checkPath[checkPath.length - 1], "_", ""), '-');
		} catch (Exception e) {
			throw new PendriveException(
					PendriveExceptionInfo.PENDRIVE_DAMAGED_LABEL);
		}

		Pattern p4 = Pattern.compile("^([0-9a-fA-F]){4}$");
		Pattern p8 = Pattern.compile("^([0-9a-fA-F]){8}$");
		Pattern p12 = Pattern.compile("^([0-9a-fA-F]){12}$");

		// 8 hex-digits, then 4, 4, 4, and then 12. Hyphen-separated

		if (!checkPath[0].equals("media")
				|| !p8.matcher(checkLabel[0]).matches()
				|| !p4.matcher(checkLabel[1]).matches()
				|| !p4.matcher(checkLabel[2]).matches()
				|| !p4.matcher(checkLabel[3]).matches()
				|| !p12.matcher(checkLabel[4]).matches()) {
			throw new PendriveException(
					PendriveExceptionInfo.PENDRIVE_DAMAGED_LABEL);
		}

		return true;
	}

	/**
	 * Parses a list of strings (typically, the stdout of a script), and for
	 * each line, checks if it is an expected error line (based on partial
	 * matches), and authorized, or if it is unexpected and problematic.
	 * 
	 * @param standardErrorOut
	 *            the list of strings to check
	 * @param authorizedErrorsList
	 *            list of (sub)strings, which if found in a string of the list
	 *            to check, will OK that line.
	 * @return true, if is an authorized error
	 */
	public static boolean isAnExpectedOutput(String[] standardErrorOut,
			List<String> authorizedErrorsList) {

		boolean lineIsOk = false;

		for (String line : standardErrorOut) {

			lineIsOk = false;

			for (String permittedError : authorizedErrorsList) {
				if (StringUtils.contains(line, permittedError)) {
					lineIsOk = true;
					break;
				}
			}

			// as soon as a line is problematic, we can return.
			if (lineIsOk == false) {
				logger.warn("Unexpected line found in standard error output: "
						+ line);
				return false;
			}

		}

		return true;
	}

	/**
	 * Changes the owner to 1000:1000
	 * 
	 * @throws PendriveException
	 */
	public static void chown10001000(String mountedPath)
			throws PendriveException {
		long startSyncTime = System.currentTimeMillis();

		executePendriveUtilScript("chown", mountedPath, null, false);

		long stopSyncTime = System.currentTimeMillis();

		logger.info("Sync Time: "
				+ TimestampUtil.prettyTimeDifference(startSyncTime,
						stopSyncTime));
	}

	/**
	 * Sync pendrive.
	 * 
	 * @throws PendriveException
	 */
	public static void sync() throws PendriveException {
		long startSyncTime = System.currentTimeMillis();

		executePendriveUtilScript("sync", StringUtils.EMPTY, null, true);

		long stopSyncTime = System.currentTimeMillis();

		logger.info("Sync Time: "
				+ TimestampUtil.prettyTimeDifference(startSyncTime,
						stopSyncTime));
	}

	/**
	 * Unmount pendrive.
	 * 
	 * @param partitionName
	 *            the partition name
	 * 
	 * @param excecuteWithoutSudo
	 *            true if the script should be executed without sudo.
	 * @throws PendriveException
	 *             Signals that some exception has occurred executing the
	 *             command.
	 */
	public static void unmountPendrive(String partitionName,
			boolean excecuteWithoutSudo) throws PendriveException {

		try {
			List<String> authorizedErrorsList = new ArrayList<String>();
			authorizedErrorsList.add("lsof: WARNING: can't stat()");
			authorizedErrorsList.add("Output information may be incomplete.");

			logger.info("Umounting Master USB");

			long startUmountTime = System.currentTimeMillis();

			executePendriveUtilScript("umount", partitionName,
					authorizedErrorsList, excecuteWithoutSudo);

			logger.info("Umount Time: "
					+ TimestampUtil.prettyTimeDifference(startUmountTime,
							System.currentTimeMillis()));

			logger.info("Finished umount process");

		} catch (PendriveException e) {
			throw e;
		} catch (Exception e) {
			throw new PendriveException(e,
					PendriveExceptionInfo.UNEXPECTED_ERROR_EXECUTING_COMMAND,
					StringUtils.defaultString(e.getMessage()));
		}
	}

	/**
	 * Executes rsync to transfer just the differences between SAESLocalUnsigned
	 * to SAESLocal folder
	 * 
	 * @param pathToMedia
	 *            the path to media
	 * @param excecuteWithoutSudo
	 *            true if the script should be executed without sudo.
	 * @throws PendriveException
	 *             Signals that some exception has occurred executing the
	 *             command.
	 */
	public static void mergeSAESLocalSAESLocalUnsigned(String pathToMedia,
			boolean excecuteWithoutSudo) throws PendriveException {

		logger.info("Merging SAESLocal and SAESLocalUnsigned");

		executePendriveUtilScript("syncAndMerge", pathToMedia, null,
				excecuteWithoutSudo);

		logger.info("SAESLocal and SAESLocalUnsigned merged successfully");
	}

	/**
	 * Method used to execute one of the commands supported by the
	 * pendriveutil.sh script.
	 * 
	 * @param path
	 *            the parameter to be passed to the script.
	 * 
	 * @param command
	 *            command to be called from the script
	 * 
	 * @param excecuteWithoutSudo
	 *            true is the script should be executed without sudo.
	 * @throws PendriveException
	 *             Signals that some exception has occurred executing the
	 *             command.
	 */
	private static void executePendriveUtilScript(String command, String path,
			List<String> authorizedErrorsList, boolean excecuteWithoutSudo)
			throws PendriveException {

		try {

			logger.info("Executing command " + command);

			String scriptPath = getScriptPath();

			ProcessBuilder builder = new ProcessBuilder();

			String commandFormat;

			if (excecuteWithoutSudo) {
				commandFormat = "%s %s %s";
			} else {
				commandFormat = "sudo %s %s %s";
			}

			builder.command("sh", "-c",
					String.format(commandFormat, scriptPath, command, path));

			logger.info("Excecuting prendrive util command: "
					+ builder.command());

			process = builder.start();

			try {
				process.waitFor();

				if (process != null) {
					String stdOut = IOUtils.toString(process.getInputStream());

					logger.info("Standard out prendrive util:\n" + stdOut);

					String standardError = IOUtils.toString(process
							.getErrorStream());

					if (!standardError.isEmpty()) {
						if (authorizedErrorsList != null) {
							boolean messageIsOk = isAnExpectedOutput(
									StringUtils.split(standardError, "\n"),
									authorizedErrorsList);

							if (!messageIsOk) {
								throw new PendriveException(
										PendriveExceptionInfo.UNEXPECTED_ERROR_EXECUTING_COMMAND,
										"\n" + standardError);
							}
						} else {
							throw new PendriveException(
									PendriveExceptionInfo.UNEXPECTED_ERROR_EXECUTING_COMMAND,
									"\n" + standardError);
						}
					}
					logger.info("Command executed successfully");
				}

			} finally {
				if (process != null) {
					process.destroy();
				}
			}

		} catch (PendriveException e) {
			throw e;
		} catch (Exception e) {
			throw new PendriveException(e,
					PendriveExceptionInfo.UNEXPECTED_ERROR_EXECUTING_COMMAND,
					StringUtils.defaultString(e.getMessage()));
		}
	}

	/**
	 * Destroy the process with the command execution.
	 */
	public static void destroyProcess() {
		if (process != null) {
			process.destroy();
			process = null;
		}
	}

	/**
	 * Gets the pendriveutil.sh script path.
	 * 
	 * @return the pendriveUtil.sh path
	 */
	private static String getScriptPath() {

		SAESCommonDestinations defaultDestinations = SAESLocalPersistenceManager
				.getInstance().getDefaultDestinations();

		return defaultDestinations.pendriveUtilScript.getAbsolutePath();
	}
}
