/**
 * PrinterStatus.java
 *
 * Revision: $Revision$  (Last Modified: $Date$)
 * Last Modified by: [$Author$]
 * 
 * Copyright (c) 2004 Smartmatic Corp.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of
 * Smartmatic Corp. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Corp.
 */
package com.smartmatic.waffle.ecm.core.printer;

import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.concurrent.TimeoutException;

import org.apache.commons.io.IOUtils;
import org.apache.commons.lang.StringUtils;

import com.smartmatic.waffle.log.SMTTLogger;

/**
 * Class used to holds the printer status.
 */
public class PrinterStatus implements IPrinterStatusQuery {

	/** For logging purposes. */
	private static SMTTLogger logger = SMTTLogger
			.getLogger(PrinterStatus.class);

	/** The printer name. */
	private String printerName;

	/** The printer url. */
	private String printerUrl;

	/** The printer query. */
	private IPrinterStatusQuery printerQuery;

	/** The listeners. */
	private List<IPrinterStatusChangedListener> listeners = new LinkedList<IPrinterStatusChangedListener>();

	/** The timer. */
	private Thread timer;

	/** The enabled. */
	private Boolean enabled;

	/**
	 * The Class PrinterStatusTask.
	 */
	class PrinterStatusTask implements Runnable {

		/*
		 * (non-Javadoc)
		 * 
		 * @see java.lang.Runnable#run()
		 */
		@Override
		public void run() {
			logger.debug("Executing printing tasks");

			while (true) {
				try {
					Map<String, Object> changed = null;
					if (PrinterStatus.this.printerQuery == null) {
						logger.debug("printerQuery == null");
						changed = init();
						updateCupsStatus(changed);
					}

					if (PrinterStatus.this.printerQuery == null) {

						Thread.sleep(500);

						continue;
					}

					if (changed == null) {
						try {
							changed = PrinterStatus.this.printerQuery
									.updateStatus();
							updateCupsStatus(changed);
						} catch (TimeoutException e) {
							PrinterStatus.this.printerQuery = null;

							Thread.sleep(500);

							continue;

						}
					}

					if (changed.isEmpty() == false
							&& PrinterStatus.this.listeners.isEmpty() == false) {

						for (IPrinterStatusChangedListener listener : PrinterStatus.this.listeners) {

							listener.printerStatusChanged(changed);
						}
					}
					Thread.sleep(500);
				} catch (Exception e) {
					logger.error(e,
							"There is an error executing printing tasks");
				}
			}

		}
	}

	/**
	 * Instantiates a new printer status.
	 * 
	 * @throws IOException
	 *             Signals that an I/O exception has occurred.
	 */
	public PrinterStatus() throws IOException {
		Map<String, Object> changed = init();
		updateCupsStatus(changed);
		this.timer = new Thread(new PrinterStatusTask(), "PrinterStatus");
		this.timer.start();
	}

	/**
	 * Inits the.
	 * 
	 * @return the map
	 * @throws IOException
	 *             Signals that an I/O exception has occurred.
	 */
	private Map<String, Object> init() throws IOException {
		List<String> lines;
		try {
			lines = executeProcess("lpstat", "-d");

		} catch (TimeoutException e) {
			return null;
		}

		if (lines.isEmpty()) {
			logger.debug("lpstat -d [result : EMPTY]");
			return null;
		}

		this.printerName = StringUtils.substringAfter(lines.get(0),
				"system default destination: ");

		logger.debug("System default destination: [" + this.printerName + "]");

		if (StringUtils.isEmpty(this.printerName)) {
			logger.debug("System default destination: [EMPTY]");

			try {
				lines = executeProcess("lpstat", "-v");
			} catch (TimeoutException e) {
				return null;
			}

			if (lines.isEmpty()) {
				logger.debug("lpstat -v [result : EMPTY]");
				return null;
			}

			this.printerName = StringUtils.substringBetween(lines.get(0),
					"device for ", ":");
			logger.debug("Device for [result : " + this.printerName + "]");

		}

		try {
			lines = executeProcess("lpstat", "-v", this.printerName);
		} catch (TimeoutException e) {
			return null;
		}

		logger.debug("lpstat -v " + this.printerName + "");

		if (lines.isEmpty()) {
			logger.debug("lpstat -v " + this.printerName + " [result : EMPTY]");
			return null;
		}

		this.printerUrl = StringUtils.substringAfter(lines.get(0),
				"device for " + this.printerName + ": ");
		logger.debug("Device for [result : " + this.printerName + "]");

		if (this.printerUrl.startsWith("hp")) {
			logger.debug("PRINTER STARTS WITH HP");
			this.printerQuery = new HPLIPPrinterStatusQuery(this.printerUrl);
			Map<String, Object> changed;
			try {
				changed = this.printerQuery.updateStatus();
			} catch (TimeoutException e) {
				this.printerQuery = null;

				return null;
			}
			return changed;
		}

		logger.debug("Finish init() method with null");

		return null;
	}

	/**
	 * Update cups status.
	 * 
	 * @param changed
	 *            the changed
	 * @throws IOException
	 *             Signals that an I/O exception has occurred.
	 */
	private void updateCupsStatus(Map<String, Object> changed)
			throws IOException {

		if (StringUtils.isEmpty(this.printerName) || changed == null) {
			logger.debug("update cups status [PRINTERNAME='" + this.printerName
					+ "', changed == null ] return;");
			return;
		}

		Boolean lastValue = this.enabled;
		logger.debug("lastValue = " + lastValue);

		List<String> lines;
		try {
			lines = executeProcess("lpstat", "-p", this.printerName);
		} catch (TimeoutException e) {

			return;
		}

		logger.debug("lpstat -p '" + this.printerName + "'");

		if (lines.isEmpty() == false) {

			String disabledStr = StringUtils.substringAfter(lines.get(0),
					"printer " + this.printerName + " ");
			logger.debug("disabledStr : '" + disabledStr + "'");
			if (disabledStr.startsWith("disabled")) {

				this.enabled = false;
			} else {

				this.enabled = true;
			}
		}

		logger.debug("enabled : '" + this.enabled + "'");

		if (lastValue == null
				|| lastValue.booleanValue() != this.enabled.booleanValue()) {
			changed.put("enabled", this.enabled);
		}
		return;
	}

	/**
	 * Execute process.
	 * 
	 * @param command
	 *            the command
	 * @return the list
	 * @throws IOException
	 *             Signals that an I/O exception has occurred.
	 * @throws TimeoutException
	 *             the timeout exception
	 */
	public static List<String> executeProcess(String... command)
			throws IOException, TimeoutException {
		ProcessBuilder pb = new ProcessBuilder(command);
		Process process = pb.start();
		logger.debug("Processing command : '" + command[0] + "'");
		try {
			boolean running = true;
			long timeout = System.currentTimeMillis() + 7000L;

			while (running) {
				if (System.currentTimeMillis() > timeout) {
					logger.debug("'" + command[0] + "' Throwing timeout: "
							+ timeout);
					List<String> status = new ArrayList<String>();
					status.add("state: unknow");
					return status;
				}

				try {
					int exitValue = process.exitValue();
					logger.debug("'" + command[0] + "'  finished with value ["
							+ exitValue + "]");
					running = false;
				} catch (IllegalThreadStateException e) {
				}
			}

			return IOUtils.readLines(process.getInputStream());
		} finally {
			process.destroy();
		}
	}

	/**
	 * Getter of <code>printerName</code> property.
	 * 
	 * @return <code>printerName</code> property value.
	 * @see #printerName
	 */
	public String getPrinterName() {
		return this.printerName;
	}

	/**
	 * Getter of <code>printerUrl</code> property.
	 * 
	 * @return <code>printerUrl</code> property value.
	 * @see #printerUrl
	 */
	public String getPrinterUrl() {
		return this.printerUrl;
	}

	/**
	 * Adds the listener.
	 * 
	 * @param listener
	 *            the listener
	 */
	public void addListener(IPrinterStatusChangedListener listener) {
		this.listeners.add(listener);
	}

	/**
	 * Removes the listener.
	 * 
	 * @param listener
	 *            the listener
	 */
	public void removeListener(IPrinterStatusChangedListener listener) {
		this.listeners.remove(listener);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * com.smartmatic.waffle.ecm.core.printer.IPrinterStatusQuery#getDeviceState
	 * ()
	 */
	@Override
	public DeviceState getDeviceState() {
		if (this.printerQuery != null) {

			synchronized (this.printerQuery) {

				return this.printerQuery.getDeviceState();
			}

		} else {
			return DeviceState.NOT_FOUND;
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * com.smartmatic.waffle.ecm.core.printer.IPrinterStatusQuery#updateStatus
	 * ()
	 */
	@Override
	public Map<String, Object> updateStatus() throws IOException {

		if (this.printerQuery != null) {

			synchronized (this.printerQuery) {

				try {
					return this.printerQuery.updateStatus();
				} catch (TimeoutException e) {
					return new HashMap<String, Object>();
				}
			}

		} else {
			return new HashMap<String, Object>();
		}

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * com.smartmatic.waffle.ecm.core.printer.IPrinterStatusQuery#getPrinterStatus
	 * ()
	 */
	@Override
	public StatusPrinter getPrinterStatus() {

		if (this.printerQuery != null) {
			synchronized (this.printerQuery) {

				return this.printerQuery.getPrinterStatus();
			}

		} else {
			return StatusPrinter.OFFLINE;
		}

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.smartmatic.waffle.ecm.core.printer.IPrinterStatusQuery#
	 * getInkLevelPercent()
	 */
	@Override
	public int getInkLevelPercent() {

		if (this.printerQuery != null) {
			synchronized (this.printerQuery) {

				return this.printerQuery.getInkLevelPercent();
			}

		} else {
			return 0;
		}
	}

	/**
	 * Getter of <code>enabled</code> property.
	 * 
	 * @return <code>enabled</code> property value.
	 * @see #enabled
	 */
	public Boolean isEnabled() {
		return this.enabled;
	}

	/**
	 * Setter of <code>enabled</code> property.
	 * 
	 * @param enabled
	 *            The <code>enabled</code> to set.
	 * @see #enabled
	 */
	public void setEnabled(Boolean enabled) {
		this.enabled = enabled;
	}

	/**
	 * Getter of <code>printerQuery</code> property.
	 * 
	 * @return <code>printerQuery</code> property value.
	 * @see #printerQuery
	 */
	public IPrinterStatusQuery getPrinterQuery() {
		return this.printerQuery;
	}
}
