/**
 * ECMBulkDataLoadServiceImpl.java
 *
 * Revision: $Revision$  (Last Modified: $Date$)
 * Last Modified by: [$Author$]
 * 
 * Copyright (c) 2004 Smartmatic Corp.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of
 * Smartmatic Corp. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Corp.
 */
package com.smartmatic.waffle.ecm.service.impl;

import static com.smartmatic.waffle.ecm.service.lang.BulkDataLoadException.BulkDataLoadExceptionOperationExceptionInfo.UNEXPECTED_ERROR_EXECUTING_BULK_DATA_LOAD;

import java.io.File;
import java.io.IOException;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.List;

import javax.annotation.Resource;
import javax.ejb.EJB;
import javax.ejb.Local;
import javax.ejb.Remote;
import javax.ejb.Stateless;
import javax.sql.DataSource;

import org.apache.commons.io.FileUtils;
import org.apache.commons.lang.StringUtils;

import com.smartmatic.waffle.domain.contest.ContestData;
import com.smartmatic.waffle.ecm.bulk.service.lang.BulkLoadException;
import com.smartmatic.waffle.ecm.bulk.service.spec.IBulkServiceLocal;
import com.smartmatic.waffle.ecm.data.transfer.objects.BulkResultDTO;
import com.smartmatic.waffle.ecm.data.transfer.objects.EMLTransformationResultDTO;
import com.smartmatic.waffle.ecm.data.transfer.type.BulkStoreMode;
import com.smartmatic.waffle.ecm.data.transfer.type.EMLFileType;
import com.smartmatic.waffle.ecm.data.transfer.type.SmttFileType;
import com.smartmatic.waffle.ecm.service.lang.BulkDataLoadException;
import com.smartmatic.waffle.ecm.service.lang.BulkDataLoadException.BulkDataLoadExceptionOperationExceptionInfo;
import com.smartmatic.waffle.ecm.service.lang.TransformationException;
import com.smartmatic.waffle.ecm.service.lang.UtilitiesException;
import com.smartmatic.waffle.ecm.service.spec.IECMBulkDataLoadService;
import com.smartmatic.waffle.ecm.service.spec.IECMBulkDataLoadServiceLocal;
import com.smartmatic.waffle.ecm.service.spec.IECMUtilitiesServiceLocal;
import com.smartmatic.waffle.ecm.service.transformData.DataTransformation;
import com.smartmatic.waffle.log.SMTTLogger;
import com.smartmatic.waffle.properties.ApplicationProperties;

/**
 * Session bean that handles all server side operations related with Bulk Data
 * Load for CODI and SMTT files. This class implements all the server services
 * defined in <code>IECMBulkDataLoadService</code>
 */
@Stateless(name = IECMBulkDataLoadService.SERVICE_NAME)
@Remote(IECMBulkDataLoadService.class)
@Local(IECMBulkDataLoadServiceLocal.class)
public class ECMBulkDataLoadServiceImpl implements IECMBulkDataLoadServiceLocal {

	/**
	 * Instance of IBulkServiceLocal
	 */
	@EJB
	private IBulkServiceLocal bulkService;

	/**
	 * Instance of IPersistenceServiceLocal
	 */
	@EJB
	private IECMUtilitiesServiceLocal utilitiesService;

	/**
	 * DataTransformation instance
	 */
	protected DataTransformation dataTransformation;

	/**
	 * ECM Data source
	 */
	@Resource(mappedName = "java:ECM_DS")
	private DataSource dataSource;

	/**
	 * Static reference to log any info or error which occurred executing
	 * methods from this class.
	 */
	private static SMTTLogger logger = SMTTLogger
			.getLogger(ECMBulkDataLoadServiceImpl.class);

	/**
	 * Sets the instance of IBulkServiceLocal.
	 * 
	 * @param bulkService
	 *            the new instance of IBulkServiceLocal
	 */
	public void setBulkService(IBulkServiceLocal bulkService) {
		this.bulkService = bulkService;
	}

	/**
	 * @inheritDoc
	 */
	@Override
	public BulkResultDTO uploadSmttFileIntoDataBase(String fileToUploadName,
			byte[] fileToUploadContent, SmttFileType fileType,
			byte[] resourceFileContent) throws BulkDataLoadException {
		try {

			initDataTransformation();

			String templateName = "bulk.template.path." + fileType;

			String kettleTransformationFileName = findTemplateName(templateName);

			if (fileType.equals(SmttFileType.SITES)) {
				List<byte[]> xmlFileList = this.dataTransformation
						.executeSitesFilesTransformation(
								kettleTransformationFileName,
								fileToUploadContent, false);

				BulkResultDTO bulkResultDTO = executeSmttSiteBulkDataLoad(
						xmlFileList, resourceFileContent, fileToUploadName);

				return bulkResultDTO;
			} else {
				byte[] xmlFile = this.dataTransformation
						.executeSmttFilesTransformation(
								kettleTransformationFileName,
								fileToUploadContent);

				BulkResultDTO bulkResultDTO = executeSmttBulkDataLoad(xmlFile,
						resourceFileContent, fileToUploadName, fileType);

				return bulkResultDTO;
			}

        } catch (TransformationException ex) {
            logger.error(ex, "Error processing Smtt files");
            throw new BulkDataLoadException(ex, ex.getInfo());

        } catch (Exception ex) {
            logger.error(ex, "Error uploading Smtt files into data base");
            throw new BulkDataLoadException(
                ex, UNEXPECTED_ERROR_EXECUTING_BULK_DATA_LOAD
            );
        }
    }

	/**
	 * @inheritDoc
	 */
	@Override
	public BulkResultDTO uploadEMLFilesIntoDataBase(String fileToUploadName,
			byte[] fileToUploadContent, EMLFileType fileType,
			byte[] resourceFileContent) throws BulkDataLoadException {
		try {

			initDataTransformation();

			if (fileType.equals(EMLFileType.SITES)) {

				String kettleTransformationFileName = findKettleTemplateName(EMLFileType.SITES);

				boolean useRegionNisAsPollingPlaceCode = this.utilitiesService
						.getActiveElection().isUseRegionNisAsPollingPlaceCode();

				List<byte[]> xmlFileList = this.dataTransformation
						.executeSitesFilesTransformation(
								kettleTransformationFileName,
								fileToUploadContent,
								useRegionNisAsPollingPlaceCode);

				BulkResultDTO bulkResultDTO = executeSmttSiteBulkDataLoad(
						xmlFileList, resourceFileContent, fileToUploadName);

				return bulkResultDTO;
			} else {

				String kettleTransformationFileName = findKettleTemplateName(EMLFileType.ELECTION);

				EMLTransformationResultDTO transformationResultDTO = this.dataTransformation
						.executeEMLFilesTransformation(
								kettleTransformationFileName,
								fileToUploadContent);

				BulkResultDTO bulkResultDTO = executeEMLBulkDataLoad(
						transformationResultDTO, resourceFileContent,
						fileToUploadName, fileType);

				String contestCustomCode = transformationResultDTO
						.getContestCustomCode();

				saveEMLFileInServer(fileToUploadName, fileToUploadContent);
				setEMLFileNameToContest(fileToUploadName, contestCustomCode);

				return bulkResultDTO;
			}
		} catch (Exception e) {
			logger.error(e, StringUtils.defaultString(e.getMessage()));
			throw new BulkDataLoadException(
					e,
					BulkDataLoadExceptionOperationExceptionInfo.UNEXPECTED_ERROR_EXECUTING_BULK_DATA_LOAD,
					StringUtils.defaultString(e.getMessage()));
		}
	}

	/**
	 * Search the file name of template in the properties file.
	 * 
	 * @param templateName
	 *            the template name
	 * 
	 * @return the name of tamplate used to bulk upload
	 */
	protected String findTemplateName(String templateName) {
		String referenceFileName = ApplicationProperties.getValue(StringUtils
				.lowerCase(templateName));
		return referenceFileName;
	}

	/**
	 * Search the file name of the template to transform files into a property
	 * file.
	 * 
	 * @param templateName
	 *            the template name
	 * 
	 * @return the name the template used to bulk upload
	 */
	protected String findKettleTemplateName(EMLFileType emlFileType) {
		String propertyFileKey = "bulk.eml.template.path." + emlFileType;

		String referenceFileName = ApplicationProperties.getValue(StringUtils
				.lowerCase(propertyFileKey));

		return referenceFileName;
	}

	/**
	 * Creates an instance of DataTransformation
	 */
	protected void initDataTransformation() {
		if (this.dataTransformation == null) {
			this.dataTransformation = new DataTransformation();
		}
	}

	/**
	 * Saves the EML file in server.
	 * 
	 * @param fileToUploadName
	 *            EML file name
	 * @param fileToUploadContent
	 *            EML file content
	 * @throws IOException
	 *             Signals that an I/O exception has occurred.
	 */
	protected void saveEMLFileInServer(String fileToUploadName,
			byte[] fileToUploadContent) throws IOException {

		File emlDir = this.utilitiesService.getEMLFilesDir();

		if (!emlDir.exists()) {
			emlDir.mkdirs();
		}

		File eml = new File(emlDir, fileToUploadName);

		FileUtils.writeByteArrayToFile(eml, fileToUploadContent);
	}

	/**
	 * Sets the EML file name to contest.
	 * 
	 * @param fileToUploadName
	 *            name to set in contest
	 * @param contestCustomCode
	 *            the custom code to search the contest and set the EML file
	 *            name
	 * @throws UtilitiesException
	 *             signals that an exception has occurred retrieving the object
	 * @throws BulkDataLoadException
	 *             signals that an exception has occurred updating the
	 *             IBaseDataObject object in the database
	 */
	protected void setEMLFileNameToContest(String fileToUploadName,
			String contestCustomCode) throws UtilitiesException,
			BulkDataLoadException {

		ContestData contestLoaded = this.utilitiesService.getItemByCustomCode(
				contestCustomCode, ContestData.class);

		contestLoaded.setCodiFile(fileToUploadName);

		this.utilitiesService.updateObject(contestLoaded);
	}

	/**
	 * Executes the bulk data load to database of an XML-transformed smartmatic
	 * file.
	 * 
	 * @param xmlFile
	 *            XML result file, in byte-representation, of the
	 *            XML-transformation of the smartmatic file
	 * @param zipFile
	 *            the resources file
	 * @param fileName
	 *            the file name
	 * @param fileType
	 *            the file type
	 * @return a dto with the result of bulk data load
	 * 
	 * @throws BulkLoadException
	 *             signals that an exception has occurred executing bulk upload
	 */
	private BulkResultDTO executeSmttBulkDataLoad(byte[] xmlFile,
			byte[] zipFile, String fileName, SmttFileType fileType)
			throws BulkLoadException {

		if (zipFile != null) {
			BulkResultDTO bulkResultDTO = this.bulkService.bulkLoad(xmlFile,
					zipFile, null, BulkStoreMode.PERSIST);

			return bulkResultDTO;
		} else {
			BulkResultDTO bulkResultDTO = this.bulkService.bulkLoad(xmlFile,
					null, BulkStoreMode.PERSIST);

			return bulkResultDTO;
		}
	}

	/**
	 * Executes the bulk data load to database of an XML-transformed smartmatic
	 * site file.
	 * 
	 * @param xmlFileList
	 *            list of the XML result files, in byte-representation, of the
	 *            XML-transformation of the smartmatic site file
	 * @param resourceFileContent
	 *            the resources file
	 * @param fileToUploadName
	 *            the file name
	 * 
	 * @return a dto with the result of bulk data load
	 * 
	 * @throws BulkLoadException
	 *             signals that an exception has occurred executing bulk upload
	 */
	private BulkResultDTO executeSmttSiteBulkDataLoad(List<byte[]> xmlFileList,
			byte[] zipFile, String fileName) throws BulkLoadException {

		BulkResultDTO finalBulkUploadResult = new BulkResultDTO();

		for (byte[] indexFile : xmlFileList) {
			if (zipFile != null) {
				BulkResultDTO bulkUploadResult = this.bulkService.bulkLoad(
						indexFile, zipFile, null, BulkStoreMode.PERSIST);

				finalBulkUploadResult
						.setCorrectCount(finalBulkUploadResult
								.getCorrectCount()
								+ bulkUploadResult.getCorrectCount());
				finalBulkUploadResult.setIncorrectCount(finalBulkUploadResult
						.getIncorrectCount()
						+ bulkUploadResult.getIncorrectCount());

				finalBulkUploadResult.getErrorReportList().addAll(
						bulkUploadResult.getErrorReportList());

				if (bulkUploadResult.getIncorrectCount() > 0) {
					break;
				}
			} else {
				BulkResultDTO bulkUploadResult = this.bulkService.bulkLoad(
						indexFile, null, BulkStoreMode.PERSIST);

				finalBulkUploadResult
						.setCorrectCount(finalBulkUploadResult
								.getCorrectCount()
								+ bulkUploadResult.getCorrectCount());
				finalBulkUploadResult.setIncorrectCount(finalBulkUploadResult
						.getIncorrectCount()
						+ bulkUploadResult.getIncorrectCount());

				finalBulkUploadResult.getErrorReportList().addAll(
						bulkUploadResult.getErrorReportList());

				if (bulkUploadResult.getIncorrectCount() > 0) {
					break;
				}
			}
		}

		return finalBulkUploadResult;
	}

	/**
	 * Executes the bulk data load to database of an XML-transformed EML and
	 * site file.
	 * 
	 * @param EMLTransformationResultDTO
	 *            object with the list of the XML result files, in
	 *            byte-representation, of the XML-transformation of the EML or
	 *            site file and the contest CustomCode loaded in case of EML
	 *            file
	 * @param resourceFileContent
	 *            the resources file
	 * @param fileToUploadName
	 *            the file name
	 * 
	 * @return a dto with the result of bulk data load
	 * 
	 * @throws BulkLoadException
	 *             signals that an exception has occurred executing bulk upload
	 */
	private BulkResultDTO executeEMLBulkDataLoad(
			EMLTransformationResultDTO emlTransformationResultDTO,
			byte[] resourceFileContent, String fileToUploadName,
			EMLFileType fileType) throws BulkLoadException {

		BulkStoreMode mode;

		BulkResultDTO finalBulkUploadResult = new BulkResultDTO();

		List<byte[]> xmlFileList = emlTransformationResultDTO
				.getResultFilesContent();

		int i = 1;

		for (byte[] indexFile : xmlFileList) {
			if (i == 2) {
				mode = BulkStoreMode.ADD;
			} else {
				mode = BulkStoreMode.PERSIST;
			}
			if (resourceFileContent != null) {
				BulkResultDTO bulkUploadResult = this.bulkService.bulkLoad(
						indexFile, resourceFileContent, null, mode);

				finalBulkUploadResult
						.setCorrectCount(finalBulkUploadResult
								.getCorrectCount()
								+ bulkUploadResult.getCorrectCount());
				finalBulkUploadResult.setIncorrectCount(finalBulkUploadResult
						.getIncorrectCount()
						+ bulkUploadResult.getIncorrectCount());

				finalBulkUploadResult.getErrorReportList().addAll(
						bulkUploadResult.getErrorReportList());

				if (bulkUploadResult.getIncorrectCount() > 0) {
					break;
				}

			} else {

				BulkResultDTO bulkUploadResult = this.bulkService.bulkLoad(
						indexFile, null, mode);

				finalBulkUploadResult
						.setCorrectCount(finalBulkUploadResult
								.getCorrectCount()
								+ bulkUploadResult.getCorrectCount());
				finalBulkUploadResult.setIncorrectCount(finalBulkUploadResult
						.getIncorrectCount()
						+ bulkUploadResult.getIncorrectCount());

				finalBulkUploadResult.getErrorReportList().addAll(
						bulkUploadResult.getErrorReportList());

				if (bulkUploadResult.getIncorrectCount() > 0) {
					break;
				}

			}

			i++;
		}
		ECMBulkDataLoadServiceImpl.logger
				.info("Finishing executeCodiBulkDataLoad");
		return finalBulkUploadResult;
	}

	/**
	 * @inheritDoc
	 */
	@Override
	public void checkConfiguredRegionLaguages() throws BulkDataLoadException {

		int regionBadConfigured = 0;
		Connection connection = null;
		PreparedStatement preparedStatement = null;
		ResultSet resultSet = null;

		StringBuilder sqlQuery = new StringBuilder();
		sqlQuery.append("SELECT COUNT(reChild.region_code) ");
		sqlQuery.append("FROM region re INNER JOIN ");
		sqlQuery.append("region_all_parent rap ON re.region_code = rap.region_parent_code INNER JOIN ");
		sqlQuery.append("region reChild ON rap.region_child_code = reChild.region_code INNER JOIN ");
		sqlQuery.append("region_election_language rel ON rel.region_code = reChild.region_code INNER JOIN ");
		sqlQuery.append("election_language el ON el.election_language_code = rel.election_language_code ");
		sqlQuery.append("WHERE re.audio_voting_enable = true ");
		sqlQuery.append("AND reChild.category_code = (SELECT MAX(category_code) from category) ");
		sqlQuery.append("AND (el.voice_name_hdr IS NULL OR el.voice_name_dat IS NULL) ");

		try {
			connection = this.dataSource.getConnection();
			preparedStatement = connection
					.prepareStatement(sqlQuery.toString());

			resultSet = preparedStatement.executeQuery();

			if (resultSet != null) {
				while (resultSet.next()) {
					regionBadConfigured = resultSet.getInt(1);
				}
			}

		} catch (Exception e) {
			logger.error(e, StringUtils.defaultString(e.getMessage()));
			throw new BulkDataLoadException(
					e,
					BulkDataLoadExceptionOperationExceptionInfo.UNEXPECTED_ERROR_EXECUTING_BULK_DATA_LOAD,
					StringUtils.defaultString(e.getMessage()));
		} finally {
			try {
				if (resultSet != null) {
					resultSet.close();
				}
				if (preparedStatement != null) {
					preparedStatement.close();
				}
				if (connection != null) {
					connection.close();
				}
			} catch (SQLException e) {
				logger.error(e, StringUtils.defaultString(e.getMessage()));
				throw new BulkDataLoadException(
						e,
						BulkDataLoadExceptionOperationExceptionInfo.UNEXPECTED_ERROR_EXECUTING_BULK_DATA_LOAD,
						StringUtils.defaultString(e.getMessage()));
			}
		}

		if (regionBadConfigured > 0) {
			throw new BulkDataLoadException(
					BulkDataLoadExceptionOperationExceptionInfo.BADLY_CONFIGURED_AUDIO_VOTING_FOR_REGION);
		}
	}
}