/**
 * ECMElectoralDemographyServiceImpl.java
 *
 * Revision: $Revision$  (Last Modified: $Date$)
 * Last Modified by: [$Author$]
 * 
 * Copyright (c) 2004 Smartmatic Corp.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of
 * Smartmatic Corp. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Corp.
 */
package com.smartmatic.waffle.ecm.service.impl;

import java.sql.CallableStatement;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.annotation.Resource;
import javax.ejb.EJB;
import javax.ejb.Local;
import javax.ejb.Remote;
import javax.ejb.Stateless;
import javax.sql.DataSource;

import org.apache.commons.lang.StringUtils;

import com.smartmatic.waffle.domain.contest.ContestClassData;
import com.smartmatic.waffle.domain.contest.ContestData;
import com.smartmatic.waffle.domain.contest.ContestGroupData;
import com.smartmatic.waffle.domain.demographics.DistrictData;
import com.smartmatic.waffle.domain.demographics.RegionData;
import com.smartmatic.waffle.domain.device.LogicalVotingGroupData;
import com.smartmatic.waffle.domain.i18n.LanguageData;
import com.smartmatic.waffle.ecm.data.transfer.comparators.ContestCustomCodeComparator;
import com.smartmatic.waffle.ecm.data.transfer.objects.ContestContestClassDTO;
import com.smartmatic.waffle.ecm.service.lang.ElectoralDemographyException;
import com.smartmatic.waffle.ecm.service.lang.ElectoralDemographyException.ElectoralDemographyExceptionInfo;
import com.smartmatic.waffle.ecm.service.spec.IECMDemographyServiceLocal;
import com.smartmatic.waffle.ecm.service.spec.IECMElectoralDemographyService;
import com.smartmatic.waffle.ecm.service.spec.IECMElectoralDemographyServiceLocal;
import com.smartmatic.waffle.log.SMTTLogger;
import com.smartmatic.waffle.persistence.search.SearchExpression;
import com.smartmatic.waffle.persistence.search.criteria.CriteriaFactory;
import com.smartmatic.waffle.persistence.service.spec.IPersistenceServiceLocal;

/**
 * Session beans that handles all server side operations related with Electoral
 * Organizations. This class implements all the server services defined in
 * <code>ECMElectoralDemographyService</code>
 */
@Stateless(name = IECMElectoralDemographyService.SERVICE_NAME)
@Remote(IECMElectoralDemographyService.class)
@Local(IECMElectoralDemographyServiceLocal.class)
public class ECMElectoralDemographyServiceImpl implements
		IECMElectoralDemographyServiceLocal {

	/** Object to generate logs during the application execution. */
	private static SMTTLogger logger = SMTTLogger
			.getLogger(ECMElectoralDemographyServiceImpl.class);

	/**
	 * Instance of IECMDemographyServiceLocal
	 */
	@EJB
	IECMDemographyServiceLocal ecmDemographyService;

	/**
	 * PersistenceServiceLocal Instance to do any database persistence
	 * operation.
	 */
	@EJB
	private IPersistenceServiceLocal persistenceService;

	/**
	 * ECM Data source
	 */
	@Resource(mappedName = "java:ECM_DS")
	private DataSource dataSource;

	/**
	 * Sets the persistenceServiceLocal Instance to do any database persistence
	 * operation. This method is for testing proposes.
	 * 
	 * @param persistenceService
	 *            the new persistenceServiceLocal Instance to do any database
	 *            persistence operation
	 */
	public void setPersistenceService(
			IPersistenceServiceLocal persistenceService) {
		this.persistenceService = persistenceService;
	}

	/**
	 * Sets the DataSource Instance to do any database persistence operation.
	 * This method is for testing proposes.
	 * 
	 * @param dataSource
	 *            the new DataService Instance to do any database persistence
	 *            operation
	 */
	public void setDataSource(DataSource dataSource) {
		this.dataSource = dataSource;
	}

	/**
	 * Sets the IECMDemographyServiceLocal Instance. This method is for testing
	 * proposes.
	 * 
	 * @param ecmDemographyService
	 *            the new IECMDemographyServiceLocal Instance.
	 */
	public void setDemographyService(
			IECMDemographyServiceLocal ecmDemographyService) {
		this.ecmDemographyService = ecmDemographyService;
	}

	/**
	 * @inheritDoc
	 */
	@Override
	@SuppressWarnings("unchecked")
	public List<ContestData> getContestByDistrict(
			Set<DistrictData> districtDataSet)
			throws ElectoralDemographyException {

		SearchExpression search = new SearchExpression(ContestData.class);

		search.addSearchCriteria(CriteriaFactory
				.in("district", districtDataSet));

		search.addSearchCriteria(CriteriaFactory.collectionSizeNotEq(
				"eligibleOptions", 0));

		try {

			String[] path = { "contestGroupList" };

			List<ContestData> contestList = (List<ContestData>) this.persistenceService
					.searchObjects(search, path, null);

			return contestList;
		} catch (Exception e) {
			logger.error(e, StringUtils.defaultString(e.getMessage()));
			throw new ElectoralDemographyException(
					e,
					ElectoralDemographyExceptionInfo.PROBLEM_RETRIEVING_CONTEST_BY_DISTRICT,
					StringUtils.defaultString(e.getMessage()));
		}
	}

	/**
	 * @inheritDoc
	 */
	@Override
	public List<ContestData> getContestsByLastLevelRegions(
			List<Long> regionCodes) throws ElectoralDemographyException {

		try {
			List<ContestData> contestsByLastLevelRegionsList = new ArrayList<ContestData>();

			List<RegionData> regionsWithParents = this.ecmDemographyService
					.getRegionsWithParentsByRegionCodes(regionCodes);

			for (RegionData regionWithParents : regionsWithParents) {

				Set<DistrictData> regionDistrictSet = new HashSet<DistrictData>();

				Set<RegionData> regionParents = regionWithParents
						.getAllParents();

				for (RegionData regionParent : regionParents) {
					regionDistrictSet.addAll(regionParent.getDistricts());
				}

				contestsByLastLevelRegionsList
						.addAll(getContestByDistrict(regionDistrictSet));
			}

			return contestsByLastLevelRegionsList;
		} catch (Exception e) {
			logger.error(e, StringUtils.defaultString(e.getMessage()));
			throw new ElectoralDemographyException(
					e,
					ElectoralDemographyExceptionInfo.PROBLEM_RETRIEVING_CONTEST_LIST_BY_LAST_LEVEL_REGION,
					StringUtils.defaultString(e.getMessage()));
		}
	}

	/**
	 * @inheritDoc
	 */
	@Override
	@SuppressWarnings("unchecked")
	public List<ContestGroupData> getContestGroupByLastLevelRegion(
			RegionData lastLevelRegion) throws ElectoralDemographyException {

		try {
			Set<RegionData> regionParent = lastLevelRegion.getAllParents();

			List<Long> regionCodes = new ArrayList<Long>();

			for (Iterator<RegionData> iterator = regionParent.iterator(); iterator
					.hasNext();) {
				RegionData regionData = iterator.next();
				regionCodes.add(regionData.getCode());
			}

			SearchExpression search = new SearchExpression(
					ContestGroupData.class);

			search.createAlias("regions", "regions");

			search.addSearchCriteria(CriteriaFactory.in("regions.code",
					regionCodes));

			String[] path = { "contests" };
			List<ContestGroupData> contestGroupList = (List<ContestGroupData>) this.persistenceService
					.searchObjects(search, path, null);

			return contestGroupList;
		} catch (Exception e) {
			logger.error(e, StringUtils.defaultString(e.getMessage()));
			throw new ElectoralDemographyException(
					e,
					ElectoralDemographyExceptionInfo.PROBLEM_RETRIEVING_CONTEST_GROUP_LIST_BY_LAST_LEVEL_REGION,
					StringUtils.defaultString(e.getMessage()));
		}
	}

	/**
	 * @inheritDoc
	 */
	@Override
	@SuppressWarnings("unchecked")
	public List<ContestGroupData> getContestGroupsLoadedInDatabase()
			throws ElectoralDemographyException {

		try {
			SearchExpression search = new SearchExpression(
					ContestGroupData.class);
			String[] path = { "contests" };
			List<ContestGroupData> contestGroupList = (List<ContestGroupData>) this.persistenceService
					.searchObjects(search, path, null);

			return contestGroupList;
		} catch (Exception e) {
			logger.error(e, StringUtils.defaultString(e.getMessage()));
			throw new ElectoralDemographyException(
					e,
					ElectoralDemographyExceptionInfo.PROBLEM_RETRIEVING_CONTEST_GROUP_LOADED_IN_DATABASE,
					StringUtils.defaultString(e.getMessage()));
		}
	}

	/**
	 * @inheritDoc
	 */
	@Override
	public List<ContestGroupData> makeContestsSpecificContestGroups(
			final List<ContestData> regionContests)
			throws ElectoralDemographyException {

		List<ContestGroupData> contestGroupList = null;
		List<ContestContestClassDTO> contestsContestClass;
		List<ContestGroupData> newContestGroupList = new ArrayList<ContestGroupData>();

		try {

			for (ContestData contest : regionContests) {

				contestGroupList = new ArrayList<ContestGroupData>(
						contest.getContestGroupList());

				for (ContestGroupData indexContestGroup : contestGroupList) {
					List<ContestData> contestListAux = new ArrayList<ContestData>();
					contestListAux.add(contest);
					for (int i = 0; i < contestListAux.size(); i++) {
						if (!indexContestGroup.getContests().contains(
								contestListAux.get(i))) {
							contestListAux.remove(i);
							i--;
						}
					}

					Map<ContestClassData, List<ContestData>> listAuxContestClass = new HashMap<ContestClassData, List<ContestData>>();
					for (ContestData indexContest : contestListAux) {
						if (listAuxContestClass.containsKey(indexContest
								.getContestClass())) {
							listAuxContestClass.get(
									indexContest.getContestClass()).add(
									indexContest);
						} else {
							List<ContestData> listAux = new ArrayList<ContestData>();
							listAux.add(indexContest);
							listAuxContestClass.put(
									indexContest.getContestClass(), listAux);
						}
					}

					contestsContestClass = new ArrayList<ContestContestClassDTO>();
					for (ContestClassData indexContestClass : listAuxContestClass
							.keySet()) {
						ContestContestClassDTO auxDto = new ContestContestClassDTO();
						auxDto.setContestClass(indexContestClass);
						auxDto.setContests(listAuxContestClass
								.get(indexContestClass));
						contestsContestClass.add(auxDto);
					}

					generateContestGroup(contest, indexContestGroup,
							newContestGroupList);
				}
			}

		} catch (Exception e) {
			logger.error(e, StringUtils.defaultString(e.getMessage()));
			throw new ElectoralDemographyException(
					e,
					ElectoralDemographyExceptionInfo.PROBLEM_RETRIEVING_CONTEST_LIST_BY_CONTESTS_PERMUTATION,
					StringUtils.defaultString(e.getMessage()));
		}
		return newContestGroupList;
	}

	/**
	 * @inheritDoc
	 */
	@Override
	public List<ContestGroupData> makeRegionSpecificContestGroups(
			final RegionData lastLevelRegion,
			final List<ContestData> regionContests,
			long contestGroupCodeReference) throws ElectoralDemographyException {
		try {
			// will contain the global (ie, in database) contest groups,
			// filtered out so they only contain contests applying to the
			// current lastLevelRegion.
			final List<ContestGroupData> contestGroupsToArrange;

			// will contain the region-specific contest groups. Each
			// region-specific contest group contains one contest from each
			// contest-class, and all contests in a region-specific contest
			// group come from a single global (ie, in database) contest group
			List<ContestGroupData> newContestGroupList = new ArrayList<ContestGroupData>();

			Collections.sort(regionContests, new ContestCustomCodeComparator());

			if (lastLevelRegion != null) {
				contestGroupsToArrange = getContestGroupByLastLevelRegion(lastLevelRegion);
			} else {
				contestGroupsToArrange = getContestGroupsLoadedInDatabase();
			}

			for (ContestGroupData contestGroup : contestGroupsToArrange) {

				Set<ContestData> allContestsInGroup = contestGroup
						.getContests();

				// calculate the intersection of regionContests and
				// allContestsInGroup

				List<ContestData> regionContestsInGroup = new ArrayList<ContestData>(
						regionContests);

				for (int i = 0; i < regionContestsInGroup.size(); i++) {

					if (!allContestsInGroup.contains(regionContestsInGroup
							.get(i))) {
						regionContestsInGroup.remove(i);
						i--;
					}
				}

				// At this point regionContestsInGroup is the intersection of
				// regionContests and allContestsInGroup, meaning it contains
				// all contests applying to the region and coming from the group
				// we are currently looking at.

				List<List<ContestData>> contestsGroupedByContestClasses = getContestsGroupedByContestClasses(regionContestsInGroup);

				combineContestsIntoRegionSpecificContestGroups(0, contestGroup,
						new ArrayList<ContestData>(),
						contestsGroupedByContestClasses, newContestGroupList,
						contestGroupCodeReference);

				contestGroupCodeReference += newContestGroupList.size();
			}

			return newContestGroupList;
		} catch (Exception e) {
			logger.error(e, StringUtils.defaultString(e.getMessage()));
			throw new ElectoralDemographyException(
					e,
					ElectoralDemographyExceptionInfo.PROBLEM_RETRIEVING_CONTEST_LIST_BY_CONTESTS_PERMUTATION,
					StringUtils.defaultString(e.getMessage()));
		}
	}

	/**
	 * Method that generate a Contest Group.
	 * 
	 * @param contestResult
	 *            Contest to generate the contest group.
	 * @param contestGroup
	 *            Contest Group generated.
	 * @param newContestGroupList
	 *            Parameter passed by reference for the list of contest group
	 *            data.
	 */
	private void generateContestGroup(ContestData contestResult,
			ContestGroupData contestGroup,
			List<ContestGroupData> newContestGroupList) {
		Set<ContestData> list = new HashSet<ContestData>();
		ContestGroupData newContestGroupData = new ContestGroupData();
		String customCode = contestGroup.getCustomCode() + "-";
		customCode += contestResult.getCustomCode();
		list.add(contestResult);
		if (!list.isEmpty()) {
			newContestGroupData.setCode(contestGroup.getCode());
			newContestGroupData.setCustomCode(customCode);
			newContestGroupData.setContestGroupName(contestGroup
					.getContestGroupName());
			newContestGroupData.setOriginalCustomCode(contestGroup
					.getCustomCode());
			newContestGroupData.setDescription(contestGroup.getDescription());
			newContestGroupData.setContests(list);

			newContestGroupList.add(newContestGroupData);
		}
	}

	@Override
	public ContestData getContestByCode(long contestCode)
			throws ElectoralDemographyException {

		SearchExpression search = new SearchExpression(ContestData.class);

		search.addSearchCriteria(CriteriaFactory.eq("code", contestCode));

		try {

			ContestData contest = (ContestData) this.persistenceService
					.searchObject(search, null);

			return contest;
		} catch (Exception e) {
			logger.error(e, StringUtils.defaultString(e.getMessage()));
			throw new ElectoralDemographyException(
					e,
					ElectoralDemographyExceptionInfo.PROBLEM_GETTING_CONTEST_BY_CODE,
					StringUtils.defaultString(e.getMessage()));
		}
	}

	@Override
	public LogicalVotingGroupData getLogicalVotingGroup(String customCode)
			throws ElectoralDemographyException {

		SearchExpression search = new SearchExpression(
				LogicalVotingGroupData.class);

		search.addSearchCriteria(CriteriaFactory.eq("customCode", customCode));

		try {
			LogicalVotingGroupData logicalVotingGroup = (LogicalVotingGroupData) this.persistenceService
					.searchObject(search, null);
			return logicalVotingGroup;
		} catch (Exception e) {
			logger.error(e, StringUtils.defaultString(e.getMessage()));
			throw new ElectoralDemographyException(
					e,
					ElectoralDemographyExceptionInfo.PROBLEM_GETTING_LOGICAL_VOTING_GROUP,
					StringUtils.defaultString(e.getMessage()));
		}
	}

	@SuppressWarnings("unchecked")
	@Override
	public List<LogicalVotingGroupData> getLogicalVotingGroupByContestGroup(
			String customCode, String contestGroupName)
			throws ElectoralDemographyException {

		SearchExpression search = new SearchExpression(
				LogicalVotingGroupData.class);

		search.createAlias("contestGroup", "contestGroup");

		String[] values = customCode.split("-");

		search.addSearchCriteria(CriteriaFactory.like("customCode", "%"
				+ values[1] + "%"));

		search.addSearchCriteria(CriteriaFactory.eq(
				"contestGroup.contestGroupName", contestGroupName));

		try {
			List<LogicalVotingGroupData> logicalVotingGroupList = (List<LogicalVotingGroupData>) this.persistenceService
					.searchObjects(search, null);
			return logicalVotingGroupList;
		} catch (Exception e) {
			logger.error(e, StringUtils.defaultString(e.getMessage()));
			throw new ElectoralDemographyException(
					e,
					ElectoralDemographyExceptionInfo.PROBLEM_GETTING_LOGICAL_VOTING_GROUP_BY_CONTEST_GROUP,
					StringUtils.defaultString(e.getMessage()));
		}
	}

	@Override
	public ContestGroupData getContestGroupByCustomCode(String customCode)
			throws ElectoralDemographyException {

		SearchExpression search = new SearchExpression(ContestGroupData.class);

		search.addSearchCriteria(CriteriaFactory.eq("customCode", customCode));

		try {

			ContestGroupData contestGroupData = (ContestGroupData) this.persistenceService
					.searchObject(search, null);

			return contestGroupData;
		} catch (Exception e) {
			logger.error(e, StringUtils.defaultString(e.getMessage()));
			throw new ElectoralDemographyException(
					e,
					ElectoralDemographyExceptionInfo.PROBLEM_GETTING_CONTEST_GROUP_BY_CUSTOM_CODE,
					StringUtils.defaultString(e.getMessage()));
		}
	}

	@Override
	public List<LanguageData> getDistrictLanguages(String customCode)
			throws ElectoralDemographyException {

		try {
			SearchExpression searchRegion = new SearchExpression(
					DistrictData.class);

			searchRegion.addSearchCriteria(CriteriaFactory.eq("customCode",
					customCode));

			DistrictData districtData = (DistrictData) this.persistenceService
					.searchObject(searchRegion, null);

			Map<String, LanguageData> languageDataMap = new HashMap<String, LanguageData>();

			Set<LanguageData> languageDataSet = new HashSet<LanguageData>();

			for (Iterator<RegionData> iterator = districtData.getRegions()
					.iterator(); iterator.hasNext();) {
				RegionData regionData = iterator.next();

				// regionData = getRegionParents(regionData);

				for (Iterator<RegionData> iterator2 = regionData
						.getAllChildren().iterator(); iterator2.hasNext();) {
					RegionData regionDataSimple = iterator2.next();

					for (Iterator iterator3 = regionDataSimple
							.getElectionsLanguage().iterator(); iterator3
							.hasNext();) {
						LanguageData languageData = (LanguageData) iterator3
								.next();

						languageDataMap.put(languageData.getCustomCode(),
								languageData);

						languageDataSet.add(languageData);
					}
				}

				for (Iterator<RegionData> iterator2 = regionData
						.getAllParents().iterator(); iterator2.hasNext();) {

					RegionData regionDataSimple = iterator2.next();

					for (Iterator iterator3 = regionDataSimple
							.getElectionsLanguage().iterator(); iterator3
							.hasNext();) {
						LanguageData languageData = (LanguageData) iterator3
								.next();

						languageDataMap.put(languageData.getCustomCode(),
								languageData);

						languageDataSet.add(languageData);
					}
				}
			}

			return new ArrayList<LanguageData>(languageDataSet);
		} catch (Exception e) {
			logger.error(e, StringUtils.defaultString(e.getMessage()));
			throw new ElectoralDemographyException(
					e,
					ElectoralDemographyExceptionInfo.PROBLEM_GETTING_DISTRICT_LANGUAGES,
					StringUtils.defaultString(e.getMessage()));
		}
	}

	/**
	 * Given a list of contests, will return a list of lists of these contests,
	 * where each list only contains contests of a same contest class.
	 * 
	 * @param contests
	 *            the list of contests to group
	 * 
	 * @return the contests grouped by contest classes
	 */
	private List<List<ContestData>> getContestsGroupedByContestClasses(
			List<ContestData> contests) {

		// auxiliary map which will allow the grouping of contests by class.
		Map<ContestClassData, List<ContestData>> contestsInContestClassesMap = new HashMap<ContestClassData, List<ContestData>>();

		List<List<ContestData>> contestsGroupedByContestClasses = new ArrayList<List<ContestData>>();

		for (ContestData contest : contests) {

			if (contestsInContestClassesMap.containsKey(contest
					.getContestClass())) {
				contestsInContestClassesMap.get(contest.getContestClass()).add(
						contest);
			} else {
				List<ContestData> listWithPreviouslyUnseenClass = new ArrayList<ContestData>();
				listWithPreviouslyUnseenClass.add(contest);
				contestsInContestClassesMap.put(contest.getContestClass(),
						listWithPreviouslyUnseenClass);
			}
		}

		for (List<ContestData> contestClassContestList : contestsInContestClassesMap
				.values()) {

			contestsGroupedByContestClasses.add(contestClassContestList);
		}

		return contestsGroupedByContestClasses;
	}

	/**
	 * Creates a new region-specific contest group. This new contest group is
	 * added in newContestGroupList (modified via its reference). The new
	 * contest group will have as custom code the concatenation of the custom
	 * code of the referenceContestGroup and all the custom codes of the
	 * contests it contains.
	 * 
	 * @param contestsForGroup
	 *            the list of all contests to be placed in the new group (this
	 *            list is a sublist of the contests in referenceContestGroup)
	 * @param referenceContestGroup
	 *            the global (database-stored) contest group from which all
	 *            contests in contestsForGroup where extracted.
	 * @param newContestGroupList
	 *            list of contest group generated, each contest group take one
	 *            element from each contest list. ie: [ [ EU1,CK1 ] , [ EU1,
	 *            CK2] [EU2,CK1] , [ EU2,CK2] ]. This is the main output of the
	 *            recursive calls.
	 * @param contestGroupCodeReference
	 *            the last contest group code generated for the current election
	 *            when generating this is useful when generating for multiple
	 *            contest to avoid duplicate codes.
	 */
	private void appendNewContestGroup(List<ContestData> contestsForGroup,
			final ContestGroupData referenceContestGroup,
			List<ContestGroupData> newContestGroupList,
			long contestGroupCodeReference) {

		String originalCustomCode = referenceContestGroup.getCustomCode();

		String contestGroupCustomCode = originalCustomCode + "-";

		String contestGroupName = referenceContestGroup.getContestGroupName();

		String contestGroupDescription = referenceContestGroup.getDescription();

		Set<ContestData> contestSetForNewContestGroup = new HashSet<ContestData>();

		Collections.sort(contestsForGroup, new ContestCustomCodeComparator());

		for (ContestData contestData : contestsForGroup) {
			if ((contestData.getContestLink() != null)
					&& (!contestsForGroup
							.contains(contestData.getContestLink()))) {
				continue;
			} else {
				contestGroupCustomCode += contestData.getCustomCode();
				contestSetForNewContestGroup.add(contestData);
			}
		}

		if (!contestSetForNewContestGroup.isEmpty()) {

			ContestGroupData newContestGroupData = new ContestGroupData();

			long lastCode = contestGroupCodeReference;

			if (newContestGroupList.size() > 0) {
				lastCode = newContestGroupList.get(
						newContestGroupList.size() - 1).getCode();

				lastCode++;

			}

			newContestGroupData.setCode(lastCode);

			newContestGroupData.setCustomCode(contestGroupCustomCode);

			newContestGroupData.setContestGroupName(contestGroupName);

			newContestGroupData.setDescription(contestGroupDescription);

			newContestGroupData.setOriginalCustomCode(originalCustomCode);

			newContestGroupData.setContests(contestSetForNewContestGroup);

			newContestGroupList.add(newContestGroupData);
		}
	}

	/**
	 * Recursive method to get all possible contests that a voter can select
	 * (contest group). This new contests group are added in
	 * newContestGroupList. Every recursion step increases the size of the
	 * currentlyBuildingContestList (adds all possible candidates, and goes to
	 * the next stage). Once the currentlyBuildingContestList has one item from
	 * each contest class, it is a complete group, and is added to the
	 * newContestGroupList.
	 * 
	 * @param currentListPosition
	 *            index of the list inside contestGroupedByContestClasses from
	 *            which to pick the new contests to add to the
	 *            currentlyBuildingContestList.
	 * @param currentlyBuildingContestList
	 *            the list of contests we will place in all contest groups built
	 *            in the following recursion steps
	 * @param referenceContestGroup
	 *            the global (database-stored) contest group from which all
	 *            contests in contestsGroupedByContestClasses were extracted.
	 * @param newContestGroupList
	 *            list of contest group generated, each contest group takes one
	 *            element from each contest class. ie: [ [ EU1,CK1 ] , [ EU1,
	 *            CK2] [EU2,CK1] , [ EU2,CK2] ]. This is the main output of the
	 *            recursive calls.
	 * @param contestsGroupedByContestClasses
	 *            holds the contests grouped in sublists coming from the same
	 *            contest classes. ie: [ [ EU1, EU2 ], [ CK1, CK2] ]. Note that
	 *            all contests come from a same global contest group
	 *            (referenceContestGroup). In the example, EU1, EU2, CK1, CK2
	 *            would all be from a contest group ("Belge" for instance) for
	 *            people allowed to vote in those contests. Also note that EU1,
	 *            EU2, CK1, CK2 would all be valid contests for a voter in a
	 *            given place, so this place would be kind of an exceptional
	 *            place, since one has a choice of 2 chambers to vote for and 2
	 *            European colleges. Finally: EU1 and EU2 come from a same "EU"
	 *            class of contests; CK1 and CK2 come from a "CK" class of
	 *            contests.
	 * @param contestGroupCodeReference
	 *            the last contest group code generated for the current election
	 *            when generating this is useful when generating for multiple
	 *            contest to avoid duplicate codes.
	 */
	private void combineContestsIntoRegionSpecificContestGroups(
			int currentListPosition,
			final ContestGroupData referenceContestGroup,
			List<ContestData> currentlyBuildingContestList,
			final List<List<ContestData>> contestsGroupedByContestClasses,
			List<ContestGroupData> newContestGroupList,
			long contestGroupCodeReference) {

		// end of recursion : we have built one new contest group of the
		// required size. We now add this currentlyBuildingContestGroup to the
		// newContestGroupList and will build the next one.
		if (currentlyBuildingContestList.size() == contestsGroupedByContestClasses
				.size()) {
			appendNewContestGroup(currentlyBuildingContestList,
					referenceContestGroup, newContestGroupList,
					contestGroupCodeReference);

			return;
		}

		List<ContestData> nextListOfContestsOfSameClassToProcess = contestsGroupedByContestClasses
				.get(currentListPosition);

		for (ContestData contestData : nextListOfContestsOfSameClassToProcess) {

			List<ContestData> nextContestListToBuild = new ArrayList<ContestData>(
					currentlyBuildingContestList);

			nextContestListToBuild.add(contestData);

			combineContestsIntoRegionSpecificContestGroups(
					currentListPosition + 1, referenceContestGroup,
					nextContestListToBuild, contestsGroupedByContestClasses,
					newContestGroupList, contestGroupCodeReference);

		}
	}

	/**
	 * @inheritDoc
	 */
	@Override
	public Map<Long, ContestGroupData> getContestsForLastLevelRegion(
			long regionCode) throws ElectoralDemographyException {

		Map<Long, ContestGroupData> contestGroupMap = new HashMap<Long, ContestGroupData>();
		ContestGroupData contestGroup = null;
		ContestGroupData contestGroupPivot = null;
		ContestData contest = null;
		ContestData contestLink = null;
		ContestClassData contestClass = null;
		long contestLinkCode = 0;

		Connection connection = null;
		PreparedStatement preparedStatement = null;
		ResultSet resultSet = null;

		StringBuilder sqlQuery = new StringBuilder("SELECT DISTINCT ");
		sqlQuery.append("contest_group_by_region.contest_group_code, contest_group_by_region.contest_group_custom_code, ");
		sqlQuery.append("contest_group_by_region.contest_code, contest_group_by_region.contest_custom_code, ");
		sqlQuery.append("contest_group_by_region.contest_link, contest_group_by_region.contest_link_custom_code, ");
		sqlQuery.append("contest_group_by_region.contest_class_code, contest_by_districts.region_code ");
		sqlQuery.append("FROM (SELECT ");
		sqlQuery.append("contest_group.contest_group_code, contest_group.contest_group_custom_code, ");
		sqlQuery.append("contest.contest_code, contest.contest_custom_code, contest.contest_class_code, ");
		sqlQuery.append("contest.contest_link, contestLink.contest_custom_code AS contest_link_custom_code ");
		sqlQuery.append("FROM contest ");
		sqlQuery.append("INNER JOIN contest_group_contest ON contest.contest_code = contest_group_contest.contest_code ");
		sqlQuery.append("INNER JOIN contest_group ON contest_group_contest.contest_group_code = contest_group.contest_group_code ");
		sqlQuery.append("INNER JOIN region_contest_group ON region_contest_group.contest_group_code = contest_group.contest_group_code ");
		sqlQuery.append("INNER JOIN region regionTop ON regionTop.region_code = region_contest_group.region_code ");
		sqlQuery.append("INNER JOIN region_all_parent ON regionTop.region_code = region_all_parent.region_parent_code ");
		sqlQuery.append("LEFT OUTER JOIN contest contestLink ON contest.contest_link = contestLink.contest_code ");
		sqlQuery.append("WHERE region_all_parent.region_child_code = ?) AS contest_group_by_region ");
		sqlQuery.append("INNER JOIN (SELECT ");
		sqlQuery.append("contest.contest_code  AS contest_code, regionLow.region_code AS region_code ");
		sqlQuery.append("FROM contest ");
		sqlQuery.append("INNER JOIN district ON contest.district_code = district.district_code ");
		sqlQuery.append("INNER JOIN region_district ON region_district.district_code = district.district_code ");
		sqlQuery.append("INNER JOIN region regionTop ON region_district.region_code = regionTop.region_code ");
		sqlQuery.append("INNER JOIN region_all_parent ON region_all_parent.region_parent_code = regionTop.region_code ");
		sqlQuery.append("INNER JOIN region regionlow ON region_all_parent.region_parent_code = regionTop.region_code AND ");
		sqlQuery.append("region_all_parent.region_child_code = regionLow.region_code ");
		sqlQuery.append("WHERE regionLow.region_code = ?) AS contest_by_districts ");
		sqlQuery.append("ON contest_group_by_region.contest_code = contest_by_districts.contest_code ");
		sqlQuery.append("WHERE ? <> (SELECT COUNT(*) ");
		sqlQuery.append("FROM eligible_option ");
		sqlQuery.append("WHERE contest_group_by_region.contest_code = eligible_option.contest_code) ");
		sqlQuery.append("ORDER BY contest_by_districts.region_code, contest_group_by_region.contest_group_code, ");
		sqlQuery.append("contest_group_by_region.contest_custom_code");

		try {
			connection = this.dataSource.getConnection();
			preparedStatement = connection
					.prepareStatement(sqlQuery.toString());

			preparedStatement.setLong(1, regionCode);
			preparedStatement.setLong(2, regionCode);
			preparedStatement.setInt(3, 0);

			resultSet = preparedStatement.executeQuery();

			if (resultSet != null) {
				while (resultSet.next()) {

					contestGroupPivot = new ContestGroupData();
					contestGroupPivot.setCode(resultSet
							.getLong("contest_group_code"));
					contestGroupPivot.setCustomCode(resultSet
							.getString("contest_group_custom_code"));

					contestGroup = contestGroupMap.get(contestGroupPivot
							.getCode());

					if (contestGroup == null) {
						contestGroupPivot
								.setContests(new HashSet<ContestData>());
					} else {
						contestGroupPivot.setContests(contestGroup
								.getContests());
					}

					contest = new ContestData();
					contest.setCode(resultSet.getLong("contest_code"));
					contest.setCustomCode(resultSet
							.getString("contest_custom_code"));
					contest.setContestClassCode(resultSet
							.getLong("contest_class_code"));

					contestLinkCode = resultSet.getLong("contest_link");

					if (contestLinkCode > 0) {
						contestLink = new ContestData();
						contestLink.setCode(contestLinkCode);
						contestLink.setCustomCode(resultSet
								.getString("contest_link_custom_code"));
						contest.setContestLink(contestLink);
					}

					contestClass = new ContestClassData();
					contestClass.setCode(resultSet
							.getLong("contest_class_code"));
					contest.setContestClass(contestClass);

					contestGroupPivot.getContests().add(contest);
					contestGroupMap.put(contestGroupPivot.getCode(),
							contestGroupPivot);

					contestLink = null;
					contestLinkCode = 0;
				}
			}

			return contestGroupMap;

		} catch (Exception e) {
			logger.error(e, StringUtils.defaultString(e.getMessage()));
			throw new ElectoralDemographyException(
					e,
					ElectoralDemographyExceptionInfo.PROBLEM_RETRIEVING_CONTEST_GROUP_LIST_BY_LAST_LEVEL_REGION,
					StringUtils.defaultString(e.getMessage()));
		} finally {
			try {
				if (resultSet != null) {
					resultSet.close();
				}
				if (preparedStatement != null) {
					preparedStatement.close();
				}
				if (connection != null) {
					connection.close();
				}
			} catch (SQLException e) {
				logger.error(e, StringUtils.defaultString(e.getMessage()));
				throw new ElectoralDemographyException(
						e,
						ElectoralDemographyExceptionInfo.PROBLEM_RETRIEVING_CONTEST_GROUP_LIST_BY_LAST_LEVEL_REGION,
						StringUtils.defaultString(e.getMessage()));
			}
		}
	}

	@Override
	public void buildSpecificContestGroup(
			Map<Long, ContestGroupData> contestsForLastLevelRegion,
			long regionCode) throws ElectoralDemographyException {
		Map<ContestClassData, List<ContestData>> listAuxContestClass = null;

		if (contestsForLastLevelRegion != null) {
			for (ContestGroupData contestGroup : contestsForLastLevelRegion
					.values()) {
				listAuxContestClass = new HashMap<ContestClassData, List<ContestData>>();

				Set<ContestData> contestOfContestGroup = contestGroup
						.getContests();

				for (ContestData indexContest : contestOfContestGroup) {
					if (listAuxContestClass.containsKey(indexContest
							.getContestClass())) {
						listAuxContestClass.get(indexContest.getContestClass())
								.add(indexContest);
					} else {
						List<ContestData> listAux = new ArrayList<ContestData>();
						listAux.add(indexContest);
						listAuxContestClass.put(indexContest.getContestClass(),
								listAux);
					}
				}

				List<ContestContestClassDTO> contestContestClass = new ArrayList<ContestContestClassDTO>();

				for (ContestClassData indexContestClass : listAuxContestClass
						.keySet()) {
					ContestContestClassDTO auxDto = new ContestContestClassDTO();
					auxDto.setContestClass(indexContestClass);
					auxDto.setContests(listAuxContestClass
							.get(indexContestClass));
					contestContestClass.add(auxDto);
				}

				this.buildLogicalVotingGroupByContestClasses(0,
						new ArrayList<ContestData>(), contestContestClass,
						contestGroup, regionCode);
			}
		}

	}

	/**
	 * Method recursive to get all permutations for logical voting group by
	 * contest classes.
	 * 
	 * @param posContestClass
	 *            Index to build the permutations
	 * @param listContestResult
	 *            Permutations of contest
	 * @param contestGroup
	 *            Contest group to add to the permutation.
	 * @param regionCode
	 *            The last level region code.
	 * @throws ElectoralDemographyException
	 *             An electoral demography exception to notify the client that
	 *             there is an error persisting the logical voting group
	 * @author Javier Santos(javier.santos@smartmatic.com)
	 */
	private void buildLogicalVotingGroupByContestClasses(int posContestClass,
			List<ContestData> listContestResult,
			List<ContestContestClassDTO> contestContestClass,
			ContestGroupData contestGroup, long regionCode)
			throws ElectoralDemographyException {

		Set<ContestData> list = new HashSet<ContestData>();
		if (listContestResult.size() == contestContestClass.size()) {
			String customCode = contestGroup.getCustomCode() + "-";

			Collections.sort(listContestResult,
					new ContestCustomCodeComparator());

			for (int i = 0; i < listContestResult.size(); i++) {
				if ((listContestResult.get(i).getContestLink() != null)
						&& (!listContestResult.contains(listContestResult
								.get(i).getContestLink()))) {
					continue;
				} else {
					customCode += listContestResult.get(i).getCustomCode();
					list.add(listContestResult.get(i));
				}
			}
			if (!list.isEmpty()) {
				try {
					this.persistLogicalVotingGroup(regionCode, customCode,
							contestGroup.getCode());
				} catch (ElectoralDemographyException e) {
					throw new ElectoralDemographyException(
							e,
							ElectoralDemographyExceptionInfo.PROBLEM_EXECUTING_PERSIST_PROCEDURE,
							StringUtils.defaultString(e.getMessage()));
				}
			}
			return;
		}
		for (int i = 0; i < contestContestClass.get(posContestClass)
				.getContests().size(); i++) {
			List<ContestData> listContestResultAux = new ArrayList<ContestData>(
					listContestResult);
			listContestResultAux.add(contestContestClass.get(posContestClass)
					.getContests().get(i));
			buildLogicalVotingGroupByContestClasses(posContestClass + 1,
					listContestResultAux, contestContestClass, contestGroup,
					regionCode);
		}
	}

	/**
	 * Call procedure to persist logical voting group and their relationship
	 * with the voting booths.
	 * 
	 * @param regionCode
	 *            The last level region code.
	 * @param logicalVotingGroupCustomCode
	 *            The logical voting group custom code.
	 * @param contestGroupCode
	 *            The contest group code.
	 * @throws ElectoralDemographyException
	 *             An electoral demography exception to notify the client that
	 *             there is an error persisting the logical voting group.
	 */
	private void persistLogicalVotingGroup(long regionCode,
			String logicalVotingGroupCustomCode, long contestGroupCode)
			throws ElectoralDemographyException {

		Connection connection = null;
		CallableStatement callableStatement = null;
		ResultSet resultSet = null;

		try {
			connection = this.dataSource.getConnection();
			callableStatement = connection
					.prepareCall("{call persist_logical_voting_group_x_voting_booth(?, ?, ?) }");

			callableStatement.setLong(1, regionCode);
			callableStatement.setString(2, logicalVotingGroupCustomCode);
			callableStatement.setLong(3, contestGroupCode);

			callableStatement.execute();

		} catch (SQLException e) {
			logger.error(e, StringUtils.defaultString(e.getMessage()));
			throw new ElectoralDemographyException(
					e,
					ElectoralDemographyExceptionInfo.PROBLEM_EXECUTING_PERSIST_PROCEDURE,
					StringUtils.defaultString(e.getMessage()));
		} catch (Exception e) {
			logger.error(e, StringUtils.defaultString(e.getMessage()));
			throw new ElectoralDemographyException(
					e,
					ElectoralDemographyExceptionInfo.PROBLEM_EXECUTING_PERSIST_PROCEDURE,
					StringUtils.defaultString(e.getMessage()));
		} finally {
			try {
				if (resultSet != null) {
					resultSet.close();
				}
				if (callableStatement != null) {
					callableStatement.close();
				}
				if (connection != null) {
					connection.close();
				}
			} catch (SQLException e) {
				logger.error(e, StringUtils.defaultString(e.getMessage()));
				throw new ElectoralDemographyException(
						e,
						ElectoralDemographyExceptionInfo.PROBLEM_EXECUTING_PERSIST_PROCEDURE,
						StringUtils.defaultString(e.getMessage()));
			}
		}
	}

	/**
	 * Call procedure to persist logical voting group and their relationship
	 * with the voting booths.
	 * 
	 * @return The max contest group code.
	 * @throws ElectoralDemographyException
	 *             An electoral demography exception to notify the client that
	 *             there is an error getting the max contest group code.
	 */
	@Override
	public long searchMaxContestGroupCode() throws ElectoralDemographyException {
		long maxContestGroupCode = 0;
		Connection connection = null;
		PreparedStatement preparedStatement = null;
		ResultSet resultSet = null;

		try {
			connection = this.dataSource.getConnection();
			preparedStatement = connection
					.prepareCall("SELECT MAX(contest_group_code) FROM CONTEST_GROUP");

			resultSet = preparedStatement.executeQuery();

			if (resultSet != null) {
				while (resultSet.next()) {

					maxContestGroupCode = resultSet.getLong(1);
				}
			}

		} catch (Exception e) {
			logger.error(e, StringUtils.defaultString(e.getMessage()));
			throw new ElectoralDemographyException(
					e,
					ElectoralDemographyExceptionInfo.PROBLEM_GETTING_MAX_CONTEST_GROUP_CODE,
					StringUtils.defaultString(e.getMessage()));
		} finally {
			try {
				if (resultSet != null) {
					resultSet.close();
				}
				if (preparedStatement != null) {
					preparedStatement.close();
				}
				if (connection != null) {
					connection.close();
				}
			} catch (SQLException e) {
				logger.error(e, StringUtils.defaultString(e.getMessage()));
				throw new ElectoralDemographyException(
						e,
						ElectoralDemographyExceptionInfo.PROBLEM_GETTING_MAX_CONTEST_GROUP_CODE,
						StringUtils.defaultString(e.getMessage()));
			}
		}

		return maxContestGroupCode;
	}
}
