/**
 * ECMServerCommonImpl.java
 *
 * Revision: $Revision$  (Last Modified: $Date$)
 * Last Modified by: [$Author$]
 * 
 * Copyright (c) 2004 Smartmatic Corp.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of
 * Smartmatic Corp. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Corp.
 */
package com.smartmatic.waffle.ecm.service.impl;

import java.math.BigInteger;
import java.security.KeyPair;
import java.security.KeyStore;
import java.security.PrivateKey;
import java.security.cert.Certificate;
import java.security.cert.X509Certificate;

import org.apache.commons.codec.binary.Base64;
import org.apache.commons.io.output.ByteArrayOutputStream;
import org.apache.commons.lang.RandomStringUtils;

import com.smartmatic.waffle.domain.eligibleoption.ImageWrapper;
import com.smartmatic.waffle.ecm.data.transfer.objects.ElectionKeyStoreComponentsDTO;
import com.smartmatic.waffle.properties.ApplicationProperties;
import com.smartmatic.waffle.security.asymmetric.AsymmetricKeyManager;
import com.smartmatic.waffle.security.asymmetric.CertificateGenerator;
import com.smartmatic.waffle.security.lang.CryptographicStaticException;
import com.smartmatic.waffle.security.symmetric.SymmetricKeyManager;

/**
 * Class that implement all commons methods used by EJB services.
 */
public class ECMServerCommonImpl {

	/**
	 * Method to generate AES 256 bits symmetric key encoded in Base64
	 * 
	 * @return symmetric key encoded in Base64
	 * 
	 * @throws CryptographicStaticException
	 *             In case decryption fails because of inappropriate system
	 *             configuration. This exception implies that the server is not
	 *             properly configured. If this exception occurs the only way to
	 *             solve the problem is by reconfiguring the server.
	 */
	public static String generateSymmetricKey()
			throws CryptographicStaticException {

		String protectionKey = new String(Base64.encodeBase64(
				SymmetricKeyManager.generateSymmetricKey().getEncoded(), false));

		return protectionKey;
	}

	/**
	 * Creates a random string whose length is the number of characters
	 * specified.
	 * 
	 * @param count
	 *            the length of random string to create use the set of all
	 *            chars.
	 * 
	 * @return the random String.
	 */
	public static String generateRandomString(int count) {
		return RandomStringUtils.random(count, true, false);
	}

	/**
	 * Creates a random string whose length is the number of characters
	 * specified.
	 * 
	 * @param count
	 *            the length of random string to create
	 * @param chars
	 *            the set of chars to choose randoms from. If null, then it will
	 *            use the set of all chars.
	 * 
	 * @return the random String
	 */
	public static String generateRandomString(int count, char[] chars) {
		return RandomStringUtils.random(count, chars);
	}

	/**
	 * Creates a random string whose length is the number of characters
	 * specified.
	 * 
	 * @param count
	 *            the length of random string to create
	 * 
	 * @return the random String encoded numeric
	 */
	public static String generateRandomNumericString(int count) {
		return RandomStringUtils.randomNumeric(count);
	}

	/**
	 * Type of algorithm used to sign the files generated by ECM.
	 */
	public static final int getCertificateKeyLength() {

		int certificateKeyLength = 521;

		if (ApplicationProperties.isApplicationPropertiesInstantiated()) {

			certificateKeyLength = Integer
					.parseInt((String) ApplicationProperties
							.getValue("certificate.key.length"));
		}

		return certificateKeyLength;
	}

	/**
	 * Generates asymmetric keys.
	 * 
	 * @param code
	 *            the code used as a certificate serial number
	 * @param customCode
	 *            the custom code used as a certificate common name.
	 * @param protectionKey
	 *            the protection key used as a certificate passphrase.
	 * 
	 * @return the image wrapper object with the asymmetric key built
	 * @throws Exception
	 *             A signal that an error occurred generating the asymmetric
	 *             key.
	 */
	public static ImageWrapper generateAsymmetricKey(long code,
			String customCode, String protectionKey, String certificateAlias,
			ElectionKeyStoreComponentsDTO electionKeyStoreComponents)
			throws Exception {

		PrivateKey electionPrivateKey = electionKeyStoreComponents
				.getElectionPrivateKey();

		X509Certificate electionX509Certificate = electionKeyStoreComponents
				.getElectionX509certificate();

		BigInteger serial = new BigInteger(Long.toString(code));

		X509Certificate certificate = null;

		String certificatePassphrase = protectionKey;

		String certificatePassword = protectionKey;

		int keyLength = ECMServerCommonImpl.getCertificateKeyLength();

		KeyPair keyPair = AsymmetricKeyManager.generateKeyPair("ECDSA", "BC",
				keyLength);

		String algorithmType = "SHA256withECDSA";

		certificate = CertificateGenerator.generateCertificate(
				keyPair.getPublic(), electionPrivateKey,
				electionX509Certificate, serial, customCode,
				electionX509Certificate.getNotBefore(),
				electionX509Certificate.getNotAfter(), algorithmType);

		Certificate[] chainOfTrust = { certificate };

		KeyStore keyStore = AsymmetricKeyManager.createKeystore(
				keyPair.getPrivate(), certificateAlias, certificatePassphrase,
				certificatePassword, chainOfTrust);

		ByteArrayOutputStream keyStoreByteArray = new ByteArrayOutputStream();

		keyStore.store(keyStoreByteArray, certificatePassphrase.toCharArray());

		ImageWrapper asymmetricKey = new ImageWrapper();

		asymmetricKey.setBinaryData(keyStoreByteArray.toByteArray());

		return asymmetricKey;
	}
}
