/**
 * TransformationException.java
 *
 * Revision: $Revision$  (Last Modified: $Date$)
 * Last Modified by: [$Author$]
 * 
 * Copyright (c) 2004 Smartmatic Corp.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of
 * Smartmatic Corp. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Corp.
 */
package com.smartmatic.waffle.ecm.service.lang;

import java.util.Collections;
import java.util.Map;

import org.apache.commons.lang.StringUtils;

import com.smartmatic.waffle.ecm.core.lang.ECMBaseException;
import com.smartmatic.waffle.ecm.core.lang.IECMExceptionInfo;

/**
 * Class to handle any exception that may occur executing any transformation
 * operation.
 */
public class TransformationException extends ECMBaseException {

    /**
     * Holds the version of the class. It is used by the serialization engine to
     * determine when a class its equivalent to another
     */
    private static final long serialVersionUID = -3594882071113060694L;


    /**
     * Map of tuples {@code (K,V)} where {@code K} represents a given
     * parameter and {@code V} its associated value.
     * 
     * Such map contains contextual information associated to this
     * {@link TransformationException}.
     */
    private final Map<String, String> ctxt;


    /**
     * Instantiates a TransformationException.
     * 
     * @param throwable
     *      a nested exception
     * @param exceptionInfo
     *      an enumeration value regarding the cause of the exception
     * @param exceptionMessage
     *      Exception message of nested exception
     */
    public TransformationException(
        Throwable throwable, IECMExceptionInfo exceptionInfo,
        String exceptionMessage
    ) {
        super(throwable, exceptionInfo, exceptionMessage);
        this.ctxt = Collections.emptyMap();
    }


    /**
     * Instantiates a TransformationException.
     * 
     * @param exceptionInfo
     *      an enumeration value regarding the cause of the exception
     */
    public TransformationException(IECMExceptionInfo exceptionInfo) {
        this(exceptionInfo, Collections.<String,String>emptyMap());
    }


    /**
     * Instantiates a TransformationException.
     * 
     * @param exceptionInfo
     *      an enumeration value regarding the cause of the exception
     * @param ctxt
     *      Map that contains contextual information of the exception
     *      being instantiated. Such contextual information can be used
     *      later to customize exception messages even more!
     *
     * @throws NullPointerException
     *      if {@code ctxt} is {@code null}
     */
    public TransformationException(
        IECMExceptionInfo exceptionInfo, Map<String,String> ctxt
    ) {
        super(exceptionInfo, toMessage(exceptionInfo, ctxt));
        this.ctxt =  Collections.unmodifiableMap(ctxt);
    }


    /**
     * Extracts an exception message from the given parameters.
     *
     * @param info
     *      {@code IECMExceptionInfo} instance
     * @param ctxt
     *      Contextual map
     * @return
     *      {@code String}
     */
    private static String toMessage(
        IECMExceptionInfo info, Map<String,String> ctxt
    ) {
        String ss;           // ss: SearchString
        String theMsg;       // Resulting Message

        theMsg = (info.getExceptionMessage() != null)
                      ? info.getExceptionMessage()
                      : info.getBundleKey();

        for (Map.Entry<String, String> entry : ctxt.entrySet()) {
            ss     = String.format("${%s}", entry.getKey());
            theMsg = StringUtils.replace(theMsg, ss, entry.getValue());
        }
        return theMsg;
    }


    /**
     * {@inheritDoc}
     */
    @Override
    public String getBundleKey() {
        return this.getClass().getSimpleName()
           + "." + this.exceptionInfo.getBundleKey();
    }


    /**
     * Returns a map of contextual information associated to this {@link
     * TransformationException}.
     *
     * @return
     *      {@code Map<String, String>} (never {@code null})
     */
    public Map<String, String> getContext() {
        return this.ctxt;
    }


    /**
     * Enum that contains all the possible reasons for which the
     * TransformationException can be thrown.
     */
    public enum TransformationExceptionInfo implements IECMExceptionInfo {

        DEV_PARAM_INVALID_VALUE(
            "device.parameter.invalid.value",
            "Device parameter: ${param}, has an invalid value."
                + " Provided value: ${value}"
        ),
        FIELD_INVALID_VALUE(
            "field.invalid.value",
            "Field: \"${field}\" (row:${row},col:${col}), has an invalid "
                + "value. Provided value: ${value}"
        ),
        MISSING_REQUIRED_FIELDS_ERROR("missing.required.fields.error"),
        UNEXPECTED_TRANSFORMATION_ERROR("unexpected.transformation.error");

        /**
         * Variable to hold the bundle key to show a user understandable
         * message.
         */
        private String bundleKey;

        /**
         * Variable to hold the exception message.
         */
        private String exceptionMessage;

        /**
         * {@inheritDoc}
         */
        @Override
        public String getBundleKey() {
            return this.bundleKey;
        }

        /**
         * {@inheritDoc}
         */
        @Override
        public String getExceptionMessage() {
            return this.exceptionMessage;
        }

        /**
         * Constructor.
         * 
         * @param bundleKey
         *      Resource bundle's key message
         */
        private TransformationExceptionInfo(String bundleKey) {
            this.bundleKey = bundleKey;
        }

        /**
         * Constructor.
         * 
         * @param bundleKey
         *      Resource bundle's key message
         * @param exceptionMessage
         *      Exception message
         */
        private TransformationExceptionInfo(
            String bundleKey, String exceptionMessage
        ) {
            this.bundleKey        = bundleKey;
            this.exceptionMessage = exceptionMessage;
        }
    }
}
