/**
 * IECMDemographyService.java
 *
 * Revision: $Revision$  (Last Modified: $Date$)
 * Last Modified by: [$Author$]
 * 
 * Copyright (c) 2004 Smartmatic Corp.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of
 * Smartmatic Corp. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Corp.
 */
package com.smartmatic.waffle.ecm.service.spec;

import java.util.List;
import java.util.Set;

import javax.ejb.Remote;

import com.smartmatic.waffle.domain.demographics.PathData;
import com.smartmatic.waffle.domain.demographics.RegionData;
import com.smartmatic.waffle.ecm.data.transfer.objects.ProgrammingUnitLastLevelRegionDTO;
import com.smartmatic.waffle.ecm.service.lang.DemographyException;
import com.smartmatic.waffle.ecm.service.lang.XmlGenerationException;

/**
 * Remote interface which establishes methods contract that must be implemented
 * by <code>ECMDemographyServiceImpl</code> class
 */
@Remote
public interface IECMDemographyService {

	/** SERVICE_NAME Constant. */
	public static final String SERVICE_NAME = "com.smartmatic.waffle.ecm.service.spec.IECMDemographyService";

	/**
	 * Method to retrieve the the Programming Units amount loaded in database.
	 * 
	 * @return Programming units amount loaded in database
	 * 
	 * @throws DemographyException
	 *             Signal that an exception has occurred retrieving the
	 *             Programming Units amount
	 */
	public int retrieveProgrammingUnitsAmount() throws DemographyException;

	/**
	 * Method that sets the protection key to programming unit and update the
	 * object in database.
	 * 
	 * @param pageNumber
	 *            Current page number
	 * @param pageSize
	 *            Page size
	 * @throws DemographyException
	 *             Signal that an exception has occurred updating the
	 *             Programming Units Protection Key
	 */
	public void setProtectionKeyToProgrammingUnit(int pageNumber, int pageSize)
			throws DemographyException;

	/**
	 * Deletes the security components generated for programming units from data
	 * base.
	 * 
	 * @return number of removed records
	 * 
	 * @throws DemographyException
	 *             Signal that an exception has occurred deleting the
	 *             Programming Units Protection Key
	 */
	public int deleteProgrammingUnitSecurityComponents()
			throws DemographyException;

	/**
	 * Gets region data objects (with their parents) by supplying a list of
	 * region codes.
	 * 
	 * @param regionCodes
	 *            the last level region codes
	 * 
	 * @return a region list with their parents included
	 * 
	 * @throws DemographyException
	 *             signals that an exception has occurred retrieving the list of
	 *             last level regions with their parents.
	 */
	List<RegionData> getRegionsWithParentsByRegionCodes(List<Long> regionCodes)
			throws DemographyException;

	/**
	 * Gets only last level region data objects from a list of region data if
	 * provided, if not it will return all last level region data objects stored
	 * in database.
	 * 
	 * @param regionsFilterList
	 *            the region data list to filter the search
	 * @return the last level region data list.
	 * @throws DemographyException
	 *             Signal that an error has occurred getting the last level
	 *             region data objects.
	 */
	List<RegionData> getLastLevelRegions(List<RegionData> regionsFilterList)
			throws DemographyException;

	/**
	 * Gets only last level region data objects(but only region code and region
	 * name of each matched object) from a list of region data if provided, if
	 * not it will return all last level region data objects stored in database.
	 * 
	 * This method will load faster than <code>getLastLevelRegions</code> method
	 * but use only in cases that you do not need more fields that the ones
	 * indicated above.
	 * 
	 * @param regionsFilterList
	 *            the region data list to filter the search
	 * @return the last level region data list.
	 * @throws DemographyException
	 *             Signal that an error has occurred getting the last level
	 *             region data objects.
	 */
	List<RegionData> getLastLevelRegionsLight(List<RegionData> regionsFilterList)
			throws DemographyException;

	/**
	 * Gets a set of programming units DTO from a list of regions.
	 * 
	 * @param regionList
	 *            the region data list to filter the search
	 * @return the set of programming units DTO
	 * @throws DemographyException
	 *             Signal that an error has occurred getting the last level
	 *             region data objects.
	 */
	Set<ProgrammingUnitLastLevelRegionDTO> getProgrammingUnitsByRegions(
			List<RegionData> regionList) throws DemographyException;

	/**
	 * Gets a region with all its parents regions set.
	 * 
	 * @param region
	 *            the region to find a set the parents.
	 * @return the region with all its parents region set.
	 * @throws DemographyException
	 *             A signal that an error has occurred getting all parents
	 *             regions.
	 */
	RegionData getAllParentsRegions(RegionData region)
			throws DemographyException;

	/**
	 * Checks if the given region is associated with the given path.
	 * 
	 * @param region
	 *            the region to check
	 * @param path
	 *            the path which the region must be check
	 * @return the region object if match with the path.
	 * @throws DemographyException
	 *             A signal that an error has occurred checking the region path.
	 */
	RegionData checkRegionParentPath(RegionData region, PathData path)
			throws DemographyException;

	/**
	 * Returns a list of all available regions to fill the regions widget.
	 * 
	 * @return a list of region data objects only with the code set, to load
	 *         faster.
	 * @throws DemographyException
	 *             A signal that an error has occurred getting the available
	 *             regions.
	 */
	List<RegionData> getAvailableRegionsForWidget() throws DemographyException;

	/**
	 * Method to load and assign the all parent for a region.
	 * 
	 * @param region
	 *            Region to find its parent
	 * 
	 * @return Same RegionData object provided with its region all parents
	 * 
	 * @throws XmlGenerationException
	 *             Wrapper of Exception to notify the client that a problem
	 *             occurred
	 * 
	 * @author Frederick Faria (frederick.faria@smartmatic.com)
	 */
	RegionData getAllRegionParentsWithDistricts(RegionData region)
			throws XmlGenerationException;
}
