/**
 * BaseKeyListener.java
 *
 * Revision: $Revision$  (Last Modified: $Date$)
 * Last Modified by: [$Author$]
 * 
 * Copyright (c) 2004 Smartmatic Corp.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of
 * Smartmatic Corp. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Corp.
 */

package com.smartmatic.waffle.ecm.ui;

import java.awt.Component;
import java.awt.Container;
import java.awt.event.ContainerEvent;
import java.awt.event.ContainerListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;

import javax.swing.JDialog;

/**
 * The listener interface for receiving baseKey events. The class that is
 * interested in processing a baseKey event implements this interface, and the
 * object created with that class is registered with a component using the
 * component's <code>addBaseKeyListener<code> method. When
 * the baseKey event occurs, that object's appropriate
 * method is invoked.
 * 
 * @see BaseKeyEvent
 */
public class BaseKeyListener implements KeyListener, ContainerListener {

	/**
	 * Instantiates a new base key listener.
	 */
	public BaseKeyListener() {
		super();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.awt.event.KeyListener#keyPressed(java.awt.event.KeyEvent)
	 */
	@Override
	public void keyPressed(KeyEvent e) {
		int code = e.getKeyCode();
		if (code == KeyEvent.VK_ESCAPE) {
			Component actualParent = (Component) e.getSource();
			while (!(actualParent instanceof JDialog) && actualParent != null) {
				actualParent = actualParent.getParent();
			}
			if (actualParent != null) {
				if (actualParent instanceof BaseFullScreenJDialog) {
					BaseFullScreenJDialog dispatcher = (BaseFullScreenJDialog) actualParent;
					if (dispatcher.getCurrentDispatcher() != null) {
						if (dispatcher.getCurrentDispatcher()
								.isCriticalProcessRunning()) {
							return;
						}
					}
				}
				((JDialog) actualParent).dispose();
			}
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.awt.event.KeyListener#keyReleased(java.awt.event.KeyEvent)
	 */
	@Override
	public void keyReleased(KeyEvent e) {
		return;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.awt.event.KeyListener#keyTyped(java.awt.event.KeyEvent)
	 */
	@Override
	public void keyTyped(KeyEvent e) {
		return;
	}

	/**
	 * This function is called whenever a Component or a Container is added to
	 * another Container belonging to this BaseMDPPanelDispatch.
	 * 
	 * @param e
	 *            the e
	 * @inheritDoc
	 */
	@Override
	public void componentAdded(ContainerEvent e) {
		synchronized (this) {
			this.addKeyAndContainerListenerRecursively(e.getChild());
		}
		return;
	}

	/**
	 * This function is called whenever a Component or a Container is removed
	 * from another Container belonging to this BaseMDPPanelDispatch.
	 * 
	 * @param e
	 *            the e
	 * @inheritDoc
	 */
	@Override
	public void componentRemoved(ContainerEvent e) {
		synchronized (this) {
			this.removeKeyAndContainerListenerRecursively(e.getChild());
		}
		return;
	}

	/**
	 * It is used to prevent anyone calling it from other classes The function
	 * takes a Component as an argument and adds this Dialog as a KeyListener to
	 * it. Besides it checks if the component is actually a Container and if it
	 * is.
	 * 
	 * @param components
	 *            the components
	 */
	public void addKeyAndContainerListenerRecursively(Component components) {
		// Add KeyListener to the Component passed as an argument
		components.addKeyListener(this);
		// Check if the Component is a Container
		if (components instanceof Container) {
			// Component c is a Container. The following cast is safe.
			Container cont = (Container) components;
			// Add ContainerListener to the Container.
			cont.addContainerListener(this);
			// Get the Container's array of children Components.
			Component[] children = cont.getComponents();
			// For every child repeat the above operation.
			for (int i = 0; i < children.length; i++) {
				this.addKeyAndContainerListenerRecursively(children[i]);
			}
		}
	}

	/**
	 * Removes this Dialog from the listener lists of Components.
	 * 
	 * @param components
	 *            the components
	 */
	public void removeKeyAndContainerListenerRecursively(Component components) {
		components.removeKeyListener(this);

		if (components instanceof Container) {

			Container cont = (Container) components;

			cont.removeContainerListener(this);

			Component[] children = cont.getComponents();

			for (int i = 0; i < children.length; i++) {
				this.removeKeyAndContainerListenerRecursively(children[i]);
			}
		}
	}

}
