/**
 * MachineUtils.java
 *
 * Revision: $Revision$  (Last Modified: $Date$)
 * Last Modified by: [$Author$]
 * 
 * Copyright (c) 2004 Smartmatic Corp.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of
 * Smartmatic Corp. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Corp.
 */
package com.smartmatic.waffle.ecm.ui.core;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.net.NetworkInterface;
import java.net.SocketException;
import java.util.Enumeration;
import java.util.Formatter;

import org.apache.commons.io.IOUtils;

import com.smartmatic.waffle.localpersistence.ApplianceCommonDestinations;
import com.smartmatic.waffle.log.SMTTLogger;

/**
 * Class used to retrieve information and perform tasks on the machine.
 */
public class MachineUtils {

	/** The logger. */
	private static SMTTLogger logger = SMTTLogger.getLogger(MachineUtils.class);

	/** Singleton instance of this class. */
	private static MachineUtils singletonInstance = new MachineUtils();

	/**
	 * Instantiates a new machine utils.
	 */
	private MachineUtils() {
		super();
	}

	/**
	 * Obtains the singleton instance from the MachineUtils.
	 * 
	 * @return The singleton instance of the MachineUtils
	 */
	public static MachineUtils getInstance() {
		return MachineUtils.singletonInstance;
	}

	/**
	 * Method used to obtain the server MAC address. This method Mnemoizate the
	 * MAC address of the server to avoid the interaction with the operating
	 * system every time.
	 * 
	 * @return String The MAC address of the server. If the server has multiple
	 *         network interfaces all of those are returned separate by a
	 *         semicolon ";"
	 * @version 1.0 (Jun 23, 2009)
	 * @author Daniel Bastidas daniel.bastidas@smartmatic.com
	 */
	public String getHardwareAddress() {
		String macAddress;

		StringBuilder sb = new StringBuilder();

		try {
			Enumeration<NetworkInterface> e = NetworkInterface
					.getNetworkInterfaces();
			while (e.hasMoreElements()) {
				byte[] b = e.nextElement().getHardwareAddress();

				sb.append(this.getFormattedId(b));

				sb.append(";");
			}
		} catch (SocketException e) {
			logger.error("Error trying to obtain the server MAC address\n"
					+ e.getMessage());
		}

		macAddress = sb.toString().substring(0, sb.length() - 1);
		return macAddress;
	}

	/**
	 * Gets the formatted id.
	 * 
	 * @param byteData
	 *            the byte data
	 * @return the formatted id
	 */
	private String getFormattedId(byte[] byteData) {
		StringBuilder sb = new StringBuilder();
		Formatter f = new Formatter(sb);
		if (byteData != null) {
			for (int i = 0; i < byteData.length; i++) {
				f.format("%02X%s", byteData[i], i < byteData.length - 1 ? ":"
						: "");
			}
		}
		return sb.toString();
	}

	/**
	 * Shutdown machine.
	 * 
	 * @param commonDestination
	 *            the common destination
	 */
	public void shutdownMachine(ApplianceCommonDestinations commonDestination) {

		Process process = null;
		BufferedReader br = null;
		InputStream is = null;
		ProcessBuilder processBuilder = new ProcessBuilder();

		try {
			processBuilder.command("sh", "-c", String.format("sudo %s %s",
					commonDestination.deviceUtilScript.getAbsolutePath(),
					"SHUTDOWN_MACHINE"));

			process = processBuilder.start();

			is = process.getErrorStream();

			br = new BufferedReader(new InputStreamReader(is));

			String aux = br.readLine();

			if (aux != null && aux.length() > 0) {
				logger.error("Is not possible to shutdown the machine " + aux);
			}
		} catch (IOException e) {
			logger.error("There was an error trying shutdown machine "
					+ e.getMessage());
		} finally {
			IOUtils.closeQuietly(br);
			IOUtils.closeQuietly(is);
			if (process != null) {
				process.destroy();
			}
		}
	}
}
