/**
 * LanguageController.java
 *
 * Revision: $Revision$  (Last Modified: $Date$)
 * Last Modified by: [$Author$]
 * 
 * Copyright (c) 2004 Smartmatic Corp.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of
 * Smartmatic Corp. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Corp.
 */
package com.smartmatic.waffle.ecm.ui.core.i18n.controller;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Locale;
import java.util.ResourceBundle;

import javax.swing.JFrame;
import javax.swing.UIManager;

import org.jdesktop.application.ApplicationContext;
import org.jdesktop.application.ResourceMap;

import com.smartmatic.waffle.domain.i18n.LanguageData;
import com.smartmatic.waffle.ecm.ui.DialogListener;
import com.smartmatic.waffle.ecm.ui.StageListener;
import com.smartmatic.waffle.ecm.ui.core.i18n.LanguageSelectionDialog;
import com.smartmatic.waffle.ecm.ui.votingexperience.VotingExperienceDialog;
import com.smartmatic.waffle.log.SMTTLogger;

/**
 * Class that contains utility methods to load the Supported Languages from a
 * configuration File.
 */
public class LanguageController implements DialogListener {

	/** The logger. */
	private static SMTTLogger logger = SMTTLogger
			.getLogger(LanguageController.class);

	/** Singleton instance of this class. */
	private static LanguageController singletonInstance = new LanguageController();

	/**
	 * Singleton running Application class's Context.
	 */
	private ApplicationContext applicationContext = null;

	/**
	 * Holds the list of language loaded from the configuration file loaded.
	 */
	private ArrayList<LanguageData> languageList;

	/** Holds the selected language used in the application. */
	private LanguageData selectedLanguage;

	/** The language selection dialog. */
	private LanguageSelectionDialog languageSelectionDialog;

	/** The stage listener. */
	private StageListener stageListener;

	/**
	 * Obtains the singleton instance from the LanguageSelectionController.
	 * 
	 * @return The singleton instance of the LanguageSelectionController
	 */
	public static LanguageController getInstance() {
		// DNM. Logger.
		// logger.info("Getting instance of LanguageController");
		return LanguageController.singletonInstance;
	}

	/**
	 * Inits the lang selection.
	 * 
	 * @param languageCollection
	 *            the language collection
	 * @throws Exception
	 *             the exception
	 */
	public void initLangSelection(Collection<LanguageData> languageCollection) {
		// Loads the supported languages from the list of configured files
		this.languageList = new ArrayList<LanguageData>();
		this.languageList.addAll(languageCollection);
	}

	/**
	 * Sets the desired default language used by the application. This is
	 * performed asking the user which language he prefers between all the
	 * supported languages.
	 */
	public void startLangSelection() {
		logger.info("Starting language selection");
		if (this.languageList.size() > 1) {
			if (this.languageSelectionDialog == null) {
				this.languageSelectionDialog = new LanguageSelectionDialog();
			}
			this.languageSelectionDialog
					.setSupportedLanguages(this.languageList);
			this.languageSelectionDialog.setListener(this);

			if (this.stageListener == null) {
				new VotingExperienceDialog((JFrame) null, true);
			}
			VotingExperienceDialog.instance
					.setFullPanel(this.languageSelectionDialog);
			VotingExperienceDialog.instance.setVisible(true);
		} else {
			this.selectedLanguage = this.languageList.get(0);
			Locale selectedLocale = this
					.getLocaleFromLanguage(this.selectedLanguage);
			Locale.setDefault(selectedLocale);
			if (this.stageListener != null) {
				this.stageListener.goToNextStage();
			}
		}
		this.initializeDialogLocale();
	}

	/**
	 * Creates a Locale instance from a given language representation. This
	 * method uses the fields language, country and variant to create the Locale
	 * 
	 * @param languageData
	 *            The language that contains the info to creates the locale.
	 * @return The locale that match the language data received or the default
	 *         locale if the param does not contain enough information to create
	 *         a locale
	 */
	public Locale getLocaleFromLanguage(LanguageData languageData) {
		String lang = languageData.getLanguageRFC().toLowerCase();
		String country = languageData.getCountry().toUpperCase();
		String variant = languageData.getVariant();

		if (variant != null && variant.length() > 0) {
			return new Locale(lang, country, variant);
		}

		if (country != null && country.length() > 0) {
			return new Locale(lang, country);
		}

		if (lang != null && lang.length() > 0) {
			return new Locale(lang);
		}

		return Locale.getDefault();
	}

	/**
	 * Getter for the <code>languageList</code> attribute.
	 * 
	 * @return <code>languageList</code> attribute value.
	 * @see LanguageController#languageList
	 */
	public ArrayList<LanguageData> getLanguageList() {
		return this.languageList;
	}

	/**
	 * Getter for the <code>selectedLanguage</code> attribute.
	 * 
	 * @return <code>selectedLanguage</code> attribute value.
	 * @see LanguageController#selectedLanguage
	 */
	public LanguageData getSelectedLanguage() {
		return this.selectedLanguage;
	}

	/**
	 * Setter for the <code>selectedLanguage</code> attribute.
	 * 
	 * @param selectedLanguage
	 *            the selectedLanguage to set
	 * @see LanguageController#selectedLanguage
	 */
	public void setSelectedLanguage(LanguageData selectedLanguage) {
		this.selectedLanguage = selectedLanguage;
	}

	/**
	 * Sets the application context for this Language Context. If the
	 * application context has already been assigned, this method will throw an
	 * IllegalStateException.
	 * 
	 * @param applicationContext
	 *            the application context
	 */
	public void initApplicationContext(ApplicationContext applicationContext) {
		if (this.applicationContext == null) {
			this.applicationContext = applicationContext;
		} else {
			throw new IllegalStateException(
					"The application context for the currently running application has already been set. Apllication contexts cannot change Dynamically");
		}
	}

	/**
	 * Getter for the application context of this language controller. If the
	 * application context has not already been assigned, this method will throw
	 * an IllegalStateException.
	 * 
	 * @return the singleton running Application class's Context
	 */
	public ApplicationContext getApplicationContext() {
		if (this.applicationContext != null) {
			return this.applicationContext;
		}
		throw new IllegalStateException(
				"the application context has not yet been set for the currently running application.");
	}

	/**
	 * Gets the resource map.
	 * 
	 * @return the resource map
	 */
	public ResourceMap getResourceMap() {
		return this.applicationContext.getResourceMap();
	}

	/**
	 * Initializes Dialog Messages with selected Locale.
	 */
	private void initializeDialogLocale() {
		ResourceBundle LangBundle = ResourceBundle
				.getBundle("com/smartmatic/waffle/ecm/ui/resources/generalLang");
		// jOptionPane properties
		UIManager.put("OptionPane.cancelButtonText",
				LangBundle.getString("myDialog.cancelButtonText"));
		UIManager.put("OptionPane.noButtonText",
				LangBundle.getString("myDialog.noButtonText"));
		UIManager.put("OptionPane.yesButtonText",
				LangBundle.getString("myDialog.yesButtonText"));
		UIManager.put("OptionPane.okButtonText",
				LangBundle.getString("myDialog.okButtonText"));
		// FileChooser properties
		UIManager.put("FileChooser.acceptAllFileFilterText",
				LangBundle.getString("myDialog.acceptAllFileFilterText"));
		UIManager.put("FileChooser.lookInLabelText",
				LangBundle.getString("myDialog.lookInLabelText"));
		UIManager.put("FileChooser.openDialogTitleText",
				LangBundle.getString("myDialog.openButtonText"));
		UIManager.put("FileChooser.openButtonText",
				LangBundle.getString("myDialog.openButtonText"));
		UIManager.put("FileChooser.cancelButtonText",
				LangBundle.getString("myDialog.cancelButtonText"));
		UIManager.put("FileChooser.fileNameLabelText",
				LangBundle.getString("myDialog.fileNameLabelText"));
		UIManager.put("FileChooser.filesOfTypeLabelText",
				LangBundle.getString("myDialog.filesOfTypeLabelText"));
		UIManager.put("FileChooser.openButtonToolTipText",
				LangBundle.getString("myDialog.openButtonToolTipText"));
		UIManager.put("FileChooser.cancelButtonToolTipText",
				LangBundle.getString("myDialog.cancelButtonText"));
		UIManager.put("FileChooser.saveButtonText",
				LangBundle.getString("myDialog.saveButtonText"));
		UIManager.put("FileChooser.saveDialogTitleText",
				LangBundle.getString("myDialog.saveButtonText"));
		UIManager.put("FileChooser.fileNameHeaderText",
				LangBundle.getString("myDialog.fileNameLabelText"));
		UIManager.put("FileChooser.upFolderToolTipText",
				LangBundle.getString("myDialog.upFolderToolTipText"));
		UIManager.put("FileChooser.homeFolderToolTipText",
				LangBundle.getString("myDialog.homeFolderToolTipText"));
		UIManager.put("FileChooser.newFolderToolTipText",
				LangBundle.getString("myDialog.newFolderButtonText"));
		UIManager.put("FileChooser.listViewButtonToolTipText",
				LangBundle.getString("myDialog.listViewButtonToolTipText"));
		UIManager.put("FileChooser.newFolderButtonText",
				LangBundle.getString("myDialog.newFolderButtonText"));
		UIManager.put("FileChooser.renameFileButtonText",
				LangBundle.getString("myDialog.renameFileButtonText"));
		UIManager.put("FileChooser.deleteFileButtonText",
				LangBundle.getString("myDialog.deleteFileButtonText"));
		UIManager.put("FileChooser.filterLabelText",
				LangBundle.getString("myDialog.filesOfTypeLabelText"));
		UIManager.put("FileChooser.detailsViewButtonToolTipText",
				LangBundle.getString("myDialog.detailsViewButtonToolTipText"));
		UIManager.put("FileChooser.fileSizeHeaderText",
				LangBundle.getString("myDialog.fileSizeHeaderText"));
		UIManager.put("FileChooser.fileDateHeaderText",
				LangBundle.getString("myDialog.fileDateHeaderText"));
		UIManager.put("FileChooser.saveButtonToolTipText",
				LangBundle.getString("myDialog.saveButtonText"));

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.smartmatic.waffle.ecm.ui.DialogListener#ok()
	 */
	@Override
	public void ok() {
		this.selectedLanguage = this.languageSelectionDialog
				.getSelectedLanguage();
		Locale selectedLocale = this
				.getLocaleFromLanguage(this.selectedLanguage);
		Locale.setDefault(selectedLocale);

		if (this.stageListener != null) {
			this.stageListener.goToNextStage();
		} else {
			VotingExperienceDialog.instance.setVisible(false);
		}
		return;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.smartmatic.waffle.ecm.ui.DialogListener#cancel()
	 */
	@Override
	public void cancel() {
		return;
	}
}
