/*
 * LocalPersistenceManager.java
 *
 * 1.0 (May 25, 2009)
 *
 * Copyright (c) 2004 Smartmatic Corp.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and propietary information of
 * Smartmatic Corp. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Corp.
 */
package com.smartmatic.waffle.localpersistence;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.Hashtable;

import javax.crypto.CipherInputStream;
import javax.crypto.CipherOutputStream;
import javax.crypto.SecretKey;

import org.apache.commons.io.IOUtils;
import org.apache.commons.lang.StringUtils;
import org.apache.log4j.Logger;
import org.dom4j.Document;
import org.dom4j.DocumentException;
import org.dom4j.DocumentHelper;
import org.dom4j.io.OutputFormat;
import org.dom4j.io.XMLWriter;
import org.iso_relax.verifier.Schema;
import org.iso_relax.verifier.VerifierConfigurationException;
import org.xml.sax.EntityResolver;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

import com.smartmatic.waffle.security.lang.CryptographicRuntimeException;
import com.smartmatic.waffle.security.lang.CryptographicStaticException;
import com.smartmatic.waffle.security.symmetric.PoolableSymmetricCipher;
import com.sun.msv.verifier.jarv.FactoryImpl;
import com.sun.msv.verifier.jarv.TheFactoryImpl;
import com.thoughtworks.xstream.XStream;

/**
 * Class that handles all the persistence operations on files system.
 * 
 * @version 1.0 (Jun 10, 2011)
 * @author Carlos Ramirez (cramirez@smartmatic.com)
 */
public class LocalPersistenceManager {

	/**
	 * For logging purposes.
	 */
	private final static Logger logger = Logger
			.getLogger(LocalPersistenceManager.class);

	/**
	 * Used to management of entities processed as xml data.
	 */
	private XStream xstream;

	/**
	 * 
	 */
	private static Hashtable<String, Schema> schemaCache = new Hashtable<String, Schema>();

	/**
	 * Getter of <code>xstream</code> property.
	 * 
	 * @return <code>xstream</code> property value.
	 * @see #xstream
	 */

	public XStream getXstream() {
		return this.xstream;
	}

	/**
	 * Setter of <code>xstream</code> property.
	 * 
	 * @param xstream
	 *            The <code>xstream</code> to set.
	 * @see #xstream
	 */
	public void setXstream(XStream xstream) {
		this.xstream = xstream;

		return;
	}

	protected static LocalPersistenceManager instance;

	/**
	 * Getter of <code>instance</code> property.
	 * 
	 * @return <code>instance</code> property value.
	 * @see #instance
	 */

	public static synchronized LocalPersistenceManager getInstance() {
		if (instance == null) {
			instance = new LocalPersistenceManager();

			instance.reset();
		}

		return instance;
	}

	/**
	 * Build a new XStream instance. It is used to avoid keeping
	 * read-only-load-at-startup configuration files in memory.
	 */
	public void reset() {
		this.xstream = new XStream();
	}

	/**
	 * Private constructor for singleton design pattern
	 */
	protected LocalPersistenceManager() {
		return;
	}

	/**
	 * 
	 * @param annotatedClassesToProcess
	 */
	@SuppressWarnings("rawtypes")
	public void processAnnotations(Class... annotatedClassesToProcess) {
		this.xstream.processAnnotations(annotatedClassesToProcess);
	}

	/**
	 * Method used to indicate which fields to the given class will be omitted.
	 * 
	 * @param annotatedClassesToProcess
	 *            the annotated class
	 * @param field
	 *            the field to be omitted
	 */
	@SuppressWarnings("rawtypes")
	public void omitField(Class annotatedClassesToProcess, String field) {
		this.xstream.omitField(annotatedClassesToProcess, field);
	}

	/**
	 * Gets an object instance from a xml string
	 * 
	 * @param objectToLoadXML
	 *            object to be loaded in string format
	 * @return an instance of object represented by the given string
	 */
	public Object loadFromXML(String objectToLoadXML) {
		return this.xstream.fromXML(objectToLoadXML);
	}

	/**
	 * Gets an object instance from a xml encrypted base64 string, given its
	 * secret key to decrypt the string.
	 * 
	 * @param stringObject
	 *            the representation of the object in string format
	 * @param password
	 *            the secret key which will be decrypted
	 * @return an instance of object represented by the given string
	 * @throws CryptographicStaticException
	 *             If an exception occurs because exist a configuration error in
	 *             the system.
	 * @throws CryptographicRuntimeException
	 *             If an exception occurs because a problem with related to
	 *             cryptographic configuration.
	 */
	public Object loadBase64Object(String stringObject, SecretKey password)
			throws CryptographicStaticException, CryptographicRuntimeException {

		PoolableSymmetricCipher cipher = new PoolableSymmetricCipher(password);
		String decryptedString = cipher.decrypt(stringObject);

		return this.xstream.fromXML(decryptedString);
	}

	/**
	 * Gets an object instance from an input stream
	 * 
	 * @param objectToLoadStream
	 *            the input stream
	 * @return an instance of object represented by the given input.
	 */
	public Object loadFromXML(InputStream objectToLoadStream) {
		return this.xstream.fromXML(objectToLoadStream);
	}

	/**
	 * Gets an object instance from a xml encrypted input stream, given its
	 * secret key to decrypt the string.
	 * 
	 * @param objectToLoadStream
	 *            the representation of the object
	 * @param password
	 *            the secret key which will be decrypted
	 * @return an instance of object represented by the given string
	 * @throws CryptographicStaticException
	 *             If an exception occurs because exist a configuration error in
	 *             the system.
	 * @throws CryptographicRuntimeException
	 *             If an exception occurs because a problem with related to
	 *             cryptographic configuration.
	 */
	public Object loadFromXML(InputStream objectToLoadStream, SecretKey password)
			throws CryptographicStaticException, CryptographicRuntimeException {

		CipherInputStream decryptedStream = null;

		try {
			PoolableSymmetricCipher cipher = new PoolableSymmetricCipher(
					password);

			decryptedStream = cipher.decrypt(objectToLoadStream);

			return this.xstream.fromXML(decryptedStream);
		} finally {
			if (decryptedStream != null) {
				IOUtils.closeQuietly(decryptedStream);
			}
		}
	}

	/**
	 * Gets an object instance from a xml file.
	 * 
	 * @param objectToLoadFile
	 *            the xml file
	 * @return an instance of object represented by the given file
	 * @throws IOException
	 *             Signals that an I/O exception of some sort has occurred. This
	 *             class is the general class of exceptions produced by failed
	 *             or interrupted I/O operations.
	 */
	public Object loadFromXML(File objectToLoadFile) throws IOException {
		return this.xstream.fromXML(objectToLoadFile);
	}

	/**
	 * Gets an object instance from a xml encrypted file, given its secret key
	 * to decrypt the file.
	 * 
	 * @param objectToLoadFile
	 *            the file
	 * @param password
	 *            the secret key which will be decrypted
	 * @return an instance of object represented by the given file
	 * @throws IOException
	 *             Signals that an I/O exception of some sort has occurred. This
	 *             class is the general class of exceptions produced by failed
	 *             or interrupted I/O operations.
	 * @throws CryptographicStaticException
	 *             If an exception occurs because exist a configuration error in
	 *             the system.
	 * @throws CryptographicRuntimeException
	 *             If an exception occurs because a problem with related to
	 *             cryptographic configuration.
	 */
	public Object loadFromXML(File objectToLoadFile, SecretKey password)
			throws IOException, CryptographicStaticException,
			CryptographicRuntimeException {
		InputStream stream = new BufferedInputStream(new FileInputStream(
				objectToLoadFile));

		try {
			return this.loadFromXML(stream, password);
		} finally {
			stream.close();
		}

	}

	/**
	 * It Serialize an object to persist it in a xml format.
	 * 
	 * @param objectToPersist
	 *            the instance object to be persisted
	 * @return a xml representation of the given object
	 */
	public String persistToXML(Object objectToPersist) {
		return this.xstream.toXML(objectToPersist);
	}

	/**
	 * It Serialize an object to persist it in a xml format and written it in
	 * the given output stream.
	 * 
	 * @param objectToPersist
	 *            the instance object to be written
	 * @param stream
	 *            the result
	 * @throws DocumentException
	 *             If an exception occurs during the processing of a DOM4J
	 *             document.
	 * @throws IOException
	 *             Signals that an I/O exception of some sort has occurred. This
	 *             class is the general class of exceptions produced by failed
	 *             or interrupted I/O operations.
	 */
	public void persistToXML(Object objectToPersist, OutputStream stream)
			throws DocumentException, IOException {
		String xml = this.xstream.toXML(objectToPersist);

		/* Writes the data in correct encoding and put the xml header */
		Document doc = DocumentHelper.parseText(xml);

		OutputFormat outformat = OutputFormat.createPrettyPrint();
		outformat.setEncoding("UTF-8");

		ByteArrayOutputStream tempOut = new ByteArrayOutputStream();
		XMLWriter writer = new XMLWriter(tempOut, outformat);
		try {
			writer.write(doc);

		} finally {

			writer.close();
		}

		tempOut.writeTo(stream);
	}

	/**
	 * It Serialize a representation in string format of an object to persist it
	 * in a xml format and written it in the given output stream.
	 * 
	 * @param objectToPersist
	 *            the string representation of an object
	 * @param stream
	 *            the result
	 * @throws DocumentException
	 *             If an exception occurs during the processing of a DOM4J
	 *             document.
	 * @throws IOException
	 *             Signals that an I/O exception of some sort has occurred. This
	 *             class is the general class of exceptions produced by failed
	 *             or interrupted I/O operations.
	 */
	public void persistToXML(String objectToPersist, OutputStream stream)
			throws DocumentException, IOException {
		/* Writes the data in correct encoding and put the xml header */
		Document doc = DocumentHelper.parseText(objectToPersist);

		OutputFormat outformat = OutputFormat.createPrettyPrint();
		outformat.setEncoding("UTF-8");

		ByteArrayOutputStream tempOut = new ByteArrayOutputStream();
		XMLWriter writer = new XMLWriter(tempOut, outformat);
		try {
			writer.write(doc);

		} finally {

			writer.close();
		}
		tempOut.writeTo(stream);
	}

	/**
	 * It Serialize an object to persist it in a xml format and written it in
	 * the given file.
	 * 
	 * @param objectToPersist
	 *            the string representation of an object
	 * @param file
	 *            the result file
	 * @throws DocumentException
	 *             If an exception occurs during the processing of a DOM4J
	 *             document.
	 * @throws IOException
	 *             Signals that an I/O exception of some sort has occurred. This
	 *             class is the general class of exceptions produced by failed
	 *             or interrupted I/O operations.
	 */
	public void persistToXML(Object objectToPersist, File file)
			throws DocumentException, IOException {

		OutputStream stream = new BufferedOutputStream(new FileOutputStream(
				file));
		try {

			this.persistToXML(objectToPersist, stream);
		} finally {
			stream.close();
		}
	}

	/**
	 * It Serialize an object to persist in a xml format, written it encrypted
	 * with the given secret key in the given file.
	 * 
	 * @param objectToPersist
	 *            the object to be persisted
	 * @param file
	 *            the file which will be written
	 * @param password
	 *            the secret key which will be encrypted
	 * @throws DocumentException
	 *             If an exception occurs during the processing of a DOM4J
	 *             document.
	 * @throws IOException
	 *             Signals that an I/O exception of some sort has occurred. This
	 *             class is the general class of exceptions produced by failed
	 *             or interrupted I/O operations.
	 * @throws CryptographicStaticException
	 *             If an exception occurs because exist a configuration error in
	 *             the system.
	 * @throws CryptographicRuntimeException
	 *             If an exception occurs because a problem with related to
	 *             cryptographic configuration.
	 */
	public void persistToXML(Object objectToPersist, File file,
			SecretKey password) throws DocumentException, IOException,
			CryptographicStaticException, CryptographicRuntimeException {

		OutputStream stream = new BufferedOutputStream(new FileOutputStream(
				file));
		try {

			this.persistToXML(objectToPersist, stream, password);
		} finally {
			stream.close();
		}

	}

	/**
	 * It Serialize a representation in string format of an object to persist
	 * it, written it encrypted with the given secret key in the given file.
	 * 
	 * @param objectToPersist
	 *            the object to be persisted
	 * @param file
	 *            the file which will be written
	 * @param password
	 *            the secret key which will be encrypted
	 * @throws DocumentException
	 *             If an exception occurs during the processing of a DOM4J
	 *             document.
	 * @throws IOException
	 *             Signals that an I/O exception of some sort has occurred. This
	 *             class is the general class of exceptions produced by failed
	 *             or interrupted I/O operations.
	 * @throws CryptographicStaticException
	 *             If an exception occurs because exist a configuration error in
	 *             the system.
	 * @throws CryptographicRuntimeException
	 *             If an exception occurs because a problem with related to
	 *             cryptographic configuration.
	 */
	public void persistToXML(String objectToPersist, File file,
			SecretKey password) throws DocumentException, IOException,
			CryptographicStaticException, CryptographicRuntimeException {

		OutputStream stream = new BufferedOutputStream(new FileOutputStream(
				file));
		try {

			this.persistToXML(objectToPersist, stream, password);
		} finally {
			stream.close();
		}

	}

	/**
	 * It Serialize an object to persist it in a xml format and written it
	 * encrypted with the given secret key in the given output stream.
	 * 
	 * @param objectToPersist
	 *            the object to be persisted
	 * @param stream
	 *            the stream which will be written
	 * @param password
	 *            the secret key which will be encrypted
	 * @throws DocumentException
	 *             If an exception occurs during the processing of a DOM4J
	 *             document.
	 * @throws IOException
	 *             Signals that an I/O exception of some sort has occurred. This
	 *             class is the general class of exceptions produced by failed
	 *             or interrupted I/O operations.
	 * @throws CryptographicStaticException
	 *             If an exception occurs because exist a configuration error in
	 *             the system.
	 * @throws CryptographicRuntimeException
	 *             If an exception occurs because a problem with related to
	 *             cryptographic configuration.
	 */
	public void persistToXML(Object objectToPersist, OutputStream stream,
			SecretKey password) throws CryptographicStaticException,
			CryptographicRuntimeException, DocumentException, IOException {

		PoolableSymmetricCipher cipher = new PoolableSymmetricCipher(password);

		ByteArrayOutputStream tempOut = new ByteArrayOutputStream();
		CipherOutputStream encrypedStream = cipher.encryptBytes(tempOut);
		try {

			this.persistToXML(objectToPersist, encrypedStream);
		} finally {
			encrypedStream.close();

		}

		tempOut.writeTo(stream);
	}

	/**
	 * It Serialize a representation of an object to persist it in a xml format
	 * and written it encrypted with the given secret key in the given output
	 * stream.
	 * 
	 * @param objectToPersist
	 *            the object to be persisted as a string
	 * @param stream
	 *            the stream which will be written
	 * @param password
	 *            the secret key which will be encrypted
	 * @throws DocumentException
	 *             If an exception occurs during the processing of a DOM4J
	 *             document.
	 * @throws IOException
	 *             Signals that an I/O exception of some sort has occurred. This
	 *             class is the general class of exceptions produced by failed
	 *             or interrupted I/O operations.
	 * @throws CryptographicStaticException
	 *             If an exception occurs because exist a configuration error in
	 *             the system.
	 * @throws CryptographicRuntimeException
	 *             If an exception occurs because a problem with related to
	 *             cryptographic configuration.
	 */
	public void persistToXML(String objectToPersist, OutputStream stream,
			SecretKey password) throws CryptographicStaticException,
			CryptographicRuntimeException, DocumentException, IOException {

		PoolableSymmetricCipher cipher = new PoolableSymmetricCipher(password);

		ByteArrayOutputStream tempOut = new ByteArrayOutputStream();
		CipherOutputStream encrypedStream = cipher.encryptBytes(tempOut);
		try {

			this.persistToXML(objectToPersist, encrypedStream);
		} finally {
			encrypedStream.close();

		}

		tempOut.writeTo(stream);
	}

	/**
	 * This method take a xml file reference a check it against its xsd schema,
	 * returning its status.
	 * 
	 * @param xmlFile
	 *            the file to be checked
	 * @param password
	 *            the secret key which was encrypted
	 * @param xsdExternalDir
	 *            dir with xsd schema path
	 * @return a detailed info instance object
	 */
	public DestinationStatus checkXmlReadable(File xmlFile, SecretKey password,
			File xsdExternalDir) {

		InputStream in;
		try {
			in = new FileInputStream(xmlFile);
		} catch (FileNotFoundException e1) {
			logger.error("Destination status not found");
			return DestinationStatus.NOT_FOUND;
		}
		try {

			if (password == null) {
				byte[] bytes = IOUtils.toByteArray(in);

				return this.validateXmlData(xmlFile, bytes, xsdExternalDir);
			} else {
				PoolableSymmetricCipher cipher = new PoolableSymmetricCipher(
						password);

				byte[] bytes = cipher.decryptBytes(IOUtils.toByteArray(in));

				return this.validateXmlData(xmlFile, bytes, xsdExternalDir);

			}
		} catch (DocumentException exception) {
			logger.error(exception.getMessage(), exception);
			return DestinationStatus.NO_WELL_FORMED;
		} catch (IOException exception) {
			logger.error(exception.getMessage(), exception);
			return DestinationStatus.INVALID;
		} catch (CryptographicStaticException exception) {
			logger.error(exception.getMessage(), exception);
			return DestinationStatus.INVALID;
		} catch (CryptographicRuntimeException exception) {
			logger.error(exception.getMessage(), exception);
			return DestinationStatus.INVALID;
		} finally {
			IOUtils.closeQuietly(in);
		}
	}

	/**
	 * @param xmlFilePath
	 * @param data
	 * @return
	 * @throws DocumentException
	 */
	private DestinationStatus validateXmlData(File xmlFilePath, byte[] data,
			File xsdExternalDir) throws DocumentException {
		String name = xmlFilePath.getName();
		String nameXsd = StringUtils.substringBeforeLast(name, ".") + ".xsd";

		File xsd = new File(xmlFilePath.getParentFile(), nameXsd);
		if (xsd.exists() && xsd.isFile()) {
			if (this.validateXSD(data, xsd.getAbsolutePath(), xsdExternalDir)) {
				return DestinationStatus.VALID;
			} else {
				logger.error("The file " + xmlFilePath + " is invalid. XSD ("
						+ xsd.getAbsolutePath() + ")");

				return DestinationStatus.INVALID;
			}
		} else {
			return DestinationStatus.VALID;
		}
	}

	/**
	 * 
	 * @param xmlFile
	 * @param xsdExternalDir
	 * @return
	 */
	public DestinationStatus checkXmlReadable(File xmlFile, File xsdExternalDir) {
		return this.checkXmlReadable(xmlFile, null, xsdExternalDir);
	}

	/**
	 * 
	 * @param file
	 * @param password
	 * @return
	 */
	public boolean checkFileReadable(File file, SecretKey password) {
		InputStream is;
		try {
			is = new FileInputStream(file);
		} catch (FileNotFoundException e) {
			logger.error(e.getMessage(), e);
			return false;
		}

		try {
			if (password == null) {
				/* Test if the file can be fully read */
				IOUtils.toByteArray(is);
			} else {
				PoolableSymmetricCipher cipher = new PoolableSymmetricCipher(
						password);
				CipherInputStream decryptedStream = cipher.decrypt(is);

				/* Test if the file can be fully read */
				IOUtils.toByteArray(decryptedStream);
			}

		} catch (IOException e) {
			logger.error(e.getMessage(), e);
			return false;
		} catch (CryptographicStaticException e) {
			logger.error(e.getMessage(), e);
			return false;
		} catch (CryptographicRuntimeException e) {
			logger.error(e.getMessage(), e);
			return false;
		} finally {
			IOUtils.closeQuietly(is);
		}
		return true;
	}

	/**
	 * 
	 * @param schemaPath
	 * @param xsdExternalDir
	 * @return
	 */
	private static Schema obtainSchema(String schemaPath, File xsdExternalDir) {

		Schema schema = schemaCache.get(schemaPath);
		if (schema == null) {
			try {
				FactoryImpl factory = new TheFactoryImpl();

				final File xsdExternalFolder = xsdExternalDir;

				factory.setEntityResolver(new EntityResolver() {

					@Override
					public InputSource resolveEntity(String publicId,
							String systemId) throws SAXException, IOException {

						InputStream fileInputStream = null;

						try {
							if ("-//W3C//DTD XMLSchema 200102//EN"
									.equals(publicId)
									&& "http://www.w3.org/2001/XMLSchema.dtd"
											.equals(systemId)) {
								fileInputStream = new FileInputStream(new File(
										xsdExternalFolder, "XMLSchema.dtd"));
								return new InputSource(fileInputStream);
							}

							if ("datatypes".equals(publicId)
									&& "http://www.w3.org/2001/datatypes.dtd"
											.equals(systemId)) {
								fileInputStream = new FileInputStream(new File(
										xsdExternalFolder, "datatypes.dtd"));
								return new InputSource(fileInputStream);
							}
						} finally {
							IOUtils.closeQuietly(fileInputStream);
						}

						return null;
					}
				});

				schema = factory.compileSchema(new File(schemaPath));
				schemaCache.put(schemaPath, schema);
			} catch (Exception e) {
				logger.error(e.getMessage(), e);
			}
		}
		return schema;

	}

	/**
	 * 
	 * @param xmlBytes
	 * @param xsdPath
	 * @param xsdExternalDir
	 * @return
	 */
	public boolean validateXSD(byte[] xmlBytes, String xsdPath,
			File xsdExternalDir) {
		Schema compiledSchema;
		try {
			compiledSchema = obtainSchema(xsdPath, xsdExternalDir);

			compiledSchema.newVerifier();
		} catch (VerifierConfigurationException e) {
			logger.error(e.getMessage(), e);
			return false;
		} catch (Exception e) {
			logger.error(e.getMessage(), e);
			return false;
		}
		return true;
	}
}
