/**
 * EnvironmentManager.java
 *
 * Revision: $Revision$  (Last Modified: $Date$)
 * Last Modified by: [$Author$]
 * 
 * Copyright (c) 2004 Smartmatic Corp.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of
 * Smartmatic Corp. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Corp.
 */
package com.smartmatic.waffle.localpersistence.environment;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Set;

import javax.crypto.SecretKey;

import com.smartmatic.waffle.localpersistence.CommonResultInfo;
import com.smartmatic.waffle.localpersistence.SAESCommonDestinations;
import com.smartmatic.waffle.localpersistence.SAESLocalPersistenceManager;
import com.smartmatic.waffle.localpersistence.exception.HistoricFileException;
import com.smartmatic.waffle.localpersistence.exception.SAESLocalPersistenceException;
import com.smartmatic.waffle.log.SMTTLogger;

/**
 * Class in charged of management of files and consistency of the application's
 * core.
 * 
 * @version 1.0 (Jun 10, 2011)
 * @author Carlos Ramirez (cramirez@smartmatic.com)
 */
public class EnvironmentManager {

	/**
	 * singleton instance.
	 */
	private static EnvironmentManager instance;

	/**
	 * For logging purposes.
	 */
	private static final SMTTLogger logger = SMTTLogger
			.getLogger(EnvironmentManager.class);

	/**
	 * Instance of local persistence manager.
	 */
	private SAESLocalPersistenceManager localPersistence;

	/**
	 * Field that indicate the current status of the environment.
	 */
	private EnvironmentStatus status;

	/**
	 * Field used to store details for each common destination loaded in the
	 * application's scope.
	 */
	private Map<SAESCommonDestinations, EnvironmentDetailStatus> environmentDetailStatuses = new HashMap<SAESCommonDestinations, EnvironmentDetailStatus>();

	/**
	 * Field used to store detailed info for main destination media.
	 */
	private Map<File, CommonResultInfo> destinationStatesDefaultMedia;

	/**
	 * Field used to store detailed info for each destination media.
	 */
	private Map<SAESCommonDestinations, Map<File, CommonResultInfo>> destinationStates;

	/** Indicate isolated mode. */
	private final boolean isolatedMode;

	/**
	 * Default application's common destination.
	 */
	private final SAESCommonDestinations defaultCommonDestinations;

	/**
	 * Private constructor.
	 */
	private EnvironmentManager() {
		this.isolatedMode = false;

		this.defaultCommonDestinations = SAESLocalPersistenceManager
				.getInstance().getDefaultDestinations();

		return;

	}

	/**
	 * Instantiates a new environment manager.
	 * 
	 * @param theDefaultCommonDestinations
	 *            the the default common destinations
	 */
	public EnvironmentManager(
			SAESCommonDestinations theDefaultCommonDestinations) {
		this.defaultCommonDestinations = theDefaultCommonDestinations;
		this.isolatedMode = true;
		this.localPersistence = SAESLocalPersistenceManager.getInstance();
	}

	/**
	 * Initialize.
	 */
	public static void initialize() {
		logger.info("Initializing the Environment Manager");
		instance = new EnvironmentManager();

		instance.localPersistence = SAESLocalPersistenceManager.getInstance();

	}

	/**
	 * Enum that represents the different status for the environment's machine.
	 * 
	 * 
	 * @version 1.0 (Aug 25, 2011)
	 * @author hrodriguez
	 */
	public enum EnvironmentStatus {

		/** The empty. */
		EMPTY,
		/** The incomplete. */
		INCOMPLETE,
		/** The invalid. */
		INVALID,
		/** The valid. */
		VALID
	}

	/**
	 * Class used to wrap detailed info about the status of different file
	 * systems.
	 * 
	 * 
	 * @version 1.0 (Aug 25, 2011)
	 * @author hrodriguez
	 */
	public class EnvironmentDetailStatus {

		/** Field that represents the current status of the environment. */
		public EnvironmentStatus environmentStatus;
		/**
		 * List of invalid files.
		 */
		public final Collection<File> invalidFiles;
		/**
		 * List of valid files.
		 */
		public final Collection<File> validFiles;
		/**
		 * List of not found files.
		 */
		public final Collection<File> notFoundFiles;
		/**
		 * List of not created files.
		 */
		public final Collection<File> notCreatedFiles;
		/**
		 * List of not required level files.
		 */
		public final Collection<File> notRequiredLevelFiles;
		/**
		 * List of not found optional files.
		 */
		public final Collection<File> notFoundOptionalFiles;
		/**
		 * List of not well formed files.
		 */
		public final Collection<File> noWellFormedFiles;

		/**
		 * Default constructor.
		 */
		public EnvironmentDetailStatus() {
			this.invalidFiles = new ArrayList<File>();
			this.validFiles = new ArrayList<File>();
			this.notFoundFiles = new ArrayList<File>();
			this.notCreatedFiles = new ArrayList<File>();
			this.notRequiredLevelFiles = new ArrayList<File>();
			this.notFoundOptionalFiles = new ArrayList<File>();
			this.noWellFormedFiles = new ArrayList<File>();
		}

	}

	/**
	 * Method used to perform a check for the configured environment.
	 * 
	 * @return the map
	 * @throws IOException
	 *             Signals that an I/O exception of some sort has occurred. This
	 *             class is the general class of exceptions produced by failed
	 *             or interrupted I/O operations.
	 * @throws HistoricFileException
	 *             If an exception occurs during reading historic files.
	 * @throws SAESLocalPersistenceException
	 *             If an exception occurs during persistence operations on local
	 */
	public Map<File, CommonResultInfo> checkEnvironment() throws IOException,
			HistoricFileException, SAESLocalPersistenceException {

		logger.info("Checking environment");

		this.destinationStates = new HashMap<SAESCommonDestinations, Map<File, CommonResultInfo>>();
		try {

			SecretKey password = this.localPersistence.getKey();
			if (this.isolatedMode) {
				SAESCommonDestinations destinations = this.defaultCommonDestinations;

				Map<File, CommonResultInfo> theDestinationStates = destinations
						.checkFiles(0, password);

				this.destinationStates.put(destinations, theDestinationStates);
				this.destinationStatesDefaultMedia = theDestinationStates;

				this.status = this.calculateStatus(destinations,
						theDestinationStates);

			} else {

				EnvironmentStatus theStatus = EnvironmentStatus.VALID;
				for (SAESCommonDestinations destinations : this.localPersistence
						.getStoreMedias()) {

					Map<File, CommonResultInfo> theDestinationStates = destinations
							.checkFiles(0, password);

					if (destinations == this.defaultCommonDestinations) {
						this.destinationStatesDefaultMedia = theDestinationStates;
					}

					this.destinationStates.put(destinations,
							theDestinationStates);

					EnvironmentStatus mediaStatus = this.calculateStatus(
							destinations, theDestinationStates);

					if (theStatus == EnvironmentStatus.VALID
							&& mediaStatus != theStatus) {
						theStatus = mediaStatus;
					}

				}
				this.status = theStatus;
			}

		} catch (HistoricFileException e) {
			this.status = EnvironmentStatus.INVALID;

			logger.error(e.getMessage(), e);

			throw e;
		} catch (IllegalArgumentException e) {

			e.printStackTrace();
			logger.error(e.getMessage(), e);
		}

		return null;
	}

	/**
	 * Getter of <code>destinationStatesDefaultMedia</code> property.
	 * 
	 * @return <code>destinationStatesDefaultMedia</code> property value.
	 * @see #destinationStatesDefaultMedia
	 */

	public Map<File, CommonResultInfo> getDestinationStatesDefaultMedia() {
		return this.destinationStatesDefaultMedia;
	}

	/**
	 * Method used to determine current status of the application.
	 * 
	 * @param theDestinations
	 *            the media destination
	 * @param results
	 *            the results
	 * @return An enum value of the Environment status
	 * @see EnvironmentStatus
	 */
	public EnvironmentStatus calculateStatus(
			SAESCommonDestinations theDestinations,
			Map<File, CommonResultInfo> results) {

		logger.info("Calculating the status");

		EnvironmentDetailStatus environmentDetailStatus = this.environmentDetailStatuses
				.get(theDestinations);

		if (environmentDetailStatus == null) {
			environmentDetailStatus = new EnvironmentDetailStatus();
		}

		this.environmentDetailStatuses.put(theDestinations,
				environmentDetailStatus);

		Set<Entry<File, CommonResultInfo>> commons = results.entrySet();

		for (Entry<File, CommonResultInfo> entry : commons) {
			File file = entry.getKey();

			CommonResultInfo resultInfo = entry.getValue();

			switch (resultInfo.getStatus()) {
			case FOUND:
			case VALID:
			case CREATED:
				environmentDetailStatus.validFiles.add(file);
				break;
			case NOT_FOUND:
				environmentDetailStatus.notFoundFiles.add(file);
				break;
			case INVALID:
				environmentDetailStatus.invalidFiles.add(file);
				break;
			case NOT_CREATED:
				environmentDetailStatus.notCreatedFiles.add(file);
				break;
			case NOT_REQUIRED_LEVEL:
				environmentDetailStatus.notRequiredLevelFiles.add(file);
				break;
			case NOT_FOUND_OPTIONAL:
				environmentDetailStatus.notFoundOptionalFiles.add(file);
				break;
			case NO_WELL_FORMED:
				environmentDetailStatus.noWellFormedFiles.add(file);
				break;
			}
		}

		if (environmentDetailStatus.invalidFiles.isEmpty() == false
				|| environmentDetailStatus.noWellFormedFiles.isEmpty() == false) {

			environmentDetailStatus.environmentStatus = EnvironmentStatus.INVALID;

		} else if (environmentDetailStatus.notCreatedFiles.isEmpty() == false
				|| environmentDetailStatus.notFoundFiles.isEmpty() == false) {

			environmentDetailStatus.environmentStatus = EnvironmentStatus.INCOMPLETE;
		} else {
			environmentDetailStatus.environmentStatus = EnvironmentStatus.VALID;
		}

		return environmentDetailStatus.environmentStatus;
	}

	/**
	 * Getter of <code>instance</code> property.
	 * 
	 * @return <code>instance</code> property value.
	 * @see #instance
	 */

	public static EnvironmentManager getInstance() {
		return instance;
	}

	/**
	 * Getter of <code>status</code> property.
	 * 
	 * @return <code>status</code> property value.
	 * @see #status
	 */

	public EnvironmentStatus getStatus() {
		return this.status;
	}

	/**
	 * Getter of <code>enviroment</code> property.
	 * 
	 * @param destinations
	 *            the destinations
	 * @return <code>enviroment</code> property value.
	 * @see #environmentDetailStatuses
	 */

	public EnvironmentDetailStatus getEnviromentDetailStatus(
			SAESCommonDestinations destinations) {

		return this.environmentDetailStatuses.get(destinations);
	}

	/**
	 * Getter of <code>destinationStates</code> property.
	 * 
	 * @return <code>destinationStates</code> property value.
	 * @see #destinationStates
	 */

	public Map<SAESCommonDestinations, Map<File, CommonResultInfo>> getDestinationStates() {
		return this.destinationStates;
	}

}
