package com.smartmatic.waffle.log.progress;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.TimeZone;

import com.smartmatic.waffle.log.SMTTLogger;

/**
 * <p>
 * Utility class that helps to show a progress log. It only needs the messages
 * that will be displayed in each case and the amount of steps that will be
 * needed to complete the process.
 * </p>
 * <p>
 * Example:
 * </p>
 * <code>
       int amount = 15; 
       ProgressLogger progressLogger = new ProgressLogger(
                logger,
                "HH 'hours' mm 'minutes' ss 'seconds'",
                "Begin process",
                "Processed %d of %d (%d%%) [elapsed: %s]",
                "Processed %d of %d (%d%%) [elapsed: %s, remaining (estimated): %s]",
                "Process completed, time spent: %s", amount);

        for(int i = 0; i < amount; i++) {

            progressLogger.step();
        }

        progressLogger.showFinalMessage();

 * </code>
 * 
 * @see #logger
 * @see #progressMessageNoEstimation
 * @see #progressMessage
 * @see #finalMessage
 * 
 * @author Carlos Rafael Ramirez
 * 
 */
public class ProgressLogger {

	/**
	 * Logger that will be used to log the progress messages
	 */
	private final SMTTLogger logger;

	private final SimpleDateFormat formatter;
	private final SimpleDateFormat formatterSimple;
	private final long init;

	/**
	 * Number of steps that will be needed in order to complete the process
	 */
	private final int amount;

	/**
	 * <p>
	 * Message to indicate the progress when the percentage of completion is
	 * still 0%
	 * </p>
	 * 
	 * <p>
	 * This message is provided in the String.format's format and requires the
	 * following parameter declarations:
	 * </p>
	 * 
	 * <pre>
	 * int steps the number of steps completed
	 * int amount of steps to be completed 
	 * int percentage of completion
	 * String time elapsed since the process begun
	 * 
	 * Example:
	 * Loaded %d of %d (%d%%) [elapsed: %s]
	 * </pre>
	 * 
	 * 
	 * 
	 * @see String#format(String, Object...)
	 */
	private final String progressMessageNoEstimation;

	/**
	 * <p>
	 * Message to indicate the progress when the percentage of completion is
	 * greater than 0%
	 * </p>
	 * 
	 * <p>
	 * This message is provided in the String.format's format and requires the
	 * following parameter declarations:
	 * </p>
	 * 
	 * <pre>
	 * int steps the number of steps completed
	 * int amount of steps to be completed 
	 * int percentage of completion
	 * String time elapsed since the process begun
	 * String time estimated of completion
	 * 
	 * Example:
	 * Loaded %d of %d (%d%%) [elapsed: %s, remaining (estimated): %s]
	 * </pre>
	 * 
	 * 
	 * 
	 * @see String#format(String, Object...)
	 */
	private final String progressMessage;

	/**
	 * <p>
	 * Message to indicate that the process has finalized
	 * </p>
	 * 
	 * <p>
	 * This message is provided in the String.format's format and requires the
	 * following parameter declarations:
	 * </p>
	 * 
	 * <pre>
	 * String time of completion
	 * 
	 * Example:
	 * Time spent for bulk load operation: %s
	 * </pre>
	 * 
	 * 
	 * 
	 * @see String#format(String, Object...)
	 */
	private final String finalMessage;
	private int progress;
	private int percent;

	/**
	 * Constructs the progressLogger object and begin to measure the progress
	 * time.
	 * 
	 * @param theLogger
	 *            Logger that will be used to log the progress messages
	 * @param logTimeFormat
	 *            format used by SimpeDateFormat to show the progress in time.
	 *            Example: <code>HH 'hours' mm 'minutes' ss 'seconds'</code>.
	 *            This parameter is optional if it is not specified the default
	 *            <code>HH 'hours' mm 'minutes' ss 'seconds'</code> will be used
	 * @param initialMessage
	 *            Optional message to show that the process has begun
	 * @param progressMessageNoEstimation
	 *            Message to indicate the progress when the percentage of
	 *            completion is still 0%. See
	 *            {@link #progressMessageNoEstimation}
	 * @param progressMessage
	 *            Message to indicate the progress when the percentage of
	 *            completion is greater than 0%. See {@link #progressMessage}
	 * @param finalMessage
	 *            Optional message to indicate that the process has finalized.
	 *            See {@link #finalMessage}
	 * @param theAmount
	 *            Number of steps that will be needed in order to complete the
	 *            process
	 * 
	 * @see SimpleDateFormat
	 */
	public ProgressLogger(SMTTLogger theLogger, String logTimeFormat,
			String initialMessage, String progressMessageNoEstimation,
			String progressMessage, String finalMessage, int theAmount) {

		this.logger = theLogger;
		this.progressMessageNoEstimation = progressMessageNoEstimation;
		this.progressMessage = progressMessage;
		this.finalMessage = finalMessage;
		this.amount = theAmount;

		if (progressMessageNoEstimation == null) {
			throw new IllegalArgumentException(
					"The parameter 'progressMessageNoEstimation' must have a value.");
		}

		if (progressMessage == null) {
			throw new IllegalArgumentException(
					"The parameter 'progressMessage' must have a value.");
		}

		if (logTimeFormat == null) {
			/* Measure execution time for logging */
			this.formatter = new SimpleDateFormat(
					"HH 'hours' mm 'minutes' ss 'seconds'");

		} else {
			/* Measure execution time for logging */
			this.formatter = new SimpleDateFormat(logTimeFormat);

		}
		this.formatter.setTimeZone(TimeZone.getTimeZone("GMT"));

		this.formatterSimple = new SimpleDateFormat("HH:mm:ss");
		this.formatterSimple.setTimeZone(TimeZone.getTimeZone("GMT"));

		if (initialMessage != null) {
			this.logger.info(initialMessage);
		}

		this.init = System.currentTimeMillis();

	}

	/**
	 * Increments the progress and shows a progress message in the log
	 * 
	 * @param theProgress
	 *            how many steps has been completed
	 */
	public synchronized void step(int theProgress) {

		this.progress += theProgress;
		long end = System.currentTimeMillis();

		Date time = new Date(end - this.init);
		String elapsed = this.formatterSimple.format(time);
		double percentFrac = (this.progress * 100.0) / this.amount;
		this.percent = (int) percentFrac;

		if (percentFrac != 0) {
			time = new Date(
					(long) ((100 * (end - this.init) / percentFrac) - (end - this.init)));
			String remaining = this.formatterSimple.format(time);

			this.logger.info(String.format(this.progressMessage, this.progress,
					this.amount, this.percent, elapsed, remaining));
		} else {
			this.logger.info(String.format(this.progressMessageNoEstimation,
					this.progress, this.amount, this.percent, elapsed));

		}

	}

	/**
	 * Shortcut to advance only one step
	 */
	public void step() {
		this.step(1);
	}

	/**
	 * Shows a message in log stating that the process has finalized
	 */
	public synchronized void showFinalMessage() {

		if (this.finalMessage != null) {
			long end = System.currentTimeMillis();

			Date time = new Date(end - this.init);

			this.logger.info(String.format(this.finalMessage,
					this.formatter.format(time)));
		}
	}

	/**
	 * Getter of <code>amount</code> property.
	 * 
	 * @return <code>amount</code> property value.
	 * @see #amount
	 */
	public int getAmount() {
		return this.amount;
	}

	/**
	 * Getter of <code>progress</code> property.
	 * 
	 * @return <code>progress</code> property value.
	 * @see #progress
	 */
	public int getProgress() {
		return this.progress;
	}

	/**
	 * Getter of <code>percent</code> property.
	 * 
	 * @return <code>percent</code> property value.
	 * @see #percent
	 */
	public int getPercent() {
		return this.percent;
	}

	/**
	 * Convenient method to log the time elapsed in a human readable format
	 * 
	 * @param time
	 *            time elapsed in milliseconds
	 * @return the string representation of the provided time
	 */
	public static String formatTimeElapsed(long time) {

		/* Measure execution time for logging */
		SimpleDateFormat formatter = new SimpleDateFormat(
				"HH 'hours' mm 'minutes' ss 'seconds'");

		formatter.setTimeZone(TimeZone.getTimeZone("GMT"));
		Date timeObj = new Date(time);
		String elapsed = formatter.format(timeObj);

		return elapsed;
	}
}
