/**
 * SearchExpression.java
 *
 * Revision: $Revision$  (Last Modified: $Date$)
 * Last Modified by: [$Author$]
 * 
 * Copyright (c) 2004 Smartmatic Corp.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of
 * Smartmatic Corp. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Corp.
 */

package com.smartmatic.waffle.persistence.search;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import com.smartmatic.waffle.persistence.search.criteria.ConjunctionCriteria;
import com.smartmatic.waffle.persistence.search.criteria.CriteriaFactory;
import com.smartmatic.waffle.persistence.search.criteria.ISearchCriteria;
import com.smartmatic.waffle.persistence.search.criteria.OrderCriteria;
import com.smartmatic.waffle.persistence.search.transformation.ITransformation;

/**
 * Object that contains all the information required to search information from
 * the database. It contains a collection of SearchCriterias, also a collection
 * of orders used to filter and order the results.
 * 
 * @version 1.0 (17/02/2005)
 * @author Ithamar Torres
 */

public class SearchExpression implements Serializable {

	/**
	 * Holds the serialization id of the class
	 */

	private static final long serialVersionUID = 4519143764755967461L;

	/**
	 * Object used to define a SearchExpression that is used as a SubSearch.
	 * 
	 * @version 1.0 (17/02/2005)
	 * @author Ithamar Torres
	 */

	public final class SubSearchExpression extends SearchExpression {

		/**
		 * Holds the serialization id of the class
		 */

		private static final long serialVersionUID = -8038248522497827614L;

		/**
		 * PathData that defines the relation between this expression and his
		 * parent expression.
		 */

		private String path;

		/**
		 * Getter of <code>path</code> property.
		 * 
		 * @return <code>path</code> property value.
		 * @see #path
		 */

		public String getPath() {
			return this.path;
		}

		/**
		 * Parent Expression that contains this subSearchExpression.
		 */

		private SearchExpression parentSearchExpression;

		/**
		 * Constructor of SubSearchExpression
		 * 
		 * @param parent
		 *            Parent of this SubSearchExpression
		 * @param path
		 *            PathData that defines the relation between this expression
		 *            and his parent expression.
		 * @param alias
		 *            Alias that will be used when a criteria is referring this
		 *            subSearchExpression
		 */

		public SubSearchExpression(SearchExpression parent, String path,
				String alias) {
			super(null, alias);
			this.path = path;
			this.parentSearchExpression = parent;
		}

		/**
		 * Creates and returns a copy of this object.
		 */
		@Override
		public Object clone() throws CloneNotSupportedException {
			SubSearchExpression se = new SubSearchExpression(
					this.parentSearchExpression, this.path, this.alias);
			se.transformation = this.transformation;
			se.criterias = new ArrayList<ISearchCriteria>(this.criterias);
			se.orders = new ArrayList<OrderCriteria>(this.orders);
			se.subExpressions = new ArrayList<SubSearchExpression>(
					this.subExpressions.size());
			for (SubSearchExpression expression : this.subExpressions) {
				se.subExpressions.add((SubSearchExpression) expression.clone());
			}

			return se;
		}
	}

	/**
	 * Alias used as the default root alias in case that the programmer does not
	 * specify any alias
	 */

	private static String ROOT_ALIAS = "this";

	/**
	 * Transformation that projects the information into a grouped or a subset
	 * of the properties used in the search expression.
	 */

	protected ITransformation transformation;

	/**
	 * Alias used to refer to this SearchExpression
	 */

	protected String alias;

	/**
	 * The type of join to use when the method createAlias is used.
	 */
	protected JoinType joinType;

	/**
	 * Set a limit upon the number of objects to be retrieved.
	 * 
	 */
	protected int maxResults;

	/**
	 * Getter of <code>alias</code> property.
	 * 
	 * @return <code>alias</code> property value.
	 * @see #alias
	 */

	public String getAlias() {
		return this.alias;
	}

	/**
	 * Entity Class that is used in this SearchExpression
	 */

	protected Class entityClass;

	/**
	 * Collection of SearchCriterias used to filter the results
	 */

	protected List<ISearchCriteria> criterias = new ArrayList<ISearchCriteria>();

	/**
	 * Collection of OrderCriterias used to order the results
	 */

	protected List<OrderCriteria> orders = new ArrayList<OrderCriteria>();

	/**
	 * Collection of subSearchExpressions used to order the results
	 */

	protected List<SubSearchExpression> subExpressions = new ArrayList<SubSearchExpression>();

	/**
	 * Should the result be considered cacheable; default is to not cache
	 * (false).
	 */
	protected boolean cacheable;

	/**
	 * 
	 * the name of a query cache region, or <tt>null</tt> for the default query
	 * cache
	 * 
	 * @see #setCacheable
	 */
	protected String cacheRegion;

	/**
	 * Constructor used to create a SearchExpression using an alias defined by
	 * the programmer
	 * 
	 * @param theClass
	 *            Entity Class that its has to be searched
	 * @param alias
	 *            The alias used to refer to this SearchExpression
	 */

	public SearchExpression(Class theClass, String alias) {
		this.entityClass = theClass;
		this.alias = alias;
	}

	/**
	 * Constructor used to create a SearchExpression using as alias the default
	 * alias.
	 * 
	 * @param theClass
	 *            Entity Class that its has to be searched
	 */

	public SearchExpression(Class theClass) {
		this.entityClass = theClass;
		this.alias = SearchExpression.ROOT_ALIAS;
	}

	/**
	 * Constructor used to create a SearchExpression using as alias the default
	 * alias. Also it receives a field-value map to initialize it.
	 * 
	 * @param theClass
	 *            Entity Class that its has to be searched
	 * @param value
	 *            Field-value map to search for
	 */

	public SearchExpression(Class theClass, Map<String, Object> values) {
		// It makes a split of the values and load all it's sub expressions
		this.entityClass = theClass;
		this.alias = SearchExpression.ROOT_ALIAS;

		for (Map.Entry<String, Object> fieldValue : values.entrySet()) {

			String fieldChain = fieldValue.getKey();
			Object value = fieldValue.getValue();

			String[] fields = fieldChain.split("\\.");
			SearchExpression actualExp = this;
			for (int i = 0; i < fields.length - 1; i++) {
				String alias = fields[i];

				actualExp = actualExp.createAlias(alias, alias);
			}

			// According to the search criteria it construct it
			if (value instanceof String) {
				String[] valuesSplitted = ((String) value).split(",");
				if (valuesSplitted.length == 1) {
					actualExp.addSearchCriteria(CriteriaFactory.eq(
							fields[fields.length - 1], value));
				} else {
					actualExp.addSearchCriteria(CriteriaFactory.in(
							fields[fields.length - 1], valuesSplitted));
				}
			} else {
				actualExp.addSearchCriteria(CriteriaFactory.eq(
						fields[fields.length - 1], value));
			}
		}
	}

	/**
	 * Method that allows to create another SearchExpression to be able to use
	 * filters using another object linked to the root object returned in the
	 * SearchExpression. It is important to know that each Search only returns
	 * the list of root objects (All the collections are not loaded from the
	 * database). It makes a JoinType.INNER_JOIN reference between the objects
	 * 
	 * @param theAlias
	 *            The alias used to refer to the object that is used to filter
	 * @param path
	 *            The way to access the object from the root object
	 * 
	 * @return The search expression that is created.
	 */

	public SearchExpression createAlias(String theAlias, String path) {
		return this.createAlias(theAlias, path, JoinType.INNER_JOIN);
	}

	/**
	 * Method that allows to create another SearchExpression to be able to use
	 * filters using another object linked to the root object returned in the
	 * SearchExpression. It is important to know that each Search only returns
	 * the list of root objects (All the collections are not loaded from the
	 * database).
	 * 
	 * @param theAlias
	 *            The alias used to refer to the object that is used to filter
	 * @param path
	 *            The way to access the object from the root object
	 * @param theJoinType
	 *            The type of join to use by this association.
	 * 
	 * @return The search expression that is created.
	 */

	public SearchExpression createAlias(String theAlias, String path,
			JoinType theJoinType) {

		SubSearchExpression expression = null;

		// Search for another sub expression with the same path
		for (SubSearchExpression subExpression : this.subExpressions) {
			if (subExpression.path.equals(path)) {
				expression = subExpression;
				break;
			}
		}

		if (expression == null) {
			expression = new SubSearchExpression(this, path, theAlias);
			expression.joinType = theJoinType;
			this.subExpressions.add(expression);
		} else {
			if (theJoinType != expression.joinType) {
				throw new CriteriaDefinitionException(
						CriteriaDefinitionException.ExceptionInfo.CONFLICTING_JOIN_TYPE,
						this.getEntityClass().getName(), theAlias, path);
			}
		}
		return expression;
	}

	/**
	 * Method used to add a SearchCriteria to the current SearchExpression
	 * 
	 * @param criteria
	 *            The criteria to add
	 * 
	 * @return the same instance to allow dot chains
	 */

	public SearchExpression addSearchCriteria(ISearchCriteria criteria) {
		this.criterias.add(criteria);
		return this;
	}

	/**
	 * Method that allows to obtain all the SearchCriterias defined in this
	 * SearchExpression as an unique SearchCriteria. The criterias are used
	 * using an logical and as his binary operator.
	 * 
	 * @return A SearchCriteria that contains all the information of the
	 *         SearchExpression
	 */

	public ISearchCriteria getSearchCriteria() {
		if (this.criterias.size() > 0) {
			ConjunctionCriteria criteria = new ConjunctionCriteria();
			Iterator<ISearchCriteria> iterator = this.criterias.iterator();
			while (iterator.hasNext()) {
				criteria.add(iterator.next());
			}
			return criteria;
		} else {
			return null;
		}
	}

	/**
	 * Method used to add a OrderCriteria to the current SearchExpression
	 * 
	 * @param criteria
	 *            The order criteria to add
	 */

	public SearchExpression addOrder(OrderCriteria order) {
		this.orders.add(order);
		return this;
	}

	/**
	 * Getter of <code>entityClass</code> property.
	 * 
	 * @return <code>entityClass</code> property value.
	 * @see #entityClass
	 */

	public Class getEntityClass() {
		return this.entityClass;
	}

	/**
	 * Getter of <code>subExpressions</code> property.
	 * 
	 * @return <code>subExpressions</code> property value.
	 * @see #subExpressions
	 */

	public Iterator<SubSearchExpression> getSubExpressions() {
		return this.subExpressions.iterator();
	}

	/**
	 * Getter of <code>orders</code> property.
	 * 
	 * @return <code>orders</code> property value.
	 * @see #orders
	 */

	public List<OrderCriteria> getOrders() {
		return this.orders;
	}

	/**
	 * Getter of <code>transformation</code> property.
	 * 
	 * @return <code>transformation</code> property value.
	 * @see #transformation
	 */

	public ITransformation getTransformation() {
		return this.transformation;
	}

	/**
	 * Setter of <code>transformation</code> property.
	 * 
	 * @param transformation
	 *            The <code>transformation</code> to set.
	 * @see #transformation
	 * 
	 * @return the same instance to allow dot chains
	 */
	public SearchExpression setTransformation(ITransformation transformation) {
		this.transformation = transformation;
		return this;
	}

	/**
	 * Getter of <code>joinType</code> property.
	 * 
	 * @return <code>joinType</code> property value.
	 * @see #joinType
	 */

	public JoinType getJoinType() {
		return this.joinType;
	}

	/**
	 * Setter of <code>joinType</code> property.
	 * 
	 * @param theJoinType
	 *            The <code>joinType</code> to set.
	 * @see #joinType
	 */
	public void setJoinType(JoinType theJoinType) {
		this.joinType = theJoinType;
		return;
	}

	/**
	 * Getter of <code>maxResults</code> property.
	 * 
	 * @return <code>maxResults</code> property value.
	 * @see #maxResults
	 */

	public int getMaxResults() {
		return this.maxResults;
	}

	/**
	 * Set a limit upon the number of objects to be retrieved.
	 * 
	 * @param theMaxResults
	 *            the maximum number of results
	 * @return this (for method chaining)
	 */
	public SearchExpression setMaxResults(int theMaxResults) {
		this.maxResults = theMaxResults;

		return this;
	}

	/**
	 * Creates and returns a copy of this object.
	 */
	public @Override
	Object clone() throws CloneNotSupportedException {
		SearchExpression se = new SearchExpression(this.entityClass, this.alias);
		se.transformation = this.transformation;
		se.criterias = new ArrayList<ISearchCriteria>(this.criterias);
		se.orders = new ArrayList<OrderCriteria>(this.orders);
		se.subExpressions = new ArrayList<SubSearchExpression>(
				this.subExpressions.size());
		for (SubSearchExpression expression : this.subExpressions) {
			se.subExpressions.add((SubSearchExpression) expression.clone());
		}
		return se;
	}

	/**
	 * Enable caching of this query result, provided query caching is enabled
	 * for the underlying session factory.
	 * 
	 * @param cacheable
	 *            Should the result be considered cacheable; default is to not
	 *            cache (false).
	 * @return this (for method chaining)
	 */
	public SearchExpression setCacheable(boolean cacheable) {
		this.cacheable = cacheable;

		return this;
	}

	/**
	 * Set the name of the cache region to use for query result caching.
	 * 
	 * @param cacheRegion
	 *            the name of a query cache region, or <tt>null</tt> for the
	 *            default query cache
	 * @return this (for method chaining)
	 * 
	 * @see #setCacheable
	 */
	public SearchExpression setCacheRegion(String cacheRegion) {
		this.cacheRegion = cacheRegion;
		return this;
	}

	/**
	 * @return the cacheable
	 */
	public boolean isCacheable() {
		return this.cacheable;
	}

	/**
	 * @return the cacheRegion
	 */
	public String getCacheRegion() {
		return this.cacheRegion;
	}

	/**
	 * Method used to remove a SearchCriteria from the current SearchExpression
	 * 
	 * @param criteria
	 *            The criteria to remove
	 * @return the same instance to allow dot chains
	 */
	public void removeSearchCriteria(ISearchCriteria criteria) {
		this.criterias.remove(criteria);
	}

	/**
	 * Method used to find if the search expression contains a certain path in
	 * its sub search expressions
	 * 
	 * @param path
	 *            Path to be searched in the search expression
	 * @return true if the alias path exists in the search expression, false
	 *         otherwise.
	 */
	public boolean hasPath(String path) {
		for (Iterator<SubSearchExpression> iter = this.getSubExpressions(); iter
				.hasNext();) {
			SubSearchExpression subExp = iter.next();
			if (subExp.getPath().equals(path) || subExp.hasPath(path)) {
				return true;
			}
		}
		return false;
	}

	/**
	 * Method used to find the alias of a given path.
	 * 
	 * @param path
	 *            Path to be searched in the search expression
	 * @return If exists, the alias associated to the path, null otherwise.
	 */
	public String getAlias(String path) {
		for (Iterator<SubSearchExpression> iter = this.getSubExpressions(); iter
				.hasNext();) {
			SubSearchExpression subExp = iter.next();
			if (subExp.getPath().equals(path)) {
				return subExp.getAlias();
			}

			String subAlias = subExp.getAlias(path);
			if (subAlias != null) {
				return subAlias;
			}
		}
		return null;
	}

	/**
	 * Replace path with alias.
	 * 
	 * @param theElement
	 *            the the element
	 * @return the string
	 */
	public String replacePathWithAlias(String theElement) {
		String[] orderPath = theElement.split("\\.");
		String element = "";

		for (int i = 0; i < orderPath.length - 1; i++) {
			String path = orderPath[i];
			if (this.hasPath(path)) {
				element = this.getAlias(path) + ".";
			} else {
				this.createAlias(path, element + path);
				element = path + ".";
			}
		}
		element += orderPath[orderPath.length - 1];

		return element;
	}
}
