/**
 * ValidatorEvaluator.java
 *
 * $Revision: 1.5 $ ($Date: 2011-04-06 04:45:10 $)
 *
 * Copyright (c) 2004 Smartmatic Corp.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of
 * Smartmatic Corp. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Corp.
 */
package com.smartmatic.waffle.persistence.service.impl;

import com.smartmatic.waffle.domain.IBaseDataObject;
import com.smartmatic.waffle.lang.annotations.basic.ValidatorListener;
import com.smartmatic.waffle.validator.IObjectDataValidator;
import com.smartmatic.waffle.validator.ObjectDataValidatorException;

/**
 * Class responsible to invoke the appropriate methods in a registered listener
 * of a entity once that the persistence service stored/updated/deleted the
 * entity.
 * 
 * @version 1.0 (16/08/2007)
 * @author Héctor Rodríguez (hector.rodriguez@smartmatic.com)
 */

public class ValidatorEvaluator {

	/**
	 * Enum that indicates the different modes supported by the
	 * ValidatorEvaluator
	 * 
	 * @version 1.0 (Aug 30, 2007)
	 * @author Ithamar Torres (itorres@smartmatic.com)
	 */

	public enum ValidatorMode {
		/**
		 * Action done before persist the object
		 */
		PREPERSIST,
		/**
		 * Action done after persist the object
		 */
		POSTPERSIST,
		/**
		 * Action done before persist the object
		 */
		PREUPDATE,
		/**
		 * Action done after persist the object
		 */
		POSTUPDATE,
		/**
		 * Action done before remove the object
		 */
		PREREMOVE,
		/**
		 * Action done after remove the object
		 */
		POSTREMOVE
	}

	/**
	 * Method responsible to invoke the proper validator method depending on the
	 * invoked mode and the configured listener of the component.
	 * 
	 * @param mode
	 *            The validation mode
	 * @param bcomponent
	 *            The object to invoke
	 * @throws ObjectDataValidatorException
	 */

	public static void invokeMethod(ValidatorMode mode,
			IBaseDataObject bcomponent) throws ObjectDataValidatorException {
		// Obtains the validator name
		ValidatorListener annotation = bcomponent.getClass().getAnnotation(
				ValidatorListener.class);
		if (annotation != null) {
			String clazzName = annotation.name();
			try {
				// Validate the Object given the mode
				Class clazz = Class.forName(clazzName);
				if (clazz != null) {
					IObjectDataValidator<IBaseDataObject> validator = (IObjectDataValidator<IBaseDataObject>) clazz
							.newInstance();
					if (mode == ValidatorMode.PREPERSIST) {
						validator.prePersist(bcomponent);
					} else if (mode == ValidatorMode.POSTPERSIST) {
						validator.postPersist(bcomponent);
					} else if (mode == ValidatorMode.PREUPDATE) {
						validator.preUpdate(bcomponent);
					} else if (mode == ValidatorMode.POSTUPDATE) {
						validator.postUpdate(bcomponent);
					} else if (mode == ValidatorMode.PREREMOVE) {
						validator.preRemove(bcomponent);
					} else if (mode == ValidatorMode.POSTREMOVE) {
						validator.postRemove(bcomponent);
					}
				}
			} catch (ClassNotFoundException e) {
				throw new ObjectDataValidatorException(
						ObjectDataValidatorException.ExceptionInfo.INVALID_LISTENER,
						e);
			} catch (ClassCastException e) {
				throw new ObjectDataValidatorException(
						ObjectDataValidatorException.ExceptionInfo.INVALID_LISTENER,
						e);
			} catch (InstantiationException e) {
				throw new ObjectDataValidatorException(
						ObjectDataValidatorException.ExceptionInfo.LISTENER_INNER_PROBLEM,
						e);
			} catch (IllegalAccessException e) {
				throw new ObjectDataValidatorException(
						ObjectDataValidatorException.ExceptionInfo.LISTENER_INNER_PROBLEM,
						e);
			}
		}
	}
}
