/**
 * ClientJNDIWrapper.java
 *
 * $Revision: 1.11 $ ($Date: 2011-04-06 05:08:28 $)
 *
 * Copyright (c) 2004 Smartmatic Corp.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of
 * Smartmatic Corp. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Corp.
 */
package com.smartmatic.waffle.client;

import java.util.Properties;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.rmi.PortableRemoteObject;

/**
 * Utility class used to obtain a remote reference to any managed resource by
 * the application server. It is used to communicate directly with the
 * application server and obtains a remote reference.
 * 
 * @author Ithamar Torres (itores@smartmatic.com)
 * @version 1.0 (23/10/2006)
 */
public class ClientJNDIWrapper {
	/**
	 * Holds the singleton instance exported by this class. This instance is
	 * going to be used as the only reference used to publish the functionality
	 * provided by this class
	 * 
	 * @author Ithamar Torres (itores@smartmatic.com)
	 */

	private static ClientJNDIWrapper singletonInstance;

	/**
	 * Holds the properties required to connect to the application server using
	 * the JNI standards
	 * 
	 * @author Ithamar Torres (itores@smartmatic.com)
	 */

	private Properties jniProperties;

	/**
	 * Holds the lookup mode used to obtain the references from the application
	 * server
	 * 
	 * @author Ithamar Torres (itores@smartmatic.com)
	 */

	private JNDILookupModeEnum lookupMode;

	/**
	 * Holds the reference to the context provider object. This object is used
	 * to configure the naming prefix used in the application.
	 */

	private IContextProvider contextProvider;

	/**
	 * Initializes the ClientJNIWrapper class with the properties required to
	 * communicate with the proper application server that will provide the
	 * remote objects
	 * 
	 * @param jniProps
	 *            The properties required to connect to the application server
	 *            using the JNDI standards
	 * @param lookupMode
	 *            The mode used to search the objects in the application server
	 * @param contextProvider
	 *            The context provider. Class used to indicate the
	 * @throws JNDICommunicationException
	 *             Exception during the JNDI initialization
	 */

	public synchronized static void init(Properties jniProps,
			JNDILookupModeEnum lookupMode, IContextProvider contextProvider)
			throws JNDICommunicationException {
		if (ClientJNDIWrapper.singletonInstance != null) {
			throw new JNDICommunicationException(
					JNDICommunicationException.ExceptionInfo.CLIENT_ALREADY_INITIALIZED);
		}

		ClientJNDIWrapper.singletonInstance = new ClientJNDIWrapper(jniProps,
				lookupMode, contextProvider);
	}

	/**
	 * Resets the properties used to connect to the server. It is used to modify
	 * the server connection parameters once this class was initialized
	 * 
	 * @param jniProps
	 *            The properties required to connect to the application server
	 *            using the JNDI standards
	 */

	public static void reset(Properties jniProps) {
		ClientJNDIWrapper.singletonInstance.jniProperties = jniProps;
	}

	/**
	 * Obtains the singleton instance published by this class
	 * 
	 * @return A instance of a ClientJNIWrapper
	 * @throws JNDICommunicationException
	 *             Exception during the JNDI initialization
	 */

	public static ClientJNDIWrapper getInstance()
			throws JNDICommunicationException {
		if (ClientJNDIWrapper.singletonInstance == null) {
			throw new JNDICommunicationException(
					JNDICommunicationException.ExceptionInfo.CLIENT_NOT_INITIALIZED);
		}
		return ClientJNDIWrapper.singletonInstance;
	}

	/**
	 * * Creates a new instance of a ClientJNIWrapper. It is a private
	 * constructor because this class is implemented as a singleton.
	 * 
	 * @param jniProps
	 *            The properties required to connect to the application server
	 *            using the JNI standards
	 * @param lookupMode
	 *            The mode used to search the objects in the application server
	 * @param contextProvider
	 *            The context provider. Class used to indicate the
	 */
	private ClientJNDIWrapper(Properties jniProps,
			JNDILookupModeEnum lookupMode, IContextProvider contextProvider) {
		this.jniProperties = jniProps;
		this.lookupMode = lookupMode;
		this.contextProvider = contextProvider;
	}

	/**
	 * * Obtains the remote reference to the required object.
	 * 
	 * @author Ithamar Torres (itores@smartmatic.com)
	 * 
	 * @param <Type>
	 *            The object type to return
	 * @param prefix
	 *            The context prefix
	 * @param JNDIName
	 *            The jndi name of the object to lookup
	 * @param theClass
	 *            The class that has to be searched
	 * @return The class that has to be searched
	 * @throws JNDICommunicationException
	 *             Exception during the JNDI initialization
	 */

	@SuppressWarnings("unchecked")
	private <Type> Type getReference(String prefix, String JNDIName,
			Class<Type> theClass) throws JNDICommunicationException {
		try {
			Context ctx = new InitialContext(this.jniProperties);
			if (prefix.length() > 0) {
				prefix += "/";
			}
			String suffix = "/" + this.lookupMode.getLookupName();

			Object reference = ctx.lookup(prefix + JNDIName + suffix);
			Type result = (Type) PortableRemoteObject.narrow(reference,
					theClass);
			return result;
		} catch (NamingException e) {
			throw new JNDICommunicationException(
					JNDICommunicationException.ExceptionInfo.NAMING_EXCEPTION,
					e);
		}
	}

	/**
	 * * Obtains the remote reference to the required object.
	 * 
	 * @author Ithamar Torres (itores@smartmatic.com)
	 * 
	 * @param <Type>
	 *            The object type to return
	 * @param prefix
	 *            The context prefix
	 * @param JNDIName
	 *            The jndi name of the object to lookup
	 * @param theClass
	 *            The class that has to be searched
	 * @return The class that has to be searched
	 * @throws JNDICommunicationException
	 *             Exception during the JNDI initialization
	 */

	@SuppressWarnings("unchecked")
	private <Type> Type getReference(String prefix, String JNDIName,
			Class<Type> theClass, Properties jniProperties,
			JNDILookupModeEnum lookupMode) throws JNDICommunicationException {
		try {
			Context ctx = new InitialContext(jniProperties);
			if (prefix.length() > 0) {
				prefix += "/";
			}
			String suffix = "/" + lookupMode.getLookupName();

			Object reference = ctx.lookup(prefix + JNDIName + suffix);
			Type result = (Type) PortableRemoteObject.narrow(reference,
					theClass);
			return result;
		} catch (NamingException e) {
			throw new JNDICommunicationException(
					JNDICommunicationException.ExceptionInfo.NAMING_EXCEPTION,
					e);
		}
	}

	/**
	 * * Obtains the remote reference to the required object.
	 * 
	 * @author Ithamar Torres (itores@smartmatic.com)
	 * 
	 * @param <Type>
	 *            The object type to return
	 * @param jndiName
	 *            The jndi name of the object to lookup
	 * @param theClass
	 *            The class that has to be searched
	 * @return The object reference
	 * @throws JNDICommunicationException
	 *             Exception during the JNDI initialization
	 */

	public <Type> Type getReference(String JNDIName, Class<Type> theClass)
			throws JNDICommunicationException {
		return getReference(this.contextProvider.getContext(), JNDIName,
				theClass);
	}

	/**
	 * * Obtains the remote reference to the required object.
	 * 
	 * @author Ithamar Torres (itores@smartmatic.com)
	 * 
	 * @param <Type>
	 *            The object type to return
	 * @param jndiName
	 *            The jndi name of the object to lookup
	 * @param theClass
	 *            The class that has to be searched
	 * @param jniProperties
	 *            The properties required to connect to the application server
	 *            using the JNI standards
	 * @param lookupMode
	 *            The mode used to search the objects in the application server
	 * 
	 * @return The object reference
	 * @throws JNDICommunicationException
	 *             Exception during the JNDI initialization
	 */

	private <Type> Type getReference(String JNDIName, Class<Type> theClass,
			Properties jniProperties, JNDILookupModeEnum lookupEnum)
			throws JNDICommunicationException {
		return getReference(this.contextProvider.getContext(), JNDIName,
				theClass, jniProperties, lookupEnum);
	}

	/**
	 * Obtains a reference from the application server.
	 * 
	 * @param <Type>
	 *            The object type to return
	 * @param theClass
	 *            The class that has to be searched
	 * @return The object reference
	 * @throws JNDICommunicationException
	 *             Exception during the JNDI initialization
	 */

	public <Type> Type getReference(Class<Type> theClass)
			throws JNDICommunicationException {
		return this.getReference(theClass.getName(), theClass);
	}

	/**
	 * Obtains a reference from the application server.
	 * 
	 * @param <Type>
	 *            The object type to return
	 * @param theClass
	 *            The class that has to be searched
	 * @param jniProperties
	 *            The properties required to connect to the application server
	 *            using the JNI standards
	 * @param lookupMode
	 *            The mode used to search the objects in the application server
	 * 
	 * @return The object reference
	 * @throws JNDICommunicationException
	 *             Exception during the JNDI initialization
	 */

	public <Type> Type getReference(Class<Type> theClass,
			Properties jniProperties, JNDILookupModeEnum look)
			throws JNDICommunicationException {
		return this.getReference(theClass.getName(), theClass, jniProperties,
				look);
	}

	/**
	 * Obtains a reference from the application server given the complete jndi
	 * address of the component to lookup.
	 * 
	 * @param <Type>
	 *            The object type to return
	 * @param theClass
	 *            The class that has to be searched
	 * @param jndiName
	 *            The jndi name of the object to lookup
	 * @return The object reference
	 * @throws JNDICommunicationException
	 *             Exception during the JNDI initialization
	 */

	@SuppressWarnings("unchecked")
	public <Type> Type lookup(String jndiName, Class<Type> theClass)
			throws JNDICommunicationException {
		try {
			Context ctx = new InitialContext(this.jniProperties);

			Object reference = ctx.lookup(jndiName);
			Type result = (Type) PortableRemoteObject.narrow(reference,
					theClass);
			return result;
		} catch (NamingException e) {
			throw new JNDICommunicationException(
					JNDICommunicationException.ExceptionInfo.NAMING_EXCEPTION,
					e);
		}
	}
}
