/**
 * BaseException.java
 *
 * $Revision: 1.9 $ ($Date: 2011-04-06 05:08:30 $)
 *
 * Copyright (c) 2004 Smartmatic Corp.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of
 * Smartmatic Corp. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Corp.
 */
package com.smartmatic.waffle.lang;

import com.smartmatic.waffle.text.impl.BundleKeyFormatter;
import com.smartmatic.waffle.text.impl.DefaultTextFormatter;
import com.smartmatic.waffle.text.spec.ITextFormatter;

/**
 * <p>
 * Abstract base class for all application's exceptions. All exceptions
 * belonging to the <code>smartmatic</code> package (or any of its sub-packages)
 * must extend from this class.
 * </p>
 * 
 * 
 * <p>
 * All objects extending from this class must define an enumerator implementing
 * the <code>IExceptionInfo</code> interface. This enumerator should list all
 * possible causes why that exception in particular might be thrown. The
 * enumerator will then be used as parameter to this class' constructor to
 * specify the cause by which the exception is being thrown at the moment of
 * creation.
 * </p>
 * 
 * <p>
 * In summary, any exception inside the <code>smartmatic</code> package (or any
 * of its sub-packages) must:
 * <ol>
 * <li>Extend from this class (<code>BaseException</code>).</li>
 * <li>Define a public enumeration for each possible cause why that exception
 * might be thrown. The enumeration must implement the
 * <code>IExceptionInfo</code> interface. See the documentation of
 * <code>IExceptionInfo</code> for more details.</li>
 * <li>Require one of the mentioned enumeration's specific value in each of its
 * constructors and passed the value to its superclass'.</li>
 * </ol>
 * </p>
 * 
 * @author Ithamar Torres (itorres@smartmatic.com)
 * @version 1.0 (03/05/2006)
 */
public class BaseException extends Exception {
	/**
	 * Holds the name of the resource bundle that is used to obtain the error
	 * messages displayed when a exception is thrown
	 */

	private static final String ERROR_RESOURCE_NAME = "properties.exception_messages";

	/**
	 * Provides the logic to obtain the ITextFormatter that has to be used in
	 * the exception handling in the system.
	 * 
	 * @return A properly configured ITextFormatter
	 */

	private static ITextFormatter getErrorTextFormatter() {
		ITextFormatter result = null;
		try {
			result = new BundleKeyFormatter(BaseException.ERROR_RESOURCE_NAME);
		} catch (Exception e) {
			result = new DefaultTextFormatter();
		}
		return result;
	}

	/**
	 * Sets the Textformatter used to set locale specific information in the
	 * system
	 * 
	 * @param formatter
	 *            The text formatter to set
	 */

	public static void setErrorTextFormatter(ITextFormatter formatter) {
		BaseException.textFormatter = formatter;
	}

	/*
	 * This class contains 1.-an enumerated value named 'info' corresponding to
	 * the cause of the exception itself, 2.-an optional array of strings (named
	 * 'messageParameters') used to provide values for possible place holders in
	 * the exception's message, 3.-a Throwable (named 'throwable' and defined in
	 * the superclass) corresponding to a possible nested exception that in fact
	 * triggered this one. All aplication's custom made exceptions must extend
	 * from this class to ensure a consistent management of errors.
	 */
	/**
	 * Holds the version of the class. It is used by the serialization engine to
	 * determine when a class its equivalent to another
	 * 
	 * @author Edgar Trejo (edgar.trejo@smartmatic.com)
	 */

	private static final long serialVersionUID = -3548133297716600138L;

	/**
	 * <p>
	 * An enumerated value regarding the cause of the exception.
	 * </p>
	 * 
	 * @see #IExceptionInfo
	 */

	protected IExceptionInfo info;

	/**
	 * <p>
	 * The <code>ITextFormatter</code> responsible to format the exception
	 * messages.
	 * </p>
	 * The instance that will be used to obtain the implementation of the
	 * ITextFormatter will be obtained through a JNDI lookup of the name
	 * <code>java:/smartmatic/services/smartmatic.text.ITextFormatter</code>
	 */

	public static ITextFormatter textFormatter = BaseException
			.getErrorTextFormatter();

	/**
	 * <p>
	 * An array of <code>Object</code> with all message placeholder values for
	 * the <code>BaseException.info.getMessage()</code> error message.
	 * </p>
	 */

	protected Object[] messageParameters;

	/**
	 * The exception message.
	 */

	private String message;

	/**
	 * <p>
	 * Constructor for an instance of this class having only the enumerated
	 * cause of the exception.
	 * </p>
	 * 
	 * @author Ithamar Torres (itorres@smartmatic.com)
	 * @param exceptionInfo
	 *            an enumeration value regarding the cause of the exception
	 * @revision (03/05/2006) Initial revision (submitted by
	 *           itorres@smartmatic.com)
	 */
	public BaseException(IExceptionInfo exceptionInfo) {
		super();
		this.info = exceptionInfo;
		this.message = BaseException.textFormatter.getText(this.info
				.getMessage());
	}

	/**
	 * <p>
	 * Constructor for an instance of this class having both the enumerated
	 * cause of the exception and the nested <code>Throwable</code> that might
	 * have triggered this exception.
	 * </p>
	 * 
	 * @author Ithamar Torres (itorres@smartmatic.com)
	 * @param exceptionInfo
	 *            an enumeration value regarding the cause of the exception
	 * @param throwable
	 *            a nested exception
	 * @revision (03/05/2006) Initial revision (submitted by
	 *           itorres@smartmatic.com)
	 */
	public BaseException(IExceptionInfo exceptionInfo, Throwable throwable) {
		super(throwable);
		this.info = exceptionInfo;
		this.message = BaseException.textFormatter.getText(this.info
				.getMessage());
	}

	/**
	 * <p>
	 * Constructor for an instance of this class having the enumerated cause of
	 * the exception, the nested <code>Throwable</code> that might have
	 * triggered this exception, and an array of placeholder values used as
	 * parameter for this exception's message contained in the <code>info</code>
	 * attribute.
	 * </p>
	 * 
	 * @author Ithamar Torres (itorres@smartmatic.com)
	 * @param exceptionInfo
	 *            an enumeration value regarding the cause of the exception
	 * @param throwable
	 *            a nested exception
	 * @param parameters
	 *            the array of placeholder values used as parameters for this
	 *            exception's enumerated cause's related message
	 * @revision (03/05/2006) Initial revision (submitted by
	 *           itorres@smartmatic.com)
	 */
	public BaseException(IExceptionInfo exceptionInfo, Throwable throwable,
			Object... parameters) {
		super(throwable);
		this.info = exceptionInfo;
		this.messageParameters = parameters;
		this.message = BaseException.textFormatter.getText(
				this.info.getMessage(), parameters);
	}

	/**
	 * <p>
	 * Constructor for an instance of this class having the cause of the
	 * exception, and an array of placeholder values used as parameter for this
	 * exception's message contained in the <code>info</code> attribute.
	 * </p>
	 * 
	 * @author Ithamar Torres (itorres@smartmatic.com)
	 * @param exceptionInfo
	 *            an enumeration value regarding the cause of the exception
	 * @param parameters
	 *            the array of placeholder values used as parameters for this
	 *            exception's enumerated cause's related message
	 * @revision (03/05/2006) Initial revision (submitted by
	 *           itorres@smartmatic.com)
	 */
	public BaseException(IExceptionInfo exceptionInfo, Object... parameters) {
		super();
		this.info = exceptionInfo;
		this.messageParameters = parameters;
		this.message = BaseException.textFormatter.getText(
				this.info.getMessage(), parameters);
	}

	/**
	 * <p>
	 * Getter. Returns information regarding the cause of the exception.
	 * </p>
	 * 
	 * @author Ithamar Torres (itorres@smartmatic.com)
	 * @return the enumerated value of the cause of this exception.
	 * @see #info
	 * @revision (03/05/2006) Initial revision (submitted by
	 *           itorres@smartmatic.com)
	 */
	public IExceptionInfo getInfo() {
		return this.info;
	}

	/**
	 * <p>
	 * Getter. Returns all parameters for the natural language message
	 * associated with this exception.
	 * </p>
	 * 
	 * @author Ithamar Torres (itorres@smartmatic.com)
	 * @return all message's placeholder parameters.
	 * @see #messageParameters
	 * @revision (03/05/2006) Initial revision (submitted by
	 *           itorres@smartmatic.com)
	 */
	public Object[] getMessageParameters() {
		return this.messageParameters;
	}

	/**
	 * <p>
	 * Returns the natural language message regarding the cause of this
	 * exception.
	 * </p>
	 * 
	 * @author Ithamar Torres (itorres@smartmatic.com)
	 * @return the natural language message regarding the cause if this
	 *         exception.
	 * @revision (03/05/2006) Initial revision (submitted by
	 *           itorres@smartmatic.com)
	 */
	@Override
	public String getMessage() {
		return this.message;
	}

	/**
	 * Returns the message key used to locate the exception message
	 * 
	 * @return
	 */

	public String getMessageKey() {
		return this.info.getMessage();
	}

}