/**
 * ZipCompressor.java
 *
 * $Revision: 1.4 $ ($Date: 2011-04-06 05:08:28 $)
 *
 * Copyright (c) 2004 Smartmatic Corp.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of
 * Smartmatic Corp. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Corp.
 */
package com.smartmatic.waffle.util.zip;

import java.io.ByteArrayOutputStream;
import java.util.zip.DataFormatException;
import java.util.zip.Deflater;
import java.util.zip.Inflater;

import com.smartmatic.waffle.lang.ZipWrongMessageFormatException;

/**
 * <p>
 * Description: Class used to compress and decompress information. This class
 * has as objective reduce the amount of information that needs to be sent in
 * the network. The transmission protocol specifies that the information sent by
 * the voting machines is compressed. This is the class responsible of
 * decompress the information sent by the voting machines. The algorithm used to
 * compress / decompress used by this class is the same algorithm that the .ZIP
 * files
 * </p>
 * 
 * @author Ithamar Torres (itorres@smartmatic.com)
 * @version 1.0
 */
public class ZipCompressor {
	/**
	 * Field that defines the size of the blocks that are going to be
	 * compressed/decompressed. This field does not limit the amount of
	 * information that can be compressed/decompressed. Only is for internal use
	 * and in case of receive a block bigger than the block size the process of
	 * compression / decompression is made spiting the information in blocks
	 * small enough to be processed. This process is done internally and the
	 * caller does not have to know anything about the block size
	 */

	private static final int BLOCK_SIZE = 4096;

	/**
	 * Compression rate that will be used by default if the programmer does not
	 * specify another compression rate
	 */

	private static final int DEFAULT_COMPRESSION_METHOD = Deflater.BEST_COMPRESSION;

	/**
	 * Method that allows to decompress information that previously was
	 * compressed
	 * 
	 * @param compressedData
	 *            byte[] information that needs to be decompressed
	 * @return byte[] the information decompressed
	 * @throws ZipWrongMessageFormatException
	 *             If there is a problem in the format of the information
	 * @throws ZipCompressionException
	 *             thrown in the process of compress or decompress a message.
	 */
	public static byte[] decompress(byte[] compressedData)
			throws ZipCompressionException {
		try {
			// Decompress the compressed data given in BLOCK_SIZE blocks
			Inflater decompresser = new Inflater(false);
			decompresser.setInput(compressedData);

			byte[] buffer = new byte[ZipCompressor.BLOCK_SIZE];
			ByteArrayOutputStream byteStream = new ByteArrayOutputStream();

			int length = -1;
			while (!decompresser.finished()) {
				length = decompresser.inflate(buffer);
				byteStream.write(buffer, 0, length);
			}
			decompresser.end();
			return byteStream.toByteArray();
		} catch (DataFormatException e) {
			throw new ZipCompressionException(
					ZipCompressionException.ExceptionInfo.INVALID_DATA_FORMAT,
					e);
		}
	}

	/**
	 * Method that allow to compress information using the default compression
	 * rate
	 * 
	 * @param rawData
	 *            byte[] Information that needs to be compressed
	 * @return byte[] The compressed information
	 */

	public static byte[] compress(byte[] rawData) {
		return compress(rawData, ZipCompressor.DEFAULT_COMPRESSION_METHOD);
	}

	/**
	 * Method that allow to compress information using a compression rate
	 * specified by the caller.
	 * 
	 * @param rawData
	 *            byte[] The information that needs to be compressed
	 * @param compressionRate
	 *            int Compression rate that will be used in the compression
	 * @return byte[] The compressed information
	 */

	public static byte[] compress(byte[] rawData, int compressionRate) {
		// Compress the data given in BLOCK_SIZE blocks
		ByteArrayOutputStream byteStream = new ByteArrayOutputStream();
		byte[] buffer = new byte[ZipCompressor.BLOCK_SIZE];

		Deflater compresser = new Deflater(compressionRate, false);
		compresser.setInput(rawData);
		compresser.finish();

		while (!compresser.finished()) {
			int length = compresser.deflate(buffer);
			byteStream.write(buffer, 0, length);
		}
		compresser.end();
		return byteStream.toByteArray();
	}
}
