/**
 * @file DiagnosticPrintDocument.cxx
 * @brief Body diagnostic print document class
 * @date 19/09/2012
 * @author Yadickson Soto
 */

#include "DiagnosticPrintDocument.hxx"
#include <Voting/OperationStatus.hxx>
#include "Resources/election-string-resources-manager.h"
#include "Printing/TranslatorTextConfiguration.h"
#include "Environment/Configurations/resource-path-provider.h"
#include <Log/SMTTLogManager.h>
#include <Voting/Election/ElectionInstalationLanguages.h>
#include "Voting/set-language-functionality.h"
#include <Graphics/Encoding2D/QREncoding.h>
#include <System/Utils/StringUtils.hxx>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

#define N_(String) (String) ///<macro for gettext not translate

static const int MAX_CHARACTERS_PER_LINE = 18;

using namespace Smartmatic::SAES::Voting::VotingDevice;
using namespace Smartmatic::SAES::Configuration;
using namespace Smartmatic::SAES::Common::Configuration;
using namespace Smartmatic::SAES::Printing;
using namespace Smartmatic::SAES::Voting;
using namespace Smartmatic::SAES::Resources;
using namespace Smartmatic::System::Exception;
using namespace Smartmatic::System;
using namespace Smartmatic::Environment;
using namespace Smartmatic::Log;
using namespace Smartmatic::Graphics::Encoding2D;
using namespace Smartmatic::SAES::Voting::Status;

/**
 * @brief  Evaluates if the line needs to be splitted in two.
 * @author manuel.argyles
 * @date   May  7, 2018
 * @param  line The line to print.
 * @return True if the line needs to be split, false if not
 */
static bool needsToBeSplit(const std::string& line)
{
    return line.size() > MAX_CHARACTERS_PER_LINE;
}

/**
 * @brief  Split lines into a vector of lines.
 * @author manuel.arguelles
 * @date   May  8, 2018
 */
static std::vector<std::string> splitLine(const std::string& line)
{
    std::vector<std::string> lines;

    if (line.size() > MAX_CHARACTERS_PER_LINE)
    {
        std::string newLine;
        std::vector<std::string> bySpaces;
        std::vector<std::string>::const_iterator it;

        Utils::StringUtils::splitToVector(line, " ", bySpaces);

        for (it = bySpaces.begin(); it != bySpaces.end(); ++it)
        {
            if (it->size() + newLine.size() <= MAX_CHARACTERS_PER_LINE)
            {
                newLine += (*it + " ");
            }
            else
            {
                lines.push_back(newLine);
                newLine = (*it + " ");
            }
        }

        if (lines.back().size() + newLine.size() <= MAX_CHARACTERS_PER_LINE)
        {
            lines.back() += newLine;
        }
        else
        {
            lines.push_back(newLine);
        }
    }
    else
    {
        lines.push_back(line);
    }

    return lines;
}

ISMTTLog * DiagnosticPrintDocument::logger = SMTTLogManager::GetLogger ( "Smartmatic.SAES.Printing.DiagnosticPrintDocument", GETTEXT_PACKAGE );

DiagnosticPrintDocument::DiagnosticPrintDocument ( OperationStatus * operationStatus )
    : BasePrintDocument(), operationStatus ( operationStatus )
{
    configuration = SAESConfigurationManager::getInstance()->getConfiguration()->getConfigurationPrinter().getConfigurationDiagnosticReport();
    this->multilanguage = configuration.getMultilanguage();
    bool process = configuration.getProcessQRCode();

    data = operationStatus->getQRDiagnosticResult ( process );
}

void DiagnosticPrintDocument::PrintDoc()
{
    std::list<Election::Language> languages;
    languages = Election::ElectionInstalationLanguages::getInstance()->getLanguages();

    if ( multilanguage && !languages.empty() )
    {
        std::list<Election::Language>::iterator it;

        std::string previousLanguage;
        std::string previousCountry;
        SetLanguageFunctionality::getCurrentLanguageCountry ( previousLanguage, previousCountry );

        for ( it = languages.begin();
                it != languages.end();
                ++it )
        {
            SetLanguageFunctionality::setNewVotingLanguageByLanguageCountry ( ( *it ).getLanguage(), ( *it ).getCountry() );
            processDocument();
        }

        setQR();
        SetLanguageFunctionality::setNewVotingLanguageByLanguageCountry ( previousLanguage, previousCountry );
    }
    else
    {
        processDocument();
        setQR();
    }
}

void DiagnosticPrintDocument::processDocument()
{
    DocumentTextOptions textInstallationTitleLevel1 = TranslatorTextConfiguration::TextConfigurationToDocumentTextOptions ( configuration.getTextInstallationTitleLevel1() );
    DocumentTextOptions textInstallationTitleLevel2 = TranslatorTextConfiguration::TextConfigurationToDocumentTextOptions ( configuration.getTextInstallationTitleLevel2() );
    DocumentTextOptions textInstallationTitleLevel3 = TranslatorTextConfiguration::TextConfigurationToDocumentTextOptions ( configuration.getTextInstallationTitleLevel3() );
    DocumentTextOptions textInstallationInfo = TranslatorTextConfiguration::TextConfigurationToDocumentTextOptions ( configuration.getTextInstallationInfo() );
    std::list<DocumentTextOptions> listDocumentTextOptions;
    listDocumentTextOptions.push_back ( textInstallationTitleLevel3 );
    listDocumentTextOptions.push_back ( textInstallationInfo );
    // title
    this->AddBreak();
    this->NewTextBlock ( textInstallationTitleLevel1 );
    this->DrawString ( " ", true );
    this->Indent ( 2 );

    Glib::ustring title = Glib::ustring::compose ( _ ( "Smartmatic.SAES.Printing.DiagnosticPrintDocument.Title" ), PACKAGE_VERSION );

    this->DrawString ( title, true );
    this->UnIndent ( 2 );
    this->DrawString ( " ", true );
    this->AddBreak ();
    this->NewTextColunmsBlock ( listDocumentTextOptions );
    this->Indent ( 2 );

    std::string serialDetected = operationStatus->getVotingMachineStatusWrapper()->getSerialDetected();
    std::string serial = operationStatus->getVotingMachineStatusWrapper()->getSerial();
    std::string abledSerialDetected = operationStatus->getVotingMachineStatusWrapper()->getAbled20SerialDetected();
    std::string observation = operationStatus->getVotingMachineStatusWrapper()->getObservation();

    std::string entry = _ ( "Serial Detected" );

    printEntryValue(entry, serialDetected);
    printEntryValue(" ", " ");

    if ( !serial.empty() )
    {
        entry = _ ( "Smartmatic.SAES.Printing.DiagnosticPrintDocument.Serial" );

        printEntryValue(entry, serial);
        printEntryValue(" ", " ");
    }

    if ( !abledSerialDetected.empty() )
    {
        entry = _ ( "Smartmatic.SAES.Printing.DiagnosticPrintDocument.AbledSerialDetected" );

        printEntryValue(entry, abledSerialDetected);
        printEntryValue(" ", " ");
    }

    std::string date = operationStatus->getVotingMachineStatusWrapper()->getDate();

    if ( !date.empty() )
    {
        entry = _ ( "Smartmatic.SAES.Printing.DiagnosticPrintDocument.Date" );

        printEntryValue(entry, date);
        printEntryValue(" ", " ");
    }

    std::string zipCode = operationStatus->getVotingMachineStatusWrapper()->getZipCode();

    if ( !zipCode.empty() )
    {
        entry = _ ( "Smartmatic.SAES.Printing.DiagnosticPrintDocument.ZipCode" );

        printEntryValue(entry, zipCode);
    }

    this->UnIndent ( 2 );
    this->DrawString ( " ", false );
    this->AddBreak ();
    // result
    this->NewTextBlock ( textInstallationTitleLevel2 );
    //this->Indent ( 2 );
    this->DrawString ( _ ( "Smartmatic.SAES.Printing.DiagnosticPrintDocument.Result" ), true );
    this->DrawString ( " ", true );
    //this->UnIndent ( 2 );
    this->AddBreak ();
    this->NewTextColunmsBlock ( listDocumentTextOptions );
    this->Indent ( 2 );

    std::list <Status::DeviceStatus> devices;
    std::list <Status::DeviceStatus>::iterator it;

    devices = operationStatus->getVotingMachineStatusWrapper()->getDeviceStatusList ();

    for ( it = devices.begin();
            it != devices.end();
            ++it )
    {
        Glib::ustring device = getType ( ( *it ).getVMType() );
        Glib::ustring status = getStatus ( ( *it ).getVMStatus().getStatus().get() );

        printEntryValue(device, status);
        printEntryValue(" ", " ");
    }

    this->UnIndent ( 2 );
    this->AddBreak ();
    this->DrawString ( " ", true );
    this->DrawString ( " ", true );
    this->AddBreak();

    VotingMachineStatus::Custom_checklistSequence & cseq ( operationStatus->getVotingMachineStatusWrapper()->getVotingMachineStatus().getCustom_checklist() );

    if ( !cseq.empty() )
    {
        VotingMachineStatus::Custom_checklistIterator cit;

        this->NewTextBlock ( textInstallationTitleLevel2 );
        //this->Indent ( 2 );
        this->DrawString ( _ ( "Checklist" ), true );
        this->DrawString ( " ", true );
        //this->UnIndent ( 2 );
        this->AddBreak ();

        this->NewTextColunmsBlock ( listDocumentTextOptions );
        this->Indent ( 2 );

        for ( cit = cseq.begin();
                cit != cseq.end();
                ++cit )
        {
            Glib::ustring customCode = ( ( *cit ).getCustomCode().present() ? ( *cit ).getCustomCode().get() : "null" );
            Glib::ustring status = ( ( *cit ).getValue().get() == "1" ? _ ( "Smartmatic.SAES.Printing.DiagnosticPrintDocument.Ok" ) : _ ( "Smartmatic.SAES.Printing.DiagnosticPrintDocument.NOk" ) );

            printEntryValue(customCode, status);
            printEntryValue(" ", " ");
        }

        this->UnIndent ( 2 );
        this->AddBreak ();
        this->DrawString ( " ", true );
        this->DrawString ( " ", true );
        this->AddBreak();
    }

    if ( !observation.empty() )
    {
        // observation
        this->NewTextBlock ( textInstallationTitleLevel2 );
        //this->Indent ( 2 );
        this->DrawString ( _ ( "Custom Remark" ), true );
        this->DrawString ( " ", true );
        //this->UnIndent ( 2 );
        this->AddBreak ();

        this->NewTextBlock ( textInstallationInfo );
        this->DrawString ( " ", true );
        this->DrawString ( observation, true );
        this->DrawString ( " ", true );
        this->AddBreak ();
    }

    this->NewTextBlock ( textInstallationTitleLevel2 );
    this->Indent ( 2 );
    this->DrawString ( " ", true );
    this->DrawString ( " ", true );
    this->UnIndent ( 2 );
    this->AddBreak ();

    logger->Audit ( N_ ( "Smartmatic.SAES.Printing.DiagnosticPrintDocument.PrintReport" ) );
}

void DiagnosticPrintDocument::BeginPrint()
{
}

void DiagnosticPrintDocument::EndPrint()
{
}

std::string DiagnosticPrintDocument::getType ( Smartmatic::SAES::Common::Configuration::DeviceType & type )
{
    std::string text;

    switch ( type )
    {
        case Smartmatic::SAES::Common::Configuration::DeviceType::CONFIGURATION_P168A:
        case Smartmatic::SAES::Common::Configuration::DeviceType::CONFIGURATION_ABLED:
            text = _ ( "Smartmatic.SAES.Printing.DiagnosticPrintDocument.Able" );
            break;

        case Smartmatic::SAES::Common::Configuration::DeviceType::CONFIGURATION_ALARMBOX:
            text = _ ( "Smartmatic.SAES.Printing.DiagnosticPrintDocument.AlarmBox" );
            break;

        case Smartmatic::SAES::Common::Configuration::DeviceType::CONFIGURATION_BARCODE:
            text = _ ( "Smartmatic.SAES.Printing.DiagnosticPrintDocument.Barcode" );
            break;

        case Smartmatic::SAES::Common::Configuration::DeviceType::CONFIGURATION_FLASH:
            text = _ ( "Smartmatic.SAES.Printing.DiagnosticPrintDocument.Flash" );
            break;

        case Smartmatic::SAES::Common::Configuration::DeviceType::CONFIGURATION_PRINTER:
            text = _ ( "Smartmatic.SAES.Printing.DiagnosticPrintDocument.Printer" );
            break;

        case Smartmatic::SAES::Common::Configuration::DeviceType::CONFIGURATION_SMARTCARD:
            text = _ ( "Smartmatic.SAES.Printing.DiagnosticPrintDocument.Smartcard" );
            break;

        case Smartmatic::SAES::Common::Configuration::DeviceType::CONFIGURATION_TOUCHSCREENCALIBRATION:
            text = _ ( "Smartmatic.SAES.Printing.DiagnosticPrintDocument.TouchScreenCalibration" );
            break;

        case Smartmatic::SAES::Common::Configuration::DeviceType::CONFIGURATION_ETHERNET:
            text = _ ( "Smartmatic.SAES.Printing.DiagnosticPrintDocument.Ethernet" );
            break;

        case Smartmatic::SAES::Common::Configuration::DeviceType::CONFIGURATION_EURN:
            text = _ ( "eUrn" );
            break;

        case Smartmatic::SAES::Common::Configuration::DeviceType::CONFIGURATION_MOUSE:
        case Smartmatic::SAES::Common::Configuration::DeviceType::CONFIGURATION_KEYBOARD:
        case Smartmatic::SAES::Common::Configuration::DeviceType::CONFIGURATION_UNKNOWN:
        case Smartmatic::SAES::Common::Configuration::DeviceType::CONFIGURATION_USBSTORAGEMOUNTED:
        default:
            text = _ ( "Smartmatic.SAES.Printing.DiagnosticPrintDocument.Unknown" );
            break;
    }

    return text;
}

std::string DiagnosticPrintDocument::getStatus ( Status::VMStatus & status )
{
    std::string text;

    switch ( status )
    {
        case Status::VMStatus::VOTING_MACHINE_STATUS_OK:
            text = _ ( "Smartmatic.SAES.Printing.DiagnosticPrintDocument.Ok" );
            break;

        case Status::VMStatus::VOTING_MACHINE_STATUS_NOTDIAGNOSED:
            text = _ ( "Smartmatic.SAES.Printing.DiagnosticPrintDocument.N/A" );
            break;

        case Status::VMStatus::VOTING_MACHINE_STATUS_RUN:
        case Status::VMStatus::VOTING_MACHINE_STATUS_ERROR:
        default:
            text = _ ( "Smartmatic.SAES.Printing.DiagnosticPrintDocument.NOk" );
            break;
    }

    return text;
}

void DiagnosticPrintDocument::setQR()
{
    if ( configuration.getShowBarCode() )
    {
        switch ( getBackendType() )
        {
            case PrinterBackendType::CONFIGURATION_GTKPRINTING:
                setQRCodeImage();
                break;

            default:
            case PrinterBackendType::CONFIGURATION_DIRECTPRINTING:
                setQRCode();
                break;
        }

        DocumentTextOptions textInstallationTitleLevel2 = TranslatorTextConfiguration::TextConfigurationToDocumentTextOptions ( configuration.getTextInstallationTitleLevel2() );
        this->NewTextBlock ( textInstallationTitleLevel2 );
        this->Indent ( 2 );
        this->DrawString ( " ", true );
        this->DrawString ( " ", true );
        this->UnIndent ( 2 );
        this->AddBreak ();
        this->AddBreak ();
    }
}

void DiagnosticPrintDocument::setQRCode()
{
    SafeBuffer toEncode ( data );
    Glib::RefPtr<Gdk::Pixbuf> backgroundImage;
    QRCodeConfig config = configuration.getQRCodeConfig();
    int pixel_size = config.getSizeQRCode();
    int margin = config.getMarginQRCode();
    int version = config.getVersionQRCode();
    QRecLevel correctionLevel = SAESConfigurationManager::getInstance()->GetCorrectionModeQRCode ( config.getCorrectionModeQRCode() );

    if ( toEncode.GetSize() > 0 )
    {
        this->DrawQRCode (
            toEncode
            , pixel_size
            , margin
            , correctionLevel
            , version
            // Image content
            , backgroundImage
            , Center );
    }
}

void DiagnosticPrintDocument::setQRCodeImage()
{
    Glib::RefPtr<Gdk::Pixbuf> backgroundImage;
    QRCodeConfig config = configuration.getQRCodeConfig();
    int pixel_size = config.getSizeQRCode();
    int margin = config.getMarginQRCode();
    int version = config.getVersionQRCode();
    QRecLevel correctionLevel = SAESConfigurationManager::getInstance()->GetCorrectionModeQRCode ( config.getCorrectionModeQRCode() );
    Glib::RefPtr<Gdk::Bitmap> bitmap ( QREncoding::GenerateSymbol ( ( guchar * ) data.GetData(),
                                       data.GetSize(),
                                       pixel_size,
                                       margin,
                                       correctionLevel,
                                       version ) );
    Glib::RefPtr<Gdk::Drawable> drwptr = bitmap;
    gint width ( 0 );
    gint height ( 0 );
    bitmap->get_size ( width, height );
    this->pixBuff = Gdk::Pixbuf::create ( drwptr, 0, 0, width, height );
    this->DrawImage ( pixBuff, Center );
}

void DiagnosticPrintDocument::printEntryValue(const std::string& entry, const std::string& value)
{
    if (needsToBeSplit(entry))
    {
        std::vector<std::string> lines = splitLine(entry);
        std::vector<std::string>::const_iterator it;

        for (it = lines.begin(); it != lines.end(); ++it)
        {
            this->DrawString(*it, false);

            if (it + 1 == lines.end())
                this->DrawString(value, false);
            else
                this->DrawString(" ", false);
        }
    }
    else
    {
        this->DrawString(entry, false);
        this->DrawString(value, false);
    }
}
