/**
 * @file	InstallationPrintDocument.cc
 * @brief	Implementation of entity that prints the installation report.
 * @author	Youssef.tourkmani
 * @date	31/10/2012
 */


#include "InstallationPrintDocument.h"
#include <Voting/ElectoralConfiguration.hxx>
#include <Voting/OperationStatus.hxx>
#include "Resources/election-string-resources-manager.h"
#include "Printing/TranslatorTextConfiguration.h"
#include "Environment/Configurations/resource-path-provider.h"
#include <System/Compression/compression-class.h>
#include <Log/SMTTLogManager.h>
#include <Security/Encryption.hxx>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

#define N_(String) (String) ///<macro for gettext not translate

using namespace Smartmatic::SAES::Voting::VotingDevice;
using namespace Smartmatic::SAES::Configuration;
using namespace Smartmatic::SAES::Printing;
using namespace Smartmatic::SAES::Voting;
using namespace Smartmatic::Graphics::Encoding2D;
using namespace Smartmatic::SAES::Security;
using namespace Smartmatic::SAES::Resources;
using namespace Smartmatic::System::Exception;
using namespace Smartmatic::System;
using namespace Smartmatic::Environment;
using namespace Smartmatic::Log;

ISMTTLog * InstallationPrintDocument::logger = SMTTLogManager::GetLogger ( "Smartmatic.SAES.Printing.InstallationPrintDocument", GETTEXT_PACKAGE );

InstallationPrintDocument::InstallationPrintDocument ( ElectoralConfiguration * electoralConfiguration, OperationStatus * operationStatus, bool installationReport, bool createBarcodeOnConstructor )
    : BasePrintDocument(), _electoralConfiguration ( electoralConfiguration ), operationStatus ( operationStatus )
{
    isPasswordRetry = false;

    try
    {
        electoralConfiguration->getVotingDevice();
        electoralConfiguration->getElection();
    }
    catch ( ... )
    {
        _electoralConfiguration = NULL;
    }

    this->installationReport = installationReport;
    this->configurationInstallation = SAESConfigurationManager::getInstance()->getConfiguration()->getConfigurationPrinter().getConfigurationInstallation();

    if ( SAESConfigurationManager::getInstance()->showBarCodeInstallationReport() && createBarcodeOnConstructor )
    {
        pixbufBarCode = GetQRCodeVotingMachineStatus();
    }
}

void InstallationPrintDocument::setIsPasswordRetry ( bool isPasswordRetryA )
{
    isPasswordRetry = isPasswordRetryA;
}

void InstallationPrintDocument::PrintDoc()
{
    DocumentTextOptions textInstallationTitleLevel1 = TranslatorTextConfiguration::TextConfigurationToDocumentTextOptions ( configurationInstallation.getTextInstallationTitleLevel1() );
    DocumentTextOptions textInstallationTitleLevel2 = TranslatorTextConfiguration::TextConfigurationToDocumentTextOptions ( configurationInstallation.getTextInstallationTitleLevel2() );
    DocumentTextOptions textInstallationTitleLevel3 = TranslatorTextConfiguration::TextConfigurationToDocumentTextOptions ( configurationInstallation.getTextInstallationTitleLevel3() );
    DocumentTextOptions textInstallationInfo = TranslatorTextConfiguration::TextConfigurationToDocumentTextOptions ( configurationInstallation.getTextInstallationInfo() );
    this->AddBreak();
    AddHeader ( textInstallationTitleLevel1, textInstallationTitleLevel2, textInstallationTitleLevel3, textInstallationInfo );

    if ( _electoralConfiguration != NULL )
    {
        AddRegions ( textInstallationTitleLevel1, textInstallationTitleLevel2, textInstallationTitleLevel3, textInstallationInfo );
    }

    if ( isPasswordRetry == false )
    {
        AddDeviceStatus ( textInstallationTitleLevel1, textInstallationTitleLevel2, textInstallationTitleLevel3, textInstallationInfo );
    }

    if ( installationReport )
    {
        AddKeys ( textInstallationTitleLevel1, textInstallationTitleLevel2, textInstallationTitleLevel3, textInstallationInfo );
    }

    if ( this->configurationInstallation.getShowBarCode() && isPasswordRetry == false )
    {
        if ( this->configurationInstallation.getQRcodePosition() == QRcodePosition::CONFIGURATION_BOTTOM_ALL )
        {
            this->DrawImage ( pixbufBarCode, Smartmatic::SAES::Printing::Center );
            this->Indent ( 5 );
        }
    }

    this->AddBreak();
    logger->Audit ( N_ ( "Smartmatic.SAES.Printing.InstallationPrintDocument.PrintInstallReport" ) );
}

void InstallationPrintDocument::AddHeader (
    Smartmatic::SAES::Printing::DocumentTextOptions & textInstallationTitleLevel1,
    Smartmatic::SAES::Printing::DocumentTextOptions & textInstallationTitleLevel2,
    Smartmatic::SAES::Printing::DocumentTextOptions & textInstallationTitleLevel3,
    Smartmatic::SAES::Printing::DocumentTextOptions & textInstallationInfo )
{
    Voting_booth votingBooth;

    if ( _electoralConfiguration != NULL )
    {
        Voting_device device;
        device = _electoralConfiguration->getVotingDevice()->getFirstVotingDevice();
        votingBooth = ( device.getVoting_booths().getVoting_booth() ) [0];
    }

    std::list<DocumentTextOptions> listDocumentTextOptions;
    listDocumentTextOptions.push_back ( textInstallationTitleLevel3 );
    listDocumentTextOptions.push_back ( textInstallationInfo );
    Glib::RefPtr<Gdk::Pixbuf> pixBuff  = Gdk::Pixbuf::create_from_file ( ResourcePathProvider::getInstance()->GetResourcePath ( PACKAGE_NAME, "ui/installation-report-logo.bmp" ) );
    this->DrawImage ( pixBuff, Left, InLineWithText, Border_Right, 10, 20, 10, 10 );

    if ( this->configurationInstallation.getShowBarCode() && isPasswordRetry == false )
    {
        if ( this->configurationInstallation.getQRcodePosition() != QRcodePosition::CONFIGURATION_BOTTOM_ALL )
        {
            this->DrawImage ( pixbufBarCode, Smartmatic::SAES::Printing::Center );
            this->Indent ( 5 );
        }
    }

    NewTextBlock ( textInstallationTitleLevel1 );
    this->DrawString ( " ", false );
    this->Indent ( 4 );

    if ( installationReport )
    {
        this->DrawString ( _ ( "Smartmatic.SAES.Printing.InstallationPrintDocument.InstallationReportTitle" ), false );
    }
    else
    {
        this->DrawString ( _ ( "Smartmatic.SAES.Printing.InstallationPrintDocument.DiagnosticReportTitle" ), false );
    }

    this->UnIndent ( 4 );
    this->DrawString ( " ", false );

    if ( _electoralConfiguration != NULL )
    {
        // ELECTION NAME
        Glib::ustring strElectionName = ElectionStringResourcesManager::getInstance()->getResource ( _electoralConfiguration->getElection()->getElection().getName_key() );
        this->NewTextBlock ( textInstallationTitleLevel2 );
        this->Indent ( 6 );
        this->DrawString ( strElectionName, false );
        this->UnIndent ( 6 );
        // Voting booths
        Glib::ustring strVotingBooth = ElectionStringResourcesManager::getInstance()->getResource ( votingBooth.getCustom_code() );
        this->NewTextBlock ( textInstallationTitleLevel2 );
        this->Indent ( 6 );
        Glib::ustring strTitle = _ ( "Smartmatic.SAES.Printing.InstallationPrintDocument.VotingBoothTitle" );
        this->DrawString ( ( Glib::ustring::compose ( strTitle, strVotingBooth ) ), false );
        this->UnIndent ( 6 );
        Glib::ustring votingCenterValue = ElectionStringResourcesManager::getInstance()->getResource ( votingBooth.getPolling_place_name () );
        this->NewTextBlock ( textInstallationTitleLevel2 );
        this->Indent ( 6 );
        Glib::ustring strVotingCenter = _ ( "Smartmatic.SAES.Printing.InstallationPrintDocument.VotingCenter" );
        Glib::ustring composedVotingCenter = ( Glib::ustring::compose ( strVotingCenter, votingCenterValue ) );
        this->DrawString ( composedVotingCenter, false );
        this->UnIndent ( 6 );
        Glib::ustring votingCenterAddress = ElectionStringResourcesManager::getInstance()->getResource ( votingBooth.getPolling_place_address () );
        this->NewTextBlock ( textInstallationTitleLevel2 );
        this->Indent ( 6 );
        Glib::ustring strVotingCenterAddress = _ ( "Smartmatic.SAES.Printing.InstallationPrintDocument.VotingCenterAddress" );
        Glib::ustring composedVotingCenterAddress = ( Glib::ustring::compose ( strVotingCenterAddress, votingCenterAddress ) );
        this->DrawString ( composedVotingCenterAddress, false );
        this->UnIndent ( 6 );
    }
    else
    {
        std::string line = _ ( "Smartmatic.SAES.Printing.InstallationPrintDocument.DiagnosticAppReportDescLine1" );
        this->NewTextBlock ( textInstallationTitleLevel2 );
        this->Indent ( 6 );

        if ( line.size() > 0 &&  line != N_ ( "Smartmatic.SAES.Printing.InstallationPrintDocument.DiagnosticAppReportDescLine1" ) )
        {
            this->DrawString ( line, false );
        }

        line = _ ( "Smartmatic.SAES.Printing.InstallationPrintDocument.DiagnosticAppReportDescLine2" );

        if ( line.size() > 0 &&  line != N_ ( "Smartmatic.SAES.Printing.InstallationPrintDocument.DiagnosticAppReportDescLine2" ) )
        {
            this->DrawString ( line, false );
        }

        line = _ ( "Smartmatic.SAES.Printing.InstallationPrintDocument.DiagnosticAppReportDescLine3" );

        if ( line.size() > 0 &&  line != N_ ( "Smartmatic.SAES.Printing.InstallationPrintDocument.DiagnosticAppReportDescLine3" ) )
        {
            this->DrawString ( line, false );
        }

        line = _ ( "Smartmatic.SAES.Printing.InstallationPrintDocument.DiagnosticAppReportDescLine4" );

        if ( line.size() > 0 &&  line != N_ ( "Smartmatic.SAES.Printing.InstallationPrintDocument.DiagnosticAppReportDescLine4" ) )
        {
            this->DrawString ( line, false );
        }

        this->UnIndent ( 6 );
    }

    this->AddBreak ();
    NewTextBlock ( textInstallationTitleLevel1 );
    this->DrawLine ( 550.0, Center );
    this->DrawString ( " ", true );
}


void InstallationPrintDocument::AddRegions (
    Smartmatic::SAES::Printing::DocumentTextOptions & textInstallationTitleLevel1,
    Smartmatic::SAES::Printing::DocumentTextOptions & textInstallationTitleLevel2,
    Smartmatic::SAES::Printing::DocumentTextOptions & textInstallationTitleLevel3,
    Smartmatic::SAES::Printing::DocumentTextOptions & textInstallationInfo )
{
    try
    {
        Voting_device device = _electoralConfiguration->getVotingDevice()->getFirstVotingDevice();
        Voting_booth votingBooth = ( device.getVoting_booths().getVoting_booth() ) [0];

        std::list<Glib::ustring> vRegions;
        GetRegionsStrings ( votingBooth, vRegions );

        if ( !vRegions.empty() )
        {
            std::list<DocumentTextOptions> listDocumentTextOptions;
            listDocumentTextOptions.push_back ( textInstallationTitleLevel3 );
            listDocumentTextOptions.push_back ( textInstallationInfo );

            NewTextBlock ( textInstallationTitleLevel2 );
            this->Indent ( 2 );
            Glib::ustring strRegion = _ ( "Smartmatic.SAES.Printing.InstallationPrintDocument.RegionTitle" );
            this->DrawString ( strRegion, true );
            this->DrawString ( " ", true );
            this->UnIndent ( 2 );
            this->NewTextColunmsBlock ( listDocumentTextOptions );
            this->Indent ( 4 );

            this->DrawStrings ( vRegions );
            this->UnIndent ( 4 );
            NewTextBlock ( textInstallationTitleLevel1 );
            this->DrawString ( " ", true );
            this->DrawLine ( 550.0, Center );
            this->DrawString ( " ", true );
        }
    }
    catch ( ... )
    {

    }
}

void InstallationPrintDocument::AddDeviceStatus (
    Smartmatic::SAES::Printing::DocumentTextOptions & textInstallationTitleLevel1,
    Smartmatic::SAES::Printing::DocumentTextOptions & textInstallationTitleLevel2,
    Smartmatic::SAES::Printing::DocumentTextOptions & textInstallationTitleLevel3,
    Smartmatic::SAES::Printing::DocumentTextOptions & textInstallationInfo )
{
    std::list <Status::DeviceStatus > devices;
    devices = operationStatus->getVotingMachineStatusWrapper()->getDeviceStatusList ();

    if ( !devices.empty() )
    {
        std::list <Status::DeviceStatus >::iterator it;
        std::list<DocumentTextOptions> listDocumentTextOptions;

        listDocumentTextOptions.push_back ( textInstallationTitleLevel3 );
        listDocumentTextOptions.push_back ( textInstallationInfo );

        NewTextBlock ( textInstallationTitleLevel2 );
        this->Indent ( 2 );
        Glib::ustring strDeviceStatus = _ ( "Smartmatic.SAES.Printing.InstallationPrintDocument.DevicesStatus" );

        this->DrawString ( strDeviceStatus, true );
        this->DrawString ( " ", true );
        this->UnIndent ( 2 );
        this->NewTextColunmsBlock ( listDocumentTextOptions );
        this->Indent ( 4 );

        for ( it = devices.begin();
                it != devices.end();
                ++it )
        {
            Glib::ustring device = getType ( ( *it ).getVMType() );
            Glib::ustring status = getStatus ( ( *it ).getVMStatus().getStatus().get() );
            this->DrawString ( device, false );
            this->DrawString ( status, false );
        }

        this->UnIndent ( 4 );
        NewTextBlock ( textInstallationTitleLevel1 );
        this->DrawString ( " ", true );
        this->DrawLine ( 550.0, Center );
        this->DrawString ( " ", true );
    }
}

void InstallationPrintDocument::AddKeys (
    Smartmatic::SAES::Printing::DocumentTextOptions & textInstallationTitleLevel1,
    Smartmatic::SAES::Printing::DocumentTextOptions & textInstallationTitleLevel2,
    Smartmatic::SAES::Printing::DocumentTextOptions & textInstallationTitleLevel3,
    Smartmatic::SAES::Printing::DocumentTextOptions & textInstallationInfo )
{
    std::list<DocumentTextOptions> listDocumentTextOptions;
    Voting_device device = _electoralConfiguration->getVotingDevice()->getFirstVotingDevice();
    listDocumentTextOptions.push_back ( textInstallationTitleLevel3 );
    listDocumentTextOptions.push_back ( textInstallationInfo );
    NewTextBlock ( textInstallationTitleLevel2 );
    this->Indent ( 2 );
    Glib::ustring strKeys = _ ( "Smartmatic.SAES.Printing.InstallationPrintDocument.ActivationKeys" );
    this->DrawString ( strKeys, true );
    this->DrawString ( " ", true );
    this->UnIndent ( 2 );
    Operation_mode operationMode = device.getOperation_modes().getOperation_mode() [0];
    this->NewTextColunmsBlock ( listDocumentTextOptions );
    this->Indent ( 4 );
    Glib::ustring strPasswordLine1 = Glib::ustring::compose ( _ ( "Smartmatic.SAES.Printing.InstallationPrintDocument.PasswordLine1" ), operationMode.getUnblocking_key (),
                                     operationMode.getRoles ().getRole () [0].getKey () );
    Glib::ustring strPasswordLine2 = Glib::ustring::compose ( _ ( "Smartmatic.SAES.Printing.InstallationPrintDocument.PasswordLine2" ), operationMode.getUnblocking_key (),
                                     operationMode.getRoles ().getRole () [0].getKey () );
    Glib::ustring strPasswordLine3 = Glib::ustring::compose ( _ ( "Smartmatic.SAES.Printing.InstallationPrintDocument.PasswordLine3" ), operationMode.getUnblocking_key (),
                                     operationMode.getRoles ().getRole () [0].getKey () );
    Glib::ustring strPasswordLine4 = Glib::ustring::compose ( _ ( "Smartmatic.SAES.Printing.InstallationPrintDocument.PasswordLine4" ), operationMode.getUnblocking_key (),
                                     operationMode.getRoles ().getRole () [0].getKey () );
    Glib::ustring strPasswordLine1Title = _ ( "Smartmatic.SAES.Printing.InstallationPrintDocument.PasswordLine1Title" );
    Glib::ustring strPasswordLine2Title = _ ( "Smartmatic.SAES.Printing.InstallationPrintDocument.PasswordLine2Title" );
    Glib::ustring strPasswordLine3Title = _ ( "Smartmatic.SAES.Printing.InstallationPrintDocument.PasswordLine3Title" );
    Glib::ustring strPasswordLine4Title = _ ( "Smartmatic.SAES.Printing.InstallationPrintDocument.PasswordLine4Title" );

    if ( strPasswordLine1 != " " )
    {
        this->DrawString ( strPasswordLine1Title, false );
        this->DrawString ( strPasswordLine1, false );
    }

    if ( strPasswordLine2 != " " )
    {
        this->DrawString ( strPasswordLine2Title, false );
        this->DrawString ( strPasswordLine2, false );
    }

    if ( strPasswordLine3 != " " )
    {
        this->DrawString ( strPasswordLine3Title, false );
        this->DrawString ( strPasswordLine3, false );
    }

    if ( strPasswordLine4 != " " )
    {
        this->DrawString ( strPasswordLine4Title, false );
        this->DrawString ( strPasswordLine4, false );
    }

    this->UnIndent ( 4 );
    NewTextBlock ( textInstallationTitleLevel1 );
    this->DrawString ( " ", true );
    this->DrawLine ( 550.0, Center );
    this->DrawString ( " ", true );
}


void InstallationPrintDocument::BeginPrint()
{
}

void InstallationPrintDocument::EndPrint()
{
}
void InstallationPrintDocument::GetRegionsStrings (
    const Voting_booth & votingBooth,
    std::list<Glib::ustring> & list )
{
    std::list <Region_category> listCategoryRegion;
    _electoralConfiguration->getVotingDevice()->getRegionCategoryList ( listCategoryRegion );

    std::list <Region_category>::iterator it;

    ConfigurationInstallation & configurationInstallation ( SAESConfigurationManager::getInstance()->getConfiguration()->getConfigurationPrinter().getConfigurationInstallation() );
    unsigned int numleves = configurationInstallation.getRegionCategoryLevel().getDepth();
    RegionCategoryLevel::CategoryLevelSequence & levels ( configurationInstallation.getRegionCategoryLevel().getCategoryLevel() );
    RegionCategoryLevel::CategoryLevelIterator rit;

    for ( rit = levels.begin();
            rit != levels.end();
            ++rit )
    {
        for ( it = listCategoryRegion.begin() ; it != listCategoryRegion.end(); ++it )
        {
            if ( ( *it ).getCategory_level().present() && ( *it ).getCategory_level().get() == ( *rit ) )
            {
                list.push_back ( ElectionStringResourcesManager::getInstance()->getResource ( ( *it ).getName_key() ) );
                list.push_back ( ElectionStringResourcesManager::getInstance()->getResource ( ( *it ).getRegion().getName_key() ) );

                logger->Debug ( "Found Category Level " + ( *rit ) );

                listCategoryRegion.erase ( it );
                break;
            }
        }
    }

    while ( list.size() / 2 > numleves )
    {
        list.pop_front();
        list.pop_front();
    }
}

Glib::RefPtr<Gdk::Pixbuf> InstallationPrintDocument::GetQRCodeVotingMachineStatus()
{
    Smartmatic::SAES::Configuration::QRCodeConfig config =
        SAESConfigurationManager::getInstance()->getQRCodeInstallConfig();
    int pixelSize = config.getSizeQRCode();
    int version = config.getVersionQRCode() ;
    int margin = config.getMarginQRCode();
    QRecLevel correctionLevel = SAESConfigurationManager::getInstance()->GetCorrectionModeQRCode ( config.getCorrectionModeQRCode() );
    Smartmatic::SAES::Voting::VMStatusTypes::NamespaceInfomap map;
    map[""].name = NAMESPACE_VOTING_STATUS;
    map[""].schema = "";

    std::ostringstream ss;

    serializeVotingMachineStatus ( ss,
                                   operationStatus->getVotingMachineStatusWrapper()->getVotingMachineStatus(),
                                   map, "UTF-8", Smartmatic::SAES::Voting::VMStatusTypes::Flags::dont_pretty_print );

    //compress vote
    std::string toDeflate ( ss.str() );
    SafeBuffer dataIn ( ( unsigned char * ) toDeflate.c_str(), toDeflate.length() );
    SafeBuffer dataDeflate;
    Smartmatic::System::Compression::CompressionClass::getInstance()->Deflate ( dataIn, dataDeflate );
    SafeBuffer dataOut;
    Smartmatic::SAES::Security::Encryption::getInstance()->cryptoBufferToBuffer ( dataDeflate, dataOut, Smartmatic::SAES::Security::Encryption::PRINTED_VOTE );
    gint width ( 0 );
    gint height ( 0 );
    Glib::RefPtr<Gdk::Drawable> drwptr;

    try
    {
        Glib::RefPtr<Gdk::Bitmap> bitmap ( QREncoding::GenerateSymbol ( ( guchar * ) dataOut.GetData(), dataOut.GetSize(), pixelSize, margin, correctionLevel, version ) );
        bitmap->get_size ( width, height );
        drwptr = bitmap;
    }
    catch ( QREncodingException & ex )
    {
        std::stringstream ss;
        ss << "FATAL - " << __func__ << " " << ex.what();
        logger->Debug ( ss.str() );
    }

    Glib::RefPtr<Gdk::Pixbuf> pixBuf =  Gdk::Pixbuf::create ( drwptr, 0, 0, width, height );
    return pixBuf;
}

void InstallationPrintDocument::setPixbufBarCode ( Glib::RefPtr<Gdk::Pixbuf> pixbufBarCode )
{
    this->pixbufBarCode = pixbufBarCode;
}


std::string InstallationPrintDocument::getType ( Smartmatic::SAES::Common::Configuration::DeviceType & type )
{
    std::string text;

    switch ( type )
    {
        case Smartmatic::SAES::Common::Configuration::DeviceType::CONFIGURATION_P168A:
        case Smartmatic::SAES::Common::Configuration::DeviceType::CONFIGURATION_ABLED:
            text = _ ( "Smartmatic.SAES.Printing.InstallationPrintDocument.Able" );
            break;

        case Smartmatic::SAES::Common::Configuration::DeviceType::CONFIGURATION_ALARMBOX:
            text = _ ( "Smartmatic.SAES.Printing.InstallationPrintDocument.AlarmBox" );
            break;

        case Smartmatic::SAES::Common::Configuration::DeviceType::CONFIGURATION_BARCODE:
            text = _ ( "Smartmatic.SAES.Printing.InstallationPrintDocument.Barcode" );
            break;

        case Smartmatic::SAES::Common::Configuration::DeviceType::CONFIGURATION_FLASH:
            text = _ ( "Smartmatic.SAES.Printing.InstallationPrintDocument.Flash" );
            break;

        case Smartmatic::SAES::Common::Configuration::DeviceType::CONFIGURATION_PRINTER:
            text = _ ( "Smartmatic.SAES.Printing.InstallationPrintDocument.Printer" );
            break;

        case Smartmatic::SAES::Common::Configuration::DeviceType::CONFIGURATION_SMARTCARD:
            text = _ ( "Smartmatic.SAES.Printing.InstallationPrintDocument.Smartcard" );
            break;

        case Smartmatic::SAES::Common::Configuration::DeviceType::CONFIGURATION_TOUCHSCREENCALIBRATION:
            text = _ ( "Smartmatic.SAES.Printing.InstallationPrintDocument.TouchScreenCalibration" );
            break;

        case Smartmatic::SAES::Common::Configuration::DeviceType::CONFIGURATION_ETHERNET:
            text = _ ( "Smartmatic.SAES.Printing.InstallationPrintDocument.Ethernet" );
            break;

        case Smartmatic::SAES::Common::Configuration::DeviceType::CONFIGURATION_EURN:
            text = _ ( "Smartmatic.SAES.Printing.InstallationPrintDocument.Eurn" );
            break;

        case Smartmatic::SAES::Common::Configuration::DeviceType::CONFIGURATION_MOUSE:
        case Smartmatic::SAES::Common::Configuration::DeviceType::CONFIGURATION_KEYBOARD:
        case Smartmatic::SAES::Common::Configuration::DeviceType::CONFIGURATION_UNKNOWN:
        case Smartmatic::SAES::Common::Configuration::DeviceType::CONFIGURATION_USBSTORAGEMOUNTED:
        default:
            text = _ ( "Smartmatic.SAES.Printing.InstallationPrintDocument.Unknown" );
            break;
    }

    return text;
}

std::string InstallationPrintDocument::getStatus ( Status::VMStatus & status )
{
    std::string text;

    switch ( status )
    {
        case Status::VMStatus::VOTING_MACHINE_STATUS_OK:
            text = _ ( "Smartmatic.SAES.Printing.InstallationPrintDocument.Ok" );
            break;

        case Status::VMStatus::VOTING_MACHINE_STATUS_NOTDIAGNOSED:
            text = _ ( "Smartmatic.SAES.Printing.InstallationPrintDocument.NotDiagnosed" );
            break;

        case Status::VMStatus::VOTING_MACHINE_STATUS_RUN:
        case Status::VMStatus::VOTING_MACHINE_STATUS_ERROR:
            text = _ ( "Smartmatic.SAES.Printing.InstallationPrintDocument.Error" );
            break;

        default:
            break;
    }

    return text;
}
