/**
 * @file	LogPrintDocument.h
 * @brief	Declaration of entity that prints the log report.
 * @author	Youssef.tourkmani
 * @date	31/10/2012
 */

#ifndef LOGPRINTDOCUMENT_H_
#define LOGPRINTDOCUMENT_H_

#include <gtkmm.h>
#include <Voting/ElectoralConfiguration.hxx>
#include <Voting/VotingDevice/voting-devices-schema.hxx>
#include <Printing/BasePrintDocument.h>
#include "Configuration/s-a-e-s-conf-manager.h"
#include <Log/LogReader.h>
#include <Log/LogLine.h>
#include <Log/ISMTTLog.h>

namespace Smartmatic
{

    namespace SAES
    {

        namespace Printing
        {

            /**
             * @class	LogPrintDocument
             * @brief	Log report which contains the printing operation.
             * @author	Youssef.tourkmani
             * @date	31/10/2012
             */
            class LogPrintDocument : public BasePrintDocument
            {
                public:

                    /**
                     * @fn	LogPrintDocument::LogPrintDocument ();
                     * @brief	Default constructor initializes an instance of the LogPrintDocument class.
                     * @author	Youssef.tourkmani
                     * @date	31/10/2012
                     */
                    LogPrintDocument ();

                    /**
                     * @fn	LogPrintDocument::LogPrintDocument(Smartmatic::SAES::Log::LogReader* reader,
                     * 			Smartmatic::SAES::Voting::ElectoralConfiguration * electoralConfiguration);
                     * @brief	Constructor initializes an instance of the LogPrintDocument class.
                     * @author	Youssef.tourkmani
                     * @date	31/10/2012
                     * @param [in,out]	reader The pointer reference to log information.
                     * @param [in,out]	electoralConfiguration The pointer reference to electoral context.
                     * This method does not support null reference.
                     */
                    LogPrintDocument ( Smartmatic::SAES::Log::LogReader * reader, Smartmatic::SAES::Voting::ElectoralConfiguration * electoralConfiguration );

                    /**
                     * @fn	virtual LogPrintDocument::~LogPrintDocument();
                     * @brief Finalizes the instance of the LogPrintDocument class.
                     * @author	Youssef.tourkmani
                     * @date	31/10/2012
                     */
                    virtual ~LogPrintDocument();

                protected:

                    /**
                     * @fn	virtual std::string LogPrintDocument::GetLogDocName () = 0;
                     * @brief	Gets log document name.
                     * @author	Youssef.tourkmani
                     * @date	31/10/2012
                     * @return	The log document name.
                     */
                    virtual std::string GetLogDocName () = 0;

                    /**
                     * @fn	virtual void LogPrintDocument::PrintDoc();
                     * @brief	Print document.
                     * @author	Youssef.tourkmani
                     * @date	31/10/2012
                     */
                    virtual void PrintDoc();

                    /**
                     * @fn	virtual void LogPrintDocument::BeginPrint();
                     * @brief	Begins a print.
                     * @author	Youssef.tourkmani
                     * @date	31/10/2012
                     */
                    virtual void BeginPrint();

                    /**
                     * @fn	virtual void LogPrintDocument::EndPrint();
                     * @brief	Ends a print.
                     * @author	Youssef.tourkmani
                     * @date	31/10/2012
                     */
                    virtual void EndPrint();

                private:

                    std::vector<Smartmatic::SAES::Log::LogLine> m_Entries;	///< The collection the log lines.
                    static Smartmatic::Log::ISMTTLog * logger; /**< The reference pointer of a logger instance.*/
                    Smartmatic::SAES::Voting::ElectoralConfiguration * _electoralConfiguration; ///< The electoral configuration

                    /**
                     * @fn	void LogPrintDocument::AddHeaderTextBlock ( const std::string& text ,
                     * 			const Smartmatic::SAES::Configuration::TextConfiguration& textConf);
                     * @brief	Adds a header text block to 'textConf'.
                     * @author	Youssef.tourkmani
                     * @date	31/10/2012
                     * @param	text			The text.
                     * @param	textConf	The text conf.
                     */
                    void AddHeaderTextBlock (
                        const std::string & text
                        , const Smartmatic::SAES::Configuration::TextConfiguration & textConf );
            };

        }

    }

}

#endif /* LOGPRINTDOCUMENT_H_ */
