/**
 * @file	VotePrintDocument.cc
 * @brief	vote print document class.
 * @author	Youssef.tourkmani
 * @date	31/10/2012
 */

#include "Resources/election-string-resources-manager.h"
#include "Graphics/Encoding2D/QREncoding.h"
#include "Printing/TranslatorTextConfiguration.h"
#include <Security/Encryption.hxx>
#include <Graphics/Encoding2D/QREncoding.h>
#include <Voting/Vote/VoteCode.hxx>
#include <System/IO/File.h>
#include <System/OptParser.hxx>
#include "Printing/VotePrintDocument.h"
#include <Voting/Vote/VoteWrapper.hxx>
#include "Voting/set-language-functionality.h"
#include <System/Utils/StringUtils.hxx>
#include <Log/SMTTLogManager.h>
#include <Voting/Vote/VoteMaxCalculate.hxx>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

#define N_(String) EvaluateText(String) ///<macro for evaluate text to translate

using namespace Smartmatic::SAES::Printing;
using namespace Smartmatic::SAES::Configuration;
using namespace Smartmatic::SAES::Voting;
using namespace Smartmatic::SAES::Voting::VotingDevice;
using namespace Smartmatic::SAES::Voting::Election;
using namespace Smartmatic::SAES::Resources;
using namespace Smartmatic::SAES::Security;
using namespace Smartmatic::Graphics::Encoding2D;
using namespace Smartmatic::System::Exception;
using namespace Smartmatic::System::Security::Cryptography;
using namespace Smartmatic::System;
using namespace Smartmatic::Log;
using namespace Smartmatic::System::IO;
using namespace Smartmatic::System::Utils;

ISMTTLog * VotePrintDocument::logger = SMTTLogManager::GetLogger ( "Smartmatic.SAES.Printing.VotePrintDocument", GETTEXT_PACKAGE );

void VotePrintDocument::PrintDoc()
{
    viewIdLine = false;
    PrintVote ( _vote );
}

void VotePrintDocument::BeginPrint()
{
}

void VotePrintDocument::EndPrint()
{
}

VotePrintDocument::VotePrintDocument ( Smartmatic::SAES::Voting::ElectoralConfiguration * electoralConfiguration,
                                       const Vote::Vote & vote, bool forceGtkPrinting, bool showBarCode, bool screenPreview )
    : BasePrintDocument ( forceGtkPrinting ), _vote ( vote )
{
    maxList = electoralConfiguration->getVoteMaxCalculate()->getMaxList();
    maxBallotOptions = electoralConfiguration->getVoteMaxCalculate()->getMaxBallotOptions();

    this->showBarCode = showBarCode;
    this->electoralConfiguration = electoralConfiguration;
    this->recoveryVoteMap = electoralConfiguration->getVoteRecovery()->LogicalToRecovery ( vote );

    if ( screenPreview )
    {
        logger->Debug ( "Loading screen preview configuration layout" );
        this->printConfiguration = SAESConfigurationManager::getInstance()->getPrintScreenVoteConfiguration();
        layout = SAESConfigurationManager::getInstance()->getLayoutContestConfigurationPreviewScreen();
    }
    else
    {
        logger->Debug ( "Loading print configuration and layout" );
        this->printConfiguration = SAESConfigurationManager::getInstance()->getPrintVoteConfiguration();
        layout = SAESConfigurationManager::getInstance()->getLayoutContestConfiguration();
    }

    this->_maxBallotOptionLastNameChars = printConfiguration.getMaxBallotOptionLastNameChars();
    this->_maxBallotOptionNameChars = printConfiguration.getMaxBallotOptionNameChars();
    this->_maxBallotOptionCodeChars = printConfiguration.getMaxBallotOptionCodeChars();
    this->_textOptionGeneric = TranslatorTextConfiguration::TextConfigurationToDocumentTextOptions ( printConfiguration.getTextGeneric() );
    this->_charPadding = printConfiguration.getPaddingChar();
    this->separatorName = printConfiguration.getBallotOptionNameSeparator();
    this->isMultiLanguage = printConfiguration.getMultiLanguageConfiguration().getEnable() && electoralConfiguration->getLanguages().getLanguage().size() > 1;
    this->maxCharByLanguage = printConfiguration.getMultiLanguageConfiguration().getMaxCharsByLanguage();
    QRCodePrintingModeType qr_print_mode = printConfiguration.getQRCodePrintingMode();

    if ( qr_print_mode == QRCodePrintingModeType::CONFIGURATION_IMAGE_BASED )
    {
        pixbufBarCode = GetQRCodeVote ( vote );
    }

    Voting_booths::Voting_boothSequence & votingBooths =  electoralConfiguration->getVotingDevice()->getVotingBooths().getVoting_booth();

    if ( votingBooths.size() > 0 )
    {
        this->votingBooth = votingBooths[0];
    }
    else
    {
        logger->Debug ( "FATAL - Error not found voting booths." );
    }

    this->customAttributeDefinition = electoralConfiguration->getVotingDevice()->getCustomAttributeDefinitionList ( Machine_type::VOTING_DEVICES_PM );
}

Glib::ustring VotePrintDocument::EvaluateText ( Glib::ustring key )
{
    Glib::ustring translated = _ ( key.c_str() );

    if ( key == translated )
    {
        return "";
    }

    return translated;
}

void VotePrintDocument::AddVoteText ( Glib::ustring toCompose,
                                      TextConfiguration textConfiguration,
                                      std::vector<Glib::ustring> & vParam )
{
    if ( !toCompose.empty() && toCompose != " " )
    {
        int paramSize = vParam.size();

        if ( paramSize < 9 )
            for ( int i = paramSize; i < 9; i++ )
            {
                vParam.push_back ( "" );
            }

        DocumentTextOptions textOption = TranslatorTextConfiguration::TextConfigurationToDocumentTextOptions ( textConfiguration );

        if ( textConfiguration.getVisible() )
        {
            Glib::ustring composed = Glib::ustring::compose ( toCompose, vParam[0],
                                     vParam[1], vParam[2],
                                     vParam[3], vParam[4],
                                     vParam[5], vParam[6],
                                     vParam[7], vParam[8] );

            if ( !composed.empty() && composed != " " )
            {
                this->NewTextBlock ( textOption );
                this->DrawString ( composed, true );
            }
        }
    }
}

void VotePrintDocument::AddVoteHeader ( const Vote::Vote & vote )
{
    Smartmatic::SAES::Voting::Election::Election & election ( electoralConfiguration->getElection()->getElection() );
    strElectionName = getResource ( election.getName_key() );
    strElectoralOrg =  getResource ( election.getElectoral_organization() );
    strPollingPlaceName = getResource ( votingBooth.getPolling_place_name() );
    strPollingPlaceAddress = getResource ( votingBooth.getPolling_place_address() );
    // Voting booths
    Glib::ustring strVotingBooth = vote.getVotingDeviceCode();
    //lvg code
    Glib::ustring strLVGCode = vote.getLogicalVoterGroup();
    // Vote Code
    std::string cbcMac = vote.getCode();
    Vote::VoteCode::encode ( cbcMac );
    SafeBuffer checksumBuffer;
    Vote::VoteCode::generateChecksum( vote, checksumBuffer );
    // Checksum
    Glib::ustring strVoteChecksum = checksumBuffer.GetBase64Data();
    std::string machineModel;
    this->NewTextBlock ( _textOptionGeneric );

    switch ( vote.getMode() )
    {
        case Vote::VoteMode::VOTE_D:
            this->DrawString ( "", true );
            this->DrawString ( "", true );
            this->DrawString ( strDemoTitle, true );
            break;

        case Vote::VoteMode::VOTE_S:
            this->DrawString ( "", true );
            this->DrawString ( "", true );
            this->DrawString ( strSimulatedTitle, true );
            break;

        case Vote::VoteMode::VOTE_O:

            if ( !strOfficialTitle.empty() && strOfficialTitle != " " )
            {
                this->DrawString ( "", true );
                this->DrawString ( "", true );
                this->DrawString ( strOfficialTitle, true );
            }

            break;

        default:
            break;
    }

    this->NewTextBlock ( _textOptionGeneric );
    Glib::ustring line = N_ ( "Smartmatic.SAES.Printing.VotePrintDocument.HorizontalLineBeginHeader" );

    if ( !line.empty() && line != " " )
    {
        this->DrawString ( line, true );
    }

    std::vector<Glib::ustring> vHeaderParams;
    vHeaderParams.push_back ( strElectionName );
    vHeaderParams.push_back ( strElectoralOrg );
    vHeaderParams.push_back ( strVotingBooth );
    vHeaderParams.push_back ( strPollingPlaceName );
    vHeaderParams.push_back ( cbcMac );
    vHeaderParams.push_back ( strVoteChecksum );
    vHeaderParams.push_back ( strPollingPlaceAddress );
    vHeaderParams.push_back ( strLVGCode );

    OptParser::getInstance()->getArg(OptParser::MACHINE_MODEL, machineModel);

    if ( vote.getMode() == Vote::VoteMode::VOTE_D )
    {
        AddVoteText ( N_ ( "Smartmatic.SAES.Printing.VotePrintDocument.HeaderDemo1" ), printConfiguration.getTextHeader1(), vHeaderParams );
        AddVoteText ( N_ ( "Smartmatic.SAES.Printing.VotePrintDocument.HeaderDemo2" ), printConfiguration.getTextHeader2(), vHeaderParams );
        AddVoteText ( N_ ( "Smartmatic.SAES.Printing.VotePrintDocument.HeaderDemo3" ), printConfiguration.getTextHeader3(), vHeaderParams );
        AddVoteText ( N_ ( "Smartmatic.SAES.Printing.VotePrintDocument.HeaderDemo4" ), printConfiguration.getTextHeader4(), vHeaderParams );
        AddVoteText ( N_ ( "Smartmatic.SAES.Printing.VotePrintDocument.HeaderDemo5" ), printConfiguration.getTextHeader5(), vHeaderParams );
        AddVoteText ( N_ ( "Smartmatic.SAES.Printing.VotePrintDocument.HeaderDemo6" ), printConfiguration.getTextHeader6(), vHeaderParams );
        AddVoteText ( N_ ( "Smartmatic.SAES.Printing.VotePrintDocument.HeaderDemo7" ), printConfiguration.getTextHeader7(), vHeaderParams );
    }
    else if ( machineModel != "VM-A4517" )
    {
        AddVoteText ( N_ ( "Smartmatic.SAES.Printing.VotePrintDocument.Header1" ), printConfiguration.getTextHeader1(), vHeaderParams );
        AddVoteText ( N_ ( "Smartmatic.SAES.Printing.VotePrintDocument.Header2" ), printConfiguration.getTextHeader2(), vHeaderParams );
        AddVoteText ( N_ ( "Smartmatic.SAES.Printing.VotePrintDocument.Header3" ), printConfiguration.getTextHeader3(), vHeaderParams );
        AddVoteText ( N_ ( "Smartmatic.SAES.Printing.VotePrintDocument.Header4" ), printConfiguration.getTextHeader4(), vHeaderParams );
        AddVoteText ( N_ ( "Smartmatic.SAES.Printing.VotePrintDocument.Header5" ), printConfiguration.getTextHeader5(), vHeaderParams );
        AddVoteText ( N_ ( "Smartmatic.SAES.Printing.VotePrintDocument.Header6" ), printConfiguration.getTextHeader6(), vHeaderParams );
        AddVoteText ( N_ ( "Smartmatic.SAES.Printing.VotePrintDocument.Header7" ), printConfiguration.getTextHeader7(), vHeaderParams );
    }

    addCustomAttributesHeader ( vote );
}

void VotePrintDocument::setBallotTitle ( const Ballot & ballot )
{
    if ( printConfiguration.getShowBallotLines() )
    {
        this->NewTextBlock ( _textOptionGeneric );
        Glib::ustring line = N_ ( "Smartmatic.SAES.Printing.VotePrintDocument.HorizontalLineBallotNameBegin" );

        if ( !line.empty() && line != " " )
        {
            this->DrawString ( line, true );
        }
    }

    if ( printConfiguration.getTextBallotName().getVisible() )
    {
        DocumentTextOptions textOptionBallotTitle =
            TranslatorTextConfiguration::TextConfigurationToDocumentTextOptions (
                printConfiguration.getTextBallotName() );
        Glib::ustring strBallotTitle = getResource ( ballot.getTitle_key() );
        this->NewTextBlock ( textOptionBallotTitle );

        if ( !strBallotTitle.empty() && strBallotTitle != " " )
        {
            this->DrawString ( strBallotTitle, true );
        }
    }

    if ( printConfiguration.getShowBallotLines() )
    {
        this->NewTextBlock ( _textOptionGeneric );
        Glib::ustring line = N_ ( "Smartmatic.SAES.Printing.VotePrintDocument.HorizontalLineBallotNameEnd" );

        if ( !line.empty() && line != " " )
        {
            this->DrawString ( line, true );
        }
    }
}

void VotePrintDocument::PrintVote ( const Vote::Vote & vote )
{
    translateText();
    setTopVote ( vote );
    setCenterVote ( vote );
    setBlankSpaceVote();
    setBottomVote ( vote );
}

void VotePrintDocument::setTopVote ( const Vote::Vote & vote )
{
    QRCodePrintingModeType qr_print_mode = printConfiguration.getQRCodePrintingMode();

    if ( qr_print_mode == QRCodePrintingModeType::CONFIGURATION_IMAGE_BASED )
    {
        if ( ( printConfiguration.getQRcodePosition() == QRcodePosition::CONFIGURATION_TOP_ALL ) && showBarCode )
        {
            this->DrawImage ( pixbufBarCode, Smartmatic::SAES::Printing::Center );
        }

        AddVoteHeader ( vote );

        if ( ( printConfiguration.getQRcodePosition() == QRcodePosition::CONFIGURATION_BOTTOM_HEADER ) && showBarCode )
        {
            this->DrawImage ( pixbufBarCode, Smartmatic::SAES::Printing::Center );
        }
    }
    else if ( qr_print_mode == QRCodePrintingModeType::CONFIGURATION_COMMAND_BASED )
    {
        if ( ( printConfiguration.getQRcodePosition() == QRcodePosition::CONFIGURATION_TOP_ALL ) && showBarCode )
        {
            CommandPrintQRCode ( vote );
        }

        AddVoteHeader ( vote );

        if ( ( printConfiguration.getQRcodePosition() == QRcodePosition::CONFIGURATION_BOTTOM_HEADER ) && showBarCode )
        {
            CommandPrintQRCode ( vote );
        }
    }
}

void VotePrintDocument::setCenterVote ( const Vote::Vote & vote )
{
    std::list < Smartmatic::SAES::Voting::Election::Ballot * > & sequenceBallot ( electoralConfiguration->getBallots() );
    std::list < Smartmatic::SAES::Voting::Election::Ballot * >::iterator ballotIt;
    Vote::Vote::SSequence sequenceS = vote.getSelections();
    Vote::Vote::SIterator its;
    int count = sequenceS.size();
    this->countContest = sequenceS.size();
    this->countLinesTotalList = 0;
    this->countLinesTotalBallotOption = 0;
    this->countContestName1 = 0;
    this->countContestName2 = 0;
    this->countContestName3 = 0;

    for ( ballotIt = sequenceBallot.begin();
            ballotIt != sequenceBallot.end();
            ++ballotIt )
    {
        setBallotTitle ( * ( *ballotIt ) );
        Ballot_cards::Ballot_cardSequence  & ballotCardSequence ( electoralConfiguration->getElection()->getBallotCards ( * ( *ballotIt ) ).getBallot_card () );
        Ballot_cards::Ballot_cardIterator ballotCardIt;

        for ( ballotCardIt = ballotCardSequence.begin();
                ballotCardIt != ballotCardSequence.end();
                ++ballotCardIt )
        {
            Contests::ContestSequence & contestSequence ( electoralConfiguration->getElection()->getContests ( *ballotCardIt ).getContest () );
            Contests::ContestIterator contestIt;

            for ( its = sequenceS.begin();
                    its != sequenceS.end();
                    ++its )
            {
                for ( contestIt = contestSequence.begin();
                        contestIt != contestSequence.end();
                        ++contestIt )
                {
                    if ( ( *its ).getContest() != ( *contestIt ).getCode() )
                    {
                        continue;
                    }

                    count--;
                    setContest ( vote, *contestIt, *its );
                    setContestBlankSpace ( ( *contestIt ).getCode(), ( unsigned ) count > 0 );

                    if ( count <= 0 )
                    {
                        return;
                    }
                }
            }
        }
    }
}

void VotePrintDocument::setSuperNull()
{
}

void VotePrintDocument::setContest ( const Vote::Vote & vote, Contest & contest, const Vote::Selection & selection )
{
    setBallotOptionInfo ( vote, contest, selection );
    setContestTitle ( vote, contest );
    bool isQuestion = contest.getIs_question();
    this->countLinesList = 0;
    this->countLinesBallotOption = 0;
    this->viewPrincipalTitle = false;
    this->viewStandInTitle = false;

    if ( selection.getBallotOptions().size() == 0 )
    {
        DocumentTextOptions format = TranslatorTextConfiguration::TextConfigurationToDocumentTextOptions ( printConfiguration.getTextBlankVote() );
        this->NewTextBlock ( format );

        if ( isQuestion )
        {
            this->DrawString ( strNullReferendum, true );
        }
        else
        {
            this->DrawString ( strNullVote, true );
        }

        this->viewPrincipalTitle = true;
        countLinesTotalList += electoralConfiguration->getVoteMaxCalculate()->getMaxLineListOptionsByContestCode ( contest.getCode() );
        countLinesTotalBallotOption += electoralConfiguration->getVoteMaxCalculate()->getMaxLineBallotOptionsByContestCode ( contest.getCode() );
    }
    else
    {
        std::list < Ballot_option * > & listBallotOptionPartyList = electoralConfiguration->getEligibleOptionsListByContestCode ( contest.getCode() );

        if ( isQuestion )
        {
            std::list < Ballot_option * > listBallotOptionStandIn;
            setTitleReferendum();
            setContestBallotOption ( vote, contest, selection, listBallotOptionPartyList, listBallotOptionStandIn, isQuestion );
        }
        else
        {
            std::list < Ballot_option * > & listBallotOptionPrincipal = electoralConfiguration->getEligibleOptionsPrincipalByContestCode ( contest.getCode() );
            std::list < Ballot_option * > & listBallotOptionStandIn = electoralConfiguration->getEligibleOptionsStandInByContestCode ( contest.getCode() );
            setContestList ( vote, contest, listBallotOptionPartyList, selection );
            setContestBallotOption ( vote, contest, selection, listBallotOptionPrincipal, listBallotOptionStandIn, isQuestion );
        }

        countLinesTotalList += electoralConfiguration->getVoteMaxCalculate()->getMaxLineListOptionsByContestCode ( contest.getCode() );
        countLinesTotalBallotOption += electoralConfiguration->getVoteMaxCalculate()->getMaxLineBallotOptionsByContestCode ( contest.getCode() );
    }
}

void VotePrintDocument::setBallotOptionInfo ( const Vote::Vote & vote, Contest & contest, const Vote::Selection & selection )
{
    strEligibleOptionBallotName = "";
    strBallotOptionPartyName = "";
    strBallotOptionPartyAbbreviation = "";
    Vote::Selection::BSequence seq = selection.getBallotOptions();
    Vote::Selection::BIterator itb;

    for ( itb = seq.begin();
            itb != seq.end();
            ++itb )
    {
        Voting::Election::Eligible_option * eligibleOption = electoralConfiguration->getEligibleOptionFromBallotOptionCode ( contest.getCode(), ( *itb ) );
        Voting::Election::Ballot_options & ballots ( eligibleOption->getBallot_options() );

        if ( ballots.getBallot_option().size() > 0 )
        {
            Voting::Election::Ballot_option & ballotOption ( ballots.getBallot_option() [0] );
            strEligibleOptionBallotName = getResource ( eligibleOption->getBallot_name_key() );

            setPartyName(contest.getCode(), ballotOption);

            return;
        }
    }
}

void VotePrintDocument::setPartyName ( const std::string& contestCode, const Voting::Election::Ballot_option & ballotOption )
{
    const Glib::ustring format = "%1 %2";
    Glib::ustring partyName;
    Glib::ustring partyAbbreviation;
    std::string tallyPosition;

    const std::list < Smartmatic::SAES::Voting::Election::Ballot_option * > & partiesByContest ( electoralConfiguration->getBallotOptionPartyByContest ( contestCode ) );

    for (std::list<Smartmatic::SAES::Voting::Election::Ballot_option * >::const_iterator it = partiesByContest.begin();  it != partiesByContest.end(); ++it)
    {
        if ( ( *it )->getParty().getCode() == ballotOption.getParty().getCode() )
        {
            partyName = ElectionStringResourcesManager::getInstance()->getResource ( ( ballotOption.getParty().getName() != NULL ? ballotOption.getParty().getName().get() : "" ) );
            partyAbbreviation = ElectionStringResourcesManager::getInstance()->getResource ( ( ballotOption.getParty().getAbbreviation() != NULL ? ballotOption.getParty().getAbbreviation().get() : "" ) );
            tallyPosition = StringUtils::longToString( ( *it )->getTally_position());

            strBallotOptionPartyName = Glib::ustring::compose ( format, tallyPosition, partyName );
            strBallotOptionPartyAbbreviation = Glib::ustring::compose ( format, tallyPosition, partyAbbreviation );

            break;
        }
    }
}

void VotePrintDocument::setContestTitle ( const Vote::Vote & vote, const Contest & contest )
{
    Contest_class * contestClass = electoralConfiguration->getContestClassByCode ( contest.getContest_class_code() );
    strContestClassName = getResource ( contestClass->getName_key() );
    strContestClassDescription = getResource ( contestClass->getDescription_key() );
    strContestTitle = getResource ( contest.getTitle_key() );
    strContestAbstract = getResource ( contest.getAbstract_key() );
    strContestHelp = getResource ( contest.getHelp_key() );
    strContestBinaryRecovery =  "";

    if ( vote.getMode() != Vote::VoteMode::VOTE_D )
    {
        std::map<std::string, std::string>::iterator contestBinaryIterator;
        contestBinaryIterator = recoveryVoteMap.find ( contest.getCode() );

        if ( contestBinaryIterator != recoveryVoteMap.end() )
        {
            strContestBinaryRecovery = contestBinaryIterator->second;
        }
    }

    int countBallot = vote.getSelections().size();
    // CustomAttributeValue
    customAttributeValue = GetCustomAttributeValue ( contest, vote, contestClass );

    if ( !customAttributeValue.empty() && customAttributeValue != " " )
    {
        customAttributeValue = getResource ( customAttributeValue );
    }

    // Contest Name 1
    if ( printConfiguration.getTextContestName1().getVisible() )
    {
        Glib::ustring messageToCompose = "";

        if ( contest.getIs_question() )
        {
            messageToCompose = _ ( "Smartmatic.SAES.Printing.VotePrintDocument.ContestName1Referendum" );
        }
        else if ( countBallot == 0 )
        {
            messageToCompose = _ ( "Smartmatic.SAES.Printing.VotePrintDocument.ContestName1NullVote" );
        }
        else
        {
            messageToCompose = _ ( "Smartmatic.SAES.Printing.VotePrintDocument.ContestName1NormalVote" );
        }

        Glib::ustring strMessage = getComposeMessage ( messageToCompose );

        if ( !strMessage.empty() && strMessage != " " )
        {
            DocumentTextOptions textOptionContestTitle = TranslatorTextConfiguration::TextConfigurationToDocumentTextOptions ( printConfiguration.getTextContestName1() );
            this->NewTextBlock ( textOptionContestTitle );
            this->DrawString ( strMessage, true );
            countContestName1++;
        }
    }

    // Contest Name 2
    if ( printConfiguration.getTextContestName2().getVisible() )
    {
        Glib::ustring messageToCompose = "";

        if ( contest.getIs_question() )
        {
            messageToCompose = _ ( "Smartmatic.SAES.Printing.VotePrintDocument.ContestName2Referendum" );
        }
        else if ( countBallot == 0 )
        {
            messageToCompose = _ ( "Smartmatic.SAES.Printing.VotePrintDocument.ContestName2NullVote" );
        }
        else
        {
            messageToCompose = _ ( "Smartmatic.SAES.Printing.VotePrintDocument.ContestName2NormalVote" );
        }

        Glib::ustring strMessage = getComposeMessage ( messageToCompose );

        if ( !strMessage.empty() && strMessage != " " )
        {
            DocumentTextOptions textOptionContestTitle = TranslatorTextConfiguration::TextConfigurationToDocumentTextOptions ( printConfiguration.getTextContestName2() );
            this->NewTextBlock ( textOptionContestTitle );
            this->DrawString ( strMessage, true );
            countContestName2++;
        }
    }

    // Contest Name 3
    if ( printConfiguration.getTextContestName3().getVisible() )
    {
        Glib::ustring messageToCompose = "";

        if ( contest.getIs_question() )
        {
            messageToCompose = _ ( "Smartmatic.SAES.Printing.VotePrintDocument.ContestName3Referendum" );
        }
        else if ( countBallot == 0 )
        {
            messageToCompose = _ ( "Smartmatic.SAES.Printing.VotePrintDocument.ContestName3NullVote" );
        }
        else
        {
            messageToCompose = _ ( "Smartmatic.SAES.Printing.VotePrintDocument.ContestName3NormalVote" );
        }

        Glib::ustring strMessage = getComposeMessage ( messageToCompose );

        if ( !strMessage.empty() && strMessage != " " )
        {
            DocumentTextOptions textOptionContestTitle = TranslatorTextConfiguration::TextConfigurationToDocumentTextOptions ( printConfiguration.getTextContestName3() );
            this->NewTextBlock ( textOptionContestTitle );
            this->DrawString ( strMessage, true );
            countContestName3++;
        }
    }
}

std::string VotePrintDocument::getDistrictDescription( const Contest & contest )
{
    const std::string descriptionKey = contest.getDistrict().getDescription_key();

    return ElectionStringResourcesManager::getInstance()->getResource ( descriptionKey );
}

std::string VotePrintDocument::getHelp( const Contest & contest )
{
    const std::string helpKey = contest.getHelp_key();

    return ElectionStringResourcesManager::getInstance()->getResource(helpKey);
}

Glib::ustring VotePrintDocument::getComposeMessage ( Glib::ustring messageToCompose )
{
    Glib::ustring strMessage = Glib::ustring::compose (
                                   messageToCompose,
                                   strContestClassName,
                                   strContestClassDescription,
                                   strContestTitle,
                                   strContestAbstract,
                                   strContestHelp,
                                   customAttributeValue,
                                   strEligibleOptionBallotName,
                                   strBallotOptionPartyName,
                                   strBallotOptionPartyAbbreviation );
    return Glib::ustring::compose ( strMessage, strContestBinaryRecovery );
}

void VotePrintDocument::setContestBlankSpace ( Glib::ustring contestCode, bool addEndSpace )
{
    if ( layout.getCompleteWithSpaces() )
    {
        int maxListContest = electoralConfiguration->getVoteMaxCalculate()->getMaxLineListOptionsByContestCode ( contestCode );
        int maxBallotOptionContest = electoralConfiguration->getVoteMaxCalculate()->getMaxLineBallotOptionsByContestCode ( contestCode );

        if ( !viewPrincipalTitle )
        {
            setContestTitlePrincipalSpace ();
        }

        if ( !viewStandInTitle )
        {
            setContestTitleStandInSpace();
        }

        if ( addEndSpace )
        {
            setContestEndSpace();
        }

        setContestListSpace ( maxListContest - countLinesList );
        setContestBallotSpace ( maxBallotOptionContest - countLinesBallotOption );
    }
}

void VotePrintDocument::setContestTitlePrincipalSpace ( int count )
{
    if ( printConfiguration.getTextPrincipalsMessage().getVisible() )
    {
        DocumentTextOptions format = TranslatorTextConfiguration::TextConfigurationToDocumentTextOptions ( printConfiguration.getTextPrincipalsMessage() );
        this->NewTextBlock ( format );

        for ( int i = 0; i < count; i++ )
        {
            this->DrawString ( ( viewIdLine ? "TP" : _charPadding ), true );
        }
    }
}

void VotePrintDocument::setContestTitleStandInSpace ( int count )
{
    if ( printConfiguration.getTextStandInMessage().getVisible() )
    {
        DocumentTextOptions format = TranslatorTextConfiguration::TextConfigurationToDocumentTextOptions ( printConfiguration.getTextStandInMessage() );
        this->NewTextBlock ( format );

        for ( int i = 0; i < count; i++ )
        {
            this->DrawString ( ( viewIdLine ? "TS" : _charPadding ), true );
        }
    }
}

void VotePrintDocument::setContestEndSpace ( int count )
{
    for ( int i = 0; i < count; i++ )
    {
        static int spaceAdd = printConfiguration.getAddBlankSpaceFullContest();
        setContestBallotSpace ( spaceAdd );
    }
}

void VotePrintDocument::setExtraSpaceForMissingContest ( int missingContestLines )
{
    if (missingContestLines > 0)
    {
        setContestEndSpace(missingContestLines/25);
    }
}

void VotePrintDocument::setContestListSpace ( int countList )
{
    if ( printConfiguration.getTextParentOptionSelected().getVisible() )
    {
        DocumentTextOptions textOptionContestTitle = TranslatorTextConfiguration::TextConfigurationToDocumentTextOptions ( printConfiguration.getTextParentOptionSelected() );
        this->NewTextBlock ( textOptionContestTitle );

        for ( int iterador = 0; iterador < countList; iterador++ )
        {
            this->DrawString ( ( viewIdLine ? "L" : _charPadding ), true );
        }
    }
}

void VotePrintDocument::setContestBallotSpace ( int countBallot )
{
    DocumentTextOptions textOptionBallotOption = TranslatorTextConfiguration::TextConfigurationToDocumentTextOptions ( layout.getTextBallotOption() );

    this->NewTextBlock ( textOptionBallotOption );

    for ( int iterador = 0; iterador < countBallot; iterador++ )
    {
        this->DrawString ( ( viewIdLine ? "B" : _charPadding ), true );
    }
}

void VotePrintDocument::setBlankSpaceVote()
{
    if ( printConfiguration.getCompleteVoteWithSpaces() )
    {
        int maxContest = electoralConfiguration->getMaxContestByVote();
        int maxCustomAttribute = electoralConfiguration->getMaxCustomAttributeByVote();

        // Contest Name 1
        if ( printConfiguration.getTextContestName1().getVisible() )
        {
            int count = ( printConfiguration.getIsTextContestName1CustomAttribute() ? maxCustomAttribute : maxContest );
            DocumentTextOptions format = TranslatorTextConfiguration::TextConfigurationToDocumentTextOptions ( printConfiguration.getTextContestName1() );
            this->NewTextBlock ( format );

            for ( int i = 0 ; i < count - countContestName1; i++ )
            {
                this->DrawString ( ( viewIdLine ? "TN1" : _charPadding ), true );
            }
        }

        // Contest Name 2
        if ( printConfiguration.getTextContestName2().getVisible() )
        {
            int count = ( printConfiguration.getIsTextContestName2CustomAttribute() ? maxCustomAttribute : maxContest );
            DocumentTextOptions format = TranslatorTextConfiguration::TextConfigurationToDocumentTextOptions ( printConfiguration.getTextContestName2() );
            this->NewTextBlock ( format );

            for ( int i = 0 ; i < count - countContestName2; i++ )
            {
                this->DrawString ( ( viewIdLine ? "TN2" : _charPadding ), true );
            }
        }

        // Contest Name 3
        if ( printConfiguration.getTextContestName3().getVisible() )
        {
            int count = ( printConfiguration.getIsTextContestName3CustomAttribute() ? maxCustomAttribute : maxContest );
            DocumentTextOptions format = TranslatorTextConfiguration::TextConfigurationToDocumentTextOptions ( printConfiguration.getTextContestName3() );
            this->NewTextBlock ( format );

            for ( int i = 0 ; i < count - countContestName3; i++ )
            {
                this->DrawString ( ( viewIdLine ? "TN3" : _charPadding ), true );
            }
        }

        setContestTitlePrincipalSpace ( maxContest - countContest );
        setContestTitleStandInSpace ( maxContest - countContest );
        setContestEndSpace ( maxContest - countContest );
        setContestListSpace ( ( maxList * maxContest ) - countLinesTotalList );
        setContestBallotSpace ( maxBallotOptions - countLinesTotalBallotOption );
        setExtraSpaceForMissingContest ( maxBallotOptions - countLinesTotalBallotOption );
    }
}

void VotePrintDocument::setTitleReferendum()
{
    if ( printConfiguration.getTextParentOptionSelected().getVisible() )
    {
        if ( !strTitleReferendum.empty() && strTitleReferendum != " " )
        {
            DocumentTextOptions format = TranslatorTextConfiguration::TextConfigurationToDocumentTextOptions ( printConfiguration.getTextParentOptionSelected() );
            this->NewTextBlock ( format );
            this->DrawString ( strTitleReferendum, true );
            countLinesList++;
        }
    }
}

void VotePrintDocument::setContestList ( const Vote::Vote & vote, Contest & contest, std::list < Ballot_option * > & list, const Vote::Selection & selection )
{
    if ( printConfiguration.getTextParentOptionSelected().getVisible() )
    {
        Glib::ustring messageToCompose;
        messageToCompose = _ ( "Smartmatic.SAES.Printing.VotePrintDocument.ParentOptionWasSelected" );
        std::list < Ballot_option * >::iterator it;
        Vote::Selection::BSequence seq = selection.getBallotOptions();
        Vote::Selection::BIterator itb;

        for ( itb = seq.begin();
                itb != seq.end();
                ++itb )
        {
            for ( it = list.begin();
                    it != list.end();
                    ++it )
            {
                if ( ( *it )->getCode() != ( *itb ) )
                {
                    continue;
                }

                Voting::Election::Eligible_option * eligibleOption = electoralConfiguration->getEligibleOptionFromBallotOptionCode ( contest.getCode(), ( *itb ) );
                Voting::Election::Ballot_options & ballots ( eligibleOption->getBallot_options() );

                if ( ballots.getBallot_option().size() > 0 )
                {
                    Voting::Election::Ballot_option & ballotOption ( ballots.getBallot_option() [0] );
                    strEligibleOptionBallotName = ElectionStringResourcesManager::getInstance()->getResource ( eligibleOption->getBallot_name_key() );
                    strBallotOptionPartyName = ElectionStringResourcesManager::getInstance()->getResource ( ( ballotOption.getParty().getName() != NULL ? ballotOption.getParty().getName().get() : "" ) );
                    strBallotOptionPartyAbbreviation = ElectionStringResourcesManager::getInstance()->getResource ( ( ballotOption.getParty().getAbbreviation() != NULL ? ballotOption.getParty().getAbbreviation().get() : "" ) );
                    Glib::ustring strMessage = getComposeMessage ( messageToCompose );

                    textToPrintMap[ contest.getCode() ][ ballotOption.getCode() ] = ballotOption.getParty().getName().get();

                    if ( !strMessage.empty() && strMessage != " " )
                    {
                        DocumentTextOptions format = TranslatorTextConfiguration::TextConfigurationToDocumentTextOptions ( printConfiguration.getTextParentOptionSelected() );
                        this->NewTextBlock ( format );
                        strMessage = strParentOptionMessage + strMessage;
                        this->DrawString ( strMessage, true );
                        countLinesList++;
                    }
                }
            }
        }
    }
}

void VotePrintDocument::sortBallotOptionsListByTallyPosition ( std::vector < Ballot_option * > & ballotOptionList )
{
    std::multimap<int, Ballot_option * > map;
    std::multimap<int, Ballot_option * >::iterator itmap;
    std::vector < Ballot_option * >::iterator it;

    for ( it = ballotOptionList.begin();
            it != ballotOptionList.end();
            ++it )
    {
        map.insert ( std::pair<int, Ballot_option * > ( ( *it )->getTally_position(), *it ) );
    }

    ballotOptionList.clear();

    for ( itmap = map.begin();
            itmap != map.end();
            ++itmap )
    {
        ballotOptionList.push_back ( ( *itmap ).second );
    }

    map.clear();
}

void VotePrintDocument::setContestBallotOption ( const Vote::Vote & vote,
        Contest & contest,
        const Vote::Selection & selection,
        std::list < Ballot_option * > & listBallotOptionPrincipal,
        std::list < Ballot_option * > & listBallotOptionStandIn, bool isQuestion )
{
    try
    {
        Vote::Selection::BSequence sequenceB = selection.getBallotOptions();
        std::vector <Ballot_option * > listBallotOptionSelected;
        std::vector < Ballot_option * > listBallotOptionStandInSelected;

        if ( sequenceB.size() != 0 )
        {
            std::list< Ballot_option * >::iterator pos;

            for ( unsigned int var = 0; var < sequenceB.size(); ++var )
            {
                for ( pos = listBallotOptionPrincipal.begin();
                        pos != listBallotOptionPrincipal.end();
                        ++pos )
                {
                    if ( ( *pos )->getCode() == sequenceB[var] )
                    {
                        listBallotOptionSelected.push_back ( *pos );
                        break;
                    }
                }
            }

            for ( unsigned int var = 0; var < sequenceB.size(); ++var )
            {
                for ( pos = listBallotOptionStandIn.begin();
                        pos != listBallotOptionStandIn.end();
                        ++pos )
                {
                    if ( ( *pos )->getCode() == sequenceB[var] )
                    {
                        listBallotOptionStandInSelected.push_back ( *pos );
                        break;
                    }
                }
            }

            sortBallotOptionsListByTallyPosition( listBallotOptionSelected );
            sortBallotOptionsListByTallyPosition( listBallotOptionStandInSelected );
            //Getting Sorted for two columns vertical distribution
            if ( layout.getLayoutContestType() == Smartmatic::SAES::Configuration::LayoutContestType::CONFIGURATION_TRUNCATED )
            {
                std::vector <Ballot_option * > listBallotOptionSelectedTemp;
                std::vector < Ballot_option * > listBallotOptionStandInSelectedTemp;
                unsigned int principalsFirstHalf = listBallotOptionSelected.size();
                unsigned int standInFirstHalf = listBallotOptionStandInSelected.size();
                principalsFirstHalf = ( principalsFirstHalf / 2 ) + ( principalsFirstHalf % 2 );
                standInFirstHalf = ( standInFirstHalf / 2 ) + ( standInFirstHalf % 2 );

                for ( unsigned int i = 0; i < principalsFirstHalf; i++ )
                {
                    listBallotOptionSelectedTemp.push_back ( listBallotOptionSelected[i] );

                    if ( ( principalsFirstHalf + i ) < listBallotOptionSelected.size() )
                    {
                        listBallotOptionSelectedTemp.push_back ( listBallotOptionSelected[principalsFirstHalf + i] );
                    }
                }

                for ( unsigned int i = 0; i < standInFirstHalf; i++ )
                {
                    listBallotOptionStandInSelectedTemp.push_back ( listBallotOptionStandInSelected[i] );

                    if ( ( standInFirstHalf + i ) < listBallotOptionStandInSelected.size() )
                    {
                        listBallotOptionStandInSelectedTemp.push_back ( listBallotOptionStandInSelected[standInFirstHalf + i] );
                    }
                }

                listBallotOptionSelected = listBallotOptionSelectedTemp;
                listBallotOptionStandInSelected = listBallotOptionStandInSelectedTemp;
            }
        }

        // BALLOT OPTION
        //
        DocumentTextOptions textOptionBallotOption = TranslatorTextConfiguration::TextConfigurationToDocumentTextOptions ( layout.getTextBallotOption() );

        if ( printConfiguration.getLayoutBallotOptiontType() == ConfigurationVote::LayoutBallotOptiontTypeType::CONFIGURATION_SEPARATEPRINCIPALANDSTANDIN )
        {
            //The List should be printed separated

            //Principals
            if ( printConfiguration.getTextPrincipalsMessage().getVisible() )
            {
                Glib::ustring message;

                if ( isQuestion )
                {
                    message = strPrincipalTitleReferendum;
                }
                else if ( listBallotOptionSelected.empty() )
                {
                    message = strPrincipalNoSelectectedMessage;
                }
                else
                {
                    message = strPrincipalMessage;
                }

                if ( !message.empty() && message != " " )
                {
                    DocumentTextOptions textOptionContestTitle = TranslatorTextConfiguration::TextConfigurationToDocumentTextOptions ( printConfiguration.getTextPrincipalsMessage() );
                    this->NewTextBlock ( textOptionContestTitle );
                    this->DrawString ( message, true );
                    viewPrincipalTitle = true;
                }
            }

            std::list<DocumentTextOptions> listDocumentTextOptions;

            if ( layout.getLayoutContestType() == Smartmatic::SAES::Configuration::LayoutContestType::CONFIGURATION_SIMPLE )
            {
                listDocumentTextOptions.push_back ( textOptionBallotOption );
            }
            else
            {
                listDocumentTextOptions.push_back ( textOptionBallotOption );
                listDocumentTextOptions.push_back ( textOptionBallotOption );
            }

            this->NewTextColunmsBlock ( listDocumentTextOptions );
            setBallotOptions ( contest, listBallotOptionSelected, layout, true, isQuestion );

            //Stand In
            if ( printConfiguration.getTextStandInMessage().getVisible() )
            {
                Glib::ustring message;

                if ( isQuestion )
                {
                    message = strStandInTitleReferendum;
                }
                else if ( listBallotOptionStandInSelected.empty() )
                {
                    message = strStandInNoSelectedMessage;
                }
                else
                {
                    message = strStandInMessage;
                }

                if ( !message.empty() && message != " " )
                {
                    DocumentTextOptions textOptionContestTitle = TranslatorTextConfiguration::TextConfigurationToDocumentTextOptions ( printConfiguration.getTextStandInMessage() );
                    this->NewTextBlock ( textOptionContestTitle );
                    this->DrawString ( message, true );
                    viewStandInTitle = true;
                }
            }

            listDocumentTextOptions.clear();

            if ( layout.getLayoutContestType() == Smartmatic::SAES::Configuration::LayoutContestType::CONFIGURATION_SIMPLE )
            {
                listDocumentTextOptions.push_back ( textOptionBallotOption );
            }
            else
            {
                listDocumentTextOptions.push_back ( textOptionBallotOption );
                listDocumentTextOptions.push_back ( textOptionBallotOption );
            }

            this->NewTextColunmsBlock ( listDocumentTextOptions );
            setBallotOptions ( contest, listBallotOptionStandInSelected, layout, false, isQuestion );
        }
        else
        {
            std::list<DocumentTextOptions> listDocumentTextOptions;

            if ( layout.getLayoutContestType() == Smartmatic::SAES::Configuration::LayoutContestType::CONFIGURATION_SIMPLE )
            {
                listDocumentTextOptions.push_back ( textOptionBallotOption );
            }
            else
            {
                listDocumentTextOptions.push_back ( textOptionBallotOption );
                listDocumentTextOptions.push_back ( textOptionBallotOption );
            }

            this->NewTextColunmsBlock ( listDocumentTextOptions );
            setBallotOptions ( contest, listBallotOptionSelected, layout, true, isQuestion );
            setBallotOptions ( contest, listBallotOptionStandInSelected, layout, false, isQuestion );
        }
    }
    catch ( ... )
    {
        std::stringstream ss;
        ss << "FATAL - Error contest not found.";
        logger->Debug ( ss.str() );
    }
}

void VotePrintDocument::setBallotOptions ( Contest & contest,
        std::vector < Ballot_option * > & list,
        LayoutContestConfiguration & layout,
        bool isPrincipal,
        bool isQuestion )
{
    std::vector < Ballot_option * >::iterator it;
    std::list < Glib::ustring > text;

    for ( it = list.begin();
            it != list.end();
            ++it )
    {
        Eligible_option * eligibleOption =  electoralConfiguration->getEligibleOptionFromBallotOptionCode ( contest.getCode(), ( *it )->getCode() );
        Glib::ustring str;

        if ( isQuestion )
        {
            Glib::ustring strEligibleOption = getResource ( eligibleOption->getBallot_name_key() );
            Glib::ustring strBallotOptionCustomCode = Glib::ustring::format ( ( *it )->getTally_position() );
            Glib::ustring strPartyName = getResource ( ( *it )->getParty().getName() != NULL ? ( *it )->getParty().getName().get() : "" );
            str = Glib::ustring::compose ( _ ( "Smartmatic.SAES.Printing.VotePrintDocument.FormatReferendum" ),
                                           strBallotOptionCustomCode, strEligibleOption, strPartyName );
        }
        else
        {
            Glib::ustring strBallotOptionCustomCode = Glib::ustring::format ( ( *it )->getTally_position() );
            str = ProcessTextBallotOption ( isPrincipal, strBallotOptionCustomCode, eligibleOption, _maxBallotOptionLastNameChars, _maxBallotOptionNameChars, _maxBallotOptionCodeChars );
        }

        text.push_back ( str );

        if ( layout.getLayoutContestType() == Smartmatic::SAES::Configuration::LayoutContestType::CONFIGURATION_SIMPLE )
        {
            this->DrawString ( str );
            countLinesBallotOption++;
        }
    }

    if ( layout.getLayoutContestType() == Smartmatic::SAES::Configuration::LayoutContestType::CONFIGURATION_TRUNCATED )
    {
        this->DrawStrings ( text );
        int elements = text.size();
        elements = ( elements % 2 != 0 ? elements + 1 : elements );
        countLinesBallotOption += elements / 2;
    }
}

void VotePrintDocument::setBottomVote ( const Vote::Vote & vote )
{
    this->NewTextBlock ( _textOptionGeneric );
    this->DrawString ( ".", true );

    if ( ( printConfiguration.getQRcodePosition() == QRcodePosition::CONFIGURATION_BOTTOM_ALL ) && showBarCode )
    {
        this->DrawImage ( pixbufBarCode, Smartmatic::SAES::Printing::Center );
    }

    this->NewTextBlock ( _textOptionGeneric );

    Glib::ustring strEnd;
    Glib::ustring strBottom;

    switch ( vote.getMode() )
    {
        case Vote::VoteMode::VOTE_D:
            strEnd = strDemoEnd;
            strBottom = strTextBottomDemo;
            break;

        case Vote::VoteMode::VOTE_S:
            strEnd = strSimulatedEnd;
            strBottom = strTextBottomSimulate;
            break;

        case Vote::VoteMode::VOTE_O:
            strEnd = strOfficialEnd;
            strBottom = strTextBottomOfficial;
            break;

        default:
            break;
    }

    if ( !strEnd.empty() && strEnd != " " )
    {
        this->DrawString ( strEnd, true );
    }

    if ( !strBottom.empty() && strBottom != " " )
    {
        this->DrawString ( "", true );
        this->DrawString ( strBottom, true );
    }
}

std::string VotePrintDocument::ProcessTextBallotOption ( bool principal, Glib::ustring code, const Eligible_option * eligibleOption, int maxLastNameChars, int maxNameChars, int maxCodeChars )
{
    Glib::ustring messageToCompose =  principal ?
                                      _ ( "Smartmatic.SAES.Printing.VotePrintDocument.PrincipalFormat" ) :
                                      _ ( "Smartmatic.SAES.Printing.VotePrintDocument.StandInFormat" );
    int posCode = code.find ( separatorName );
    Glib::ustring codeOut;

    if ( posCode >= 0 )
    {
        codeOut = code.substr ( posCode + 1 );
    }
    else
    {
        codeOut = code;
    }

    if ( ( int ) codeOut.length() > maxCodeChars )
    {
        int dif = codeOut.length() - maxCodeChars;
        codeOut = codeOut.substr ( dif );
    }

    std::string firstName_, lastName_, initialFirstName_;
    electoralConfiguration->getBallotOptionNames ( eligibleOption, firstName_, lastName_, initialFirstName_ );

    Glib::ustring firstName(firstName_), lastName(lastName_), initialFirstName(initialFirstName_);

    if ( maxLastNameChars > 0 )
    {
        lastName = lastName.substr ( 0, maxLastNameChars );
    }

    if ( maxNameChars > 0 )
    {
        firstName =  firstName.substr ( 0, maxNameChars );
    }

    try
    {
        Glib::ustring result = Glib::ustring::compose ( messageToCompose,
                               codeOut,
                               lastName,
                               firstName,
                               initialFirstName );
        return result;
    }
    catch (const Glib::ConvertError& e)
    {
      return e.what();
    }
}

SafeBuffer VotePrintDocument::getBuffer ()
{
    SafeBuffer dataOut;

    try
    {
        Voting::Vote::VoteWrapper::packVoteForQR ( electoralConfiguration, _vote, dataOut );
    }
    catch ( ... )
    {

    }

    return dataOut;
}

void VotePrintDocument::CommandPrintQRCode ( const Vote::Vote & vote )
{
    Glib::RefPtr<Gdk::Pixbuf> backgroundImage;
    Smartmatic::System::SafeBuffer digest = getBuffer();

    Smartmatic::SAES::Configuration::QRCodeConfig config =
        SAESConfigurationManager::getInstance()->getQRCodeVoteConfig ( digest.GetSize() );
    int pixel_size = config.getSizeQRCode();
    int margin = config.getMarginQRCode();
    int version = config.getVersionQRCode();
    QRecLevel correctionLevel = SAESConfigurationManager::getInstance()->GetCorrectionModeQRCode ( config.getCorrectionModeQRCode() );
    logger->Debug ( "----------QRCodeConfig-----------" );
    {
        std::stringstream ss;
        ss << "DataSize             : " << digest.GetSize();
        logger->Debug ( ss.str() );
    }
    {
        std::stringstream ss;
        ss << "CorrectionModeQRCode : " << config.getCorrectionModeQRCode();
        logger->Debug ( ss.str() );
    }
    {
        std::stringstream ss;
        ss << "Pixel                : " << config.getSizeQRCode();
        logger->Debug ( ss.str() );
    }
    {
        std::stringstream ss;
        ss << "VersionQRCode        : " << config.getVersionQRCode();
        logger->Debug ( ss.str() );
    }

    if ( digest.GetSize() > 0 )
    {
        this->DrawQRCode (
            digest
            , pixel_size
            , margin
            , correctionLevel
            , version
            , backgroundImage
            , Smartmatic::SAES::Printing::Center );
    }
}

std::map < std::string, std::map < std::string, std::string > > VotePrintDocument::getTextToPrintMap()
{
    return textToPrintMap;
}

Glib::RefPtr<Gdk::Pixbuf> VotePrintDocument::GetQRCodeVote ( const Vote::Vote & vote )
{
    Smartmatic::System::SafeBuffer digest = getBuffer();

    Smartmatic::SAES::Configuration::QRCodeConfig config =
        SAESConfigurationManager::getInstance()->getQRCodeVoteConfig ( digest.GetSize() );
    Glib::RefPtr<Gdk::Pixbuf> pixBuf;
    int pixelSize = config.getSizeQRCode();
    int margin = config.getMarginQRCode();
    int version = config.getVersionQRCode();
    QRecLevel correctionLevel = SAESConfigurationManager::getInstance()->GetCorrectionModeQRCode ( config.getCorrectionModeQRCode() );
    logger->Debug ( "----------QRCodeConfig-----------" );
    {
        std::stringstream ss;
        ss << "DataSize             : " << digest.GetSize();
        logger->Debug ( ss.str() );
    }
    {
        std::stringstream ss;
        ss << "CorrectionModeQRCode : " << config.getCorrectionModeQRCode();
        logger->Debug ( ss.str() );
    }
    {
        std::stringstream ss;
        ss << "Pixel                : " << config.getSizeQRCode();
        logger->Debug ( ss.str() );
    }
    {
        std::stringstream ss;
        ss << "VersionQRCode        : " << config.getVersionQRCode();
        logger->Debug ( ss.str() );
    }

    if ( digest.GetSize() == 0 )
    {
        return pixBuf;
    }

    Glib::RefPtr<Gdk::Drawable> drwptr;
    gint width ( 0 );
    gint height ( 0 );

    try
    {
        Glib::RefPtr<Gdk::Bitmap> bitmap ( QREncoding::GenerateSymbol ( ( guchar * ) digest.GetData(), digest.GetSize(), pixelSize, margin, correctionLevel, version ) );
        bitmap->get_size ( width, height );
        drwptr = bitmap;
    }
    catch ( QREncodingException & ex )
    {
        std::stringstream ss;
        ss << "ERROR - " << ex.what();
        logger->Debug ( ss.str() );
    }

    pixBuf =  Gdk::Pixbuf::create ( drwptr, 0, 0, width, height );

    return pixBuf;
}

void VotePrintDocument::addCustomAttributesHeader ( const Smartmatic::SAES::Voting::Vote::Vote & vote )
{
    Glib::ustring logicalVoterGroupCode = vote.getLogicalVoterGroup ();
    Logical_voter_group lvg;

    try
    {
        lvg = electoralConfiguration->getVotingDevice()->getLogicalVoterGroup ( logicalVoterGroupCode );
    }
    catch ( const std::exception & e )
    {
        std::stringstream ss;
        ss << "ERROR - " << e.what();
        logger->Debug ( ss.str() );
        return;
    }

    std::list < Custom_attribute > customAttributes = electoralConfiguration->getVotingDevice()->getCustomAttributeList ( lvg );
    std::list<Custom_attribute>::iterator itAtt;

    for ( itAtt = customAttributes.begin(); itAtt != customAttributes.end(); ++itAtt )
    {
        std::list<Custom_attribute_definition>::iterator itDefinitions;

        for ( itDefinitions = customAttributeDefinition.begin(); itDefinitions != customAttributeDefinition.end(); ++itDefinitions )
        {
            if ( ( *itDefinitions ).getCode() == ( *itAtt ).getCode() )
            {
                Glib::ustring messageToCompose =  N_ ( "Smartmatic.SAES.Printing.VotePrintDocument.CustomAttributeHeader" );

                if ( !messageToCompose.empty() && messageToCompose != " " )
                {
                    this->NewTextBlock ( _textOptionGeneric );
                    Glib::ustring CustomAttributeMessage = Glib::ustring::compose ( messageToCompose,
                                                           getResource ( ( *itDefinitions ).getName_key() ),
                                                           getResource ( ( *itDefinitions ).getDescription_key() ),
                                                           getResource ( ( *itAtt ).getValue() ) );

                    if ( !CustomAttributeMessage.empty() && CustomAttributeMessage != " " )
                    {
                        this->DrawString ( CustomAttributeMessage, true );
                    }
                }
            }
        }
    }
}


Glib::ustring VotePrintDocument::GetCustomAttributeValue (
    const Contest & contest,
    const Vote::Vote & vote,
    const Contest_class * contestClass )
{
    Glib::ustring customAttributeValue = "";
    Glib::ustring contestClassCode = contest.getContest_class_code ();

    if ( contestClass->getCustom_code() == NULL )
    {
        return customAttributeValue;
    }

    Glib::ustring customCodeContestClass = contestClass->getCustom_code().get();
    Logical_voter_groups logicalVoterGroups = electoralConfiguration->getVotingDevice()->getLogicalVoterGroups();
    Custom_attribute_definitions CustomAttributeDefinitions;

    if ( logicalVoterGroups.getCustom_attribute_definitions().present() )
    {
        CustomAttributeDefinitions = logicalVoterGroups.getCustom_attribute_definitions().get();
    }
    else
    {
        return customAttributeValue;
    }

    Custom_attribute_definitions::Custom_attribute_definitionSequence CustomAttributeDefinitionSequence = CustomAttributeDefinitions.getCustom_attribute_definition();
    Glib::ustring logicalVoterGroupCode = vote.getLogicalVoterGroup ();
    Logical_voter_group lvg;

    try
    {
        lvg = electoralConfiguration->getVotingDevice()->getLogicalVoterGroup ( logicalVoterGroupCode );
    }
    catch ( const std::exception & e )
    {
        return customAttributeValue;
    }

    Custom_attributes customAttributes = lvg.getCustom_attributes ().get();
    Custom_attributes::Custom_attributeSequence & customAttributeSequence ( customAttributes.getCustom_attribute () );
    int sizeCustomAttributeSequence = customAttributeSequence.size();
    int	SizeCustomAttributeDefinitionSequence = CustomAttributeDefinitionSequence.size();

    for ( int i = 0; i < SizeCustomAttributeDefinitionSequence; i++ )
    {
        if ( CustomAttributeDefinitionSequence[i].getCode() == customCodeContestClass )
        {
            for ( int j = 0 ; j < sizeCustomAttributeSequence; j++ )
            {
                if ( CustomAttributeDefinitionSequence[i].getCode() == customAttributeSequence[j].getCode() )
                {
                    std::string contestClassName = ElectionStringResourcesManager::getInstance()->getResource(contestClass->getName_key());

                    if ( CustomAttributeDefinitionSequence[i].getCustom_attribute_definition_values().getCustom_attribute_definition_value().size() > 1 )
                    {
                        const std::string help = getHelp ( contest );

                        if ( StringUtils::toLowerUtf8 ( contestClassName ) != StringUtils::toLowerUtf8 ( help ) )
                            customAttributeValue = contest.getHelp_key();
                    }
                    else
                    {
                        const std::string description = getDistrictDescription ( contest );

                        if ( StringUtils::toLowerUtf8 ( contestClassName ) != StringUtils::toLowerUtf8 ( description ) )
                            customAttributeValue = contest.getDistrict().getDescription_key();
                    }

                    break;
                }
            }

            break;
        }
    }

    return customAttributeValue;
}

void VotePrintDocument::AddFinalSpaces()
{
}

void VotePrintDocument::translateText()
{
    bool found = false;
    clearText();

    if ( isMultiLanguage )
    {
        separatorLanguage = printConfiguration.getMultiLanguageConfiguration().getSeparator();
        Smartmatic::SAES::Voting::Election::Languages & languages ( electoralConfiguration->getLanguages() );
        Smartmatic::SAES::Voting::Election::Languages::LanguageIterator it;

        if ( languages.getLanguage().size() > 1 )
        {
            std::string previousLanguage;
            std::string previousCountry;
            SetLanguageFunctionality::getCurrentLanguageCountry ( previousLanguage, previousCountry );
            found = true;
            Glib::ustring separator = separatorLanguage;
            separatorLanguage = "";

            for ( it = languages.getLanguage().begin();
                    it != languages.getLanguage().end();
                    ++it )
            {
                SetLanguageFunctionality::setNewVotingLanguageByLanguageCountry ( ( *it ).getLanguage(), ( *it ).getCountry() );
                concatenateText();
                separatorLanguage = separator;
            }

            SetLanguageFunctionality::setNewVotingLanguageByLanguageCountry ( previousLanguage, previousCountry );
        }
    }

    if ( !found )
    {
        separatorLanguage = "";
        concatenateText();
    }
}

void VotePrintDocument::clearText()
{
    strDemoTitle = "";
    strSimulatedTitle = "";
    strOfficialTitle = "";
    strDemoEnd = "";
    strRecoveryEnd = "";
    strSimulatedEnd = "";
    strOfficialEnd = "";
    strNullReferendum = "";
    strNullVote = "";
    strTitleReferendum = "";
    strParentOptionMessage = "";
    strParentOptionWasSelected = "";
    strPrincipalTitleReferendum = "";
    strPrincipalNoSelectectedMessage = "";
    strPrincipalMessage = "";
    strStandInTitleReferendum = "";
    strStandInNoSelectedMessage = "";
    strStandInMessage = "";
    strTextBottomDemo = "";
    strTextBottomSimulate = "";
    strTextBottomOfficial = "";
}

void VotePrintDocument::concatenateText()
{
    concatenateText ( _ ( "Smartmatic.SAES.Printing.VotePrintDocument.DemoTitle" ), strDemoTitle );
    concatenateText ( _ ( "Smartmatic.SAES.Printing.VotePrintDocument.SimulatedTitle" ), strSimulatedTitle );
    concatenateText ( _ ( "Smartmatic.SAES.Printing.VotePrintDocument.OfficialTitle" ), strOfficialTitle );
    concatenateText ( _ ( "Smartmatic.SAES.Printing.VotePrintDocument.DemoEnd" ), strDemoEnd );
    concatenateText ( _ ( "Smartmatic.SAES.Printing.VotePrintDocument.RecoveryEnd" ), strRecoveryEnd );
    concatenateText ( _ ( "Smartmatic.SAES.Printing.VotePrintDocument.SimulatedEnd" ), strSimulatedEnd );
    concatenateText ( _ ( "Smartmatic.SAES.Printing.VotePrintDocument.OfficialEnd" ), strOfficialEnd );
    concatenateText ( _ ( "Smartmatic.SAES.Printing.VotePrintDocument.NullReferendum" ), strNullReferendum );
    concatenateText ( _ ( "Smartmatic.SAES.Printing.VotePrintDocument.NullVote" ), strNullVote );
    concatenateText ( _ ( "Smartmatic.SAES.Printing.VotePrintDocument.TitleReferendum" ), strTitleReferendum );
    concatenateText ( _ ( "Smartmatic.SAES.Printing.VotePrintDocument.ParentOptionMessage" ), strParentOptionMessage );
    concatenateText ( _ ( "Smartmatic.SAES.Printing.VotePrintDocument.ParentOptionWasSelected" ), strParentOptionWasSelected );
    concatenateText ( _ ( "Smartmatic.SAES.Printing.VotePrintDocument.PrincipalTitleReferendum" ), strPrincipalTitleReferendum );
    concatenateText ( _ ( "Smartmatic.SAES.Printing.VotePrintDocument.PrincipalNoSelectectedMessage" ), strPrincipalNoSelectectedMessage );
    concatenateText ( _ ( "Smartmatic.SAES.Printing.VotePrintDocument.PrincipalMessage" ), strPrincipalMessage );
    concatenateText ( _ ( "Smartmatic.SAES.Printing.VotePrintDocument.StandInTitleReferendum" ), strStandInTitleReferendum );
    concatenateText ( _ ( "Smartmatic.SAES.Printing.VotePrintDocument.StandInNoSelectedMessage" ), strStandInNoSelectedMessage );
    concatenateText ( _ ( "Smartmatic.SAES.Printing.VotePrintDocument.StandInMessage" ), strStandInMessage );
    concatenateText ( _ ( "Smartmatic.SAES.Printing.VotePrintDocument.TextBottomDemo" ), strTextBottomDemo );
    concatenateText ( _ ( "Smartmatic.SAES.Printing.VotePrintDocument.TextBottomSimulate" ), strTextBottomSimulate );
    concatenateText ( _ ( "Smartmatic.SAES.Printing.VotePrintDocument.TextBottomOfficial" ), strTextBottomOfficial );
}

void VotePrintDocument::concatenateText ( Glib::ustring text, Glib::ustring & result )
{
    if ( isMultiLanguage )
    {
        result += ( ( text.empty() || text == " " ) ? "" : ( separatorLanguage + text ) );
    }
    else
    {
        result = text;
    }
}

Glib::ustring VotePrintDocument::getResource ( Glib::ustring key )
{
    Glib::ustring text;
    bool found = false;

    if ( isMultiLanguage )
    {
        Smartmatic::SAES::Voting::Election::Languages & languages ( electoralConfiguration->getLanguages() );
        Smartmatic::SAES::Voting::Election::Languages::LanguageSequence & langSequence ( languages.getLanguage() );
        Smartmatic::SAES::Voting::Election::Languages::LanguageIterator it;

        if ( langSequence.size() > 1 )
        {
            found = true;
            Glib::ustring separator = separatorLanguage;
            separatorLanguage = "";

            for ( it = langSequence.begin();
                    it != langSequence.end();
                    ++it )
            {
                Glib::ustring temp = ElectionStringResourcesManager::getInstance()->getResource ( key, ( *it ).getCustom_code() );
                text += separatorLanguage + temp.substr ( 0, maxCharByLanguage );
                separatorLanguage = separator;
            }
        }
    }

    if ( !found )
    {
        separatorLanguage = "";
        text = ElectionStringResourcesManager::getInstance()->getResource ( key );
    }

    return text;
}
