/**
 * @file X7SSecurity.cxx
 * @brief Body class x7s security
 * @date 19/03/2012
 * @author Yadickson Soto
 */

#include "X7SSecurity.hxx"
#include <Configuration/s-a-e-s-conf-manager.h>
#include <Log/ISMTTLog.h>
#include <Log/SMTTLogManager.h>
#include <Security/Encryption.hxx>
#include <Security/KeyManager.hxx>
#include <System/Encoding/Base64Encoder.h>
#include <System/IO/Path.h>
#include <System/SafeBuffer.h>
#include <System/Security/Cryptography/CMSSigner.hxx>
#include <System/Security/Cryptography/CryptoCMS.hxx>
#include <System/Security/Cryptography/CryptoPKCS7.hxx>
#include <System/Security/Cryptography/PKCS7Signer.hxx>
#include <Voting/ElectoralConfiguration.hxx>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

#define N_(String) (String) ///<macro for gettext not translate

using namespace Smartmatic::Log;
using namespace Smartmatic::SAES::Common::Security;
using namespace Smartmatic::SAES::Configuration;
using namespace Smartmatic::SAES::Runtime::Environment;
using namespace Smartmatic::SAES::Security;
using namespace Smartmatic::SAES::Voting::EmlX7S;
using namespace Smartmatic::SAES::Voting;
using namespace Smartmatic::System::Encoding;
using namespace Smartmatic::System::Exception;
using namespace Smartmatic::System::IO;
using namespace Smartmatic::System::Security::Cryptography;
using namespace Smartmatic::System::Security;
using namespace Smartmatic::System;

ISMTTLog * X7SSecurity::m_logger = SMTTLogManager::GetLogger(
    "Smartmatic.SAES.Voting.Eml.X7SSecurity",
    GETTEXT_PACKAGE);

X7SSecurity::X7SSecurity()
{
    m_electoralConfiguration = NULL;
    m_directories            = NULL;
}

X7SSecurity::~X7SSecurity()
{
}

void X7SSecurity::init(ElectoralConfiguration* electoralConfiguration,
                       SaesDirectories* directories)
{
    m_electoralConfiguration = electoralConfiguration;
    m_directories = directories;
}

void X7SSecurity::signBuffer(const SafeBuffer& bufferIn, SafeBuffer& bufferOut)
{
    X7SSigned signedType = SAESConfigurationManager::getInstance()->getEmlFileConfiguration().getX7SSigned();

    if ( signedType != X7SSigned::CONFIGURATION_NONE )
    {
        SafeBuffer sign;
        X7SSecurityType certType = SAESConfigurationManager::getInstance()->getEmlFileConfiguration().getX7SSignedCertified();
        std::string signedSecretKey;
        std::string certifiedFile;
        CertUtil::CertFileFormat format = CertUtil::CertFormat_PEM;

        switch ( certType )
        {
            case X7SSecurityType::CONFIGURATION_MACHINE_KEYS_FILE:
                {
                    m_logger->Debug ( "Signed X7S EML file with machine keys" );
                    signedSecretKey = m_electoralConfiguration->getVotingDevice()->getFirstVotingDevice().getKeys().getProtection_key();
                    certifiedFile = m_directories->getMachineKeysFileFullPath();
                    format = CertUtil::CertFormat_PKCS12;
                }
                break;

            default:
            case X7SSecurityType::CONFIGURATION_PEM_FILES:
                {
                    m_logger->Debug ( "Signed X7S EML file with pem files" );
                    certifiedFile = m_directories->getX7SSignedCertifiedFileFullPath();
                    signedSecretKey = m_electoralConfiguration->getVotingDevice()->getFirstVotingDevice().getKeys().getBackup_zip_file_password();
                }
                break;
        }

        switch ( signedType )
        {
            case X7SSigned::CONFIGURATION_PKCS7_SIGNER:
                m_logger->Debug ( "Signed X7S EML file with pkcs7" );
                {
                    PKCS7Signer obj;
                    obj.SetCertificate ( certifiedFile, format , signedSecretKey );
                    obj.signBuffer ( bufferIn, sign, PKCS7Base::DER );
                }
                break;

            case X7SSigned::CONFIGURATION_CMS_SIGNER:
                m_logger->Debug ( "Signed X7S EML file with cms" );
                {
                    CMSSigner obj;
                    obj.SetCertificate ( certifiedFile, format , signedSecretKey );
                    obj.signBuffer ( bufferIn, sign);
                }
                break;

            default:
            case X7SSigned::CONFIGURATION_NONE:
                break;
        }

        bufferOut.Clear();
        bufferOut.AddData ( sign );
    }
    else
    {
        bufferOut.Clear();
        bufferOut.AddData ( bufferIn );
    }
}

void X7SSecurity::encryptBuffer(const SafeBuffer& bufferIn, SafeBuffer& bufferOut)
{
    X7SCrypto cryptoType = SAESConfigurationManager::getInstance()->getEmlFileConfiguration().getX7SCrypto();

    if ( cryptoType != X7SCrypto::CONFIGURATION_NONE )
    {
        SafeBuffer crypto;
        X7SSecurityType certType = SAESConfigurationManager::getInstance()->getEmlFileConfiguration().getX7SCryptoCertified();
        std::string cryptoSecretKey;
        std::string certifiedFile;
        CertUtil::CertFileFormat format = CertUtil::CertFormat_PKCS12;

        switch ( certType )
        {
            case X7SSecurityType::CONFIGURATION_MACHINE_KEYS_FILE:
                m_logger->Debug ( "Crypto X7S EML file with machine keys" );
                cryptoSecretKey = m_electoralConfiguration->getVotingDevice()->getFirstVotingDevice().getKeys().getProtection_key();
                certifiedFile = m_directories->getMachineKeysFileFullPath();
                break;

            default:
            case X7SSecurityType::CONFIGURATION_PEM_FILES:
                m_logger->Debug ( "Crypto X7S EML file with pem files" );
                cryptoSecretKey = m_electoralConfiguration->getElection()->getElection().getResults_cert_protection_key();
                certifiedFile = m_directories->getX7SCryptoKeystoreFileFullPath();
                break;
        }

        switch ( cryptoType )
        {
            case X7SCrypto::CONFIGURATION_PKCS7_CRYPTO:
                m_logger->Debug ( "Crypto X7S EML file with pkcs7" );
                {
                    CryptoPKCS7 obj;
                    obj.SetCertificate ( certifiedFile, format , cryptoSecretKey );
                    obj.encryptBuffer ( bufferIn, crypto, PKCS7Base::SMIME );
                }
                break;

            case X7SCrypto::CONFIGURATION_SYMMETRIC_KEY:
                {
                    m_logger->Debug ( "Crypto X7S EML file with symmetric key" );
                    SafeBuffer key;
                    std::string password = m_electoralConfiguration->getVotingDevice()->getFirstVotingDevice().getKeys().getBackup_zip_file_password();
                    Base64Encoder::ConvertFromBase64 ( password, key );
                    Encryption::getInstance()->cryptoBufferToBuffer ( bufferIn, crypto, key );
                }
                break;


            case X7SCrypto::CONFIGURATION_CMS_CRYPTO:
                m_logger->Debug ( "Crypto X7S EML file with cms" );
                {
                    CryptoCMS obj;
                    obj.SetCertificate ( certifiedFile, format, cryptoSecretKey );
                    obj.encryptBuffer ( bufferIn, crypto);
                }
                break;

            default:
            case X7SCrypto::CONFIGURATION_NONE:
                break;
        }

        bufferOut.Clear();
        bufferOut.AddData ( crypto );
    }
    else
    {
        bufferOut.Clear();
        bufferOut.AddData ( bufferIn );
    }
}
