/*
 * NationalityTypeManager.hxx
 *
 *  Created on: 22/11/2013
 *      Author: soto
 */

#ifndef NATIONALITYTYPEMANAGER_HXX_
#define NATIONALITYTYPEMANAGER_HXX_

#include <map>
#include <string>

namespace Smartmatic
{
    namespace Log
    {
        class ISMTTLog;
    }

    namespace System
    {
        class SafeBuffer;
    }

    namespace SAES
    {
        namespace Runtime
        {
            namespace Environment
            {
                class SaesDirectories;
            }
        }

        namespace Voting
        {
            class ElectoralConfiguration;
            class RecountVoteStatistics;

            namespace Eml
            {
                class EmlFileManager;
            }

            namespace PBReports
            {
                class VoteStatisticsWrapper;
            }

            class NationalityTypeManager
            {

                public:

                    /**
                     * @brief Getter method: return unique instance of class
                     * @return instance of class
                     * @date 27/08/2013
                     * @author Yadickson Soto
                     */
                    static NationalityTypeManager * getInstance();

                    /**
                     * @brief Class destructor
                     * @date 27/08/2013
                     * @author Yadickson Soto
                     */
                    ~NationalityTypeManager();

                public:

                    /**
                     * @brief Method initialize of the class
                     * @param electoralConfiguration the electoral configuration
                     * @param directories the saes directories
                     * @param voteStatistics the vote statistics
                     * @param recountVoteStatistics the vote statistics by nationality
                     * @date 27/08/2013
                     * @author Yadickson Soto
                     * @throws FileSystemError On error.
                     */
                    void init(SAES::Runtime::Environment::SaesDirectories* directories,
                              PBReports::VoteStatisticsWrapper* voteStatistics,
                              RecountVoteStatistics* recountVoteStatistics,
                              Eml::EmlFileManager* emlFiles);
                    /**
                     * @brief Persistence voting results using structure from voting-results-schema.xsd
                     * @return true if persistence is successful
                     * @date 27/08/2013
                     * @author Yadickson Soto
                     * @throws FileSystemException On error.
                     */
                    void persistence();

                    /**
                     * @brief Verify voting results file
                     * @return true if verify is successful
                     * @date 27/08/2013
                     * @author Yadickson Soto
                     */
                    bool validate();

                    /**
                     * @brief Delete voting results file
                     * @date 27/08/2013
                     * @author Yadickson Soto
                     */
                    void deleteFiles();

                    /**
                     * @brief  Returns a map from the buffer.
                     * @author manuel.arguelles
                     * @date   Mar  5, 2018
                     * @param  buffer  The input buffer.
                     * @param  resuolt The result map with the nationality type and code.
                     */
                    void getNationalityMap(const System::SafeBuffer& buffer, std::map<std::string, std::string>& result);

                    /**
                     * @brief  Gets the result string.
                     * @author manuel.arguelles
                     * @date   Mar  5, 2018
                     * @param  map The nationality type and code map to use.
                     * @return A formatted string with the results by type.
                     */
                    std::string getStringResult(const std::map<std::string, std::string>& map);

                    /**
                     * @brief  Gets the result string.
                     * @author manuel.arguelles
                     * @date   Mar  5, 2018
                     * @return A formatted string with the results by type.
                     */
                    std::string getStringResult() const;

                    /**
                     * @brief  Gets the total of votes by nationality type.
                     * @author manuel.arguelles
                     * @date   Mar  5, 2018
                     */
                    std::map<std::string, long> getTotalsByType() const;

                private:

                    /**
                     * @brief Class constructor
                     * @date 27/08/2013
                     * @author Yadickson Soto
                     */
                    NationalityTypeManager();

                    /**
                     * @brief  Gets the voting results file for Dom memory.
                     * @author manuel.arguelles
                     * @date   Mar  6, 2018
                     * @return The full path of the voting results file.
                     */
                    std::string getVotingResultsDomFile() const;

                    /**
                     * @brief  Gets the voting results file for Flash memory.
                     * @author manuel.arguelles
                     * @date   Mar  6, 2018
                     * @return The full path of the voting results file.
                     */
                    std::string getVotingResultsFlashFile() const;

                private:

                    SAES::Runtime::Environment::SaesDirectories* directories; ///< The saes directories
                    PBReports::VoteStatisticsWrapper* voteStatistics; ///< The voting statistics
                    RecountVoteStatistics* recountVoteStatistics; ///< The vote statistics by nationality
                    std::map<std::string, std::string> typeCodeMap; ///< The nationality map
                    static Smartmatic::Log::ISMTTLog* logger;  ///< The logger
                    static NationalityTypeManager* instance; ///< instance of class
                    std::string machineCode;
                    std::string customNisCode;
            };
        }
    }
}
#endif /* NATIONALITYTYPEMANAGER_HXX_ */
