/**
 * @file VoteResultsWrapper.cxx
 * @brief Body class vote result wrapper
 * @date 25/03/2014
 * @author Yadickson Soto
 */

#include "VoteResultsWrapper.hxx"
#include <Resources/election-string-resources-manager.h>
#include <System/Utils/StringUtils.hxx>
#include <Voting/Vote/station-tally-schema.hxx>

using namespace Smartmatic::SAES::Voting::PBReports;
using namespace Smartmatic::SAES::Voting;
using namespace Smartmatic::SAES::Voting::Election;
using namespace Smartmatic::SAES::Voting::Vote;
using namespace Smartmatic::SAES::Resources;
using namespace Smartmatic::System::Utils;

VoteResultsWrapper::VoteResultsWrapper ( ElectoralConfiguration * electoralConfiguration, StationTallyWrapper * stationTallyWrapper )
    : electoralConfiguration ( electoralConfiguration ), stationTallyWrapper ( stationTallyWrapper )
{
}

VoteResultsWrapper::~VoteResultsWrapper()
{
}

VoteResults VoteResultsWrapper::getVoteResults ()
{
    VoteResults voteResults;

    if ( electoralConfiguration == NULL || stationTallyWrapper == NULL )
    {
        return voteResults;
    }

    ElectionStringResourcesManager * resource = ElectionStringResourcesManager::getInstance();

    VoteResults::ContestResultsSequence contestSeq;

    std::list <Smartmatic::SAES::Voting::Vote::StationTally *> stList;
    std::list <Smartmatic::SAES::Voting::Vote::StationTally *>::iterator stit;

    stList = stationTallyWrapper->getStationTally();

    for ( stit = stList.begin();
            stit != stList.end();
            ++stit )
    {
        Smartmatic::SAES::Voting::Vote::StationTally * st = ( * stit );

        ContestResults contest;

        std::string contestCode = st->getContest().get();

        contest.setBlank_votes ( st->getEmptyVotes().present() ? st->getEmptyVotes().get() : 0 );
        contest.setTotal_votes ( st->getProcessedVotes().present() ? st->getProcessedVotes().get() : 0 );
        contest.setNon_blank_votes ( contest.getTotal_votes() - contest.getBlank_votes() );

        Smartmatic::SAES::Voting::Election::Contest * electoralContest = electoralConfiguration->getContestByCode ( contestCode );
        Smartmatic::SAES::Voting::Election::Contest_class * contestClass = electoralConfiguration->getContestClassByCode ( electoralContest->getContest_class_code() );

        contest.setCode ( contestCode );
        contest.setContest_name ( resource->getResource ( contestClass->getName_key() ) );

        ContestResults::PartyResultsSequence partyResultsSeq;
        Smartmatic::SAES::Voting::Vote::StationTally::RSequence rseq = st->getRegisters();
        Smartmatic::SAES::Voting::Vote::StationTally::RIterator rit;

        std::map < std::string , PartyResults * > partyCodeMap;
        std::map < int , PartyResults * > partyTallyMap;
        std::map < int , PartyResults * >::iterator partyTallyIt;

        for ( rit = rseq.begin();
                rit != rseq.end();
                ++ rit )
        {
            Smartmatic::SAES::Voting::Vote::Register & regis ( *rit );

            std::string ballotCode = regis.getBallotOption().get();

            Smartmatic::SAES::Voting::Election::Eligible_option * eligible;
            eligible = electoralConfiguration->getEligibleOptionFromBallotOptionCode ( contestCode, ballotCode );

            Smartmatic::SAES::Voting::Election::Ballot_option ballot = * ( eligible->getBallot_options().getBallot_option().begin() );

            if ( eligible->getParent_code().empty() )
            {
                PartyResults * party = new PartyResults();

                int tally = ballot.getTally_position();

                party->setCode ( eligible->getCode() );
                party->setName ( resource->getResource ( ballot.getParty().getName().present() ? ballot.getParty().getName().get() : "" ) );
                party->setTally_position ( tally );

                int onlyListVotes = regis.getCountList().present() ? regis.getCountList().get() : 0;
                int onlyListPVotes = regis.getCountListP().present() ? regis.getCountListP().get() : 0;
                int onlyListPSVotes = regis.getCountListPS().present() ? regis.getCountListPS().get() : 0;
                int onlyListSVotes = regis.getCountListS().present() ? regis.getCountListS().get() : 0;

                party->setTotal_votes ( onlyListVotes + onlyListPVotes + onlyListPSVotes + onlyListSVotes );
                party->setList_votes ( regis.getTotal().present() ? regis.getTotal().get() : 0 );
                party->setTotal_preference_votes ( 0 );

                partyCodeMap [ eligible->getCode() ] = party;
                partyTallyMap [ tally ] = party;
            }
        }

        for ( rit = rseq.begin();
                rit != rseq.end();
                ++ rit )
        {
            Smartmatic::SAES::Voting::Vote::Register & regis ( *rit );

            std::string ballotCode = regis.getBallotOption().get();

            Smartmatic::SAES::Voting::Election::Eligible_option * eligible;
            eligible = electoralConfiguration->getEligibleOptionFromBallotOptionCode ( contestCode, ballotCode );

            Smartmatic::SAES::Voting::Election::Ballot_option ballot = * ( eligible->getBallot_options().getBallot_option().begin() );

            if ( eligible->getParent_code().empty() )
            {
                continue;
            }

            PartyResults * party = partyCodeMap [ eligible->getParent_code() ];
            ItemResult item;

            std::string firstName, lastName, initialFirstName;
            electoralConfiguration->getBallotOptionNames ( eligible, firstName, lastName, initialFirstName );

            item.setCode ( ballot.getCode() );
            item.setTally_position ( ballot.getTally_position() );
            item.setPreference_votes ( regis.getTotal().present() ? regis.getTotal().get() : 0 );
            item.setName ( firstName );
            item.setLastname ( lastName );

            party->setTotal_preference_votes ( party->getTotal_preference_votes() + item.getPreference_votes() );

            switch ( ballot.getCategory() )
            {
                default:
                case Category::ELECTION_LIST:
                case Category::ELECTION_PRINCIPAL:

                    if ( !party->getCandidateResults().present() )
                    {
                        ItemsResults candidateResults;
                        ItemsResults::CandidateSequence seq;
                        candidateResults.setCandidate ( seq );
                        party->setCandidateResults ( candidateResults );
                    }

                    party->getCandidateResults().get().getCandidate().push_back ( item );
                    break;

                case Category::ELECTION_STAND_IN:

                    if ( !party->getSubstituteResults().present() )
                    {
                        ItemsResults substituteResults;
                        ItemsResults::CandidateSequence seq;
                        substituteResults.setCandidate ( seq );
                        party->setSubstituteResults ( substituteResults );
                    }

                    party->getSubstituteResults().get().getCandidate().push_back ( item );
                    break;
            }
        }

        for ( partyTallyIt = partyTallyMap.begin();
                partyTallyIt != partyTallyMap.end();
                ++partyTallyIt )
        {
            PartyResults * party = ( *partyTallyIt ).second;
            partyResultsSeq.push_back ( *party );
            delete party;
        }

        contest.setPartyResults ( partyResultsSeq );
        contestSeq.push_back ( contest );
    }

    voteResults.setContestResults ( contestSeq );

    return voteResults;
}
