#include "RecountVoteStatistics.h"
#include <Log/SMTTLogManager.h>
#include "Log/ISMTTLog.h"
#include <System/IO/File.h>
#include <System/IO/Directory.h>
#include <Voting/Vote/VoteWrapper.hxx>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

#define N_(String) (String) ///<macro for gettext not translate

using namespace Smartmatic::Log;
using namespace Smartmatic::SAES::Voting::VotingDevice;
using namespace Smartmatic::SAES::Voting;
using namespace Smartmatic::SAES::Voting::Vote;
using namespace Smartmatic::SAES::Runtime::Environment;
using namespace Smartmatic::System::IO;
using namespace Smartmatic::SAES::Common::Configuration;

#define LOGGER_PROJECT "Smartmatic.SAES.Voting.Recount" ///< logger project name

#ifdef GETTEXT_PACKAGE
ISMTTLog * logger = SMTTLogManager::GetLogger ( LOGGER_PROJECT, GETTEXT_PACKAGE );
#else
ISMTTLog * logger = SMTTLogManager::GetLogger ( LOGGER_PROJECT, "" );
#endif

RecountVoteStatistics::RecountVoteStatistics ()
{
    this->votingDeviceWrapper = NULL;
}

RecountVoteStatistics::~RecountVoteStatistics ()
{

}

void RecountVoteStatistics::initialize ( VotingDeviceWrapper * votingDeviceWrapper )
{
    this->votingDeviceWrapper = votingDeviceWrapper;

    clear();
    voterTypes.clear();

    std::list <Custom_attribute_definition> listCAD;

    try
    {
        listCAD	= votingDeviceWrapper->getCustomAttributeDefinitionList ( Machine_type::VOTING_DEVICES_PM );
    }
    catch ( Smartmatic::System::Exception::VotingDeviceException & vde )
    {
        logger->Debug ( vde.getMessage() );
    }
    catch ( ... )
    {
        logger->Debug ( "Unable to fetch CAD!" );
    }

    for ( std::list<Custom_attribute_definition>::iterator iter = listCAD.begin(); iter != listCAD.end(); ++iter )
    {
        m_nationalityKeyName = iter->getCode();
        Custom_attribute_definition_values::Custom_attribute_definition_valueSequence & seq ( iter->getCustom_attribute_definition_values().getCustom_attribute_definition_value() );
        Custom_attribute_definition_values::Custom_attribute_definition_valueIterator cait;

        for ( cait = seq.begin();
                cait != seq.end();
                ++cait )
        {
            voterTypes.insert ( cait->getValue() );
        }
    }
}

void RecountVoteStatistics::ProcessVote ( Smartmatic::SAES::Voting::Vote::Vote & vote )
{
    if ( !votingDeviceWrapper )
    {
        return;
    }

    if ( m_processedVotes.find ( vote.getCode() ) != m_processedVotes.end() )
    {
        logger->Debug ( "VOTE PROCESSED ALREADY!" );
        return;
    }

    const std::string & sessionId = vote.getSessionID();

    int index = GetVoterIndex ( GetVoter ( vote ) );

    if ( index == -1 )
    {
        logger->Debug ( "VOTER TYPE UNRECOGNISED!" );
        return;
    }

    if (vote.getMode() != Smartmatic::SAES::Voting::Vote::VoteMode::VOTE_O)
    {
        logger->Debug ( "VOTE MODE IS NOT OFFICIAL!" );
        return;
    }

    if ( index > 11 )
    {
        std::stringstream ss;
        ss << "WRONG INDEX: " << index;
        logger->Debug ( ss.str() );
        return;
    }

    {
        int sessionNum = GetSessionNum ( sessionId );
        std::vector< std::pair< int, std::list< std::string > > > & sessionResults = GetSessionResults ( sessionId );
        sessionResults[index].first++;
        std::string voteFileName = Smartmatic::SAES::Voting::Vote::VoteWrapper::getFileName ( vote );
        sessionResults[index].second.push_back ( voteFileName );
        //		sessionResults[index].second.push_back(vote.getCode());
        std::pair<int, int> val ( index, sessionNum );
        m_updateQueue.push_back ( val );
    }

    m_processedVotes.insert ( vote.getCode() );
}

bool RecountVoteStatistics::GetUpdateQueue ( std::list< std::pair<int, int> > & updates, std::list<std::string> & updateSessionNames )
{
    updates = m_updateQueue;
    m_updateQueue.clear();

    for ( std::list< std::pair<int, int> >::iterator it = updates.begin(); it != updates.end(); ++it )
    {
        if ( it->first > 6 )
        {
            it->first++; //move after Total
        }

        for ( std::map<std::string, int>::iterator itMap = m_sessionNums.begin(); itMap != m_sessionNums.end(); ++itMap )
        {
            if ( itMap->second == it->second )
            {
                updateSessionNames.push_back ( itMap->first );
                break;
            }
        }
    }

    if ( updates.size() != updateSessionNames.size() )
    {
        logger->Audit ( _( "Inconsistency found in RecountVoteStatistics::GetUpdateQueue" ) );
    }

    return !updates.empty();
}

std::string RecountVoteStatistics::GetVoter ( Smartmatic::SAES::Voting::Vote::Vote & vote )
{
    std::list < Custom_attribute > caList;

    try
    {
        caList = votingDeviceWrapper->getCustomAttributeListFromLVGCode ( vote.getLogicalVoterGroup() );
    }
    catch ( Smartmatic::System::Exception::VotingDeviceException & vde )
    {
        logger->Debug ( vde.getMessage() );
        return "";
    }

    std::string voter;

    for ( std::list < Custom_attribute >::iterator it = caList.begin(); it != caList.end(); ++it )
    {
        if ( it->getCode() == m_nationalityKeyName )
        {
            voter = it->getValue();
            break;
        }
    }

    return voter;
}

std::list<std::string> RecountVoteStatistics::GetVoterTypes()
{
    std::list<std::string> res;

    for ( std::set< std::string >::iterator it = voterTypes.begin(); it != voterTypes.end(); ++it )
    {
        res.push_back ( *it );
    }

    return res;
}

int RecountVoteStatistics::GetVoterIndex ( const std::string & voter )
{
    for ( std::set< std::string >::iterator it = voterTypes.begin(); it != voterTypes.end(); ++it )
    {
        if ( *it == voter )
        {
            return std::distance ( voterTypes.begin(), it );
        }
    }

    return -1;
}

int RecountVoteStatistics::GetSessionNum ( const std::string & sessionId )
{
    std::map<std::string, int>::iterator it = m_sessionNums.find ( sessionId );
    int num;

    if ( it == m_sessionNums.end() )
    {
        num = m_sessionNums.size() + 1;
        m_sessionNums.insert ( std::pair<std::string, int> ( sessionId, num ) );
    }
    else
    {
        num = it->second;
    }

    return num;
}

std::vector< std::pair<int, std::list<std::string> > > & RecountVoteStatistics::GetSessionResults ( const std::string & sessionId )
{
    int num = GetSessionNum ( sessionId );

    if ( num > ( int ) m_sessionResults.size() )
    {
        std::pair< int, std::list< std::string > > tmplt ( 0, std::list< std::string > ( 0 ) );
        m_sessionResults.push_back ( std::vector< std::pair< int, std::list< std::string > > > ( 14, tmplt ) );
        return m_sessionResults.back();
    }
    else
    {
        std::list< std::vector< std::pair<int, std::list<std::string> > > >::iterator itSessions = m_sessionResults.begin();
        std::advance ( itSessions, num - 1 );
        return *itSessions;
    }
}

void RecountVoteStatistics::Persist ( std::list< std::pair<int, int> > deleteQueue )
{
    Smartmatic::SAES::Runtime::Environment::SaesDirectories directories;
    const std::string & votesDomDir = directories.getVotesFolderFullPath ( BasePathType::CONFIGURATION_DOM );
    const std::string & votesFlashDir = directories.getVotesFolderFullPath ( BasePathType::CONFIGURATION_FLASH );

    if ( !Directory::getInstance()->DirectoryExists ( votesDomDir ) || !Directory::getInstance()->DirectoryExists ( votesFlashDir ) )
    {
        logger->Debug ( "Either or both DOM and FLASH directory does not exist" );
        return;
    }

    for ( std::list< std::pair<int, int> >::iterator it = deleteQueue.begin(); it != deleteQueue.end(); ++it )
    {
        if ( ( unsigned int ) it->second >= m_sessionResults.size() )
        {
            logger->Audit ( _( "Persist Inconsistent session number!" ) );
            continue;
        }

        std::list< std::vector< std::pair<int, std::list<std::string> > > >::iterator itS = m_sessionResults.begin();
        std::advance ( itS, it->second );

        if ( ( unsigned int ) it->first >= itS->size() )
        {
            logger->Audit ( _( "Persist Inconsistent field number!" ) );
            continue;
        }

        const std::list<std::string> & deleteList = itS->at ( it->first ).second;

        for ( std::list<std::string>::const_iterator itD = deleteList.begin(); itD != deleteList.end(); ++itD )
        {
#if	0
            Smartmatic::SAES::Voting::Vote::Vote vote;
            std::string f = votesDomDir + "/" + *itD;
            std::stringstream ss;
            ss << "FILE to delete " << *itD;

            if ( VoteWrapper::voteFileToVote ( f, vote ) )
            {
                ss << " voter " << GetVoter ( vote ) << " session " << m_sessionNums.find ( vote.getSessionID() )->second;
            }

            logger->Debug ( ss.str() );
#endif
            File::getInstance()->DeleteFile ( votesDomDir + "/" + *itD );
            File::getInstance()->DeleteFile ( votesFlashDir + "/" + *itD );
        }
    }
}

void RecountVoteStatistics::clear()
{
    m_processedVotes.clear();
}
