/**
 * @file	VMStatusWrapper.cxx
 * @brief	voting machine status wrapper class.
 * @author	Youssef.tourkmani
 * @date	31/10/2012
 */

#include "VMStatusWrapper.hxx"

#include <glibmm.h>
#include "DiagnoseStepWrapper.hxx"
#include <System/Utils/StringUtils.hxx>
#include <System/Utils/DateTimeUtils.hxx>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

using namespace Smartmatic::SAES::Voting::Status;
using namespace Smartmatic::System::Utils;
using namespace Smartmatic::SAES::Common::Configuration;

static const std::string SERIAL_NUMBER_DETECTED_FUNCTIONALITY           = "XMLMSerialNumberFunctionality";
static const std::string SERIAL_NUMBER_FUNCTIONALITY                    = "XMLMSerialNumberMFunctionality";
static const std::string ZIP_CODE_FUNCTIONALITY                         = "XMLMZipCodeFunctionality";
static const std::string DATE_DIAGNOSTIC_FUNCTIONALITY                  = "XMLMDateDiagnosticFunctionality";
static const std::string EURN_DIAGNOSIS_FUNCTIONALITY                   = "XMLMEurnDiagnosisFunctionality";
static const std::string ABLE_DIAGNOSTIC_FUNCTIONALITY                  = "XMLMAbleDiagnosticFunctionality";
static const std::string ABLE_SERIAL_NUMBER_FUNCTIONALITY               = "XMLMAbleSNDiagnosticFunctionality";
static const std::string CALIBRATION_DIAGNOSIS_FUNCTIONALITY            = "XMLMCalibrationDiagnosisFunctionality";
static const std::string PRINTER_DIAGNOSIS_FUNCTIONALITY                = "XMLMPrinterDiagnosisFunctionality";
static const std::string BARCODE_DIAGNOSIS_FUNCTIONALITY                = "XMLMBarcodeDiagnosisFunctionality";
static const std::string SMARTCARD_READER_DIAGNOSIS_FUNCTIONALITY       = "XMLMSCReaderDiagnosisFunctionality";
static const std::string ALARMBOX_DIAGNOSIS_FUNCTIONALITY               = "XMLMAlarmBoxDiagnosisFunctionality";
static const std::string MOUSE_FUNCTIONALITY                            = "XMLMMouseFunctionality";
static const std::string CUSTOMIZABLE_CHECK_LIST_DIAGNOSE_FUNCTIONALITY = "XMLMCustomizableCheckListDiagnoseFunctionality";
static const std::string CUSTOM_REMARK_DIAGNOSE_FUNCTIONALITY           = "XMLMCustomRemarkDiagnoseFunctionality";

/**
 * @brief method to compare two devices (based on their DateTime of detection)
 * @param d1 device one
 * @param d2 device two
 * @return true is d1 < d2
 * @date 19/09/2012
 * @author Yadickson Soto
 */
static bool sortDevices ( DeviceStatus d1, DeviceStatus d2 )
{
    return ( d1.getVMDateTime().compare ( d2.getVMDateTime() ) < 0 );
}

/**
 * @brief method to get a map containing the run functionality descriptors.
 * @return A map containing the run functionality descriptors.
 * @author eduardo.fernandez
 * @date 01/03/2018
 */
static std::map<std::string, std::string> getRunFunctionalityDescriptorsMap()
{
    std::map<std::string, std::string> runFunctionalityDescriptorsMap;
    runFunctionalityDescriptorsMap[SERIAL_NUMBER_DETECTED_FUNCTIONALITY] = DiagnoseCode(DiagnoseCode::VOTING_MACHINE_STATUS_SND);
    runFunctionalityDescriptorsMap[SERIAL_NUMBER_FUNCTIONALITY] = DiagnoseCode(DiagnoseCode::VOTING_MACHINE_STATUS_SNM);
    runFunctionalityDescriptorsMap[ZIP_CODE_FUNCTIONALITY] = DiagnoseCode(DiagnoseCode::VOTING_MACHINE_STATUS_Z);
    runFunctionalityDescriptorsMap[DATE_DIAGNOSTIC_FUNCTIONALITY] = DiagnoseCode(DiagnoseCode::VOTING_MACHINE_STATUS_D);
    runFunctionalityDescriptorsMap[EURN_DIAGNOSIS_FUNCTIONALITY] = DiagnoseCode(DiagnoseCode::VOTING_MACHINE_STATUS_EU);
    runFunctionalityDescriptorsMap[ABLE_DIAGNOSTIC_FUNCTIONALITY] = DiagnoseCode(DiagnoseCode::VOTING_MACHINE_STATUS_AA);
    runFunctionalityDescriptorsMap[ABLE_SERIAL_NUMBER_FUNCTIONALITY] = DiagnoseCode(DiagnoseCode::VOTING_MACHINE_STATUS_AA_SND);
    runFunctionalityDescriptorsMap[CALIBRATION_DIAGNOSIS_FUNCTIONALITY] = DiagnoseCode(DiagnoseCode::VOTING_MACHINE_STATUS_T);
    runFunctionalityDescriptorsMap[PRINTER_DIAGNOSIS_FUNCTIONALITY] = DiagnoseCode(DiagnoseCode::VOTING_MACHINE_STATUS_P);
    runFunctionalityDescriptorsMap[BARCODE_DIAGNOSIS_FUNCTIONALITY] = DiagnoseCode(DiagnoseCode::VOTING_MACHINE_STATUS_B);
    runFunctionalityDescriptorsMap[SMARTCARD_READER_DIAGNOSIS_FUNCTIONALITY] = DiagnoseCode(DiagnoseCode::VOTING_MACHINE_STATUS_S);
    runFunctionalityDescriptorsMap[ALARMBOX_DIAGNOSIS_FUNCTIONALITY] = DiagnoseCode(DiagnoseCode::VOTING_MACHINE_STATUS_A);
    runFunctionalityDescriptorsMap[MOUSE_FUNCTIONALITY] = DiagnoseCode(DiagnoseCode::VOTING_MACHINE_STATUS_M);
    runFunctionalityDescriptorsMap[CUSTOMIZABLE_CHECK_LIST_DIAGNOSE_FUNCTIONALITY] = DiagnoseCode(DiagnoseCode::VOTING_MACHINE_STATUS_C);
    runFunctionalityDescriptorsMap[CUSTOM_REMARK_DIAGNOSE_FUNCTIONALITY] = DiagnoseCode(DiagnoseCode::VOTING_MACHINE_STATUS_R);

    return runFunctionalityDescriptorsMap;
}

/**
 * @brief method to get a specific diagnose result from the results vector got from the results string.
 * @param diagnoseCode The code of the diagnose result to get.
 * @param resultsVector The results vector got from the results string.
 * @return A specific diagnose result from the results vector got from the results string.
 * @author eduardo.fernandez
 * @date 01/03/2018
 */
static std::string getDiagnoseResultFromResultsVector(const std::string &diagnoseCode, const std::vector<std::string> &resultsVector)
{
    std::string diagnoseResult = "";

    for (size_t i = 0; i < resultsVector.size(); ++i)
    {
        std::vector<std::string> resultVector;
        StringUtils::splitToVector((resultsVector[i]), ":", resultVector);

        if (resultVector[0] == diagnoseCode)
        {
            diagnoseResult = resultsVector[i];
            break;
        }
    }

    return diagnoseResult;
}

/**
 * @brief method to add a specific diagnose result to the cumulative ordered results string.
 * @param orderedList The cumulative ordered results string.
 * @param diagnoseResultToAdd The diagnose result to add.
 * @author eduardo.fernandez
 * @date 01/03/2018
 */
static void addDiagnoseResult(std::string &orderedList, const std::string &diagnoseResultToAdd)
{
    if ((diagnoseResultToAdd.empty() == false) && (StringUtils::regex(diagnoseResultToAdd, orderedList) == false))
    {
        orderedList += Glib::ustring::compose("%1,", diagnoseResultToAdd);
    }
}

/**
 * @brief method to add a specific diagnose result that is not included in the run functionality descriptors list.
 * @param orderedList The cumulative ordered results string.
 * @param runFunctionalityDescriptorsMap Map containing the run functionality descriptors.
 * @param diagnoseCode The code of the diagnose result to add.
 * @param resultsVector The results vector got from the results string.
 * @author eduardo.fernandez
 * @date 01/03/2018
 */
static void addComplementaryDiagnoseResult(
    std::string &orderedList,
    std::map<std::string, std::string> &runFunctionalityDescriptorsMap,
    const std::string &diagnoseCode,
    const std::vector<std::string> &resultsVector)
{
    std::string diagnoseResult;

    if (diagnoseCode == runFunctionalityDescriptorsMap[SERIAL_NUMBER_DETECTED_FUNCTIONALITY])
    {
        diagnoseResult = getDiagnoseResultFromResultsVector(runFunctionalityDescriptorsMap[SERIAL_NUMBER_FUNCTIONALITY], resultsVector);

        addDiagnoseResult(orderedList, diagnoseResult);
    }

    if (diagnoseCode == runFunctionalityDescriptorsMap[ABLE_DIAGNOSTIC_FUNCTIONALITY])
    {
        diagnoseResult = getDiagnoseResultFromResultsVector(runFunctionalityDescriptorsMap[ABLE_SERIAL_NUMBER_FUNCTIONALITY], resultsVector);

        addDiagnoseResult(orderedList, diagnoseResult);
    }
}

/**
 * @brief method to include the results that couldn't be found among the run functionality descriptors or
 *        that may be repeated (f. e. customcheck list diagnose).
 * @param orderedList The cumulative ordered results string.
 * @param resultsVector The results vector got from the results string.
 * @author eduardo.fernandez
 * @date 05/03/2018
 */
static void addRemainingDiagnosticsResults(std::string &orderedList, const std::vector<std::string> &resultsVector)
{
    std::vector<std::string> orderedResultsVector;
    StringUtils::splitToVector((orderedList), ",", orderedResultsVector);

    for (size_t i = 0; i < resultsVector.size(); ++i)
    {
        bool found = false;

        for (size_t j = 0; j < orderedResultsVector.size(); ++j)
        {
            if (resultsVector[i] == orderedResultsVector[j])
            {
                found = true;
                break;
            }
        }

        if (found == false)
        {
            addDiagnoseResult(orderedList, resultsVector[i]);
        }
    }
}

/**
 * @brief method to reorder the diagnosis results string.
 * @param diagnosticsResults The unordered diagnosis results string.
 * @param runFunctionalityDescriptors The currently configured run functionality descriptors.
 * @author eduardo.fernandez
 * @date 01/03/2018
 */
static std::string reorderDiagnosticsResults(const std::string &diagnosticsResults, std::list<std::string> &runFunctionalityDescriptors)
{
    std::string orderedList = "";

    if (diagnosticsResults.empty() == false)
    {
        std::vector<std::string> resultsVector;
        StringUtils::splitToVector((diagnosticsResults), ",", resultsVector);

        std::list<std::string>::const_iterator descriptorsIterator = runFunctionalityDescriptors.begin();
        const std::list<std::string>::const_iterator end = runFunctionalityDescriptors.end();

        std::map<std::string, std::string> runFunctionalityDescriptorsMap = getRunFunctionalityDescriptorsMap();

        for (; descriptorsIterator != end; ++descriptorsIterator)
        {
            const std::string diagnoseCode = runFunctionalityDescriptorsMap[*descriptorsIterator];
            const std::string diagnoseResult = getDiagnoseResultFromResultsVector(diagnoseCode, resultsVector);

            addDiagnoseResult(orderedList, diagnoseResult);
            addComplementaryDiagnoseResult(orderedList, runFunctionalityDescriptorsMap, diagnoseCode, resultsVector);
        }

        addRemainingDiagnosticsResults(orderedList, resultsVector);
    }
    else
    {
        orderedList = diagnosticsResults;
    }

    return orderedList;
}

VotingMachineStatusWrapper::VotingMachineStatusWrapper ( const std::list<std::string>& runFunctionalityDescriptors )
{
    VotingMachineStatus::DeviceStatusSequence dseq;
    VotingMachineStatus::Custom_checklistSequence seq;

    DiagnoseCode code ( DiagnoseCode::VOTING_MACHINE_STATUS_X );
    DiagnoseStep unknown ( code );

    status.setSerial_detected ( unknown );
    status.setSerial ( unknown );
    status.setDate ( unknown );
    status.setZipCode ( unknown );
    status.setCustom_remark ( unknown );

    status.setDeviceStatus ( dseq );
    status.setCustom_checklist ( seq );

    status.setAbled_serial_detected(unknown);

    m_runFunctionalityDescriptors = runFunctionalityDescriptors;
}

VotingMachineStatusWrapper::~VotingMachineStatusWrapper()
{
}

VotingMachineStatus & VotingMachineStatusWrapper::getVotingMachineStatus()
{
    return status;
}

std::string VotingMachineStatusWrapper::getSerial()
{
    std::string serial;

    if ( getVotingMachineStatus().getSerial().getValue().present() )
    {
        serial = getVotingMachineStatus().getSerial().getValue().get();
    }

    return serial;
}

std::string VotingMachineStatusWrapper::getSerialDetected()
{
    std::string serial;

    if ( getVotingMachineStatus().getSerial_detected().getValue().present() )
    {
        serial = getVotingMachineStatus().getSerial_detected().getValue().get();
    }

    return serial;
}

std::string VotingMachineStatusWrapper::getAbled20SerialDetected()
{
    std::string serial;

    if ( getVotingMachineStatus().getAbled_serial_detected().getValue().present() )
    {
        serial = getVotingMachineStatus().getAbled_serial_detected().getValue().get();
    }

    return serial;
}

std::string VotingMachineStatusWrapper::getDate()
{
    std::string date;

    if ( getVotingMachineStatus().getDate().getValue().present() )
    {
        date = getVotingMachineStatus().getDate().getValue().get();
    }

    return date;
}

std::string VotingMachineStatusWrapper::getZipCode()
{
    std::string zipCode;

    if ( getVotingMachineStatus().getZipCode().getValue().present() )
    {
        zipCode = getVotingMachineStatus().getZipCode().getValue().get();
    }

    return zipCode;
}

std::string VotingMachineStatusWrapper::getObservation()
{
    std::string observation;

    if ( getVotingMachineStatus().getCustom_remark().getValue().present() )
    {
        observation = getVotingMachineStatus().getCustom_remark().getValue().get();
    }

    return observation;
}

void VotingMachineStatusWrapper::setSerial ( const std::string & serial )
{
    DiagnoseCode code ( DiagnoseCode::VOTING_MACHINE_STATUS_SNM );
    DiagnoseStep serialDevice ( code );
    serialDevice.setValue ( serial );
    getVotingMachineStatus().setSerial ( serialDevice );
}

void VotingMachineStatusWrapper::setSerialDetected ( const std::string & serial )
{
    DiagnoseCode code ( DiagnoseCode::VOTING_MACHINE_STATUS_SND );
    DiagnoseStep serialDevice ( code );
    serialDevice.setValue ( serial );
    getVotingMachineStatus().setSerial_detected ( serialDevice );
}

void VotingMachineStatusWrapper::setAbled20SerialDetected ( const std::string & serial )
{
    DiagnoseCode code ( DiagnoseCode::VOTING_MACHINE_STATUS_AA_SND );
    DiagnoseStep serialDevice ( code );
    serialDevice.setValue ( serial );
    getVotingMachineStatus().setAbled_serial_detected ( serialDevice );
}

void VotingMachineStatusWrapper::setDate ( const std::string & date )
{
    DiagnoseCode code ( DiagnoseCode::VOTING_MACHINE_STATUS_D );
    DiagnoseStep dateDiagnose ( code );
    dateDiagnose.setValue ( date );
    getVotingMachineStatus().setDate ( dateDiagnose );
}

void VotingMachineStatusWrapper::setZipCode ( const std::string & zipCode )
{
    DiagnoseCode code ( DiagnoseCode::VOTING_MACHINE_STATUS_Z );
    DiagnoseStep zipcodeDiagnose ( code );
    zipcodeDiagnose.setValue ( zipCode );
    getVotingMachineStatus().setZipCode ( zipcodeDiagnose );
}

void VotingMachineStatusWrapper::setObservation ( const std::string & observation, const unsigned int maxLines )
{
    if ( !observation.empty() )
    {
        std::string remark = observation;
        std::string newLine ( "\n" );
        std::vector <std::string> result;
        StringUtils::splitToVector ( remark, newLine, result );

        if ( result.size() > maxLines )
        {
            remark = StringUtils::replaceAll ( remark, newLine, " " );
        }

        DiagnoseCode code ( DiagnoseCode::VOTING_MACHINE_STATUS_R );
        DiagnoseStep customRemark ( code );
        customRemark.setValue ( remark );
        getVotingMachineStatus().setCustom_remark ( customRemark );
    }
}

void VotingMachineStatusWrapper::addCustomCheckList ( const std::string & customCode, bool value )
{
    VotingMachineStatus::Custom_checklistSequence & seq ( getVotingMachineStatus().getCustom_checklist() );

    DiagnoseCode code ( DiagnoseCode::VOTING_MACHINE_STATUS_C );
    DiagnoseStep checklist ( code );
    checklist.setValue ( ( value ? "1" : "0" ) );
    checklist.setCustomCode ( customCode );

    seq.push_back ( checklist );
}

DeviceStatus & VotingMachineStatusWrapper::addDevice ( DeviceType type )
{
    VotingMachineStatus::DeviceStatusSequence & seq ( getVotingMachineStatus().getDeviceStatus() );
    VotingMachineStatus::DeviceStatusIterator it;

    for ( it = seq.begin();
            it != seq.end();
            ++it )
    {
        if ( ( *it ).getVMType() == type )
        {
            return ( *it );
        }
    }

    DeviceStatus device;
    device.setVMType ( type );
    device.setMandatory ( false );
    device.setVMDateTime ( DateTimeUtils::getCurrentDateTime() );

    DiagnoseStep statusD = DiagnoseStepWrapper::createDiagnoseStep ( type );
    statusD.setStatus ( VMStatus::VOTING_MACHINE_STATUS_ERROR );
    device.setVMStatus ( statusD );

    DeviceStatus::DiagnoseStepSequence diagSeq;
    device.setDiagnoseStep ( diagSeq );
    device.setCode ( statusD.getCode() );

    getVotingMachineStatus().getDeviceStatus().push_back ( device );

    return getVotingMachineStatus().getDeviceStatus().back();
}

void VotingMachineStatusWrapper::clearDevice ( DeviceType type )
{
    DeviceStatus & device ( addDevice ( type ) );
    device.getDiagnoseStep().clear ();
    device.getVMStatus().setStatus ( VMStatus::VOTING_MACHINE_STATUS_NOTDIAGNOSED );
}

std::string VotingMachineStatusWrapper::getVersion (
    const std::string & fieldSeparator,
    const std::string & interSeparator )
{
    std::stringstream ss;

    ss << "V" << interSeparator << VERSION << fieldSeparator;

    return ss.str();
}

std::string VotingMachineStatusWrapper::getResult (
    const std::string & fieldSeparator,
    const std::string & interSeparator,
    const std::string & ok,
    const std::string & nok,
    const std::string & na,
    const bool able20IsConnected)
{
    VotingMachineStatus::DeviceStatusSequence & seq ( getVotingMachineStatus().getDeviceStatus() );
    VotingMachineStatus::DeviceStatusIterator it;

    std::stringstream ss;

    ss << DiagnoseStepWrapper::getDiagnoseStepString ( getVotingMachineStatus().getSerial_detected(), fieldSeparator, interSeparator, ok, nok );
    ss << DiagnoseStepWrapper::getDiagnoseStepString ( getVotingMachineStatus().getSerial(), fieldSeparator, interSeparator, ok, nok );

    if (able20IsConnected)
    {
        ss << DiagnoseStepWrapper::getDiagnoseStepString ( getVotingMachineStatus().getAbled_serial_detected(), fieldSeparator, interSeparator, ok, nok );
    }

    ss << DiagnoseStepWrapper::getDiagnoseStepString ( getVotingMachineStatus().getDate(), fieldSeparator, interSeparator, ok, nok );
    ss << DiagnoseStepWrapper::getDiagnoseStepString ( getVotingMachineStatus().getZipCode(), fieldSeparator, interSeparator, ok, nok );

    for ( it = seq.begin();
            it != seq.end();
            ++it )
    {
        ss << DiagnoseStepWrapper::getDiagnoseStepString ( ( *it ).getVMStatus(), "", interSeparator, ok, nok );

        DeviceStatus::DiagnoseStepSequence & diagSeq ( ( *it ).getDiagnoseStep() );
        DeviceStatus::DiagnoseStepIterator diagIt;

        for ( diagIt = diagSeq.begin();
                diagIt != diagSeq.end();
                ++diagIt )
        {
            ss << "-" << ( *diagIt ).getCode();
        }

        ss << fieldSeparator;
    }

    ss << DiagnoseStepWrapper::getDiagnoseStepString ( getVotingMachineStatus().getCustom_remark(), fieldSeparator, interSeparator, ok, nok );

    VotingMachineStatus::Custom_checklistSequence & cseq ( getVotingMachineStatus().getCustom_checklist() );
    VotingMachineStatus::Custom_checklistIterator cit;

    for ( cit = cseq.begin();
            cit != cseq.end();
            ++cit )
    {
        ss << DiagnoseStepWrapper::getDiagnoseStepString ( ( *cit ), fieldSeparator, interSeparator, ok, nok );
    }

    return reorderDiagnosticsResults(ss.str(), m_runFunctionalityDescriptors);
}

void VotingMachineStatusWrapper::changeStatus ( DeviceType type, VMStatus status )
{
    DeviceStatus & device ( addDevice ( type ) );
    device.getVMStatus().setStatus ( status );
}

void VotingMachineStatusWrapper::addDiagnoseStepToDevice ( DeviceType type, const DiagnoseStep & step )
{
    DeviceStatus & device ( addDevice ( type ) );

    DeviceStatus::DiagnoseStepSequence & dseq ( device.getDiagnoseStep() );
    DeviceStatus::DiagnoseStepIterator dit;

    bool found = false;

    for ( dit = dseq.begin() ;
            !found && dit != dseq.end();
            ++dit )
    {
        found = ( *dit ).getCode() == step.getCode();
    }

    if ( !found )
    {
        device.getDiagnoseStep().push_back ( step );
    }

    DeviceStatus::DiagnoseStepSequence & seq ( device.getDiagnoseStep() );
    DeviceStatus::DiagnoseStepIterator it;

    bool validate = true;

    for ( it = seq.begin();
            it != seq.end() && validate;
            ++it )
    {
        validate = validate && ( ( *it ).getStatus().get() == VMStatus::VOTING_MACHINE_STATUS_OK );
    }

    device.getVMStatus().setStatus ( validate ? VMStatus::VOTING_MACHINE_STATUS_OK : VMStatus::VOTING_MACHINE_STATUS_ERROR );
}

void VotingMachineStatusWrapper::setAlarmBoxFullStatus ( bool deviceDetected, bool buzzerTest, bool redLedTest, bool greenLedTest, bool buttonTest )
{
    clearDevice ( DeviceType::CONFIGURATION_ALARMBOX );

    if ( deviceDetected && buzzerTest && redLedTest && greenLedTest && buttonTest )
    {
        changeStatus ( DeviceType::CONFIGURATION_ALARMBOX, VMStatus::VOTING_MACHINE_STATUS_OK );
    }

    if ( !deviceDetected )
    {
        addErrorStatusCode ( DeviceType::CONFIGURATION_ALARMBOX, DiagnoseCode ( DiagnoseCode::VOTING_MACHINE_STATUS_ND ) );
        return;
    }

    if ( !buzzerTest )
    {
        addErrorStatusCode ( DeviceType::CONFIGURATION_ALARMBOX, DiagnoseCode ( DiagnoseCode::VOTING_MACHINE_STATUS_BZ ) );
    }

    if ( !redLedTest )
    {
        addErrorStatusCode ( DeviceType::CONFIGURATION_ALARMBOX, DiagnoseCode ( DiagnoseCode::VOTING_MACHINE_STATUS_RL ) );
    }

    if ( !greenLedTest )
    {
        addErrorStatusCode ( DeviceType::CONFIGURATION_ALARMBOX, DiagnoseCode ( DiagnoseCode::VOTING_MACHINE_STATUS_GL ) );
    }

    if ( !buttonTest )
    {
        addErrorStatusCode ( DeviceType::CONFIGURATION_ALARMBOX, DiagnoseCode ( DiagnoseCode::VOTING_MACHINE_STATUS_BT ) );
    }
}

void VotingMachineStatusWrapper::setBarcodeReaderFullStatus ( bool deviceDetected, bool configurationSuccessful, bool scanSuccessful, bool QRMatch )
{
    if ( !deviceDetected )
    {
        addErrorStatusCode ( DeviceType::CONFIGURATION_BARCODE, DiagnoseCode ( DiagnoseCode::VOTING_MACHINE_STATUS_ND ) );
    }
    else if ( !configurationSuccessful )
    {
        addErrorStatusCode ( DeviceType::CONFIGURATION_BARCODE, DiagnoseCode ( DiagnoseCode::VOTING_MACHINE_STATUS_CE ) );
    }
    else if ( !scanSuccessful )
    {
        addErrorStatusCode ( DeviceType::CONFIGURATION_BARCODE, DiagnoseCode ( DiagnoseCode::VOTING_MACHINE_STATUS_FS ) );
    }
    else if ( !QRMatch )
    {
        addErrorStatusCode ( DeviceType::CONFIGURATION_BARCODE, DiagnoseCode ( DiagnoseCode::VOTING_MACHINE_STATUS_QR ) );
    }
    else
    {
        changeStatus ( DeviceType::CONFIGURATION_BARCODE, VMStatus::VOTING_MACHINE_STATUS_OK );
    }
}

void VotingMachineStatusWrapper::setEurnFullStatus ( bool deviceDetected, bool doorOpenResult, bool doorCloseResult, bool greenLedResult, bool redLedResult)
{
    if ( !deviceDetected )
    {
        addErrorStatusCode ( DeviceType::CONFIGURATION_EURN, DiagnoseCode ( DiagnoseCode::VOTING_MACHINE_STATUS_ND ) );
    }
    else if ( !doorOpenResult )
    {
        addErrorStatusCode ( DeviceType::CONFIGURATION_EURN, DiagnoseCode ( DiagnoseCode::VOTING_MACHINE_STATUS_EOD ) );
    }
    else if ( !doorCloseResult )
    {
        addErrorStatusCode ( DeviceType::CONFIGURATION_EURN, DiagnoseCode ( DiagnoseCode::VOTING_MACHINE_STATUS_ECD ) );
    }
    else if ( !greenLedResult )
    {
        addErrorStatusCode ( DeviceType::CONFIGURATION_EURN, DiagnoseCode ( DiagnoseCode::VOTING_MACHINE_STATUS_EGL ) );
    }
    else if( !redLedResult)
    {
        addErrorStatusCode ( DeviceType::CONFIGURATION_EURN, DiagnoseCode ( DiagnoseCode::VOTING_MACHINE_STATUS_ERL ) );
    }
    else
    {
        changeStatus ( DeviceType::CONFIGURATION_EURN, VMStatus::VOTING_MACHINE_STATUS_OK );
    }
}

void VotingMachineStatusWrapper::setStatusSmartcardReaderNotDetected ()
{
    addErrorStatusCode ( DeviceType::CONFIGURATION_SMARTCARD, DiagnoseCode ( DiagnoseCode::VOTING_MACHINE_STATUS_ND ) );
}

void VotingMachineStatusWrapper::setStatusSmartcardReaderWriteError ()
{
    addErrorStatusCode ( DeviceType::CONFIGURATION_SMARTCARD, DiagnoseCode ( DiagnoseCode::VOTING_MACHINE_STATUS_WE ) );
}

void VotingMachineStatusWrapper::setStatusSmartcardReaderReadError ()
{
    addErrorStatusCode ( DeviceType::CONFIGURATION_SMARTCARD, DiagnoseCode ( DiagnoseCode::VOTING_MACHINE_STATUS_RE ) );
}

void VotingMachineStatusWrapper::setStatusSmartcardReaderInsertionError ()
{
    addErrorStatusCode ( DeviceType::CONFIGURATION_SMARTCARD, DiagnoseCode ( DiagnoseCode::VOTING_MACHINE_STATUS_IE ) );
}

void VotingMachineStatusWrapper::setStatusSmartcardReaderExtractionError ()
{
    addErrorStatusCode ( DeviceType::CONFIGURATION_SMARTCARD, DiagnoseCode ( DiagnoseCode::VOTING_MACHINE_STATUS_EE ) );
}

void VotingMachineStatusWrapper::setStatusSmartcardReaderLedStripError ()
{
    addErrorStatusCode ( DeviceType::CONFIGURATION_SMARTCARD, DiagnoseCode ( DiagnoseCode::VOTING_MACHINE_STATUS_LSE ) );
}

void VotingMachineStatusWrapper::setStatusSmartcardReaderPowerCycleOffError ()
{
    addErrorStatusCode ( DeviceType::CONFIGURATION_SMARTCARD, DiagnoseCode ( DiagnoseCode::VOTING_MACHINE_STATUS_PC0 ) );
}

void VotingMachineStatusWrapper::setStatusSmartcardReaderPowerCycleOnError ()
{
    addErrorStatusCode ( DeviceType::CONFIGURATION_SMARTCARD, DiagnoseCode ( DiagnoseCode::VOTING_MACHINE_STATUS_PC1 ) );
}

void VotingMachineStatusWrapper::addErrorStatusCode ( DeviceType deviceType, const std::string & code )
{
    addStatusCode ( deviceType, code, VMStatus::VOTING_MACHINE_STATUS_ERROR );
}

void VotingMachineStatusWrapper::addStatusCode ( DeviceType deviceType, const std::string & code, VMStatus status )
{
    DiagnoseStep device;
    device.setCode ( code );
    device.setStatus ( status );

    addDiagnoseStepToDevice ( deviceType, device );
}

void VotingMachineStatusWrapper::clearPrinterStatus ()
{
    clearDevice ( DeviceType::CONFIGURATION_PRINTER );
}

void VotingMachineStatusWrapper::setStatusPrinterNotDetected ()
{
    clearPrinterStatus();
    addErrorStatusCode ( DeviceType::CONFIGURATION_PRINTER, DiagnoseCode ( DiagnoseCode::VOTING_MACHINE_STATUS_ND ) );
}

void VotingMachineStatusWrapper::addStatusPrinterNoPaper ()
{
    addErrorStatusCode ( DeviceType::CONFIGURATION_PRINTER, DiagnoseCode ( DiagnoseCode::VOTING_MACHINE_STATUS_NP ) );
}

void VotingMachineStatusWrapper::addStatusPrinterPaperJam ()
{
    addErrorStatusCode ( DeviceType::CONFIGURATION_PRINTER, DiagnoseCode ( DiagnoseCode::VOTING_MACHINE_STATUS_PJ ) );
}

void VotingMachineStatusWrapper::addStatusPrinterCoverOpen ()
{
    addErrorStatusCode ( DeviceType::CONFIGURATION_PRINTER, DiagnoseCode ( DiagnoseCode::VOTING_MACHINE_STATUS_CO ) );
}

void VotingMachineStatusWrapper::addStatusPrinterFeederOpen ()
{
    addErrorStatusCode ( DeviceType::CONFIGURATION_PRINTER, DiagnoseCode ( DiagnoseCode::VOTING_MACHINE_STATUS_FO ) );
}

void VotingMachineStatusWrapper::addStatusPrinterNearEndOfPaper ()
{
    addErrorStatusCode ( DeviceType::CONFIGURATION_PRINTER, DiagnoseCode ( DiagnoseCode::VOTING_MACHINE_STATUS_NE ) );
}

void VotingMachineStatusWrapper::addStatusPrinterSupplyLowWarn ()
{
    addErrorStatusCode ( DeviceType::CONFIGURATION_PRINTER, DiagnoseCode ( DiagnoseCode::VOTING_MACHINE_STATUS_SL ) );
}

void VotingMachineStatusWrapper::addStatusPrinterNoToner ()
{
    addErrorStatusCode ( DeviceType::CONFIGURATION_PRINTER, DiagnoseCode ( DiagnoseCode::VOTING_MACHINE_STATUS_NT ) );
}

void VotingMachineStatusWrapper::addStatusPrinterNoCartridge ()
{
    addErrorStatusCode ( DeviceType::CONFIGURATION_PRINTER, DiagnoseCode ( DiagnoseCode::VOTING_MACHINE_STATUS_NC ) );
}

void VotingMachineStatusWrapper::addStatusPrintingCanceled ()
{
    addErrorStatusCode ( DeviceType::CONFIGURATION_PRINTER, DiagnoseCode ( DiagnoseCode::VOTING_MACHINE_STATUS_PC ) );
}

void VotingMachineStatusWrapper::addStatusPrintingError ()
{
    addErrorStatusCode ( DeviceType::CONFIGURATION_PRINTER, DiagnoseCode ( DiagnoseCode::VOTING_MACHINE_STATUS_PE ) );
}

void VotingMachineStatusWrapper::addStatusAbled10Ok()
{
    changeStatus( DeviceType::CONFIGURATION_P168A, VMStatus::VOTING_MACHINE_STATUS_OK );
}

void VotingMachineStatusWrapper::addStatusAbled10Error()
{
    addErrorStatusCode ( DeviceType::CONFIGURATION_P168A, DiagnoseCode ( DiagnoseCode::VOTING_MACHINE_STATUS_AE ) );
}

void VotingMachineStatusWrapper::addStatusAbled10SoundError()
{
    addErrorStatusCode ( DeviceType::CONFIGURATION_P168A, DiagnoseCode ( DiagnoseCode::VOTING_MACHINE_STATUS_ASE ) );
}

void VotingMachineStatusWrapper::addStatusAbled20Ok()
{
    changeStatus( DeviceType::CONFIGURATION_ABLED, VMStatus::VOTING_MACHINE_STATUS_OK );
}

void VotingMachineStatusWrapper::addStatusAbled20Error()
{
    addErrorStatusCode ( DeviceType::CONFIGURATION_ABLED, DiagnoseCode ( DiagnoseCode::VOTING_MACHINE_STATUS_AE ) );
}

void VotingMachineStatusWrapper::addStatusAbled20SoundError()
{
    addErrorStatusCode ( DeviceType::CONFIGURATION_ABLED, DiagnoseCode ( DiagnoseCode::VOTING_MACHINE_STATUS_ASE ) );
}

void VotingMachineStatusWrapper::setStatusAbledNotDiagnosed()
{
    changeStatus( DeviceType::CONFIGURATION_P168A, VMStatus::VOTING_MACHINE_STATUS_NOTDIAGNOSED );
}

void VotingMachineStatusWrapper::setStatusPrinterManualError ()
{
    clearPrinterStatus();
    addErrorStatusCode ( DeviceType::CONFIGURATION_PRINTER, DiagnoseCode ( DiagnoseCode::VOTING_MACHINE_STATUS_ME ) );
}

std::list <DeviceStatus > VotingMachineStatusWrapper::getDeviceStatusList ()
{
    std::list <DeviceStatus > devices;
    VotingMachineStatus::DeviceStatusSequence & seq ( getVotingMachineStatus().getDeviceStatus() );

    devices.clear();
    devices.insert ( devices.end(), seq.begin(), seq.end() );
    devices.sort ( sortDevices );

    return devices;
}

void VotingMachineStatusWrapper::clearDeviceStatusList()
{
    getVotingMachineStatus().getDeviceStatus().clear();
}

DeviceStatus & VotingMachineStatusWrapper::getDeviceStatus ( DeviceType type )
{
    return addDevice ( type );
}

void VotingMachineStatusWrapper::setDiagnosisStatus ( DeviceType diagnosis, VMStatus status, bool mandatory )
{
    DeviceStatus & device ( getDeviceStatus ( diagnosis ) );

    device.getVMStatus().setStatus ( status );
    device.setVMDateTime ( DateTimeUtils::getCurrentDateTime() );
    device.setMandatory ( mandatory );
}
