/**
 * @file	TransmissionPackageWrapper.hxx
 * @brief	transmission package wrapper class.
 * @author	Youssef.tourkmani
 * @date	31/10/2012
 */

#ifndef _TRANSMISSION_PACKAGES_WRAPPER_H_
#define _TRANSMISSION_PACKAGES_WRAPPER_H_

#include <Voting/Vote/transmission-package-schema.hxx>
#include <string>
#include <list>

namespace Smartmatic
{
    namespace Log
    {
        class ISMTTLog;
    }
    namespace SAES
    {
        namespace Runtime
        {
            namespace Environment
            {
                class SaesDirectories;
            }
        }

        namespace Voting
        {
            class ElectoralConfiguration;

            namespace Vote
            {
                class StationTally;

                /**
                 * @class	TransmissionPackageWrapper
                 * @brief	Transmission package wrapper.
                 * @author	Eduardo.campione
                 * @date	28/04/2011
                 */

                class TransmissionPackageWrapper
                {
                    public:

                        /**
                         * @fn	TransmissionPackageWrapper::TransmissionPackageWrapper();
                         * @brief	Default constructor.
                         * @author	Eduardo.campione
                         * @date	28/04/2011
                         */
                        TransmissionPackageWrapper();

                        /**
                         * @brief	Finaliser.
                         * @author	Eduardo.campione
                         * @date	28/04/2011
                         */
                        virtual ~TransmissionPackageWrapper();

                    public:

                        /**
                         * @brief	Init class.
                         * @author	Yadickson Soto
                         * @date	17/02/2012
                         * @param	electoralConfiguration	electoral configuration.
                         * @param	directories saes directories.
                         */
                        void init ( Smartmatic::SAES::Voting::ElectoralConfiguration * electoralConfiguration,
                                    Smartmatic::SAES::Runtime::Environment::SaesDirectories * directories );

                        /**
                         * @brief	Dispose of this object, cleaning up any resources it uses.
                         * @author	Eduardo.campione
                         * @date	28/04/2011
                         */
                        void dispose();

                        /**
                         * @brief	Adds a vote.
                         * @author	Eduardo.campione
                         * @date	28/04/2011
                         * @param	vote	The vote.
                         */
                        void addVote ( Vote & vote );

                        /**
                         * @brief	Method delete vote in list.
                         * @author	Yadickson Soto
                         * @date	17/02/2012
                         */
                        void deleteVotes();

                        /**
                         * @brief	getter all votes.
                         * @author	Yadickson Soto yadickson.soto@smartmatic.com
                         * @date	05/05/2011
                         * @return	list all vote.
                         */
                        const std::list<Vote>& getAllVote();

                        /**
                         * @brief	getter all vote demo.
                         * @author	Yadickson Soto yadickson.soto@smartmatic.com
                         * @date	05/05/2011
                         * @return	list all vote demo.
                         */
                        const std::list<Vote>& getAllVoteDemo();

                        /**
                         * @brief	Persist the transmission package.
                         * @author	Yadickson Soto
                         * @date	24/03/2014
                         * @param       stationTallies A list of the StationTally objects to include.
                         * @throws	CryptoException	Thrown when a crypto error condition occurs.
                         * @throws	SignedException	Thrown when a signed error condition occurs.
                         * @throws	XmlException	Thrown when an XML error condition occurs.
                         */
                        void persistTransmissionPackage(const std::list<StationTally*>& stationTallies);

                        /**
                         * @brief	Method validate files.
                         * @author	Yadickson Soto
                         * @date	17/02/2012
                         * @return	true is validate files.
                         */
                        bool validate();

                        /**
                         * @brief	Method delete files.
                         * @author	Yadickson Soto
                         * @date	17/02/2012
                         */
                        void deleteFiles();

                    private:

                        /**
                         * @brief	Persist transmission package secret key.
                         * @author	Youssef.tourkmani
                         * @date	31/10/2012
                         * @throws	CryptoException	Thrown when a crypto error condition occurs.
                         * @throws	SignedException	Thrown when a signed error condition occurs.
                         * @throws	XmlException	Thrown when an XML error condition occurs.
                         * @param 	data            The data.
                         */
                        void persistTransmissionPackageSecretKey(std::string& data);

                        /**
                         * @brief	Persist transmission package asymmetric key.
                         * @author	Youssef.tourkmani
                         * @date	31/10/2012
                         * @throws	CryptoException	Thrown when a crypto error condition occurs.
                         * @throws	SignedException	Thrown when a signed error condition occurs.
                         * @throws	XmlException	Thrown when an XML error condition occurs.
                         * @param	data            The data.
                         */
                        void persistTransmissionPackageAsymmetricKey(std::string& data);

                        /**
                         * @brief	Persist transmission package hash.
                         * @author	Youssef.tourkmani
                         * @date	31/10/2012
                         * @throws	CryptoException	Thrown when a crypto error condition occurs.
                         * @throws	SignedException	Thrown when a signed error condition occurs.
                         * @throws	XmlException	Thrown when an XML error condition occurs.
                         */
                        void persistTransmissionPackageHash();

                    private:

                        std::list<Vote>                              votes;
                        std::list<Vote>                              demoVotes;
                        static Smartmatic::Log::ISMTTLog*            logger;
                        Voting::ElectoralConfiguration*              electoralConfiguration;
                        SAES::Runtime::Environment::SaesDirectories* directories;
                };
            }
        }
    }
}

#endif /* _TRANSMISSION_PACKAGES_WRAPPER_H_ */
