/**
 * @file   VoteCode.hxx
 * @brief  vote code class.
 * @author Youssef.tourkmani
 * @author manuel.arguelles
 * @date   31/10/2012
 */

#ifndef _VOTE_CODE_H_
#define _VOTE_CODE_H_

#include <System/SafeBuffer.h>
#include <Voting/Vote/vote-schema.hxx>
#include <string>

namespace Smartmatic
{
    namespace Log
    {
        class ISMTTLog;
    }
    namespace SAES
    {
        namespace Voting
        {
            namespace Vote
            {
                /**
                 * @class  VoteCode
                 * @brief  Vote Code.
                 * @author Yadickson Soto yadickson.soto@smartmatic.com
                 * @date   02/06/2011
                 */
                class VoteCode
                {
                    protected:

                        /**
                         * @brief  Default constructor.
                         * @author Yadickson Soto yadickson.soto@smartmatic.com
                         * @date   02/06/2011
                         */
                        VoteCode();

                    public:

                        /**
                         * @brief  Destroy.
                         * @author Yadickson Soto yadickson.soto@smartmatic.com
                         * @date   02/06/2011
                         */
                        virtual ~VoteCode();

                        /**
                         * @brief  Method generate cbc-mac.
                         * @author Yadickson Soto yadickson.soto@smartmatic.com
                         * @date   02/06/2011
                         * @param  vote The vote object.
                         * @return The compose vote code (uuid + checksum)
                         */
                        static std::string generateCbcMac(const Vote& vote);

                        /**
                         * @brief  Method get compose code.
                         * @author Yadickson Soto yadickson.soto@smartmatic.com
                         * @date   02/06/2011
                         * @param  uuid The vote code.
                         * @param  checksum The checksum of the vote.
                         * @param  dataOut The compose code.
                         */

                        static void getCbcMac(
                            const Smartmatic::System::SafeBuffer& uuid,
                            const Smartmatic::System::SafeBuffer& checksum,
                            Smartmatic::System::SafeBuffer& dataOut);

                        /**
                         * @brief  Method generate selections buffer.
                         * @author eduardo.fernandez
                         * @date   17/01/2019
                         * @param  vote The vote object.
                         * @param  selections The vote selections.
                         */
                        static void getSelections(
                            const Vote& vote,
                            Smartmatic::System::SafeBuffer& selections);

                        /**
                         * @brief  Method generate checksum vote.
                         * @author Yadickson Soto yadickson.soto@smartmatic.com
                         * @date   02/06/2011
                         * @param  vote The vote object.
                         * @param  dataOut The checksum.
                         */
                        static void generateChecksum(
                            const Vote& vote,
                            Smartmatic::System::SafeBuffer& checksum);

                        /**
                         * @brief  Validate vote code.
                         * @author Yadickson Soto yadickson.soto@smartmatic.com
                         * @date   02/06/2011
                         * @param  The vote object.
                         * @return True if the vote code is valid, false otherwise.
                         */
                        static bool validateCbcMac(const Vote& vote);

                        /**
                         * @brief         Encodes the given compose code.
                         * @author        Youssef.tourkmani
                         * @date          31/10/2012
                         * @param[in,out] cbcMac The cbc-mac.
                         */
                        static void encode(std::string& cbcMac);

                        /**
                         * @brief         Decodes the given compose code.
                         * @author        Youssef.tourkmani
                         * @date          31/10/2012
                         * @param[in,out] cbcMac The cbc-mac.
                         */
                        static void decode(std::string& cbcMac);

                    private:

                        /**
                         * @brief  Sort selection precedes.
                         * @author Yadickson Soto yadickson.soto@smartmatic.com
                         * @date   02/06/2011
                         * @param  first The first selection.
                         * @param  second The second selection.
                         * @return True if it succeeds, false if it fails.
                         */
                        static bool sortSelectionVote(const Selection& first,
                                                      const Selection& second);

                        /**
                         * @brief  Sort ballot vote.
                         * @author Youssef.tourkmani
                         * @date   31/10/2012
                         * @param  first The first.
                         * @param  second The second.
                         * @return True if it succeeds, false if it fails.
                         */
                        static bool sortBallotVote(const std::string& first,
                                                   const std::string& second);

                        /**
                         * @brief  Crypto code.
                         * @author Youssef.tourkmani
                         * @date   31/10/2012
                         * @param  buffer The code to encrypt.
                         * @param  dataOut The data out.
                         */
                        static void cryptoCode(
                            const Smartmatic::System::SafeBuffer& buffer,
                            Smartmatic::System::SafeBuffer& dataOut);

                        /**
                         * @brief  Decrypto code.
                         * @author Youssef.tourkmani
                         * @date   31/10/2012
                         * @param  composeCode The compose code.
                         * @param  dataOut The data out.
                         */
                        static void decryptoCode(
                            const std::string& composeCode,
                            Smartmatic::System::SafeBuffer& dataOut);

                    private:

                        static Smartmatic::Log::ISMTTLog* m_logger;
                };
            }
        }
    }
}

#endif /* _VOTE_CODE_H_ */
