/**
 * @file VoteOptimized.hxx
 * @brief Header class for transformation full vote to size optimized
 * @date 21/05/2013
 * @author Yadickson Soto
 */

#ifndef VOTEOPTIMIZED_HXX_
#define VOTEOPTIMIZED_HXX_

#include <System/SafeBuffer.h>
#include <Voting/Vote/vote-schema.hxx>
#include <Voting/Vote/VoteOptimizedSigned.hxx>
#include <Voting/Vote/VoteOptimizedSelection.hxx>

namespace Smartmatic
{
    namespace SAES
    {
        namespace Voting
        {
            namespace Vote
            {

                /**
                 * @class VoteOptimized
                 * @brief Transformation full vote to size optimized
                 * @date 21/05/2013
                 * @author Yadickson Soto
                 */
                class VoteOptimized
                {

                    private:

                        /**
                         * @enum SizeOptimized
                         * @brief Define position data input buffer size optimized
                         * @date 21/05/2013
                         * @author Yadickson Soto
                         */
                        enum SizeOptimized
                        {
                            VOTE_CODE,
                            LOGICAL_VOTER_GROUP_CODE,
                            VOTING_DEVICE_CODE,
                            VOTE_MODE,
                            BALLOT_ID,
                            SESSION_ID
                        };

                        /**
                         * @brief Class constructor
                         * @date 21/05/2013
                         * @author Yadickson Soto
                         */
                        VoteOptimized();

                    public:

                        /**
                         * @brief Class destructor
                         * @date 21/05/2013
                         * @author Yadickson Soto
                         */
                        virtual ~VoteOptimized();

                        /**
                         * @fn static bool sizeOptimizedToVote (
                        		Smartmatic::System::SafeBuffer & bufferIn,
                        		Vote *& vote,
                        		VoteOptimizedSelection * selectionManager,
                        		VoteOptimizedSigned * signedManager );
                         * @brief Transformation buffer size optimized to full vote
                         * @date 21/05/2013
                         * @author Yadickson Soto
                         * @param bufferIn input buffer whit size optimized vote
                         * @param vote vote result
                         * @param selectionManager selection manager for transformation of checksum selection to candidates
                         * @param signedManager signed manager for validate the signed
                         * @return true if the transformation vote is success full
                         */
                        static bool sizeOptimizedToVote (
                            Smartmatic::System::SafeBuffer & bufferIn,
                            Vote *& vote,
                            VoteOptimizedSelection * selectionManager,
                            VoteOptimizedSigned * signedManager );

                        /**
                         * @fn static bool voteToSizeOptimized (
                        		const Vote * vote,
                        		Smartmatic::System::SafeBuffer & bufferOut,
                        		VoteOptimizedSelection * selectionManager,
                        		VoteOptimizedSigned * optimizedSigned );
                         * @brief Transformation full vote to buffer size optimized
                         * @date 21/05/2013
                         * @author Yadickson Soto
                         * @param vote input vote
                         * @param bufferOut buffer out result of size optimized
                         * @param selectionManager selection manager for transformation of checksum selection to candidates
                         * @param signedManager signed manager for validate the signed
                         * @return true if the transformation vote is success full
                         */
                        static bool voteToSizeOptimized (
                            const Vote * vote,
                            Smartmatic::System::SafeBuffer & bufferOut,
                            VoteOptimizedSelection * selectionManager,
                            VoteOptimizedSigned * optimizedSigned );

                };
            }
        }
    }
}
#endif /* VOTEOPTIMIZED_HXX_ */
