/**
 * @file	VoteRecovery.h
 * @brief	Declares the vote recovery class.
 * @author	Youssef.tourkmani
 * @date	31/10/2012
 */

#ifndef VOTERECOVERY_H_
#define VOTERECOVERY_H_


#include <vector>
#include <map>
#include <list>
#include <Voting/Election/election-schema.hxx>
#include <Voting/Vote/vote-schema.hxx>

namespace Smartmatic
{
    namespace Log
    {
        class ISMTTLog;
    }

    namespace SAES
    {

        namespace Voting
        {
            /**
             * @class	ElectoralConfiguration
             * @brief	Electoral configuration.
             * @author	Youssef.tourkmani
             * @date	31/10/2012
             */
            class ElectoralConfiguration;

            namespace Vote
            {

                /**
                 * @class	VoteRecovery
                 * @brief	Vote recovery.
                 * @author	Youssef.tourkmani
                 * @date	31/10/2012
                 */
                class VoteRecovery
                {
                    public:

                        /**
                         * @fn	VoteRecovery::VoteRecovery(Smartmatic::SAES::Voting::ElectoralConfiguration * electoralConfiguration);
                         * @brief	Constructor.
                         * @author	Youssef.tourkmani
                         * @date	31/10/2012
                         * @param [in,out]	electoralConfiguration	If non-null, the electoral configuration.
                         */
                        VoteRecovery ( Smartmatic::SAES::Voting::ElectoralConfiguration * electoralConfiguration );
                        /**
                         * @fn	virtual VoteRecovery::~VoteRecovery();
                         * @brief	Destructor.
                         * @author	Youssef.tourkmani
                         * @date	31/10/2012
                         */
                        virtual ~VoteRecovery();


                        /**
                         * @fn	std::map<std::string, std::string> VoteRecovery::LogicalToRecovery (const Vote& vote);
                         * @brief	Computes a compact representation of the vote object.
                         * @author	mcordovi
                         * @date	Jun 6, 2011
                         * @param	vote	logical vote object.
                         * @return	a plain, compact representation of the logical vote.
                         */
                        std::map<std::string, std::string> LogicalToRecovery ( const Vote & vote );

                        /**
                         * @fn	void VoteRecovery::RecoveryToLogical (std::map<std::string, std::string> & recoverySelections,
                         * 			Smartmatic::SAES::Voting::Vote::Vote& outVote);
                         * @brief	Recovers a logical vote object out of the compact recovery representation.
                         * @author	mcordovi
                         * @date	Jun 6, 2011
                         * @param	recoverySelections	List of base64 strings representing the selections per contest.
                         * @param	outVote							The out vote.
                         */
                        void RecoveryToLogical ( std::map<std::string, std::string> & recoverySelections, Smartmatic::SAES::Voting::Vote::Vote & outVote );

                        /**
                         * @fn	bool VoteRecovery::IsCodeValid (std::string conetstCode, std::string recoveryCode);
                         * @brief	Query if 'conetstCode' is code valid.
                         * @author	Youssef.tourkmani
                         * @date	31/10/2012
                         * @param	conetstCode 	The conetst code.
                         * @param	recoveryCode	The recovery code.
                         * @return	true if code valid, false if not.
                         */
                        bool IsCodeValid ( std::string conetstCode, std::string recoveryCode );

                        /**
                         * @fn	bool VoteRecovery::IsCodeValid (std::map<std::string, std::string> & recoverySelections);
                         * @brief	Query if 'recoverySelections' is code valid.
                         * @author	Youssef.tourkmani
                         * @date	31/10/2012
                         * @param [in,out]	recoverySelections	The recovery selections.
                         * @return	true if code valid, false if not.
                         */
                        bool IsCodeValid ( std::map<std::string, std::string> & recoverySelections );

                    private:

                        Smartmatic::SAES::Voting::ElectoralConfiguration * electoralConfiguration;	///< The electoral configuration

                        /**
                         * @fn	bool VoteRecovery::IsBitmaskCodeValid (std::string conetstCode, std::string recoveryCode);
                         * @brief	validation check.
                         * @author	Youssef.tourkmani
                         * @date	31/10/2012
                         * @param	conetstCode 	The conetst code.
                         * @param	recoveryCode	The recovery code.
                         * @return	true if bitmask code valid, false if not.
                         */

                        bool IsBitmaskCodeValid ( std::string conetstCode, std::string recoveryCode );

                        /**
                         * @fn	std::map<std::string, std::string> VoteRecovery::BitmaskEncode (const Vote& vote);
                         * @brief	encoding.
                         * @author	Youssef.tourkmani
                         * @date	31/10/2012
                         * @param	vote	The vote.
                         * @return	.
                         */

                        std::map<std::string, std::string> BitmaskEncode ( const Vote & vote );

                        /**
                         * @fn	void VoteRecovery::BitmaskDecode (std::map<std::string, std::string>& selectionCodes,
                         * 			Vote& outVote);
                         * @brief	Bitmask decode.
                         * @author	Youssef.tourkmani
                         * @date	31/10/2012
                         * @param [in,out]	selectionCodes	The selection codes.
                         * @param [in,out]	outVote					The out vote.
                         */
                        void BitmaskDecode ( std::map<std::string, std::string> & selectionCodes, Vote & outVote );

                        /**
                         * @fn	unsigned char VoteRecovery::PairToIndex (const std::string& contestCode,
                         * 			const std::string& partyCode);
                         * @brief	auxiliars.
                         * @author	Youssef.tourkmani
                         * @date	31/10/2012
                         * @param	contestCode	The contest code.
                         * @param	partyCode	 	The party code.
                         * @return	.
                         */

                        unsigned char PairToIndex ( const std::string & contestCode, const std::string & partyCode );
                        /**
                         * @fn	void VoteRecovery::IndexToPair (int idx, std::string& contestCode, std::string& partyCode);
                         * @brief	method getter index to pair
                         * @author	Yadickson Soto
                         * @date	31/10/2012
                         * @param       idx index
                         * @param	contestCode	The contest code.
                         * @param	partyCode 	The party code.
                         */
                        void IndexToPair ( int idx, std::string & contestCode, std::string & partyCode );

                        static Smartmatic::Log::ISMTTLog * logger;  ///< The logger
                };
            }
        }
    }
}

#endif /* VOTERECOVERY_H_ */
