/**
 * @file	VoteWrapper.cxx
 * @brief	vote wrapper class.
 * @author	Youssef.tourkmani
 * @date	31/10/2012
 */


#include "VoteWrapper.hxx"
#include <Log/SMTTLogManager.h>
#include <glibmm.h>
#include <Configuration/s-a-e-s-conf-manager.h>
#include "Resources/election-string-resources-manager.h"
#include <Security/Encryption.hxx>
#include <System/Utils/StringUtils.hxx>
#include "VoteCode.hxx"
#include <System/Compression/compression-class.h>
#include <System/Encoding/Base64Encoder.h>
#include <Voting/set-language-functionality.h>
#include <Voting/IDManager/BallotIDManager.hxx>
#include <Voting/IDManager/SessionIDManager.hxx>
#include <Voting/Vote/VoteOptimized.hxx>
#include <Runtime/Environment/filename-hasher.h>
#include "VoteFactory.hxx"
#include <Voting/OperationStatus.hxx>
#include <System/Exception/XmlException.hxx>
#include <System/Exception/CryptoException.hxx>
#include "Exceptions/Voting/Vote/QRVoteInvalidFormatException.h"
#include "Exceptions/Voting/Vote/QRVoteSignatureException.h"


#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

#define N_(String) (String) ///<macro for gettext not translate

using namespace Smartmatic::SAES::Voting::Vote;
using namespace Smartmatic::SAES::Configuration;
using namespace Smartmatic::Log;
using namespace Smartmatic::SAES::Resources;
using namespace Smartmatic::SAES::Security;
using namespace Smartmatic::System::Security::Cryptography;
using namespace Smartmatic::System;
using namespace Smartmatic::System::Encoding;
using namespace Smartmatic::SAES::Exceptions::Voting::Vote;



ISMTTLog * VoteWrapper::logger = SMTTLogManager::GetLogger ( "Smartmatic.SAES.Voting.Vote.VoteWrapper", GETTEXT_PACKAGE );

VoteWrapper::VoteWrapper()
{
}

VoteWrapper::~VoteWrapper()
{
}

std::string VoteWrapper::generateCode ( Vote & vote )
{
    return VoteCode::generateCbcMac ( vote );
}

bool VoteWrapper::voteFileToVote ( std::string & file, Vote & vote )
{
    try
    {
        std::string data = Smartmatic::SAES::Security::Encryption::getInstance()->openXmlFile (
                               file,
                               Smartmatic::SAES::Security::Encryption::EXCHANGE_FILE,
                               Smartmatic::SAES::Common::Configuration::KeyType::CONFIGURATION_SECRET_KEY );
        std::istringstream istr ( data );
        vote = ( * ( Smartmatic::SAES::Voting::Vote::parseV ( istr, Smartmatic::SAES::Voting::VoteTypes::Flags::dont_validate ) ) );
    }
    catch ( ... )
    {
        logger->Debug ( "ERROR - load vote file " + file );
        return false;
    }

    return true;
}

void VoteWrapper::voteToString ( const Vote & vote, std::string & outStringVote )
{
    std::ostringstream str;
    VoteTypes::NamespaceInfomap mapv;
    mapv[""].name = NAMESPACE_VOTING_VOTE;
    mapv[""].schema = "";

    try
    {
        serializeV ( str, vote, mapv, "UTF-8", VoteTypes::Flags::dont_validate );
        outStringVote = str.str();
    }
    catch ( const ::xsd::cxx::tree::exception< char > & e )
    {
        std::stringstream ss;
        ss << "FATAL - " << __func__ << e;
        logger->Error ( ss.str() );
        throw Smartmatic::System::Exception::XmlException ( _ ( "Smartmatic.SAES.Voting.Vote.VoteWrapper.ErrorSerializaData" ), N_ ( "Smartmatic.SAES.Voting.Vote.VoteWrapper.ErrorSerializaData" ), logger );
    }
    catch ( ... )
    {
        std::stringstream ss;
        ss << "FATAL - " << __func__;
        logger->Error ( ss.str() );
        throw Smartmatic::System::Exception::XmlException ( _ ( "Smartmatic.SAES.Voting.Vote.VoteWrapper.ErrorSerializaData" ), N_ ( "Smartmatic.SAES.Voting.Vote.VoteWrapper.ErrorSerializaData" ), logger );
    }
}

Smartmatic::SAES::Voting::PBReports::PVVote VoteWrapper::voteToPVVote (
    Smartmatic::SAES::Voting::ElectoralConfiguration * electoralConfiguration, Vote & vote )
{
    logger->Debug ( "Start vote to pvvote" );
    Smartmatic::SAES::Voting::PBReports::PVVote pvVote;
    Smartmatic::SAES::Voting::Election::ElectionWrapper * election;
    Smartmatic::SAES::Voting::VotingDevice::VotingDeviceWrapper * votingDevice;
    election = electoralConfiguration->getElection();
    votingDevice = electoralConfiguration->getVotingDevice();
    VotingDevice::Voting_device firstVotingDevice = votingDevice->getFirstVotingDevice();
    /*********************************************************************/
    std::string lvgCode = vote.getLogicalVoterGroup();

    VotingDevice::Logical_voter_group lvg = votingDevice->getLogicalVoterGroup ( firstVotingDevice, lvgCode );
    std::string lvgCustomCode = lvg.getCustom_code();
    std::string lvgNameKey = lvg.getName_key();
    std::string toComposeLVG =  _ ( "Smartmatic.SAES.Voting.Vote.VoteWrapper.LogicalVoterGroup" );
    lvgNameKey = Glib::ustring::compose ( toComposeLVG,
                                          lvgCode,
                                          lvgCustomCode,
                                          ElectionStringResourcesManager::getInstance()->getResource ( lvgNameKey ) );

    pvVote.setLogical_voter_group ( lvgNameKey );
    /*********************************************************************/
    std::string pvCode = vote.getCode();
    pvVote.setVote_code ( pvCode );
    /*********************************************************************/
    std::string pvVotingDeviceCode = firstVotingDevice.getCode();
    pvVote.setVoting_device_code ( pvVotingDeviceCode );
    /*********************************************************************/
    Vote::Vote::SSequence & seqContest ( vote.getSelections() );
    Vote::Vote::SIterator seqConstIterator;

    Smartmatic::SAES::Voting::PBReports::PVVote::ContestSequence seqpvContest;

    for ( seqConstIterator = seqContest.begin();
            seqConstIterator != seqContest.end();
            ++seqConstIterator )
    {
        Vote::SType s = ( *seqConstIterator );
        std::string contestCode = s.getContest();

        Smartmatic::SAES::Voting::PBReports::PVContest pvContest;
        Smartmatic::SAES::Voting::PBReports::PVContest::PrincipalsSelectedSequence pvPSeq;
        Smartmatic::SAES::Voting::PBReports::PVContest::StandInSelectedSequence pvSSeq;
        Election::Contest * contest = electoralConfiguration->getContestByCode ( contestCode );
        std::string pvReportName = "";
        std::string contestTitle = contest->getTitle_key();
        std::string contestAbstract = contest->getAbstract_key();
        std::string contestHelp = contest->getHelp_key();
        std::string pvParty = "";
        std::string pvPartyName = "";
        std::string pvPartyAbreviation = "";
        std::string pvPartyOrder = "";
        Selection::BSequence & seqbo ( s.getBallotOptions() );
        Selection::BIterator itbo;
        pvContest.setNull_info ( seqbo.size() == 0 ?
                                 _ ( "Smartmatic.SAES.Voting.Vote.VoteWrapper.VoteNULL" ) :
                                 _ ( "Smartmatic.SAES.Voting.Vote.VoteWrapper.VoteNotNULL" ) );
        std::list<std::string> codes;

        for ( itbo = seqbo.begin();
                itbo != seqbo.end();
                ++itbo )
        {
            codes.push_back ( ( *itbo ) );
        }

        /********************************************************************************/
        std::list <Election::Ballot_option * > boList;
        std::list <Election::Ballot_option * > boPrincipal;
        std::list <Election::Ballot_option * > boStandIn;
        election->getBallotOptionsListByContest ( *contest, codes, boList );
        election->getBallotOptionsByContest ( *contest, codes, boPrincipal, Election::Category::ELECTION_PRINCIPAL );
        election->getBallotOptionsByContest ( *contest, codes, boStandIn, Election::Category::ELECTION_STAND_IN );
        std::list <Election::Ballot_option * >::iterator it;

        /* list */

        for ( it = boList.begin();
                it != boList.end();
                ++it )
        {
            if ( it == boList.begin() )
            {
                getPartyInfo ( * ( *it ),
                               pvPartyAbreviation,
                               pvPartyName );
            }

            Election::Eligible_option * eo = electoralConfiguration->getEligibleOptionFromBallotOptionCode ( contest->getCode(), ( *it )->getCode() );
            pvPSeq.push_back ( getCandidateInfo ( *eo, * ( *it ) ) );
        }

        /* principal */

        for ( it = boPrincipal.begin();
                it != boPrincipal.end();
                ++it )
        {
            if ( it == boPrincipal.begin() )
            {
                getPartyInfo ( * ( *it ),
                               pvPartyAbreviation,
                               pvPartyName );
            }

            Election::Eligible_option * eo = electoralConfiguration->getEligibleOptionFromBallotOptionCode ( contest->getCode(), ( *it )->getCode() );
            pvPSeq.push_back ( getCandidateInfo ( *eo, * ( *it ) ) );
        }

        /* stand in */

        for ( it = boStandIn.begin();
                it != boStandIn.end();
                ++it )
        {
            if ( it == boPrincipal.begin() )
            {
                getPartyInfo ( * ( *it ),
                               pvPartyAbreviation,
                               pvPartyName );
            }

            Election::Eligible_option * eo = electoralConfiguration->getEligibleOptionFromBallotOptionCode ( contest->getCode(), ( *it )->getCode() );
            pvSSeq.push_back ( getCandidateInfo ( *eo, * ( *it ) ) );
        }

        /********************************************************************************/
        Glib::ustring toComposeContest =  _ ( "Smartmatic.SAES.Voting.Vote.VoteWrapper.Contest" );
        pvReportName = Glib::ustring::compose ( toComposeContest,
                                                ElectionStringResourcesManager::getInstance()->getResource ( contestTitle ),
                                                ElectionStringResourcesManager::getInstance()->getResource ( contestAbstract ),
                                                ElectionStringResourcesManager::getInstance()->getResource ( contestHelp ) );
        Glib::ustring toComposeParty =  _ ( "Smartmatic.SAES.Voting.Vote.VoteWrapper.Party" );
        pvParty = Glib::ustring::compose ( toComposeParty,
                                           ElectionStringResourcesManager::getInstance()->getResource ( pvPartyName ),
                                           ElectionStringResourcesManager::getInstance()->getResource ( pvPartyAbreviation ),
                                           ElectionStringResourcesManager::getInstance()->getResource ( pvPartyOrder ) );
        pvContest.setPv_report_name ( pvReportName );
        pvContest.setPv_party_name ( pvParty );
        pvContest.setPrincipalsSelected ( pvPSeq );
        pvContest.setStandInSelected ( pvSSeq );
        seqpvContest.push_back ( pvContest );
    }

    pvVote.setContest ( seqpvContest );
    logger->Debug ( "Success vote to pvvote" );
    return pvVote;
}

void VoteWrapper::getPartyInfo ( Election::Ballot_option & ballotOption,
                                 std::string & pvPartyAbreviation,
                                 std::string & pvPartyName )
{
    if ( ballotOption.getParty().getAbbreviation() != NULL )
    {
        pvPartyAbreviation = ballotOption.getParty().getAbbreviation().get();
    }

    if ( ballotOption.getParty().getName() != NULL )
    {
        pvPartyName = ballotOption.getParty().getName().get();
    }
}

Smartmatic::SAES::Voting::PBReports::PVCandidate VoteWrapper::getCandidateInfo (
    Election::Eligible_option & eligibleOption, Election::Ballot_option & ballotOption )
{
    Smartmatic::SAES::Voting::PBReports::PVCandidate candidate;
    std::string pvCandidateInfo;
    long long unsigned int pvCandidateTallyPosition = 0;
    static std::string pvCharacter = SAESConfigurationManager::getInstance()->getPrintVoteConfiguration().getBallotOptionNameSeparator();
    pvCandidateInfo = eligibleOption.getBallot_name_key();
    pvCandidateTallyPosition = ballotOption.getTally_position();
    pvCandidateInfo = Smartmatic::System::Utils::StringUtils::replaceAll ( pvCandidateInfo, pvCharacter, " " );
    std::stringstream ss;
    ss << pvCandidateTallyPosition;
    std::string pvName;
    std::string toComposeCandidate =  _ ( "Smartmatic.SAES.Voting.Vote.VoteWrapper.Candidate" );
    pvName = Glib::ustring::compose ( toComposeCandidate,
                                      ElectionStringResourcesManager::getInstance()->getResource ( pvCandidateInfo ),
                                      ElectionStringResourcesManager::getInstance()->getResource ( ss.str() ) );
    candidate.setInfo ( pvName );
    return candidate;
}

bool VoteWrapper::validateVote (
    Smartmatic::SAES::Voting::ElectoralConfiguration * electoralConfiguration,
    Vote & vote )
{
    bool success = true;
    success = validateCbcMac ( vote ) && success;
    success = validateLogicalVoterGroupVote ( electoralConfiguration, vote ) && success;
    success = validateVotingDeviceCodeVote ( electoralConfiguration, vote ) && success;

    if ( !success )
    {
        logger->Debug ( "Error validate vote" );
    }

    return success;
}

bool VoteWrapper::validateCbcMac ( Vote & vote )
{
    return VoteCode::validateCbcMac ( vote );
}

bool VoteWrapper::validateLogicalVoterGroupVote (
    Smartmatic::SAES::Voting::ElectoralConfiguration * electoralConfiguration,
    Vote & vote )
{
    Smartmatic::SAES::Voting::VotingDevice::VotingDeviceWrapper * votingDevice;
    votingDevice = electoralConfiguration->getVotingDevice();
    VotingDevice::Voting_device firstVotingDevice = votingDevice->getFirstVotingDevice();
    std::string lvgCode = vote.getLogicalVoterGroup();

    try
    {
        votingDevice->getLogicalVoterGroup ( firstVotingDevice, lvgCode );
    }
    catch ( ... )
    {
        logger->Debug ( "Error getter logical voter group " + lvgCode );
        return false;
    }

    return true;
}

bool VoteWrapper::validateVotingDeviceCodeVote (
    Smartmatic::SAES::Voting::ElectoralConfiguration * electoralConfiguration,
    Vote & vote )
{
    Smartmatic::SAES::Voting::VotingDevice::VotingDeviceWrapper * votingDevice;
    votingDevice = electoralConfiguration->getVotingDevice();
    VotingDevice::Voting_device firstVotingDevice = votingDevice->getFirstVotingDevice();
    std::string vdCode = firstVotingDevice.getCode();

    if ( vote.getVotingDeviceCode() != vdCode )
    {
        logger->Debug ( "Error in voting device code " + vote.getVotingDeviceCode() + " != " + vdCode );
        return false;
    }

    return true;
}

void VoteWrapper::packVoteForQR ( Smartmatic::SAES::Voting::ElectoralConfiguration * electoralConfiguration, const Vote & vote, Smartmatic::System::SafeBuffer & outBuffer, bool deflate )
{
    static Smartmatic::SAES::Configuration::VoteQRCodeModeEnum mode = SAESConfigurationManager::getInstance()->getPrintVoteConfiguration().getVoteQRCodeMode();
    std::string strVote;
    SafeBuffer dataIn;

    switch ( mode )
    {
        case Smartmatic::SAES::Configuration::VoteQRCodeModeEnum::CONFIGURATION_XMLREDUCED:
            //process vote
            Voting::Vote::VoteWrapper::getReducedXmlVoteString ( electoralConfiguration, vote, strVote );
            Encryption::getInstance()->signedXml ( strVote, Smartmatic::SAES::Security::Encryption::PRINTED_VOTE );
            break;

        case Smartmatic::SAES::Configuration::VoteQRCodeModeEnum::CONFIGURATION_XMLFULL:
            Voting::Vote::VoteWrapper::voteToString ( vote, strVote );
            Encryption::getInstance()->signedXml ( strVote, Smartmatic::SAES::Security::Encryption::PRINTED_VOTE );
            break;

        case Smartmatic::SAES::Configuration::VoteQRCodeModeEnum::CONFIGURATION_SIZEOPTIMIZED:
            Voting::Vote::VoteWrapper::getSizeOptimizedVoteString ( electoralConfiguration, vote, strVote );
            break;

        default:
            break;
    }

    //std::string str;
    //voteToString(vote, str);
    //logger->Debug(str);
    //logger->Debug(strVote);
    //setup data for compression
    dataIn.AddData ( ( unsigned char * ) strVote.c_str(), strVote.length() );

    //compress data

    if ( deflate )
    {
        SafeBuffer dataDeflate;
        Smartmatic::System::Compression::CompressionClass::getInstance()->Deflate ( dataIn, dataDeflate );
        Smartmatic::SAES::Security::Encryption::getInstance()->cryptoBufferToBuffer ( dataDeflate, outBuffer, Smartmatic::SAES::Security::Encryption::PRINTED_VOTE );
    }
    else
    {
        Smartmatic::SAES::Security::Encryption::getInstance()->cryptoBufferToBuffer ( dataIn, outBuffer, Smartmatic::SAES::Security::Encryption::PRINTED_VOTE );
    }
}

void VoteWrapper::getReducedXmlVoteString ( Smartmatic::SAES::Voting::ElectoralConfiguration * electoralConfiguration, Vote vote, std::string & outStringVote )
{
    std::map <std::string, std::string> codedSeletions = electoralConfiguration->getVoteRecovery()->LogicalToRecovery ( vote );
    Vote::Vote::SSequence & sequenceS ( vote.getSelections () );
    Vote::Vote::SIterator its = sequenceS.begin();

    while ( its != sequenceS.end() )
    {
        //clear the list and write the coded selections
        its->setContestCodedSelections ( codedSeletions[its->getContest()] );
        its->getBallotOptions().clear();
        ++its;
    }

    //serialize the vote
    VoteWrapper::voteToString ( vote, outStringVote );
}

void VoteWrapper::getExpandedXmlVote ( Smartmatic::SAES::Voting::ElectoralConfiguration * electoralConfiguration, Vote & inOutVote )
{
    std::map <std::string, std::string> codedSeletions;
    Vote::Vote::SSequence & sequenceS ( inOutVote.getSelections () );
    Vote::Vote::SIterator its = sequenceS.begin();

    //Build the map
    while ( its != sequenceS.end() )
    {
        //clear the list and write the coded selections
        if ( its->getContestCodedSelections().present() )
        {
            codedSeletions[its->getContest()] = its->getContestCodedSelections().get();
            its->getContestCodedSelections().reset();
        }
        else
        {
            //throw and exception
            throw QRVoteInvalidFormatException ( _ ( "Smartmatic.SAES.Voting.Vote.VoteWrapper.InvalidXmlDataFormat" ), N_ ( "Smartmatic.SAES.Voting.Vote.VoteWrapper.InvalidXmlDataFormat" ), logger );
        }

        ++its;
    }

    //Get the vote with the selections
    electoralConfiguration->getVoteRecovery()->RecoveryToLogical ( codedSeletions, inOutVote );
}

bool VoteWrapper::unpackVoteFromQR ( Smartmatic::SAES::Voting::ElectoralConfiguration * electoralConfiguration, Smartmatic::System::SafeBuffer & inBuffer, Vote & outVote, bool inflate )
{
    static Smartmatic::SAES::Configuration::VoteQRCodeModeEnum mode = SAESConfigurationManager::getInstance()->getPrintVoteConfiguration().getVoteQRCodeMode();
    Smartmatic::System::SafeBuffer decryptBuffer;
    Smartmatic::SAES::Security::Encryption::getInstance()->decryptoBufferToBuffer ( inBuffer, decryptBuffer, Smartmatic::SAES::Security::Encryption::PRINTED_VOTE );
    Smartmatic::System::SafeBuffer inflateData;

    if ( inflate )
    {
        Smartmatic::System::Compression::CompressionClass::getInstance()->Inflate ( decryptBuffer, inflateData );
    }
    else
    {
        inflateData.AddData ( decryptBuffer );
    }

    if ( inflateData.GetSize() > 0 )
    {
        std::string inflatedString;
        inflatedString.append ( ( const char * ) inflateData.GetData(), inflateData.GetSize() );
        //inflatedString.resize(inflateData.GetSize());
        std::istringstream ss ( inflatedString );

        //std::cout << std::endl << ss.str() << std::endl;

        switch ( mode )
        {
            case Smartmatic::SAES::Configuration::VoteQRCodeModeEnum::CONFIGURATION_XMLFULL:
                Smartmatic::SAES::Security::Encryption::getInstance()->validateSignedString ( inflatedString, Smartmatic::SAES::Security::Encryption::PRINTED_VOTE );
                outVote = ( * ( Smartmatic::SAES::Voting::Vote::parseV ( ss, Smartmatic::SAES::Voting::VoteTypes::Flags::dont_validate ) ) );
                break;

            case Smartmatic::SAES::Configuration::VoteQRCodeModeEnum::CONFIGURATION_XMLREDUCED:
                Smartmatic::SAES::Security::Encryption::getInstance()->validateSignedString ( inflatedString, Smartmatic::SAES::Security::Encryption::PRINTED_VOTE );
                outVote = ( * ( Smartmatic::SAES::Voting::Vote::parseV ( ss, Smartmatic::SAES::Voting::VoteTypes::Flags::dont_validate ) ) );
                Voting::Vote::VoteWrapper::getExpandedXmlVote ( electoralConfiguration, outVote );
                break;

            case Smartmatic::SAES::Configuration::VoteQRCodeModeEnum::CONFIGURATION_SIZEOPTIMIZED:
                Voting::Vote::VoteWrapper::getVoteFromSizeOptimizedString ( electoralConfiguration, inflatedString, outVote );
                break;

            default:
                break;
        }

        //std::string str;
        //voteToString(outVote, str);
        //logger->Debug(str);
        //logger->Debug(inflatedString);
    }
    else
    {
        return false;
    }

    return true;
}

void VoteWrapper::getSizeOptimizedVoteString ( Smartmatic::SAES::Voting::ElectoralConfiguration * electoralConfiguration, const Vote & vote, std::string & outStringVote )
{
    SafeBuffer bufferVote;
    outStringVote.clear();

    VoteOptimizedSelection voteOptimizedSelection ( electoralConfiguration );
    VoteOptimizedSigned voteOptimizedSigned;

    if ( VoteOptimized::voteToSizeOptimized ( &vote, bufferVote, &voteOptimizedSelection, &voteOptimizedSigned ) )
    {
        outStringVote.assign ( ( const char * ) bufferVote.GetData(), bufferVote.GetSize() );
    }
    else
    {
        throw Smartmatic::System::Exception::CryptoException ( _ ( "Smartmatic.SAES.Voting.Vote.VoteWrapper.InvalidSignature" ), N_ ( "Smartmatic.SAES.Voting.Vote.VoteWrapper.InvalidSignature" ), logger );
    }
}

void VoteWrapper::getVoteFromSizeOptimizedString ( Smartmatic::SAES::Voting::ElectoralConfiguration * electoralConfiguration, std::string & stringVote, Vote & outVote )
{
    Vote * newVote = NULL;
    SafeBuffer bufferVote ( stringVote );

    VoteOptimizedSelection voteOptimizedSelection ( electoralConfiguration );
    VoteOptimizedSigned voteOptimizedSigned;

    if ( VoteOptimized::sizeOptimizedToVote ( bufferVote, newVote, &voteOptimizedSelection, &voteOptimizedSigned ) )
    {
        outVote = *newVote;
        delete newVote;
        newVote = NULL;
    }
    else
    {
        throw QRVoteSignatureException ( _ ( "Smartmatic.SAES.Voting.Vote.VoteWrapper.InvalidSignature" ), N_ ( "Smartmatic.SAES.Voting.Vote.VoteWrapper.InvalidSignature" ), logger );
    }
}

Smartmatic::SAES::Voting::Vote::Vote * VoteWrapper::createRandomVote ( Smartmatic::SAES::Voting::ElectoralConfiguration * electoralConfiguration, Smartmatic::SAES::Voting::OperationStatus * operationStatus, bool full, bool empty )
{
    //Get the list of LVG
    Smartmatic::SAES::Voting::VotingDevice::Voting_device votingDevice = electoralConfiguration->getVotingDevice()->getFirstVotingDevice();
    std::list < Smartmatic::SAES::Voting::VotingDevice::Logical_voter_group * > & lvgs ( electoralConfiguration->getLogicalVoterGroupList() );
    std::list < Smartmatic::SAES::Voting::VotingDevice::Logical_voter_group * >::iterator itlvg;
    std::vector < Smartmatic::SAES::Voting::VotingDevice::Logical_voter_group * > logicalVoterGroups;
    std::map < std::string, std::vector < std::string > > contestByContestGroup;
    std::map < std::string, std::vector < std::string > > partiesByContest;

    for ( itlvg = lvgs.begin();
            itlvg != lvgs.end();
            ++itlvg )
    {
        Smartmatic::SAES::Voting::VotingDevice::Logical_voter_group * lvg = ( *itlvg );
        logicalVoterGroups.push_back ( lvg );
        std::string contestGroupCode = lvg->getContest_group_code();
        std::list < Smartmatic::SAES::Voting::Election::Contest * > & contestList ( electoralConfiguration->getListContestByContestGroupCode ( contestGroupCode ) );
        std::list < Smartmatic::SAES::Voting::Election::Contest * >::iterator itcontest;
        logger->Debug ( "contestGroupCode : " + contestGroupCode );

        for ( itcontest = contestList.begin();
                itcontest != contestList.end();
                ++itcontest )
        {
            Smartmatic::SAES::Voting::Election::Contest * contest = ( *itcontest );
            std::string contestCode = contest->getCode();
            contestByContestGroup[contestGroupCode].push_back ( contestCode );
            std::list < Smartmatic::SAES::Voting::Election::Party * > & parties ( electoralConfiguration->getListPartyByContestCode ( contestCode ) );
            std::list < Smartmatic::SAES::Voting::Election::Party * >::iterator itparty;
            logger->Debug ( " - contest " + contestCode );

            for ( itparty = parties.begin();
                    itparty != parties.end();
                    ++itparty )
            {
                partiesByContest[contestCode].push_back ( ( *itparty )->getCode() );
                logger->Debug ( " - party " + ( *itparty )->getCode() );
            }
        }
    }

    Smartmatic::SAES::Voting::Vote::Vote * vote = VoteFactory::createVote();
    int maxLVG = logicalVoterGroups.size();
    int lvgoffset = rand() % maxLVG;
    Smartmatic::SAES::Voting::VotingDevice::Logical_voter_group * lvgSim = logicalVoterGroups[lvgoffset];
    vote->setVotingDeviceCode ( votingDevice.getCode() );
    vote->setMode ( Smartmatic::SAES::Voting::Vote::VoteMode::VOTE_O );
    vote->setLogicalVoterGroup ( lvgSim->getCode() );
    vote->setBallotID ( BallotIDManager::getInstance()->getID() );
    vote->setSessionID ( SessionIDManager::getInstance()->getID ( operationStatus ) );
    vote->getSelections().clear();

    Smartmatic::SAES::Voting::Vote::Vote::SSequence sseq;

    for ( unsigned int j = 0; j < contestByContestGroup[lvgSim->getContest_group_code()].size() && !empty; j++ )
    {
        std::string contSim = contestByContestGroup[lvgSim->getContest_group_code()][j];
        int maxParties = partiesByContest[contSim].size();
        int partyOffset = rand() % maxParties;
        std::string partySim = ( partiesByContest[contSim] ) [partyOffset];
        std::list < Smartmatic::SAES::Voting::Election::Ballot_option * > & eligibleOptionList ( electoralConfiguration->getBallotOptionsListByContestParty ( contSim, partySim ) );
        std::list < Smartmatic::SAES::Voting::Election::Ballot_option * > & eligibleOptionPrincipal ( electoralConfiguration->getBallotOptionsPrincipalByContestParty ( contSim, partySim ) );
        std::list < Smartmatic::SAES::Voting::Election::Ballot_option * > & eligibleOptionStandIn ( electoralConfiguration->getBallotOptionsStandInByContestParty ( contSim, partySim ) );
        std::list < Smartmatic::SAES::Voting::Election::Ballot_option * >::iterator iteo;
        std::vector<std::string> boCodes;

        for ( iteo = eligibleOptionList.begin();
                iteo != eligibleOptionList.end();
                ++iteo )
        {
            if ( full || rand() % ( eligibleOptionList.size() + 1 ) % 2 == 0 )
            {
                boCodes.push_back ( ( *iteo )->getCode() );
            }
        }

        for ( iteo = eligibleOptionPrincipal.begin();
                iteo != eligibleOptionPrincipal.end();
                ++iteo )
        {
            if ( full || rand() % ( eligibleOptionPrincipal.size() + 1 ) % 2 == 0 )
            {
                boCodes.push_back ( ( *iteo )->getCode() );
            }
        }

        for ( iteo = eligibleOptionStandIn.begin();
                iteo != eligibleOptionStandIn.end();
                ++iteo )
        {
            if ( full || rand() % ( eligibleOptionStandIn.size() + 1 ) % 2 == 0 )
            {
                boCodes.push_back ( ( *iteo )->getCode() );
            }
        }

        Smartmatic::SAES::Voting::Vote::Selection::BSequence bseq;
        Smartmatic::SAES::Voting::Vote::Selection selection;
        selection.setContest ( contSim );

        for ( std::vector<std::string>::iterator boI ( boCodes.begin () );
                boI != boCodes.end (); ++boI )
        {
            bseq.push_back ( *boI );
        }

        selection.setBallotOptions ( bseq );
        sseq.push_back ( selection );
    }

    vote->setSelections ( sseq );
    std::string voteCode = generateCode ( *vote );
    vote->setCode ( voteCode );
    return vote;
}

Vote * VoteWrapper::createRandomVote ( Smartmatic::SAES::Voting::ElectoralConfiguration * electoralConfiguration, Smartmatic::SAES::Voting::OperationStatus * operationStatus )
{
    return createRandomVote ( electoralConfiguration, operationStatus, false, false );
}

Vote * VoteWrapper::createEmptyRandomVote ( Smartmatic::SAES::Voting::ElectoralConfiguration * electoralConfiguration, Smartmatic::SAES::Voting::OperationStatus * operationStatus )
{
    return createRandomVote ( electoralConfiguration, operationStatus, false, true );
}

Vote * VoteWrapper::createFullRandomVote ( Smartmatic::SAES::Voting::ElectoralConfiguration * electoralConfiguration, Smartmatic::SAES::Voting::OperationStatus * operationStatus )
{
    return createRandomVote ( electoralConfiguration, operationStatus, true, false );
}

std::string VoteWrapper::getFileName ( const Vote & vote )
{
    Smartmatic::SAES::Runtime::Environment::FileNameHasher hasher;
    std::string fileCode = hasher.GetFileNameHasher ( vote.getCode() ) + ".vt";
    return fileCode;
}
