/**
 * @file	VotingCentersWrapper.hxx
 * @brief	voting centers wrapper class.
 * @author	Youssef.tourkmani
 * @date	31/10/2012
 */

#ifndef _VOTING_CENTERS_WRAPPER_H_
#define _VOTING_CENTERS_WRAPPER_H_

#include <Voting/VotingCenters/voting-centers-schema.hxx>
#include <System/Exception/CryptoException.hxx>
#include <System/Exception/SignedException.hxx>
#include <System/Exception/XmlException.hxx>
#include <Configuration/SaesCommonConfiguration-schema.hxx>
#include <string>
#include <list>

namespace Smartmatic
{
    namespace Log
    {
        class ISMTTLog;
    }

    namespace SAES
    {
        namespace Voting
        {
            namespace VotingCenters
            {
                /**
                 * @class	VotingCentersWrapper
                 * @brief	Voting centers wrapper.
                 * @author	Eduardo.campione
                 * @date	28/04/2011
                 */

                class VotingCentersWrapper
                {
                    private:

                        /**
                         * @fn	VotingCentersWrapper::VotingCentersWrapper();
                         * @brief	Default constructor.
                         * @author	Eduardo.campione
                         * @date	28/04/2011
                         */

                        VotingCentersWrapper();
                        static VotingCentersWrapper * instance; ///< The instance

                    public:

                        /**
                         * @fn	bool VotingCentersWrapper::isInstall();
                         * @brief	Query if this object is install.
                         * @author	Youssef.tourkmani
                         * @date	31/10/2012
                         * @return	true if install, false if not.
                         */
                        bool isInstall();

                        /**
                         * @fn	Smartmatic::SAES::Common::Configuration::BasePathType VotingCentersWrapper::getMasterDevice();
                         * @brief	Gets master device.
                         * @author	Youssef.tourkmani
                         * @date	31/10/2012
                         * @return	The master device.
                         */
                        Smartmatic::SAES::Common::Configuration::BasePathType getMasterDevice();

                        /**
                         * @fn	std::list <VotingCenter> & VotingCentersWrapper::getVotingCenters();
                         * @brief	Gets the voting centers.
                         * @author	Eduardo.campione
                         * @date	28/04/2011
                         * @return	The voting centers.
                         */

                        std::list <VotingCenter> & getVotingCenters();

                        /**
                         * @fn	void VotingCentersWrapper::setInstallationVotingCenter(VotingCenter &votingCenter);
                         * @brief	Sets an installation voting device.
                         * @author	Eduardo.campione
                         * @date	28/04/2011
                         * @param	votingCenter	The voting center.
                         */

                        void setInstallationVotingCenter ( VotingCenter & votingCenter );

                        /**
                         * @fn	void VotingCentersWrapper::getInstallationVotingCenter(VotingCenter &votingCenter);
                         * @brief	Gets installation voting center.
                         * @author	Youssef.tourkmani
                         * @date	31/10/2012
                         * @param [in,out]	votingCenter	The voting center.
                         */
                        void getInstallationVotingCenter ( VotingCenter & votingCenter );

                        /**
                         * @fn	virtual VotingCentersWrapper::~VotingCentersWrapper();
                         * @brief	Finaliser.
                         * @author	Eduardo.campione
                         * @date	28/04/2011
                         */

                        virtual ~VotingCentersWrapper();

                        /**
                         * @fn	void VotingCentersWrapper::persistVotingCenters() throw (Smartmatic::System::Exception::CryptoException,
                         * 			Smartmatic::System::Exception::SignedException, Smartmatic::System::Exception::XmlException);
                         * @brief	Persist voting devices.
                         * @author	Eduardo.campione
                         * @date	28/04/2011
                         * @exception	Smartmatic::System::Exception::CryptoException	Thrown when a crypto error condition
                         * 																														occurs.
                         * @exception	Smartmatic::System::Exception::SignedException	Thrown when a signed error condition
                         * 																														occurs.
                         * @exception	Smartmatic::System::Exception::XmlException			Thrown when an XML error condition
                         * 																														occurs.
                         */

                        void persistVotingCenters()
                        throw ( Smartmatic::System::Exception::CryptoException,
                                Smartmatic::System::Exception::SignedException,
                                Smartmatic::System::Exception::XmlException );

                        /**
                         * @fn	static VotingCentersWrapper * VotingCentersWrapper::getInstance();
                         * @brief	Gets the instance.
                         * @author	Eduardo.campione
                         * @date	28/04/2011
                         * @return	null if it fails, else the instance.
                         */

                        static VotingCentersWrapper * getInstance();

                        /**
                         * @fn	bool VotingCentersWrapper::LoadFile(std::string filePath);
                         * @brief	Loads a file.
                         * @author	Eduardo.campione
                         * @date	28/04/2011
                         * @param	filePath	Full pathname of the file.
                         * @return	true if it succeeds, false if it fails.
                         */
                        bool LoadFile ( std::string filePath );

                        /**
                         * @fn	RunningMode getRunningMode();
                         * @brief	Gets running mode application, if not present attribute return official mode
                         * @author	yadickson Soto
                         * @date	23/05/2013
                         * @return	running mode application
                         */
                        RunningMode getRunningMode();

                    private:
                        bool install; ///< true to install
                        Smartmatic::SAES::Common::Configuration::BasePathType type; ///< The type
                        VotingCenter votingCenter;	///< The voting center
                        std::list <VotingCenter> votingCenters;   ///< The voting centers
                        static Smartmatic::Log::ISMTTLog * logger; ///< The logger
                };
            }
        }
    }
}

#endif /* _VOTING_CENTERS_WRAPPER_H_ */
