/**
 * @file VotingResultsWrapper.hxx
 * @brief Header of voting results class, responsible for the generation of voting results by nationality.
 * @date 27/08/2013
 * @author Yadickson Soto
 */

#ifndef VOTINGRESULTS_HXX_
#define VOTINGRESULTS_HXX_

#include <Voting/ElectoralConfiguration.hxx>
#include <Runtime/Environment/saes-directories.h>
#include <Voting/Recount/RecountVoteStatistics.h>

namespace Smartmatic
{
    namespace Log
    {
        class ISMTTLog;
    }

    namespace SAES
    {
        namespace Voting
        {
            /**
             * @class VotingResultsWrapper
             * @brief Class responsible for the generation of voting results by nationality.
             * @date 27/08/2013
             * @author Yadickson Soto
             */
            namespace PBReports
            {
                class VoteStatisticsWrapper;
            }

            class VotingResultsWrapper
            {
                private:
                    /**
                     * @fn VotingResultsWrapper();
                     * @brief Class constructor
                     * @date 27/08/2013
                     * @author Yadickson Soto
                     */
                    VotingResultsWrapper ();

                    static VotingResultsWrapper * instance; ///< instance of class

                public:

                    /**
                     * @fn virtual ~VotingResultsWrapper();
                     * @brief Class destructor
                     * @date 27/08/2013
                     * @author Yadickson Soto
                     */
                    virtual ~VotingResultsWrapper();
                    /**
                     * @fn void init ( Smartmatic::SAES::Voting::ElectoralConfiguration * electoralConfiguration,
                     * Smartmatic::SAES::Runtime::Environment::SaesDirectories * directories,
                    		Smartmatic::SAES::Voting::PBReports::VoteStatisticsWrapper * voteStatistics,
                    		Smartmatic::SAES::Voting::RecountVoteStatistics * recountVoteStatistics );
                     * @brief Method initialize of the class
                     * @param electoralConfiguration the electoral configuration
                     * @param directories the saes directories
                     * @param voteStatistics the vote statistics
                     * @param recountVoteStatistics the vote statistics by nationality
                     * @date 27/08/2013
                     * @author Yadickson Soto
                     */
                    void init ( Smartmatic::SAES::Voting::ElectoralConfiguration * electoralConfiguration,
                                Smartmatic::SAES::Runtime::Environment::SaesDirectories * directories,
                                Smartmatic::SAES::Voting::PBReports::VoteStatisticsWrapper * voteStatistics,
                                Smartmatic::SAES::Voting::RecountVoteStatistics * recountVoteStatistics );

                    /**
                     * @brief Persistence voting results using structure from voting-results-schema.xsd
                     * @return true if persistence is successful
                     * @throws CryptoException Thrown when a crypto error condition
                     * @throws SignedException Thrown when a signed error condition
                     * @throws XmlException	  Thrown when an XML error condition
                     * @date 27/08/2013
                     * @author Yadickson Soto
                     */
                    void persistence();

                    /**
                     * @fn bool validate();
                     * @brief Verify voting results file
                     * @return true if verify is successful
                     * @date 27/08/2013
                     * @author Yadickson Soto
                     */
                    bool validate();
                    /**
                     * @fn void deleteFiles();
                     * @brief Delete voting results file
                     * @date 27/08/2013
                     * @author Yadickson Soto
                     */
                    void deleteFiles();
                    /**
                     * @fn static VotingResultsWrapper * getInstance();
                     * @brief Getter method: return unique instance of class
                     * @return instance of class
                     * @date 27/08/2013
                     * @author Yadickson Soto
                     */
                    static VotingResultsWrapper * getInstance();

                private:

                    Smartmatic::SAES::Voting::ElectoralConfiguration * electoralConfiguration; ///< The electoral configuration
                    Smartmatic::SAES::Runtime::Environment::SaesDirectories * directories; ///< The saes directories
                    Smartmatic::SAES::Voting::PBReports::VoteStatisticsWrapper * voteStatistics; ///< The voting statistics
                    Smartmatic::SAES::Voting::RecountVoteStatistics * recountVoteStatistics; ///< The vote statistics by nationality
                    static Smartmatic::Log::ISMTTLog * logger;  ///< The logger
            };
        } /* namespace Voting */
    } /* namespace SAES */
} /* namespace Smartmatic */
#endif /* VOTINGRESULTS_HXX_ */
