#ifndef VOTING_MACHINE_STATUS_TEST_H_
#define VOTING_MACHINE_STATUS_TEST_H_

#ifdef ECUT_MAIN

#include <cppunit/extensions/HelperMacros.h>
#include <Voting/Status/VMStatusWrapper.hxx>
#include <unistd.h>

class VotingMachineStatusTest : public CPPUNIT_NS::TestFixture
{
    private:

        typedef Smartmatic::SAES::Voting::Status::VotingMachineStatusWrapper VotingMachineStatusWrapper;
        typedef Smartmatic::SAES::Common::Configuration::DeviceType DeviceType;
        typedef Smartmatic::SAES::Voting::Status::DeviceStatus DeviceStatus;
        typedef Smartmatic::SAES::Voting::Status::DiagnoseCode DiagnoseCode;
        typedef Smartmatic::SAES::Voting::Status::VMStatus VMStatus;

        VotingMachineStatusWrapper * wrapper;

    public:

        VotingMachineStatusTest ()
        {
            wrapper = NULL;
        }

        void setUp()
        {
            std::list<std::string> runFunctionalityDescriptors;
            runFunctionalityDescriptors.push_back("XMLMSerialNumberFunctionality");
            runFunctionalityDescriptors.push_back("XMLMSerialNumberMFunctionality");
            runFunctionalityDescriptors.push_back("XMLMZipCodeFunctionality");
            runFunctionalityDescriptors.push_back("XMLMDateDiagnosticFunctionality");
            runFunctionalityDescriptors.push_back("XMLMEurnDiagnosisFunctionality");
            runFunctionalityDescriptors.push_back("XMLMAbleDiagnosticFunctionality");
            runFunctionalityDescriptors.push_back("XMLMAbleSNDiagnosticFunctionality");
            runFunctionalityDescriptors.push_back("XMLMCalibrationDiagnosisFunctionality");
            runFunctionalityDescriptors.push_back("XMLMPrinterDiagnosisFunctionality");
            runFunctionalityDescriptors.push_back("XMLMBarcodeDiagnosisFunctionality");
            runFunctionalityDescriptors.push_back("XMLMSCReaderDiagnosisFunctionality");
            runFunctionalityDescriptors.push_back("XMLMAlarmBoxDiagnosisFunctionality");
            runFunctionalityDescriptors.push_back("XMLMCustomizableCheckListDiagnoseFunctionality");
            runFunctionalityDescriptors.push_back("XMLMCustomRemarkDiagnoseFunctionality");

            wrapper = new VotingMachineStatusWrapper ( runFunctionalityDescriptors );
        }

        void tearDown()
        {
            if ( wrapper )
            {
                delete wrapper;
                wrapper = NULL;
            }
        }

        void testResultEmptyInformation()
        {
            std::string result;

            result = wrapper->getResult ( ",", ":", "OK", "NOK" );

            CPPUNIT_ASSERT ( true == result.empty() );
        }

        void testAddDeviceDefaultValues()
        {
            wrapper->addDevice ( DeviceType::CONFIGURATION_ALARMBOX );

            DeviceStatus device = wrapper->getDeviceStatus ( DeviceType::CONFIGURATION_ALARMBOX );

            CPPUNIT_ASSERT ( DeviceType::CONFIGURATION_ALARMBOX == device.getVMType() );
            CPPUNIT_ASSERT ( true == device.getMandatory().present() );
            CPPUNIT_ASSERT ( false == device.getMandatory().get() );
            CPPUNIT_ASSERT ( true == device.getVMStatus().getStatus().present() );
            CPPUNIT_ASSERT ( VMStatus::VOTING_MACHINE_STATUS_ERROR == device.getVMStatus().getStatus().get() );
        }

        void testResultAddOneDevice()
        {
            std::string result;
            std::stringstream expected;

            expected << "A:NOK,";

            wrapper->addDevice ( DeviceType::CONFIGURATION_ALARMBOX );

            result = wrapper->getResult ( ",", ":", "OK", "NOK" );

            CPPUNIT_ASSERT ( false == result.empty() );
            CPPUNIT_ASSERT ( true == ( result.compare ( expected.str() ) == 0 ) );
        }

        void testResultAddSameDevice()
        {
            std::string result;
            std::stringstream expected;

            expected << "A:NOK,";

            wrapper->addDevice ( DeviceType::CONFIGURATION_ALARMBOX );
            wrapper->addDevice ( DeviceType::CONFIGURATION_ALARMBOX );

            result = wrapper->getResult ( ",", ":", "OK", "NOK" );

            CPPUNIT_ASSERT ( false == result.empty() );
            CPPUNIT_ASSERT ( true == ( result.compare ( expected.str() ) == 0 ) );
        }

        void testResultAddTwoDevice()
        {
            std::string result;
            std::stringstream expected;

            expected << "B:NOK,A:NOK,";

            wrapper->addDevice ( DeviceType::CONFIGURATION_ALARMBOX );
            wrapper->addDevice ( DeviceType::CONFIGURATION_BARCODE );

            result = wrapper->getResult ( ",", ":", "OK", "NOK" );

            CPPUNIT_ASSERT ( false == result.empty() );
            CPPUNIT_ASSERT ( true == ( result.compare ( expected.str() ) == 0 ) );
        }

        void testResultAddThreeDevice()
        {
            std::string result;
            std::stringstream expected;

            expected << "B:NOK,S:NOK,A:NOK,";

            wrapper->addDevice ( DeviceType::CONFIGURATION_ALARMBOX );
            wrapper->addDevice ( DeviceType::CONFIGURATION_BARCODE );
            wrapper->addDevice ( DeviceType::CONFIGURATION_SMARTCARD );

            result = wrapper->getResult ( ",", ":", "OK", "NOK" );

            CPPUNIT_ASSERT ( false == result.empty() );
            CPPUNIT_ASSERT ( true == ( result.compare ( expected.str() ) == 0 ) );
        }

        void testResultAddSerialDetectedDevice()
        {
            std::string result;
            std::stringstream expected;

            expected << "SND:ABCD123456789,";

            wrapper->setSerialDetected ( "ABCD123456789" );
            result = wrapper->getResult ( ",", ":" );

            CPPUNIT_ASSERT ( false == result.empty() );
            CPPUNIT_ASSERT ( true == ( result.compare ( expected.str() ) == 0 ) );
        }

        void testResultAddManualSerialNumberdDevice()
        {
            std::string result;
            std::stringstream expected;

            expected << "SNM:ABCD123456789,";

            wrapper->setSerial ( "ABCD123456789" );

            result = wrapper->getResult ( ",", ":" );

            CPPUNIT_ASSERT ( false == result.empty() );
            CPPUNIT_ASSERT ( true == ( result.compare ( expected.str() ) == 0 ) );
        }

        void testResultAddBothSerialNumberdDevice()
        {
            std::string result;
            std::stringstream expected;

            expected << "SND:ABCD987654321,SNM:ABCD123456789,";

            wrapper->setSerialDetected ( "ABCD987654321" );
            wrapper->setSerial ( "ABCD123456789" );

            result = wrapper->getResult ( ",", ":" );

            CPPUNIT_ASSERT ( false == result.empty() );
            CPPUNIT_ASSERT ( true == ( result.compare ( expected.str() ) == 0 ) );
        }

        void testResultAddDateDiagnose()
        {
            std::string result;
            std::stringstream expected;

            expected << "D:01/01/2013,";

            wrapper->setDate ( "01/01/2013" );

            result = wrapper->getResult ( ",", ":" );

            CPPUNIT_ASSERT ( false == result.empty() );
            CPPUNIT_ASSERT ( true == ( result.compare ( expected.str() ) == 0 ) );
        }

        void testResultAddZipCodeDiagnose()
        {
            std::string result;
            std::stringstream expected;

            expected << "Z:1234,";

            wrapper->setZipCode ( "1234" );

            result = wrapper->getResult ( ",", ":" );

            CPPUNIT_ASSERT ( false == result.empty() );
            CPPUNIT_ASSERT ( true == ( result.compare ( expected.str() ) == 0 ) );
        }

        void testResultAddCustomRemarkDiagnose()
        {
            std::string result;
            std::stringstream expected;

            expected << "R:Hola,";

            wrapper->setObservation ( "Hola", 10 );

            result = wrapper->getResult ( ",", ":" );

            CPPUNIT_ASSERT ( false == result.empty() );
            CPPUNIT_ASSERT ( true == ( result.compare ( expected.str() ) == 0 ) );
        }

        void testResultWithCustomRemarkEmpty()
        {
            std::string result;

            wrapper->setObservation ( "", 10 );

            result = wrapper->getResult ( ",", ":" );

            CPPUNIT_ASSERT ( true == result.empty() );
        }

        void testResultWithCustomRemarkNotTooManyLines()
        {
            std::string result;
            std::stringstream expected;

            expected << "1\n2\n3\n4\n5\n6\n7\n8\n9\n10";

            wrapper->setObservation ( "1\n2\n3\n4\n5\n6\n7\n8\n9\n10", 10 );

            result = wrapper->getObservation();

            CPPUNIT_ASSERT ( false == result.empty() );
            CPPUNIT_ASSERT ( true == ( result.compare ( expected.str() ) == 0 ) );
        }

        void testResultWithCustomRemarkTooManyLines()
        {
            std::string result;
            std::stringstream expected;

            expected << "1 2 3 4 5 6 7 8 9 10 11";

            wrapper->setObservation ( "1\n2\n3\n4\n5\n6\n7\n8\n9\n10\n11", 10 );

            result = wrapper->getObservation();

            CPPUNIT_ASSERT ( false == result.empty() );
            CPPUNIT_ASSERT ( true == ( result.compare ( expected.str() ) == 0 ) );
        }

        void testResultAddCustomCheckListDiagnose()
        {
            std::string result;
            std::stringstream expected;

            expected << "C:1234:1,";

            wrapper->addCustomCheckList ( "1234", true );

            result = wrapper->getResult ( ",", ":" );

            CPPUNIT_ASSERT ( false == result.empty() );
            CPPUNIT_ASSERT ( true == ( result.compare ( expected.str() ) == 0 ) );
        }

        void testResultAddTwoCustomCheckListDiagnose()
        {
            std::string result;
            std::stringstream expected;

            expected << "C:1234:1,C:4321:0,";

            wrapper->addCustomCheckList ( "1234", true );
            wrapper->addCustomCheckList ( "4321", false );

            result = wrapper->getResult ( ",", ":" );

            CPPUNIT_ASSERT ( false == result.empty() );
            CPPUNIT_ASSERT ( true == ( result.compare ( expected.str() ) == 0 ) );
        }

        void testResultChangeStatusDiagnose()
        {
            std::string result;
            std::stringstream expected;

            expected << "A:OK,";

            wrapper->changeStatus ( DeviceType::CONFIGURATION_ALARMBOX, VMStatus::VOTING_MACHINE_STATUS_OK );

            result = wrapper->getResult ( ",", ":", "OK", "NOK" );

            CPPUNIT_ASSERT ( false == result.empty() );
            CPPUNIT_ASSERT ( true == ( result.compare ( expected.str() ) == 0 ) );
        }

        void testResultOtherChangeStatusDiagnose()
        {
            std::string result;
            std::stringstream expected;

            expected << "B:NOK,A:OK,";

            wrapper->addDevice ( DeviceType::CONFIGURATION_BARCODE );
            wrapper->changeStatus ( DeviceType::CONFIGURATION_ALARMBOX, VMStatus::VOTING_MACHINE_STATUS_OK );

            result = wrapper->getResult ( ",", ":", "OK", "NOK" );

            CPPUNIT_ASSERT ( false == result.empty() );
            CPPUNIT_ASSERT ( true == ( result.compare ( expected.str() ) == 0 ) );
        }

        void testResultAddToAlamboxSmallDiagnoseStep()
        {
            std::string result;
            std::stringstream expected;

            expected << "A:OK-BT,";

            wrapper->addStatusCode ( DeviceType::CONFIGURATION_ALARMBOX, DiagnoseCode ( DiagnoseCode::VOTING_MACHINE_STATUS_BT ) );

            result = wrapper->getResult ( ",", ":", "OK", "NOK" );

            CPPUNIT_ASSERT ( false == result.empty() );
            CPPUNIT_ASSERT ( true == ( result.compare ( expected.str() ) == 0 ) );
        }

        void testResultAddAllToAlamboxSmallDiagnoseStepDeviceNotFound()
        {
            std::string result;
            std::stringstream expected;

            expected << "A:NOK-ND,";

            wrapper->setAlarmBoxFullStatus ( false );

            result = wrapper->getResult ( ",", ":", "OK", "NOK" );

            CPPUNIT_ASSERT ( false == result.empty() );
            CPPUNIT_ASSERT ( true == ( result.compare ( expected.str() ) == 0 ) );
        }

        void testResultAddAllToAlamboxSmallDiagnoseStepFail()
        {
            std::string result;
            std::stringstream expected;

            expected << "A:NOK-BZ-RL-GL-BT,";

            wrapper->setAlarmBoxFullStatus ( true, false, false, false, false );

            result = wrapper->getResult ( ",", ":", "OK", "NOK" );

            CPPUNIT_ASSERT ( false == result.empty() );
            CPPUNIT_ASSERT ( true == ( result.compare ( expected.str() ) == 0 ) );
        }

        void testResultAddAllToAlamboxSmallDiagnoseStepSuccess()
        {
            std::string result;
            std::stringstream expected;

            expected << "A:OK,";

            wrapper->setAlarmBoxFullStatus ( true, true, true, true, true );

            result = wrapper->getResult ( ",", ":", "OK", "NOK" );

            CPPUNIT_ASSERT ( false == result.empty() );
            CPPUNIT_ASSERT ( true == ( result.compare ( expected.str() ) == 0 ) );
        }

        void testResultAddAllToBarcodeReaderSmallDiagnoseStepDeviceNotFound()
        {
            std::string result;
            std::stringstream expected;

            expected << "B:NOK-ND,";

            wrapper->setBarcodeReaderFullStatus ( false );

            result = wrapper->getResult ( ",", ":", "OK", "NOK" );

            CPPUNIT_ASSERT ( false == result.empty() );
            CPPUNIT_ASSERT ( true == ( result.compare ( expected.str() ) == 0 ) );
        }

        void testResultAddAllToBarcodeReaderSmallDiagnoseStepConfigureError()
        {
            std::string result;
            std::stringstream expected;

            expected << "B:NOK-CE,";

            wrapper->setBarcodeReaderFullStatus ( true, false );

            result = wrapper->getResult ( ",", ":", "OK", "NOK" );

            CPPUNIT_ASSERT ( false == result.empty() );
            CPPUNIT_ASSERT ( true == ( result.compare ( expected.str() ) == 0 ) );
        }

        void testResultAddAllToBarcodeReaderSmallDiagnoseStepOnlyFailScan()
        {
            std::string result;
            std::stringstream expected;

            expected << "B:NOK-FS,";

            wrapper->setBarcodeReaderFullStatus ( true, true, false, false );

            result = wrapper->getResult ( ",", ":", "OK", "NOK" );

            CPPUNIT_ASSERT ( false == result.empty() );
            CPPUNIT_ASSERT ( true == ( result.compare ( expected.str() ) == 0 ) );
        }

        void testResultAddAllToBarcodeReaderSmallDiagnoseStepOnlyFailQR()
        {
            std::string result;
            std::stringstream expected;

            expected << "B:NOK-QR,";

            wrapper->setBarcodeReaderFullStatus ( true, true, true, false );

            result = wrapper->getResult ( ",", ":", "OK", "NOK" );

            CPPUNIT_ASSERT ( false == result.empty() );
            CPPUNIT_ASSERT ( true == ( result.compare ( expected.str() ) == 0 ) );
        }

        void testResultAddAllToBarcodeReaderSmallDiagnoseStepSuccess()
        {
            std::string result;
            std::stringstream expected;

            expected << "B:OK,";

            wrapper->setBarcodeReaderFullStatus ( true, true, true, true );

            result = wrapper->getResult ( ",", ":", "OK", "NOK" );

            CPPUNIT_ASSERT ( false == result.empty() );
            CPPUNIT_ASSERT ( true == ( result.compare ( expected.str() ) == 0 ) );
        }

        void testResultSmartcardReaderNotDetected()
        {
            std::string result;
            std::stringstream expected;

            expected << "S:NOK-ND,";

            wrapper->setStatusSmartcardReaderNotDetected ();

            result = wrapper->getResult ( ",", ":", "OK", "NOK" );

            CPPUNIT_ASSERT ( false == result.empty() );
            CPPUNIT_ASSERT ( true == ( result.compare ( expected.str() ) == 0 ) );
        }

        void testResultSmartcardReaderWriteError()
        {
            std::string result;
            std::stringstream expected;

            expected << "S:NOK-WE,";

            wrapper->setStatusSmartcardReaderWriteError ();

            result = wrapper->getResult ( ",", ":", "OK", "NOK" );

            CPPUNIT_ASSERT ( false == result.empty() );
            CPPUNIT_ASSERT ( true == ( result.compare ( expected.str() ) == 0 ) );
        }

        void testResultSmartcardReaderReadError()
        {
            std::string result;
            std::stringstream expected;

            expected << "S:NOK-RE,";

            wrapper->setStatusSmartcardReaderReadError ();

            result = wrapper->getResult ( ",", ":", "OK", "NOK" );

            CPPUNIT_ASSERT ( false == result.empty() );
            CPPUNIT_ASSERT ( true == ( result.compare ( expected.str() ) == 0 ) );
        }

        void testResultSmartcardReaderInsertionError()
        {
            std::string result;
            std::stringstream expected;

            expected << "S:NOK-IE,";

            wrapper->setStatusSmartcardReaderInsertionError ();

            result = wrapper->getResult ( ",", ":", "OK", "NOK" );

            CPPUNIT_ASSERT ( false == result.empty() );
            CPPUNIT_ASSERT ( true == ( result.compare ( expected.str() ) == 0 ) );
        }

        void testResultSmartcardReaderExtractionError()
        {
            std::string result;
            std::stringstream expected;

            expected << "S:NOK-EE,";

            wrapper->setStatusSmartcardReaderExtractionError ();

            result = wrapper->getResult ( ",", ":", "OK", "NOK" );

            CPPUNIT_ASSERT ( false == result.empty() );
            CPPUNIT_ASSERT ( true == ( result.compare ( expected.str() ) == 0 ) );
        }

        void testResultSmartcardReaderPowerCycleOffError()
        {
            std::string result;
            std::stringstream expected;

            expected << "S:NOK-PC0,";

            wrapper->setStatusSmartcardReaderPowerCycleOffError ();

            result = wrapper->getResult ( ",", ":", "OK", "NOK" );

            CPPUNIT_ASSERT ( false == result.empty() );
            CPPUNIT_ASSERT ( true == ( result.compare ( expected.str() ) == 0 ) );
        }

        void testResultSmartcardReaderPowerCycleOnError()
        {
            std::string result;
            std::stringstream expected;

            expected << "S:NOK-PC1,";

            wrapper->setStatusSmartcardReaderPowerCycleOnError ();

            result = wrapper->getResult ( ",", ":", "OK", "NOK" );

            CPPUNIT_ASSERT ( false == result.empty() );
            CPPUNIT_ASSERT ( true == ( result.compare ( expected.str() ) == 0 ) );
        }

        void testResultPrinterNotDetected()
        {
            std::string result;
            std::stringstream expected;

            expected << "P:NOK-ND,";

            wrapper->setStatusPrinterNotDetected ();

            result = wrapper->getResult ( ",", ":", "OK", "NOK" );

            CPPUNIT_ASSERT ( false == result.empty() );
            CPPUNIT_ASSERT ( true == ( result.compare ( expected.str() ) == 0 ) );
        }

        void testResultPrinterNotDetectedDualCall()
        {
            std::string result;
            std::stringstream expected;

            expected << "P:NOK-ND,";

            wrapper->setStatusPrinterNotDetected ();
            wrapper->setStatusPrinterNotDetected ();

            result = wrapper->getResult ( ",", ":", "OK", "NOK" );

            CPPUNIT_ASSERT ( false == result.empty() );
            CPPUNIT_ASSERT ( true == ( result.compare ( expected.str() ) == 0 ) );
        }

        void testResultPrinterPaperJam()
        {
            std::string result;
            std::stringstream expected;

            expected << "P:NOK-PJ,";

            wrapper->addStatusPrinterPaperJam ();

            result = wrapper->getResult ( ",", ":", "OK", "NOK" );

            CPPUNIT_ASSERT ( false == result.empty() );
            CPPUNIT_ASSERT ( true == ( result.compare ( expected.str() ) == 0 ) );
        }

        void testResultPrinterPaperJamDualCall()
        {
            std::string result;
            std::stringstream expected;

            expected << "P:NOK-PJ,";

            wrapper->addStatusPrinterPaperJam ();
            wrapper->addStatusPrinterPaperJam ();

            result = wrapper->getResult ( ",", ":", "OK", "NOK" );

            CPPUNIT_ASSERT ( false == result.empty() );
            CPPUNIT_ASSERT ( true == ( result.compare ( expected.str() ) == 0 ) );
        }

        void testResultPrinterNoPaper()
        {
            std::string result;
            std::stringstream expected;

            expected << "P:NOK-NP,";

            wrapper->addStatusPrinterNoPaper ();

            result = wrapper->getResult ( ",", ":", "OK", "NOK" );

            CPPUNIT_ASSERT ( false == result.empty() );
            CPPUNIT_ASSERT ( true == ( result.compare ( expected.str() ) == 0 ) );
        }

        void testResultPrinterNoPaperDualCall()
        {
            std::string result;
            std::stringstream expected;

            expected << "P:NOK-NP,";

            wrapper->addStatusPrinterNoPaper ();
            wrapper->addStatusPrinterNoPaper ();

            result = wrapper->getResult ( ",", ":", "OK", "NOK" );

            CPPUNIT_ASSERT ( false == result.empty() );
            CPPUNIT_ASSERT ( true == ( result.compare ( expected.str() ) == 0 ) );
        }

        void testResultPrinterCoverOpen()
        {
            std::string result;
            std::stringstream expected;

            expected << "P:NOK-CO,";

            wrapper->addStatusPrinterCoverOpen ();

            result = wrapper->getResult ( ",", ":", "OK", "NOK" );

            CPPUNIT_ASSERT ( false == result.empty() );
            CPPUNIT_ASSERT ( true == ( result.compare ( expected.str() ) == 0 ) );
        }

        void testResultPrinterCoverOpenDualCall()
        {
            std::string result;
            std::stringstream expected;

            expected << "P:NOK-CO,";

            wrapper->addStatusPrinterCoverOpen ();
            wrapper->addStatusPrinterCoverOpen ();

            result = wrapper->getResult ( ",", ":", "OK", "NOK" );

            CPPUNIT_ASSERT ( false == result.empty() );
            CPPUNIT_ASSERT ( true == ( result.compare ( expected.str() ) == 0 ) );
        }

        void testResultPrinterFeederOpen()
        {
            std::string result;
            std::stringstream expected;

            expected << "P:NOK-FO,";

            wrapper->addStatusPrinterFeederOpen ();

            result = wrapper->getResult ( ",", ":", "OK", "NOK" );

            CPPUNIT_ASSERT ( false == result.empty() );
            CPPUNIT_ASSERT ( true == ( result.compare ( expected.str() ) == 0 ) );
        }

        void testResultPrinterFeederOpenDualCall()
        {
            std::string result;
            std::stringstream expected;

            expected << "P:NOK-FO,";

            wrapper->addStatusPrinterFeederOpen ();
            wrapper->addStatusPrinterFeederOpen ();

            result = wrapper->getResult ( ",", ":", "OK", "NOK" );

            CPPUNIT_ASSERT ( false == result.empty() );
            CPPUNIT_ASSERT ( true == ( result.compare ( expected.str() ) == 0 ) );
        }

        void testResultPrinterNearEndOfPaper()
        {
            std::string result;
            std::stringstream expected;

            expected << "P:NOK-NE,";

            wrapper->addStatusPrinterNearEndOfPaper ();

            result = wrapper->getResult ( ",", ":", "OK", "NOK" );

            CPPUNIT_ASSERT ( false == result.empty() );
            CPPUNIT_ASSERT ( true == ( result.compare ( expected.str() ) == 0 ) );
        }

        void testResultPrinterNearEndOfPaperDualCall()
        {
            std::string result;
            std::stringstream expected;

            expected << "P:NOK-NE,";

            wrapper->addStatusPrinterNearEndOfPaper ();
            wrapper->addStatusPrinterNearEndOfPaper ();

            result = wrapper->getResult ( ",", ":", "OK", "NOK" );

            CPPUNIT_ASSERT ( false == result.empty() );
            CPPUNIT_ASSERT ( true == ( result.compare ( expected.str() ) == 0 ) );
        }

        void testResultPrinterSupplyLow()
        {
            std::string result;
            std::stringstream expected;

            expected << "P:NOK-SL,";

            wrapper->addStatusPrinterSupplyLowWarn ();

            result = wrapper->getResult ( ",", ":", "OK", "NOK" );

            CPPUNIT_ASSERT ( false == result.empty() );
            CPPUNIT_ASSERT ( true == ( result.compare ( expected.str() ) == 0 ) );
        }

        void testResultPrinterSupplyLowDualCall()
        {
            std::string result;
            std::stringstream expected;

            expected << "P:NOK-SL,";

            wrapper->addStatusPrinterSupplyLowWarn ();
            wrapper->addStatusPrinterSupplyLowWarn ();

            result = wrapper->getResult ( ",", ":", "OK", "NOK" );

            CPPUNIT_ASSERT ( false == result.empty() );
            CPPUNIT_ASSERT ( true == ( result.compare ( expected.str() ) == 0 ) );
        }

        void testResultPrinterNoToner()
        {
            std::string result;
            std::stringstream expected;

            expected << "P:NOK-NT,";

            wrapper->addStatusPrinterNoToner ();

            result = wrapper->getResult ( ",", ":", "OK", "NOK" );

            CPPUNIT_ASSERT ( false == result.empty() );
            CPPUNIT_ASSERT ( true == ( result.compare ( expected.str() ) == 0 ) );
        }

        void testResultPrinterNoTonerDualCall()
        {
            std::string result;
            std::stringstream expected;

            expected << "P:NOK-NT,";

            wrapper->addStatusPrinterNoToner ();
            wrapper->addStatusPrinterNoToner ();

            result = wrapper->getResult ( ",", ":", "OK", "NOK" );

            CPPUNIT_ASSERT ( false == result.empty() );
            CPPUNIT_ASSERT ( true == ( result.compare ( expected.str() ) == 0 ) );
        }

        void testResultPrinterNoCartridge()
        {
            std::string result;
            std::stringstream expected;

            expected << "P:NOK-NC,";

            wrapper->addStatusPrinterNoCartridge ();

            result = wrapper->getResult ( ",", ":", "OK", "NOK" );

            CPPUNIT_ASSERT ( false == result.empty() );
            CPPUNIT_ASSERT ( true == ( result.compare ( expected.str() ) == 0 ) );
        }

        void testResultPrinterNoCartridgeDualCall()
        {
            std::string result;
            std::stringstream expected;

            expected << "P:NOK-NC,";

            wrapper->addStatusPrinterNoCartridge ();
            wrapper->addStatusPrinterNoCartridge ();

            result = wrapper->getResult ( ",", ":", "OK", "NOK" );

            CPPUNIT_ASSERT ( false == result.empty() );
            CPPUNIT_ASSERT ( true == ( result.compare ( expected.str() ) == 0 ) );
        }

        void testResultPrinterPrintingCanceled()
        {
            std::string result;
            std::stringstream expected;

            expected << "P:NOK-PC,";

            wrapper->addStatusPrintingCanceled();

            result = wrapper->getResult ( ",", ":", "OK", "NOK" );

            CPPUNIT_ASSERT ( false == result.empty() );
            CPPUNIT_ASSERT ( true == ( result.compare ( expected.str() ) == 0 ) );
        }

        void testResultPrinterPrintingCanceledDualCall()
        {
            std::string result;
            std::stringstream expected;

            expected << "P:NOK-PC,";

            wrapper->addStatusPrintingCanceled ();
            wrapper->addStatusPrintingCanceled ();

            result = wrapper->getResult ( ",", ":", "OK", "NOK" );

            CPPUNIT_ASSERT ( false == result.empty() );
            CPPUNIT_ASSERT ( true == ( result.compare ( expected.str() ) == 0 ) );
        }

        void testResultPrinterPrintingError()
        {
            std::string result;
            std::stringstream expected;

            expected << "P:NOK-PE,";

            wrapper->addStatusPrintingError ();

            result = wrapper->getResult ( ",", ":", "OK", "NOK" );

            CPPUNIT_ASSERT ( false == result.empty() );
            CPPUNIT_ASSERT ( true == ( result.compare ( expected.str() ) == 0 ) );
        }

        void testResultPrinterPrintingErrorDualCall()
        {
            std::string result;
            std::stringstream expected;

            expected << "P:NOK-PE,";

            wrapper->addStatusPrintingError ();
            wrapper->addStatusPrintingError ();

            result = wrapper->getResult ( ",", ":", "OK", "NOK" );

            CPPUNIT_ASSERT ( false == result.empty() );
            CPPUNIT_ASSERT ( true == ( result.compare ( expected.str() ) == 0 ) );
        }

        void testResultPrinterManualError()
        {
            std::string result;
            std::stringstream expected;

            expected << "P:NOK-ME,";

            wrapper->setStatusPrinterManualError ();

            result = wrapper->getResult ( ",", ":", "OK", "NOK" );

            CPPUNIT_ASSERT ( false == result.empty() );
            CPPUNIT_ASSERT ( true == ( result.compare ( expected.str() ) == 0 ) );
        }

        void testResultPrinterManualErrorDualCall()
        {
            std::string result;
            std::stringstream expected;

            expected << "P:NOK-ME,";

            wrapper->setStatusPrinterManualError ();
            wrapper->setStatusPrinterManualError ();

            result = wrapper->getResult ( ",", ":", "OK", "NOK" );

            CPPUNIT_ASSERT ( false == result.empty() );
            CPPUNIT_ASSERT ( true == ( result.compare ( expected.str() ) == 0 ) );
        }

        void testResultPrinterNearEndOfPaperAndFeederOpen()
        {
            std::string result;
            std::stringstream expected;

            expected << "P:NOK-NE-FO,";

            wrapper->addStatusPrinterNearEndOfPaper ();
            wrapper->addStatusPrinterFeederOpen ();

            result = wrapper->getResult ( ",", ":", "OK", "NOK" );

            CPPUNIT_ASSERT ( false == result.empty() );
            CPPUNIT_ASSERT ( true == ( result.compare ( expected.str() ) == 0 ) );
        }

        void testResultPrinterHPAllError()
        {
            std::string result;
            std::stringstream expected;

            expected << "P:NOK-NP-PJ-CO-SL-NC-NT-PC-PE,";

            wrapper->addStatusPrinterNoPaper();
            wrapper->addStatusPrinterPaperJam ();
            wrapper->addStatusPrinterCoverOpen ();
            wrapper->addStatusPrinterSupplyLowWarn();
            wrapper->addStatusPrinterNoCartridge();
            wrapper->addStatusPrinterNoToner();
            wrapper->addStatusPrintingCanceled();
            wrapper->addStatusPrintingError();

            result = wrapper->getResult ( ",", ":", "OK", "NOK" );

            CPPUNIT_ASSERT ( false == result.empty() );
            CPPUNIT_ASSERT ( true == ( result.compare ( expected.str() ) == 0 ) );
        }

        void testResultPrinterAPSAllError()
        {
            std::string result;
            std::stringstream expected;

            expected << "P:NOK-NP-FO-NE-PE,";

            wrapper->addStatusPrinterNoPaper();
            wrapper->addStatusPrinterFeederOpen ();
            wrapper->addStatusPrinterNearEndOfPaper ();
            wrapper->addStatusPrintingError();

            result = wrapper->getResult ( ",", ":", "OK", "NOK" );

            CPPUNIT_ASSERT ( false == result.empty() );
            CPPUNIT_ASSERT ( true == ( result.compare ( expected.str() ) == 0 ) );
        }

        void testResultFullDiagnose()
        {
            std::string result;
            std::stringstream expected;

            expected << "SND:ABCD1234,";
            expected << "SNM:ABCD4321,";
            expected << "Z:1234,";
            expected << "D:01/01/2013,";
            expected << "P:OK,";
            expected << "B:OK,";
            expected << "S:NOK,";
            expected << "A:NOK,";
            expected << "R:Hello,";

            wrapper->setSerialDetected ( "ABCD1234" );
            wrapper->setSerial ( "ABCD4321" );
            wrapper->setDate ( "01/01/2013" );
            wrapper->setZipCode ( "1234" );
            wrapper->changeStatus ( DeviceType::CONFIGURATION_ALARMBOX, VMStatus::VOTING_MACHINE_STATUS_ERROR );
            wrapper->changeStatus ( DeviceType::CONFIGURATION_BARCODE, VMStatus::VOTING_MACHINE_STATUS_OK );
            wrapper->changeStatus ( DeviceType::CONFIGURATION_SMARTCARD, VMStatus::VOTING_MACHINE_STATUS_ERROR );
            wrapper->changeStatus ( DeviceType::CONFIGURATION_PRINTER, VMStatus::VOTING_MACHINE_STATUS_OK );
            wrapper->setObservation ( "Hello", 10 );

            result = wrapper->getResult ( ",", ":", "OK", "NOK" );

            CPPUNIT_ASSERT ( false == result.empty() );
            CPPUNIT_ASSERT ( true == ( result.compare ( expected.str() ) == 0 ) );
        }

        void testSerial()
        {
            std::string result;

            result = wrapper->getSerial();

            CPPUNIT_ASSERT ( true == result.empty() );

            std::string expected = "serial";

            wrapper->setSerial ( expected );
            result = wrapper->getSerial();

            CPPUNIT_ASSERT ( expected == result );
        }

        void testSerialDetected()
        {
            std::string result;

            result = wrapper->getSerialDetected();

            CPPUNIT_ASSERT ( true == result.empty() );

            std::string expected = "serial";

            wrapper->setSerialDetected ( expected );
            result = wrapper->getSerialDetected();

            CPPUNIT_ASSERT ( expected == result );
        }

        void testDate()
        {
            std::string result;

            result = wrapper->getDate();

            CPPUNIT_ASSERT ( true == result.empty() );

            std::string expected = "date";

            wrapper->setDate ( expected );
            result = wrapper->getDate();

            CPPUNIT_ASSERT ( expected == result );
        }

        void testZipCode()
        {
            std::string result;

            result = wrapper->getZipCode();

            CPPUNIT_ASSERT ( true == result.empty() );

            std::string expected = "zip";

            wrapper->setZipCode ( expected );
            result = wrapper->getZipCode();

            CPPUNIT_ASSERT ( expected == result );
        }

        void testObservation()
        {
            std::string result;

            result = wrapper->getObservation();

            CPPUNIT_ASSERT ( true == result.empty() );

            std::string expected = "observation";

            wrapper->setObservation ( expected , 20 );
            result = wrapper->getObservation();

            CPPUNIT_ASSERT ( expected == result );
        }

        void testResultAddAllToAlamboxSmallDiagnoseStepDeviceNotFoundDualCall()
        {
            std::string result;
            std::stringstream expected;

            expected << "A:NOK-ND,";

            wrapper->setAlarmBoxFullStatus ( false );
            wrapper->setAlarmBoxFullStatus ( false );

            result = wrapper->getResult ( ",", ":", "OK", "NOK" );

            CPPUNIT_ASSERT ( false == result.empty() );
            CPPUNIT_ASSERT ( true == ( result.compare ( expected.str() ) == 0 ) );
        }

        void testResultAddAllToAlamboxSmallDiagnoseStepFailDualCall()
        {
            std::string result;
            std::stringstream expected;

            expected << "A:NOK-BZ-RL-GL-BT,";

            wrapper->setAlarmBoxFullStatus ( true, false, false, false, false );
            wrapper->setAlarmBoxFullStatus ( true, false, false, false, false );

            result = wrapper->getResult ( ",", ":", "OK", "NOK" );

            CPPUNIT_ASSERT ( false == result.empty() );
            CPPUNIT_ASSERT ( true == ( result.compare ( expected.str() ) == 0 ) );
        }

        void testResultSmartcardReaderNotDetectedDualCall()
        {
            std::string result;
            std::stringstream expected;

            expected << "S:NOK-ND,";

            wrapper->setStatusSmartcardReaderNotDetected ();
            wrapper->setStatusSmartcardReaderNotDetected ();

            result = wrapper->getResult ( ",", ":", "OK", "NOK" );

            CPPUNIT_ASSERT ( false == result.empty() );
            CPPUNIT_ASSERT ( true == ( result.compare ( expected.str() ) == 0 ) );
        }

        void testResultSmartcardReaderWriteErrorDualCall()
        {
            std::string result;
            std::stringstream expected;

            expected << "S:NOK-WE,";

            wrapper->setStatusSmartcardReaderWriteError ();
            wrapper->setStatusSmartcardReaderWriteError ();

            result = wrapper->getResult ( ",", ":", "OK", "NOK" );

            CPPUNIT_ASSERT ( false == result.empty() );
            CPPUNIT_ASSERT ( true == ( result.compare ( expected.str() ) == 0 ) );
        }

        void testResultSmartcardReaderReadErrorDualCall()
        {
            std::string result;
            std::stringstream expected;

            expected << "S:NOK-RE,";

            wrapper->setStatusSmartcardReaderReadError ();
            wrapper->setStatusSmartcardReaderReadError ();

            result = wrapper->getResult ( ",", ":", "OK", "NOK" );

            CPPUNIT_ASSERT ( false == result.empty() );
            CPPUNIT_ASSERT ( true == ( result.compare ( expected.str() ) == 0 ) );
        }

        void testResultSmartcardReaderInsertionErrorDualCall()
        {
            std::string result;
            std::stringstream expected;

            expected << "S:NOK-IE,";

            wrapper->setStatusSmartcardReaderInsertionError ();
            wrapper->setStatusSmartcardReaderInsertionError ();

            result = wrapper->getResult ( ",", ":", "OK", "NOK" );

            CPPUNIT_ASSERT ( false == result.empty() );
            CPPUNIT_ASSERT ( true == ( result.compare ( expected.str() ) == 0 ) );
        }

        void testResultSmartcardReaderExtractionErrorDualCall()
        {
            std::string result;
            std::stringstream expected;

            expected << "S:NOK-EE,";

            wrapper->setStatusSmartcardReaderExtractionError ();
            wrapper->setStatusSmartcardReaderExtractionError ();

            result = wrapper->getResult ( ",", ":", "OK", "NOK" );

            CPPUNIT_ASSERT ( false == result.empty() );
            CPPUNIT_ASSERT ( true == ( result.compare ( expected.str() ) == 0 ) );
        }

        void testResultSmartcardReaderPowerCycleOffErrorDualCall()
        {
            std::string result;
            std::stringstream expected;

            expected << "S:NOK-PC0,";

            wrapper->setStatusSmartcardReaderPowerCycleOffError ();
            wrapper->setStatusSmartcardReaderPowerCycleOffError ();

            result = wrapper->getResult ( ",", ":", "OK", "NOK" );

            CPPUNIT_ASSERT ( false == result.empty() );
            CPPUNIT_ASSERT ( true == ( result.compare ( expected.str() ) == 0 ) );
        }

        void testResultSmartcardReaderPowerCycleOnErrorDualCall()
        {
            std::string result;
            std::stringstream expected;

            expected << "S:NOK-PC1,";

            wrapper->setStatusSmartcardReaderPowerCycleOnError ();
            wrapper->setStatusSmartcardReaderPowerCycleOnError ();

            result = wrapper->getResult ( ",", ":", "OK", "NOK" );

            CPPUNIT_ASSERT ( false == result.empty() );
            CPPUNIT_ASSERT ( true == ( result.compare ( expected.str() ) == 0 ) );
        }

        void testAddStatusCodeOKToDiagnoseOK()
        {
            std::string result;
            std::stringstream expected;

            expected << "S:OK-D0001,";

            wrapper->changeStatus ( DeviceType::CONFIGURATION_SMARTCARD, VMStatus::VOTING_MACHINE_STATUS_OK );
            wrapper->addStatusCode ( DeviceType::CONFIGURATION_SMARTCARD, "D0001" );

            result = wrapper->getResult ( ",", ":", "OK", "NOK" );

            CPPUNIT_ASSERT ( false == result.empty() );
            CPPUNIT_ASSERT ( true == ( result.compare ( expected.str() ) == 0 ) );
        }

        void testAddStatusCodeNOKToDiagnoseOK()
        {
            std::string result;
            std::stringstream expected;

            expected << "S:NOK-D0001,";

            wrapper->changeStatus ( DeviceType::CONFIGURATION_SMARTCARD, VMStatus::VOTING_MACHINE_STATUS_OK );
            wrapper->addStatusCode ( DeviceType::CONFIGURATION_SMARTCARD, "D0001", VMStatus::VOTING_MACHINE_STATUS_ERROR );

            result = wrapper->getResult ( ",", ":", "OK", "NOK" );

            CPPUNIT_ASSERT ( false == result.empty() );
            CPPUNIT_ASSERT ( true == ( result.compare ( expected.str() ) == 0 ) );
        }

        void testAddStatusCodeOKToDiagnoseNOK()
        {
            std::string result;
            std::stringstream expected;

            expected << "S:NOK-EE-D0004,";

            wrapper->setStatusSmartcardReaderExtractionError();
            wrapper->addStatusCode ( DeviceType::CONFIGURATION_SMARTCARD, "D0004" );

            result = wrapper->getResult ( ",", ":", "OK", "NOK" );

            CPPUNIT_ASSERT ( false == result.empty() );
            CPPUNIT_ASSERT ( true == ( result.compare ( expected.str() ) == 0 ) );
        }

        void testAddStatusCodeNOKToDiagnoseNOK()
        {
            std::string result;
            std::stringstream expected;

            expected << "S:NOK-EE-D0008,";

            wrapper->setStatusSmartcardReaderExtractionError();
            wrapper->addStatusCode ( DeviceType::CONFIGURATION_SMARTCARD, "D0008", VMStatus::VOTING_MACHINE_STATUS_ERROR );

            result = wrapper->getResult ( ",", ":", "OK", "NOK" );

            CPPUNIT_ASSERT ( false == result.empty() );
            CPPUNIT_ASSERT ( true == ( result.compare ( expected.str() ) == 0 ) );
        }

        void testGetVersion()
        {
            std::string result;
            std::stringstream expected;

            expected << "V:" << VERSION << ",";

            result = wrapper->getVersion ( ",", ":" );

            CPPUNIT_ASSERT ( false == result.empty() );
            CPPUNIT_ASSERT ( true == ( result.compare ( expected.str() ) == 0 ) );
        }

        void testGetDeviceList()
        {
            wrapper->addDevice ( DeviceType::CONFIGURATION_ALARMBOX );
            wrapper->addDevice ( DeviceType::CONFIGURATION_BARCODE );

            std::list <DeviceStatus > devices = wrapper->getDeviceStatusList();
            std::list <DeviceStatus >::iterator it;

            CPPUNIT_ASSERT ( 2 == devices.size() );

            it = devices.begin();

            CPPUNIT_ASSERT ( DeviceType::CONFIGURATION_ALARMBOX == ( *it ).getVMType() );

            ++it;

            CPPUNIT_ASSERT ( DeviceType::CONFIGURATION_BARCODE == ( *it ).getVMType() );
        }

        void testGetDeviceListEmpty()
        {
            wrapper->addDevice ( DeviceType::CONFIGURATION_ALARMBOX );
            wrapper->addDevice ( DeviceType::CONFIGURATION_BARCODE );

            std::list <DeviceStatus > devices = wrapper->getDeviceStatusList();

            CPPUNIT_ASSERT ( 2 == devices.size() );

            wrapper->clearDeviceStatusList();

            devices = wrapper->getDeviceStatusList();

            CPPUNIT_ASSERT ( true == devices.empty() );
        }

        void testGetDeviceListChangeSort()
        {
            wrapper->addDevice ( DeviceType::CONFIGURATION_ALARMBOX );
            wrapper->addDevice ( DeviceType::CONFIGURATION_BARCODE );

            std::list <DeviceStatus > devices = wrapper->getDeviceStatusList();
            std::list <DeviceStatus >::iterator it;

            CPPUNIT_ASSERT ( 2 == devices.size() );

            it = devices.begin();

            CPPUNIT_ASSERT ( DeviceType::CONFIGURATION_ALARMBOX == ( *it ).getVMType() );

            ++it;

            CPPUNIT_ASSERT ( DeviceType::CONFIGURATION_BARCODE == ( *it ).getVMType() );

            sleep ( 1 );

            wrapper->setDiagnosisStatus ( DeviceType::CONFIGURATION_ALARMBOX, VMStatus::VOTING_MACHINE_STATUS_OK, true );
            devices = wrapper->getDeviceStatusList();

            CPPUNIT_ASSERT ( 2 == devices.size() );

            it = devices.begin();

            CPPUNIT_ASSERT ( DeviceType::CONFIGURATION_BARCODE == ( *it ).getVMType() );
            CPPUNIT_ASSERT ( false == ( *it ).getMandatory().get() );
            CPPUNIT_ASSERT ( VMStatus::VOTING_MACHINE_STATUS_ERROR == ( *it ).getVMStatus().getStatus().get() );

            ++it;

            CPPUNIT_ASSERT ( DeviceType::CONFIGURATION_ALARMBOX == ( *it ).getVMType() );
            CPPUNIT_ASSERT ( true == ( *it ).getMandatory().get() );
            CPPUNIT_ASSERT ( VMStatus::VOTING_MACHINE_STATUS_OK == ( *it ).getVMStatus().getStatus().get() );

        }

        CPPUNIT_TEST_SUITE ( VotingMachineStatusTest );

        CPPUNIT_TEST ( testSerial );
        CPPUNIT_TEST ( testSerialDetected );
        CPPUNIT_TEST ( testDate );
        CPPUNIT_TEST ( testZipCode );
        CPPUNIT_TEST ( testObservation );

        CPPUNIT_TEST ( testResultEmptyInformation );
        CPPUNIT_TEST ( testAddDeviceDefaultValues );

        CPPUNIT_TEST ( testResultAddOneDevice );
        CPPUNIT_TEST ( testResultAddSameDevice );
        CPPUNIT_TEST ( testResultAddTwoDevice );
        CPPUNIT_TEST ( testResultAddThreeDevice );

        CPPUNIT_TEST ( testResultAddSerialDetectedDevice );
        CPPUNIT_TEST ( testResultAddManualSerialNumberdDevice );
        CPPUNIT_TEST ( testResultAddBothSerialNumberdDevice );

        CPPUNIT_TEST ( testResultAddDateDiagnose );
        CPPUNIT_TEST ( testResultAddZipCodeDiagnose );
        CPPUNIT_TEST ( testResultAddCustomRemarkDiagnose );
        CPPUNIT_TEST ( testResultWithCustomRemarkEmpty );
        CPPUNIT_TEST ( testResultWithCustomRemarkNotTooManyLines );
        CPPUNIT_TEST ( testResultWithCustomRemarkTooManyLines );
        CPPUNIT_TEST ( testResultAddCustomCheckListDiagnose );
        CPPUNIT_TEST ( testResultAddTwoCustomCheckListDiagnose );

        CPPUNIT_TEST ( testResultChangeStatusDiagnose );
        CPPUNIT_TEST ( testResultOtherChangeStatusDiagnose );
        CPPUNIT_TEST ( testResultAddToAlamboxSmallDiagnoseStep );
        CPPUNIT_TEST ( testResultAddAllToAlamboxSmallDiagnoseStepDeviceNotFound );
        CPPUNIT_TEST ( testResultAddAllToAlamboxSmallDiagnoseStepFail );
        CPPUNIT_TEST ( testResultAddAllToAlamboxSmallDiagnoseStepSuccess );

        CPPUNIT_TEST ( testResultAddAllToBarcodeReaderSmallDiagnoseStepDeviceNotFound );
        CPPUNIT_TEST ( testResultAddAllToBarcodeReaderSmallDiagnoseStepConfigureError );
        CPPUNIT_TEST ( testResultAddAllToBarcodeReaderSmallDiagnoseStepOnlyFailScan );
        CPPUNIT_TEST ( testResultAddAllToBarcodeReaderSmallDiagnoseStepOnlyFailQR );
        CPPUNIT_TEST ( testResultAddAllToBarcodeReaderSmallDiagnoseStepSuccess );

        CPPUNIT_TEST ( testResultFullDiagnose );

        CPPUNIT_TEST ( testResultSmartcardReaderNotDetected );
        CPPUNIT_TEST ( testResultSmartcardReaderWriteError );
        CPPUNIT_TEST ( testResultSmartcardReaderReadError );
        CPPUNIT_TEST ( testResultSmartcardReaderInsertionError );
        CPPUNIT_TEST ( testResultSmartcardReaderExtractionError );
        CPPUNIT_TEST ( testResultSmartcardReaderPowerCycleOffError );
        CPPUNIT_TEST ( testResultSmartcardReaderPowerCycleOnError );

        CPPUNIT_TEST ( testResultAddAllToAlamboxSmallDiagnoseStepDeviceNotFoundDualCall );
        CPPUNIT_TEST ( testResultAddAllToAlamboxSmallDiagnoseStepFailDualCall );

        CPPUNIT_TEST ( testResultSmartcardReaderNotDetectedDualCall );
        CPPUNIT_TEST ( testResultSmartcardReaderWriteErrorDualCall );
        CPPUNIT_TEST ( testResultSmartcardReaderReadErrorDualCall );
        CPPUNIT_TEST ( testResultSmartcardReaderInsertionErrorDualCall );
        CPPUNIT_TEST ( testResultSmartcardReaderExtractionErrorDualCall );
        CPPUNIT_TEST ( testResultSmartcardReaderPowerCycleOffErrorDualCall );
        CPPUNIT_TEST ( testResultSmartcardReaderPowerCycleOnErrorDualCall );

        CPPUNIT_TEST ( testResultPrinterNotDetected );
        CPPUNIT_TEST ( testResultPrinterNotDetectedDualCall );
        CPPUNIT_TEST ( testResultPrinterPaperJam );
        CPPUNIT_TEST ( testResultPrinterPaperJamDualCall );
        CPPUNIT_TEST ( testResultPrinterNoPaper );
        CPPUNIT_TEST ( testResultPrinterNoPaperDualCall );
        CPPUNIT_TEST ( testResultPrinterCoverOpen );
        CPPUNIT_TEST ( testResultPrinterCoverOpenDualCall );
        CPPUNIT_TEST ( testResultPrinterFeederOpen );
        CPPUNIT_TEST ( testResultPrinterFeederOpenDualCall );
        CPPUNIT_TEST ( testResultPrinterNearEndOfPaper );
        CPPUNIT_TEST ( testResultPrinterNearEndOfPaperDualCall );
        CPPUNIT_TEST ( testResultPrinterSupplyLow );
        CPPUNIT_TEST ( testResultPrinterSupplyLowDualCall );
        CPPUNIT_TEST ( testResultPrinterNoToner );
        CPPUNIT_TEST ( testResultPrinterNoTonerDualCall );
        CPPUNIT_TEST ( testResultPrinterNoCartridge );
        CPPUNIT_TEST ( testResultPrinterNoCartridgeDualCall );
        CPPUNIT_TEST ( testResultPrinterPrintingCanceled );
        CPPUNIT_TEST ( testResultPrinterPrintingCanceledDualCall );
        CPPUNIT_TEST ( testResultPrinterPrintingError );
        CPPUNIT_TEST ( testResultPrinterPrintingErrorDualCall );
        CPPUNIT_TEST ( testResultPrinterManualError );
        CPPUNIT_TEST ( testResultPrinterManualErrorDualCall );

        CPPUNIT_TEST ( testResultPrinterNearEndOfPaperAndFeederOpen );
        CPPUNIT_TEST ( testResultPrinterHPAllError );
        CPPUNIT_TEST ( testResultPrinterAPSAllError );

        CPPUNIT_TEST ( testAddStatusCodeOKToDiagnoseOK );
        CPPUNIT_TEST ( testAddStatusCodeNOKToDiagnoseOK );
        CPPUNIT_TEST ( testAddStatusCodeOKToDiagnoseNOK );
        CPPUNIT_TEST ( testAddStatusCodeNOKToDiagnoseNOK );

        CPPUNIT_TEST ( testGetVersion );

        CPPUNIT_TEST ( testGetDeviceList );
        CPPUNIT_TEST ( testGetDeviceListEmpty );
        CPPUNIT_TEST ( testGetDeviceListChangeSort );

        CPPUNIT_TEST_SUITE_END();
};

#endif /* ECUT_MAIN */

#endif /* VOTING_MACHINE_STATUS_TEST_H_ */
