/**
 * @file ConnectionDriverInterface.hxx
 * @brief Declaration of the interface for a reader driver, which implements low-level operations of connection, sending commands and getting responses.
 * @author Yadickson Soto
 * @date 28/03/2014
 */

#ifndef _CONNECTION_DRIVER_INTERFACE_HXX_
#define _CONNECTION_DRIVER_INTERFACE_HXX_

#include <System/SafeBuffer.h>

namespace Smartmatic
{
    namespace SAES
    {
        namespace Card
        {

            /**
             * @class ConnectionDriverInterface
             * @brief Interface for a reader driver, which implements low-level operations of connection, sending commands and getting responses.
             * @author Yadickson Soto
             * @date 28/03/2014
             */
            class ConnectionDriverInterface
            {
                public:

                    /**
                     * @fn virtual ~ConnectionDriverInterface();
                     * @brief Class destructor.
                     * @author Yadickson Soto
                     * @date 28/03/2014
                     */
                    virtual ~ConnectionDriverInterface() {};

                    /**
                     * @fn virtual unsigned long establishContext() = 0;
                     * @brief Establishes a context for the connection driver.
                     * @author Yadickson Soto
                     * @date 28/03/2014
                     * @return result code, zero "0" if successful.
                     */
                    virtual unsigned long establishContext () = 0;

                    /**
                     * @fn virtual unsigned long releaseContext() = 0;
                     * @brief Releases the context.
                     * @author Yadickson Soto
                     * @date 28/03/2014
                     * @return result code, zero "0" if successful.
                     */
                    virtual unsigned long releaseContext() = 0;

                    /**
                     * @fn virtual unsigned long connect() = 0;
                     * @brief Connects to the device.
                     * @author Yadickson Soto
                     * @date 28/03/2014
                     * @return result code, zero "0" if successful.
                     */
                    virtual unsigned long connect () = 0;

                    /**
                     * @fn virtual unsigned long disconnect() = 0;
                     * @brief Disconnects from the device.
                     * @author Yadickson Soto
                     * @date 28/03/2014
                     * @return result code, zero "0" if successful.
                     */
                    virtual unsigned long disconnect() = 0;

                    /**
                     * @fn virtual unsigned long getReaderStatus() = 0;
                     * @brief Gets the status of the device.
                     * @author Yadickson Soto
                     * @date 05/06/2014
                     * @return result code, zero "0" if successful.
                     */
                    virtual unsigned long getReaderStatus () = 0;

                    /**
                     * @fn virtual unsigned long selectAID ( const Smartmatic::System::SafeBuffer & aid ) = 0;
                     * @brief Gets the card's applet by ID.
                     * @author Yadickson Soto
                     * @date 28/03/2014
                     * @param aid The aid.
                     * @return result code, zero "0" if successful.
                     */
                    virtual unsigned long selectAID ( const Smartmatic::System::SafeBuffer & aid ) = 0;

                    /**
                     * @fn virtual unsigned long sendCommand ( const Smartmatic::System::SafeBuffer & command ) = 0;
                     * @brief Sends a command to reader.
                     * @author Yadickson Soto
                     * @date 28/03/2014
                     * @param command Command to be sent by the reader to the card.
                     * @return result code, zero "0" if successful.
                     */
                    virtual unsigned long sendCommand ( const Smartmatic::System::SafeBuffer & command ) = 0;

                    /**
                     * @fn virtual Smartmatic::System::SafeBuffer getResponse () = 0;
                     * @brief Gets the response to a previous command from the reader to the card.
                     * @author Yadickson Soto
                     * @date 28/03/2014
                     * @return The buffer containing the response sent by the card.
                     */
                    virtual Smartmatic::System::SafeBuffer getResponse () = 0;
            };
        }
    }
}

#endif /* _SMARTCARDCONNECTION_DRIVER_INTERFACE_HXX_ */
