/**
 * @file GlobalPlatformConnectionDriver.hxx
 * @brief Declaration of the GlobalPlatform-specific driver, which implements low-level operations of connection, sending commands and getting responses.
 * @author Yadickson Soto
 * @date 20/05/2014
 */

#ifndef _GLOBAL_PLATFORM_CONNECTION_DRIVER_HXX_
#define _GLOBAL_PLATFORM_CONNECTION_DRIVER_HXX_

#include <Card/ConnectionDriverInterface.hxx>
#include <Card/GlobalPlatformMode.hxx>

namespace Smartmatic
{
    namespace Log
    {
        class ISMTTLog;
    }

    namespace SAES
    {
        namespace Card
        {

            /**
             * @class GlobalPlatformConnectionDriver
             * @brief The GlobalPlaform reader driver, which uses globalplatform for connecting and interacting with the reader.
             * @author Yadickson Soto
             * @date 20/05/2014
             */
            class GlobalPlatformConnectionDriver : public ConnectionDriverInterface
            {
                public:

                    /**
                     * @fn GlobalPlatformConnectionDriver::GlobalPlatformConnectionDriver();
                     * @brief Class constructor.
                     * @author Yadickson Soto
                     * @date 20/05/2014
                     */

                    GlobalPlatformConnectionDriver();

                    /**
                     * @fn virtual GlobalPlatformConnectionDriver::~GlobalPlatformConnectionDriver();
                     * @brief Class destructor.
                     * @author Yadickson Soto
                     * @date 20/05/2014
                     */

                    virtual ~GlobalPlatformConnectionDriver();

                    /**
                     * @fn virtual unsigned long establishContext();
                     * @brief Establishes a context for the connection driver.
                     * @author Yadickson Soto
                     * @date 20/05/2014
                     * @return result code, zero "0" if successful.
                     */
                    virtual unsigned long establishContext ();

                    /**
                     * @fn virtual unsigned long releaseContext();
                     * @brief Releases the context.
                     * @author Yadickson Soto
                     * @date 20/05/2014
                     * @return result code, zero "0" if successful.
                     */
                    virtual unsigned long releaseContext();

                    /**
                     * @fn virtual unsigned long getReaderStatus();
                     * @brief Gets the status of the device.
                     * @author Yadickson Soto
                     * @date 05/06/2014
                     * @return result code, zero "0" if successful.
                     */
                    virtual unsigned long getReaderStatus ();

                    /**
                     * @fn virtual unsigned long connect();
                     * @brief Connects to the device.
                     * @author Yadickson Soto
                     * @date 20/05/2014
                     * @return result code, zero "0" if successful.
                     */
                    virtual unsigned long connect ();

                    /**
                     * @fn virtual unsigned long disconnect();
                     * @brief Disconnects from the device.
                     * @author Yadickson Soto
                     * @date 20/05/2014
                     * @return result code, zero "0" if successful.
                     */
                    virtual unsigned long disconnect();

                    /**
                     * @fn virtual unsigned long selectAID ( const Smartmatic::System::SafeBuffer & aid );
                     * @brief Gets the card's applet by ID.
                     * @author Yadickson Soto
                     * @date 20/05/2014
                     * @param aid The aid.
                     * @return result code, zero "0" if successful.
                     */
                    virtual unsigned long selectAID ( const Smartmatic::System::SafeBuffer & aid );

                    /**
                     * @fn virtual unsigned long sendCommand ( const Smartmatic::System::SafeBuffer & command );
                     * @brief Sends a command to reader.
                     * @author Yadickson Soto
                     * @date 20/05/2014
                     * @param command Command to be sent by the reader to the card.
                     * @return result code, zero "0" if successful.
                     */
                    virtual unsigned long sendCommand ( const Smartmatic::System::SafeBuffer & command );

                    /**
                     * @fn virtual Smartmatic::System::SafeBuffer getResponse ();
                     * @brief Gets the response to a previous command from the reader to the card.
                     * @author Yadickson Soto
                     * @date 20/05/2014
                     * @return The buffer containing the response sent by the card.
                     */
                    virtual Smartmatic::System::SafeBuffer getResponse ();

                protected:

                    /**
                     * @fn virtual unsigned long checkStatus( const OPGP_ERROR_STATUS & status );
                     * @brief Checks the reader's status.
                     * @author Yadickson Soto
                     * @date 20/05/2014
                     * @return status code.
                     */
                    virtual unsigned long checkStatus ( const OPGP_ERROR_STATUS & status );

                    /**
                     * @fn virtual unsigned long openSecureChannel();
                     * @brief Sets the mutual authentication implemented according to the selected platform.
                     * @author Yadickson Soto
                     * @date 20/05/2014
                     * @return result code, zero "0" if successful.
                     */
                    virtual unsigned long openSecureChannel();

                private:

                    OPGP_CARD_INFO cardInfo; ///< Information describing the card
                    OPGP_CARD_CONTEXT cardContext; ///< Context for the card

                    GlobalPlatformMode * platformMode; ///< the platform mode determined by configuration at construction time.

                    std::string readerName; ///< The reader name
                    Smartmatic::System::SafeBuffer responseBuffer; ///< The response to the last command.

                    static Smartmatic::Log::ISMTTLog * logger;  ///< The reference pointer of a logger instance.
            };
        }
    }
}

#endif /* _GLOBAL_PLATFORM_CONNECTION_DRIVER_HXX_ */
