/**
 * @file GlobalPlatformMode.hxx
 * @brief The global platform connection mode
 * @author Yadickson Soto
 * @date 20/05/2014
 */

#ifndef GLOBALPLATFORMMODE_HXX_
#define GLOBALPLATFORMMODE_HXX_

#include <System/SafeBuffer.h>
#include <globalplatform/globalplatform.h>

namespace Smartmatic
{
    namespace Log
    {
        class ISMTTLog;
    }

    namespace SAES
    {
        namespace Card
        {
            /**
             * @class GlobalPlatformMode
             * @brief The global platform connection mode.
             * @author Yadickson Soto
             * @date 20/05/2014
             */
            class GlobalPlatformMode
            {
                public:

                    /**
                     * @fn GlobalPlatformMode();
                     * @brief Class constructor.
                     * @author Yadickson Soto
                     * @date 05/06/2014
                     * @param cardInfo Information describing the card
                     * @param cardContext Context for the card
                     */
                    GlobalPlatformMode ( OPGP_CARD_INFO & cardInfo, OPGP_CARD_CONTEXT & cardContext );

                    /**
                     * @fn virtual ~GlobalPlatformMode();
                     * @brief Class destructor.
                     * @author Yadickson Soto
                     * @date 05/06/2014
                     */
                    virtual ~GlobalPlatformMode();

                    /**
                     * @fn virtual OPGP_ERROR_STATUS openSecureChannel();
                     * @brief Opens a secure channel, using the mutual authentication implemented according to the selected platform.
                     * @author Yadickson Soto
                     * @date 05/06/2014
                     * @return code status result
                     */
                    virtual OPGP_ERROR_STATUS openSecureChannel();

                    /**
                     * @fn virtual OPGP_ERROR_STATUS sendCommand ( const Smartmatic::System::SafeBuffer & command, Smartmatic::System::SafeBuffer & response );
                     * @brief Sends a command to reader and gets the response.
                     * @author Yadickson Soto
                     * @date 05/06/2014
                     * @param command Command sent by the reader to the card.
                     * @param [in,out] response Response given to the command
                     * @return code status result
                     */
                    virtual OPGP_ERROR_STATUS sendCommand ( const Smartmatic::System::SafeBuffer & command, Smartmatic::System::SafeBuffer & response );

                    /**
                     * @fn virtual unsigned int getPlatformMode() = 0;
                     * @brief Gets the platform mode value.
                     * @author Yadickson Soto
                     * @date 05/06/2014
                     * @return The platform mode value
                     */
                    virtual unsigned int getPlatformMode() = 0;

                protected:

                    /**
                     * @fn virtual unsigned int getPlatformMode() = 0;
                     * @brief Gets the security key used for the secure channel.
                     * @author Yadickson Soto
                     * @date 05/06/2014
                     * @return The security key.
                     */
                    virtual unsigned char * getSecurityKey();

                    /**
                     * @fn virtual OPGP_ERROR_STATUS setupSecureChannel() = 0;
                     * @brief Sets the mutual authentication implemented according to the selected platform.
                     * @author Yadickson Soto
                     * @date 05/06/2014
                     * @return status result code
                     */
                    virtual OPGP_ERROR_STATUS setupSecureChannel() = 0;

                    /**
                     * @fn virtual OPGP_ERROR_STATUS processCommand ( const Smartmatic::System::SafeBuffer & command, unsigned char * response, unsigned long & length ) = 0;
                     * @brief Sends a command to the card reader and gets the response. Platform-mode specific implementation.
                     * @author Yadickson Soto
                     * @date 05/06/2014
                     * @param command Command sent by the reader to the card.
                     * @param response buffer for the response
                     * @param length response buffer length
                     * @return result status code
                     */
                    virtual OPGP_ERROR_STATUS processCommand ( const Smartmatic::System::SafeBuffer & command, unsigned char * response, unsigned long & length ) = 0;

                protected:

                    OPGP_CARD_INFO & cardInfo; ///< Information describing the card
                    OPGP_CARD_CONTEXT & cardContext; ///< Context for the card

                    unsigned char keySetVersion; ///< The key set version.
                    unsigned char keyIndex; ///< zero-based index of the key.
                    unsigned char keyDerivation; ///< The key derivation.

                    bool secureChannelEnabled; ///< true to enable, false to disable the secure channel

                private:

                    static Smartmatic::Log::ISMTTLog * logger;  ///< The pointer to the logger instance.
            };

        } /* namespace Card */
    } /* namespace SAES */
} /* namespace Smartmatic */

#endif /* GLOBALPLATFORMMODEINTERFACE_HXX_ */
