/**
 * @file	DeviceTypeMonitorConfigurationManager.hxx
 * @brief	Declaration of the device type monitor configuration manager.
 * @author	Yadickson Soto
 * @date	07/05/2014
 */

#ifndef _DEVICE_MONITOR_CONFIGURATION_MANAGER_HXX_
#define _DEVICE_MONITOR_CONFIGURATION_MANAGER_HXX_

#include <string>
#include <Configuration/device-monitor-configuration-schema.hxx>

namespace Smartmatic
{
    namespace SAES
    {
        namespace Common
        {
            namespace Configuration
            {
                /**
                 * @class	DeviceTypeMonitorConfigurationManager
                 * @brief	The device monitor configuration manager.
                 * @author	Yadickson Soto
                 * @date	07/05/2014
                 */
                class DeviceTypeMonitorConfigurationManager
                {
                    public:

                        /**
                         * @fn	static DeviceTypeMonitorConfigurationManager * getInstance( DeviceTypeMonitorConfigurationManager * externManager = NULL );
                         * @brief	Gets the current Device Type Monitor Configuration Manager.
                         * @author	Yadickson Soto
                         * @date	07/05/2014
                         * @param 	externManager external manager to unit test
                         * @return	instance of the class.
                         */
                        static DeviceTypeMonitorConfigurationManager * getInstance ( DeviceTypeMonitorConfigurationManager * externManager = NULL );

                        /**
                         * @fn	virtual DeviceTypeMonitorConfiguration getDeviceTypeMonitorConfiguration( DeviceType type );
                         * @brief	Gets a copy of the device context selected according to the type
                         * @author	Yadickson Soto
                         * @date	07/05/2014
                         * @param	type device type
                         * @return	The device context.
                         */
                        virtual DeviceTypeMonitorConfiguration getDeviceTypeMonitorConfiguration ( DeviceType type );

                        /**
                         * @fn	DeviceTypeMonitorPoolConfiguration::DeviceTypeMonitorSequence & getDeviceTypeMonitorSequence();
                         * @brief	Gets a copy of the all devices configuration
                         * @author	Yadickson Soto
                         * @date	12/05/2014
                         * @return	All devices.
                         */
                        DeviceTypeMonitorPoolConfiguration::DeviceTypeMonitorSequence & getDeviceTypeMonitorSequence();

                        /**
                         * @fn	std::string getCustomNode ( const std::string & node, DeviceType type );
                         * @brief	Gets custom node to check device mounted
                         * @author	Yadickson Soto
                         * @date	09/07/2015
                         * @param 	node node to fill command
                         * @param	type device type
                         * @return	custom node to check device mounted
                         */
                        std::string getCustomNode ( const std::string & node, DeviceType type );

                        /**
                         * @fn	bool simulationEnabled ();
                         * @brief	Get if simulation mode is enabled
                         * @author	Yadickson Soto
                         * @date	04/06/2015
                         * @return	true if the simulation mode is enabled.
                         */
                        bool simulationEnabled ();

                        /**
                         * @fn	std::string getSimulatorFilePath ();
                         * @brief	Gets the simulator file path
                         * @author	Yadickson Soto
                         * @date	04/06/2015
                         * @return	simulator file path
                         */
                        std::string getSimulatorFilePath ();

                        /**
                         * @fn	virtual ~DeviceTypeMonitorConfigurationManager();
                         * @brief	Class destructor.
                         * @author	Yadickson Soto
                         * @date	07/05/2014
                         */
                        virtual ~DeviceTypeMonitorConfigurationManager();

                    protected:

                        /**
                         * @fn	DeviceTypeMonitorConfigurationManager();
                         * @brief	Class constructor.
                         * @author	Yadickson Soto
                         * @date	07/05/2014
                         */
                        DeviceTypeMonitorConfigurationManager();

                        /**
                         * @fn	virtual DeviceTypeMonitorPoolConfiguration & getDeviceTypeMonitorPoolConfiguration ();
                         * @brief	Gets a reference to the common context of the device monitor which contains the cryptographic data and the reader device settings.
                         * @author	Yadickson Soto
                         * @date	07/05/2014
                         * @return	The reference to the common context of the device monitor device
                         */
                        virtual DeviceTypeMonitorPoolConfiguration & getDeviceTypeMonitorPoolConfiguration ();

                    private:

                        static DeviceTypeMonitorConfigurationManager * instance; ///< The pointer to the singleton instance of the device monitor configuration manager
                };
            }
        }
    }
}

#endif // _DEVICE_MONITOR_CONFIGURATION_MANAGER_HXX_
