/**
 * @file	s-a-e-s-common-conf-manager.h
 * @brief	Declaration the configuration manager which provides and persists the SAESCommon settings.
 * This class is implemented in terms of SaesConfiguration.
 * @author	Youssef.tourkmani
 * @date	30/10/2012
 */

#ifndef _S_A_E_S_COMMON_CONFIGURATION_MANAGER_H_
#define _S_A_E_S_COMMON_CONFIGURATION_MANAGER_H_

#include <string>
#include <Configuration/SaesCommonConfiguration-schema.hxx>

namespace Smartmatic
{
    namespace SAES
    {
        namespace Common
        {
            namespace Configuration
            {
                /**
                 * @class	SAESCommonConfigurationManager
                 * @brief	Configuration manager Class that provide the SAESCommons settings.
                 * @author	Cesar.bonilla
                 * @date	4/28/2011
                 */
                class SAESCommonConfigurationManager
                {
                    public:

                        /**
                         * @fn	static SAESCommonConfigurationManager * getInstance( SAESCommonConfigurationManager * externManager = NULL );
                         * @brief	Gets the current instance of the singleton that manage the SAESCommon settings.
                         * This function throw a fatal error if it can't read 'SAESCommon' config file.
                         * @author	Yadickson Soto
                         * @date 	28/03/2014
                         * @param 	externManager external manager for unit tests
                         * @return	A pointer reference to the configuration manager.
                         */
                        static SAESCommonConfigurationManager * getInstance ( SAESCommonConfigurationManager * externManager = NULL );

                        /**
                         * @fn	SaesCommonConfiguration * getConfiguration();
                         * @brief	Gets an instance of the configuration provider 'SAESCommon'.
                         * @author	Youssef.tourkmani
                         * @date	30/10/2012
                         * @return	A pointer reference to the configuration provider instance.
                         */
                        SaesCommonConfiguration * getConfiguration();

                        /**
                         * @fn	void SAESCommonConfigurationManager::loadDefault();
                         * @brief	Loads the default settings. This function is invoked only if there's no an initial configuration file.
                         * @author	Yadickson Soto
                         * @date	28/03/2014
                         */
                        void loadDefault();

                        /**
                         * @fn	SecurityConfiguration & SAESCommonConfigurationManager::getSecurityConfiguration();
                         * @brief	Gets the provider of the security configuration.
                         * @author	Cesar.bonilla
                         * @date	4/28/2011
                         * @return	The reference to a security configuration instance.
                         */
                        SecurityConfiguration & getSecurityConfiguration();

                        /**
                         * @fn	FileSystemVoteConfiguration & SAESCommonConfigurationManager::getFileSystemVoteConfiguration();
                         * @brief	Gets the reference to a file system vote configuration.
                         * @author	Cesar.bonilla
                         * @date	4/28/2011
                         * @return	The reference to a file system vote configuration.
                         */
                        FileSystemVoteConfiguration & getFileSystemVoteConfiguration();

                        /**
                         * @fn	PrinterBackendType & SAESCommonConfigurationManager::getPrinterBackendType();
                         * @brief	Gets printer backend type used to select the printer strategy.
                         * @author	Yadickson Soto yadickson.soto@smartmatic.com
                         * @date	03/06/2011
                         * @return	The Enumerator reference that represents the printer backend type.
                         */
                        PrinterBackendType & getPrinterBackendType();

                        /**
                         * @fn	ConfigurationBasePath & SAESCommonConfigurationManager::getConfigBasePath();
                         * @brief	Gets the context reference that describes the directory hierarchy such as: RulesBasePath,
                         *  backupFileName, languageImageDirectory, etc.
                         * @author	Youssef.tourkmani
                         * @date	30/10/2012
                         * @return	The context reference of the directory hierarchy.
                         */
                        ConfigurationBasePath & getConfigBasePath();

                        /**
                         * @fn	std::string SAESCommonConfigurationManager::getDateDiagnosticFileName();
                         * @brief	Gets the 'DateDiagnosticFileName' field from the election context of the directory hierarchy.
                         * @author	Youssef.tourkmani
                         * @date	30/10/2012
                         * @return	The 'DateDiagnosticFileName' field the election context of the directory hierarchy.
                         */
                        std::string  getDateDiagnosticFileName();

                        /**
                         * @fn	bool SAESCommonConfigurationManager::isVerifyExternElements();
                         * @brief	Gets whether should do verifying of signature the resources.
                         * @author	Yadickson Soto yadickson.soto@smartmatic.com
                         * @date	03/06/2011
                         * @return <tt>true</tt> if it should verifying of signature the resources as part of checking task; otherwise <tt>false</tt>.
                         */
                        bool isVerifyExternElements();

                        /**
                         * @fn	bool SAESCommonConfigurationManager::isCompressElection();
                         * @brief	Gets whether should do uncompress the electoral data.
                         * @author	Yadickson Soto yadickson.soto@smartmatic.com
                         * @date	03/06/2011
                         * @return	<tt>true</tt> if it should uncompress the electoral data.
                         */
                        bool isCompressElection();

                        /**
                         * @fn	bool SAESCommonConfigurationManager::isCompressVotingDevices();
                         * @brief	Gets whether should do uncompress the voting device file.
                         * @author	Yadickson Soto yadickson.soto@smartmatic.com
                         * @date	03/06/2011
                         * @return	<tt>true</tt> if it should uncompress the voting device file; otherwise <tt>false</tt>.
                         */
                        bool isCompressVotingDevices();

                        /**
                         * @fn	bool SAESCommonConfigurationManager::isUseAsymmetricKeyForTransmissionPackages ();
                         * @brief	Gets wheter should used transmission package with asymmetric-key.
                         * @author	Yadickson Soto yadickson.soto@smartmatic.com
                         * @date	08/06/2011
                         * @return <tt>true</tt> if it should be used transmission package with asymmetric-key; otherwise <tt>false</tt>.
                         */
                        bool isUseAsymmetricKeyForTransmissionPackages ();

                        /**
                         * @fn	PCSCDConfiguration & SAESCommonConfigurationManager::getPCSCDConfiguration ();
                         * @brief	Gets the context which  handles restartind of PCSCD service them while the system is running.
                         * @author	Youssef.tourkmani
                         * @date	30/10/2012
                         * @return	The reference of context the PCSCD smartcard.
                         */
                        PCSCDConfiguration & getPCSCDConfiguration ();

                        /**
                         * @fn	bool SAESCommonConfigurationManager::getStatusMonitorCancel ();
                         * @brief	Gets status monitor cancel.
                         * @author	Youssef.tourkmani
                         * @date	30/10/2012
                         * @return	true if it succeeds, false if it fails.
                         */
                        bool getStatusMonitorCancel ();

                        /**
                         * @fn	int SAESCommonConfigurationManager::getStatusMonitorTimeOut ();
                         * @brief	Gets status monitor time out.
                         * @author	Youssef.tourkmani
                         * @date	30/10/2012
                         * @return	The value that indicate the timeout used in the status monitor.
                         */
                        int getStatusMonitorTimeOut ();

                        /**
                         * @fn	bool SAESCommonConfigurationManager::getDirectQrUseOptimization ();
                         * @brief	Gets whether should do the <tt>Qr</tt> optimization of the direct printer strategy.
                         * @author	Youssef.tourkmani
                         * @date	30/10/2012
                         * @return	<tt>true</tt> if it should sent <tt>Qr</tt> command; otherwise <tt>false</tt>.
                         */
                        bool getDirectQrUseOptimization ();

                        /**
                         * @fn	std::string SAESCommonConfigurationManager::getDirectQrCommandHeader ();
                         * @brief	Gets <tt>Qr</tt> command header used for the optimization of the direct printer strategy.
                         * @author	Youssef.tourkmani
                         * @date	30/10/2012
                         * @return	the string value with <tt>Qr</tt> command header.
                         */
                        std::string getDirectQrCommandHeader ();

                        /**
                         * @fn	std::string SAESCommonConfigurationManager::getDirectQrCommandFooter ();
                         * @brief	Gets <tt>Qr</tt> command footer used for the optimization of the direct printer strategy.
                         * @author	Youssef.tourkmani
                         * @date	30/10/2012
                         * @return	the string value with <tt>Qr</tt> command header.
                         */
                        std::string getDirectQrCommandFooter ();

                        /**
                         * @fn	bool SAESCommonConfigurationManager::isPVInAllLanguages();
                         * @brief	Query if this object is pv in all languages.
                         * @author	Youssef.tourkmani
                         * @date	30/10/2012
                         * @return	true if pv in all languages, false if not.
                         */
                        bool isPVInAllLanguages();

                        /**
                         * @fn	bool SAESCommonConfigurationManager::isForceReduceElection();
                         * @brief	Query if this object is force reduce election.
                         * @author	Youssef.tourkmani
                         * @date	30/10/2012
                         * @return	true if force reduce election, false if not.
                         */
                        bool isForceReduceElection();

                        /**
                         * @fn	OSConfigurationFiles & SAESCommonConfigurationManager::getOSConfigurationFiles();
                         * @brief	Gets operating system configuration files.
                         * @author	Youssef.tourkmani
                         * @date	30/10/2012
                         * @return	The operating system configuration files.
                         */
                        OSConfigurationFiles & getOSConfigurationFiles();

                        /**
                         * @fn	ConfigurationPrinter & SAESCommonConfigurationManager::getConfigurationPrinter();
                         * @brief	Gets a reference to the printer context which contains the settings used for initialize the strategy.
                         * @author	Youssef.tourkmani
                         * @date	30/10/2012
                         * @return	The reference to the printer context.
                         */
                        ConfigurationPrinter & getConfigurationPrinter();

                        /**
                         * @fn	virtual SAESCommonConfigurationManager::~SAESCommonConfigurationManager();
                         * @brief	Destructor (destroys the singleton instance)
                         * @author	Yadickson Soto
                         * @date	02/04/2014
                         */

                        virtual ~SAESCommonConfigurationManager();

                    protected:

                        /**
                         * @fn	SAESCommonConfigurationManager::SAESCommonConfigurationManager();
                         * @brief	Default constructor initializes an instance of the SAESCommonConfigurationManager class.
                         * @author	Cesar.bonilla
                         * @date	26/04/2011
                         */

                        SAESCommonConfigurationManager();

                        /**
                         * @fn	virtual void initialize();
                         * @brief	Initializes the class members with the information obtains from the 'SAESCommon' config file.
                         * @author	Yadickson Soto
                         * @date	28/03/2014
                         */
                        virtual void initialize();

                    private:

                        static SAESCommonConfigurationManager * instance; ///< The pointer to the singleton instance of the configuration manager
                        SaesCommonConfiguration * configuration;	///< The pointer to the saescommon configuration obtained from the 'SAESCommon' config file
                };
            }
        }
    }
}

#endif // _S_A_E_S_COMMON_CONFIGURATION_MANAGER_H_
