/**
 * @file	LogReader.h
 * @brief	Declaration of reader that provides access to log data obtained from the files.
 * @author	Youssef.tourkmani
 * @date	30/10/2012
 */
#ifndef _LOGREADER_H_
#define _LOGREADER_H_

#include "Log/LogLine.h"
#include <Log/LogPage.h>
#include <vector>
#include <string>
#include <Log/ISMTTLog.h>
#include <System/Exception/LogReaderException.h>
#include <Configuration/SaesCommonConfiguration-schema.hxx>

namespace Smartmatic
{
    namespace SAES
    {
        namespace Log
        {
            /**
             * @class	LogReader
             * @brief	Represents a reader that provides access to log data.
             * @author	Rafael Ferrero &lt;rafael.ferrero@smartmatic.com&gt;
             * @date	30/10/2012
             */
            class LogReader
            {
                public:
                    /**
                     * @fn	LogReader::LogReader( const std::string& logFile, unsigned int sizePage, const Smartmatic::SAES::Common::Configuration::CryptoType& type );
                     * @brief	Default constructor initializes an instance of the LogReader class.
                     * @author	Youssef.tourkmani
                     * @date	30/10/2012
                     * @param	logFile 	path of the log file to read.
                     * @param	sizePage	The number of lines associated to a page.
                     * @param	type			The type.
                     */
                    LogReader ( const std::string& logFile, unsigned int sizePage, const Smartmatic::SAES::Common::Configuration::CryptoType& type );

                    /**
                     * @fn	std::vector<LogLine> LogReader::getLogLines();
                     * @brief	Gets a copy of vector which contains the lines obtained from file.
                     * @author	Youssef.tourkmani
                     * @date	30/10/2012
                     * @return	The vector which contains the lines stored in the instance.
                     */
                    std::vector<LogLine> getLogLines();

                    /**
                     * @fn	unsigned int LogReader::getPageCount();
                     * @brief	Gets the count of total pages stored in the instance.
                     * @author	Youssef.tourkmani
                     * @date	30/10/2012
                     * @return The total of pages.
                     */
                    unsigned int getPageCount();

                    /**
                     * @fn	LogPage LogReader::getPage(unsigned int page) throw(Smartmatic::System::Exception::LogReaderException);
                     * @brief	Gets a copy of the page selected.
                     * @author	Youssef.tourkmani
                     * @date	30/10/2012
                     * @exception	Smartmatic::System::Exception::LogReaderException	Thrown when a log reader error
                     * 																															condition occurs.
                     * @param	page	The page.
                     * @return	LogPage page.
                     */
                    LogPage getPage ( unsigned int page )
                    throw ( Smartmatic::System::Exception::LogReaderException );

                private:

                    /**
                     * @fn	void LogReader::StringToMessagelog( const std::string& line, LogLine& logLine );
                     * @brief	Parses a line obtained from file and retrieves the message associated.
                     * @author	Youssef.tourkmani
                     * @date	30/10/2012
                     * @param	line	The line obtained from file.
                     * @param [in,out]	logLine	The reference to logLine instance used for retrieve the result.
                     */
                    void StringToMessagelog ( const std::string& line, LogLine& logLine );

                    /**
                     * @fn	LogLine::LevelLog LogReader::StringToLevelLog(std::string level);
                     * @brief	transfor a string into a LevelLog if not match return LevelLog::TRACE.
                     * @author	Youssef.tourkmani
                     * @date	30/10/2012
                     * @param	level	to transform.
                     * @return	LevelLog.
                     */
                    LogLine::LevelLog StringToLevelLog ( const std::string& level );
                    /**
                     * @fn	LogLine LogReader::ReadLine(std::string line);
                     * @brief	Parses a line obtained from file and retrieves a new logLine with the information associated.
                     * @author	Youssef.tourkmani
                     * @date	30/10/2012
                     * @param	line	A string readable of log.
                     * @return	The lobLine instance with the information associated.
                     */
                    LogLine ReadLine ( std::string line );

                    /**
                     * @fn	void ProcessFilesToGetLogLines ( const std::string& logFile );
                     * @brief	Processes the information contained in the log file.
                     * @author	Youssef.tourkmani
                     * @date	30/10/2012
                     * @param	logFile	The log file.
                     */
                    void ProcessFilesToGetLogLines ( const std::string& logFile );

                    static Smartmatic::Log::ISMTTLog * logger; /**<The reference pointer of a logger instance.*/
                    std::vector<LogLine> logLines; /**<The vector which contains the lines stored in the instance.*/
                    unsigned int sizePage; /**<The number of lines associated to a page.*/
                    Smartmatic::SAES::Common::Configuration::CryptoType type; ///< The type

            };
        }
    }
}

#endif // _LOGREADER_H_
