/**
 * @file UdevMonitor.cxx
 * @brief Body for the UdevMonitor class which interacts with the UDEV daemon.
 * @date 11/12/2012
 * @author Yadickson Soto
 */

#include "UdevMonitor.hxx"
#include <System/Utils/StringUtils.hxx>
#include <Log/ISMTTLog.h>
#include <Log/SMTTLogManager.h>
#include <System/OptParser.hxx>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

using namespace Smartmatic::SAES::Common::Operation::Monitor;
using namespace Smartmatic::System::Utils;
using namespace Smartmatic::System;
using namespace Smartmatic::Log;

ISMTTLog * UdevMonitor::logger = SMTTLogManager::GetLogger ( "Smartmatic.SAES.Common.Operation.Monitor.UdevMonitor", GETTEXT_PACKAGE );

UdevMonitor::UdevMonitor ()
{
    this->context = NULL;
    this->monitor = NULL;
    this->handle = 0;
}

UdevMonitor::~UdevMonitor()
{
    dispose();
}

bool UdevMonitor::initialize()
{
    if ( context != NULL )
    {
        return true;
    }

    context = udev_new ();

    if ( context == NULL )
    {
        logger->Debug ( "Can't create udev" );
    }
    else
    {
        monitor = udev_monitor_new_from_netlink ( context, "udev" );
        udev_monitor_enable_receiving ( monitor );
        handle = udev_monitor_get_fd ( monitor );
    }

    return context != NULL;
}

void UdevMonitor::dispose()
{
    if ( context != NULL )
    {
        udev_unref ( context );
        context = NULL;
    }

    if ( monitor != NULL )
    {
        udev_monitor_unref ( monitor );
        monitor = NULL;
    }
}

SafeList < MonitorInfo * > UdevMonitor::searchDevices ()
{
    SafeList < MonitorInfo * > list;

    if ( !context )
    {
        return list;
    }

    struct udev_enumerate * enumerate;

    struct udev_list_entry *it = NULL;

    enumerate = udev_enumerate_new ( context );

    udev_enumerate_scan_devices ( enumerate );

    struct udev_list_entry * devices = udev_enumerate_get_list_entry ( enumerate );

    udev_list_entry_foreach ( it, devices )
    {
        struct udev_device * dev;

        dev = udev_device_new_from_syspath ( udev_enumerate_get_udev ( enumerate ), udev_list_entry_get_name ( it ) );
        addMonitorInfo ( dev, list );
        udev_device_unref ( dev );
    }

    udev_enumerate_unref ( enumerate );

    return list;
}

SafeList < MonitorInfo * > UdevMonitor::eventDevices()
{
    SafeList < MonitorInfo * > list;
    struct udev_device * dev;

    while ( true )
    {
        fd_set fds;
        struct timeval tv;
        int ret;

        FD_ZERO ( &fds );
        FD_SET ( handle, &fds );
        tv.tv_sec = 0;
        tv.tv_usec = 250000;

        ret = select ( handle + 1, &fds, NULL, NULL, &tv );

        if ( ret > 0 && FD_ISSET ( handle, &fds ) )
        {
            dev = udev_monitor_receive_device ( monitor );
            addMonitorInfo ( dev, list );
            udev_device_unref ( dev );
        }
        else
        {
            break;
        }
    }

    return list;
}

void UdevMonitor::addMonitorInfo ( struct udev_device * dev, SafeList < MonitorInfo * > & list )
{
    if ( !dev )
    {
        return;
    }

    struct udev_list_entry *it = NULL;;

    std::string action = udev_device_get_action ( dev ) ? udev_device_get_action ( dev ) : "";

    MonitorInfo * info = new MonitorInfo ( action != "remove" );

    struct udev_list_entry * properties = udev_device_get_properties_list_entry ( dev );

    udev_list_entry_foreach ( it, properties )
    {
        const char * prop = udev_list_entry_get_name ( it );
        const char * value = udev_device_get_property_value ( dev, prop );

        if ( prop && value )
        {
            info->push ( prop, value );
        }
    }

    list.push_back ( info );
}
