/**
 * @file	BasePrintDocument.h
 * @brief	Declares of mediator which contains the printer backend and encapsulated the operations as: rendering operations, printer, callback result, etc.
 * @author	Youssef.tourkmani
 * @date	30/10/2012
 */

#ifndef BASEPRINTDOCUMENT_H_
#define BASEPRINTDOCUMENT_H_


#include <queue>
#include <qrencode.h>
#include <gtkmm.h>
#include <gdkmm.h>
#include <memory>

#include <Configuration/SaesCommonConfiguration-schema.hxx>
#include "Printing/DocumentOptions.h"
#include "Printing/GraphicContextPrintingBackend.h"
#include "Printing/PrintDocumentResult.h"
#include "IBasePrintDocument.hxx"
#include <set>
#include <Log/ISMTTLog.h>

namespace Smartmatic
{
    namespace SAES
    {
        namespace Printing
        {
            class IPrintingBackend;

            /**
             * @class	BasePrintDocument
             * @brief	Base printer that interacts with the printer backend.
             * @author	Cesar.bonilla
             * @date	4/28/2011
             */
            class BasePrintDocument : public IBasePrintDocument
            {
                public:

                    /**
                     * @fn	BasePrintDocument::BasePrintDocument(bool forceGTKPrinting = false);
                     * @brief	Default constructor initializes an instance of the BasePrintDocument class.
                     * @author	Cesar.bonilla
                     * @date	4/28/2011
                     * @param	forceGTKPrinting	true to force gtk printing.
                     */

                    BasePrintDocument ( bool forceGTKPrinting = false );

                    /**
                     * @fn	virtual BasePrintDocument::~BasePrintDocument();
                     * @brief	Finalizes the instance of the BasePrintDocument class.
                     * @author	Cesar.bonilla
                     * @date	4/28/2011
                     */

                    virtual ~BasePrintDocument();

                    /**
                     * @fn	void BasePrintDocument::setInitialPageSetup(Glib::RefPtr<Gtk::PageSetup> m_refPageSetup);
                     * @brief	Sets an initial page setup.
                     * @author	Cesar.bonilla
                     * @date	4/28/2011
                     * @param	m_refPageSetup	The reference page setup.
                     */

                    void setInitialPageSetup ( Glib::RefPtr<Gtk::PageSetup> m_refPageSetup );

                    /**
                     * @fn	void BasePrintDocument::setInitialPrintSetings(Glib::RefPtr<Gtk::PrintSettings> m_refSettings);
                     * @brief	Sets the printer settings.
                     * @author	Cesar.bonilla
                     * @date	4/28/2011
                     * @param	m_refSettings The pointer reference to print settings.
                     */

                    void setInitialPrintSetings ( Glib::RefPtr<Gtk::PrintSettings> m_refSettings );

                    /**
                     * @fn	void BasePrintDocument::Print();
                     * @brief	Prints the document using the assigned backend.
                     * This method triggers the callback client which describes the printer operation.
                     * @author	Cesar.bonilla
                     * @date	4/28/2011
                     */

                    void Print();

                    /**
                     * @fn	void BasePrintDocument::PrintPreview ( const ::Cairo::RefPtr<Cairo::Context>& graphicCtx ,
                     * 			Glib::RefPtr<Pango::Context> destPangoContext , double renderAreaWidth);
                     * @brief	Print preview.
                     * This method triggers the callback client which describes the printer operation.
                     * @author	Cesar.bonilla
                     * @date	4/28/2011
                     * @param	graphicCtx The pointer reference that stores global information used to control graphic rendering.
                     * @param	destPangoContext The pointer reference that stores global information used to control font rendering.
                     * @param	renderAreaWidth 	Width of the render area.
                     */

                    void PrintPreview (
                        const ::Cairo::RefPtr<Cairo::Context> & graphicCtx
                        , Glib::RefPtr<Pango::Context> destPangoContext
                        , double renderAreaWidth );

                    /**
                     * @fn	void BasePrintDocument::CalculatePreviewDimensions ( ::Cairo::RefPtr<Cairo::Context> cr ,
                     * 			Glib::RefPtr<Pango::Context> pg , double areaWidth );
                     * @brief	Calculates the preview dimensions.
                     * @author	Cesar.bonilla
                     * @date	4/28/2011
                     * @param	cr The pointer reference that stores global information used to control graphic rendering.
                     * @param	pg The pointer reference that stores global information used to control font rendering.
                     * @param	areaWidth	Width of the area.
                     */

                    void CalculatePreviewDimensions (
                        ::Cairo::RefPtr<Cairo::Context> cr
                        , Glib::RefPtr<Pango::Context>  pg
                        , double areaWidth
                    );

                    /**
                     * @fn	double BasePrintDocument::GetWidth ()
                     * @brief	Gets the width of the document.
                     * @author	Cesar.bonilla
                     * @date	4/28/2011
                     * @return	The width.
                     */

                    double GetWidth ()
                    {
                        return mDocumentWidth;
                    }

                    /**
                     * @fn	double BasePrintDocument::GetHeight ()
                     * @brief	Gets the height.
                     * @author	Cesar.bonilla
                     * @date	4/28/2011
                     * @return	The height.
                     */

                    double GetHeight ()
                    {
                        return mDocumentHeight;
                    }

                    /**
                     * @fn	void BasePrintDocument::setClient (PrintDocumentClient * client);
                     * @brief	Sets the reference to the client used to retrieve the printer result.
                     * @author	Cesar.bonilla
                     * @date	4/28/2011
                     * @param	client	The pointer reference to client instance.
                     */

                    void setClient ( PrintDocumentClient * client );

                    /**
                     * @fn	PrintDocumentResult BasePrintDocument::getDocumentResult ()
                     * @brief	Gets the document result which contains the details of the operation.
                     * @author	Cesar.bonilla
                     * @date	4/28/2011
                     * @return	The document result.
                     */

                    PrintDocumentResult getDocumentResult ()
                    {
                        return this->res;
                    }

                    /**
                     * @fn	void BasePrintDocument::PreRenderDocument ();
                     * @brief	Pre render document.
                     * @author	Youssef.tourkmani
                     * @date	30/10/2012
                     */
                    void PreRenderDocument ();

                    /**
                     * @fn	virtual void BasePrintDocument::cleanPrintingQueue();
                     * @brief	Cleans the printing queue from any previous job.
                     * @author	Daniel Ampuero
                     * @date	25/09/2012
                     */
                    virtual void cleanPrintingQueue();

                protected:

                    /**
                     * @fn	virtual void BasePrintDocument::PrintDoc()=0;
                     * @brief	Print document.
                     * @author	Cesar.bonilla
                     * @date	4/28/2011
                     */

                    virtual void PrintDoc() = 0;

                    /**
                     * @fn	virtual void BasePrintDocument::BeginPrint()=0;
                     * @brief	Begins a print.
                     * @author	Cesar.bonilla
                     * @date	4/28/2011
                     */

                    virtual void BeginPrint() = 0;

                    /**
                     * @fn	virtual void BasePrintDocument::EndPrint()=0;
                     * @brief	Ends a print.
                     * @author	Cesar.bonilla
                     * @date	4/28/2011
                     */

                    virtual void EndPrint() = 0;

                    /**
                     * @fn	void BasePrintDocument::NewTextBlock(DocumentTextOptions& textOptions);
                     * @brief	Creates a new text block.
                     * @author	Cesar.bonilla
                     * @date	4/28/2011
                     * @param	textOptions	Options for controlling the text.
                     */

                    void NewTextBlock ( DocumentTextOptions & textOptions );

                    /**
                     * @fn	void BasePrintDocument::NewTextColunmsBlock(std::list<DocumentTextOptions>& textOptionList);
                     * @brief	Creates a new text colunms block.
                     * @author	Cesar.bonilla
                     * @date	4/28/2011
                     * @param	textOptionList	List of text options.
                     */

                    void NewTextColunmsBlock ( std::list<DocumentTextOptions> & textOptionList );

                    /**
                     * @fn	void BasePrintDocument::ChangeFont(DocumentFontSize newFont);
                     * @brief	Change font.
                     * @author	Cesar.bonilla
                     * @date	4/28/2011
                     * @param	newFont	The new font.
                     */

                    void ChangeFont ( DocumentFontSize newFont );

                    /**
                     * @fn	void BasePrintDocument::Indent(gint levelsToIndent =1);
                     * @brief	Indents.
                     * @author	Cesar.bonilla
                     * @date	4/28/2011
                     * @param	levelsToIndent	The levels to indent.
                     */

                    void Indent ( gint levelsToIndent = 1 );

                    /**
                     * @fn	void BasePrintDocument::UnIndent(gint levelsToUnIndent =1);
                     * @brief	Un indent.
                     * @author	Cesar.bonilla
                     * @date	4/28/2011
                     * @param	levelsToUnIndent	The levels to un indent.
                     */

                    void UnIndent ( gint levelsToUnIndent = 1 );

                    /**
                     * @fn	void BasePrintDocument::Rotate();
                     * @brief	Rotates this object.
                     * @author	Cesar.bonilla
                     * @date	4/28/2011
                     */

                    void Rotate();

                    /**
                     * @fn	void BasePrintDocument::DrawString(const Glib::ustring& text, bool addDocBreak = true);
                     * @brief	Draw string.
                     * @author	Cesar.bonilla
                     * @date	4/28/2011
                     * @param	text			 	The text.
                     * @param	addDocBreak	true to add document break.
                     */

                    void DrawString ( const Glib::ustring & text, bool addDocBreak = true );

                    /**
                     * @fn	void BasePrintDocument::DrawStrings(std::list<Glib::ustring>& text,
                     * 			bool addDocBreak = true);
                     * @brief	Draw strings.
                     * @author	Cesar.bonilla
                     * @date	4/28/2011
                     * @param	text			 	The text.
                     * @param	addDocBreak	true to add document break.
                     */

                    void DrawStrings ( std::list<Glib::ustring> & text, bool addDocBreak = true );

                    /**
                     * @fn	void BasePrintDocument::AddBreak();
                     * @brief	Adds break.
                     * @author	Youssef.tourkmani
                     * @date	30/10/2012
                     */
                    void AddBreak();

                    /**
                     * @fn	void BasePrintDocument::DrawImage( Glib::RefPtr<Gdk::Pixbuf> image ,
                     * 			Smartmatic::SAES::Printing::DocumentImageAlignmet imgAlignment ,
                     * 			Smartmatic::SAES::Printing::DocumentImageLayout imgLayout = Exclusive ,
                     * 			CellBorderType imageBorder = Border_None , int leftPadding = 0 , int rightPadding = 0 ,
                     * 			int topPadding = 0 , int bottomPadding = 0);
                     * @brief	Draw image.
                     * @author	Cesar.bonilla
                     * @date	4/28/2011
                     * @param	image				 	The image.
                     * @param	imgAlignment	 	The image alignment.
                     * @param	imgLayout		 	The image layout.
                     * @param	imageBorder	 	The image border.
                     * @param	leftPadding	 	The left padding.
                     * @param	rightPadding 	The right padding.
                     * @param	topPadding	 	The top padding.
                     * @param	bottomPadding	The bottom padding.
                     */

                    void DrawImage (
                        Glib::RefPtr<Gdk::Pixbuf> image
                        , Smartmatic::SAES::Printing::DocumentImageAlignmet imgAlignment
                        , Smartmatic::SAES::Printing::DocumentImageLayout imgLayout = Exclusive
                                , CellBorderType imageBorder = Border_None
                                        , int leftPadding = 0
                                                            , int rightPadding = 0
                                                                    , int topPadding = 0
                                                                            , int bottomPadding = 0 );

                    /**
                     * @fn	void BasePrintDocument::DrawQRCode (QRCodeContent& qr_content);
                     * @brief	Draw qr code.
                     * @author	Youssef.tourkmani
                     * @date	30/10/2012
                     * @param [in,out]	qr_content	The qr content.
                     */
                    void DrawQRCode ( QRCodeContent & qr_content );

                    /**
                     * @fn	void BasePrintDocument::DrawQRCode ( const Smartmatic::System::SafeBuffer& data ,
                     * 			gint pixelSize , gint margin , QRecLevel level , gint symbolVersion ,
                     * 			Glib::RefPtr<Gdk::Pixbuf> backgroundImage , DocumentImageAlignmet imgAlignment = Left ,
                     * 			DocumentImageLayout imgLayout = Exclusive , CellBorderType imageBorder = Border_None ,
                     * 			int leftPadding = 0 , int rightPadding = 0 , int topPadding = 0 , int bottomPadding = 0 );
                     * @brief	Draw qr code.
                     * @author	Youssef.tourkmani
                     * @date	30/10/2012
                     * @param	data					 	The data.
                     * @param	pixelSize			 	Size of the pixel.
                     * @param	margin				 	The margin.
                     * @param	level					 	The level.
                     * @param	symbolVersion	 	The symbol version.
                     * @param	backgroundImage	The background image.
                     * @param	imgAlignment		 	(optional) the image alignment.
                     * @param	imgLayout			 	(optional) the image layout.
                     * @param	imageBorder		 	(optional) the image border.
                     * @param	leftPadding		 	(optional) the left padding.
                     * @param	rightPadding	 	(optional) the right padding.
                     * @param	topPadding		 	(optional) the top padding.
                     * @param	bottomPadding	 	(optional) the bottom padding.
                     */
                    void DrawQRCode (
                        const Smartmatic::System::SafeBuffer & data
                        , gint pixelSize
                        , gint margin
                        , QRecLevel level
                        , gint symbolVersion
                        , Glib::RefPtr<Gdk::Pixbuf> backgroundImage
                        , DocumentImageAlignmet imgAlignment = Left
                                , DocumentImageLayout imgLayout = Exclusive
                                        , CellBorderType imageBorder = Border_None
                                                , int leftPadding = 0
                                                        , int rightPadding = 0
                                                                , int topPadding = 0
                                                                        , int bottomPadding = 0
                    );

                    /**
                     * @fn	void BasePrintDocument::SetSingleColumnLayout ();
                     * @brief	Sets single column layout.
                     * @author	Youssef.tourkmani
                     * @date	30/10/2012
                     */
                    void SetSingleColumnLayout ();

                    /**
                     * @fn	void BasePrintDocument::SetDoubleColumnLayout ();
                     * @brief	Sets double column layout.
                     * @author	Youssef.tourkmani
                     * @date	30/10/2012
                     */
                    void SetDoubleColumnLayout ();

                    /**
                     * @fn	void BasePrintDocument::NewLine();
                     * @brief	Creates a new line.
                     * @author	Cesar.bonilla
                     * @date	4/28/2011
                     */

                    void NewLine();

                    /**
                     * @fn	void BasePrintDocument::DrawLine (double lenght, DocumentImageAlignmet alignment);
                     * @brief	Draw line.
                     * @author	Cesar.bonilla
                     * @date	4/28/2011
                     * @param	lenght	 	The lenght.
                     * @param	alignment	The alignment.
                     */

                    void DrawLine ( double lenght, DocumentImageAlignmet alignment );

                    /**
                     * @fn	virtual void BasePrintDocument::OnSetupPrintOperation(Glib::RefPtr<Gtk::PrintOperation>& operation);
                     * @brief	Executes the setup print operation action.
                     * @author	Cesar.bonilla
                     * @date	4/28/2011
                     * @param	operation	The operation.
                     */

                    virtual void OnSetupPrintOperation ( Glib::RefPtr<Gtk::PrintOperation> & operation );

                    /**
                     * @fn	virtual void BasePrintDocument::OnBeginPrint(const Glib::RefPtr<Gtk::PrintContext>& context);
                     * @brief	Executes the begin print action.
                     * @author	Cesar.bonilla
                     * @date	4/28/2011
                     * @param	context	The context.
                     */

                    virtual void OnBeginPrint ( const Glib::RefPtr<Gtk::PrintContext> & context );

                    /**
                     * @fn	virtual void BasePrintDocument::OnDrawPage(const Glib::RefPtr<Gtk::PrintContext>& context ,
                     * 			int page_nr);
                     * @brief	Executes the draw page action.
                     * @author	Cesar.bonilla
                     * @date	4/28/2011
                     * @param	context	The context.
                     * @param	page_nr	The page nr.
                     */

                    virtual void OnDrawPage ( const Glib::RefPtr<Gtk::PrintContext> & context , int page_nr );

                    /**
                     * @fn	virtual void BasePrintDocument::OnDone(Gtk::PrintOperationResult result);
                     * @brief	Executes the done action.
                     * @author	Cesar.bonilla
                     * @date	4/28/2011
                     * @param	result	The result.
                     */

                    virtual void OnDone ( Gtk::PrintOperationResult result );

                    /**
                     * @fn	virtual void BasePrintDocument::OnStatusChanged(const Glib::RefPtr<Gtk::PrintOperation>& operation);
                     * @brief	Executes the status changed action.
                     * @author	Cesar.bonilla
                     * @date	4/28/2011
                     * @param	operation	The operation.
                     */

                    virtual void OnStatusChanged ( const Glib::RefPtr<Gtk::PrintOperation> & operation );

                    /**
                     * @fn	virtual void BasePrintDocument::OnRequestPageSetup(const Glib::RefPtr<Gtk::PrintContext>& context,
                     * 			int page_no, const Glib::RefPtr<Gtk::PageSetup>& setup);
                     * @brief	Executes the request page setup action.
                     * @author	Cesar.bonilla
                     * @date	4/28/2011
                     * @param	context	The context.
                     * @param	page_no	The page no.
                     * @param	setup	 	The setup.
                     */

                    virtual void OnRequestPageSetup ( const Glib::RefPtr<Gtk::PrintContext> & context, int page_no, const Glib::RefPtr<Gtk::PageSetup> & setup );

                    /**
                     * @fn	bool BasePrintDocument::OnPrintOperationTimeout ();
                     * @brief	Executes the print operation timeout action.
                     * @author	Youssef.tourkmani
                     * @date	30/10/2012
                     * @return	true if it succeeds, false if it fails.
                     */
                    bool OnPrintOperationTimeout ();

                    /**
                     * @fn	virtual void BasePrintDocument::ConnectToSignals(const Glib::RefPtr<Gtk::PrintOperation>& operation);
                     * @brief	Connects to signals.
                     * @author	Cesar.bonilla
                     * @date	4/28/2011
                     * @param	operation	The operation.
                     */

                    virtual void ConnectToSignals ( const Glib::RefPtr<Gtk::PrintOperation> & operation );

                    /**
                     * @fn	void BasePrintDocument::DisconnectFromSignals();
                     * @brief	Disconnects from signals.
                     * @author	Cesar.bonilla
                     * @date	4/28/2011
                     */
                    void DisconnectFromSignals();

                    /**
                     * @fn	void BasePrintDocument::OnPrinterMonitorThreadBody ();
                     * @brief	Executes the printer monitor thread body action.
                     * @author	Youssef.tourkmani
                     * @date	30/10/2012
                     */
                    void OnPrinterMonitorThreadBody ();

                private:

                    /**
                     * @enum	DocumentContentType
                     * @brief	Values that represent DocumentContentType.
                     */
                    enum DocumentContentType
                    {
                        Text,
                        RotatedText,
                        TextColumns,
                        RotatedTextColumns,
                        Image,
                        Break,
                        LineSep,
                        TwoColumnRequest,
                        SingleColumnRequest,
                        QRCode
                    };

                    // Document Content Representation

                    std::queue<DocumentContentType> contentTypeQueue;   ///< Queue of content types
                    std::queue<TextContent> contentTextQueue;   ///< Queue of content texts
                    std::queue<ImageContent> contentImageQueue; ///< Queue of content images
                    std::queue<SeparatorContent> contentSepQueue;   ///< Queue of content separators
                    std::queue<QRCodeContent> contentQRCodeQeue;	///< The content qr code qeue

                    std::list<Glib::ustring> currentContent;	///< The current content
                    std::list<DocumentTextOptions> currentContentOptions;   ///< Options for controlling the current content
                    DocumentContentType currenContentType;  ///< Type of the curren content

                    // Printing management

                    Glib::RefPtr<Gtk::PrintOperation> currentPrintOperation;	///< The current print operation
                    Glib::RefPtr<Gtk::PageSetup> m_refPageSetup;	///< The reference page setup
                    Glib::RefPtr<Gtk::PrintSettings> m_refSettings; ///< The reference settings

                    bool aborted_by_thread; ///< true to aborted by thread

                    std::list<sigc::connection> mSignalPool;	///< The signal pool
                    std::list<sigc::connection> mTimeoutPool;	///< The signal pool

                    Glib::Mutex m_PrintOpMutex; ///< The print operation mutex
                    Glib::Cond m_JobSentCond; ///< The job sent condition
                    Glib::Cond m_MonitorFinishedCond; ///< The monitor finished condition
                    static Smartmatic::Log::ISMTTLog * logger;  ///< The logger

                    // Document build and formatting operations

                    /**
                     * @fn	void BasePrintDocument::SplitIntoPages ();
                     * @brief	Split document content into pages.
                     * @author	Youssef.tourkmani
                     * @date	30/10/2012
                     * ### param [in,out]	gtkBackEnd	The gtk back end.
                     */
                    void SplitIntoPages ();

                    /**
                     * @fn	double BasePrintDocument::RenderDocument (GraphicContextPrintingBackend & gtkBackEnd);
                     * @brief	Renders the document described by gtkBackEnd.
                     * @author	Cesar.bonilla
                     * @date	4/28/2011
                     * @param	gtkBackEnd	The gtk back end.
                     * @return	.
                     */

                    double RenderDocument ( GraphicContextPrintingBackend & gtkBackEnd );

                    /**
                     * @fn	void BasePrintDocument::CommitCurrentTextBlock();
                     * @brief	Commit current text block.
                     * @author	Cesar.bonilla
                     * @date	4/28/2011
                     */

                    void CommitCurrentTextBlock();

                    /**
                     * @fn	void BasePrintDocument::UpdateContentType();
                     * @brief	Updates the content type.
                     * @author	Cesar.bonilla
                     * @date	4/28/2011
                     */

                    void UpdateContentType();

                    /**
                     * @fn	void BasePrintDocument::PrintWithGTK();
                     * @brief	Print with gtk.
                     * @author	Cesar.bonilla
                     * @date	4/28/2011
                     */

                    void PrintWithGTK();

                    /**
                     * @fn	void BasePrintDocument::PrintDirect();
                     * @brief	Print direct.
                     * @author	Cesar.bonilla
                     * @date	4/28/2011
                     */

                    void PrintDirect();

                    gint isRotated; ///< The is rotated
                    bool usingGtkPrinting;  ///< true to using gtk printing
                    bool runWholeProcessInSeperatedThread;  ///< true to run whole process in seperated thread
                    bool exportDoc; ///< true to export document
                    bool mIsPrinterOpComplete;	///< true if this object is printer operation complete
                    //bool contextRetreived;
                    PrintDocumentResult res;	///< The resource

                    GraphicContextPrintingBackend m_GraphicBackend; ///< The graphic backend

                    // Dimensions

                    double mDocumentWidth;  ///< Width of the document
                    double mDocumentHeight; ///< Height of the document

                    std::auto_ptr<IPrintingBackend> directBackEnd;
                    PrintDocumentClient * client;	///< The client

                    Gtk::PrintStatus prev_status; ///< The previous status
                    Glib::Thread * m_thread; ///< The thread pointer
            };
        }
    }
}
#endif /* BASEPRINTDOCUMENT_H_ */
