/**
 * @file	GraphicContextPrintingBackend.h
 * @brief	Declares the graphic context printing backend class.
 * @author	Youssef.tourkmani
 * @date	30/10/2012
 */

#ifndef GRAPHICCONTEXTPRINTINGBACKEND_H_
#define GRAPHICCONTEXTPRINTINGBACKEND_H_

#include <queue>
#include <gtkmm.h>
#include <gdkmm.h>
#include <qrencode.h>

#include <System/SafeBuffer.h>
#include <System/Exception/QREncodingException.h>

#include "Printing/IPrintingBackend.h"
#include "Printing/DocumentOptions.h"
#include "Printing/GraphicContentPage.h"


namespace Smartmatic
{
    namespace Log
    {
        class ISMTTLog;
    }

    namespace SAES
    {

        namespace Printing
        {
            /**
             * @class	GraphicContextPrintingBackend
             * @brief	Graphic context printing backend.
             * @author	Cesar.bonilla
             * @date	4/28/2011
             */

            class GraphicContextPrintingBackend : IPrintingBackend
            {

                public:


                    // Lifecycle

                    /**
                     * @fn	GraphicContextPrintingBackend::GraphicContextPrintingBackend ( Cairo::RefPtr<Cairo::Context> graphicCtx ,
                     * 			Glib::RefPtr<Pango::Context> destPangoContext , double renderAreaWidth);
                     * @brief	Constructor.
                     * @author	Cesar.bonilla
                     * @date	4/28/2011
                     * @param	graphicCtx				Context for the graphic.
                     * @param	destPangoContext	Context for the destination pango.
                     * @param	renderAreaWidth 	Width of the render area.
                     */

                    GraphicContextPrintingBackend (
                        Cairo::RefPtr<Cairo::Context> graphicCtx
                        , Glib::RefPtr<Pango::Context> destPangoContext
                        , double renderAreaWidth );

                    /**
                     * @fn	GraphicContextPrintingBackend::GraphicContextPrintingBackend( Cairo::RefPtr<Cairo::Context> cairoCtx ,
                     * 			Glib::RefPtr<Pango::Context> pangoCtx , double pageWidth , double pageHeight);
                     * @brief	Constructor.
                     * @author	Youssef.tourkmani
                     * @date	30/10/2012
                     * @param	cairoCtx		Context for the cairo.
                     * @param	pangoCtx		Context for the pango.
                     * @param	pageWidth 	Width of the page.
                     * @param	pageHeight	Height of the page.
                     */
                    GraphicContextPrintingBackend (
                        Cairo::RefPtr<Cairo::Context> cairoCtx
                        , Glib::RefPtr<Pango::Context>  pangoCtx
                        , double pageWidth
                        , double pageHeight );

                    /**
                     * @fn	virtual GraphicContextPrintingBackend::~GraphicContextPrintingBackend();
                     * @brief	Finaliser.
                     * @author	Cesar.bonilla
                     * @date	4/28/2011
                     */

                    virtual ~GraphicContextPrintingBackend();


                    /**
                     * @fn	GraphicContextPrintingBackend& GraphicContextPrintingBackend::operator= (const GraphicContextPrintingBackend& other);
                     * @brief	Assignment operator.
                     * @author	Youssef.tourkmani
                     * @date	30/10/2012
                     * @param	other	The other.
                     * @return	A shallow copy of this object.
                     */
                    GraphicContextPrintingBackend & operator= ( const GraphicContextPrintingBackend & other );



                    // IPrintingBackend implementation

                    /**
                     * @fn	virtual void GraphicContextPrintingBackend::Init();
                     * @brief	Initialises this object.
                     * @author	Cesar.bonilla
                     * @date	4/28/2011
                     */

                    virtual void Init();

                    /**
                     * @fn	virtual void GraphicContextPrintingBackend::End();
                     * @brief	Ends this object.
                     * @author	Cesar.bonilla
                     * @date	4/28/2011
                     */

                    virtual void End();

                    /**
                     * @fn	void GraphicContextPrintingBackend::DrawString(std::list<Glib::ustring>& textContent ,
                     * 			std::list<DocumentTextOptions>& textContentOptionList);
                     * @brief	Draw string.
                     * @author	Cesar.bonilla
                     * @date	4/28/2011
                     * @param	textContent					 	The text content.
                     * @param	textContentOptionList	List of text content options.
                     */

                    void DrawString ( std::list<Glib::ustring> & textContent
                                      , std::list<DocumentTextOptions> & textContentOptionList );

                    /**
                     * @fn	void GraphicContextPrintingBackend::DrawImage (Glib::RefPtr<Gdk::Pixbuf> image);
                     * @brief	Draw image.
                     * @author	Cesar.bonilla
                     * @date	4/28/2011
                     * @param	image	The image.
                     */

                    void DrawImage ( Glib::RefPtr<Gdk::Pixbuf> image );

                    /**
                     * @fn	void GraphicContextPrintingBackend::DrawImage (ImageContent content);
                     * @brief	Draw image.
                     * @author	Cesar.bonilla
                     * @date	4/28/2011
                     * @param	content	The content.
                     */

                    void DrawImage ( ImageContent content );

                    // gtkBackend owns

                    /**
                     * @fn	void GraphicContextPrintingBackend::DrawLine (SeparatorContent line);
                     * @brief	Draw line.
                     * @author	Cesar.bonilla
                     * @date	4/28/2011
                     * @param	line	The line.
                     */

                    void DrawLine ( SeparatorContent line );

                    /**
                     * @fn	void GraphicContextPrintingBackend::DrawQRCode (QRCodeContent qr) throw (Smartmatic::System::Exception::QREncodingException);
                     * @brief	Draw qr code.
                     * @author	Youssef.tourkmani
                     * @date	30/10/2012
                     * @param	qr The qr code.
                     */
                    virtual void DrawQRCode ( QRCodeContent& qr );

                    /**
                     * @fn	double GraphicContextPrintingBackend::Height ()
                     * @brief	Gets the height.
                     * @author	Cesar.bonilla
                     * @date	4/28/2011
                     * @return	.
                     */

                    double Height ()
                    {
                        return mPrintContextHeight;
                    }

                    /**
                     * @fn	double GraphicContextPrintingBackend::Width ()
                     * @brief	Gets the width.
                     * @author	Cesar.bonilla
                     * @date	4/28/2011
                     * @return	.
                     */

                    double Width ()
                    {
                        return mPrintContextWidth;
                    }


                    /**
                     * @fn	int GraphicContextPrintingBackend::GetNumPages ()
                     * @brief	Gets number pages.
                     * @author	Youssef.tourkmani
                     * @date	30/10/2012
                     * @return	The number pages.
                     */
                    int GetNumPages ()
                    {
                        return ( int ) m_Pages.size();
                    }

                    /**
                     * @fn	void GraphicContextPrintingBackend::RenderPage (Cairo::RefPtr<Cairo::Context> drawableCtx);
                     * @brief	Renders the page described by drawableCtx.
                     * @author	Youssef.tourkmani
                     * @date	30/10/2012
                     * @param	drawableCtx	Context for the drawable.
                     */
                    void RenderPage ( Cairo::RefPtr<Cairo::Context> drawableCtx );

                    /**
                     * @fn	void GraphicContextPrintingBackend::PreDrawString( std::list<Glib::ustring>& textContent ,
                     * 			std::list<DocumentTextOptions>& textContentOptions);
                     * @brief	Pre draw string.
                     * @author	Youssef.tourkmani
                     * @date	30/10/2012
                     * @param [in,out]	textContent					The text content.
                     * @param [in,out]	textContentOptions	Options for controlling the text content.
                     */
                    void PreDrawString (
                        std::list<Glib::ustring> & textContent
                        , std::list<DocumentTextOptions> & textContentOptions );

                    /**
                     * @fn	bool GraphicContextPrintingBackend::PreDrawRow ( std::vector<Glib::ustring>::iterator pRowContent ,
                     * 			std::vector<DocumentTextOptions> textContentOptions , int numCellsToRender);
                     * @brief	Pre draw row.
                     * @author	Youssef.tourkmani
                     * @date	30/10/2012
                     * @param	pRowContent					The row content.
                     * @param	textContentOptions	Options for controlling the text content.
                     * @param	numCellsToRender		Number of cells to renders.
                     * @return	true if it succeeds, false if it fails.
                     */
                    bool PreDrawRow (
                        std::vector<Glib::ustring>::iterator pRowContent
                        , std::vector<DocumentTextOptions> textContentOptions
                        , int numCellsToRender );

                    /**
                     * @fn	bool GraphicContextPrintingBackend::PreDrawImage(ImageContent content);
                     * @brief	Pre draw image.
                     * @author	Youssef.tourkmani
                     * @date	30/10/2012
                     * @param	content	The content.
                     * @return	true if it succeeds, false if it fails.
                     */
                    bool PreDrawImage ( ImageContent content );

                    /**
                     * @fn	void GraphicContextPrintingBackend::PreDrawQRCode (QRCodeContent qr) throw (Smartmatic::System::Exception::QREncodingException);
                     * @brief	Pre draw qr code.
                     * @author	Youssef.tourkmani
                     * @date	30/10/2012
                     * @exception	Smartmatic::System::Exception::QREncodingException	Thrown when a qr encoding error
                     * 																																condition occurs.
                     * @param	qr	The qr.
                     */
                    void PreDrawQRCode ( QRCodeContent qr )
                    throw ( Smartmatic::System::Exception::QREncodingException );

                    /**
                     * @fn	bool GraphicContextPrintingBackend::PreDrawSeparator (SeparatorContent line);
                     * @brief	Pre draw separator.
                     * @author	Youssef.tourkmani
                     * @date	30/10/2012
                     * @param	line	The line.
                     * @return	true if it succeeds, false if it fails.
                     */
                    bool PreDrawSeparator ( SeparatorContent line );

                    /**
                     * @fn	void GraphicContextPrintingBackend::PreJumpInlineImage ();
                     * @brief	Pre jump inline image.
                     * @author	Youssef.tourkmani
                     * @date	30/10/2012
                     */
                    void PreJumpInlineImage ();

                    /**
                     * @fn	void GraphicContextPrintingBackend::PreSetSingleColumn ();
                     * @brief	Pre set single column.
                     * @author	Youssef.tourkmani
                     * @date	30/10/2012
                     */
                    void PreSetSingleColumn ();

                    /**
                     * @fn	void GraphicContextPrintingBackend::PreSetDoubleColumn ();
                     * @brief	Pre set double column.
                     * @author	Youssef.tourkmani
                     * @date	30/10/2012
                     */
                    void PreSetDoubleColumn ();

                    /**
                     * @fn	GraphicContextPrintingBackend::GraphicContextPrintingBackend();
                     * @brief	Default constructor.
                     * @author	Cesar.bonilla
                     * @date	4/28/2011
                     */

                    GraphicContextPrintingBackend();

                    /**
                     * @brief  Draw a QR code.
                     * @author manuel.arguelles
                     * @date   Sep  8, 2017
                     * @param  qrCode The QR code to draw.
                     */
                    //virtual void DrawQRCode(QRCodeContent& qrCode);

                    /**
                     * @brief  Gets the result of the print.
                     * @author manuel.arguelles
                     * @date   Sep 11, 2017
                     * @return The print result.
                     */
                    virtual PrintDocumentResult getPrintResult();

                    /**
                     * @brief  Cleans the queue to print.
                     * @author manuel.arguelles
                     * @date   Sep  8, 2017
                     */
                    virtual void cleanPrintQueue();

                protected:


                private:

                    static Smartmatic::Log::ISMTTLog * logger; ///< The logger

                    Glib::RefPtr<Gtk::PrintContext> mPrintContext;  ///< Context for the print
                    Cairo::RefPtr<Cairo::Context> mCairoCtx;	///< Context for the cairo

                    Pango::FontDescription mCurrentBlockFont;   ///< The current block font

                    int mImageRightPadding; ///< The image right padding
                    int mImageLeftPadding;  ///< The image left padding
                    int mInlineImageBottomBorder;   ///< The inline image bottom border

                    // print context related properties

                    double mPrintContextWidth;  ///< Width of the print context
                    double mPrintContextHeight; ///< Height of the print context

                    Smartmatic::SAES::Printing::GraphicContentPage::ColumnModeEnumType m_CurrentColumnLayout; ///< The current column layout

                    Glib::RefPtr<Pango::Context> mPrintPangoContext;	///< Context for the print pango



                    std::queue<Smartmatic::SAES::Printing::GraphicContentPage> m_Pages; ///< The pages


                    /**
                     * @fn	void GraphicContextPrintingBackend::mDrawRow ( std::vector<Glib::ustring>::iterator pRowContent ,
                     * 			std::vector<DocumentTextOptions> textContentOptions , int numCellsToRender);
                     * @brief	Draw row.
                     * @author	Cesar.bonilla
                     * @date	4/28/2011
                     * @param	pRowContent					The row content.
                     * @param	textContentOptions	Options for controlling the text content.
                     * @param	numCellsToRender		Number of cells to renders.
                     */

                    void mDrawRow (
                        std::vector<Glib::ustring>::iterator pRowContent
                        , std::vector<DocumentTextOptions> textContentOptions
                        , int numCellsToRender );

                    /**
                     * @fn	void GraphicContextPrintingBackend::mApplyTextFormat ( Glib::RefPtr<Pango::Layout> textLayout ,
                     * 			DocumentTextOptions& options);
                     * @brief	Applies the text format.
                     * @author	Cesar.bonilla
                     * @date	4/28/2011
                     * @param	textLayout	The text layout.
                     * @param	options			Options for controlling the operation.
                     */

                    void mApplyTextFormat (
                        Glib::RefPtr<Pango::Layout> textLayout
                        , DocumentTextOptions & options );

                    /**
                     * @fn	void GraphicContextPrintingBackend::mSetRowMargin ();
                     * @brief	Ms the row margin.
                     * @author	Cesar.bonilla
                     * @date	4/28/2011
                     */

                    void mSetRowMargin ();

                    /**
                     * @fn	void GraphicContextPrintingBackend::mPreApplyTextFormat( Glib::RefPtr<Pango::Layout> textLayout ,
                     * 			DocumentTextOptions& options);
                     * @brief	Pre apply text format.
                     * @author	Youssef.tourkmani
                     * @date	30/10/2012
                     * @param	textLayout			 	The text layout.
                     * @param [in,out]	options	Options for controlling the operation.
                     */
                    void mPreApplyTextFormat (
                        Glib::RefPtr<Pango::Layout> textLayout
                        , DocumentTextOptions & options );

            };

        }

    }

}

#endif /* GRAPHICCONTEXTPRINTINGBACKEND_H_ */
