/**
 * @file   IB31K1PrintingBackend.hxx
 * @brief  Declaration of IB31K1PrintingBackend class.
 * @author manuel.arguelles
 * @date   Sep  8, 2017
 */

#ifndef IB31K1_PRINTING_BACKEND_HXX
#define IB31K1_PRINTING_BACKEND_HXX

#include "DocumentOptions.h"
#include "IPrintingBackend.h"
#include <string>
#include <list>
#include <gtkmm.h>
#include <queue>

namespace Smartmatic
{
    namespace SAES
    {
        namespace Printing
        {
            /**
             * @class  IB31K1PrintingBackend
             * @brief  Direct printing backend for the IB31K1 printer.
             * @author manuel.arguelles
             * @date   Sep  8, 2017
             */
            class IB31K1PrintingBackend : public IPrintingBackend
            {
                public:

                    /**
                     * @brief  Creates an instance of the class.
                     * @author manuel.arguelles
                     * @date   Sep  8, 2017
                     */
                    IB31K1PrintingBackend();

                    /**
                     * @brief  Destroys an instance of the class.
                     * @author manuel.arguelles
                     * @date   Sep  8, 2017
                     */
                    virtual ~IB31K1PrintingBackend();

                public:

                    /**
                     * @brief  Initializes the object.
                     * @author manuel.arguelles
                     * @date   Sep  8, 2017
                     */
                    virtual void Init();

                    /**
                     * @brief  Ends the confection of the documents and ends it to the spooler/printer.
                     * @author manuel.arguelles
                     * @date   Sep  8, 2017
                     */
                    virtual void End();

                    /**
                     * @brief  Draws a string.
                     * @author manuel.arguelles
                     * @date   Sep  8, 2017
                     * @param  textContent           The text to draw.
                     * @param  textContentOptionList The options to use for drawing the text.
                     */
                    virtual void DrawString(std::list<Glib::ustring>& textContent,
                                            std::list<DocumentTextOptions>& textContentOptions);

                    /**
                     * @brief  Draws and image.
                     * @author manuel.arguelles
                     * @date   Sep  8, 2017
                     * @param  image The image to draw.
                     */
                    virtual void DrawImage(Glib::RefPtr<Gdk::Pixbuf> image);

                    /**
                     * @brief  Draw a QR code.
                     * @author manuel.arguelles
                     * @date   Sep  8, 2017
                     * @param  qrCode The QR code to draw.
                     */
                    virtual void DrawQRCode(QRCodeContent& qrCode);

                    /**
                     * @brief  Gets the result of the printing.
                     * @author manuel.arguelles
                     * @date   Sep  8, 2017
                     * @return The result.
                     */
                    virtual PrintDocumentResult getPrintResult();

                    /**
                     * @brief  Cleans the queue to print.
                     * @author manuel.arguelles
                     * @date   Sep  8, 2017
                     */
                    virtual void cleanPrintQueue();

                    /**
                     * @brief  Configure near end of paper sensor value.
                     * @author manuel.arguelles
                     * @date   Mar 16, 2018
                     */
                    void configureNeopSensor();

                    /**
                     * @brief  Update the printer status.
                     * @author manuel.arguelles
                     * @date   Apr 19, 2018
                     */
                    void updateStatus();

                private:

                    /**
                     * @brief  Adds a line feed.
                     * @author manuel.arguelles
                     * @date   Sep  8, 2017
                     */
                    void addLineFeedCommand();

                    /**
                     * @brief  Adds a full cut.
                     * @author manuel.arguelles
                     * @date   Nov 17, 2017
                     */
                    void addNewFullCutCommand();

                    /**
                     * @brief  Send the final commands required by configuration.
                     * @author manuel.arguelles
                     * @date   Nov 17, 2017
                     */
                    void addFinalCommands();

                    /**
                     * @brief  Adds a command to feed the paper.
                     * @author manuel.arguelles
                     * @date   Nov 17, 2017
                     * @param  dots The amount of dots (0.125 mm) to feed.
                     */
                    void addFeedPaperCommand(unsigned char dots);

                    /**
                     * @brief  Change the font attributes.
                     * @author manuel.arguelles
                     * @date   Sep 11, 2017
                     * @param  fontSize  The new font size.
                     * @param  fontStyle The new font style.
                     */
                    void changeFont(DocumentFontSize fontSize, DocumentFontStyle fontStyle);

                    /**
                     * @brief  Change the text justification.
                     * @author manuel.arguelles
                     * @date   Sep 11, 2017
                     * @param  justification The new justification to use.
                     */
                    void changeJustification(DocumentJustifitacion justification);

                    /**
                     * @brief  Change the text options.
                     * @author manuel.arguelles
                     * @date   Sep 11, 2017
                     * @param  options              The new text options to use.
                     * @param  includeJustification If justification should be set or not.
                     */
                    void updateTextOptions(DocumentTextOptions& options, const bool includeJustification);

                    /**
                     * @brief  Draw text in a column or cell.
                     * @author manuel.arguelles
                     * @date   Sep 11, 2017
                     * @param  options The document text options to use.
                     * @param  text    The text to draw.
                     * @param  columns The amount of columns.
                     */
                    void drawCell(DocumentTextOptions& options, const std::string& text, int columns);

                    /**
                     * @brief  Push the current file to the queue and use a new one.
                     * @author manuel.arguelles
                     * @date   Sep 11, 2017
                     */
                    void changeFile();

                    /**
                     * @brief  Change the left margin
                     * @author manuel.arguelles
                     * @date   Nov 20, 2017
                     */
                    void changeLeftMargin(unsigned char dots);

                    /**
                     * @brief  Change the max characters per line.
                     * @author manuel.arguelles
                     * @date   Dec  5, 2017
                     * @param  characters The maximum number of characters.
                     */
                    void changeCharactersPerLine(unsigned char characters);

                    /**
                     * @brief  Draw a QR code by sending it directly to the
                     * printer.
                     * @author manuel.arguelles
                     * @date   Dec  6, 2017
                     * @param  qr The qr code.
                     */
                    void DrawQRCodeBin(QRCodeContent& qr);

                private:

                    PrintDocumentResult     m_overallResult;
                    std::queue<std::string> m_files;
                    std::string             m_currentFile;
                    DocumentTextOptions     m_textOptions;
            };

        } /* namespace Printing */
    } /* namespace SAES */
} /* namespace Smartmatic */

#endif /* IB31K1_PRINTING_BACKEND_HXX */
