/**
 * @file	PrintDocumentResult.h
 * @brief	Declares the print document result class.
 * @author	Youssef.tourkmani
 * @date	30/10/2012
 */

#ifndef PRINTDOCUMENTRESULT_H_
#define PRINTDOCUMENTRESULT_H_

#include <string>
#include <Printing/PrinterStatusNotifier.h>

namespace Smartmatic
{
    namespace SAES
    {
        namespace Printing
        {
            /**
             * @class	PrintDocumentResult
             * @brief	Print document result.
             * @author	Cesar.bonilla
             * @date	4/28/2011
             */

            class PrintDocumentResult
            {
                public:

                    /**
                     * @typedef	enum PrintingResultEnum
                     * @brief	Defines an alias representing the printing result enum.
                     */
                    typedef enum PrintingResultEnum
                    {
                        // Cups job related error codes

                        PRINTING_SERVER_ERROR,     /* Error has occurred while attempting status retrieval */
                        PRINTING_NO_JOBS,          /* No print request has been made so far*/
                        PRINTING_NO_PRINTER_FOUND, /* No printer was found */
                        PRINTING_PENDING,          /* Job is waiting to be printed */
                        PRINTING_HELD,             /* Job is waiting to be printed */
                        PRINTING_PROCESSING,       /* Job is currently being printed */
                        PRINTING_STOPPED,          /* Job has been stopped */
                        PRINTING_CANCELED,         /* Job has been canceled */
                        PRINTING_ABORTED,          /* Job has aborted due to error */
                        PRINTING_COMPLETED,        /* Job has completed successfully */

                        // Gtk print related error codes

                        PRINTING_INTERNAL_ERROR,
                        PRINTING_PREPARING,
                        PRINTING_GENERATING_DATA,
                        PRINTING_SENDING_DATA,
                        PRINTING_NONE,

                        // Device related error codes

                        PRINTING_DEVICE_ERROR,     /* Hardware level error condition */

                        // Document creation logic related errors

                        PRINTING_UNEXPECTED_ERROR,
                        PRINTING_PREPARING_ERROR,

                        // Status receiving logic

                        PRINTING_JOB_NOT_SENT,
                        PRINTING_JOB_ABORTED_BY_SYSTEM

                    } PrintingResultEnumType; ///< Type of the printing result enum

                    /**
                     * @fn	PrintDocumentResult::PrintDocumentResult();
                     * @brief	Default constructor.
                     * @author	Cesar.bonilla
                     * @date	4/28/2011
                     */

                    PrintDocumentResult();

                    /**
                     * @fn	PrintDocumentResult::PrintDocumentResult ( const PrintDocumentResult & object );
                     * @brief	Default constructor.
                     * @author	Yadickson Soto
                     * @date	03/08/2015
                     * @param	object The object to get copy
                     */
                    PrintDocumentResult ( const PrintDocumentResult & object );

                    /**
                     * @fn	PrintDocumentResult::PrintDocumentResult(
                     * 	bool success,
                     * 	const std::string & printerStatus,
                     * 	PrintingResultEnum printResult,
                     * 	PrinterStatusNotifier::PhysicalStateEnumType printerStatusNotifier = PrinterStatusNotifier::PSTATE_NONE);
                     * @brief	Constructor.
                     * @author	Cesar.bonilla
                     * @date	4/28/2011
                     * @param	success			true if the operation was a success, false if it failed.
                     * @param	printerStatus	The printer status.
                     * @param	printResult		The print result.
                     * @param 	printerStatusNotifier The printer core status result
                     */
                    PrintDocumentResult ( bool success,
                                          const std::string & printerStatus,
                                          PrintingResultEnum printResult,
                                          PrinterStatusNotifier::PhysicalStateEnumType printerStatusNotifier = PrinterStatusNotifier::PSTATE_NONE );
                    /**
                     * @fn	virtual PrintDocumentResult::~PrintDocumentResult();
                     * @brief	Finaliser.
                     * @author	Cesar.bonilla
                     * @date	4/28/2011
                     */
                    virtual ~PrintDocumentResult();
                    /**
                     * @fn	void PrintDocumentResult::setResult(bool wasOk, const std::string & printStatus,
                     * 			PrintingResultEnum printResult, PrinterStatusNotifier::PhysicalStateEnumType printerStatusNotifier );
                     * @brief	Sets a result.
                     * @author	Yadickson Soto
                     * @date	13/01/2014
                     * @param	status	true if the was operation was a success, false if it failed.
                     * @param	printStatus	The print status.
                     * @param	printResult	The print result.
                     * @param 	printerStatusNotifier The printer core status result
                     */
                    void setResult ( bool status,
                                     const std::string & printStatus,
                                     PrintingResultEnum printResult,
                                     PrinterStatusNotifier::PhysicalStateEnumType printerStatusNotifier = PrinterStatusNotifier::PSTATE_NONE );

                    friend class BasePrintDocument; ///< The base print document

                    /**
                     * @fn	bool PrintDocumentResult::getSuccess();
                     * @brief	Gets the success.
                     * @author	Youssef.tourkmani
                     * @date	30/10/2012
                     * @return	true if it succeeds, false if it fails.
                     */
                    bool getSuccess();
                    /**
                     * @fn	std::string PrintDocumentResult::getPrinterStatusDetail();
                     * @brief	Gets printer status detail.
                     * @author	Youssef.tourkmani
                     * @date	30/10/2012
                     * @return	The printer status detail.
                     */
                    std::string getPrinterStatusDetail();
                    /**
                     * @fn	PrintingResultEnumType PrintDocumentResult::getPrintingResult();
                     * @brief	Gets printing result.
                     * @author	Youssef.tourkmani
                     * @date	30/10/2012
                     * @return	The printing result.
                     */
                    PrintingResultEnumType getPrintingResult();
                    /**
                     * @fn	PrinterStatusNotifier::PhysicalStateEnumType PrintDocumentResult::getPrinterStatusNotifierResult();
                     * @brief	Gets printer core status result.
                     * @author	Yadickson Soto
                     * @date	13/01/2014
                     * @return	The printer core status result.
                     */
                    PrinterStatusNotifier::PhysicalStateEnumType getPrinterStatusNotifierResult();
                    /**
                     * @fn  PrintDocumentResult operator = ( const PrintDocumentResult & obj );
                     * @brief   Operator equal
                     * @author  eduardo.fernandez
                     * @date    21/01/2019
                     * @param   obj Object
                     * @return  return object
                     */
                    PrintDocumentResult & operator = ( const PrintDocumentResult & obj );

                private:

                    bool wasSuccess;	///< true if the was operation was a success, false if it failed

                    std::string printerStatusDetail;  ///< The printer status detail
                    PrintingResultEnumType printingResult;  ///< The printing results
                    PrinterStatusNotifier::PhysicalStateEnumType printerStatusNotifierResult; ///< The printer core status result
            };
        }
    }
}

#endif /* PRINTDOCUMENTRESULT_H_ */
