/**
 * @file	PrinterStatusNotifier.h
 * @brief	Declares the printer status notifier class.
 * @author	Youssef.tourkmani
 * @date	30/10/2012
 */


#ifndef PRINTERSTATUSNOTIFIER_H_
#define PRINTERSTATUSNOTIFIER_H_

#include <string>
#include <Configuration/s-a-e-s-common-conf-manager.h>

namespace Smartmatic
{
    namespace Log
    {
        class ISMTTLog;
    }
    namespace SAES
    {
        namespace Printing
        {
            /**
             * @class	PrinterStatusNotifier
             * @brief	Printer status notifier.
             * @author	Youssef.tourkmani
             * @date	30/10/2012
             */
            class PrinterStatusNotifier
            {
                public:

                    /**
                     * @typedef	enum PhysicalStateEnum
                     * @brief	Defines an alias representing the physical state enum.
                     */
                    typedef enum PhysicalStateEnum
                    {
                        PSTATE_NONE = 0,
                        PSTATE_READY = 1,
                        PSTATE_OUT_OF_PAPER = 2,
                        PSTATE_PROCESSING = 4,
                        PSTATE_PAPER_JAM = 8,
                        PSTATE_COVER_OPEN = 16,
                        PSTATE_FEEDER_OPEN = 32,
                        PSTATE_OFFLINE = 64,
                        PSTATE_REJECTING_JOBS = 128,
                        PSTATE_NEAR_END_OF_PAPER = 256,

                        /* President machine printer only */

                        PSTATE_SUPPLYLOW_WARN = 512,
                        PSTATE_OUT_OF_TONER = 1024,
                        PSTATE_CANCELING = 2048,
                        PSTATE_CARTRIDGE_NOT_PRESENT = 4096,
                        PSTATE_UNKNOWN = 8192

                    } PhysicalStateEnumType; ///< Type of the physichal state enum


                    // Life cycle

                    /**
                     * @fn	PrinterStatusNotifier::PrinterStatusNotifier();
                     * @brief	Default constructor.
                     * @author	Cesar.bonilla
                     * @date	4/28/2011
                     */

                    PrinterStatusNotifier();

                    /**
                     * @fn	virtual PrinterStatusNotifier::~PrinterStatusNotifier();
                     * @brief	Finaliser.
                     * @author	Cesar.bonilla
                     * @date	4/28/2011
                     */

                    virtual ~PrinterStatusNotifier();


                    // Access

                    /**
                     * @fn	const std::string& PrinterStatusNotifier::getStatusMessage (bool useCachedValue = false);
                     * @brief	returns comma separated, human readable string value for each state detected.
                     * @author	Youssef.tourkmani
                     * @date	30/10/2012
                     * @param	useCachedValue	(optional) the use cached value.
                     * @return	The status message.
                     */
                    const std::string & getStatusMessage ( bool useCachedValue = false );


                    /**
                     * @fn	PhysicalStateEnumType PrinterStatusNotifier::getPrinterState (bool useCachedValue = false);
                     * @brief	returns a flagged type containing all detected state bits on, including warnings and the
                     * 				"processing" state.
                     * @author	Youssef.tourkmani
                     * @date	30/10/2012
                     * @param	useCachedValue	(optional) the use cached value.
                     * @return	The printer state.
                     */
                    PhysicalStateEnumType getPrinterState ( bool useCachedValue = false );

                    /**
                     * @fn	const std::string PrinterStatusNotifier::getStatusMessage (PhysicalStateEnumType state);
                     * @brief	Gets status message.
                     * @author	Youssef.tourkmani
                     * @date	30/10/2012
                     * @param	state	The state.
                     * @return	The status message.
                     */
                    const std::string getStatusMessage ( PhysicalStateEnumType state ); // TODO: Legacy. remove.
                    /**
                     * @fn	const std::string PrinterStatusNotifier::getCurrentStatusMessage ();
                     * @brief	Gets current status message.
                     * @author	Youssef.tourkmani
                     * @date	30/10/2012
                     * @return	The current status message.
                     */
                    const std::string getCurrentStatusMessage ();

                    /**
                     * @fn	int PrinterStatusNotifier::getErrorCondition () const;
                     * @brief	returns a flagged type with all error conditions of the current state bits on (processing
                     * 				is not considered an error condition). Can be used as a mask for the state obtained from
                     * 				"getPrinterState" to extract errors when warnings and "printer ready" state values are
                     * 				not of interest.
                     * @author	Youssef.tourkmani
                     * @date	30/10/2012
                     * @return	The error condition.
                     */
                    int getErrorCondition () const;

                    /**
                     * @fn	void PrinterStatusNotifier::setExternalStatusTrigger (bool arg);
                     * @brief	Sets a use print black resources.
                     * @author	Cesar.bonilla
                     * @date	4/28/2011
                     * @param	arg	true to argument.
                     */

                    void setExternalStatusTrigger ( bool arg );
                    /**
                     * @fn	static void PrinterStatusNotifier::CheckPrinterState (bool check);
                     * @brief	Check printer state.
                     * @author	Youssef.tourkmani
                     * @date	30/10/2012
                     * @param	check	true to check.
                     */
                    static void CheckPrinterState ( bool check );
                    /**
                     * @fn	static void PrinterStatusNotifier::runPrintBlank();
                     * @brief	Executes the print blank operation.
                     * @author	Youssef.tourkmani
                     * @date	30/10/2012
                     */
                    static void runPrintBlank();

                    /**
                     * @brief  Executes the print blank operation.
                     * @author manuel.arguelles
                     * @date   Nov 18, 2017
                     */
                    static void runForcedPrintBlank();

                protected:

                private:

                    static bool CheckCurrentState;	///< true to check current state
                    Smartmatic::SAES::Common::Configuration::PrinterBackendType printerBackendType; ///< printer backend type

                    PhysicalStateEnumType mPhState;	///< State of the ph
                    std::string mStatusMessage;   ///< Message describing the status
                    bool externalStatusTrigger;	///< true to use external status trigger

                    /**
                     * @fn	void PrinterStatusNotifier::mRefreshStatusMessage (bool skipBlankPage = false);
                     * @brief	Refresh status message.
                     * @author	Cesar.bonilla
                     * @date	4/28/2011
                     * @param	skipBlankPage	true to skip blank page.
                     */

                    void mRefreshStatusMessage ( bool skipBlankPage = false );
                    /**
                     * @fn	const std::string PrinterStatusNotifier::mapStatusMessage (PhysicalStateEnumType state);
                     * @brief	Map status message.
                     * @author	Youssef.tourkmani
                     * @date	30/10/2012
                     * @param	state	The state.
                     * @return	.
                     */
                    const std::string mapStatusMessage ( PhysicalStateEnumType state );

                    static Smartmatic::Log::ISMTTLog * logger; ///< The logger
            };

        }

    }

}

#endif /* PRINTERSTATUSNOTIFIER_H_ */

