/**
 * @file	string-resources-manager-base.cc
 * @brief	string resources manager base class.
 * @author	Youssef.tourkmani
 * @date	30/10/2012
 */


#include "Resources/string-resources-manager-base.h"
#include <Security/Encryption.hxx>
#include <System/Utils/StringUtils.hxx>
#include <System/IO/Directory.h>
#include <System/IO/Path.h>
#include <Log/SMTTLogManager.h>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

#define N_(String) (String) ///<macro for gettext not translate

using namespace Smartmatic::System::IO;
using namespace Smartmatic::SAES::Resources;
using namespace Smartmatic::Log;
using namespace Smartmatic::System::Utils;

ISMTTLog * StringResourcesManagerBase::logger = SMTTLogManager::GetLogger ( "Smartmatic.SAES.Resources.StringResourcesManagerBase", GETTEXT_PACKAGE );

void StringResourcesManagerBase::Initialize()
{
    Dispose();
    std::vector<std::string> files = std::vector<std::string>();

    if ( Directory::getInstance()->DirectoryExists ( this->getFullFolderPath() ) )
    {
        logger->Debug ( "Open directory: " + this->getFullFolderPath() );
    }
    else
    {
        logger->Debug ( "Directory " + this->getFullFolderPath() + " not found." );
        return;
    }

    Directory::getInstance()->GetFiles ( this->getFullFolderPath(), files );
    int size = files.size();
    bool errorLoading = false;
    bool fileLoaded = false;

    for ( int i = 0; i < size; i++ )
    {
        std::string fileToLook = this->getFileName();
        fileToLook.erase ( fileToLook.end() - 4, fileToLook.end() );
        std::string fileLooked = files[i];
        std::transform ( fileToLook.begin(), fileToLook.end(), fileToLook.begin(), ::toupper );
        std::transform ( fileLooked.begin(), fileLooked.end(), fileLooked.begin(), ::toupper );
        int pos = fileLooked.find ( fileToLook );

        if ( pos >= 0 )
        {
            try
            {
                std::string file = Path::Combine ( this->getFullFolderPath(), files[i] );
                logger->Debug ( "Open file: " + file );
                std::string data = Smartmatic::SAES::Security::Encryption::getInstance()->openXmlFile ( file, Smartmatic::SAES::Security::Encryption::RESOURCE_FILE );
                std::istringstream istr ( data );
                std::auto_ptr< Strings_resources> resources =  parseStrings_resources ( istr, ResourcesTypes::Flags::dont_validate );
                files[i].erase ( files[i].begin(), files[i].begin() + fileToLook.size() );
                files[i].erase ( files[i].end() - 4, files[i].end() );
                std::string culture = StringUtils::toLower ( files[i] );
                std::map<std::string, std::string> * cultureResources = new std::map<std::string, std::string>();
                Strings_resources::SrIterator iterator ( resources->getSr().begin() );
                Strings_resources::SrIterator enditerator ( resources->getSr().end() );
                std::stringstream ss;
                ss << "Load culture: " << culture << " - number resources keys: " << resources->getSr().size();
                logger->Debug ( ss.str() );

                while ( iterator != enditerator )
                {
                    const Sr & resource ( *iterator++ );
                    ( *cultureResources ) [resource.getKey()] = resource.getValue();
                }

                cultureMap[culture] = cultureResources;
                fileLoaded = true;
            }
            catch ( const ::xsd::cxx::tree::exception< char > & e )
            {
                errorLoading = true;
                std::stringstream ss;
                ss << "FATAL - " << __func__ << " " << e;
                logger->Debug ( ss.str() );
            }
            catch ( Smartmatic::System::Exception::CryptoException & e )
            {
                errorLoading = true;
                std::stringstream ss;
                ss << "FATAL - " << __func__ << " " << e.what();
                logger->Debug ( ss.str() );
                throw;
            }
            catch ( Smartmatic::System::Exception::SignedException & e )
            {
                errorLoading = true;
                std::stringstream ss;
                ss << "FATAL - " << __func__ << " " << e.what();
                logger->Debug ( ss.str() );
            }
            catch ( Smartmatic::System::Exception::XmlException & e )
            {
                errorLoading = true;
                std::stringstream ss;
                ss << "FATAL - " << __func__ << " " << e.what();
                logger->Debug ( ss.str() );
            }
            catch ( ... )
            {
                errorLoading = true;
                std::stringstream ss;
                ss << "FATAL - " << __func__;
                logger->Debug ( ss.str() );
            }
        }
    }

    if ( fileLoaded == true  && errorLoading == false )
    {
        resourcesLoaded = true;
    }

    //job done
}

void StringResourcesManagerBase::Dispose()
{
    std::map<std::string , std::map<std::string, std::string>* >::iterator it;

    for ( it = cultureMap.begin();
            it != cultureMap.end();
            ++it )
    {
        std::map<std::string, std::string> * resources = ( *it ).second;
        delete resources;
        resources = NULL;
    }

    cultureMap.clear();
}

const std::string & StringResourcesManagerBase::getResource ( const std::string & key, const std::string & culture ) const
{
    std::string fixedCulture = StringUtils::toLower ( culture );
    std::map<std::string , std::map<std::string, std::string>* >::const_iterator cultureResources = cultureMap.find ( fixedCulture );
    std::map<std::string, std::string> * resources = NULL;

    if ( cultureResources != cultureMap.end() )
    {
        resources = cultureResources->second;
    }

    if ( resources != NULL && ( *resources ).find ( key ) != ( *resources ).end() )
    {
        return ( *resources ).find ( key )->second;
    }
    else
    {
        std::map<std::string , std::map<std::string, std::string>* >::const_iterator cultureCurrentResources = cultureMap.find ( getCurrentCulture() );

        if ( cultureCurrentResources != cultureMap.end() )
        {
            resources = cultureCurrentResources->second;
        }

        if ( resources != NULL && ( *resources ).find ( key ) != ( *resources ).end() )
        {
            return ( *resources ).find ( key )->second;
        }
        else
        {
            std::map<std::string , std::map<std::string, std::string>* >::const_iterator cultureDefaultResources = cultureMap.find ( getDefaultCulture() );

            if ( cultureDefaultResources != cultureMap.end() )
            {
                resources = cultureDefaultResources->second;
            }

            if ( resources != NULL && ( *resources ).find ( key ) != ( *resources ).end() )
            {
                return ( *resources ).find ( key )->second;
            }
            else
            {
                std::map<std::string , std::map<std::string, std::string>* >::const_iterator cultureGlobalResources = cultureMap.find ( "" );

                if ( cultureGlobalResources != cultureMap.end() )
                {
                    resources = cultureGlobalResources->second;
                }

                if ( resources != NULL && ( *resources ).find ( key ) != ( *resources ).end() )
                {
                    return ( *resources ).find ( key )->second;
                }
                else
                {
                    return key;
                }
            }
        }
    }
}

void StringResourcesManagerBase::setCurrentCulture ( const std::string & newCulture )
{
    currentCulture = StringUtils::toLower ( newCulture );
}

StringResourcesManagerBase::~StringResourcesManagerBase()
{
    Dispose();
}
