/**
 * @file	string-resources-manager-base.h
 * @brief	Declaration of abstract manager of the string resources.
 * @author	Youssef.tourkmani
 * @date	30/10/2012
 */


#ifndef _STRING_RESOURCES_MANAGER_BASE_H_
#define _STRING_RESOURCES_MANAGER_BASE_H_

#include <string>
#include <map>
#include <vector>
#include <Resources/strings-resources-schema.hxx>
#include <Log/ISMTTLog.h>

namespace Smartmatic
{
    namespace SAES
    {
        namespace Resources
        {
            /**
             * @class	StringResourcesManagerBase
             * @brief	Abstract collection which contains the map the string resources index according to its culture.
             * Additionally exposes the getters that provide the file name and full path of the string resources.
             * @author	Cesar.bonilla
             * @date	4/28/2011
             */

            class StringResourcesManagerBase
            {
                public:

                    /**
                     * @fn	virtual std::string & StringResourcesManagerBase::getFileName() = 0;
                     * @brief	Gets the file name.
                     * @author	Cesar.bonilla
                     * @date	4/28/2011
                     * @return	The file name.
                     */

                    virtual std::string & getFileName() = 0;

                    /**
                     * @fn	virtual std::string & StringResourcesManagerBase::getFullFolderPath() = 0;
                     * @brief	Gets the full folder path that contains the resources files.
                     * @author	Cesar.bonilla
                     * @date	4/28/2011
                     * @return	The reference to full folder path.
                     */

                    virtual std::string & getFullFolderPath() = 0;

                    /**
                     * @fn	void StringResourcesManagerBase::setCurrentCulture(const std::string& newCulture);
                     * @brief	Sets a current culture.
                     * @author	Cesar.bonilla
                     * @date	4/28/2011
                     * @param	newCulture	The new culture.
                     */

                    void setCurrentCulture ( const std::string & newCulture );

                    /**
                     * @fn	const std::string & StringResourcesManagerBase::getCurrentCulture() const
                     * @brief	Gets the current culture.
                     * @author	Cesar.bonilla
                     * @date	4/28/2011
                     * @return	The current culture.
                     */

                    const std::string & getCurrentCulture() const
                    {
                        return currentCulture;
                    }

                    /**
                     * @fn	void StringResourcesManagerBase::setDefaultCulture(const std::string& newDefault)
                     * @brief	Sets a default culture.
                     * @author	Cesar.bonilla
                     * @date	4/28/2011
                     * @param	newDefault	The new default.
                     */

                    void setDefaultCulture ( const std::string & newDefault )
                    {
                        defaultCulture = newDefault;
                    }

                    /**
                     * @fn	const std::string & StringResourcesManagerBase::getDefaultCulture() const
                     * @brief	Gets the default culture.
                     * @author	Cesar.bonilla
                     * @date	4/28/2011
                     * @return	The default culture.
                     */

                    const std::string & getDefaultCulture() const
                    {
                        return defaultCulture;
                    }

                    /**
                     * @fn	virtual std::string StringResourcesManagerBase::getResource(const std::string & key)
                     * @brief	Gets a resource.
                     * @author	Yadickson Soto
                     * @date	22/11/2013
                     * @param	key	The key.
                     * @return	The resource.
                     */

                    virtual std::string getResource ( const std::string & key )
                    {
                        return getResource ( key, currentCulture );
                    }

                    /**
                     * @fn	virtual const std::string & StringResourcesManagerBase::getResource(const std::string & key,
                     * 			const std::string & culture) const;
                     * @brief	Gets a resource.
                     * @author	Yadickson Soto
                     * @date	22/11/2013
                     * @param	key		 	The key.
                     * @param	culture	The culture.
                     * @return	The resource.
                     */

                    virtual const std::string & getResource ( const std::string & key, const std::string & culture ) const;

                    /**
                     * @fn	virtual StringResourcesManagerBase::~StringResourcesManagerBase();
                     * @brief	Finalizes an instance of the StringResourcesManagerBase class.
                     * @author	Cesar.bonilla
                     * @date	4/28/2011
                     */

                    virtual ~StringResourcesManagerBase();
                protected:

                    /**
                     * @fn	void StringResourcesManagerBase::Initialize();
                     * @brief	Initializes the collection with the string resources obtained from the files in the directory.
                     * @author	Cesar.bonilla
                     * @date	4/28/2011
                     */

                    void Initialize();

                    /**
                     * @fn	void StringResourcesManagerBase::Dispose();
                     * @brief	Disposes the resource map.
                     * @author	Yadickson Soto
                     * @date	05/11/2012
                     */

                    void Dispose();

                    /**
                     * @fn	StringResourcesManagerBase::StringResourcesManagerBase()
                     * @brief	Default constructor initializes an instance of the SmartcardConnected class.
                     * @author	Cesar.bonilla
                     * @date	4/28/2011
                     */

                    StringResourcesManagerBase() :
                        currentCulture ( "en" ), defaultCulture ( "en" ), resourcesLoaded ( false )
                    {}

                protected:

                    std::map<std::string , std::map<std::string, std::string>* > cultureMap; ///< The map the string resources index according to its culture
                    std::string currentCulture;  ///< The current culture

                private:


                    std::string defaultCulture;  ///< The default culture
                    bool resourcesLoaded;   ///< This flag represents if was loaded the resources.
                    static Smartmatic::Log::ISMTTLog * logger; /**< The reference pointer of a logger instance.*/
            };
        }
    }
}

#endif // _STRING_RESOURCES_MANAGER_BASE_H_
