/**
 * @file	DirectoryManager.cxx
 * @brief	directory manager class.
 * @author	Youssef.tourkmani
 * @date	30/10/2012
 */

#include "DirectoryManager.hxx"
#include <System/IO/Path.h>
#include <System/IO/File.h>
#include <System/IO/Directory.h>
#include <Log/SMTTLogManager.h>
#include <System/Runtime/ProcessLauncher.h>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

#define N_(String) (String) ///<macro for gettext not translate

using namespace Smartmatic::SAES::Runtime::Environment;
using namespace Smartmatic::SAES::Security;
using namespace Smartmatic::System::Exception;
using namespace Smartmatic::SAES::Common::Configuration;
using namespace Smartmatic::Log;
using namespace Smartmatic::System::Runtime;
using namespace Smartmatic::System::IO;

ISMTTLog * DirectoryManager::logger = SMTTLogManager::GetLogger ( "Smartmatic.SAES.Runtime.Environment.DirectoryManager", GETTEXT_PACKAGE );

DirectoryManager::DirectoryManager ( Smartmatic::SAES::Runtime::Environment::SaesDirectories & directories )
    : directories(directories)
{
}

DirectoryManager::~DirectoryManager()
{
}

bool DirectoryManager::backup ( bool copyDomToFlash )
{
    std::string languageImageDirectory = directories.getLanguageImageInstalationDirectoryFullPath();
    std::string languageImageDirectoryLocal = directories.getLanguageImageDirectoryFullPath();
    bool success = true;

    if ( Directory::getInstance()->DirectoryExists ( languageImageDirectory ) )
    {
        success = copyDir ( languageImageDirectory, languageImageDirectoryLocal ) && success;
    }

    std::string masterDir = directories.getBasePath ( BasePathType::CONFIGURATION_DOM );
    std::string backupDir = directories.getBasePath ( BasePathType::CONFIGURATION_FLASH );

    if ( copyDomToFlash )
    {
        success = copyDir ( masterDir, backupDir ) && success;
    }
    else
    {
        success = copyDir ( backupDir, masterDir ) && success;
    }

    return success;
}

bool DirectoryManager::copyDir ( std::string origDir, std::string destDir, bool removeDestDir )
{
    bool result = true;

    if ( removeDestDir )
    {
        result = Directory::getInstance()->Delete(destDir);
    }

    if ( result )
    {
        result = File::getInstance()->copyDirectory ( origDir, destDir );
        Directory::getInstance()->addAllPermission ( destDir );
    }

    return result;
}

void DirectoryManager::createMachineIdFile ( std::string machineId, bool force )
{
    std::string machineIdPath = directories.getBackupFileFullPath ( BasePathType::CONFIGURATION_DOM );
    std::string machineIdFlashPath = directories.getBackupFileFullPath ( BasePathType::CONFIGURATION_FLASH );

    if ( !File::getInstance()->canOpenFile ( machineIdPath ) || force )
    {
        try
        {
            Encryption::getInstance()->cryptoStringToFile ( machineId, machineIdPath, Encryption::INTERNAL_FILE );
        }
        catch ( CryptoException & ex )
        {
            std::stringstream ss;
            ss << __func__ << " - ERROR " << machineIdPath << " " << ex.what();
            logger->Debug ( ss.str() );
        }
    }

    if ( force )
    {
        try
        {
            Encryption::getInstance()->cryptoStringToFile ( machineId, machineIdFlashPath, Encryption::INTERNAL_FILE );
        }
        catch ( CryptoException & ex )
        {
            std::stringstream ss;
            ss << __func__ << " - ERROR " << machineIdFlashPath << " " << ex.what();
            logger->Debug ( ss.str() );
        }
    }
}

bool DirectoryManager::validateMachineIdFile ( const std::string& machineId )
{
    std::string machineIdFilePath = directories.getBackupFileFullPath ( BasePathType::CONFIGURATION_DOM );
    std::string machineIdData;
    bool validate = false;

    try
    {
        Encryption::getInstance()->decryptoFileToString ( machineIdFilePath, machineIdData,
                Encryption::INTERNAL_FILE );
        validate = machineIdData == machineId;
    }
    catch ( CryptoException & ex )
    {
        std::stringstream ss;
        ss << __func__ << " - ERROR " << machineIdFilePath << " " << ex.what();
        logger->Debug ( ss.str() );
    }

    if ( validate )
    {
        logger->Debug ( "Validate Machine ID Success" );
    }
    else
    {
        logger->Debug ( "Validate Machine ID Error" );
    }

    return validate;
}


bool DirectoryManager::validateMachineIdFileDOMFlash ()
{
    std::string backupFileM = directories.getBackupFileFullPath ( BasePathType::CONFIGURATION_DOM );
    std::string backupFileB = directories.getBackupFileFullPath ( BasePathType::CONFIGURATION_FLASH );
    bool validate = File::getInstance()->compareFiles ( backupFileM, backupFileB );

    if ( !validate )
    {
        logger->Debug ( "MachineId file Faild" );
    }
    else
    {
        logger->Debug ( "MachineId file Ok" );
    }

    return validate;
}
