/**
 * @file	MemoryPathController.h
 * @brief	Declares the memory path controller class.
 * @author	Youssef.tourkmani
 * @date	30/10/2012
 */


#ifndef MEMORYPATHCONTROLLER_H_
#define MEMORYPATHCONTROLLER_H_

#include <string>
#include <Configuration/s-a-e-s-common-conf-manager.h>
#include <set>
#include <glibmm.h>

namespace Smartmatic
{
    namespace Log
    {
        class ISMTTLog;
    }

    namespace SAES
    {
        namespace Runtime
        {
            namespace Environment
            {
                /**
                 * @class	MemoryPathController
                 * @brief	Memory Path Manager.
                 * @author	Freddy.Gomez
                 * @date	18/05/2011
                 */

                class MemoryPathController
                {
                    public:

                        /**
                         * @fn	MemoryPathController::MemoryPathController();
                         * @brief	Default constructor.
                         * @author	Freddy.Gomez
                         * @date	18/05/2011
                         */
                        MemoryPathController();

                        /**
                         * @fn	virtual MemoryPathController::~MemoryPathController();
                         * @brief	Finalizer.
                         * @author	Freddy.Gomez
                         * @date	18/05/2011
                         */
                        virtual ~MemoryPathController();

                        /**
                         * @fn	std::string& MemoryPathController::getStartupMemoryBasePath();
                         * @brief	Get the path of the startup meomry.
                         * @author	Freddy.Gomez
                         * @date	18/05/2011
                         * @return	the path of the startup memory.
                         */
                        std::string & getStartupMemoryBasePath();

                        /**
                         * @fn	std::string& MemoryPathController::getLiveFileSystemBasePath();
                         * @brief	Get the path of the memory mapped file system.
                         * @author	Freddy.Gomez
                         * @date	18/05/2011
                         * @return	the path of live file system.
                         */
                        std::string & getLiveFileSystemBasePath();

                        /**
                         * @fn  std::string MemoryPathController::getIssuerCertificatesFolderFullPath();
                         * @brief   Get the path of the issuer certificates folder.
                         * @author  eduardo.fernandez
                         * @date    29/11/2018
                         * @return  The path of the issuer certificates folder.
                         */
                        std::string getIssuerCertificatesFolderFullPath();

                        /**
                         * @fn	std::string& MemoryPathController::getDomMemoryBasePath();
                         * @brief	Get the path of the dom memory.
                         * @author	Freddy.Gomez
                         * @date	18/05/2011
                         * @return	The path of the Dom memory, if connected.
                         */
                        std::string & getDomMemoryBasePath();

                        /**
                         * @fn	std::string& MemoryPathController::getFlashMemoryBasePath();
                         * @brief	Get the path of the flash memory.
                         * @author	Freddy.Gomez
                         * @date	18/05/2011
                         * @return	The path of the Flash memory, if connected.
                         */
                        std::string & getFlashMemoryBasePath();

                        /**
                         * @fn	std::string& MemoryPathController::getBackupMemoryBasePath();
                         * @brief	Get the path of the backup memory.
                         * @author	Yadickson Soto
                         * @date	14/05/2012
                         * @return	The path of the backup memory.
                         */
                        std::string & getBackupMemoryBasePath();

                        /**
                         * @fn	static MemoryPathController * MemoryPathController::getInstance();
                         * @brief	Get the unique instance of the class.
                         * @author	Freddy.Gomez
                         * @date	18/05/2011
                         * @return	The instance of the class.
                         */
                        static MemoryPathController * getInstance();
                        /**
                         * @fn	Smartmatic::SAES::Common::Configuration::RulesBasePathType & MemoryPathController::getRule();
                         * @brief	method getter rules config.
                         * @author	Yadickson Soto
                         * @date	04/06/2012
                         * @return	The rule.
                         */
                        Smartmatic::SAES::Common::Configuration::RulesBasePathType & getRule();
                        /**
                         * @fn	bool MemoryPathController::isAllDevices();
                         * @brief	method getter is all devices.
                         * @author	Yadickson Soto
                         * @date	04/06/2012
                         * @return	true is all devices.
                         */
                        bool isAllDevices();

                        /**
                         * @fn	void addPath( std::string path );
                         * @brief	Add path
                         * @author	Yadickson Soto
                         * @date	18/12/2012
                         * @param path add path
                         */
                        void addPath ( std::string path );

                        /**
                         * @fn	void removePath( const std::string& path );
                         * @brief	Remove path
                         * @author	Yadickson Soto
                         * @date	18/12/2012
                         * @param path remove path
                         */
                        void removePath ( const std::string& path );

                        /**
                         * @fn	void MemoryPathController::updatePaths();
                         * @brief	Update the Paths registered.
                         * @author	Freddy.Gomez
                         * @date	18/05/2011
                         */
                        void updatePaths();

                    private:

                        static MemoryPathController * instance; ///< instance of class MemoryPathController
                        std::string liveFileSystemBasePath; ///< path of the live folder
                        std::string issuerCertificatesFolderFullPath; ///< path of the issuer certificates folder.
                        std::string domMemoryBasePath; ///< path of the DOM memory
                        std::string flashMemoryBasePath;///< path of the Flash Memory
                        std::string backupMemoryBasePath; ///< path of the backup memory
                        Smartmatic::SAES::Common::Configuration::RulesBasePathType rule; ///< current rule
                        std::vector < std::string > pathList; ///< path list
                        Glib::Mutex mutex; ///< The mutex
                        static Smartmatic::Log::ISMTTLog* logger;
                };
            }
        }
    }
}
#endif /* MEMORYPATHCONTROLLER_H_ */
