/**
 * @file CertificateValidatorWrapper
 * @brief Implementation of the entity that validates a signature certificate.
 * @author eduardo.fernandez
 * @date   26/10/2018
 */

#include "CertificateValidatorWrapper.hxx"

#include <Log/SMTTLogManager.h>
#include <Security/CertificateValidatorWrapper.hxx>
#include <Security/CertificateValidatorWrapper.hxx>
#include <System/Security/Cryptography/CertUtil.h>
#include <System/Exception/CryptoException.hxx>
#include <System/IO/File.h>
#include <System/IO/Path.h>
#include <System/SafeBuffer.h>
#include <System/Security/OpenSSLCertificate.hxx>
#include <System/XML/ExtractCertificate.hxx>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

#define N_(String) (String) ///<macro for gettext not translate

using namespace Smartmatic::Log;
using namespace Smartmatic::SAES::Common::Configuration;
using namespace Smartmatic::SAES::Security;
using namespace Smartmatic::System;
using namespace Smartmatic::System::Exception;
using namespace Smartmatic::System::IO;
using namespace Smartmatic::System::Security;
using namespace Smartmatic::System::Security::Cryptography;
using namespace Smartmatic::System::XML;

static const std::string CERTIFICATE_VALIDATION_EXCEPTION_MESSAGE  = _("Smartmatic.SAES.Security.CertificateStore.ErrorValidatingCertificate");
static const std::string CERTIFICATE_NOT_VALIDATED_MESSAGE         = "Certificate could not be validated.";
static const std::string CLIENT_EMPTY_PATH_MESSAGE                 = "Client certificate path is empty. Nothing to validate";
static const std::string CLIENT_NOT_VALIDATED_MESSAGE              = "Client certificate could not be validated.";
static const std::string CLIENT_VALIDATION_START_MESSAGE           = "Validating client certificate located at ";
static const std::string CLIENT_VALIDATION_EXCEPTION_MESSAGE       = _("Smartmatic.SAES.Security.CertificateStore.ErrorValidatingClientCertificate");
static const std::string CLIENT_VALIDATION_SUCCESS_MESSAGE         = "Client certificate has been successfully validated.";
static const std::string EMPTY_CERTIFICATE_EXCEPTION_MESSAGE       = _("Smartmatic.SAES.Security.CertificateStore.ErrorValidatingEmptyCertificate");
static const std::string EMPTY_CHAR                                = "";
static const std::string INTERMEDIATE_EMPTY_PATH_MESSAGE           = "Intermediate certificate path cannot be empty.";
static const std::string INTERMEDIATE_NOT_VALIDATED_MESSAGE        = "Intermediate certificate could not be validated.";
static const std::string INTERMEDIATE_VALIDATION_EXCEPTION_MESSAGE = _("Smartmatic.SAES.Security.CertificateStore.ErrorValidatingIntermediateCertificate");
static const std::string INTERMEDIATE_VALIDATION_START_MESSAGE     = "Validating intermediate certificate located at ";
static const std::string INTERMEDIATE_VALIDATION_SUCCESS_MESSAGE   = "Intermediate certificate has been successfully validated.";
static const std::string ROOT_EMPTY_PATH_MESSAGE                   = "Root certificate path cannot be empty.";
static const std::string ROOT_NOT_VALIDATED_MESSAGE                = "Root certificate could not be validated.";
static const std::string ROOT_VALIDATION_EXCEPTION_MESSAGE         = _("Smartmatic.SAES.Security.CertificateStore.ErrorValidatingRootCertificate");
static const std::string ROOT_VALIDATION_START_MESSAGE             = "Validating root certificate fingerprint located at ";
static const std::string ROOT_VALIDATION_SUCCESS_MESSAGE           = "Root certificate has been successfully validated.";

ISMTTLog *CertificateValidatorWrapper::m_logger =
    SMTTLogManager::GetLogger("Smartmatic.SAES.Security.CertificateValidatorWrapper", GETTEXT_PACKAGE);

CertificateValidatorWrapper *CertificateValidatorWrapper::m_instance = NULL;

/**
 * @brief Sets the verification options.
 * @author eduardo.fernandez
 * @date   16/11/2018
 * @param  validator The certificate validator instance.
 * @param  resourceFileType The type of resource file.
 */
static void
setVerificationOptions(CertificateValidator& validator, const Encryption::ResourceFileType& resourceFileType)
{
    const int certificateValidatorFlags = Encryption::getInstance()->getCertificateValidatorFlags(resourceFileType);
    validator.SetOptions((CertificateValidationFlags) certificateValidatorFlags);
}

/**
 * @brief Sets the verification time.
 * @author eduardo.fernandez
 * @date   29/10/2018
 * @param  validator The certificate validator instance.
 */
static void
setVerificationTime(CertificateValidator& validator)
{
    const time_t timeCertified = Encryption::getInstance()->getCertificateValidatorTime();

    if (timeCertified != 0)
    {
        validator.SetVerificationTime(timeCertified);
    }
}

CertificateValidatorWrapper::CertificateValidatorWrapper()
{

}

CertificateValidatorWrapper::~CertificateValidatorWrapper()
{

}

CertificateValidatorWrapper*
CertificateValidatorWrapper::getInstance(CertificateValidatorWrapper * const externalInstance)
{
    if (m_instance == NULL || externalInstance != NULL)
    {
        m_instance = (externalInstance == NULL ? new CertificateValidatorWrapper() : externalInstance);
    }

    return m_instance;
}

void
CertificateValidatorWrapper::checkIssuerCertificatePath(
    const std::string& issuerCertificatePath,
    const std::string& loggerMessage,
    const std::string& exceptionMessage)
{
    if (Path::GetFileName(issuerCertificatePath) == EMPTY_CHAR)
    {
        m_logger->Debug(loggerMessage);
        throw CryptoException(exceptionMessage, exceptionMessage, m_logger);
    }
}

void
CertificateValidatorWrapper::checkValidationResult(
    CertValidationResult& result,
    const std::string& loggerMessage,
    const std::string& exceptionMessage)
{
    if (result.GetSucces() == false)
    {
        m_logger->Debug(loggerMessage);
        throw CryptoException(exceptionMessage, exceptionMessage, m_logger);
    }
}

void
CertificateValidatorWrapper::validateRootCertificate(const std::string& path, const std::string& configuredFingerprint)
{
    checkIssuerCertificatePath(path, ROOT_EMPTY_PATH_MESSAGE, EMPTY_CERTIFICATE_EXCEPTION_MESSAGE);

    m_logger->Debug(ROOT_VALIDATION_START_MESSAGE + path);

    if (Encryption::getInstance()->getRootCAFingerprint(path) != configuredFingerprint)
    {
        m_logger->Debug(ROOT_NOT_VALIDATED_MESSAGE);
        throw CryptoException(ROOT_VALIDATION_EXCEPTION_MESSAGE, ROOT_VALIDATION_EXCEPTION_MESSAGE, m_logger);
    }

    m_logger->Debug(ROOT_VALIDATION_SUCCESS_MESSAGE);

    m_validator.SetCACertificate(path, CertUtil::CertFormat_PEM);
}

void
CertificateValidatorWrapper::validateIntermediateCertificate(const std::string& path)
{
    checkIssuerCertificatePath(path, INTERMEDIATE_EMPTY_PATH_MESSAGE, EMPTY_CERTIFICATE_EXCEPTION_MESSAGE);

    m_logger->Debug(INTERMEDIATE_VALIDATION_START_MESSAGE + path);

    setVerificationOptions(m_validator, Encryption::INSTALL_FILE);
    setVerificationTime(m_validator);

    SafeBuffer buffer;
    File::getInstance()->openFileToBuffer(path, buffer);
    CertUtil::updateCertificateBuffer(buffer);
    OpenSSLCertificate *certificate =  new OpenSSLCertificate(buffer);

    OpenSSLCryptoX509 * const x509   = certificate->getX509();
    CertValidationResult      result = m_validator.VerifyX509Certificate(x509, DefaultPurpose, true);
    checkValidationResult(result, INTERMEDIATE_NOT_VALIDATED_MESSAGE, INTERMEDIATE_VALIDATION_EXCEPTION_MESSAGE);
    CertUtil::updateCertificateBuffer(buffer, true);

    m_validator.SetIntermediateCACertificate(buffer, CertUtil::CertFormat_PEM);

    m_logger->Debug(INTERMEDIATE_VALIDATION_SUCCESS_MESSAGE);

    if (certificate != NULL)
    {
        delete certificate;
        certificate = NULL;
    }
}

void
CertificateValidatorWrapper::validateClientCertificate(
    const std::string& path,
    const CertUtil::CertFileFormat& format,
    const std::string& password)
{
    if (Path::GetFileName(path) != EMPTY_CHAR)
    {
        m_logger->Debug(CLIENT_VALIDATION_START_MESSAGE + path);

        CertValidationResult result = m_validator.VerifyX509Certificate(path, format, DefaultPurpose, password);

        if (result.GetSucces() == false)
        {
            m_logger->Debug(CLIENT_NOT_VALIDATED_MESSAGE);
            throw CryptoException(CLIENT_VALIDATION_EXCEPTION_MESSAGE, CLIENT_VALIDATION_EXCEPTION_MESSAGE, m_logger);
        }

        m_logger->Debug(CLIENT_VALIDATION_SUCCESS_MESSAGE);
    }
    else
    {
        m_logger->Debug(CLIENT_EMPTY_PATH_MESSAGE);
    }
}

void
CertificateValidatorWrapper::validateCertificate(OpenSSLCryptoX509 * const certificate)
{
    CertValidationResult result = m_validator.VerifyX509Certificate(certificate, DefaultPurpose);

    if (result.GetSucces() == false)
    {
        m_logger->Debug(CERTIFICATE_NOT_VALIDATED_MESSAGE);
        throw CryptoException(CERTIFICATE_VALIDATION_EXCEPTION_MESSAGE, CERTIFICATE_VALIDATION_EXCEPTION_MESSAGE, m_logger);
    }
}
