/**
 * @file CertificateValidatorWrapper.hxx
 * @brief Declaration of the entity that wraps the certificate validator.
 * @author eduardo.fernandez
 * @date   26/10/2018
 */

#ifndef CERTIFICATE_VALIDATOR_WRAPPER_H
#define CERTIFICATE_VALIDATOR_WRAPPER_H

#include <Security/Encryption.hxx>
#include <System/Security/Cryptography/CertificateValidator.h>

namespace Smartmatic
{
    namespace Log
    {
        class ISMTTLog;
    }

    namespace System
    {
        namespace Security
        {
            class OpenSSLCertificate;
        }
    }

    namespace SAES
    {
        namespace Security
        {
            /**
             * @class CertificateValidatorWrapper
             * @brief Wrapper of the certificate validator class.
             * @author eduardo.fernandez
             * @date   26/10/2018
             */
            class CertificateValidatorWrapper
            {

                public:

                    /**
                     * @brief Gets the class current instance.
                     * @author eduardo.fernandez
                     * @date   31/10/2018
                     * @param  externalInstance Any external instance with testing purposes.
                     * @return Current instance of the class.
                     */
                    static CertificateValidatorWrapper *getInstance(
                        CertificateValidatorWrapper * const externalInstance = NULL);

                    /**
                     * @brief  Finalizes the instance of the SignatureCertificateValidator class.
                     * @author eduardo.fernandez
                     * @date   26/10/2018
                     */
                    virtual ~CertificateValidatorWrapper();

                public:

                    /**
                     * @brief Validates the root certificate against a configured fingerprint.
                     * @author eduardo.fernandez
                     * @date   29/10/2018
                     * @param  path Path to the root certificate.
                     * @param  configuredFingerprint Configured root certificate fingerprint.
                     */
                    void validateRootCertificate(
                        const std::string& path,
                        const std::string& configuredFingerprint);

                    /**
                     * @brief Validates an intermediate certificate against a root certificate.
                     * @author eduardo.fernandez
                     * @date   26/10/2018
                     * @param  path Intermediate certificate to validate.
                     */
                    void validateIntermediateCertificate(const std::string& path);

                    /**
                     * @brief Validates a client certificate chain of trust.
                     * @author eduardo.fernandez
                     * @date   29/10/2018
                     * @param  path Path to the client certificate.
                     * @param  format Format of the client certificate.
                     * @param  password Password of the client certificate.
                     */
                    void validateClientCertificate(
                        const std::string& path,
                        const Smartmatic::System::Security::Cryptography::CertUtil::CertFileFormat& format,
                        const std::string& password);

                    /**
                     * @brief Validates a certificate's chain of trust.
                     * @author eduardo.fernandez
                     * @date   04/12/2018
                     * @param  certificate The certificate.
                     */
                    void validateCertificate(OpenSSLCryptoX509 * const certificate);

                private:

                    /**
                     * @brief Default constructor.
                     * @author eduardo.fernandez
                     * @date   26/10/2018
                     */
                    CertificateValidatorWrapper();

                private:

                    /**
                     * @brief Checks the issuer certificate path.
                     * @author eduardo.fernandez
                     * @date   12/11/2018
                     * @param  issuerCertificatePath The path to the issuer certificate.
                     * @param  loggerMessage The message to log.
                     * @param  exceptionMessage The message of the exception to throw.
                     */
                    void checkIssuerCertificatePath(
                        const std::string& issuerCertificatePath,
                        const std::string& loggerMessage,
                        const std::string& exceptionMessage);

                    /**
                     * @brief Checks the result of the certificate validation.
                     * @author eduardo.fernandez
                     * @date   12/11/2018
                     * @param[out] result The certificate validation result.
                     * @param[in]  loggerMessage The message to log.
                     * @param[in]  exceptionMessage The message of the exception to throw.
                     */
                    void checkValidationResult(
                        Smartmatic::System::Security::Cryptography::CertValidationResult& result,
                        const std::string& loggerMessage,
                        const std::string& exceptionMessage);

                private:

                    static CertificateValidatorWrapper *m_instance;
                    static Smartmatic::Log::ISMTTLog *m_logger;

                    Smartmatic::System::Security::Cryptography::CertificateValidator m_validator;
            };
        }
    }
}

#endif /*CERTIFICATE_VALIDATOR_WRAPPER_H*/
