/**
 * @file Passphrase.hxx
 * @brief Declaration of the singleton entity that provides the cryto passphrase.
 * @date 17/05/2013
 * @author Yadickson Soto
 */

#ifndef PASSPHRASE_HXX_
#define PASSPHRASE_HXX_

#include <string>
#include <System/SafeBuffer.h>

namespace Smartmatic
{
    namespace SAES
    {
        namespace Common
        {
            namespace Security
            {
                /**
                 * @brief Singleton provider of the cryto passphrase.
                 * @author  Yadickson Soto
                 */
                class Passphrase
                {

                    private:
                        /**
                         * @brief Default constructor initializes an instance of the Passphrase class.
                         */
                        Passphrase();

                        static Passphrase * instance; ///< The pointer reference to Passphrase required to access to the instance.
                        std::string passphrase; ///< base 64 passphrase

                    public:
                        /**
                         * @brief Finalizes the instance of the Passphrase class.
                         */
                        virtual ~Passphrase();

                        /**
                         * @fn		void setPlainPassphrase( const std::string & plainPassphrase );
                         * @brief	Sets the passphrase.
                         * @author	Yadickson Soto
                         * @date	08/05/2013
                         * @param	plainPassphrase	The reference to the readable string used to generates el passphrase.
                         */
                        void setPlainPassphrase ( const std::string & plainPassphrase );

                        /**
                         * @fn		std::string getPlainPassphrase();
                         * @brief	Getter method: user provided plain passphrase.
                         * @author	Yadickson Soto
                         * @date	08/05/2013
                         * @return	The plain passphrase.
                         */
                        std::string getPlainPassphrase();

                        /**
                         * @fn		bool getCryptoPassphrase( Smartmatic::System::SafeBuffer & bufferOut );
                         * @brief	Gets the crypto plain passphrase with init key.
                         * @author	Yadickson Soto
                         * @date	08/05/2013
                         * @param 	bufferOut	The crypto plain passphrase result.
                         * @return	true if success crypto plain passphrase.
                         */
                        bool getCryptoPassphrase ( Smartmatic::System::SafeBuffer & bufferOut );

                        /**
                         * @fn    bool getDecryptPassphrase ( Smartmatic::System::SafeBuffer & bufferIn, std::string & pass );
                         * @brief Gets the password obtained from the input buffer.
                         * @author  Yadickson Soto
                         * @param bufferIn The reference to input buffer.
                         * @param pass The reference to string used to retrieve the password.
                         * @return  <tt>true</tt> if obtains the password was successful; otherwise <tt>false</tt>.
                         */
                        bool getDecryptPassphrase ( Smartmatic::System::SafeBuffer & bufferIn, std::string & pass );

                        /**
                         * @fn	static Passphrase * getInstance();
                         * @brief	Gets the only instance of the Passphrase. If this instance does not exist, then it creates a new one.
                         * @author	Yadickson Soto
                         * @date	17/05/2013
                         * @return 	The pointer reference to Passphrase required to access to the instance.
                         */
                        static Passphrase * getInstance();
                };

            } /* namespace Security */
        } /* namespace Common */
    } /* namespace SAES */
} /* namespace Smartmatic */
#endif /* PASSPHRASE_HXX_ */
