/**
 * @file SecretKey.cpp
 * @brief Implementation of reader the secret key which contains the methods required for parse the encrypted file.
 * @date 05/04/2011
 * @author Marcel Cordovi
 */

#include "SecretKey.h"
#include "Configuration/s-a-e-s-common-conf-manager.h"
#include "System/Security/Cryptography/PBEKeyDerivatorHMAC.h"
#include <Security/KeyManager.hxx>
#include <Log/SMTTLogManager.h>
#include <Log/ISMTTLog.h>
#include <System/IO/File.h>
#include <System/Encoding/Base64Encoder.h>
#include <System/Security/Cryptography/MsgDigestUtil.h>
#include <System/Security/Cryptography/HmacSha256.hxx>
#include <System/Utils/StringUtils.hxx>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

#define N_(String) (String) ///<macro for gettext not translate

static const unsigned int HMAC_KEY_LEN    = 32; ///< hmac key length
static const unsigned int ENC_KEY_LEN     = 32; ///< encryption key length
static const unsigned int HMAC_LEN        = 32; ///< size of the hmac

using namespace Smartmatic::System::Encoding;
using namespace Smartmatic::System::Security;
using namespace Smartmatic::System::Security::Cryptography;
using namespace Smartmatic::System::Exception;
using namespace Smartmatic::SAES::Common::Configuration;
using namespace Smartmatic::SAES::Common::Security;
using namespace Smartmatic::System::IO;
using namespace Smartmatic::System;
using namespace Smartmatic::Log;

ISMTTLog * SecretKey::logger = SMTTLogManager::GetLogger ( "Smartmatic.SAES.Common.Security.SecretKey", GETTEXT_PACKAGE );

// ================= Utility ==========================

void SecretKey::mClearKey ( Smartmatic::System::SafeBuffer & key )
{
    key.Clear();
}

void SecretKey::mAssignKey ( const Smartmatic::System::SafeBuffer & keyIn,
                             Smartmatic::System::SafeBuffer & keyOut )
{
    keyOut.Clear();
    keyOut.AddData ( keyIn );
}

// ================= Operator Overloading ======================

SecretKey & SecretKey::operator= ( const SecretKey & sk )
{
    if ( this != &sk )
    {
        // clear current state
        mClearKey ( mp_DecryptionKey );
        mClearKey ( mp_SecretKey );
        mClearKey ( mp_PassphraseEncrypted );
        // copy new assigned values
        this->m_FilePath.assign ( sk.m_FilePath );
        this->m_IsFileEncrypted = sk.m_IsFileEncrypted;
        this->m_Passphrase = sk.m_Passphrase;
        this->m_rnd = sk.m_rnd;
        this->m_votingCenterCode = sk.m_votingCenterCode;
        mAssignKey (
            sk.mp_DecryptionKey,
            this->mp_DecryptionKey );
        mAssignKey (
            sk.mp_SecretKey,
            this->mp_SecretKey );
        mAssignKey (
            sk.mp_MacKey,
            this->mp_MacKey );
        mAssignKey (
            sk.mp_PassphraseEncrypted,
            this->mp_PassphraseEncrypted );
    }

    return *this;
}


// ================= Access =======================================

std::string SecretKey::getKeyBase64 () const
{
    return mp_SecretKey.GetBase64Data();
}

void SecretKey::getKey ( unsigned char *& key, int & keyLen )
{
    key = ( unsigned char * ) mp_SecretKey.GetData();
    keyLen = mp_SecretKey.GetSize();
}

const unsigned char * SecretKey::getKey ( int & keyLen ) const
{
    keyLen = mp_SecretKey.GetSize();
    return mp_SecretKey.GetData();
}

const unsigned char * SecretKey::getPassphrase ( size_t & bufferLen ) const
{
    bufferLen = mp_PassphraseEncrypted.GetSize();
    return mp_PassphraseEncrypted.GetData();
}


// ================= Life Cycle ==================================

SecretKey::SecretKey()
{
    m_IsFileEncrypted = true;
}

SecretKey::SecretKey ( const SecretKey & sk )
{
    this->m_FilePath.assign ( sk.m_FilePath );
    this->m_IsFileEncrypted = sk.m_IsFileEncrypted;
    this->m_Passphrase = sk.m_Passphrase;
    this->m_rnd = sk.m_rnd;
    this->m_votingCenterCode = sk.m_votingCenterCode;
    mAssignKey (
        sk.mp_DecryptionKey,
        this->mp_DecryptionKey );
    mAssignKey (
        sk.mp_SecretKey,
        this->mp_SecretKey );
    mAssignKey (
        sk.mp_MacKey,
        this->mp_MacKey );
    mAssignKey (
        sk.mp_PassphraseEncrypted,
        this->mp_PassphraseEncrypted );
}

SecretKey::SecretKey ( const std::string & filePath, const std::string & passPhrase, const std::string & votingCenterCode, const std::string & rnd)
{
    processKey ( filePath, passPhrase, votingCenterCode, rnd );
}

void SecretKey::processKey ( const std::string & filePath, const std::string & passPhrase, const std::string & votingCenterCode, const std::string & rnd )
{
    std::string passphrase ( passPhrase );
    m_Passphrase = passPhrase;
    m_rnd = rnd;
    m_votingCenterCode = votingCenterCode;
    /* State initialization */
    logger->Debug ( "Open file: " + filePath );
    m_FilePath.assign ( filePath );
    m_IsFileEncrypted = SAESCommonConfigurationManager::getInstance()->getConfiguration()->
                        getMachineSecuritySchema().getSecretKeyEncrypted();

    /* Calculate file's decryption key  */

    if ( m_IsFileEncrypted )
    {
        SafeBuffer salt;
        int iterations = 0;
        SaltSourceType salt_conf = SAESCommonConfigurationManager::getInstance()->getConfiguration()->
                                   getMachineSecuritySchema().getSaltSource();

        if ( salt_conf.getCalcSalt() ) // check whether salt must be calculated from the rnd and voting center code
        {
            std::string saltSeed;
            saltSeed=votingCenterCode + "-" + rnd;
            SafeBuffer dbg_saltSeed ( saltSeed );
            MsgDigestUtil digest ( dbg_saltSeed );
            digest.getDigest ( salt, MsgDigestUtil::DGSTUTIL_SHA256 );
            iterations = SAESCommonConfigurationManager::getInstance()->getConfiguration()->
                getMachineSecuritySchema().getIterations();
        }
        else // obtain salt the old way: A SHA256 hash of the passphrase
        {
            SafeBuffer dbg_salt ( passphrase );
            MsgDigestUtil digest ( dbg_salt );

            digest.getDigest ( salt, MsgDigestUtil::getDefaultSecretKeyDigest() );
            iterations = SAESCommonConfigurationManager::getInstance()->getConfiguration()->
                         getMachineSecuritySchema().getIterations();
        }

        mObtainSymmetricKey ( passphrase, salt, iterations );
    }

    mEncryptPassphrase ( passphrase );
    /* Extract key from file */
    mUnwrap ();
}

SecretKey::~SecretKey()
{
    mClearKey ( mp_DecryptionKey );
    mClearKey ( mp_SecretKey );
    mClearKey ( mp_MacKey );
    mClearKey ( mp_PassphraseEncrypted );
}


// ================= Key Processing ======================

void SecretKey::mProcessInputPassphrase (
    std::string & passphrase,
    std::string & salt,
    std::string sep_pattern,
    int & iterations )
{
    std::vector<std::string> tokens;
    size_t num_tokens;

    if ( sep_pattern == "|" ) // TODO: use regular expression for regex's reserved tokens detection and scapeping
    {
        sep_pattern = "\\" + sep_pattern;    // preppend scape character
    }

    Utils::StringUtils::splitToVector ( passphrase, sep_pattern, tokens );

    num_tokens = tokens.size();

    if ( num_tokens == 0 || num_tokens > 3 )
    {
        throw SecretKeyException ( _ ( "Smartmatic.SAES.Common.Security.SecretKey.ErrorInvalidPassphraseFormat" ), N_ ( "Smartmatic.SAES.Common.Security.SecretKey.ErrorInvalidPassphraseFormat" ), logger );
    }

    passphrase.assign ( tokens[0] );

    if ( num_tokens >= 2 )
    {
        salt = tokens[1];
    }
    else // no provided
    {
        salt.assign ( "" );
    }

    if ( num_tokens == 3 )
    {
        iterations = atoi ( tokens[2].c_str () );
    }
    else // no provided
    {
        iterations = 0;
    }
}

void SecretKey::mObtainSymmetricKey ( const std::string & passPhrase, const SafeBuffer & salt, const int iterations )
{
    try
    {
        SafeBuffer          outputBuffer;
        SafeBuffer          hmacKeyBuffer;
        SafeBuffer          encKeyBuffer;
        SafeBuffer          cbcMacKeyBuffer;
        PBEKeyDerivatorHMAC deriver;

        deriver.SetSalt ( salt );

        deriver.GenerateKey (
            passPhrase,
            outputBuffer,
            MsgDigestUtil::getDefaultGenerateKeyDigest(),
            HMAC_KEY_LEN + ENC_KEY_LEN,
            iterations );

        if (outputBuffer.GetSize() != HMAC_KEY_LEN + ENC_KEY_LEN)
        {
            throw SecretKeyException (
                _( "Smartmatic.SAES.Common.Security.SecretKey.ErrorReadingSecretKey" ),
                N_( "Smartmatic.SAES.Common.Security.SecretKey.ErrorReadingSecretKey" ),
                logger );
        }

        encKeyBuffer.AddData(outputBuffer.GetData(), ENC_KEY_LEN);
        outputBuffer.removeData(0, ENC_KEY_LEN);
        hmacKeyBuffer.AddData(outputBuffer.GetData(), HMAC_KEY_LEN);

        mAssignKey ( encKeyBuffer, mp_DecryptionKey );
        mAssignKey ( hmacKeyBuffer, mp_MacKey );
    }
    catch ( CryptoException & ce )
    {
        throw SecretKeyException (
            _( "Smartmatic.SAES.Common.Security.SecretKey.ErrorReadingSecretKey" ),
            N_( "Smartmatic.SAES.Common.Security.SecretKey.ErrorReadingSecretKey" ),
            logger );
    }
}

void SecretKey::mUnwrap ()
{
    if ( !File::getInstance()->canOpenFile ( m_FilePath ) )
    {
        throw SecretKeyException (
            _( "Smartmatic.SAES.Common.Security.SecretKey.ErrorOpenFileSecretKey" ),
            N_( "Smartmatic.SAES.Common.Security.SecretKey.ErrorOpenFileSecretKey" ),
            logger );
    }

    SafeBuffer dataIn;
    File::getInstance()->openFileToBuffer ( m_FilePath, dataIn );

    if ( m_IsFileEncrypted )
    {
        try
        {
            Crypto crypt;

            crypt.init ( Smartmatic::SAES::Common::Security::KeyManager::getInstance()->getKeySize(),
                         Smartmatic::SAES::Common::Security::KeyManager::getInstance()->getModeType(), true );

            Smartmatic::System::SafeBuffer dataOut;
            Smartmatic::System::SafeBuffer hmac;

            hmac.AddData(dataIn.GetData(), HMAC_LEN);
            dataIn.removeData(0, HMAC_LEN);

            HmacSha256 hmacSha256(mp_MacKey);

            hmacSha256.update(dataIn);

            if (!HmacSha256Validate(hmacSha256, hmac))
            {
                logger->Debug("HMAC validation failed for " + m_FilePath);

                throw SecretKeyException (
                    _( "Smartmatic.SAES.Common.Security.SecretKey.ErrorHMAC" ),
                    N_( "Smartmatic.SAES.Common.Security.SecretKey.ErrorHMAC" ),
                    logger );

            }

            crypt.decryptBufferToBufferAndGetIV(
                dataIn,
                dataOut,
                mp_DecryptionKey.GetData());

            mAssignKey ( dataOut, mp_SecretKey );
        }
        catch ( CryptoException & ce )
        {
            throw SecretKeyException (
                _( "Smartmatic.SAES.Common.Security.SecretKey.ErrorReadingSecretKey" ),
                N_( "Smartmatic.SAES.Common.Security.SecretKey.ErrorReadingSecretKey" ),
                logger );
        }
    }
    else
    {
        SafeBuffer decode;
        Base64Encoder::ConvertFromBase64 ( dataIn.GetBase64Data(), decode );
        mAssignKey ( decode, mp_SecretKey );
    }
}

void SecretKey::mEncryptPassphrase ( const std::string & passphrase )
{
    Smartmatic::System::SafeBuffer passEncrypted;
    Smartmatic::System::SafeBuffer data ( ( unsigned char * ) passphrase.data(), ( int ) passphrase.size () );

    Smartmatic::System::SafeBuffer initKey;

    Smartmatic::SAES::Common::Security::KeyManager::getInstance()->getKey (
        Smartmatic::SAES::Common::Configuration::KeyType::CONFIGURATION_INIT_KEY, initKey );

    try
    {
        Crypto crypt;
        crypt.init ( Smartmatic::SAES::Common::Security::KeyManager::getInstance()->getKeySize(),
                     Smartmatic::SAES::Common::Security::KeyManager::getInstance()->getModeType(), true );
        crypt.encryptBufferToBufferAndCreateIV (
            data
            , passEncrypted
            , initKey.GetData()
            );
    }
    catch ( CryptoException & ce )
    {
        throw SecretKeyException (
            _ ( "Smartmatic.SAES.Common.Security.SecretKey.ErrorReadingSecretKey" ),
            N_ ( "Smartmatic.SAES.Common.Security.SecretKey.ErrorReadingSecretKey" ), logger );
    }

    mAssignKey ( passEncrypted, mp_PassphraseEncrypted );
}

void SecretKey::getSafeBuffer ( Smartmatic::System::SafeBuffer & buffer )
{
    buffer.Clear();
    buffer.AddData ( mp_SecretKey );
}
