/**
 * @file   SecretKey.h
 * @brief  Declaration of reader the secret key which contains the methods required for parse the encrypted file.
 * @author Youssef.tourkmani
 * @date   30/10/2012
 */


#ifndef SECRETKEY_H_
#define SECRETKEY_H_

#include <string>
#include <Configuration/SaesCommonConfiguration-schema.hxx>
#include <System/Exception/SecretKeyException.h>
#include <System/SafeBuffer.h>

namespace Smartmatic
{
    namespace Log
    {
        class ISMTTLog;
    }

    namespace SAES
    {
        namespace Security
        {
            class Encryption;
        }

        namespace Common
        {
            namespace Security
            {
                class KeyManager;

                /**
                 * @class  SecretKey
                 * @brief  Reader which contains the functionalities for obtain and expose the secret key.
                 * @author Cesar.bonilla
                 * @date   4/28/2011
                 */

                class SecretKey
                {

                    public:

                        /**
                         * @brief Default constructor initializes an instance of the SecretKey class.
                         * @author Cesar.bonilla
                         * @date 4/28/2011
                         */

                        SecretKey();

                        /**
                         * @brief   Constructor initializes an instance of the SecretKey class.
                         * @author  Cesar.bonilla
                         * @date    4/28/2011
                         * @throw   SecretKeyException Thrown when a secret key error condition occurs.
                         * @param   filePath         Full pathname of the file.
                         * @param   passPhrase       The pass phrase.
                         * @param   votingCenterCode The voting center code.
                         * @param   rnd              The random string used to generate the salt.
                         */
                        SecretKey (
                            const std::string & filePath,
                            const std::string & passPhrase,
                            const std::string & votingCenterCode,
                            const std::string & rnd );

                        /**
                         * @brief  Copy Constructor initializes an instance of the SecretKey class.
                         * @author Cesar.bonilla
                         * @date   4/28/2011
                         * @param  other The other.
                         */
                        SecretKey ( const SecretKey & other );

                        /**
                         * @brief  Finalizes the instance of the SecretKey class.
                         * @author Cesar.bonilla
                         * @date   4/28/2011
                         */
                        virtual ~SecretKey();

                        /**
                         * @brief  Assignment operator.
                         * @author Cesar.bonilla
                         * @date   4/28/2011
                         * @param  sk The sk.
                         * @return A shallow copy of this object.
                         */
                        SecretKey & operator= ( const SecretKey & sk );


                    public:

                        /**
                         * @brief  Accessor to the file stored secret key.
                         * @author Cesar.bonilla
                         * @date   4/28/2011
                         * @return The key contained in the file in a base64 encoding format.
                         */
                        std::string getKeyBase64 () const;

                        /**
                         * @brief  Accessor to the file stored secret key.
                         * @author Youssef.tourkmani
                         * @date   30/10/2012
                         * @param[out]  key    Is allocated with a new buffer containing the key bytes.
                         * @param[out]  keyLen Will contain the key buffer length.
                         */
                        void getKey ( unsigned char *& key, int & keyLen );

                        /**
                         * @brief  Accessor to the file stored secret key.
                         * @author Youssef.tourkmani
                         * @date   30/10/2012
                         * @param[out] keyLen It is filled with the key bytes.
                         * @return pointer to the key content.
                         */
                        const unsigned char * getKey ( int & keyLen ) const;

                        /**
                         * @brief  Clear passphrase.
                         * @author Yadickson Soto
                         * @date   08/05/2013
                         */
                        void clearPlainPassphrase ()
                        {
                            m_Passphrase.clear();
                        }

                        /**
                         * @brief  Accesor to the encrypted passphrase.
                         * @author Youssef.tourkmani
                         * @date   30/10/2012
                         * @param[out] bufferLen Mutable integer wich will contain the size of the buffer.
                         * @return a buffer with the encrypted passphrase.
                         */
                        const unsigned char * getPassphrase ( size_t & bufferLen ) const;

                        /**
                         * @brief  Getter method: secret key binary buffer
                         * @author Yadickson Soto
                         * @date   07/05/2013
                         * @param  buffer Result buffer
                         */
                        void getSafeBuffer ( Smartmatic::System::SafeBuffer & buffer );

                    protected:

                        /**
                         * @brief  Accesor to the user provided passphrase.
                         * @author Youssef.tourkmani
                         * @date   30/10/2012
                         * @return The plain passphrase.
                         */
                        const std::string getPlainPassphrase () const
                        {
                            return m_Passphrase;
                        }


                    private:

                        /**
                         * @brief  Extract the secret key contained in the file decrypting it if necessary.
                         * @author Cesar.bonilla
                         * @date   4/28/2011
                         * @throw  SecretKeyException Thrown when a secret key error condition occurs.
                         */
                        void mUnwrap();

                        /**
                         * @brief  Parse the string provided by the client to obtain the parameters needed by the
                         *         cryptography algorithms involved in the extraction of the secret key.
                         * @author Cesar.bonilla
                         * @date   4/28/2011
                         * @throw  SecretKeyException Thrown when a secret key error condition occurs.
                         * @param  passphrase  The passphrase.
                         * @param  salt        The salt.
                         * @param  sep_pattern The separator pattern.
                         * @param  iterations  The iterations.
                         */
                        void mProcessInputPassphrase (
                            std::string & passphrase,
                            std::string & salt,
                            std::string sep_pattern,
                            int & iterations );

                        /**
                         * @brief  Derives PBE key from the passphrase provided by the user.
                         * This method only support to Hashed Message Authentication Code for key derivation function.
                         * @author Cesar.bonilla
                         * @date   4/28/2011
                         * @throw  SecretKeyException Thrown on error obtaining the key.
                         * @param  passPhrase The pass phrase.
                         * @param  salt       The salt.
                         * @param  iterations The iterations.
                         */
                        void mObtainSymmetricKey (
                            const std::string & passPhrase,
                            const Smartmatic::System::SafeBuffer & salt,
                            const int iterations );

                        /**
                         * @brief  Securely free a key buffer resources.
                         * @author Yadickson Soto
                         * @date   05/11/2012
                         * @param  key Pointer to the buffer to be erased.
                         */
                        void mClearKey ( Smartmatic::System::SafeBuffer & key );

                        /**
                         * @brief  Securely fills a key buffer with a new key value.
                         * @author Yadickson Soto
                         * @date   05/11/2012
                         * @param[in]  keyIn  Source buffer pointer to copy from.
                         * @param[out] keyOut Destination buffer.
                         */
                        void mAssignKey ( const Smartmatic::System::SafeBuffer & keyIn,
                                          Smartmatic::System::SafeBuffer & keyOut );

                        /**
                         * @brief  Store provided plain phassphrase encrypted with the default encryption key.
                         * @author Cesar.bonilla
                         * @date   4/28/2011
                         * @throw  SecretKeyException Thrown when a secret key error condition occurs.
                         * @param  passphrase The passphrase.
                         */
                        void mEncryptPassphrase ( const std::string & passphrase );

                        /**
                         * @brief  Method process file key.
                         * @author Yadickson Soto
                         * @date   13/02/2012
                         * @throw  SecretKeyException Thrown when a secret key error condition occurs.
                         * @param  filePath         The file path.
                         * @param  passPhrase       The passphrase.
                         * @param  votingCenterCode The voting center code.
                         * @param  rnd              The random string used to generate the salt.
                         */
                        void processKey (
                            const std::string & filePath,
                            const std::string & passPhrase,
                            const std::string & votingCenterCode,
                            const std::string & rnd );

                    private:

                        friend class Smartmatic::SAES::Common::Security::KeyManager;
                        friend class Smartmatic::SAES::Security::Encryption;

                        std::string m_FilePath;   ///< Full pathname of the file
                        std::string m_Passphrase; ///< The passphrase
                        std::string m_votingCenterCode; ///< The voting center code
                        std::string m_rnd; ///< The random number used to generate the salt

                        bool m_IsFileEncrypted; ///< true if is file encrypted

                        Smartmatic::System::SafeBuffer mp_DecryptionKey; ///< The decryption key
                        Smartmatic::System::SafeBuffer mp_SecretKey; ///< The secret key
                        Smartmatic::System::SafeBuffer mp_MacKey; ///< The MAC key
                        Smartmatic::System::SafeBuffer mp_PassphraseEncrypted; ///< The passphrase encrypted

                        static Smartmatic::Log::ISMTTLog * logger; ///< The reference pointer of a logger instance.
                };
            }
        }
    }
}

#endif /* SECRETKEY_H_ */
