/**
 * @file XmlFileSignature.cxx
 * @brief Body class for XmlFileSignature file.
 * @author  Eduardo Fernández
 * @date    04/12/2018
 */

#include "XmlFileSignature.hxx"

#include <Log/SMTTLogManager.h>
#include <Security/CertificateStore.h>
#include <Security/CertificateValidatorWrapper.hxx>
#include <Security/Encryption.hxx>
#include <System/Exception/CryptoException.hxx>
#include <System/Security/Cryptography/CertUtil.h>
#include <System/Security/OpenSSLCertificate.hxx>
#include <System/XML/ExtractCertificate.hxx>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

using namespace Smartmatic::Log;
using namespace Smartmatic::SAES::Security;
using namespace Smartmatic::System;
using namespace Smartmatic::System::Exception;
using namespace Smartmatic::System::Security;
using namespace Smartmatic::System::Security::Cryptography;
using namespace Smartmatic::System::XML;

static const std::string CERTIFICATE_NOT_DETECTED_MESSAGE        = "Chain of trust validation is expected but certificate was not detected.";
static const std::string CERTIFICATE_DETECTION_EXCEPTION_MESSAGE = _("Smartmatic.SAES.Security.CertificateStore.ErrorDetectingCertificate");

ISMTTLog *XmlFileSignature::m_logger =
    SMTTLogManager::GetLogger("Smartmatic.SAES.Security.XmlFileSignature", GETTEXT_PACKAGE);

XmlFileSignature::XmlFileSignature()
{
}

XmlFileSignature::~XmlFileSignature()
{
}

void
XmlFileSignature::validateSignatureCertificate(const SafeBuffer& xmlFileBuffer, const bool mustBeValidated)
{
    if (mustBeValidated)
    {
        SafeBuffer base64Cert;
        ExtractCertificate extractCertificate(xmlFileBuffer);

        if (extractCertificate.getBase64Certificate(base64Cert))
        {
            OpenSSLCertificate cert(base64Cert);
            OpenSSLCryptoX509 * const x509 = cert.getX509();

            CertificateValidatorWrapper::getInstance()->validateCertificate(x509);
        }
        else
        {
            m_logger->Debug(CERTIFICATE_NOT_DETECTED_MESSAGE);
            throw CryptoException(CERTIFICATE_DETECTION_EXCEPTION_MESSAGE, CERTIFICATE_DETECTION_EXCEPTION_MESSAGE, m_logger);
        }
    }
}
