#ifndef TEST_SIMULATOR_SMARTCARD_TEST
#define TEST_SIMULATOR_SMARTCARD_TEST

#ifdef ECUT_MAIN

#include <cppunit/extensions/HelperMacros.h>
#include <Card/SimulatorSmartcard.hxx>
#include <tests/MockFile.hxx>
#include <tests/StoreMockFile.hxx>

class SimulatorSmartcardTest : public CPPUNIT_NS::TestFixture
{
    private:

        typedef Smartmatic::SAES::Card::SimulatorSmartcard SimulatorSmartcard;
        typedef Smartmatic::System::SafeBuffer SafeBuffer;
        typedef Smartmatic::SAES::Security::CryptoData CryptoData;
        typedef Smartmatic::System::IO::File File;

    public:

        SimulatorSmartcardTest ()
        {

        }

        void setUp()
        {

        }

        void tearDown()
        {

        }

        void testInitializeFail()
        {
            MockFile file;
            File::getInstance ( &file );

            MOCKPP_CHAINER_FOR_EXT ( MockFile, canOpenFile, ext ) canOpenFile_mock ( &file );

            canOpenFile_mock
            .expects ( MOCKPP_NS::once() )
            .will ( MOCKPP_NS::returnValue<bool> ( false ) );

            SimulatorSmartcard smartcard ( "eventFile", "contentFile" );

            CPPUNIT_ASSERT ( false == smartcard.open ( "file" ) );
        }

        void testInitializeSuccess()
        {
            MockFile file;
            File::getInstance ( &file );

            MOCKPP_CHAINER_FOR_EXT ( MockFile, canOpenFile, ext ) canOpenFile_mock ( &file );

            canOpenFile_mock
            .expects ( MOCKPP_NS::once() )
            .will ( MOCKPP_NS::returnValue<bool> ( true ) );

            SimulatorSmartcard smartcard ( "eventFile", "contentFile" );

            CPPUNIT_ASSERT ( true == smartcard.open ( "file" ) );
        }

        void testDispose()
        {
            SimulatorSmartcard smartcard ( "eventFile", "contentFile" );

            CPPUNIT_ASSERT ( true == smartcard.close() );
            CPPUNIT_ASSERT ( true == smartcard.close() );
        }

        void testRead_File_Not_Found()
        {
            MockFile file;
            File::getInstance ( &file );

            MOCKPP_CHAINER_FOR_EXT ( MockFile, openFileToBuffer, ext ) openFileToBuffer_mock ( &file );

            openFileToBuffer_mock
            .expects ( MOCKPP_NS::once() )
            .will ( MOCKPP_NS::returnValue<bool> ( false ) );

            SimulatorSmartcard smartcard ( "eventFile", "contentFile" );

            SafeBuffer buffer;
            CryptoData crypto;

            CPPUNIT_ASSERT ( false == smartcard.read ( buffer, crypto ) );

            file.verify();
        }

        void testReadFile_Buffer_Empty()
        {
            StoreMockFile file;
            File::getInstance ( &file );

            file.openFileValues.push_back ( StoreMockFile::OpenFileValues ( SafeBuffer (), true ) );

            SimulatorSmartcard smartcard ( "eventFile", "contentFile" );

            SafeBuffer buffer;
            CryptoData crypto;

            CPPUNIT_ASSERT ( false == smartcard.read ( buffer, crypto ) );
            CPPUNIT_ASSERT ( false == file.openFileListValues.empty() );
            CPPUNIT_ASSERT ( "contentFile" == file.openFileListValues.front() );
        }

        void testReadFile_Buffer_Only_Password()
        {
            StoreMockFile file;
            File::getInstance ( &file );

            SafeBuffer fileBuffer;

            fileBuffer.AddData ( SafeBuffer ( "012345678901234567890123456789ab" ) );

            file.openFileValues.push_back ( StoreMockFile::OpenFileValues ( fileBuffer, true ) );

            SimulatorSmartcard smartcard ( "eventFile", "contentFile" );

            SafeBuffer buffer ( "garbage" );
            CryptoData crypto;

            CPPUNIT_ASSERT ( false == smartcard.read ( buffer, crypto ) );
            CPPUNIT_ASSERT ( false == file.openFileListValues.empty() );
            CPPUNIT_ASSERT ( "contentFile" == file.openFileListValues.front() );
            CPPUNIT_ASSERT ( 0 == buffer.GetSize() );
        }

        void testReadFile_Buffer_Success()
        {
            StoreMockFile file;
            File::getInstance ( &file );

            SafeBuffer fileBuffer;

            SafeBuffer key ( "012345678901234567890123456789ab" );
            SafeBuffer content ( "Hello" );

            fileBuffer.AddData ( key );
            fileBuffer.AddData ( content );

            file.openFileValues.push_back ( StoreMockFile::OpenFileValues ( fileBuffer, true ) );

            SimulatorSmartcard smartcard ( "eventFile", "contentFile" );

            SafeBuffer buffer;
            CryptoData crypto;

            CPPUNIT_ASSERT ( true == smartcard.read ( buffer, crypto ) );
            CPPUNIT_ASSERT ( false == file.openFileListValues.empty() );
            CPPUNIT_ASSERT ( "contentFile" == file.openFileListValues.front() );
            CPPUNIT_ASSERT ( buffer == content );
            CPPUNIT_ASSERT ( key == crypto.getKey() );
        }

        void testWrite_File_Fail()
        {
            MockFile file;
            File::getInstance ( &file );

            MOCKPP_CHAINER_FOR_EXT ( MockFile, saveBufferToFile, ext ) saveBufferToFile_mock ( &file );

            saveBufferToFile_mock
            .expects ( MOCKPP_NS::once() )
            .will ( MOCKPP_NS::returnValue<bool> ( false ) );

            SimulatorSmartcard smartcard ( "eventFile", "contentFile" );

            SafeBuffer buffer;
            CryptoData crypto;

            CPPUNIT_ASSERT ( false == smartcard.write ( buffer, crypto ) );

            file.verify();
        }

        void testWriteFile_Buffer()
        {
            StoreMockFile file;
            File::getInstance ( &file );

            file.saveReturnValues.push_back ( true );

            SimulatorSmartcard smartcard ( "eventFile", "contentFile" );

            CryptoData crypto;

            SafeBuffer key ( "key" );
            SafeBuffer content ( "Hello" );

            crypto.setKey ( key );

            SafeBuffer expected ( "keyHello" );

            CPPUNIT_ASSERT ( true == smartcard.write ( content, crypto ) );
            CPPUNIT_ASSERT ( false == file.saveBufferValues.empty() );
            CPPUNIT_ASSERT ( "contentFile" == file.saveBufferValues.front().file );
            CPPUNIT_ASSERT ( expected == file.saveBufferValues.front().buffer );
        }

        void testCardInserted_File_Not_Found()
        {
            MockFile file;
            File::getInstance ( &file );

            MOCKPP_CHAINER_FOR_EXT ( MockFile, openFileToBuffer, ext ) openFileToBuffer_mock ( &file );

            openFileToBuffer_mock
            .expects ( MOCKPP_NS::once() )
            .will ( MOCKPP_NS::returnValue<bool> ( false ) );

            SimulatorSmartcard smartcard ( "eventFile", "contentFile" );

            CPPUNIT_ASSERT ( false == smartcard.isCardPresent() );

            file.verify();
        }

        void testCardInserted_Buffer_Empty()
        {
            StoreMockFile file;
            File::getInstance ( &file );

            SafeBuffer fileBuffer;

            file.openFileValues.push_back ( StoreMockFile::OpenFileValues ( fileBuffer, true ) );

            SimulatorSmartcard smartcard ( "eventFile", "contentFile" );

            CPPUNIT_ASSERT ( false == smartcard.isCardPresent() );
            CPPUNIT_ASSERT ( false == file.openFileListValues.empty() );
            CPPUNIT_ASSERT ( "eventFile" == file.openFileListValues.front() );
        }

        void testCardInserted_Buffer_TwoLine_Success()
        {
            StoreMockFile file;
            File::getInstance ( &file );

            SafeBuffer fileBuffer;
            fileBuffer.AddData ( "I\n" );

            file.openFileValues.push_back ( StoreMockFile::OpenFileValues ( fileBuffer, true ) );

            SimulatorSmartcard smartcard ( "eventFile", "contentFile" );

            CPPUNIT_ASSERT ( true == smartcard.isCardPresent() );
            CPPUNIT_ASSERT ( false == file.openFileListValues.empty() );
            CPPUNIT_ASSERT ( "eventFile" == file.openFileListValues.front() );
        }

        void testCardInserted_Buffer_Insert_With_Spaces_Success()
        {
            StoreMockFile file;
            File::getInstance ( &file );

            SafeBuffer fileBuffer;
            fileBuffer.AddData ( " I " );

            file.openFileValues.push_back ( StoreMockFile::OpenFileValues ( fileBuffer, true ) );

            SimulatorSmartcard smartcard ( "eventFile", "contentFile" );

            CPPUNIT_ASSERT ( true == smartcard.isCardPresent() );
            CPPUNIT_ASSERT ( false == file.openFileListValues.empty() );
            CPPUNIT_ASSERT ( "eventFile" == file.openFileListValues.front() );
        }


        void testCardInserted_Buffer_Insert_Success()
        {
            StoreMockFile file;
            File::getInstance ( &file );

            SafeBuffer fileBuffer;
            fileBuffer.AddData ( "I" );

            file.openFileValues.push_back ( StoreMockFile::OpenFileValues ( fileBuffer, true ) );

            SimulatorSmartcard smartcard ( "eventFile", "contentFile" );

            CPPUNIT_ASSERT ( true == smartcard.isCardPresent() );
            CPPUNIT_ASSERT ( false == file.openFileListValues.empty() );
            CPPUNIT_ASSERT ( "eventFile" == file.openFileListValues.front() );
        }

        CPPUNIT_TEST_SUITE ( SimulatorSmartcardTest );

        CPPUNIT_TEST ( testInitializeFail );
        CPPUNIT_TEST ( testInitializeSuccess );
        CPPUNIT_TEST ( testDispose );

        CPPUNIT_TEST ( testRead_File_Not_Found );
        CPPUNIT_TEST ( testReadFile_Buffer_Empty );
        CPPUNIT_TEST ( testReadFile_Buffer_Only_Password );
        CPPUNIT_TEST ( testReadFile_Buffer_Success );

        CPPUNIT_TEST ( testWrite_File_Fail );
        CPPUNIT_TEST ( testWriteFile_Buffer );

        CPPUNIT_TEST ( testCardInserted_File_Not_Found );
        CPPUNIT_TEST ( testCardInserted_Buffer_Empty );
        CPPUNIT_TEST ( testCardInserted_Buffer_TwoLine_Success );
        CPPUNIT_TEST ( testCardInserted_Buffer_Insert_With_Spaces_Success );
        CPPUNIT_TEST ( testCardInserted_Buffer_Insert_Success );


        CPPUNIT_TEST_SUITE_END();
};

#endif /* ECUT_MAIN */

#endif /* TEST_SIMULATOR_SMARTCARD_TEST */
