/**
 * @file	ResultsPackageTransmissionConfManager.hxx
 * @brief	Declaration of the results package transmission configuration manager.
 * @author	Yadickson Soto
 * @date	09/04/2014
 */

#ifndef _RESULTS_PACAKGE_TRANSMISSION_CONFIGURATION_MANAGER_HXX_
#define _RESULTS_PACAKGE_TRANSMISSION_CONFIGURATION_MANAGER_HXX_

#include <string>
#include <Configuration/results-package-transmission-schema.hxx>

namespace Smartmatic
{
    namespace SAES
    {
        namespace Functionality
        {
            namespace Configuration
            {
                /**
                 * @class	ResultsPackageTransmissionConfManager
                 * @brief	The results package transmission configuration manager.
                 * @author	Yadickson Soto
                 * @date	09/04/2014
                 */
                class ResultsPackageTransmissionConfManager
                {
                    public:

                        /**
                         * @fn	static ResultsPackageTransmissionConfManager * getInstance( ResultsPackageTransmissionConfManager * externManager = NULL );
                         * @brief	Gets the current ResultsPackageTransmission Configuration Manager.
                         * @author	Yadickson Soto
                         * @date	09/04/2014
                         * @param 	externManager external manager to unit test
                         * @return	instance of the class.
                         */
                        static ResultsPackageTransmissionConfManager * getInstance ( ResultsPackageTransmissionConfManager * externManager = NULL );

                        /**
                         * @fn	virtual std::string getNetworkManagerScript();
                         * @brief	Gets the network manager script path.
                         * @author	Yadickson Soto
                         * @date	09/04/2014
                         * @return	the network manager script path.
                         */
                        virtual std::string getNetworkManagerScript();

                        /**
                         * @fn	virtual std::string getResultsTransmissionScript();
                         * @brief	Gets the send results package transmission script path.
                         * @author	Yadickson Soto
                         * @date	09/04/2014
                         * @return	the send results package transmission script path.
                         */
                        virtual std::string getResultsTransmissionScript();

                        /**
                         * @fn	virtual bool isUpDownInterfaceScriptNeeded();
                         * @brief	Gets whether it is necessary to call network manager script for up/down interface.
                         * @author	Yadickson Soto
                         * @date	09/04/2014
                         * @return	true if call network manager script.
                         */
                        virtual bool isUpDownInterfaceScriptNeeded();

                        /**
                         * @fn	virtual bool isSendResultsPackageScriptNeeded();
                         * @brief	Gets whether the results package script should be called.
                         * @author	Yadickson Soto
                         * @date	09/04/2014
                         * @return	true if call send results script.
                         */
                        virtual bool isSendResultsPackageScriptNeeded();

                        /**
                         * @fn	virtual std::string getConfigFileName();
                         * @brief	Gets configuration file name.
                         * @author	Yadickson Soto
                         * @date	09/04/2014
                         * @return	configuration file name.
                         */
                        virtual std::string getConfigFileName();

                        /**
                         * @fn	virtual std::string getDebugFileName();
                         * @brief	Gets debug configuration file name.
                         * @author	Yadickson Soto
                         * @date	09/04/2014
                         * @return	debug configuration file name.
                         */
                        virtual std::string getDebugFileName();

                        /**
                         * @fn	virtual std::string getFileListToArchive ( const std::string & devicePath );
                         * @brief	Gets filename list, as a whitespace-separated string, of files part of the results package to transmit.
                         * @author	Yadickson Soto
                         * @date	09/04/2014
                         * @param	devicePath The device path
                         * @return	full file list name separate with whitespace.
                         */
                        virtual std::string getFileListToArchive ( const std::string & devicePath );

                        /**
                         * @fn	virtual std::string getResultsFileName();
                         * @brief	Gets the name of the results file to create and transmit.
                         * @author	Yadickson Soto
                         * @date	09/04/2014
                         * @return	results file name.
                         */
                        virtual std::string getResultsFileName();

                        /**
                         * @fn	virtual std::string getResponseFileName();
                         * @brief	Gets the filename where to save the HTTP upload response.
                         * @author	Yadickson Soto
                         * @date	09/04/2014
                         * @return	response file name.
                         */
                        virtual std::string getResponseFileName();

                        /**
                         * @fn	virtual bool useStaticIP();
                         * @brief	Whether to use static IP or not
                         * @author	Yadickson Soto
                         * @date	10/04/2014
                         * @return	true if DHCP configuration is not enable.
                         */
                        virtual bool useStaticIP();

                        /**
                         * @fn	virtual std::string getFullCommand( const std::string & centerName, const std::string & interface, const std::string & devicePath );
                         * @brief	ets the full command for sending the results package.
                         * @author	Yadickson Soto
                         * @date	10/04/2014
                         * @param	centerName The polling place name
                         * @param	interface The network interface name
                         * @param	devicePath The device path
                         * @return	full command to send results package transmission.
                         */
                        virtual std::string getFullCommandToSendResults ( const std::string & centerName, const std::string & interface, const std::string & devicePath );

                        /**
                         * @fn	virtual ~ResultsPackageTransmissionConfManager();
                         * @brief	Class destructor.
                         * @author	Yadickson Soto
                         * @date	09/04/2014
                         */
                        virtual ~ResultsPackageTransmissionConfManager();

                    protected:

                        /**
                         * @fn	ResultsPackageTransmissionConfManager();
                         * @brief	Class constructor.
                         * @author	Yadickson Soto
                         * @date	09/04/2014
                         */
                        ResultsPackageTransmissionConfManager();

                        /**
                         * @fn	virtual ResultsPackageTransmissionConfiguration & getResultsPackageTransmissionConfiguration ();
                         * @brief	Gets a reference to the results package transmission configuration.
                         * @author	Yadickson Soto
                         * @date	09/04/2014
                         * @return	The reference of the results package transmission configuration.
                         */
                        virtual ResultsPackageTransmissionConfiguration & getResultsPackageTransmissionConfiguration ();

                    private:

                        static ResultsPackageTransmissionConfManager * instance; ///< The pointer to the singleton instance of the results package transmission configuration manager
                };
            }
        }
    }
}

#endif // _RESULTS_PACAKGE_TRANSMISSION_CONFIGURATION_MANAGER_HXX_
