/**
 * @file s-a-e-s-functionality-conf-manager.h
 * @brief Declaration of the SAESFunctionalityConfigurationManager class.
 * @author Youssef.tourkmani
 * @date 01/11/2012
 */


#ifndef _S_A_E_S_FUNCTIONALITY_CONFIGURATION_MANAGER_H_
#define _S_A_E_S_FUNCTIONALITY_CONFIGURATION_MANAGER_H_

#include <list>
#include <string>
#include <Configuration/SaesFunctionalityConfiguration-schema.hxx>
#include <Configuration/menu-option-type-schema.hxx>

namespace Smartmatic
{
    namespace SAES
    {
        namespace Functionality
        {
            namespace Configuration
            {
                /**
                 * @class SAESFunctionalityConfigurationManager
                 * @brief Manager for SAES functionality configurations.
                 * @author Youssef.tourkmani
                 * @date 01/11/2012
                 */

                class SAESFunctionalityConfigurationManager
                {
                    public:

                        /**
                         * @fn static SAESFunctionalityConfigurationManager * getInstance( SAESFunctionalityConfigurationManager * externManager = NULL );
                         * @brief	Gets the current SAES Functionality Configuration Manager.
                         * @author	Yadickson Soto
                         * @date	09/04/2014
                         * @param 	externManager external manager to unit test
                         * @return	instance of the class
                         */
                        static SAESFunctionalityConfigurationManager * getInstance ( SAESFunctionalityConfigurationManager * externManager = NULL );

                        /**
                         * @fn SaesFunctionalityConfiguration * getConfiguration();
                         * @brief Gets the configuration.
                         * @author Youssef.tourkmani
                         * @date 01/11/2012
                         * @return The current configuration. Returns NULL if the method fails.
                         */
                        SaesFunctionalityConfiguration * getConfiguration();

                        /**
                         * @fn void loadDefault();
                         * @brief Loads the default configuration.
                         * @author Yadickson Soto
                         * @date 09/04/2014
                         */
                        void loadDefault();

                        /**
                         * @fn VotingExperienceControllerConfiguration getVotingExperienceControllerConfiguration();
                         * @brief Gets the voting experience controller configuration.
                         * @author Youssef.tourkmani
                         * @date 01/11/2012
                         * @return voting experience controller configuration.
                         */

                        VotingExperienceControllerConfiguration getVotingExperienceControllerConfiguration();

                        /**
                         * @fn AudioVotingExperienceControllerConfiguration getAudioVotingExperienceControllerConfiguration();
                         * @brief Gets the audio voting experience controller configuration.
                         * @author eduardo.fernandez
                         * @date 15/11/2017
                         * @return audio voting experience controller configuration.
                         */

                        AudioVotingExperienceControllerConfiguration getAudioVotingExperienceControllerConfiguration();

                        /**
                         * @fn OperationConfiguration getOperationConfiguration();
                         * @brief method to obtain the operation's configuration.
                         * @author Youssef.tourkmani
                         * @date 01/11/2012
                         * @return operation's configuration.
                         */
                        OperationConfiguration getOperationConfiguration();

                        /**
                         * @fn ScanVoteConfiguration & getScanVoteConfiguration();
                         * @brief Gets scan vote configuration.
                         * @author Youssef.tourkmani
                         * @date 01/11/2012
                         * @return The scan vote configuration.
                         */
                        ScanVoteConfiguration & getScanVoteConfiguration();

                        /**
                         * @fn FlowStatesConfiguration & getFlowStatesConfiguration();
                         * @brief Gets the flow step configuration.
                         * @author Yadickson Soto
                         * @date 07/01/2013
                         * @return The flow step configuration.
                         */
                        FlowStatesConfiguration & getFlowStatesConfiguration();

                        /**
                         * @fn SystemRoleConfiguration getSystemRoleConfiguration();
                         * @brief Gets the system role configuration.
                         * @author Youssef.tourkmani
                         * @date 01/11/2012
                         * @return system role configuration.
                         */

                        SystemRoleConfiguration getSystemRoleConfiguration();

                        /**
                         * @fn int getSuccesMessageTime();
                         * @brief Gets voting machine succes message time end machine initiator.
                         * @author Youssef.tourkmani
                         * @date 01/11/2012
                         * @return The voting machine succes message time.
                         */
                        int getSuccesMessageTime();
                        /**
                         * @fn PVFormTransformationConfig & getPVFormTransformation();
                         * @brief Gets pv form transformation.
                         * @author Youssef.tourkmani
                         * @date 01/11/2012
                         * @return The pv form transformation.
                         */
                        PVFormTransformationConfig & getPVFormTransformation();
                        /**
                         * @fn	StructureInstallationType & getStructureInstallationType();
                         * @brief	Gets structure installation type.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         * @return	The structure installation type.
                         */
                        StructureInstallationType & getStructureInstallationType();
                        /**
                         * @fn EnqueueVoteConfiguration & getEnqueueVoteConfiguration();
                         * @brief Gets enqueue vote configuration.
                         * @author Youssef.tourkmani
                         * @date 01/11/2012
                         * @return The enqueue vote configuration.
                         */
                        EnqueueVoteConfiguration & getEnqueueVoteConfiguration();
                        /**
                         * @fn BeforePrintingVoteComplianceVerificationConfiguration & getBeforePrintingVoteComplianceVerificationConfiguration();
                         * @brief Gets before printing vote compliance verification configuration.
                         * @author eduardo.fernandez
                         * @date 10/02/2018
                         * @return Before printing vote compliance verification configuration.
                         */
                        BeforePrintingVoteComplianceVerificationConfiguration & getBeforePrintingVoteComplianceVerificationConfiguration();
                        /**
                         * @fn AfterScanningVoteComplianceVerificationConfiguration & getAfterScanningVoteComplianceVerificationConfiguration();
                         * @brief Gets after scanning vote compliance verification configuration.
                         * @author eduardo.fernandez
                         * @date 10/02/2018
                         * @return After scanning vote compliance verification configuration.
                         */
                        AfterScanningVoteComplianceVerificationConfiguration & getAfterScanningVoteComplianceVerificationConfiguration();
                        /**
                         * @fn BeforeGeneratingResultsComplianceVerificationConfiguration & getBeforeGeneratingResultsComplianceVerificationConfiguration();
                         * @brief Gets before generating results compliance verification configuration.
                         * @author eduardo.fernandez
                         * @date 10/02/2018
                         * @return Before generating results compliance verification configuration.
                         */
                        BeforeGeneratingResultsComplianceVerificationConfiguration & getBeforeGeneratingResultsComplianceVerificationConfiguration();
                        /**
                         * @fn ProcessStatus getNextProcessStatus( ProcessStatus status );
                         * @brief Gets next state by current state
                         * @author Yadickson Soto
                         * @date 12/11/2012
                         * @param status current status
                         * @return next state configuration
                         */
                        ProcessStatus getNextProcessStatus ( ProcessStatus status );

                        /**
                         * @fn std::list<Smartmatic::SAES::Functionality::Configuration::MenuOptionType> getRunFunctionalityDescriptors();
                         * @brief Gets the run functionality descriptor list.
                         * @author Yadickson Soto
                         * @date 24/05/2013
                         * @return The list of run functionality descriptors.
                         */
                        std::list<Smartmatic::SAES::Functionality::Configuration::MenuOptionType> getRunFunctionalityDescriptors();

                        /**
                         * @fn std::list<Smartmatic::SAES::Functionality::Configuration::MenuOptionType> getRunFunctionalityDescriptorsByMode( Smartmatic::SAES::Voting::VotingCenters::::RunningMode mode );
                         * @brief Gets the run functionality descriptor list by running mode.
                         * @author Yadickson Soto
                         * @date 24/05/2013
                         * @param mode mode run functionalities, official or recount
                         * @return The list of run functionality descriptors by running mode.
                         */
                        std::list<Smartmatic::SAES::Functionality::Configuration::MenuOptionType> getRunFunctionalityDescriptorsByMode ( Smartmatic::SAES::Voting::VotingCenters::RunningMode mode );

                        /**
                         * @fn virtual ~SAESFunctionalityConfigurationManager();
                         * @brief Finalizes an instance of the SAESFunctionalityConfigurationManager class.
                         * @author Youssef.tourkmani
                         * @date 01/11/2012
                         */

                        virtual ~SAESFunctionalityConfigurationManager();

                    protected:

                        /**
                         * @fn SAESFunctionalityConfigurationManager();
                         * @brief Default constructor for the SAESFunctionalityConfigurationManager class.
                         * @author Youssef.tourkmani
                         * @date 01/11/2012
                         */
                        SAESFunctionalityConfigurationManager();

                        /**
                         * @fn virtual void initialize();
                         * @brief Initializes this object.
                         * @author Yadickson Soto
                         * @date 09/04/2014
                         */
                        virtual void initialize();

                    private:

                        static SAESFunctionalityConfigurationManager * instance; ///< The singleton instance
                        SaesFunctionalityConfiguration * configuration; ///< The configuration
                };
            }
        }
    }
}

#endif // _S_A_E_S_FUNCTIONALITY_CONFIGURATION_MANAGER_H_
