/**
 * @file	functionality.h
 *
 * @brief	Header file to the functionality class.
 */

#ifndef _FUNCTIONALITY_H_
#define _FUNCTIONALITY_H_

#include <glibmm.h>
#include <Log/ISMTTLog.h>

namespace Smartmatic
{
    namespace Functionality
    {
        /**
         * @class	FunctionalityManager
         * @brief	Manager for functionalities class.
         * @author	Rafael.gasperi
         * @date	28/04/2011
         * @tparam	TMenuOptionDescriptor	Type of the menu option descriptor.
         */
        template <class TMenuOptionDescriptor>
        class FunctionalityManager;

        /**
         * @class	Functionality
         * @brief	Functionality class.
         * @author	Rafael.gasperi
         * @date	28/04/2011
         * @tparam	TMenuOptionDescriptor	Type of the menu option descriptor.
         */
        template <class TMenuOptionDescriptor>
        class Functionality
        {

            public:

                /**
                 * @fn	Functionality::Functionality<TMenuOptionDescriptor>()
                 * @brief	Default constructor.
                 * @author	Rafael.gasperi
                 * @date	28/04/2011
                 * @tparam	TMenuOptionDescriptor	Type of the menu option descriptor.
                 */
                Functionality<TMenuOptionDescriptor>()
                    : loaded ( false )
                {
                    operation_manager = NULL;
                    loaded = false;
                }

                /**
                 * @fn	virtual Functionality::~Functionality<TMenuOptionDescriptor>()
                 * @brief	Finaliser.
                 * @author	Rafael.gasperi
                 * @date	28/04/2011
                 * @tparam	TMenuOptionDescriptor	Type of the menu option descriptor.
                 */
                virtual ~Functionality<TMenuOptionDescriptor>() {}

                /**
                 * @fn	void Functionality::Do()
                 * @brief	Does this object.
                 * @author	Rafael.gasperi
                 * @date	28/04/2011
                 * @exception	e	Thrown when Excute() fails.
                 */
                void Do()
                {
                    try
                    {
                        Execute();
                    }
                    catch ( std::exception & e )
                    {
                        getLogger()->Debug ( "Error execute functionality" );
                    }
                }

                /**
                 * @fn	void Functionality::BeginExecute();
                 * @brief	Begins an execute.
                 * @author	Rafael.gasperi
                 * @date	28/04/2011
                 */
                void BeginExecute();

                /**
                 * @fn	void Functionality::Register(FunctionalityManager<TMenuOptionDescriptor> *funct_manager)
                 * @brief	Registers this object.
                 * @author	Rafael.gasperi
                 * @date	28/04/2011
                 * @exception	e	Thrown when .
                 * @param	funct_manager	If non-null, manager for funct.
                 */
                void Register ( FunctionalityManager<TMenuOptionDescriptor> * funct_manager )
                {
                    try
                    {
                        funct_manager->Register ( this );
                    }
                    catch ( std::exception & e )
                    {
                        getLogger()->Debug ( "Error register functionality" );
                    }
                }

                /**
                 * @fn	virtual void Functionality::DisplayUnableMessage() = 0;
                 * @brief	Pure Virtual Function (Abstract Members)
                 * @author	Rafael.gasperi
                 * @date	28/04/2011
                 */
                virtual void DisplayUnableMessage() = 0;

                /**
                 * @fn	virtual void Functionality::Execute() = 0;
                 * @brief	Executes this object.
                 * @author	Rafael.gasperi
                 * @date	28/04/2011
                 */
                virtual void Execute() = 0;

                /**
                 * @fn	virtual bool Functionality::CanExecute() = 0;
                 * @brief	Return if this functionality can be executed.
                 * @author	Rafael.gasperi
                 * @date	28/04/2011
                 * @return	true if can execute functionality.
                 */
                virtual bool CanExecute() = 0;

                /**
                 * @fn	virtual bool Functionality::isRunning() = 0;
                 * @brief	Method getter is running.
                 * @author	Yadickson Soto
                 * @date	19/09/2012
                 * @return	true is running.
                 */
                virtual bool isRunning() = 0;

                /**
                 * @fn	virtual bool Functionality::Enable() = 0;
                 * @brief	Enables this object.
                 * @author	Rafael.gasperi
                 * @date	28/04/2011
                 * @return	true if it succeeds, false if it fails.
                 */
                virtual bool Enable() = 0;

                /**
                 * @fn	virtual void Functionality::ExitFunctionality() = 0;
                 * @brief	Virtual method exit functionality.
                 * @author	Yadickson Soto
                 * @date	28/10/2012
                 */
                virtual void ExitFunctionality() = 0;

                /**
                 * @fn	virtual TMenuOptionDescriptor Functionality::GetMenuOptionType() = 0;
                 * @brief	Gets the menu option type.
                 * @author	Rafael.gasperi
                 * @date	28/04/2011
                 * @return	The menu option type.
                 */
                virtual TMenuOptionDescriptor GetMenuOptionType() = 0;

                sigc::signal<void> ExecutionFinishedSigc; ///< dispatcher

            protected:

                void * operation_manager; ///< Manager for operation

                bool loaded; ///< true if the data was loaded

                /**
                 * @fn	virtual Smartmatic::Log::ISMTTLog* Functionality::getLogger() = 0;
                 * @brief	Gets the logger.
                 * @author	Rafael.gasperi
                 * @date	28/04/2011
                 * @return	null if it fails, else the logger.
                 */
                virtual Smartmatic::Log::ISMTTLog * getLogger() = 0;

                /**
                 * @fn	virtual Glib::ustring Functionality::getStartLogMessage() = 0;
                 * @brief	Gets the start log message.
                 * @author	Rafael.gasperi
                 * @date	28/04/2011
                 * @return	The start log message.
                 */
                virtual Glib::ustring getStartLogMessage() = 0;

                /**
                 * @fn	virtual Glib::ustring Functionality::getFinishLogMessage() = 0;
                 * @brief	Gets the finish log message.
                 * @author	Rafael.gasperi
                 * @date	28/04/2011
                 * @return	The finish log message.
                 */
                virtual Glib::ustring getFinishLogMessage() = 0;

            private:


        };
    }
}

#endif // _FUNCTIONALITY_H_
