/**
 * @file CheckDataManager.cxx
 * @brief Body class to check data and synchronize
 * @data 29/07/2015
 * @author Yadickson Soto
 */

#include <Operation/Monitor/DeviceTypeMonitor.hxx>
#include <Operation/Monitor/DeviceTypeMonitorPool.hxx>
#include <Operation/CheckDataManager.hxx>
#include <Operation/Devices/Polling/BCRPolling.hxx>
#include <Operation/Devices/Polling/CardPolling.hxx>
#include <Operation/MemoryEventManager.hxx>
#include <Operation/MachineOperationManager.hxx>
#include <Operation/LoginKeyManager.hxx>
#include <Runtime/Environment/MemoryPathController.h>
#include <System/Runtime/ProcessLauncher.h>
#include <Log/SMTTLogManager.h>

using namespace Smartmatic::SAES::Operation;
using namespace Smartmatic::SAES::Operation::Devices;
using namespace Smartmatic::SAES::Common::Configuration;
using namespace Smartmatic::SAES::Runtime::Environment;
using namespace Smartmatic::System::Runtime;
using namespace Smartmatic::SAES::Operation::Monitor;
using namespace Smartmatic::SAES::Common::Operation::Monitor;
using namespace Smartmatic::Log;

CheckDataManager * CheckDataManager::instance = NULL;

static void
checkPrematureExtraction()
{
    DeviceTypeMonitor* const device = DeviceTypeMonitorPool::getInstance()->getDeviceTypeMonitor(DeviceType::CONFIGURATION_USBSTORAGEMOUNTED);

    if (device->getNumberOfDevices() == 1)
        MemoryEventManager::getInstance()->addMemoryEvent ( MemoryEventManager::SAFE_EJECT );
}

CheckDataManager::CheckDataManager()
    : isCheckData(false), isSynchronize(false), isSafeEject(false), isWaitingMount(false), isInterrupted(false), hasNoMemories(false)
{
}

CheckDataManager::~CheckDataManager()
{
    instance = NULL;
}

void CheckDataManager::startDetection()
{
    MemoryEventManager::getInstance()->startEmitSignal();
}

void CheckDataManager::stopDetection()
{
    MemoryEventManager::getInstance()->stopEmitSignal();
}

void CheckDataManager::checkInsertion( unsigned int numberOfDevices )
{
    if ( !MemoryEventManager::getInstance()->isCanEmitSignal() )
    {
        return;
    }

    BCRPolling::getInstance()->pause ();
    CardPolling::getInstance()->pause ();

    std::string domMedia = MemoryPathController::getInstance()->getDomMemoryBasePath();
    std::string flashMedia = MemoryPathController::getInstance()->getFlashMemoryBasePath();

    if ( isInterrupted && !domMedia.empty() && !flashMedia.empty() )
    {
        start ( 0 ); // Resume operation
    }
    else if (isWaitingMount && !domMedia.empty() && !flashMedia.empty() && !isCheckData)
    {
        isWaitingMount = false;
        isCheckData = true;
        start ( 0 ); // Resume operation
    }
    else if (numberOfDevices == 2 && !isWaitingMount)
    {
        isWaitingMount = true;
    }
}

void CheckDataManager::checkExtraction ( unsigned int numberOfDevices )
{
    if ( !MemoryEventManager::getInstance()->isCanEmitSignal() )
    {
        return;
    }

    if ( numberOfDevices == 0 && isInterrupted && !hasNoMemories)
    {
        MemoryEventManager::getInstance()->addMemoryEvent ( MemoryEventManager::INTERRUPTED_NO_MEMORY );
        hasNoMemories = true;
    }
    else if ( isSafeEject && numberOfDevices == 1 )
    {
        BCRPolling::getInstance()->resume();
        CardPolling::getInstance()->resume();
        MemoryEventManager::getInstance()->addMemoryEvent ( MemoryEventManager::SAFE_EJECT );
    }
    else if ( numberOfDevices < 2 && !isInterrupted && !isSafeEject )
    {
        BCRPolling::getInstance()->pause ();
        CardPolling::getInstance()->pause();
        MemoryEventManager::getInstance()->addMemoryEvent ( numberOfDevices == 1 ? MemoryEventManager::ONE_MEMORY : MemoryEventManager::INTERRUPTED_NO_MEMORY );
    }
    else if ( numberOfDevices == 0 && isSafeEject )
    {
        BCRPolling::getInstance()->pause ();
        CardPolling::getInstance()->pause();
        MemoryEventManager::getInstance()->addMemoryEvent ( MemoryEventManager::INTERRUPTED_NO_MEMORY );
    }
}

void CheckDataManager::startSafeSynchronize()
{
    if ( !isSynchronize )
    {
        isSynchronize = true;

        BCRPolling::getInstance()->pause ();
        CardPolling::getInstance()->pause();
        start ( 0 ); // Starting synchronize
    }
}

void CheckDataManager::clearSynchronize()
{
    isSafeEject = false;
    isSynchronize = false;

    SMTTLogManager::getInstance()->shutdown();
    SMTTLogManager::getInstance()->Reconfigure();

    LoginKeyManager::getInstance()->clear();

    try
    {
        MachineOperationManager::getInstance()->restoreVoteInAllMemory();
    }
    catch (...)
    {
    }

    BCRPolling::getInstance()->resume();
    CardPolling::getInstance()->resume();
}

bool CheckDataManager::process ()
{
    if ( isCheckData )
    {
        checkDataDevice();
    }
    else if ( isSynchronize )
    {
        safeSynchronize();
    }

    return false;
}

void CheckDataManager::checkDataDevice()
{
    MemoryEventManager::getInstance()->addMemoryEvent ( MemoryEventManager::TWO_MEMORY );
    DeviceTypeMonitor * device = DeviceTypeMonitorPool::getInstance()->getDeviceTypeMonitor ( DeviceType::CONFIGURATION_USBSTORAGEMOUNTED );

    stopDetection();
    isCheckData = false;
    isInterrupted = false;

    if ( MachineOperationManager::getInstance()->checkData() )
    {
        startDetection();
        clearSynchronize();
        MemoryEventManager::getInstance()->addMemoryEvent ( MemoryEventManager::CHECK_MEMORY_OK );
    }
    else
    {
        g_usleep(750000);

        if ( device->getNumberOfDevices() == 2 )
        {
            startDetection();
            MemoryEventManager::getInstance()->addMemoryEvent ( MemoryEventManager::CHECK_MEMORY_ERROR );
        }
        else
        {
            MemoryEventManager::MemoryEvent event = MemoryEventManager::INTERRUPTED;

            hasNoMemories = false;

            if (device->getNumberOfDevices() == 0)
            {
                event = MemoryEventManager::INTERRUPTED_NO_MEMORY;
                hasNoMemories = true;
            }

            isInterrupted = true;
            isCheckData = true;
            startDetection();
            MemoryEventManager::getInstance()->addMemoryEvent(event);
        }
    }
}

void CheckDataManager::safeSynchronize ()
{
    DeviceTypeMonitor* device = DeviceTypeMonitorPool::getInstance()->getDeviceTypeMonitor(DeviceType::CONFIGURATION_USBSTORAGEMOUNTED);

    MemoryEventManager::getInstance()->addMemoryEvent(MemoryEventManager::START_SYNCHRONIZE);
    stopDetection();
    isSynchronize = false;
    isInterrupted = false;

    if (MachineOperationManager::getInstance()->synchronize())
    {
        isSafeEject = true;
        startDetection();
        MemoryEventManager::getInstance()->addMemoryEvent(MemoryEventManager::END_SYNCHRONIZE);
        Glib::signal_timeout().connect_once(sigc::ptr_fun(&checkPrematureExtraction), 250);
    }
    else
    {
        isSafeEject = false;
        g_usleep(750000);

        if (device->getNumberOfDevices() == 2)
        {
            startDetection();
            MemoryEventManager::getInstance()->addMemoryEvent(MemoryEventManager::CHECK_MEMORY_ERROR);
        }
        else
        {
            MemoryEventManager::MemoryEvent event = MemoryEventManager::INTERRUPTED;

            hasNoMemories = false;

            if (device->getNumberOfDevices() == 0)
            {
                event = MemoryEventManager::INTERRUPTED_NO_MEMORY;
                hasNoMemories = true;
            }

            isInterrupted = true;
            isSynchronize = true;
            startDetection();
            MemoryEventManager::getInstance()->addMemoryEvent(event);
        }
    }
}

CheckDataManager * CheckDataManager::getInstance()
{
    if ( instance == NULL )
    {
        instance = new CheckDataManager();
    }

    return instance;
}
