/**
 * @file   A4517AbledController.hxx
 * @brief  Declaration of A4517AbledController class.
 * @author eduardo.fernandez
 * @date   Jan 11, 2018
 */

#ifndef A4517_ABLED_CONTROLLER_HXX
#define A4517_ABLED_CONTROLLER_HXX

#include <glibmm.h>
#include <string>
#include <sigc++/sigc++.h>

namespace Smartmatic
{
    namespace Log
    {
        class ISMTTLog;
    }

    namespace SAES
    {
        namespace HW
        {
            namespace Devices
            {
                class AbledInterface;
            }

            namespace Operation
            {
                class AbleDButtonState;
            }
        }

        namespace Operation
        {
            namespace Devices
            {
                /**
                 * @class  A4517AbledController
                 * @brief  Basic A4517 abled controller class.
                 * @author eduardo.fernandez
                 * @date   Jan 11, 2018
                 */
                class A4517AbledController
                {
                    public:

                        /**
                         * @brief  Destroy an instance of the class.
                         * @author eduardo.fernandez
                         * @date   Jan 11, 2018
                         */
                        virtual ~A4517AbledController();

                    public:

                        /**
                         * @fn static A4517AbledController * A4517AbledController::getInstance();
                         * @brief Gets the current instance of the A4517AbledController class.
                         * @author eduardo.fernandez
                         * @date Jan 13, 2018
                         * @return instance class.
                         */
                        static A4517AbledController * getInstance(A4517AbledController * externInstance = NULL);

                    public:

                        /**
                         * @fn bool open (const std::string & device);
                         * @brief Opens a connection to the abled device.
                         * @author eduardo.fernandez
                         * @date Jan 13, 2018
                         * @param  device the abled device.
                         * @return instance class.
                         */
                        bool open (const std::string & device);

                        /**
                         * @fn bool close ();
                         * @brief Closes current connection to the abled device.
                         * @author eduardo.fernandez
                         * @date Jan 13, 2018
                         * @return instance class.
                         */
                        bool close ();

                        /**
                         * @brief  Enables or disables abled notifications.
                         * @author eduardo.fernandez
                         * @date   Jan 12, 2018
                         * @return Whether the abled notifications were enabled or disabled successfully.
                         * @param  enable Whether to enable the abled notifications.
                         */
                        bool enableNotifications(const bool enable);

                        /**
                         * @brief  Gets the FW version from the PDB's.
                         * @author manuel.arguelles
                         * @date   Nov  2, 2017
                        */
                        std::string getFWVersion();

                        /**
                         * @brief  Gets the serial number from the PDB's EEPROM.
                         * @author manuel.arguelles
                         * @date   Nov  2, 2017
                        */
                        std::string getSerialNumber();

                        /**
                         * @brief  Sets the serial number to the PDB's EEPROM.
                         * @author manuel.arguelles
                         * @date   Nov  2, 2017
                         */
                        void setSerialNumber ( const std::string & serial, bool save );

                        /**
                         * @brief  Method getter able-d button status.
                         * @author eduardo.fernandez
                         * @date   Jan 15, 2018
                         * @return AbleD button status
                         */
                        SAES::HW::Operation::AbleDButtonState* getA4517AbleDButtonState ();

                        /**
                         * @brief  Checks if the abled is present.
                         * @author manuel.arguelles
                         * @date   Jan 29, 2018
                         * @return True if present, false otherwise.
                         */
                        virtual bool isPresent() const;

                    protected:

                        /**
                         * @brief  Create an instance of the class
                         * @author eduardo.fernandez
                         * @date   Jan 11, 2018
                         */
                        A4517AbledController();

                    private:

                        static A4517AbledController *                    m_instance; ///< Instance
                        Glib::Mutex                                      m_mutex;  ///< The mutex
                        static Smartmatic::Log::ISMTTLog *               m_logger;  ///< The logger
                        std::auto_ptr<SAES::HW::Devices::AbledInterface> m_device;
                        bool                                             m_present;
                };

            } /* namespace AudioVoting */
        } /* namespace Operation */
    } /* namespace SAES */
} /* namespace Smartmatic */

#endif /* A4517_ABLED_CONTROLLER_HXX */
