/**
 * @file AlarmController.cxx
 * @brief Body class alarm box monitor
 * @date 07/06/2015
 * @author Yadickson Soto
 */

#include "AlarmController.hxx"
#include <Log/ISMTTLog.h>
#include <Log/SMTTLogManager.h>
#include <Devices/FactoryDevice.hxx>
#include <unistd.h>
#include <Configuration/AlarmConfigurationManager.hxx>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

#define N_(String) (String) ///<macro for gettext not translate

using namespace Smartmatic::SAES::Operation::Devices;
using namespace Smartmatic::SAES::Functionality::Configuration;
using namespace Smartmatic::SAES::HW::Devices;
using namespace Smartmatic::System;
using namespace Smartmatic::Log;

ISMTTLog * AlarmController::logger = SMTTLogManager::GetLogger ( "Smartmatic.SAES.Operation.Card.AlarmController", GETTEXT_PACKAGE );
AlarmController * AlarmController::instance = NULL;

AlarmController::AlarmController()
    : message(" ")
    , alarmBoxDevice(NULL)
{
}

AlarmController::~AlarmController()
{
    if ( alarmBoxDevice )
    {
        delete alarmBoxDevice;
        alarmBoxDevice = NULL;
    }

    instance = NULL;
}

bool AlarmController::open ( const std::string & device )
{
    Glib::Mutex::Lock lock ( mutex );
    alarmBoxDevice = ( alarmBoxDevice == NULL ? FactoryDevice::getAlarmBoxDevice() : alarmBoxDevice );
    return alarmBoxDevice->open ( device );
}

bool AlarmController::close()
{
    Glib::Mutex::Lock lock ( mutex );
    return alarmBoxDevice != NULL && alarmBoxDevice->close();
}

bool AlarmController::sendPing()
{
    Glib::Mutex::Lock lock ( mutex );
    if(alarmBoxDevice != NULL)
    {
        for(int retries = 0; retries < 6; retries++)
        {
            bool valid = alarmBoxDevice->sendPing();
            if(valid)
            {
                return true;
            }
        }
        return false;
    }
    else
    {
        return false;
    }
}

bool AlarmController::turnOnRedLed ( unsigned char period )
{
    Glib::Mutex::Lock lock ( mutex );
    return alarmBoxDevice != NULL && alarmBoxDevice->turnOnRedLed ( period );
}

bool AlarmController::turnOffRedLed()
{
    Glib::Mutex::Lock lock ( mutex );
    return alarmBoxDevice != NULL && alarmBoxDevice->turnOffRedLed();
}

bool AlarmController::turnOnGreenLed ( unsigned char period )
{
    Glib::Mutex::Lock lock ( mutex );
    return alarmBoxDevice != NULL && alarmBoxDevice->turnOnGreenLed ( period );
}

bool AlarmController::turnOffGreenLed()
{
    Glib::Mutex::Lock lock ( mutex );
    return alarmBoxDevice != NULL && alarmBoxDevice->turnOffGreenLed();
}

bool AlarmController::turnOnBuzzer ( unsigned char period )
{
    Glib::Mutex::Lock lock ( mutex );
    return alarmBoxDevice != NULL && alarmBoxDevice->turnOnBuzzer ( period );
}

bool AlarmController::turnOffBuzzer()
{
    Glib::Mutex::Lock lock ( mutex );
    return alarmBoxDevice != NULL && alarmBoxDevice->turnOffBuzzer();
}

bool AlarmController::isButtonPressed ()
{
    Glib::Mutex::Lock lock ( mutex );
    return alarmBoxDevice != NULL && alarmBoxDevice->isButtonPressed();
}

void AlarmController::addAlarm ( const AlarmEnum & alarm )
{
    switch ( alarm )
    {
        case AlarmEnum::CONFIGURATION_UNABLETOREAD:
            logger->Audit ( N_ ( "Smartmatic.SAES.Operation.AlarmController.LogUnableToReadCard" ) );
            logger->Debug ( "Alarm sent was: LogUnableToReadCard" );
            break;

        case AlarmEnum::CONFIGURATION_UNABLETOWRITE:
            logger->Audit ( N_ ( "Smartmatic.SAES.Operation.AlarmController.LogUnableToWriteCard" ) );
            logger->Debug ( "Alarm sent was: Unable to write card" );
            break;

        case AlarmEnum::CONFIGURATION_CARDVALIDITYPERIODEXPIRED:
            logger->Audit ( N_ ( "Smartmatic.SAES.Operation.AlarmController.LogUnableToWriteCard" ) );
            logger->Debug ( "Alarm sent was: Unable to write card" );
            break;

        case  AlarmEnum::CONFIGURATION_CRYPTOERROR:
            logger->Audit ( N_ ( "Smartmatic.SAES.Operation.AlarmController.LogCryptographicError" ) );
            logger->Debug ( "Alarm sent was: Cryptographic error" );
            break;

        case AlarmEnum::CONFIGURATION_INVALIDCONTENT:
            logger->Audit ( N_ ( "Smartmatic.SAES.Operation.AlarmController.LogInvalidCard" ) );
            logger->Debug ( "Alarm sent was: Invalid card" );
            break;

        case AlarmEnum::CONFIGURATION_CARDUSED:
            logger->Audit ( N_ ( "Smartmatic.SAES.Operation.AlarmController.LogCardUsed" ) );
            logger->Debug ( "Alarm sent was: Card used" );
            break;

        case AlarmEnum::CONFIGURATION_INVALIDVOTINGDATA:
            logger->Audit ( N_ ( "Smartmatic.SAES.Operation.AlarmController.LogInvalidVotingCardData" ) );
            logger->Debug ( "Alarm sent was: Invalid voting card data" );
            break;

        case AlarmEnum::CONFIGURATION_PRINTERNOTREADYFORVOTE:
            logger->Audit ( N_ ( "Smartmatic.SAES.Operation.AlarmController.LogPrinterNotReadyForVoterActivation" ) );
            logger->Debug ( "Alarm sent was: Printer not ready for voter activation" );
            break;

        case AlarmEnum::CONFIGURATION_VOTEPRINTINGFAILURE:
            logger->Audit ( N_ ( "Smartmatic.SAES.Operation.AlarmController.LogPrinterFailure" ) );
            logger->Debug ( "Alarm sent was: Printer device failure" );
            break;

        case AlarmEnum::CONFIGURATION_VOTECOMPLIANCEFAILURE:
            logger->Audit ( N_ ( "Smartmatic.SAES.Operation.AlarmController.LogVoteComplianceFailure" ) );
            logger->Debug ( "Alarm sent was: Vote compliance failure" );
            break;

        case AlarmEnum::CONFIGURATION_PRINTERNEOPAFTERVOTE:
            logger->Audit ( N_ ( "Smartmatic.SAES.Operation.AlarmController.LogPrinterNearEndOfPaperAfterVoting" ) );
            logger->Debug ( "Alarm sent was: Printer near end of paper after last vote printed" );
            break;

        case AlarmEnum::CONFIGURATION_PRINTERNEOPBEFOREVOTE:
            logger->Audit ( N_ ( "Smartmatic.SAES.Operation.AlarmController.LogPrinterNearEndOfPaperBeforeVoting" ) );
            logger->Debug ( "Alarm sent was: Printer near end of paper before last vote printed" );
            break;

        case AlarmEnum::CONFIGURATION_TIMEOUT:
            logger->Audit ( N_ ( "Smartmatic.SAES.Operation.AlarmController.LogTimeOut" ) );
            logger->Debug ( "Alarm sent was: Time out" );
            break;

        default:
        case AlarmEnum::CONFIGURATION_UNEXPECTED:
            logger->Audit ( N_ ( "Smartmatic.SAES.Operation.AlarmController.LogUnexpected" ) );
            logger->Debug ( "Alarm sent was: Unexpected" );
            break;

        case AlarmEnum::CONFIGURATION_CARDNOTREMOVEDAFTERVOTING:
            logger->Audit ( N_ ( "Smartmatic.SAES.Operation.AlarmController.LogCardNotRemovedAfterVoting" ) );
            logger->Debug ( "Alarm sent was: CardNotRemovedAfterVoting" );
            break;

        case AlarmEnum::CONFIGURATION_CARDEXTRACTIONCOUNTER:
            logger->Audit ( N_ ( "Smartmatic.SAES.Operation.AlarmControllAlarmControllerer.LogMaximunCardExtractionCounterAlarm" ) );
            logger->Debug ( "Alarm sent was: CardExtractionCounterAlarm" );
            break;

        case AlarmEnum::CONFIGURATION_ALARMBOXDISCONNECTED:
            logger->Audit ( N_ ( "Smartmatic.SAES.Operation.AlarmController.AlarmBoxDisconnected" ) );
            logger->Debug ( "Alarm sent was: AlarmBoxDisconnected" );
            break;

        case AlarmEnum::CONFIGURATION_ALARMBOXCONNECTED:
            logger->Audit ( N_ ( "Smartmatic.SAES.Operation.AlarmController.AlarmBoxConnected" ) );
            logger->Debug ( "Alarm sent was: AlarmBoxConnected" );
            break;

        case AlarmEnum::CONFIGURATION_CARDREADERUNRESPONSIVE:
            logger->Audit ( N_ ( "Smartmatic.SAES.Operation.AlarmController.CardReaderUnresponsive" ) );
            logger->Debug ( "Alarm sent was: CardReaderUnresponsive" );
            break;

        case AlarmEnum::CONFIGURATION_DETECTEDAUDIOCARDBUTNOTCONFIGUREDFORAUDIO:
            logger->Audit ( N_ ( "Smartmatic.SAES.Operation.AlarmController.DetectedAudioCardButNotConfiguredForAudio" ));
            logger->Debug ( "Alarm sent was: DetectedAudioCardButNotConfiguredForAudio" );
            break;

        case AlarmEnum::CONFIGURATION_DETECTEDAUDIOCARDBUTNOABLEPRESENT:
            logger->Audit ( N_ ( "Smartmatic.SAES.Operation.AlarmController.DetectedAudioCardButNoAblePresent" ));
            logger->Debug ( "Alarm sent was: DetectedAudioCardButNoablePresent" );
            break;

        case AlarmEnum::CONFIGURATION_DETECTEDAUDIOCARDBUTNOVOICESPRESENT:
            logger->Audit ( N_ ( "Smartmatic.SAES.Operation.AlarmController.DetectedAudioCardButNoVoicesPresent" ));
            logger->Debug ( "Alarm sent was: DetectedAudioCardButNoVoicesPresent" );
            break;
    }

    if ( AlarmConfigurationManager::getInstance()->getCategoryOfAlarm ( alarm ) != AlarmCategoryEnum::CONFIGURATION_NOTALARM )
    {
        alarmList.push_back ( alarm );
    }
}

void AlarmController::clearAlarms()
{
    logger->Audit ( N_ ( "Smartmatic.SAES.Operation.AlarmController.LogStackOfAlarmsCleared" ) );
    logger->Debug ( "Stack of Alarms Cleared" );
    alarmList.clear();
}

bool AlarmController::isCriticalAlarmPending()
{
    bool criticalAlarmPending = false;
    std::vector<AlarmEnum>::iterator it;

    for ( it = alarmList.begin();
            it != alarmList.end();
            ++it )
    {
        if ( AlarmConfigurationManager::getInstance()->getCategoryOfAlarm ( *it ) == AlarmCategoryEnum::CONFIGURATION_CRITICALALARM )
        {
            criticalAlarmPending = true;
            break;
        }
    }

    return criticalAlarmPending;
}

bool AlarmController::isGoToAuthentication()
{
    bool goToAuthentication = false;
    std::vector<AlarmEnum>::iterator it;

    for ( it = alarmList.begin();
            it != alarmList.end();
            ++it )
    {
        if ( AlarmConfigurationManager::getInstance()->getBehaviorOfAlarm ( *it ) == AlarmBehaviorEnum::CONFIGURATION_GOAUTHENTICATION )
        {
            goToAuthentication = true;
            break;
        }
    }

    return goToAuthentication;
}

bool AlarmController::isAlarmPending()
{
    return !alarmList.empty();
}

void AlarmController::setMessage ( const std::string & message )
{
    this->message = message;
}

std::string AlarmController::getMessage()
{
    return message;
}

void AlarmController::soundAlarm ( const AlarmEnum & alarm )
{
    Alarm::AlarmCommandsSequence alarmCommands = AlarmConfigurationManager::getInstance()->getCommandsOfAlarm ( alarm );
    Alarm::AlarmCommandsIterator it;

    for ( it = alarmCommands.begin();
            it != alarmCommands.end();
            ++it )
    {
        switch ( ( *it ).getTypeOfCommand() )
        {
            case AlarmCommandEnum::CONFIGURATION_GREENLED:

                logger->Debug ( "Device is Green Led" );

                if ( ( *it ).getIsOff() )
                {
                    turnOffGreenLed();
                }
                else
                {
                    turnOnGreenLed ( ( *it ).getPeriod() );
                }

                break;

            case AlarmCommandEnum::CONFIGURATION_REDLED:

                logger->Debug ( "Device is Red Led" );

                if ( ( *it ).getIsOff() )
                {
                    turnOffRedLed();
                }
                else
                {
                    turnOnRedLed ( ( *it ).getPeriod() );
                }

                break;

            case AlarmCommandEnum::CONFIGURATION_BUZZER:

                logger->Debug ( "Device is Buzzer" );

                if ( ( *it ).getIsOff() )
                {
                    turnOffBuzzer();
                }
                else
                {
                    turnOnBuzzer ( ( *it ).getPeriod() );
                }

                break;

            default:
                break;
        }
    }
}

void AlarmController::shutDownAlarm()
{
    turnOffRedLed();
    turnOffGreenLed();
    turnOffBuzzer();
}

bool AlarmController::test ( unsigned int wait )
{
    bool validate = sendPing();

    if ( validate )
    {
        turnOnRedLed ( 1 );
        turnOnGreenLed ( 1 );
        turnOnBuzzer ( 1 );

        usleep ( wait * 1000 );

        turnOffRedLed();
        turnOffGreenLed();
        turnOffBuzzer();
    }

    return validate;
}

AlarmController * AlarmController::getInstance()
{
    if ( !instance )
    {
        instance = new AlarmController();
    }

    return instance;
}
