/*
 * EurnController.cxx
 *
 *  Created on: Oct 25, 2017
 *      Author: juanvalera
 */
#include "EurnController.hxx"
#include <Log/ISMTTLog.h>
#include <Log/SMTTLogManager.h>
#include <Devices/FactoryDevice.hxx>
#include <System/Utils/StringUtils.hxx>
#include <Operation/Voting/VotingEnqueuer.hxx>
#include <Operation/Devices/Polling/BCRPolling.hxx>
#include <Configuration/EurnDeviceConfigurationManager.hxx>
#include <unistd.h>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

using namespace Smartmatic::SAES::Operation::Devices;
using namespace Smartmatic::SAES::Operation;
using namespace Smartmatic::Log;
using namespace Smartmatic::SAES::HW::Devices;
using namespace Smartmatic::SAES::HW::Configuration;
using namespace Smartmatic::System;
using namespace Smartmatic::System::Utils;
using namespace Smartmatic::SAES::Operation::Voting;

EurnController * EurnController::instance = NULL;
ISMTTLog * EurnController::logger = SMTTLogManager::GetLogger ( "Smartmatic.SAES.Operation.BCR.EurnController", GETTEXT_PACKAGE );

EurnController::EurnController()
{
    this->eurnDevice = NULL;
    this->deviceStatus = EurnController::NOT_CONNECTED;
    this->lastDeviceStatus = EurnController::NOT_CONNECTED;
    abortConnection = false;
    allowProcessingCommandsQueue();
}

EurnController::~EurnController()
{

}

EurnController * EurnController::getInstance(EurnController * externInstance)
{
    if ( !instance || externInstance)
    {
        instance = ( externInstance == NULL ? new EurnController() : externInstance );
    }

    return instance;
}

bool EurnController::open ( const std::string & device )
{
    abortConnection = false;
    Glib::Mutex::Lock lock ( mutex );

    logger->Debug ( "Open Eurn device" );

    eurnDevice = eurnDevice == NULL ? FactoryDevice::getEurnDevice() : eurnDevice;
    bool connected = false;

    for(int i=0;i<15;i++)
    {
        if(!abortConnection)
        {
            connected = eurnDevice->open ( device );
        }
        if(abortConnection || connected)
        {
            break;
        }
        usleep(200000);
    }

    if(connected)
    {
       signalEurnInteraction = VotingEnqueuer::getInstance()->getSignalEurnInteration().connect ( sigc::mem_fun ( *this, &EurnController::processEurnInteraction ) );
       allowProcessingCommandsQueue();
       this->deviceStatus = EurnController::CONNECTED_WORKING;
    }

    logger->Debug ( std::string ( "Open EurnDevice device : " ) + ( connected ? "Success" : "Fail" ) );
    evaluateAndNotifyEurnStatus();

    return connected;
}

bool EurnController::close()
{
    abortConnection = true;;
    signalEurnInteraction.disconnect();
    stopProcessingCommandsQueue();
    Glib::Mutex::Lock lock ( mutex );

    logger->Debug ( "Close Eurn device" );
    this->deviceStatus = EurnController::NOT_CONNECTED;
    evaluateAndNotifyEurnStatus();

    return eurnDevice != NULL && eurnDevice->close();
}

void EurnController::allowProcessingCommandsQueue()
{
    this->stopProcessingQueue = false;
}

void EurnController::stopProcessingCommandsQueue()
{
    this->stopProcessingQueue = true;
}

void EurnController::processSuccesEurnInteraction()
{
    Glib::Mutex::Lock lock ( mutex );

    if (eurnDevice != NULL)
    {
        bool result = eurnDevice->openDoorForAFiniteTimeWithLedActions();

        if (!result)
        {
            this->deviceStatus = EurnController::CONNECTED_NOT_RESPONDING;
            evaluateAndNotifyEurnStatus();
            return;
        }

        evaluateAndNotifyEurnStatus();
    }
}

void EurnController::processFailedEurnInteraction()
{
    Glib::Mutex::Lock lock ( mutex );

    if (eurnDevice != NULL)
    {
        bool result = false;
        const bool blinkRedLed = EurnDeviceConfigurationManager::getInstance()->getConfiguration().getBlinkRedLed();

        if (blinkRedLed)
        {
            result = eurnDevice->blinkRedLedOn();
        }
        else
        {
            result = eurnDevice->turnRedLedOnIndefinitely();
        }

        if (!result)
        {
            this->deviceStatus = EurnController::CONNECTED_NOT_RESPONDING;
            evaluateAndNotifyEurnStatus();

            return;
        }

        const bool closeWhenInvalidVote = EurnDeviceConfigurationManager::getInstance()->getConfiguration().getCloseWhenInvalidVote();
        const bool closeDoor = eurnDevice->closeDoor();

        if ((closeWhenInvalidVote) && (closeDoor == false))
        {
            this->deviceStatus = EurnController::CONNECTED_NOT_RESPONDING;
        }

        evaluateAndNotifyEurnStatus();
    }
}

void EurnController::processEurnInteraction(bool result)
{
    Glib::Mutex::Lock lock ( mutex, Glib::TRY_LOCK );

    if(lock.locked() && !this->stopProcessingQueue)
    {
       if(result)
       {
          m_pool.push(sigc::mem_fun(*this,&EurnController::processSuccesEurnInteraction) );
       }
       else
       {
          m_pool.push(sigc::mem_fun(*this,&EurnController::processFailedEurnInteraction) );
       }
    }
    else
    {
        logger->Debug (  "EurnController is busy........... " );
    }
}

EurnController::SignalEurnStatusChanged EurnController::getSignalEurnSatusChanged()
{
    return signalEurnStatusChanged;
}

void EurnController::evaluateAndNotifyEurnStatus()
{
   if (this->lastDeviceStatus != this->deviceStatus)
   {
       this->lastDeviceStatus = this->deviceStatus;
       notifyEurnStatus();
   }
}

void EurnController::notifyEurnStatus()
{
    getSignalEurnSatusChanged().emit(this->deviceStatus);
}

std::string EurnController::getVersion()
{
    Glib::Mutex::Lock lock ( mutex );

    if (eurnDevice == NULL)
    {
       return "not present";
    }

    std::string version = eurnDevice->getVersion();

    return version;
}

bool EurnController::runDiagnosticSequence()
{
    Glib::Mutex::Lock lock ( mutex );

    if (eurnDevice == NULL)
    {
       return false;
    }

    bool result = eurnDevice->openDoorForAFiniteTime();

    if (!result || stopProcessingQueue)
    {
       logger->Debug(  "Error diagnosing open door" );
       return result;
    }

    result = eurnDevice->blinkGreenLedOnWithRedLedTurnOnAfter();

    if (!result || stopProcessingQueue)
    {
       logger->Debug(  "Error diagnosing turning on GreenLed" );
       return result;
    }

    return result;
}

bool EurnController::runDiagnoseOpenDoor()
{
    Glib::Mutex::Lock lock ( mutex );

    if (eurnDevice == NULL)
    {
       return false;
    }

    bool result = eurnDevice->openDoor();

    if (!result || stopProcessingQueue)
    {
       logger->Debug(  "Error diagnosing open door" );
    }

    return result;
}

bool EurnController::runDiagnoseCloseDoor()
{
    Glib::Mutex::Lock lock ( mutex );

    if (eurnDevice == NULL)
    {
       return false;
    }

    bool result = eurnDevice->closeDoor();

    if (!result || stopProcessingQueue)
    {
       logger->Debug(  "Error diagnosing close door" );
    }

    return result;
}

bool EurnController::runDiagnoseTurnOnGreenLed()
{
    Glib::Mutex::Lock lock ( mutex );

    if (eurnDevice == NULL)
    {
       return false;
    }

    bool result = eurnDevice->blinkGreenLedOn();

    if (!result || stopProcessingQueue)
    {
       logger->Debug(  "Error diagnosing blinking on green led" );
    }

    return result;
}

bool EurnController::runDiagnoseTurnOnRedLed()
{
    Glib::Mutex::Lock lock ( mutex );

    if (eurnDevice == NULL)
    {
       return false;
    }

    bool result = eurnDevice->turnRedLedOnIndefinitely();

    if (!result || stopProcessingQueue)
    {
       logger->Debug(  "Error diagnosing red led" );
    }

    return result;
}

bool EurnController::runDiagnoseTurnOffGreenLed()
{
    Glib::Mutex::Lock lock ( mutex );

    if (eurnDevice == NULL)
    {
       return false;
    }

    bool result = eurnDevice->turnGreenLedOff();

    if (!result || stopProcessingQueue)
    {
       logger->Debug(  "Error diagnosing turn off green led" );
    }

    return result;
}

bool EurnController::runDiagnoseTurnOffRedLed()
{
    Glib::Mutex::Lock lock ( mutex );

    if (eurnDevice == NULL)
    {
       return false;
    }

    bool result = eurnDevice->blinkRedLedOff();

    if (!result || stopProcessingQueue)
    {
       logger->Debug(  "Error diagnosing turn off red led" );
    }

    return result;
}

void EurnController::turnRedLedOnIndefinitely()
{
    bool result = eurnDevice->turnRedLedOnIndefinitely();

    if (!result)
    {
       logger->Debug( "Error turning red led on indefinitely" );
    }
}
