/*
 * EurnController.hxx
 *
 *  Created on: Oct 25, 2017
 *      Author: juanvalera
 */

#ifndef EURNCONTROLLER_HXX_
#define EURNCONTROLLER_HXX_

#include <glibmm.h>
#include <Devices/EurnInterface.hxx>

namespace Smartmatic
{
    namespace Log
    {
        class ISMTTLog;
    }

    namespace SAES
    {
        namespace Operation
        {
            namespace Diagnosis
            {
                class EurnDiagnosis;
            }

            namespace Devices
            {
                /**
                 * @class EurnController
                 * @brief Eurn Controller is the singleton instance which provide the support to the device.
                 * @author Juan Valera
                 * @date 25/10/2017
                 */
                class EurnController
                {
                    public:

                       enum DeviceStatus
                       {
                          CONNECTED_WORKING,
                          CONNECTED_NOT_RESPONDING,
                          NOT_CONNECTED
                       };

                       /**
                        * @typedef sigc::signal<void, DeviceStatus > SignalEurnStatusChanged
                        * @brief Defines an alias representing the signal Eurn status changed.
                        */
                       typedef sigc::signal<void, DeviceStatus > SignalEurnStatusChanged; ///< typedef signal status

                        /**
                         * @fn virtual EurnController::~EurnController();
                         * @brief Finalizes an instance of the EurnController class.
                         * @author Juan Valera
                         * @date 25/10/2017
                         */
                        virtual ~EurnController();

                        /**
                         * @fn static EurnController * EurnController::getInstance();
                         * @brief Gets the current instance of the EurnController class.
                         * @author Juan Valera
                         * @date 25/10/2017
                         * @return instance class.
                         */
                        static EurnController * getInstance(EurnController * externInstance = NULL);

                        /**
                         * @fn	virtual bool open( const std::string & device );
                         * @brief	Opens the connections.
                         * @author	Juan Valera
                         * @date	25/10/2017
                         * @param 	device device path
                         * @return	true if successful.
                         */
                        virtual bool open ( const std::string & device );

                        /**
                         * @fn	virtual bool close();
                         * @brief	Closes the connections.
                         * @author	Juan Valera
                         * @date	25/10/2017
                         * @return	true if successful.
                         */
                        virtual bool close();

                        /**
                         * @fn  virtual void allowProcessingCommandsQueue ();
                         * @brief Allow processing commands queue.
                         * @author  Juan Valera
                         * @date  26/10/2017
                         */
                        virtual void allowProcessingCommandsQueue ();

                        /**
                         * @fn  virtual void stopProcessingCommandsQueue ();
                         * @brief Stop processing commands queue.
                         * @author  Juan Valera
                         * @date  26/10/2017
                         */
                        virtual void stopProcessingCommandsQueue ();

                        /**
                         * @fn  virtual void processEurnInteraction (bool result);
                         * @brief process Eurn interaction.
                         * @author  Juan Valera
                         * @date  27/10/2017
                         */
                        virtual void processEurnInteraction (bool result);

                        /**
                         * @fn  virtual void processSuccesEurnInteraction ();
                         * @brief process succes Eurn interaction.
                         * @author  Juan Valera
                         * @date  27/10/2017
                         */
                        virtual void processSuccesEurnInteraction ();

                        /**
                         * @fn  virtual void processFailedEurnInteraction ();
                         * @brief process failed Eurn interaction.
                         * @author  Juan Valera
                         * @date  27/10/2017
                         */
                        virtual void processFailedEurnInteraction ();

                        /**
                         * @fn SignalEurnStatusChanged EurnController::getSignalEurnSatusChanged();
                         * @brief Gets the signal that occurs when status of Eurn changes.
                         * @author Juan Valera
                         * @date 30/10/2017
                         * @return signal.
                         */
                        SignalEurnStatusChanged getSignalEurnSatusChanged();

                        /**
                         * @fn  virtual void evaluateAndNotifyEurnStatus ();
                         * @brief notify Eurn status if the deviceStatus is different of lastDeviceStatus
                         * @author  Juan Valera
                         * @date  30/10/2017
                         */
                        virtual void evaluateAndNotifyEurnStatus ();

                        /**
                         * @fn  virtual void notifyEurnStatus ();
                         * @brief notify Eurn status
                         * @author  Juan Valera
                         * @date  30/10/2017
                         */
                        virtual void notifyEurnStatus();

                        /**
                         * @fn  virtual void runDiagnosticSequence ();
                         * @brief run the diagnostic Sequence
                         * @author  Juan Valera
                         * @date  02/11/2017
                         */
                        virtual bool runDiagnosticSequence();

                        /**
                         * @fn  virtual void runDiagnoseOpenDoor ();
                         * @brief run the diagnose open door
                         * @author  Juan Valera
                         * @date  08/11/2017
                         */
                        virtual bool runDiagnoseOpenDoor();

                        /**
                         * @fn  virtual void runDiagnoseCloseDoor ();
                         * @brief run the diagnose close door
                         * @author  Juan Valera
                         * @date  08/11/2017
                         */
                        virtual bool runDiagnoseCloseDoor();

                        /**
                         * @fn  virtual void runDiagnoseTurnOnGreenLed ();
                         * @brief run the diagnose turn on gren led
                         * @author  Juan Valera
                         * @date  08/11/2017
                         */
                        virtual bool runDiagnoseTurnOnGreenLed();

                        /**
                         * @fn  virtual void runDiagnoseTurnOnRedLed ();
                         * @brief run the diagnose turn on red led
                         * @author  Juan Valera
                         * @date  08/11/2017
                         */
                        virtual bool runDiagnoseTurnOnRedLed();

                        /**
                         * @fn  virtual void runDiagnoseTurnOffGreenLed ();
                         * @brief run the diagnose turn off green led
                         * @author  Juan Valera
                         * @date  08/11/2017
                         */
                        virtual bool runDiagnoseTurnOffGreenLed();

                        /**
                         * @fn  virtual void runDiagnoseTurnOffRedLed ();
                         * @brief run the diagnose turn off red led
                         * @author  Juan Valera
                         * @date  08/11/2017
                         */
                        virtual bool runDiagnoseTurnOffRedLed();

                        /**
                         * @fn  virtual void turnRedLedOnIndefinitely ();
                         * @brief turns red led on indefinitely
                         * @author  eduardo.fernandez
                         * @date  22/01/2018
                         */
                        virtual void turnRedLedOnIndefinitely();

                        /**
                         * @fn  virtual std::string getVersion ();
                         * @brief gets the version of the eURN add-on
                         * @author  Juan Valera
                         * @date  08/11/2017
                         */
                        virtual std::string getVersion();

                    protected:

                        /**
                         * @fn EurnController::EurnController();
                         * @brief Default constructor for the EurnController class.
                         * @author Juan Valera
                         * @date 25/10/2017
                         */
                        EurnController();

                    private:

                        Glib::Mutex mutex;  ///< The mutex
                        Smartmatic::SAES::HW::Devices::EurnInterface * eurnDevice; ///< Eurn device interface
                        bool stopProcessingQueue;
                        bool abortConnection; ///<flag to abort connections
                        DeviceStatus deviceStatus;
                        DeviceStatus lastDeviceStatus;
                        SignalEurnStatusChanged signalEurnStatusChanged; ///< signal Eurn

                        sigc::connection signalEurnInteraction;
                        Glib::ThreadPool m_pool;
                        static EurnController * instance; ///< Instance
                        static Smartmatic::Log::ISMTTLog * logger;  ///< The logger
                };
            }
        }
    }
}




#endif /* EURNCONTROLLER_HXX_ */
