/**
 * @file BCRPolling.hxx
 * @brief Declaration of the BCRPolling class.
 * @date 13/07/2015
 * @author Yadickson Soto
 */

#ifndef BCR_POLLING_HXX_
#define BCR_POLLING_HXX_

#include <System/SafeBuffer.h>
#include <System/SafeThread.hxx>
#include <System/SafeDispatcher.hxx>

namespace Smartmatic
{
    namespace Log
    {
        class ISMTTLog;
    }

    namespace SAES
    {
        namespace Operation
        {
            namespace Devices
            {
                /**
                 * @class BCRPolling
                 * @brief Class alarm controller.
                 * @author Yadickson Soto &lt;yadickson.soto@smartmatic.com&gt;
                 * @date 13/07/2015
                 */
                class BCRPolling : public Smartmatic::System::SafeThread
                {
                    protected:

                        /**
                         * @fn BCRPolling::BCRPolling();
                         * @brief Default constructor for the BCRPolling class.
                         * @author Yadickson Soto &lt;yadickson.soto@smartmatic.com&gt;
                         * @date 13/07/2015
                         */
                        BCRPolling();

                        /**
                         * @fn	    virtual void preStart ();
                         * @brief	Callback from thread before start thread.
                         * @author	Yadickson Soto
                         * @date	13/07/2015
                         */
                        virtual void preStart ();

                        /**
                         * @fn	virtual bool process ();
                         * @brief	Callback from thread any time.
                         * @author	Yadickson Soto
                         * @date	13/07/2015
                         * @return true to continue process
                         */
                        virtual bool process ();

                        /**
                         * @fn	    virtual void postStop ();
                         * @brief	Callback from thread after stop thread.
                         * @author	Yadickson Soto
                         * @date	13/07/2015
                         */
                        virtual void postStop ();

                        /**
                         * @fn	    void onRawObject ( void * object );
                         * @brief	Callback when emit sync signal.
                         * @author	Yadickson Soto
                         * @date	23/07/2015
                         * @param 	object The object to process
                         */
                        void onRawObject ( void * object );

                    public:

                        typedef sigc::signal<void, Smartmatic::System::SafeBuffer *> BufferSignal; ///< The buffer events' signal

                        /**
                         * @fn virtual BCRPolling::~BCRPolling();
                         * @brief Finalizes an instance of the BCRPolling class.
                         * @author Yadickson Soto &lt;yadickson.soto@smartmatic.com&gt;
                         * @date 13/07/2015
                         */
                        virtual ~BCRPolling();

                        /**
                         * @fn void pause ();
                         * @brief Pause read device.
                         * @author Yadickson Soto
                         * @date 13/07/2015
                         */
                        void pause ();

                        /**
                         * @fn void resume ();
                         * @brief Resume read device.
                         * @author Yadickson Soto
                         * @date 13/07/2015
                         */
                        void resume ();

                        /**
                         * @fn BufferSignal getBufferSignal();
                         * @brief Gets the signal that corresponds to the buffer signal
                         * @date 23/07/2015
                         * @author Yadickson Soto
                         * @return The signal for the buffer signal.
                         */
                        BufferSignal getBufferSignal();

                        /**
                         * @fn bool getIsPaused();
                         * @brief Gets the value of flagPause
                         * @date 27/01/2017
                         * @author Juan Valera
                         * @return true if is paused.
                         */
                         bool getIsPaused();

                        /**
                         * @fn static BCRPolling * BCRPolling::getInstance();
                         * @brief Gets the current instance of the BCRPolling.
                         * @author Yadickson Soto
                         * @date 13/07/2015
                         * @return pointer to the current instance of the BCRPolling class.
                         */
                        static BCRPolling * getInstance();

                    private:

                        Smartmatic::System::SafeDispatcher safeSync; ///< The sync signal
                        BufferSignal bufferSignal; ///< The buffer signal

                        bool flagPause; ///< The flag pause

                        std::string inCommand; ///< The in command
                        std::string outCommand; ///< The out command

                        bool sendInCommand; ///< The flag to send the in command
                        bool sendOutCommand; ///< The flag to send the out command

                        static BCRPolling * instance; ///< instance
                        static Smartmatic::Log::ISMTTLog * logger;  ///< The logger
                };
            }
        }
    }
}

#endif /* BCR_POLLING_HXX_ */
