/**
 * @file FlashDiagnosis.cxx
 * @brief Implementation of the FlashDiagnosis class.
 * @date 20/10/2011
 * @author Yadickson Soto
 */

#include "FlashDiagnosis.hxx"
#include <Runtime/Environment/MemoryPathController.h>
#include <System/IO/Directory.h>
#include <System/IO/Path.h>
#include <System/IO/File.h>
#include <Log/SMTTLogManager.h>
#include <System/Runtime/StorageDevice.hxx>
#include <Configuration/s-a-e-s-functionality-conf-manager.h>
#include <Operation/MachineOperationManager.hxx>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

#define N_(String) (String) ///<macro for gettext not translate

using namespace Smartmatic::Log;
using namespace Smartmatic::SAES::Functionality::Configuration;
using namespace Smartmatic::SAES::Operation;
using namespace Smartmatic::SAES::Operation::Diagnosis;
using namespace Smartmatic::SAES::Runtime::Environment;
using namespace Smartmatic::SAES::Voting::Status;
using namespace Smartmatic::SAES::Voting;
using namespace Smartmatic::System::IO;
using namespace Smartmatic::System::Runtime;
using namespace Smartmatic::System;

ISMTTLog * FlashDiagnosis::logger = SMTTLogManager::GetLogger ( "Smartmatic.SAES.Operation.Diagnosis.FlashDiagnosis", GETTEXT_PACKAGE );

FlashDiagnosis::FlashDiagnosis ( bool checkBoth )
    : IDiagnosis ( Smartmatic::SAES::Common::Configuration::DeviceType::CONFIGURATION_FLASH ),
      checkBoth ( checkBoth )
{
    this->errorMessage = N_ ( "Smartmatic.SAES.Operation.Diagnosis.FlashDiagnosis.DiagnosticFailed" );
    updateMessage();
}

FlashDiagnosis::~FlashDiagnosis()
{
}

void FlashDiagnosis::run()
{
    bool validate = false;

    if ( checkBoth )
    {
        std::string domPath = MemoryPathController::getInstance()->getDomMemoryBasePath();
        std::string flashPath = MemoryPathController::getInstance()->getFlashMemoryBasePath();
        validate = getDeviceStatus ( domPath ) && getDeviceStatus ( flashPath );
    }
    else
    {
        std::string domPath = MemoryPathController::getInstance()->getStartupMemoryBasePath();
        validate = getDeviceStatus ( domPath );
    }

    if (validate && !getFat32Status())
    {
        if (SAESFunctionalityConfigurationManager::getInstance()->getOperationConfiguration().getIgnoreFatErrors())
        {
            logger->Audit(N_("Diagnostic of fat partition failed. Fatal error suppressed by configuration."));
        }
        else
        {
            logger->Audit(N_("Diagnostic of fat partition failed."));
            validate = false;
        }
    }

    setStatus ( validate ? VMStatus::VOTING_MACHINE_STATUS_OK : VMStatus::VOTING_MACHINE_STATUS_ERROR );
}

bool FlashDiagnosis::getDeviceStatus ( std::string device )
{
    bool validate = false;

    if ( Directory::getInstance()->DirectoryExists ( device ) )
    {
        std::string pathComplete = Path::Combine ( device, "test.bin" );
        unsigned char data[30000];
        srand ( time ( NULL ) );

        for ( unsigned int i = 0; i < sizeof ( data ); i++ )
        {
            data[i] = rand() % 256;
        }

        srand ( 1 );
        SafeBuffer bufferWrite ( data, sizeof ( data ) );
        SafeBuffer bufferRead;

        if ( !File::getInstance()->saveBufferToFile ( bufferWrite, pathComplete ) )
        {
            logger->Debug ( "Error save file in " + device );
            errorMessage = N_ ( "Smartmatic.SAES.Operation.Diagnosis.FlashDiagnosis.WriteDeviceError" );
        }
        else if ( !File::getInstance()->openFileToBuffer ( pathComplete, bufferRead ) )
        {
            logger->Debug ( "Error open file in " + device );
            errorMessage = N_ ( "Smartmatic.SAES.Operation.Diagnosis.FlashDiagnosis.ReadDeviceError" );
        }
        else if ( bufferWrite.GetBase64Data() != bufferRead.GetBase64Data() )
        {
            logger->Debug ( "Error compare data file in " + device );
            errorMessage = N_ ( "Smartmatic.SAES.Operation.Diagnosis.FlashDiagnosis.DataWriteDeviceError" );
        }
        else if ( !File::getInstance()->DeleteFile ( pathComplete ) )
        {
            logger->Debug ( "Error delete file in " + device );
            errorMessage = N_ ( "Smartmatic.SAES.Operation.Diagnosis.FlashDiagnosis.DeleteDeviceError" );
        }
        else
        {
            validate = true;
        }
    }
    else
    {
        logger->Debug ( "Error - device " + device + " not found" );
        errorMessage = N_ ( "Smartmatic.SAES.Operation.Diagnosis.FlashDiagnosis.DeviceNotFound" );
    }

    return validate;
}

bool FlashDiagnosis::getFat32Status()
{
    bool                                 valid         = true;
    std::string::size_type               uniqueDevices = (checkBoth ? 2 : 1);
    std::vector<StorageDevice>           devices       = StorageDevice::getUniqueDevicesByFileSystem("vfat");
    std::vector<StorageDevice>::iterator it;

    if (devices.size() == uniqueDevices)
    {
        for (it = devices.begin(); it != devices.end(); ++it)
        {
            if (!checkFat32Device(*it))
            {
                valid = false;
                break;
            }
        }
    }
    else
    {
        logger->Debug("Number of available fat partitions mismatch");
        errorMessage = N_("Smartmatic.SAES.Operation.Diagnosis.FlashDiagnosis.DeviceNotFound");
        valid = false;
    }

    return valid;
}

bool FlashDiagnosis::checkFat32Device(StorageDevice& device)
{
    bool result = false;
    SafeBuffer bufferWrite("test.bin");
    SafeBuffer bufferRead;

    if (!device.isMounted())
    {
        device.mount();
    }

    if (device.isMounted() && !device.getMountPoint().empty())
    {
        std::string pathComplete = Path::Combine(device.getMountPoint(), "test.bin");

        if (!File::getInstance()->saveBufferToFile(bufferWrite, pathComplete))
        {
            logger->Debug("Error save file in " + pathComplete);
            errorMessage = N_("Smartmatic.SAES.Operation.Diagnosis.FlashDiagnosis.WriteDeviceError");
        }
        else if (!File::getInstance()->openFileToBuffer(pathComplete, bufferRead))
        {
            logger->Debug("Error open file in " + pathComplete);
            errorMessage = N_("Smartmatic.SAES.Operation.Diagnosis.FlashDiagnosis.ReadDeviceError");
        }
        else if (bufferWrite != bufferRead)
        {
            logger->Debug("Error compare data file in " + pathComplete);
            errorMessage = N_("Smartmatic.SAES.Operation.Diagnosis.FlashDiagnosis.DataWriteDeviceError");
        }
        else if (!File::getInstance()->DeleteFile(pathComplete))
        {
            logger->Debug("Error delete file in " + pathComplete);
            errorMessage = N_("Smartmatic.SAES.Operation.Diagnosis.FlashDiagnosis.DeleteDeviceError");
        }
        else if (!MachineOperationManager::getInstance()->isCloseElection() &&
                 !MachineOperationManager::getInstance()->isCloseVotingExperience())
        {
            Directory* directory = Directory::getInstance();
            std::string path = device.getMountPoint();
            std::vector<std::string> files;
            std::vector<std::string> directories;

            directory->GetFiles(path, files);
            directory->GetDirectories(path, directories);

            if (files.empty() && directories.size() <= 2)
            {
                result = true;
            }
            else
            {
                logger->Debug("Error fat directory " + path + " is not empty");
                errorMessage = N_("Smartmatic.SAES.Operation.Diagnosis.FlashDiagnosis.NotEmptyDeviceError");
            }
        }
        else
        {
            result = true;
        }
    }

    device.unmount();

    return result;
}

void FlashDiagnosis::updateMessage()
{
    const std::string messageFooter = N_ ( "Smartmatic.SAES.Operation.Diagnosis.FlashDiagnosis.Info" );

    switch ( getStatus() )
    {
        default:
        case VMStatus::VOTING_MACHINE_STATUS_NOTDIAGNOSED:
            setMessageFooter ( messageFooter );
            setMessage ( N_ ( "Smartmatic.SAES.Operation.Diagnosis.FlashDiagnosis.InitializingDiagnostic" ) );
            break;

        case VMStatus::VOTING_MACHINE_STATUS_RUN:
            setMessageFooter ( messageFooter );
            setMessage ( N_ ( "Smartmatic.SAES.Operation.Diagnosis.FlashDiagnosis.CheckingForMediaInserted" ) );
            break;

        case VMStatus::VOTING_MACHINE_STATUS_OK:
            setMessageFooter ( messageFooter );
            setMessage ( N_ ( "Smartmatic.SAES.Operation.Diagnosis.FlashDiagnosis.DiagnosticSuccessful" ) );
            break;

        case VMStatus::VOTING_MACHINE_STATUS_ERROR:
            setMessageFooter ( " " );
            setMessage ( errorMessage );
            break;
    }
}
