/**
 * @file IDiagnosis.cxx
 * @brief Implementation of the IDiagnosis class.
 * @date 20/10/2011
 * @author Yadickson Soto
 */

#include "IDiagnosis.hxx"
#include <Log/SMTTLogManager.h>
#include <Operation/MachineOperationManager.hxx>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

#define N_(String) (String) ///<macro for gettext not translate

using namespace Smartmatic::SAES::Operation::Diagnosis;
using namespace Smartmatic::SAES::Voting;
using namespace Smartmatic::SAES::Operation;
using namespace Smartmatic::Log;
using namespace Smartmatic::SAES::Voting::Status;

ISMTTLog * IDiagnosis::logger = SMTTLogManager::GetLogger ( "Smartmatic.SAES.Operation.Diagnosis.IDiagnosis", GETTEXT_PACKAGE );

IDiagnosis::IDiagnosis ( Smartmatic::SAES::Common::Configuration::DeviceType type )
    : mandatory ( false ), valueTime ( 1 ), message ( " " ), messageFooter ( " " ),
      type ( type ), status ( VMStatus::VOTING_MACHINE_STATUS_NOTDIAGNOSED )
{
    signalDistpacherEnd.getSyncSignal().connect ( sigc::mem_fun ( *this, &IDiagnosis::onRawObject ) );
}

IDiagnosis::~IDiagnosis()
{

}

Smartmatic::SAES::Common::Configuration::DeviceType IDiagnosis::getType()
{
    return type;
}

void IDiagnosis::setMandatory ( bool mandatory )
{
    this->mandatory = mandatory;
}

bool IDiagnosis::getMandatory()
{
    return mandatory;
}

void IDiagnosis::setMessage ( const std::string& message )
{
    this->message = message;
}

std::string IDiagnosis::getMessage()
{
    const std::string translatedMessage = _ ( message.c_str() );

    return translatedMessage;
}

void IDiagnosis::setTime ( int valueTime )
{
    this->valueTime = valueTime;
}

int IDiagnosis::getTime()
{
    return valueTime;
}

void IDiagnosis::setStatus ( VMStatus status )
{
    switch ( type )
    {
        case Smartmatic::SAES::Common::Configuration::DeviceType::CONFIGURATION_ALARMBOX:

            switch ( status )
            {
                case VMStatus::VOTING_MACHINE_STATUS_RUN:
                    logger->Audit ( N_ ( "Smartmatic.SAES.Operation.Diagnosis.IDiagnosis.AlarmBox.Run" ) );
                    break;

                case VMStatus::VOTING_MACHINE_STATUS_OK:
                    logger->Audit ( N_ ( "Smartmatic.SAES.Operation.Diagnosis.IDiagnosis.AlarmBox.Ok" ) );
                    break;

                case VMStatus::VOTING_MACHINE_STATUS_ERROR:
                    logger->Audit ( N_ ( "Smartmatic.SAES.Operation.Diagnosis.IDiagnosis.AlarmBox.Error" ) );
                    break;

                case VMStatus::VOTING_MACHINE_STATUS_NOTDIAGNOSED:
                default:
                    break;
            }

            break;

        case Smartmatic::SAES::Common::Configuration::DeviceType::CONFIGURATION_BARCODE:

            switch ( status )
            {
                case VMStatus::VOTING_MACHINE_STATUS_RUN:
                    logger->Audit ( N_ ( "Smartmatic.SAES.Operation.Diagnosis.IDiagnosis.Barcode.Run" ) );
                    break;

                case VMStatus::VOTING_MACHINE_STATUS_OK:
                    logger->Audit ( N_ ( "Smartmatic.SAES.Operation.Diagnosis.IDiagnosis.Barcode.Ok" ) );
                    break;

                case VMStatus::VOTING_MACHINE_STATUS_ERROR:
                    logger->Audit ( N_ ( "Smartmatic.SAES.Operation.Diagnosis.IDiagnosis.Barcode.Error" ) );
                    break;

                case VMStatus::VOTING_MACHINE_STATUS_NOTDIAGNOSED:
                default:
                    break;
            }

            break;

        case Smartmatic::SAES::Common::Configuration::DeviceType::CONFIGURATION_PRINTER:

            switch ( status )
            {
                case VMStatus::VOTING_MACHINE_STATUS_RUN:
                    logger->Audit ( N_ ( "Smartmatic.SAES.Operation.Diagnosis.IDiagnosis.Printer.Run" ) );
                    break;

                case VMStatus::VOTING_MACHINE_STATUS_OK:
                    logger->Audit ( N_ ( "Smartmatic.SAES.Operation.Diagnosis.IDiagnosis.Printer.Ok" ) );
                    break;

                case VMStatus::VOTING_MACHINE_STATUS_ERROR:
                    logger->Audit ( N_ ( "Smartmatic.SAES.Operation.Diagnosis.IDiagnosis.Printer.Error" ) );
                    break;

                case VMStatus::VOTING_MACHINE_STATUS_NOTDIAGNOSED:
                default:
                    break;
            }

            break;

        case Smartmatic::SAES::Common::Configuration::DeviceType::CONFIGURATION_SMARTCARD:

            switch ( status )
            {
                case VMStatus::VOTING_MACHINE_STATUS_RUN:
                    logger->Audit ( N_ ( "Smartmatic.SAES.Operation.Diagnosis.IDiagnosis.Smartcard.Run" ) );
                    break;

                case VMStatus::VOTING_MACHINE_STATUS_OK:
                    logger->Audit ( N_ ( "Smartmatic.SAES.Operation.Diagnosis.IDiagnosis.Smartcard.Ok" ) );
                    break;

                case VMStatus::VOTING_MACHINE_STATUS_ERROR:
                    logger->Audit ( N_ ( "Smartmatic.SAES.Operation.Diagnosis.IDiagnosis.Smartcard.Error" ) );
                    break;

                case VMStatus::VOTING_MACHINE_STATUS_NOTDIAGNOSED:
                default:
                    break;
            }

            break;

        case Smartmatic::SAES::Common::Configuration::DeviceType::CONFIGURATION_FLASH:

            switch ( status )
            {
                case VMStatus::VOTING_MACHINE_STATUS_RUN:
                    logger->Audit ( N_ ( "Smartmatic.SAES.Operation.Diagnosis.IDiagnosis.Flash.Run" ) );
                    break;

                case VMStatus::VOTING_MACHINE_STATUS_OK:
                    logger->Audit ( N_ ( "Smartmatic.SAES.Operation.Diagnosis.IDiagnosis.Flash.Ok" ) );
                    break;

                case VMStatus::VOTING_MACHINE_STATUS_ERROR:
                    logger->Audit ( N_ ( "Smartmatic.SAES.Operation.Diagnosis.IDiagnosis.Flash.Error" ) );
                    break;

                case VMStatus::VOTING_MACHINE_STATUS_NOTDIAGNOSED:
                default:
                    break;
            }

            break;

        case Smartmatic::SAES::Common::Configuration::DeviceType::CONFIGURATION_TOUCHSCREENCALIBRATION:

            switch ( status )
            {
                case VMStatus::VOTING_MACHINE_STATUS_RUN:
                    logger->Audit ( N_ ( "Smartmatic.SAES.Operation.Diagnosis.IDiagnosis.TouchScreenCalibration.Run" ) );
                    break;

                case VMStatus::VOTING_MACHINE_STATUS_OK:
                    logger->Audit ( N_ ( "Smartmatic.SAES.Operation.Diagnosis.IDiagnosis.TouchScreenCalibration.Ok" ) );
                    break;

                case VMStatus::VOTING_MACHINE_STATUS_ERROR:
                    logger->Audit ( N_ ( "Smartmatic.SAES.Operation.Diagnosis.IDiagnosis.TouchScreenCalibration.Error" ) );
                    break;

                case VMStatus::VOTING_MACHINE_STATUS_NOTDIAGNOSED:
                default:
                    break;
            }

            break;

        case Smartmatic::SAES::Common::Configuration::DeviceType::CONFIGURATION_P168A:
        case Smartmatic::SAES::Common::Configuration::DeviceType::CONFIGURATION_ABLED:

            switch ( status )
            {
                case VMStatus::VOTING_MACHINE_STATUS_RUN:
                    logger->Audit ( N_ ( "Smartmatic.SAES.Operation.Diagnosis.IDiagnosis.Able.Run" ) );
                    break;

                case VMStatus::VOTING_MACHINE_STATUS_OK:
                    logger->Audit ( N_ ( "Smartmatic.SAES.Operation.Diagnosis.IDiagnosis.Able.Ok" ) );
                    break;

                case VMStatus::VOTING_MACHINE_STATUS_ERROR:
                    logger->Audit ( N_ ( "Smartmatic.SAES.Operation.Diagnosis.IDiagnosis.Able.Error" ) );
                    break;

                case VMStatus::VOTING_MACHINE_STATUS_NOTDIAGNOSED:
                default:
                    break;
            }

            break;

        case Smartmatic::SAES::Common::Configuration::DeviceType::CONFIGURATION_EURN:

            switch ( status )
            {
                case VMStatus::VOTING_MACHINE_STATUS_RUN:
                    logger->Audit ( N_ ( "The diagnostic of the eUrn is executing" ) );
                    break;

                case VMStatus::VOTING_MACHINE_STATUS_OK:
                    logger->Audit ( N_ ( "The diagnostic of the eUrn was executed succesfully" ) );
                    break;

                case VMStatus::VOTING_MACHINE_STATUS_ERROR:
                    logger->Audit ( N_ ( "There was an error executing diagnostic of the eUrn" ) );
                    break;

                case VMStatus::VOTING_MACHINE_STATUS_NOTDIAGNOSED:
                default:
                    break;
            }

            break;

        case Smartmatic::SAES::Common::Configuration::DeviceType::CONFIGURATION_ETHERNET:
        case Smartmatic::SAES::Common::Configuration::DeviceType::CONFIGURATION_MOUSE:
        case Smartmatic::SAES::Common::Configuration::DeviceType::CONFIGURATION_KEYBOARD:
        case Smartmatic::SAES::Common::Configuration::DeviceType::CONFIGURATION_USBSTORAGEMOUNTED:
        case Smartmatic::SAES::Common::Configuration::DeviceType::CONFIGURATION_UNKNOWN:
        default:
            break;
    }

    this->status = status;
    updateMessage();
    MachineOperationManager::getInstance()->getOperationStatus()->getVotingMachineStatusWrapper()->setDiagnosisStatus ( getType(), getStatus(), getMandatory() );
}

VMStatus IDiagnosis::getStatus()
{
    return status;
}

sigc::signal<void> IDiagnosis::getSignalEnd()
{
    return signalEnd;
}

void IDiagnosis::setMessageFooter ( const std::string& messageFooter )
{
    this->messageFooter = messageFooter;
}

std::string IDiagnosis::getMessageFooter()
{
    const std::string translatedFooter = _ ( messageFooter.c_str() );

    return translatedFooter;
}

void IDiagnosis::test()
{
    start ( 0 );
}

void IDiagnosis::postStop ()
{
    signalDistpacherEnd.emit ( NULL );
}

bool IDiagnosis::process()
{
    run();
    return false;
}

void IDiagnosis::onRawObject ( void * object )
{
    getSignalEnd().emit();
}
