/**
 * @file FileKeyLoginManager.cxx
 * @brief Body of the file key login manager class.
 * @date 23/05/2013
 * @author Yadickson Soto
 */

#include "FileKeyLoginManager.hxx"
#include <Voting/VotingCenters/VotingCentersWrapper.hxx>
#include <Security/Encryption.hxx>
#include <System/IO/File.h>
#include <Log/SMTTLogManager.h>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

using namespace Smartmatic::SAES::Operation;
using namespace Smartmatic::SAES::Voting::VotingCenters;
using namespace Smartmatic::SAES::Common::Configuration;
using namespace Smartmatic::System::IO;
using namespace Smartmatic::Log;
using namespace Smartmatic::SAES::Common::Security;

ISMTTLog * FileKeyLoginManager::logger = SMTTLogManager::GetLogger ( "Smartmatic.SAES.Operation.FileKeyLoginManager", GETTEXT_PACKAGE );

FileKeyLoginManager::FileKeyLoginManager()
    : dir(NULL)
{
}

FileKeyLoginManager::~FileKeyLoginManager()
{

}

bool FileKeyLoginManager::authentication ( std::string & password , std::string & codeOfVotingCenter, std::string & rnd)
{
    this->password = password;
    this->codeOfVotingCenter = codeOfVotingCenter;
    this->rnd = rnd;

    if ( !isInstall() )
    {
        if ( loadOfficialKey() )
        {
            setMode ( RunningMode::VOTING_CENTER_OFFICIAL );
            return true;
        }
        else if ( loadRecountKey() )
        {
            setMode ( RunningMode::VOTING_CENTER_RECOUNT );
            return true;
        }
    }
    else
    {
        if ( getMode() == RunningMode::VOTING_CENTER_OFFICIAL )
        {
            return loadOfficialKey();
        }
        else if ( getMode() == RunningMode::VOTING_CENTER_RECOUNT )
        {
            return loadRecountKey();
        }
    }

    return false;
}

bool FileKeyLoginManager::isInstall()
{
    if ( VotingCentersWrapper::getInstance()->isInstall() )
    {
        VotingCentersWrapper::getInstance()->getInstallationVotingCenter ( votingCenter );
        return true;
    }

    return false;
}

bool FileKeyLoginManager::loadOfficialKey()
{
    BasePathType type = VotingCentersWrapper::getInstance()->getMasterDevice();
    std::string secretKeyFile = dir->getSecretKeyFileFullPath ( type );
    std::string upKeyFile = dir->getUPKeyFileFullPath ( type );

    logger->Debug ( "Load Official Keys" );

    return loadKey ( secretKeyFile, upKeyFile );
}

bool FileKeyLoginManager::loadRecountKey()
{
    BasePathType type = VotingCentersWrapper::getInstance()->getMasterDevice();
    std::string secretKeyFile = dir->getRecountSecretKeyFileName ( type );
    std::string upKeyFile = dir->getRecountUPKeyFileName ( type );

    logger->Debug ( "Load Recount Keys" );

    return loadKey ( secretKeyFile, upKeyFile );
}

void FileKeyLoginManager::setMode ( const RunningMode & mode )
{
    votingCenter.setRunning_mode ( mode );
}

RunningMode FileKeyLoginManager::getMode ()
{
    return ( votingCenter.getRunning_mode().present() ? votingCenter.getRunning_mode().get() : RunningMode ( RunningMode::VOTING_CENTER_OFFICIAL ) );
}

bool FileKeyLoginManager::loadKey ( std::string keyFile, std::string upKeyFile )
{
    bool validate = false;

    try
    {
        Smartmatic::SAES::Security::Encryption::getInstance()->dispose();

        if ( File::getInstance()->canOpenFile ( keyFile ) )
        {
            logger->Debug ( "Open key file in " + keyFile );
            SecretKey sk ( keyFile, password, codeOfVotingCenter, rnd);
            Smartmatic::SAES::Security::Encryption::getInstance()->initCrypto ( sk, Smartmatic::SAES::Common::Configuration::KeyType::CONFIGURATION_SECRET_KEY );
        }

        if ( File::getInstance()->canOpenFile ( upKeyFile ) )
        {
            logger->Debug ( "Open key file in " + upKeyFile );
            SecretKey sk ( upKeyFile, password, codeOfVotingCenter, rnd);
            Smartmatic::SAES::Security::Encryption::getInstance()->initCrypto ( sk, Smartmatic::SAES::Common::Configuration::KeyType::CONFIGURATION_UP_KEY );
        }

        validate = true;
    }
    catch ( ... )
    {

    }

    return validate;
}
