/**
 * @file IMachineInitiator.hxx
 * @brief Declares the IMachineInitiator interface.
 * @author Youssef.tourkmani
 * @date 01/11/2012
 */

#ifndef _IMACHINE_INITIATOR_H_
#define _IMACHINE_INITIATOR_H_

#include <list>
#include <Operation/Initiator/ICheckCondition.hxx>
#include <Operation/Initiator/InitiatorSignal.hxx>
#include <Voting/ElectoralConfiguration.hxx>
#include "Log/ISMTTLog.h"
#include <System/SafeThread.hxx>
#include <System/SafeDispatcher.hxx>

namespace Smartmatic
{
    namespace SAES
    {
        namespace Operation
        {
            namespace Initiator
            {
                /**
                 * @class IMachineInitiator
                 * @brief Represents a machine initiator.
                 * @author Yadickson Soto &lt;yadickson.soto@smartmatic.com&gt;
                 * @date 02-03-2011
                 */

                class IMachineInitiator : public Smartmatic::System::SafeThread
                {
                    public:

                        /**
                         * @enum RunModeType
                         * @brief Represents the run mode type.
                         */

                        enum RunModeType
                        {
                            FIRST_TIME
                            , NORMAL_MODE
                            , RECOVERY_MODE
                            , REPLACE_MACHINE_MODE
                        };

                        typedef sigc::signal<void, InitiatorSignal * > SignalInitiatorProcess; ///< signal initiator process

                        /**
                         * @fn IMachineInitiator::IMachineInitiator();
                         * @brief Default constructor for the IMachineInitiator class.
                         * @author Youssef.tourkmani
                         * @date 01/11/2012
                         */

                        IMachineInitiator();

                        /**
                         * @fn virtual IMachineInitiator::~IMachineInitiator();
                         * @brief Finalizes an instance of the IMachineInitiator class.
                         * @author Youssef.tourkmani
                         * @date 01/11/2012
                         */
                        virtual ~IMachineInitiator();

                        /**
                         * @fn virtual int IMachineInitiator::getSuccessMessageTime() = 0;
                         * @brief Gets success message time.
                         * @author Youssef.tourkmani
                         * @date 01/11/2012
                         * @return The success message time.
                         */

                        virtual int getSuccessMessageTime() = 0;

                        /**
                         * @fn virtual void IMachineInitiator::dispose() = 0;
                         * @brief Disposes an instance of the IMachineInitiator class.
                         * @author Youssef.tourkmani
                         * @date 01/11/2012
                         */

                        virtual void dispose() = 0;

                        /**
                         * @fn virtual bool IMachineInitiator::checkData() = 0;
                         * @brief Verifies that the data on the internal and external storage devices is correct.
                         * @author Yadickson Soto yadickson.soto@smartmatic.com
                         * @date 25/05/2011
                         * @return true if the data is correct. Otherwise, false.
                         */

                        virtual bool checkData() = 0;

                        /**
                         * @fn virtual Smartmatic::Log::ISMTTLog * getLogger() = 0;
                         * @brief Getter logger class
                         * @author Yadickson Soto yadickson.soto@smartmatic.com
                         * @date 09/09/2013
                         * @return logger
                         */
                        virtual Smartmatic::Log::ISMTTLog * getLogger() = 0;

                        /**
                         * @fn virtual void IMachineInitiator::setMode ( RunModeType mode );
                         * @brief Sets the run mode type.
                         * @author Youssef.tourkmani
                         * @date 01/11/2012
                         * @param mode New run mode type.
                         */

                        virtual void setMode ( RunModeType mode );

                        /**
                         * @fn void IMachineInitiator::addCondition ( ICheckCondition * condition );
                         * @brief Add a condition to check when this initiatior runs.
                         * @author Yadickson Soto
                         * @date 04/10/2012
                         * @param condition new condition to check.
                         */

                        void addCondition ( ICheckCondition * condition );

                        /**
                         * @fn void IMachineInitiator::init ( Smartmatic::SAES::Voting::ElectoralConfiguration * electoralConfiguration );
                         * @brief Initialises this machine initiator.
                         * @author Youssef.tourkmani
                         * @date 01/11/2012
                         * @param electoralConfiguration the electoral configuration.
                         */

                        void init ( Smartmatic::SAES::Voting::ElectoralConfiguration * electoralConfiguration );

                        /**
                         * @fn void addMessage ( const InitializationStatus & status, bool finished = false );
                         * @brief Signals a change in the status of the initialization process.
                         * @author Yadickson Soto
                         * @date 19/06/2013
                         * @param status Initialization status.
                         * @param finished indicates if the initiator running is finished.
                         */

                        void addMessage ( const InitializationStatus & status, bool finished = false );

                        /**
                         * @fn SignalInitiatorProcess getSignalDetected();
                         * @brief Gets the signal that occurs when the status of the initialization process is updated.
                         * @author Yadickson Soto
                         * @date 19/06/2013
                         * @return signal SignalInitiatorProcess signal.
                         */

                        SignalInitiatorProcess getSignalDetected();

                    private:

                        /**
                         * @fn	    void onRawObject ( void * object );
                         * @brief	Callback when emit sync signal. Force the exit of this diagnostic.
                         * @author	Yadickson Soto
                         * @date	03/08/2015
                         * @param 	object The object to process
                         */
                        void onRawObject ( void * object );

                    protected:

                        SignalInitiatorProcess signalDetected; ///< The signal detected
                        RunModeType mode; ///< The mode
                        Smartmatic::SAES::Voting::ElectoralConfiguration * electoralConfiguration; ///< The electoral configuration instance
                        Smartmatic::System::SafeDispatcher signalSendProcessStatus; ///< process status
                        std::list < ICheckCondition * > conditions; ///< list conditions
                };
            }
        }
    }
}

#endif // _IMACHINE_INITIATOR_H_
