/**
 * @file PMachineInitiator.cxx
 * @brief Implementation of the PMachineInitiator class.
 * @author Youssef.tourkmani
 * @date 01/11/2012
 */

#include "PMachineInitiator.hxx"
#include <System/IO/Directory.h>
#include <Operation/Initiator/CheckFlashPresence.hxx>
#include <Operation/MachineOperationManager.hxx>
#include <Printing/PrintDocumentResult.h>
#include <Log/SMTTLogManager.h>
#include <Voting/VotingCenters/VotingCentersWrapper.hxx>
#include <Runtime/Environment/DirectoryManager.hxx>
#include <Operation/Initiator/CheckForUsedMemory.h>
#include <Operation/Initiator/CheckForMemoryComplete.h>
#include <Runtime/Environment/MemoryPathController.h>
#include <Operation/Initiator/CheckFlashStructure.hxx>
#include <Operation/Initiator/CheckFilesPresence.hxx>
#include <Operation/Initiator/CheckDirectoryStructure.hxx>
#include <Operation/Initiator/CheckResourceFiles.hxx>
#include <Operation/Monitor/DeviceTypeMonitorPool.hxx>
#include <Operation/CheckDataManager.hxx>
#include <Configuration/s-a-e-s-functionality-conf-manager.h>
#include <Operation/Voting/VotingEnqueuer.hxx>
#include <System/OptParser.hxx>
#include <System/Exception/VoteException.hxx>
#include <System/Exception/FileSystemException.hxx>
#include <unistd.h>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

#define N_(String) (String) ///<macro for gettext not translate

using namespace Smartmatic::SAES::Operation::Initiator;
using namespace Smartmatic::SAES::Functionality::Configuration;
using namespace Smartmatic::SAES::Runtime::Environment;
using namespace Smartmatic::SAES::Printing;
using namespace Smartmatic::SAES::Operation;
using namespace Smartmatic::System::Exception;
using namespace Smartmatic::System::IO;
using namespace Smartmatic::Log;
using namespace Smartmatic::SAES::Common::Configuration;
using namespace Smartmatic::SAES::Runtime::Environment;
using namespace Smartmatic::SAES::Voting::VotingCenters;
using namespace Smartmatic::SAES::Common::Operation::Monitor;
using namespace Smartmatic::SAES::Common::Configuration;
using namespace Smartmatic::SAES::Operation::Monitor;
using namespace Smartmatic::SAES::Operation::Voting;
using namespace Smartmatic::System;

ISMTTLog * PMachineInitiator::logger = SMTTLogManager::GetLogger ( "Smartmatic.SAES.Operation.Initiator.PMachineInitiator", GETTEXT_PACKAGE );

PMachineInitiator::PMachineInitiator ()
{
    runNormal = false;
    runMode = NORMAL_MODE;
    BasePathType type = VotingCentersWrapper::getInstance()->getMasterDevice();
    dirCondition = new CheckDirectoryStructure ( MachineOperationManager::getInstance()->getSaesDirectories(), type );
    filesCondition = new CheckFilesPresence ( MachineOperationManager::getInstance()->getSaesDirectories(), type );
    addCondition ( dirCondition );
    addCondition ( filesCondition );
    init ( MachineOperationManager::getInstance()->getElectoralConfiguration() );
}

PMachineInitiator::~PMachineInitiator ()
{
    delete dirCondition;
    delete filesCondition;
}

void PMachineInitiator::preStart()
{
    InitializationStatus status;

    status.setMessage ( _ ( "Smartmatic.SAES.Operation.Initiator.PMachineInitiator.DoNotRemoveUSB" ) );
    logger->Audit ( N_ ( "Smartmatic.SAES.Operation.Initiator.StartPMAchine" ) );
}

bool PMachineInitiator::process()
{
    bool success = true;
    InitializationStatus status;

    SaesDirectories directories;
    std::list < ICheckCondition * >::iterator it;

    for ( it = conditions.begin();
            it != conditions.end() && success;
            ++it )
    {
        ICheckCondition * condition = ( *it );
        status.setMessage ( condition->getMessage() );
        addMessage ( status );
        TestStatus::Tests test = condition->check();
        success = success && ( test == TestStatus::Passed );
        status.setStatus ( success ? TestStatus::Passed : TestStatus::Failed );
        addMessage ( status );
    }

    CheckFlashPresence flashCondition;

    if ( success )
    {
        DeviceTypeMonitor * device = DeviceTypeMonitorPool::getInstance()->getDeviceTypeMonitor ( DeviceType::CONFIGURATION_USBSTORAGEMOUNTED );

        logger->Debug ( "Check USB Count Device Mount" );

        if ( device != NULL && device->getNumberOfDevices() < 2 )
        {
            status.setMessage ( flashCondition.getMessage() );
            status.setStatus ( TestStatus::BlockDevice );
            addMessage ( status );

            while ( device->getNumberOfDevices() < 2 )
            {
                sleep ( 1 );
            }

            status.setMessage ( _ ( "Smartmatic.SAES.Operation.Initiator.PMachineInitiator.RunningCheckConfiguration" ) );
            status.setStatus ( TestStatus::Passed );
            addMessage ( status );
        }
    }

    std::string flashMedia = MemoryPathController::getInstance()->getFlashMemoryBasePath();

    if ( success && !Directory::getInstance()->DirectoryExists ( flashMedia ) )
    {
        status.setMessage ( flashCondition.getMessage() );
        status.setStatus ( TestStatus::BlockDevice );

        addMessage ( status );
        TestStatus::Tests test = flashCondition.check();
        success = success && ( test == TestStatus::Passed );
        status.setStatus ( success ? TestStatus::Passed : TestStatus::Failed );
        addMessage ( status );
    }

    if ( success )
    {
        runNormal = true;
        success = checkData() && checkRunFiles();
        runNormal = false;
    }

    status.setStatus ( success ? TestStatus::Passed : TestStatus::Failed );

    if ( success )
    {
        switch ( runMode )
        {
            default:
            case FIRST_TIME:
                status.setMessage ( _ ( "Smartmatic.SAES.Operation.Initiator.PMachineInitiator.RunningFirstTimeSuccessEnd" ) );
                break;

            case NORMAL_MODE:
                status.setMessage ( _ ( "Smartmatic.SAES.Operation.Initiator.PMachineInitiator.CheckingConfigurationSuccess" ) );
                break;

            case RECOVERY_MODE:
                status.setMessage ( _ ( "Smartmatic.SAES.Operation.Initiator.PMachineInitiator.RunningMemoryRecoverySuccess" ) );
                break;

            case REPLACE_MACHINE_MODE:
                status.setMessage ( _ ( "Smartmatic.SAES.Operation.Initiator.PMachineInitiator.RunningReplaceMachineModeSuccess" ) );
                break;
        }

        CheckDataManager::getInstance()->startDetection();
    }
    else
    {
        switch ( runMode )
        {
            default:
            case FIRST_TIME:
                status.setMessage ( _ ( "Smartmatic.SAES.Operation.Initiator.PMachineInitiator.RunningFirstTimeError" ) );
                break;

            case NORMAL_MODE:
                status.setMessage ( _ ( "Smartmatic.SAES.Operation.Initiator.PMachineInitiator.CheckingConfigurationError" ) );
                break;

            case RECOVERY_MODE:
                status.setMessage ( _ ( "Smartmatic.SAES.Operation.Initiator.PMachineInitiator.RunningMemoryRecoveryError" ) );
                break;

            case REPLACE_MACHINE_MODE:
                status.setMessage ( _ ( "Smartmatic.SAES.Operation.Initiator.PMachineInitiator.RunningReplaceMachineModeError" ) );
                break;
        }

        status.setFooterMessage( _ ( "Smartmatic.SAES.Operation.Initiator.PMachineInitiator.ContactHelpdesk" ) );
    }

    if ( status.getStatus() == TestStatus::Passed )
    {
        VotingEnqueuer::getInstance()->enableSaveVote ( !OptParser::getInstance()->isSet ( OptParser::DISABLE_SAVE_VOTE ) );
        VotingEnqueuer::getInstance()->validateID ( !OptParser::getInstance()->isSet ( OptParser::DISABLE_BID_SID_VALIDATE ) );
    }

    addMessage ( status, true );

    return false;
}

bool PMachineInitiator::checkRunFiles()
{
    logger->Debug ( "Start check run files" );
    bool success = true;
    InitializationStatus status;

    try
    {
        SaesDirectories directories;
        CheckFlashStructure checkDom ( directories, Smartmatic::SAES::Common::Configuration::BasePathType::CONFIGURATION_DOM );
        CheckFlashStructure checkFlash ( directories, Smartmatic::SAES::Common::Configuration::BasePathType::CONFIGURATION_FLASH );

        if ( runNormal )
        {
            status.setMessage ( checkFlash.getMessage() );
            addMessage ( status );
        }

        TestStatus::Tests testD = checkDom.check();
        TestStatus::Tests testF = checkFlash.check();

        success = ( testD == TestStatus::Passed && testF == TestStatus::Passed );

        if ( runNormal )
        {
            status.setStatus ( success ? TestStatus::Passed : TestStatus::Failed );
            addMessage ( status );
        }

        if ( !success )
        {
            status.setMessage ( _ ( "Smartmatic.SAES.Operation.Initiator.PMachineInitiator.NotOperatingSystemInFlash" ) );
            return false;
        }

        success = MachineOperationManager::getInstance()->validateSAESSignature();
        success = success && MachineOperationManager::getInstance()->validateEmlSignature();
        success = success && MachineOperationManager::getInstance()->validateSaesLocalSignature();
    }
    catch ( ... )
    {
        success = false;
    }

    return success;
}

bool PMachineInitiator::checkData()
{
    bool success = true;

    logger->Debug ( "Start check data" );

    InitializationStatus status;

    try
    {
        SaesDirectories directories;

        if ( !MachineOperationManager::getInstance()->checkVotingCentersFiles() )
        {
            logger->Debug ( "Invalid voting centers files." );
            return false;
        }

        CheckForUsedMemoryCondition usedDomCond ( &directories );
        CheckForMemoryCompleteCondition completeDomCond ( &directories );
        CheckForUsedMemoryCondition usedFlashCond ( &directories, BasePathType::CONFIGURATION_FLASH );
        CheckForMemoryCompleteCondition completeFlashCond ( &directories, BasePathType::CONFIGURATION_FLASH );
        TestStatus::Tests usedDOM = usedDomCond.check();
        TestStatus::Tests completeDOM = completeDomCond.check();
        TestStatus::Tests usedFlash = usedFlashCond.check();
        TestStatus::Tests completeFlash = completeFlashCond.check();

        if ( runNormal )
        {
            status.setStatus ( success ? TestStatus::Passed : TestStatus::Failed );
            addMessage ( status );
        }

        if ( runNormal && usedDOM == TestStatus::Failed && usedFlash == TestStatus::Failed )
        {
            logger->Debug ( "Run" );
            runMode = mode;
            success = runFirstTime() && checkBasicConfig();
        }
        else if ( completeDOM == TestStatus::Passed && usedFlash == TestStatus::Failed )
        {
            logger->Debug ( "Run recovery mode" );
            // run with DOM
            runMode = RECOVERY_MODE;
            MachineOperationManager::getInstance()->refreshOperationStatus();
            success = MachineOperationManager::getInstance()->isClosingElection() ? false : runRecoveryMode ( true );
        }
        else if ( usedDOM == TestStatus::Failed && completeFlash == TestStatus::Passed )
        {
            logger->Debug ( "Run recovery mode" );
            // run with Flash
            runMode = RECOVERY_MODE;
            MachineOperationManager::getInstance()->refreshOperationStatus();
            success = MachineOperationManager::getInstance()->isClosingElection() ? false : runRecoveryMode ( false );
        }
        else if ( completeDOM == TestStatus::Passed && completeFlash == TestStatus::Passed )
        {
            success = checkBasicConfig() && checkIssuerCertificates();

            if ( success )
            {
                if ( MachineOperationManager::getInstance()->validateMachineId() )
                {
                    logger->Debug ( "Run normal mode" );
                    runMode = NORMAL_MODE;
                    success = runNormalMode();
                }
                else if ( runNormal )
                {
                    logger->Debug ( "Run replace machine mode" );
                    runMode = REPLACE_MACHINE_MODE;
                    success = runReplaceMachineMode();
                }
                else
                {
                    logger->Debug ( "Run check mode" );
                    success = true;
                }
            }
        }
        else
        {
            success = false;

            if ( completeDOM == TestStatus::Failed )
            {
                logger->Debug ( "Error: DOM is not complete" );
            }

            if ( completeFlash == TestStatus::Failed )
            {
                logger->Debug ( "Error: Flash is not complete" );
            }

            if ( usedDOM == TestStatus::Failed && usedFlash == TestStatus::Passed )
            {
                logger->Debug ( "Error: DOM not used but flash used" );
            }

            status.setMessage ( _ ( "Smartmatic.SAES.Operation.Initiator.PMachineInitiator.InvalidDomAndFlashData" ) );
        }
    }
    catch ( ... )
    {
        logger->Debug ( "Error check data" );
        success = false;
    }

    if ( runNormal )
    {
        status.setStatus ( success ? TestStatus::Passed : TestStatus::Failed );
        addMessage ( status );
    }

    logger->Debug ( "End check data" );

    return success;
}

bool PMachineInitiator::runFirstTime()
{
    bool success = false;
    logger->Audit ( N_ ( "Smartmatic.SAES.Operation.Initiator.PMachineInitiator.RunningFirstTimeAudit" ) );
    InitializationStatus status;

    if ( runNormal )
    {
        status.setMessage ( _ ( "Smartmatic.SAES.Operation.Initiator.PMachineInitiator.RunningFirstTime" ) );
        addMessage ( status );
    }

    success = CheckConfigurationLoad();

    if ( success )
    {
        MachineOperationManager::getInstance()->createMachineId();
        SaesDirectories directories;
        Smartmatic::SAES::Runtime::Environment::DirectoryManager directorymanager ( directories );
        success = directorymanager.backup ( true );
    }

    if ( success )
    {
        logger->Debug ( "Running first time OK" );
        status.setMessage ( _ ( "Smartmatic.SAES.Operation.Initiator.PMachineInitiator.RunningFirstTimeSuccess" ) );
    }
    else
    {
        logger->Debug ( "Running first time Failed" );
        status.setMessage ( _ ( "Smartmatic.SAES.Operation.Initiator.PMachineInitiator.RunningFirstTimeError" ) );
        status.setFooterMessage( _ ( "Smartmatic.SAES.Operation.Initiator.PMachineInitiator.ContactHelpdesk" ) );
    }

    if ( runNormal )
    {
        addMessage ( status );
    }

    return success;
}

bool PMachineInitiator::runRecoveryMode ( bool copyDomToFlash )
{
    bool success = false;
    logger->Audit ( N_ ( "Smartmatic.SAES.Operation.Initiator.PMachineInitiator.RunningMemoryRecovery" ) );
    InitializationStatus status;

    if ( runNormal )
    {
        status.setMessage ( _ ( "Smartmatic.SAES.Operation.Initiator.PMachineInitiator.RunningMemoryRecovery" ) );
        addMessage ( status );
        status.setMessage ( _ ( "Smartmatic.SAES.Operation.Initiator.PMachineInitiator.CheckVotes" ) );
        addMessage ( status );
    }

    SaesDirectories directories;
    Smartmatic::SAES::Runtime::Environment::DirectoryManager directorymanager ( directories );

    if ( !directorymanager.backup ( copyDomToFlash ) )
    {
        logger->Debug ( "Error backup memory." );
    }
    else if ( !CheckConfigurationLoad() )
    {
        logger->Debug ( "Error check configuration load." );
    }
    else if ( !checkBasicConfig() )
    {
        logger->Debug ( "Error check basic configuration." );
    }
    else if ( !MachineOperationManager::getInstance()->checkVoteFiles() )
    {
        logger->Debug ( "Invalid votes file in DOM and Flash." );
    }
    else if ( runNormal && !MachineOperationManager::getInstance()->loadVoteFiles() )
    {
        logger->Debug ( "Invalid load votes file in DOM and Flash." );
    }
    else if ( !MachineOperationManager::getInstance()->checkVotes () )
    {
        logger->Debug ( "Invalid votes." );
    }
    else
    {
        success = true;
    }

    if ( success )
    {
        logger->Debug ( "Running recovery mode OK" );
        status.setMessage ( _ ( "Smartmatic.SAES.Operation.Initiator.PMachineInitiator.MemoryRecoverySuccess" ) );
    }
    else
    {
        logger->Debug ( "Running recovery mode Failed" );
        status.setMessage ( _ ( "Smartmatic.SAES.Operation.Initiator.PMachineInitiator.MemoryRecoveryError" ) );
    }

    if ( runNormal )
    {
        addMessage ( status );
    }

    return success;
}

bool PMachineInitiator::runNormalMode()
{
    bool success = false;
    logger->Audit ( N_ ( "Smartmatic.SAES.Operation.Initiator.PMachineInitiator.RunningNormalMode" ) );
    InitializationStatus status;

    if ( runNormal )
    {
        status.setMessage ( _ ( "Smartmatic.SAES.Operation.Initiator.PMachineInitiator.RunningNormalMode" ) );
        addMessage ( status );
    }

    if ( runNormal )
    {
        status.setMessage ( _ ( "Smartmatic.SAES.Operation.Initiator.PMachineInitiator.CheckVotes" ) );
        addMessage ( status );
    }

    if ( !MachineOperationManager::getInstance()->checkVoteFiles() )
    {
        logger->Debug ( "Invalid votes file in DOM and Flash." );
    }
    else if ( runNormal && !MachineOperationManager::getInstance()->loadVoteFiles() )
    {
        logger->Debug ( "Invalid load votes file in DOM and Flash." );
    }
    else if ( !MachineOperationManager::getInstance()->checkVotes () )
    {
        logger->Debug ( "Invalid votes." );
    }
    else if ( !MachineOperationManager::getInstance()->checkSAESLocal() )
    {
        logger->Debug ( "Invalid SaesLocal." );
    }
    else if ( !MachineOperationManager::getInstance()->checkVotingResultFiles() )
    {
        logger->Debug ( "Invalid voting results file." );
    }
    else if ( !MachineOperationManager::getInstance()->checkFatFiles() )
    {
        logger->Debug ( "Invalid files in FAT partition." );
    }
    else if ( !MachineOperationManager::getInstance()->checkNationalitiesFile() )
    {
        logger->Debug ( "Invalid nationalities files." );
    }
    else
    {
        success = true;
    }

    if ( success )
    {
        logger->Debug ( "Running normal mode OK" );
        status.setMessage ( _ ( "Smartmatic.SAES.Operation.Initiator.PMachineInitiator.RunningNormalModeSuccess" ) );
    }
    else
    {
        logger->Debug ( "Running normal mode Failed" );
        status.setMessage ( _ ( "Smartmatic.SAES.Operation.Initiator.PMachineInitiator.RunningNormalModeError" ) );
    }

    if ( runNormal )
    {
        addMessage ( status );
    }

    return success;
}

bool PMachineInitiator::runReplaceMachineMode()
{
    logger->Audit ( N_ ( "Smartmatic.SAES.Operation.Initiator.PMachineInitiator.RunningReplaceMachineMode" ) );
    InitializationStatus status;

    if ( runNormal )
    {
        status.setStatus ( TestStatus::Passed );
        status.setMessage ( _ ( "Smartmatic.SAES.Operation.Initiator.PMachineInitiator.RunningReplaceMachineMode" ) );
        addMessage ( status );
    }

    // no se permiten que exista diferencia de votos
    bool success = false;

    if ( runNormal )
    {
        status.setMessage ( _ ( "Smartmatic.SAES.Operation.Initiator.PMachineInitiator.CheckVotes" ) );
        addMessage ( status );
    }

    if ( !MachineOperationManager::getInstance()->checkVoteFiles() )
    {
        logger->Debug ( "Invalid votes file in DOM and Flash." );
    }
    else if ( runNormal && !MachineOperationManager::getInstance()->loadVoteFiles() )
    {
        logger->Debug ( "Invalid load votes file in DOM and Flash." );
    }
    else if ( !MachineOperationManager::getInstance()->checkVotes () )
    {
        logger->Debug ( "Invalid votes." );
    }
    else if ( !MachineOperationManager::getInstance()->checkSAESLocal() )
    {
        logger->Debug ( "Invalid SaesLocal." );
    }
    else if ( !MachineOperationManager::getInstance()->checkVotingResultFiles() )
    {
        logger->Debug ( "Invalid voting results file." );
    }
    else if ( !MachineOperationManager::getInstance()->checkFatFiles() )
    {
        logger->Debug ( "Invalid files in FAT partition." );
    }
    else
    {
        success = true;
    }

    if ( success )
    {
        // se crea el nuevo machine id en dom y flash
        MachineOperationManager::getInstance()->createMachineId ( true );
        logger->Debug ( "Running replace machine mode OK" );
        status.setMessage ( _ ( "Smartmatic.SAES.Operation.Initiator.PMachineInitiator.RunningReplaceMachineModeSuccess" ) );
    }
    else
    {
        logger->Debug ( "Running replace machine mode Failed" );
        status.setMessage ( _ ( "Smartmatic.SAES.Operation.Initiator.PMachineInitiator.RunningReplaceMachineModeError" ) );
    }

    if ( runNormal )
    {
        status.setStatus ( success ? TestStatus::Passed : TestStatus::Failed );
        addMessage ( status );
    }

    return success;
}

bool PMachineInitiator::checkResources ( )
{
    const std::string message = _ ( "Checking resource files, do not remove memory." );

    logger->Debug ( "Start check resources files" );

    bool success = true;
    SaesDirectories directories;
    InitializationStatus status;

    CheckResourceFiles checkResourcesDom ( &directories, Smartmatic::SAES::Common::Configuration::BasePathType::CONFIGURATION_DOM, message );
    CheckResourceFiles checkResourcesFlash ( &directories, Smartmatic::SAES::Common::Configuration::BasePathType::CONFIGURATION_FLASH, message );

    if ( runNormal )
    {
        status.setMessage ( checkResourcesDom.getMessage() );
        addMessage ( status );
    }

    TestStatus::Tests testResourcesD = checkResourcesDom.check();
    TestStatus::Tests testResourcesF = checkResourcesFlash.check();

    success = ( testResourcesD == TestStatus::Passed && testResourcesF == TestStatus::Passed );

    if ( runNormal )
    {
        status.setStatus ( success ? TestStatus::Passed : TestStatus::Failed );
        addMessage ( status );
    }

    if ( !success )
    {
        status.setMessage ( _ ( "Smartmatic.SAES.Operation.Initiator.PMachineInitiator.ErrorResourceFiles" ) );
        return false;
    }

    if ( runNormal )
    {
        status.setStatus ( success ? TestStatus::Passed : TestStatus::Failed );
        addMessage ( status );
    }

    logger->Debug ( "End check resources files : " + std::string ( success ? "Success" : "Fail" ) );

    return success;
}

bool PMachineInitiator::checkIssuerCertificates()
{
    bool success = MachineOperationManager::getInstance()->validateIssuerCertificates();

    success ?
        logger->Debug ( "Running check issuer certificates ok." ) :
        logger->Debug ( "Error validate issuer certificates." );

    return success;
}

bool PMachineInitiator::checkBasicConfig()
{
    logger->Audit ( N_ ( "Smartmatic.SAES.Operation.Initiator.PMachineInitiator.RunningCheckConfigurationAudit" ) );
    InitializationStatus status;

    if ( runNormal )
    {
        status.setMessage ( _ ( "Smartmatic.SAES.Operation.Initiator.PMachineInitiator.RunningCheckConfiguration" ) );
        addMessage ( status );
    }

    bool success = false;

    if ( !MachineOperationManager::getInstance()->checkVoteStatistics() )
    {
        logger->Debug ( "Invalid vote statistics file." );
    }
    else if ( !MachineOperationManager::getInstance()->checkActivatedCardStatistics() )
    {
        logger->Debug ( "Invalid activated card statistics file." );
    }
    else if ( !MachineOperationManager::getInstance()->checkSignature() )
    {
        logger->Debug ( "Invalid signature file." );
    }
    else if ( !MachineOperationManager::getInstance()->checkElectionStatus() )
    {
        logger->Debug ( "Invalid election status file." );
    }
    else if ( !MachineOperationManager::getInstance()->checkElection() )
    {
        logger->Debug ( "Invalid election file." );
    }
    else if ( !MachineOperationManager::getInstance()->checkDeviceParameters() )
    {
        logger->Debug ( "Invalid device parameters file." );
    }
    else if ( !MachineOperationManager::getInstance()->checkVotingDevices() )
    {
        logger->Debug ( "Invalid voting devices file." );
    }
    else if ( !MachineOperationManager::getInstance()->checkMachineId() )
    {
        logger->Debug ( "Invalid machine id file." );
    }
    else if ( !CheckConfigurationLoad() )
    {
        logger->Debug ( "Error load config" );
    }
    else if ( !MachineOperationManager::getInstance()->checkPVReport() )
    {
        logger->Debug ( "Invalid pv report file." );
    }
    else if ( !MachineOperationManager::getInstance()->checkTransmissionPackages() )
    {
        logger->Debug ( "Invalid transmission package file." );
    }
    else if ( !MachineOperationManager::getInstance()->validateEmlSignature() )
    {
        logger->Debug ( "Error validate EML files." );
    }
    else if ( !MachineOperationManager::getInstance()->validateSaesLocalSignature() )
    {
        logger->Debug ( "Error validate SAES local files." );
    }

    else if ( !checkResources() )
    {
        logger->Debug ( "Error validate resources files." );
    }
    else
    {
        success = true;
    }

    if ( success )
    {
        logger->Debug ( "Running check configuration OK" );
        status.setMessage ( _ ( "Smartmatic.SAES.Operation.Initiator.PMachineInitiator.RunningCheckConfigurationSuccess" ) );
    }
    else
    {
        logger->Debug ( "Running check configuration Failed" );
        status.setMessage ( _ ( "Smartmatic.SAES.Operation.Initiator.PMachineInitiator.RunningCheckConfigurationError" ) );
    }

    if ( runNormal )
    {
        addMessage ( status );
    }

    return success;
}

bool PMachineInitiator::CheckConfigurationLoad()
{
    try
    {
        MachineOperationManager::getInstance()->startUp ( runMode == FIRST_TIME );
        return true;
    }
    catch ( ElectionException & ex )
    {
        std::stringstream ss;
        ss << "FATAL " << __func__ << ex.what();
        logger->Debug ( ss.str() );
    }
    catch ( VotingDeviceException & ex )
    {
        std::stringstream ss;
        ss << "FATAL " << __func__ << ex.what();
        logger->Debug ( ss.str() );
    }
    catch ( CryptoException & ex )
    {
        std::stringstream ss;
        ss << "FATAL " << __func__ << ex.what();
        logger->Debug ( ss.str() );
    }
    catch ( SignedException & ex )
    {
        std::stringstream ss;
        ss << "FATAL " << __func__ << ex.what();
        logger->Debug ( ss.str() );
    }
    catch ( XmlException & ex )
    {
        std::stringstream ss;
        ss << "FATAL " << __func__ << ex.what();
        logger->Debug ( ss.str() );
    }
    catch ( GuIdException & ex )
    {
        std::stringstream ss;
        ss << "FATAL " << __func__ << ex.what();
        logger->Debug ( ss.str() );
    }
    catch ( VoteException & ex )
    {
        std::stringstream ss;
        ss << "FATAL " << __func__ << ex.what();
        logger->Debug ( ss.str() );
    }
    catch ( FileSystemException & ex )
    {
        std::stringstream ss;
        ss << "FATAL " << __func__ << ex.what();
        logger->Debug ( ss.str() );
    }
    catch ( std::exception & ex )
    {
        std::stringstream ss;
        ss << "FATAL " << __func__ << ex.what();
        logger->Debug ( ss.str() );
    }
    catch ( ... )
    {
        std::stringstream ss;
        ss << "FATAL " << __func__;
        logger->Debug ( ss.str() );
    }

    return false;
}

void PMachineInitiator::dispose()
{
}

int PMachineInitiator::getSuccessMessageTime()
{
    return Smartmatic::SAES::Functionality::Configuration::SAESFunctionalityConfigurationManager::getInstance()->getSuccesMessageTime();
}

Smartmatic::Log::ISMTTLog * PMachineInitiator::getLogger()
{
    return logger;
}
