/**
 * @file InstallMachine.cxx
 * @brief Body class install machine
 * @date 18/10/2011
 * @author Yadickson Soto
 */

#include "InstallMachine.hxx"

#include <Configuration/s-a-e-s-common-conf-manager.h>
#include <Log/SMTTLogManager.h>
#include <Operation/MachineOperationManager.hxx>
#include <Runtime/Environment/EnvironmentBuilder.hxx>
#include <System/Utils/StringUtils.hxx>
#include <Voting/VotingCenters/VotingCentersWrapper.hxx>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

#define N_(String) (String) ///<macro for gettext not translate

using namespace Smartmatic::SAES::Operation;
using namespace Smartmatic::Log;
using namespace Smartmatic::SAES::Runtime::Environment;
using namespace Smartmatic::SAES::Voting::VotingCenters;
using namespace Smartmatic::SAES::Common::Configuration;
using namespace Smartmatic::SAES::Voting;
using namespace Smartmatic::SAES::Runtime::Environment;
using namespace Smartmatic::System::Utils;

ISMTTLog * InstallMachine::logger = SMTTLogManager::GetLogger ( "Smartmatic.SAES.Operation.InstallMachine", GETTEXT_PACKAGE );

InstallMachine::InstallMachine ( )
    : saesDir(NULL)
    , installDir(NULL)
{
}

InstallMachine::~InstallMachine()
{
}

bool InstallMachine::authentication (
    std::string & center,
    std::string & password,
    bool centerCaseSensitive,
    Smartmatic::SAES::Runtime::Environment::SaesDirectories * saesDir,
    Smartmatic::SAES::Runtime::Environment::SaesInstallDirectories * installDir )
{
    bool validate = false;
    std::string folder;
    std::string programUnit;
    std::string rnd;

    this->saesDir = saesDir;
    this->installDir = installDir;

    if ( !fileKeyManger.isInstall() )
    {
        std::list< VotingCenter > & votingCenters ( VotingCentersWrapper::getInstance()->getVotingCenters() );
        std::list< VotingCenter >::iterator it;

        for ( it = votingCenters.begin();
                it != votingCenters.end();
                ++it )
        {
            std::string interCenter = ( *it ).getCode();

            if ( !centerCaseSensitive )
            {
                interCenter = StringUtils::toLowerUtf8 ( ( *it ).getCode() );
                center = StringUtils::toLowerUtf8 ( center );
            }

            if ( interCenter == center )
            {
                fileKeyManger.votingCenter = ( *it );
                folder = ( *it ).getFolder_name();
                rnd = ( *it ).getRnd();
                programUnit = ( ( *it ).getProgram_unit() != NULL ? ( *it ).getProgram_unit().get() : "" );
                break;
            }
        }
    }
    else
    {
        if ( !centerCaseSensitive )
        {
            center = StringUtils::toLowerUtf8 ( center );
        }
        rnd = fileKeyManger.votingCenter.getRnd();
        folder = fileKeyManger.votingCenter.getFolder_name();
        programUnit = ( fileKeyManger.votingCenter.getProgram_unit() != NULL ? fileKeyManger.votingCenter.getProgram_unit().get() : "" );
    }

    bool secretKeyEncrypted = SAESCommonConfigurationManager::getInstance()->getSecurityConfiguration().getSecretKeyEncrypted();

    if ( !secretKeyEncrypted && password != folder && password != "AZERTY" )
    {
        password = "";
    }

    if ( !folder.empty() && !password.empty() )
    {
        installDir->setWorkingPath ( folder );
        installDir->setProgramUnitName ( programUnit );

        fileKeyManger.dir = ( !fileKeyManger.isInstall() ? installDir : saesDir );
        validate = fileKeyManger.authentication(password, center, rnd) && loadConfig();
    }

    if ( validate )
    {
        logger->Debug ( "Login ok" );
    }
    else
    {
        logger->Debug ( "ERROR - authentication machine" );
    }

    return validate;
}

bool InstallMachine::loadConfig ()
{
    bool validate = false;
    BasePathType type = VotingCentersWrapper::getInstance()->getMasterDevice();

    try
    {
        ElectoralConfiguration * electoralConfiguration = MachineOperationManager::getInstance()->getElectoralConfiguration();
        electoralConfiguration->dispose();
        electoralConfiguration->init ( fileKeyManger.dir, type );
        validate = true;
    }
    catch ( ... )
    {
        logger->Debug ( "Error load config in " + type );
    }

    return validate;
}

bool InstallMachine::install()
{
    bool validate = true;

    if ( !fileKeyManger.isInstall() )
    {
        EnvironmentBuilder builder;
        builder.install ( installDir, saesDir );

        try
        {
            VotingCentersWrapper::getInstance()->setInstallationVotingCenter ( fileKeyManger.votingCenter );
            VotingCentersWrapper::getInstance()->persistVotingCenters();
        }
        catch ( ... )
        {
            logger->Debug ( "FATAL - persistVotingCenters" );
            validate = false;
        }
    }

    return validate;
}
