/**
 * @file DeviceTypeMonitorPool.cxx
 * @brief Implementation of the observer that contains all the device type to be monitored.
 * @date 12/12/2012
 * @author Yadickson Soto
 */

#include "DeviceTypeMonitorPool.hxx"
#include <System/Utils/StringUtils.hxx>
#include <Log/ISMTTLog.h>
#include <Log/SMTTLogManager.h>
#include <Operation/Monitor/DeviceTypeMonitor.hxx>
#include <Operation/Monitor/DeviceMonitorFactory.hxx>
#include <Configuration/DeviceTypeMonitorConfigurationManager.hxx>
#include <Operation/Monitor/FlashMonitor.hxx>
#include <Operation/Monitor/SmartcardMonitor.hxx>
#include <Operation/Monitor/PrinterMonitor.hxx>
#include <Operation/Monitor/EurnMonitor.hxx>
#include <Operation/Monitor/BCRMonitor.hxx>
#include <Operation/Monitor/AlarmMonitor.hxx>
#include <Operation/Monitor/P168AMonitor.hxx>
#include <Operation/Monitor/MouseMonitor.hxx>
#include <Operation/Monitor/USBStorageMonitor.hxx>
#include <Operation/Monitor/AbledMonitor.hxx>
#include <Operation/Monitor/EthernetMonitor.hxx>
#include <Operation/Monitor/DeviceMonitorSignal.hxx>
#include <System/OptParser.hxx>
#include <unistd.h>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

using namespace Smartmatic::SAES::Operation::Monitor;
using namespace Smartmatic::SAES::Common::Operation::Monitor;
using namespace Smartmatic::SAES::Common::Configuration;
using namespace Smartmatic::System::Utils;
using namespace Smartmatic::System;
using namespace Smartmatic::Log;

DeviceTypeMonitorPool * DeviceTypeMonitorPool::instance = NULL;
ISMTTLog * DeviceTypeMonitorPool::logger = SMTTLogManager::GetLogger ( "Smartmatic.SAES.Common.Operation.Monitor.DeviceTypeMonitorPool", GETTEXT_PACKAGE );

static const std::string EXPECTED_HID_ID_TAG = "HID_ID";

DeviceTypeMonitorPool::DeviceTypeMonitorPool ()
{
    monitor = DeviceMonitorFactory::getMonitor();
    rawSignal.getSyncSignal().connect ( sigc::mem_fun ( *this, &DeviceTypeMonitorPool::onRawObject ) );
}

DeviceTypeMonitorPool::~DeviceTypeMonitorPool()
{
    if ( monitor )
    {
        delete monitor;
        monitor = NULL;
    }

    dispose();

    instance = NULL;
}

void DeviceTypeMonitorPool::initialize()
{
    monitor->initialize();

    DeviceTypeMonitorPoolConfiguration::DeviceTypeMonitorSequence & list ( DeviceTypeMonitorConfigurationManager::getInstance()->getDeviceTypeMonitorSequence() );
    DeviceTypeMonitorPoolConfiguration::DeviceTypeMonitorIterator it;

    for ( it = list.begin();
            it != list.end();
            ++it )
    {
        DeviceTypeMonitor * interface = NULL;

        switch ( ( *it ).getType() )
        {
            case DeviceType::CONFIGURATION_FLASH:
                interface = new FlashMonitor ();
                break;

            case DeviceType::CONFIGURATION_PRINTER:
                interface = new PrinterMonitor ();
                break;

            case DeviceType::CONFIGURATION_SMARTCARD:
                interface = new SmartcardMonitor ();
                break;

            case DeviceType::CONFIGURATION_BARCODE:
                interface = new BCRMonitor();
                break;

            case DeviceType::CONFIGURATION_EURN:
                interface = new EurnMonitor();
                break;

            case DeviceType::CONFIGURATION_MOUSE:
                interface = new MouseMonitor();
                break;

            case DeviceType::CONFIGURATION_USBSTORAGEMOUNTED:
                interface = new USBStorageMonitor();
                break;

            case DeviceType::CONFIGURATION_ETHERNET:
                interface = new EthernetMonitor();
                break;

            case DeviceType::CONFIGURATION_P168A:
                interface = new P168AMonitor();
                break;

            case DeviceType::CONFIGURATION_ALARMBOX:
                interface = new AlarmMonitor();
                break;

            case DeviceType::CONFIGURATION_ABLED:
                interface = new AbledMonitor();
                break;

            case DeviceType::CONFIGURATION_TOUCHSCREENCALIBRATION:
            case DeviceType::CONFIGURATION_KEYBOARD:
            case DeviceType::CONFIGURATION_UNKNOWN:
            default:
                break;
        }

        if ( interface != NULL )
        {
            logger->Debug ( "Add device type : " + ( *it ).getType() );
            devices.push_back ( interface );
        }
    }

    SafeList < MonitorInfo * > devlist = monitor->searchDevices ();
    processList ( devlist );

    while ( rawSignal.getQueueSize() != 0 )
    {
        logger->Debug ( "Wait" );
        usleep ( 100000 );
    }
}

void DeviceTypeMonitorPool::dispose()
{
    std::list < DeviceTypeMonitor * >::iterator it;

    for ( it = devices.begin();
            it != devices.end();
            ++it )
    {
        delete ( *it );
    }

    devices.clear();
}

bool DeviceTypeMonitorPool::process ()
{
    SafeList < MonitorInfo * > list;

    list = monitor->eventDevices();
    processList ( list );

    return true;
}

void DeviceTypeMonitorPool::deviceAdded ( MonitorInfo * info )
{
    std::list < DeviceTypeMonitor * >::iterator it;

    if ( OptParser::getInstance()->isSet ( OptParser::DBUS_INFO ) )
    {
        logger->Debug ( "--add--" );
        info->viewInfo();
    }

    for ( it = devices.begin();
            it != devices.end();
            ++it )
    {
        DeviceTypeMonitor * device = *it;
        std::string node = getNode ( info, device );

        if ( info->isAdded() && devicePropertiesFound ( info, device ) )
        {
            std::string customNode = DeviceTypeMonitorConfigurationManager::getInstance()->getCustomNode ( node, device->getType() );
            std::string path = device->getDeviceMounted ( node, customNode );

            if ( !path.empty() && device->addNode ( node ) )
            {
                logger->Debug ( "Device added (" + device->getType() + ":" + path + ")" );
                rawSignal.emit ( new DeviceMonitorSignal ( true, path, device ) );
            }

            break;
        }
    }
}

void DeviceTypeMonitorPool::deviceRemoved ( MonitorInfo * info )
{
    std::list < DeviceTypeMonitor * >::iterator it;

    if ( OptParser::getInstance()->isSet ( OptParser::DBUS_INFO ) )
    {
        logger->Debug ( "--remove--" );
        info->viewInfo();
    }

    for ( it = devices.begin();
            it != devices.end();
            ++it )
    {
        DeviceTypeMonitor * device = *it;
        std::string node = getNode ( info, device );

        if ( ! info->isAdded() && device->removeNode ( node ) )
        {
            logger->Debug ( "Device removed (" + device->getType() + ":" + node + ")" );
            rawSignal.emit ( new DeviceMonitorSignal ( false, node, device ) );
            break;
        }
    }
}

std::list < DeviceTypeMonitor * > & DeviceTypeMonitorPool::getDeviceTypeMonitors()
{
    return devices;
}

DeviceTypeMonitor * DeviceTypeMonitorPool::getDeviceTypeMonitor ( DeviceType type )
{
    DeviceTypeMonitor * device = NULL;

    std::list < DeviceTypeMonitor * >::iterator it;

    for ( it = devices.begin();
            it != devices.end();
            ++it )
    {
        if ( ( *it )->getType() == type )
        {
            device = ( *it );
            break;
        }
    }

    return device;
}

unsigned int DeviceTypeMonitorPool::getNumberOfDevicesByType ( DeviceType type )
{
    unsigned int numberOfDevices = 0;

    DeviceTypeMonitor * device = getDeviceTypeMonitor ( type );

    if ( device != NULL )
    {
        numberOfDevices = device->getNumberOfDevices();
    }

    return numberOfDevices;
}

bool DeviceTypeMonitorPool::isMousePresent(const std::string& expectedHidId)
{
    bool isPresent = false;

    const SafeList<MonitorInfo*> list = monitor->searchDevices();

    SafeList<MonitorInfo*>::const_iterator       it  = list.begin();
    const SafeList<MonitorInfo*>::const_iterator end = list.end();

    for (; it != end; ++it)
    {
        const std::string hidIdDetected = (*it)->getValue(EXPECTED_HID_ID_TAG);

        std::vector<std::string> hidDetectedVector;
        StringUtils::splitToVector(hidIdDetected, ":", hidDetectedVector);

        if (hidDetectedVector[0].find(expectedHidId) != std::string::npos)
        {
            isPresent = true;
            break;
        }
    }

    return isPresent;
}

bool DeviceTypeMonitorPool::isDevicePresent(const std::string& expectedHidId, std::string& expectedVendorId, std::string& expectedProductId)
{
    bool isPresent = false;

    const SafeList<MonitorInfo*> list = monitor->searchDevices();

    SafeList<MonitorInfo*>::const_iterator       it  = list.begin();
    const SafeList<MonitorInfo*>::const_iterator end = list.end();

    for (; it != end; ++it)
    {
        const std::string hidIdDetected = (*it)->getValue(EXPECTED_HID_ID_TAG);

        std::vector<std::string> hidDetectedVector;
        StringUtils::splitToVector(hidIdDetected, ":", hidDetectedVector);

        if (hidDetectedVector[0].find(expectedHidId) != std::string::npos)
        {
            const std::string vendorIdDetected  = StringUtils::toLower(hidDetectedVector[1]);
            const std::string productIdDetected = StringUtils::toLower(hidDetectedVector[2]);

            if ((vendorIdDetected.find(expectedVendorId) != std::string::npos) &&
                (productIdDetected.find(expectedProductId) != std::string::npos))
            {
                isPresent = true;
            }
            else
            {
                expectedVendorId  = vendorIdDetected.substr(4);
                expectedProductId = productIdDetected.substr(4);
            }

            break;
        }
    }

    return isPresent;
}

bool DeviceTypeMonitorPool::devicePropertiesFound ( MonitorInfo * info, DeviceTypeMonitor * device )
{
    bool result = false;

    DeviceTypeMonitorConfiguration::PropertySequence seq = DeviceTypeMonitorConfigurationManager::getInstance()->getDeviceTypeMonitorConfiguration ( device->getType() ).getProperty();
    DeviceTypeMonitorConfiguration::PropertyIterator it;

    for ( it = seq.begin();
            it != seq.end();
            ++it )
    {
        std::string value = info->getValue ( ( *it ).getName() );
        std::string expected = ( *it ).getExpected();

        if ( expected.empty() || StringUtils::regex ( expected, value ) )
        {
            result = true;
        }
        else
        {
            result = false;
            break;
        }
    }

    return result;
}

std::string DeviceTypeMonitorPool::getNode ( MonitorInfo * info, DeviceTypeMonitor * device )
{
    std::string property = DeviceTypeMonitorConfigurationManager::getInstance()->getDeviceTypeMonitorConfiguration ( device->getType() ).getNode();
    return info->getValue ( property );
}

void DeviceTypeMonitorPool::processList ( SafeList < MonitorInfo * > & list )
{
    SafeList < MonitorInfo * >::iterator it;

    for ( it = list.begin();
            it != list.end();
            ++it )
    {
        MonitorInfo * info = ( *it );

        if ( info->isAdded() )
        {
            deviceAdded ( ( *it ) );
        }
        else
        {
            deviceRemoved ( ( *it ) );
        }

        delete info;
    }
}

DeviceTypeMonitorPool::DeviceSignal DeviceTypeMonitorPool::getDeviceChangedSignal()
{
    return deviceEventSignal;
}

void DeviceTypeMonitorPool::onRawObject ( void * object )
{
    DeviceMonitorSignal * info = static_cast < DeviceMonitorSignal * > ( object );
    DeviceTypeMonitor * device = info->getDevice();

    if ( info->isAdded() )
    {
        logger->Debug ( "Added : " + info->getNode() );
        device->added ( info->getNode() );
    }
    else
    {
        logger->Debug ( "Removed : " + info->getNode() );
        device->removed ( info->getNode() );
    }

    logger->Debug ( "Emit signal : " + device->getType() );
    getDeviceChangedSignal().emit ( device );

    delete info;
}

DeviceTypeMonitorPool * DeviceTypeMonitorPool::getInstance ( DeviceTypeMonitorPool * externManager )
{
    if ( instance == NULL )
    {
        instance = ( externManager == NULL ? new DeviceTypeMonitorPool () : externManager );
    }

    return instance;
}
