/**
 * @file DeviceTypeMonitorPool.hxx
 * @brief Declaration of the singleton observer that contains all the DeviceTypeMonitors and notifies them when a change occurs in a monitored device.
 * @date 12/12/2012
 * @author Yadickson Soto
 */

#ifndef DEVICE_TYPE_MONITOR_POOL_HXX_
#define DEVICE_TYPE_MONITOR_POOL_HXX_

#include <Operation/Monitor/DeviceTypeMonitor.hxx>
#include <Operation/Monitor/DeviceMonitorInterface.hxx>
#include <System/SafeThread.hxx>
#include <System/SafeDispatcher.hxx>

namespace Smartmatic
{
    namespace Log
    {
        class ISMTTLog;
    }

    namespace SAES
    {
        namespace Operation
        {
            namespace Monitor
            {
                /**
                 * @class DeviceTypeMonitorPool
                 * @brief Singleton observer that contains all the DeviceTypeMonitors and notifies them when a change occurs in a monitored device.
                 * This class acts as a bridge between the Monitor provider and the individual Smartmatic::SAES::Common::Operation::Monitor::DeviceTypeMonitor instances.
                 * @date 12/12/2012
                 * @author Yadickson Soto
                 */
                class DeviceTypeMonitorPool: public Smartmatic::System::SafeThread
                {
                    public:

                        typedef sigc::signal<void, Smartmatic::SAES::Common::Operation::Monitor::DeviceTypeMonitor *> DeviceSignal; ///< The device events' signal

                        /**
                         * @fn virtual ~DeviceTypeMonitorPool();
                         * @brief Class destructor.
                         * @date 12/12/2012
                         * @author Yadickson Soto
                         */
                        virtual ~DeviceTypeMonitorPool();

                        /**
                         * @fn void initialize();
                         * @brief Default initialize class.
                         * @date 07/06/2015
                         * @author Yadickson Soto
                         */
                        void initialize();

                        /**
                         * @fn DeviceSignal getDeviceChangedSignal();
                         * @brief Gets the signal that corresponds to the change event of a device type in the collection.
                         * @date 12/12/2012
                         * @author Yadickson Soto
                         * @return The signal for the changing event.
                         */
                        DeviceSignal getDeviceChangedSignal();

                        /**
                         * @fn std::list < Smartmatic::SAES::Common::Operation::Monitor::DeviceTypeMonitor * > & getDeviceTypeMonitors();
                         * @brief Gets a reference to the Smartmatic::SAES::Common::Operation::Monitor::DeviceTypeMonitor collection.
                         * @date 12/12/2012
                         * @author Yadickson Soto
                         * @return The reference to the device collection.
                         */
                        std::list < Smartmatic::SAES::Common::Operation::Monitor::DeviceTypeMonitor * > & getDeviceTypeMonitors();

                        /**
                         * @fn virtual Smartmatic::SAES::Common::Operation::Monitor::DeviceTypeMonitor * getDeviceTypeMonitor ( Smartmatic::SAES::Common::Configuration::DeviceType type );
                         * @brief Gets the Smartmatic::SAES::Common::Operation::Monitor::DeviceTypeMonitor for the specified device type.
                         * @date 12/12/2012
                         * @author Yadickson Soto
                         * @param type device type
                         * @return the Smartmatic::SAES::Common::Operation::Monitor::DeviceTypeMonitor for the specified device type.
                         */
                        virtual Smartmatic::SAES::Common::Operation::Monitor::DeviceTypeMonitor * getDeviceTypeMonitor ( Smartmatic::SAES::Common::Configuration::DeviceType type );

                        /**
                         * @fn	virtual unsigned int getNumberOfDevicesByType ( Smartmatic::SAES::Common::Configuration::DeviceType type );
                         * @brief	Gets the current number of devices detected of a specified type.
                         * @author	Yadickson Soto
                         * @date	23/05/2014
                         * @param	type The type of DeviceMonitor
                         * @return	the number of monitored devices detected for the specified type.
                         */
                        virtual unsigned int getNumberOfDevicesByType ( Smartmatic::SAES::Common::Configuration::DeviceType type );

                        /**
                         * @fn  bool isMousePresent(const std::string& expectedHidId);
                         * @brief   Gets whether a mouse device is present.
                         * @author  eduardo.fernandez
                         * @date    05/04/2019
                         * @param   expectedHidId The expected hid id.
                         * @return  Whether a mouse device is present.
                         */
                        bool isMousePresent(const std::string& expectedHidId);

                        /**
                         * @fn  bool isDevicePresent(const std::string& expectedHidId, std::string& vendorId, std::string& productId);
                         * @brief   Gets whether the specified device is present.
                         * @author  eduardo.fernandez
                         * @date    26/03/2019
                         * @param   expectedHidId The expected hid id.
                         * @param   expectedVendorId The expected vendor id.
                         * @param   expectedProductId The expected product id.
                         * @return  Whether the specified device is present.
                         */
                        bool isDevicePresent(const std::string& expectedHidId, std::string& expectedVendorId, std::string& expectedProductId);

                        /**
                         * @fn static DeviceTypeMonitorPool * getInstance( DeviceTypeMonitorPool * externManager = NULL );
                         * @brief Gets the instance of the DeviceTypeMonitorPool singleton.
                         * @date 12/12/2012
                         * @author Yadickson Soto
                         * @param 	externManager external manager (for unit tests only)
                         * @return The pointer reference to device monitor that provide access to the instance.
                         */
                        static DeviceTypeMonitorPool * getInstance ( DeviceTypeMonitorPool * externManager = NULL );

                    protected:

                        /**
                         * @fn DeviceTypeMonitorPool();
                         * @brief Default singleton constructor (protected so unit-tests can still subclass).
                         * @date 12/12/2012
                         * @author Yadickson Soto
                         */
                        DeviceTypeMonitorPool();

                        /**
                         * @fn void dispose();
                         * @brief Default dispose class.
                         * @date 07/06/2015
                         * @author Yadickson Soto
                         */
                        void dispose();

                        /**
                         * @fn void processList ( Smartmatic::System::SafeList < Smartmatic::SAES::Common::Operation::Monitor::MonitorInfo * > & list );
                         * @brief Process search devices or event devices, call add or remove device method.
                         * @date 12/12/2012
                         * @author Yadickson Soto
                         * @param list list devices to be process
                         */
                        void processList ( Smartmatic::System::SafeList < Smartmatic::SAES::Common::Operation::Monitor::MonitorInfo * > & list );

                        /**
                         * @fn virtual void deviceAdded ( Smartmatic::SAES::Common::Operation::Monitor::MonitorInfo * info );
                         * @brief This method is a callback that is only invoked by the provider when a device is added.
                         * @date 12/12/2012
                         * @author Yadickson Soto
                         * @param info The information from provider service.
                         */
                        virtual void deviceAdded ( Smartmatic::SAES::Common::Operation::Monitor::MonitorInfo * info );

                        /**
                         * @fn virtual void deviceRemoved ( Smartmatic::SAES::Common::Operation::Monitor::MonitorInfo * info );
                         * @brief This method is a callback that is only invoked by the provider when a device is removed.
                         * @date 12/12/2012
                         * @author Yadickson Soto
                         * @param info The information from provider service.
                         */
                        virtual void deviceRemoved ( Smartmatic::SAES::Common::Operation::Monitor::MonitorInfo * info );

                        /**
                         * @fn virtual bool devicePropertiesFound ( Smartmatic::SAES::Common::Operation::Monitor::MonitorInfo * info, Smartmatic::SAES::Common::Operation::Monitor::DeviceTypeMonitor * device );
                         * @brief whether the device has all the expected properties mentioned in the configuration.
                         * @date 03/06/2015
                         * @author Yadickson Soto
                         * @param info The information from provider service.
                         * @param device The pointer reference to device.
                         * @return true if matches successful; otherwise false.
                         */
                        virtual bool devicePropertiesFound ( Smartmatic::SAES::Common::Operation::Monitor::MonitorInfo * info, Smartmatic::SAES::Common::Operation::Monitor::DeviceTypeMonitor * device );

                        /**
                         * @fn virtual std::string getNode ( Smartmatic::SAES::Common::Operation::Monitor::MonitorInfo * info, Smartmatic::SAES::Common::Operation::Monitor::DeviceTypeMonitor * device );
                         * @brief Gets the device path expected in the configuration.
                         * @date 03/06/2015
                         * @author Yadickson Soto
                         * @param info The information from provider service.
                         * @param device The pointer reference to device.
                         * @return the node.
                         */
                        virtual std::string getNode ( Smartmatic::SAES::Common::Operation::Monitor::MonitorInfo * info, Smartmatic::SAES::Common::Operation::Monitor::DeviceTypeMonitor * device );

                        /**
                         * @fn	    virtual bool process ();
                         * @brief	Callback from thread any time.
                         * @author	Yadickson Soto
                         * @date	22/07/2015
                         * @return true for continue thread
                         */
                        virtual bool process ();

                        /**
                         * @fn	    void onRawObject ( void * object );
                         * @brief	Callback when emit sync signal.
                         * @author	Yadickson Soto
                         * @date	23/07/2015
                         * @param 	object The object to process
                         */
                        void onRawObject ( void * object );

                    private:

                        Smartmatic::SAES::Common::Operation::Monitor::DeviceMonitorInterface * monitor; ///< Monitor interface to load plug, unplug and change devices
                        std::list < Smartmatic::SAES::Common::Operation::Monitor::DeviceTypeMonitor * > devices; ///< The  collection of monitors.

                        DeviceSignal deviceEventSignal;	///< The device event's signal
                        Smartmatic::System::SafeDispatcher rawSignal; ///< The raw signal

                        static DeviceTypeMonitorPool * instance; ///< ///< The pointer to the instance.
                        static Smartmatic::Log::ISMTTLog * logger;  ///< logger
                };
            }
        }
    }
}

#endif /* DEVICE_TYPE_MONITOR_POOL_HXX_ */
