/**
 * @file   AudioVotingCheckUtility.cxx
 * @brief  Implementation of class to check initial conditions of audio voting.
 * @author marcel.mancini
 * @date   Jan 23, 2018
 */

#include "AudioVotingCheckUtility.hxx"
#include <Configuration/MachineModelEnum.hxx>
#include "Configuration/s-a-e-s-functionality-conf-manager.h"
#include <Log/SMTTLogManager.h>
#include <Log/ISMTTLog.h>
#include <Operation/Devices/P168AController.hxx>
#include <Operation/MachineOperationManager.hxx>
#include <Operation/Devices/A4517AbledController.hxx>
#include <Runtime/Environment/saes-directories.h>
#include <System/Utils/StringUtils.hxx>
#include <System/IO/Directory.h>
#include <System/OptParser.hxx>
#include <vector>
#include <string>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

using namespace Smartmatic::Configuration;
using namespace Smartmatic::Log;
using namespace Smartmatic::SAES::Functionality::Configuration;
using namespace Smartmatic::SAES::Operation;
using namespace Smartmatic::SAES::Operation::Devices;
using namespace Smartmatic::SAES::Operation::AudioVoting;
using namespace Smartmatic::SAES::Runtime::Environment;
using namespace Smartmatic::System::IO;
using namespace Smartmatic::System::Utils;
using namespace Smartmatic::System;

static ISMTTLog* logger = SMTTLogManager::GetLogger(
    "Smartmatic.SAES.Operation.AudioVoting.AudioVotingCheckUtility",
    GETTEXT_PACKAGE);

static const std::string DEFAULT_EXPECTED_SERIAL_NUMBER = "P51";

AudioVotingCheckUtility::AudioVotingCheckUtility()
:    m_abledConnected(ABLED10)
{
}

AudioVotingCheckUtility::~AudioVotingCheckUtility()
{
}

bool AudioVotingCheckUtility::pollingPlaceConfiguredForAudioVoting()
{
    bool isAudioVoting;

    isAudioVoting = MachineOperationManager::getInstance()->getElectoralConfiguration()->getVotingDevice()->getVotingBooths().getVoting_booth()[0].getAudio_voting_enable();

    return isAudioVoting;
}

bool AudioVotingCheckUtility::ableIsConnected(const bool ignoreCache)
{
    bool abledPresent = false;

    if (A4517AbledController::getInstance()->isPresent())
    {
        abledPresent = true;
        m_abledConnected = ABLED20;
    }
    else
    {
        std::string machineModel;
        OptParser::getInstance()->getArg( OptParser::MACHINE_MODEL, machineModel );

        if (machineModel != EnumToString( VM_A4517 ))
        {
            std::string serial;

            if (ignoreCache)
                serial = P168AController::getInstance()->getSerialMachine();
            else
                serial = P168AController::getInstance()->getSerialNumber();

            AudioVotingExperienceControllerConfiguration configuration = SAESFunctionalityConfigurationManager::getInstance()->getAudioVotingExperienceControllerConfiguration();
            std::string expectedSerialNumber = DEFAULT_EXPECTED_SERIAL_NUMBER;

            if (configuration.getVotingMachineExpectedSerialNumber().present())
            {
                expectedSerialNumber = StringUtils::toUpper(configuration.getVotingMachineExpectedSerialNumber().get());
            }
            else
            {
                logger->Debug( Glib::ustring::compose( _( "VotingMachineExpectedSerialNumber setting is not included in the current configuration. Evaluating default expected serial number %1." ), expectedSerialNumber ) );
            }

            if (StringUtils::regex ( expectedSerialNumber, serial ))
            {
                abledPresent = true;
                m_abledConnected = ABLED10;
            }
        }
    }

    return abledPresent;
}

bool AudioVotingCheckUtility::voicesArePresent()
{
    bool             arePresent      = false;
    Directory*       directory       = Directory::getInstance();
    SaesDirectories* saesDirectories = MachineOperationManager::getInstance()->getSaesDirectories();
    std::string      path            = saesDirectories->getTextToSpeechVoicesFolderFullPath();

    std::vector<std::string> fileList;
    std::vector<std::string> dirList;

    directory->GetFiles(path, fileList);
    directory->GetDirectories(path, dirList);

    if (!fileList.empty() || dirList.size() > 2)
    {
        arePresent = true;
    }

    return arePresent;
}

bool AudioVotingCheckUtility::removeVoices()
{
    Directory*       directory       = Directory::getInstance();
    SaesDirectories* saesDirectories = MachineOperationManager::getInstance()->getSaesDirectories();
    std::string      path            = saesDirectories->getTextToSpeechVoicesFolderFullPath();

    return directory->Delete(path);
}

AudioVotingCheckUtility::ABLED_CONNECTED AudioVotingCheckUtility::getAbledConnected()
{
    return m_abledConnected;
}
