/**
 * @file   AudioVotingExperienceController.hxx
 * @brief  Declaration of AudioVotingExperienceController class.
 * @author manuel.arguelles
 * @date   Oct 11, 2017
 */

#ifndef AUDIO_VOTING_EXPERIENCE_CONTROLLER_HXX
#define AUDIO_VOTING_EXPERIENCE_CONTROLLER_HXX

#include "AudioVotingLanguageSelection.hxx"
#include "AudioVotingPartySelection.hxx"
#include "AudioVotingPrincipalSelection.hxx"
#include "AudioVotingStandInSelection.hxx"
#include "AudioVotingSelectionsReview.hxx"
#include "AudioVotingReviewConfirmation.hxx"
#include "AudioVotingSelectionsEdition.hxx"
#include "AudioVotingCustomSelection.hxx"
#include "AbledController.hxx"
#include <Voting/Election/election-schema.hxx>
#include <System/SafeThread.hxx>
#include <System/Audio/MixerControl.hxx>
#include <memory>
#include <list>

namespace Smartmatic
{
    namespace Log
    {
        class ISMTTLog;
    }

    namespace SAES
    {
        namespace Resources
        {
            class GeneralAudioResourcesManager;
        }

        namespace Voting
        {
            namespace Vote
            {
                class VoteMode;
            }
        }

        namespace Operation
        {
            namespace Voting
            {
                class VotingExperienceController;

            } /* namespace Voting */

            namespace AudioVoting
            {
                enum CurrentStage
                {
                    NEW_VOTING_EXPERIENCE,
                    LANGUAGE_SELECTION,
                    CUSTOM_ATTRIBUTE_SELECTION,
                    PARTY_SELECTION,
                    EMPTY_PARTY_SELECTION,
                    PRINCIPAL_SELECTION,
                    STAND_IN_SELECTION,
                    REVIEW_STAGE,
                    REVIEW_CONFIRMATION_STAGE,
                    EDIT_SELECTIONS,
                    INTERRUPTED,
                    CANCELED_VOTING_EXPERIENCE
                };

                /**
                 * @class  AudioVotingExperienceController
                 * @brief  Audio voting experience controller.
                 * @author manuel.arguelles
                 * @date   Oct 11, 2017
                 */
                class AudioVotingExperienceController : public System::SafeThread
                {
                    public:

                        /**
                         * @brief  Signal type for message selection.
                         * @author manuel.arguelles
                         * @date   Dec 28, 2017
                         */
                        typedef sigc::signal<void> SignalLanguageSelected;

                        /**
                         * @brief  Signal type for end of voting experience.
                         * @author manuel.arguelles
                         * @date   Dec 28, 2017
                         */
                        typedef sigc::signal<void> SignalEndVotingExperience;

                    public:

                        /**
                         * @brief  Creates an instance of the class.
                         * @author manuel.arguelles
                         * @date   Oct 11, 2017
                         */
                        AudioVotingExperienceController();

                        /**
                         * @brief  Destroys an instance of the class.
                         * @author manuel.arguelles
                         * @date   Oct 11, 2017
                         */
                        ~AudioVotingExperienceController();

                    public:

                        /**
                         * @brief  Starts the voting experience controller.
                         * @author manuel.arguelles
                         * @date   Oct 12, 2017
                         */
                        void start();

                        /**
                         * @brief  Called when the voting experience ends.
                         * @author manuel.arguelles
                         * @date   Dec 28, 2017
                         * @return The end voting experience signal.
                         */
                        SignalEndVotingExperience getSignalEndAudioVotingExperience();

                        /**
                         * @brief  Speaks a message.
                         * @author manuel.arguelles
                         * @date   Dec 28, 2017
                         * @param  messageKey      The message resource to play.
                         * @param  waitUntilFinish To wait until the current message ends.
                         */
                        void playMessage(const std::string& messageKey, const bool waitUntilFinish = true);

                        /**
                         * @brief  Interrupt the voting experience.
                         * @author manuel.arguelles
                         * @date   Dec 28, 2017
                         */
                        void interrupt();

                        /**
                         * @brief  Resume the voting experience.
                         * @author manuel.arguelles
                         * @date   Dec 28, 2017
                         */
                        void resume();

                        /**
                         * @brief  Cancels current audio voting experience.
                         * @author eduardo.fernandez
                         * @date   Mar 27, 2018
                         */
                        void cancel();

                        /**
                         * @brief  Kills current audio voting experience.
                         * @author eduardo.fernandez
                         * @date   Jan 23, 2018
                         */
                        void killVotingExperience();

                        /**
                         * @brief  Return the activation code.
                         * @author manuel.arguelles
                         * @date   Dec 28, 2017
                         * @return The activation code.
                         */
                        std::string getActivationCode();

                        /**
                         * @brief  Return the voting mode.
                         * @author manuel.arguelles
                         * @date   Dec 28, 2017
                         * @return The voting mode.
                         */
                        Smartmatic::SAES::Voting::Vote::VoteMode getVotingMode();

                        /**
                         * @brief  Return the current stage.
                         * @author manuel.arguelles
                         * @date   Dec 28, 2017
                         * @return The current stage.
                         */
                        CurrentStage getCurrentStage();

                        /**
                         * @brief  Called when the language is selected.
                         * @author manuel.arguelles
                         * @date   Dec 28, 2017
                         * @return Language selection signal.
                         */
                        SignalLanguageSelected getSignalLanguageSelected();

                        /**
                         * @brief  Stops the audio voting experience.
                         * @author manuel.arguelles
                         * @date   Dec 28, 2017
                         */
                        void dispose();

                        /**
                         * @brief  Increase volume.
                         * @author manuel.arguelles
                         * @date   Dec 29, 2017
                         */
                        void increaseVolume();

                        /**
                         * @brief  Decrease volume.
                         * @author manuel.arguelles
                         * @date   Dec 29, 2017
                         */
                        void decreaseVolume();

                    private:

                        /**
                         * @brief  Main thread loop.
                         * @author manuel.arguelles
                         * @date   Dec 21, 2017
                         * @return True while the audio experience is active, false when done.
                         */
                        virtual bool process();

                        /**
                         * @brief  Called after the voting experience.
                         * @author manuel.arguelles
                         * @date   Dec 28, 2017
                         */
                        virtual void postStop();

                        /**
                         * @brief  Called when the voter selected the language.
                         * @author manuel.arguelles
                         * @date   Oct 12, 2017
                         */
                        void languageSelected();

                        /**
                         * @brief  Loads the contest list.
                         * @author manuel.arguelles
                         * @date   Oct 12, 2017
                         */
                        void loadContestList();

                        /**
                         * @brief  Advances a step in the voting experience.
                         * @author manuel.arguelles
                         * @date   Oct 12, 2017
                         */
                        void nextStep();

                        /**
                         * @brief  Advances to the next contest.
                         * @author manuel.arguelles
                         * @date   Oct 12, 2017
                         */
                        void nextContest();

                        /**
                         * @brief  Sets the stage to party selection.
                         * @author manuel.arguelles
                         * @date   Dec 28, 2017
                         */
                        void getPartySelection();

                        /**
                         * @brief  Called when a party is selected.
                         * @author manuel.arguelles
                         * @date   Dec 28, 2017
                         * @param  party The selected party.
                         */
                        void onPartySelected(SAES::Voting::Election::Party& party);

                        /**
                         * @brief  Called when no party is selected.
                         * @author manuel.arguelles
                         * @date   Dec 28, 2017
                         */
                        void onEmptyPartySelected();

                        /**
                         * @brief  Called to return to party selection.
                         * @author manuel.arguelles
                         * @date   Dec 28, 2017
                         */
                        void onPartyReturnRequested();

                        /**
                         * @brief  Called to return to the principal selection.
                         * @author manuel.arguelles
                         * @date   Dec 28, 2017
                         */
                        void onPrincipalReturnRequested();

                        /**
                         * @brief  Called to return to the stand in selection.
                         * @author manuel.arguelles
                         * @date   Dec 28, 2017
                         */
                        void onStandInReturnRequested();

                        /**
                         * @brief  Selects the actual contest.
                         * @author manuel.arguelles
                         * @date   Dec 28, 2017
                         * @param  index The contest index.
                         */
                        void setActualContestFromIndex(int index);

                        /**
                         * @brief  Starts the principal selection stage.
                         * @author manuel.arguelles
                         * @date   Dec 28, 2017
                         */
                        void getPrincipalSelection();

                        /**
                         * @brief  Starts the review confirmation stage.
                         * @author manuel.arguelles
                         * @date   Dec 28, 2017
                         */
                        void getReviewConfirmationStage();

                        /**
                         * @brief  Called when the review confirmation ends.
                         * @author manuel.arguelles
                         * @date   Dec 28, 2017
                         * @param  editSelections If the edition is required.
                         */
                        void onReviewConfirmationEnded(const bool editSelections);

                        /**
                         * @brief  Called when returning to the selections review stage.
                         * @author manuel.arguelles
                         * @date   Feb 23, 2018
                         */
                        void onReviewConfirmationReturnRequested();

                        /**
                         * @brief  Starts the review stage.
                         * @author manuel.arguelles
                         * @date   Dec 28, 2017
                         */
                        void getReviewStage();

                        /**
                         * @brief  Called when a ballot selection is made.
                         * @author manuel.arguelles
                         * @date   Dec 28, 2017
                         * @param  ballotOptionCode The ballot option code.
                         * @param  isSelected       If the ballot option was selected or not.
                         */
                        void onBallotSelected(const std::string& ballotOptionCode, bool isSelected);

                        /**
                         * @brief  Called when an option is selected for edit.
                         * @author manuel.arguelles
                         * @date   Dec 28, 2017
                         * @param  mementoPosition The position to edit in the memento.
                         * @param  contestsIndex   The index of the contest.
                         */
                        void onReviewOptionSelected(const int mementoPosition, const int contestsIndex);

                        /**
                         * @brief  Called when the principal is selected.
                         * @author manuel.arguelles
                         * @date   Dec 28, 2017
                         * @param  anySelection If a selection was made.
                         */
                        void onPrincipalSelected(bool anySelection);

                        /**
                         * @brief  Called when the stand in is selected.
                         * @author manuel.arguelles
                         * @date   Dec 28, 2017
                         */
                        void onStandInSelected();

                        /**
                         * @brief  Called when the principal selection process ends.
                         * @author manuel.arguelles
                         * @date   Dec 28, 2017
                         */
                        void onPrincipalDone();

                        /**
                         * @brief  Called when the stand in selection process ends.
                         * @author manuel.arguelles
                         * @date   Dec 28, 2017
                         */
                        void onStandInDone();

                        /**
                         * @brief  Called when a customer attribute is selected.
                         * @author manuel.arguelles
                         * @date   Dec 28, 2017
                         * @param  code The code of the selection.
                         */
                        void onCustomAttributeSelected(const std::string& code);

                        /**
                         * @brief  Starts the custom attribute selection process.
                         * @author manuel.arguelles
                         * @date   Dec 28, 2017
                         */
                        void askCustomAttribute();

                        /**
                         * @brief  Finalizes the audio voting experience.
                         * @author manuel.arguelles
                         * @date   Dec 28, 2017
                         */
                        void endVotingExperience();

                        /**
                         * @brief  Check if the stage changed.
                         * @author manuel.arguelles
                         * @date   Dec 21, 2017
                         * @return True if the stage changed, false if not.
                         */
                        bool isNewStage();

                        /**
                         * @brief  Starts a new stage.
                         * @author manuel.arguelles
                         * @date   Dec 21, 2017
                         */
                        void startNewStage();

                        void checkRunningStages();

                        /**
                         * @brief  Stop all running stages.
                         * @author manuel.arguelles
                         * @date   Dec 29, 2017
                         */
                        void stopAllStages();

                    private:

                        std::auto_ptr<System::Audio::TextToSpeech::ITextToSpeech> m_textToSpeech;
                        AudioVotingLanguageSelection                              m_languageSelection;
                        AudioVotingPartySelection                                 m_partySelection;
                        AudioVotingPrincipalSelection                             m_principalSelection;
                        AudioVotingStandInSelection                               m_standInSelection;
                        AudioVotingSelectionsReview                               m_selectionsReview;
                        AudioVotingReviewConfirmation                             m_reviewConfirmation;
                        AudioVotingSelectionsEdition                              m_selectionsEdition;
                        AudioVotingCustomSelection                                m_customSelection;
                        Operation::Voting::VotingExperienceController*            m_votingExperienceController;
                        int                                                       m_contestIndex;
                        int                                                       m_totalContests;
                        std::list<SAES::Voting::Election::Contest*>               m_contestList;
                        std::list<SAES::Voting::Election::Contest*>::iterator     m_contestIterator;
                        SAES::Voting::Election::Contest*                          m_contest;
                        SAES::Voting::Election::Party                             m_party;
                        SAES::Voting::VotingDevice::Custom_attribute_definition   m_currentCustomAttribute;
                        SignalEndVotingExperience                                 m_endAudioVotingExperienceSignal;
                        bool                                                      m_partySelected;
                        bool                                                      m_anySelection;
                        SAES::Resources::GeneralAudioResourcesManager*            m_generalAudioResources;
                        CurrentStage                                              m_previousStage;
                        CurrentStage                                              m_currentStage;
                        SignalLanguageSelected                                    m_signalLanguageSelected;
                        AbledController                                           m_abled;
                        System::Audio::MixerControl                               m_mixer;
                        static Smartmatic::Log::ISMTTLog*                         m_logger;
                        bool                                                      m_goToReview;
                        bool                                                      m_customAttributeSelected;
                        int                                                       m_audioVotingDefaultVolume;
                };

            } /* namespace AudioVoting */
        } /* namespace Operation */
    } /* namespace SAES */
} /* namespace Smartmatic */

#endif /* AUDIO_VOTING_EXPERIENCE_CONTROLLER_HXX */
