/**
 * @file   AudioVotingLanguageSelection.cxx
 * @brief  Implementation of AudioVotingLanguageSelection class.
 * @author manuel.arguelles
 * @date   Oct 11, 2017
 */

#include "AudioVotingLanguageSelection.hxx"
#include <Log/ISMTTLog.h>
#include <Log/SMTTLogManager.h>
#include <System/Audio/TextToSpeech/ITextToSpeech.hxx>
#include <Voting/set-language-functionality.h>
#include <Operation/MachineOperationManager.hxx>
#include <Resources/ElectionAudioResourcesManager.hxx>
#include <Voting/ElectoralConfiguration.hxx>
#include <Voting/Election/ElectionWrapper.hxx>
#include <Voting/VotingDevice/voting-devices-schema.hxx>
#include <Operation/Devices/A4517AbledController.hxx>
#include <unistd.h>

using namespace Smartmatic::Log;
using namespace Smartmatic::SAES::Operation;
using namespace Smartmatic::SAES::Operation::Devices;
using namespace Smartmatic::SAES::Operation::AudioVoting;
using namespace Smartmatic::SAES::Voting::Election;
using namespace Smartmatic::SAES::Voting;
using namespace Smartmatic::System::Audio::TextToSpeech;
using namespace Smartmatic::SAES::Resources;
using namespace Smartmatic::SAES::HW::Operation;
using namespace Smartmatic::SAES::Voting::VotingDevice;

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#include <libintl.h>
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

#define N_(String) (String) ///<macro for gettext not translate

static const unsigned int IDLE_INTERVAL_S                             = 12;
static const unsigned int IDLE_INITIAL_INTERVAL_S                     = 7;
static const unsigned int SLEEP_AFTER_STOP_US                         = 200000;
static const std::string  LANGUAGE_FRENCH                             = "FR";
static const std::string  LANGUAGE_FRENCH_BRUSSELS                    = "FR-BRU";
static const std::string  LANGUAGE_NON_OFFICIAL_FRENCH                = "NON-FR";
static const std::string  LANGUAGE_DUTCH                              = "NL";
static const std::string  LANGUAGE_DUTCH_BRUSSELS                     = "NL-BRU";
static const std::string  LANGUAGE_NON_OFFICIAL_DUTCH                 = "NON-NL";
static const std::string  LANGUAGE_GERMAN                             = "DE";
static const std::string  LANGUAGE_NON_OFFICIAL_GERMAN                = "NON-DE";
static const std::string  INITIAL_INSTRUCTIONS_SINGLE_MESSAGE_KEY     = "smartmatic.saes.operation.audiovoting.AudioVotingLanguageSelection.InitialInstructionsSingleMessage";
static const std::string  INITIAL_INSTRUCTIONS_MULTIPLE_MESSAGE_KEY   = "smartmatic.saes.operation.audiovoting.AudioVotingLanguageSelection.InitialInstructionsMultipleMessage";
static const std::string  LANGUAGE_SELECTION_INSTRUCTIONS_MESSAGE_KEY = "smartmatic.saes.operation.audiovoting.AudioVotingLanguageSelection.LanguageSelectionInstructions";
static const std::string  CONFIRMATION_MESSAGE_KEY                    = "smartmatic.saes.operation.audiovoting.AudioVotingLanguageSelection.ConfirmationMessage";
static const std::string  WELCOME_MESSAGE_KEY                         = "smartmatic.saes.operation.audiovoting.AudioVotingLanguageSelection.WelcomeMessage";

ISMTTLog* AudioVotingLanguageSelection::m_logger = SMTTLogManager::GetLogger(
    "Smartmatic.SAES.Operation.AudioVoting.AudioVotingLanguageSelection",
    GETTEXT_PACKAGE);

static std::string getLanguageNameFromCode(const std::string& customCode)
{
    std::string languageName = "Belgian Dutch";

    if ((customCode == LANGUAGE_FRENCH) || (customCode == LANGUAGE_FRENCH_BRUSSELS) || (customCode == LANGUAGE_NON_OFFICIAL_FRENCH))
    {
        languageName = "French";
    }
    else if ((customCode == LANGUAGE_DUTCH) || (customCode == LANGUAGE_DUTCH_BRUSSELS) || (customCode == LANGUAGE_NON_OFFICIAL_DUTCH))
    {
        languageName = "Belgian Dutch";
    }
    else if ((customCode == LANGUAGE_GERMAN) || (customCode == LANGUAGE_NON_OFFICIAL_GERMAN))
    {
        languageName = "German";
    }

    return languageName;
}

AudioVotingLanguageSelection::AudioVotingLanguageSelection(ITextToSpeech* textToSpeech)
    : m_textToSpeech(textToSpeech)
    , m_electoralAudioResources(ElectionAudioResourcesManager::getInstance())
    , m_canInstructionsBeInterrupted(false)
    , m_canWelcomeBeInterrupted(false)
{
    Languages::LanguageSequence languages =
        MachineOperationManager::getInstance()->getElectoralConfiguration()->getLanguages().getLanguage();
    Languages::LanguageIterator it;

    for (it = languages.begin(); it != languages.end(); ++it)
    {
        m_languages.push_back(*it);
    }

    m_languages.sort(by_precedence());

    setLanguagesMap();
}

AudioVotingLanguageSelection::~AudioVotingLanguageSelection()
{
}

void AudioVotingLanguageSelection::start()
{
    m_currentStage = INITIAL_STAGE;
    m_canInstructionsBeInterrupted = false;
    m_canWelcomeBeInterrupted = false;
    m_currentOption = (m_languages.size() > 1) ? m_options.end() : m_options.begin();
    m_currentLanguage = m_languagesMap.begin();

    SafeThread::start(100);
}

void AudioVotingLanguageSelection::preStart()
{
    m_selectConnection = m_abled.signalSelectButton().connect(
        sigc::mem_fun(*this, &AudioVotingLanguageSelection::handleSelectPress));
    m_downConnection = m_abled.signalDownButton().connect(
        sigc::mem_fun(*this, &AudioVotingLanguageSelection::handleDownPress));
    m_upConnection = m_abled.signalUpButton().connect(
        sigc::mem_fun(*this, &AudioVotingLanguageSelection::handleUpPress));
    m_leftConnection = m_abled.signalLeftButton().connect(
        sigc::mem_fun(*this, &AudioVotingLanguageSelection::handleLeftPress));
    m_rightConnection = m_abled.signalRightButton().connect(
        sigc::mem_fun(*this, &AudioVotingLanguageSelection::handleRightPress));
    m_helpConnection = m_abled.signalHelpButton().connect(
        sigc::mem_fun(*this, &AudioVotingLanguageSelection::handleHelpPress));
    m_moreSpeedConnection = m_abled.signalSpeedUpButton().connect(
        sigc::mem_fun(*this, &AudioVotingLanguageSelection::handleSpeedPress));
    m_lessSpeedConnection = m_abled.signalSpeedDownButton().connect(
        sigc::mem_fun(*this, &AudioVotingLanguageSelection::handleSpeedPress));
}

void AudioVotingLanguageSelection::postStop()
{
    m_selectConnection.disconnect();
    m_helpConnection.disconnect();
    m_leftConnection.disconnect();
    m_rightConnection.disconnect();
    m_downConnection.disconnect();
    m_upConnection.disconnect();
    m_moreSpeedConnection.disconnect();
    m_lessSpeedConnection.disconnect();
}

bool AudioVotingLanguageSelection::process()
{
    if (m_currentStage == INITIAL_STAGE)
    {
        playInitialMessage();
        m_currentStage = INITIAL_WAIT_STAGE;
        m_idleTime.reset();
    }
    else if (m_currentStage == INITIAL_WAIT_STAGE)
    {
        if (m_idleTime.elapsed() > IDLE_INITIAL_INTERVAL_S)
        {
            m_currentStage = INITIAL_STAGE;
        }
    }
    else if (m_currentStage == LANGUAGE_SELECT_STAGE)
    {
        if (m_idleTime.elapsed() > IDLE_INTERVAL_S)
        {
            m_currentStage = INITIAL_STAGE;
        }
    }
    else if (m_currentStage == CONFIRMATION_STAGE)
    {
        confirmSelection();
        m_currentStage = WAIT_USER_STAGE;
        m_idleTime.reset();
    }
    else if (m_currentStage == WAIT_USER_STAGE)
    {
        if (m_idleTime.elapsed() > IDLE_INTERVAL_S)
        {
            m_currentStage = CONFIRMATION_STAGE;
        }
    }

    return true;
}

void AudioVotingLanguageSelection::playInitialMessage()
{
    bool multipleLanguages = m_languages.size() > 1;
    std::string initialMessageKey = multipleLanguages ? INITIAL_INSTRUCTIONS_MULTIPLE_MESSAGE_KEY : INITIAL_INSTRUCTIONS_SINGLE_MESSAGE_KEY;

    if (m_currentLanguage == m_languagesMap.end())
    {
        m_currentLanguage = m_languagesMap.begin();
        m_canInstructionsBeInterrupted = true;
    }

    SetLanguageFunctionality::SetNewVotingLanguage(m_currentLanguage->first);
    m_textToSpeech->waitWhileSpeaking();
    m_textToSpeech->setLanguage(m_currentLanguage->second);

    std::string initialMessage = m_generalAudioResources.getResource(initialMessageKey);

    m_logger->Debug(initialMessage);
    m_textToSpeech->speak(initialMessage);
    m_textToSpeech->waitWhileSpeaking();

    ++m_currentLanguage;
}

void AudioVotingLanguageSelection::setLanguagesMap()
{
    std::list<SAES::Voting::Election::Language>::iterator it;

    for (it = m_languages.begin(); it != m_languages.end(); ++it)
    {
        const std::string customCode = it->getCustom_code();
        const std::string languageName = getLanguageNameFromCode(customCode);

        m_languagesMap.insert(std::pair<std::string, std::string>(it->getCustom_code(), languageName));
        m_options.push_back(*it);
    }
}

void AudioVotingLanguageSelection::nextSelection()
{
    if (m_currentOption == m_options.end())
    {
        m_currentOption = m_options.begin();
    }
    else
    {
        ++m_currentOption;

        if (m_currentOption == m_options.end())
        {
            m_currentOption = m_options.begin();
        }
    }

    playSelection();
}

void AudioVotingLanguageSelection::previousSelection()
{
    if (m_currentOption == m_options.begin())
    {
        m_currentOption = m_options.end();
    }

    --m_currentOption;

    playSelection();
}

std::string AudioVotingLanguageSelection::getRegionName(ElectoralConfiguration *electoralConfiguration)
{
    Voting_booths::Voting_boothSequence votingBoothSequence =
        electoralConfiguration->getVotingDevice()->getFirstVotingDevice().getVoting_booths().getVoting_booth();

    Voting_booth votingBooth = (*(votingBoothSequence.begin()));

    return m_electoralAudioResources->getResource(votingBooth.getPolling_place_name());
}

std::string AudioVotingLanguageSelection::getWelcomeMessage()
{
    ElectoralConfiguration *electoralConfiguration = MachineOperationManager::getInstance()->getElectoralConfiguration();

    std::string electionNameKey = electoralConfiguration->getElection()->getElection().getName_key();
    std::string electionTitle = m_electoralAudioResources->getResource(electionNameKey);
    std::string municipalityName = getRegionName(electoralConfiguration);
    std::string unformattedWelcomeMessage = m_generalAudioResources.getResource(WELCOME_MESSAGE_KEY);

    Glib::ustring welcomeMessage = Glib::ustring::compose(unformattedWelcomeMessage, electionTitle, municipalityName);

    return welcomeMessage;
}

void AudioVotingLanguageSelection::confirmSelection()
{
    SetLanguageFunctionality::SetNewVotingLanguage(m_currentOption->getCustom_code());

    std::string welcomeMessage = getWelcomeMessage();
    std::string audioText;

    if (m_languages.size() > 1)
    {
        std::string optionAudioText = m_currentOption->getName_key();
        std::string confirmation = Glib::ustring::compose(m_generalAudioResources.getResource(CONFIRMATION_MESSAGE_KEY), optionAudioText);

        audioText = confirmation + " " + welcomeMessage;
    }
    else
    {
        audioText = welcomeMessage;
    }

    m_logger->Debug(audioText);
    m_textToSpeech->stop();
    m_textToSpeech->speak(audioText);
    m_textToSpeech->waitWhileSpeaking();
    m_canWelcomeBeInterrupted = true;
}

void AudioVotingLanguageSelection::playSelection()
{
    const std::string customCode = m_currentOption->getCustom_code();
    const std::string language = m_languagesMap[customCode];

    SetLanguageFunctionality::SetNewVotingLanguage(customCode);
    std::string audioText = m_generalAudioResources.getResource(LANGUAGE_SELECTION_INSTRUCTIONS_MESSAGE_KEY);

    m_logger->Debug(audioText);
    m_textToSpeech->stop();
    m_textToSpeech->waitWhileSpeaking();
    m_textToSpeech->setLanguage(language);
    m_textToSpeech->speak(audioText);
}

void AudioVotingLanguageSelection::end()
{
    m_logger->Debug("Language selection ended");

    stop();
    m_languageSelectedSignal.emit();
}

void AudioVotingLanguageSelection::handleRightPress()
{
    if (m_currentStage == LANGUAGE_SELECT_STAGE)
    {
        nextSelection();
        m_idleTime.reset();
    }
    else if (m_currentStage == INITIAL_WAIT_STAGE && !m_textToSpeech->isSpeaking())
    {
        if (m_languages.size() == 1)
        {
            m_currentStage = CONFIRMATION_STAGE;
        }
        else
        {
            m_currentStage = LANGUAGE_SELECT_STAGE;
            nextSelection();
            m_idleTime.reset();
        }
    }
}

void AudioVotingLanguageSelection::handleLeftPress()
{
    if (m_currentStage == LANGUAGE_SELECT_STAGE)
    {
        previousSelection();
        m_idleTime.reset();
    }
    else if (m_currentStage == INITIAL_WAIT_STAGE && !m_textToSpeech->isSpeaking())
    {
        if (m_languages.size() == 1)
        {
            m_currentStage = CONFIRMATION_STAGE;
        }
        else
        {
            m_currentStage = LANGUAGE_SELECT_STAGE;
            previousSelection();
            m_idleTime.reset();
        }
    }
}

void AudioVotingLanguageSelection::handleDownPress()
{
    if (m_currentStage == INITIAL_WAIT_STAGE && m_languages.size() == 1)
    {
        m_currentStage = CONFIRMATION_STAGE;
    }
}

void AudioVotingLanguageSelection::handleUpPress()
{
    if (m_currentStage == INITIAL_WAIT_STAGE && m_languages.size() == 1)
    {
        m_currentStage = CONFIRMATION_STAGE;
    }
}

void AudioVotingLanguageSelection::handleSelectPress()
{
    if (m_currentOption != m_options.end())
    {
        if (m_currentStage == LANGUAGE_SELECT_STAGE)
        {
            m_currentStage = CONFIRMATION_STAGE;
        }
        else if (m_currentStage == WAIT_USER_STAGE ||
                 (m_currentStage == CONFIRMATION_STAGE && m_canWelcomeBeInterrupted))
        {
            m_textToSpeech->stop();
            Glib::usleep(SLEEP_AFTER_STOP_US);
            end();
        }
    }
}

void AudioVotingLanguageSelection::handleHelpPress()
{
    if (m_currentStage == INITIAL_WAIT_STAGE || m_currentStage == LANGUAGE_SELECT_STAGE)
    {
        m_currentStage = INITIAL_STAGE;
    }
    else if (m_currentStage == WAIT_USER_STAGE)
    {
        m_currentStage = CONFIRMATION_STAGE;
    }
}

void AudioVotingLanguageSelection::handleSpeedPress()
{
    if (m_currentStage == LANGUAGE_SELECT_STAGE)
    {
        playSelection();
        m_idleTime.reset();
    }
    else if (m_currentStage == CONFIRMATION_STAGE || m_currentStage == WAIT_USER_STAGE)
    {
        m_textToSpeech->stop();
        Glib::usleep(SLEEP_AFTER_STOP_US);
        m_currentStage = CONFIRMATION_STAGE;
    }
    else if (m_currentStage == INITIAL_STAGE || m_currentStage == INITIAL_WAIT_STAGE)
    {
        m_textToSpeech->stop();
        Glib::usleep(SLEEP_AFTER_STOP_US);
        --m_currentLanguage;
        m_currentStage = INITIAL_STAGE;
    }
}

AudioVotingLanguageSelection::LanguageSelectedSignal AudioVotingLanguageSelection::signalLanguageSelected()
{
    return m_languageSelectedSignal;
}
