/**
 * @file   AudioVotingReviewConfirmation.cxx
 * @brief  Implementation of AudioVotingReviewConfirmation class.
 * @author eduardo.fernandez
 * @date   Dec 20, 2017
 */

#include "AudioVotingReviewConfirmation.hxx"
#include <Configuration/MachineModelEnum.hxx>
#include <Log/ISMTTLog.h>
#include <Log/SMTTLogManager.h>
#include <Operation/MachineOperationManager.hxx>
#include <System/Audio/TextToSpeech/ITextToSpeech.hxx>
#include <System/OptParser.hxx>
#include <System/Utils/StringUtils.hxx>
#include <Voting/Election/election-schema.hxx>
#include <Voting/ElectoralConfiguration.hxx>
#include <Voting/Vote/vote-schema.hxx>

using namespace Smartmatic::Configuration;
using namespace Smartmatic::Log;
using namespace Smartmatic::SAES::HW::Operation;
using namespace Smartmatic::SAES::Operation::AudioVoting;
using namespace Smartmatic::SAES::Operation::Voting;
using namespace Smartmatic::SAES::Resources;
using namespace Smartmatic::SAES::Voting::Election;
using namespace Smartmatic::SAES::Voting::Vote;
using namespace Smartmatic::SAES::Voting;
using namespace Smartmatic::System::Audio::TextToSpeech;
using namespace Smartmatic::System::Utils;
using namespace Smartmatic::System;

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#include <libintl.h>
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

#define N_(String) (String) ///<macro for gettext not translate

static const unsigned int IDLE_INTERVAL_S                  = 10;
static const unsigned int INTERVAL_MS                      = 10;
static const unsigned int SLEEP_AFTER_STOP_US              = 200000;
static const std::string  GENERAL_INSTRUCTIONS_MESSAGE_KEY = "smartmatic.saes.operation.AudioVoting.AudioVotingReviewConfirmation.GeneralInstructionsMessage";
static const std::string  EDIT_SELECTIONS_MESSAGE_KEY      = "smartmatic.saes.operation.AudioVoting.AudioVotingReviewConfirmation.EditSelectionsMessage";
static const std::string  PRINT_SELECTIONS_MESSAGE_KEY     = "smartmatic.saes.operation.AudioVoting.AudioVotingReviewConfirmation.PrintSelectionsMessage";

ISMTTLog* AudioVotingReviewConfirmation::m_logger = SMTTLogManager::GetLogger(
    "Smartmatic.SAES.Operation.AudioVoting.AudioVotingReviewConfirmation",
    GETTEXT_PACKAGE);

AudioVotingReviewConfirmation::AudioVotingReviewConfirmation(ITextToSpeech* textToSpeech)
    : m_textToSpeech(textToSpeech)
    , m_editSelections(false)
    , m_canInstructionsBeInterrupted(false)
    , m_optionPlayed(false)
{
}

AudioVotingReviewConfirmation::~AudioVotingReviewConfirmation()
{
}

void AudioVotingReviewConfirmation::start()
{
    m_editSelections = false;
    m_currentStage = INITIAL_STAGE;
    m_canInstructionsBeInterrupted = false;

    SafeThread::start(100);
}

void AudioVotingReviewConfirmation::preStart()
{
    std::string currentMachineModel;
    OptParser::getInstance()->getArg( OptParser::MACHINE_MODEL, currentMachineModel );

    m_selectConnection = m_abled.signalSelectButton().connect(
        sigc::mem_fun(*this, &AudioVotingReviewConfirmation::handleSelectPress));
    m_helpConnection = m_abled.signalHelpButton().connect(
        sigc::mem_fun(*this, &AudioVotingReviewConfirmation::handleHelpPress));
    m_leftConnection = m_abled.signalLeftButton().connect(
        sigc::mem_fun(*this, &AudioVotingReviewConfirmation::handleLeftOrRightPress));
    m_rightConnection = m_abled.signalRightButton().connect(
        sigc::mem_fun(*this, &AudioVotingReviewConfirmation::handleLeftOrRightPress));
    m_upConnection = m_abled.signalUpButton().connect(
        sigc::mem_fun(*this, &AudioVotingReviewConfirmation::handleUpPress));
    m_moreSpeedConnection = m_abled.signalSpeedUpButton().connect(
        sigc::mem_fun(*this, &AudioVotingReviewConfirmation::handleSpeedPress));
    m_lessSpeedConnection = m_abled.signalSpeedDownButton().connect(
        sigc::mem_fun(*this, &AudioVotingReviewConfirmation::handleSpeedPress));
}

bool AudioVotingReviewConfirmation::process()
{
    if (m_currentStage == INITIAL_STAGE)
    {
        playInstructions();
        m_idleTime.reset();
        m_currentStage = SELECTION_STAGE;
    }
    else if (m_currentStage == SELECTION_STAGE && m_textToSpeech->isSpeaking())
    {
        m_idleTime.reset();
    }
    else if (m_currentStage == END_STAGE)
    {
        Glib::signal_timeout().connect_once(
            sigc::mem_fun(*this, &AudioVotingReviewConfirmation::end),
            INTERVAL_MS);
    }

    if (m_idleTime.elapsed() > IDLE_INTERVAL_S)
    {
        m_currentStage = INITIAL_STAGE;
    }

    return true;
}

void AudioVotingReviewConfirmation::postStop()
{
    m_selectConnection.disconnect();
    m_helpConnection.disconnect();
    m_leftConnection.disconnect();
    m_rightConnection.disconnect();
    m_moreSpeedConnection.disconnect();
    m_lessSpeedConnection.disconnect();
}

void AudioVotingReviewConfirmation::playInstructions()
{
    std::string generalInstructions = m_generalAudioResources.getResource(GENERAL_INSTRUCTIONS_MESSAGE_KEY);

    m_optionPlayed = false;

    m_logger->Debug(generalInstructions);
    m_textToSpeech->stop();
    m_textToSpeech->speak(generalInstructions);
    m_textToSpeech->waitWhileSpeaking();
    m_canInstructionsBeInterrupted = true;
}

void AudioVotingReviewConfirmation::playCurrentOption()
{
    std::string currentOption = "";

    if (m_editSelections)
    {
        m_editSelections = false;
        currentOption = m_generalAudioResources.getResource(PRINT_SELECTIONS_MESSAGE_KEY);
    }
    else
    {
        m_editSelections = true;
        currentOption = m_generalAudioResources.getResource(EDIT_SELECTIONS_MESSAGE_KEY);
    }

    m_logger->Debug(currentOption);
    m_textToSpeech->stop();
    Glib::usleep(SLEEP_AFTER_STOP_US);
    m_textToSpeech->speak(currentOption);
    m_optionPlayed = true;
}

void AudioVotingReviewConfirmation::end()
{
    stop();
    m_doneSignal.emit(m_editSelections);
}

AudioVotingReviewConfirmation::DoneSignal AudioVotingReviewConfirmation::signalDone()
{
    return m_doneSignal;
}

AudioVotingReviewConfirmation::ReturnSignal AudioVotingReviewConfirmation::signalReturn()
{
    return m_returnSignal;
}

void AudioVotingReviewConfirmation::handleHelpPress()
{
    m_idleTime.reset();
    m_currentStage = INITIAL_STAGE;
}

void AudioVotingReviewConfirmation::handleLeftOrRightPress()
{
    m_idleTime.reset();

    if (m_currentStage == INITIAL_STAGE && m_canInstructionsBeInterrupted)
    {
        playCurrentOption();
    }
    else if (m_currentStage == SELECTION_STAGE)
    {
        playCurrentOption();
    }
}

void AudioVotingReviewConfirmation::handleSelectPress()
{
    m_idleTime.reset();

    if (m_currentStage == INITIAL_STAGE && m_canInstructionsBeInterrupted)
    {
        playCurrentOption();
    }
    else if (m_currentStage == SELECTION_STAGE)
    {
        m_textToSpeech->stop();
        m_currentStage = END_STAGE;
    }
}

void AudioVotingReviewConfirmation::handleUpPress()
{
    m_idleTime.reset();

    if ((m_currentStage == INITIAL_STAGE && m_canInstructionsBeInterrupted) ||
        (m_currentStage == SELECTION_STAGE))
    {
        returnToPreviousStage();
    }
}

void AudioVotingReviewConfirmation::handleSpeedPress()
{
    m_idleTime.reset();

    if (m_currentStage == INITIAL_STAGE)
    {
        m_textToSpeech->stop();
        Glib::usleep(SLEEP_AFTER_STOP_US);
        m_currentStage = INITIAL_STAGE;
    }
    else if (m_currentStage == SELECTION_STAGE && m_optionPlayed)
    {
        std::string currentOption;

        if (m_editSelections)
        {
            currentOption = m_generalAudioResources.getResource(EDIT_SELECTIONS_MESSAGE_KEY);
        }
        else
        {
            currentOption = m_generalAudioResources.getResource(PRINT_SELECTIONS_MESSAGE_KEY);
        }

        m_logger->Debug(currentOption);
        m_textToSpeech->stop();
        Glib::usleep(SLEEP_AFTER_STOP_US);
        m_textToSpeech->speak(currentOption);
    }
    else if (m_currentStage == SELECTION_STAGE && !m_optionPlayed)
    {
        m_currentStage = INITIAL_STAGE;
    }
}

void AudioVotingReviewConfirmation::returnToPreviousStage()
{
    m_textToSpeech->stop();
    stop();
    m_returnSignal.emit();
}
