/**
 * @file   AudioVotingSelectionsEdition.cxx
 * @brief  Implementation of AudioVotingSelectionsEdition class.
 * @author eduardo.fernandez
 * @date   Dec 11, 2017
 */

#include "AudioVotingSelectionsEdition.hxx"
#include <Configuration/MachineModelEnum.hxx>
#include <Log/ISMTTLog.h>
#include <Log/SMTTLogManager.h>
#include <Operation/MachineOperationManager.hxx>
#include <System/Audio/TextToSpeech/ITextToSpeech.hxx>
#include <System/OptParser.hxx>
#include <System/Utils/StringUtils.hxx>
#include <Voting/Election/election-schema.hxx>
#include <Voting/ElectoralConfiguration.hxx>
#include <Voting/Vote/VoteFactory.hxx>

using namespace Smartmatic::Configuration;
using namespace Smartmatic::Log;
using namespace Smartmatic::SAES::HW::Operation;
using namespace Smartmatic::SAES::Operation::AudioVoting;
using namespace Smartmatic::SAES::Operation::Voting;
using namespace Smartmatic::SAES::Resources;
using namespace Smartmatic::SAES::Voting::Election;
using namespace Smartmatic::SAES::Voting::Vote;
using namespace Smartmatic::SAES::Voting;
using namespace Smartmatic::System::Audio::TextToSpeech;
using namespace Smartmatic::System::Utils;
using namespace Smartmatic::System;

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#include <libintl.h>
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

#define N_(String) (String) ///<macro for gettext not translate

static const unsigned int INTERVAL_MS                      = 10;
static const unsigned int IDLE_INTERVAL_S                  = 10;
static const unsigned int SLEEP_AFTER_STOP_US              = 200000;
static const std::string  GENERAL_INSTRUCTIONS_MESSAGE_KEY = "smartmatic.saes.operation.AudioVoting.AudioVotingSelectionsEdition.GeneralInstructionsMessage";

ISMTTLog* AudioVotingSelectionsEdition::m_logger = SMTTLogManager::GetLogger(
    "Smartmatic.SAES.Operation.AudioVoting.AudioVotingSelectionsEdition",
    GETTEXT_PACKAGE);

AudioVotingSelectionsEdition::AudioVotingSelectionsEdition(ITextToSpeech* textToSpeech)
    : m_textToSpeech(textToSpeech)
    , m_selectedOptionsIndex(0)
    , m_currentMementoPosition(0)
    , m_currentMementoContestsIndex(0)
    , m_currentStage(INITIAL_STAGE)
    , m_canInstructionsBeInterrupted(false)
    , m_optionPlayed(false)
{
}

AudioVotingSelectionsEdition::~AudioVotingSelectionsEdition()
{
}

void AudioVotingSelectionsEdition::start()
{
    m_selectedOptions = MachineOperationManager::getInstance()->getVotingExperienceController()->getSelectedOptions(true);
    m_selectedOptionsIndex = 0;
    m_currentMementoPosition = 0;
    m_currentMementoContestsIndex = 0;
    m_currentStage = INITIAL_STAGE;
    m_canInstructionsBeInterrupted = false;

    SafeThread::start(100);
}

void AudioVotingSelectionsEdition::preStart()
{
    std::string currentMachineModel;
    OptParser::getInstance()->getArg( OptParser::MACHINE_MODEL, currentMachineModel );

    m_leftConnection = m_abled.signalLeftButton().connect(
        sigc::mem_fun(*this, &AudioVotingSelectionsEdition::handleLeftPress));
    m_rightConnection = m_abled.signalRightButton().connect(
        sigc::mem_fun(*this, &AudioVotingSelectionsEdition::handleRightPress));
    m_selectConnection = m_abled.signalSelectButton().connect(
        sigc::mem_fun(*this, &AudioVotingSelectionsEdition::handleSelectPress));
    m_helpConnection = m_abled.signalHelpButton().connect(
        sigc::mem_fun(*this, &AudioVotingSelectionsEdition::handleHelpPress));
    m_upConnection = m_abled.signalUpButton().connect(
        sigc::mem_fun(*this, &AudioVotingSelectionsEdition::handleUpPress));
    m_moreSpeedConnection = m_abled.signalSpeedUpButton().connect(
        sigc::mem_fun(*this, &AudioVotingSelectionsEdition::handleSpeedPress));
    m_lessSpeedConnection = m_abled.signalSpeedDownButton().connect(
        sigc::mem_fun(*this, &AudioVotingSelectionsEdition::handleSpeedPress));
}

void AudioVotingSelectionsEdition::postStop()
{
    m_leftConnection.disconnect();
    m_rightConnection.disconnect();
    m_selectConnection.disconnect();
    m_helpConnection.disconnect();
    m_upConnection.disconnect();
    m_moreSpeedConnection.disconnect();
    m_lessSpeedConnection.disconnect();
}

bool AudioVotingSelectionsEdition::process()
{
    if (m_currentStage == INITIAL_STAGE)
    {
        playInstructions();
        m_currentStage = SELECTION_STAGE;
        m_idleTime.reset();
    }
    else if (m_currentStage == SELECTION_STAGE && m_textToSpeech->isSpeaking())
    {
        m_idleTime.reset();
    }
    else if (m_currentStage == FINAL_STAGE)
    {
        Glib::signal_timeout().connect_once(
            sigc::mem_fun(*this, &AudioVotingSelectionsEdition::end),
            INTERVAL_MS);
    }
    else if (m_currentStage == RETURN_STAGE)
    {
        Glib::signal_timeout().connect_once(
            sigc::mem_fun(*this, &AudioVotingSelectionsEdition::returnRequested),
            INTERVAL_MS);
    }

    if (m_idleTime.elapsed() > IDLE_INTERVAL_S)
    {
        m_currentStage = INITIAL_STAGE;
    }

    return true;
}

std::string AudioVotingSelectionsEdition::parseSelectedOptionAudioMessage(const std::string& selectedOption)
{
    std::vector<std::string> fields;
    std::string audioMessage;

    StringUtils::splitToVector(selectedOption, " | ", fields);

    m_currentMementoPosition      = StringUtils::stringToInt(fields[0]);
    m_currentMementoContestsIndex = StringUtils::stringToInt(fields[1]);

    for (size_t i = 2; i < fields.size(); ++i)
    {
        audioMessage += Glib::ustring::compose("%1 ", fields[i]);
    }

    audioMessage = audioMessage.substr(0, audioMessage.size() - 1);

    return audioMessage;
}

void AudioVotingSelectionsEdition::playSelectedOption(const std::string& selectedOption)
{
    std::string audioText = parseSelectedOptionAudioMessage(selectedOption);

    m_logger->Debug(audioText);
    m_textToSpeech->stop();
    Glib::usleep(SLEEP_AFTER_STOP_US);
    m_textToSpeech->speak(audioText);
    m_optionPlayed = true;
}

void AudioVotingSelectionsEdition::playInstructions()
{
    std::string generalInstructions = m_generalAudioResources.getResource(GENERAL_INSTRUCTIONS_MESSAGE_KEY);

    m_optionPlayed = false;

    generalInstructions += " " + parseSelectedOptionAudioMessage(m_selectedOptions[m_selectedOptionsIndex]);

    m_logger->Debug(generalInstructions);
    m_textToSpeech->stop();
    m_textToSpeech->speak(generalInstructions);
    m_textToSpeech->waitWhileSpeaking();
    m_canInstructionsBeInterrupted = true;
}

void AudioVotingSelectionsEdition::end()
{
    stop();
    m_optionSelectedSignal.emit(m_currentMementoPosition, m_currentMementoContestsIndex);
}

void AudioVotingSelectionsEdition::returnRequested()
{
    stop();
    m_doneSignal.emit();
}

void AudioVotingSelectionsEdition::nextSelection()
{
    ++m_selectedOptionsIndex;

    if (m_selectedOptionsIndex == m_selectedOptions.size())
    {
        m_selectedOptionsIndex = 0;
    }

    playSelectedOption(m_selectedOptions[m_selectedOptionsIndex]);
}

void AudioVotingSelectionsEdition::previousSelection()
{
    if (m_selectedOptionsIndex == 0)
    {
        m_selectedOptionsIndex = m_selectedOptions.size();
    }

    --m_selectedOptionsIndex;
    playSelectedOption(m_selectedOptions[m_selectedOptionsIndex]);
}

void AudioVotingSelectionsEdition::handleLeftPress()
{
    m_idleTime.reset();

    if (m_currentStage == SELECTION_STAGE || (m_currentStage == INITIAL_STAGE && m_canInstructionsBeInterrupted))
    {
        previousSelection();
    }
}

void AudioVotingSelectionsEdition::handleRightPress()
{
    m_idleTime.reset();

    if (m_currentStage == SELECTION_STAGE || (m_currentStage == INITIAL_STAGE && m_canInstructionsBeInterrupted))
    {
        nextSelection();
    }
}

void AudioVotingSelectionsEdition::handleSelectPress()
{
    m_idleTime.reset();

    if (m_currentStage == SELECTION_STAGE)
    {
        m_currentStage = FINAL_STAGE;
    }
}

void AudioVotingSelectionsEdition::handleUpPress()
{
    m_idleTime.reset();

    if (m_currentStage == INITIAL_STAGE && m_canInstructionsBeInterrupted)
    {
        m_textToSpeech->stop();
        Glib::usleep(SLEEP_AFTER_STOP_US);
    }
    if (m_currentStage == SELECTION_STAGE)
    {
        m_currentStage = RETURN_STAGE;
    }
}

void AudioVotingSelectionsEdition::handleHelpPress()
{
    m_idleTime.reset();
    m_currentStage = INITIAL_STAGE;
}

void AudioVotingSelectionsEdition::handleSpeedPress()
{
    m_idleTime.reset();

    if (m_currentStage == INITIAL_STAGE)
    {
        m_textToSpeech->stop();
        Glib::usleep(SLEEP_AFTER_STOP_US);
        m_currentStage = INITIAL_STAGE;
    }
    else if (m_currentStage == SELECTION_STAGE && m_optionPlayed)
    {
        playSelectedOption(m_selectedOptions[m_selectedOptionsIndex]);
    }
    else if (m_currentStage == SELECTION_STAGE && !m_optionPlayed)
    {
        m_currentStage = INITIAL_STAGE;
    }
}

AudioVotingSelectionsEdition::OptionSelectedSignal AudioVotingSelectionsEdition::signalOptionSelected()
{
    return m_optionSelectedSignal;
}

AudioVotingSelectionsEdition::DoneSignal AudioVotingSelectionsEdition::signalDone()
{
    return m_doneSignal;
}
