/*
 * VoteComplianceVerificationManager.hxx
 *
 *  Created on: Feb 10, 2018
 *      Author: eduardo.fernandez
 */

#ifndef VOTE_COMPLIANCE_VERIFICATION_MANAGER_HXX
#define VOTE_COMPLIANCE_VERIFICATION_MANAGER_HXX

#include <string>
#include <list>
#include <vector>
#include <map>
#include <Log/ISMTTLog.h>

namespace Smartmatic
{
    namespace SAES
    {
        namespace Voting
        {
            class ElectoralConfiguration;

            namespace Vote
            {
                class Vote;
                class Selection;
            }

            namespace PBReports
            {
                class VoteResults;
                class VoteStatisticsWrapper;
            }

            namespace Eml
            {
                class EML;
            }
        }

        namespace Operation
        {
            namespace VoteCompliance
            {
                /**
                 * @class	VoteComplianceVerificationManager
                 * @brief	Header class VoteComplianceVerificationManager.
                 * @author	eduardo.fernandez
                 * @date	Feb 10, 2018
                 */
                class VoteComplianceVerificationManager
                {
                    public:

                        /**
                         * @fn	VoteComplianceVerificationManager::VoteComplianceVerificationManager ( Smartmatic::SAES::Voting::ElectoralConfiguration * electoralConfiguration );
                         * @brief	Class constructor.
                         * @param	electoralConfiguration The electoral configuration.
                         * @author	eduardo.fernandez
                         * @date	Feb 06, 2018
                         */
                        explicit VoteComplianceVerificationManager(
                            Smartmatic::SAES::Voting::ElectoralConfiguration * electoralConfiguration);

                        /**
                         * @fn	virtual BeforePrintingVoteComplianceVerificator::~BeforePrintingVoteComplianceVerificator();
                         * @brief	Finalizer.
                         * @author	eduardo.fernandez
                         * @date	Feb 06, 2018
                         */
                        virtual ~VoteComplianceVerificationManager();


                        /**
                         * @fn  bool VoteComplianceVerificationManager::checkMinimumNumberOfSelections(const Smartmatic::SAES::Voting::Vote::Vote &vote);
                         * @brief    Checks minimum number of selections in the given vote.
                         * @param    vote Vote containing the voter selections.
                         * @return   Whether the vote satisfies the minimum number of selections.
                         * @author   eduardo.fernandez
                         * @date     Feb 07, 2018
                         */
                        bool checkMinimumNumberOfSelections(const Smartmatic::SAES::Voting::Vote::Vote &vote);

                        /**
                         * @fn  bool VoteComplianceVerificationManager::checkSelectionsBelongToSameParty(const Smartmatic::SAES::Voting::Vote::Vote &vote);
                         * @brief    Checks whether the given vote's selections belong to the same party.
                         * @param    vote Vote containing the voter selections.
                         * @return   Whether the given vote's selections belong to the same party.
                         * @author   eduardo.fernandez
                         * @date     Feb 07, 2018
                         */
                        bool checkSelectionsBelongToSameParty(const Smartmatic::SAES::Voting::Vote::Vote &vote);

                        /**
                         * @fn  bool VoteComplianceVerificationManager::checkSelectionsApplicableToConstituency(const Smartmatic::SAES::Voting::Vote::Vote &vote);
                         * @brief    Checks whether the given vote's selections' party belong to the same contest.
                         * @param    vote Vote containing the voter selections.
                         * @return   Whether the given vote's selections' party belong to the same contest.
                         * @author   eduardo.fernandez
                         * @date     Feb 08, 2018
                         */
                        bool checkSelectionsApplicableToConstituency(const Smartmatic::SAES::Voting::Vote::Vote &vote);

                        /**
                         * @fn  bool VoteComplianceVerificationManager::checkUniquenessInSelections(const Smartmatic::SAES::Voting::Vote::Vote &vote);
                         * @brief    Checks whether the given vote's selections are unique within the vote.
                         * @param    vote Vote containing the voter selections.
                         * @return   Whether the given vote's selections' party belong to the same contest.
                         * @author   eduardo.fernandez
                         * @date     Feb 08, 2018
                         */
                        bool checkUniquenessInSelections(const Smartmatic::SAES::Voting::Vote::Vote &vote);

                        /**
                         * @fn  bool VoteComplianceVerificationManager::checkListHeaderResourceCorrectness(const Smartmatic::SAES::Voting::Vote::Vote &vote, const std::map < std::string, std::map < std::string, std::string > > &textToPrintMap);
                         * @brief    Checks whether the string resource of the parties to print matches the string resource
                         *           of the vote's parties.
                         * @param    vote Vote containing the voter selections.
                         * @param    textToPrintMap Map containing the parties to print.
                         * @return   Whether the string resource of the parties to print matches the string resource
                         *           of the vote's parties.
                         * @author   eduardo.fernandez
                         * @date     Feb 09, 2018
                         */
                        bool checkListHeaderResourceCorrectness(
                            const Smartmatic::SAES::Voting::Vote::Vote &vote,
                            std::map < std::string, std::map < std::string, std::string > > textToPrintMap);

                        /**
                         * @fn  bool VoteComplianceVerificationManager::checkCumulativeNumberOfSelections(const Smartmatic::SAES::Voting::Vote::Vote &vote);
                         * @brief    Checks whether the cumulative number of selected candidates are less than or equal
                         *           to the number of candidates registered on the party.
                         * @param    vote Vote containing the voter selections.
                         * @return   Whether the cumulative number of selected candidates are less than or equal
                         *           to the number of candidates registered on the party.
                         * @author   eduardo.fernandez
                         * @date     Feb 08, 2018
                         */
                        bool checkCumulativeNumberOfSelections(const Smartmatic::SAES::Voting::Vote::Vote &vote);

                        /**
                         * @fn  bool VoteComplianceVerificationManager::checkTotalCumulativeSelections();
                         * @brief    Checks whether the cumulative number of categorized votes don't exceed the total number of votes.
                         * @param    totalVotes The total number of votes.
                         * @param    voteResults The vote results.
                         * @return   Whether the cumulative number of categorized votes don't exceed the total number of votes.
                         * @author   eduardo.fernandez
                         * @date     Feb 14, 2018
                         */
                        bool checkTotalCumulativeSelections(
                            const size_t totalVotes,
                            const Smartmatic::SAES::Voting::PBReports::VoteResults &voteResults);

                        /**
                         * @fn  bool VoteComplianceVerificationManager::checkIndividualCumulativeSelections(const Smartmatic::SAES::Voting::PBReports::VoteResults &voteResults);
                         * @brief    Checks whether the cumulative number of categorized votes for a single list don't
                         *           exceed the total number of votes for that list.
                         * @param    voteResults The vote results.
                         * @return   Whether the cumulative number of categorized votes for a single list don't
                         *           exceed the total number of votes for that list.
                         * @author   eduardo.fernandez
                         * @date     Feb 15, 2018
                         */
                        bool checkIndividualCumulativeSelections(
                            const Smartmatic::SAES::Voting::PBReports::VoteResults &voteResults);

                        /*
                         * @fn bool VoteComplianceVerificationManager::checkEmlVotesDeposited(const std::map<std::string, Voting::Eml::EML>& plainEmlFiles);
                         * @brief   Checks whether the total votes deposited matches the sum of valid and invalid votes.
                         * @param   plainEmlFiles The map containing the plain EML files to verify.
                         * @return  Whether the total votes deposited matches the sum of valid and invalid votes.
                         * @author  eduardo.fernandez
                         * @date    Nov 13, 2018
                         */
                        bool checkEmlVotesDeposited(const std::map<std::string, Voting::Eml::EML>& plainEmlFiles);

                        /*
                         * @fn bool VoteComplianceVerificationManager::checkEmlVotesByVoterType(const std::map<std::string, Voting::Eml::EML>& plainEmlFiles);
                         * @brief   Checks whether the sum of votes by voter type matches the total voters number.
                         * @param   plainEmlFiles The map containing the plain EML files to verify.
                         * @return  Whether the sum of votes by voter type matches the total voters number.
                         * @author  eduardo.fernandez
                         * @date    Nov 13, 2018
                         */
                        bool checkEmlVotesByVoterType(const std::map<std::string, Voting::Eml::EML>& plainEmlFiles);

                        /**
                         * @fn bool VoteComplianceVerificationManager::checkEmlVotesCounter(const std::map<std::string, Voting::Eml::EML>& plainEmlFiles, Smartmatic::SAES::Voting::PBReports::VoteStatisticsWrapper * const voteStatisticsWrapper);
                         * @brief Checks whether the EML votes deposited match the vote statistics votes counter by contest.
                         * @param plainEmlFiles The map containing the plain EML files to verify.
                         * @param voteStatisticsWrapper Thw vote statistics wrapper.
                         */
                        bool checkEmlVotesCounter(
                            const std::map<std::string, Voting::Eml::EML>& plainEmlFiles,
                            Smartmatic::SAES::Voting::PBReports::VoteStatisticsWrapper * const voteStatisticsWrapper);

                    private:

                        /**
                         * @fn  bool VoteComplianceVerificationManager::checkMinimumNumberOfSelectionsInContest(const Smartmatic::SAES::Voting::Vote::Selection &selection);
                         * @brief    Checks minimum number of selections in the given vote.
                         * @param    selection Voter selections.
                         * @return   Whether the selection satisfies the minimum number of selections.
                         * @author   eduardo.fernandez
                         * @date     Feb 07, 2018
                         */
                        bool checkMinimumNumberOfSelectionsInContest(const Smartmatic::SAES::Voting::Vote::Selection &selection);

                        /**
                         * @fn  bool VoteComplianceVerificationManager::checkSelectionsBelongToSamePartyInContest(const Smartmatic::SAES::Voting::Vote::Selection &selection);
                         * @brief    Checks whether the given vote's selections belong to the same party.
                         * @param    selection Voter selections.
                         * @return   Whether the selections belong to the same party.
                         * @author   eduardo.fernandez
                         * @date     Feb 07, 2018
                         */
                        bool checkSelectionsBelongToSamePartyInContest(const Smartmatic::SAES::Voting::Vote::Selection &selection);

                        /**
                         * @fn  bool VoteComplianceVerificationManager::checkSelectionsApplicableToConstituencyInContest(const Smartmatic::SAES::Voting::Vote::Selection &selection);
                         * @brief    Checks whether the given vote's selections' party belong to the same contest.
                         * @param    vote Vote containing the voter selections.
                         * @return   Whether the given vote's selections' party belong to the same contest.
                         * @author   eduardo.fernandez
                         * @date     Feb 08, 2018
                         */
                        bool checkSelectionsApplicableToConstituencyInContest(const Smartmatic::SAES::Voting::Vote::Selection &selection);

                        /**
                         * @fn  bool VoteComplianceVerificationManager::checkUniquenessInSelectionsInContest(const Smartmatic::SAES::Voting::Vote::Selection &selection);
                         * @brief    Checks whether the given vote's selections are unique within the vote.
                         * @param    currentSelection Current voter selection.
                         * @param    vote Vote containing the voter selections.
                         * @return   Whether the given vote's selections' party belong to the same contest.
                         * @author   eduardo.fernandez
                         * @date     Feb 08, 2018
                         */
                        bool checkUniquenessInSelectionsInContest(
                            const Smartmatic::SAES::Voting::Vote::Selection &currentSelection,
                            const Smartmatic::SAES::Voting::Vote::Vote &vote);

                    private:

                        static Smartmatic::Log::ISMTTLog * m_logger;	///< the logger
                        Smartmatic::SAES::Voting::ElectoralConfiguration * m_electoralConfiguration;   ///< electoral configuration
                };
            }
        }
    }
}

#endif /* VOTE_COMPLIANCE_VERIFICATION_MANAGER_HXX */
