/**
 * @file	VotingState.hxx
 * @brief	voting state class.
 * @author	Youssef.tourkmani
 * @date	01/11/2012
 */

#ifndef __VOTING_STATE_H__
#define __VOTING_STATE_H__

#include <string>
#include <list>
#include <map>
#include <Voting/VotingDevice/voting-devices-schema.hxx>

namespace Smartmatic
{
    namespace SAES
    {
        namespace Operation
        {
            namespace Voting
            {
                /**
                 * @class	VotingState
                 * @brief	Header class VotingState.
                 * @author	Yadickson Soto
                 * @date	07-03-2011
                 */

                class VotingState
                {
                    public:

                        /**
                         * @enum	TypeResponse
                         * @brief	Values that represent TypeResponse.
                         */
                        enum TypeResponse
                        {
                            START
                            , USER_RESPONSE
                            , AUTOMATIC_RESPONSE
                        };

                        /**
                         * @typedef	std::list < Smartmatic::SAES::Voting::VotingDevice::Logical_voter_group > ListLogicalVoterGroup
                         * @brief	Defines an alias representing list logical voter group for state.
                         */

                        typedef std::list < Smartmatic::SAES::Voting::VotingDevice::Logical_voter_group * > ListLogicalVoterGroup; /**< list logical voter group for state*/

                        /**
                         * @typedef	std::list < Smartmatic::SAES::Voting::VotingDevice::Custom_attribute_definition > ListCustomAttributeDefinition
                         * @brief	Defines an alias representing the list custom attribute definition for state.
                         */

                        typedef std::list < Smartmatic::SAES::Voting::VotingDevice::Custom_attribute_definition > ListCustomAttributeDefinition;

                        /**
                         * @typedef	std::map < std::string, std::list < std::string > > MapList
                         * @brief	Defines an alias representing list of maps for state.
                         */

                        typedef std::map < std::string, std::list < std::string > > MapList;

                        /**
                         * @typedef	std::map < std::string, MapList > MapContest
                         * @brief	Defines an alias representing the map contest for state.
                         */

                        typedef std::map < std::string, MapList > MapContest;

                    private:

                        std::string code;	///< response code
                        std::string value;   ///< response value
                        TypeResponse response;   ///< response of user or automatic
                        ListLogicalVoterGroup logicalVoterGroups;	///< list logical voter group for state
                        ListCustomAttributeDefinition customAttributeDefinitions;	///< list custom attribute definition for state
                        MapContest mapContest;   ///< map contest
                        MapList mapContestGroup; ///< map contest group
                        std::list <std::string> contestByContestClass; ///< contest by contest class
                        std::list <std::string> contestResolved; ///< contest resolved

                    public:

                        /**
                         * @fn	VotingState::VotingState ( std::string code, std::string value,
                         * 			TypeResponse response = START );
                         * @brief	class constructor.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         * @param	code			The response code.
                         * @param	value			The response value.
                         * @param	response	true to user response.
                         */
                        VotingState ( std::string code, std::string value, TypeResponse response = START );

                        /**
                         * @fn	std::string VotingState::getCode();
                         * @brief	method to get the response code.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         * @return	response code.
                         */

                        std::string getCode();

                        /**
                         * @fn	std::string VotingState::getValue();
                         * @brief	method to get the response value.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         * @return	response value.
                         */

                        std::string getValue();

                        /**
                         * @fn	TypeResponse VotingState::getResponse();
                         * @brief	query if it is user response.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         * @return	true if is user response.
                         */

                        TypeResponse getResponse();

                        /**
                         * @fn	void VotingState::addBallotOption ( std::string contestCode, std::string partyCode,
                         * 			std::string ballotOptionCode );
                         * @brief	method to add ballot option.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         * @param	contestCode				contest code.
                         * @param	partyCode					party code.
                         * @param	ballotOptionCode	ballot option.
                         */

                        void addBallotOption ( const std::string& contestCode, const std::string& partyCode, const std::string& ballotOptionCode );

                        /**
                         * @fn	void VotingState::removeContest ( std::string contestCode );
                         * @brief	method to remove contest.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         * @param	contestCode	contest code.
                         */

                        void removeContest ( const std::string& contestCode );

                        /**
                         * @fn	void VotingState::removeAllContest ();
                         * @brief	Removes all contest.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         */
                        void removeAllContest ();

                        /**
                         * @fn	void VotingState::removePartyByContest ( std::string contestCode, std::string partyCode );
                         * @brief	method to remove party by contest.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         * @param	contestCode	contest code.
                         * @param	partyCode	 	party code.
                         */

                        void removePartyByContest ( const std::string& contestCode, const std::string& partyCode );

                        /**
                         * @fn	void VotingState::removeBallotOption ( std::string contestCode, std::string partyCode,
                         * 			std::string ballotOptionCode );
                         * @brief	method to remove ballot options.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         * @param	contestCode				contest code.
                         * @param	partyCode					party code.
                         * @param	ballotOptionCode	ballot option.
                         */

                        void removeBallotOption ( const std::string& contestCode, const std::string& partyCode, const std::string& ballotOptionCode );

                        /**
                         * @fn	void VotingState::addContestByContestGroup ( std::string contestGroupCode,
                         * 			std::string contestCode );
                         * @brief	add contest to contest group.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         * @param	contestGroupCode	contest group code.
                         * @param	contestCode				contest code.
                         */

                        void addContestByContestGroup ( const std::string& contestGroupCode, const std::string& contestCode );

                        /**
                         * @fn	void VotingState::removeContestByContestGroup ( std::string contestGroupCode,
                         * 			std::string contestCode );
                         * @brief	removes contest from contest group.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         * @param	contestGroupCode	contest group code.
                         * @param	contestCode				contest code.
                         */

                        void removeContestByContestGroup ( const std::string& contestGroupCode, const std::string& contestCode );

                        /**
                         * @fn	void VotingState::removeContestGroup ( std::string contestGroupCode );
                         * @brief	removes contest group.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         * @param	contestGroupCode	contest group code.
                         */

                        void removeContestGroup ( const std::string& contestGroupCode );

                        /**
                         * @fn	void VotingState::setLogicalVoterGroups ( ListLogicalVoterGroup logicalVoterGroups );
                         * @brief	sets the list of logical voter groups.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         * @param	logicalVoterGroups	list of logical voter groups.
                         */

                        void setLogicalVoterGroups ( ListLogicalVoterGroup logicalVoterGroups );

                        /**
                         * @fn	void VotingState::setCustomAttributeDefinitions ( ListCustomAttributeDefinition customAttributeDefinitions );
                         * @brief	sets the list of custom attribute definitions.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         * @param	customAttributeDefinitions	list of custom attribute definitions.
                         */

                        void setCustomAttributeDefinitions ( const ListCustomAttributeDefinition& customAttributeDefinitions );

                        /**
                         * @fn	void VotingState::setContestByContestClass ( std::list <std::string> contest );
                         * @brief	sets the list of contest by contest class.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         * @param	contest	list of contest by contest class.
                         */

                        void setContestByContestClass ( const std::list <std::string>& contest );

                        /**
                         * @fn	std::list < std::string > VotingState::getContests ();
                         * @brief	gets the list of contests.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         * @return	list of contests.
                         */

                        std::list < std::string > getContests ();

                        /**
                         * @fn	std::list < std::string > VotingState::getPartys ( std::string contestCode );
                         * @brief	gets the list of partys.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         * @param	contestCode	contest code.
                         * @return	list of partys.
                         */

                        std::list < std::string > getPartys ( const std::string& contestCode );

                        /**
                         * @fn	std::list < std::string > VotingState::getBallotOptions ( std::string contestCode,
                         * 			std::string partyCode );
                         * @brief	gets the list of ballot options.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         * @param	contestCode	contest code.
                         * @param	partyCode	 	party code.
                         * @return	list of ballot options.
                         */

                        std::list < std::string > getBallotOptions ( const std::string& contestCode, const std::string& partyCode );

                        /**
                         * @fn	std::list < std::string > VotingState::getContestGroups ();
                         * @brief	gets the list of contest groups.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         * @return	list of contest group.
                         */

                        std::list < std::string > getContestGroups ();

                        /**
                         * @fn	std::list < std::string > & VotingState::getContestsByContestGroup ( std::string contestGroupCode );
                         * @brief	gets the list of contests by contest group.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         * @param	contestGroupCode	contest group code.
                         * @return	list of contests by contest group.
                         */

                        std::list < std::string > getContestsByContestGroup ( const std::string& contestGroupCode );

                        /**
                         * @fn	std::list < std::string > VotingState::getAllContestsByContestGroup ();
                         * @brief	gets the list of contests by contest group.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         * @return	list of contests by contest group.
                         */

                        std::list < std::string > getAllContestsByContestGroup ();

                        /**
                         * @fn	ListLogicalVoterGroup & VotingState::getLogicalVoterGroups ();
                         * @brief	gets the list of logical voter groups.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         * @return	list of logical voter groups.
                         */

                        ListLogicalVoterGroup getLogicalVoterGroups ();

                        /**
                         * @fn	ListCustomAttributeDefinition & VotingState::getCustomAttributeDefinitions ();
                         * @brief	gets the list of custom attribute definitions.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         * @return	list of custom attribute definitions.
                         */

                        ListCustomAttributeDefinition getCustomAttributeDefinitions ();

                        /**
                         * @fn	std::list <std::string> & VotingState::getContestByContestClass();
                         * @brief	gets the list of contest by contest class.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         * @return	list of contest by contest class.
                         */

                        std::list <std::string> getContestByContestClass();
                };
            }
        }
    }
}

#endif /*__VOTING_STATE_H__*/
