#ifndef TEST_CRYPTO_SMARTCARD_TEST
#define TEST_CRYPTO_SMARTCARD_TEST

#ifdef ECUT_MAIN

#include <cppunit/extensions/HelperMacros.h>
#include <Security/KeyManager.hxx>
#include <Operation/Security/CryptoSmartcard.hxx>
#include <System/Security/OpenSSLSecurity.hxx>
#include <tests/MockKeyManager.hxx>

class CryptoSmartcardTest : public CPPUNIT_NS::TestFixture
{
    private:

        Smartmatic::SAES::Operation::Security::CryptoSmartcard * cryptoSmartcard;
        Smartmatic::SAES::Common::Security::KeyManager * keyManager;

    public:

        CryptoSmartcardTest ()
        {
            cryptoSmartcard = NULL;
            keyManager = NULL;
        }

        void setUp()
        {
            MockKeyManager* mockedKeyManager = new MockKeyManager();

            Smartmatic::System::Security::OpenSSLSecurity::init();

            cryptoSmartcard = new Smartmatic::SAES::Operation::Security::CryptoSmartcard ();
            keyManager = Smartmatic::SAES::Common::Security::KeyManager::getInstance(mockedKeyManager);
        }

        void tearDown()
        {
            if ( cryptoSmartcard )
            {
                delete cryptoSmartcard;
                cryptoSmartcard = NULL;
            }

            if ( keyManager )
            {
                delete keyManager;
                keyManager = NULL;
            }

            Smartmatic::System::Security::OpenSSLSecurity::clear();
        }

        void testGeneratePINWithSecretKeyNULL()
        {
            cryptoSmartcard->generatePin();
            const Smartmatic::System::SafeBuffer & buffer ( cryptoSmartcard->getPin() );

            CPPUNIT_ASSERT ( 0 == buffer.GetSize() );
        }

        void testGeneratePINWithSecretKey()
        {
            Smartmatic::System::SafeBuffer key ( "abcd1234567890" );
            keyManager->addKeyByType ( key, Smartmatic::SAES::Common::Configuration::KeyType::CONFIGURATION_SECRET_KEY );

            cryptoSmartcard->generatePin();

            const Smartmatic::System::SafeBuffer & buffer ( cryptoSmartcard->getPin() );

            CPPUNIT_ASSERT ( PASSWORD_PIN_LEN == buffer.GetSize() );
        }

        void testGenerateKeys()
        {
            cryptoSmartcard->generateKeys();

            const Smartmatic::System::SafeBuffer & key ( cryptoSmartcard->getKey() );

            CPPUNIT_ASSERT ( PASSWORD_FILE_KEY == key.GetSize() );
        }

        void testCryptoDataRegresion()
        {
            Smartmatic::System::SafeBuffer key ( "abcd1234567890" );
            keyManager->addKeyByType ( key, Smartmatic::SAES::Common::Configuration::KeyType::CONFIGURATION_SECRET_KEY );

            Smartmatic::System::SafeBuffer buffer ( "ABCD" );
            Smartmatic::System::SafeBuffer crypto;
            Smartmatic::System::SafeBuffer decrypto;

            cryptoSmartcard->generatePin();
            cryptoSmartcard->generateKeys();

            CPPUNIT_ASSERT ( true == cryptoSmartcard->crypto ( buffer, crypto ) );
            CPPUNIT_ASSERT ( true == cryptoSmartcard->decrypto ( crypto, decrypto ) );
            CPPUNIT_ASSERT ( buffer.GetBase64Data() == decrypto.GetBase64Data() );
        }

        CPPUNIT_TEST_SUITE ( CryptoSmartcardTest );
        CPPUNIT_TEST ( testGeneratePINWithSecretKeyNULL );
        CPPUNIT_TEST ( testGeneratePINWithSecretKey );
        CPPUNIT_TEST ( testGenerateKeys );
        CPPUNIT_TEST ( testCryptoDataRegresion );
        CPPUNIT_TEST_SUITE_END();
};

#endif /* ECUT_MAIN */

#endif /* TEST_CRYPTO_SMARTCARD_TEST */
