/**
 * @file DataUtilsValidatorFunctionality.cxx
 * @brief Body class data utils validator functionality
 * @date 18/07/2012
 * @author Yadickson Soto
 */

#include "DataUtilsValidatorFunctionality.hxx"
#include "Gui/Widgets/FactoryMethodWidget.hxx"
#include "Gui/Widgets/AbstractFactoryWidget.hxx"
#include <Voting/VotingCenters/VotingCentersWrapper.hxx>
#include <Operation/MachineOperationManager.hxx>
#include <Operation/CheckDataManager.hxx>
#include <Voting/CardData/card-data-wrapper.h>
#include <Log/ISMTTLog.h>
#include <Log/SMTTLogManager.h>
#include <System/IO/Path.h>
#include <System/IO/Directory.h>
#include <System/IO/File.h>
#include <System/Utils/StringUtils.hxx>
#include <System/Utils/SafeBufferUtils.hxx>
#include <System/Exception/FileSystemException.hxx>
#include <Configuration/s-a-e-s-common-conf-manager.h>
#include <Resources/election-string-resources-manager.h>
#include <Voting/Election/ElectionInstalationLanguages.h>
#include <Voting/set-language-functionality.h>
#include <Operation/Initiator/CheckSAESResourceFiles.hxx>
#include <Operation/Initiator/CheckCODIResourceFiles.hxx>
#include <Operation/Initiator/CheckResourceFiles.hxx>
#include <Voting/Status/VMProcessStatusWrapper.hxx>
#include <Runtime/Environment/EnvironmentBuilder.hxx>
#include <Voting/Vote/TransmissionPackageWrapper.hxx>
#include <Voting/PBReports/PVFormWrapper.hxx>
#include <Runtime/Environment/DirectoryManager.hxx>
#include <algorithm>
#include <Voting/ResultValidation.hxx>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#include <libintl.h>
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

#define N_(String) (String) ///<macro for gettext not translate

using namespace Smartmatic::Log;
using namespace Smartmatic::SAES::Common::Configuration;
using namespace Smartmatic::SAES::Configuration;
using namespace Smartmatic::SAES::Functionality;
using namespace Smartmatic::SAES::GUI::Configuration;
using namespace Smartmatic::SAES::GUI::Functionality;
using namespace Smartmatic::SAES::GUI::Widgets;
using namespace Smartmatic::SAES::Operation::Initiator;
using namespace Smartmatic::SAES::Operation::Voting;
using namespace Smartmatic::SAES::Operation;
using namespace Smartmatic::SAES::Resources;
using namespace Smartmatic::SAES::Runtime::Environment;
using namespace Smartmatic::SAES::Voting::Election;
using namespace Smartmatic::SAES::Voting::Status;
using namespace Smartmatic::SAES::Voting::VotingCenters;
using namespace Smartmatic::SAES::Voting::VotingDevice;
using namespace Smartmatic::SAES::Voting;
using namespace Smartmatic::System::Exception;
using namespace Smartmatic::System::IO;
using namespace Smartmatic::System::Utils;
using namespace Smartmatic::System;

/**
 * @fn static bool sortVotingCenter ( const VotingCenter & v1, const VotingCenter & v2 )
 * @brief method sort voting center by language
 * @param v1 voting center one
 * @param v2 voting center two
 * @return true is v1-code < v2-code
 * @date 12/06/2013
 * @author Yadickson Soto
 */
static bool sortVotingCenter ( const VotingCenter & v1, const VotingCenter & v2 )
{
    std::string value1 = StringUtils::toUpper ( v1.getCode() );
    std::string value2 = StringUtils::toUpper ( v2.getCode() );
    return ( value1.compare ( value2 ) < 0 );
}


ISMTTLog * DataUtilsValidatorFunctionality::logger = SMTTLogManager::GetLogger ( "Smartmatic.SAES.GUI.Functionality.DataUtilsValidatorFunctionality", GETTEXT_PACKAGE );

DataUtilsValidatorFunctionality * DataUtilsValidatorFunctionality::instance = NULL;

DataUtilsValidatorFunctionality::DataUtilsValidatorFunctionality()
{
    dataUtilsWidget = NULL;
    electoralConfiguration = NULL;
    votingExperienceController = NULL;

    m_thread_check_data = NULL;
    m_thread_validate_center = NULL;
    m_thread_export_results = NULL;

    currentDirectories = MachineOperationManager::getInstance()->getSaesInstallDirectories();
    centerCaseSensitive = SAESGUIConfigurationManager::getInstance()->getPMInstallationWidgetConfig().getLoginCaseSensitiveConfiguration();

    directoriesTempValidation = new SaesTempValidatorDirectories ( SAESGUIConfigurationManager::getInstance()->getDataUtilsValidatorFunctionalityConfiguration().getResultFolder() );

    signalStartValidate.connect ( sigc::mem_fun ( *this, &DataUtilsValidatorFunctionality::startValidate ) );
    signalFailValidate.connect ( sigc::mem_fun ( *this, &DataUtilsValidatorFunctionality::failValidate ) );
    signalNextVotingCenter.connect ( sigc::mem_fun ( *this , &DataUtilsValidatorFunctionality::nextVotingCenter ) );
    signalEndFunctionality.connect ( sigc::mem_fun ( *this , &DataUtilsValidatorFunctionality::endFunctionalityProcess ) );
}

DataUtilsValidatorFunctionality::~DataUtilsValidatorFunctionality()
{
    deleteElements();
}

DataUtilsValidatorFunctionality * DataUtilsValidatorFunctionality::getInstance()
{
    if ( !instance )
    {
        instance = new DataUtilsValidatorFunctionality();
    }

    return instance;
}

std::string DataUtilsValidatorFunctionality::GetWindowTitle()
{
    return N_ ( "Smartmatic.SAES.GUI.Functionality.DataUtilsValidatorFunctionality.WindowTitle" );
}

bool DataUtilsValidatorFunctionality::ShowWindowTitle()
{
    return false;
}

Smartmatic::Log::ISMTTLog * DataUtilsValidatorFunctionality::getLogger()
{
    return logger;
}

Glib::ustring DataUtilsValidatorFunctionality::getStartLogMessage()
{
    return N_ ( "Smartmatic.SAES.GUI.Functionality.DataUtilsValidatorFunctionality.LogStartFunctionality" );
}

Glib::ustring DataUtilsValidatorFunctionality::getFinishLogMessage()
{
    return N_ ( "Smartmatic.SAES.GUI.Functionality.DataUtilsValidatorFunctionality.LogCloseFunctionality" );
}

Smartmatic::SAES::Functionality::Configuration::MenuOptionType DataUtilsValidatorFunctionality::GetMenuOptionType()
{
    return MenuOptionType::CONFIGURATION_XMLMDATAUTILSVALIDATORFUNCTIONALITY;
}

Gtk::Widget * DataUtilsValidatorFunctionality::GetFrame()
{
    dataUtilsWidget = AbstractFactoryWidget::createDataUtilsValidatorWidget();
    dataUtilsWidget->showValidateFiles();
    return dataUtilsWidget;
}

void DataUtilsValidatorFunctionality::Dispose()
{
    VMProcessStatusWrapper::getInstance()->stop();
    VMProcessStatusWrapper::getInstance()->persistence();
}

void DataUtilsValidatorFunctionality::Initialize()
{
    electoralConfiguration = new ElectoralConfiguration();
    votingExperienceController = new VotingExperienceController ( electoralConfiguration );

    CheckDataManager::getInstance()->stopDetection();
    MachineOperationManager::getInstance()->setElectoralConfiguration ( electoralConfiguration );
    MachineOperationManager::getInstance()->setVotingExperienceController ( votingExperienceController );

    dataUtilsWidget->showValidateFiles();

    SetLanguageFunctionality::getCurrentLanguageCountry ( currentLanguage, currentCountry );
    VMProcessStatusWrapper::getInstance()->dispose();
    VMProcessStatusWrapper::getInstance()->start();

    if ( !Glib::thread_supported() )
    {
        Glib::thread_init();
    }

    if ( m_thread_check_data != NULL )
    {
        m_thread_check_data->join();
        m_thread_check_data = NULL;
    }

    m_thread_check_data = Glib::Thread::create ( sigc::mem_fun ( *this, &DataUtilsValidatorFunctionality::checkData ), true );
}

void DataUtilsValidatorFunctionality::deleteElements()
{
    if ( dataUtilsWidget )
    {
        delete dataUtilsWidget;
        dataUtilsWidget = NULL;
    }

    if ( electoralConfiguration )
    {
        delete electoralConfiguration;
        electoralConfiguration = NULL;
    }

    if ( votingExperienceController )
    {
        delete votingExperienceController;
        votingExperienceController = NULL;
    }

    if ( m_thread_check_data != NULL )
    {
        m_thread_check_data->join();
        m_thread_check_data = NULL;
    }

    if ( m_thread_validate_center != NULL )
    {
        m_thread_validate_center->join();
        m_thread_validate_center = NULL;
    }

    if ( m_thread_export_results != NULL )
    {
        m_thread_export_results->join();
        m_thread_export_results = NULL;
    }
}

void DataUtilsValidatorFunctionality::block()
{
    logger->Debug ( "Block functionality" );
}

void DataUtilsValidatorFunctionality::partial()
{
    logger->Debug ( "Partial functionality" );
}

void DataUtilsValidatorFunctionality::resume()
{
    logger->Debug ( "Resume functionality" );
}

void DataUtilsValidatorFunctionality::checkData()
{
    bool status = checkSAESResources();

    if ( status )
    {
        signalStartValidate.emit();
    }
    else
    {
        signalFailValidate.emit();
    }
}

void DataUtilsValidatorFunctionality::failValidate()
{
    dataUtilsWidget->showStatusValidateFiles ( false );
}

void DataUtilsValidatorFunctionality::startValidate()
{
    Directory::getInstance()->Delete ( directoriesTempValidation->getResultFolderFullPath() );

    dataUtilsWidget->showProgress();
    keyMap.clear();
    votingCenters.clear();
    std::string votingCentersFilePath = currentDirectories->getVotingCentersInstalationFileFullPath();
    bool loadVotingCenter = VotingCentersWrapper::getInstance()->LoadFile ( votingCentersFilePath );
    bool loadKey = loadKeys();

    if ( loadVotingCenter && loadKey )
    {
        logger->Debug ( "Voting Centers File path is: " + votingCentersFilePath );

        std::list< VotingCenter > & list ( VotingCentersWrapper::getInstance()->getVotingCenters() );
        std::list< VotingCenter >::iterator itVc;
        {
            std::stringstream ss;
            ss << "Number of voting centers: " << list.size();
            logger->Debug ( ss.str() );
        }

        for ( itVc = list.begin(); itVc != list.end(); ++itVc )
        {
            votingCenters.push_back ( ( *itVc ) );
        }

        std::stable_sort ( votingCenters.begin(), votingCenters.end(), sortVotingCenter );

        currentCenter = 0;
        signalNextVotingCenter.emit();
    }
    else if ( !loadVotingCenter )
    {
        dataUtilsWidget->showError ( _ ( "Smartmatic.SAES.GUI.Functionality.DataUtilsValidatorFunctionality.ErrorLoadVotingCenterFile" ) );
        logger->Debug ( "Could not open file: " + votingCentersFilePath );
    }
    else if ( !loadKey )
    {
        dataUtilsWidget->showError ( _ ( "Smartmatic.SAES.GUI.Functionality.DataUtilsValidatorFunctionality.ErrorLoadVotingCenterFile" ) );
        logger->Debug ( "Could not open key file" );
    }
}

void DataUtilsValidatorFunctionality::nextVotingCenter()
{
    if ( currentCenter < votingCenters.size() )
    {
        center = votingCenters[currentCenter];
        VMProcessStatusWrapper::getInstance()->init ( currentLanguage, currentCountry, center.getCode() );

        currentCenter++;

        logger->Debug ( "Process voting center " + center.getCode() );
        dataUtilsWidget->showProgress ( center.getCode(), currentCenter, votingCenters.size() );
        std::string folderName = center.getFolder_name();
        std::string programUnit = ( center.getProgram_unit() != NULL ? center.getProgram_unit().get() : "" );
        currentDirectories->setWorkingPath ( folderName );
        currentDirectories->setProgramUnitName ( programUnit );
        logger->Debug ( "Generate Center: " + folderName );

        directoriesTempValidation->setFolderCenter ( StringUtils::replaceAll ( center.getCode(), " ", "_" ) );

        Directory::getInstance()->Delete ( directoriesTempValidation->getResultFolderWorking() );
        Directory::getInstance()->Delete ( directoriesTempValidation->getBasePath() );

        folderPath = Path::Combine ( currentDirectories->getVotingExperienceValidationFolderFullPath(), folderName );
        VMProcessStatusWrapper::getInstance()->addSuccess ( GETTEXT_PACKAGE, N_ ( "Smartmatic.SAES.GUI.Functionality.DataUtilsValidatorFunctionality.InitializesTheCenterValidation" ) );

        if ( !Glib::thread_supported() )
        {
            Glib::thread_init();
        }

        if ( m_thread_validate_center != NULL )
        {
            m_thread_validate_center->join();
            m_thread_validate_center = NULL;
        }

        m_thread_validate_center = Glib::Thread::create ( sigc::mem_fun ( *this, &DataUtilsValidatorFunctionality::validateCenter ), true );
    }
    else
    {
        Directory::getInstance()->Delete ( directoriesTempValidation->getResultFolderWorking() );
        Directory::getInstance()->Delete ( directoriesTempValidation->getBasePath() );

        dataUtilsWidget->showValidateFiles();

        if ( !Glib::thread_supported() )
        {
            Glib::thread_init();
        }

        if ( m_thread_export_results != NULL )
        {
            m_thread_export_results->join();
            m_thread_export_results = NULL;
        }

        m_thread_export_results = Glib::Thread::create ( sigc::mem_fun ( *this, &DataUtilsValidatorFunctionality::exportResults ), true );
    }
}

void DataUtilsValidatorFunctionality::validateCenter()
{
    std::string currentDirectoryPath = currentDirectories->getBasePath();
    Directory::getInstance()->Delete ( directoriesTempValidation->getResultFolderWorking() );

    if ( Smartmatic::System::IO::Directory::getInstance()->DirectoryExists ( currentDirectoryPath ) )
    {
        logger->Debug ( "Loading Configuration of: " + currentDirectoryPath );

        if ( loadElectoralConfiguration() && validateCenterResources() && loadLanguages() && validateCloseElection() )
        {
            try
            {
                VMProcessStatusWrapper::getInstance()->addSuccess ( GETTEXT_PACKAGE, N_ ( "Smartmatic.SAES.GUI.Functionality.DataUtilsValidatorFunctionality.CreateCards" ) );

                Voting_device vd = MachineOperationManager::getInstance()->getElectoralConfiguration()->getVotingDevice()->getFirstVotingDevice();
                std::list < Custom_attribute_definition > listCAD =
                    MachineOperationManager::getInstance()->getElectoralConfiguration()->getVotingDevice()->getCustomAttributeDefinitionList ( vd, VotingDevice::Machine_type::VOTING_DEVICES_PM );
                std::vector<Custom_attribute > attributes; ///< The attributes

                for ( std::list<VotingDevice::Custom_attribute_definition>::iterator l ( listCAD.begin() ); l != listCAD.end(); ++l )
                {
                    for ( VotingDevice::Custom_attribute_definition_values::Custom_attribute_definition_valueIterator c ( ( *l ).getCustom_attribute_definition_values().getCustom_attribute_definition_value().begin() );
                            c != ( *l ).getCustom_attribute_definition_values().getCustom_attribute_definition_value().end(); ++c )
                    {
                        VotingDevice::Custom_attribute attribute;
                        attribute.setCode ( l->getCode() );
                        attribute.setValue ( ( *c ).getValue() );
                        attributes.push_back ( attribute );

                        try
                        {
                            VMProcessStatusWrapper::getInstance()->addSuccess ( GETTEXT_PACKAGE, N_ ( "Smartmatic.SAES.GUI.Functionality.DataUtilsValidatorFunctionality.CreateCard" ), ( *c ).getValue() );
                            CardData::Card_data cardData;
                            CardData::CardDataWrapper::GenerateCardData ( MachineOperationManager::getInstance()->getElectoralConfiguration(), MachineOperationManager::getInstance()->getOperationStatus(), attributes, cardData, false, Smartmatic::SAES::Voting::CardData::Card_type::CARD_DATA_DEMOSTRATION_VOTE_CARD );

                            if ( !votingExperienceController->init ( cardData ) )
                            {
                                logger->Debug ( "Error validate info card" );
                                VMProcessStatusWrapper::getInstance()->addFail ( GETTEXT_PACKAGE, N_ ( "Smartmatic.SAES.GUI.Functionality.DataUtilsValidatorFunctionality.InitVotingExperienceFail" ) );
                            }
                        }
                        catch ( CryptoException & ex )
                        {
                            VMProcessStatusWrapper::getInstance()->addFail ( ex.getMessage() );
                        }
                        catch ( ElectionException & ex )
                        {
                            VMProcessStatusWrapper::getInstance()->addFail ( ex.getMessage() );
                        }
                        catch ( VotingDeviceException & ex )
                        {
                            VMProcessStatusWrapper::getInstance()->addFail ( ex.getMessage() );
                        }
                        catch ( ... )
                        {
                            VMProcessStatusWrapper::getInstance()->addFail ( _ ( "Smartmatic.SAES.GUI.Functionality.DataUtilsValidatorFunctionality.InitVotingExperienceFail" ) );
                        }

                        attributes.clear();
                        votingExperienceController->dispose();
                    }
                }
            }
            catch ( ElectionException & ex )
            {
                VMProcessStatusWrapper::getInstance()->addFail ( ex.getMessage() );
            }
            catch ( VotingDeviceException & ex )
            {
                VMProcessStatusWrapper::getInstance()->addFail ( ex.getMessage() );
            }
            catch ( ... )
            {
                logger->Debug ( "Error validate center" );
                VMProcessStatusWrapper::getInstance()->addFail ( GETTEXT_PACKAGE, N_ ( "Smartmatic.SAES.GUI.Functionality.DataUtilsValidatorFunctionality.InitVotingExperienceFail" ) );
            }
        }
        else
        {
            logger->Debug ( "FAIL " + currentDirectoryPath );
        }
    }
    else
    {
        VMProcessStatusWrapper::getInstance()->addFail ( GETTEXT_PACKAGE, N_ ( "Smartmatic.SAES.GUI.Functionality.DataUtilsValidatorFunctionality.CenterFolderNotFound" ) );
        logger->Debug ( "Directory no exist: " + currentDirectoryPath );
    }

    signalNextVotingCenter.emit();
}

bool DataUtilsValidatorFunctionality::loadElectoralConfiguration()
{
    bool validate = true;
    std::string password;
    VMProcessStatusWrapper::getInstance()->addSuccess ( GETTEXT_PACKAGE, N_ ( "Smartmatic.SAES.GUI.Functionality.DataUtilsValidatorFunctionality.StartLoadConfiguration" ) );

    try
    {
        electoralConfiguration->dispose();
        votingExperienceController->dispose();

        if ( getPassword ( password ) )
        {
            if ( !password.empty() )
            {
                std::string centerCode = center.getCode();

                VMProcessStatusWrapper::getInstance()->addSuccess ( GETTEXT_PACKAGE, N_ ( "Smartmatic.SAES.GUI.Functionality.DataUtilsValidatorFunctionality.Authentication" ) );
                validate = MachineOperationManager::getInstance()->authentication ( centerCode, password, centerCaseSensitive );
            }

            if ( validate )
            {
                electoralConfiguration->loadAllData();
            }
            else
            {
                VMProcessStatusWrapper::getInstance()->addFail ( GETTEXT_PACKAGE, N_ ( "Smartmatic.SAES.GUI.Functionality.DataUtilsValidatorFunctionality.AuthenticationFail" ) );
            }
        }
        else
        {
            VMProcessStatusWrapper::getInstance()->addFail ( GETTEXT_PACKAGE, N_ ( "Smartmatic.SAES.GUI.Functionality.DataUtilsValidatorFunctionality.PasswordNotFound" ) );
            validate = false;
        }
    }
    catch ( ElectionException & ex )
    {
        VMProcessStatusWrapper::getInstance()->addFail ( ex.getMessage() );
        validate = false;
    }
    catch ( VotingDeviceException & ex )
    {
        VMProcessStatusWrapper::getInstance()->addFail ( ex.getMessage() );
        validate = false;
    }
    catch ( CryptoException & ex )
    {
        VMProcessStatusWrapper::getInstance()->addFail ( ex.getMessage() );
        validate = false;
    }
    catch ( SignedException & ex )
    {
        VMProcessStatusWrapper::getInstance()->addFail ( ex.getMessage() );
        validate = false;
    }
    catch ( XmlException & ex )
    {
        VMProcessStatusWrapper::getInstance()->addFail ( ex.getMessage() );
        validate = false;
    }
    catch ( FileSystemException & ex )
    {
        VMProcessStatusWrapper::getInstance()->addFail ( ex.getMessage() );
        validate = false;
    }
    catch ( ... )
    {
        VMProcessStatusWrapper::getInstance()->addFail ( GETTEXT_PACKAGE, N_ ( "Smartmatic.SAES.GUI.Functionality.DataUtilsValidatorFunctionality.ErrorLoadElectoralConfiguration" ) );
        validate = false;
    }

    if ( !validate )
    {
        logger->Debug ( "Error loading the configuration" );
    }

    return validate;
}

bool DataUtilsValidatorFunctionality::validateCenterResources()
{
    SaesDirectories saesDirectories;

    Directory::getInstance()->Delete ( directoriesTempValidation->getBasePath() );
    Directory::getInstance()->Mkdir ( directoriesTempValidation->getBasePath() );

    EnvironmentBuilder builder;
    builder.install ( &saesDirectories, directoriesTempValidation );
    builder.install ( currentDirectories, directoriesTempValidation );

    CheckResourceFiles resourceFiles ( directoriesTempValidation );
    bool validate = resourceFiles.check() == TestStatus::Passed;

    if ( validate )
    {
        VMProcessStatusWrapper::getInstance()->addSuccess ( GETTEXT_PACKAGE, N_ ( "Smartmatic.SAES.GUI.Functionality.DataUtilsValidatorFunctionality.ValidateCenterResourcesSuccess" ) );
    }
    else
    {
        VMProcessStatusWrapper::getInstance()->addFail ( GETTEXT_PACKAGE, N_ ( "Smartmatic.SAES.GUI.Functionality.DataUtilsValidatorFunctionality.ValidateCenterResourcesFail" ) );
    }

    return validate;
}

bool DataUtilsValidatorFunctionality::loadLanguages()
{
    VMProcessStatusWrapper::getInstance()->addSuccess ( GETTEXT_PACKAGE, N_ ( "Smartmatic.SAES.GUI.Functionality.DataUtilsValidatorFunctionality.LoadLanguages" ) );
    Languages::LanguageSequence & sequence ( electoralConfiguration->getLanguages().getLanguage() );

    if ( sequence.empty() )
    {
        logger->Debug ( "Could not load languages of the configuration" );
        VMProcessStatusWrapper::getInstance()->addFail ( GETTEXT_PACKAGE, N_ ( "Smartmatic.SAES.GUI.Functionality.DataUtilsValidatorFunctionality.LanguageNotFound" ) );
        return false;
    }
    else
    {
        Languages::LanguageIterator it;
        ElectionInstalationLanguages::getInstance()->clearLanguage();

        for ( it = sequence.begin(); it != sequence.end(); ++it )
        {
            ElectionInstalationLanguages::getInstance()->addInstallationLanguages ( ( *it ) );
        }
    }

    return true;
}

bool DataUtilsValidatorFunctionality::validateCloseElection()
{
    Directory::getInstance()->Delete ( directoriesTempValidation->getResultFolderWorking() );
    Directory::getInstance()->Mkdir ( directoriesTempValidation->getDocsFolderFullPath () );
    Directory::getInstance()->Mkdir ( directoriesTempValidation->getEmlFolderFullPath () );

    ResultValidation::getInstance()->initialize();
    VMProcessStatusWrapper::getInstance()->addSuccess ( GETTEXT_PACKAGE, N_ ( "Smartmatic.SAES.GUI.Functionality.CloseElectionFunctionality.StartCloseElection" ) );

    try
    {
        MachineOperationManager::getInstance()->initializeClass ( directoriesTempValidation, false, false );
        MachineOperationManager::getInstance()->loadVoteFiles ( false );
    }
    catch ( ... )
    {
        VMProcessStatusWrapper::getInstance()->addFail ( GETTEXT_PACKAGE, N_ ( "Smartmatic.SAES.GUI.Functionality.CloseElectionFunctionality.InitializeClassFail" ) );
        return false;
    }

    if ( MachineOperationManager::getInstance()->persistAllResult() )
    {
        VMProcessStatusWrapper::getInstance()->addSuccess ( GETTEXT_PACKAGE, N_ ( "Smartmatic.SAES.GUI.Functionality.CloseElectionFunctionality.PersistPlainCodiFilesSuccessfully" ) );
    }
    else
    {
        VMProcessStatusWrapper::getInstance()->addFail ( GETTEXT_PACKAGE, N_ ( "Smartmatic.SAES.GUI.Functionality.CloseElectionFunctionality.PersistPlainCodiFilesFail" ) );
        return false;
    }

    try
    {
        MachineOperationManager::getInstance()->persistVotingResultsFile();
        VMProcessStatusWrapper::getInstance()->addSuccess ( GETTEXT_PACKAGE, N_ ( "Persist voting results file successfully" ) );
    }
    catch ( ... )
    {
        VMProcessStatusWrapper::getInstance()->addFail ( GETTEXT_PACKAGE, N_ ( "Persist voting results file fail" ) );
        return false;
    }

    if ( SAESGUIConfigurationManager::getInstance()->getDataUtilsValidatorFunctionalityConfiguration().getExportResults() )
    {
        std::string origDir = directoriesTempValidation->getResultFolderWorking();
        std::string destDir = directoriesTempValidation->getResultFolderFullPath();
        DirectoryManager::copyDir ( origDir, destDir, false );
    }

    return true;
}

bool DataUtilsValidatorFunctionality::checkSAESResources()
{
    CheckSAESResourceFiles resourceFiles;
    CheckCODIResourceFiles checkCODIResourceFiles;
    return resourceFiles.check() == TestStatus::Passed && checkCODIResourceFiles.check() == TestStatus::Passed;
}

bool DataUtilsValidatorFunctionality::loadKeys()
{
    bool validate = false;

    if ( SAESCommonConfigurationManager::getInstance()->getSecurityConfiguration().getSecretKeyEncrypted() )
    {
        std::string file = SAESGUIConfigurationManager::getInstance()->getDataUtilsValidatorFunctionalityConfiguration().getKeyFile();
        std::string separator = SAESGUIConfigurationManager::getInstance()->getDataUtilsValidatorFunctionalityConfiguration().getSeparator();
        unsigned int positionCode = SAESGUIConfigurationManager::getInstance()->getDataUtilsValidatorFunctionalityConfiguration().getPositionCode();
        unsigned int positionPassword = SAESGUIConfigurationManager::getInstance()->getDataUtilsValidatorFunctionalityConfiguration().getPositionPassword();
        std::string dom = currentDirectories->getDOMMemoryBasePath();
        std::string filePath = Path::Combine ( dom, file );

        SafeBuffer buffer;

        if ( File::getInstance ()->openFileToBuffer ( filePath, buffer ) )
        {
            validate = true;
            SafeBufferUtils util ( buffer );
            std::vector < SafeBuffer > result = util.splitByEndLine ();
            std::vector < SafeBuffer >::iterator it;

            for ( it = result.begin ();
                    it != result.end ();
                    ++it )
            {
                std::string contents ( ( const char * ) ( *it ).GetData(), ( *it ).GetSize() );
                std::vector <std::string> vector;
                StringUtils::splitToVector ( contents, separator, vector );

                if ( vector.size() >= positionCode && vector.size() >= positionPassword )
                {
                    std::string code = StringUtils::trim ( vector[positionCode] );
                    std::string pass = StringUtils::trim ( vector[positionPassword] );
                    keyMap[code] = pass;
                }
            }
        }
        else
        {
            logger->Debug ( "Error load file " + filePath );
        }
    }
    else
    {
        validate = true;
    }

    return validate;
}

bool DataUtilsValidatorFunctionality::getPassword ( std::string & password )
{
    bool validate = true;

    if ( SAESCommonConfigurationManager::getInstance()->getSecurityConfiguration().getSecretKeyEncrypted() )
    {
        if ( keyMap.find ( center.getCode() ) != keyMap.end() )
        {
            password = keyMap[center.getCode()];
        }
        else
        {
            logger->Debug ( "Not password for " + center.getCode() );
            validate = false;
        }
    }

    return validate;
}

bool DataUtilsValidatorFunctionality::ForceExecute()
{
    return true;
}

void DataUtilsValidatorFunctionality::endFunctionalityProcess ()
{
    ExitFunctionality();
}

void DataUtilsValidatorFunctionality::exportResults()
{
    sleep ( 2 );

    if ( SAESGUIConfigurationManager::getInstance()->getDataUtilsValidatorFunctionalityConfiguration().getExportResults() )
    {
        std::string destDir = directoriesTempValidation->getResultFolderFullPath();
        Directory::getInstance()->addAllPermission ( destDir );
    }

    signalEndFunctionality.emit();
}
