/**
 * @file	alarm-box-diagnosis-functionality.h
 * @brief	Declares the alarm box diagnosis functionality class.
 * @author	Youssef.tourkmani
 * @date	01/11/2012
 */


#ifndef _ALARM_BOX_DIAGNOSIS_FUNCTIONALITY_H_
#define _ALARM_BOX_DIAGNOSIS_FUNCTIONALITY_H_

#include <gtkmm.h>
#include <string>
#include <Gui/Widgets/Menu/menu-option.h>
#include <Functionalities/graphic-display-functionality.h>
#include <Widgets/smartmatic-button.h>
#include <Gui/Widgets/Diagnose/AlarmBoxDiagnoseWidget.hxx>
#include <Log/ISMTTLog.h>

namespace Smartmatic
{
    namespace SAES
    {
        namespace GUI
        {
            namespace Functionality
            {
                /**
                 * @class	AlarmBoxDiagnosisFunctionality
                 * @brief	Alarm box diagnosis functionality.
                 * @author	Youssef.tourkmani
                 * @date	01/11/2012
                 */
                class AlarmBoxDiagnosisFunctionality: public GraphicDisplayFunctionality
                {

                    public:

                        /**
                         * @fn	bool AlarmBoxDiagnosisFunctionality::DontAllowHelp();
                         * @brief	Gets the dont allow help.
                         * @author	Luiggi.mendez
                         * @date	28/04/2011
                         * @return	true if it succeeds, false if it fails.
                         */

                        bool DontAllowHelp();

                        /**
                         * @fn	std::vector<Smartmatic::GUI::Widgets::KeysActionsCode> AlarmBoxDiagnosisFunctionality::getHelpSeqKey();
                         * @brief	Gets the help sequence key.
                         * @author	Luiggi.mendez
                         * @date	28/04/2011
                         * @return	The help sequence key.
                         */

                        std::vector<Smartmatic::GUI::Widgets::KeysActionsCode> getHelpSeqKey();

                        /**
                         * @fn	bool AlarmBoxDiagnosisFunctionality::DontAllowExit();
                         * @brief	Gets the dont allow exit.
                         * @author	Luiggi.mendez
                         * @date	28/04/2011
                         * @return	true if it succeeds, false if it fails.
                         */

                        bool DontAllowExit();

                        /**
                         * @fn	std::vector<Smartmatic::GUI::Widgets::KeysActionsCode> AlarmBoxDiagnosisFunctionality::getSeqKey();
                         * @brief	Gets the sequence key.
                         * @author	Luiggi.mendez
                         * @date	28/04/2011
                         * @return	The sequence key.
                         */

                        std::vector<Smartmatic::GUI::Widgets::KeysActionsCode> getSeqKey();

                        /**
                         * @fn	static AlarmBoxDiagnosisFunctionality * AlarmBoxDiagnosisFunctionality::getInstance();
                         * @brief	Gets the current.
                         * @author	Luiggi.mendez
                         * @date	28/04/2011
                         * @return	null if it fails, else.
                         */

                        static AlarmBoxDiagnosisFunctionality * getInstance();

                    protected:

                        /**
                         * @fn	Smartmatic::Log::ISMTTLog * AlarmBoxDiagnosisFunctionality::getLogger();
                         * @brief	Gets the logger.
                         * @author	Luiggi.mendez
                         * @date	28/04/2011
                         * @return	null if it fails, else the logger.
                         */

                        Smartmatic::Log::ISMTTLog * getLogger();

                        /**
                         * @fn	Glib::ustring AlarmBoxDiagnosisFunctionality::getStartLogMessage();
                         * @brief	Gets the start log message.
                         * @author	Luiggi.mendez
                         * @date	28/04/2011
                         * @return	The start log message.
                         */

                        Glib::ustring getStartLogMessage();

                        /**
                         * @fn	Glib::ustring AlarmBoxDiagnosisFunctionality::getFinishLogMessage();
                         * @brief	Gets the finish log message.
                         * @author	Luiggi.mendez
                         * @date	28/04/2011
                         * @return	The finish log message.
                         */

                        Glib::ustring getFinishLogMessage();

                    private:

                        Smartmatic::SAES::GUI::Widgets::AlarmBoxDiagnoseWidget * alarmWidget;  ///< The alarm widget
                        sigc::connection connFinishDiagnosis;	///< The connection finish diagnosis
                        static AlarmBoxDiagnosisFunctionality * current_instance;	///< The current instance

                        /**
                         * @fn	AlarmBoxDiagnosisFunctionality::AlarmBoxDiagnosisFunctionality();
                         * @brief	Default constructor.
                         * @author	Luiggi.mendez
                         * @date	28/04/2011
                         */

                        AlarmBoxDiagnosisFunctionality();

                        /**
                         * @fn	virtual AlarmBoxDiagnosisFunctionality::~AlarmBoxDiagnosisFunctionality();
                         * @brief	Finaliser.
                         * @author	Luiggi.mendez
                         * @date	28/04/2011
                         */

                        virtual ~AlarmBoxDiagnosisFunctionality();

                        /**
                         * @fn	virtual void AlarmBoxDiagnosisFunctionality::deleteElements();
                         * @brief	Deletes all elements.
                         * @author	Yadickson Soto
                         * @date	03/04/2012
                         */
                        virtual void deleteElements();

                        /**
                         * @fn	void AlarmBoxDiagnosisFunctionality::Dispose();
                         * @brief	Dispose of this object, cleaning up any resources it uses.
                         * @author	Luiggi.mendez
                         * @date	28/04/2011
                         */

                        void Dispose();

                        /**
                         * @fn	void AlarmBoxDiagnosisFunctionality::Initialize();
                         * @brief	Initializes this object.
                         * @author	Luiggi.mendez
                         * @date	28/04/2011
                         */

                        void Initialize();

                        /**
                         * @fn	Gtk::Widget * AlarmBoxDiagnosisFunctionality::GetFrame();
                         * @brief	Gets the frame.
                         * @author	Luiggi.mendez
                         * @date	28/04/2011
                         * @return	null if it fails, else the frame.
                         */

                        Gtk::Widget * GetFrame();

                        /**
                         * @fn	std::string AlarmBoxDiagnosisFunctionality::GetWindowTitle();
                         * @brief	Gets the window title.
                         * @author	Luiggi.mendez
                         * @date	28/04/2011
                         * @return	The window title.
                         */

                        std::string GetWindowTitle();

                        /**
                         * @fn	Smartmatic::SAES::Functionality::Configuration::MenuOptionType AlarmBoxDiagnosisFunctionality::GetMenuOptionType();
                         * @brief	Gets the menu option type.
                         * @author	Luiggi.mendez
                         * @date	28/04/2011
                         * @return	The menu option type.
                         */

                        Smartmatic::SAES::Functionality::Configuration::MenuOptionType GetMenuOptionType();
                        /**
                         * @fn	void AlarmBoxDiagnosisFunctionality::endDiagnosis();
                         * @brief	method end diagnosis.
                         * @author	Yadickson Soto
                         * @date	18/09/2012
                         */
                        void endDiagnosis();

                        static Smartmatic::Log::ISMTTLog * logger; /**<logger*/ ///< The logger
                };
            }
        }
    }
}


#endif // _ALARM_BOX_DIAGNOSIS_FUNCTIONALITY_H_
