/**
 * @file	all-diagnosis-functionality.cc
 * @brief	all diagnosis functionality class.
 * @author	Youssef.tourkmani
 * @date	01/11/2012
 */


#include <Voting/Status/voting-machine-status-schema.hxx>
#include "Functionalities/all-diagnosis-functionality.h"
#include "Configuration/s-a-e-s-g-u-i-conf-manager.h"
#include <Operation/MachineOperationManager.hxx>
#include <Log/SMTTLogManager.h>
#include <Operation/Devices/Polling/BCRPolling.hxx>
#include <Gui/Widgets/AbstractFactoryWidget.hxx>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#include <libintl.h>
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

#define N_(String) (String) ///<macro for gettext not translate

using namespace Smartmatic::Functionality;
using namespace Smartmatic::Environment;
using namespace Smartmatic::SAES::GUI::Widgets;
using namespace Smartmatic::SAES::GUI::Configuration;
using namespace Smartmatic::SAES::GUI::Functionality;
using namespace Smartmatic::SAES::Functionality;
using namespace Smartmatic::SAES::Voting;
using namespace Smartmatic::SAES::Operation;
using namespace Smartmatic::GUI::Windows;
using namespace Smartmatic::GUI::Widgets;
using namespace Smartmatic::Log;
using namespace Smartmatic::SAES::Configuration;
using namespace Smartmatic::SAES::Operation::Devices;

ISMTTLog * AllDiagnosisFunctionality::logger = SMTTLogManager::GetLogger ( "Smartmatic.SAES.GUI.Functionality.AllDiagnosisFunctionality", GETTEXT_PACKAGE );

AllDiagnosisFunctionality * AllDiagnosisFunctionality::current_instance = NULL;

AllDiagnosisFunctionality * AllDiagnosisFunctionality::getInstance()
{
    if ( !current_instance )
    {
        current_instance = new AllDiagnosisFunctionality();
    }

    return current_instance;
}

AllDiagnosisFunctionality::AllDiagnosisFunctionality()
{
    alarmWidget = NULL;
    bcreaderWidget = NULL;
    screaderWidget = NULL;
    printerWidget = NULL;
    calibrationWidget = NULL;
    flashWidget = NULL;
    ableWidget = NULL;
    resultWidget = NULL;
    framePrincipal = NULL;
    printResultWidget = NULL;
    startPrint = false;
}

AllDiagnosisFunctionality::~AllDiagnosisFunctionality()
{
    deleteElements();
}

void AllDiagnosisFunctionality::deleteElements()
{
    if ( alarmWidget != NULL )
    {
        delete ( alarmWidget );
        alarmWidget = NULL;
    }

    if ( bcreaderWidget != NULL )
    {
        delete ( bcreaderWidget );
        bcreaderWidget = NULL;
    }

    if ( screaderWidget != NULL )
    {
        delete ( screaderWidget );
        screaderWidget = NULL;
    }

    if ( printerWidget != NULL )
    {
        delete ( printerWidget );
        printerWidget = NULL;
    }

    if ( calibrationWidget != NULL )
    {
        delete ( calibrationWidget );
        calibrationWidget = NULL;
    }

    if ( flashWidget != NULL )
    {
        delete ( flashWidget );
        flashWidget = NULL;
    }

    if ( ableWidget != NULL )
    {
        delete ( ableWidget );
        ableWidget = NULL;
    }

    if ( framePrincipal != NULL )
    {
        delete ( framePrincipal );
        framePrincipal = NULL;
    }

    deleteResultElement();
}

void AllDiagnosisFunctionality::deleteResultElement()
{
    if ( resultWidget )
    {
        delete resultWidget;
        resultWidget = NULL;
    }

    if ( printResultWidget )
    {
        delete printResultWidget;
        printResultWidget = NULL;
    }
}

void AllDiagnosisFunctionality::Initialize()
{
    MachineOperationManager::getInstance()->getOperationStatus()->getVotingMachineStatusWrapper()->clearDeviceStatusList();
    BCRPolling::getInstance()->pause();

    connShowResults.disconnect();
    connEnd.disconnect();

    connShowResults = signalDispatcherShowResults.connect ( sigc::mem_fun ( *this , &AllDiagnosisFunctionality::showResults ) );
    connEnd = signalDispatcherEnd.connect ( sigc::mem_fun ( *this , &AllDiagnosisFunctionality::endAllDiagnostic ) );

    index = 0;
    std::vector<Smartmatic::SAES::Configuration::DiagnosticType> listDiagnosis = SAESGUIConfigurationManager::getInstance()->getAllDiagnosisList();
    listOfDiagnosis.clear();

    for ( unsigned int i = 0; i < listDiagnosis.size(); i++ )
    {
        switch ( listDiagnosis[i].getDevice() )
        {
            case Smartmatic::SAES::Common::Configuration::DeviceType::CONFIGURATION_ALARMBOX:
                alarmWidget = new Smartmatic::SAES::GUI::Widgets::AlarmBoxDiagnoseWidget ();
                listOfDiagnosis.push_back ( alarmWidget );
                break;

            case Smartmatic::SAES::Common::Configuration::DeviceType::CONFIGURATION_BARCODE:
                bcreaderWidget = new Smartmatic::SAES::GUI::Widgets::BarcodeReaderDiagnoseWidget ();
                listOfDiagnosis.push_back ( bcreaderWidget );
                break;

            case Smartmatic::SAES::Common::Configuration::DeviceType::CONFIGURATION_TOUCHSCREENCALIBRATION:
                calibrationWidget = new Smartmatic::SAES::GUI::Widgets::CalibrationDiagnosisWidget ();
                listOfDiagnosis.push_back ( calibrationWidget );
                break;

            case Smartmatic::SAES::Common::Configuration::DeviceType::CONFIGURATION_PRINTER:
                printerWidget = new Smartmatic::SAES::GUI::Widgets::PrinterDiagnosisWidget ();
                listOfDiagnosis.push_back ( printerWidget );
                break;

            case Smartmatic::SAES::Common::Configuration::DeviceType::CONFIGURATION_SMARTCARD:
                screaderWidget = new Smartmatic::SAES::GUI::Widgets::SmartcardDiagnosisWidget ();
                listOfDiagnosis.push_back ( screaderWidget );
                break;

            case Smartmatic::SAES::Common::Configuration::DeviceType::CONFIGURATION_FLASH:
                flashWidget = new Smartmatic::SAES::GUI::Widgets::FlashDiagnosisWidget ();
                listOfDiagnosis.push_back ( flashWidget );
                break;

            case Smartmatic::SAES::Common::Configuration::DeviceType::CONFIGURATION_P168A:
            case Smartmatic::SAES::Common::Configuration::DeviceType::CONFIGURATION_ABLED:
                ableWidget = new Smartmatic::SAES::GUI::Widgets::AbleDiagnosticWidget ();
                listOfDiagnosis.push_back ( ableWidget );
                break;

            default:
            case Smartmatic::SAES::Common::Configuration::DeviceType::CONFIGURATION_EURN:
            case Smartmatic::SAES::Common::Configuration::DeviceType::CONFIGURATION_MOUSE:
            case Smartmatic::SAES::Common::Configuration::DeviceType::CONFIGURATION_KEYBOARD:
            case Smartmatic::SAES::Common::Configuration::DeviceType::CONFIGURATION_USBSTORAGEMOUNTED:
            case Smartmatic::SAES::Common::Configuration::DeviceType::CONFIGURATION_ETHERNET:
            case Smartmatic::SAES::Common::Configuration::DeviceType::CONFIGURATION_UNKNOWN:
                break;
        }
    }

    if ( alarmWidget )
    {
        connAlarmDiag = alarmWidget->getSignalEnd().connect ( sigc::mem_fun ( this, &AllDiagnosisFunctionality::on_AlarmDiagFinished ) );
    }

    if ( bcreaderWidget )
    {
        connBCDiag = bcreaderWidget->getSignalEnd().connect ( sigc::mem_fun ( this, &AllDiagnosisFunctionality::on_BCDiagFinished ) );
    }

    if ( screaderWidget )
    {
        connSCDiag = screaderWidget->getSignalEnd().connect ( sigc::mem_fun ( this, &AllDiagnosisFunctionality::on_SCDiagFinished ) );
    }

    if ( printerWidget )
    {
        connPrinterDiag = printerWidget->getSignalEnd().connect ( sigc::mem_fun ( this, &AllDiagnosisFunctionality::on_PrinterDiagFinished ) );
    }

    if ( calibrationWidget )
    {
        connCalibDiag = calibrationWidget->getSignalEnd().connect ( sigc::mem_fun ( this, &AllDiagnosisFunctionality::on_CalibrationDiagFinished ) );
    }

    if ( flashWidget )
    {
        connFlashDiag = flashWidget->getSignalEnd().connect ( sigc::mem_fun ( this, &AllDiagnosisFunctionality::on_FlashDiagFinished ) );
    }

    if ( ableWidget )
    {
        connAbleDiag = ableWidget->getSignalEnd().connect ( sigc::mem_fun ( this, &AllDiagnosisFunctionality::on_AbleDiagFinished ) );
    }

    listOfDiagnosis[index]->initialize();
    framePrincipal->set_shadow_type ( Gtk::SHADOW_NONE );
    framePrincipal->remove();
    framePrincipal->set_visible ( true );
    framePrincipal->add ( * ( listOfDiagnosis[index] ) );
    framePrincipal->show_all_children();
}

void AllDiagnosisFunctionality::printResults()
{
    deleteResultElement();
    printResultWidget = AbstractFactoryWidget::createDeviceDiagnosticPrintWidget();
    printResultWidget->getSignalEnd().connect ( sigc::mem_fun ( this, &AllDiagnosisFunctionality::endPrintResults ) );

    framePrincipal->remove();
    framePrincipal->add ( *printResultWidget );
    framePrincipal->show_all_children();
    printResultWidget->show_all_children();
    printResultWidget->start();
}

void AllDiagnosisFunctionality::endPrintResults()
{
    signalDispatcherShowResults.emit();
}

void AllDiagnosisFunctionality::showResults()
{
    int labelWidth = SAESGUIConfigurationManager::getInstance()->getAllDiagnosisConfiguration().getDiagnosticReviewWidth();
    int labelHeight = SAESGUIConfigurationManager::getInstance()->getAllDiagnosisConfiguration().getDiagnosticReviewHeight();
    resultWidget = new DiagnosticResultWidget ( false, labelWidth, labelHeight );
    framePrincipal->remove();
    framePrincipal->add ( *resultWidget );
    framePrincipal->show_all_children();
    resultWidget->show_all_children();
    resultWidget->refresh();
    MachineOperationManager::getInstance()->getOperationStatus()->persistDiagnosticFile(false);
    logger->Audit ( N_ ( "Smartmatic.SAES.GUI.Functionality.AllDiagnosisFunctionality.AllDiagnosticFinished" ) );
    signalDispatcherEnd.emit();
}

void AllDiagnosisFunctionality::endAllDiagnostic()
{
    enableExitButton();
    ExitFunctionality();
}

bool AllDiagnosisFunctionality::RefreshCalib ( int time_num )
{
    calibrationWidget->dispose();
    endDiagnostic();
    return false;
}

bool AllDiagnosisFunctionality::RefreshAlarm ( int time_num )
{
    alarmWidget->dispose();
    endDiagnostic();
    return false;
}

bool AllDiagnosisFunctionality::RefreshBC ( int time_num )
{
    bcreaderWidget->dispose();
    endDiagnostic();
    return false;
}
bool AllDiagnosisFunctionality::RefreshPrinter ( int time_num )
{
    SetPrinterWidget();
    return false;
}

void AllDiagnosisFunctionality::SetPrinterWidget()
{
    printerWidget->dispose();
    endDiagnostic();
}

bool AllDiagnosisFunctionality::RefreshSC ( int time_num )
{
    screaderWidget->dispose();
    endDiagnostic();
    return false;
}

bool AllDiagnosisFunctionality::RefreshFlash ( int time_num )
{
    flashWidget->Dispose();
    endDiagnostic();
    return false;
}

bool AllDiagnosisFunctionality::RefreshAble ( int time_num )
{
    ableWidget->Dispose();
    endDiagnostic();
    return false;
}

void AllDiagnosisFunctionality::on_AlarmDiagFinished()
{
    sigc::slot<bool> my_slot1 = sigc::bind ( sigc::mem_fun ( *this,
                                &AllDiagnosisFunctionality::RefreshAlarm ), 0 );
    connRefreshAlarm = Glib::signal_timeout().connect ( my_slot1, SAESGUIConfigurationManager::getInstance()->getAllDiagnosisAlarmWaitTime() );
}

void AllDiagnosisFunctionality::on_BCDiagFinished()
{
    sigc::slot<bool> my_slot2 = sigc::bind ( sigc::mem_fun ( *this,
                                &AllDiagnosisFunctionality::RefreshBC ), 0 );
    connRefreshBC = Glib::signal_timeout().connect ( my_slot2, SAESGUIConfigurationManager::getInstance()->getAllDiagnosisBarcodeWaitTime() );
    logger->Audit ( N_ ( "Smartmatic.SAES.GUI.Functionality.AllDiagnosisFunctionality.BarCodeDiagnosisFinished" ) );
}

void AllDiagnosisFunctionality::on_SCDiagFinished()
{
    sigc::slot<bool> my_slot3 = sigc::bind ( sigc::mem_fun ( *this,
                                &AllDiagnosisFunctionality::RefreshSC ), 0 );
    connRefreshSC = Glib::signal_timeout().connect ( my_slot3, SAESGUIConfigurationManager::getInstance()->getAllDiagnosisSmartcardWaitTime() );
    logger->Audit ( N_ ( "Smartmatic.SAES.GUI.Functionality.AllDiagnosisFunctionality.SmartcardReaderDiagnosisFinished" ) );
}

void AllDiagnosisFunctionality::on_PrinterDiagFinished()
{
    sigc::slot<bool> my_slot4 = sigc::bind ( sigc::mem_fun ( *this,
                                &AllDiagnosisFunctionality::RefreshPrinter ), 0 );
    connRefreshPrinter = Glib::signal_timeout().connect ( my_slot4, SAESGUIConfigurationManager::getInstance()->getAllDiagnosisPrinterWaitTime() );
    logger->Audit ( N_ ( "Smartmatic.SAES.GUI.Functionality.AllDiagnosisFunctionality.PrinterFinished" ) );
}

void AllDiagnosisFunctionality::on_CalibrationDiagFinished()
{
    sigc::slot<bool> my_slot7 = sigc::bind ( sigc::mem_fun ( *this,
                                &AllDiagnosisFunctionality::RefreshCalib ), 0 );
    connRefreshCalib = Glib::signal_timeout().connect ( my_slot7, SAESGUIConfigurationManager::getInstance()->getAllDiagnosisCalibrationWaitTime() );
    logger->Audit ( N_ ( "Smartmatic.SAES.GUI.Functionality.AllDiagnosisFunctionality.CalibrationDiagnosisFinished" ) );
}

void AllDiagnosisFunctionality::on_FlashDiagFinished()
{
    sigc::slot<bool> my_slot8 = sigc::bind ( sigc::mem_fun ( *this,
                                &AllDiagnosisFunctionality::RefreshFlash ), 0 );
    connRefreshFlash = Glib::signal_timeout().connect ( my_slot8, SAESGUIConfigurationManager::getInstance()->getAllDiagnosisFlashWaitTime() );
    logger->Audit ( N_ ( "Smartmatic.SAES.GUI.Functionality.AllDiagnosisFunctionality.FlashDiagnosisFinished" ) );
}

void AllDiagnosisFunctionality::on_AbleDiagFinished()
{
    sigc::slot<bool> my_slot8 = sigc::bind ( sigc::mem_fun ( *this,
                                &AllDiagnosisFunctionality::RefreshAble ), 0 );
    connRefreshAble = Glib::signal_timeout().connect ( my_slot8, SAESGUIConfigurationManager::getInstance()->getAllDiagnosisAbleWaitTime() );
    logger->Audit ( N_ ( "Smartmatic.SAES.GUI.Functionality.AllDiagnosisFunctionality.AbleDiagnosisFinished" ) );
}

Gtk::Widget * AllDiagnosisFunctionality::GetFrame()
{
    startPrint = false;
    framePrincipal = new Gtk::Frame();
    framePrincipal->set_shadow_type ( Gtk::SHADOW_NONE );
    return framePrincipal;
}

std::string AllDiagnosisFunctionality::GetWindowTitle()
{
    return N_ ( "Smartmatic.SAES.GUI.Functionality.AllDiagnosisFunctionality.AllDiagnosis" );
}

Smartmatic::SAES::Functionality::Configuration::MenuOptionType AllDiagnosisFunctionality::GetMenuOptionType()
{
    return MenuOptionType::CONFIGURATION_XMLMALLDIAGNOSISFUNCTIONALITY;
}

void AllDiagnosisFunctionality::onBeforeExit()
{
    BCRPolling::getInstance()->resume();
}

void AllDiagnosisFunctionality::Dispose()
{
    connShowResults.disconnect();
    connEnd.disconnect();

    startPrint = false;
    connRefreshAlarm.disconnect();
    connRefreshPrinter.disconnect();
    connRefreshPrinterOnOk.disconnect();
    connRefreshPrinterOnCancel.disconnect();
    connRefreshCalib.disconnect();
    connRefreshBC.disconnect();
    connRefreshSC.disconnect();
    connRefreshFlash.disconnect();
    connRefreshAble.disconnect();
    connAlarmDiag.disconnect();
    connBCDiag.disconnect();
    connSCDiag.disconnect();
    connCalibDiag.disconnect();
    connPrinterDiag.disconnect();
    connPrinterDiagOnOk.disconnect();
    connPrinterDiagOnCancel.disconnect();
    connFlashDiag.disconnect();
    connAbleDiag.disconnect();
}

Smartmatic::Log::ISMTTLog * AllDiagnosisFunctionality::getLogger()
{
    return logger;
}
Glib::ustring AllDiagnosisFunctionality::getStartLogMessage()
{
    return N_ ( "Smartmatic.SAES.GUI.Functionality.AllDiagnosisFunctionality.LogStartFunctionality" );
}

Glib::ustring AllDiagnosisFunctionality::getFinishLogMessage()
{
    return N_ ( "Smartmatic.SAES.GUI.Functionality.AllDiagnosisFunctionality.LogCloseFunctionality" );
}

void AllDiagnosisFunctionality::endDiagnostic()
{
    framePrincipal->remove();

    if ( index < listOfDiagnosis.size() - 1 )
    {
        index++;
        listOfDiagnosis[index]->initialize();
        framePrincipal->add ( ( *listOfDiagnosis[index] ) );
        framePrincipal->show_all_children();
    }
    else
    {
        printResults();
    }
}
