/**
 * @file	close-votation-functionality.h
 * @brief	Declares the close votation functionality class.
 * @author	Youssef.tourkmani
 * @date	01/11/2012
 */

#ifndef _CLOSE_VOTATION_FUNCTIONALITY_H_

#define _CLOSE_VOTATION_FUNCTIONALITY_H_

#include <gtkmm.h>
#include <string>
#include <Functionalities/graphic-display-functionality.h>
#include <Widgets/smartmatic-button.h>
#include <Log/ISMTTLog.h>

namespace Smartmatic
{
    namespace SAES
    {
        namespace GUI
        {
            namespace Functionality
            {
                /**
                 * @class	CloseVotationFunctionality
                 * @brief	Close votation functionality.
                 * @author	Youssef.tourkmani
                 * @date	01/11/2012
                 */
                class CloseVotationFunctionality: public GraphicDisplayFunctionality
                {

                    public:

                        /**
                         * @fn	static CloseVotationFunctionality * CloseVotationFunctionality::getInstance();
                         * @brief	Gets the current.
                         * @author	Luiggi.mendez
                         * @date	28/04/2011
                         * @return	null if it fails, else.
                         */

                        static CloseVotationFunctionality * getInstance();
                        /**
                         * @fn	virtual void CloseVotationFunctionality::deleteElements();
                         * @brief	Deletes all elements.
                         * @author	Yadickson Soto
                         * @date	03/04/2012
                         */
                        virtual void deleteElements();

                    protected:
                        Smartmatic::GUI::Widgets::SmartmaticButton * closeVotationButton;	///< The close votation button

                        /**
                         * @fn	void CloseVotationFunctionality::on_closeVotationButton_clicked();
                         * @brief	Handles close votation button clicked signals.
                         * @author	Luiggi.mendez
                         * @date	28/04/2011
                         */

                        void on_closeVotationButton_clicked();

                        /**
                         * @fn	Smartmatic::Log::ISMTTLog * CloseVotationFunctionality::getLogger();
                         * @brief	Gets the logger.
                         * @author	Luiggi.mendez
                         * @date	28/04/2011
                         * @return	null if it fails, else the logger.
                         */

                        Smartmatic::Log::ISMTTLog * getLogger();

                        /**
                         * @fn	Glib::ustring CloseVotationFunctionality::getStartLogMessage();
                         * @brief	Gets the start log message.
                         * @author	Luiggi.mendez
                         * @date	28/04/2011
                         * @return	The start log message.
                         */

                        Glib::ustring getStartLogMessage();

                        /**
                         * @fn	Glib::ustring CloseVotationFunctionality::getFinishLogMessage();
                         * @brief	Gets the finish log message.
                         * @author	Luiggi.mendez
                         * @date	28/04/2011
                         * @return	The finish log message.
                         */

                        Glib::ustring getFinishLogMessage();

                        /**
                         * @fn	void CloseVotationFunctionality::on_confirmButton_clicked();
                         * @brief	Handles confirm button clicked signals.
                         *
                         * @author	Juan.Delgado
                         * @date	29/06/2011
                         */
                        void on_confirmButton_clicked();

                        /**
                         * @fn	void CloseVotationFunctionality::on_cancelButton_clicked();
                         * @brief	Handles cancel button clicked signal.
                         * @author	Juan.Delgado
                         * @date	29/06/2011
                         */
                        void on_cancelButton_clicked();

                    private:

                        Gtk::Frame * externalFrame; ///< The external frame
                        Gtk::Label * titleLabel;	///< The title label
                        Gtk::Alignment * align; ///< The alignment
                        Smartmatic::GUI::Widgets::SmartmaticButton * confirmButton; ///< The confirm button
                        Smartmatic::GUI::Widgets::SmartmaticButton * cancelButton;	///< The cancel button
                        Gtk::HButtonBox * buttonBox;	///< The button box
                        Gtk::Label * labelConfirmation; ///< The label confirmation
                        Gtk::Label * titleLabelID;	///< Identifier for the title label

                        static CloseVotationFunctionality * current_instance;	///< The current instance

                        Gtk::VBox * vbox; ///< The vbox

                        /**
                         * @fn	CloseVotationFunctionality::CloseVotationFunctionality();
                         * @brief	Default constructor.
                         * @author	Luiggi.mendez
                         * @date	28/04/2011
                         */

                        CloseVotationFunctionality();

                        /**
                         * @fn	virtual CloseVotationFunctionality::~CloseVotationFunctionality();
                         * @brief	Destructor.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         */
                        virtual ~CloseVotationFunctionality();

                        /**
                         * @fn	void CloseVotationFunctionality::Dispose();
                         * @brief	Dispose of this object, cleaning up any resources it uses.
                         * @author	Luiggi.mendez
                         * @date	28/04/2011
                         */

                        void Dispose();

                        /**
                         * @fn	void CloseVotationFunctionality::Initialize();
                         * @brief	Initializes this object.
                         * @author	Luiggi.mendez
                         * @date	28/04/2011
                         */

                        void Initialize();

                        /**
                         * @fn	Gtk::Widget * CloseVotationFunctionality::GetFrame();
                         * @brief	Gets the frame.
                         * @author	Luiggi.mendez
                         * @date	28/04/2011
                         * @return	null if it fails, else the frame.
                         */

                        Gtk::Widget * GetFrame();

                        /**
                         * @fn	std::string CloseVotationFunctionality::GetWindowTitle();
                         * @brief	Gets the window title.
                         * @author	Luiggi.mendez
                         * @date	28/04/2011
                         * @return	The window title.
                         */

                        std::string GetWindowTitle();

                        /**
                         * @fn	Smartmatic::SAES::Functionality::Configuration::MenuOptionType CloseVotationFunctionality::GetMenuOptionType();
                         * @brief	Gets the menu option type.
                         * @author	Luiggi.mendez
                         * @date	28/04/2011
                         * @return	The menu option type.
                         */

                        Smartmatic::SAES::Functionality::Configuration::MenuOptionType GetMenuOptionType();

                        /**
                         * @fn	void CloseVotationFunctionality::closeVotingExperience();
                         * @brief	Execute background process for close votation functionality.
                         * @author	Gerardo.Hernandez
                         * @date	10/06/2011
                         */
                        void closeVotingExperience();

                        /**
                         * @fn	void CloseVotationFunctionality::LoadAfterConfirmation();
                         * @brief	Load frame widget after confirmation.
                         * @author	Juan.Delgado
                         * @date	29/06/2011
                         */
                        void LoadAfterConfirmation();

                        static Smartmatic::Log::ISMTTLog * logger;  ///< The logger

                };
            }
        }
    }
}


#endif // _CLOSE_VOTATION_FUNCTIONALITY_H_
