/**
 * @file	flash-diagnosis-functionality.h
 * @brief	Declares the flash diagnosis functionality class.
 * @author	Youssef.tourkmani
 * @date	01/11/2012
 */


#ifndef _FLASH_DIAGNOSIS_FUNCTIONALITY_H_
#define _FLASH_DIAGNOSIS_FUNCTIONALITY_H_

#include <gtkmm.h>
#include <string>
#include <Functionalities/graphic-display-functionality.h>
#include <Gui/Widgets/Diagnose/FlashDiagnosisWidget.hxx>
#include <Log/ISMTTLog.h>

namespace Smartmatic
{
    namespace SAES
    {
        namespace GUI
        {
            namespace Functionality
            {
                /**
                 * @class	FlashDiagnosisFunctionality
                 * @brief	Flash diagnosis functionality.
                 * @author	Youssef.tourkmani
                 * @date	01/11/2012
                 */
                class FlashDiagnosisFunctionality: public GraphicDisplayFunctionality
                {

                    public:

                        /**
                         * @fn	bool FlashDiagnosisFunctionality::DontAllowHelp();
                         * @brief	Gets the dont allow help.
                         * @author	Luiggi.mendez
                         * @date	28/04/2011
                         * @return	true if it succeeds, false if it fails.
                         */

                        bool DontAllowHelp();

                        /**
                         * @fn	std::vector<Smartmatic::GUI::Widgets::KeysActionsCode> FlashDiagnosisFunctionality::getHelpSeqKey();
                         * @brief	Gets the help sequence key.
                         * @author	Luiggi.mendez
                         * @date	28/04/2011
                         * @return	The help sequence key.
                         */

                        std::vector<Smartmatic::GUI::Widgets::KeysActionsCode> getHelpSeqKey();

                        /**
                         * @fn	bool FlashDiagnosisFunctionality::DontAllowExit();
                         * @brief	Gets the dont allow exit.
                         * @author	Luiggi.mendez
                         * @date	28/04/2011
                         * @return	true if it succeeds, false if it fails.
                         */

                        bool DontAllowExit();

                        /**
                         * @fn	std::vector<Smartmatic::GUI::Widgets::KeysActionsCode> FlashDiagnosisFunctionality::getSeqKey();
                         * @brief	Gets the sequence key.
                         * @author	Luiggi.mendez
                         * @date	28/04/2011
                         * @return	The sequence key.
                         */

                        std::vector<Smartmatic::GUI::Widgets::KeysActionsCode> getSeqKey();

                        /**
                         * @fn	static FlashDiagnosisFunctionality * FlashDiagnosisFunctionality::getInstance();
                         * @brief	Gets the current.
                         * @author	Luiggi.mendez
                         * @date	28/04/2011
                         * @return	null if it fails, else.
                         */

                        static FlashDiagnosisFunctionality * getInstance();

                    protected:

                        /**
                         * @fn	Smartmatic::Log::ISMTTLog * FlashDiagnosisFunctionality::getLogger();
                         * @brief	Gets the logger.
                         * @author	Luiggi.mendez
                         * @date	28/04/2011
                         * @return	null if it fails, else the logger.
                         */

                        Smartmatic::Log::ISMTTLog * getLogger();

                        /**
                         * @fn	Glib::ustring FlashDiagnosisFunctionality::getStartLogMessage();
                         * @brief	Gets the start log message.
                         * @author	Luiggi.mendez
                         * @date	28/04/2011
                         * @return	The start log message.
                         */

                        Glib::ustring getStartLogMessage();

                        /**
                         * @fn	Glib::ustring FlashDiagnosisFunctionality::getFinishLogMessage();
                         * @brief	Gets the finish log message.
                         * @author	Luiggi.mendez
                         * @date	28/04/2011
                         * @return	The finish log message.
                         */

                        Glib::ustring getFinishLogMessage();

                        /**
                         * @fn	virtual void FlashDiagnosisFunctionality::deleteElements();
                         * @brief	virtual Deletes all elements.
                         * @author	Yadickson Soto
                         * @date	03/04/2012
                         */
                        virtual void deleteElements();

                    private:

                        Smartmatic::SAES::GUI::Widgets::FlashDiagnosisWidget * flashWidget; ///< The flash widget
                        static FlashDiagnosisFunctionality * current_instance;  ///< The current instance

                        /**
                         * @fn	FlashDiagnosisFunctionality::FlashDiagnosisFunctionality();
                         * @brief	Default constructor.
                         * @author	Luiggi.mendez
                         * @date	28/04/2011
                         */

                        FlashDiagnosisFunctionality();

                        /**
                         * @fn	virtual FlashDiagnosisFunctionality::~FlashDiagnosisFunctionality();
                         * @brief	Finaliser.
                         * @author	Luiggi.mendez
                         * @date	28/04/2011
                         */

                        virtual ~FlashDiagnosisFunctionality();

                        /**
                         * @fn	Smartmatic::SAES::Functionality::Configuration::MenuOptionType FlashDiagnosisFunctionality::GetMenuOptionType();
                         * @brief	Gets the menu option type.
                         * @author	Luiggi.mendez
                         * @date	28/04/2011
                         * @return	The menu option type.
                         */

                        Smartmatic::SAES::Functionality::Configuration::MenuOptionType GetMenuOptionType();

                        /**
                         * @fn	void FlashDiagnosisFunctionality::Dispose();
                         * @brief	Dispose of this object, cleaning up any resources it uses.
                         * @author	Luiggi.mendez
                         * @date	28/04/2011
                         */

                        void Dispose();

                        /**
                         * @fn	void FlashDiagnosisFunctionality::Initialize();
                         * @brief	Initializes this object.
                         * @author	Luiggi.mendez
                         * @date	28/04/2011
                         */

                        void Initialize();

                        /**
                         * @fn	Gtk::Widget * FlashDiagnosisFunctionality::GetFrame();
                         * @brief	Gets the frame.
                         * @author	Luiggi.mendez
                         * @date	28/04/2011
                         * @return	null if it fails, else the frame.
                         */

                        Gtk::Widget * GetFrame();

                        /**
                         * @fn	std::string FlashDiagnosisFunctionality::GetWindowTitle();
                         * @brief	Gets the window title.
                         * @author	Luiggi.mendez
                         * @date	28/04/2011
                         * @return	The window title.
                         */

                        std::string GetWindowTitle();

                        sigc::connection connFlashDiag; ///< connection flash diagnosis

                        /**
                         * @fn	void FlashDiagnosisFunctionality::on_FlashDiagFinished();
                         * @brief	Handles flash diagram finished signals.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         */
                        void on_FlashDiagFinished();

                        static Smartmatic::Log::ISMTTLog * logger; ///< The logger
                };
            }
        }
    }
}


#endif // _FLASH_DIAGNOSIS_FUNCTIONALITY_H_
