/**
* @file	graphic-display-functionality.cc
* @brief	Declares the graphic display functionality class.
* @author	Youssef.tourkmani
* @date	01/11/2012
*/

#include "graphic-display-functionality.h"
#include <Runtime/Environment/MemoryPathController.h>
#include <Operation/WindowsManager.hxx>
#include <Configuration/s-a-e-s-functionality-conf-manager.h>
#include <Operation/FunctionalityWindowManager.hxx>
#include <Operation/Monitor/DeviceTypeMonitorPool.hxx>

#define N_(String) (String) ///<macro for gettext not translate

using namespace Smartmatic::SAES::GUI::Functionality;
using namespace Smartmatic::SAES::Common::Operation::Monitor;
using namespace Smartmatic::SAES::Common::Configuration;
using namespace Smartmatic::SAES::Operation::Monitor;

GraphicDisplayFunctionality::GraphicDisplayFunctionality()
    : Smartmatic::SAES::Functionality::SAESFunctionality<Smartmatic::SAES::Functionality::Configuration::MenuOptionType>()
{
    exitWidget.setInfo ( N_ ( "Smartmatic.SAES.GUI.Functionality.GraphicDisplayFunctionality.DefaultExitMessage" ) );
    frame = NULL;
    helpWidget = NULL;
    baseWindow = NULL;
    enableBackgroundProcess = true;
    flagEndNotify = false;
    flagExecute = false;
    forceToLoad = Smartmatic::SAES::GUI::Configuration::SAESGUIConfigurationManager::getInstance()->GetForceToLoadFrame();
}

GraphicDisplayFunctionality::~GraphicDisplayFunctionality()
{
}


void GraphicDisplayFunctionality::Execute()
{
    flagExecute = true;
    getLogger()->Audit ( getStartLogMessage() );
    BeginExecute();

    if ( enableBackgroundProcess )
    {
        Glib::signal_timeout().connect_once ( sigc::mem_fun ( *this, &GraphicDisplayFunctionality::ExcuteBackgroundProcess ), 1500 );
    }
}

void GraphicDisplayFunctionality::ExcuteBackgroundProcess()
{
}


void GraphicDisplayFunctionality::EnableBackgroundProcess()
{
    enableBackgroundProcess = true;
}


void GraphicDisplayFunctionality::PreLoad()
{
    enableBackgroundProcess = false;
    deleteElements();
    frame = GetFrame();
    this->loaded = true;
}


void GraphicDisplayFunctionality::DisplayUnableMessage()
{
    baseWindow = ( Smartmatic::SAES::GUI::Windows::EmptyWindow * ) Smartmatic::GUI::Operation::WindowsManager::getInstance()->current();

    if ( baseWindow )
    {
        int delay = Smartmatic::SAES::GUI::Configuration::SAESGUIConfigurationManager::getInstance()->GetUnableMessageCloseTime();

        initWidget.setTitle ( false );
        initWidget.setHelp ( false );
        initWidget.setExit ( false );
        initWidget.setTimeout ( delay );

        baseWindow->showInitMessageWidget ( initWidget );
    }
}


void GraphicDisplayFunctionality::ForceShowHelp()
{
}

bool GraphicDisplayFunctionality::CanExecute()
{
    bool canExecute = true;

    helpConfiguration = Smartmatic::SAES::GUI::Configuration::SAESGUIConfigurationManager::getInstance()->getHelpConfiguration ( GetMenuOptionType() );

    if ( this->ForceExecute() || Smartmatic::SAES::GUI::Configuration::SAESGUIConfigurationManager::getInstance()->getForceExecuteFunctionality() )
    {
        return canExecute;
    }

    Smartmatic::SAES::Voting::OperationStatus * operationStatus =
        Smartmatic::SAES::Operation::MachineOperationManager::getInstance()->getOperationStatus();

    if ( operationStatus == NULL )
    {
        getLogger()->Debug ( "Operation status is null" );
        return canExecute;
    }

    if ( !checkMemoryPath() )
    {
        canExecute = false;
    }
    else if ( checkOperationStatus ( operationStatus ) )
    {
        canExecute = checkNecessaryDevices();
    }
    else
    {
        canExecute = false;
    }

    return canExecute;
}


bool GraphicDisplayFunctionality::checkMemoryPath()
{
    bool result = Smartmatic::SAES::Runtime::Environment::MemoryPathController::getInstance()->isAllDevices();

    if ( !result )
    {
        initWidget.setInfo ( N_ ( "Smartmatic.SAES.GUI.Functionality.GraphicDisplayFunctionality.ErrorMemoryPath" ) );
    }

    return result;
}


bool GraphicDisplayFunctionality::ForceExecute()
{
    return true;
}


bool GraphicDisplayFunctionality::checkOperationStatus ( Smartmatic::SAES::Voting::OperationStatus * operationStatus )
{
    bool canExecute = true;
    getLogger()->Debug ( "Start check operation status" );
    Smartmatic::SAES::Voting::Status::ProcessStatus actualStatus = operationStatus->getElectionState();
    Smartmatic::SAES::Configuration::ElectionBlockStatusList::ElectionStatusSequence seq =
        Smartmatic::SAES::GUI::Configuration::SAESGUIConfigurationManager::getInstance()->getDisableStateList ( GetMenuOptionType() );
    Smartmatic::SAES::Configuration::ElectionBlockStatusList::ElectionStatusIterator iter;

    for ( iter = seq.begin(); iter != seq.end(); iter++ )
    {
        Smartmatic::SAES::Voting::Status::ProcessStatus checkStatus = ( *iter );

        if ( actualStatus.compare ( checkStatus ) == 0 )
        {
            switch ( actualStatus )
            {
                default:
                    break;

                case Smartmatic::SAES::Voting::Status::ProcessStatus::ELECTION_STATUS_NOTINITIATED:
                    initWidget.setInfo ( N_ ( "Smartmatic.SAES.GUI.Functionality.GraphicDisplayFunctionality.ElectionStatusNotInitiated" ) );
                    break;

                case Smartmatic::SAES::Voting::Status::ProcessStatus::ELECTION_STATUS_INITIATED:
                    initWidget.setInfo ( N_ ( "Smartmatic.SAES.GUI.Functionality.GraphicDisplayFunctionality.ElectionStatusInitiated" ) );
                    break;

                case Smartmatic::SAES::Voting::Status::ProcessStatus::ELECTION_STATUS_CLOSING:
                    initWidget.setInfo ( N_ ( "Start of election closing" ) );
                    break;

                case Smartmatic::SAES::Voting::Status::ProcessStatus::ELECTION_STATUS_CLOSED:
                    initWidget.setInfo ( N_ ( "Smartmatic.SAES.GUI.Functionality.GraphicDisplayFunctionality.ElectionStatusClosed" ) );
                    break;

                case Smartmatic::SAES::Voting::Status::ProcessStatus::ELECTION_STATUS_RESULTSTRANSMITTED:
                    initWidget.setInfo ( N_ ( "Smartmatic.SAES.GUI.Functionality.GraphicDisplayFunctionality.ElectionStatusResultsTransmitted" ) );
                    break;

                case Smartmatic::SAES::Voting::Status::ProcessStatus::ELECTION_STATUS_NOTINSTALLED:
                    initWidget.setInfo ( N_ ( "Smartmatic.SAES.GUI.Functionality.GraphicDisplayFunctionality.ElectionStatusNotInstalled" ) );
                    break;

                case Smartmatic::SAES::Voting::Status::ProcessStatus::ELECTION_STATUS_NOTDEMOSTRATIONVOTE:
                    initWidget.setInfo ( N_ ( "Smartmatic.SAES.GUI.Functionality.GraphicDisplayFunctionality.ElectionStatusNotDemostrationVote" ) );
                    break;
            }

            canExecute = false;
            break;
        }
        else
        {
            initWidget.setInfo ( " " );
        }
    }

    canExecute = canExecute && Enable();
    getLogger()->Debug ( "End check operation status " + std::string ( canExecute ? "success" : "fail" ) );
    return canExecute;
}


bool GraphicDisplayFunctionality::checkNecessaryDevices()
{
    bool canExecute = true;
    getLogger()->Debug ( "Start check necessary devices" );
    Smartmatic::SAES::Configuration::NecessaryDevicesConfiguration::DeviceConfigurationSequence seq =
        Smartmatic::SAES::GUI::Configuration::SAESGUIConfigurationManager::getInstance()->getDeviceConfiguration ( GetMenuOptionType() );
    Smartmatic::SAES::Configuration::NecessaryDevicesConfiguration::DeviceConfigurationIterator iter;

    for ( iter = seq.begin(); iter != seq.end(); iter++ )
    {
        DeviceTypeMonitor * deviceTypeMonitor = DeviceTypeMonitorPool::getInstance()->getDeviceTypeMonitor ( *iter );

        canExecute = false;

        switch ( *iter )
        {
            case DeviceType::CONFIGURATION_ALARMBOX:
            case DeviceType::CONFIGURATION_TOUCHSCREENCALIBRATION:
            case DeviceType::CONFIGURATION_FLASH:
            case DeviceType::CONFIGURATION_P168A:
            case DeviceType::CONFIGURATION_MOUSE:
            case DeviceType::CONFIGURATION_KEYBOARD:
            case DeviceType::CONFIGURATION_USBSTORAGEMOUNTED:
            case DeviceType::CONFIGURATION_ETHERNET:
            case DeviceType::CONFIGURATION_EURN:
            case DeviceType::CONFIGURATION_ABLED:
            case DeviceType::CONFIGURATION_UNKNOWN:
            default:
                break;

            case DeviceType::CONFIGURATION_BARCODE:
                canExecute = checkBarcode ( deviceTypeMonitor );
                break;

            case DeviceType::CONFIGURATION_SMARTCARD:
                canExecute = checkSmartcard ( deviceTypeMonitor );
                break;

            case DeviceType::CONFIGURATION_PRINTER:
                canExecute = checkPrinter ( deviceTypeMonitor );
                break;
        }

        if ( !canExecute )
        {
            break;
        }
    }

    getLogger()->Debug ( "End check necessary devices " + std::string ( canExecute ? "success" : "fail" ) );
    return canExecute;
}


bool GraphicDisplayFunctionality::checkBarcode ( DeviceTypeMonitor * deviceTypeMonitor )
{
    bool canExecute = true;

    if ( deviceTypeMonitor )
    {
        canExecute = deviceTypeMonitor->getNumberOfDevices() != 0;
    }

    if ( !canExecute )
    {
        initWidget.setInfo ( N_ ( "Smartmatic.SAES.GUI.Functionality.GraphicDisplayFunctionality.BarcodeNotFound" ) );
        getLogger()->Audit ( N_ ( "Smartmatic.SAES.GUI.Functionality.GraphicDisplayFunctionality.LoggerAuditBarcodeNotFound" ) );
    }

    return canExecute;
}


bool GraphicDisplayFunctionality::checkSmartcard ( DeviceTypeMonitor * deviceTypeMonitor )
{
    bool canExecute = true;

    if ( deviceTypeMonitor )
    {
        canExecute = deviceTypeMonitor->getNumberOfDevices() != 0;
    }

    if ( !canExecute )
    {
        initWidget.setInfo ( N_ ( "Smartmatic.SAES.GUI.Functionality.GraphicDisplayFunctionality.SmartcardNotFound" ) );
        getLogger()->Audit ( N_ ( "Smartmatic.SAES.GUI.Functionality.GraphicDisplayFunctionality.LoggerAuditSmartcardNotFound" ) );
    }

    return canExecute;
}


bool GraphicDisplayFunctionality::checkPrinter ( DeviceTypeMonitor * deviceTypeMonitor )
{
    bool canExecute = true;

    if ( deviceTypeMonitor )
    {
        canExecute = deviceTypeMonitor->getNumberOfDevices() != 0;
    }
    else
    {
        Smartmatic::SAES::Printing::PrinterStatusNotifier printNotifier;
        Smartmatic::SAES::Printing::PrinterStatusNotifier::PhysicalStateEnumType printerState = printNotifier.getPrinterState();

        if ( printerState != Smartmatic::SAES::Printing::PrinterStatusNotifier::PSTATE_READY )
        {
            canExecute = false;
        }
    }

    if ( !canExecute )
    {
        initWidget.setInfo ( N_ ( "Smartmatic.SAES.GUI.Functionality.GraphicDisplayFunctionality.PrinterError" ) );
        getLogger()->Audit ( N_ ( "Smartmatic.SAES.GUI.Functionality.GraphicDisplayFunctionality.LoggerAuditPrinterError" ) );
    }

    return canExecute;
}


void GraphicDisplayFunctionality::ForceExit()
{
}

std::string GraphicDisplayFunctionality::getDescriptionMessage()
{
    return N_ ( "Smartmatic.SAES.GUI.Functionality.GraphicDisplayFunctionality.DescriptionMessage" );
}


void GraphicDisplayFunctionality::Exit()
{

}

bool GraphicDisplayFunctionality::ShowFinalMessage()
{
    return Smartmatic::SAES::GUI::Configuration::SAESGUIConfigurationManager::getInstance()->GetShowExitMessage ( GetMenuOptionType() );
}


void GraphicDisplayFunctionality::setExitButtonSensitive ( bool sensitive )
{
    stepWidget.setExitSensitive ( sensitive );

    if ( sensitive )
    {
        baseWindow->enableExitButton();
    }
    else
    {
        baseWindow->disableExitButton();
    }
}


void GraphicDisplayFunctionality::setHelpButtonSensitive ( bool sensitive )
{
    stepWidget.setHelpSensitive ( sensitive );

    if ( sensitive )
    {
        baseWindow->enableHelpButton();
    }
    else
    {
        baseWindow->disableHelpButton();
    }
}


void GraphicDisplayFunctionality::enableExitButton()
{
    setExitButtonSensitive ( true );
}


void GraphicDisplayFunctionality::disableExitButton()
{
    setExitButtonSensitive ( false );
}


void GraphicDisplayFunctionality::enableHelpButton()
{
    setHelpButtonSensitive ( true );
}


void GraphicDisplayFunctionality::disableHelpButton()
{
    setHelpButtonSensitive ( false );
}


void GraphicDisplayFunctionality::onBeforeExit()
{
}


void GraphicDisplayFunctionality::onExit()
{
    ExecuteDispose();
    Exit();
    getLogger()->Audit ( getFinishLogMessage() );

    flagExecute = false;
    endNotify();
}


Smartmatic::GUI::Widgets::StepWidget * GraphicDisplayFunctionality::getHelpWidget()
{
    if ( !helpWidget )
    {
        helpWidget = new Smartmatic::SAES::GUI::Widgets::HelpWidget();
    }

    return helpWidget;
}


void GraphicDisplayFunctionality::endNotify()
{
    if ( isRunning() )
    {
        enableExitButton();
    }

    if ( flagEndNotify && Smartmatic::SAES::GUI::Operation::FunctionalityWindowManager::getInstance()->getCount() == 1 )
    {
        getLogger()->Debug ( "End notify functionality" );
        flagEndNotify = false;
        this->ExecutionFinishedSigc.emit();
    }
}

void GraphicDisplayFunctionality::ExitFunctionality()
{
    Glib::signal_timeout().connect_once ( sigc::mem_fun ( *this, &GraphicDisplayFunctionality::onExitFunctionality ), 0 );
}

void GraphicDisplayFunctionality::onExitFunctionality ()
{
    RefreshInfo();
    baseWindow->ShowExit();
}

void GraphicDisplayFunctionality::OnHelpButtonClicked()
{

}

void GraphicDisplayFunctionality::OnHelpButtonClickedConnect()
{

}

void GraphicDisplayFunctionality::ExecuteDispose()
{
    Dispose();

    if ( this->forceToLoad == true )
    {
        getLogger()->Debug ( "Execute Delete Element" );

        if ( helpWidget )
        {
            delete helpWidget;
            helpWidget = NULL;
        }

        deleteElements();
    }
}


bool GraphicDisplayFunctionality::Enable()
{
    return true;
}


bool GraphicDisplayFunctionality:: isRunning()
{
    return flagExecute;
}


void GraphicDisplayFunctionality::BeginExecute()
{
    if ( this->loaded == false || this->forceToLoad == true )
    {
        bool showExit = true;
        baseWindow = ( Smartmatic::SAES::GUI::Windows::EmptyWindow * ) Smartmatic::GUI::Operation::WindowsManager::getInstance()->current();

        if ( !baseWindow )
        {
            baseWindow = new Smartmatic::SAES::GUI::Windows::EmptyWindow ( NULL, " ", false, false, false );
            Smartmatic::GUI::Operation::WindowsManager::getInstance()->push ( baseWindow );
            showExit = false;
            baseWindow->show();
        }

        deleteElements();
        frame = GetFrame();
        bool enableExit = Smartmatic::SAES::GUI::Operation::FunctionalityWindowManager::getInstance()->getCount() > 0;
        bool enableHelp = ( this->ShowHelp() || helpConfiguration.getUseHelp() ) && enableExit;
        stepWidget.setWidget ( frame );
        stepWidget.setTitleText ( this->GetWindowTitle() );
        stepWidget.setTitle ( this->ShowWindowTitle() );
        stepWidget.setExit ( showExit && enableExit );
        stepWidget.setHelp ( enableHelp );
        startHelpPrintConn.disconnect();
        stopHelpPrintConn.disconnect();
        shownHelpConn.disconnect();
        closedHelpConn.disconnect();

        if ( this->ShowHelp() )
        {
            Smartmatic::SAES::GUI::Widgets::HelpWidget * widget = ( Smartmatic::SAES::GUI::Widgets::HelpWidget * ) getHelpWidget();
            stepWidget.setHelpWidget ( widget );
            startHelpPrintConn = widget->getSignalStartPrint().connect ( sigc::mem_fun ( *this, &GraphicDisplayFunctionality::forceDisableExitButton ) );
            stopHelpPrintConn = widget->getSignalEndPrint().connect ( sigc::mem_fun ( *this, &GraphicDisplayFunctionality::enableExitButton ) );
            widget->setStateListener ( this );
        }
        else if ( enableHelp )
        {
            bool enablePrinting = true;

            if ( helpConfiguration.getEnableFilePrinting().present() )
            {
                enablePrinting = helpConfiguration.getEnableFilePrinting().get();
            }

            getHelpWidget();
            helpWidget->setInfo ( helpConfiguration.getTitleKey(), helpConfiguration.getImageHelp(), helpConfiguration.getPdfHelp(), enablePrinting );
            stepWidget.setHelpWidget ( helpWidget );
            startHelpPrintConn = helpWidget->getSignalStartPrint().connect ( sigc::mem_fun ( *this, &GraphicDisplayFunctionality::forceDisableExitButton ) );
            stopHelpPrintConn = helpWidget->getSignalEndPrint().connect ( sigc::mem_fun ( *this, &GraphicDisplayFunctionality::enableExitButton ) );
            shownHelpConn = helpWidget->getSignalShown().connect ( sigc::mem_fun ( *this, &GraphicDisplayFunctionality::onHelpShown ) );
            closedHelpConn = helpWidget->getSignalClosed().connect ( sigc::mem_fun ( *this, &GraphicDisplayFunctionality::onHelpClosed ) );
            helpWidget->setStateListener ( this );
        }
        else
        {
            stepWidget.setHelpWidget ( NULL );
        }

        if ( ShowFinalMessage() && enableExit )
        {
            int delay = Smartmatic::SAES::GUI::Configuration::SAESGUIConfigurationManager::getInstance()->GetExitFunctionalityMessageCloseTime();
            exitWidget.setTitleText ( this->GetWindowTitle() );
            exitWidget.setTitle ( stepWidget.isTitle() );
            exitWidget.setHelp ( helpConfiguration.getUseHelp() && enableExit );
            exitWidget.setExit ( stepWidget.isExit() && enableExit );
            exitWidget.setHelpSensitive ( false );
            exitWidget.setExitSensitive ( false );
            exitWidget.setTimeout ( delay );
            stepWidget.setExitWidget ( &exitWidget );
        }
        else
        {
            stepWidget.setExitWidget ( NULL );
        }

        stepWidget.setRefreshListener ( this );
        stepWidget.setStateListener ( this );
        baseWindow->pushStepWidget ( &stepWidget );
        baseWindow->setKeywordNavigation ( this );

        Smartmatic::SAES::GUI::Operation::FunctionalityWindowManager::getInstance()->push ( this );
        this->loaded = true;
    }
    else
    {
    }

    flagEndNotify = true;
    disableExitButton();
    Initialize();
}

void GraphicDisplayFunctionality::RefreshInfo()
{
    if ( baseWindow )
    {
        baseWindow->refreshTitleText();
    }
}

void GraphicDisplayFunctionality::refresh()
{
    baseWindow->setKeywordNavigation ( this );
}

void GraphicDisplayFunctionality::block()
{

}

void GraphicDisplayFunctionality::partial()
{

}

void GraphicDisplayFunctionality::resume()
{

}

void GraphicDisplayFunctionality::onEmitSetKey()
{
    onSetKey ( currentKey );
}

void GraphicDisplayFunctionality::onSetKey ( Smartmatic::GUI::Widgets::KeysActionsCode value )
{

}

void GraphicDisplayFunctionality::SetKey ( Smartmatic::GUI::Widgets::KeysActionsCode value )
{
    currentKey = value;
    Glib::signal_timeout().connect_once ( sigc::mem_fun ( *this, &GraphicDisplayFunctionality::onEmitSetKey ), 0 );
}

void GraphicDisplayFunctionality::forceDisableExitButton ()
{
    baseWindow->forceDisableExitButton();
}

void GraphicDisplayFunctionality::onHelpShown()
{

}

void GraphicDisplayFunctionality::onHelpClosed()
{

}
