/**
 * @file	graphic-display-functionality.h
 * @brief	Declares the graphic display functionality class.
 * @author	Youssef.tourkmani
 * @date	01/11/2012
 */


#ifndef _GRAPHIC_DISPLAY_FUNCTIONALITY_H_
#define _GRAPHIC_DISPLAY_FUNCTIONALITY_H_

#include <gtkmm.h>
#include <string>
#include "Functionalities/saes-functionality.h"
#include "Gui/Windows/empty-window.h"
#include <Configuration/menu-option-type-schema.hxx>
#include "Configuration/s-a-e-s-g-u-i-conf-manager.h"
#include <Printing/PrinterStatusNotifier.h>
#include <Widgets/StepWidget.hxx>
#include <Widgets/RefreshListener.hxx>
#include <Widgets/StateListener.hxx>
#include <Windows/ExitWindowListener.hxx>
#include <Windows/HelpWindowListener.hxx>
#include <Gui/Widgets/HelpWidget.hxx>
#include <Gui/Widgets/InfoStepWidget.hxx>
#include <Widgets/keyword-navigation-interface.h>
#include <Operation/Monitor/DeviceTypeMonitor.hxx>

namespace Smartmatic
{
    namespace SAES
    {
        namespace GUI
        {
            namespace Functionality
            {

                /**
                 * @class	GraphicDisplayFunctionality
                 * @brief	Graphic display functionality.
                 * @author	Youssef.tourkmani
                 * @date	05/11/2012
                 * @tparam	TMenuOptionDescriptor	Type of the menu option descriptor.
                 */
                class GraphicDisplayFunctionality: public Smartmatic::SAES::Functionality::SAESFunctionality<Smartmatic::SAES::Functionality::Configuration::MenuOptionType>,
                    public Smartmatic::GUI::Widgets::RefreshListener,
                    public Smartmatic::GUI::Widgets::StateListener,
                    public Smartmatic::GUI::Windows::ExitWindowListener,
                    public Smartmatic::GUI::Windows::HelpWindowListener,
                    protected Smartmatic::GUI::Widgets::keywordNavigationInterface
                {


                    public:

                        /**
                         * @fn	GraphicDisplayFunctionality();
                         * @brief	Default constructor.
                         * @author	Luiggi.mendez
                         * @date	28/04/2011
                         */

                        GraphicDisplayFunctionality();

                        /**
                         * @fn	~GraphicDisplayFunctionality();
                         * @brief	Finaliser.
                         * @author	Luiggi.mendez
                         * @date	28/04/2011
                         */

                        virtual ~GraphicDisplayFunctionality();

                        /**
                         * @fn	void GraphicDisplayFunctionality::BeginExecute();
                         * @brief	Begins an execute.
                         * @author	Luiggi.mendez
                         * @date	28/04/2011
                         */

                        void BeginExecute();

                        /**
                         * @fn	virtual void GraphicDisplayFunctionality::ExcuteBackgroundProcess();
                         * @brief	Execute a Background process functionality.
                         * @author	Gerardo.Hernandez
                         * @date	10/06/2011
                         */
                        virtual void ExcuteBackgroundProcess();

                        /**
                         * @fn	virtual TMenuOptionDescriptor GraphicDisplayFunctionality::GetMenuOptionType() = 0;
                         * @brief	Gets the menu option type.
                         * @author	Rafael.gasperi
                         * @date	28/04/2011
                         * @return	The menu option type.
                         */

                        virtual Smartmatic::SAES::Functionality::Configuration::MenuOptionType GetMenuOptionType() = 0;

                        /**
                         * @fn	void GraphicDisplayFunctionality::Execute();
                         * @brief	Executes this object.
                         * @author	Luiggi.mendez
                         * @date	28/04/2011
                         */

                        void Execute();

                        /**
                         * @fn	void GraphicDisplayFunctionality::PreLoad();
                         * @brief	Pre load.
                         * @author	Luiggi.mendez
                         * @date	28/04/2011
                         */

                        void PreLoad();

                        //Pure Virtual Function

                        /**
                         * @fn	virtual Gtk::Widget * GraphicDisplayFunctionality::GetFrame() = 0;
                         * @brief	Gets the frame.
                         * @author	Luiggi.mendez
                         * @date	28/04/2011
                         * @return	null if it fails, else the frame.
                         */

                        virtual  Gtk::Widget * GetFrame() = 0;

                        /**
                         * @fn	virtual void GraphicDisplayFunctionality::Dispose() = 0;
                         * @brief	Dispose of this object, cleaning up any resources it uses.
                         * @author	Luiggi.mendez
                         * @date	28/04/2011
                         */

                        virtual void Dispose() = 0;

                        /**
                         * @fn	virtual void GraphicDisplayFunctionality::Initialize() = 0;
                         * @brief	Initializes this object.
                         * @author	Luiggi.mendez
                         * @date	28/04/2011
                         */

                        virtual void Initialize() = 0;

                        /**
                         * @fn	void GraphicDisplayFunctionality::DisplayUnableMessage();
                         * @brief	Displays a unable message.
                         * @author	Luiggi.mendez
                         * @date	28/04/2011
                         */

                        void DisplayUnableMessage();

                        /**
                         * @fn	void ForceShowHelp();
                         * @brief	Force show help.
                         * @author	Luiggi.mendez
                         * @date	28/04/2011
                         */

                        void ForceShowHelp();

                        /**
                         * @fn	virtual bool ForceExecute();
                         * @brief	Method force execute functionality
                         * @author	Yadickson Soto
                         * @date	16/11/2012
                         * @return	true if force execute
                         */
                        virtual bool ForceExecute();

                        /**
                         * @fn	virtual bool CanExecute();
                         * @brief	Return if this functionality can be executed.
                         * @author	Luiggi.mendez
                         * @date	28/04/2011
                         * @return	true if can execute functionality.
                         */

                        virtual bool CanExecute();

                        /**
                         * @fn	virtual bool Enable();
                         * @brief	Enables this object.
                         * @author	Luiggi.mendez
                         * @date	28/04/2011
                         * @return	true if it succeeds, false if it fails.
                         */

                        virtual bool Enable();

                        /**
                         * @fn	void ForceExit();
                         * @brief	Force exit.
                         * @author	Luiggi.mendez
                         * @date	28/04/2011
                         */

                        void ForceExit();

                        /**
                         * @fn	virtual std::string getDescriptionMessage();
                         * @brief	Gets the description message.
                         * @author	Luiggi.mendez
                         * @date	28/04/2011
                         * @return	The description message.
                         */

                        virtual std::string getDescriptionMessage();

                        /**
                         * @fn	void setExitButtonSensitive ( bool sensitive );
                         * @brief	Sets an exit button sensitive.
                         * @author	Luiggi.mendez
                         * @date	28/04/2011
                         * @param	sensitive	true to sensitive.
                         */

                        void setExitButtonSensitive ( bool sensitive );

                        /**
                         * @fn	void setHelpButtonSensitive ( bool sensitive );
                         * @brief	Sets a help button sensitive.
                         * @author	Luiggi.mendez
                         * @date	28/04/2011
                         * @param	sensitive	true to sensitive.
                         */
                        void setHelpButtonSensitive ( bool sensitive );

                        /**
                         * @fn	void enableExitButton();
                         * @brief	Method enable exit button.
                         * @author	Yadickson Soto
                         * @date	09/04/2012
                         */
                        void enableExitButton();

                        /**
                         * @fn	void disableExitButton();
                         * @brief	Method disable exit button.
                         * @author	Yadickson Soto
                         * @date	09/04/2012
                         */
                        void disableExitButton();

                        /**
                         * @fn	void enableHelpButton();
                         * @brief	Method enable help button.
                         * @author	Yadickson Soto
                         * @date	09/04/2012
                         */
                        void enableHelpButton();

                        /**
                         * @fn	void disableHelpButton();
                         * @brief	Method disable help button.
                         * @author	Yadickson Soto
                         * @date	09/04/2012
                         */
                        void disableHelpButton();

                    protected:

                        sigc::connection startHelpPrintConn;	///< The start print connection
                        sigc::connection stopHelpPrintConn;	///< The stop print connection
                        sigc::connection shownHelpConn;	///< The shown connection
                        sigc::connection closedHelpConn;	///< The closed connection

                        Smartmatic::SAES::GUI::Windows::EmptyWindow * baseWindow;	///< The base window
                        Smartmatic::GUI::Widgets::StepWidget stepWidget; ///< step widget
                        Smartmatic::SAES::GUI::Widgets::InfoStepWidget initWidget; ///< init widget
                        Smartmatic::SAES::GUI::Widgets::HelpWidget * helpWidget; ///< help widget
                        Smartmatic::SAES::GUI::Widgets::InfoStepWidget exitWidget; ///< exit widget

                        /**
                         * @fn	bool ShowFinalMessage();
                         * @brief	Show final message.
                         * @author	Juan.Delgado
                         * @date	28/04/2011
                         * @return	true if show final message.
                         */

                        bool ShowFinalMessage();

                        /**
                         * @fn	virtual void ExitFunctionality();
                         * @brief	Call onExitFunctionality method.
                         * @author	Yadickson Soto
                         * @date	04/09/2013
                         */
                        virtual void ExitFunctionality();

                        /**
                         * @fn	void OnHelpButtonClicked();
                         * @brief	Executes the help button clicked action.
                         * @author	Luiggi.mendez
                         * @date	28/04/2011
                         */

                        void OnHelpButtonClicked();

                        /**
                         * @fn	void OnHelpButtonClickedConnect();
                         * @brief	Executes the help button clicked action.
                         * @author	Luiggi.mendez
                         * @date	28/04/2011
                         */

                        void OnHelpButtonClickedConnect();

                        /**
                         * @fn	virtual Smartmatic::Log::ISMTTLog * getLogger() = 0;
                         * @brief	Gets the logger.
                         * @author	Luiggi.mendez
                         * @date	28/04/2011
                         * @return	null if it fails, else the logger.
                         */

                        virtual Smartmatic::Log::ISMTTLog * getLogger() = 0;

                        /**
                         * @fn	virtual Glib::ustring getStartLogMessage() = 0;
                         * @brief	Gets the start log message.
                         * @author	Luiggi.mendez
                         * @date	28/04/2011
                         * @return	The start log message.
                         */

                        virtual Glib::ustring getStartLogMessage() = 0;

                        /**
                         * @fn	virtual Glib::ustring getFinishLogMessage() = 0;
                         * @brief	Gets the finish log message.
                         * @author	Luiggi.mendez
                         * @date	28/04/2011
                         * @return	The finish log message.
                         */

                        virtual Glib::ustring getFinishLogMessage() = 0;

                        /**
                         * @fn	void EnableBackgroundProcess();
                         * @brief	Enable background process functionality.
                         * @author	Gerardo.Hernandez
                         * @date	10/06/2011
                         */
                        void EnableBackgroundProcess();

                        /**
                         * @fn	virtual void deleteElements() = 0;
                         * @brief	virtual Deletes all elements.
                         * @author	Yadickson Soto
                         * @date	03/04/2012
                         */
                        virtual void deleteElements() = 0;

                        /**
                         * @fn	void RefreshInfo();
                         * @brief	Refresh information.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         */
                        void RefreshInfo();
                        /**
                         * @fn	virtual void refresh();
                         * @brief	refresh widget method.
                         * @author	Yadickson Soto
                         * @date	30/03/2012
                         */
                        virtual void refresh();
                        /**
                         * @fn	virtual void block();
                         * @brief	method block functionality.
                         * @author	Yadickson Soto
                         * @date	19/04/2012
                         */
                        virtual void block();
                        /**
                         * @fn	virtual void partial();
                         * @brief	method partial functionality.
                         * @author	Yadickson Soto
                         * @date	19/04/2012
                         */
                        virtual void partial();
                        /**
                         * @fn	virtual void resume();
                         * @brief	method resume functionality.
                         * @author	Yadickson Soto
                         * @date	19/04/2012
                         */
                        virtual void resume();
                        /**
                         * @fn	void endNotify();
                         * @brief	method end notify.
                         * @author	Yadickson Soto
                         * @date	19/04/2012
                         */
                        void endNotify();
                        /**
                         * @fn	virtual bool isRunning();
                         * @brief	Method getter is running.
                         * @author	Yadickson Soto
                         * @date	19/09/2012
                         * @return	true is running.
                         */
                        virtual bool isRunning();
                        /**
                         * @fn	virtual void onBeforeExit();
                         * @brief	virtual method before exit.
                         * @author	Yadickson Soto
                         * @date	03/10/2012
                         */
                        virtual void onBeforeExit();
                        /**
                         * @fn	virtual void onExit();
                         * @brief	virtual method exit.
                         * @author	Yadickson Soto
                         * @date	03/10/2012
                         */
                        virtual void onExit();
                        /**
                         * @fn	virtual Smartmatic::GUI::Widgets::StepWidget * getHelpWidget();
                         * @brief	virtual method refresh.
                         * @author	Yadickson Soto
                         * @date	04/10/2012
                         * @return	null if it fails, else the help widget.
                         */
                        virtual Smartmatic::GUI::Widgets::StepWidget * getHelpWidget();

                        /**
                         * @fn	virtual void onSetKey( Smartmatic::GUI::Widgets::KeysActionsCode value);
                         * @brief	Sets a key.
                         * @author	Yadickson Soto
                         * @date	20/11/2012
                         * @param	value	The value.
                         */
                        virtual void onSetKey ( Smartmatic::GUI::Widgets::KeysActionsCode value );

                        /**
                         * @fn	void onHelpShown();
                         * @brief	Method on help shown.
                         * @author	Juan Valera
                         * @date	09/02/2017
                         */
                        virtual void onHelpShown();

                        /**
                         * @fn	void onHelpClosed();
                         * @brief	Method on help closed.
                         * @author	Juan Valera
                         * @date	09/02/2017
                         */
                        virtual void onHelpClosed();

                    private:

                        Gtk::Widget * frame; ///< The frame
                        bool forceToLoad; ///< true to force to load
                        bool enableBackgroundProcess; ///<enable background process
                        Smartmatic::SAES::Configuration::HelpConfiguration helpConfiguration; ///<the help configuration
                        Smartmatic::GUI::Widgets::KeysActionsCode currentKey; ///< current key

                        /**
                         * @fn	virtual void onExitFunctionality();
                         * @brief	Exit functionality.
                         * @author	Yadickson Soto
                         * @date	04/09/2013
                         */
                        void onExitFunctionality();

                        /**
                         * @fn	void onEmitSetKey ( );
                         * @brief	Method emit signal key press
                         * @author	Yadickson Soto
                         * @date	20/11/2012
                         */
                        void onEmitSetKey ( );

                        /**
                         * @fn	virtual void SetKey( Smartmatic::GUI::Widgets::KeysActionsCode value);
                         * @brief	Sets a key.
                         * @author	Yadickson Soto
                         * @date	20/11/2012
                         * @param	value	The value.
                         */
                        virtual void SetKey ( Smartmatic::GUI::Widgets::KeysActionsCode value );

                        /**
                         * @fn	void ExecuteDispose();
                         * @brief	Executes the dispose operation.
                         * @author	Luiggi.mendez
                         * @date	28/04/2011
                         */

                        void ExecuteDispose();

                        /**
                         * @fn	bool checkMemoryPath();
                         * @brief	Determines if we can check memory path.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         * @return	true if it succeeds, false if it fails.
                         */
                        bool checkMemoryPath();

                        /**
                         * @fn	bool checkOperationStatus ( Smartmatic::SAES::Voting::OperationStatus * operationStatus );
                         * @brief	Check operation status.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         * @param [in,out]	operationStatus	If non-null, the operation status.
                         * @return	true if it succeeds, false if it fails.
                         */
                        bool checkOperationStatus ( Smartmatic::SAES::Voting::OperationStatus * operationStatus );

                        /**
                         * @fn	bool checkNecessaryDevices();
                         * @brief	Determines if we can check necessary devices.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         * @return	true if it succeeds, false if it fails.
                         */
                        bool checkNecessaryDevices();

                        /**
                         * @fn	bool checkBarcode( Smartmatic::SAES::Common::Operation::Monitor::DeviceTypeMonitor * deviceTypeMonitor );
                         * @brief	Determines if we can check barcode.
                         * @author	Yadickson Soto
                         * @date	01/11/2012
                         * @param deviceTypeMonitor device type monitor to test
                         * @return	true if it succeeds, false if it fails.
                         */
                        bool checkBarcode ( Smartmatic::SAES::Common::Operation::Monitor::DeviceTypeMonitor * deviceTypeMonitor );

                        /**
                         * @fn	bool checkSmartcard( Smartmatic::SAES::Common::Operation::Monitor::DeviceTypeMonitor * deviceTypeMonitor );
                         * @brief	Determines if we can check smartcard.
                         * @author	Yadickson Soto
                         * @date	01/11/2012
                         * @param deviceTypeMonitor device type monitor to test
                         * @return	true if it succeeds, false if it fails.
                         */
                        bool checkSmartcard ( Smartmatic::SAES::Common::Operation::Monitor::DeviceTypeMonitor * deviceTypeMonitor );

                        /**
                         * @fn	bool checkPrinter( Smartmatic::SAES::Common::Operation::Monitor::DeviceTypeMonitor * deviceTypeMonitor );
                         * @brief	Determines if we can check printer.
                         * @author	Yadickson Soto
                         * @date	01/11/2012
                         * @param deviceTypeMonitor device type monitor to test
                         * @return	true if it succeeds, false if it fails.
                         */
                        bool checkPrinter ( Smartmatic::SAES::Common::Operation::Monitor::DeviceTypeMonitor * deviceTypeMonitor );

                        /**
                         * @fn	void Exit();
                         * @brief	Exits this object.
                         * @author	Luiggi.mendez
                         * @date	28/04/2011
                         */

                        void Exit(); //Call OnExitButtonClicked if want to exit functionality

                        /**
                         * @fn	void forceDisableExitButton();
                         * @brief	Force disable exit button
                         * @author	Yadickson Soto
                         * @date	30/11/2012
                         */
                        void forceDisableExitButton();

                        bool flagEndNotify; ///< flag end notify
                        bool flagExecute; ///< flag run execute
                };
            }
        }
    }
}

#endif /* _GRAPHIC_DISPLAY_FUNCTIONALITY_H_ */
