/**
 * @file	voting-experience-validation-functionality.cpp
 * @brief	Implements the voting experience validation functionality class.
 * @author	Youssef.tourkmani
 * @date	01/11/2012
 */

#include "voting-experience-validation-functionality.h"
#include "Operation/MachineOperationManager.hxx"
#include <System/IO/Path.h>
#include <System/IO/File.h>
#include <System/IO/Directory.h>
#include <Log/SMTTLogManager.h>
#include <Resources/election-string-resources-manager.h>
#include <Voting/VotingCenters/VotingCentersWrapper.hxx>
#include "Configuration/s-a-e-s-g-u-i-conf-manager.h"
#include <Voting/Election/ElectionInstalationLanguages.h>
#include <Voting/set-language-functionality.h>
#include <Voting/CardData/card-data-wrapper.h>
#include <Voting/Election/election-schema.hxx>
#include <Voting/VotingDevice/voting-devices-schema.hxx>
#include <Operation/Voting/VotingOperation.hxx>
#include <Gui/Widgets/AbstractFactoryWidget.hxx>
#include <Gui/Widgets/Voting/FactoryVotingWidget.hxx>
#include <System/Runtime/ProcessLauncher.h>
#include <Runtime/Environment/DirectoryManager.hxx>
#include <Graphics/Image/CheckImageFile.hxx>
#include <System/ZenityWrapper.hxx>
#include <Voting/Status/VMProcessStatusWrapper.hxx>
#include <System/Utils/StringUtils.hxx>
#include <Graphics/Capture/PrintScreenManager.h>
#include <System/Sounds/smartmatic-sounds-manager.h>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#include <libintl.h>
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

#define N_(String) (String) ///<macro for gettext not translate

using namespace Smartmatic::System::Utils;
using namespace Smartmatic::System::IO;
using namespace Smartmatic::Environment;
using namespace Smartmatic::GUI::Widgets;
using namespace Smartmatic::Functionality;
using namespace Smartmatic::SAES::Operation;
using namespace Smartmatic::SAES::GUI::Functionality;
using namespace Smartmatic::SAES::Runtime::Environment;
using namespace Smartmatic::SAES::GUI::Operation;
using namespace Smartmatic::SAES::GUI::Operation::Voting;
using namespace Smartmatic::SAES::GUI::Widgets::Voting;
using namespace Smartmatic::SAES::GUI::Widgets;
using namespace Smartmatic::Log;
using namespace Smartmatic::SAES::Voting;
using namespace Smartmatic::SAES::Voting::Election;
using namespace Smartmatic::SAES::Voting::VotingDevice;
using namespace Smartmatic::SAES::Voting::VotingCenters;
using namespace Smartmatic::SAES::Resources;
using namespace Smartmatic::SAES::GUI::Configuration;
using namespace Smartmatic::SAES::Operation::Voting;
using namespace Smartmatic::SAES::GUI::Operation::Voting;
using namespace Smartmatic::System::Runtime;
using namespace Smartmatic::Graphics::Image;
using namespace Smartmatic::System;
using namespace Smartmatic::SAES::Runtime::Environment;
using namespace Smartmatic::SAES::Configuration;
using namespace Smartmatic::SAES::Functionality;
using namespace Smartmatic::SAES::Voting::Status;
using namespace Smartmatic::System::Sounds;

ISMTTLog * VotingExperienceValidationFunctionality::logger = SMTTLogManager::GetLogger ( "Smartmatic.SAES.GUI.Functionality.VotingExperienceValidationFunctionality", GETTEXT_PACKAGE );

VotingExperienceValidationFunctionality * VotingExperienceValidationFunctionality::current_instance = NULL;
VotingExperienceValidationFunctionality * VotingExperienceValidationFunctionality::getInstance()
{
    if ( !current_instance )
    {
        current_instance = new VotingExperienceValidationFunctionality();
    }

    return current_instance;
}

VotingExperienceValidationFunctionality::VotingExperienceValidationFunctionality()
{
    electoralConfiguration = NULL;
    votingExperienceValidationWidget = NULL;
    votingExperienceController = NULL;
    screenVotingWidget = NULL;
    forceDelete = false;
    m_thread_generate_center = NULL;
    m_thread_capture = NULL;

    currentDirectories = MachineOperationManager::getInstance()->getSaesInstallDirectories();
    checkImageFiles = SAESGUIConfigurationManager::getInstance()->GetVotingExperienceValidationScreensFunctionalityConfiguration().getCheckImageFile();
    sound = SAESGUIConfigurationManager::getInstance()->GetVotingExperienceValidationScreensFunctionalityConfiguration().getScreenCaptureSound();

    badCount = 0;
    badTotalCount = 0;
    currentCounter = 0;

    signalGenerate.connect ( sigc::mem_fun ( *this , &VotingExperienceValidationFunctionality::Generate ) );
    signalNextVotingCenter.connect ( sigc::mem_fun ( *this , &VotingExperienceValidationFunctionality::nextVotingCenter ) );
    signalNextStepWidget.connect ( sigc::mem_fun ( *this , &VotingExperienceValidationFunctionality::onNextStepWidget ) );
    signalSaveImage.connect ( sigc::mem_fun ( *this , &VotingExperienceValidationFunctionality::saveImage ) );
    signalEndCapturePrintScreen.connect ( sigc::mem_fun ( *this , &VotingExperienceValidationFunctionality::captureImage ) );
}

VotingExperienceValidationFunctionality::~VotingExperienceValidationFunctionality()
{
    deleteElements();
}

void VotingExperienceValidationFunctionality::deleteElements()
{
    if ( votingExperienceValidationWidget )
    {
        delete ( votingExperienceValidationWidget );
        votingExperienceValidationWidget = NULL;
    }

    if ( electoralConfiguration )
    {
        delete ( electoralConfiguration );
        electoralConfiguration = NULL;
    }

    if ( votingExperienceController )
    {
        delete votingExperienceController;
        votingExperienceController = NULL;
    }

    if ( screenVotingWidget )
    {
        delete screenVotingWidget;
        screenVotingWidget = NULL;
    }

    if ( m_thread_generate_center != NULL )
    {
        m_thread_generate_center->join();
        m_thread_generate_center = NULL;
    }

    if ( m_thread_capture != NULL )
    {
        m_thread_capture->join();
        m_thread_capture = NULL;
    }
}

void VotingExperienceValidationFunctionality::Dispose()
{
    VMProcessStatusWrapper::getInstance()->stop();
    VMProcessStatusWrapper::getInstance()->persistence();
}

void VotingExperienceValidationFunctionality::setStepWidget ( Smartmatic::GUI::Widgets::StepWidget * widget )
{
    currentWidget = widget;
    widget->refresh();
    container->setWidget ( widget );
}

Gtk::Widget * VotingExperienceValidationFunctionality::GetFrame()
{
    container = new StepWidget();
    return container;
}

std::string VotingExperienceValidationFunctionality::GetWindowTitle()
{
    return N_ ( "Smartmatic.SAES.GUI.Functionality.VotingExperienceValidationFunctionality.Title" );
}

Smartmatic::SAES::Functionality::Configuration::MenuOptionType VotingExperienceValidationFunctionality::GetMenuOptionType()
{
    return MenuOptionType::CONFIGURATION_XMLMVOTINGEXPERIENCEVALIDATIONFUNCTIONALITY;
}

void VotingExperienceValidationFunctionality::Initialize()
{
    SetLanguageFunctionality::getCurrentLanguageCountry ( currentLanguage, currentCountry );
    VMProcessStatusWrapper::getInstance()->dispose();
    VMProcessStatusWrapper::getInstance()->start();

    electoralConfiguration = new ElectoralConfiguration();
    machineOperationManager = MachineOperationManager::getInstance();
    VotingWidgetManager = VotingWidgetManager::getInstance();
    imageFileSequence = 1;
    imageFileByCenter = 0;
    screenVotingWidget = FactoryVotingWidget::createScreenVotingWidget();
    screenVotingWidget->getSignalEnd().connect ( sigc::mem_fun ( *this, &VotingExperienceValidationFunctionality::onShutdownFunctionality ) );

    votingExperienceValidationWidget = FactoryVotingWidget::createVotingExperineceValidationWidget();
    votingExperienceValidationWidget->getSignalAll().connect ( sigc::mem_fun ( *this, &VotingExperienceValidationFunctionality::LoadScreenConfig ) );
    votingExperienceValidationWidget->getSignalBallot().connect ( sigc::mem_fun ( *this, &VotingExperienceValidationFunctionality::LoadScreenConfig ) );

    baseWindow->set_name ( SAESGUIConfigurationManager::getInstance()->GetVotingExperienceWindowName() );

    bool autorun = SAESGUIConfigurationManager::getInstance()->GetVotingExperienceValidationScreensFunctionalityConfiguration().getAutorun();

    if ( autorun )
    {
        int startTime = SAESGUIConfigurationManager::getInstance()->GetVotingExperienceValidationScreensFunctionalityConfiguration().getTimeForStart();
        Glib::signal_timeout().connect_once ( sigc::mem_fun ( *this, &VotingExperienceValidationFunctionality::LoadScreenConfig ), startTime );
    }
    else
    {
        setStepWidget ( votingExperienceValidationWidget );
    }
}

void VotingExperienceValidationFunctionality::LoadScreenConfig()
{
    currentCounter = 0;
    signalGenerate.emit();
}

void VotingExperienceValidationFunctionality::Generate()
{
    std::string votingCentersFilePath = currentDirectories->getVotingCentersInstalationFileFullPath();
    bool success = VotingCentersWrapper::getInstance()->LoadFile ( votingCentersFilePath );
    logger->Debug ( "Voting Centers File path is: " + votingCentersFilePath );

    if ( success )
    {
        std::list< VotingCenter > & list ( VotingCentersWrapper::getInstance()->getVotingCenters() );
        std::list< VotingCenter >::iterator itVc;
        {
            std::stringstream ss;
            ss << "Number of voting centers: " << list.size();
            logger->Debug ( ss.str() );
        }

        votingCenters.clear();

        for ( itVc = list.begin(); itVc != list.end(); ++itVc )
        {
            votingCenters.push_back ( ( *itVc ) );
        }

        imageFileByCenter = 0;
        currentCenter = 0;
        screenVotingWidget->setCurrentCenter ( currentCenter );
        screenVotingWidget->setTotalCenter ( votingCenters.size() );
        signalNextVotingCenter.emit();
    }
    else
    {
        onExitFunctionality();
        logger->Debug ( "Could not open file: " + votingCentersFilePath );
    }

    setStepWidget ( screenVotingWidget );
}

void VotingExperienceValidationFunctionality::saveCurrentCenter()
{
    if ( currentCenter != 0 )
    {
        std::string folderName = infoCenter.getFolder_name();
        std::string destImages = Path::Combine ( currentDirectories->getVotingExperienceValidationFolderFullPath(), folderName );
        std::string orgImages = Path::Combine ( currentDirectories->getVotingExperienceValidationFolderFullPath ( Smartmatic::SAES::Common::Configuration::BasePathType::CONFIGURATION_LIVE ), infoCenter.getFolder_name() );
        DirectoryManager::copyDir ( orgImages, destImages, false );
        Directory::getInstance()->Delete ( orgImages );

        std::stringstream totalImagesText;
        totalImagesText << "Total images by center = " << imageFileByCenter;
        logger->Debug ( totalImagesText.str() );

        VMProcessStatusWrapper::getInstance()->addSuccess ( GETTEXT_PACKAGE, N_ ( "Smartmatic.SAES.GUI.Functionality.DataUtilsValidatorFunctionality.TotalImages" ), StringUtils::intToString ( imageFileByCenter ) );

        imageFileByCenter = 0;
    }
}

void VotingExperienceValidationFunctionality::nextVotingCenter()
{
    saveCurrentCenter();

    if ( !votingCenters.empty() )
    {
        currentCenter++;
        infoCenter = votingCenters.front();
        votingCenters.erase ( votingCenters.begin(), votingCenters.begin() + 1 );
        logger->Debug ( "Process voting center " + infoCenter.getCode() );

        VMProcessStatusWrapper::getInstance()->init ( currentLanguage, currentCountry, infoCenter.getCode() );

        screenVotingWidget->setCenterName ( " " );
        screenVotingWidget->setCurrentCenter ( ++currentCounter );

        setStepWidget ( screenVotingWidget );
        std::string folderName = infoCenter.getFolder_name();
        std::string programUnit = ( infoCenter.getProgram_unit() != NULL ? infoCenter.getProgram_unit().get() : "" );
        currentDirectories->setWorkingPath ( folderName );
        currentDirectories->setProgramUnitName ( programUnit );
        logger->Debug ( "Generate Center: " + folderName );

        VMProcessStatusWrapper::getInstance()->addSuccess ( GETTEXT_PACKAGE, N_ ( "Smartmatic.SAES.GUI.Functionality.DataUtilsValidatorFunctionality.LoadFolder" ), folderName );

        folderPath = Path::Combine ( currentDirectories->getVotingExperienceValidationFolderFullPath ( Smartmatic::SAES::Common::Configuration::BasePathType::CONFIGURATION_LIVE ), folderName );

        if ( !Glib::thread_supported() )
        {
            Glib::thread_init();
        }

        if ( m_thread_generate_center != NULL )
        {
            m_thread_generate_center->join();
            m_thread_generate_center = NULL;
        }

        m_thread_generate_center = Glib::Thread::create ( sigc::mem_fun ( *this, &VotingExperienceValidationFunctionality::generateCenter ), true );
    }
    else
    {
        showEndResult();
    }
}

void VotingExperienceValidationFunctionality::showEndResult()
{
    Smartmatic::System::IO::Directory::getInstance()->addAllPermission ( currentDirectories->getVotingExperienceValidationFolderFullPath() );

    Directory::getInstance()->Delete ( currentDirectories->getVotingExperienceValidationFolderFullPath ( Smartmatic::SAES::Common::Configuration::BasePathType::CONFIGURATION_LIVE ) );
    bool autoclose = SAESGUIConfigurationManager::getInstance()->GetVotingExperienceValidationScreensFunctionalityConfiguration().getAutoclose();

    std::stringstream totalImagesText;
    totalImagesText << "Total images = " << imageFileSequence - 1;
    logger->Debug ( totalImagesText.str() );

    if ( autoclose )
    {
        onExitFunctionality();
    }
}

void VotingExperienceValidationFunctionality::onNextStepWidget()
{
    Glib::signal_timeout().connect_once ( sigc::mem_fun ( *this , &VotingExperienceValidationFunctionality::nextStepWidget ), 0 );
}

void VotingExperienceValidationFunctionality::nextStepWidget()
{
    if ( !stepWidgets.empty() )
    {
        imageFileByCenter++;
        forceDelete = stepWidgets.front().forceDelete;
        StepWidget * widget = stepWidgets.front().widget;
        std::string fileName = stepWidgets.front().fileName;
        std::string language = stepWidgets.front().language;
        std::string country = stepWidgets.front().country;
        configurePrintScreenManager ( fileName );
        SetLanguageFunctionality::setNewVotingLanguageByLanguageCountry ( language, country );
        stepWidgets.erase ( stepWidgets.begin(), stepWidgets.begin() + 1 );
        setStepWidget ( widget );
        unsigned int timeToShow = SAESGUIConfigurationManager::getInstance()->GetVotingExperienceValidationScreensFunctionalityConfiguration().getTimeToShow();
        Glib::signal_timeout().connect_once ( sigc::mem_fun ( *this , &VotingExperienceValidationFunctionality::capturePrintScreen ), timeToShow );
    }
    else
    {
        generateXml.persist ( folderPath );
        signalNextVotingCenter.emit();
    }
}

void VotingExperienceValidationFunctionality::captureImage()
{
    bool validate = true;

    if ( checkImageFiles )
    {
        std::string imageFile = Smartmatic::System::IO::Path::Combine ( currentImageFolder, currentImageName );
        logger->Debug ( "Check image file " + imageFile );
        validate = CheckImageFile::check ( imageFile );

        if ( !validate )
        {
            badCount++;
            badTotalCount++;
            std::stringstream ss;
            ss << imageFile << ".bad (" << badCount << ").";
            File::getInstance()->renameFile ( imageFile, ss.str() );
        }

        logger->Debug ( "Check image file " + std::string ( validate ? "Success" : "Fail" ) );
    }

    if ( validate || badCount > 2 )
    {
        SMTTSoundsManager::getInstance()->Sound ( sound );
        badCount = 0;
        signalSaveImage.emit();
    }
    else
    {
        unsigned int timeAfterCapture = SAESGUIConfigurationManager::getInstance()->GetVotingExperienceValidationScreensFunctionalityConfiguration().getTimeToShow();
        Glib::signal_timeout().connect_once ( sigc::mem_fun ( *this , &VotingExperienceValidationFunctionality::captureImage ), timeAfterCapture );
    }
}

void VotingExperienceValidationFunctionality::saveImage()
{
    if ( forceDelete && currentWidget )
    {
        delete currentWidget;
        currentWidget = NULL;
    }

    signalNextStepWidget.emit();
}

void VotingExperienceValidationFunctionality::generateCenter()
{
    sleep ( 1 );

    stepWidgets.clear();
    std::string currentDirectoryPath = currentDirectories->getBasePath();

    if ( Smartmatic::System::IO::Directory::getInstance()->DirectoryExists ( currentDirectoryPath ) )
    {
        logger->Debug ( "Loading Configuration of: " + currentDirectoryPath );

        if ( LoadElectoralConfiguration() )
        {
            generateXml.init();
            //Configure selectable languages
            machineOperationManager->setVotingExperienceController ( votingExperienceController );
            Languages::LanguageSequence sequence = electoralConfiguration->getLanguages().getLanguage();
            Languages::LanguageIterator it;

            if ( sequence.empty() )
            {
                logger->Debug ( "Could not load languages of the configuration" );
                VMProcessStatusWrapper::getInstance()->addFail ( GETTEXT_PACKAGE, N_ ( "Smartmatic.SAES.GUI.Functionality.DataUtilsValidatorFunctionality.LanguageNotFound" ) );
                signalNextVotingCenter.emit();
                return;
            }

            //Clear previous language
            ElectionInstalationLanguages::getInstance()->clearLanguage();
            /**
             * LANGUAGE ITERATION
             */

            for ( it = sequence.begin(); it != sequence.end(); ++it )
            {
                ElectionInstalationLanguages::getInstance()->addInstallationLanguages ( ( *it ) );
            }

            for ( it = sequence.begin(); it != sequence.end(); ++it )
            {
                std::string language = it->getLanguage();
                std::string country = it->getCountry();

                logger->Debug ( "Generating images in language: " + language + "_" + country );

                VMProcessStatusWrapper::getInstance()->addSuccess ( GETTEXT_PACKAGE, N_ ( "Smartmatic.SAES.GUI.Functionality.DataUtilsValidatorFunctionality.LoadLanguage" ) , language + "_" + country );

                try
                {
                    ClearVariables();
                    ElectionInstalationLanguages::getInstance()->addInstallationLanguages ( ( *it ) );
                    Voting_device vd = machineOperationManager->getElectoralConfiguration()->getVotingDevice()->getFirstVotingDevice();
                    std::list < Custom_attribute_definition > listCAD =
                        machineOperationManager->getElectoralConfiguration()->getVotingDevice()->getCustomAttributeDefinitionList ( vd, VotingDevice::Machine_type::VOTING_DEVICES_PM );
                    std::vector<Custom_attribute> attributes; ///< The attributes

                    /**
                     * CUSTOM ATTRIBUTE MP ITERATION / SIMULATE
                     */
                    for ( std::list<VotingDevice::Custom_attribute_definition>::iterator l ( listCAD.begin() ); l != listCAD.end(); ++l )
                    {
                        /**
                         * ITERATION IN VALUES OF CURRENT CUSTOM ATTRIBUTE
                         * GET ALL FIXED CONTEST
                         */
                        for ( VotingDevice::Custom_attribute_definition_values::Custom_attribute_definition_valueIterator c ( ( *l ).getCustom_attribute_definition_values().getCustom_attribute_definition_value().begin() );
                                c != ( *l ).getCustom_attribute_definition_values().getCustom_attribute_definition_value().end();
                                ++c )
                        {
                            VotingDevice::Custom_attribute attribute;
                            attribute.setCode ( l->getCode() );
                            attribute.setValue ( ( *c ).getValue() );
                            attributes.push_back ( attribute );
                            CardData::Card_data cardData;

                            VMProcessStatusWrapper::getInstance()->addSuccess ( GETTEXT_PACKAGE, N_ ( "Smartmatic.SAES.GUI.Functionality.DataUtilsValidatorFunctionality.CreateCard" ), ( *c ).getValue() );

                            CardData::CardDataWrapper::GenerateCardData ( MachineOperationManager::getInstance()->getElectoralConfiguration(), MachineOperationManager::getInstance()->getOperationStatus(), attributes, cardData, false, Smartmatic::SAES::Voting::CardData::Card_type::CARD_DATA_DEMOSTRATION_VOTE_CARD );

                            if ( votingExperienceController->init ( cardData ) )
                            {
                                VotingOperation::refresh();
                                UpdateFixedContests();
                            }
                            else
                            {
                                logger->Debug ( "Error writing info card for images in: " + currentDirectoryPath + " for " + it->getCode() );
                                VMProcessStatusWrapper::getInstance()->addFail ( GETTEXT_PACKAGE, N_ ( "Smartmatic.SAES.GUI.Functionality.DataUtilsValidatorFunctionality.InitVotingExperienceFail" ) );
                            }

                            attributes.clear();
                            votingExperienceController->dispose();
                        }

                        generateImages ( language, country );
                    }
                }
                catch ( ... )
                {
                    logger->Debug ( "Error generating and showing the images in: " + currentDirectoryPath + " for " + it->getCode() );
                    VMProcessStatusWrapper::getInstance()->addFail ( GETTEXT_PACKAGE, N_ ( "Smartmatic.SAES.GUI.Functionality.DataUtilsValidatorFunctionality.InitVotingExperienceFail" ) );
                }
            }

            signalNextStepWidget.emit();
        }
        else
        {
            logger->Debug ( "Error loading the configuration" );
            signalNextVotingCenter.emit();
        }
    }
    else
    {
        logger->Debug ( "Directory no exist: " + currentDirectoryPath );
        VMProcessStatusWrapper::getInstance()->addFail ( GETTEXT_PACKAGE, N_ ( "Smartmatic.SAES.GUI.Functionality.DataUtilsValidatorFunctionality.ErrorLoadElectoralConfiguration" ) );
        signalNextVotingCenter.emit();
    }
}

void VotingExperienceValidationFunctionality::ClearVariables()
{
    fixedContest.clear();
}

void VotingExperienceValidationFunctionality::UpdateFixedContests()
{
    std::list<std::string> fixed = machineOperationManager->getVotingExperienceController()->getFixedContests();

    for ( std::list<std::string>::iterator iter = fixed.begin(); iter != fixed.end(); ++iter )
    {
        std::list<std::string>::iterator itFind = find ( fixedContest.begin(), fixedContest.end(), *iter );

        if ( itFind == fixedContest.end() )
        {
            fixedContest.push_back ( *iter );
        }
    }
}

bool VotingExperienceValidationFunctionality::LoadElectoralConfiguration()
{
    bool returnVal = TRUE;

    VMProcessStatusWrapper::getInstance()->addSuccess ( GETTEXT_PACKAGE, N_ ( "Smartmatic.SAES.GUI.Functionality.DataUtilsValidatorFunctionality.StartLoadConfiguration" ) );

    try
    {

        electoralConfiguration->init ( currentDirectories );
        VMProcessStatusWrapper::getInstance()->addSuccess ( GETTEXT_PACKAGE, N_ ( "Smartmatic.SAES.GUI.Functionality.DataUtilsValidatorFunctionality.LoadElectoralConfiguration" ) );
        electoralConfiguration->loadAllData();
        ElectionStringResourcesManager::getInstance()->init ( currentDirectories );
        machineOperationManager->setElectoralConfiguration ( electoralConfiguration );
        VotingWidgetManager->SetElectoralConfiguration ( electoralConfiguration );
        VotingWidgetManager->SetSaesDirectories ( currentDirectories );
        VotingWidgetManager->init();

        if ( votingExperienceController )
        {
            delete votingExperienceController;
            votingExperienceController = NULL;
        }

        votingExperienceController = new VotingExperienceController ( electoralConfiguration );
    }
    catch ( ... )
    {
        VMProcessStatusWrapper::getInstance()->addFail ( GETTEXT_PACKAGE, N_ ( "Smartmatic.SAES.GUI.Functionality.DataUtilsValidatorFunctionality.ErrorLoadElectoralConfiguration" ) );
        returnVal = FALSE;
    }

    return returnVal;
}

void VotingExperienceValidationFunctionality::generateWaitingScreen ( Smartmatic::SAES::Configuration::ScreenImageConfiguration & screenConfig, std::string & language, std::string & country )
{
    std::string formatName = screenConfig.getFormat();

    if ( !formatName.empty() )
    {
        logger->Debug ( "Generate welcome screen" );
        std::string fileName = Glib::ustring::compose ( formatName, language, country, imageFileSequence );
        WaitingForCardWidget * stepWidget = FactoryVotingWidget::createWaitingForCardWidget();
        stepWidget->setName ( VotingOperation::getWindowName() );
        generateXml.addImage ( fileName, language, country, screenConfig.getType() );
        addStepWidget ( stepWidget, fileName, language, country );
    }
}

void VotingExperienceValidationFunctionality::generateReadCard ( Smartmatic::SAES::Configuration::ScreenImageConfiguration & screenConfig, std::string & language, std::string & country )
{
    std::string formatName = screenConfig.getFormat();

    if ( !formatName.empty() )
    {
        logger->Debug ( "Generate welcome screen" );
        std::string fileName = Glib::ustring::compose ( formatName, language, country, imageFileSequence );
        WaitingForCardWidget * stepWidget = FactoryVotingWidget::createWaitingForCardWidget();
        stepWidget->forceNotChange ();
        stepWidget->viewCardValidate();
        stepWidget->setName ( VotingOperation::getWindowName() );
        generateXml.addImage ( fileName, language, country, screenConfig.getType() );
        addStepWidget ( stepWidget, fileName, language, country );
    }
}

void VotingExperienceValidationFunctionality::generateCardUsed ( Smartmatic::SAES::Configuration::ScreenImageConfiguration & screenConfig, std::string & language, std::string & country )
{
    std::string formatName = screenConfig.getFormat();

    if ( !formatName.empty() )
    {
        logger->Debug ( "Generate card used" );
        std::string fileName = Glib::ustring::compose ( formatName, language, country, imageFileSequence );
        WaitingForCardWidget * stepWidget = FactoryVotingWidget::createWaitingForCardWidget();
        stepWidget->forceNotChange ();
        stepWidget->viewErrorCardUsed();
        stepWidget->setName ( VotingOperation::getWindowName() );
        generateXml.addImage ( fileName, language, country, screenConfig.getType() );
        addStepWidget ( stepWidget, fileName, language, country );
    }
}

void VotingExperienceValidationFunctionality::generateErrorCard ( Smartmatic::SAES::Configuration::ScreenImageConfiguration & screenConfig, std::string & language, std::string & country )
{
    std::string formatName = screenConfig.getFormat();

    if ( !formatName.empty() )
    {
        logger->Debug ( "Generate error card" );
        std::string fileName = Glib::ustring::compose ( formatName, language, country, imageFileSequence );
        WaitingForCardWidget * stepWidget = FactoryVotingWidget::createWaitingForCardWidget();
        stepWidget->forceNotChange ();
        stepWidget->viewErrorCardValidate();
        stepWidget->setName ( VotingOperation::getWindowName() );
        generateXml.addImage ( fileName, language, country, screenConfig.getType() );
        addStepWidget ( stepWidget, fileName, language, country );
    }
}

void VotingExperienceValidationFunctionality::generateErrorCardAID ( Smartmatic::SAES::Configuration::ScreenImageConfiguration & screenConfig, std::string & language, std::string & country )
{
    std::string formatName = screenConfig.getFormat();

    if ( !formatName.empty() )
    {
        logger->Debug ( "Generate error card AID" );
        std::string fileName = Glib::ustring::compose ( formatName, language, country, imageFileSequence );
        WaitingForCardWidget * stepWidget = FactoryVotingWidget::createWaitingForCardWidget();
        stepWidget->forceNotChange ();
        stepWidget->viewErrorCardAID();
        stepWidget->setName ( VotingOperation::getWindowName() );
        generateXml.addImage ( fileName, language, country, screenConfig.getType() );
        addStepWidget ( stepWidget, fileName, language, country );
    }
}

void VotingExperienceValidationFunctionality::generateErrorCardPIN ( Smartmatic::SAES::Configuration::ScreenImageConfiguration & screenConfig, std::string & language, std::string & country )
{
    std::string formatName = screenConfig.getFormat();

    if ( !formatName.empty() )
    {
        logger->Debug ( "Generate error card PIN" );
        std::string fileName = Glib::ustring::compose ( formatName, language, country, imageFileSequence );
        WaitingForCardWidget * stepWidget = FactoryVotingWidget::createWaitingForCardWidget();
        stepWidget->forceNotChange ();
        stepWidget->viewErrorCardPIN();
        stepWidget->setName ( VotingOperation::getWindowName() );
        generateXml.addImage ( fileName, language, country, screenConfig.getType() );
        addStepWidget ( stepWidget, fileName, language, country );
    }
}

void VotingExperienceValidationFunctionality::generateErrorCardGeneric ( Smartmatic::SAES::Configuration::ScreenImageConfiguration & screenConfig, std::string & language, std::string & country )
{
    std::string formatName = screenConfig.getFormat();

    if ( !formatName.empty() )
    {
        logger->Debug ( "Generate error card PIN" );
        std::string fileName = Glib::ustring::compose ( formatName, language, country, imageFileSequence );
        WaitingForCardWidget * stepWidget = FactoryVotingWidget::createWaitingForCardWidget();
        stepWidget->forceNotChange ();
        stepWidget->viewErrorCardGeneric();
        stepWidget->setName ( VotingOperation::getWindowName() );
        generateXml.addImage ( fileName, language, country, screenConfig.getType() );
        addStepWidget ( stepWidget, fileName, language, country );
    }
}

void VotingExperienceValidationFunctionality::generateLanguage ( Smartmatic::SAES::Configuration::ScreenImageConfiguration & screenConfig, std::string & language, std::string & country )
{
    std::string formatName = screenConfig.getFormat();

    if ( !formatName.empty() )
    {
        Languages::LanguageSequence langInstall = electoralConfiguration->getLanguages().getLanguage();

        if ( langInstall.size() > 1 )
        {
            logger->Debug ( "Generate language screen" );
            std::string fileName = Glib::ustring::compose ( formatName, language, country, imageFileSequence );
            ILanguageSelectionWidget * stepWidget = FactoryVotingWidget::createVotingLanguageSelectionWidget();
            stepWidget->setName ( VotingOperation::getWindowName() );
            generateXml.addImage ( fileName, language, country, screenConfig.getType() );
            addStepWidget ( stepWidget, fileName, language, country );
        }
    }
}

void VotingExperienceValidationFunctionality::generateLanguageConfirmation ( Smartmatic::SAES::Configuration::ScreenImageConfiguration & screenConfig, std::string & language, std::string & country )
{
    std::string formatName = screenConfig.getFormat();

    if ( !formatName.empty() )
    {
        logger->Debug ( "Generate language confirmation screen" );

        Languages::LanguageSequence langInstall = electoralConfiguration->getLanguages().getLanguage();

        if ( langInstall.size() > 1 )
        {
            Languages::LanguageIterator iti;

            int i = 0;

            for ( iti = langInstall.begin();
                    iti != langInstall.end();
                    ++iti, i++ )
            {
                ILanguageSelectionWidget * stepWidget = FactoryVotingWidget::createVotingLanguageSelectionWidget();
                stepWidget->setName ( VotingOperation::getWindowName() );
                stepWidget->SimulateButtonNPressed ( i );
                std::string currentCode = iti->getLanguage() + "_" + iti->getCountry();
                std::string fileName = Glib::ustring::compose ( formatName, language, country, currentCode, imageFileSequence );
                generateXml.addSelectionImage ( fileName, language, country, screenConfig.getType(), currentCode );
                addStepWidget ( stepWidget, fileName, language, country );
            }
        }
    }
}

void VotingExperienceValidationFunctionality::generateCustomAttribute ( Smartmatic::SAES::Configuration::ScreenImageConfiguration & screenConfig, std::string & language, std::string & country )
{
    std::string formatName = screenConfig.getFormat();

    if ( !formatName.empty() )
    {
        logger->Debug ( "Generate custom attributes" );
        std::list < Custom_attribute_definition > customAttrList;
        std::list <Custom_attribute_definition>::iterator iter;
        Contest_classes::Contest_classSequence contestClassSeq;
        Voting_device votDev = machineOperationManager->getElectoralConfiguration()->getVotingDevice()->getFirstVotingDevice();
        customAttrList = machineOperationManager->getElectoralConfiguration()->getVotingDevice()->getCustomAttributeDefinitionList ( votDev );
        contestClassSeq = machineOperationManager->getElectoralConfiguration()->getElection()->getContestClasses().getContest_class();

        for ( iter = customAttrList.begin(); iter != customAttrList.end(); ++iter )
        {
            contestClassIndex = 0;

            for ( unsigned int i = 0; i < contestClassSeq.size(); i++ )
            {
                if ( contestClassSeq[i].getCustom_code().get() == iter->getCode() )
                {
                    std::list < Smartmatic::SAES::Voting::Election::Contest * > & allContestList =
                        machineOperationManager->getElectoralConfiguration()->getContestList();

                    for ( std::list < Smartmatic::SAES::Voting::Election::Contest * >::iterator iterContest = allContestList.begin();
                            iterContest != allContestList.end(); ++iterContest )
                    {
                        if ( contestClassSeq[i].getCode() == ( *iterContest )->getContest_class_code() )
                        {
                            contestClassIndex = i;
                            break;
                        }
                    }
                }
            }

            //compare custom class code
            if ( iter->getCustom_attribute_definition_values().getCustom_attribute_definition_value().size() > 1 )
            {
                // give a format to the custom attribute name key
                std::string formattedNameKey = iter->getName_key();
                FormatName ( formattedNameKey );
                std::string fileName = Glib::ustring::compose ( formatName, language, country, imageFileSequence );
                ICustomAttributeSelectionWidget * stepWidget = FactoryVotingWidget::createCustomAttributeWidget ( *iter );
                stepWidget->setName ( VotingOperation::getWindowName() );
                generateXml.addCustomAttributes ( fileName, language, country, *iter );
                addStepWidget ( stepWidget, fileName, language, country );
            }
        }
    }
}

void VotingExperienceValidationFunctionality::generateCustomAttributeConfirmation ( Smartmatic::SAES::Configuration::ScreenImageConfiguration & screenConfig, std::string & language, std::string & country )
{
    std::string formatName = screenConfig.getFormat();

    if ( !formatName.empty() )
    {
        logger->Debug ( "Generate custom attribute confirmation" );
        std::list < Custom_attribute_definition > customAttrList;
        std::list <Custom_attribute_definition>::iterator iter;
        Contest_classes::Contest_classSequence contestClassSeq;
        Voting_device votDev = machineOperationManager->getElectoralConfiguration()->getVotingDevice()->getFirstVotingDevice();
        customAttrList = machineOperationManager->getElectoralConfiguration()->getVotingDevice()->getCustomAttributeDefinitionList ( votDev );
        contestClassSeq = machineOperationManager->getElectoralConfiguration()->getElection()->getContestClasses().getContest_class();

        for ( iter = customAttrList.begin(); iter != customAttrList.end(); ++iter )
        {
            contestClassIndex = 0;

            for ( unsigned int i = 0; i < contestClassSeq.size(); i++ )
            {
                if ( contestClassSeq[i].getCustom_code().get() == iter->getCode() )
                {
                    std::list < Smartmatic::SAES::Voting::Election::Contest * > & allContestList =
                        machineOperationManager->getElectoralConfiguration()->getContestList();

                    for ( std::list < Smartmatic::SAES::Voting::Election::Contest * >::iterator iterContest = allContestList.begin();
                            iterContest != allContestList.end(); ++iterContest )
                    {
                        if ( contestClassSeq[i].getCode() == ( *iterContest )->getContest_class_code() )
                        {
                            contestClassIndex = i;
                            break;
                        }
                    }
                }
            }

            //compare custom class code
            if ( iter->getCustom_attribute_definition_values().getCustom_attribute_definition_value().size() > 1 )
            {
                for ( unsigned int i = 0; i < iter->getCustom_attribute_definition_values().getCustom_attribute_definition_value().size(); i++ )
                {
                    std::string formattedValue = iter->getCustom_attribute_definition_values().getCustom_attribute_definition_value() [i].getValue();
                    FormatName ( formattedValue );
                    std::string fileName = Glib::ustring::compose ( formatName, language, country, imageFileSequence );
                    ICustomAttributeSelectionWidget * stepWidget = FactoryVotingWidget::createCustomAttributeWidget ( *iter );
                    stepWidget->setName ( VotingOperation::getWindowName() );
                    stepWidget->ForceSelectedOption ( i );
                    generateXml.addSelectionImage ( fileName, language, country, screenConfig.getType(), formattedValue );
                    addStepWidget ( stepWidget, fileName, language, country );
                }
            }
        }
    }
}

void VotingExperienceValidationFunctionality::generatePrintVote ( Smartmatic::SAES::Configuration::ScreenImageConfiguration & screenConfig, std::string & language, std::string & country )
{
    std::string formatName = screenConfig.getFormat();

    if ( !formatName.empty() )
    {
        logger->Debug ( "Generate print vote" );
        std::string fileName = Glib::ustring::compose ( formatName, language, country, imageFileSequence );
        PrintVoteWidget * stepWidget = FactoryVotingWidget::createPrintVoteWidget();
        stepWidget->setName ( VotingOperation::getWindowName() );
        generateXml.addImage ( fileName, language, country, screenConfig.getType() );
        addStepWidget ( stepWidget, fileName, language, country );
    }
}

void VotingExperienceValidationFunctionality::generateErrorPrintVote ( Smartmatic::SAES::Configuration::ScreenImageConfiguration & screenConfig, std::string & language, std::string & country )
{
    std::string formatName = screenConfig.getFormat();

    if ( !formatName.empty() )
    {
        logger->Debug ( "Generate error print vote" );
        std::string fileName = Glib::ustring::compose ( formatName, language, country, imageFileSequence );
        PrintVoteErrorWidget * stepWidget = FactoryVotingWidget::createPrintVoteErrorWidget();
        stepWidget->setName ( VotingOperation::getWindowName() );
        generateXml.addImage ( fileName, language, country, screenConfig.getType() );
        addStepWidget ( stepWidget, fileName, language, country );
    }
}

void VotingExperienceValidationFunctionality::generateRemoveCard ( Smartmatic::SAES::Configuration::ScreenImageConfiguration & screenConfig, std::string & language, std::string & country )
{
    std::string formatName = screenConfig.getFormat();

    if ( !formatName.empty() )
    {
        logger->Debug ( "Generate remove card" );
        std::string fileName = Glib::ustring::compose ( formatName, language, country, imageFileSequence );
        RemoveCardWidget * stepWidget = FactoryVotingWidget::createRemoveCardWidget();
        stepWidget->setName ( VotingOperation::getWindowName() );
        generateXml.addImage ( fileName, language, country, screenConfig.getType() );
        addStepWidget ( stepWidget, fileName, language, country );
    }
}

void VotingExperienceValidationFunctionality::generateCardRemoved ( Smartmatic::SAES::Configuration::ScreenImageConfiguration & screenConfig, std::string & language, std::string & country )
{
    std::string formatName = screenConfig.getFormat();

    if ( !formatName.empty() )
    {
        logger->Debug ( "Generate card removed" );
        std::string fileName = Glib::ustring::compose ( formatName, language, country, imageFileSequence );
        CardRemovedWidget * stepWidget = FactoryVotingWidget::createCardRemovedWidget();
        stepWidget->setName ( VotingOperation::getWindowName() );
        generateXml.addImage ( fileName, language, country, screenConfig.getType() );
        addStepWidget ( stepWidget, fileName, language, country );
    }
}

void VotingExperienceValidationFunctionality::generateEndPrintVote ( Smartmatic::SAES::Configuration::ScreenImageConfiguration & screenConfig, std::string & language, std::string & country )
{
    std::string formatName = screenConfig.getFormat();

    if ( !formatName.empty() )
    {
        logger->Debug ( "Generate end print vote" );
        std::string formatName = screenConfig.getFormat();
        std::string fileName = Glib::ustring::compose ( formatName, language, country, imageFileSequence );
        FoldPrintVoteWidget * stepWidget = FactoryVotingWidget::createFoldPrintVoteWidget();
        stepWidget->setName ( VotingOperation::getWindowName() );
        generateXml.addImage ( fileName, language, country, screenConfig.getType() );
        addStepWidget ( stepWidget, fileName, language, country );
    }
}

void VotingExperienceValidationFunctionality::generatePrintFoldVote ( Smartmatic::SAES::Configuration::ScreenImageConfiguration & screenConfig, std::string & language, std::string & country )
{
    std::string formatName = screenConfig.getFormat();

    if ( !formatName.empty() )
    {
        logger->Debug ( "Generate end print vote" );
        std::string fileName = Glib::ustring::compose ( formatName, language, country, imageFileSequence );
        PrintAndFoldVoteWidget * stepWidget = FactoryVotingWidget::createPrintAndFoldVoteWidget();
        stepWidget->setName ( VotingOperation::getWindowName() );
        generateXml.addImage ( fileName, language, country, screenConfig.getType() );
        addStepWidget ( stepWidget, fileName, language, country );
    }
}

void VotingExperienceValidationFunctionality::generateTimeOut ( Smartmatic::SAES::Configuration::ScreenImageConfiguration & screenConfig, std::string & language, std::string & country )
{
    std::string formatName = screenConfig.getFormat();

    if ( !formatName.empty() )
    {
        logger->Debug ( "Generate time out" );
        std::string fileName = Glib::ustring::compose ( formatName, language, country, imageFileSequence );
        PrintTimeOutWidget * stepWidget = FactoryVotingWidget::createPrintTimeOutWidget();
        stepWidget->setName ( VotingOperation::getWindowName() );
        generateXml.addImage ( fileName, language, country, screenConfig.getType() );
        addStepWidget ( stepWidget, fileName, language, country );
    }
}

void VotingExperienceValidationFunctionality::generateScanVoteScreen ( Smartmatic::SAES::Configuration::ScreenImageConfiguration & screenConfig, std::string & language, std::string & country )
{
    std::string formatName = screenConfig.getFormat();

    if ( !formatName.empty() )
    {
        logger->Debug ( "Generate scan vote screen" );
        std::string fileName = Glib::ustring::compose ( formatName, language, country, imageFileSequence );
        ScanVoteWidget * stepWidget = FactoryVotingWidget::createScanVoteWidget();
        stepWidget->setName ( VotingOperation::getWindowName() );
        generateXml.addImage ( fileName, language, country, screenConfig.getType() );
        addStepWidget ( stepWidget, fileName, language, country );
    }
}

void VotingExperienceValidationFunctionality::generateParties ( Smartmatic::SAES::Configuration::ScreenImageConfiguration & screenConfig, std::string & language, std::string & country )
{
    std::string formatName = screenConfig.getFormat();

    if ( !formatName.empty() )
    {
        logger->Debug ( "Generate parties" );

        std::map<std::string, Smartmatic::SAES::GUI::Widgets::PartySelectionWidget *>::iterator iterPartySel;
        std::map<std::string, Smartmatic::SAES::GUI::Widgets::PartySelectionWidget *> & mapPartySelectionWidget ( VotingWidgetManager->getMapPartySelectionWidget() );

        //Loading Party Selections
        //Map indexed by the contest code

        for ( iterPartySel = mapPartySelectionWidget.begin(); iterPartySel !=  mapPartySelectionWidget.end(); ++iterPartySel )
        {
            contestClassIndex = GetContestClassIndexFromContestCode ( iterPartySel->second->getContest()->getContest_class_code() );
            std::list<std::string>::iterator itFind = find ( fixedContest.begin(), fixedContest.end(), iterPartySel->second->getContest()->getCode() );

            if ( itFind == fixedContest.end() )
            {
                iterPartySel->second->ShowBackButton();
            }
            else
            {
                iterPartySel->second->SetBackButton ( false );
            }

            std::string fileName = Glib::ustring::compose ( formatName, language, country, iterPartySel->first, contestClassIndex, imageFileSequence );
            iterPartySel->second->setName ( VotingOperation::getWindowName() );
            addStepWidget ( iterPartySel->second, fileName, language, country, false );
            generateXml.addParty ( fileName, language, country, iterPartySel->second->getContest() );
        }
    }
}

void VotingExperienceValidationFunctionality::generateBallots ( Smartmatic::SAES::Configuration::ScreenImageConfiguration & screenConfig, std::string & language, std::string & country )
{
    std::string formatName = screenConfig.getFormat();

    if ( !formatName.empty() )
    {
        logger->Debug ( "Generate ballots" );

        //Loading Ballot Options Selections
        std::map<std::string, std::map < std::string, Smartmatic::SAES::GUI::Widgets::BallotOptionSelectionWidget * > >::iterator itb1;
        std::map<std::string, Smartmatic::SAES::GUI::Widgets::BallotOptionSelectionWidget * >::iterator itb2;
        std::map < std::string , std::map < std::string, Smartmatic::SAES::GUI::Widgets::BallotOptionSelectionWidget * > >	& mapBallotOptionSelectionWidget =
            VotingWidgetManager->getMapBallotOptionSelectionWidget();

        for ( itb1 = mapBallotOptionSelectionWidget.begin(); itb1 != mapBallotOptionSelectionWidget.end(); ++itb1 )
        {
            std::list < Party * > & partyList = machineOperationManager->getElectoralConfiguration()->getListPartyByContestCode ( itb1->first );
            std::map < std::string, Smartmatic::SAES::GUI::Widgets::BallotOptionSelectionWidget * > & mapBallots ( ( *itb1 ).second );

            for ( itb2 = mapBallots.begin(); itb2 != mapBallots.end();  ++itb2 )
            {
                Smartmatic::SAES::GUI::Widgets::BallotOptionSelectionWidget * ballotTmp = ( *itb2 ).second;

                contestClassIndex = GetContestClassIndexFromContestCode ( ballotTmp->getContest()->getContest_class_code() );
                std::list<std::string>::iterator itFind = find ( fixedContest.begin(), fixedContest.end(), ballotTmp->getContest()->getCode() );

                if ( itFind == fixedContest.end() )
                {
                    ballotTmp->ShowBackButton();
                }
                else
                {
                    ballotTmp->SetBackButton ( false );
                }

                std::string fileName = Glib::ustring::compose ( formatName, language, country, itb1->first, FormatTallyPosition ( ballotTmp->GetTallyPosition() ), contestClassIndex, imageFileSequence );
                ballotTmp->setName ( VotingOperation::getWindowName() );

                Party * p = machineOperationManager->getElectoralConfiguration()->getPartyByCode ( itb2->first );

                if ( SAESGUIConfigurationManager::getInstance()->GetVotingExperienceValidationScreensFunctionalityConfiguration().getScreenOnlyFirstBallot() )
                {
                    if ( itb2->first == ( * ( partyList.begin() ) )->getCode() )
                    {
                        addStepWidget ( ballotTmp, fileName, language, country, false );
                        generateXml.addBallot ( fileName, language, country, p );
                    }
                }
                else
                {
                    addStepWidget ( ballotTmp, fileName, language, country, false );
                    generateXml.addBallot ( fileName, language, country, p );
                }
            }
        }
    }
}

void VotingExperienceValidationFunctionality::generateCardRemovedCounter ( Smartmatic::SAES::Configuration::ScreenImageConfiguration & screenConfig, std::string & language, std::string & country )
{
    std::string formatName = screenConfig.getFormat();

    if ( !formatName.empty() )
    {
        logger->Debug ( "Generate interrupt voting experience" );
        std::string fileName = Glib::ustring::compose ( formatName, language, country, imageFileSequence );
        InterruptWidget * stepWidget = FactoryVotingWidget::createInterruptWidget();
        stepWidget->forceNotChange();
        stepWidget->showCardCounter();
        stepWidget->setName ( VotingOperation::getWindowName() );
        generateXml.addImage ( fileName, language, country, screenConfig.getType() );
        addStepWidget ( stepWidget, fileName, language, country );
    }
}

void VotingExperienceValidationFunctionality::generateCardRemovedCancelProcess ( Smartmatic::SAES::Configuration::ScreenImageConfiguration & screenConfig, std::string & language, std::string & country )
{
    std::string formatName = screenConfig.getFormat();

    if ( !formatName.empty() )
    {
        logger->Debug ( "Generate interrupt voting experience" );
        std::string fileName = Glib::ustring::compose ( formatName, language, country, imageFileSequence );
        InterruptWidget * stepWidget = FactoryVotingWidget::createInterruptWidget();
        stepWidget->forceNotChange();
        stepWidget->showCancelProcess();
        stepWidget->setName ( VotingOperation::getWindowName() );
        generateXml.addImage ( fileName, language, country, screenConfig.getType() );
        addStepWidget ( stepWidget, fileName, language, country );
    }
}

void VotingExperienceValidationFunctionality::generateInterruptVoting ( Smartmatic::SAES::Configuration::ScreenImageConfiguration & screenConfig, std::string & language, std::string & country )
{
    std::string formatName = screenConfig.getFormat();

    if ( !formatName.empty() )
    {
        logger->Debug ( "Generate interrupt voting experience" );
        std::string fileName = Glib::ustring::compose ( formatName, language, country, imageFileSequence );
        InterruptWidget * stepWidget = FactoryVotingWidget::createInterruptWidget();
        stepWidget->forceNotChange();
        stepWidget->showKeypad();
        stepWidget->setName ( VotingOperation::getWindowName() );
        generateXml.addImage ( fileName, language, country, screenConfig.getType() );
        addStepWidget ( stepWidget, fileName, language, country );
    }
}

void VotingExperienceValidationFunctionality::generateNullVotingExperience ( Smartmatic::SAES::Configuration::ScreenImageConfiguration & screenConfig, std::string & language, std::string & country )
{
    std::string formatName = screenConfig.getFormat();

    if ( !formatName.empty() )
    {
        logger->Debug ( "Generate null voting experience" );
        std::string fileName = Glib::ustring::compose ( formatName, language, country, imageFileSequence );
        InterruptWidget * stepWidget = FactoryVotingWidget::createInterruptWidget();
        stepWidget->showEndVoting();
        stepWidget->setName ( VotingOperation::getWindowName() );
        generateXml.addImage ( fileName, language, country, screenConfig.getType() );
        addStepWidget ( stepWidget, fileName, language, country );
    }
}

void VotingExperienceValidationFunctionality::generateImages ( std::string & language, std::string & country )
{
    VotingExperienceValidationScreensFunctionalityConfiguration::ImageSequence seq = SAESGUIConfigurationManager::getInstance()->GetVotingExperienceValidationScreensFunctionalityConfiguration().getImage();
    VotingExperienceValidationScreensFunctionalityConfiguration::ImageIterator it;

    for ( it = seq.begin();
            it != seq.end();
            ++it )
    {
        ScreenImageConfiguration & screenConfig ( *it );

        switch ( screenConfig.getType() )
        {
            case Smartmatic::SAES::Voting::Screen::BasicImageType::GENERATED_IMAGES_WAITING_CARD:
                generateWaitingScreen ( screenConfig, language, country );
                break;

            case Smartmatic::SAES::Voting::Screen::BasicImageType::GENERATED_IMAGES_READ_CARD:
                generateReadCard ( screenConfig, language, country );
                break;

            case Smartmatic::SAES::Voting::Screen::BasicImageType::GENERATED_IMAGES_CARD_USED:
                generateCardUsed ( screenConfig, language, country );
                break;

            case Smartmatic::SAES::Voting::Screen::BasicImageType::GENERATED_IMAGES_ERROR_CARD:
                generateErrorCard ( screenConfig, language, country );
                break;

            case Smartmatic::SAES::Voting::Screen::BasicImageType::GENERATED_IMAGES_ERROR_CARD_AID:
                generateErrorCardAID ( screenConfig, language, country );
                break;

            case Smartmatic::SAES::Voting::Screen::BasicImageType::GENERATED_IMAGES_ERROR_CARD_PIN:
                generateErrorCardPIN ( screenConfig, language, country );
                break;

            case Smartmatic::SAES::Voting::Screen::BasicImageType::GENERATED_IMAGES_ERROR_CARD_GENERIC:
                generateErrorCardGeneric ( screenConfig, language, country );
                break;

            case Smartmatic::SAES::Voting::Screen::BasicImageType::GENERATED_IMAGES_LANGUAGE:
                generateLanguage ( screenConfig, language, country );
                break;

            case Smartmatic::SAES::Voting::Screen::BasicImageType::GENERATED_IMAGES_LANGUAGE_CONFIRMATION:
                generateLanguageConfirmation ( screenConfig, language, country );
                break;

            case Smartmatic::SAES::Voting::Screen::BasicImageType::GENERATED_IMAGES_CUSTOM_ATTRIBUTE:
                generateCustomAttribute ( screenConfig, language, country );
                break;

            case Smartmatic::SAES::Voting::Screen::BasicImageType::GENERATED_IMAGES_CUSTOM_ATTRIBUTE_CONFIRMATION:
                generateCustomAttributeConfirmation ( screenConfig, language, country );
                break;

            case Smartmatic::SAES::Voting::Screen::BasicImageType::GENERATED_IMAGES_PARTY:
                generateParties ( screenConfig, language, country );
                break;

            case Smartmatic::SAES::Voting::Screen::BasicImageType::GENERATED_IMAGES_PARTY_CONFIRMATION:
                break;

            case Smartmatic::SAES::Voting::Screen::BasicImageType::GENERATED_IMAGES_NULL_PARTY:
                break;

            case Smartmatic::SAES::Voting::Screen::BasicImageType::GENERATED_IMAGES_BALLOT:
                generateBallots ( screenConfig, language, country );
                break;

            case Smartmatic::SAES::Voting::Screen::BasicImageType::GENERATED_IMAGES_PRINT_VOTE:
                generatePrintVote ( screenConfig, language, country );
                break;

            case Smartmatic::SAES::Voting::Screen::BasicImageType::GENERATED_IMAGES_ERROR_PRINT_VOTE:
                generateErrorPrintVote ( screenConfig, language, country );
                break;

            case Smartmatic::SAES::Voting::Screen::BasicImageType::GENERATED_IMAGES_FOLD_VOTE:
                generateEndPrintVote ( screenConfig, language, country );
                break;

            case Smartmatic::SAES::Voting::Screen::BasicImageType::GENERATED_IMAGES_REMOVE_CARD:
                generateRemoveCard ( screenConfig, language, country );
                break;

            case Smartmatic::SAES::Voting::Screen::BasicImageType::GENERATED_IMAGES_CARD_REMOVED:
                generateCardRemoved ( screenConfig, language, country );
                break;

            case Smartmatic::SAES::Voting::Screen::BasicImageType::GENERATED_IMAGES_CARD_REMOVED_COUNTER:
                generateCardRemovedCounter ( screenConfig, language, country );
                break;

            case Smartmatic::SAES::Voting::Screen::BasicImageType::GENERATED_IMAGES_CARD_REMOVED_CANCEL_PROCESS:
                generateCardRemovedCancelProcess ( screenConfig, language, country );
                break;

            case Smartmatic::SAES::Voting::Screen::BasicImageType::GENERATED_IMAGES_INTERRUPT_VOTING_EXPERIENCE:
                generateInterruptVoting ( screenConfig, language, country );
                break;

            case Smartmatic::SAES::Voting::Screen::BasicImageType::GENERATED_IMAGES_NULL_VOTING_EXPERIENCE:
                generateNullVotingExperience ( screenConfig, language, country );
                break;

            case Smartmatic::SAES::Voting::Screen::BasicImageType::GENERATED_IMAGES_PRINT_FOLD_VOTE:
                generatePrintFoldVote ( screenConfig, language, country );
                break;

            case Smartmatic::SAES::Voting::Screen::BasicImageType::GENERATED_IMAGES_TIME_OUT_VOTING_EXPERIENCE:
                generateTimeOut ( screenConfig, language, country );
                break;

            case Smartmatic::SAES::Voting::Screen::BasicImageType::GENERATED_IMAGES_SCAN_VOTE:
                generateScanVoteScreen ( screenConfig, language, country );
                break;

            default:
                break;
        }
    }
}

std::string VotingExperienceValidationFunctionality::FormatTallyPosition ( int position )
{
    std::stringstream out;
    out << position;
    std::string formattedPosition = out.str();

    if ( formattedPosition.size() == 1 )
    {
        formattedPosition.insert ( 0, "000" );
    }
    else if ( formattedPosition.size() == 2 )
    {
        formattedPosition.insert ( 0, "00" );
    }
    else if ( formattedPosition.size() == 3 )
    {
        formattedPosition.insert ( 0, "0" );
    }

    return formattedPosition;
}

int VotingExperienceValidationFunctionality::GetContestClassIndexFromContestCode ( const std::string& contestCode )
{
    Contest_classes::Contest_classSequence contestClassSeq = machineOperationManager->getElectoralConfiguration()->getElection()->getContestClasses().getContest_class();

    for ( unsigned int i = 0; i < contestClassSeq.size(); i++ )
    {
        if ( contestClassSeq[i].getCode() == contestCode )
        {
            return i;
        }
    }

    return 0;
}

void VotingExperienceValidationFunctionality::FormatName ( std::string & fileName )
{
    size_t found = fileName.find ( "|" );
    fileName.erase ( found );

    if ( fileName.size() > 5 )
    {
        fileName.erase ( fileName.begin() + 5, fileName.end() );
    }
}

void VotingExperienceValidationFunctionality::configurePrintScreenManager ( const std::string& imageName )
{
    currentImageFolder = folderPath;
    currentImageName = imageName;
}

void VotingExperienceValidationFunctionality::capturePrintScreen ( )
{
    if ( !Glib::thread_supported() )
    {
        Glib::thread_init();
    }

    if ( m_thread_capture != NULL )
    {
        m_thread_capture->join();
        m_thread_capture = NULL;
    }

    m_thread_capture = Glib::Thread::create ( sigc::mem_fun ( *this, &VotingExperienceValidationFunctionality::threadCapturePrintScreen ), true );
}

void VotingExperienceValidationFunctionality::threadCapturePrintScreen ( )
{
    if ( Smartmatic::System::IO::Directory::getInstance()->DirectoryExists ( currentImageFolder ) == false )
    {
        Smartmatic::System::IO::Directory::getInstance()->Mkdir ( currentImageFolder );
    }

    Smartmatic::Graphics::Capture::PrintScreenManager::getInstance()->capture ( Smartmatic::System::IO::Path::Combine ( currentImageFolder, currentImageName ) );

    signalEndCapturePrintScreen.emit();
}

Smartmatic::Log::ISMTTLog * VotingExperienceValidationFunctionality::getLogger()
{
    return logger;
}

Glib::ustring VotingExperienceValidationFunctionality::getStartLogMessage()
{
    return N_ ( "Smartmatic.SAES.GUI.Functionality.VotingExperienceValidationFunctionality.LogStartFunctionality" );
}

Glib::ustring VotingExperienceValidationFunctionality::getFinishLogMessage()
{
    return N_ ( "Smartmatic.SAES.GUI.Functionality.VotingExperienceValidationFunctionality.LogCloseFunctionality" );
}

void VotingExperienceValidationFunctionality::addStepWidget ( Smartmatic::GUI::Widgets::StepWidget * widget, std::string fileName, std::string & language, std::string & country, bool forceDel )
{
    LanguageStepWidget config;
    config.forceDelete = forceDel;
    config.fileName = fileName;
    config.language = language;
    config.country = country;
    config.widget = widget;
    stepWidgets.push_back ( config );
    logger->Debug ( "file name: " + fileName + " - " + language + "_" + country );
    imageFileSequence++;
}

void VotingExperienceValidationFunctionality::onExitFunctionality()
{
    ExitFunctionality();
}

void VotingExperienceValidationFunctionality::onShutdownFunctionality()
{

}

bool VotingExperienceValidationFunctionality::ShowWindowTitle()
{
    return false;
}

void VotingExperienceValidationFunctionality::refresh()
{
    baseWindow->set_name ( SAESGUIConfigurationManager::getInstance()->GetVotingExperienceWindowName() );
    GraphicDisplayFunctionality::refresh();
}

bool VotingExperienceValidationFunctionality::ForceExecute()
{
    return true;
}
