/**
 * @file voting-experience-validation-functionality.h
 * @brief Header class voting experience validation functionality
 * @date 06/04/2011
 * @author Gerardo Hernandez, Juan Delgado, Yadickson Soto
 */

#ifndef VOTING_EXPERIENCE_VALIDATION_FUNCTIONALITY_H_
#define VOTING_EXPERIENCE_VALIDATION_FUNCTIONALITY_H_

#include <gtkmm.h>
#include <string>
#include <map>
#include <Functionalities/graphic-display-functionality.h>
#include "Operation/Voting/VotingWidgetManager.hxx"
#include <Runtime/Environment/SaesInstallDirectories.hxx>
#include <Log/ISMTTLog.h>
#include <Gui/Widgets/Voting/VotingExperienceValidationWidget.hxx>
#include <Widgets/StepWidget.hxx>
#include <Voting/VotingCenters/voting-centers-schema.hxx>
#include <Operation/Voting/VotingExperienceController.hxx>
#include <Gui/Widgets/Voting/ScreenVotingWidget.hxx>
#include <Gui/Widgets/voting-frame-base.h>
#include <Operation/GenerateXMLImage.h>

namespace Smartmatic
{
    namespace SAES
    {
        namespace GUI
        {
            namespace Functionality
            {

                /**
                 * @class	VotingExperienceValidationFunctionality
                 * @brief	Voting experience validation functionality.
                 * @author	Youssef.tourkmani
                 * @date	01/11/2012
                 */
                class VotingExperienceValidationFunctionality : public GraphicDisplayFunctionality
                {
                    public:

                        /**
                         * @fn	static VotingExperienceValidationFunctionality * VotingExperienceValidationFunctionality::getInstance();
                         * @brief	Gets the current.
                         * @author	Luiggi.mendez
                         * @date	28/04/2011
                         * @return	null if it fails, else.
                         */

                        static VotingExperienceValidationFunctionality * getInstance();

                        /**
                         * @fn	static void VotingExperienceValidationFunctionality::FormatName ( std::string & fileName );
                         * @brief	Format name.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         * @param [in,out]	fileName	Filename of the file.
                         */
                        static void FormatName ( std::string & fileName );

                    protected:

                        /**
                         * @fn	Smartmatic::Log::ISMTTLog * VotingExperienceValidationFunctionality::getLogger();
                         * @brief	Gets the logger.
                         * @author	Luiggi.mendez
                         * @date	28/04/2011
                         * @return	null if it fails, else the logger.
                         */

                        Smartmatic::Log::ISMTTLog * getLogger();

                        /**
                         * @fn	Glib::ustring VotingExperienceValidationFunctionality::getStartLogMessage();
                         * @brief	Gets the start log message.
                         * @author	Luiggi.mendez
                         * @date	28/04/2011
                         * @return	The start log message.
                         */

                        Glib::ustring getStartLogMessage();

                        /**
                         * @fn	Glib::ustring VotingExperienceValidationFunctionality::getFinishLogMessage();
                         * @brief	Gets the finish log message.
                         * @author	Luiggi.mendez
                         * @date	28/04/2011
                         * @return	The finish log message.
                         */

                        Glib::ustring getFinishLogMessage();

                    private:

                        /**
                         * @fn	std::string VotingExperienceValidationFunctionality::FormatTallyPosition ( int position );
                         * @brief	Default constructor.
                         * @author	Luiggi.mendez
                         * @date	28/04/2011
                         * @param	position	The position.
                         * @return	The formatted tally position.
                         */

                        std::string FormatTallyPosition ( int position );

                        /**
                         * @fn	VotingExperienceValidationFunctionality::VotingExperienceValidationFunctionality();
                         * @brief	Default constructor.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         */
                        VotingExperienceValidationFunctionality();

                        static VotingExperienceValidationFunctionality * current_instance;	///< The current instance

                        /**
                         * @fn	virtual VotingExperienceValidationFunctionality::~VotingExperienceValidationFunctionality();
                         * @brief	Finaliser.
                         * @author	Luiggi.mendez
                         * @date	28/04/2011
                         */

                        virtual ~VotingExperienceValidationFunctionality();

                        /**
                         * @fn	void VotingExperienceValidationFunctionality::Dispose();
                         * @brief	Dispose of this object, cleaning up any resources it uses.
                         * @author	Luiggi.mendez
                         * @date	28/04/2011
                         */

                        void Dispose();

                        /**
                         * @fn	void VotingExperienceValidationFunctionality::Initialize();
                         * @brief	Initializes this object.
                         * @author	Luiggi.mendez
                         * @date	28/04/2011
                         */

                        void Initialize();

                        /**
                         * @fn	Gtk::Widget * VotingExperienceValidationFunctionality::GetFrame();
                         * @brief	Gets the frame.
                         * @author	Luiggi.mendez
                         * @date	28/04/2011
                         * @return	null if it fails, else the frame.
                         */

                        Gtk::Widget * GetFrame();

                        /**
                         * @fn	std::string VotingExperienceValidationFunctionality::GetWindowTitle();
                         * @brief	Gets the window title.
                         * @author	Luiggi.mendez
                         * @date	28/04/2011
                         * @return	The window title.
                         */

                        std::string GetWindowTitle();

                        /**
                         * @fn	Smartmatic::SAES::Functionality::Configuration::MenuOptionType VotingExperienceValidationFunctionality::GetMenuOptionType();
                         * @brief	Gets the menu option type.
                         * @author	Luiggi.mendez
                         * @date	28/04/2011
                         * @return	The menu option type.
                         */

                        Smartmatic::SAES::Functionality::Configuration::MenuOptionType GetMenuOptionType();

                    private:

                        /**
                         * @fn	void VotingExperienceValidationFunctionality::Generate();
                         * @brief	Generates this object.
                         * @author	Luiggi.mendez
                         * @date	28/04/2011
                         */
                        void Generate();
                        /**
                         * @fn	void VotingExperienceValidationFunctionality::LoadScreenConfig();
                         * @brief	Load screen config
                         * @author	Yadickson Soto
                         * @date	09/11/2012
                         */
                        void LoadScreenConfig();
                        /**
                         * @fn	bool VotingExperienceValidationFunctionality::LoadElectoralConfiguration();
                         * @brief	Loads the electoral configuration.
                         * @author	Luiggi.mendez
                         * @date	28/04/2011
                         * @return	true if it succeeds, false if it fails.
                         */

                        bool LoadElectoralConfiguration();

                        /**
                         * @fn	void VotingExperienceValidationFunctionality::capturePrintScreen ( );
                         * @brief	Capture print screen.
                         * @author	Luiggi.mendez
                         * @date	28/04/2011
                         */

                        void capturePrintScreen ( );

                        /**
                         * @fn	void VotingExperienceValidationFunctionality::threadCapturePrintScreen ( );
                         * @brief	Capture print screen.
                         * @author	Yadickson Soto
                         * @date	17/07/2013
                         */
                        void threadCapturePrintScreen ( );

                        /**
                         * @fn	void VotingExperienceValidationFunctionality::configurePrintScreenManager ( const std::string& imageName );
                         * @brief	Configure print screen manager.
                         * @author	Luiggi.mendez
                         * @date	28/04/2011
                         * @param	imageName	Name of the image.
                         */

                        void configurePrintScreenManager ( const std::string& imageName );
                        /**
                         * @fn	void VotingExperienceValidationFunctionality::UpdateFixedContests();
                         * @brief	Updated fixed contest.
                         * @author	Juan.Delgado
                         * @date	18/07/2011
                         */
                        void UpdateFixedContests();

                        /**
                         * @fn	void VotingExperienceValidationFunctionality::ClearVariables();
                         * @brief	Clear variables.
                         * @author	Juan.Delgado
                         * @date	18/07/2011
                         */
                        void ClearVariables();

                        /**
                         * @fn	int VotingExperienceValidationFunctionality::GetContestClassIndexFromContestCode ( const std::string& contestClassCode );
                         * @brief	Gets the contest class index from contest class code.
                         * @author	Juan.Delgado
                         * @date	18/07/2011
                         * @param	contestClassCode	contest class code.
                         * @return	Index of contest class.
                         */
                        int GetContestClassIndexFromContestCode ( const std::string& contestClassCode );

                    protected:

                        /**
                         * @fn	void VotingExperienceValidationFunctionality::generateCenter();
                         * @brief	Generates a center.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         */
                        void generateCenter();
                        /**
                         * @fn	virtual void VotingExperienceValidationFunctionality::deleteElements();
                         * @brief	Deletes all elements.
                         * @author	Yadickson Soto
                         * @date	03/04/2012
                         */
                        virtual void deleteElements();
                        /**
                         * @fn	void VotingExperienceValidationFunctionality::saveCurrentCenter();
                         * @brief	method save current center images.
                         * @author	Yadickson Soto
                         * @date	01/10/2012
                         */
                        void saveCurrentCenter();
                        /**
                         * @fn	void VotingExperienceValidationFunctionality::nextVotingCenter();
                         * @brief	method next voting center.
                         * @author	Yadickson Soto
                         * @date	25/04/2012
                         */
                        void nextVotingCenter();
                        /**
                         * @fn	void VotingExperienceValidationFunctionality::nextStepWidget();
                         * @brief	method next step widget.
                         * @author	Yadickson Soto
                         * @date	25/04/2012
                         */
                        void nextStepWidget();
                        /**
                         * @fn	void VotingExperienceValidationFunctionality::onNextStepWidget();
                         * @brief	method next step widget.
                         * @author	Yadickson Soto
                         * @date	25/04/2012
                         */
                        void onNextStepWidget();
                        /**
                         * @fn	void VotingExperienceValidationFunctionality::captureImage();
                         * @brief	method capture image.
                         * @author	Yadickson Soto
                         * @date	25/04/2012
                         */
                        void captureImage();
                        /**
                         * @fn	void VotingExperienceValidationFunctionality::saveImage();
                         * @brief	method save image.
                         * @author	Yadickson Soto
                         * @date	25/04/2012
                         */
                        void saveImage();
                        /**
                         * @fn	void generateWaitingScreen ( Smartmatic::SAES::Configuration::ScreenImageConfiguration & screenConfig, std::string & language, std::string & country );
                         * @brief	Generate waiting screen.
                         * @author	Yadickson Soto
                         * @date	25/04/2012
                         * @param 	screenConfig screen configuration
                         * @param language The language
                         * @param country The country
                         */
                        void generateWaitingScreen ( Smartmatic::SAES::Configuration::ScreenImageConfiguration & screenConfig, std::string & language, std::string & country );
                        /**
                         * @fn	void generateReadCard ( Smartmatic::SAES::Configuration::ScreenImageConfiguration & screenConfig, std::string & language, std::string & country );
                         * @brief	Generate waiting screen for read card.
                         * @author	Yadickson Soto
                         * @date	21/03/2013
                         * @param 	screenConfig screen configuration
                         * @param language The language
                         * @param country The country
                         */
                        void generateReadCard ( Smartmatic::SAES::Configuration::ScreenImageConfiguration & screenConfig, std::string & language, std::string & country );
                        /**
                         * @fn	void generateCardUsed ( Smartmatic::SAES::Configuration::ScreenImageConfiguration & screenConfig, std::string & language, std::string & country );
                         * @brief	Generate waiting screen for card used.
                         * @author	Yadickson Soto
                         * @date	21/03/2013
                         * @param 	screenConfig screen configuration
                         * @param language The language
                         * @param country The country
                         */
                        void generateCardUsed ( Smartmatic::SAES::Configuration::ScreenImageConfiguration & screenConfig, std::string & language, std::string & country );
                        /**
                         * @fn	void generateErrorCard ( Smartmatic::SAES::Configuration::ScreenImageConfiguration & screenConfig, std::string & language, std::string & country );
                         * @brief	Generate waiting screen for error card.
                         * @author	Yadickson Soto
                         * @date	21/03/2013
                         * @param 	screenConfig screen configuration
                         * @param language The language
                         * @param country The country
                         */
                        void generateErrorCard ( Smartmatic::SAES::Configuration::ScreenImageConfiguration & screenConfig, std::string & language, std::string & country );
                        /**
                         * @fn	void generateErrorCardAID ( Smartmatic::SAES::Configuration::ScreenImageConfiguration & screenConfig, std::string & language, std::string & country );
                         * @brief	Generate waiting screen for error card AID.
                         * @author	Yadickson Soto
                         * @date	21/03/2013
                         * @param 	screenConfig screen configuration
                         * @param language The language
                         * @param country The country
                         */
                        void generateErrorCardAID ( Smartmatic::SAES::Configuration::ScreenImageConfiguration & screenConfig, std::string & language, std::string & country );
                        /**
                         * @fn	void generateErrorCardPIN ( Smartmatic::SAES::Configuration::ScreenImageConfiguration & screenConfig, std::string & language, std::string & country );
                         * @brief	Generate waiting screen for error card PIN.
                         * @author	Yadickson Soto
                         * @date	21/03/2013
                         * @param 	screenConfig screen configuration
                         * @param language The language
                         * @param country The country
                         */
                        void generateErrorCardPIN ( Smartmatic::SAES::Configuration::ScreenImageConfiguration & screenConfig, std::string & language, std::string & country );
                        /**
                         * @fn	void generateErrorCardGeneric ( Smartmatic::SAES::Configuration::ScreenImageConfiguration & screenConfig, std::string & language, std::string & country );
                         * @brief	Generate waiting screen for error card generic.
                         * @author	Yadickson Soto
                         * @date	21/03/2013
                         * @param 	screenConfig screen configuration
                         * @param language The language
                         * @param country The country
                         */
                        void generateErrorCardGeneric ( Smartmatic::SAES::Configuration::ScreenImageConfiguration & screenConfig, std::string & language, std::string & country );
                        /**
                         * @fn	void generateLanguage ( Smartmatic::SAES::Configuration::ScreenImageConfiguration & screenConfig, std::string & language, std::string & country );
                         * @brief	Generate languages.
                         * @author	Yadickson Soto
                         * @date	25/04/2012
                         * @param 	screenConfig screen configuration
                         * @param language The language
                         * @param country The country
                         */
                        void generateLanguage ( Smartmatic::SAES::Configuration::ScreenImageConfiguration & screenConfig, std::string & language, std::string & country );
                        /**
                         * @fn	void generateLanguageConfirmation ( Smartmatic::SAES::Configuration::ScreenImageConfiguration & screenConfig, std::string & language, std::string & country );
                         * @brief	Generate language confiration.
                         * @author	Yadickson Soto
                         * @date	20/03/2013
                         * @param 	screenConfig screen configuration
                         * @param language The language
                         * @param country The country
                         */
                        void generateLanguageConfirmation ( Smartmatic::SAES::Configuration::ScreenImageConfiguration & screenConfig, std::string & language, std::string & country );
                        /**
                         * @fn	void generateCustomAttribute ( Smartmatic::SAES::Configuration::ScreenImageConfiguration & screenConfig, std::string & language, std::string & country );
                         * @brief	Generate custom attribute images.
                         * @author	Yadickson Soto
                         * @date	25/04/2012
                         * @param 	screenConfig screen configuration
                         * @param language The language
                         * @param country The country
                         */
                        void generateCustomAttribute ( Smartmatic::SAES::Configuration::ScreenImageConfiguration & screenConfig, std::string & language, std::string & country );
                        /**
                         * @fn	void generateCustomAttributeConfirmation ( Smartmatic::SAES::Configuration::ScreenImageConfiguration & screenConfig, std::string & language, std::string & country );
                         * @brief	Generate custom attribute images.
                         * @author	Yadickson Soto
                         * @date	20/03/2013
                         * @param 	screenConfig screen configuration
                         * @param language The language
                         * @param country The country
                         */
                        void generateCustomAttributeConfirmation ( Smartmatic::SAES::Configuration::ScreenImageConfiguration & screenConfig, std::string & language, std::string & country );
                        /**
                         * @fn	void generatePrintVote ( Smartmatic::SAES::Configuration::ScreenImageConfiguration & screenConfig, std::string & language, std::string & country );
                         * @brief	Generate print vote.
                         * @author	Yadickson Soto
                         * @date	25/04/2012
                         * @param 	screenConfig screen configuration
                         * @param language The language
                         * @param country The country
                         */
                        void generatePrintVote ( Smartmatic::SAES::Configuration::ScreenImageConfiguration & screenConfig, std::string & language, std::string & country );
                        /**
                         * @fn	void generateErrorPrintVote ( Smartmatic::SAES::Configuration::ScreenImageConfiguration & screenConfig, std::string & language, std::string & country );
                         * @brief	Generate error print vote.
                         * @author	Yadickson Soto
                         * @date	25/04/2012
                         * @param 	screenConfig screen configuration
                         * @param language The language
                         * @param country The country
                         */
                        void generateErrorPrintVote ( Smartmatic::SAES::Configuration::ScreenImageConfiguration & screenConfig, std::string & language, std::string & country );
                        /**
                         * @fn	void generateRemoveCard ( Smartmatic::SAES::Configuration::ScreenImageConfiguration & screenConfig, std::string & language, std::string & country );
                         * @brief	Generate remove card.
                         * @author	Yadickson Soto
                         * @date	25/04/2012
                         * @param 	screenConfig screen configuration
                         * @param language The language
                         * @param country The country
                         */
                        void generateRemoveCard ( Smartmatic::SAES::Configuration::ScreenImageConfiguration & screenConfig, std::string & language, std::string & country );
                        /**
                         * @fn	void generateCardRemoved ( Smartmatic::SAES::Configuration::ScreenImageConfiguration & screenConfig, std::string & language, std::string & country );
                         * @brief	Generate card removed.
                         * @author	Yadickson Soto
                         * @date	25/04/2012
                         * @param 	screenConfig screen configuration
                         * @param language The language
                         * @param country The country
                         */
                        void generateCardRemoved ( Smartmatic::SAES::Configuration::ScreenImageConfiguration & screenConfig, std::string & language, std::string & country );
                        /**
                         * @fn	void generateEndPrintVote ( Smartmatic::SAES::Configuration::ScreenImageConfiguration & screenConfig, std::string & language, std::string & country );
                         * @brief	Generate print vote.
                         * @author	Yadickson Soto
                         * @date	25/04/2012
                         * @param 	screenConfig screen configuration
                         * @param language The language
                         * @param country The country
                         */
                        void generateEndPrintVote ( Smartmatic::SAES::Configuration::ScreenImageConfiguration & screenConfig, std::string & language, std::string & country );
                        /**
                         * @fn	void generatePrintFoldVote ( Smartmatic::SAES::Configuration::ScreenImageConfiguration & screenConfig, std::string & language, std::string & country );
                         * @brief	Generate print fold vote.
                         * @author	Yadickson Soto
                         * @date	20/03/2013
                         * @param 	screenConfig screen configuration
                         * @param language The language
                         * @param country The country
                         */
                        void generatePrintFoldVote ( Smartmatic::SAES::Configuration::ScreenImageConfiguration & screenConfig, std::string & language, std::string & country );

                        /**
                         * @fn	void generateTimeOut ( Smartmatic::SAES::Configuration::ScreenImageConfiguration & screenConfig, std::string & language, std::string & country );
                         * @brief	Generate timeout.
                         * @author	Yadickson Soto
                         * @date	02/04/2013
                         * @param 	screenConfig screen configuration
                         * @param language The language
                         * @param country The country
                         */
                        void generateTimeOut ( Smartmatic::SAES::Configuration::ScreenImageConfiguration & screenConfig, std::string & language, std::string & country );

                        /**
                         * @fn	void generateScanVoteScreen ( Smartmatic::SAES::Configuration::ScreenImageConfiguration & screenConfig, std::string & language, std::string & country );
                         * @brief	Generates the scan vote widget which asks the voter to go scan his vote at the end of the voting process.
                         * @author	Yadickson Soto
                         * @date	27/03/2014
                         * @param 	screenConfig screen configuration
                         * @param language The language
                         * @param country The country
                         */
                        void generateScanVoteScreen ( Smartmatic::SAES::Configuration::ScreenImageConfiguration & screenConfig, std::string & language, std::string & country );

                        /**
                         * @fn	void generateParties ( Smartmatic::SAES::Configuration::ScreenImageConfiguration & screenConfig, std::string & language, std::string & country );
                         * @brief	Generates parties
                         * @author	Yadickson Soto
                         * @date	20/03/2013
                         * @param 	screenConfig screen configuration
                         * @param language The language
                         * @param country The country
                         */
                        void generateParties ( Smartmatic::SAES::Configuration::ScreenImageConfiguration & screenConfig, std::string & language, std::string & country );
                        /**
                         * @fn	void generateBallots ( Smartmatic::SAES::Configuration::ScreenImageConfiguration & screenConfig, std::string & language, std::string & country );
                         * @brief	Generates ballots.
                         * @author	Yadickson Soto
                         * @date	20/03/2013
                         * @param 	screenConfig screen configuration
                         * @param language The language
                         * @param country The country
                         */
                        void generateBallots ( Smartmatic::SAES::Configuration::ScreenImageConfiguration & screenConfig, std::string & language, std::string & country );

                        void generateCardRemovedCancelProcess ( Smartmatic::SAES::Configuration::ScreenImageConfiguration & screenConfig, std::string & language, std::string & country );
                        void generateCardRemovedCounter ( Smartmatic::SAES::Configuration::ScreenImageConfiguration & screenConfig, std::string & language, std::string & country );
                        /**
                         * @fn	void generateInterruptVoting ( Smartmatic::SAES::Configuration::ScreenImageConfiguration & screenConfig, std::string & language, std::string & country );
                         * @brief	Generates interrupt voting experience.
                         * @author	Yadickson Soto
                         * @date	21/03/2013
                         * @param 	screenConfig screen configuration
                         * @param language The language
                         * @param country The country
                         */
                        void generateInterruptVoting ( Smartmatic::SAES::Configuration::ScreenImageConfiguration & screenConfig, std::string & language, std::string & country );
                        /**
                         * @fn	void generateNullVotingExperience ( Smartmatic::SAES::Configuration::ScreenImageConfiguration & screenConfig, std::string & language, std::string & country );
                         * @brief	Generates null voting experience.
                         * @author	Yadickson Soto
                         * @date	21/03/2013
                         * @param 	screenConfig screen configuration
                         * @param language The language
                         * @param country The country
                         */
                        void generateNullVotingExperience ( Smartmatic::SAES::Configuration::ScreenImageConfiguration & screenConfig, std::string & language, std::string & country );
                        /**
                         * @fn	void generateImages( std::string & language, std::string & country );
                         * @brief	Generates the dialogs.
                         * @author	Yadickson Soto
                         * @date	20/03/2013
                         * @param language The language
                         * @param country The country
                         */
                        void generateImages ( std::string & language, std::string & country );
                        /**
                         * @fn	void VotingExperienceValidationFunctionality::addStepWidget ( Smartmatic::GUI::Widgets::StepWidget * widget,
                         * 			std::string & language, std::string & country, std::string fileName );
                         * @brief	Add step widget.
                         * @author	Yadickson Soto
                         * @date	15/05/2012
                         * @param	widget		widget.
                         * @param	fileName	file name.
                         * @param language The language
                         * @param country The country
                         * @param forceDel force delete widget
                         */
                        void addStepWidget ( Smartmatic::GUI::Widgets::StepWidget * widget, std::string fileName, std::string & language, std::string & country, bool forceDel = true );
                        /**
                         * @fn	void VotingExperienceValidationFunctionality::showEndResult();
                         * @brief	show end result.
                         * @author	Yadickson Soto
                         * @date	01/07/2012
                         */
                        void showEndResult();
                        /**
                         * @fn	std::string VotingExperienceValidationFunctionality::onExitFunctionality ( );
                         * @brief	On exit functionality.
                         * @author	Yadickson Soto
                         * @date	06/11/2012
                         */
                        void onExitFunctionality();
                        /**
                         * @fn	virtual bool VotingExperienceValidationFunctionality::ShowWindowTitle();
                         * @brief	getter isshow window title.
                         * @author	Yadickson Soto
                         * @date	18/07/2012
                         * @return	true is show window title.
                         */
                        virtual bool ShowWindowTitle();
                        /**
                         * @fn	virtual void VotingExperienceValidationFunctionality::refresh();
                         * @brief	refresh widget method.
                         * @author	Yadickson Soto
                         * @date	09/11/2012
                         */
                        virtual void refresh();
                        /**
                         * @fn	void VotingExperienceValidationFunctionality::onShutdownFunctionality ( );
                         * @brief	On shut down functionality.
                         * @author	Yadickson Soto
                         * @date	07/11/2012
                         */
                        void onShutdownFunctionality();
                        /**
                         * @fn	void VotingExperienceValidationFunctionality::setStepWidget( Smartmatic::GUI::Widgets::StepWidget * widget );
                         * @brief	set widget
                         * @author	Yadickson Soto
                         * @date	09/11/2012
                         * @param widget widget
                         */
                        void setStepWidget ( Smartmatic::GUI::Widgets::StepWidget * widget );

                        /**
                         * @fn	virtual bool ForceExecute();
                         * @brief	Method force execute functionality
                         * @author	Yadickson Soto
                         * @date	16/11/2012
                         * @return	true if force execute
                         */
                        virtual bool ForceExecute();

                    private:

                        std::string folderPath;	///< The folder path
                        std::string sound; ///< screen capture sound

                        std::string currentLanguage; ///< the current language
                        std::string currentCountry; ///< the current country

                        std::list<std::string> fixedContest;	///< The fixed contest

                        std::string currentImageFolder; ///< Pathname of the current image folder
                        std::string currentImageName;   ///< The current image name
                        std::vector<std::string> files; ///< The files
                        int contestClassIndex;	///< The contest class index

                        Smartmatic::SAES::Operation::MachineOperationManager * machineOperationManager; ///< Manager for machine operation
                        Operation::Voting::VotingWidgetManager * VotingWidgetManager;	///< Manager for voting widget
                        Smartmatic::SAES::Runtime::Environment::SaesInstallDirectories * currentDirectories;   ///< The current directories
                        Smartmatic::SAES::Voting::ElectoralConfiguration * electoralConfiguration;  ///< The electoral configuration

                        unsigned long imageFileSequence; ///< The image file sequence
                        unsigned long imageFileByCenter; ///< The image file by centers

                        Smartmatic::SAES::Operation::Voting::VotingExperienceController * votingExperienceController; ///< voting experience controller
                        Smartmatic::SAES::GUI::Widgets::Voting::VotingExperienceValidationWidget * votingExperienceValidationWidget; ///< voting experinece validation widget
                        Smartmatic::SAES::GUI::Widgets::Voting::ScreenVotingWidget * screenVotingWidget; ///< screen voting widget

                        Glib::Dispatcher signalGenerate; ///< signal generate center
                        Glib::Dispatcher signalNextVotingCenter; ///< signal next voting center
                        Glib::Dispatcher signalNextStepWidget; ///< signal next step widget
                        Glib::Dispatcher signalSaveImage; ///< signal save image
                        Glib::Dispatcher signalEndCapturePrintScreen; ///< signal end capture print screen

                        Glib::Thread * m_thread_generate_center; ///< The thread pointer to generate center
                        Glib::Thread * m_thread_capture; ///< The thread pointer to capture print screen

                        /**
                         * @struct	LanguageStepWidget
                         * @brief	Struct language step widget.
                         * @author	Yadickson Soto
                         * @date	15/05/2012
                         */
                        struct LanguageStepWidget
                        {
                            LanguageStepWidget ()
                            {
                                widget = NULL;
                                forceDelete = false;
                            }

                            std::string fileName; ///< file name
                            std::string language; ///< language
                            std::string country; ///< country
                            bool forceDelete; ///< flag force delete
                            Smartmatic::GUI::Widgets::StepWidget * widget; ///< widget
                        };

                        Smartmatic::SAES::Voting::VotingCenters::VotingCenter infoCenter; ///< info center
                        std::vector< Smartmatic::SAES::Voting::VotingCenters::VotingCenter > votingCenters; ///< all voting centers
                        std::vector< LanguageStepWidget > stepWidgets; ///< all step widgets

                        int currentCounter; ///< current cunter
                        int currentCenter; ///< current center
                        bool forceDelete; ///< flag force delete
                        bool checkImageFiles;  ///< flag check image files
                        unsigned int badCount; ///< count bad files
                        unsigned int badTotalCount; ///< count total bad files

                        Smartmatic::GUI::Widgets::StepWidget * container; ///< widget container
                        Smartmatic::GUI::Widgets::StepWidget * currentWidget; ///< current widget
                        Smartmatic::SAES::GUI::Operation::GenerateXMLImage generateXml; ///< generate xml file
                        static Smartmatic::Log::ISMTTLog * logger;  ///< The logger
                };
            }
        }
    }
}

#endif /* VOTING_EXPERIENCE_VALIDATION_FUNCTIONALITY_H_ */
