/**
 * @file   AudioVotingExperienceWidget.cxx
 * @brief  Implementation of AudioVotingExperienceWidget class.
 * @author manuel.arguelles
 * @date   Oct 11, 2017
 */

#include "AudioVotingExperienceWidget.hxx"
#include "Printing/SmartcardRemovedPrintDocument.h"
#include <iostream>
#include <Gui/Widgets/Voting/FactoryVotingWidget.hxx>
#include <Operation/MachineOperationManager.hxx>
#include <Printing/PrinterStatusNotifier.h>
#include "Configuration/s-a-e-s-g-u-i-conf-manager.h"
#include "Configuration/s-a-e-s-functionality-conf-manager.h"
#include <Operation/AlarmMessage.hxx>
#include <Operation/Devices/AlarmController.hxx>
#include <Log/ISMTTLog.h>
#include <Log/SMTTLogManager.h>
#include <Gui/Widgets/AudioVoting/EndAudioVotingSequenceWidget.hxx>
#include <Operation/Devices/CardController.hxx>
#include <Voting/CardData/card-data-schema.hxx>
#include <Voting/Vote/vote-schema.hxx>
#include <Voting/set-language-functionality.h>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#include <libintl.h>
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///< macro for gettext
#else
#define _(String) (String) ///< macro without gettext
#endif

#define N_(String) (String) ///< macro not translate

using namespace Smartmatic::SAES::GUI::Widgets::AudioVoting;
using namespace Smartmatic::SAES::GUI::Widgets::Voting;
using namespace Smartmatic::SAES::Operation;
using namespace Smartmatic::SAES::Operation::Devices;
using namespace Smartmatic::SAES::Printing;
using namespace Smartmatic::SAES::Voting;
using namespace Smartmatic::SAES::Voting::Vote;
using namespace Smartmatic::SAES::GUI::Configuration;
using namespace Smartmatic::SAES::GUI::Operation;
using namespace Smartmatic::Log;
using namespace Smartmatic::SAES::Functionality::Configuration;
using namespace Smartmatic::Environment;
using namespace Smartmatic::SAES::Operation::AudioVoting;

static const std::string CARD_EXTRACTED_TOO_MANY_TIMES_MESSAGE_KEY = _("smartmatic.saes.gui.widgets.AudioVoting.AudioVotingExperienceWidget.CardExtractedTooManyTimesMessage");

ISMTTLog * AudioVotingExperienceWidget::logger = SMTTLogManager::GetLogger ( "Smartmatic.SAES.GUI.Widgets.AudioVotingExperienceWidget", GETTEXT_PACKAGE );

AudioVotingExperienceWidget::AudioVotingExperienceWidget()
{
    std::string imageFile = ResourcePathProvider::getInstance()->GetResourcePath ( PACKAGE_NAME, "ui/speaker.png" );
    m_image.set(imageFile);

    m_audioVotingExperienceController = MachineOperationManager::getInstance()->getAudioVotingExperienceController();
    m_audioVotingExperienceController->getSignalEndAudioVotingExperience().connect(sigc::mem_fun( this, &AudioVotingExperienceWidget::endAudioVotingExperience ) );
    m_audioVotingExperienceController->getSignalLanguageSelected().connect(sigc::mem_fun( this, &AudioVotingExperienceWidget::onLanguageSelected ) );

    m_endAudioVotingSequenceWidget = FactoryVotingWidget::createEndAudioVotingSequenceWidget();
    m_endAudioVotingSequenceWidget->getSignalEnd().connect ( sigc::mem_fun ( this, &AudioVotingExperienceWidget::endVoting ) );
    m_endAudioVotingSequenceWidget->getSignalError().connect ( sigc::mem_fun ( this, &AudioVotingExperienceWidget::errorVoting ) );
    m_endAudioVotingSequenceWidget->getSignalPlayAudio().connect ( sigc::mem_fun ( this, &AudioVotingExperienceWidget::playAudio ) );

    m_cardRemovedWidget = FactoryVotingWidget::createCardRemovedWidget();

    m_interruptWidget = FactoryVotingWidget::createInterruptWidget();
    m_interruptWidget->getSignalEnd().connect ( sigc::mem_fun ( this, &AudioVotingExperienceWidget::nullVoting ) );
}

AudioVotingExperienceWidget::~AudioVotingExperienceWidget()
{
    if ( m_interruptWidget )
    {
        delete m_interruptWidget;
        m_interruptWidget = NULL;
    }
}

void AudioVotingExperienceWidget::dispose()
{
    if ( m_endAudioVotingSequenceWidget )
    {
        delete m_endAudioVotingSequenceWidget;
        m_endAudioVotingSequenceWidget = NULL;
    }

    if ( m_cardRemovedWidget )
    {
        delete m_cardRemovedWidget;
        m_cardRemovedWidget = NULL;
    }
}

void AudioVotingExperienceWidget::start()
{
    m_detectCardExtractionEnable = true;
    m_invalidCard = false;
    m_isInInterrupt = false;
    m_extractionCounter = 0;

    setWidget(&m_image);
}

void AudioVotingExperienceWidget::endVoting()
{
    logger->Debug ( "End voting" );

    PrinterStatusNotifier notifier;
    PrinterStatusNotifier::PhysicalStateEnumType state = notifier.getPrinterState();

    bool neop_state = ( ( state &  PrinterStatusNotifier::PSTATE_NEAR_END_OF_PAPER ) == PrinterStatusNotifier::PSTATE_NEAR_END_OF_PAPER );
    bool is_neop_blocking = SAESGUIConfigurationManager::getInstance ()->getWaitingForCardWindowConfiguration ().getIsNeopBlockingCondition ();

    if ( neop_state && is_neop_blocking )
    {
        AlarmMessage::alarm ( AlarmEnum::CONFIGURATION_PRINTERNEOPAFTERVOTE );
    }

    m_audioVotingExperienceController->dispose();
    emit();
}

void AudioVotingExperienceWidget::errorVoting()
{
    logger->Debug ( "Error voting" );

    m_audioVotingExperienceController->dispose();
    emit();
}

void AudioVotingExperienceWidget::playAudio(const std::string& messageKey)
{
    m_audioVotingExperienceController->playMessage(messageKey);
}

void AudioVotingExperienceWidget::showStepWidget ( StepWidget * widget )
{
    remove();
    widget->refresh();
    setWidget ( widget );
}

bool AudioVotingExperienceWidget::CardIsValid()
{
    CardData::Card_data cardDataCheck;
    bool isGood = false;

    try
    {
        if ( CardController::getInstance()->readCardData ( cardDataCheck ) )
        {
            if ( cardDataCheck.getCard_status().compare ( "" ) == 0 )
            {
                logger->Debug ( "invalid data: card status could not be resolved" );
                isGood = false;
            }
            else
            {
                bool cardUsed = !SAESGUIConfigurationManager::getInstance()->getMarkCardAsUsed () ||
                                ( cardDataCheck.getCard_status() == Smartmatic::SAES::Voting::CardData::Card_status::CARD_DATA_USED ) ||
                                SAESFunctionalityConfigurationManager::getInstance()->getOperationConfiguration().getSimulateVotes();

                if ( cardUsed && cardDataCheck.getActivation_code().compare ( m_audioVotingExperienceController->getActivationCode() ) == 0 )
                {
                    logger->Debug ( "card is Good" );
                    isGood = true;
                }
                else if ( m_audioVotingExperienceController->getVotingMode() != VoteMode::VOTE_S )
                {
                    logger->Debug ( "error: card not used" );
                    isGood = false;
                }
                else
                {
                    logger->Debug ( "Card simulate" );
                    isGood = true;
                }
            }
        }
    }
    catch ( ... )
    {
        isGood = false;
        logger->Debug ( "Error read card data" );
    }

    return isGood;
}

void AudioVotingExperienceWidget::cardInsertEvent()
{
    logger->Debug ( "Card insert event." );

    if (m_audioVotingExperienceController->getCurrentStage() == LANGUAGE_SELECTION)
    {
        m_audioVotingExperienceController->dispose();
        m_signalStartAudioVotingExperience.emit();
        return;
    }

    if (m_detectCardExtractionEnable)
    {
        if ( m_isInInterrupt && m_invalidCard == false )
        {
            bool cardValid = CardIsValid();

            if ( cardValid )
            {
                m_isInInterrupt = false;
                setWidget(&m_image);
                m_audioVotingExperienceController->resume();
            }
            else
            {
                m_invalidCard = true;
                m_interruptWidget->showCancelProcess();
                m_audioVotingExperienceController->cancel();
            }
        }
        else if ( m_isInInterrupt && m_invalidCard )
        {
            bool cardValid = CardIsValid();

            if ( cardValid )
            {
                m_invalidCard = false;
                m_isInInterrupt = false;
                setWidget(&m_image);
                m_audioVotingExperienceController->resume();
            }
        }
    }
}

void AudioVotingExperienceWidget::cardRemoveEvent()
{
    logger->Debug ( "Card remove event." );

    if (m_audioVotingExperienceController->getCurrentStage() == LANGUAGE_SELECTION)
    {
        m_audioVotingExperienceController->dispose();
        m_isInInterrupt = true;
        return;
    }

    if ( m_detectCardExtractionEnable )
    {
        if ( m_isInInterrupt == false )
        {
            m_extractionCounter++;
            unsigned int extractionsBeforeStop = SAESGUIConfigurationManager::getInstance()->getNumberOfExtractionsBeforeStop();

            if ( ( extractionsBeforeStop != 0 ) && ( m_extractionCounter == extractionsBeforeStop ) )
            {
                m_detectCardExtractionEnable = false;
                m_audioVotingExperienceController->dispose();
                m_audioVotingExperienceController->playMessage(CARD_EXTRACTED_TOO_MANY_TIMES_MESSAGE_KEY, false);
                RefreshByTooManyExtracts();
            }
            else
            {
                m_audioVotingExperienceController->interrupt();
                m_isInInterrupt = true;
                const std::string currentLanguageCode = SetLanguageFunctionality::getCurrentLanguageCode();
                m_interruptWidget->showCardCounter();
                m_interruptWidget->refresh();
                setWidget ( m_interruptWidget );
                SetLanguageFunctionality::SetNewVotingLanguage(currentLanguageCode);
            }
        }
    }
}

void AudioVotingExperienceWidget::nullVoting()
{
    m_detectCardExtractionEnable = false;
    showStepWidget ( m_interruptWidget );
    m_interruptWidget->showEndVoting();
    m_invalidCard = false;
    m_isInInterrupt = false;
    m_audioVotingExperienceController->stop();

    unsigned int time = SAESGUIConfigurationManager::getInstance()->getWaitTimeByBasicImageType ( Smartmatic::SAES::Voting::Screen::BasicImageType::GENERATED_IMAGES_NULL_VOTING_EXPERIENCE );
    Glib::signal_timeout().connect_once ( sigc::mem_fun ( *this, &AudioVotingExperienceWidget::endVoting ), time );
}

AudioVotingExperienceWidget::SignalStartAudioVotingExperience AudioVotingExperienceWidget::getSignalStartAudioVotingExperience()
{
    return m_signalStartAudioVotingExperience;
}

AudioVotingExperienceWidget::SignalLanguageSelected AudioVotingExperienceWidget::getSignalLanguageSelected()
{
    return m_signalLanguageSelected;
}

void AudioVotingExperienceWidget::RefreshByTooManyExtracts()
{
    showStepWidget(m_cardRemovedWidget);

    unsigned int time = SAESGUIConfigurationManager::getInstance()->getWaitTimeByBasicImageType ( Smartmatic::SAES::Voting::Screen::BasicImageType::GENERATED_IMAGES_CARD_REMOVED_COUNTER );
    Glib::signal_timeout().connect_once ( sigc::mem_fun ( *this, &AudioVotingExperienceWidget::stopByExtractProcedure ), time );
}

void AudioVotingExperienceWidget::stopByExtractProcedure()
{
    try
    {
        SmartcardRemovedPrintDocument smartcardRemovedPrintDocument (
            Smartmatic::SAES::Operation::MachineOperationManager::getInstance()->getElectoralConfiguration(),
            "",
            m_extractionCounter );
        //smartcardRemovedPrintDocument.setClient( this );
        smartcardRemovedPrintDocument.Print();

        PrintDocumentResult res = smartcardRemovedPrintDocument.getDocumentResult ();

        if ( !res.getSuccess() )
        {
            AlarmMessage::alarm ( AlarmEnum::CONFIGURATION_VOTEPRINTINGFAILURE );
        }
    }
    catch ( Smartmatic::System::Exception::ElectionException & ex )
    {
        std::stringstream ss;
        ss  << "FATAL - " << __func__ << " " << ex.what();
        logger->Debug ( ss.str() );
    }

    Glib::signal_timeout().connect_once ( sigc::mem_fun ( *this, &AudioVotingExperienceWidget::AlarmExtractionCounter ), 100 );
}

void AudioVotingExperienceWidget::AlarmExtractionCounter()
{
    logger->Debug ( "AlarmExtractionCounter" );
    AlarmMessage::alarm ( AlarmEnum::CONFIGURATION_CARDEXTRACTIONCOUNTER );
    m_audioVotingExperienceController->dispose();
    emit();
}

void AudioVotingExperienceWidget::onLanguageSelected()
{
    m_signalLanguageSelected.emit();
}

void AudioVotingExperienceWidget::endAudioVotingExperience()
{
    m_detectCardExtractionEnable = false;
    m_endAudioVotingSequenceWidget->start();
    showStepWidget ( m_endAudioVotingSequenceWidget );
}
