/**
 * @file   AudioVotingExperienceWidget.hxx
 * @brief  Declaration of AudioVotingExperienceWidget class.
 * @author manuel.arguelles
 * @date   Oct 11, 2017
 */

#ifndef AUDIO_VOTING_EXPERIENCE_WIDGET_HXX
#define AUDIO_VOTING_EXPERIENCE_WIDGET_HXX

#include <Widgets/StepWidget.hxx>
#include <Widgets/SmartmaticBasicImage.hxx>
#include <Operation/Voting/AudioVoting/AudioVotingExperienceController.hxx>

namespace Smartmatic
{
    namespace Log
    {
        class ISMTTLog;
    }

    namespace SAES
    {
        namespace GUI
        {
            namespace Widgets
            {
                namespace Voting
                {
                    class EndAudioVotingSequenceWidget;
                    class CardRemovedWidget;
                    class InterruptWidget;
                }

                namespace AudioVoting
                {

                    /**
                     * @class  AudioVotingExperienceWidget
                     * @brief  Class showing that audio voting is in progress.
                     * @author manuel.arguelles
                     * @date   Oct 11, 2017
                     */
                    class AudioVotingExperienceWidget : public Smartmatic::GUI::Widgets::StepWidget
                    {
                        public:

                            typedef sigc::signal<void> SignalStartAudioVotingExperience;
                            typedef sigc::signal<void> SignalLanguageSelected;

                        public:

                            /**
                             * @brief  Creates an instance of the class.
                             * @author manuel.arguelles
                             * @date   Oct 11, 2017
                             */
                            AudioVotingExperienceWidget();

                            /**
                             * @brief  Destroys an instance of the class.
                             * @author manuel.arguelles
                             * @date   Oct 11, 2017
                             */
                            virtual ~AudioVotingExperienceWidget();

                        public:

                            /**
                             * @fn  void AudioVotingExperienceWidget::dispose();
                             * @brief   Performs application-defined tasks associated with freeing, releasing, or resetting
                             *              unmanaged resources.
                             * @author  Eduardo Fernández
                             * @date    23/11/2017
                             */
                            void dispose();

                            /**
                             * @fn  void AudioVotingExperienceWidget::start();
                             * @brief   Method start process.
                             * @author  Eduardo Fernández
                             * @date    23/11/2017
                             */
                            void start();

                            /**
                             * @fn  void AudioVotingExperienceWidget::endAudioVotingExperience();
                             * @brief   End audio voting experience
                             * @author  Eduardo Fernández
                             * @date    23/11/2017
                             */
                            void endAudioVotingExperience();

                            /**
                             * @fn  void AudioVotingExperienceWidget::showStepWidget ( Smartmatic::GUI::Widgets::StepWidget * widget );
                             * @brief   method view step widget.
                             * @author  Eduardo Fernández
                             * @date    09/11/2017
                             * @param   widget  step widget.
                             */
                            void showStepWidget ( Smartmatic::GUI::Widgets::StepWidget * widget );

                            /**
                             * @fn  bool AudioVotingExperienceWidget::CardIsValid();
                             * @brief   card detection: queries if the card is valid.
                             * @author  Eduardo Fernández
                             * @date    23/11/2017
                             * @return  true if it succeeds, false if it fails.
                             */
                            bool CardIsValid();

                            /**
                             * @fn  void AudioVotingExperienceWidget::cardInsertEvent();
                             * @brief   method card insert event.
                             * @author  Eduardo Fernández
                             * @date    23/11/2017
                             */
                            void cardInsertEvent();

                            /**
                             * @fn  void AudioVotingExperienceWidget::cardRemoveEvent();
                             * @brief   method card remove event.
                             * @author  Eduardo Fernández
                             * @date    23/11/2017
                             */
                            void cardRemoveEvent();

                            /**
                             * @fn  void AudioVotingExperienceWidget::nullVoting();
                             * @brief   null voting.
                             * @author  Eduardo Fernández
                             * @date    23/11/2017
                             */
                            void nullVoting();

                            /**
                             * @fn  void AudioVotingExperienceWidget::getSignalStartAudioVotingExperience();
                             * @brief   get signal start audio voting experience.
                             * @author  Eduardo Fernández
                             * @date    27/11/2017
                             */
                            SignalStartAudioVotingExperience getSignalStartAudioVotingExperience();

                            /**
                             * @fn  void AudioVotingExperienceWidget::getSignalLanguageSelected();
                             * @brief   get signal language selected.
                             * @author  Eduardo Fernández
                             * @date    27/11/2017
                             */
                            SignalLanguageSelected getSignalLanguageSelected();

                        protected:

                            /**
                             * @fn  void AudioVotingExperienceWidget::endVoting();
                             * @brief   force end voting.
                             * @author  Eduardo Fernández
                             * @date    23/11/2017
                             */
                            void endVoting();

                            /**
                             * @fn  void AudioVotingExperienceWidget::errorVoting();
                             * @brief   error voting.
                             * @author  Eduardo Fernández
                             * @date    23/11/2017
                             */
                            void errorVoting();

                            /**
                             * @fn  void AudioVotingExperienceWidget::playAudio();
                             * @brief   play audio.
                             * @author  Eduardo Fernández
                             * @date    23/11/2017
                             * @param   messageKey the key of the message to play.
                             */
                            void playAudio(const std::string& messageKey);

                            /**
                             * @fn  void AudioVotingExperienceWidget::RefreshByTooManyExtracts ();
                             * @brief   card detection: Refresh by too many extracts.
                             * @author  Eduardo Fernández
                             * @date    23/11/2017
                             */
                            void RefreshByTooManyExtracts();

                            /**
                             * @fn  void AudioVotingExperienceWidget::stopByExtractProcedure ( );
                             * @brief   card detection: stop by extract procedure.
                             * @author  Eduardo Fernández
                             * @date    23/11/2017
                             */
                            void stopByExtractProcedure();

                            /**
                             * @fn  void AudioVotingExperienceWidget::AlarmExtractionCounter ();
                             * @brief   card detection: Alarm and back.
                             * @author  Eduardo Fernández
                             * @date    23/11/2017
                             */
                            void AlarmExtractionCounter();

                            void onLanguageSelected();

                        private:

                            Smartmatic::GUI::Widgets::SmartmaticBasicImage                  m_image;
                            SAES::Operation::AudioVoting::AudioVotingExperienceController * m_audioVotingExperienceController;
                            GUI::Widgets::Voting::EndAudioVotingSequenceWidget *            m_endAudioVotingSequenceWidget; ///< end voting sequence widget
                            GUI::Widgets::Voting::CardRemovedWidget *                       m_cardRemovedWidget; ///< card removed widget
                            GUI::Widgets::Voting::InterruptWidget *                         m_interruptWidget; ///< interrupt widget
                            static Smartmatic::Log::ISMTTLog *                              logger; ///< The logger
                            bool                                                            m_detectCardExtractionEnable; ///< true if detect card extraction enable
                            bool                                                            m_isInInterrupt; ///< true if is in interrupt
                            bool                                                            m_invalidCard;
                            unsigned int                                                    m_extractionCounter; ///< extraction counter
                            SignalStartAudioVotingExperience                                m_signalStartAudioVotingExperience;
                            SignalLanguageSelected                                          m_signalLanguageSelected;
                    };

                } /* namespace AudioVoting */
            } /* namespace Widgets */
        } /* namespace GUI */
    } /* namespace SAES */
} /* namespace Smartmatic */

#endif /* AUDIO_VOTING_EXPERIENCE_WIDGET_HXX */
