/**
 * @file	CardActivatorEventBox.h
 * @brief	Declares the card activator event box class.
 * @author	Youssef.tourkmani
 * @date	01/11/2012
 */


#ifndef CARDACTIVATOREVENTBOX_H_
#define CARDACTIVATOREVENTBOX_H_

#include <gtkmm.h>
#include <Voting/VotingDevice/voting-devices-schema.hxx>
#include <Widgets/EventboxSelectionWidget.h>
#include <Configuration/label-configuration.h>
#include <Widgets/SmartmaticBasicVBox.hxx>
#include <Operation/Devices/Polling/CardPolling.hxx>

namespace Smartmatic
{
    namespace SAES
    {
        namespace GUI
        {
            namespace Widgets
            {
                /**
                 * @class	CardActivatorEventBox
                 * @brief	Card activator event box.
                 * @author	Youssef.tourkmani
                 * @date	01/11/2012
                 */
                class CardActivatorEventBox : public Smartmatic::GUI::Widgets::SmartmaticBasicVBox
                {
                    public:
                        /**
                         * @fn	CardActivatorEventBox::CardActivatorEventBox ( int width );
                         * @brief	Constructor.
                         * @author	Juan Delgado
                         * @date	01/11/2012
                         * @param	width	of widget.
                         */
                        explicit CardActivatorEventBox ( int width );

                        /**
                         * @fn	virtual CardActivatorEventBox::~CardActivatorEventBox();
                         * @brief	finalizer.
                         * @author	Juan Delgado
                         * @date	01/11/2012
                         */
                        virtual ~CardActivatorEventBox();

                        /**
                         * @fn	std::vector< Smartmatic::SAES::Voting::VotingDevice::Custom_attribute > CardActivatorEventBox::GetCustomAttributes();
                         * @brief	Retur vector with custom attributes selected.
                         * @author	Juan Delgado
                         * @date	01/11/2012
                         * @return	vector with custom attributes selecteds.
                         */
                        std::vector< Smartmatic::SAES::Voting::VotingDevice::Custom_attribute > GetCustomAttributes();

                        /**
                         * @typedef	sigc::signal< void, std::vector< Smartmatic::SAES::Voting::VotingDevice::Custom_attribute > > type_vectorCA_changed
                         * @brief	Signal returns vector of custom attribute selected.
                         */
                        typedef sigc::signal< void, std::vector< Smartmatic::SAES::Voting::VotingDevice::Custom_attribute > > type_vectorCA_changed;

                        /**
                         * @typedef sigc::signal<void> CustomAttributeSelectedSignal;
                         * @brief   Signal emitted when selecting a voter type.
                         */
                        typedef sigc::signal<void> CustomAttributeSelectedSignal;

                        /**
                         * @fn	type_vectorCA_changed CardActivatorEventBox::signal_vectorCustomAttribute_changed();
                         * @brief	Signal to receive when a custom attribute change.
                         * @author	Juan Delgado
                         * @date	01/11/2012
                         * @return	connection signal.
                         */
                        type_vectorCA_changed signal_vectorCustomAttribute_changed();

                        /**
                         * @fn  CustomAttributeSelectedSignal CardActivatorEventBox::getCustomAttributeSelectedSignal();
                         * @brief   Returns custom attribute selected signal.
                         * @author  Eduardo Fernández
                         * @date    04/01/2018
                         * @return  Custom attribute selected signal.
                         */
                        CustomAttributeSelectedSignal getCustomAttributeSelectedSignal();

                        /**
                         * @fn	void CardActivatorEventBox::setFont ( Smartmatic::GUI::Configuration::LabelConfiguration & titleLabelConfiguration,
                         * 			Smartmatic::GUI::Configuration::LabelConfiguration & selectedLabelConfiguration,
                         * 			Smartmatic::GUI::Configuration::LabelConfiguration & unselectedLabelConfiguration );
                         * @brief	Sets a font.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         * @param [in,out]	titleLabelConfiguration				The title label configuration.
                         * @param [in,out]	selectedLabelConfiguration		The selected label configuration.
                         * @param [in,out]	unselectedLabelConfiguration	The unselected label configuration.
                         */
                        void setFont ( Smartmatic::GUI::Configuration::LabelConfiguration & titleLabelConfiguration,
                                       Smartmatic::GUI::Configuration::LabelConfiguration & selectedLabelConfiguration,
                                       Smartmatic::GUI::Configuration::LabelConfiguration & unselectedLabelConfiguration );

                        /**
                         * @fn	void CardActivatorEventBox::reset ();
                         * @brief	Method reset to default select
                         * @author	Yadickson Soto
                         * @date	13/11/2012
                         */
                        void reset ();

                        /**
                         * @fn  void CardActivatorEventBox::reconnectEventBox();
                         * @brief   Method to reconnect the user event on the event boxes.
                         * @author  Eduardo Fernández
                         * @date    04/01/2018
                         */
                        void reconnectEventBox();

                        /**
                         * @fn  void CardActivatorEventBox::disconnectEventBox();
                         * @brief   Method to disconnect the user event on the event boxes.
                         * @author  Eduardo Fernández
                         * @date    08/01/2018
                         */
                        void disconnectEventBox();

                    protected:

                        type_vectorCA_changed m_type_vector_changed;	///< The type vector changed

                    private:
                        /**
                         * @fn	void CardActivatorEventBox::on_EventboxClicked ( std::list< std::string > valuesSelected );
                         * @brief	handles eventBox clicked event.
                         * @author	Juan Delgado
                         * @date	01/11/2012
                         * @param	valuesSelected	of values selected.
                         */
                        void on_EventboxClicked ( std::list< std::string > valuesSelected );

                        /**
                         * @fn  void CardActivatorEventBox::on_SignalDisconnectionEvent();
                         * @brief   handles eventBox signal disconnection event.
                         * @author  Eduardo Fernández
                         * @date    05/01/2018
                         * @param   initializing Whether the event box is being initialized.
                         */
                        void on_SignalDisconnectionEvent(const bool initializing);

                        /**
                         * @fn  void onSignalCardChange( Smartmatic::SAES::Operation::Devices::CardPolling::CardPollingStatus status );
                         * @brief   Method on signal card change
                         * @author  Eduardo Fernández
                         * @date    05/01/2018
                         * @param   status  card status
                         */
                        void onSignalCardChange(Smartmatic::SAES::Operation::Devices::CardPolling::CardPollingStatus status);

                        std::list < Smartmatic::SAES::Voting::VotingDevice::Custom_attribute_definition > listCAD;	///< The list of custom attributes definition
                        std::vector<Smartmatic::SAES::Voting::VotingDevice::Custom_attribute> attributes;  ///< The attributes
                        Smartmatic::GUI::Widgets::EventboxSelectionWidget * eventboxSelectionWidget;	///< The eventbox selection widget

                        CustomAttributeSelectedSignal m_customAttributeSelectedSignal;
                        sigc::connection m_eventBoxConnection;
                };
            }
        }
    }
}

#endif /* CARDACTIVATOREVENTBOX_H_ */
