/**
 * @file	ClosingElectionStatusWidget.cc
 * @brief	ClosingElectionStatusWidget class.
 * @author	eduardo.fernandez
 * @date	10/12/2018
 */


#include <Gui/Widgets/ClosingElectionStatusWidget.h>
#include <Operation/MachineOperationManager.hxx>
#include <System/IO/File.h>
#include "Environment/Configurations/resource-path-provider.h"
#include <Log/ISMTTLog.h>
#include <System/Exception/CryptoException.hxx>
#include <System/Exception/SignedException.hxx>
#include <System/Exception/XmlException.hxx>
#include <Log/SMTTLogManager.h>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#include <libintl.h>
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

#define N_(String) (String) ///<macro for gettext not translate

using namespace Smartmatic::SAES::Operation;
using namespace Smartmatic::SAES::GUI::Widgets;
using namespace Smartmatic::Environment;
using namespace Smartmatic::System::IO;
using namespace Smartmatic::System::Exception;
using namespace Smartmatic::Log;

Smartmatic::Log::ISMTTLog * ClosingElectionStatusWidget::logger = Smartmatic::Log::SMTTLogManager::GetLogger ( "Smartmatic.SAES.GUI.Widgets.ClosingElectionStatusWidget", GETTEXT_PACKAGE );

ClosingElectionStatusWidget::ClosingElectionStatusWidget ( ProgressWidget * progress )
    : WizardWidgetBase(), progress ( progress )
{
    m_thread = NULL;

    this->progress->setImage ( "ui/close-election.gif" );
    this->progress->setInfoText ( N_ ( "Start of election closing" ) );
    setWidget ( progress );

    setAccessibleName ( "ClosingElectionStatusWidget" );
}

ClosingElectionStatusWidget::~ClosingElectionStatusWidget()
{
    dispose();

    this->remove();

    if ( progress )
    {
        delete progress;
        progress = NULL;
    }
}

void ClosingElectionStatusWidget::initialize()
{
    m_disable_close_signal.emit();

    if ( !Glib::thread_supported() )
    {
        Glib::thread_init();
    }

    if ( m_thread != NULL )
    {
        m_thread->join();
        m_thread = NULL;
    }

    m_thread = Glib::Thread::create ( sigc::mem_fun ( *this, &ClosingElectionStatusWidget::start ), true );
}

void ClosingElectionStatusWidget::dispose()
{
    if ( m_thread != NULL )
    {
        m_thread->join();
        m_thread = NULL;
    }
}

void ClosingElectionStatusWidget::start()
{
    bool errorHappen = true;

    try
    {
        MachineOperationManager::getInstance()->startClosingElection();
        errorHappen = false;
    }
    catch ( CryptoException & ex )
    {
        logger->Debug ( "ClosingElectionStatusWidget CryptoException" );
    }
    catch ( SignedException & ex )
    {
        logger->Debug ( "ClosingElectionStatusWidget SignedException" );
    }
    catch ( XmlException & ex )
    {
        logger->Debug ( "ClosingElectionStatusWidget XmlException" );
    }
    catch ( ... )
    {
        logger->Debug ( "ClosingElectionStatusWidget Error" );
    }

    if ( !errorHappen )
        Glib::signal_timeout().connect_once ( sigc::mem_fun ( *this, &ClosingElectionStatusWidget::end ), 500 );
    else
        emitError();
}

void ClosingElectionStatusWidget::end()
{
    emit();
}

void ClosingElectionStatusWidget::updateLabel ( std::string text )
{
    progress->hideInfoTitle();
    progress->hideImage();
    progress->setInfoText ( text );
    m_stop_signal.emit();
}

bool ClosingElectionStatusWidget::isEnabledButtonExitWizard()
{
    return false;
}
