/**
 * @file	ConsolidationStatsWidget.h
 * @brief	Declares the consolidation stats widget class.
 * @author	Youssef.tourkmani
 * @date	01/11/2012
 */


#ifndef CONSOLIDATIONSTATSWIDGET_H_
#define CONSOLIDATIONSTATSWIDGET_H_

#include <gtkmm.h>
#include <Log/ISMTTLog.h>

#include <System/SafeBuffer.h>
#include <Voting/Vote/vote-schema.hxx>
//#include <Voting/PBReports/VoteStatisticsWrapper.hxx>
#include <Widgets/StatisticWidget.h>
#include <Configuration/SaesGUIConfiguration-schema.hxx>
#include <Operation/Voting/VotePersistence.hxx>
#include <Operation/Monitor/DeviceTypeMonitor.hxx>
#include <Operation/Devices/EurnController.hxx>
#include "Voting/DetailedStatistics.hxx"
#include <Widgets/SmartmaticBasicVBox.hxx>
#include <Widgets/SmartmaticBasicLabel.hxx>
#include <Widgets/SmartmaticBasicImage.hxx>
#include <Widgets/SmartmaticBasicEventBox.hxx>

#define ENABLE_EXTENDED_STATISTICS	1

namespace Smartmatic
{
    namespace SAES
    {
        namespace Voting
        {
            namespace PBReports
            {
                class VoteStatisticsWrapper;
            }
        }

        namespace GUI
        {
            namespace Widgets
            {
                /**
                 * @class	ConsolidationStatsWidget
                 * @brief	Consolidation stats widget.
                 * @author	Youssef.tourkmani
                 * @date	01/11/2012
                 */
                class ConsolidationStatsWidget : public Smartmatic::GUI::Widgets::SmartmaticBasicVBox
                {
                    public:
                        /**
                         * @fn	ConsolidationStatsWidget::ConsolidationStatsWidget ( Smartmatic::SAES::Configuration::CountConfiguration & totalCountType,
                         * 			Smartmatic::SAES::Configuration::CountConfiguration & officialCountType );
                         * @brief	Constructor.
                         * @author	Juan Delgado
                         * @date	01/11/2012
                         * @param [in,out]	totalCountType	 	Type of the total count.
                         * @param [in,out]	officialCountType	Type of the official count.
                         */
                        ConsolidationStatsWidget ( Smartmatic::SAES::Configuration::CountConfiguration & totalCountType,
                                                   Smartmatic::SAES::Configuration::CountConfiguration & officialCountType );

                        /**
                         * @fn	virtual ConsolidationStatsWidget::~ConsolidationStatsWidget();
                         * @brief	Finializer.
                         * @author	Juan Delgado
                         * @date	01/11/2012
                         */
                        virtual ~ConsolidationStatsWidget();

                        /**
                         * @fn	void ConsolidationStatsWidget::Initialize();
                         * @brief	Initialize widget.
                         * @author	Juan Delgado
                         * @date	01/11/2012
                         */
                        void Initialize();

                        /**
                         * @fn	void ConsolidationStatsWidget::Dispose();
                         * @brief	Dispose widget.
                         * @author	Juan Delgado
                         * @date	01/11/2012
                         */
                        void Dispose();

                        /**
                         * @fn	void ConsolidationStatsWidget::SetOneMemoryMode ( bool oneMemory );
                         * @brief	Set one memory mode operation.
                         * @author	Juan Delgado
                         * @date	01/11/2012
                         * @param	oneMemory	if only one memory.
                         */
                        void SetOneMemoryMode ( bool oneMemory );

                        /**
                         * @typedef	sigc::signal<void, bool> signal_void_bool
                         * @brief	Defines an alias representing the type signal_barcodeConnected.
                         */
                        typedef sigc::signal<void, bool> signal_void_bool;

                        /**
                         * @fn	signal_void_bool ConsolidationStatsWidget::barcode_SendCommands();
                         * @brief	Signal that handles the barcode commands send.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         * @return	signal connection.
                         */
                        signal_void_bool barcode_SendCommands();

                        /**
                         * @fn	void ConsolidationStatsWidget::ContinueOperation();
                         * @brief	Continue operation.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         */
                        void ContinueOperation();

                        /**
                         * @fn	void ConsolidationStatsWidget::StopOperation();
                         * @brief	Stop operation.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         */
                        void StopOperation();

                        /**
                         * @fn	void ConsolidationStatsWidget::pauseOperation();
                         * @brief	Pause operation.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         */
                        void pauseOperation();

                        /**
                         * @fn	void ConsolidationStatsWidget::resumeOperation();
                         * @brief	Resume operation.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         */
                        void resumeOperation();

                        /**
                         * @fn	void ConsolidationStatsWidget::UpdateMemoryPresence();
                         * @brief	Updates the memory presence.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         */
                        void UpdateMemoryPresence();

                        /**
                         * @fn	void ConsolidationStatsWidget::UpdateBarcodePresence();
                         * @brief	Updates the barcode presence.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         */
                        void UpdateBarcodePresence();

                        /**
                         * @fn	void ConsolidationStatsWidget::updateEurnStatus();
                         * @brief	Updates the Eurn status.
                         * @author	Juan Valera
                         * @date	30/10/2017
                         */
                        void updateEurnStatus(Smartmatic::SAES::Operation::Devices::EurnController::DeviceStatus status);

                    private:

                        signal_void_bool m_signal_sendCommands; ///< The signal send commands

                        /**
                         * @fn	void ConsolidationStatsWidget::BarcodeCommandsSent ( bool commandsOk );
                         * @brief	Barcode commands sent.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         * @param	commandsOk	true if the commands operation was a success, false if it failed.
                         */
                        void BarcodeCommandsSent ( bool commandsOk );

                        /**
                         * @fn	void ConsolidationStatsWidget::BarcodeConnected();
                         * @brief	Barcode connected.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         */
                        void BarcodeConnected();

                        /**
                         * @fn	void ConsolidationStatsWidget::BarcodeDisconnected();
                         * @brief	Barcode disconnected.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         */
                        void BarcodeDisconnected();

                        /**
                         * @fn	bool ConsolidationStatsWidget::UpdateInstructionLabel();
                         * @brief	Updates the instruction label.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         * @return	true if it succeeds, false if it fails.
                         */
                        bool UpdateInstructionLabel();

                        /**
                         * @fn	void ConsolidationStatsWidget::UpdateStats();
                         * @brief	Updates the stats.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         */
                        void UpdateStats();

                        /**
                         * @fn	void ConsolidationStatsWidget::ShowNormalState();
                         * @brief	Shows the normal state.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         */
                        void ShowNormalState();

                        /**
                         * @fn	void ConsolidationStatsWidget::ConnectSignals();
                         * @brief	Connects the signals.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         */
                        void ConnectSignals();

                        /**
                         * @fn	void ConsolidationStatsWidget::DisconnectSignals();
                         * @brief	Disconnects the signals.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         */
                        void DisconnectSignals();

                        /**
                         * @fn	std::string ConsolidationStatsWidget::getValue ( Smartmatic::SAES::Configuration::CountConfiguration::CountTypeSequence & seq );
                         * @brief	Gets a value.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         * @param [in,out]	seq	The sequence.
                         * @return	The value.
                         */
                        std::string getValue ( Smartmatic::SAES::Configuration::CountConfiguration::CountTypeSequence & seq );
                        /**
                         * @fn	void ConsolidationStatsWidget::onSignalState ( Smartmatic::SAES::Operation::Voting::VotePersistence::PersistenceStatusVote status );
                         * @brief	Callback signal vote persisted, if success update votes count
                         * @author	Juan Valera
                         * @date	26/01/2017
                         * @param	status	status vote process.
                         */
                        void onSignalState ( Smartmatic::SAES::Operation::Voting::VotePersistence::PersistenceStatusVote status );
                        /**
                         * @fn	void ConsolidationStatsWidget::onSignalEurnStatus ( Smartmatic::SAES::Operation::Devices::EurnController::DeviceStatus status );
                         * @brief	Callback signal Eurn status changed
                         * @author	Juan Valera
                         * @date	30/10/2017
                         * @param	status	status Eurn.
                         */
                        void onSignalEurnStatus ( Smartmatic::SAES::Operation::Devices::EurnController::DeviceStatus status );

                        /**
                         * @fn	void deviceChange( Smartmatic::SAES::Common::Operation::Monitor::DeviceTypeMonitor * deviceTypeMonitor );
                         * @brief	getter device monitor change
                         * @author	Yadickson Soto
                         * @date	14/12/2012
                         * @param 	deviceTypeMonitor device change
                         */
                        void deviceChange ( Smartmatic::SAES::Common::Operation::Monitor::DeviceTypeMonitor * deviceTypeMonitor );
                        /**
                         * @fn	bool onPressSafeEject ( GdkEventButton * event );
                         * @brief	Callback signal button press in safe eject image, change image to over
                         * @author	Yadickson Soto
                         * @date	18/06/2013
                         * @param 	event gdk event button
                         * @return false
                         */
                        bool onPressSafeEject ( GdkEventButton * event );
                        /**
                         * @fn	bool onPressSafeEject ( GdkEventButton * event );
                         * @brief	Callback signal button release in safe eject image, change to normal image
                         * @author	Yadickson Soto
                         * @date	18/06/2013
                         * @param 	event gdk event button
                         * @return false
                         */
                        bool onReleaseSafeEject ( GdkEventButton * event );

                        sigc::connection signalChange; ///< The signal change device
#if	!ENABLE_EXTENDED_STATISTICS
                        Smartmatic::GUI::Widgets::StatisticWidget * statsWidget; ///< The card image
#else
                        Voting::DetailedStatistics * detailedStatistics;
#endif
                        Smartmatic::GUI::Widgets::SmartmaticBasicLabel * infoScanVoteLabel; ///< The info scan vote label
                        Smartmatic::GUI::Widgets::SmartmaticBasicLabel * instructionLabel; ///< The instruction label
                        Smartmatic::GUI::Widgets::SmartmaticBasicLabel * barcodeLabel; ///< The barcode label
                        Smartmatic::GUI::Widgets::SmartmaticBasicLabel * eurnLabel; ///< The Eurn label
                        Smartmatic::GUI::Widgets::SmartmaticBasicLabel * memoryLabel; ///< The memory label
                        Smartmatic::GUI::Widgets::SmartmaticBasicLabel * safeEjectLabel; ///< The safe eject label
                        Smartmatic::GUI::Widgets::SmartmaticBasicImage * barcodeImage; ///< The barcode image
                        Smartmatic::GUI::Widgets::SmartmaticBasicImage * eurnImage; ///< The Eurn image
                        Smartmatic::GUI::Widgets::SmartmaticBasicImage * memoryImage; ///< The memory image
                        Smartmatic::GUI::Widgets::SmartmaticBasicImage * safeEjectImage; ///< The safe eject image
                        Gtk::VBox * vbox; ///< The veBarcodeInParamsrtical box
                        Smartmatic::GUI::Widgets::SmartmaticBasicEventBox * safeEjectEventBox; ///< Safe eject even box

                        bool consolidateVoteWithOneMemory;///< True if consolidate with one memory
                        bool oneMemory;///< True if only one memory
                        bool barcodeConnected;///< True if barcode is connected
                        bool disposeExecuted;///< True if dispose was executed
                        bool sendingCommands;///< True if sending commands
                        bool stopOperationAfterSend;///< True if stop operation after send
                        bool m_isSafeEjectRunning;

                        sigc::connection signalStatusVote;	///< The signal status vote

                        std::string eurnConnectedAndWorkingImage;///< The eurn connected working image path
                        std::string eurnConnectedAndNotWorkingImage;///< The eurn connected not working image path
                        std::string eurnNotConnectedImage;///< The eurn not connected image path

                        std::string barcodeConnectedImage;///< The barcode connected image path
                        std::string barcodeNotConnectedImage;///< The barcode not connected image path
                        std::string oneMemoryConnectedImage;///< The one memory connected image path
                        std::string twoMemoryConnectedImage;///< The two memory connected image path

                        std::string oneMemorySafeEjectImage;///< The one memory safe eject image path
                        std::string twoMemorySafeEjectImage;///< The two memory safe eject image path
                        std::string twoMemorySafeEjectOverImage;///< The two memory safe eject over image path

                        sigc::connection conn_barcodeCommandsSend;///< The connection for barcode commands send
                        sigc::connection conn_updateLabelInstruction;///< The connection for update label instruction
                        sigc::connection conn_updateEurnStatus;///< The connection for update Eurn status

                        Smartmatic::SAES::Voting::PBReports::VoteStatisticsWrapper * voteWrapper; ///< The vote stats
                        Glib::Mutex mutex;///< The mutex
                        Glib::Mutex windowMutex;///< The window mutex
                        Glib::Mutex popupMutex;///< The window mutex

                        Smartmatic::SAES::Configuration::CountConfiguration totalCountType; ///< Type of the total count
                        Smartmatic::SAES::Configuration::CountConfiguration officialCountType;	///< Type of the official count
                        static Smartmatic::Log::ISMTTLog * logger; /**<logger*/ ///< The logger

                };
            }
        }
    }
}

#endif /* CONSOLIDATIONSTATSWIDGET_H_ */
