/**
 * @file AbleDiagnosticWidget.cxx
 * @brief Body class able diagnostic widget
 * @date 05/09/2012
 * @author Yadickson Soto
 */

#include "AbleDiagnosticWidget.hxx"
#include <Log/ISMTTLog.h>
#include <Log/SMTTLogManager.h>
#include "Configuration/label-configuration.h"
#include "Configuration/s-a-e-s-g-u-i-conf-manager.h"
#include <Operation/MachineOperationManager.hxx>
#include <Configuration/device-type-schema.hxx>
#include <System/OptParser.hxx>
#include <Configuration/MachineModelEnum.hxx>
#include <System/Utils/StringUtils.hxx>
#include <Operation/Voting/AudioVoting/AudioVotingCheckUtility.hxx>
#include <Operation/Devices/A4517AbledController.hxx>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#include <libintl.h>
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

#define N_(String) (String) ///<macro for gettext not translate

using namespace Smartmatic::Configuration;
using namespace Smartmatic::SAES::GUI::Widgets;
using namespace Smartmatic::GUI::Widgets;
using namespace Smartmatic::SAES::Voting;
using namespace Smartmatic::GUI::Configuration;
using namespace Smartmatic::SAES::GUI::Configuration;
using namespace Smartmatic::Log;
using namespace Smartmatic::SAES::Operation;
using namespace Smartmatic::SAES::Operation::AudioVoting;
using namespace Smartmatic::SAES::Operation::Devices;
using namespace Smartmatic::SAES::Voting::Status;
using namespace Smartmatic::System;
using namespace Smartmatic::SAES::Common::Configuration;
using namespace Smartmatic::System::Utils;

static const std::string ABLED_NOT_DETECTED_MESSAGE      = "NOT DETECTED";
static const std::string ABLED10_RESULT_OK_IMAGE_PATH    = "ui/Able10DiagnosticResultOk.png";
static const std::string ABLED10_RESULT_ERROR_IMAGE_PATH = "ui/Able10DiagnosticResultError.png";
static const std::string ABLED20_RESULT_OK_IMAGE_PATH    = "ui/Able20DiagnosticResultOk.png";
static const std::string ABLED20_RESULT_ERROR_IMAGE_PATH = "ui/Able20DiagnosticResultError.png";
static const size_t      TIME_TO_DISPLAY_DIAGNOSE_RESULT = 5000;
static const int         GET_SERIAL_MAX_RETRIES          = 5;

ISMTTLog * AbleDiagnosticWidget::logger = SMTTLogManager::GetLogger ( "Smartmatic.SAES.GUI.Widgets.AbleDiagnosticWidget", GETTEXT_PACKAGE );

/**
 * @fn static bool AbleDiagnosticWidget::abledSerialNumberIsValid(const std::string &abledSerialNumber)
 * @brief  Checks whether given able-d serial number is valid.
 * @param  abledSerialNumber Able-d serial number to validate
 * @return Whether given able-d serial number is valid.
 * @author eduardo.fernandez
 * @date   Feb 19, 2018
 */
static bool abledSerialNumberIsValid(std::string &abledSerialNumber)
{
    bool validSerialNumber = true;

    for (size_t i = 0; i < abledSerialNumber.size(); ++i)
    {
        const char currentChar = abledSerialNumber[i];

        if (!(((currentChar >= 'a') && (abledSerialNumber[i] <= 'z')) ||
            ((abledSerialNumber[i] >= 'A') && (abledSerialNumber[i] <= 'Z')) ||
            ((abledSerialNumber[i] >= '0') && (abledSerialNumber[i] <= '9'))))
        {
            validSerialNumber = false;
            break;
        }
    }

    return validSerialNumber;
}

AbleDiagnosticWidget::AbleDiagnosticWidget ()
    : BasicDiagnosticWidget ( true, true )
{
    progressWidget = new ProgressWidget();
    ableButtonWidget = new AbleButtonDiagnosticWidget();
    ableDiagnosticSoundWidget = new AbleDiagnosticSoundWidget();
    ableDiagnosticSoundWidget->getSignalOk().connect(sigc::mem_fun(*this, &AbleDiagnosticWidget::endDiagnosis));
    ableDiagnosticSoundWidget->getSignalCancel().connect(sigc::mem_fun(*this, &AbleDiagnosticWidget::endError));
    LabelConfiguration diagConf = SAESGUIConfigurationManager::getInstance()->GetDiagnosisLabelConfiguration();
    setOkButtonSensitive(false);
    signalCancelDetected.connect ( sigc::mem_fun ( *this, &AbleDiagnosticWidget::endError ) );
    signalOkDetected.connect ( sigc::mem_fun ( *this, &AbleDiagnosticWidget::endDiagnosis ) );
    ableButtonWidget->getSignalAllButtonsPressed().connect ( sigc::mem_fun ( *this, &AbleDiagnosticWidget::enableOkButton ) );
    setWidget ( ableButtonWidget );
    audioVotingCheckUtility = MachineOperationManager::getInstance()->getAudioVotingCheckUtility();
    votingMachineStatusWrapper = MachineOperationManager::getInstance()->getOperationStatus()->getVotingMachineStatusWrapper();
}

AbleDiagnosticWidget::~AbleDiagnosticWidget()
{
    Dispose();
    this->remove();

    if ( ableButtonWidget )
    {
        delete ableButtonWidget;
        ableButtonWidget = NULL;
    }

    if ( ableDiagnosticSoundWidget )
    {
        delete ableDiagnosticSoundWidget;
        ableDiagnosticSoundWidget = NULL;
    }

    if ( progressWidget )
    {
        delete progressWidget;
        progressWidget = NULL;
    }
}

void AbleDiagnosticWidget::initialize()
{
    soundTestPerformed = false;

    if (audioVotingCheckUtility.ableIsConnected(true))
    {
        status = true;

        if (audioVotingCheckUtility.getAbledConnected() == AudioVotingCheckUtility::ABLED20)
        {
            std::string abledSerialNumber;

            A4517AbledController::getInstance()->enableNotifications(false);

            for (int i = 0; i < GET_SERIAL_MAX_RETRIES; ++i)
            {
                abledSerialNumber = A4517AbledController::getInstance()->getSerialNumber();

                if (abledSerialNumberIsValid(abledSerialNumber))
                {
                    break;
                }
                else
                {
                    abledSerialNumber = ABLED_NOT_DETECTED_MESSAGE;
                }
            }

            votingMachineStatusWrapper->setAbled20SerialDetected(abledSerialNumber);
            A4517AbledController::getInstance()->enableNotifications(true);
        }

        setWidget ( ableButtonWidget );
        ableButtonWidget->initialize();
    }
    else
    {
        votingMachineStatusWrapper->setStatusAbledNotDiagnosed();
        emit();
    }
}

void AbleDiagnosticWidget::Dispose()
{
    ableButtonWidget->dispose();
}

void AbleDiagnosticWidget::endError()
{
    status = false;

    if (soundTestPerformed == false)
    {
        soundTestPerformed = true;
    }

    endDiagnosis();
}

void AbleDiagnosticWidget::enableOkButton()
{
    setOkButtonSensitive(true);
}

void AbleDiagnosticWidget::setStatusAbledOk()
{
    if (audioVotingCheckUtility.getAbledConnected() == AudioVotingCheckUtility::ABLED20)
    {
        votingMachineStatusWrapper->addStatusAbled20Ok();
    }
    else
    {
        votingMachineStatusWrapper->addStatusAbled10Ok();
    }
}

void AbleDiagnosticWidget::setStatusAbledError()
{
    if (audioVotingCheckUtility.getAbledConnected() == AudioVotingCheckUtility::ABLED20)
    {
        votingMachineStatusWrapper->addStatusAbled20Error();
    }
    else
    {
        votingMachineStatusWrapper->addStatusAbled10Error();
    }
}

void AbleDiagnosticWidget::setStatusAbledSoundError()
{
    if (audioVotingCheckUtility.getAbledConnected() == AudioVotingCheckUtility::ABLED20)
    {
        votingMachineStatusWrapper->addStatusAbled20SoundError();
    }
    else
    {
        votingMachineStatusWrapper->addStatusAbled10SoundError();
    }
}

void AbleDiagnosticWidget::setDiagnosticResult()
{
    if (status)
    {
        if (ableDiagnosticSoundWidget->getSoundTestIsCorrect())
        {
            setStatusAbledOk();
        }
        else
        {
            setStatusAbledSoundError();
        }
    }
    else
    {
        setStatusAbledError();
    }
}

void AbleDiagnosticWidget::setResultWidget()
{
    const bool  abled20Connected = audioVotingCheckUtility.getAbledConnected() == AudioVotingCheckUtility::ABLED20;
    std::string resultImage      = abled20Connected ? ABLED20_RESULT_OK_IMAGE_PATH : ABLED10_RESULT_OK_IMAGE_PATH;
    std::string resultMessage    = _( "Able-d diagnose has been successfully executed." );

    if (status == false)
    {
        resultImage = abled20Connected ? ABLED20_RESULT_ERROR_IMAGE_PATH : ABLED10_RESULT_ERROR_IMAGE_PATH;
        resultMessage = _( "Able-d diagnose has finished with errors." );
    }

    progressWidget->setImage ( resultImage );
    progressWidget->setInfoText ( resultMessage );
    setWidget(progressWidget);

    Glib::signal_timeout().connect_once (
        sigc::mem_fun ( *this, &AbleDiagnosticWidget::emit ), TIME_TO_DISPLAY_DIAGNOSE_RESULT );
}

void AbleDiagnosticWidget::endDiagnosis()
{
    if ((audioVotingCheckUtility.ableIsConnected(true)) &&
        (soundTestPerformed == false))
    {
        setWidget ( ableDiagnosticSoundWidget );
        ableDiagnosticSoundWidget->initialize();
        soundTestPerformed = true;
        showButtons(false, false);
    }
    else
    {
        setDiagnosticResult();
        setResultWidget();
    }
}
