/**
 * @file	AutomaticDiagnosticWidget.cxx
 * @brief	automatic diagnostic widget class.
 * @author	Youssef.tourkmani
 * @date	01/11/2012
 */

#include "AutomaticDiagnosticWidget.hxx"
#include <Log/ISMTTLog.h>
#include <Log/SMTTLogManager.h>
#include <Configuration/s-a-e-s-g-u-i-conf-manager.h>
#include <Configuration/s-a-e-s-functionality-conf-manager.h>
#include <Operation/Diagnosis/AlarmBoxDiagnosis.hxx>
#include <Operation/Diagnosis/BarCodeDiagnosis.hxx>
#include <Operation/Diagnosis/EurnDiagnosis.hxx>
#include <Operation/Diagnosis/AbledDiagnosis.hxx>
#include <Operation/Diagnosis/CalibrationDiagnosis.hxx>
#include <Operation/Diagnosis/FlashDiagnosis.hxx>
#include <Operation/Diagnosis/PrinterDiagnosis.hxx>
#include <Operation/Diagnosis/SmartcardReaderDiagnosis.hxx>
#include <Configuration/label-configuration.h>
#include <Gui/Widgets/AbstractFactoryWidget.hxx>
#include <Operation/MachineOperationManager.hxx>
#include <Voting/VotingCenters/VotingCentersWrapper.hxx>
#include <Gui/Windows/MemoryMessageWindow.hxx>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#include <libintl.h>
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

#define N_(String) (String) ///<macro for gettext not translate

using namespace Smartmatic::SAES::GUI::Widgets;
using namespace Smartmatic::Log;
using namespace Smartmatic::SAES::Operation::Diagnosis;
using namespace Smartmatic::SAES::GUI::Configuration;
using namespace Smartmatic::SAES::Functionality::Configuration;
using namespace Smartmatic::SAES::Configuration;
using namespace Smartmatic::SAES::Voting;
using namespace Smartmatic::GUI::Configuration;
using namespace Smartmatic::SAES::Operation;
using namespace Smartmatic::SAES::Voting::Status;
using namespace Smartmatic::SAES::Voting::VotingCenters;
using namespace Smartmatic::SAES::Common::Configuration;
using namespace Smartmatic::SAES::GUI::Windows;

ISMTTLog * AutomaticDiagnosticWidget::logger = SMTTLogManager::GetLogger ( "Smartmatic.SAES.GUI.Widgets.AutomaticDiagnosticWidget", GETTEXT_PACKAGE );

AutomaticDiagnosticWidget::AutomaticDiagnosticWidget ( ProgressWidget * progress )
    : Smartmatic::GUI::Widgets::StepWidget(), progress ( progress )
{
    this->current = 0;
    this->resultWidget = NULL;
    this->diagnosisBlockedWidget = NULL;
    this->bothMemoriesConnected = true;
    this->initialize();
    this->setWidget ( progress );
}

AutomaticDiagnosticWidget::~AutomaticDiagnosticWidget()
{
    dispose();
}

void AutomaticDiagnosticWidget::start()
{
    bool blockDiagnostic = SAESGUIConfigurationManager::getInstance()->GetAutomaticWindowConfigurationForOfficialMode().getBlockDiagnostic();

    if ( VotingCentersWrapper::getInstance()->getRunningMode() == RunningMode::VOTING_CENTER_RECOUNT )
        blockDiagnostic = SAESGUIConfigurationManager::getInstance()->GetAutomaticWindowConfigurationForRecountMode().getBlockDiagnostic();

    if ( blockDiagnostic )
        block();
    else
        unblock();
}

void AutomaticDiagnosticWidget::block()
{
    diagnosisBlockedWidget = AbstractFactoryWidget::createDiagnosisBlockedWidget();
    diagnosisBlockedWidget->getSignalClicked().connect ( sigc::mem_fun ( *this, &AutomaticDiagnosticWidget::unblock ) );
    this->setWidget ( diagnosisBlockedWidget );
}

void AutomaticDiagnosticWidget::unblock()
{
    this->setWidget ( progress );
    diagnostic();
}

void AutomaticDiagnosticWidget::initialize()
{
    MachineOperationManager::getInstance()->getOperationStatus()->getVotingMachineStatusWrapper()->clearDeviceStatusList();
    LabelConfiguration diagConf = SAESGUIConfigurationManager::getInstance()->GetDiagnosisLabelConfiguration();
    progress->setInfoTextConfig ( diagConf );
    LabelConfiguration diagFooterConf = SAESGUIConfigurationManager::getInstance()->getDiagnosisFooterLabelConfiguration();
    progress->setInfoFooterConfig ( diagFooterConf );

    if ( SAESGUIConfigurationManager::getInstance()->getShowFooterInAutoDiagnosis() )
    {
        progress->showInfoFooter();
    }
    else
    {
        progress->hideInfoFooter();
    }

    progress->setImage ( Smartmatic::SAES::GUI::Operation::ImageResourcePath::NONE );
    progress->setInfoText ( " " );
    progress->setInfoFooter ( " " );
    std::vector <DiagnosticType> configList = SAESGUIConfigurationManager::getInstance()->getAutoDiagnosisList();
    std::vector <DiagnosticType>::iterator it;

    for ( it = configList.begin();
            it != configList.end();
            ++it )
    {
        IDiagnosis * interface = NULL;

        switch ( ( *it ).getDevice() )
        {
            case Smartmatic::SAES::Common::Configuration::DeviceType::CONFIGURATION_ALARMBOX:
                interface = new AlarmBoxDiagnosis();
                break;

            case Smartmatic::SAES::Common::Configuration::DeviceType::CONFIGURATION_BARCODE:
                interface = new BarCodeDiagnosis ( SAESGUIConfigurationManager::getInstance()->getBarcodeDiagnosisConfiguration().getBarcodeCommandsName() );
                break;

            case Smartmatic::SAES::Common::Configuration::DeviceType::CONFIGURATION_PRINTER:
                interface = new PrinterDiagnosis ( SAESGUIConfigurationManager::getInstance()->getBarcodeDiagnosisConfiguration().getStringToEncode(),
                                                   !SAESGUIConfigurationManager::getInstance()->getPrinterDiagnosisConfiguration().getDoNotCheckBeforePrintDiagnostic() );
                break;

            case Smartmatic::SAES::Common::Configuration::DeviceType::CONFIGURATION_SMARTCARD:
                interface = new SmartcardReaderDiagnosis();
                break;

            case Smartmatic::SAES::Common::Configuration::DeviceType::CONFIGURATION_FLASH:
                interface = new FlashDiagnosis ( true );
                break;

            case Smartmatic::SAES::Common::Configuration::DeviceType::CONFIGURATION_TOUCHSCREENCALIBRATION:
                interface = new CalibrationDiagnosis ( SAESGUIConfigurationManager::getInstance()->getTouchScreenCalibrationConfiguration().getNumberOfCalibrationPoints() );
                break;

            case Smartmatic::SAES::Common::Configuration::DeviceType::CONFIGURATION_EURN:
                interface = new EurnDiagnosis ( );
                break;

            case Smartmatic::SAES::Common::Configuration::DeviceType::CONFIGURATION_ABLED:
                interface = new AbledDiagnosis ( );
                break;

            case Smartmatic::SAES::Common::Configuration::DeviceType::CONFIGURATION_P168A:
            case Smartmatic::SAES::Common::Configuration::DeviceType::CONFIGURATION_MOUSE:
            case Smartmatic::SAES::Common::Configuration::DeviceType::CONFIGURATION_KEYBOARD:
            case Smartmatic::SAES::Common::Configuration::DeviceType::CONFIGURATION_USBSTORAGEMOUNTED:
            case Smartmatic::SAES::Common::Configuration::DeviceType::CONFIGURATION_ETHERNET:
            case Smartmatic::SAES::Common::Configuration::DeviceType::CONFIGURATION_UNKNOWN:
            default:
                break;
        }

        if ( interface )
        {
            interface->setTime ( ( *it ).getWaitTime() );
            interface->setMandatory ( ( *it ).getMandatory() );
            diagnosisList.push_back ( interface );
            diagnosisConnectionList.push_back ( interface->getSignalEnd().connect ( sigc::mem_fun ( *this, &AutomaticDiagnosticWidget::checkEndDiagnosis ) ) );
        }
    }

    connectMemorySignals();
}

void AutomaticDiagnosticWidget::dispose()
{
    this->remove();
    std::vector <IDiagnosis *>::iterator it;

    for ( it = diagnosisList.begin();
            it != diagnosisList.end();
            ++it )
    {
        IDiagnosis * obj = ( *it );

        if ( obj )
        {
            delete obj;
        }
    }

    diagnosisConnectionList.clear();
    diagnosisList.clear();
    disconnectMemorySignals();

    if ( progress )
    {
        delete progress;
        progress = NULL;
    }

    if ( diagnosisBlockedWidget )
    {
        delete diagnosisBlockedWidget;
        diagnosisBlockedWidget = NULL;
    }

    if ( resultWidget )
    {
        delete resultWidget;
        resultWidget = NULL;
    }
}

void AutomaticDiagnosticWidget::end()
{
    dispose();
    emit();
}

void AutomaticDiagnosticWidget::diagnostic()
{
    if ( ( ( diagnosisList.empty() || diagnosisList.size() == current ) ) && ( bothMemoriesConnected ) )
        showResult();
    else if ( bothMemoriesConnected )
        firstMessage();
}

void AutomaticDiagnosticWidget::firstMessage()
{
    diagnosisList[current]->updateMessage();
    showMessage();
    Glib::signal_timeout().connect_once ( sigc::mem_fun ( *this , &AutomaticDiagnosticWidget::runningMessage ), 1000 );
}

void AutomaticDiagnosticWidget::runningMessage()
{
    if (diagnosisList[current]->getStatus() != VMStatus::VOTING_MACHINE_STATUS_NOTDIAGNOSED)
    {
        diagnosisList[current]->setStatus ( VMStatus::VOTING_MACHINE_STATUS_RUN );
    }

    showMessage();
    Glib::signal_timeout().connect_once ( sigc::mem_fun ( *this , &AutomaticDiagnosticWidget::test ), 1000 );
}

void AutomaticDiagnosticWidget::test()
{
    diagnosisList[current]->test();
}

void AutomaticDiagnosticWidget::checkEndDiagnosis()
{
    diagnosisConnectionList[current].disconnect();

    if ( bothMemoriesConnected )
    {
        showMessage();
        Glib::signal_timeout().connect_once ( sigc::mem_fun ( *this , &AutomaticDiagnosticWidget::nextPage ), 200 + diagnosisList[current]->getTime() );
    }
}

void AutomaticDiagnosticWidget::nextPage()
{
    current++;
    Glib::signal_timeout().connect_once ( sigc::mem_fun ( *this , &AutomaticDiagnosticWidget::unblock ), 200 );
}

void AutomaticDiagnosticWidget::showMessage()
{
    switch ( diagnosisList[current]->getType() )
    {
        case Smartmatic::SAES::Common::Configuration::DeviceType::CONFIGURATION_ALARMBOX:
            progress->setImage ( "ui/alarmButton.gif" );
            progress->setInfoText ( diagnosisList[current]->getMessage() );
            progress->setInfoFooter ( diagnosisList[current]->getMessageFooter() );
            break;

        case Smartmatic::SAES::Common::Configuration::DeviceType::CONFIGURATION_BARCODE:
            progress->setImage ( Smartmatic::SAES::GUI::Operation::ImageResourcePath::RECONFIGURE_BARCODE );
            progress->setInfoText ( diagnosisList[current]->getMessage() );
            progress->setInfoFooter ( diagnosisList[current]->getMessageFooter() );
            break;

        case Smartmatic::SAES::Common::Configuration::DeviceType::CONFIGURATION_PRINTER:
            progress->setImage ( Smartmatic::SAES::GUI::Operation::ImageResourcePath::PRINTING_TEST );
            progress->setInfoText ( diagnosisList[current]->getMessage() );
            progress->setInfoFooter ( diagnosisList[current]->getMessageFooter() );
            break;

        case Smartmatic::SAES::Common::Configuration::DeviceType::CONFIGURATION_SMARTCARD:
            progress->setImage ( Smartmatic::SAES::GUI::Operation::ImageResourcePath::SMARTCARD_AUTO_DIAGNOSIS );
            progress->setInfoText ( diagnosisList[current]->getMessage() );
            progress->setInfoFooter ( diagnosisList[current]->getMessageFooter() );
            break;

        case Smartmatic::SAES::Common::Configuration::DeviceType::CONFIGURATION_FLASH:
            progress->setImage ( "ui/Flash.png" );
            progress->setInfoText ( diagnosisList[current]->getMessage() );
            progress->setInfoFooter ( diagnosisList[current]->getMessageFooter() );
            break;

        case Smartmatic::SAES::Common::Configuration::DeviceType::CONFIGURATION_TOUCHSCREENCALIBRATION:
            progress->setImage ( "ui/calibration.jpg" );
            progress->setInfoText ( diagnosisList[current]->getMessage() );
            progress->setInfoFooter ( diagnosisList[current]->getMessageFooter() );
            break;

        case Smartmatic::SAES::Common::Configuration::DeviceType::CONFIGURATION_EURN:
            progress->setImage ( "ui/eUrnDiagnostic.gif" );
            progress->setInfoText ( diagnosisList[current]->getMessage() );
            progress->setInfoFooter ( diagnosisList[current]->getMessageFooter() );
            break;

        case Smartmatic::SAES::Common::Configuration::DeviceType::CONFIGURATION_ABLED:
            if (diagnosisList[current]->getStatus() != VMStatus::VOTING_MACHINE_STATUS_NOTDIAGNOSED)
            {
                progress->setImage ( "ui/Able20Buttons.png" );
                progress->setInfoText ( diagnosisList[current]->getMessage() );
                progress->setInfoFooter ( diagnosisList[current]->getMessageFooter() );
            }
            break;

        case Smartmatic::SAES::Common::Configuration::DeviceType::CONFIGURATION_P168A:
        case Smartmatic::SAES::Common::Configuration::DeviceType::CONFIGURATION_MOUSE:
        case Smartmatic::SAES::Common::Configuration::DeviceType::CONFIGURATION_KEYBOARD:
        case Smartmatic::SAES::Common::Configuration::DeviceType::CONFIGURATION_USBSTORAGEMOUNTED:
        case Smartmatic::SAES::Common::Configuration::DeviceType::CONFIGURATION_ETHERNET:
        case Smartmatic::SAES::Common::Configuration::DeviceType::CONFIGURATION_UNKNOWN:
        default:
            break;
    }
}

void AutomaticDiagnosticWidget::showResult()
{
    int labelWidth = SAESGUIConfigurationManager::getInstance()->GetAutomaticWindowConfigurationForOfficialMode().getDiagnosticReviewWidth();
    int labelHeight = SAESGUIConfigurationManager::getInstance()->GetAutomaticWindowConfigurationForOfficialMode().getDiagnosticReviewHeight();

    if ( VotingCentersWrapper::getInstance()->getRunningMode() == RunningMode::VOTING_CENTER_RECOUNT )
    {
        labelWidth = SAESGUIConfigurationManager::getInstance()->GetAutomaticWindowConfigurationForRecountMode().getDiagnosticReviewWidth();
        labelHeight = SAESGUIConfigurationManager::getInstance()->GetAutomaticWindowConfigurationForRecountMode().getDiagnosticReviewHeight();
    }

    MachineOperationManager::getInstance()->getOperationStatus()->persistDiagnosticFile(false);
    resultWidget = new DiagnosticResultWidget ( true, labelWidth, labelHeight );
    resultWidget->getSignalEnd().connect ( sigc::mem_fun ( *this, &AutomaticDiagnosticWidget::end ) );
    resultWidget->refresh();
    setWidget ( resultWidget );
}

void AutomaticDiagnosticWidget::onSignalBlock()
{
    bothMemoriesConnected = false;
    logger->Error( N_ ( "Remove one USB not safely" ) );
}

void AutomaticDiagnosticWidget::onSignalResume()
{
    bothMemoriesConnected = true;
    logger->Debug( N_ ( "USB added" ) );

    current = 0;
    resultWidget = NULL;
    diagnosisBlockedWidget = NULL;
    diagnosisConnectionList.clear();
    diagnosisList.clear();
    disconnectMemorySignals();
    initialize();

    start();
}

void AutomaticDiagnosticWidget::connectMemorySignals()
{
    signalBlock = MemoryMessageWindow::getInstance()->getSignalBlock().connect( sigc::mem_fun ( *this, &AutomaticDiagnosticWidget::onSignalBlock ) );
    signalResume = MemoryMessageWindow::getInstance()->getSignalResume().connect( sigc::mem_fun ( *this, &AutomaticDiagnosticWidget::onSignalResume ) );
}

void AutomaticDiagnosticWidget::disconnectMemorySignals()
{
    signalBlock.disconnect();
    signalResume.disconnect();
}

std::string AutomaticDiagnosticWidget::getTitleText()
{
    return N_ ( "Smartmatic.SAES.GUI.Windows.AutomaticDiagnosticWindow.Title" );
}
