/**
 * @file	BarcodeReaderDetectedWidget.cxx
 * @brief	Body barcode reader detected widget class.
 * @author	Yadickson Soto
 * @date	14/10/2013
 */

#include "BarcodeReaderDetectedWidget.hxx"
#include "Configuration/s-a-e-s-g-u-i-conf-manager.h"
#include <Log/ISMTTLog.h>
#include <Log/SMTTLogManager.h>
#include <Operation/MachineOperationManager.hxx>
#include <Operation/Monitor/DeviceTypeMonitorPool.hxx>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#include <libintl.h>
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

#define N_(String) (String) ///<macro for gettext not translate

using namespace Smartmatic::GUI::Configuration;
using namespace Smartmatic::SAES::GUI::Configuration;
using namespace Smartmatic::SAES::GUI::Widgets;
using namespace Smartmatic::SAES::Operation;
using namespace Smartmatic::SAES::Operation::Monitor;
using namespace Smartmatic::SAES::Common::Operation::Monitor;
using namespace Smartmatic::SAES::Common::Configuration;

Smartmatic::Log::ISMTTLog * BarcodeReaderDetectedWidget::logger = Smartmatic::Log::SMTTLogManager::GetLogger ( "Smartmatic.SAES.GUI.Widgets.BarcodeReaderDetectedWidget", GETTEXT_PACKAGE );

BarcodeReaderDetectedWidget::BarcodeReaderDetectedWidget ( ProgressWidget * widget )
    : BasicDiagnosticWidget ( false, true, true ),  widget ( widget )
{
    timeOK = SAESGUIConfigurationManager::getInstance()->getBarcodeDiagnosisConfiguration().getReconfigureTimeOk();
    timeFail = SAESGUIConfigurationManager::getInstance()->getBarcodeDiagnosisConfiguration().getReconfigureTimeFail();

    widget->setInfoText ( N_ ( "Detecting the Barcode Reader Device." ) );
    widget->setImage ( Smartmatic::SAES::GUI::Operation::ImageResourcePath::RECONFIGURE_BARCODE );

    setRetryButtonLabel ( _ ( "Not Detected" ) );
    setCancelButtonLabel ( _ ( "I do not wish to Test" ) );

    signalCancelDetected.connect ( sigc::mem_fun ( *this, &BarcodeReaderDetectedWidget::skipDetection ) );
    signalRetryDetected.connect ( sigc::mem_fun ( *this, &BarcodeReaderDetectedWidget::onNotDetected ) );

    setWidget ( widget );
}

BarcodeReaderDetectedWidget::~BarcodeReaderDetectedWidget()
{
    remove();
    dispose();

    if ( widget )
    {
        delete widget;
        widget = NULL;
    }
}

void BarcodeReaderDetectedWidget::initialize()
{
    dispose();
    showButtons ( false, true, true );

    DeviceTypeMonitor * deviceTypeMonitor = DeviceTypeMonitorPool::getInstance()->getDeviceTypeMonitor ( DeviceType::CONFIGURATION_BARCODE );

    if ( deviceTypeMonitor != NULL && deviceTypeMonitor->getNumberOfDevices() > 0 )
    {
        deviceDetected ();
    }
    else
    {
        connDevice.disconnect();
        connDevice = DeviceTypeMonitorPool::getInstance()->getDeviceChangedSignal().connect ( sigc::mem_fun ( *this, &BarcodeReaderDetectedWidget::deviceChange ) );

        reconnectDevice ();
    }
}

void BarcodeReaderDetectedWidget::dispose()
{
    connDevice.disconnect();
}

void BarcodeReaderDetectedWidget::deviceDetected ()
{
    showButtons ( false, false, false );
    connDevice.disconnect();

    widget->setInfoText ( N_ ( "Barcode Reader Device Detected." ) );
    widget->setImage ( Smartmatic::SAES::GUI::Operation::ImageResourcePath::RECONFIGURE_BARCODE_SUCCESS );
    Glib::signal_timeout().connect_once ( sigc::mem_fun ( *this, &BarcodeReaderDetectedWidget::end ), timeOK );
}

void BarcodeReaderDetectedWidget::reconnectDevice ()
{
    showButtons ( false, true, true );

    widget->setInfoText ( N_ ( "Please connect the Barcode Reader at this time.\n"
                               "This message should go away in the 10 seconds following the\n"
                               "connection, otherwise please press 'Not Detected'.\n"
                               "If the Barcode Reader is already physically connected and you\n"
                               "are getting this message, please also press 'Not Detected'." ) );
    widget->setImage ( Smartmatic::SAES::GUI::Operation::ImageResourcePath::RECONFIGURE_BARCODE_FAIL );
}

void BarcodeReaderDetectedWidget::onNotDetected ()
{
    showButtons ( false, false, false );

    widget->setInfoText ( N_ ( "Barcode Reader Device Not Detected." ) );
    widget->setImage ( Smartmatic::SAES::GUI::Operation::ImageResourcePath::RECONFIGURE_BARCODE_FAIL );

    MachineOperationManager::getInstance()->getOperationStatus()->getVotingMachineStatusWrapper()->setBarcodeReaderFullStatus ( false );

    Glib::signal_timeout().connect_once ( sigc::mem_fun ( *this, &BarcodeReaderDetectedWidget::error ), timeFail );
}

void BarcodeReaderDetectedWidget::skipDetection ()
{
    showButtons ( false, false, false );
    cancel();
}

void BarcodeReaderDetectedWidget::error()
{
    on_stop_signal_emit().emit();
}

void BarcodeReaderDetectedWidget::cancel()
{
    on_cancel_signal_emit().emit();
}

void BarcodeReaderDetectedWidget::end()
{
    emit();
}

void BarcodeReaderDetectedWidget::deviceChange ( DeviceTypeMonitor * deviceTypeMonitor )
{
    if ( deviceTypeMonitor->getType() == DeviceType::CONFIGURATION_BARCODE && deviceTypeMonitor->getNumberOfDevices() > 0 )
    {
        deviceDetected();
    }
}
