/**
 * @file	BarcodeReaderScanWidget.cxx
 * @brief	bar code diagnosis widget class.
 * @author	Youssef.tourkmani
 * @date	01/11/2012
 */


#include "BarcodeReaderScanWidget.hxx"
#include "Configuration/s-a-e-s-g-u-i-conf-manager.h"
#include <Log/ISMTTLog.h>
#include <Log/SMTTLogManager.h>
#include <Operation/Devices/BCRController.hxx>
#include <Graphics/Encoding2D/QREncoding.h>
#include "Configuration/s-a-e-s-conf-manager.h"
#include <Operation/MachineOperationManager.hxx>
#include <System/Utils/SafeBufferUtils.hxx>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#define N_(String) (String) ///<macro for gettext not translate

using namespace Smartmatic::GUI::Configuration;
using namespace Smartmatic::SAES::GUI::Configuration;
using namespace Smartmatic::SAES::GUI::Widgets;
using namespace Smartmatic::SAES::Operation::Devices;
using namespace Smartmatic::System;
using namespace Smartmatic::SAES::Configuration;
using namespace Smartmatic::Graphics::Encoding2D;
using namespace Smartmatic::SAES::Operation;
using namespace Smartmatic::System::Utils;

Smartmatic::Log::ISMTTLog * BarcodeReaderScanWidget::logger = Smartmatic::Log::SMTTLogManager::GetLogger ( "Smartmatic.SAES.GUI.Widgets.BarcodeReaderScanWidget", GETTEXT_PACKAGE );

BarcodeReaderScanWidget::BarcodeReaderScanWidget ( ProgressWidget * widget )
    : BasicDiagnosticWidget ( false, true )
    , widget(widget)
    , timeOK(SAESGUIConfigurationManager::getInstance()->getBarcodeDiagnosisConfiguration().getScanTimeOk())
    , timeFail(SAESGUIConfigurationManager::getInstance()->getBarcodeDiagnosisConfiguration().getScanTimeFail())
    , commandsIn(SAESGUIConfigurationManager::getInstance()->getBarcodeDiagnosisConfiguration().getBarcodeInParams())
    , commandsOut(SAESGUIConfigurationManager::getInstance()->getBarcodeDiagnosisConfiguration().getBarcodeOutParams())
{
    signalDispatcher.getSyncSignal().connect ( sigc::mem_fun ( *this, &BarcodeReaderScanWidget::onRawObject ) );

    setWidget ( widget );
}

BarcodeReaderScanWidget::~BarcodeReaderScanWidget()
{
    remove();
    dispose();

    if ( widget )
    {
        delete widget;
        widget = NULL;
    }
}

void BarcodeReaderScanWidget::initialize()
{
    std::string data = SAESGUIConfigurationManager::getInstance()->getBarcodeDiagnosisConfiguration().getStringToEncode();
    SafeBuffer buffer ( data );
    QRCodeConfig config = SAESConfigurationManager::getInstance()->getQRCodeBarcodeConfig();
    int pixelSize = config.getSizeQRCode();
    int margin = config.getMarginQRCode();
    int version = config.getVersionQRCode();
    QRecLevel correctionLevel = SAESConfigurationManager::getInstance()->GetCorrectionModeQRCode ( config.getCorrectionModeQRCode() );
    Glib::RefPtr<Gdk::Bitmap> bitmap ( QREncoding::GenerateSymbol ( ( guchar * ) buffer.GetData(), buffer.GetSize(), pixelSize, margin, correctionLevel, version ) );
    Glib::RefPtr<Gdk::Drawable> drwptr = bitmap;
    gint width ( 0 );
    gint height ( 0 );
    bitmap->get_size ( width, height );
    Glib::RefPtr<Gdk::Pixbuf> pixBuffBarcode = Gdk::Pixbuf::create ( drwptr, 0, 0, width, height );

    widget->setInfoText ( N_ ( "Smartmatic.SAES.GUI.Widgets.BarCodeDiagnosisWidget.PleaseScanThePrintedBarcodeOrTheScreenBarcode" ) );
    widget->setImage ( pixBuffBarcode );

    connCancel.disconnect();
    connCancel = signalCancelDetected.connect ( sigc::mem_fun ( *this, &BarcodeReaderScanWidget::onCancel ) );

    setCancelButtonSensitive ( true );

    start ( 100 );
}

void BarcodeReaderScanWidget::preStart ()
{
}

bool BarcodeReaderScanWidget::process ()
{
    SafeBuffer buffer = BCRController::getInstance()->readBuffer();

    if ( buffer.GetSize() > 0 )
    {
        dispose();

        bool validate = false;

        SafeBufferUtils buffUtil ( buffer );
        std::vector < SafeBuffer > result = buffUtil.splitByEndLine();

        std::string recievedData;

        if ( ! result.empty() )
        {
            SafeBuffer item ( result.front() );
            recievedData.assign ( ( const char * ) item.GetData(), item.GetSize() );
            recievedData.resize ( item.GetSize() );
        }

        logger->Debug ( " Read data from device (" + recievedData + ")" );

        validate = recievedData == SAESGUIConfigurationManager::getInstance()->getBarcodeDiagnosisConfiguration().getStringToEncode();

        signalDispatcher.emit ( new bool ( validate ) );
    }

    return buffer.GetSize() == 0;
}

void BarcodeReaderScanWidget::postStop ()
{
    BCRController::getInstance()->sendReconfigureCommand ( commandsOut );
}

void BarcodeReaderScanWidget::dispose()
{
    connCancel.disconnect();
}

void BarcodeReaderScanWidget::onRawObject ( void * object )
{
    bool * status = static_cast < bool * > ( object );

    if ( *status )
    {
        logger->Debug ( "Scan Device Successfully" );

        showButtons ( false, false, false );

        widget->setInfoText ( N_ ( "Smartmatic.SAES.GUI.Widgets.BarCodeDiagnosisWidget.DiagnosticSuccessful" ) );
        Glib::signal_timeout().connect_once ( sigc::mem_fun ( *this, &BarcodeReaderScanWidget::emit ), timeOK );
    }
    else
    {
        logger->Debug ( "Scan Device Fail" );

        showButtons ( false, false, false );

        widget->setInfoText ( N_ ( "Smartmatic.SAES.GUI.Widgets.BarCodeDiagnosisWidget.DiagnosticFailed" ) );
        MachineOperationManager::getInstance()->getOperationStatus()->getVotingMachineStatusWrapper()->setBarcodeReaderFullStatus ( true, true, true, false );

        Glib::signal_timeout().connect_once ( sigc::mem_fun ( *this, &BarcodeReaderScanWidget::error ), timeFail );
    }

    delete status;
}

void BarcodeReaderScanWidget::onCancel()
{
    stop();

    dispose ();

    showButtons ( false, false, false );
    widget->setInfoText ( N_ ( "Smartmatic.SAES.GUI.Widgets.BarCodeDiagnosisWidget.DiagnosticFailed" ) );

    MachineOperationManager::getInstance()->getOperationStatus()->getVotingMachineStatusWrapper()->setBarcodeReaderFullStatus ( true, true, false );

    Glib::signal_timeout().connect_once ( sigc::mem_fun ( *this, &BarcodeReaderScanWidget::error ), timeFail );
}

void BarcodeReaderScanWidget::error()
{
    on_stop_signal_emit().emit();
}
