/**
 * @file MouseWidget.cxx
 * @brief Body class mouse widget
 * @date 22/03/2019
 * @author eduardo.fernandez
 */

#include "MouseWidget.hxx"
#include <Log/ISMTTLog.h>
#include <Log/SMTTLogManager.h>
#include <Gui/Widgets/FactoryMethodWidget.hxx>
#include <Environment/Configurations/resource-path-provider.h>
#include "Configuration/label-configuration.h"
#include <Configuration/MachineModelEnum.hxx>
#include "Configuration/s-a-e-s-g-u-i-conf-manager.h"
#include <Configuration/device-type-schema.hxx>
#include <Operation/MachineOperationManager.hxx>
#include <System/Utils/StringUtils.hxx>
#include <System/Sounds/smartmatic-sounds-manager.h>
#include <Configuration/s-m-t-t-g-u-i-conf-manager.h>
#include <Voting/Status/voting-machine-status-schema.hxx>
#include <Voting/Status/VMStatusWrapper.hxx>
#include <Operation/Monitor/DeviceTypeMonitor.hxx>
#include <Operation/Monitor/DeviceTypeMonitorPool.hxx>
#include <Configuration/SaesCommonConfiguration-schema.hxx>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#include <libintl.h>
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

#define N_(String) (String) ///<macro for gettext not translate
#define KEYPAD_TABLE_MAX_HEIGHT 250
#define KEYPAD_BUTTONS_MAX_WIDTH 60
#define KEYPAD_BUTTONS_MAX_HEIGHT 60

using namespace Smartmatic::Configuration;
using namespace Smartmatic::GUI::Widgets;
using namespace Smartmatic::SAES::Common::Configuration;
using namespace Smartmatic::SAES::GUI::Widgets;
using namespace Smartmatic::Log;
using namespace Smartmatic::GUI::Configuration;
using namespace Smartmatic::Environment;
using namespace Smartmatic::SAES::GUI::Configuration;
using namespace Smartmatic::SAES::HW::Configuration;
using namespace Smartmatic::SAES::Configuration;
using namespace Smartmatic::SAES::Operation;
using namespace Smartmatic::SAES::Common::Operation::Monitor;
using namespace Smartmatic::SAES::Operation::Monitor;
using namespace Smartmatic::SAES::Voting::Status;
using namespace Smartmatic::System::Utils;
using namespace Smartmatic::System::Sounds;

ISMTTLog *MouseWidget::logger = SMTTLogManager::GetLogger("Smartmatic.SAES.GUI.Widgets.MouseWidget", GETTEXT_PACKAGE);

static const std::string EMPTY_STRING           = "";
static const std::string MOUSE_DETECTED_MESSAGE = "VID-%1;PID-%2";

MouseWidget::MouseWidget() : BasicDiagnosticWidget ( true )
{
}

MouseWidget::~MouseWidget()
{
}

void MouseWidget::setDiagnosticResult(
    const bool isMousePresent,
    const bool isSpecificMousePresent,
    const std::string& vendorId,
    const std::string& productId)
{
    VotingMachineStatusWrapper *statusWrapper =
        MachineOperationManager::getInstance()->getOperationStatus()->getVotingMachineStatusWrapper();

    if (isMousePresent && isSpecificMousePresent)
    {
        statusWrapper->changeStatus(DeviceType::CONFIGURATION_MOUSE, VMStatus::VOTING_MACHINE_STATUS_OK);
    }
    else if (isMousePresent && isSpecificMousePresent == false)
    {
        const std::string mouseInfo = StringUtils::compose(MOUSE_DETECTED_MESSAGE, vendorId, productId);

        statusWrapper->addStatusCode(DeviceType::CONFIGURATION_MOUSE, mouseInfo, VMStatus::VOTING_MACHINE_STATUS_ERROR);
    }
    else
    {
        statusWrapper->changeStatus(DeviceType::CONFIGURATION_MOUSE, VMStatus::VOTING_MACHINE_STATUS_NOTDIAGNOSED);
    }

    statusWrapper = NULL;
    delete statusWrapper;
}

void MouseWidget::initialize()
{
    bool isMousePresent         = false;
    bool isSpecificMousePresent = false;

    std::string expectedHidId;
    std::string expectedVendorId;
    std::string expectedProductId;

    SaesCommonConfiguration *configuration = SAESCommonConfigurationManager::getInstance()->getConfiguration();

    if (configuration->getMouseDiagnosticConfiguration().present())
    {
        expectedHidId     = StringUtils::toLower(configuration->getMouseDiagnosticConfiguration().get().getExpectedHidId());
        expectedVendorId  = StringUtils::toLower(configuration->getMouseDiagnosticConfiguration().get().getExpectedVendorId());
        expectedProductId = StringUtils::toLower(configuration->getMouseDiagnosticConfiguration().get().getExpectedProductId());
    }

    configuration = NULL;
    delete configuration;

    DeviceTypeMonitor *deviceTypeMonitor = DeviceTypeMonitorPool::getInstance()->getDeviceTypeMonitor(
        DeviceType::CONFIGURATION_MOUSE);

    if ( deviceTypeMonitor != NULL && deviceTypeMonitor->getNumberOfDevices() > 0 )
    {
        isMousePresent = DeviceTypeMonitorPool::getInstance()->isMousePresent(expectedHidId);

        if (isMousePresent)
        {
            if ((expectedHidId != EMPTY_STRING) &&
                (expectedVendorId != EMPTY_STRING) &&
                (expectedProductId != EMPTY_STRING))
            {
                isSpecificMousePresent = DeviceTypeMonitorPool::getInstance()->isDevicePresent(
                    expectedHidId, expectedVendorId, expectedProductId);
            }
        }
    }

    setDiagnosticResult(isMousePresent, isSpecificMousePresent, expectedVendorId, expectedProductId);
}
