/**
 * @file	SmartcardPowerWidget.cxx
 * @brief	smartcard reader power OFF/ON diagnosis widget class.
 * @author	Yadickson Soto
 * @date	12/09/2013
 */

#include "SmartcardPowerWidget.hxx"
#include <Configuration/s-a-e-s-g-u-i-conf-manager.h>
#include <Card/ISmartcardReaderPowerManager.hxx>
#include <Operation/Devices/SmartcardReaderPowerManagerFactory.hxx>
#include <Log/ISMTTLog.h>
#include <Log/SMTTLogManager.h>
#include <Operation/MachineOperationManager.hxx>
#include <Operation/Monitor/DeviceTypeMonitorPool.hxx>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#include <libintl.h>
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

#define N_(String) (String) ///<macro for gettext not translate

using namespace Smartmatic::GUI::Configuration;
using namespace Smartmatic::SAES::GUI::Widgets;
using namespace Smartmatic::SAES::GUI::Configuration;
using namespace Smartmatic::SAES::Configuration;
using namespace Smartmatic::Log;
using namespace Smartmatic::GUI::Widgets;
using namespace Smartmatic::SAES::Card;
using namespace Smartmatic::SAES::Operation;
using namespace Smartmatic::SAES::Operation::Devices;
using namespace Smartmatic::SAES::Common::Operation::Monitor;
using namespace Smartmatic::SAES::Common::Configuration;
using namespace Smartmatic::SAES::Operation::Monitor;

ISMTTLog * SmartcardPowerWidget::logger = SMTTLogManager::GetLogger ( "Smartmatic.SAES.GUI.Widgets.PrinterDiagnosisWidget", GETTEXT_PACKAGE );

SmartcardPowerWidget::SmartcardPowerWidget ( Smartmatic::SAES::GUI::Widgets::ProgressWidget * progressWidget )
    : WizardWidgetBase (), progressWidget ( progressWidget )
{
    currentStep = OFF;

    timeOk = SAESGUIConfigurationManager::getInstance()->getSmartcardReaderDiagnosisConfiguration().getTimeOk();
    timeFail = SAESGUIConfigurationManager::getInstance()->getSmartcardReaderDiagnosisConfiguration().getTimeFail();
    timeStep = SAESGUIConfigurationManager::getInstance()->getSmartcardReaderDiagnosisConfiguration().getTimeStep();

    signalThread.getSyncSignal().connect ( sigc::mem_fun ( *this, &SmartcardPowerWidget::onRawObject ) );

    setWidget ( progressWidget );
}

SmartcardPowerWidget::~SmartcardPowerWidget()
{
    remove();

    if ( progressWidget )
    {
        delete progressWidget;
        progressWidget = NULL;
    }
}

void SmartcardPowerWidget::initialize()
{
    currentStep = OFF;
    nextStep();
}

void SmartcardPowerWidget::nextStep()
{
    switch ( ( SmartcardPowerStep ) currentStep )
    {
        case OFF:
            powerOff ();
            break;

        case ON:
            powerOn();
            break;

        default:
            success();
            break;
    }
}

void SmartcardPowerWidget::powerOff()
{
    progressWidget->setInfoText ( _ ( "Power OFF Smartcard Reader, please wait" ) );
    progressWidget->setImage ( Smartmatic::SAES::GUI::Operation::ImageResourcePath::PROGRESS );
    start ( 0 );
}

void SmartcardPowerWidget::powerOn()
{
    progressWidget->setInfoText ( _ ( "Power ON Smartcard Reader, please wait" ) );
    progressWidget->setImage ( Smartmatic::SAES::GUI::Operation::ImageResourcePath::PROGRESS );
    start ( 0 );
}

bool SmartcardPowerWidget::process()
{
    std::auto_ptr<ISmartcardReaderPowerManager> powerManager(
        SmartcardReaderPowerManagerFactory::getSmartcardReaderPowerManager());

    if ( ( SmartcardPowerStep ) currentStep == OFF )
    {
        powerManager->callPowerOffSmartcard();
    }
    else
    {
        powerManager->callPowerOnSmartcard();
    }

    sleep ( 2 );

    unsigned int numberOfDevices = DeviceTypeMonitorPool::getInstance()->getNumberOfDevicesByType ( DeviceType::CONFIGURATION_SMARTCARD );

    if ( ( SmartcardPowerStep ) currentStep == OFF )
    {
        if ( numberOfDevices == 0 )
        {
            logger->Debug ( "Power OFF successfully" );
        }
        else
        {
            MachineOperationManager::getInstance()->getOperationStatus()->getVotingMachineStatusWrapper()->setStatusSmartcardReaderPowerCycleOffError();
            logger->Debug ( "Power OFF fail" );
            messageError = N_ ( "Power OFF Fail" );
        }

        signalThread.emit ( new bool ( numberOfDevices == 0 ) );
    }
    else
    {
        if ( numberOfDevices != 0 )
        {
            logger->Debug ( "Power ON successfully" );
        }
        else
        {
            MachineOperationManager::getInstance()->getOperationStatus()->getVotingMachineStatusWrapper()->setStatusSmartcardReaderPowerCycleOnError();
            logger->Debug ( "Power ON fail" );
            messageError = N_ ( "Power ON Fail" );
        }

        signalThread.emit ( new bool ( numberOfDevices != 0 ) );
    }

    return false;
}

void SmartcardPowerWidget::onRawObject ( void * object )
{
    bool * status = static_cast < bool * > ( object );

    if ( *status )
    {
        currentStep++;
        Glib::signal_timeout().connect_once ( sigc::mem_fun ( *this, &SmartcardPowerWidget::nextStep ), timeStep );
    }
    else
    {
        progressWidget->setImage ( Smartmatic::SAES::GUI::Operation::ImageResourcePath::SMARTCARD_DISABLED );
        progressWidget->setInfoText ( _ ( messageError.c_str() ) );
        Glib::signal_timeout().connect_once ( sigc::mem_fun ( *this, &SmartcardPowerWidget::error ), timeFail );
    }

    delete status;
}

void SmartcardPowerWidget::success()
{
    progressWidget->setImage ( Smartmatic::SAES::GUI::Operation::ImageResourcePath::SMARTCARD_DETECT );
    progressWidget->setInfoText ( _ ( "Power OFF/ON Successfully" ) );
    Glib::signal_timeout().connect_once ( sigc::mem_fun ( *this, &SmartcardPowerWidget::emit ), timeOk );
}

void SmartcardPowerWidget::error()
{
    on_stop_signal_emit().emit();
}

void SmartcardPowerWidget::dispose()
{

}
