/**
 * @file ZipCodeWidget.cxx
 * @brief Body class zip code widget
 * @date 27/09/2012
 * @author Yadickson Soto
 */

#include "ZipCodeWidget.hxx"
#include <Log/ISMTTLog.h>
#include <Log/SMTTLogManager.h>
#include <Gui/Widgets/FactoryMethodWidget.hxx>
#include <Environment/Configurations/resource-path-provider.h>
#include "Configuration/label-configuration.h"
#include <Configuration/MachineModelEnum.hxx>
#include "Configuration/s-a-e-s-g-u-i-conf-manager.h"
#include <System/Utils/StringUtils.hxx>
#include <System/Sounds/smartmatic-sounds-manager.h>
#include <Configuration/s-m-t-t-g-u-i-conf-manager.h>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#include <libintl.h>
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

#define N_(String) (String) ///<macro for gettext not translate
#define KEYPAD_TABLE_MAX_HEIGHT 250
#define KEYPAD_BUTTONS_MAX_WIDTH 60
#define KEYPAD_BUTTONS_MAX_HEIGHT 60

using namespace Smartmatic::Configuration;
using namespace Smartmatic::GUI::Widgets;
using namespace Smartmatic::SAES::GUI::Widgets;
using namespace Smartmatic::Log;
using namespace Smartmatic::GUI::Configuration;
using namespace Smartmatic::Environment;
using namespace Smartmatic::SAES::GUI::Configuration;
using namespace Smartmatic::SAES::HW::Configuration;
using namespace Smartmatic::SAES::Configuration;
using namespace Smartmatic::System::Utils;
using namespace Smartmatic::System::Sounds;

ISMTTLog * ZipCodeWidget::logger = SMTTLogManager::GetLogger ( "Smartmatic.SAES.GUI.Widgets.ZipCodeWidget", GETTEXT_PACKAGE );

ZipCodeWidget::ZipCodeWidget ( bool showKeyBoard )
    : BasicDiagnosticWidget ( true )
{
    this->showKeyBoard = showKeyBoard;

    mainFrame = NULL;
    mainAlignment = NULL;
    tableAlignment = NULL;
    keyBoardAlignment = NULL;
    mainVbox = NULL;
    titleText = NULL;
    zipCodeText = NULL;
    messageText = NULL;
    table = NULL;
    keyboard = NULL;
    zipCodeEntry = NULL;
    separator = NULL;

    keyboard = new KeypadWidget ( CorrigeConfirma, true, KEYPAD_TABLE_MAX_HEIGHT, KEYPAD_BUTTONS_MAX_WIDTH, KEYPAD_BUTTONS_MAX_HEIGHT, true );
    keyboard->keypadinterface = this;
    Glib::RefPtr<Gtk::Builder> builder;
    std::string gladeFile = ResourcePathProvider::getInstance()->GetResourcePath ( PACKAGE_NAME, "ui/ZipCodeWidget.glade" );

    try
    {
        builder = Gtk::Builder::create_from_file ( gladeFile );
    }
    catch ( const Glib::FileError & ex )
    {
        logger->Debug ( "FATAL - ZipCodeWidget Initialize" );
    }

    builder->get_widget ( "main-frame", mainFrame );
    builder->get_widget ( "main-vbox", mainVbox );
    builder->get_widget ( "main-alignment", mainAlignment );
    builder->get_widget ( "table-alignment", tableAlignment );
    builder->get_widget ( "keyboard", keyBoardAlignment );
    builder->get_widget ( "label-title", titleText );
    builder->get_widget ( "table", table );
    builder->get_widget ( "label-zipcode", zipCodeText );
    builder->get_widget ( "label-message", messageText );
    builder->get_widget ( "hseparator1", separator );
    builder->get_widget_derived ( "entry-zipcode", zipCodeEntry );

    bool runningVIU = SMTTGUIConfigurationManager::getInstance()->getSmartmaticGUIMachineModelConfiguration().getId() == EnumToString( PM_VIU805 );

    separator->set_no_show_all();
    separator->set_visible ( showKeyBoard );

    keyBoardAlignment->add ( *keyboard );
    keyBoardAlignment->show_all_children();
    zipCodeEntry->SetType ( SmartmaticEntry::Mask );
    zipCodeEntry->setMandatory ( true );
    zipCodeEntry->setMax ( SAESGUIConfigurationManager::getInstance()->getZipCodeConfiguration().getMax() );
    zipCodeEntry->setMask ( SAESGUIConfigurationManager::getInstance()->getZipCodeConfiguration().getMask() );
    zipCodeEntry->setRegex ( SAESGUIConfigurationManager::getInstance()->getZipCodeConfiguration().getRegex() );
    zipCodeEntry->setBGError();

    keyBoardAlignment->set_no_show_all();
    keyBoardAlignment->set_visible ( showKeyBoard || runningVIU );
    zipCodeEntry->signal_changed().connect ( sigc::mem_fun ( *this, &ZipCodeWidget::clearMessage ) );

    signalOkDetected.connect ( sigc::mem_fun ( *this, &ZipCodeWidget::validate ) );

    LabelConfiguration titleConf = SAESGUIConfigurationManager::getInstance()->getZipCodeTitleConfiguration();
    LabelConfiguration::SetLabelConfiguration ( titleText, titleConf );
    LabelConfiguration infoConf = SAESGUIConfigurationManager::getInstance()->getZipCodeInfoConfiguration();
    LabelConfiguration::SetLabelConfiguration ( zipCodeText, infoConf );
    LabelConfiguration messageConf = SAESGUIConfigurationManager::getInstance()->getZipCodeMessageConfiguration();
    LabelConfiguration::SetLabelConfiguration ( messageText, messageConf );
    zipCodeText->set_text ( _ ( "Smartmatic.SAES.GUI.Widgets.ZipCodeWidget.ZipCode" ) );

    show_all_children();

    setWidget ( mainFrame );
}

ZipCodeWidget::~ZipCodeWidget()
{
    remove();

    if ( mainFrame )
    {
        delete mainFrame;
        mainFrame = NULL;
    }

    if ( mainAlignment )
    {
        delete mainAlignment;
        mainAlignment = NULL;
    }

    if ( tableAlignment )
    {
        delete tableAlignment;
        tableAlignment = NULL;
    }

    if ( keyBoardAlignment )
    {
        delete keyBoardAlignment;
        keyBoardAlignment = NULL;
    }

    if ( mainVbox )
    {
        delete mainVbox;
        mainVbox = NULL;
    }

    if ( titleText )
    {
        delete titleText;
        titleText = NULL;
    }

    if ( zipCodeText )
    {
        delete zipCodeText;
        zipCodeText = NULL;
    }

    if ( messageText )
    {
        delete messageText;
        messageText = NULL;
    }

    if ( separator )
    {
        delete separator;
        separator = NULL;
    }

    if ( table )
    {
        delete table;
        table = NULL;
    }

    if ( keyboard )
    {
        delete keyboard;
        keyboard = NULL;
    }

    if ( zipCodeEntry )
    {
        delete zipCodeEntry;
        zipCodeEntry = NULL;
    }
}

void ZipCodeWidget::initialize()
{
    setOkButtonSensitive ( false );
    clearMessage();
    zipCodeEntry->focus();
}

std::string ZipCodeWidget::getZipCode()
{
    return zipCodeEntry->get_text();
}

void ZipCodeWidget::setZipCode ( std::string zipCode )
{
    std::string title;

    if ( zipCode.empty() )
    {
        title = _ ( "Zip Code Empty" );
    }
    else
    {
        title = Glib::ustring::compose ( _ ( "Smartmatic.SAES.GUI.Widgets.ZipCodeWidget.Title" ), zipCode );
    }

    titleText->set_text ( title );
    zipCodeEntry->set_text ( zipCode );
    zipCodeEntry->focus();
}

void ZipCodeWidget::clearMessage()
{
    setOkButtonSensitive ( zipCodeEntry->validate() );
    keyboard->setSensitiveButton ( kConfirma, zipCodeEntry->validate() );
    messageText->set_text ( " " );
}

void ZipCodeWidget::SetKey ( KeysActionsCode value )
{
    switch ( value )
    {
        case Key_0:
        case Key_Aaccent:
        case Key_Arroba:
        case Key_Pad0:
        case Key_Caps0:
            OnKeyPress ( 0 );
            break;

        case Key_1:
        case Key_And:
        case Key_Alt1:
        case Key_F1:
        case Key_Pad1:
            OnKeyPress ( 1 );
            break;

        case Key_2:
        case Key_Eaccent:
        case Key_Alt2:
        case Key_F2:
        case Key_Pad2:
        case Key_Caps2:
            OnKeyPress ( 2 );
            break;

        case Key_3:
        case Key_Quote:
        case Key_F3:
        case Key_Pad3:
            OnKeyPress ( 3 );
            break;

        case Key_4:
        case Key_SingleQuote:
        case Key_OpenKey:
        case Key_F4:
        case Key_Pad4:
            OnKeyPress ( 4 );
            break;

        case Key_5:
        case Key_OpenParentheses:
        case Key_OpenBrace:
        case Key_F5:
        case Key_Pad5:
            OnKeyPress ( 5 );
            break;

        case Key_6:
        case Key_Minus:
        case Key_Or:
        case Key_F6:
        case Key_Pad6:
            OnKeyPress ( 6 );
            break;

        case Key_7:
        case Key_Eaccent2:
        case Key_Accent2:
        case Key_F7:
        case Key_Pad7:
        case Key_Caps7:
            OnKeyPress ( 7 );
            break;

        case Key_8:
        case Key_MinusBig:
        case Key_BackSlash:
        case Key_F8:
        case Key_Pad8:
            OnKeyPress ( 8 );
            break;

        case Key_9:
        case Key_Caccent:
        case Key_InversedV:
        case Key_F9:
        case Key_Pad9:
        case Key_Caps9:
            OnKeyPress ( 9 );
            break;

        case Key_Numeral:
            clearMessage();
            validate();
            break;

        case Key_BS:
        case Key_Asterisk:
            clearMessage();
            zipCodeEntry->set_text ( "" );
            break;

        case Key_ESC:
        case Key_DEL:
        case Key_space:
        case Key_ENTER:
        case Key_Dollar:
        case Key_Percent:
        case Key_Plus:
        case Key_Comma:
        case Key_Point:
        case Key_Slash:
        case Key_TwoPoints:
        case Key_PointComma:
        case Key_Equal:
        case Key_Question:
        case Key_A:
        case Key_B:
        case Key_C:
        case Key_D:
        case Key_E:
        case Key_F:
        case Key_G:
        case Key_H:
        case Key_I:
        case Key_J:
        case Key_K:
        case Key_L:
        case Key_M:
        case Key_N:
        case Key_O:
        case Key_P:
        case Key_Q:
        case Key_R:
        case Key_S:
        case Key_T:
        case Key_U:
        case Key_V:
        case Key_W:
        case Key_X:
        case Key_Y:
        case Key_Z:
        case Key_a:
        case Key_b:
        case Key_c:
        case Key_d:
        case Key_e:
        case Key_f:
        case Key_g:
        case Key_h:
        case Key_i:
        case Key_j:
        case Key_k:
        case Key_l:
        case Key_m:
        case Key_n:
        case Key_o:
        case Key_p:
        case Key_q:
        case Key_r:
        case Key_s:
        case Key_t:
        case Key_u:
        case Key_v:
        case Key_w:
        case Key_x:
        case Key_y:
        case Key_z:
        case Key_PadAsterisk:
        case Key_PadPlus:
        case Key_PadMinus:
        case Key_Invalid:
        default:
            break;
    }
}

void ZipCodeWidget::OnKeyPress ( int data )
{
    clearMessage();

    if ( data == kConfirma )
    {
        validate();
    }
    else if ( data == kCorrige )
    {
        zipCodeEntry->set_text ( "" );
    }
    else
    {
        std::string text = zipCodeEntry->get_text();
        text.append ( Glib::Ascii::dtostr ( data ) );
        zipCodeEntry->set_text ( text );
    }
}

void ZipCodeWidget::OnKeyPressT9 ( char data )
{
}

void ZipCodeWidget::validate()
{
    if ( zipCodeEntry->validate() )
    {
        emit();
    }
    else
    {
        std::string message = Glib::ustring::compose ( _ ( "Smartmatic.SAES.GUI.Widgets.ZipCodeWidget.ZipCodeError" ), SAESGUIConfigurationManager::getInstance()->getZipCodeConfiguration().getMax() );
        messageText->set_text ( message );
    }
}
