/**
 * @file   ExternalPrintWizardWidget.cxx
 * @brief  Implementation of ExternalPrintWizardWidget class.
 * @author manuel.arguelles
 * @date   Jun  6, 2017
 */

#include "ExternalPrintWizardWidget.hxx"
#include <Configuration/MachineModelEnum.hxx>
#include <Configuration/s-a-e-s-g-u-i-conf-manager.h>
#include <Configuration/s-m-t-t-g-u-i-conf-manager.h>
#include <Gui/Widgets/ProgressWidget.hxx>
#include <Gui/Windows/MemoryMessageWindow.hxx>
#include <Log/SMTTLogManager.h>
#include <Operation/CheckDataManager.hxx>
#include <Operation/MachineOperationManager.hxx>
#include <Operation/Monitor/DeviceTypeMonitorPool.hxx>
#include <Runtime/Environment/MemoryPathController.h>
#include <System/Runtime/PowerDown.hxx>
#include <System/Sounds/smartmatic-sounds-manager.h>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#include <libintl.h>
#define _(String) dgettext (GETTEXT_PACKAGE, String)
#else
#define _(String) (String)
#endif

#define N_(String) (String)

#define BUTTON_WIDTH 300
#define BUTTON_HEIGHT 50
#define MAIN_BUTTON_MAX_WIDTH 1250
#define MAIN_BUTTON_MAX_HEIGHT 85

using namespace Smartmatic::Configuration;
using namespace Smartmatic::Log;
using namespace Smartmatic::SAES::Common::Operation::Monitor;
using namespace Smartmatic::GUI::Configuration;
using namespace Smartmatic::SAES::GUI::Configuration;
using namespace Smartmatic::SAES::GUI::Widgets;
using namespace Smartmatic::SAES::GUI::Windows;
using namespace Smartmatic::SAES::Operation::Monitor;
using namespace Smartmatic::SAES::Operation;
using namespace Smartmatic::SAES::Runtime::Environment;
using namespace Smartmatic::System::Runtime;
using namespace Smartmatic::System::Sounds;

static ISMTTLog* logger = SMTTLogManager::GetLogger(
    "Smartmatic.SAES.GUI.ExternalPrintWizardWidget",
    GETTEXT_PACKAGE);

bool ExternalPrintWizardWidget::m_firstRun = true;

ExternalPrintWizardWidget::ExternalPrintWizardWidget(ProgressWidget* progress)
    : StepWidget(), m_progress(progress)
{
    int buttonWidth = BUTTON_WIDTH;
    int buttonHeight = BUTTON_HEIGHT;
    int mainButtonWidth = SAESGUIConfigurationManager::getInstance ()->getExternalPrintWizardWidgetConfiguration().getButtonConf().getButtonWidth();
    int mainButtonHeight = SAESGUIConfigurationManager::getInstance ()->getExternalPrintWizardWidgetConfiguration().getButtonConf().getButtonHeight();

    if ( SMTTGUIConfigurationManager::getInstance()->getSmartmaticGUIMachineModelConfiguration().getId() == EnumToString( PM_VIU805 ) )
    {
        double guiReconfigurationFactor = SMTTGUIConfigurationManager::getInstance()->getConfiguration()->getSmartmaticGUIMachineModelConfiguration().getGuiReconfigurationFactor();

        buttonWidth *= guiReconfigurationFactor;
        buttonHeight *= guiReconfigurationFactor;
        mainButtonWidth = MAIN_BUTTON_MAX_WIDTH;
        mainButtonHeight = MAIN_BUTTON_MAX_HEIGHT;
    }

    m_progress->hideInfoTitle();
    m_progress->hideInfoText();
    m_progress->hideInfoFooter();

    m_mainButton.hide();
    m_mainButton.set_no_show_all();
    m_mainButton.set_size_request( mainButtonWidth, mainButtonHeight );

    m_okButton.hide();
    m_okButton.set_no_show_all();
    m_okButton.set_size_request( buttonWidth, buttonHeight );
    m_okButton.set_label(_("Smartmatic.SAES.GUI.Widgets.VotingMachineInstallationWizardWidget.FinishButton"));
    m_okButton.signal_clicked().connect(sigc::mem_fun(*this, &ExternalPrintWizardWidget::onOkButtonClicked));
    m_okHBox.pack_start(m_okButton, true, false);

    m_mainVBox.pack_start(*m_progress, true, true);
    m_mainVBox.pack_start(m_okHBox, true, false);
    m_mainVBox.pack_end(m_mainButton, false, false);

    setWidget(&m_mainVBox);
    set_visible(true);
}

ExternalPrintWizardWidget::~ExternalPrintWizardWidget()
{
    remove();

    if (m_progress)
    {
        delete m_progress;
        m_progress = NULL;
    }

    m_buttonConnection.disconnect();
    restoreMonitor();
}

void ExternalPrintWizardWidget::Initialize()
{
    MemoryMessageWindow::getInstance()->stop();
    m_memoryEventConnection = MemoryEventManager::getInstance()->getMemorySignal().connect(
        sigc::mem_fun(*this, &ExternalPrintWizardWidget::onMemoryEvent));
    CheckDataManager::getInstance()->startSafeSynchronize();
}

ExternalPrintWizardWidget::FinishSignal ExternalPrintWizardWidget::getFinishSignal()
{
    return m_finishSignal;
}

void ExternalPrintWizardWidget::onMemoryEvent(MemoryEventManager::MemoryEvent event)
{
    switch (event)
    {
        case MemoryEventManager::ONE_MEMORY:
        case MemoryEventManager::INTERRUPTED:
            MemoryMessageWindow::getInstance()->showOneMemory();
            break;
        case MemoryEventManager::INTERRUPTED_NO_MEMORY:
            MemoryMessageWindow::getInstance()->showReinsertBothMemories();
            break;
        case MemoryEventManager::START_SYNCHRONIZE:
            MemoryMessageWindow::getInstance()->QuitDialog();
            m_progress->setImage("ui/check-flash-structure.gif");
            m_progress->setInfoText(N_("Process synchronization, before printing."));
            m_progress->showInfoText();
            m_mainButton.hide();
            m_okButton.hide();
            break;
        case MemoryEventManager::TWO_MEMORY:
            MemoryMessageWindow::getInstance()->QuitDialog();
            m_progress->setImage("ui/check-flash-structure.gif");
            m_progress->setInfoText(N_("Process synchronization, after printing."));
            m_progress->showInfoText();
            m_mainButton.hide();
            m_okButton.hide();
            break;
        case MemoryEventManager::END_SYNCHRONIZE:
            MemoryMessageWindow::getInstance()->QuitDialog();
            SMTTSoundsManager::getInstance()->Sound ( "SoundRemoveMemory" );
            onWaitingToPrint();
            break;
        case MemoryEventManager::SAFE_EJECT:
            onPrinting();
            break;
        case MemoryEventManager::CHECK_MEMORY_OK:
            MemoryMessageWindow::getInstance()->QuitDialog();
            onComplete();
            break;
        case MemoryEventManager::CHECK_MEMORY_ERROR:
            MemoryMessageWindow::getInstance()->showMemoryCheckError();
            break;
    }
}

void ExternalPrintWizardWidget::restoreMonitor()
{
    m_memoryEventConnection.disconnect();
    MemoryMessageWindow::getInstance()->start();
}

void ExternalPrintWizardWidget::onCancel()
{
    CheckDataManager::getInstance()->clearSynchronize();
    restoreMonitor();
    m_finishSignal.emit();
}

void ExternalPrintWizardWidget::onComplete()
{
    m_progress->setImage("ui/check-directory-structure_ok.png");
    m_progress->setInfoText(N_("Printing Process Successful"));
    m_buttonConnection.disconnect();
    m_mainButton.hide();
    m_okButton.show();
}

void ExternalPrintWizardWidget::onWaitingToPrint()
{
    m_progress->setInfoText(N_("Please remove one USB to continue with printing process"));
    m_progress->setImage("ui/wiz_extract_media.gif");
    m_progress->showInfoText();

    if (!m_firstRun)
    {
        m_mainButton.SetLabel(_("Cancel safe eject"));
        m_buttonConnection = m_mainButton.signal_clicked().connect(
            sigc::mem_fun(*this, &ExternalPrintWizardWidget::onCancel));
        m_mainButton.show();
    }
}

void ExternalPrintWizardWidget::onPrinting()
{
    m_progress->setInfoText(N_("Please use the USB to print the Election Report in a Voting Machine\n and reconnect it to continue"));
    m_progress->setImage("ui/print_election_report.gif");
    m_progress->showInfoText();
    m_mainButton.hide();
    m_firstRun = false;
}

void ExternalPrintWizardWidget::onOkButtonClicked()
{
    restoreMonitor();
    m_finishSignal.emit();
}
