/**
 * @file GenericPrintingWidget.cxx
 * @brief Body class generic widget
 * @date 29/11/2012
 * @author Yadickson Soto
 */

#include "GenericPrintingWidget.hxx"
#include <Configuration/s-a-e-s-g-u-i-conf-manager.h>
#include <Printing/PrinterStatusNotifier.h>
#include <Log/ISMTTLog.h>
#include <Log/SMTTLogManager.h>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#include <libintl.h>
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

#define N_(String) (String) ///<macro for gettext not translate

using namespace Smartmatic::SAES::GUI::Widgets;
using namespace Smartmatic::Log;
using namespace Smartmatic::GUI::Configuration;
using namespace Smartmatic::SAES::GUI::Configuration;
using namespace Smartmatic::GUI::Widgets;
using namespace Smartmatic::SAES::Printing;
using namespace Smartmatic::SAES::Configuration;
using namespace Smartmatic::SAES::Common::Configuration;
using namespace Smartmatic::SAES::Operation::Printer;
using namespace Smartmatic::SAES::Operation::Devices;

ISMTTLog * GenericPrintingWidget::logger = SMTTLogManager::GetLogger ( "Smartmatic.SAES.GUI.Widgets.GenericPrintingWidget", GETTEXT_PACKAGE );

GenericPrintingWidget::GenericPrintingWidget ( GenericWidget * centerWidget )
    : WizardWidgetBase(), centerWidget ( centerWidget )
{
    centerWidget->setImage ( Smartmatic::SAES::GUI::Operation::ImageResourcePath::PREPARE_PRINT );
    centerWidget->setTextButtons ( _ ( "Smartmatic.SAES.GUI.Widgets.DeviceDiagnosticPrintWidget.Retry" ), _ ( "Smartmatic.SAES.GUI.Widgets.DeviceDiagnosticPrintWidget.Cancel" ) );
    centerWidget->showButtons ( false, false );

    centerWidget->getSignalAcceptButtonClicked().connect ( sigc::mem_fun ( *this, &GenericPrintingWidget::onSignalAccept ) );
    centerWidget->getSignalCancelButtonClicked().connect ( sigc::mem_fun ( *this, &GenericPrintingWidget::onSignalCancel ) );

    status = false;
    printObj = NULL;

    showRetry = false;
    showCancel = false;
    pixbuff = Glib::RefPtr<Gdk::Pixbuf> ( NULL );

    printer = new PrinterController();

    this->setWidget ( centerWidget );

    setAccessibleName ( "GenericPrintingWidget" );
}

GenericPrintingWidget::~GenericPrintingWidget()
{
    connOk.disconnect();
    connError.disconnect();

    remove();

    if ( centerWidget )
    {
        delete centerWidget;
        centerWidget = NULL;
    }

    if ( printer )
    {
        delete printer;
        printer = NULL;
    }
}

void GenericPrintingWidget::setInfoTitle ( std::string text )
{
    title = text;
    centerWidget->setInfoTitle ( text );
}

void GenericPrintingWidget::setInfoText ( std::string text )
{
    message = text;
    centerWidget->setInfoText ( text );
}

void GenericPrintingWidget::setTextButtons ( std::string ok, std::string cancel )
{
    centerWidget->setTextButtons ( ok, cancel );
}

void GenericPrintingWidget::start ( Smartmatic::SAES::Printing::IBasePrintDocument * printObj, bool showRetry, bool showCancel, Glib::RefPtr<Gdk::Pixbuf> pixbuff )
{
    connOk.disconnect();
    connError.disconnect();

    connOk = printer->getSignalOk().connect ( sigc::mem_fun ( *this, &GenericPrintingWidget::endPrinting ) );
    connError = printer->getSignalError().connect ( sigc::mem_fun ( *this, &GenericPrintingWidget::errorPrint ) );

    this->printObj = printObj;
    this->showRetry = showRetry;
    this->showCancel = showCancel;
    this->pixbuff = pixbuff;

    startPrinting();
}

void GenericPrintingWidget::startPrinting()
{
    PrinterStatusNotifier::runForcedPrintBlank();
    sleep ( 1 );

    on_disable_close_signal_emit().emit();

    if ( pixbuff != Glib::RefPtr<Gdk::Pixbuf> ( NULL ) )
    {
        centerWidget->setImage ( pixbuff );
    }
    else
    {
        centerWidget->setImage ( Smartmatic::SAES::GUI::Operation::ImageResourcePath::PRINTING );
    }

    centerWidget->setInfoTitle ( title );
    centerWidget->setInfoText ( message );

    centerWidget->showInfoFooter ( false );
    centerWidget->showButtons ( false, false );

    printer->start ( printObj );
}

void GenericPrintingWidget::endPrinting()
{
    status = true;
    centerWidget->setInfoText ( _ ( "Smartmatic.SAES.GUI.Widgets.DeviceDiagnosticPrintWidget.PrintOk" ) );
    centerWidget->showInfoFooter ( false );

    if ( pixbuff != Glib::RefPtr<Gdk::Pixbuf> ( NULL ) )
    {
        centerWidget->setImage ( pixbuff );
    }
    else
    {
        centerWidget->setImage ( Smartmatic::SAES::GUI::Operation::ImageResourcePath::PRINTING_OK );
    }

    centerWidget->showButtons ( false, false );
    Glib::signal_timeout().connect_once ( sigc::mem_fun ( *this, &GenericPrintingWidget::end ), 1500 );
}

void GenericPrintingWidget::end()
{
    logger->Debug ( "end" );
    emit();
}

void GenericPrintingWidget::onSignalCancel()
{
    centerWidget->showInfoFooter ( false );
    centerWidget->showButtons ( false, false );
    signalError.emit();
}

void GenericPrintingWidget::onSignalAccept()
{
    startPrinting();
}

bool GenericPrintingWidget::getStatus()
{
    return status;
}

GenericPrintingWidget::SignalErrorPrinting GenericPrintingWidget::getSignalErrorPrinting()
{
    return signalErrorPrinting;
}

sigc::signal<void> GenericPrintingWidget::getSignalError()
{
    return signalError;
}

void GenericPrintingWidget::errorPrint ( std::string errorMessage )
{
    centerWidget->showButtons ( showRetry, showCancel );

    centerWidget->setInfoTitle ( _ ( "Smartmatic.SAES.GUI.Widgets.DeviceDiagnosticPrintWidget.PrintError" ) );
    centerWidget->setInfoText ( errorMessage );

    std::string message;

    switch ( SAESGUIConfigurationManager::getInstance()->getStartConfiguration().getInfoMachine() )
    {
        case InfoMachineConfiguration::CONFIGURATION_PM:
            message = _ ( "Smartmatic.SAES.GUI.Widgets.PrinterDiagnosisWidget.AditionalMessagePM" );
            break;

        default:
        case InfoMachineConfiguration::CONFIGURATION_VM:
            message = _ ( "Smartmatic.SAES.GUI.Widgets.PrinterDiagnosisWidget.AditionalMessageVM" );
            break;
    }

    centerWidget->setInfoFooter ( message );

    if ( pixbuff != Glib::RefPtr<Gdk::Pixbuf> ( NULL ) )
    {
        centerWidget->setImage ( pixbuff );
    }
    else
    {
        centerWidget->setImage ( Smartmatic::SAES::GUI::Operation::ImageResourcePath::PRINTING_ERROR );
    }

    signalErrorPrinting.emit();

    if ( !showRetry && !showCancel )
    {
        Glib::signal_timeout().connect_once ( sigc::mem_fun ( *this, &GenericPrintingWidget::end ), 5000 );
        return;
    }
}

