/**
 * @file LoginWidget.cxx
 * @brief Body class Login widget
 * @date 17/10/2011
 * @author Yadickson Soto
 */

#include "LoginWidget.hxx"
#include <Configuration/MachineModelEnum.hxx>
#include <Configuration/s-a-e-s-g-u-i-conf-manager.h>
#include <Configuration/s-m-t-t-g-u-i-conf-manager.h>
#include <Environment/Configurations/resource-path-provider.h>
#include <Log/ISMTTLog.h>
#include <Log/SMTTLogManager.h>
#include <System/Utils/StringUtils.hxx>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#include <libintl.h>
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

#define N_(String) (String) ///<macro for gettext not translate

#define MAIN_ALIGNMENT_BOTTOM_PADDING 350
#define BUTTON_WIDTH  200
#define BUTTON_HEIGHT 70
#define KEYBOARD_WIDTH 1250
#define KEYBOARD_HEIGHT 400
#define KEYBOARD_DEFAULT_LANGUAGE "fr_BE.UTF8"

using namespace Smartmatic::SAES::GUI::Widgets;
using namespace Smartmatic::Configuration;
using namespace Smartmatic::Environment;
using namespace Smartmatic::Log;
using namespace Smartmatic::GUI::Widgets;
using namespace Smartmatic::SAES::Configuration;
using namespace Smartmatic::GUI::Configuration;
using namespace Smartmatic::SAES::GUI::Configuration;

ISMTTLog * LoginWidget::logger = SMTTLogManager::GetLogger ( "Smartmatic.SAES.GUI.Widgets.LoginWidget", GETTEXT_PACKAGE );

static const Glib::ustring KEYBOARD_MAIN_KEYS_ARRAY[] =
{
    "~", "1", "2", "3", "4", "5", "6", "7", "8", "9", "0", "-", "=",
    "a", "z", "e", "r", "t", "y", "u", "i", "o", "p", "[", "]", "\\",
    "q", "s", "d", "f", "g", "h", "j", "k", "l", "m", "'",
    "w", "x", "c", "v", "b", "n", ",", ";", ":",
    "_"
};

static const Glib::ustring KEYBOARD_SECOND_KEYS_ARRAY[] =
{
    "à", "â", "è", "ê", "ì", "î", "ò", "ô", "ù", "û", "ß", "ÿ", "æ",
    "á", "°", "é", "\"", "í", "§", "ó", "{", "ú", "/", "!", "¡", "œ",
    "ä", "@", "ë", "#", "ï", "ç", "ö", "}", "ü", "µ", "^",
    "&", "(", ")", "$", "?", "¿", "€", "+", "",
    ""
};

LoginWidget::LoginWidget ( bool onlyIcons, EntryDividedWidgetConfiguration & config )
    : SmartmaticBasicFrame()
{
    this->mainFrame = NULL;
    this->tableLogin = NULL;
    this->labelTitle = NULL;
    this->labelLogin = NULL;
    this->labelPassword = NULL;
    this->imageLogin = NULL;
    this->imagePassword = NULL;
    this->login = NULL;
    this->password = NULL;
    this->buttonOk = NULL;
    this->tableKeyboard = NULL;
    this->keyboard = NULL;
    this->onlyIcons = onlyIcons;
    this->entryDivisions = config.getNumberOfDivision();
    this->entryMaxChar = config.getMaxCharacters();
    this->entryWidth = SAESGUIConfigurationManager::getInstance()->getCurrentMachineModelConfiguration().getLoginWidgetEntryConfiguration().getEntryWidth();
    this->entryHeight = SAESGUIConfigurationManager::getInstance()->getCurrentMachineModelConfiguration().getLoginWidgetEntryConfiguration().getEntryHeight();
    this->entryVisibility = config.getVisibility();
    this->set_shadow_type ( Gtk::SHADOW_NONE );
    this->initialize();
    this->add ( *mainFrame );
    this->show_all_children();

    this->setAccessibleName ( "LoginWidget" );
}

LoginWidget::~LoginWidget()
{
    dispose();
}

void LoginWidget::onKeyPress ( std::string data )
{
    std::string currentData = "";

    if ( password->hasFocus() == false )
    {
        currentData = login->get_text();
        login->set_text( currentData + data );
    }
    else
    {
        password->AddData( data );
    }
}

void LoginWidget::onKeyEnterPress ()
{
    pressLoginButton();
}

void LoginWidget::onKeyDeletePress()
{
    std::string currentData = "";

    if ( password->hasFocus() == false )
    {
        currentData = login->get_text();

        if ( currentData.size() > 0 )
            currentData.erase( currentData.begin() + currentData.size() - 1 );

        login->set_text( currentData );
    }
    else
    {
        password->OnKeyDeletePress();
    }
}

bool LoginWidget::on_login_focus_in ( GdkEventFocus * event )
{
    password->setHasFocus( false );

    return true;
}

void LoginWidget::initialize()
{
    dispose();
    Glib::RefPtr<Gtk::Builder> builder;
    std::string gladeFile;
    std::string iconLoginFile;
    std::string iconPasswordFile;
    gladeFile = ResourcePathProvider::getInstance()->GetResourcePath ( PACKAGE_NAME, "ui/LoginWidget.glade" );
    iconLoginFile = ResourcePathProvider::getInstance()->GetResourcePath ( PACKAGE_NAME, "ui/PM_login.gif" );
    iconPasswordFile = ResourcePathProvider::getInstance()->GetResourcePath ( PACKAGE_NAME, "ui/PM_pass.gif" );

    int buttonWidth  = BUTTON_WIDTH;
    int buttonHeight = BUTTON_HEIGHT;

    try
    {
        builder = Gtk::Builder::create_from_file ( gladeFile );
    }
    catch ( const Glib::FileError & ex )
    {
        logger->Debug ( "FATAL - LoginWidget Initialize" );
    }

    builder->get_widget_derived ( "main-frame", mainFrame );
    builder->get_widget_derived ( "table-login", tableLogin );
    builder->get_widget_derived ( "entry-login", login );
    builder->get_widget_derived ( "label-title", labelTitle );
    builder->get_widget_derived ( "label-login", labelLogin );
    builder->get_widget_derived ( "label-password", labelPassword );
    builder->get_widget_derived ( "image-login", imageLogin );
    builder->get_widget_derived ( "image-password", imagePassword );
    builder->get_widget_derived ( "button-ok", buttonOk );

    if ( SMTTGUIConfigurationManager::getInstance()->getSmartmaticGUIMachineModelConfiguration().getId() == EnumToString( PM_VIU805 ) )
    {
        double guiReconfigurationFactor = SMTTGUIConfigurationManager::getInstance()->getConfiguration()->getSmartmaticGUIMachineModelConfiguration().getGuiReconfigurationFactor();

        buttonWidth *= guiReconfigurationFactor;
        buttonHeight *= guiReconfigurationFactor;

        builder->get_widget_derived ( "table-keyboard", tableKeyboard );
        setKeyboard();
    }

    password = new SmartmaticEntryDivided ( entryDivisions, entryMaxChar, entryWidth, entryHeight, entryVisibility );
    password->signalEntryComplete().connect(sigc::mem_fun(*this, &LoginWidget::pressLoginButton));

    login->signal_focus_in_event().connect ( sigc::mem_fun ( *this, &LoginWidget::on_login_focus_in ) );

    tableLogin->attach ( *password, 1, 2, 1, 2, Gtk::SHRINK, Gtk::SHRINK );
    labelTitle->set_text ( !onlyIcons ? _ ( "Smartmatic.SAES.GUI.Widgets.PMInstallationWidget.InsertYourLoginAndPassword" ) : " " );
    labelLogin->set_text ( !onlyIcons ? _ ( "Smartmatic.SAES.GUI.Widgets.PMInstallationWidget.Login" ) : " " );
    labelPassword->set_text ( !onlyIcons ? _ ( "Smartmatic.SAES.GUI.Widgets.PMInstallationWidget.Password" ) : " " );

    imageLogin->set ( Gdk::PixbufAnimation::create_from_file ( iconLoginFile ) );
    imagePassword->set ( Gdk::PixbufAnimation::create_from_file ( iconPasswordFile ) );

    buttonOk->SetButtonType ( SmartmaticButton::Ok );
    buttonOk->WidgetsPosition ( 0, 5, 0 );
    buttonOk->set_size_request( buttonWidth, buttonHeight );

    std::vector<SmartmaticEntry *> & entryList ( password->getEntriesVector() );
    std::vector<SmartmaticEntry *>::iterator it;
    int i = 1;

    for ( it = entryList.begin();
            it != entryList.end();
            ++it, i++ )
    {
        ( *it )->setAccessibleName ( "Password" + Smartmatic::System::Utils::StringUtils::intToString ( i ) );
    }

    mainFrame->setAccessibleName ( "MainContainerLoginWidget" ); // container
    tableLogin->setAccessibleName ( "ContainerLoginWidget" );
    labelTitle->setAccessibleName ( "Title" );
    labelLogin->setAccessibleName ( "Login" );
    labelPassword->setAccessibleName ( "Password" );

    login->set_size_request( -1, entryHeight );
    login->setAccessibleName ( "Login" );
    imageLogin->setAccessibleName ( "Login" );

    password->setAccessibleName ( "PasswordContainer" );
    imagePassword->setAccessibleName ( "Password" );

    buttonOk->setAccessibleName ( "LoginOk" );
}

void LoginWidget::dispose()
{
    this->remove();

    if ( mainFrame )
    {
        delete mainFrame;
        mainFrame = NULL;
    }

    if ( tableLogin )
    {
        delete tableLogin;
        tableLogin = NULL;
    }

    if ( labelTitle )
    {
        delete labelTitle;
        labelTitle = NULL;
    }

    if ( labelLogin )
    {
        delete labelLogin;
        labelLogin = NULL;
    }

    if ( labelPassword )
    {
        delete labelPassword;
        labelPassword = NULL;
    }

    if ( imageLogin )
    {
        delete imageLogin;
        imageLogin = NULL;
    }

    if ( imagePassword )
    {
        delete imagePassword;
        imagePassword = NULL;
    }

    if ( login )
    {
        delete login;
        login = NULL;
    }

    if ( password )
    {
        delete password;
        password = NULL;
    }

    if ( buttonOk )
    {
        delete buttonOk;
        buttonOk = NULL;
    }

    if ( tableKeyboard )
    {
        delete tableKeyboard;
        tableKeyboard = NULL;
    }

    if ( keyboard )
    {
        delete keyboard;
        keyboard = NULL;
    }
}

void LoginWidget::setKeyboard()
{
    std::setlocale ( LC_ALL, KEYBOARD_DEFAULT_LANGUAGE );

    const std::vector<std::string> mainKeys(KEYBOARD_MAIN_KEYS_ARRAY, KEYBOARD_MAIN_KEYS_ARRAY + sizeof KEYBOARD_MAIN_KEYS_ARRAY / sizeof KEYBOARD_MAIN_KEYS_ARRAY[0]);
    const std::vector<std::string> secondKeys(KEYBOARD_SECOND_KEYS_ARRAY, KEYBOARD_SECOND_KEYS_ARRAY + sizeof KEYBOARD_SECOND_KEYS_ARRAY / sizeof KEYBOARD_SECOND_KEYS_ARRAY[0]);

    keyboard = new Keyboard (mainKeys, secondKeys);

    keyboard->getDetectSignal().connect ( sigc::mem_fun ( *this, &LoginWidget::onKeyPress ) );
    keyboard->getEnterSignal().connect ( sigc::mem_fun ( *this, &LoginWidget::onKeyEnterPress ) );
    keyboard->getDeleteSignal().connect ( sigc::mem_fun ( *this, &LoginWidget::onKeyDeletePress ) );

    keyboard->setSensitiveButton ( escape, false );
    keyboard->setSensitiveButton ( upArrow, false );
    keyboard->setSensitiveButton ( downArrow, false );
    keyboard->setSensitiveButton ( leftArrow, false );
    keyboard->setSensitiveButton ( rightArrow, false );

    keyboard->set_visible( true );
    keyboard->setAccessibleName( "Keyboard" );
    keyboard->set_size_request( KEYBOARD_WIDTH, KEYBOARD_HEIGHT );
    tableKeyboard->attach ( *keyboard, 0, 1, 0, 1, Gtk::EXPAND, Gtk::EXPAND );
}

Glib::SignalProxy0< void > LoginWidget::getSignalButtonOkClicked()
{
    return buttonOk->signal_clicked();
}

void LoginWidget::enabledLogin ( bool enabled )
{
    login->set_editable ( enabled );
    login->set_sensitive ( enabled );
}

void LoginWidget::getLoginText ( std::string & text )
{
    text = login->get_text();
}

void LoginWidget::setLoginText ( std::string & text )
{
    login->set_text ( text );
}

void LoginWidget::getPasswordText ( std::string & text )
{
    text = password->GetText();
}

void LoginWidget::clearLogin()
{
    login->set_text ( "" );
}

void LoginWidget::clearPassword()
{
    password->Clean();
    password->focusFirstEntry();
}

void LoginWidget::focusLogin()
{
    login->focus();
}

void LoginWidget::focusPassword()
{
    password->focusFirstEntry();
}

void LoginWidget::pressLoginButton()
{
    buttonOk->activate();
}
