/**
 * @file MachineVotingWidget.cxx
 * @brief Body class voting experience widget
 * @date 08/03/2012
 * @author Yadickson Soto
 */

#include "MachineVotingWidget.hxx"
#include <Gui/Widgets/Voting/FactoryVotingWidget.hxx>
#include <Gui/Windows/BarcodeReconfigureWindow.hxx>
#include <Log/ISMTTLog.h>
#include <Log/SMTTLogManager.h>
#include <Operation/Devices/CardController.hxx>
#include <Operation/Devices/AlarmController.hxx>
#include <Operation/Devices/Polling/BCRPolling.hxx>
#include <Configuration/s-a-e-s-g-u-i-conf-manager.h>
#include <Operation/AlarmMessage.hxx>
#include <Operation/WindowsManager.hxx>
#include "Printing/PrinterStatusNotifier.h"
#include <Voting/CardData/card-data-wrapper.h>
#include <Operation/Voting/VotingOperation.hxx>
#include <Operation/Voting/AudioVoting/AudioVotingCheckUtility.hxx>
#include <Functionalities/saes-functionality-manager.h>
#include <Operation/Devices/BCRController.hxx>
#include <Operation/Monitor/DeviceTypeMonitorPool.hxx>
#include <Card/ISmartcardReaderPowerManager.hxx>
#include <Operation/Devices/SmartcardReaderPowerManagerFactory.hxx>
#include <Operation/Devices/LEDStripManagerFactory.hxx>
#include <Operation/MachineOperationManager.hxx>
#include <Configuration/AlarmConfigurationManager.hxx>
#include <Configuration/s-a-e-s-functionality-conf-manager.h>
#include <Operation/Voting/VotingEnqueuer.hxx>
#include <memory>
#include <Gui/Widgets/AudioVoting/AudioVotingExperienceWidget.hxx>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#include <libintl.h>
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

#define N_(String) (String) ///<macro for gettext not translate

using namespace Smartmatic::GUI::Configuration;
using namespace Smartmatic::GUI::Operation;
using namespace Smartmatic::GUI::Widgets;
using namespace Smartmatic::GUI::Windows;
using namespace Smartmatic::Log;
using namespace Smartmatic::SAES::Card;
using namespace Smartmatic::SAES::Common::Operation::Monitor;
using namespace Smartmatic::SAES::Configuration;
using namespace Smartmatic::SAES::Functionality::Configuration;
using namespace Smartmatic::SAES::Functionality;
using namespace Smartmatic::SAES::GUI::Configuration;
using namespace Smartmatic::SAES::GUI::Operation::Voting;
using namespace Smartmatic::SAES::GUI::Operation;
using namespace Smartmatic::SAES::GUI::Widgets;
using namespace Smartmatic::SAES::GUI::Widgets::AudioVoting;
using namespace Smartmatic::SAES::GUI::Widgets::Voting;
using namespace Smartmatic::SAES::GUI::Windows;
using namespace Smartmatic::SAES::HW::Devices;
using namespace Smartmatic::SAES::Operation::AudioVoting;
using namespace Smartmatic::SAES::Operation::Devices;
using namespace Smartmatic::SAES::Operation::Monitor;
using namespace Smartmatic::SAES::Operation::Voting;
using namespace Smartmatic::SAES::Operation;
using namespace Smartmatic::SAES::Printing;
using namespace Smartmatic::SAES::Voting;

ISMTTLog * MachineVotingWidget::logger = SMTTLogManager::GetLogger ( "Smartmatic.SAES.GUI.Widgets.MachineVotingWidget", GETTEXT_PACKAGE );

MachineVotingWidget::MachineVotingWidget()
    : RunMachineWidget()
{
    this->setHelp ( false );
    this->setExit ( false );
    this->setShowBorder ( true );
    this->setTitle ( false );
    this->isInterrupted = false;

    audioVoting = false;

    m_thread = NULL;

    stepVoting = WAITING;

    setName ( SAESGUIConfigurationManager::getInstance()->GetVotingExperienceWindowName() );

    initialize();

    simulateVote = SAESFunctionalityConfigurationManager::getInstance()->getOperationConfiguration().getSimulateVotes();

    mainWidget->showInfo ();
    viewWidget ( waitingVotingWidget );
}

MachineVotingWidget::~MachineVotingWidget()
{
    dispose();

    delete mainWidget;
}

void MachineVotingWidget::initialize()
{
    waitingVotingWidget = FactoryVotingWidget::createWaitingVotingWidget();
    simulationWidget = FactoryVotingWidget::createSimulationWidget();
    languageSelectionWidget = FactoryVotingWidget::createVotingLanguageSelectionWidget();
    previewWidget = FactoryVotingWidget::createPreviewWidget();
    votingExperienceWidget = FactoryVotingWidget::createVotingExperienceWidget();
    audioVotingExperienceWidget = FactoryVotingWidget::createAudioVotingExperienceWidget();

    waitingVotingWidget->getSignalSim().connect ( sigc::mem_fun ( *this, &MachineVotingWidget::simCardData ) );
    waitingVotingWidget->getSignalBack().connect ( sigc::mem_fun ( *this, &MachineVotingWidget::back ) );
    waitingVotingWidget->getSignalAudioVoting().connect ( sigc::mem_fun ( *this, &MachineVotingWidget::setAudioVoting ) );
    simulationWidget->getSignalEnd().connect ( sigc::mem_fun ( *this, &MachineVotingWidget::createCardData ) );
    previewWidget->getSignalEnd().connect ( sigc::mem_fun ( *this, &MachineVotingWidget::restoreWidget ) );
    votingExperienceWidget->getSignalEnd().connect ( sigc::mem_fun ( this, &MachineVotingWidget::stopVotingExperience ) );
    audioVotingExperienceWidget->getSignalEnd().connect ( sigc::mem_fun ( this, &MachineVotingWidget::stopVotingExperience ) );

    signalCardError.getSyncSignal().connect ( sigc::mem_fun ( *this, &MachineVotingWidget::cardError ) );
    signalCardErrorAID.getSyncSignal().connect ( sigc::mem_fun ( *this, &MachineVotingWidget::cardErrorAID ) );
    signalCardErrorPIN.getSyncSignal().connect ( sigc::mem_fun ( *this, &MachineVotingWidget::cardErrorPIN ) );
    signalCardErrorGeneric.getSyncSignal().connect ( sigc::mem_fun ( *this, &MachineVotingWidget::cardErrorGeneric ) );
    signalCardUsed.getSyncSignal().connect ( sigc::mem_fun ( *this, &MachineVotingWidget::showCardUsed ) );
    signalShowLanguage.getSyncSignal().connect ( sigc::mem_fun ( *this, &MachineVotingWidget::showSelectionLanguage ) );

    signalStartAudioVoting.getSyncSignal().connect ( sigc::mem_fun ( *this, &MachineVotingWidget::startAudioVotingExperience ) );
    audioVotingExperienceWidget->getSignalStartAudioVotingExperience().connect ( sigc::mem_fun ( this, &MachineVotingWidget::startValidateCardData ) );

    audioVotingExperienceWidget->getSignalLanguageSelected().connect ( sigc::mem_fun ( this, &MachineVotingWidget::audioVotingLanguageSelected ) );

    signalAlarm.getSyncSignal().connect ( sigc::mem_fun ( *this, &MachineVotingWidget::safeAlarm ) );
    signalBack.connect ( sigc::mem_fun ( *this, &MachineVotingWidget::back ) );
    signalRefresh.connect ( sigc::mem_fun ( *this, &MachineVotingWidget::refresh ) );

    mainWidget = FactoryVotingWidget::createDualWaitingForCardWidget();
    waitingForCardWidget = mainWidget->getWaitingForCardWidget();

    waitingVotingWidget->setWidget ( mainWidget );
}

void MachineVotingWidget::start()
{
    stopAll();

    logger->Debug ( "Start MachineVotingWidget" );

    connLanguage = languageSelectionWidget->getSignalEnd().connect ( sigc::mem_fun ( *this, &MachineVotingWidget::languageSelected ) );

    signalDeviceChange = DeviceTypeMonitorPool::getInstance()->getDeviceChangedSignal().connect ( sigc::mem_fun ( *this, &MachineVotingWidget::deviceChange ) );

    unsigned int timer = SAESGUIConfigurationManager::getInstance()->getAlarmBoxMonitorConfiguration().getMonitoringTime();
    connAlarmMonitor = Glib::signal_timeout().connect ( sigc::bind ( sigc::mem_fun ( *this, &MachineVotingWidget::detectAlarm ), 0 ), timer );

    CardPolling::getInstance()->resume ();

    clearCardData();

    stepVoting = WAITING;

    startCheckAlarmButton();
    startCheckCardPresent();
    startCheckBarcode();

    std::auto_ptr<IStripLEDManager> ledManager(
              LEDStripManagerFactory::getLEDStripManager());
    ledManager->turnAllLedsOn();
    viewWidget ( waitingVotingWidget );

    refresh();

    logger->Debug ( "End Start MachineVotingWidget" );
}

void MachineVotingWidget::viewWidget ( Smartmatic::GUI::Widgets::StepWidget * widget )
{
    logger->Debug ( "View Widget" );
    widget->refresh();
    setWidget ( widget );
    logger->Debug ( "End View Widget" );
}

void MachineVotingWidget::dispose()
{
    stopAll();

    this->remove();

    if ( waitingVotingWidget )
    {
        delete waitingVotingWidget;
        waitingVotingWidget = NULL;
    }

    if ( simulationWidget )
    {
        delete simulationWidget;
        simulationWidget = NULL;
    }

    if ( languageSelectionWidget )
    {
        delete languageSelectionWidget;
        languageSelectionWidget = NULL;
    }

    if ( previewWidget )
    {
        delete previewWidget;
        previewWidget = NULL;
    }

    if ( votingExperienceWidget )
    {
        delete votingExperienceWidget;
        votingExperienceWidget = NULL;
    }

    if ( audioVotingExperienceWidget )
    {
        delete audioVotingExperienceWidget;
        audioVotingExperienceWidget = NULL;
    }

    if ( m_thread != NULL )
    {
        m_thread->join();
        m_thread = NULL;
    }
}

void MachineVotingWidget::restoreWidget()
{
    logger->Debug ( "Restore Widget" );
    stepVoting = WAITING;
    startCheckAlarmButton();
    startCheckCardPresent();
    std::auto_ptr<IStripLEDManager> ledManager(
                LEDStripManagerFactory::getLEDStripManager());
    ledManager->turnAllLedsOn();
    viewWidget ( waitingVotingWidget );
}

void MachineVotingWidget::startCheckCardPresent()
{
    logger->Debug ( "Start check card present" );
    connCardPollingEvent.disconnect();
    connCardPollingEvent = CardPolling::getInstance()->getSignalCard().connect ( sigc::mem_fun ( *this, &MachineVotingWidget::onSignalCardChange ) );
}

void MachineVotingWidget::stopCheckCardPresent()
{
    logger->Debug ( "Stop check card" );
    connCardPollingEvent.disconnect();
}

void MachineVotingWidget::cardInsert()
{
    mainWidget->showInfo ( false );
    waitingVotingWidget->hideButtons();
    std::auto_ptr<IStripLEDManager> ledManager(
                LEDStripManagerFactory::getLEDStripManager());
    ledManager->turnAllLedsOff();
    logger->Debug ( "Card Insert" );

    switch ( stepVoting )
    {
        case WAITING:
            logger->Debug ( "WAITING" );
            stepVoting = VALIDATE;
            startValidateCardData();
            break;

        case VALIDATE:
            logger->Debug ( "VALIDATE" );
            startValidateCardData();
            break;

        case ERROR_VALIDATE:
            logger->Debug ( "ERROR_VALIDATE" );
            startValidateCardData();
            break;

        case SHOW_LANGUAGE:
            logger->Debug ( "SHOW_LANGUAGE" );
            startValidateCardData();
            break;

        case VOTING:
            logger->Debug ( "VOTING" );
            votingExperienceWidget->cardInsertEvent();
            break;

        case AUDIO_VOTING_LANGUAGE_SELECTION:
            logger->Debug ( "AUDIO_VOTING_LANGUAGE_SELECTION" );
            audioVotingExperienceWidget->cardInsertEvent();
            break;

        case AUDIO_VOTING:
            logger->Debug ( "AUDIO_VOTING" );
            audioVotingExperienceWidget->cardInsertEvent();
            break;

        case NONE:
        default:
            logger->Debug ( "NONE" );
            break;
    }
}

void MachineVotingWidget::cardRemove()
{
    logger->Debug ( "Card Remove" );

    std::auto_ptr<IStripLEDManager> ledManager(
         LEDStripManagerFactory::getLEDStripManager());
    ledManager->turnAllLedsOn();
    mainWidget->showInfo();
    waitingVotingWidget->showButtons();

    switch ( stepVoting )
    {
        case WAITING:
            logger->Debug ( "WAITING" );
            startCheckBarcode();
            clearCardData();
            waitingForCardWidget->refresh();
            waitingVotingWidget->refresh();
            break;

        case VALIDATE:
            logger->Debug ( "VALIDATE" );
            startCheckBarcode();
            waitingForCardWidget->refresh();
            waitingVotingWidget->refresh();
            break;

        case ERROR_VALIDATE:
            logger->Debug ( "ERROR_VALIDATE" );
            startCheckBarcode();
            clearCardData();
            waitingForCardWidget->refresh();
            waitingVotingWidget->refresh();
            viewWidget ( waitingVotingWidget );
            break;

        case SHOW_LANGUAGE:
            logger->Debug ( "SHOW_LANGUAGE" );
            startCheckBarcode();
            waitingForCardWidget->refresh();
            viewWidget ( waitingVotingWidget );
            break;

        case VOTING:
            logger->Debug ( "VOTING" );
            clearCardData();
            votingExperienceWidget->cardRemoveEvent();
            break;

        case AUDIO_VOTING_LANGUAGE_SELECTION:
            logger->Debug ( "AUDIO_VOTING_LANGUAGE_SELECTION" );
            audioVotingExperienceWidget->cardRemoveEvent();
            waitingForCardWidget->refresh();
            viewWidget ( waitingVotingWidget );
            break;

        case AUDIO_VOTING:
            logger->Debug ( "AUDIO_VOTING" );
            clearCardData();
            audioVotingExperienceWidget->cardRemoveEvent();
            break;

        case NONE:
        default:
            break;
    }
}

void MachineVotingWidget::cardReaderUnresponsive()
{
    if ( !SAESGUIConfigurationManager::getInstance()->getWaitingForCardWindowConfiguration().getShowSimulateButton() )
    {
        logger->Debug ( "SCR Error" );
        stopAll();
        setAlarm ( AlarmEnum::CONFIGURATION_CARDREADERUNRESPONSIVE );
    }
}

void MachineVotingWidget::startCheckBarcode()
{
    if ( SAESGUIConfigurationManager::getInstance()->getWaitingForCardWindowConfiguration().getAllowVotePreview() )
    {
        logger->Debug ( "Check barcode event." );

        signalVote.disconnect();
        signalStatusVote.disconnect();

        signalVote = VotingEnqueuer::getInstance()->getSignalVote().connect ( sigc::mem_fun ( *this, &MachineVotingWidget::onSignalVote ) );
        signalStatusVote = VotingEnqueuer::getInstance()->getSignalStatus().connect ( sigc::mem_fun ( *this, &MachineVotingWidget::onSignalState ) );

        BarcodeReconfigureWindow::getInstance()->start();
        VotingEnqueuer::getInstance()->resume ();
        BCRPolling::getInstance()->resume ();
    }
}

void MachineVotingWidget::stopCheckBarcode()
{
    signalVote.disconnect();
    signalStatusVote.disconnect();

    VotingEnqueuer::getInstance()->pause();
    BCRPolling::getInstance()->pause();
    BarcodeReconfigureWindow::getInstance()->stop();
}

void MachineVotingWidget::connectBarcode()
{
}

void MachineVotingWidget::disconnectBarcode()
{
}

void MachineVotingWidget::refreshAll()
{

}

void MachineVotingWidget::startCheckAlarmButton()
{
    logger->Debug ( "Start check alarm button" );
    connAlarm.disconnect();
    connAlarm = AlarmPolling::getInstance()->getSignalPressButton().connect ( sigc::mem_fun ( *this, &MachineVotingWidget::onButtonPresed ) );
}

void MachineVotingWidget::stopCheckAlarmButton()
{
    logger->Debug ( "Stop check alarm button" );
    connAlarm.disconnect();
}

void MachineVotingWidget::refresh()
{
    if ( stepVoting == VOTING )
    {
        stopVotingExperience();
    }

    mainWidget->refresh();
    mainWidget->showInfo ();
    waitingForCardWidget->refresh();
    waitingVotingWidget->refresh();
    isInterrupted = false;
}

void MachineVotingWidget::simCardData()
{
    logger->Debug ( "Simulate card data" );
    stopCheckCardPresent();
    stopCheckBarcode();
    viewWidget ( simulationWidget );
}

void MachineVotingWidget::setAudioVoting()
{
    audioVoting = true;
    simCardData();
}

void MachineVotingWidget::startValidateCardData()
{
    logger->Debug ( "Start validate card data" );
    stopCheckBarcode();
    waitingForCardWidget->viewCardValidate();
    mainWidget->showInfo ( false );
    waitingVotingWidget->hideButtons();

    simulateProcess = false;

    if ( !Glib::thread_supported() )
    {
        Glib::thread_init();
    }

    if ( m_thread != NULL )
    {
        m_thread->join();
        m_thread = NULL;
    }

    m_thread = Glib::Thread::create ( sigc::mem_fun ( *this, &MachineVotingWidget::readCardData ), true );
}

void MachineVotingWidget::startAudioVotingExperience(void*)
{
    AudioVotingExperienceController* audioVotingExperienceController;

    audioVotingExperienceController = MachineOperationManager::getInstance()->getAudioVotingExperienceController();
    audioVotingExperienceController->start();

    stopCheckAlarmButton();
    logger->Debug ( "Start audio voting experience" );
    stepVoting = AUDIO_VOTING_LANGUAGE_SELECTION;
    audioVotingExperienceWidget->start();

    viewWidget ( audioVotingExperienceWidget );
}

void MachineVotingWidget::stopAudioVotingExperience()
{
    AudioVotingExperienceController* audioVotingExperienceController;

    audioVotingExperienceController = MachineOperationManager::getInstance()->getAudioVotingExperienceController();
    audioVotingExperienceController->killVotingExperience();
    audioVotingExperienceController->dispose();
}

void MachineVotingWidget::audioVotingLanguageSelected()
{
    clearCardData();
    stepVoting = AUDIO_VOTING;
}

void MachineVotingWidget::startVotingExperience()
{
    stopCheckAlarmButton();
    logger->Debug ( "Start voting experience" );
    clearCardData();
    stepVoting = VOTING;

    votingExperienceWidget->setSimulationMode ( simulateProcess );
    votingExperienceWidget->start();

    viewWidget ( votingExperienceWidget );
}

void MachineVotingWidget::stopVotingExperience()
{
    audioVoting = false;
    stepVoting = WAITING;
    votingExperienceWidget->dispose();

    logger->Debug ( "Stop voting experience" );
    languageSelectionWidget->SetBackLanguage();

    if ( AlarmController::getInstance()->isGoToAuthentication() )
    {
        logger->Debug ( "Alarm go to Authentication" );
        signalBack.emit();
    }
    else
    {
        startCheckAlarmButton();
        startCheckCardPresent();
        startCheckBarcode();

        mainWidget->showInfo ();
        waitingVotingWidget->refresh();
        waitingForCardWidget->refresh();

        viewWidget ( waitingVotingWidget );
    }
}

void MachineVotingWidget::readCardData()
{
    logger->Debug ( "Start read card data" );

    if ( !checkPrinter() )
    {
        return;
    }

    bool validate = false;
    bool insert = true;
    bool block = SAESGUIConfigurationManager::getInstance()->getWaitingForCardWindowConfiguration().getBlockOnCardInsertionError();

    try
    {
        validate = CardController::getInstance()->readCardData ( currentCardData );
        insert = CardController::getInstance()->isCardPresent();

        if ( ! validate && insert )
        {
            signalCardErrorGeneric.emit ( NULL );
        }
    }
    catch ( ... )
    {
        logger->Debug ( "Error read card data" );
        block  ? signalAlarm.emit ( new AlarmEnum ( AlarmEnum::CONFIGURATION_INVALIDVOTINGDATA ) ) : signalCardErrorGeneric.emit ( NULL );
    }

    if ( validate && insert )
    {
        validateCardData();
    }

    logger->Debug ( "End read card data" );
}

void MachineVotingWidget::createCardData()
{
    logger->Debug ( "Create card data" );
    simulateProcess = true;
    simulationWidget->getCardData ( currentCardData );
    validateCardData();
}

void MachineVotingWidget::validateCardData()
{
    logger->Debug ( "Validate card data" );
    connAlarmMonitor.disconnect();

    try
    {
        bool notCompletelyUsed = false;
        bool cond1 = ( currentCardData.getCard_type() == CardData::Card_type::CARD_DATA_OFFICIAL_VOTE_CARD ) ||
                     ( currentCardData.getCard_type() == CardData::Card_type::CARD_DATA_DEMOSTRATION_VOTE_CARD );
        bool cond2 = true;

        if ( CardData::CardDataWrapper::isUsed ( currentCardData ) )
        {
            if ( currentCardDataCode == currentCardData.getActivation_code() )
            {
                notCompletelyUsed = true;
                cond2 = true;
            }
            else
            {
                notCompletelyUsed = false;
                cond2 = false;
            }
        }

        cond2 |= !Smartmatic::SAES::GUI::Configuration::SAESGUIConfigurationManager::getInstance()->getMarkCardAsUsed ();

        currentCardDataCode = currentCardData.getActivation_code();

        if ( currentCardData.getBallotID().empty() )
        {
            currentCardData.setBallotID ( currentBallotIDCardData );
        }
        else
        {
            currentBallotIDCardData = currentCardData.getBallotID();
        }

        if ( currentCardData.getSessionID().empty() )
        {
            currentCardData.setSessionID ( currentSessionIDCardData );
        }
        else
        {
            currentSessionIDCardData = currentCardData.getSessionID();
        }

        if ( cond1 && cond2 )
        {
            logger->Debug ( "Check activation code." );
            std::string verifCode = currentCardData.getActivation_code ();
            std::string uncypheredDate = CardData::CardDataWrapper::GetUncypheredActivationCode ( verifCode );

            if ( uncypheredDate.compare ( "" ) != 0 && CardData::CardDataWrapper::checkIfCardTimeIsValid ( uncypheredDate ) )
            {
                bool condition = currentCardData.getCard_type() == CardData::Card_type::CARD_DATA_DEMOSTRATION_VOTE_CARD;

                if ( notCompletelyUsed || simulateProcess || markUsedCardData() )
                {
                    VotingExperienceController * votingExperienceController;
                    votingExperienceController = MachineOperationManager::getInstance()->getVotingExperienceController();

                    if ( votingExperienceController->init ( currentCardData ) )
                    {
                        if ( simulateVote && condition )
                        {
                            logger->Debug ( "Demo Simulate" );
                            votingExperienceController->setVotingMode ( Vote::VoteMode::VOTE_D );
                        }
                        else if ( simulateVote )
                        {
                            logger->Debug ( "Any Simulate" );
                            votingExperienceController->setVotingMode ( Vote::VoteMode::VOTE_S );
                        }
                        else
                        {
                            logger->Debug ( "Init Vote" );
                        }

                        VotingOperation::refresh();

                        if (audioVoting || currentCardData.getIsAudio())
                        {
                            if (!pollingPlaceConfiguredForAudioVoting())
                            {
                                logger->Debug ( "ERROR - Detected Audio voting card but polling place not configured for audio voting" );
                                signalAlarm.emit ( new AlarmEnum ( AlarmEnum::CONFIGURATION_DETECTEDAUDIOCARDBUTNOTCONFIGUREDFORAUDIO ) );
                            }
                            else if (!ableIsConnected())
                            {
                                logger->Debug ( "ERROR - Detected Audio voting card but able is not connected" );
                                signalAlarm.emit ( new AlarmEnum ( AlarmEnum::CONFIGURATION_DETECTEDAUDIOCARDBUTNOABLEPRESENT ) );
                            }
                            else if (!voicesArePresent())
                            {
                                logger->Debug ( "ERROR - Detected Audio voting card but voices are not present" );
                                signalAlarm.emit ( new AlarmEnum ( AlarmEnum::CONFIGURATION_DETECTEDAUDIOCARDBUTNOVOICESPRESENT ) );
                            }
                            else
                            {
                               signalStartAudioVoting.emit ( NULL );
                            }
                        }
                        else
                        {
                            signalShowLanguage.emit ( NULL );
                        }
                    }
                    else
                    {
                        signalAlarm.emit ( new AlarmEnum ( AlarmEnum::CONFIGURATION_INVALIDVOTINGDATA ) );
                    }
                }
                else
                {
                    logger->Debug ( "ERROR - Problem write data card" );
                    signalAlarm.emit ( new AlarmEnum ( AlarmEnum::CONFIGURATION_UNABLETOWRITE ) );
                }
            }
            else
            {
                logger->Debug ( "ERROR - Time of card has been expired" );
                signalAlarm.emit ( new AlarmEnum ( AlarmEnum::CONFIGURATION_CARDVALIDITYPERIODEXPIRED ) );
            }
        }
        else
        {
            clearCardData();
            signalCardUsed.emit ( NULL );
        }
    }
    catch ( ... )
    {
        signalAlarm.emit ( new AlarmEnum ( AlarmEnum::CONFIGURATION_INVALIDVOTINGDATA ) );
    }
}

bool MachineVotingWidget::pollingPlaceConfiguredForAudioVoting()
{
    AudioVotingCheckUtility audioVotingCheck = MachineOperationManager::getInstance()->getAudioVotingCheckUtility();
    return audioVotingCheck.pollingPlaceConfiguredForAudioVoting();
}

bool MachineVotingWidget::ableIsConnected()
{
    AudioVotingCheckUtility audioVotingCheck = MachineOperationManager::getInstance()->getAudioVotingCheckUtility();
    return audioVotingCheck.ableIsConnected();
}

bool MachineVotingWidget::voicesArePresent()
{
    AudioVotingCheckUtility audioVotingCheck = MachineOperationManager::getInstance()->getAudioVotingCheckUtility();
    return audioVotingCheck.voicesArePresent();
}

void MachineVotingWidget::showSelectionLanguage ( void * p )
{
    logger->Debug ( "Show selection language" );

    if (!CardPolling::getInstance()->isCardPresent() &&
        !SAESGUIConfigurationManager::getInstance()->getWaitingForCardWindowConfiguration().getShowSimulateButton()) return;

    if ( languageSelectionWidget->ShowWidget() )
    {
        stepVoting = SHOW_LANGUAGE;
        languageSelectionWidget->start();
        viewWidget ( languageSelectionWidget );
    }
    else
    {
        languageSelected();
    }
}

void MachineVotingWidget::languageSelected()
{
    logger->Debug ( "Language selected" );
    startVotingExperience();
}

void MachineVotingWidget::setAlarm ( AlarmEnum alarmType )
{
    stepVoting = WAITING;

    if ( AlarmConfigurationManager::getInstance()->getCategoryOfAlarm ( alarmType ) != AlarmCategoryEnum::CONFIGURATION_NOTALARM )
    {
        stopCheckCardPresent ();
        AlarmMessage::alarm ( alarmType );

        if ( AlarmController::getInstance()->isGoToAuthentication() )
        {
            logger->Debug ( "Set Alarm" );
            signalBack.emit();
        }
        else
        {
            startCheckCardPresent ();
            signalRefresh.emit();
        }
    }
}

void MachineVotingWidget::safeAlarm ( void * alarmType )
{
    AlarmEnum * alarm = static_cast < AlarmEnum * > ( alarmType );
    setAlarm ( *alarm );
    delete alarm;
}

std::string MachineVotingWidget::getTitleText()
{
    return N_ ( "Smartmatic.SAES.GUI.Widget.MachineVotingWidget.Title" );
}

bool MachineVotingWidget::isTitle()
{
    return false;
}

void MachineVotingWidget::cardError ( void * p )
{
    stepVoting = ERROR_VALIDATE;
    waitingForCardWidget->viewErrorCardValidate();
    mainWidget->showInfo ( false );
    waitingVotingWidget->hideButtons();
}

void MachineVotingWidget::cardErrorAID ( void * p )
{
    stepVoting = ERROR_VALIDATE;
    waitingForCardWidget->viewErrorCardAID();
    mainWidget->showInfo ( false );
    waitingVotingWidget->hideButtons();
}

void MachineVotingWidget::cardErrorPIN ( void * p )
{
    stepVoting = ERROR_VALIDATE;
    waitingForCardWidget->viewErrorCardPIN();
    mainWidget->showInfo ( false );
    waitingVotingWidget->hideButtons();
}

void MachineVotingWidget::cardErrorGeneric ( void * p )
{
    stepVoting = ERROR_VALIDATE;
    waitingForCardWidget->viewErrorCardGeneric();
    mainWidget->showInfo ( false );
    waitingVotingWidget->hideButtons();
}

bool MachineVotingWidget::checkPrinter()
{
    bool ok = false, neop_state, is_neop_blocking;
    PrinterStatusNotifier::runForcedPrintBlank();
    sleep ( 1 );
    PrinterStatusNotifier printNotifier;
    PrinterStatusNotifier::PhysicalStateEnumType printerState;
    printerState = printNotifier.getPrinterState();
    std::stringstream ss;
    ss << "Printer state is: " << ( int ) printerState;
    logger->Debug ( ss.str() );
    neop_state = ( ( printerState & PrinterStatusNotifier::PSTATE_NEAR_END_OF_PAPER ) == PrinterStatusNotifier::PSTATE_NEAR_END_OF_PAPER );
    is_neop_blocking = SAESGUIConfigurationManager::getInstance ()->getWaitingForCardWindowConfiguration ().getIsNeopBlockingCondition ();

    if ( printerState & printNotifier.getErrorCondition() )
    {
        ok = false;
        signalAlarm.emit ( new AlarmEnum ( AlarmEnum::CONFIGURATION_PRINTERNOTREADYFORVOTE ) );
    }
    else if ( is_neop_blocking )
    {
        ok = ! neop_state;

        if ( !ok )
        {
            signalAlarm.emit ( new AlarmEnum ( AlarmEnum::CONFIGURATION_PRINTERNEOPBEFOREVOTE ) );
        }
    }
    else
    {
        ok = true;
    }

    return ok;
}

bool MachineVotingWidget::markUsedCardData()
{
    bool validate = true;

    try
    {
        if ( SAESGUIConfigurationManager::getInstance()->getMarkCardAsUsed () )
        {
            CardData::Card_data temp = currentCardData;
            CardData::CardDataWrapper::markUsed ( temp );

            if ( !CardController::getInstance()->writeCardData ( temp ) )
            {
                logger->Debug ( "FATAL - check used card data" );
                validate = false;
            }
        }
    }
    catch ( ... )
    {
        logger->Debug ( "FATAL - error write card data" );
        validate = false;
    }

    return validate;
}

void MachineVotingWidget::stopAll()
{
    logger->Debug ( "Stop all" );

    std::auto_ptr<IStripLEDManager> ledManager(
              LEDStripManagerFactory::getLEDStripManager());
    ledManager->turnAllLedsOff();

    stopCheckAlarmButton();
    stopCheckBarcode();
    stopCheckCardPresent();

    stopAudioVotingExperience();

    simulateProcess = false;

    connCardPollingEvent.disconnect();
    connLanguage.disconnect();
    signalDeviceChange.disconnect();

    connAlarm.disconnect();
    connAlarmMonitor.disconnect();

    signalVote.disconnect();
    signalStatusVote.disconnect();

    clearCardData();

    stepVoting = WAITING;

    CardPolling::getInstance()->pause();
}

void MachineVotingWidget::onButtonPresed ( AlarmPolling::AlarmButtonStatus status )
{
    if ( status == AlarmPolling::PRESSED )
    {
        if ( !AlarmController::getInstance()->isAlarmPending() &&
                SAESGUIConfigurationManager::getInstance()->getWaitingForCardWindowConfiguration().getCallRestartSmartcardIfPressAlarmBoxAndNotAlarm () )
        {
            std::auto_ptr<ISmartcardReaderPowerManager> powerManager(
                SmartcardReaderPowerManagerFactory::getSmartcardReaderPowerManager());

            powerManager->callRestartSmartcard(true);
        }

        logger->Debug ( "On Button Pressed" );
        back ();
    }
    else
    {
        logger->Debug ( "On Button Released" );
    }
}

void MachineVotingWidget::back()
{
    if ( !isInterrupted )
    {
        isInterrupted = true;
        stopAll();
        blockMachine();
    }
}

void MachineVotingWidget::onSignalVote ( Smartmatic::SAES::Voting::Vote::Vote * vote )
{
    this->vote = *vote;

    stopCheckCardPresent();
    stopCheckAlarmButton();

    viewWidget ( previewWidget );
    previewWidget->preview ( *vote );
}

void MachineVotingWidget::onSignalState ( Smartmatic::SAES::Operation::Voting::VotePersistence::PersistenceStatusVote status )
{
    switch ( status )
    {
        case Smartmatic::SAES::Operation::Voting::VotePersistence::PERSISTENCE_VOTE_FAIL:
        case Smartmatic::SAES::Operation::Voting::VotePersistence::PERSISTENCE_VOTE_DUPLICATE:
            viewWidget ( previewWidget );
            previewWidget->error();
            break;

        default:
        case Smartmatic::SAES::Operation::Voting::VotePersistence::PERSISTENCE_VOTE_SUCCESS:
            break;
    }

    previewWidget->refresh();
}

bool MachineVotingWidget::detectAlarm ( int time )
{
    if ( stepVoting != WAITING )
    {
        return true;
    }

    if ( AlarmController::getInstance()->isGoToAuthentication() )
    {
        logger->Debug ( "Go to Authentication" );
        signalBack.emit();
    }

    return true;
}

void MachineVotingWidget::showCardUsed ( void * p )
{
    waitingForCardWidget->viewErrorCardUsed();
    mainWidget->showInfo ( false );
    waitingVotingWidget->hideButtons();
    setAlarm ( AlarmEnum::CONFIGURATION_CARDUSED );
}

void MachineVotingWidget::onSignalCardChange ( CardPolling::CardPollingStatus status )
{
    switch ( status )
    {
        case CardPolling::INSERTED:
            cardInsert();
            break;

        case CardPolling::REMOVED:
            cardRemove();
            break;

        default:
        case CardPolling::ERROR_SCR:
            cardReaderUnresponsive();
            break;
    }
}

void MachineVotingWidget::deviceChange ( Smartmatic::SAES::Common::Operation::Monitor::DeviceTypeMonitor * deviceTypeMonitor )
{
    if ( deviceTypeMonitor->getType() == Smartmatic::SAES::Common::Configuration::DeviceType::CONFIGURATION_SMARTCARD )
    {
        if ( deviceTypeMonitor->getNumberOfDevices() != 0 )
        {
            logger->Debug ( "Smartcard connected" );
            switch (stepVoting) {
                case WAITING:
                    logger->Debug("SCR RECONNECTING WAITING");
                    break;
                case VALIDATE:
                    logger->Debug("SCR RECONNECTING VALIDATE");
                    break;
                case ERROR_VALIDATE:
                    logger->Debug("SCR RECONNECTING ERROR_VALIDATE");
                    break;
                case SHOW_LANGUAGE:
                    logger->Debug("SCR RECONNECTING SHOW_LANGUAGE");
                    break;
                case VOTING:
                    logger->Debug("SCR RECONNECTING VOTING");
                    break;
                case AUDIO_VOTING_LANGUAGE_SELECTION:
                    logger->Debug("SCR RECONNECTING AUDIO_VOTING_LANGUAGE_SELECTION");
                    break;
                case AUDIO_VOTING:
                    logger->Debug("SCR RECONNECTING AUDIO_VOTING");
                    break;
                case NONE:
                default:
                    break;
            }
            if(CardPolling::getInstance()->isCardPresent() &&
                    (stepVoting == VALIDATE || stepVoting == WAITING || stepVoting == ERROR_VALIDATE
                            || stepVoting == SHOW_LANGUAGE || stepVoting == AUDIO_VOTING_LANGUAGE_SELECTION))
            {
                clearCardData();
                logger->Debug ( "Forcing card error event" );
                signalCardErrorGeneric.emit ( NULL );
            }
        }
        else
        {
            logger->Debug ( "Smartcard disconnected" );
            setAlarm ( AlarmEnum::CONFIGURATION_CARDREADERUNRESPONSIVE );
        }
    }
    else if ( deviceTypeMonitor->getType() == Smartmatic::SAES::Common::Configuration::DeviceType::CONFIGURATION_BARCODE )
    {
        if ( deviceTypeMonitor->getNumberOfDevices() == 0 )
        {
            logger->Debug ( "BCR disconnected" );
            disconnectBarcode();
        }
        else
        {
            logger->Debug ( "BCR connected" );
            connectBarcode();
        }
    }
}

void MachineVotingWidget::clearCardData()
{
    currentCardDataCode.clear();
    currentBallotIDCardData.clear();
    currentSessionIDCardData.clear();
}
