/**
 * @file MemoryStateWidget.cxx
 * @brief Body class memory state widget
 * @date 29/03/2012
 * @author Yadickson Soto
 */

#include "MemoryStateWidget.hxx"
#include <Log/ISMTTLog.h>
#include <Log/SMTTLogManager.h>
#include <Configuration/MachineModelEnum.hxx>
#include "Configuration/s-a-e-s-g-u-i-conf-manager.h"
#include <System/Runtime/PowerDown.hxx>
#include <Environment/Configurations/resource-path-provider.h>
#include <Environment/Configurations/base-language-controller.h>
#include <Operation/CheckDataManager.hxx>
#include <Operation/MachineOperationManager.hxx>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#include <libintl.h>
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

#define N_(String) (String) ///<macro for gettext not translate

#define BUTTON_MAX_WIDTH 1250
#define BUTTON_MAX_HEIGHT 85

using namespace Smartmatic::SAES::GUI::Widgets;
using namespace Smartmatic::Configuration;
using namespace Smartmatic::Log;
using namespace Smartmatic::GUI::Widgets;
using namespace Smartmatic::SAES::GUI::Configuration;
using namespace Smartmatic::System::Runtime;
using namespace Smartmatic::Environment;
using namespace Smartmatic::GUI::Configuration;
using namespace Smartmatic::SAES::Operation;

ISMTTLog * MemoryStateWidget::logger = SMTTLogManager::GetLogger ( "Smartmatic.SAES.GUI.Widgets.MemoryStateWidget", GETTEXT_PACKAGE );

MemoryStateWidget::MemoryStateWidget ( ProgressWidget * progress )
    : StepWidget(), progress ( progress )
{
    mainEventBox = NULL;
    memoryVBox = NULL;
    progressEventBox = NULL;
    shutdownHBox = NULL;
    shutdownButton = NULL;
    cancelSafeEjectButton = NULL;

    initialize();
    setWidget ( mainEventBox );
}

MemoryStateWidget::~MemoryStateWidget()
{
    dispose();
}

void MemoryStateWidget::initialize()
{
    Glib::RefPtr<Gtk::Builder> builder;
    std::string gladeFile;
    gladeFile = ResourcePathProvider::getInstance()->GetResourcePath ( PACKAGE_NAME, "ui/MemoryStateWidget.glade" );

    try
    {
        builder = Gtk::Builder::create_from_file ( gladeFile );
    }
    catch ( const Glib::FileError & ex )
    {
        logger->Debug ( "FATAL - MemoryStateWidget Initialize" );
    }

    builder->get_widget_derived ( "memory-eventbox", mainEventBox );
    builder->get_widget_derived ( "memory-vbox", memoryVBox );
    builder->get_widget_derived ( "progress-eventbox", progressEventBox );
    builder->get_widget_derived ( "shutdown-hbox", shutdownHBox );

    builder->get_widget_derived ( "cancel-safe-eject-button", cancelSafeEjectButton );
    builder->get_widget_derived ( "shutdown-button", shutdownButton );

    shutdownHBox->set_no_show_all();
    shutdownButton->set_no_show_all();
    cancelSafeEjectButton->set_no_show_all();

    progressEventBox->add ( *progress );
    progressEventBox->show_all_children();

    int fontSize = SAESGUIConfigurationManager::getInstance ()->getMemoryStateWidgetConfiguration().getButtonConf().getFontSize();
    int widthB = SAESGUIConfigurationManager::getInstance ()->getMemoryStateWidgetConfiguration().getButtonConf().getButtonWidth();
    int heightB = SAESGUIConfigurationManager::getInstance ()->getMemoryStateWidgetConfiguration().getButtonConf().getButtonHeight();

    if ( SAESGUIConfigurationManager::getInstance()->getCurrentMachineModelConfiguration().getId() == EnumToString( PM_VIU805 ) )
    {
        widthB = BUTTON_MAX_WIDTH;
        heightB = BUTTON_MAX_HEIGHT;
    }

    LabelConfiguration textConf = SAESGUIConfigurationManager::getInstance()->getMemoryStateWidgetTextConfiguration();
    progress->setInfoTextConfig ( textConf );
    shutdownHBox->hide();
    shutdownButton->ModifyFontSize ( fontSize );
    shutdownButton->SetSound ( " " );
    shutdownButton->set_size_request ( widthB, heightB );

    cancelSafeEjectButton->ModifyFontSize ( fontSize );
    cancelSafeEjectButton->SetSound ( " " );
    cancelSafeEjectButton->set_size_request ( widthB, heightB );
    cancelSafeEjectButton->signal_clicked().connect ( sigc::mem_fun ( *this , &MemoryStateWidget::cancelSafeEject ) );

    setAccessibleName ( "MemoryStateWidget" );
    mainEventBox->setAccessibleName ( "MemoryStateMainContainer" );
    memoryVBox->setAccessibleName ( "MemoryStateVerticalContainer" );
    progressEventBox->setAccessibleName ( "MemoryStateProgressContainer" );
    shutdownHBox->setAccessibleName ( "MemoryStateButtonContainer" );
    cancelSafeEjectButton->setAccessibleName ( "CancelSafeEject" );
    shutdownButton->setAccessibleName ( "Shutdown" );
}

void MemoryStateWidget::dispose()
{
    this->remove();

    if ( mainEventBox )
    {
        delete mainEventBox;
        mainEventBox = NULL;
    }

    if ( memoryVBox )
    {
        delete memoryVBox;
        memoryVBox = NULL;
    }

    if ( progressEventBox )
    {
        delete progressEventBox;
        progressEventBox = NULL;
    }

    if ( shutdownHBox )
    {
        delete shutdownHBox;
        shutdownHBox = NULL;
    }

    if ( shutdownButton )
    {
        delete shutdownButton;
        shutdownButton = NULL;
    }

    if ( cancelSafeEjectButton )
    {
        delete cancelSafeEjectButton;
        cancelSafeEjectButton = NULL;
    }

    if ( progress )
    {
        delete progress;
        progress = NULL;
    }
}

void MemoryStateWidget::refresh()
{
}

std::string MemoryStateWidget::getTitleText()
{
    std::string tr = _ ( StepWidget::getTitleText().c_str() );
    return ( BaseLanguageController::isSetLanguage() ? tr : " " );
}

void MemoryStateWidget::configureWidget ( bool showHideButton, bool buttonClose, bool showSafeEjectButton )
{
    int button_enabling_delay = SAESGUIConfigurationManager::getInstance ()->getMemoryStateWidgetConfiguration().getButtonDelay ();
    progress->setImage ( image );
    std::string lblTex = _ ( labelKey.c_str() );

    if ( BaseLanguageController::isSetLanguage() )
    {
        progress->setInfoText ( lblTex );
    }
    else
    {
        progress->setInfoText ( " " );
    }

    if ( showHideButton )
    {
        connectButton.disconnect();
        shutdownButton->set_sensitive ( false );

        if ( buttonClose )
        {
            std::string textButton = N_ ( "Smartmatic.SAES.GUI.Windows.ErrorWindow.ButtonShutDown" );
            std::string text = _ ( textButton.c_str() );

            if ( BaseLanguageController::isSetLanguage() )
            {
                shutdownButton->set_label ( text );
            }
            else
            {
                shutdownButton->set_label ( " " );
                shutdownButton->WidgetsPosition ( 0, 5, 0 );
                shutdownButton->SetImage ( ResourcePathProvider::getInstance()->GetResourcePath ( PACKAGE_NAME, "ui/button-shut-down.png" ) );
            }

            connectButton = shutdownButton->signal_clicked().connect ( sigc::mem_fun ( this, &MemoryStateWidget::shutDown ) );
        }
        else
        {
            shutdownButton->set_label ( _ ( "Smartmatic.SAES.GUI.Windows.ErrorWindow.AllowWindowButton" ) );
            connectButton = shutdownButton->signal_clicked().connect ( sigc::mem_fun ( this, &MemoryStateWidget::cancelSafeEject ) );
        }

        Glib::signal_timeout().connect_once ( sigc::mem_fun ( *this , &MemoryStateWidget::enabledButton ), button_enabling_delay );

        cancelSafeEjectButton->hide();
        shutdownHBox->show();
        shutdownButton->show();
    }
    else if ( showSafeEjectButton )
    {
        cancelSafeEjectButton->set_label ( _ ( "Cancel safe eject" ) );
        shutdownHBox->show();
        shutdownButton->hide();
        cancelSafeEjectButton->show();
    }
    else
    {
        shutdownHBox->hide();
    }
}

void MemoryStateWidget::enabledButton()
{
    shutdownButton->set_sensitive ( true );
}

void MemoryStateWidget::shutDown()
{
    PowerDown::shutdown ( );
}

void MemoryStateWidget::showSynchronize()
{
    logger->Audit ( N_ ( "Start safe eject synchronization" ) );

    logger->Debug ( "Signal Start Synchronize" );
    setTitleText ( N_ ( "Synchronization Title" ) );
    labelKey = N_ ( "Do not remove USBs while synchronization" );
    image = "ui/check-flash-structure.gif";

    configureWidget ( false, false );
}

void MemoryStateWidget::showSafeEject()
{
    logger->Audit ( N_ ( "You can now safely remove one USB" ) );

    logger->Debug ( "Signal One Memory Allow" );
    setTitleText ( N_ ( "Safe eject Title" ) );
    labelKey = N_ ( "You can now safely remove one USB or press cancel to go back" );
    image = "ui/safe-eject-success.gif";

    configureWidget ( false, false, true );
}

void MemoryStateWidget::showOneMemory()
{
    setTitleText ( N_ ( "Smartmatic.SAES.GUI.Windows.ErrorWindow.SuspendWindowTitle" ) );

    if ( MachineOperationManager::getInstance()->isCloseElection() || MachineOperationManager::getInstance()->isClosingElection() )
        labelKey = N_ ( "Smartmatic.SAES.GUI.Windows.ErrorWindow.SuspendWindowInstructionClosed" );
    else
        labelKey = N_ ( "Smartmatic.SAES.GUI.Windows.ErrorWindow.SuspendWindowInstruction" );

    image = "ui/check-flash-presence.gif";
    logger->Debug ( "Signal One Memory Suspend" );

    configureWidget ( false, false );

    logger->Audit ( N_ ( "Remove one USB not safely" ) );
}

void MemoryStateWidget::showAllMemoryRemoved()
{
    setTitleText ( N_ ( "Smartmatic.SAES.GUI.Windows.ErrorWindow.PendriveExtracted" ) );
    labelKey = " ";
    image = "ui/invalid-memory-inserted.gif";

    configureWidget ( true, true );
}

void MemoryStateWidget::showVerifyTwoMemory()
{
    setTitleText ( N_ ( "Smartmatic.SAES.GUI.Windows.ErrorWindow.VerifyDataWindowTitle" ) );
    labelKey = N_ ( "Smartmatic.SAES.GUI.Windows.ErrorWindow.VerifyDataWindowInstruction" );
    image = "ui/check-flash-structure.gif";
    logger->Debug ( "Signal Verify Memory" );

    configureWidget ( false, false );
}

void MemoryStateWidget::showMemoryCheckError()
{
    setTitleText ( N_ ( "Smartmatic.SAES.GUI.Windows.ErrorWindow.ErrorCheckData" ) );

    if ( MachineOperationManager::getInstance()->isCloseElection() || MachineOperationManager::getInstance()->isClosingElection() )
    {
      labelKey = N_ ( "Smartmatic.SAES.GUI.Windows.ErrorWindow.ErrorCheckDataInstructionClosed" );
      image = "ui/synchronization-error.gif";
    }
    else
    {
      labelKey = N_ ( "Smartmatic.SAES.GUI.Windows.ErrorWindow.ErrorCheckDataInstruction" );
      image = "ui/check-flash-structure-failed.gif";
    }

    configureWidget ( true, true );
}

void MemoryStateWidget::showMemoryCheckErrorToRecount()
{
    setTitleText ( N_ ( "Smartmatic.SAES.GUI.Windows.ErrorWindow.ErrorCheckData" ) );

    if ( MachineOperationManager::getInstance()->isCloseElection() || MachineOperationManager::getInstance()->isClosingElection() )
        labelKey = N_ ( "Smartmatic.SAES.GUI.Windows.ErrorWindow.ErrorCheckDataInstructionClosed" );
    else
        labelKey = N_ ( "Smartmatic.SAES.GUI.Windows.ErrorWindow.ErrorCheckDataWithVotesInstruction" );

    image = "ui/synchronization-error.gif";
    configureWidget(true, true);
}

void MemoryStateWidget::showMemoryError()
{
    setTitleText ( N_ ( "Smartmatic.SAES.GUI.Windows.ErrorWindow.PendriveExtracted" ) );
    labelKey = " ";
    image = "ui/invalid-memory-inserted.gif";
    configureWidget ( true, true );
}

void MemoryStateWidget::showReturnMemory()
{
    setTitleText ( N_ ( "Smartmatic.SAES.GUI.Windows.ErrorWindow.PendriveExtracted" ) );

    if ( MachineOperationManager::getInstance()->isCloseElection() || MachineOperationManager::getInstance()->isClosingElection() )
        labelKey = N_ ( "Smartmatic.SAES.GUI.Windows.ErrorWindow.SuspendWindowInstructionClosed" );
    else
        labelKey = ( N_ ("Smartmatic.SAES.GUI.Windows.ErrorWindow.ReturnMemory") );

    image = "ui/check-flash-presence.gif";
    configureWidget ( false, false );
}

void MemoryStateWidget::showReturnBothMemories()
{
    setTitleText ( N_ ( "Smartmatic.SAES.GUI.Windows.ErrorWindow.ReturnBothMemoriesTitle" ) );

    if ( MachineOperationManager::getInstance()->isCloseElection() || MachineOperationManager::getInstance()->isClosingElection() )
        labelKey = ( N_ ("Smartmatic.SAES.GUI.Windows.ErrorWindow.ReturnBothMemoriesClosed") );
    else
        labelKey = ( N_ ("Smartmatic.SAES.GUI.Windows.ErrorWindow.ReturnBothMemories") );

    image = "ui/two_usb_warning.gif";
    configureWidget ( false, false );
}

void MemoryStateWidget::cancelSafeEject()
{
    CheckDataManager::getInstance()->clearSynchronize();

    logger->Audit ( N_ ( "Cancel safe eject synchronization" ) );

    getCancelSafeEject().emit();

    emit();
}

sigc::signal<void> MemoryStateWidget::getCancelSafeEject()
{
    return signalCancel;
}
