/*
 * RecountWidget.h
 *
 *  Created on: Apr 29, 2013
 *      Author: Georgi Demirchev
 */

#ifndef RECOUNTWIDGET_H_
#define RECOUNTWIDGET_H_

#include <Widgets/StepWidget.hxx>
#include "Recount/RecountStatPage.h"
#include "Recount/RecountInfoVoteWidget.h"
#include "Recount/ConfigBridge.h"
#include <string>
#include <map>
#include "Recount/RecountPersistConfirm.h"
#include <Widgets/notebook-container.h>

namespace Smartmatic
{
    namespace SAES
    {
        namespace GUI
        {
            namespace Widgets
            {

                class RecountWidget: public Smartmatic::GUI::Widgets::StepWidget
                {
                    public:

                        /**
                         * @brief  Definition of pause signal type.
                         * @author manuel.arguelles
                         * @date   Apr  2, 2018
                         */
                        typedef sigc::signal<void> PauseSignal;

                        /**
                         * @brief  Definition of resume signal type.
                         * @author manuel.arguelles
                         * @date   Apr  2, 2018
                         */
                        typedef sigc::signal<void> ResumeSignal;

                    public:

                        /**
                         * @brief Constructor initialized by voter types
                         * @param voterType A list of the voters type.
                         */
                        explicit RecountWidget(const std::list<std::string>& voterTypes);

                        /**
                         * @brief Destructor
                         */
                        virtual ~RecountWidget();

                        /**
                         * @brief  Refresh method.
                         */
                        virtual void refresh();

                        /**
                         * @brief Sets the data per given vote.
                         * @param num         Sequential session number.
                         * @param code        The vote code
                         * @param sessionID   The session ID.
                         * @param ballotID    The ballot ID.
                         * @param votingGroup The voting group.
                         */
                        void SetVote(int num,
                                     const std::string& code,
                                     const std::string& sessionID,
                                     const std::string& ballotID,
                                     const std::string& votingGroup);

                        /**
                         * @brief On vote failed
                         */
                        void SetVoteFailed();

                        /**
                         * @brief On vote Success
                         */
                        void SetVoteSuccess();

                        /*
                         * @brief On update with newly scanned votes
                         * @param updates            List of pairs<field name, session number>.
                         * @param updateSessionNames List of sessions name, effectively session id.
                         */
                        void Update (const std::list<std::pair<int, int> >& updates,
                                     const std::list<std::string>& updateSessionNames);

                        /**
                         * @brief Get votes marked for deletion
                         * @param deleteQueue List of votes with coordinates voter_type, session id.
                         */
                        void GetForPersistence (std::list<std::pair<int, int> >& deleteQueue);

                        /**
                         * @brief Forces the client to confirm
                         */
                        void showConfirmation();

                        /**
                         * @brief  Gets the pause signal. Called when the recount must be paused.
                         * @author manuel.arguelles
                         * @date   Apr  2, 2018
                         * @return The pause signal.
                         */
                        PauseSignal pauseSignal();

                        /**
                         * @brief  Gest the resume signal. Called when the recount must be resumed.
                         * @author manuel.arguelles
                         * @date   Apr  2, 2018
                         * @return The resume signal.
                         */
                        ResumeSignal resumeSignal();

                    protected:

                        /**
                         * @brief On confirmation canceled restores the view
                         */
                        void before();

                        /*
                         * @brief Emits End signal
                         */
                        void exit ();

                    private:

                        Recount::ConfigBridge*                       m_configBridge;
                        Recount::RecountInfoVoteWidget*              m_scanPage;
                        Recount::RecountStatPage*                    m_statPage;
                        Smartmatic::GUI::Widgets::NotebookContainer* notebookContainer;
                        PauseSignal                                  m_pauseSignal;
                        ResumeSignal                                 m_resumeSignal;
                };


            } /* namespace Widgets */
        } /* namespace GUI */
    } /* namespace SAES */
} /* namespace Smartmatic */
#endif /* RECOUNTWIDGET_H_ */
