/**
 * @file	ReportLogWidget.h
 * @brief	Declares the report log widget class.
 * @author	Youssef.tourkmani
 * @date	01/11/2012
 */


#ifndef _REPORTLOGWIDGET_H_
#define _REPORTLOGWIDGET_H_

#include <gtkmm.h>
#include <Log/LogReader.h>
#include <Gui/Widgets/LogPagingTreeViewWidget.h>
#include <Printing/AuditLogPrintDocument.h>
#include <Printing/ErrorLogPrintDocument.h>
#include <Gui/Widgets/GenericPrintingWidget.hxx>
#include <Widgets/StepWidget.hxx>

namespace Smartmatic
{
    namespace Log
    {
        class ISMTTLog;
    }
    namespace SAES
    {
        namespace GUI
        {
            namespace Widgets
            {
                /**
                 * @class	ReportLogWidget
                 * @brief	Report log widget.
                 * @author	Cesar.bonilla
                 * @date	4/29/2011
                 */

                class ReportLogWidget : public Smartmatic::GUI::Widgets::StepWidget
                {
                    public:

                        /**
                         * @enum	LogType
                         * @brief	Values that represent LogType.
                         */
                        enum LogType
                        {
                            LogType_Audit,
                            LogType_Error
                        };

                        sigc::signal<void, LogType> signalPrint_detected; ///< The signal print detected
                        /**
                         * @fn	ReportLogWidget::ReportLogWidget ( Smartmatic::SAES::Log::LogReader * logReaderAudit,
                         * 			Smartmatic::SAES::Log::LogReader * logReaderError,
                         * 			GenericPrintingWidget * genericPrintingWidget );
                         * @brief	Constructor.
                         * @author	Yadickson Soto
                         * @date	31/11/2012
                         * @param	logReaderAudit	If non-null, the log reader audit.
                         * @param	logReaderError	If non-null, the log reader error.
                         * @param 	genericPrintingWidget generic printing widget
                         */

                        ReportLogWidget ( Smartmatic::SAES::Log::LogReader * logReaderAudit,
                                          Smartmatic::SAES::Log::LogReader * logReaderError,
                                          GenericPrintingWidget * genericPrintingWidget );

                        /**
                         * @fn	ReportLogWidget::~ReportLogWidget();
                         * @brief	Finaliser.
                         * @author	Cesar.bonilla
                         * @date	4/29/2011
                         */

                        ~ReportLogWidget();

                        /**
                         * @typedef	sigc::signal<void> signalPrint
                         * @brief	Defines an alias representing the signal print.
                         */
                        typedef sigc::signal<void> signalPrint;

                        /**
                         * @fn	signalPrint ReportLogWidget::on_print_begin();
                         * @brief	Executes the print begin action.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         * @return	.
                         */
                        signalPrint on_print_begin();
                        /**
                         * @fn	signalPrint ReportLogWidget::on_print_end();
                         * @brief	Executes the print end action.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         * @return	.
                         */
                        signalPrint on_print_end();

                    protected:

                        signalPrint m_signalPrint_begin;	///< The signal print begin
                        signalPrint m_signalPrint_end;	///< The signal print end

                        /**
                         * @fn	void ReportLogWidget::printAuditLogger();
                         * @brief	Print audit logger.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         */
                        void printAuditLogger();
                        /**
                         * @fn	void ReportLogWidget::printErrorLogger();
                         * @brief	Print error logger.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         */
                        void printErrorLogger();

                    private:

                        /**
                         * @fn	void ReportLogWidget::LoadGUI();
                         * @brief	Loads the graphical user interface.
                         * @author	Cesar.bonilla
                         * @date	4/29/2011
                         */

                        void LoadGUI();
                        /**
                         * @fn	void ReportLogWidget::on_buttonPrint_clicked();
                         * @brief	Handles button print clicked signals.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         */
                        void on_buttonPrint_clicked();

                        Gtk::Table * m_TopWidgetTableLayout; ///< The top widget table layout
                        Gtk::Frame * m_FrameAudit;  ///< The frame audit
                        Gtk::Frame * m_FrameErrors; ///< The frame errors
                        Gtk::Notebook	* m_NotebookLogs; ///< The notebook logs
                        LogPagingTreeViewWidget * logPagingTreeViewAudit;	///< The log paging tree view audit
                        LogPagingTreeViewWidget * logPagingTreeViewError;	///< The log paging tree view error
                        sigc::connection connPrintAudit; ///< connection print audit
                        sigc::connection connPrintError;	///< connection print error
                        Smartmatic::SAES::Log::LogReader * logReaderAudit;  ///< The log reader audit
                        Smartmatic::SAES::Log::LogReader * logReaderError;  ///< The log reader error

                        GenericPrintingWidget * genericPrintingWidget; ///< generic printing widget
                        Smartmatic::SAES::Printing::AuditLogPrintDocument * m_AuditLogPrintReport;	///< The audit log print report
                        Smartmatic::SAES::Printing::ErrorLogPrintDocument * m_ErrorLogPrintReport;	///< The error log print report

                        std::string messagePrint; ///< The message print
                        bool statusPrint; ///< true to status print

                        /**
                         * @fn	void ReportLogWidget::showInitialImage();
                         * @brief	Shows the initial image.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         */
                        void showInitialImage();
                        /**
                         * @fn	void ReportLogWidget::endPrint();
                         * @brief	Ends a print.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         */
                        void endPrint();

                        static Smartmatic::Log::ISMTTLog * logger; /**<logger*/ ///< The logger
                };
            }
        }
    }
}

#endif // _REPORTLOGWIDGET_H_
