/**
 * @file ResultWidget.hxx
 * @brief Body class result widget
 * @date 18/07/2012
 * @author Yadickson Soto
 */

#ifndef RESULTWIDGET_HXX_
#define RESULTWIDGET_HXX_

#include <gtkmm.h>
#include <Configuration/label-configuration.h>

namespace Smartmatic
{
    namespace Log
    {
        class ISMTTLog;
    }

    namespace SAES
    {
        namespace GUI
        {
            namespace Widgets
            {
                /**
                 * @class	ResultWidget
                 * @brief	Result widget.
                 * @author	Youssef.tourkmani
                 * @date	01/11/2012
                 */
                class ResultWidget : public Gtk::EventBox
                {
                    public:
                        /**
                         * @fn	ResultWidget::ResultWidget( std::string code = "" );
                         * @brief	Default constructor.
                         * @author	Yadickson Soto
                         * @date	01/11/2012
                         * @param code code
                         */
                        explicit ResultWidget ( std::string code = "" );
                        /**
                         * @fn	virtual ResultWidget::~ResultWidget();
                         * @brief	Destructor.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         */
                        virtual ~ResultWidget();

                        /**
                         * @fn	void ResultWidget::setText ( std::string text, bool center = true );
                         * @brief	Sets a text.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         * @param	text		The text.
                         * @param	center	(optional) the center.
                         */
                        void setText ( std::string text, bool center = true );
                        /**
                         * @fn	void ResultWidget::setStatus ( bool status );
                         * @brief	Sets the status.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         * @param	status	true to status.
                         */
                        void setStatus ( bool status );

                        /**
                         * @fn	std::string ResultWidget::getText();
                         * @brief	Gets the text.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         * @return	The text.
                         */
                        std::string getText();
                        /**
                         * @fn	bool ResultWidget::getStatus();
                         * @brief	Gets the status.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         * @return	true if it succeeds, false if it fails.
                         */
                        bool getStatus();

                        /**
                         * @struct	Result
                         * @brief	Result.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         */
                        struct Result
                        {
                            Result()
                            {
                                status = false;
                            }
                            bool status;	///< true to status
                            std::string message;	///< The message
                        };

                        /**
                         * @fn	void ResultWidget::addResult ( const std::string & text, bool status );
                         * @brief	Adds a result.
                         * @author	Yadickson Soto
                         * @date	07/11/2012
                         * @param   text visual text
                         * @param status status text
                         */
                        void addResult ( const std::string & text, bool status );

                        /**
                         * @fn	std::list <Result> & ResultWidget::getResult();
                         * @brief	Gets the result.
                         * @author	Yadickson Soto
                         * @date	01/11/2012
                         * @return	The result.
                         */
                        std::list <Result> & getResult();

                        /**
                         * @fn	std::string getCode();
                         * @brief	Gets the code.
                         * @author	Yadickson Soto
                         * @date	01/11/2012
                         * @return code
                         */
                        std::string getCode();

                        /**
                         * @fn	sigc::signal<void, ResultWidget * > ResultWidget::getSignalClick();
                         * @brief	Method getter signal click.
                         * @author	Yadickson Soto
                         * @date	19/05/2012
                         * @return	signal click.
                         */
                        sigc::signal<void, ResultWidget * > getSignalClick();

                        /**
                         * @fn	void ResultWidget::setLabelConfig ( Smartmatic::GUI::Configuration::LabelConfiguration & labelConf );
                         * @brief	Sets label configuration.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         * @param [in,out]	labelConf	The label conf.
                         */
                        void setLabelConfig ( Smartmatic::GUI::Configuration::LabelConfiguration & labelConf );

                    protected:
                        /**
                         * @fn	void ResultWidget::initialize();
                         * @brief	Initializes this object.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         */
                        void initialize();
                        /**
                         * @fn	void ResultWidget::dispose();
                         * @brief	Performs application-defined tasks associated with freeing, releasing, or resetting
                         * 				unmanaged resources.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         */
                        void dispose();

                        /**
                         * @brief Method getter signal click
                         * @param event event click
                         * @return true is success
                         * @date 19/05/2012
                         * @author Yadickson Soto
                         */
                        bool clicked ( GdkEventButton * event );

                    private:

                        std::string code; ///< code
                        Gtk::Frame * mainFrame; ///< The main frame
                        Gtk::Frame * labelFrame; ///< The label frame
                        Gtk::Frame * imageFrame; ///< The image frame
                        Gtk::HBox * hbox; ///< hbox
                        Gtk::Label * label; ///< The label
                        Gtk::Image * image; ///< The image
                        bool status; ///< global status
                        std::list <Result> list; ///< list message
                        sigc::signal<void, ResultWidget * > signal; ///< signal clicked
                        static Smartmatic::Log::ISMTTLog * logger; /**<logger*/
                };
            }
        }
    }
}

#endif /* RESULTWIDGET_HXX_ */
