/**
 * @file	CustomAttributeSelectionBoxWidget.cxx
 * @brief	Body custom attribute selection box widget class.
 * @author	Yadickson Soto
 * @date	13/11/2012
 */

#include "CustomAttributeSelectionBoxWidget.hxx"

#include "Configuration/s-a-e-s-g-u-i-conf-manager.h"
#include "Configuration/voting-experience-configuration-schema.hxx"
#include "FactoryVotingWidget.hxx"
#include "PartyPreview.hxx"
#include "Resources/election-string-resources-manager.h"

#include <Operation/Voting/VotingOperation.hxx>
#include <System/Utils/StringUtils.hxx>
#include <Operation/Voting/VotingExperienceController.hxx>
#include <Voting/Election/election-schema.hxx>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#include <libintl.h>
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

#define N_(String) (String) ///<macro for gettext not translate

using namespace Smartmatic::GUI::Configuration;
using namespace Smartmatic::GUI::Widgets;
using namespace Smartmatic::SAES::Configuration;
using namespace Smartmatic::SAES::GUI::Configuration;
using namespace Smartmatic::SAES::GUI::Operation::Voting;
using namespace Smartmatic::SAES::GUI::Widgets::Voting;
using namespace Smartmatic::SAES::Operation;
using namespace Smartmatic::SAES::Operation::Voting;
using namespace Smartmatic::SAES::Resources;
using namespace Smartmatic::SAES::Voting::VotingDevice;
using namespace Smartmatic::System::Utils;

static const int PREVIEW_SPACING = 20;
static const int PREVIEW_WIDTH   = 935;
static const int PREVIEW_HEIGHT  = 520;

/**
 * @brief  Adjust the widget configuration for the party preview.
 * @author manuel.arguelles
 * @date   sep 18, 2018
 * @param  configuration The initial widget configuration.
 * @return A new configuration with the settings for the party preview.
 */
static SelectionWidgetConfiguration adjustPreviewConfiguration(const SelectionWidgetConfiguration& configuration)
{
    SelectionWidgetConfiguration newConfiguration = configuration;

    newConfiguration.setBoxSpacing(PREVIEW_SPACING);
    newConfiguration.setBoxWidth(PREVIEW_WIDTH);
    newConfiguration.setBoxHeight(PREVIEW_HEIGHT);

    return newConfiguration;
}

CustomAttributeSelectionBoxWidget::CustomAttributeSelectionBoxWidget ( Custom_attribute_definition & customAttr, bool showConfirmation, bool sortAlphabetically )
    : ICustomAttributeSelectionWidget ( customAttr, showConfirmation, sortAlphabetically )
{
    currentBox = NULL;
    headerLabel = NULL;
    bottomFrame = NULL;
    externalBox = NULL;
    tableWidget = NULL;
    bottomButtonWidget = NULL;
    flagForce = false;
    partyPreviewFont = -1;

    initialize();

    setWidget ( externalBox );
    setAccessibleName ( "CustomAttributeSelectionBox" );
}

CustomAttributeSelectionBoxWidget::~CustomAttributeSelectionBoxWidget()
{
    remove();
    dispose();
}

void CustomAttributeSelectionBoxWidget::ForceSelectedOption ( unsigned int index )
{
    flagForce = true;
    tableWidget->forceClick ( index );
    bottomButtonWidget->showButton ( true, true, false, false );
}

void CustomAttributeSelectionBoxWidget::initializeTableWidget(const bool enableClick)
{
    tableWidget = manage(new TableBoxSelectionWidget(false, false, true));

    if (enableClick)
    {
    	tableWidget->getSignalClick().connect(sigc::mem_fun(this, &CustomAttributeSelectionBoxWidget::onBoxClicked));
    }
}

void CustomAttributeSelectionBoxWidget::initializeBottomButtonWidget(const bool enableClick)
{
    bottomButtonWidget = FactoryVotingWidget::createCustomAttributeSelectionButtonWidget();
    bottomButtonWidget->update();

    if (enableClick)
    {
        bottomButtonWidget->getSignalAcceptButtonClicked().connect(
            sigc::mem_fun(this, &CustomAttributeSelectionBoxWidget::onAcceptClicked));
        bottomButtonWidget->getSignalClearButtonClicked().connect(
            sigc::mem_fun(this, &CustomAttributeSelectionBoxWidget::onClearClicked));
    }
}

void CustomAttributeSelectionBoxWidget::initializeHeaderLabel(const size_t headerHeight)
{
    headerLabel = new SmartmaticBasicLabel();
    headerLabel->show();
    headerLabel->set_size_request ( -1, headerHeight );

    SelectionWidgetConfiguration::HeaderLabelConfigurationType headerLabelConfiguration =
        SAESGUIConfigurationManager::getInstance()->getCustomAttributeSelectionWidgetConfiguration()
        .getHeaderLabelConfiguration();

    LabelConfiguration labelConfiguration =
        SAESGUIConfigurationManager::ConvertToLabelConfiguration(headerLabelConfiguration);
    LabelConfiguration::SetLabelConfiguration(headerLabel, labelConfiguration);
}

void CustomAttributeSelectionBoxWidget::initializeBottomFrame(const size_t bottomHeight)
{
    bottomFrame = new SmartmaticBasicFrame();
    bottomFrame->set_shadow_type(Gtk::SHADOW_NONE);
    bottomFrame->show();
    bottomFrame->add(*bottomButtonWidget);
    bottomFrame->set_size_request(-1, bottomHeight);
}

void CustomAttributeSelectionBoxWidget::initializeExternalBox(const size_t headerHeight)
{
    externalBox = new SmartmaticBasicVBox(false , 0);

    if (headerHeight > 0)
    {
        Gtk::HSeparator * separator = manage(new Gtk::HSeparator());
        externalBox->pack_start(*headerLabel, false, false);
        externalBox->pack_start(*separator, false, false);
    }

    externalBox->pack_start(*tableWidget);
    externalBox->pack_end(*bottomFrame, false, false, 5);
}

PartyPreview* CustomAttributeSelectionBoxWidget::createPartyPreviewFrame(
    const std::string& customAttributeCode,
    const SelectionWidgetConfiguration& widgetConfiguration,
    SmartmaticBasicLabel* const partyPreviewLabel,
    const size_t maxParties)
{
    const std::string futureContest =
        MachineOperationManager::getInstance()->getVotingExperienceController()->
        getNextContestCodeFromCustomAttribute(customAttr.getCode(), customAttributeCode);

    const size_t width  = widgetConfiguration.getBoxWidth();
    const size_t height = widgetConfiguration.getBoxHeight();

    PartyPreview* const partyPreview = manage(new PartyPreview(width, height, maxParties));

    partyPreview->setLabel(partyPreviewLabel);
    partyPreview->loadParties(futureContest);

    return partyPreview;
}

SmartmaticBasicLabel* CustomAttributeSelectionBoxWidget::createCustomAttributeLabel(
    const std::string& customAttributeCode,
    const SelectionWidgetConfiguration& widgetConfiguration)
{
    LabelConfiguration boxLabelConfiguration =
        SAESGUIConfigurationManager::ConvertToLabelConfiguration(widgetConfiguration.getBoxLabelConfiguration());

    const std::string name = ElectionStringResourcesManager::getInstance()->getResource(customAttributeCode);
    SmartmaticBasicLabel* label = manage(new SmartmaticBasicLabel(name));
    LabelConfiguration::SetLabelConfiguration(label, boxLabelConfiguration);

    return label;
}

BoxSelectionWidget* CustomAttributeSelectionBoxWidget::createCustomAttributeBox(
    const std::string& customAttributeCode,
    const bool enableClick,
    const SelectionWidgetConfiguration& widgetConfiguration,
    Widget* const widget)
{
    const Gdk::Color selected   = SAESGUIConfigurationManager::ConvertToGdkColor(widgetConfiguration.getSelectedColor());
    const Gdk::Color unselected = SAESGUIConfigurationManager::ConvertToGdkColor(widgetConfiguration.getUnSelectedColor());
    const size_t     width      = widgetConfiguration.getBoxWidth();
    const size_t     height     = widgetConfiguration.getBoxHeight();
    const bool       showBorder = widgetConfiguration.getShowBorder();
    const int        spacing    = widgetConfiguration.getBoxSpacing();

    tableWidget->setSpacings ( spacing, 0 );

    BoxSelectionWidget* const box = manage(new BoxSelectionWidget(customAttributeCode, enableClick, showBorder));
    box->setSelectedColor(selected);
    box->setUnselectedColor(unselected);
    box->set_size_request(width, height);

    box->addWidget(widget);
    box->set_name("voting_experience_custom_attribute_widget");
    box->setName("custom_attribute_widget_event_box");

    return box;
}

void CustomAttributeSelectionBoxWidget::listCustomAttributes(const bool enableClick)
{
    const SelectionWidgetConfiguration configuration =
        SAESGUIConfigurationManager::getInstance()->getCustomAttributeSelectionWidgetConfiguration();

    sortCustomAttribute();

    size_t                                 position    = 0;
    size_t                                 maxParties  = maxPartiesInContests();
    const bool                             showPreview = shouldShowPartyPreview();
    std::list<std::string>::const_iterator cadIterator = cadList.begin();

    for (; cadIterator != cadList.end(); ++cadIterator, position++)
    {
        SmartmaticBasicLabel* const label = createCustomAttributeLabel(*cadIterator, configuration);
        BoxSelectionWidget* box;

        if (showPreview)
        {
            SelectionWidgetConfiguration previewConfiguration = adjustPreviewConfiguration(configuration);
            PartyPreview* const preview = createPartyPreviewFrame(*cadIterator, previewConfiguration, label, maxParties);
            preview->signal_size_request().connect(sigc::mem_fun(*this, &CustomAttributeSelectionBoxWidget::on_party_preview_size_request));
            box = createCustomAttributeBox(*cadIterator, enableClick, previewConfiguration, preview);
        }
        else
        {
            box = createCustomAttributeBox(*cadIterator, enableClick, configuration, label);
        }

        const std::string code = StringUtils::intToString(position + 1);

        box->setAccessibleName("CustomAttributeBox" + code);
        label->setAccessibleName("CustomAttributeBox" + code);

        tableWidget->put(box, position, 0);
    }
}

void CustomAttributeSelectionBoxWidget::initialize()
{
    SAESGUIConfigurationManager* const configurationManager = SAESGUIConfigurationManager::getInstance();

    const bool   enableClick  = configurationManager->getCustomAttributeSelectionWidgetConfiguration().getEnableClick();
    const size_t headerHeight = configurationManager->getCustomAttributeSelectionWidgetConfiguration().getFrameTopHeight();
    const size_t bottomHeight = configurationManager->getCustomAttributeSelectionWidgetConfiguration().getFrameBottomHeight();

    initializeTableWidget(enableClick);
    initializeBottomButtonWidget(enableClick);

    initializeHeaderLabel(headerHeight);
    initializeBottomFrame(bottomHeight);

    initializeExternalBox(headerHeight);

    bottomButtonWidget->showButton(false, false, false, false);

    listCustomAttributes(enableClick);

    tableWidget->setAccessibleName("CustomAttributeSelectionMainTable");
    headerLabel->setAccessibleName("CustomAttributeSelectionTitle");
    bottomFrame->setAccessibleName("CustomAttributeSelectionBottom");
    externalBox->setAccessibleName("CustomAttributeSelectionContainer");
}

void CustomAttributeSelectionBoxWidget::dispose()
{
    if ( headerLabel )
    {
        delete headerLabel;
        headerLabel = NULL;
    }

    if ( bottomFrame )
    {
        delete bottomFrame;
        bottomFrame = NULL;
    }

    if ( externalBox )
    {
        delete externalBox;
        externalBox = NULL;
    }

    if ( bottomButtonWidget )
    {
        delete bottomButtonWidget;
        bottomButtonWidget = NULL;
    }
}

void CustomAttributeSelectionBoxWidget::refresh()
{
    bottomButtonWidget->update();

    Glib::ustring title;
    Glib::ustring key = _ ( "Smartmatic.SAES.GUI.Widgets.Voting.CustomAttributeSelectionBoxWidget.Title" );

    std::string nameKey = ElectionStringResourcesManager::getInstance()->getResource ( customAttr.getName_key() );
    std::string descriptionKey = ElectionStringResourcesManager::getInstance()->getResource ( customAttr.getDescription_key() );

    title = Glib::ustring::compose ( key, nameKey, descriptionKey );

    headerLabel->set_text ( title );

    if ( !flagForce )
    {
        onClearClicked();
    }
}

void CustomAttributeSelectionBoxWidget::onAcceptClicked()
{
    if ( currentBox )
    {
        signal_Answer().emit ( currentBox->getCode() );
    }
}

void CustomAttributeSelectionBoxWidget::onClearClicked()
{
    bottomButtonWidget->showButton ( false, false, false, false );
    tableWidget->clear();
    bottomButtonWidget->clearSignals();
}

void CustomAttributeSelectionBoxWidget::onBoxClicked ( std::list < Smartmatic::GUI::Widgets::BoxSelectionWidget * > & list )
{
    bool selected = !list.empty();

    if ( selected )
    {
        currentBox = ( * ( list.begin() ) );
    }
    else
    {
        currentBox = NULL;
    }

    if ( showConfirmation )
    {
        bottomButtonWidget->showButton ( selected, selected, false, false );
    }
    else if ( selected )
    {
        signal_Answer().emit ( currentBox->getCode() );
    }
}

bool CustomAttributeSelectionBoxWidget::shouldShowPartyPreview() const
{
    std::list<std::string>::const_iterator it = cadList.begin();
    bool isEnabled = false;

    for (; it != cadList.end(); ++it)
    {
        VotingExperienceController* controller = MachineOperationManager::getInstance()->getVotingExperienceController();
        const SAES::Voting::Election::Contest* contest = controller->getNextContestFromCustomAttribute(customAttr.getCode(), *it);

        if (contest->getDisplay_info().getParty_preview().present() &&
            contest->getDisplay_info().getParty_preview().get())
        {
            isEnabled = true;
        }
    }

    return isEnabled;
}

size_t CustomAttributeSelectionBoxWidget::maxPartiesInContests() const
{
    std::list<std::string>::const_iterator it = cadList.begin();
    size_t parties = 0;

    for (; it != cadList.end(); ++it)
    {
        VotingExperienceController* controller = MachineOperationManager::getInstance()->getVotingExperienceController();
        SAES::Voting::Election::Contest* contest = controller->getNextContestFromCustomAttribute(customAttr.getCode(), *it);

        std::list<Ballot_option*> partyList;
        MachineOperationManager::getInstance()->getElectoralConfiguration()->getElection()->getBallotOptionPartyByContest(*contest, partyList);

        if (partyList.size() > parties)
        {
            parties = partyList.size();
        }
    }

    return parties;

}

int CustomAttributeSelectionBoxWidget::getSmallestPartyPreviewFontSize()
{
    std::vector<Widget*> partyPreviews = tableWidget->getTable()->get_children();
    int smallest = -1;

    for (std::vector<Widget*>::iterator it = partyPreviews.begin(); it != partyPreviews.end(); ++it)
    {
        BoxSelectionWidget*      selection    = static_cast<BoxSelectionWidget*>(*it);
        SmartmaticBasicEventBox* eventBox     = selection->getEventBox();
        PartyPreview*            partyPreview = static_cast<PartyPreview*>(eventBox->get_child());
        int                      partySize    = partyPreview->getSmallestFontSize();

        if (partySize != -1 && (partySize < smallest || smallest == -1))
        {
            smallest = partySize;
        }
    }

    return smallest;
}

void CustomAttributeSelectionBoxWidget::setPartyPreviewFontSize(const int fontSize)
{
    std::vector<Widget*> partyPreviews = tableWidget->getTable()->get_children();

    for (std::vector<Widget*>::iterator it = partyPreviews.begin(); it != partyPreviews.end(); ++it)
    {
        BoxSelectionWidget*      selection    = static_cast<BoxSelectionWidget*>(*it);
        SmartmaticBasicEventBox* eventBox     = selection->getEventBox();
        PartyPreview*            partyPreview = static_cast<PartyPreview*>(eventBox->get_child());

        partyPreview->setFontSize(fontSize);
    }
}

void CustomAttributeSelectionBoxWidget::on_party_preview_size_request(Gtk::Requisition*)
{
    int size = getSmallestPartyPreviewFontSize();

    if (size != partyPreviewFont && (size < partyPreviewFont || partyPreviewFont == -1))
    {
        partyPreviewFont = size;
        setPartyPreviewFontSize(size);
    }
}
