/**
 * @file   DetailedStatistics.cxx
 * @brief  Implementation of DetailedStatistics class.
 * @author Georgi Demirchev
 * @date   Jul  8, 2013
 * @author manuel.arguelles
 * @date   Oct 19, 2017
 */

#include "DetailedStatistics.hxx"
#include <Operation/MachineOperationManager.hxx>
#include <Resources/election-string-resources-manager.h>
#include <System/Utils/StringUtils.hxx>
#include <Voting/PBReports/VoteStatisticsWrapper.hxx>
#include <Configuration/label-configuration.h>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#include <libintl.h>
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

#define N_(String) (String) ///<macro for gettext not translate

using namespace Smartmatic::GUI::Configuration;
using namespace Smartmatic::GUI::Widgets;
using namespace Smartmatic::SAES::GUI::Widgets::Voting;
using namespace Smartmatic::SAES::Operation;
using namespace Smartmatic::SAES::Resources;
using namespace Smartmatic::SAES::Voting::PBReports;
using namespace Smartmatic::SAES::Voting;
using namespace Smartmatic::System::Utils;

static const int COUNTERS_VALUE_WIDTH = 8;

/**
 * @brief  Sets the alignment of the cell renderers in the ComboBox.
 * @author manuel.arguelles
 * @date   Oct 19, 2017
 * @param  comboBox The combobox object.
 */
static void setComboBoxCellAlignment(SmartmaticBasicComboBox& comboBox)
{
    Glib::ListHandle<Gtk::CellRenderer*>           cells = comboBox.get_cells();
    Glib::ListHandle<Gtk::CellRenderer*>::iterator it    = cells.begin();

    (*it)->set_alignment(0, .5);
    ++it;
    (*it)->set_alignment(1, .5);
}

DetailedStatistics::DetailedStatistics()
{
    m_storeModel = Gtk::ListStore::create(m_columns);
    m_voteStatistics = MachineOperationManager::getInstance()->getVoteStatistics();

    m_details.set_model(m_storeModel);

    Gtk::TreeModel::Row row0 = *(m_storeModel->append());
    Gtk::TreeModel::Row row1 = *(m_storeModel->append());

    row0[m_columns.m_voterType] = _("Smartmatic.SAES.GUI.Widgets.Voting.VoteStatisticsWidget.TotalOfficialVotes");
    row0[m_columns.m_votes] = "     ";
    row1[m_columns.m_voterType] = _("Smartmatic.SAES.GUI.Widgets.Voting.VoteStatisticsWidget.DetailedStatistics.NumVoterTypes");
    row1[m_columns.m_votes] = "";

    std::list<std::string> voterTypes = MachineOperationManager::getInstance()->getRecountVoteStatistics()->GetVoterTypes();

    for (std::list<std::string>::const_iterator it = voterTypes.begin(); it != voterTypes.end(); ++it)
    {
        Gtk::TreeModel::Row row = *(m_storeModel->append());

        row[m_columns.m_voterType] = _("Smartmatic.SAES.GUI.Widgets.Voting.VoteStatisticsWidget.DetailedStatistics.Prefix") +
            ElectionStringResourcesManager::getInstance()->getResource(*it);
        row[m_columns.m_votes] = "     ";
    }

    m_details.pack_start(m_columns.m_voterType, true);
    m_details.pack_start(m_columns.m_votes, false);

    setComboBoxCellAlignment(m_details);

    m_details.set_active(0);
    m_details.set_can_focus(false);
    m_details.set_can_default(false);
    m_details.signal_changed().connect(
        sigc::mem_fun(*this, &DetailedStatistics::onSelectionChanged));
    m_details.signal_event_after().connect(
        sigc::mem_fun(*this, &DetailedStatistics::onOutOfFocus));

    m_title.setAccessibleName("StatisticTitle");
    m_details.setAccessibleName("RuningVoteCounters");

    m_buttonVoteType.set_justify(Gtk::JUSTIFY_LEFT);
    m_buttonVoteTotal.set_justify(Gtk::JUSTIFY_RIGHT);

    m_buttonHolder.pack_start(m_buttonVoteType, false, true, 5);
    m_buttonHolder.pack_end(m_buttonVoteTotal, false, true, 5);

    m_buttonFrame.add(m_buttonHolder);
    m_buttonFrame.set_shadow_type(Gtk::SHADOW_NONE);

    m_buttonExpander.add(m_buttonFrame);
    m_buttonExpander.set_can_focus(false);
    m_buttonExpander.set_can_default(false);
    m_buttonExpander.signal_clicked().connect(
        sigc::mem_fun(*this, &DetailedStatistics::onShowDetails));

    pack_start(m_title, false, false);
    pack_start(m_details, false, false);
    pack_start(m_buttonExpander, false, false);

    m_title.show();
    m_buttonExpander.show();
    m_details.hide();
}

DetailedStatistics::~DetailedStatistics()
{
}

void DetailedStatistics::updateShowDetailsButton()
{
    Gtk::TreeModel::Children entries = m_storeModel->children();

    m_buttonVoteType.set_text(entries[0].get_value(m_columns.m_voterType));
    m_buttonVoteTotal.set_text(entries[0].get_value(m_columns.m_votes));
}

void DetailedStatistics::Update()
{
    std::vector<VoteCounterByCA> votes = m_voteStatistics->getVoteCountersByCA();
    Gtk::TreeModel::Children children = m_storeModel->children();
    Gtk::TreeModel::Children::iterator it;

    it = children.begin();
    (*it)[m_columns.m_votes] = StringUtils::intToString(m_voteStatistics->getOfficialVotes());
    ++it;
    ++it;

    for (; it != children.end(); ++it)
    {
        Gtk::TreeModel::Row row = *it;

        for (std::vector<VoteCounterByCA>::iterator itv = votes.begin(); itv != votes.end(); ++itv)
        {
            std::string voter = ElectionStringResourcesManager::getInstance()->getResource(itv->getCAValue());
            std::string s(row->get_value(m_columns.m_voterType));
            s = s.substr(std::strlen(_("Smartmatic.SAES.GUI.Widgets.Voting.VoteStatisticsWidget.DetailedStatistics.Prefix")));

            if (voter == s)
            {
                std::string counter = StringUtils::intToString ( itv->getCounter() );

                if (counter.size() < COUNTERS_VALUE_WIDTH)
                {
                    int padding = COUNTERS_VALUE_WIDTH - counter.size();

                    counter.insert(0, padding, ' ');
                }

                row[m_columns.m_votes] = counter;
                break;
            }
        }
    }

    updateShowDetailsButton();

    if (m_details.property_popup_shown())
    {
        m_details.popdown();
        m_details.popup();
    }
}

void DetailedStatistics::SetTitle(const std::string &title,
                                  LabelConfiguration &conf,
                                  const int sizeX,
                                  const int sizeY)
{
    m_title.set_text(title);
    LabelConfiguration::SetLabelConfiguration(&m_title, conf);
    m_title.set_size_request(sizeX, sizeY);
    m_title.set_line_wrap(true);
}


void DetailedStatistics::onSelectionChanged()
{
    m_details.set_active(0);
}

void DetailedStatistics::onShowDetails()
{
    m_buttonExpander.hide();
    m_details.show();
    m_details.popup();
}

void DetailedStatistics::onOutOfFocus(GdkEvent* event)
{
    m_buttonExpander.show();
    m_details.hide();
}

