/**
 * @file IPrintVoteWidget.cxx
 * @brief Body class print vote widget
 * @date 29/02/2012
 * @author Yadickson Soto
 */

#include "IPrintVoteWidget.hxx"
#include <Configuration/s-a-e-s-g-u-i-conf-manager.h>
#include <Operation/MachineOperationManager.hxx>
#include <Operation/AlarmMessage.hxx>
#include <Gui/Widgets/Voting/FactoryVotingWidget.hxx>
#include <System/Sounds/smartmatic-sounds-manager.h>
#include <Operation/Voting/VoteCompliance/VoteComplianceVerificator.hxx>
#include <Log/ISMTTLog.h>
#include <Log/SMTTLogManager.h>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#include <libintl.h>
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

#define N_(String) (String) ///<macro for gettext not translate

using namespace Smartmatic::SAES::Operation::VoteCompliance;
using namespace Smartmatic::SAES::GUI::Widgets::Voting;
using namespace Smartmatic::SAES::GUI::Widgets;
using namespace Smartmatic::SAES::GUI::Configuration;
using namespace Smartmatic::SAES::GUI::Operation;
using namespace Smartmatic::SAES::Operation::Printer;
using namespace Smartmatic::SAES::Operation::Devices;
using namespace Smartmatic::SAES::Operation;
using namespace Smartmatic::SAES::Printing;
using namespace Smartmatic::SAES::Voting;
using namespace Smartmatic::SAES::Functionality::Configuration;
using namespace Smartmatic::System;
using namespace Smartmatic::Log;
using namespace Smartmatic::System::Exception;

static const std::string SUCCESS_FINAL_MESSAGE_KEY = _("smartmatic.saes.gui.widgets.voting.IPrintVoteWidget.SuccessFinalMessage");
static const std::string ERROR_FINAL_MESSAGE_KEY = _("smartmatic.saes.gui.widgets.voting.IPrintVoteWidget.ErrorFinalMessage");
static const std::string VOTE_COMPLIANCE_FAILURE_MESSAGE_KEY = _("smartmatic.saes.gui.widgets.voting.IPrintVoteWidget.VoteComplianceFailureMessage");

ISMTTLog * IPrintVoteWidget::logger = SMTTLogManager::GetLogger ( "Smartmatic.SAES.Operation.Voting.IPrintVoteWidget", GETTEXT_PACKAGE );

IPrintVoteWidget::IPrintVoteWidget ( PrintVoteErrorWidget * printVoteErrorWidget, VoteComplianceFailureWidget * voteComplianceFailureWidget )
    : WizardWidgetBase(), printVoteErrorWidget ( printVoteErrorWidget ), voteComplianceFailureWidget ( voteComplianceFailureWidget )
{
    printer = NULL;
    votePrintDocument = NULL;
    m_vote = NULL;

    setTitle ( false );
    setShowBorder ( true );
}

IPrintVoteWidget::~IPrintVoteWidget()
{
    this->remove();

    delete printer;
    delete votePrintDocument;
    delete printVoteErrorWidget;
    delete voteComplianceFailureWidget;

    printer = NULL;
    votePrintDocument = NULL;
    printVoteErrorWidget = NULL;
    voteComplianceFailureWidget = NULL;
}

void IPrintVoteWidget::initialize()
{
    if ( printer )
    {
        delete printer;
        printer = NULL;
    }

    if ( votePrintDocument )
    {
        delete votePrintDocument;
        votePrintDocument = NULL;
    }

    printer = new PrinterController();
    printer->getSignalOk().connect ( sigc::mem_fun ( *this, &IPrintVoteWidget::onOK ) );
    printer->getSignalError().connect ( sigc::mem_fun ( *this, &IPrintVoteWidget::onError ) );

    std::string sound = SAESGUIConfigurationManager::getInstance()->getVotingExperienceSound();
    Smartmatic::System::Sounds::SMTTSoundsManager::getInstance()->Sound ( sound );

    try
    {
        m_vote = MachineOperationManager::getInstance()->getVotingExperienceController()->getVote();
        votePrintDocument = new VotePrintDocument ( MachineOperationManager::getInstance()->getElectoralConfiguration(), *m_vote );

        if (VoteComplianceVerificator::getInstance(MachineOperationManager::getInstance()->getElectoralConfiguration())->checkAllBeforePrintingVoteConditions(*m_vote, votePrintDocument->getTextToPrintMap()) == false)
        {
            AlarmMessage::alarm ( AlarmEnum::CONFIGURATION_VOTECOMPLIANCEFAILURE );

            voteComplianceFailureWidget->refresh();
            setWidget ( voteComplianceFailureWidget );

            unsigned int time = SAESGUIConfigurationManager::getInstance()->getWaitTimeByBasicImageType ( Smartmatic::SAES::Voting::Screen::BasicImageType::GENERATED_IMAGES_ERROR_PRINT_VOTE );
            Glib::signal_timeout().connect_once ( sigc::mem_fun ( *this, &IPrintVoteWidget::endError ), time );
            m_play_audio_signal.emit(VOTE_COMPLIANCE_FAILURE_MESSAGE_KEY);
        }
        else
        {
            safeTime.start();
            printer->start ( votePrintDocument );
        }
    }
    catch (ElectionException &electionException)
    {
        logger->Error("Exception caught while generating the vote: " + electionException.getMessage());
        onError(electionException.getMessage());
    }
}

void IPrintVoteWidget::onOK()
{
    safeTime.stop();

    int voteTime = safeTime.getAllSeconds() * 1000;
    int time = SAESGUIConfigurationManager::getInstance()->getWaitTimeByBasicImageType ( Smartmatic::SAES::Voting::Screen::BasicImageType::GENERATED_IMAGES_PRINT_VOTE );

    int result = time - voteTime;

    if ( result <= 0 )
    {
        result = 0;
    }

    Glib::signal_timeout().connect_once ( sigc::mem_fun ( *this, &IPrintVoteWidget::endOK ), result );
    m_play_audio_signal.emit(SUCCESS_FINAL_MESSAGE_KEY);
}

void IPrintVoteWidget::onError ( const std::string& message )
{
    AlarmMessage::alarm ( AlarmEnum::CONFIGURATION_VOTEPRINTINGFAILURE );

    printVoteErrorWidget->refresh();
    setWidget ( printVoteErrorWidget );

    unsigned int time = SAESGUIConfigurationManager::getInstance()->getWaitTimeByBasicImageType ( Smartmatic::SAES::Voting::Screen::BasicImageType::GENERATED_IMAGES_ERROR_PRINT_VOTE );
    Glib::signal_timeout().connect_once ( sigc::mem_fun ( *this, &IPrintVoteWidget::endError ), time );
    m_play_audio_signal.emit(ERROR_FINAL_MESSAGE_KEY);
}

void IPrintVoteWidget::endOK()
{
    emit();
}

void IPrintVoteWidget::endError()
{
    emit();
    m_stop_signal.emit();
}
