/**
 * @file	LanguageSelectionBoxWidget.cxx
 * @brief	Body interface language selection widget.
 * @author	Yadickson Soto
 * @date	13/11/2012
 */

#include "LanguageSelectionBoxWidget.hxx"
#include "Resources/election-string-resources-manager.h"
#include "Configuration/s-a-e-s-g-u-i-conf-manager.h"
#include <Operation/FormatLabel.hxx>
#include "Voting/set-language-functionality.h"
#include "FactoryVotingWidget.hxx"
#include <System/Utils/StringUtils.hxx>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#include <libintl.h>
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

#define N_(String) (String) ///<macro for gettext not translate

using namespace Smartmatic::GUI::Widgets;
using namespace Smartmatic::SAES::GUI::Widgets::Voting;
using namespace Smartmatic::GUI::Configuration;
using namespace Smartmatic::SAES::GUI::Configuration;
using namespace Smartmatic::SAES::Voting;
using namespace Smartmatic::SAES::Voting::Election;
using namespace Smartmatic::SAES::Operation;
using namespace Smartmatic::SAES::Operation::Voting;
using namespace Smartmatic::SAES::Resources;
using namespace Smartmatic::SAES::Voting::VotingDevice;
using namespace Smartmatic::GUI::Configuration;
using namespace Smartmatic::SAES::GUI::Operation::Voting;
using namespace Smartmatic::SAES::GUI::Operation;
using namespace Smartmatic::System::Utils;

LanguageSelectionBoxWidget::LanguageSelectionBoxWidget ( bool showConfirmation, bool sortAlphabetically )
    : ILanguageSelectionWidget ( showConfirmation, sortAlphabetically )
{
    currentBox = NULL;
    headerLabel = NULL;
    bottomFrame = NULL;
    externalBox = NULL;
    tableWidget = NULL;
    buttonWidget = NULL;
    flagForce = false;

    initialize();

    setWidget ( externalBox );
    setAccessibleName ( "LanguageSelectionBox" );
}

LanguageSelectionBoxWidget::~LanguageSelectionBoxWidget()
{
    remove ();
    dispose();
}

void LanguageSelectionBoxWidget::initialize()
{
    int spacing = SAESGUIConfigurationManager::getInstance()->getLanguageSelectionWidgetConfiguration().getBoxSpacing();

    tableWidget = new TableBoxSelectionWidget ( false, false, true );
    tableWidget->setSpacings ( spacing, 0 );

    buttonWidget = FactoryVotingWidget::createLanguageSelectionButtonWidget();
    buttonWidget->update();

    bool enableClick = SAESGUIConfigurationManager::getInstance()->getLanguageSelectionWidgetConfiguration().getEnableClick();

    if ( enableClick )
    {
        tableWidget->getSignalClick().connect ( sigc::mem_fun ( this, &LanguageSelectionBoxWidget::onBoxClicked ) );
        buttonWidget->getSignalAcceptButtonClicked().connect ( sigc::mem_fun ( this, &LanguageSelectionBoxWidget::onAcceptClicked ) );
        buttonWidget->getSignalClearButtonClicked().connect ( sigc::mem_fun ( this, &LanguageSelectionBoxWidget::onClearClicked ) );
    }

    int headerHeight = SAESGUIConfigurationManager::getInstance()->getLanguageSelectionWidgetConfiguration().getFrameTopHeight();
    int bottomHeight = SAESGUIConfigurationManager::getInstance()->getLanguageSelectionWidgetConfiguration().getFrameBottomHeight();

    int boxWidth = SAESGUIConfigurationManager::getInstance()->getLanguageSelectionWidgetConfiguration().getBoxWidth();
    int boxHeight = SAESGUIConfigurationManager::getInstance()->getLanguageSelectionWidgetConfiguration().getBoxHeight();

    headerLabel = new SmartmaticBasicLabel();
    headerLabel->show();
    headerLabel->set_size_request ( -1, headerHeight );

    Smartmatic::GUI::Configuration::LabelConfiguration labelConf = SAESGUIConfigurationManager::ConvertToLabelConfiguration ( SAESGUIConfigurationManager::getInstance()->getLanguageSelectionWidgetConfiguration().getHeaderLabelConfiguration() );
    Smartmatic::GUI::Configuration::LabelConfiguration::SetLabelConfiguration ( headerLabel, labelConf );

    bottomFrame = new SmartmaticBasicFrame();
    bottomFrame->set_shadow_type ( Gtk::SHADOW_NONE );
    bottomFrame->show();
    bottomFrame->add ( *buttonWidget );
    bottomFrame->set_size_request ( -1, bottomHeight );

    externalBox = new SmartmaticBasicVBox ( false , 0 );

    if ( headerHeight > 0 )
    {
        Gtk::HSeparator * separator = manage ( new Gtk::HSeparator() );
        externalBox->pack_start ( *headerLabel, false, false );
        externalBox->pack_start ( *separator, false, false );
    }

    externalBox->pack_start ( *tableWidget );
    externalBox->pack_end ( *bottomFrame, false, false, 5 );

    buttonWidget->showButton ( false, false, false, false );

    Gdk::Color selected = SAESGUIConfigurationManager::ConvertToGdkColor ( SAESGUIConfigurationManager::getInstance()->getLanguageSelectionWidgetConfiguration().getSelectedColor() );
    Gdk::Color unselected = SAESGUIConfigurationManager::ConvertToGdkColor ( SAESGUIConfigurationManager::getInstance()->getLanguageSelectionWidgetConfiguration().getUnSelectedColor() );
    Smartmatic::GUI::Configuration::LabelConfiguration boxLabelConf = SAESGUIConfigurationManager::ConvertToLabelConfiguration ( SAESGUIConfigurationManager::getInstance()->getLanguageSelectionWidgetConfiguration().getBoxLabelConfiguration() );

    bool showBorderBox = SAESGUIConfigurationManager::getInstance()->getLanguageSelectionWidgetConfiguration().getShowBorder();

    std::list<Smartmatic::SAES::Voting::Election::Language>::iterator it;
    int pos = 0;

    for ( it = languagesList.begin();
            it != languagesList.end();
            ++it, pos++ )
    {
        std::string name = ElectionStringResourcesManager::getInstance()->getResource ( it->getName_key() );
        std::string code = it->getCustom_code();

        BoxSelectionWidget * box = manage ( new BoxSelectionWidget ( code, enableClick, showBorderBox ) );
        box->setSelectedColor ( selected );
        box->setUnselectedColor ( unselected );
        box->set_size_request ( boxWidth, boxHeight );

        SmartmaticBasicLabel * label = manage ( new SmartmaticBasicLabel ( name ) );
        Smartmatic::GUI::Configuration::LabelConfiguration::SetLabelConfiguration ( label, boxLabelConf );
        label->set_size_request ( boxWidth, boxHeight );

        box->addWidget ( label );
        box->set_name ( "voting_experience_language_widget" );
        box->setName ( "language_widget_event_box" );

        std::string language = StringUtils::toUpper ( it->getLanguage() );

        box->setAccessibleName ( "LanguageBox" + language );
        label->setAccessibleName ( "LanguageBox" + language );

        tableWidget->put ( box, pos, 0 );
    }

    tableWidget->setAccessibleName ( "LanguageSelectionMainTable" );
    headerLabel->setAccessibleName ( "LanguageSelectionTitle" );
    bottomFrame->setAccessibleName ( "LanguageSelectionBottom" );
    externalBox->setAccessibleName ( "LanguageSelectionContainer" );
}

void LanguageSelectionBoxWidget::dispose()
{
    if ( headerLabel )
    {
        delete headerLabel;
        headerLabel = NULL;
    }

    if ( bottomFrame )
    {
        delete bottomFrame;
        bottomFrame = NULL;
    }

    if ( externalBox )
    {
        delete externalBox;
        externalBox = NULL;
    }

    if ( tableWidget )
    {
        delete tableWidget;
        tableWidget = NULL;
    }

    if ( buttonWidget )
    {
        delete buttonWidget;
        buttonWidget = NULL;
    }
}

void LanguageSelectionBoxWidget::refresh()
{
    std::string title = FormatLabel::format ( N_ ( "Smartmatic.SAES.GUI.Widgets.Voting.LanguageSelectionBoxWidget.Title" ), "/" );

    buttonWidget->update();
    headerLabel->set_text ( title );

    if ( !flagForce )
    {
        onClearClicked ();
    }
}

void LanguageSelectionBoxWidget::SimulateButtonNPressed ( unsigned int index )
{
    flagForce = true;
    tableWidget->forceClick ( index );
    buttonWidget->showButton ( true, true, false, false );
}

void LanguageSelectionBoxWidget::onAcceptClicked()
{
    if ( currentBox )
    {
        setLanguage ( currentBox->getCode() );
    }
}

void LanguageSelectionBoxWidget::onClearClicked()
{
    buttonWidget->showButton ( false, false, false, false );
    tableWidget->clear();
    buttonWidget->clearSignals();
}

void LanguageSelectionBoxWidget::onBoxClicked ( std::list < Smartmatic::GUI::Widgets::BoxSelectionWidget * > & list )
{
    bool selected = !list.empty();

    if ( selected )
    {
        currentBox = ( * ( list.begin() ) );
    }
    else
    {
        currentBox = NULL;
    }

    if ( showConfirmation )
    {
        buttonWidget->showButton ( selected, selected, false, false );
    }
    else if ( selected )
    {
        setLanguage ( currentBox->getCode() );
    }
}

void LanguageSelectionBoxWidget::setLanguage ( std::string code )
{
    Smartmatic::SAES::Voting::SetLanguageFunctionality::SetNewVotingLanguage ( code );
    emit();
}
