/**
 * @file SimulatorVotesWidget.cxx
 * @brief Body simulator votes widget class
 * @date 05/10/2012
 * @author Yadickson Soto
 */

#include "SimulatorVotesWidget.hxx"
#include <Log/ISMTTLog.h>
#include <Log/SMTTLogManager.h>
#include <Environment/Configurations/resource-path-provider.h>
#include <System/Utils/StringUtils.hxx>
#include <Operation/MachineOperationManager.hxx>
#include <Voting/Vote/VoteWrapper.hxx>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#include <libintl.h>
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

#define N_(String) (String) ///<macro for gettext not translate

using namespace Smartmatic::SAES::GUI::Widgets::Voting;
using namespace Smartmatic::SAES::GUI::Widgets;
using namespace Smartmatic::GUI::Widgets;
using namespace Smartmatic::Environment;
using namespace Smartmatic::Log;
using namespace Smartmatic::SAES::Voting::Election;
using namespace Smartmatic::SAES::Voting::VotingDevice;
using namespace Smartmatic::System::Utils;
using namespace Smartmatic::SAES::Operation;
using namespace Smartmatic::SAES::Printing;
using namespace Smartmatic::SAES::Operation::Devices;

ISMTTLog * SimulatorVotesWidget::logger = SMTTLogManager::GetLogger ( "Smartmatic.SAES.GUI.Widgets.Voting.SimulatorVotesWidget", GETTEXT_PACKAGE );

SimulatorVotesWidget::SimulatorVotesWidget()
    : StepWidget()
{
    setTitle ( true );
    setTitleText ( N_ ( "Smartmatic.SAES.GUI.Widgets.Voting.SimulatorVotesWidget.Title" ) );
    mainFrame = NULL;
    mainAlignment = NULL;
    tableAlignment = NULL;
    mainVbox = NULL;
    countText = NULL;
    fullText = NULL;
    fullCheck = NULL;
    messageText = NULL;
    table = NULL;
    buttonBox = NULL;
    countEntry = NULL;
    simButton = NULL;
    exitButton = NULL;
    votePrint = NULL;

    printer = new PrinterController();
    printer->getSignalOk().connect ( sigc::mem_fun ( *this, &SimulatorVotesWidget::endPrint ) );
    printer->getSignalError().connect ( sigc::mem_fun ( *this, &SimulatorVotesWidget::errorPrint ) );

    initialize();

    setWidget ( mainFrame );
}

SimulatorVotesWidget::~SimulatorVotesWidget()
{
    remove();
    dispose();

    if ( mainFrame )
    {
        delete mainFrame;
        mainFrame = NULL;
    }

    if ( mainAlignment )
    {
        delete mainAlignment;
        mainAlignment = NULL;
    }

    if ( tableAlignment )
    {
        delete tableAlignment;
        tableAlignment = NULL;
    }

    if ( mainVbox )
    {
        delete mainVbox;
        mainVbox = NULL;
    }

    if ( countText )
    {
        delete countText;
        countText = NULL;
    }

    if ( fullText )
    {
        delete fullText;
        fullText = NULL;
    }

    if ( fullCheck )
    {
        delete fullCheck;
        fullCheck = NULL;
    }

    if ( messageText )
    {
        delete messageText;
        messageText = NULL;
    }

    if ( table )
    {
        delete table;
        table = NULL;
    }

    if ( buttonBox )
    {
        delete buttonBox;
        buttonBox = NULL;
    }

    if ( countEntry )
    {
        delete countEntry;
        countEntry = NULL;
    }

    if ( simButton )
    {
        delete simButton;
        simButton = NULL;
    }

    if ( exitButton )
    {
        delete exitButton;
        exitButton = NULL;
    }

    if ( printer )
    {
        delete printer;
        printer = NULL;
    }
}

void SimulatorVotesWidget::initialize()
{
    Glib::RefPtr<Gtk::Builder> builder;
    std::string gladeFile = ResourcePathProvider::getInstance()->GetResourcePath ( PACKAGE_NAME, "ui/SimulatorVotesWidget.glade" );

    try
    {
        builder = Gtk::Builder::create_from_file ( gladeFile );
    }
    catch ( const Glib::FileError & ex )
    {
        logger->Debug ( "FATAL - SimulatorVotesWidget Initialize" );
    }

    builder->get_widget ( "main-frame", mainFrame );
    builder->get_widget ( "main-vbox", mainVbox );
    builder->get_widget ( "main-alignment", mainAlignment );
    builder->get_widget ( "table-alignment", tableAlignment );
    builder->get_widget ( "table", table );
    builder->get_widget ( "label-count", countText );
    builder->get_widget ( "label-message", messageText );
    builder->get_widget ( "label-full-vote", fullText );
    builder->get_widget ( "hbuttonbox", buttonBox );
    builder->get_widget_derived ( "full-vote", fullCheck );
    builder->get_widget_derived ( "entry-count", countEntry );
    builder->get_widget_derived ( "sim-button", simButton );
    builder->get_widget_derived ( "exit-button", exitButton );
    countEntry->SetType ( SmartmaticEntry::Mask );
    countEntry->setMask ( "0123456789" );
    countEntry->setMax ( 3 );
    countEntry->setMandatory ( true );
    countEntry->setRegex ( "[0-9]" );
    countText->set_text ( _ ( "Smartmatic.SAES.GUI.Widgets.Voting.SimulatorVotesWidget.Votes" ) );
    fullText->set_text ( _ ( "Smartmatic.SAES.GUI.Widgets.Voting.SimulatorVotesWidget.Full" ) );
    simButton->signal_clicked().connect ( sigc::mem_fun ( *this, &SimulatorVotesWidget::validate ) );
    exitButton->signal_clicked().connect ( sigc::mem_fun ( *this, &SimulatorVotesWidget::end ) );
    simButton->SetButtonType ( SmartmaticButton::Ok );
    simButton->WidgetsPosition ( 5, 6, 0 );
    simButton->SetLabel ( _ ( "Smartmatic.SAES.GUI.Widgets.Voting.SimulatorVotesWidget.Run" ) );
    exitButton->SetButtonType ( SmartmaticButton::Cancel );
    exitButton->WidgetsPosition ( 5, 6, 0 );
    exitButton->SetLabel ( _ ( "Smartmatic.SAES.GUI.Widgets.Voting.SimulatorVotesWidget.Cancel" ) );
    message = " ";
}

void SimulatorVotesWidget::dispose()
{
    if ( votePrint )
    {
        delete votePrint;
        votePrint = NULL;
    }

    std::vector < Smartmatic::SAES::Voting::Vote::Vote * >::iterator it;

    for ( it = voteList.begin();
            it != voteList.end();
            ++it )
    {
        Smartmatic::SAES::Voting::Vote::Vote * vote = ( *it );
        delete vote;
        vote = NULL;
    }

    voteList.clear();
    count = 0;
}

void SimulatorVotesWidget::addVotes()
{
    bool full = fullCheck->get_active();
    unsigned int count = StringUtils::stringToInt ( countEntry->get_text() );

    for ( unsigned int i = 0; i < count; i++ )
    {
        Smartmatic::SAES::Voting::Vote::Vote * vote = NULL;

        if ( full )
        {
            vote = Smartmatic::SAES::Voting::Vote::VoteWrapper::createFullRandomVote ( MachineOperationManager::getInstance()->getElectoralConfiguration(), MachineOperationManager::getInstance()->getOperationStatus() );
        }
        else
        {
            vote = Smartmatic::SAES::Voting::Vote::VoteWrapper::createRandomVote ( MachineOperationManager::getInstance()->getElectoralConfiguration(), MachineOperationManager::getInstance()->getOperationStatus() );
        }

        voteList.push_back ( vote );
    }
}

void SimulatorVotesWidget::validate()
{
    if ( countEntry->validate() && StringUtils::stringToInt ( countEntry->get_text() ) > 0 )
    {
        simulate();
    }
    else
    {
        messageText->set_text ( _ ( "Smartmatic.SAES.GUI.Widgets.Voting.SimulatorVotesWidget.Error" ) );
    }
}

void SimulatorVotesWidget::simulate()
{
    dispose();
    simButton->set_sensitive ( false );
    exitButton->set_sensitive ( false );
    messageText->set_text ( _ ( "Smartmatic.SAES.GUI.Widgets.Voting.SimulatorVotesWidget.Start" ) );
    addVotes();
    nextPrint();
}

void SimulatorVotesWidget::endSimulate()
{
    simButton->set_sensitive ( true );
    exitButton->set_sensitive ( true );
    messageText->set_text ( message );
    stopPrint();
}

void SimulatorVotesWidget::messageUpdate()
{
    messageText->set_text ( message );
}

void SimulatorVotesWidget::end()
{
    emit();
}

void SimulatorVotesWidget::nextPrint()
{
    startPrint();
    Glib::ustring composed = Glib::ustring::compose (
                                 _ ( "Smartmatic.SAES.GUI.Widgets.Voting.SimulatorVotesWidget.RunMessage" ),
                                 count + 1,
                                 voteList.size() );
    messageText->set_text ( composed );

    if ( votePrint )
    {
        delete votePrint;
        votePrint = NULL;
    }

    Smartmatic::SAES::Voting::Vote::Vote * vote = voteList[count];
    votePrint = new VotePrintDocument ( MachineOperationManager::getInstance()->getElectoralConfiguration(), *vote );

    printer->start ( votePrint );
}

void SimulatorVotesWidget::endPrint()
{
    count++;

    if ( count == voteList.size() )
    {
        message = _ ( "Smartmatic.SAES.GUI.Widgets.Voting.SimulatorVotesWidget.End" );
        Glib::signal_timeout().connect_once ( sigc::mem_fun ( *this , &SimulatorVotesWidget::endSimulate ), 0 );
    }
    else
    {
        Glib::signal_timeout().connect_once ( sigc::mem_fun ( *this , &SimulatorVotesWidget::nextPrint ), 0 );
    }
}

void SimulatorVotesWidget::errorPrint ( const std::string& message )
{
    this->message = message;
    Glib::signal_timeout().connect_once ( sigc::mem_fun ( *this , &SimulatorVotesWidget::endSimulate ), 0 );
}
