/**
 * @file   VotingMachineInstallationWizardWidget.cxx
 * @brief  Implementation of VotingMachineInstallationWizardWidget class.
 * @author Youssef.tourkmani
 * @author manuel.arguelles
 * @date   01/11/2012
 */

#include "VotingMachineInstallationWizardWidget.hxx"
#include <System/Runtime/PowerDown.hxx>
#include <Operation/MachineOperationManager.hxx>
#include "Configuration/s-a-e-s-g-u-i-conf-manager.h"
#include "Configuration/s-m-t-t-g-u-i-conf-manager.h"
#include "Security/Encryption.hxx"
#include "Security/SecretKey.h"
#include <Log/ISMTTLog.h>
#include <Log/SMTTLogManager.h>
#include <Runtime/Environment/MemoryPathController.h>
#include "Functionalities/VotingMachineInstallationFunctionality.hxx"
#include <Configuration/SaesGUIConfiguration-schema.hxx>
#include <Operation/Monitor/DeviceTypeMonitorPool.hxx>
#include <Gui/Windows/MemoryMessageWindow.hxx>
#include <Operation/CheckDataManager.hxx>
#include <Gui/Widgets/ProgressWidget.hxx>
#include <Operation/Monitor/DeviceTypeMonitor.hxx>
#include <Configuration/MachineModelEnum.hxx>
#include <System/Sounds/smartmatic-sounds-manager.h>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#include <libintl.h>
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

#define N_(String) (String) ///<macro for gettext not translate

#define EVENT_BOX_STEP_WIDTH 900
#define EVENT_BOX_STEP_HEIGHT 70
#define NEXT_BUTTON_WIDTH 300
#define NEXT_BUTTON_HEIGHT 50

using namespace Gtk;
using namespace Smartmatic::Configuration;
using namespace Smartmatic::Log;
using namespace Smartmatic::Environment;
using namespace Smartmatic::GUI::Configuration;
using namespace Smartmatic::SAES::Common::Security;
using namespace Smartmatic::SAES::Configuration;
using namespace Smartmatic::SAES::GUI::Configuration;
using namespace Smartmatic::SAES::GUI::Functionality;
using namespace Smartmatic::SAES::GUI::Widgets;
using namespace Smartmatic::SAES::GUI::Windows;
using namespace Smartmatic::SAES::Operation;
using namespace Smartmatic::SAES::Security;
using namespace Smartmatic::SAES::Runtime::Environment;
using namespace Smartmatic::SAES::Configuration;
using namespace Smartmatic::SAES::Common::Operation::Monitor;
using namespace Smartmatic::SAES::Operation::Monitor;
using namespace Smartmatic::System::Runtime;
using namespace Smartmatic::System::Sounds;

ISMTTLog* VotingMachineInstallationWizardWidget::logger = SMTTLogManager::GetLogger(
    "Smartmatic.SAES.GUI.VotingMachineInstallationWizardWidget",
    GETTEXT_PACKAGE);

VotingMachineInstallationWizardWidget::VotingMachineInstallationWizardWidget(ProgressWidget* progress)
    : StepWidget(), m_progress(progress), m_usbMonitor(NULL), m_imageCounter(0),
      m_threadCheckData(NULL), m_threadSync(NULL)
{
    CardActivatorFunctionalityConfiguration* config = SAESGUIConfigurationManager::getInstance()->GetCardActivatorWidgetConfiguration();

    int eventBoxStepWidth = EVENT_BOX_STEP_WIDTH;
    int eventBoxStepHeight = EVENT_BOX_STEP_HEIGHT;
    int nextButtonWidth = NEXT_BUTTON_WIDTH;
    int nextButtonHeight = NEXT_BUTTON_HEIGHT;

    if ( SMTTGUIConfigurationManager::getInstance()->getSmartmaticGUIMachineModelConfiguration().getId() == EnumToString( PM_VIU805 ) )
    {
        double guiReconfigurationFactor = SMTTGUIConfigurationManager::getInstance()->getConfiguration()->getSmartmaticGUIMachineModelConfiguration().getGuiReconfigurationFactor();

        eventBoxStepWidth *= guiReconfigurationFactor;
        eventBoxStepHeight *= guiReconfigurationFactor;
        nextButtonWidth *= guiReconfigurationFactor;
        nextButtonHeight *= guiReconfigurationFactor;
    }

    m_imageStep.set_size_request(100, 70);
    m_imageStepColor.set_rgb(
        config->getSelectedColor().getRed(),
        config->getSelectedColor().getGreen(),
        config->getSelectedColor().getBlue());

    m_eventBoxImageStep.set_name("EventBoxImageStepInstallationWizard");
    m_eventBoxImageStep.add(m_imageStep);
    m_eventBoxImageStep.set_size_request(100, 70);
    m_eventBoxImageStep.modify_bg(Gtk::STATE_NORMAL, m_imageStepColor);

    Pango::FontDescription fontTitle = m_labelStep.get_style()->get_font();
    fontTitle.set_size(15 * Pango::SCALE);

    m_labelStep.set_alignment(0.0, 0.5);
    m_labelStep.set_padding(5, 0);
    m_labelStep.modify_font(fontTitle);
    m_labelStep.set_size_request(-1, 50);
    m_labelStep.set_label(_("Smartmatic.SAES.GUI.Widgets.VotingMachineInstallationWizardWidget.AcceptInstallation"));

    m_hboxStep.set_spacing(10);
    m_hboxStep.set_size_request(-1, 70);
    m_hboxStep.pack_start(m_eventBoxImageStep, Gtk::PACK_SHRINK);
    m_hboxStep.pack_start(m_labelStep, true, true, 10);

    m_progress->setImage("ui/check-flash-structure.gif");
    m_progress->hideInfoTitle();
    m_progress->setInfoText(N_("Smartmatic.SAES.GUI.Widgets.VotingMachineInstallationWizardWidget.LabelMessageInitial"));

    m_nextButton.hide();
    m_nextButton.set_sensitive(false);
    m_nextButton.SetLabel(_("Smartmatic.SAES.GUI.Widgets.VotingMachineInstallationWizardWidget.FinishButton"));
    m_nextButton.WidgetsPosition(5, 0, 0);
    m_nextButton.set_size_request(nextButtonWidth, nextButtonHeight);

    if (m_connClickButton.connected())
    {
        m_connClickButton.disconnect();
    }

    m_connClickButton = m_nextButton.signal_clicked().connect(
        sigc::mem_fun(
            this,
            &VotingMachineInstallationWizardWidget::on_buttonNext_clicked));

    m_nextButton.set_no_show_all();

    m_hboxButton.add(m_nextButton);

    m_hboxStepColor.set_rgb(
        config->getUnSelectedColor().getRed(),
        config->getUnSelectedColor().getGreen(),
        config->getUnSelectedColor().getBlue());

    m_eventBoxStep.set_size_request( eventBoxStepWidth, eventBoxStepHeight );
    m_eventBoxStep.set_name("EventBoxStepInstallationWizard");
    m_eventBoxStep.modify_bg(Gtk::STATE_NORMAL, m_hboxStepColor);
    m_eventBoxStep.add(m_hboxStep);

    m_vboxPrincipal.set_spacing(3);
    m_vboxPrincipal.pack_start(m_eventBoxStep, false, false);
    m_vboxPrincipal.add(*m_progress);
    m_vboxPrincipal.add(m_hboxButton);

    setWidget(&m_vboxPrincipal);
    set_visible(true);

    m_stepSequence.push_back(WIZ_EXTRACT_MEDIA);
    m_stepSequence.push_back(WIZ_INSTALL_MEDIA);
    m_stepSequence.push_back(WIZ_VERIFY_MEDIA);
    m_stepSequence.push_back(WIZ_FINALIZE);
    m_stepSequence.push_back(WIZ_END);

    m_eventBoxImageStep.setAccessibleName("EventBoxImageContainer");
    m_hboxStep.setAccessibleName("HBoxImageContainer");
    m_vboxPrincipal.setAccessibleName("VBoxMainContainer");
    m_nextButton.setAccessibleName("NextStep");

    setAccessibleName("VotingMachineInstallationWizardWidget");

    setImageByStep();
    setAccessibleNameByStep();

    m_signalEndSynchronize.connect(
        sigc::mem_fun(
            *this,
            &VotingMachineInstallationWizardWidget::endSynchronize));
}

void VotingMachineInstallationWizardWidget::mForwardProcess()
{
    DisconnectConnections();
    setImageByStep();
    setAccessibleNameByStep();

    WizardStepType current_step = *m_processTracker;

    switch (current_step)
    {
        case WIZ_EXTRACT_MEDIA:
            CheckDataManager::getInstance()->stopDetection();
            MemoryMessageWindow::getInstance()->stop();
            VotingMachineInstallationFunctionality::getInstance()->disableHelpButton();
            VotingMachineInstallationFunctionality::getInstance()->disableExitButton();
            m_labelStep.set_label(_("Smartmatic.SAES.GUI.Widgets.VotingMachineInstallationWizardWidget.PleaseExtractDevice"));
            m_progress->setInfoText(N_("Smartmatic.SAES.GUI.Widgets.VotingMachineInstallationWizardWidget.LabelMessageExtractMedia"));
            m_progress->setImage("ui/wiz_extract_media.gif");
            m_connExtract = Glib::signal_timeout().connect(sigc::mem_fun(*this, &VotingMachineInstallationWizardWidget::Extract), 1000);
            SMTTSoundsManager::getInstance()->Sound ( "SoundRemoveMemory" );
            break;

        case WIZ_INSTALL_MEDIA:
            m_labelStep.set_label(_("Smartmatic.SAES.GUI.Widgets.VotingMachineInstallationWizardWidget.InstallDevice"));
            m_progress->setInfoText(N_("Smartmatic.SAES.GUI.Widgets.VotingMachineInstallationWizardWidget.InstallAndReturn"));
            m_progress->setImage("ui/wiz_install_media.gif");
            m_connDetect = Glib::signal_timeout().connect(sigc::mem_fun(*this, &VotingMachineInstallationWizardWidget::DetectAllMount), 500);
            break;

        case WIZ_VERIFY_MEDIA:
            m_labelStep.set_label(_("Smartmatic.SAES.GUI.Widgets.VotingMachineInstallationWizardWidget.VerifyDataDevice"));
            m_progress->setInfoText(N_("Smartmatic.SAES.GUI.Widgets.VotingMachineInstallationWizardWidget.VerifyDataAndFinalize"));
            m_progress->setImage("ui/check-flash-structure.gif");

            if (!Glib::thread_supported())
            {
                Glib::thread_init();
            }

            if (m_threadCheckData != NULL)
            {
                m_threadCheckData->join();
                m_threadCheckData = NULL;
            }

            m_threadCheckData = Glib::Thread::create(sigc::mem_fun(*this, &VotingMachineInstallationWizardWidget::checkData), true);

            break;

        case WIZ_FINALIZE:
            VotingMachineInstallationFunctionality::getInstance()->enableHelpButton();
            VotingMachineInstallationFunctionality::getInstance()->enableExitButton();
            signalFinish_detected.emit();
            m_nextButton.show();
            m_nextButton.set_sensitive(true);
            m_labelStep.set_label(_("Smartmatic.SAES.GUI.Widgets.VotingMachineInstallationWizardWidget.FinalizeStep"));
            m_progress->setInfoText(N_("Smartmatic.SAES.GUI.Widgets.VotingMachineInstallationWizardWidget.Finalize"));
            m_progress->setImage("ui/wiz_finalize.gif");

            try
            {
                MachineOperationManager::getInstance()->installMachine();
            }
            catch ( ... )
            {
                logger->Debug("ERROR - Install machine");
            }

            break;

        case WIZ_END:
            signalHide_detected.emit();
            break;

        default:
            break;
    }
}

VotingMachineInstallationWizardWidget::~VotingMachineInstallationWizardWidget()
{
    if (m_progress)
    {
        delete m_progress;
        m_progress = NULL;
    }
}

bool VotingMachineInstallationWizardWidget::waitForMemoryMount()
{
    bool result = true;
    std::string domMedia;
    std::string flashMedia;

    do
    {
        domMedia = MemoryPathController::getInstance()->getDomMemoryBasePath();
        flashMedia = MemoryPathController::getInstance()->getFlashMemoryBasePath();

        if (m_usbMonitor != NULL && m_usbMonitor->getNumberOfDevices() != 2)
        {
            result = false;
            break;
        }
        g_usleep(G_USEC_PER_SEC);
    } while (domMedia.empty() || flashMedia.empty());

    return result;
}

void VotingMachineInstallationWizardWidget::checkData()
{
    if (!waitForMemoryMount())
    {
        Glib::signal_timeout().connect_once(
            sigc::mem_fun(
                *this,
                &VotingMachineInstallationWizardWidget::processMemoryRemovedWhileChecking),
            100);
    }
    else
    {
        if (Smartmatic::SAES::Operation::MachineOperationManager::getInstance()->getMachineInitiator()->checkData() &&
            (m_usbMonitor != NULL && m_usbMonitor->getNumberOfDevices() == 2))
        {
            CheckDataManager::getInstance()->startDetection();
            MemoryMessageWindow::getInstance()->start();

            Glib::signal_timeout().connect_once(
                sigc::mem_fun(
                    *this,
                    &VotingMachineInstallationWizardWidget::processCheckDataOk),
                100);
        }
        else
        {
            g_usleep(750000);

            if (m_usbMonitor != NULL && m_usbMonitor->getNumberOfDevices() < 2)
            {
                Glib::signal_timeout().connect_once(
                    sigc::mem_fun(
                        *this,
                        &VotingMachineInstallationWizardWidget::processMemoryRemovedWhileChecking),
                    100);
            }
            else
            {
                Glib::signal_timeout().connect_once(
                    sigc::mem_fun(
                        *this,
                        &VotingMachineInstallationWizardWidget::processCheckDataError),
                    100);
            }
        }
    }
}

void VotingMachineInstallationWizardWidget::processCheckDataOk()
{
    m_progress->setInfoText(N_("Smartmatic.SAES.GUI.Widgets.VotingMachineInstallationWizardWidget.VerifyDataOk"));
    m_progress->setImage("ui/check-flash-structure-success.gif");
    NextStep(2000);
}

void VotingMachineInstallationWizardWidget::processCheckDataError()
{
    if (m_connClickButton.connected())
    {
       m_connClickButton.disconnect();
    }

    SMTTSoundsManager::getInstance()->Stop();
    SMTTSoundsManager::getInstance()->Sound ( "SoundOneMemory" );
    MemoryMessageWindow::getInstance()->showMemoryCheckError();
}

void VotingMachineInstallationWizardWidget::on_buttonNext_clicked()
{
    m_imageCounter++;

    if (m_processTracker != m_stepSequence.end())
    {
        mForwardProcess();
        ++m_processTracker;
    }
}

bool VotingMachineInstallationWizardWidget::RefreshLabel()
{
    m_progress->setInfoText(N_("Smartmatic.SAES.GUI.Widgets.VotingMachineInstallationWizardWidget.ErrorWritingToken"));
    return false;
}

void VotingMachineInstallationWizardWidget::NextStep(int timer)
{
    m_connNextButton.disconnect();
    m_connNextButton = Glib::signal_timeout().connect(
        sigc::mem_fun(
            *this,
            &VotingMachineInstallationWizardWidget::on_buttonNext_clicked_signal),
        timer);
}

bool VotingMachineInstallationWizardWidget::on_buttonNext_clicked_signal()
{
    on_buttonNext_clicked();
    return false;
}

void VotingMachineInstallationWizardWidget::Dispose()
{
    DisconnectConnections();

    if (m_threadCheckData != NULL)
    {
        m_threadCheckData->join();
        m_threadCheckData = NULL;
    }

    if (m_threadSync != NULL)
    {
        m_threadSync->join();
        m_threadSync = NULL;
    }
}

void VotingMachineInstallationWizardWidget::Initialize()
{
    m_processTracker = m_stepSequence.begin();
    m_imageCounter = 0;

    MemoryMessageWindow::getInstance()->stop();

    m_labelStep.set_label(_("Smartmatic.SAES.GUI.Widgets.VotingMachineInstallationWizardWidget.AcceptInstallation"));

    m_progress->setImage("ui/check-flash-structure.gif");

    setImageByStep();
    setAccessibleNameByStep();

    m_progress->setInfoText(N_("Process synchronization, wait one moment."));

    if (!Glib::thread_supported())
    {
        Glib::thread_init();
    }

    if (m_threadSync != NULL)
    {
        m_threadSync->join();
        m_threadSync = NULL;
    }

    CheckDataManager::getInstance()->stopDetection();
    MemoryMessageWindow::getInstance()->stop();

    m_threadSync = Glib::Thread::create(
        sigc::mem_fun(
            *this,
            &VotingMachineInstallationWizardWidget::synchronize),
        true);

    m_usbMonitor = DeviceTypeMonitorPool::getInstance()->getDeviceTypeMonitor(
        Smartmatic::SAES::Common::Configuration::DeviceType::CONFIGURATION_USBSTORAGEMOUNTED);
}

bool VotingMachineInstallationWizardWidget::Extract()
{
    std::string flashMedia = MemoryPathController::getInstance()->getFlashMemoryBasePath();
    std::string domMedia = MemoryPathController::getInstance()->getDomMemoryBasePath();

    if (flashMedia.empty() || domMedia.empty())
    {
        VotingMachineInstallationFunctionality::getInstance()->disableExitButton();
        NextStep ( 500 );
        return false;
    }

    return true;
}

bool VotingMachineInstallationWizardWidget::DetectAllMount()
{
    if (m_usbMonitor != NULL && m_usbMonitor->getNumberOfDevices() == 2)
    {
        NextStep(500);
        return false;
    }

    return true;
}

void VotingMachineInstallationWizardWidget::DisconnectConnections()
{
    if (m_connDetect.connected())
    {
        m_connDetect.disconnect();
    }

    if (m_connExtract.connected())
    {
        m_connExtract.disconnect();
    }

    if (m_connNextButton.connected())
    {
        m_connNextButton.disconnect();
    }
}

void VotingMachineInstallationWizardWidget::synchronize()
{
    logger->Debug("Voting machine installation - Process synchronization." );
    if (!waitForMemoryMount())
    {
        Glib::signal_timeout().connect_once(
            sigc::mem_fun(
                *this,
                &VotingMachineInstallationWizardWidget::processMemoryRemovedWhileSynchronizing),
            500);

    }
    else
    {
        if (MachineOperationManager::getInstance()->synchronize() &&
            MachineOperationManager::getInstance()->getMachineInitiator()->checkData())
        {
            m_signalEndSynchronize.emit();
        }
        else
        {
            g_usleep(750000);

            if (m_usbMonitor != NULL && m_usbMonitor->getNumberOfDevices() == 2)
            {
                CheckDataManager::getInstance()->stopDetection();
                MemoryMessageWindow::getInstance()->stop();

                Glib::signal_timeout().connect_once(
                    sigc::mem_fun(
                        *this,
                        &VotingMachineInstallationWizardWidget::processCheckDataError),
                    500);
            }
            else
            {
                Glib::signal_timeout().connect_once(
                    sigc::mem_fun(*this, &VotingMachineInstallationWizardWidget::processMemoryRemovedWhileSynchronizing),
                    500);
            }
        }
    }
}

void VotingMachineInstallationWizardWidget::endSynchronize()
{
    Glib::signal_timeout().connect_once(
        sigc::mem_fun(
            this,
            &VotingMachineInstallationWizardWidget::onEndSynchronize),
        500);
}

void VotingMachineInstallationWizardWidget::onEndSynchronize()
{
    m_progress->setInfoText(N_("Smartmatic.SAES.GUI.Widgets.VotingMachineInstallationWizardWidget.LabelMessageInitial"));
    on_buttonNext_clicked();
}

void VotingMachineInstallationWizardWidget::setImageByStep()
{
    std::stringstream ss;
    ss << "ui/InstallationWizardStep" << m_imageCounter << ".png";

    try
    {
        m_imageStep.set(ResourcePathProvider::getInstance()->GetResourcePath(PACKAGE_NAME, ss.str()));
    }
    catch ( ... )
    {
    }
}

void VotingMachineInstallationWizardWidget::setAccessibleNameByStep ()
{
    std::stringstream ss;
    ss << m_imageCounter;

    m_imageStep.setAccessibleName("ImageStep" + ss.str());
    m_labelStep.setAccessibleName("Step" + ss.str());
    m_progress->getLabelInfo()->setAccessibleName("InfoStep" + ss.str());
}

void VotingMachineInstallationWizardWidget::processMemoryRemovedError()
{
    if (m_connClickButton.connected())
    {
       m_connClickButton.disconnect();
    }

    MemoryMessageWindow::getInstance()->showNoMemoryError();
}

void VotingMachineInstallationWizardWidget::processMemoryRemovedWhileSynchronizing()
{
    if (m_threadSync != NULL)
    {
        m_threadSync->join();
        m_threadSync = NULL;
    }

    Glib::signal_timeout().connect(
        sigc::mem_fun(*this, &VotingMachineInstallationWizardWidget::shouldResumeSynchronizing),
        500);
}

void VotingMachineInstallationWizardWidget::processMemoryRemovedWhileChecking()
{
    if (m_threadCheckData != NULL)
    {
        m_threadCheckData->join();
        m_threadCheckData = NULL;
    }

    Glib::signal_timeout().connect(
        sigc::mem_fun(*this, &VotingMachineInstallationWizardWidget::shouldResumeChecking),
        500);
}

bool VotingMachineInstallationWizardWidget::shouldResumeSynchronizing()
{
    static unsigned int lastDevices = 2;

    if (m_usbMonitor == NULL)
        return true;

    if (m_usbMonitor->getNumberOfDevices() == 2)
    {
        SMTTSoundsManager::getInstance()->Stop();
        MemoryMessageWindow::getInstance()->QuitDialog();

        lastDevices = 2;

        m_threadSync = Glib::Thread::create(
            sigc::mem_fun(*this, &VotingMachineInstallationWizardWidget::synchronize),
            true);

        return false;
    }
    else if (m_usbMonitor->getNumberOfDevices() == 1 && lastDevices == 2)
    {
        lastDevices = 1;
        MemoryMessageWindow::getInstance()->showReinsertOneMemory();
        SMTTSoundsManager::getInstance()->Stop();
        SMTTSoundsManager::getInstance()->Sound ( "SoundOneMemory" );
    }
    else if (m_usbMonitor->getNumberOfDevices() == 0 && lastDevices >= 1)
    {
        lastDevices = 0;
        MemoryMessageWindow::getInstance()->showReinsertBothMemories();
        SMTTSoundsManager::getInstance()->Stop();
        SMTTSoundsManager::getInstance()->Sound ( "SoundOneMemory" );
    }

    return true;
}

bool VotingMachineInstallationWizardWidget::shouldResumeChecking()
{
    static unsigned int lastDevices = 2;

    if (m_usbMonitor == NULL)
        return true;

    if (m_usbMonitor->getNumberOfDevices() == 2)
    {
        SMTTSoundsManager::getInstance()->Stop();
        MemoryMessageWindow::getInstance()->QuitDialog();
        lastDevices = 2;

        m_threadCheckData = Glib::Thread::create(
            sigc::mem_fun(*this, &VotingMachineInstallationWizardWidget::checkData),
            true);

        return false;
    }
    else if (m_usbMonitor->getNumberOfDevices() == 1 && lastDevices == 2)
    {
        lastDevices = 1;
        MemoryMessageWindow::getInstance()->showReinsertOneMemory();
        SMTTSoundsManager::getInstance()->Stop();
        SMTTSoundsManager::getInstance()->Sound ( "SoundOneMemory" );
    }
    else if (m_usbMonitor->getNumberOfDevices() == 0 && lastDevices >= 1)
    {
        lastDevices = 0;
        MemoryMessageWindow::getInstance()->showReinsertBothMemories();
        SMTTSoundsManager::getInstance()->Stop();
        SMTTSoundsManager::getInstance()->Sound ( "SoundOneMemory" );
    }

    return true;
}
