/**
 * @file WizardWidget.h
 * @brief Header wizard widget class
 * @date 28/05/2011
 * @author Gerardo Hernandez
 */

#ifndef WIZARDWIDGET_H_
#define WIZARDWIDGET_H_

#include <vector>
#include <gtkmm.h>
#include <Gui/Widgets/Wizard/WizardConfiguration.h>
#include <Gui/Widgets/Wizard/WizardTimeline.h>
#include <Gui/Widgets/Wizard/WizardToolkit.h>
#include "Widgets/keyword-navigation-interface.h"
#include "Widgets/smartmatic-button.h"
#include <Widgets/SmartmaticBasicFrame.hxx>
#include <Widgets/SmartmaticBasicVBox.hxx>
#include <Widgets/SmartmaticBasicHBox.hxx>
#include <Widgets/SmartmaticBasicLabel.hxx>
#include <Widgets/SmartmaticBasicEventBox.hxx>
#include <Widgets/SmartmaticBasicTable.hxx>

namespace Smartmatic
{
    namespace SAES
    {
        namespace GUI
        {
            namespace Widgets
            {
                /**
                 * @class	WizardWidget
                 * @brief	Wizard widget.
                 * @author	Youssef.tourkmani
                 * @date	25/10/2012
                 */
                class WizardWidget : public Smartmatic::GUI::Widgets::SmartmaticBasicFrame,
                    public Smartmatic::GUI::Widgets::keywordNavigationInterface
                {
                    public:

                        /**
                         * @fn	WizardWidget::WizardWidget(std::vector<WizardWidgetConfiguration> configuration,
                         * 			std::string currentState,
                         * 			std::string languagePackageA = "");
                         * @brief	Constructor.
                         * @author	Youssef.tourkmani
                         * @date	25/10/2012
                         * @param	configuration				The configuration.
                         * @param	currentState				The current state.
                         */
                        WizardWidget ( std::vector<WizardConfiguration> configuration, std::string currentState );

                        /**
                         * @fn	virtual WizardWidget::~WizardWidget();
                         * @brief	Destructor.
                         * @author	Youssef.tourkmani
                         * @date	25/10/2012
                         */
                        virtual ~WizardWidget();

                        /**
                         * @typedef	sigc::signal<void> show_menu_sigc
                         * @brief	Defines an alias representing the show menu sigc.
                         */
                        typedef sigc::signal<void> show_menu_sigc;

                        /**
                         * @fn	show_menu_sigc WizardWidget::ShowMenuSigc()
                         * @brief	Shows the menu sigc.
                         * @author	Youssef.tourkmani
                         * @date	25/10/2012
                         * @return	.
                         */
                        show_menu_sigc ShowMenuSigc()
                        {
                            return m_show_menu_sigc;
                        };

                        /**
                         * @typedef	sigc::signal<void> functionality_executed_sigc
                         * @brief	Defines an alias representing the functionality executed sigc.
                         */
                        typedef sigc::signal<void> functionality_executed_sigc;

                        /**
                         * @fn	functionality_executed_sigc WizardWidget::FunctionalityExecuteSigc()
                         * @brief	Gets the functionality execute sigc.
                         * @author	Youssef.tourkmani
                         * @date	25/10/2012
                         * @return	.
                         */
                        functionality_executed_sigc FunctionalityExecuteSigc()
                        {
                            return m_functionality_executed;
                        };


                        /**
                         * @fn	void WizardWidget::SetKey(Smartmatic::GUI::Widgets::KeysActionsCode value);
                         * @brief	Sets a key.
                         * @author	Youssef.tourkmani
                         * @date	25/10/2012
                         * @param	value	The value.
                         */
                        void SetKey ( Smartmatic::GUI::Widgets::KeysActionsCode value );

                        /**
                         * @fn	void WizardWidget::RefreshLanguage();
                         * @brief	Refresh language.
                         * @author	Youssef.tourkmani
                         * @date	25/10/2012
                         */
                        void RefreshLanguage();

                    protected:
                        show_menu_sigc m_show_menu_sigc;	///< The show menu sigc
                        functionality_executed_sigc m_functionality_executed; ///< The functionality executed

                    private:
                        std::vector<WizardConfiguration> wizardWidgetConfiguration; ///< The wizard widget configuration
                        Smartmatic::GUI::Widgets::SmartmaticBasicVBox * boxPrincipal;	///< The box principal
                        WizardToolkit * toolbox; ///< The toolbox
                        WizardTimeline * timeline; ///< The timeline
                        Smartmatic::GUI::Widgets::SmartmaticBasicHBox * boxCentral;	///< The box central
                        Smartmatic::GUI::Widgets::SmartmaticBasicVBox * principalButtonBox;	///< The principal button box
                        Smartmatic::GUI::Widgets::SmartmaticBasicLabel * helpTitle;	///< The help title
                        Smartmatic::GUI::Widgets::SmartmaticBasicLabel * helpDetail; ///< The help detail
                        Smartmatic::GUI::Widgets::SmartmaticBasicEventBox * eventBoxTimeLine;	///< The event box time line
                        Smartmatic::GUI::Widgets::SmartmaticBasicEventBox * eventBoxPrincipalButtons;	///< The event box principal buttons
                        Smartmatic::GUI::Widgets::SmartmaticBasicEventBox * eventBoxToolbox; ///< The event box toolbox
                        Smartmatic::GUI::Widgets::SmartmaticBasicEventBox * eventBoxHelpPanel; ///< The event box help panel
                        Smartmatic::GUI::Widgets::SmartmaticBasicTable * principalButtonTable; ///< The principal button table
                        Smartmatic::GUI::Widgets::SmartmaticBasicVBox * helpBox; ///< The hepl box
                        int buttonIndex;	///< Zero-based index of the button
                        std::string state; ///< The state

                        sigc::connection toolBoxConnection; ///< The tool box connection
                        WizardConfiguration currentWizardConfiguration; ///< The current wizard configuration

                        /**
                         * @fn	std::string WizardWidget::GetButtonName( const std::string& suffix );
                         * @brief	Gets a button name.
                         * @author	Luiggi.mendez
                         * @date	27/04/2011
                         * @param	suffix	The suffix.
                         * @return	The button name.
                         */

                        std::string GetButtonName ( const std::string& suffix );

                        /**
                         * @fn	void WizardWidget::on_button_clicked(Smartmatic::GUI::Widgets::SmartmaticButton *widget, MenuOption _option);
                         * @brief	Executes the button clicked action.
                         * @author	Luiggi.mendez
                         * @date	27/04/2011
                         * @param	widget 	If non-null, the widget.
                         * @param	_option	The option.
                         */
                        void on_button_clicked ( Smartmatic::GUI::Widgets::SmartmaticButton * widget, MenuOption _option );

                        /**
                         * @fn	void WizardWidget::on_toolbox_element_clicked(MenuOption option);
                         * @brief	Handles toolbox element clicked signals.
                         * @author	Youssef.tourkmani
                         * @date	25/10/2012
                         * @param	option	The option.
                         */
                        void on_toolbox_element_clicked ( MenuOption option );

                        std::map<Smartmatic::GUI::Widgets::SmartmaticButton *, MenuOption> optionsBybuttonList; ///< List of options by correspond buttons
                };
            }
        }
    }
}

#endif /* WIZARDWIDGET_H_ */
