/**
 * @file	ballot-option-selection-widget.cc
 * @brief	ballot option selection widget class.
 * @author	Youssef.tourkmani
 * @date	01/11/2012
 */


#include "AbstractFactoryWidget.hxx"
#include "ballot-option-selection-widget.h"
#include "ballot-option-widget.h"
#include "BallotOptionTableWidget.hxx"
#include "complex-ballot-option-widget.h"
#include "Configuration/s-a-e-s-g-u-i-conf-manager.h"
#include "Configuration/s-m-t-t-g-u-i-conf-manager.h"
#include "FactoryMethodWidget.hxx"
#include "Resources/election-string-resources-manager.h"
#include "Resources/election-string-resources-manager.h"
#include "System/Sounds/smartmatic-sounds-manager.h"
#include "Voting/ElectoralConfiguration.hxx"
#include <Configuration/BasicWindowConfManager.hxx>
#include <Gui/Widgets/Voting/ContestLayout.hxx>
#include <Log/ISMTTLog.h>
#include <Log/SMTTLogManager.h>
#include <Operation/FormatLabel.hxx>
#include <Operation/MachineOperationManager.hxx>
#include <Operation/Voting/VotingOperation.hxx>
#include <System/Utils/StringUtils.hxx>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#include <libintl.h>
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

#define N_(String) (String) ///<macro for gettext not translate

using namespace Gtk;
using namespace Smartmatic::SAES::Configuration;
using namespace Smartmatic::SAES::GUI::Configuration;
using namespace Smartmatic::SAES::GUI::Widgets;
using namespace Smartmatic::SAES::GUI::Widgets::Voting;
using namespace Smartmatic::GUI::Widgets;
using namespace Smartmatic::SAES::Voting::Election;
using namespace Smartmatic::SAES::Operation::Voting;
using namespace Smartmatic::SAES::Resources;
using namespace Smartmatic::SAES::GUI::Operation::Voting;
using namespace Smartmatic::SAES::GUI::Operation;
using namespace Smartmatic::System::Utils;
using namespace Smartmatic::Log;
using namespace Smartmatic::GUI::Configuration;

static const int ELEMENTS_HEADER = 2;

ISMTTLog * BallotOptionSelectionWidget::logger = SMTTLogManager::GetLogger ( "Smartmatic.SAES.GUI.Widgets.BallotOptionSelectionWidget", GETTEXT_PACKAGE );

BallotOptionSelectionWidget::BallotOptionSelectionWidget ( Contest * contest )
{
    minimumAllowed = contest->getMin_votes();
    maximumAllowed = contest->getMax_votes();

    useInfoVoting = false;
    useWarningVoting = false;

    m_widgetEnabled = true;
    tallyPosition = 0;
    ballotOptionTableWidget = NULL;
    ballotOptionButtonWidget = NULL;
    infoFrame = NULL;

    showBackButton = true;
    SoundBallotS = SAESGUIConfigurationManager::getInstance()->getBallotOptionSelectSound();
    static BallotOptionSelectionConfiguration config =
        SAESGUIConfigurationManager::getInstance()->GetBallotOptionSelectionConfiguration();
    HideStandInLabelIfEmpty = config.getHideStandInLabelIfEmpty();
    this->contest = contest;

    listBallotsSelected = 0;
    principalBallotsSelected = 0;
    standInBallotsSelected = 0;
    totalPrincipalBallots = 0;
    totalStandInBallots = 0;
    ballotOptionListWidget = NULL;
    m_isChildCustomAttribute = false;

    int x;

    SMTTGUIConfigurationManager::getInstance()->getPaddings ( x, padding );

    topHeight = config.getFrameTopHeight();
    listHeight = config.getFrameListHeight();
    infoHeight = config.getFrameInfoHeight();
    bottomHeight = config.getFrameBottomHeight();

    showInfoFrame = infoHeight > 0;

    listFrame = manage ( new SmartmaticBasicEventBox() );
    listFrame->show();
    listFrame->set_size_request ( -1, listHeight );

    ballotOptionButtonWidget = FactoryMethodWidget::createBallotOptionButtonWidget();
    ballotOptionButtonWidget->update();
    ballotOptionTableWidget = AbstractFactoryWidget::createBallotOptionTableWidget();

    titleLabel = manage ( new SmartmaticBasicLabel() );
    titleLabel->show();
    titleLabel->set_size_request ( -1, topHeight/ELEMENTS_HEADER );

    subtitleLabel = manage ( new SmartmaticBasicLabel() );
    subtitleLabel->show();
    subtitleLabel->set_size_request ( -1, topHeight/ELEMENTS_HEADER );

    Smartmatic::GUI::Configuration::LabelConfiguration titleConfig = getTitleLabelConfig();
    Smartmatic::GUI::Configuration::LabelConfiguration::SetLabelConfiguration ( titleLabel, titleConfig );

    Smartmatic::GUI::Configuration::LabelConfiguration subtitleConfig = getSubtitleLabelConfig();
    Smartmatic::GUI::Configuration::LabelConfiguration::SetLabelConfiguration ( subtitleLabel, subtitleConfig );

    bottomFrame = manage ( new SmartmaticBasicEventBox() );
    bottomFrame->show();
    bottomFrame->add ( *ballotOptionButtonWidget );
    bottomFrame->set_size_request ( -1, bottomHeight );

    clientAreaFrame = manage ( new SmartmaticBasicEventBox() );
    clientAreaFrame->show();

    infoFrame = manage ( new SmartmaticBasicEventBox() );
    infoFrame->show();
    infoFrame->set_size_request ( -1, infoHeight );

    externalBox = new SmartmaticBasicVBox ( false, 0 );
    externalBox->show();

    infoVoting = new VotingInfoWidget ( minimumAllowed, maximumAllowed );
    warningVoting = new WarningVotingInfoWidget ( maximumAllowed );

    Gtk::HSeparator * separator = manage ( new Gtk::HSeparator() );
    separator->set_size_request ( -1, padding );

    externalBox->pack_start ( *titleLabel, false, false );
    externalBox->pack_start ( *subtitleLabel, false, false );
    externalBox->pack_start ( *separator, false, false );
    externalBox->pack_start ( *listFrame, false, false );
    externalBox->pack_start ( *clientAreaFrame, true, true, padding );
    externalBox->pack_start ( *infoFrame, false, false );
    externalBox->pack_end ( *bottomFrame, false, false );

    setWidget ( externalBox );

    setTitle ( false );

    setAccessibleName ( "BallotOptionSelection" );

    listFrame->setAccessibleName ( "BallotOptionSelectionHeader" );
    titleLabel->setAccessibleName ( "BallotOptionSelectionTitle" );
    subtitleLabel->setAccessibleName ( "BallotOptionSelectionSubTitle" );
    clientAreaFrame->setAccessibleName ( "BallotOptionSelectionContainer" );
    infoFrame->setAccessibleName ( "BallotOptionVotingInfo" );
    bottomFrame->setAccessibleName ( "BallotOptionSelectionBottom" );
    externalBox->setAccessibleName ( "BallotOptionSelectionContainer" );
}

std::string BallotOptionSelectionWidget::getDistrictDescription()
{
    const std::string descriptionKey = contest->getDistrict().getDescription_key();

    return ElectionStringResourcesManager::getInstance()->getResource(descriptionKey);
}

std::string BallotOptionSelectionWidget::getHelp()
{
    const std::string helpKey = contest->getHelp_key();

    return ElectionStringResourcesManager::getInstance()->getResource(helpKey);
}

void BallotOptionSelectionWidget::buildHeader()
{
  const Smartmatic::SAES::Voting::Election::Contest_class * contestClass =
     Smartmatic::SAES::Operation::MachineOperationManager::getInstance()->getElectoralConfiguration()
    ->getContestClassByCode ( contest->getContest_class_code() );
  std::string contestClassName = ElectionStringResourcesManager::getInstance()->getResource(contestClass->getName_key());
  std::string subtitleValue    = "";

  FormatLabel::checkComposeString(contestClassName);
  titleLabel->set_text(contestClassName);

  if ( m_isChildCustomAttribute )
  {
    std::string contestHelp = getHelp();

    FormatLabel::checkComposeString(contestHelp);
    logger->Debug ("Showing Help Key");

    if ( StringUtils::toLowerUtf8(contestClassName) != StringUtils::toLowerUtf8(contestHelp) && !contestHelp.empty() )
      subtitleValue = contestHelp;
  }
  else
  {
    std::string districtDescription = getDistrictDescription();

    FormatLabel::checkComposeString(districtDescription);
    logger->Debug ("Showing District Description");

    if ( StringUtils::toLowerUtf8(contestClassName) != StringUtils::toLowerUtf8(districtDescription) && !districtDescription.empty() )
      subtitleValue = districtDescription;
  }

  subtitleLabel->set_text(subtitleValue);
}

void BallotOptionSelectionWidget::SetChildCustomAttribute ( bool value )
{
    m_isChildCustomAttribute = value;
}

void BallotOptionSelectionWidget::RefreshLabels()
{
    buildHeader();
    ballotOptionTableWidget->update();
    ballotOptionButtonWidget->update();
    infoVoting->refresh();
    warningVoting->refresh();
}

void BallotOptionSelectionWidget::ClearVisualSelection()
{
    ballotOptionListWidget->clear ();
    ballotOptionTableWidget->clearSelections();

    listBallotsSelected = 0;
    principalBallotsSelected = 0;
    standInBallotsSelected = 0;

    useInfoVoting = false;
    useWarningVoting = false;

    CheckButtonsLogic();
}

int BallotOptionSelectionWidget::GetTotalBallotsDisplayed()
{
    return totalPrincipalBallots + totalStandInBallots;
}

void BallotOptionSelectionWidget::SetWidgetEnabled ( bool enabled )
{
    m_widgetEnabled = enabled;
}

void BallotOptionSelectionWidget::Initialize ( std::list<Eligible_option * > & ballotList,
        std::list<Eligible_option * > & principalBallotList,
        std::list<Eligible_option * > & standInBallotList,
        Smartmatic::SAES::Runtime::Environment::SaesDirectories & saesDir )
{
    ContestLayout layout ( contest );
    static BallotOptionSelectionConfiguration config = SAESGUIConfigurationManager::getInstance()->GetBallotOptionSelectionConfiguration();

    totalPrincipalBallots = ( int ) principalBallotList.size();
    totalStandInBallots = ( int ) standInBallotList.size();

    clientAreaFrame->set_border_width ( 0 );
    clientAreaFrame->add ( *ballotOptionTableWidget );

    bool enableClick = config.getEnableClick();
    bool showBorder = SAESGUIConfigurationManager::getInstance()->GetBallotOptionWidgetConfiguration().getShowFrame();

    if ( !ballotList.empty() )
    {
        ballotOptionListWidget = manage ( new ComplexBallotOptionWidget ( * ( * ( ballotList.begin() ) ), saesDir, enableClick, showBorder ) );
        ballotOptionListWidget->setAccessibleName ( "ListBox" );
        ballotOptionListWidget->getSignalClick().connect ( sigc::mem_fun ( this, &BallotOptionSelectionWidget::onBoxClicked ) );
        ballotOptionListWidget->setSize ( 100, 10 );

        if ( config.getShowImageInList() )
        {
            float listRelation = config.getImageRelation();
            ballotOptionListWidget->forceShowImage ( listHeight, listRelation );
        }

        ballotOptionListWidget->show();
        listFrame->add ( *ballotOptionListWidget );
        tallyPosition = ( * ( ballotList.begin() ) )->getBallot_options().getBallot_option().begin()->getTally_position();
    }

    std::list<Smartmatic::SAES::Voting::Election::Eligible_option * >::iterator it;	///< The iterator

    int count = 0;

    //ballot_option principal
    for ( count = 1, it = principalBallotList.begin(); it != principalBallotList.end(); ++it, count++ )
    {
        BallotOptionWidget * ballot = manage(new BallotOptionWidget ( * ( *it ), saesDir, true, enableClick, showBorder, &layout ));
        ballot->setSize ( 100, 10 );
        ballot->setAccessibleName ( "CandidateBox" + StringUtils::intToString ( count ) );
        ballotOptionTableWidget->addPrincipal ( ballot );
    }

    //ballot_option substitute
    for ( count = 1, it = standInBallotList.begin(); it != standInBallotList.end(); ++it, count++ )
    {
        BallotOptionWidget * ballot = manage(new BallotOptionWidget ( * ( *it ), saesDir, true, enableClick, showBorder, &layout ));
        ballot->setSize ( 100, 10 );
        ballot->setAccessibleName ( "SubstituteBox" + StringUtils::intToString ( count ) );
        ballotOptionTableWidget->addStandIn ( ballot );
    }

    if ( enableClick )
    {
        ballotOptionTableWidget->getTableWidget()->getMinSignal().connect ( sigc::mem_fun ( this, &BallotOptionSelectionWidget::onMinSignal ) );
        ballotOptionTableWidget->getTableWidget()->getMaxSignal().connect ( sigc::mem_fun ( this, &BallotOptionSelectionWidget::onMaxSignal ) );
        ballotOptionTableWidget->getTableWidget()->getBoxSignalClick().connect ( sigc::mem_fun ( this, &BallotOptionSelectionWidget::onBoxClicked ) );

        ballotOptionButtonWidget->getSignalClearButtonClicked().connect ( sigc::mem_fun ( this, &BallotOptionSelectionWidget::on_clearButton_clicked ) );
        ballotOptionButtonWidget->getSignalAcceptButtonClicked().connect ( sigc::mem_fun ( this, &BallotOptionSelectionWidget::on_acceptButton_clicked ) );
        ballotOptionButtonWidget->getSignalBackButtonClicked().connect ( sigc::mem_fun ( this, &BallotOptionSelectionWidget::on_backButton_clicked ) );
    }

    int totalW, totalH;


    SMTTGUIConfigurationManager::getInstance()->getCenterSize ( totalW, totalH );
    totalH -= topHeight + listHeight + infoHeight + bottomHeight + 2 * config.getBallotsMargin() + 4 * padding;
    totalH -= config.getCandidatesButtonsSpacing();
    totalW -= config.getFixBorder();

    ballotOptionTableWidget->getTableWidget()->set_size_request ( totalW, totalH );
    ballotOptionTableWidget->fill ( contest, totalW, totalH );
    show();
}

void BallotOptionSelectionWidget::refresh()
{
    RefreshLabels();
    CheckButtonsLogic();
}

void BallotOptionSelectionWidget::CheckButtonsLogic()
{
    static BallotOptionSelectionConfiguration config =
        SAESGUIConfigurationManager::getInstance()->GetBallotOptionSelectionConfiguration();

    bool acceptCondition = minimumAllowed == 0 || ( principalBallotsSelected + standInBallotsSelected + listBallotsSelected) >= minimumAllowed;

    if ( !useInfoVoting && !useWarningVoting && showInfoFrame )
    {
        useInfoVoting = true;

        infoFrame->remove();
        infoFrame->add ( * infoVoting );
        infoFrame->show();
        infoFrame->show_all_children();
    }

    bool showAcceptAndCancelButtons = principalBallotsSelected + standInBallotsSelected  + listBallotsSelected > 0;

    infoVoting->showFooterInfo ( showAcceptAndCancelButtons && acceptCondition );
    infoVoting->refresh();

    if ( showAcceptAndCancelButtons )
    {
        ButtonsLogicShowHide ( true, true, config.getAllowBackWithBallotSelected() );
        ballotOptionButtonWidget->setSensitiveButton ( true, acceptCondition, true, true );
    }
    else
    {
        ButtonsLogicShowHide ( false, false, true );
    }
}

void BallotOptionSelectionWidget::ButtonsLogicShowHide ( bool clear, bool accept, bool back )
{
    ballotOptionButtonWidget->clearSignals();
    ballotOptionButtonWidget->showButton ( clear, accept, back );
    m_skipAcceptSignal = !accept;
}

void BallotOptionSelectionWidget::on_clearButton_clicked()
{
    if ( m_widgetEnabled == false )
    {
        return;
    }

    ClearVisualSelection();

    ballotOptionButtonWidget->clearSignals();

    m_clear_clicked.emit();
}

void BallotOptionSelectionWidget::on_acceptButton_clicked()
{
    if ( m_widgetEnabled == false )
    {
        return;
    }

    if ( m_skipAcceptSignal )
    {
        return;
    }

    if ( m_accept_clicked.empty() == false )
    {
        m_widgetEnabled = false;
    }

    m_accept_clicked.emit();
}

void BallotOptionSelectionWidget::on_backButton_clicked()
{
    if ( m_widgetEnabled == false )
    {
        return;
    }

    if ( m_back_clicked.empty() == false )
    {
        m_widgetEnabled = false;
    }

    m_back_clicked.emit();
    ballotOptionButtonWidget->clearSignals();
}

BallotOptionSelectionWidget::type_accept_clicked BallotOptionSelectionWidget::accept_clicked()
{
    return m_accept_clicked;
}

BallotOptionSelectionWidget::type_back_clicked BallotOptionSelectionWidget::back_clicked()
{
    return m_back_clicked;
}

BallotOptionSelectionWidget::type_clear_clicked BallotOptionSelectionWidget::clear_clicked()
{
    return m_clear_clicked;
}

BallotOptionSelectionWidget::type_ballotOption_clicked BallotOptionSelectionWidget::ballotOption_clicked()
{
    return m_ballotOption_clicked;
}

Smartmatic::SAES::Voting::Election::Contest * BallotOptionSelectionWidget::getContest()
{
    return contest;
}

BallotOptionSelectionWidget::~BallotOptionSelectionWidget()
{
    this->remove();

    if ( ballotOptionButtonWidget )
    {
        delete ballotOptionButtonWidget;
        ballotOptionButtonWidget = NULL;
    }

    if ( ballotOptionTableWidget )
    {
        delete ballotOptionTableWidget;
        ballotOptionTableWidget = NULL;
    }

    delete infoVoting;
    delete warningVoting;
    delete externalBox;
}

Smartmatic::GUI::Configuration::LabelConfiguration BallotOptionSelectionWidget::getTitleLabelConfig()
{
    return SAESGUIConfigurationManager::getInstance()->GetBallotOptionSelectionWindowTitleLabelConfiguration();
}

Smartmatic::GUI::Configuration::LabelConfiguration BallotOptionSelectionWidget::getSubtitleLabelConfig()
{
    return SAESGUIConfigurationManager::getInstance()->GetBallotOptionSelectionWindowSubtitleLabelConfiguration();
}

Smartmatic::GUI::Configuration::LabelConfiguration BallotOptionSelectionWidget::getBottomLabelConfig()
{
    return SAESGUIConfigurationManager::getInstance()->GetBallotOptionSelectionWindowBottomLabelConfiguration();
}

std::string BallotOptionSelectionWidget::getBottomText()
{
    std::string bottom;
    Glib::ustring s_BallotFooterFormat;

    switch ( VotingOperation::getVoteMode() )
    {
        case Smartmatic::SAES::Voting::Vote::VoteMode::VOTE_S:
        case Smartmatic::SAES::Voting::Vote::VoteMode::VOTE_O:
        default:
            s_BallotFooterFormat = _ ( "Smartmatic.SAES.GUI.Windows.VotingWindow.BallotFooterFormat" );
            break;

        case Smartmatic::SAES::Voting::Vote::VoteMode::VOTE_D:
            s_BallotFooterFormat = _ ( "Smartmatic.SAES.GUI.Windows.VotingWindow.BallotFooterFormat.Demo" );
            break;
    }

    bottom = FormatLabel::contestText ( *contest, s_BallotFooterFormat );
    FormatLabel::checkComposeString ( bottom );
    return bottom;
}

void BallotOptionSelectionWidget::onMinSignal ( unsigned int minValue, unsigned int size )
{
    if ( m_widgetEnabled == false )
    {
        return;
    }

    std::stringstream ss;
    ss << "Min value: ";
    ss << minValue << " ";
    ss << "Selected: ";
    ss << size;

    logger->Debug ( ss.str() );
}

void BallotOptionSelectionWidget::onMaxSignal ( unsigned int maxValue )
{
    if ( m_widgetEnabled == false )
    {
        return;
    }

    if ( !useWarningVoting && showInfoFrame )
    {
        useWarningVoting = true;
        useInfoVoting = false;

        infoFrame->remove();
        infoFrame->add ( * warningVoting );
        infoFrame->show();
        infoFrame->show_all_children();

        warningVoting->refresh();
    }

    std::stringstream ss;
    ss << "Max value: ";
    ss << maxValue;

    logger->Debug ( ss.str() );
}

void BallotOptionSelectionWidget::onBoxClicked ( Smartmatic::GUI::Widgets::BoxSelectionWidget * widget )
{
    if ( m_widgetEnabled == false )
    {
        return;
    }

    BallotOptionInfoWidget * ballotWidget = static_cast < BallotOptionInfoWidget * > ( widget );

    switch ( ballotWidget->getCategory() )
    {
        default:
        case Category::ELECTION_LIST:
            listBallotsSelected += ( ballotWidget->getStatus() ? 1 : -1 );
            break;

        case Category::ELECTION_PRINCIPAL:
            principalBallotsSelected += ( ballotWidget->getStatus() ? 1 : -1 );
            break;

        case Category::ELECTION_STAND_IN:
            standInBallotsSelected += ( ballotWidget->getStatus() ? 1 : -1 );
            break;
    }

    useWarningVoting = false;

    CheckButtonsLogic();

    m_ballotOption_clicked.emit ( ballotWidget->getCode(), ballotWidget->getStatus() );
}
