/**
 * @file	change-language-widget.cc
 * @brief	change language widget class.
 * @author	Youssef.tourkmani
 * @date	01/11/2012
 */


#include "change-language-widget.h"
#include <Configuration/MachineModelEnum.hxx>
#include "Voting/set-language-functionality.h"
#include "Environment/Configurations/base-language-controller.h"
#include "Voting/Election/election-language-schema.hxx"
#include "System/IO/File.h"
#include "System/IO/Path.h"
#include "System/IO/Directory.h"
#include "Configuration/s-a-e-s-g-u-i-conf-manager.h"
#include "Voting/Election/ElectionInstalationLanguages.h"
#include <Log/ISMTTLog.h>
#include <Log/SMTTLogManager.h>
#include <Configuration/s-a-e-s-functionality-conf-manager.h>
#include <Gui/Widgets/Menu/MenuWidget.hxx>
#include <System/Utils/StringUtils.hxx>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#include <libintl.h>
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

#define N_(String) (String) ///<macro for gettext not translate

using namespace Gtk;
using namespace Smartmatic::Configuration;
using namespace Smartmatic::Environment;
using namespace Smartmatic::GUI::Widgets;
using namespace Smartmatic::SAES::GUI::Widgets;
using namespace Smartmatic::SAES::Runtime::Environment;
using namespace Smartmatic::SAES::Voting::Election;
using namespace Smartmatic::System::IO;
using namespace Smartmatic::SAES::GUI::Configuration;
using namespace Smartmatic::System::Exception;
using namespace Smartmatic::Log;
using namespace Smartmatic::System::Utils;

ISMTTLog * ChangeLanguageWidget::logger = SMTTLogManager::GetLogger ( "Smartmatic.SAES.GUI.Functionality.ChangeLanguageWidget", GETTEXT_PACKAGE );

ChangeLanguageWidget::ChangeLanguageWidget()
    : StepWidget ()
    , saveFile(true)
    , showMessage(false)
    , labelMessage(NULL)
    , imageWidget(NULL)
    , clientAreaAlignment(NULL)
    , directories(NULL)
{
    this->setAccessibleName ( "ChangeLanguageWidget" );
}

ChangeLanguageWidget::~ChangeLanguageWidget()
{
}

void ChangeLanguageWidget::init ( Smartmatic::SAES::Runtime::Environment::SaesDirectories * directories )
{
    this->directories = directories;
}

void ChangeLanguageWidget::SetShowMessage ( bool shoMessageA )
{
    this->showMessage = shoMessageA;
}

void ChangeLanguageWidget::on_image_selected ( const std::string& imageCode )
{
    MenuWidget::reloadMenu = true;
    std::list<Smartmatic::SAES::Voting::Election::Language> listLang =
        Smartmatic::SAES::Voting::Election::ElectionInstalationLanguages::getInstance()->getLanguages();
    std::list<Smartmatic::SAES::Voting::Election::Language>::iterator it;

    for ( it = listLang.begin(); it != listLang.end(); ++it )
    {
        if ( ( it->getCode() == imageCode ) )
        {
            Smartmatic::SAES::Voting::Election::ElectionInstalationLanguages::getInstance()->setInstallationLanguages ( *it );
            Smartmatic::SAES::Voting::SetLanguageFunctionality::setNewVotingLanguageByLanguageCountry ( it->getLanguage(), it->getCountry() );

            if ( saveFile )
            {
                try
                {
                    Smartmatic::SAES::Voting::Election::ElectionInstalationLanguages::getInstance()->persistLanguage();
                }
                catch ( CryptoException & ex )
                {
                    std::stringstream ss;
                    ss  << "FATAL - " << __func__ << " " << ex.what();
                    logger->Debug ( ss.str() );
                }
                catch ( SignedException & ex )
                {
                    std::stringstream ss;
                    ss  << "FATAL - " << __func__ << " " << ex.what();
                    logger->Debug ( ss.str() );
                }
                catch ( XmlException & ex )
                {
                    std::stringstream ss;
                    ss  << "FATAL - " << __func__ << " " << ex.what();
                    logger->Debug ( ss.str() );
                }
                catch ( ... )
                {
                    std::stringstream ss;
                    ss  << "FATAL - " << __func__;
                    logger->Debug ( ss.str() );
                }
            }

            if ( labelMessage )
            {
                labelMessage->set_text ( _ ( "Smartmatic.SAES.GUI.Widgets.ChangeLanguageWidget.MessageKey" ) );
            }

            logger->Audit ( N_ ( "Smartmatic.SAES.GUI.Widgets.ChangeLanguageWidget.LanguageChanged" ) );
            m_language_changed.emit();
            break;
        }
    }
}

bool ChangeLanguageWidget::loadDefaultLanguage()
{
    std::list<Smartmatic::SAES::Voting::Election::Language> listLang =
        Smartmatic::SAES::Voting::Election::ElectionInstalationLanguages::getInstance()->getLanguages();
    std::list<Smartmatic::SAES::Voting::Election::Language>::iterator it;

    if ( listLang.size() == 1 )
    {
        on_image_selected ( ( *listLang.begin() ).getCode() );
        return true;
    }

    for ( it = listLang.begin(); it != listLang.end(); ++it )
    {
        if ( it->getIs_default() )
        {
            Smartmatic::SAES::Voting::SetLanguageFunctionality::setNewVotingLanguageByLanguageCountry ( it->getLanguage(), it->getCountry() );
            return true;
        }
    }

    return false;
}

void ChangeLanguageWidget::initialize ( std::list<Smartmatic::SAES::Voting::Election::Language> & listLang, bool showOnOneLanguage )
{
    this->remove();
    std::vector<std::string> imagesPath;
    std::vector<std::string> imagesCode;
    std::map < std::string, std::string > mapLanguageCode;

    std::string imagesFolder;
    std::string folderDom = directories->getLanguageImageDirectoryFullPath ( Smartmatic::SAES::Common::Configuration::BasePathType::CONFIGURATION_DOM );
    std::string folderFlash = directories->getLanguageImageDirectoryFullPath ( Smartmatic::SAES::Common::Configuration::BasePathType::CONFIGURATION_FLASH );

    if ( Smartmatic::System::IO::Directory::getInstance()->DirectoryExists ( folderDom ) )
    {
        imagesFolder = folderDom;
        logger->Debug ( "Load languages in DOM" );
    }
    else if ( Smartmatic::System::IO::Directory::getInstance()->DirectoryExists ( folderFlash ) )
    {
        imagesFolder = folderFlash;
        logger->Debug ( "Load languages in Flash" );
    }
    else
    {
        imagesFolder = directories->getLanguageImageInstalationDirectoryFullPath();
    }

    if ( !listLang.empty() )
    {
        std::list<Smartmatic::SAES::Voting::Election::Language>::iterator it;

        if ( listLang.size() == 1 && !showOnOneLanguage )
        {
            on_image_selected ( ( *listLang.begin() ).getCode() );
            return;
        }

        for ( it = listLang.begin(); it != listLang.end(); ++it )
        {
            Smartmatic::SAES::Voting::Election::Language lang = *it;
            std::string image = Path::Combine ( imagesFolder , lang.getLanguage() + "_" + lang.getCountry() + ".img" );
            imagesPath.push_back ( image );
            imagesCode.push_back ( lang.getCode() );
            mapLanguageCode [ lang.getCode() ] = StringUtils::toUpper ( lang.getLanguage() );
        }
    }
    else
    {
        Gtk::Image * image = manage ( new Gtk::Image() );
        std::string imageFile = ResourcePathProvider::getInstance()->GetResourcePath ( PACKAGE_NAME, "ui/shut-down.png" );
        image->set ( Gdk::PixbufAnimation::create_from_file ( imageFile ) );
        add ( *image );
        show_all_children ();
        return;
    }

    imageWidget = manage ( new ImageSelectionWidget ( imagesPath, imagesCode ) );
    imageWidget->setAccessibleName ( "ImageSelectionLanguageWidget" );
    imageWidget->clicked().connect ( sigc::mem_fun ( *this, &ChangeLanguageWidget::on_image_selected ) );

    std::vector<Smartmatic::GUI::Widgets::SmartmaticBasicEventBox *> & eventBoxs ( imageWidget->getEventBoxs() );
    std::vector<Smartmatic::GUI::Widgets::SmartmaticBasicEventBox *>::iterator eit;

    int pos = 0;

    for ( eit = eventBoxs.begin();
            eit != eventBoxs.end();
            ++eit, pos++ )
    {
        SmartmaticBasicEventBox * box = *eit;
        box->setAccessibleName ( "LanguageBox" + mapLanguageCode [ imagesCode[pos] ] );
    }

    clientAreaAlignment = manage ( new Gtk::Alignment() );
    clientAreaAlignment->add ( *imageWidget );

    if ( showMessage )
    {
        Gtk::VBox * box = manage ( new Gtk::VBox() );
        labelMessage = manage ( new Gtk::Label ( _ ( "Smartmatic.SAES.GUI.Widgets.ChangeLanguageWidget.MessageKey" ) ) );
        Pango::FontDescription fontTitle = labelMessage->get_style()->get_font();
        fontTitle.set_size ( 16 * Pango::SCALE );
        fontTitle.set_weight ( Pango::WEIGHT_BOLD );
        labelMessage->modify_font ( fontTitle );
        Gtk::EventBox * eventBox = manage ( new Gtk::EventBox() );
        eventBox->set_name ( SAESGUIConfigurationManager::getInstance()->GetChangeLanguageMessageWigdetEventboxName() );
        eventBox->add ( *labelMessage );
        eventBox->show_all();
        box->add ( *eventBox );

        if ( ( int ) imagesPath.size() == 1 )
        {
            if ( SAESGUIConfigurationManager::getInstance()->getCurrentMachineModelConfiguration().getId() == EnumToString( PM_VIU805 ) )
                clientAreaAlignment->set_padding ( 130, 320, 495, 495 );
            else
                clientAreaAlignment->set_padding ( 50, 230, 370, 370 );
        }
        else if ( ( int ) imagesPath.size() == 2 )
        {
            if ( SAESGUIConfigurationManager::getInstance()->getCurrentMachineModelConfiguration().getId() == EnumToString( PM_VIU805 ) )
                clientAreaAlignment->set_padding ( 130, 320, 265, 265 );
            else
                clientAreaAlignment->set_padding ( 50, 230, 140, 140 );
        }
        else
        {
            if ( SAESGUIConfigurationManager::getInstance()->getCurrentMachineModelConfiguration().getId() == EnumToString( PM_VIU805 ) )
                clientAreaAlignment->set_padding ( 130, 320, 185, 185 );
            else
                clientAreaAlignment->set_padding ( 50, 230, 55, 55 );
        }

        box->add ( *clientAreaAlignment );
        add ( *box );
    }
    else
    {
        if ( ( int ) imagesPath.size() == 1 )
        {
            if ( SAESGUIConfigurationManager::getInstance()->getCurrentMachineModelConfiguration().getId() == EnumToString( PM_VIU805 ) )
                clientAreaAlignment->set_padding ( 295, 295, 495, 495 );
            else
                clientAreaAlignment->set_padding ( 195, 195, 370, 370 );
        }
        else if ( ( int ) imagesPath.size() == 2 )
        {
            if ( SAESGUIConfigurationManager::getInstance()->getCurrentMachineModelConfiguration().getId() == EnumToString( PM_VIU805 ) )
                clientAreaAlignment->set_padding ( 295, 295, 265, 265 );
            else
                clientAreaAlignment->set_padding ( 195, 195, 140, 140 );
        }
        else
        {
            if ( SAESGUIConfigurationManager::getInstance()->getCurrentMachineModelConfiguration().getId() == EnumToString( PM_VIU805 ) )
                clientAreaAlignment->set_padding ( 295, 295, 185, 185 );
            else
                clientAreaAlignment->set_padding ( 195, 195, 60, 60 );
        }

        add ( *clientAreaAlignment );
    }

    if ( SAESGUIConfigurationManager::getInstance()->getStartConfiguration().getInfoMachine() == Smartmatic::SAES::Configuration::InfoMachineConfiguration::CONFIGURATION_VM )
    {
        if ( ( int ) imagesPath.size() == 2 )
        {
            imageWidget->set_spacing ( 50 );
            clientAreaAlignment->set_padding ( 70, 100, 80, 80 );
        }
        else
        {
            clientAreaAlignment->set_padding ( 150, 150, 55, 55 );
        }
    }

    show_all_children ();
}

ChangeLanguageWidget::language_changed ChangeLanguageWidget::LanguageChanged()
{
    return m_language_changed;
}

void ChangeLanguageWidget::SetSaveFile ( bool saveFileA )
{
    saveFile = saveFileA;
}
