/**
 * @file	voting-widget-base.cc
 * @brief	voting widget base class.
 * @author	Youssef.tourkmani
 * @date	01/11/2012
 */


#include "voting-widget-base.h"
#include "Configuration/s-a-e-s-g-u-i-conf-manager.h"
#include "Environment/Configurations/resource-path-provider.h"
#include "Configuration/label-configuration.h"
#include "Configuration/SaesGUIConfiguration-schema.hxx"

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#include <libintl.h>
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

#define N_(String) (String) ///<macro for gettext not translate

using namespace Smartmatic::GUI::Configuration;
using namespace Smartmatic::GUI::Configuration;
using namespace Smartmatic::GUI::Operation;
using namespace Smartmatic::GUI::Widgets;
using namespace Smartmatic::SAES::Configuration;
using namespace Smartmatic::SAES::GUI::Configuration;
using namespace Smartmatic::SAES::GUI::Widgets;
using namespace Smartmatic::SAES::Voting::Election;

static void changeLabelFontSize(SmartmaticBasicLabel* label, const int fontSize)
{
    Glib::RefPtr<Gtk::Style> labelStyle = label->get_style();

    if (labelStyle)
    {
        Pango::FontDescription font = labelStyle->get_font();

        font.set_size(fontSize * Pango::SCALE);
        label->modify_font(font);
    }
}

static bool isValidLabelAlignment(LabelAlignmentEnum alignment)
{
  bool result = false;

  switch (alignment)
  {
    case Smartmatic::GUI::Configuration::LeftAlignment:
    case Smartmatic::GUI::Configuration::CenterAlignment:
    case Smartmatic::GUI::Configuration::RigthAlignment:
      result = true;
      break;

  default:
      break;
  }

  return result;
}

VotingWidgetBase::VotingWidgetBase ( Smartmatic::SAES::Configuration::WidgetDisplayType & display, std::string displayString, std::string number, int padding, int numberWidth, int width, int height, bool showNumberA, bool isReferendum )
{
    this->numberWidth = numberWidth;
    this->number = number;
    this->display = display;
    this->padding = padding;
    this->displayString = displayString;
    this->topString = "";
    this->bottomString = "";
    this->showNumber = showNumberA;
    this->isReferendum = isReferendum;
    this->smallestAdjustedFontSize = -1;
    Initialize ( width, height );
}

VotingWidgetBase::VotingWidgetBase ( Smartmatic::SAES::Configuration::WidgetDisplayType & display, std::string displayString, std::string number, int padding, int numberWidth,
                                     Smartmatic::GUI::Configuration::LabelConfiguration labelChildConfigurationA, int width, int height, bool showNumberA )
{
    this->numberWidth = numberWidth;
    this->number = number;
    this->display = display;
    this->padding = padding;
    this->displayString = displayString;
    this->topString = "";
    this->bottomString = "";
    this->showNumber = showNumberA;
    this->isReferendum = false;
    this->smallestAdjustedFontSize = -1;
    Initialize ( width, height );
}


VotingWidgetBase::VotingWidgetBase ( std::string topString, std::string bottomString, std::string number, int padding, int numberWidth, int width, int height, bool showNumberA )
{
    this->numberWidth = numberWidth;
    this->number = number;
    this->display = WidgetDisplayType ( DisplayType::CONFIGURATION_DUAL_TEXT, 0, 0 );
    this->padding = padding;
    this->displayString = "";
    this->topString = topString;
    this->bottomString = bottomString;
    this->showNumber = showNumberA;
    this->isReferendum = false;
    this->smallestAdjustedFontSize = -1;
    Initialize ( width, height );
}

VotingWidgetBase::VotingWidgetBase (
  std::string topString,
  LabelAlignmentEnum topAlignment,
  std::string bottomString,
  LabelAlignmentEnum bottomAlignment,
  std::string number,
  int padding,
  int numberWidth,
  int width,
  int height,
  bool showNumberA )
{
  this->numberWidth   = numberWidth;
  this->number        = number;
  this->display       = WidgetDisplayType ( DisplayType::CONFIGURATION_DUAL_TEXT, 0, 0 );
  this->padding       = padding;
  this->displayString = "";
  this->topString     = topString;
  this->bottomString  = bottomString;
  this->showNumber    = showNumberA;
  this->isReferendum  = false;
  this->smallestAdjustedFontSize = -1;
  Initialize ( width, height, topAlignment, bottomAlignment);
}

VotingWidgetBase::~VotingWidgetBase()
{
    if ( image_child )
    {
        delete image_child;
    }

    if ( label_child )
    {
        delete label_child;
    }

    if ( top_child )
    {
        delete top_child;
    }

    if ( bottom_child )
    {
        delete bottom_child;
    }

    if ( number_child )
    {
        delete number_child;
    }

    initializeChilds();
}

void VotingWidgetBase::Initialize ( int width, int height, LabelAlignmentEnum topAlignment, LabelAlignmentEnum bottomAlignment)
{
  initializeChilds();
  sizeAllocateCallback = false;

  set_flags ( Gtk::NO_WINDOW );
  set_redraw_on_allocate ( false );

  if ( !topString.empty() )
  {
    LabelConfiguration topChildConfiguration = SAESGUIConfigurationManager::getInstance()->GetVotingWidgetTopChildLabelConfiguration();

    if ( isValidLabelAlignment(topAlignment) )
      top_alignment = topAlignment;
    else
      top_alignment = topChildConfiguration.GetLabelAlignment();

    top_child = getDefaultLabel(topString, "VotingWidgetBaseTopText", topChildConfiguration);
    top_child->show();
  }

  if ( !bottomString.empty() )
  {
    LabelConfiguration bottomChildConfiguration = SAESGUIConfigurationManager::getInstance()->GetVotingWidgetBottomChildLabelConfiguration();

    if ( isValidLabelAlignment(bottomAlignment) )
      botton_alignment = bottomAlignment;
    else
      botton_alignment = bottomChildConfiguration.GetLabelAlignment();

    bottom_child = getDefaultLabel(bottomString, "VotingWidgetBaseBottomText", bottomChildConfiguration);
    bottom_child->show();
  }

  showNumberLabel();
  setSize ( width, height );
  show();
}

void VotingWidgetBase::Initialize ( int width, int height )
{
    initializeChilds();
    sizeAllocateCallback = false;

    set_flags ( Gtk::NO_WINDOW );
    set_redraw_on_allocate ( false );

    switch ( display.getType() )
    {
        case DisplayType::CONFIGURATION_TEXT:
            {
                LabelConfiguration labelChildConfiguration  = SAESGUIConfigurationManager::getInstance()->GetVotingWidgetLabelChildLabelConfiguration();
                LabelConfiguration referendumConfiguration  = SAESGUIConfigurationManager::getInstance()->GetReferendumBallotsLabelConfiguration();

                label_child = getDefaultLabel(displayString,
                  "VotingWidgetBaseText",
                   !isReferendum ? labelChildConfiguration : referendumConfiguration);
                label_child->set_line_wrap ( true );
                label_child->show();
            }
            break;

        case DisplayType::CONFIGURATION_DUAL_TEXT:
            {
              LabelConfiguration topChildConfiguration    = SAESGUIConfigurationManager::getInstance()->GetVotingWidgetTopChildLabelConfiguration();
              LabelConfiguration bottomChildConfiguration = SAESGUIConfigurationManager::getInstance()->GetVotingWidgetBottomChildLabelConfiguration();

              top_alignment = topChildConfiguration.GetLabelAlignment();
              top_child = getDefaultLabel(topString, "VotingWidgetBaseTopText", topChildConfiguration);

              if ( !bottomString.empty() )
              {
                botton_alignment = bottomChildConfiguration.GetLabelAlignment();
                bottom_child = getDefaultLabel(bottomString, "VotingWidgetBaseBottomText", bottomChildConfiguration);
              }
            }
            break;

        case DisplayType::CONFIGURATION_IMAGE_FULL:
        case DisplayType::CONFIGURATION_IMAGE_FIX:
        case DisplayType::CONFIGURATION_IMAGE_RESIZE:
            {
                image_child = new SmartmaticBasicImage();
                image_child->set_parent ( *this );
                image_child->set_alignment ( display.getXAlign(), display.getYAlign() );
                image_child->clear();
                image_child->set ( displayString );
                image_child->show();
                image_child->setAccessibleName ( "VotingWidgetBaseImage" );
            }
            break;

        default:
            break;
    }

    showNumberLabel();
    setSize ( width, height );
    show();
}

SmartmaticBasicLabel * VotingWidgetBase::getDefaultLabel(
  std::string content,
  std::string accessibleName,
  LabelConfiguration& labelConfiguration)
{
  SmartmaticBasicLabel * defaultLabel = new SmartmaticBasicLabel ( content );

  defaultLabel->set_line_wrap(false);
  defaultLabel->set_parent(*this);
  defaultLabel->set_alignment(display.getXAlign(), display.getYAlign());
  Smartmatic::GUI::Configuration::LabelConfiguration::SetLabelConfigurationWithoutAlignment(defaultLabel, labelConfiguration);
  defaultLabel->setAccessibleName(accessibleName);

  return defaultLabel;
}

void VotingWidgetBase::showNumberLabel()
{
  if ( this->showNumber )
  {
    static LabelConfiguration numberChildConfiguration = SAESGUIConfigurationManager::getInstance()->GetVotingWidgetNumberChildLabelConfiguration();

    number_child = getDefaultLabel(this->number, "VotingWidgetBaseNumber", numberChildConfiguration);
    number_child->set_alignment(0,0);
    number_child->show();
  }
}

void VotingWidgetBase::setSize ( int width, int height )
{
    set_size_request ( width, height );

    switch ( display.getType() )
    {
        case DisplayType::CONFIGURATION_TEXT:
            {
                int innerNumberWidth = 	showNumber ? numberWidth : 0;
                label_child->set_size_request ( width - padding - innerNumberWidth, height - 2 * padding );
                break;
            }

        case DisplayType::CONFIGURATION_DUAL_TEXT:
            {
                if ( bottomString.empty() )
                {
                    int innerNumberWidth = 	showNumber ? numberWidth : 0;
                    top_child->set_size_request ( width - padding - innerNumberWidth, height - 2 * padding );
                }
            }
            break;

        case DisplayType::CONFIGURATION_IMAGE_FULL:
        case DisplayType::CONFIGURATION_IMAGE_FIX:
        case DisplayType::CONFIGURATION_IMAGE_RESIZE:
        default:
            break;
    }
}

void VotingWidgetBase::on_size_allocate ( Gtk::Allocation & allocation )
{
    if ( !sizeAllocateCallback )
    {
        set_allocation ( allocation );
        Gtk::Allocation child_allocation;
        int innerNumberWidth = 	this->showNumber ? numberWidth : 0;
        int width = allocation.get_width() - innerNumberWidth;
        int height = allocation.get_height();
        float position_x ( 0.0 ), position_y ( 0.0 );
        Pango::Rectangle rec, dummy;
        Glib::RefPtr<Pango::Layout> layout;

        switch ( display.getType() )
        {
            case DisplayType::CONFIGURATION_TEXT:
                {
                    if ( label_child )
                    {
                        layout = label_child->get_layout();
                        layout->get_pixel_extents ( dummy, rec );
                        child_allocation.set_width ( width - padding );
                        child_allocation.set_height ( height - 2 * padding );
                        int originalSizeFont = SAESGUIConfigurationManager::getInstance()->GetVotingWidgetLabelChildLabelConfiguration().GetSize();

                        while ( ( unsigned ) rec.get_width() > ( unsigned ) ( width - padding ) ||
                                ( unsigned ) rec.get_height() > ( unsigned ) ( height - (2 * padding) ) )
                        {
                            originalSizeFont--;
                            Pango::FontDescription fontTitle = label_child->get_style()->get_font();
                            fontTitle.set_size ( originalSizeFont * Pango::SCALE );
                            label_child->modify_font ( fontTitle );
                            layout = label_child->get_layout();
                            layout->get_pixel_extents ( dummy, rec );
                        }

                        setSmallestAdjustedFontSize(originalSizeFont);

                        if ( !isReferendum )
                        {
                            static Smartmatic::GUI::Configuration::LabelAlignmentEnum labelAlignment =
                                SAESGUIConfigurationManager::getInstance()->GetVotingWidgetLabelChildLabelConfiguration().GetLabelAlignment();

                            switch ( labelAlignment )
                            {
                                case Smartmatic::GUI::Configuration::LeftAlignment:
                                    position_x = allocation.get_x() + innerNumberWidth;
                                    break;

                                case Smartmatic::GUI::Configuration::CenterAlignment:
                                    position_x = allocation.get_x() + innerNumberWidth + width / 2 - rec.get_width() / 2;
                                    break;

                                case Smartmatic::GUI::Configuration::RigthAlignment:
                                    position_x = allocation.get_x() + innerNumberWidth + width - padding - rec.get_width();
                                    break;

                                default:
                                    break;
                            }
                        }
                        else
                        {
                            static Smartmatic::GUI::Configuration::LabelAlignmentEnum labelAlignment =
                                SAESGUIConfigurationManager::getInstance()->GetReferendumBallotsLabelConfiguration().GetLabelAlignment();

                            switch ( labelAlignment )
                            {
                                case Smartmatic::GUI::Configuration::LeftAlignment:
                                    position_x = allocation.get_x() + innerNumberWidth;
                                    break;

                                case Smartmatic::GUI::Configuration::CenterAlignment:
                                    position_x = allocation.get_x() + innerNumberWidth + width / 2 - rec.get_width() / 2;
                                    break;

                                case Smartmatic::GUI::Configuration::RigthAlignment:
                                    position_x = allocation.get_x() + innerNumberWidth + width - padding - rec.get_width();
                                    break;

                                default:
                                    break;
                            }
                        }

                        position_y = allocation.get_y() + ( height / 2 ) - ( ( rec.get_height() / 2 ) );
                        child_allocation.set_x ( position_x );
                        child_allocation.set_y ( position_y );
                        label_child->size_allocate ( child_allocation );
                    }
                }
                break;

            case DisplayType::CONFIGURATION_DUAL_TEXT:
                {
                    if ( top_child && bottom_child )
                    {
                        //label top
                        layout = top_child->get_layout();
                        layout->get_pixel_extents ( dummy, rec );
                        child_allocation.set_width ( width - padding );
                        child_allocation.set_height ( height / 2 - padding );

                        switch ( top_alignment ) //top
                        {
                            case Smartmatic::GUI::Configuration::LeftAlignment:
                                position_x = allocation.get_x() + innerNumberWidth;
                                break;

                            case Smartmatic::GUI::Configuration::CenterAlignment:
                                position_x = allocation.get_x() + innerNumberWidth + width / 2 - rec.get_width() / 2;
                                break;

                            case Smartmatic::GUI::Configuration::RigthAlignment:
                                position_x = allocation.get_x() + innerNumberWidth + width - padding - rec.get_width();
                                break;

                            default:
                                break;
                        }

                        //TODO size of text
                        child_allocation.set_x ( position_x );
                        child_allocation.set_y ( allocation.get_y() + padding );
                        top_child->size_allocate ( child_allocation );
                        //label bottom
                        layout = bottom_child->get_layout();
                        layout->get_pixel_extents ( dummy, rec );
                        child_allocation.set_width ( width - padding );
                        child_allocation.set_height ( height / 2 - padding );

                        switch ( botton_alignment ) //bottom
                        {
                            case Smartmatic::GUI::Configuration::LeftAlignment:
                                position_x = allocation.get_x() + innerNumberWidth;
                                break;

                            case Smartmatic::GUI::Configuration::CenterAlignment:
                                position_x = allocation.get_x() + innerNumberWidth + width / 2 - rec.get_width() / 2;
                                break;

                            case Smartmatic::GUI::Configuration::RigthAlignment:
                                position_x = allocation.get_x() + innerNumberWidth + width - padding - rec.get_width();
                                break;

                            default:
                                break;
                        }

                        child_allocation.set_x ( position_x );
                        child_allocation.set_y ( allocation.get_y() + height - padding - rec.get_height() );
                        bottom_child->size_allocate ( child_allocation );
                    }
                    else if ( top_child )
                    {
                        layout = top_child->get_layout();
                        layout->get_pixel_extents ( dummy, rec );
                        child_allocation.set_width ( width - padding );
                        child_allocation.set_height ( height - 2 * padding );
                        int originalSizeFont = SAESGUIConfigurationManager::getInstance()->GetVotingWidgetTopChildLabelConfiguration().GetSize();

                        while ( rec.get_width() > width - padding )
                        {
                            originalSizeFont--;
                            Pango::FontDescription fontTitle = top_child->get_style()->get_font();
                            fontTitle.set_size ( originalSizeFont * Pango::SCALE );
                            top_child->modify_font ( fontTitle );
                            layout = top_child->get_layout();
                            layout->get_pixel_extents ( dummy, rec );
                        }

                        setSmallestAdjustedFontSize(originalSizeFont);

                        static Smartmatic::GUI::Configuration::LabelAlignmentEnum labelAlignment =
                            SAESGUIConfigurationManager::getInstance()->GetReferendumBallotsLabelConfiguration().GetLabelAlignment();

                        switch ( labelAlignment )
                        {
                            case Smartmatic::GUI::Configuration::LeftAlignment:
                                position_x = allocation.get_x() + innerNumberWidth;
                                break;

                            case Smartmatic::GUI::Configuration::CenterAlignment:
                                position_x = allocation.get_x() + innerNumberWidth + width / 2 - rec.get_width() / 2;
                                break;

                            case Smartmatic::GUI::Configuration::RigthAlignment:
                                position_x = allocation.get_x() + innerNumberWidth + width - padding - rec.get_width();
                                break;

                            default:
                                break;
                        }

                        position_y = allocation.get_y() + ( height / 2 ) - ( ( rec.get_height() / 2 ) );
                        child_allocation.set_x ( position_x );
                        child_allocation.set_y ( position_y );
                        top_child->size_allocate ( child_allocation );
                    }
                }
                break;

            case DisplayType::CONFIGURATION_IMAGE_FIX:
                {
                    if ( image_child )
                    {
                        //image
                        child_allocation.set_width ( width - padding );
                        child_allocation.set_height ( height -  2 * padding );
                        child_allocation.set_x ( allocation.get_x() + innerNumberWidth );
                        child_allocation.set_y ( allocation.get_y() + padding );
                        image_child->size_allocate ( child_allocation );

                        double w = ( display.getWidth().present() ? display.getWidth().get() : child_allocation.get_width() );
                        double h = ( display.getHeight().present() ? display.getHeight().get() : child_allocation.get_height() );

                        Glib::RefPtr<Gdk::Pixbuf> tempPixbuf = image_child->get_pixbuf();
                        tempPixbuf = tempPixbuf->scale_simple ( w, h, Gdk::INTERP_HYPER );
                        image_child->set ( tempPixbuf );
                    }
                }
                break;

            case DisplayType::CONFIGURATION_IMAGE_FULL:
                {
                    if ( image_child )
                    {
                        //image
                        child_allocation.set_width ( width - padding );
                        child_allocation.set_height ( height -  2 * padding );
                        child_allocation.set_x ( allocation.get_x() + innerNumberWidth );
                        child_allocation.set_y ( allocation.get_y() + padding );
                        image_child->size_allocate ( child_allocation );

                        Glib::RefPtr<Gdk::Pixbuf> tempPixbuf = image_child->get_pixbuf();
                        tempPixbuf = tempPixbuf->scale_simple ( child_allocation.get_width(), child_allocation.get_height(), Gdk::INTERP_HYPER );
                        image_child->set ( tempPixbuf );
                    }
                }
                break;

            case DisplayType::CONFIGURATION_IMAGE_RESIZE:
                {
                    if ( image_child )
                    {
                        //image
                        child_allocation.set_width ( width - padding );
                        child_allocation.set_height ( height -  2 * padding );
                        child_allocation.set_x ( allocation.get_x() + innerNumberWidth );
                        child_allocation.set_y ( allocation.get_y() + padding );
                        image_child->size_allocate ( child_allocation );

                        double w = ( display.getWidth().present() ? display.getWidth().get() : child_allocation.get_width() );
                        double h = ( display.getHeight().present() ? display.getHeight().get() : child_allocation.get_height() );

                        if ( w > child_allocation.get_width() )
                        {
                            w = child_allocation.get_width();
                        }

                        if ( h > child_allocation.get_height() )
                        {
                            h = child_allocation.get_height();
                        }

                        Glib::RefPtr<Gdk::Pixbuf> tempPixbuf = image_child->get_pixbuf();
                        double ri = ( double ) tempPixbuf->get_height() / ( double ) tempPixbuf->get_width();

                        double nw = h / ri;
                        double nh = ri * nw;

                        if ( nw > w )
                        {
                            nw = w;
                            nh = ri * nw;
                        }

                        tempPixbuf = tempPixbuf->scale_simple ( nw, nh, Gdk::INTERP_HYPER );
                        //
                        image_child->set ( tempPixbuf );
                    }
                }
                break;

            default:
                break;
        }

        //number
        if ( this->showNumber )
        {
            layout = number_child->get_layout();
            layout->get_pixel_extents ( dummy, rec );
            position_x = allocation.get_x() + innerNumberWidth / 2 - rec.get_width() / 2;
            position_y = allocation.get_y() + height / 2  - rec.get_height() / 2;

            if ( position_x < allocation.get_x() + padding )
            {
                position_x = allocation.get_x() + padding;
            }

            if ( position_y < allocation.get_y() + padding )
            {
                position_y = allocation.get_y() + padding;
            }

            child_allocation.set_width ( innerNumberWidth - padding );
            child_allocation.set_height ( height - 2 * padding );
            child_allocation.set_x ( position_x );
            child_allocation.set_y ( position_y );
            number_child->size_allocate ( child_allocation );
        }

        sizeAllocateCallback = true;
    }
}

void VotingWidgetBase::forall_vfunc ( gboolean include_internals, GtkCallback callback, gpointer callback_data )
{
    if ( image_child )
    {
        callback ( GTK_WIDGET ( image_child->gobj() ), callback_data );
    }

    if ( label_child )
    {
        callback ( GTK_WIDGET ( label_child->gobj() ), callback_data );
    }

    if ( top_child )
    {
        callback ( GTK_WIDGET ( top_child->gobj() ), callback_data );
    }

    if ( bottom_child )
    {
        callback ( GTK_WIDGET ( bottom_child->gobj() ), callback_data );
    }

    if ( number_child )
    {
        callback ( GTK_WIDGET ( number_child->gobj() ), callback_data );
    }
}

void VotingWidgetBase::on_add ( Gtk::Widget * child )
{
    //No more widgets can be added.
}

void VotingWidgetBase::on_remove ( Gtk::Widget * child )
{
    if ( image_child )
    {
        image_child->unparent();
    }

    if ( label_child )
    {
        label_child->unparent();
    }

    if ( top_child )
    {
        top_child->unparent();
    }

    if ( bottom_child )
    {
        bottom_child->unparent();
    }

    if ( number_child )
    {
        number_child->unparent();
    }
}

GType VotingWidgetBase::child_type_vfunc() const
{
    //No more widgets can be added.
    return G_TYPE_NONE;
}

void VotingWidgetBase::refreshLabel ( std::string text )
{
    if ( label_child )
    {
        label_child->set_text ( text );
    }
}

void VotingWidgetBase::setSmallestAdjustedFontSize(const int newFontSize)
{
    if (smallestAdjustedFontSize == -1 || newFontSize < smallestAdjustedFontSize)
    {
        smallestAdjustedFontSize = newFontSize;
    }

}

int VotingWidgetBase::getSmallestAdjustedFontSize() const
{
    return smallestAdjustedFontSize;
}

void VotingWidgetBase::setFontSize(const int fontSize)
{
    if (label_child)
    {
        changeLabelFontSize(label_child, fontSize);
    }

    if (top_child)
    {
        changeLabelFontSize(top_child, fontSize);
    }

    if (bottom_child)
    {
        changeLabelFontSize(bottom_child, fontSize);
    }

    if (number_child)
    {
        changeLabelFontSize(number_child, fontSize);
    }
}

void VotingWidgetBase::initializeChilds ( )
{
  image_child = 0;
  label_child = 0;
  top_child = 0;
  bottom_child = 0;
  number_child = 0;
}