/**
 * @file	empty-window.cc
 * @brief	Body the empty Windows Form.
 * @date	03/04/2012
 * @author	Yadickson Soto
 */


#include "empty-window.h"
#include "Configuration/s-a-e-s-g-u-i-conf-manager.h"
#include "Functionalities/saes-functionality.h"
#include <Operation/MachineOperationManager.hxx>
#include <Gui/Widgets/HelpWidget.hxx>
#include <Gui/Windows/MemoryMessageWindow.hxx>
#include <Runtime/Environment/MemoryPathController.h>
#include <Log/ISMTTLog.h>
#include <Log/SMTTLogManager.h>
#include <Gui/Windows/BarcodeReconfigureWindow.hxx>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#include <libintl.h>
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

#define N_(String) (String) ///<macro for gettext not translate

using namespace Smartmatic::GUI::Widgets;
using namespace Smartmatic::SAES::GUI::Widgets;
using namespace Smartmatic::SAES::GUI::Windows;
using namespace Smartmatic::SAES::GUI::Configuration;
using namespace Smartmatic::SAES::Runtime::Environment;
using namespace Smartmatic::GUI::Windows;
using namespace Smartmatic::SAES::Common::Configuration;
using namespace Smartmatic::SAES::Functionality::Configuration;
using namespace Smartmatic::Log;

ISMTTLog * EmptyWindow::logger = SMTTLogManager::GetLogger ( "Smartmatic.SAES.GUI.Windows.EmptyWindow", GETTEXT_PACKAGE );

EmptyWindow::EmptyWindow ( Smartmatic::GUI::Widgets::StepWidget * stepWidget )
    : BasicWindow ( false )
{
    initConfig();
    pushStepWidget ( stepWidget );
}

EmptyWindow::EmptyWindow ( Gtk::Widget * widget, std::string title, bool showTitleText, bool showHelpButton, bool showExitButton )
    : BasicWindow ( showTitleText )
{
    initConfig();
    stepWidget.setTitleText ( title );
    stepWidget.setTitle ( showTitleText );
    stepWidget.setHelp ( showHelpButton );
    stepWidget.setExit ( showExitButton );
    stepWidget.setWidget ( widget );
    pushStepWidget ( &stepWidget );
}

EmptyWindow::~EmptyWindow()
{
    this->remove();
    BasicWindow::dispose();
}

void EmptyWindow::initConfig()
{
    this->viewInitMessage = false;
    this->viewExitMessage = false;
    this->viewHelpWidget = false;

    RulesBasePathType & rule = MemoryPathController::getInstance()->getRule();
    forceHide = rule != RulesBasePathType::CONFIGURATION_PM_RULE || SAESGUIConfigurationManager::getInstance()->getForceExecuteFunctionality();

    Gdk::Color color ( "black" );
    setTitleColor ( color );
    std::string fileImage = Smartmatic::Environment::ResourcePathProvider::getInstance()->GetResourcePath ( PACKAGE_NAME, "ui/one-memory-small.gif" );
    setTitleImage ( fileImage );
    setTitleLabelConfiguration ( SAESGUIConfigurationManager::getInstance()->GetEmptyWindowTitleFormConfiguration() );

    MemoryMessageWindow::getInstance()->getSignalBlock().connect ( sigc::mem_fun ( *this, &EmptyWindow::memoryBlock ) );
    MemoryMessageWindow::getInstance()->getSignalAllow().connect ( sigc::mem_fun ( *this, &EmptyWindow::restoreOneMemory ) );
    MemoryMessageWindow::getInstance()->getSignalResume().connect ( sigc::mem_fun ( *this, &EmptyWindow::restoreTwoMemory ) );
}

void EmptyWindow::memoryBlock()
{
    BarcodeReconfigureWindow::getInstance()->stop();

    logger->Debug ( "memory block" );
    StateListener * state = getCurrentWidget()->getStateListener();

    if ( state )
    {
        state->block();
    }

    disableExitButton();
    refreshTitleText();
}

void EmptyWindow::showInitMessageWidget ( InfoStepWidget & widget )
{
    if ( !viewInitMessage )
    {
        logger->Debug ( "init message widget" );
        viewInitMessage = true;
        pushStepWidget ( &widget );
        disableHelpButton();
        disableExitButton();
        Glib::signal_timeout().connect_once ( sigc::mem_fun ( *this, &EmptyWindow::exitInitMessage ), widget.getTimeout() );
    }
}

void EmptyWindow::showExitMessageWidget ( InfoStepWidget & widget )
{
    if ( !viewExitMessage )
    {
        logger->Debug ( "exit message widget" );
        popStepWidget();
        viewExitMessage = true;
        pushStepWidget ( &widget );
        disableHelpButton();
        disableExitButton();
        Glib::signal_timeout().connect_once ( sigc::mem_fun ( *this, &EmptyWindow::exitMessage ), widget.getTimeout() );
    }
}

void EmptyWindow::showMessageWidget ( InfoStepWidget & widget )
{
    pushStepWidget ( &widget );
    disableHelpButton();
    disableExitButton();
    Glib::signal_timeout().connect_once ( sigc::mem_fun ( *this, &EmptyWindow::exitInitMessage ), widget.getTimeout() );
}

void EmptyWindow::ShowHelp()
{
    Glib::Mutex::Lock lock ( mutex );
    viewHelpWidget = true;
    BasicWindow::ShowHelp();
}

void EmptyWindow::exitInitMessage()
{
    viewInitMessage = false;
    BasicWindow::Exit();
}

void EmptyWindow::exitMessage()
{
    viewExitMessage = false;
    BasicWindow::Exit();
    signalExitButton.emit();
}

void EmptyWindow::ShowExit()
{
    Glib::Mutex::Lock lock ( mutex );
    signalShowExitMessage.emit();
    StepWidget * widget = getCurrentWidget();

    if ( !viewExitMessage && widget && widget->getExitWidget() )
    {
        InfoStepWidget * widgetExit = ( InfoStepWidget * ) widget->getExitWidget();
        showExitMessageWidget ( *widgetExit );
    }
    else
    {
        exitMessage();
    }
}

void EmptyWindow::Exit()
{
    if ( viewHelpWidget )
    {
        viewHelpWidget = false;
        BasicWindow::CloseHelp();
    }
    else if ( !SAESGUIConfigurationManager::getInstance()->getForceExecuteFunctionality() && !MemoryPathController::getInstance()->isAllDevices() && !viewHelpWidget )
    {
        setExitButtonSensitive ( false );
        return;
    }
    else if ( MemoryPathController::getInstance()->isAllDevices() || forceHide )
    {
        ShowExit();
        logger->Debug ( "Emit exit button" );
    }

    enableExitButton();
}

void EmptyWindow::restoreOneMemory()
{
    BarcodeReconfigureWindow::getInstance()->start();

    StateListener * state = getCurrentWidget()->getStateListener();

    if ( state )
    {
        state->partial();
    }

    disableExitButton();
}

void EmptyWindow::restoreTwoMemory()
{
    BarcodeReconfigureWindow::getInstance()->start();

    StateListener * state = getCurrentWidget()->getStateListener();

    if ( state )
    {
        state->resume();
    }

    enableExitButton();
}

void EmptyWindow::enableExitButton()
{
    StepWidget * widget = getCurrentWidget();

    if ( widget )
    {
        bool force = SAESGUIConfigurationManager::getInstance()->getForceExecuteFunctionality();
        setExitButtonSensitive ( ( force || MemoryPathController::getInstance()->isAllDevices() || viewHelpWidget ) && widget->isExitSensitive() );
    }
    else
    {
        setExitButtonSensitive ( true );
    }
}

void EmptyWindow::disableExitButton()
{
    setExitButtonSensitive ( viewHelpWidget );
}

void EmptyWindow::refreshTitleText()
{
    StepWidget * widget = getCurrentWidget();

    if ( widget )
    {
        setTitleText ( _ ( widget->getTitleText().c_str() ) );
    }
}

sigc::signal<void> EmptyWindow::getSignalShowExitMessage()
{
    return signalShowExitMessage;
}

sigc::signal<void> EmptyWindow::getSignalExitButton()
{
    return signalExitButton;
}

void EmptyWindow::forceDisableExitButton()
{
    setExitButtonSensitive ( false );
}
