/**
 * @file RS232Connection.cxx
 * @brief	Defines the RS232 communication connection.
 * @author	Yadickson Soto
 * @date	18/02/2014
 */

#include "RS232Connection.hxx"
#include <Log/ISMTTLog.h>
#include <Log/SMTTLogManager.h>

#include <sys/ioctl.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <limits.h>
#include <string.h>
#include <errno.h>   /* Error number definitions */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

using namespace Smartmatic::SAES::HW::Connections;
using namespace Smartmatic::Log;

ISMTTLog * RS232Connection::logger = SMTTLogManager::GetLogger ( "Smartmatic.SAES.HW.Connections.RS232Connection", GETTEXT_PACKAGE );

RS232Connection::RS232Connection ( unsigned int baudrate, unsigned int dataBits, bool parity, bool enableCTSRTS )
    : port ( 0 ), baudrate ( baudrate ), dataBits ( dataBits ), parity ( parity ), enableCTSRTS ( enableCTSRTS )
{

}

RS232Connection::~RS232Connection()
{
    closeConnection();
}

bool RS232Connection::openConnection ( const std::string & device )
{
    if ( port <= 0 )
    {
        logger->Debug ( "Start open device path ( " + device + " )." );

        int baudr;

        switch ( baudrate )
        {
            default:
            case  9600 :
                baudr = B9600;
                break;

            case  19200 :
                baudr = B19200;
                break;

            case  38400 :
                baudr = B38400;
                break;

            case  57600 :
                baudr = B57600;
                break;

            case  115200 :
                baudr = B115200;
                break;
        }

        port = open ( device.c_str(), O_RDWR | O_NOCTTY | O_NDELAY );

        if ( port < 0 )
        {
            logger->Debug ( "unable to open device ( " + device + " ) : " + std::string ( strerror ( errno ) ) );
            return false;
        }

        // flushing is to be done after opening. This prevents first read and write to be spam'ish.
        tcflush ( port, TCIOFLUSH );

        int error = tcgetattr ( port, &portSettings );

        if ( error == -1 )
        {
            logger->Debug ( "unable to read port settings from device ( " + device + " ) : " + std::string ( strerror ( errno ) ) );
            closeConnection();
            return false;
        }

        portSettings.c_oflag &= ~ ( OPOST | OLCUC | OCRNL | ONOCR | ONLRET | OFILL | OFDEL );
        portSettings.c_oflag |= ONLCR;

        // Enable the receiver and set local mode...
        portSettings.c_cflag &= ~PARENB;
        portSettings.c_cflag &= ~CSTOPB;
        portSettings.c_cflag &= ~CSIZE;

        //set the port to manage data as raw input
        portSettings.c_iflag &= ~ ( IGNBRK | BRKINT | PARMRK | ISTRIP | INLCR | IGNCR | ICRNL | IXON );
        portSettings.c_lflag &= ~ ( ECHO | ECHONL | ICANON | ISIG | IEXTEN );

        cfsetispeed ( &portSettings, baudr );
        cfsetospeed ( &portSettings, baudr );

        portSettings.c_cflag |= ( CLOCAL | CREAD );

        portSettings.c_cc[VMIN] = 0;      /* block until n bytes are received */
        portSettings.c_cc[VTIME] = 0;     /* block until a timer expires (n * 100 mSec.) */

        if ( parity )
        {
            portSettings.c_cflag |= PARENB;
            portSettings.c_cflag &= ~PARODD;
            portSettings.c_cflag &= ~CSTOPB;
        }
        else
        {
            portSettings.c_cflag |= CS8;
        }

        switch ( dataBits )
        {
            case 7:
                portSettings.c_cflag |= CS7;
                break;

            default:
                portSettings.c_cflag |= CS8;
                break;
        }

        if ( enableCTSRTS )
        {
            // Turn on hardware flow control
            portSettings.c_cflag |= CRTSCTS;
        }
        else
        {
            // Turn off hardware flow control
            portSettings.c_cflag &= ~CRTSCTS;
        }

        error = tcsetattr ( port, TCSANOW, &portSettings );

        if ( error == -1 )
        {
            logger->Debug ( "unable to adjust port settings from device ( " + device + " ) : " + std::string ( strerror ( errno ) ) );
            closeConnection();
            return false;
        }
    }

    return true;
}

bool RS232Connection::sendCommand ( const std::vector<unsigned char> & command )
{
    bool validate = false;

    if ( port > 0 && !command.empty() )
    {
        validate = ! ( write ( port, &command[0], command.size() ) < 0 );
    }

    return validate;
}

Smartmatic::System::SafeVector <unsigned char> RS232Connection::readBuffer ()
{
    Smartmatic::System::SafeVector <unsigned char> buffer;

    if ( port > 0 )
    {
        int bytes;
        unsigned char buf[PIPE_BUF];

        bytes = read ( port, buf, sizeof ( buf ) );

        if ( bytes > 0 )
        {
            buffer.assign ( buf, buf + bytes );
        }
    }

    return buffer;
}

bool RS232Connection::closeConnection( )
{
    bool result = false;

    if ( port > 0 )
    {
        int result_total = 0;

        portSettings.c_cflag &= ~CRTSCTS;

        result_total += tcsetattr ( port, TCSANOW, &portSettings );
        result_total += close ( port );

        if ( result_total < 0 )
        {
            result = false;
        }
        else
        {
            result = true;
        }
    }

    port = 0;

    return result;
}

unsigned int RS232Connection::getTimeOut()
{
    return 100000;
}
