/**
 * @file RS232Connection.hxx
 * @brief	Defines the RS232 communication connection.
 * @author	Yadickson Soto
 * @date	18/02/2014
 */

#ifndef RS232CONNECTION_HXX_
#define RS232CONNECTION_HXX_

#include <Connections/IConnection.hxx>
#include <termios.h>

namespace Smartmatic
{
    namespace Log
    {
        class ISMTTLog;
    }
    namespace SAES
    {
        namespace HW
        {
            namespace Connections
            {
                /**
                 * @class	RS232Connection
                 * @brief	Defines the RS232 communication connection.
                 * @author	Yadickson Soto
                 * @date		18/02/2014
                 */
                class RS232Connection : public IConnection
                {
                    public:

                        /**
                         * @fn	RS232Connection ( unsigned int baudrate, unsigned int dataBits, bool parity, bool enableCTSRTS );
                         * @brief	Default constructor.
                         * @author	Yadickson Soto
                         * @date	18/02/2014
                         * @param	baudrate The baud rate connection
                         * @param	dataBits Set number of data bits. Default is 8.
                         * @param	parity	Enable parity. Default is false.
                         * @param	enableCTSRTS Enable CTS and RTS. Default is false.
                         */
                        RS232Connection ( unsigned int baudrate, unsigned int dataBits, bool parity, bool enableCTSRTS );

                        /**
                         * @fn	virtual ~RS232Connection();
                         * @brief	Default destructor.
                         * @author	Yadickson Soto
                         * @date	18/02/2014
                         */
                        virtual ~RS232Connection();

                        /**
                         * @fn	virtual bool openConnection ( const std::string & device );
                         * @brief	Opens the connections.
                         * @author	Yadickson Soto
                         * @date	20/02/2014
                         * @param 	device The device path
                         * @return	true if successful.
                         */

                        virtual bool openConnection ( const std::string & device );

                        /**
                         * @fn	virtual bool sendCommand ( const std::vector<unsigned char> & command );
                         * @brief	Sends command over the connection.
                         * @author	Yadickson Soto
                         * @date	20/02/2014
                         * @param	command	The command to send.
                         * @return	true if successful.
                         */

                        virtual bool sendCommand ( const std::vector<unsigned char> & command );

                        /**
                         * @fn	virtual Smartmatic::System::SafeVector <unsigned char> readBuffer ();
                         * @brief	Reads data from the connection.
                         * @author	Yadickson Soto
                         * @date	20/02/2014
                         * @return	data buffer read from the connection, empty if result is false
                         */

                        virtual Smartmatic::System::SafeVector <unsigned char> readBuffer ();

                        /**
                         * @fn	virtual bool closeConnection ();
                         * @brief	Closes the connection.
                         * @author	Yadickson Soto
                         * @date	20/02/2014
                         * @return	true if successful.
                         */

                        virtual bool closeConnection ();

                        /**
                         * @fn	virtual unsigned int getTimeOut();
                         * @brief	Get timeout to wait the read buffer.
                         * @author	Yadickson Soto
                         * @date	30/06/2015
                         */

                        virtual unsigned int getTimeOut();

                    private:

                        int port; ///< The connection's port handler.

                        unsigned int baudrate; ///< The connection's baud rate
                        unsigned int dataBits; ///< The data bits

                        bool parity; ///< Enable parity
                        bool enableCTSRTS; ///< enable CTS and RTS

                        struct termios portSettings; ///< The port settings

                        static Smartmatic::Log::ISMTTLog * logger; /**<logger*/
                };

            } /* namespace Connections */
        } /* namespace HW */
    } /* namespace SAES */
} /* namespace Smartmatic */
#endif /* RS232CONNECTION_HXX_ */
