/**
 * @file	SerialPortConnection.hxx
 * @brief	Declares the serial port connection class.
 * @author	Youssef.tourkmani
 * @date	31/10/2012
 */

#ifndef _SERIAL_PORT_CONNECTION_H_
#define _SERIAL_PORT_CONNECTION_H_

#include <Connections/IConnection.hxx>
#include <Connections/IPeekListener.hxx>

namespace Smartmatic
{
    namespace Log
    {
        class ISMTTLog;
    }

    namespace SAES
    {
        namespace HW
        {
            namespace Connections
            {
                /**
                 * @class	SerialPortConnection
                 * @brief	Manages the connection to a serial port resources.
                 * @author	Fsilva
                 * @date	28/04/2011
                 */

                class SerialPortConnection: public IConnection
                {
                    public:

                        /**
                         * @fn	SerialPortConnection ( IConnection * connection );
                         * @brief	Default constructor.
                         * @author	Yadickson Soto
                         * @date	20/02/2014
                         * @param	connection			The connection provider.
                         */
                        explicit SerialPortConnection ( IConnection * connection );

                        /**
                         * @fn	virtual ~SerialPortConnection();
                         * @brief	Destructor.
                         * @author	Youssef.tourkmani
                         * @date	31/10/2012
                         */
                        virtual ~SerialPortConnection();

                        /**
                         * @fn	virtual bool openConnection ( const std::string & device  );
                         * @brief	Opens the connections.
                         * @author	Yadickson Soto
                         * @date	20/02/2014
                         * @param 	device device path
                         * @return	true if successful.
                         */
                        virtual bool openConnection ( const std::string & device );

                        /**
                         * @fn	virtual bool sendCommand ( const std::vector<unsigned char> & command );
                         * @brief	Sends command over the connection.
                         * @author	Yadickson Soto
                         * @date	20/02/2014
                         * @param	command	The data.
                         * @return	true if successful.
                         */
                        virtual bool sendCommand ( const std::vector<unsigned char> & command );

                        /**
                         * @fn	virtual Smartmatic::System::SafeVector <unsigned char> readBuffer ();
                         * @brief	Reads data from the connection.
                         * @author	Yadickson Soto
                         * @date	20/02/2014
                         * @return	data buffer read from the connection, empty if result is false
                         */
                        virtual Smartmatic::System::SafeVector <unsigned char> readBuffer ();

                        /**
                         * @fn	virtual bool closeConnection ();
                         * @brief	Closes the connection.
                         * @author	Yadickson Soto
                         * @date	20/02/2014
                         * @return	true if successful.
                         */
                        virtual bool closeConnection ();

                        /**
                         * @fn	void setListener ( IPeekListener * clientListener );
                         * @brief	Set peek listener.
                         * @author	Yadickson Soto
                         * @date	26/06/2015
                         * @param	clientListener		The client listener
                         */
                        void setListener ( IPeekListener * clientListener );

                        /**
                         * @fn	virtual unsigned int getTimeOut();
                         * @brief	Get timeout to wait the read buffer.
                         * @author	Yadickson Soto
                         * @date	30/06/2015
                         */
                        virtual unsigned int getTimeOut();

                    protected:

                        IConnection * connection; ///< The connection provider
                        IPeekListener * clientListener; ///< client listener

                    private:

                        static Smartmatic::Log::ISMTTLog * logger; /**<logger*/
                };
            }
        }
    }
}

#endif // _SERIAL_PORT_CONNECTION_H_
