/**
 * @file	BarcodeReaderDevice.hxx
 * @brief	Declares the barcode reader device class.
 * @author	Yadickson Soto
 * @date	26/06/2015
 */

#ifndef _BARCODE_READER_DEVICE_H_
#define _BARCODE_READER_DEVICE_H_

#include <Connections/IConnection.hxx>
#include <System/SafeBuffer.h>
#include <Devices/BarcodeInterface.hxx>
#include <Connections/IPeekListener.hxx>

namespace Smartmatic
{
    namespace Log
    {
        class ISMTTLog;
    }

    namespace SAES
    {
        namespace HW
        {
            namespace Devices
            {
                /**
                 * @class	BarcodeReaderDevice
                 * @brief	Manage the connections with the barcode.
                 * @author	Yadickson Soto
                 * @date	26/06/2015
                 */
                class BarcodeReaderDevice: public BarcodeInterface, public Smartmatic::SAES::HW::Connections::IPeekListener
                {
                    public:

                        /**
                         * @fn	BarcodeReaderDevice( Smartmatic::SAES::HW::Connections::IConnection * connection );
                         * @brief	Default constructor.
                         * @author	Yadickson Soto
                         * @date	26/06/2015
                         * @param connection The serial connection to device
                         */
                        explicit BarcodeReaderDevice ( Smartmatic::SAES::HW::Connections::IConnection * connection );

                        /**
                         * @fn	virtual ~BarcodeReaderDevice();
                         * @brief	Class destroyer.
                         * @author	Yadickson Soto
                         * @date	26/06/2015
                         */
                        virtual ~BarcodeReaderDevice();

                        /**
                         * @fn	virtual bool open( const std::string & device );
                         * @brief	Opens the connections.
                         * @author	Yadickson Soto
                         * @date	26/06/2015
                         * @param 	device device path
                         * @return	true if successful.
                         */
                        virtual bool open ( const std::string & device );

                        /**
                         * @fn	virtual bool close();
                         * @brief	Closes the connections.
                         * @author	Yadickson Soto
                         * @date	26/06/2015
                         * @return	true if successful.
                         */
                        virtual bool close();

                        /**
                         * @fn	virtual std::string getDeviceModel();
                         * @brief	Getter device model.
                         * @author	Yadickson Soto
                         * @date	26/06/2015
                         * @return	device model
                         */
                        virtual std::string getDeviceModel();
                        /**
                         * @fn	virtual std::string getDeviceSerial();
                         * @brief	Getter device serial ESN.
                         * @author	Yadickson Soto
                         * @date	26/06/2015
                         * @param	factorySerial Return the factory serial instead of the user defined serial.
                         * @return	device serial ESN
                         */
                        virtual std::string getDeviceSerial(bool factorySerial = false);
                        /**
                         * @fn	virtual bool setDeviceSerial ( const std::string & serial );
                         * @brief	Setter new serial to device.
                         * @author	Yadickson Soto
                         * @date	26/06/2015
                         * @param	serial new serial
                         * @return true if successfully
                         */
                        virtual bool setDeviceSerial ( const std::string & serial );

                        /**
                         * @fn	virtual bool sendReconfigureCommand ( const std::string & command,
                         * 		unsigned int timeout,
                         * 		unsigned int retries,
                         * 		bool mandatory );
                         * @brief	Send command to device.
                         * @author	Yadickson Soto
                         * @date	26/06/2015
                         * @param	command command to send
                         * @param	command The string command to send
                         * @param	timeout The waiting time to get response command
                         * @param	retries The tries to get good response
                         * @param	mandatory If command isn't mandatory, the execute is true
                         * @return true if successfully
                         */
                        virtual bool sendReconfigureCommand ( const std::string & command,
                                                              unsigned int timeout,
                                                              unsigned int retries,
                                                              bool mandatory );

                        /**
                         * @fn	virtual Smartmatic::System::SafeBuffer readBuffer ();
                         * @brief	Read device buffer.
                         * @author	Yadickson Soto
                         * @date	26/06/2015
                         * @return  data buffer from device
                         */
                        virtual Smartmatic::System::SafeBuffer readBuffer ();

                    protected:

                        /**
                         * @fn	virtual void peekEvent( const std::vector<unsigned char> & data);
                         * @brief	Receive and processed the data.
                         * @author	Yadickson Soto
                         * @date	26/06/2015
                         * @param	data	The data.
                         */
                        virtual void peekEvent ( const std::vector<unsigned char> & data );

                    private:

                        Smartmatic::SAES::HW::Connections::IConnection * connection; ///< The connection provider
                        Smartmatic::System::SafeBuffer buffer; ///< The data buffer reader

                        static Smartmatic::Log::ISMTTLog * logger;  ///< The logger

                    protected:

                        bool processSendCommand; ///< True if get result from command
                        bool processSendConfigurationCommand; ///< True if get response from configuration commands
                };
            }
        }
    }
}

#endif // _BARCODE_READER_DEVICE_H_
