/*
 * EurnDevice.hxx
 *
 *  Created on: Oct 26, 2017
 *      Author: juanvalera
 */

#ifndef EURNDEVICE_HXX_
#define EURNDEVICE_HXX_

#include <Devices/EurnInterface.hxx>
#include <Commands/CommandBaseEurn.hxx>
#include <Configuration/EurnDeviceSchema.hxx>

namespace Smartmatic
{
    namespace Log
    {
        class ISMTTLog;
    }

    namespace SAES
    {
        namespace HW
        {
            namespace Connections
            {
                class IConnection;
            }

            namespace Devices
            {
                /**
                 * @class	EurnDevice
                 * @brief	Manage the connections with the Eurn.
                 * @author	Juan Valera
                 * @date	26/10/2017
                 */
                class EurnDevice: public EurnInterface
                {
                    public:

                        /**
                         * @fn	EurnDevice( Smartmatic::SAES::HW::Connections::IConnection * connection );
                         * @brief	Default constructor.
                         * @author	Juan Valera
                         * @date	26/10/2017
                         * @param connection The serial connection to device
                         */
                        explicit EurnDevice ( Smartmatic::SAES::HW::Connections::IConnection * connection);

                        /**
                         * @fn	virtual ~EurnDevice();
                         * @brief	Class destroyer.
                         * @author	Juan Valera
                         * @date	26/10/2017
                         */
                        virtual ~EurnDevice();

                        /**
                         * @fn	virtual bool open( const std::string & device );
                         * @brief	Opens the connections.
                         * @author	Juan Valera
                         * @date	26/10/2017
                         * @param 	device device path
                         * @return	true if successful.
                         */
                        virtual bool open ( const std::string & device );

                        /**
                         * @fn	virtual bool close();
                         * @brief	Closes the connections.
                         * @author	Juan Valera
                         * @date	26/10/2017
                         * @return	true if successful.
                         */
                        virtual bool close();

                        /**
                         * @fn  virtual std::string getVersion();
                         * @brief   gets the version of the add on
                         * @author  Juan Valera
                         * @date    27/10/2017
                         * @return  hex number if successful, 0x00 if not
                         */
                        virtual std::string getVersion();

                        /**
                         * @fn	virtual bool turnGreenLedOn();
                         * @brief	Turn on the green led.
                         * @author	Juan Valera
                         * @date	27/10/2017
                         * @return	true if successful.
                         */
                        virtual bool turnGreenLedOn();

                        /**
                         * @fn	virtual bool turnGreenLedOff();
                         * @brief	Turn off the green led.
                         * @author	Juan Valera
                         * @date	27/10/2017
                         * @return	true if successful.
                         */
                        virtual bool turnGreenLedOff();

                        /**
                         * @fn	virtual bool turnRedLedOn();
                         * @brief	Turn on the red led.
                         * @author	Juan Valera
                         * @date	27/10/2017
                         * @return	true if successful.
                         */
                        virtual bool turnRedLedOn();

                        /**
                         * @fn	virtual bool turnRedLedOff();
                         * @brief	Turn off the red led.
                         * @author	Juan Valera
                         * @date	27/10/2017
                         * @return	true if successful.
                         */
                        virtual bool turnRedLedOff();

                        /**
                         * @fn  virtual bool turnRedLedOnIndefinitely();
                         * @brief   Turn on the red led indefinitely.
                         * @author  Eduardo Fernández
                         * @date    26/12/2017
                         * @return  true if successful.
                         */
                        virtual bool turnRedLedOnIndefinitely();

                        /**
                         * @fn	virtual bool blinkGreenLedOn();
                         * @brief	Turn on and blink the green led.
                         * @author	Juan Valera
                         * @date	27/10/2017
                         * @return	true if successful.
                         */
                        virtual bool blinkGreenLedOn();

                        /**
                         * @fn  virtual bool blinkGreenLedOnWithRedLedTurnOnAfter();
                         * @brief   Turn on and blink the green led, and then turn on red led for an infinite time.
                         * @author  Juan Valera
                         * @date    27/10/2017
                         * @return  true if successful.
                         */
                        virtual bool blinkGreenLedOnWithRedLedTurnOnAfter();

                        /**
                         * @fn	virtual bool blinkGreenLedOff();
                         * @brief	Turn off the green led.
                         * @author	Juan Valera
                         * @date	27/10/2017
                         * @return	true if successful.
                         */
                        virtual bool blinkGreenLedOff();

                        /**
                         * @fn	virtual bool blinkRedLedOn();
                         * @brief	Turn on and blink the red led.
                         * @author	Juan Valera
                         * @date	27/10/2017
                         * @return	true if successful.
                         */
                        virtual bool blinkRedLedOn();

                        /**
                         * @fn	virtual bool blinkRedLedOff();
                         * @brief	Turn off the red led.
                         * @author	Juan Valera
                         * @date	27/10/2017
                         * @return	true if successful.
                         */
                        virtual bool blinkRedLedOff();

                        /**
                         * @fn	virtual bool openDoorForAFiniteTime();
                         * @brief	Open the Eurn door for a finite time.
                         * @author	Juan Valera
                         * @date	27/10/2017
                         * @return	true if successful.
                         */
                        virtual bool openDoorForAFiniteTime();

                        /**
                         * @fn  virtual bool openDoor();
                         * @brief   Open the Eurn door.
                         * @author  Juan Valera
                         * @date    27/10/2017
                         * @return  true if successful.
                         */
                        virtual bool openDoor();

                        /**
                         * @fn  virtual bool openDoorForAFiniteTimeWithLedActions();
                         * @brief   Open the Eurn door and execute special led actions, during and after the configured time.
                         * @author  Juan Valera
                         * @date    27/10/2017
                         * @return  true if successful.
                         */
                        virtual bool openDoorForAFiniteTimeWithLedActions();
                        /**
                         * @fn	virtual bool turnGreenLedOff();
                         * @brief	Close the Eurn door.
                         * @author	Juan Valera
                         * @date	27/10/2017
                         * @return	true if successful.
                         */
                        virtual bool closeDoor();


                    private:

                        Smartmatic::SAES::HW::Connections::IConnection * connection; ///< The connection provider
                        static Smartmatic::Log::ISMTTLog * logger;  ///< The logger

                    protected:

                        bool processSendCommand; ///< True if get result from command
                        Smartmatic::SAES::HW::Configuration::EurnDeviceConfiguration & configuration;
                };
            }
        }
    }
}
#endif /* EURNDEVICE_HXX_ */
