/**
 * @file FactoryDevice.cxx
 * @brief Body class to make all devices for saeshardware project
 * @date 15/06/2015
 * @author Yadickson Soto
 */

#include "FactoryDevice.hxx"
#include <Connections/FactoryConnection.hxx>
#include <Connections/SerialPortConnection.hxx>
#include <Devices/EurnDevice.hxx>
#include <Devices/AbledDevice.hxx>
#include <Devices/BarcodeReaderDevice.hxx>
#include <Devices/P168ADevice.hxx>
#include <Devices/AlarmBoxDevice.hxx>
#include <Devices/SimulatorAlarmBoxDevice.hxx>
#include <Devices/SimulatorP168ADevice.hxx>
#include <Devices/SimulatorBarcodeReaderDevice.hxx>
#include <Configuration/AlarmBoxConfigurationManager.hxx>
#include <Configuration/P168AConfigurationManager.hxx>
#include <Configuration/BarcodeDeviceConfigurationManager.hxx>
#include <Configuration/EurnDeviceConfigurationManager.hxx>
#include <Configuration/EurnDeviceSchema.hxx>

using namespace Smartmatic::SAES::HW::Devices;
using namespace Smartmatic::SAES::HW::Connections;
using namespace Smartmatic::SAES::HW::Configuration;

BarcodeInterface * FactoryDevice::getBarcodeReaderDevice()
{
    if ( BarcodeDeviceConfigurationManager::getInstance()->simulationEnabled() )
    {
        return getSimulatorBarcodeReader();
    }
    else
    {
        return getBarcodeReader();
    }
}

EurnInterface * FactoryDevice::getEurnDevice()
{
    return getEurn( );
}

P168AInterface * FactoryDevice::getP168ADevice()
{
    if ( P168AConfigurationManager::getInstance()->simulationEnabled() )
    {
        return getSimulatorP168A();
    }
    else
    {
        return getP168A();
    }
}

AlarmBoxInterface * FactoryDevice::getAlarmBoxDevice()
{
    if ( AlarmBoxConfigurationManager::getInstance()->simulationEnabled() )
    {
        return getSimulatorAlarmBox();
    }
    else
    {
        return getAlarmBox();
    }
}

BarcodeInterface * FactoryDevice::getBarcodeReader()
{
    IConnection * barcodeDeviceConnection = FactoryConnection::createBarcodeDeviceConnection();
    SerialPortConnection * connection = new SerialPortConnection ( barcodeDeviceConnection );
    BarcodeReaderDevice * device = new BarcodeReaderDevice ( connection );
    connection->setListener ( device );

    return device;
}

EurnInterface * FactoryDevice::getEurn()
{
    IConnection * eurnDeviceConnection = FactoryConnection::createEurnDeviceConnection();
    EurnDevice * device = new EurnDevice ( eurnDeviceConnection );

    return device;
}

BarcodeInterface * FactoryDevice::getSimulatorBarcodeReader()
{
    std::string modelFile = BarcodeDeviceConfigurationManager::getInstance()->getModelFile();
    std::string serialFile = BarcodeDeviceConfigurationManager::getInstance()->getSerialFile();
    std::string commandsFile = BarcodeDeviceConfigurationManager::getInstance()->getCommandsFile();
    std::string bufferFile = BarcodeDeviceConfigurationManager::getInstance()->getBufferFile();

    return new SimulatorBarcodeReaderDevice ( modelFile, serialFile, commandsFile, bufferFile );
}

P168AInterface * FactoryDevice::getP168A()
{
    IConnection * p168aConnection = FactoryConnection::createP168ADeviceConnection();
    SerialPortConnection * connection = new SerialPortConnection ( p168aConnection );

    return new P168ADevice ( connection );
}

P168AInterface * FactoryDevice::getSimulatorP168A()
{
    std::string serialFile = P168AConfigurationManager::getInstance()->getSerialMachineFile();
    std::string buttonsFile = P168AConfigurationManager::getInstance()->getAbleDButtonsFile();

    return new SimulatorP168ADevice ( serialFile, buttonsFile );
}

AlarmBoxInterface * FactoryDevice::getAlarmBox()
{
    IConnection * alarmBoxConnection = FactoryConnection::createAlarmBoxDeviceConnection();
    SerialPortConnection * connection = new SerialPortConnection ( alarmBoxConnection );

    return new AlarmBoxDevice ( connection );
}

AlarmBoxInterface * FactoryDevice::getSimulatorAlarmBox()
{
    std::string pingFile = AlarmBoxConfigurationManager::getInstance()->getAlarmBoxPingFile();
    std::string buttonFile = AlarmBoxConfigurationManager::getInstance()->getAlarmBoxButtonFile();
    std::string behaviorFile = AlarmBoxConfigurationManager::getInstance()->getAlarmBoxBehaviorFile();

    return new SimulatorAlarmBoxDevice ( pingFile, buttonFile, behaviorFile );
}

PDBDevice * FactoryDevice::getPDBDevice()
{
    IConnection * connection = FactoryConnection::createPDBDeviceConnection();

    return new PDBDevice(connection);
}

IOBDevice * FactoryDevice::getIOBDevice()
{
    IConnection * connection = FactoryConnection::createIOBDeviceConnection();

    return new IOBDevice(connection);
}

AbledInterface * FactoryDevice::getAbledDevice()
{
    IConnection * connection = FactoryConnection::createAbledDeviceConnection();

    return new AbledDevice(connection);
}
