/**
 * @file FactoryDevice.hxx
 * @brief Factory device class to make all devices for saeshardware project
 * @date 15/06/2015
 * @author Yadickson Soto
 */

#ifndef FactoryDevice_HXX_
#define FactoryDevice_HXX_

#include <Devices/EurnInterface.hxx>
#include <Devices/AbledInterface.hxx>
#include <Devices/BarcodeInterface.hxx>
#include <Devices/P168AInterface.hxx>
#include <Devices/AlarmBoxInterface.hxx>
#include <Devices/PDBDevice.hxx>
#include <Devices/IOBDevice.hxx>
#include <Devices/AbledDevice.hxx>

namespace Smartmatic
{
    namespace SAES
    {
        namespace HW
        {
            namespace Devices
            {
                /**
                 * @class FactoryDevice
                 * @brief Class to make all devices for saeshardware project
                 * @date 15/06/2015
                 * @author Yadickson Soto
                 */
                class FactoryDevice
                {
                    public:

                        /**
                         * @fn	static BarcodeInterface * getBarcodeReaderDevice();
                         * @brief	Gets a unique barcode reader device instance.
                         * @author	Yadickson Soto
                         * @date 15/06/2015
                         * @return barcode reader instance
                         */
                        static BarcodeInterface * getBarcodeReaderDevice();

                        /**
                         * @fn	static EurnInterface * getEurnDevice();
                         * @brief	Gets a unique Eurn device instance.
                         * @author	Juan Valera
                         * @date 26/10/2015
                         * @return Eurn reader instance
                         */
                        static EurnInterface * getEurnDevice();

                        /**
                         * @fn	static P168AInterface * getP168ADevice();
                         * @brief	Gets a unique p168a device instance.
                         * @author	Yadickson Soto
                         * @date 30/06/2015
                         * @return p168a reader instance
                         */
                        static P168AInterface * getP168ADevice();

                        /**
                         * @fn	static AlarmBoxInterface * getAlarmBoxDevice();
                         * @brief	Gets a unique alarmbox device instance.
                         * @author	Yadickson Soto
                         * @date 30/06/2015
                         * @return alarmbox reader instance
                         */
                        static AlarmBoxInterface * getAlarmBoxDevice();

                        /**
                         * @brief  Gets the PDB device.
                         * @author manuel.arguelles
                         * @date   Nov  1, 2017
                         * @return An instance to the PDB device.
                         */
                        static PDBDevice * getPDBDevice();

                        /**
                         * @brief  Gets the IOB device.
                         * @author eduardo.fernandez
                         * @date   Jan 2, 2018
                         * @return An instance to the IOB device.
                         */
                        static IOBDevice * getIOBDevice();

                        /**
                         * @brief  Gets the abled device.
                         * @author eduardo.fernandez
                         * @date   Jan 11, 2018
                         * @return An instance to the abled device.
                         */
                        static AbledInterface * getAbledDevice();

                    private:

                        /**
                         * @fn	static SmartcardInterface * getBarcodeReader();
                         * @brief	Gets a barcode reader implementation class.
                         * @author	Yadickson Soto
                         * @date	15/06/2015
                         * @return barcode reader instance
                         */
                        static BarcodeInterface * getBarcodeReader();

                        /**
                         * @fn	static EurnInterface * getEurn();
                         * @brief	Gets a Eurn implementation class.
                         * @author	Juan Valera
                         * @date	26/10/2017
                         * @return Eurn instance
                         */
                        static EurnInterface * getEurn();

                        /**
                         * @fn	static BarcodeInterface * getSimulatorBarcodeReader();
                         * @brief	Gets a Simulator barcode reader device.
                         * @author	Yadickson Soto
                         * @date	15/06/2015
                         * @return barcode reader instance
                         */
                        static BarcodeInterface * getSimulatorBarcodeReader();

                        /**
                         * @fn	static P168AInterface * getP168A();
                         * @brief	Gets a unique p168a device instance.
                         * @author	Yadickson Soto
                         * @date 30/06/2015
                         * @return p168a reader instance
                         */
                        static P168AInterface * getP168A();

                        /**
                         * @fn	static P168AInterface * getP168A();
                         * @brief	Gets a Simulator p168a device instance.
                         * @author	Yadickson Soto
                         * @date 30/06/2015
                         * @return p168a reader instance
                         */
                        static P168AInterface * getSimulatorP168A();

                        /**
                         * @fn	static AlarmBoxInterface * getAlarmBox();
                         * @brief	Gets a unique alarmbox device instance.
                         * @author	Yadickson Soto
                         * @date 30/06/2015
                         * @return alarmbox reader instance
                         */
                        static AlarmBoxInterface * getAlarmBox();

                        /**
                         * @fn	static AlarmBoxInterface * getSimulatorAlarmBox();
                         * @brief	Gets a simulator alarmbox device instance.
                         * @author	Yadickson Soto
                         * @date 30/06/2015
                         * @return alarmbox reader instance
                         */
                        static AlarmBoxInterface * getSimulatorAlarmBox();
                };

            } /* namespace Devices */
        } /* namespace HW */
    } /* namespace SAES */
} /* namespace Smartmatic */

#endif /* FactoryDevice_HXX_ */
