/**
 * @file   IOBDevice.cxx
 * @brief  Implementation of IOBDevice class.
 * @author eduardo.fernandez
 * @date   Jan 2, 2017
 */

#include "IOBDevice.hxx"
#include <Commands/IOBDisableUsbCommand.hxx>
#include <Commands/IOBEnableUsbCommand.hxx>
#include <Commands/IOBGetSerialNumberCommand.hxx>
#include <Commands/IOBSetSerialNumberCommand.hxx>
#include <Commands/IOBFWVersionCommand.hxx>
#include <Configuration/IOBDeviceConfigurationManager.hxx>
#include <System/Utils/StringUtils.hxx>
#include <Log/ISMTTLog.h>
#include <Log/SMTTLogManager.h>

using namespace Smartmatic::Log;
using namespace Smartmatic::System;
using namespace Smartmatic::System::Utils;
using namespace Smartmatic::SAES::HW::Devices;
using namespace Smartmatic::SAES::HW::Configuration;
using namespace Smartmatic::SAES::HW::Connections;
using namespace Smartmatic::SAES::HW::Commands;

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

ISMTTLog * IOBDevice::logger = SMTTLogManager::GetLogger ( "Smartmatic.SAES.HW.Devices.IOBDevice", GETTEXT_PACKAGE );

IOBDevice::IOBDevice(IConnection* connection)
    : m_connection(connection)
{
}

IOBDevice::~IOBDevice()
{
}

bool IOBDevice::open()
{
    return m_connection->openConnection("");
}

void IOBDevice::close()
{
    m_connection->closeConnection();
}

IOBUsbPort IOBDevice::ConvertConfigurationToActualPort(IOBPortType IOBPortConfigurationType)
{
    IOBUsbPort usbPort;

    switch(IOBPortConfigurationType)
    {
        case (IOBPortType::IOB_DEVICE_USB_1):
            usbPort = IOB_USB_1;
            break;
        case (IOBPortType::IOB_DEVICE_USB_2):
            usbPort = IOB_USB_2;
            break;
        case (IOBPortType::IOB_DEVICE_USB_3):
            usbPort = IOB_USB_3;
            break;
        case (IOBPortType::IOB_DEVICE_USB_4):
            usbPort = IOB_USB_4;
            break;
        case (IOBPortType::IOB_DEVICE_USB_5):
            usbPort = IOB_USB_5;
            break;
        case (IOBPortType::IOB_DEVICE_USB_6):
            usbPort = IOB_USB_6;
            break;
        case (IOBPortType::IOB_DEVICE_USB_7):
            usbPort = IOB_USB_7;
            break;
        case (IOBPortType::IOB_DEVICE_USB_8):
            usbPort = IOB_USB_8;
            break;
        case (IOBPortType::IOB_DEVICE_USB_9):
            usbPort = IOB_USB_9;
            break;
        case (IOBPortType::IOB_DEVICE_USB_10):
            usbPort = IOB_USB_10;
            break;
        case (IOBPortType::IOB_DEVICE_USB_11):
            usbPort = IOB_USB_11;
            break;
        case (IOBPortType::IOB_DEVICE_USB_12):
            usbPort = IOB_USB_12;
            break;
        default:
            usbPort = IOB_USB_10;
            break;
    }

    return usbPort;
}

bool IOBDevice::disableSmartcardReader()
{
    bool result = false;
    IOBUsbPort usbPort;

    usbPort = ConvertConfigurationToActualPort(IOBDeviceConfigurationManager::getInstance()->getIOBPortForVIUSCRPowerCycle());

    IOBDisableUsbCommand command(m_connection.get(), usbPort);

    if (command.execute())
    {
        result = true;
    }

    return result;
}

bool IOBDevice::enableSmartcardReader()
{
    bool result = false;
    IOBUsbPort usbPort;

    usbPort = ConvertConfigurationToActualPort(IOBDeviceConfigurationManager::getInstance()->getIOBPortForVIUSCRPowerCycle());

    IOBEnableUsbCommand command(m_connection.get(), usbPort);

    if (command.execute())
    {
        result = true;
    }

    return result;
}

std::string IOBDevice::getFWVersion()
{
    std::string version;
    IOBFWVersionCommand command(m_connection.get());

    logger->Debug("Get version from IOB");

    if (command.execute())
    {
        std::size_t HEADER_SIZE     = 24;
        std::size_t NOISE_SIZE      = 2;
        std::size_t COMMAND_ID_SIZE = 1;
        std::vector<unsigned char> response(
            command.getResponse().begin() + (HEADER_SIZE + NOISE_SIZE + COMMAND_ID_SIZE),
            command.getResponse ().end());

        version = StringUtils::arrayHexToString(response);
    }

    logger->Debug("Result : " + version);

    return version;
}

std::string IOBDevice::getSerialNumber()
{
    std::string serial;
    IOBGetSerialNumberCommand command(m_connection.get());

    logger->Debug("Get serial number from IOB");

    if (command.execute())
    {
        std::size_t HEADER_SIZE     = 24;
        std::size_t NOISE_SIZE      = 2;
        std::size_t COMMAND_ID_SIZE = 1;
        std::vector<unsigned char> response(
            command.getResponse().begin() + (HEADER_SIZE + NOISE_SIZE + COMMAND_ID_SIZE),
            command.getResponse ().end());

        serial = StringUtils::arrayHexToAsciiString(response);
    }

    logger->Debug("Result : " + serial);

    return serial;
}

bool IOBDevice::setSerialNumber(const std::string& serial)
{
    std::vector <unsigned char> data;
    StringUtils::asciiStringToArrayHex(serial, data);
    IOBSetSerialNumberCommand command(m_connection.get(), data);

    logger->Debug("Set serial number to IOB : " + serial);

    return command.execute ();
}
