/**
 * @file   IOBDevice.hxx
 * @brief  Declaration of IOBDevice class.
 * @author eduardo.fernandez
 * @date   Jan 2, 2018
 */

#ifndef IOB_DEVICE_HXX
#define IOB_DEVICE_HXX

#include <string>
#include <memory>
#include <Commands/IOBUsbPort.hxx>
#include <Configuration/IOBDeviceSchema.hxx>

using namespace Smartmatic::SAES::HW::Commands;
using namespace Smartmatic::SAES::HW::Configuration;

namespace Smartmatic
{
    namespace Log
    {
        class ISMTTLog;
    }

    namespace SAES
    {
        namespace HW
        {
            namespace Connections
            {
                class IConnection;
            }

            namespace Devices
            {
                /**
                 * @class  IOBDevice
                 * @brief  IOB Device controller.
                 * @author eduardo.fernandez
                 * @date   Jan 2, 2017
                 */
                class IOBDevice
                {
                    public:

                        /**
                         * @brief  Create an instance of the class.
                         * @author eduardo.fernandez
                         * @date   Jan 2, 2017
                         * @param  connection The connection to use.
                         * @remark Takes ownership of the IConnection pointer
                         */
                        explicit IOBDevice(HW::Connections::IConnection* connection);

                        /**
                         * @brief  Destroys an instance of the class.
                         * @author eduardo.fernandez
                         * @date   Jan 2, 2017
                         */
                        virtual ~IOBDevice();

                    public:

                        /**
                         * @brief  Opens a connection with the device.
                         * @author eduardo.fernandez
                         * @date   Jan 2, 2017
                         * @return True if the connection was established.
                         */
                        bool open();

                        /**
                         * @brief  Closes a connection with the device.
                         * @author eduardo.fernandez
                         * @date   Jan 2, 2017
                         */
                        void close();

                        /**
                         * @brief  Disables the smartcard reader.
                         * @author eduardo.fernandez
                         * @date   Jan 2, 2017
                         * @return True if the operation succeed, false if not.
                         */
                       bool disableSmartcardReader();

                        /**
                         * @brief  Enables the smartcard reader.
                         * @author eduardo.fernandez
                         * @date   Jan 2, 2017
                         * @return True if the operation succeed, false if not.
                         */
                        bool enableSmartcardReader();

                        /**
                         * @brief  Converts the value derived from the configuration into an actual PDB usb port
                         * @author eduardo.fernandez
                         * @date   Jan 2, 2017
                         * @return the actual PDB port
                        */
                        IOBUsbPort ConvertConfigurationToActualPort(IOBPortType IOBPortConfigurationType);

                        /**
                         * @brief  Gets the FW version from the PDB.
                         * @author eduardo.fernandez
                         * @date   Jan 2, 2017
                         * @return The FW version is successful, empty string if not.
                         */
                        std::string getFWVersion();

                        /**
                         * @brief  Gets the serial number from the PDB's EEPROM.
                         * @author eduardo.fernandez
                         * @date   Jan 2, 2017
                         * @return The serial number is successful, empty string if not.
                         */
                        std::string getSerialNumber();

                        /**
                         * @brief  Sets the serial number to the PDB's EEPROM.
                         * @author eduardo.fernandez
                         * @date   Jan 2, 2017
                         * @param  serial The serial value to set.
                         * @return True if the operation succeed, false if not.
                        */
                        bool setSerialNumber(const std::string& serial);

                    private:

                        std::auto_ptr<HW::Connections::IConnection> m_connection;
                        static Smartmatic::Log::ISMTTLog * logger;  ///< The logger
                };

            } /* namespace Devices */
        } /* namespace HW */
    } /* namespace SAES */
} /* namespace Smartmatic */

#endif /* IOB_DEVICE_HXX */

