/**
 * @file	P168ADevice.hxx
 * @brief	Declares the p168a device class.
 * @author	Yadickson Soto
 * @date	30/06/2015
 */

#ifndef _P168A_DEVICE_H_
#define _P168A_DEVICE_H_

#include <Connections/IConnection.hxx>
#include <Devices/P168AInterface.hxx>

namespace Smartmatic
{
    namespace Log
    {
        class ISMTTLog;
    }

    namespace SAES
    {
        namespace HW
        {
            namespace Devices
            {
                /**
                 * @class	P168ADevice
                 * @brief	Manage the configuration of the P168A device.
                 * @author	Yadickson Soto
                 * @date	30/06/2015
                 */
                class P168ADevice : public P168AInterface
                {
                    public:

                        /**
                         * @fn	P168ADevice( Smartmatic::SAES::HW::Connections::IConnection * connection );
                         * @brief	Default constructor.
                         * @author	Yadickson Soto
                         * @date	30/06/2015
                         * @param connection The serial connection to device
                         */
                        explicit P168ADevice ( Smartmatic::SAES::HW::Connections::IConnection * connection );

                        /**
                         * @fn	virtual ~P168ADevice()
                         * @brief	Class destroyer.
                         * @author	Yadickson Soto
                         * @date	30/06/2015
                         */
                        virtual ~P168ADevice();

                        /**
                         * @fn	virtual bool open( const std::string & device );
                         * @brief	Opens the connections.
                         * @author	Yadickson Soto
                         * @date	30/06/2015
                         * @param 	device device path
                         * @return	true if successful.
                         */
                        virtual bool open ( const std::string & device );

                        /**
                         * @fn	virtual bool close();
                         * @brief	Closes the connections.
                         * @author	Yadickson Soto
                         * @date	30/06/2015
                         * @return	true if successful.
                         */
                        virtual bool close();

                        /**
                         * @fn	virtual std::string getSerialMachine();
                         * @brief	Method getter serial machine.
                         * @author	Yadickson Soto
                         * @date	30/06/2015
                         * @return	serial machine.
                         */
                        virtual std::string getSerialMachine();

                        /**
                         * @fn	virtual bool setSerialMachine ( const std::string & serial );
                         * @brief	Method setter serial machine.
                         * @author	Yadickson Soto
                         * @date	30/06/2015
                         * @param	serial	serial machine.
                         * return  true if success
                         */
                        virtual bool setSerialMachine ( const std::string & serial );

                        /**
                         * @fn  virtual bool enableVolumeControl ();
                         * @brief   Method enable volume control.
                         * @author  eduardo.fernandez
                         * @date    24/01/2018
                         * return  true if success
                         */
                        virtual bool enableVolumeControl ();

                        /**
                         * @fn	virtual Smartmatic::SAES::HW::Operation::AbleButtonState * getAbleDButtonState();
                         * @brief	Method getter able-d button status.
                         * @author	Yadickson Soto
                         * @date	30/06/2015
                         * @return AbleD button status
                         */
                        virtual Smartmatic::SAES::HW::Operation::AbleDButtonState * getAbleDButtonState ();

                    private:

                        Smartmatic::SAES::HW::Connections::IConnection * connection;	///< The connection provider
                        Smartmatic::SAES::HW::Operation::AbleDButtonState * buttonStatus; ///< The Able-D button status
                        static Smartmatic::Log::ISMTTLog * logger;  ///< The logger
                };
            }
        }
    }
}

#endif // _P168_A_DEVICE_H_
