/**
 * @file   PDBDevice.cxx
 * @brief  Implementation of PDBDevice class.
 * @author manuel.arguelles
 * @date   Oct 31, 2017
 */

#include "PDBDevice.hxx"
#include <Commands/PDBDisableUsbCommand.hxx>
#include <Commands/PDBEnableUsbCommand.hxx>
#include <Commands/PDBGetSerialNumberCommand.hxx>
#include <Commands/PDBSetSerialNumberCommand.hxx>
#include <Commands/PDBControlLEDSCommand.hxx>
#include <Commands/PDBFWVersionCommand.hxx>
#include <Configuration/PDBDeviceConfigurationManager.hxx>
#include <System/OptParser.hxx>
#include <System/Utils/StringUtils.hxx>
#include <Log/ISMTTLog.h>
#include <Log/SMTTLogManager.h>

using namespace Smartmatic::Log;
using namespace Smartmatic::System;
using namespace Smartmatic::System::Utils;
using namespace Smartmatic::SAES::HW::Devices;
using namespace Smartmatic::SAES::HW::Configuration;
using namespace Smartmatic::SAES::HW::Connections;
using namespace Smartmatic::SAES::HW::Commands;

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

ISMTTLog * PDBDevice::logger = SMTTLogManager::GetLogger ( "Smartmatic.SAES.HW.Devices.PDBDevice", GETTEXT_PACKAGE );

PDBDevice::PDBDevice(IConnection* connection)
    : m_connection(connection)
{
}

PDBDevice::~PDBDevice()
{
}

bool PDBDevice::open()
{
    return m_connection->openConnection("");
}

void PDBDevice::close()
{
    m_connection->closeConnection();
}

PDBUsbPort PDBDevice::ConvertConfigurationToActualPort(PDBPortType PDBPortConfigurationType)
{
    PDBUsbPort usbPort;

    switch(PDBPortConfigurationType)
    {
        case (PDBPortType::PDB_DEVICE_USB_1):
            usbPort = USB_1;
            break;
        case (PDBPortType::PDB_DEVICE_USB_2):
            usbPort = USB_2;
            break;
        case (PDBPortType::PDB_DEVICE_USB_3):
            usbPort = USB_3;
            break;
        case (PDBPortType::PDB_DEVICE_USB_4):
            usbPort = USB_4;
            break;
        case (PDBPortType::PDB_DEVICE_USB_5):
            usbPort = USB_5;
            break;
        case (PDBPortType::PDB_DEVICE_USB_6):
            usbPort = USB_6;
            break;
        case (PDBPortType::PDB_DEVICE_USB_7):
            usbPort = USB_7;
            break;
        case (PDBPortType::PDB_DEVICE_USB_8):
            usbPort = USB_8;
            break;
        case (PDBPortType::PDB_DEVICE_USB_9):
            usbPort = USB_9;
            break;
        case (PDBPortType::PDB_DEVICE_USB_10):
            usbPort = USB_10;
            break;
        case (PDBPortType::PDB_DEVICE_USB_11):
            usbPort = USB_11;
            break;
        case (PDBPortType::PDB_DEVICE_USB_12):
            usbPort = USB_12;
            break;
        default:
            usbPort = USB_10;
            break;
    }

    return usbPort;
}

bool PDBDevice::disableSmartcardReader()
{
    bool result = false;
    std::string machineModel;
    PDBUsbPort usbPort;

    OptParser::getInstance()->getArg(OptParser::MACHINE_MODEL, machineModel);

    if (StringUtils::toUpper(machineModel) == "PM-VIU805")
    {
        usbPort = ConvertConfigurationToActualPort(PDBDeviceConfigurationManager::getInstance()->getPDBPortForVIUSCRPowerCycle());
    }
    else
    {
         usbPort = ConvertConfigurationToActualPort(PDBDeviceConfigurationManager::getInstance()->getPDBPortForA4517SCRPowerCycle());
    }

    PDBDisableUsbCommand command(m_connection.get(), usbPort);

    if (command.execute())
    {
        result = true;
    }

    return result;
}

bool PDBDevice::enableSmartcardReader()
{
    bool result = false;
    std::string machineModel;
    PDBUsbPort usbPort;

    OptParser::getInstance()->getArg(OptParser::MACHINE_MODEL, machineModel);

    if (StringUtils::toUpper(machineModel) == "PM-VIU805")
    {
        usbPort = ConvertConfigurationToActualPort(PDBDeviceConfigurationManager::getInstance()->getPDBPortForVIUSCRPowerCycle());
    }
    else
    {
        usbPort = ConvertConfigurationToActualPort(PDBDeviceConfigurationManager::getInstance()->getPDBPortForA4517SCRPowerCycle());
    }

    PDBEnableUsbCommand command(m_connection.get(), usbPort);

    if (command.execute())
    {
        result = true;
    }

    return result;
}

std::string PDBDevice::getFWVersion()
{
    std::string version;
    PDBFWVersionCommand command(m_connection.get());

    logger->Debug("Get version from PDB");

    if (command.execute())
    {
        std::size_t HEADER_SIZE     = 24;
        std::size_t NOISE_SIZE      = 2;
        std::size_t COMMAND_ID_SIZE = 1;
        std::vector<unsigned char> response(
            command.getResponse().begin() + (HEADER_SIZE + NOISE_SIZE + COMMAND_ID_SIZE),
            command.getResponse ().end());

        version = StringUtils::arrayHexToString(response);
    }

    logger->Debug("Result : " + version);

    return version;
}

std::string PDBDevice::getSerialNumber()
{
    std::string serial;
    PDBGetSerialNumberCommand command(m_connection.get());

    logger->Debug("Get serial number from PDB");

    if (command.execute())
    {
        std::size_t HEADER_SIZE     = 24;
        std::size_t NOISE_SIZE      = 2;
        std::size_t COMMAND_ID_SIZE = 1;
        std::vector<unsigned char> response(
            command.getResponse().begin() + (HEADER_SIZE + NOISE_SIZE + COMMAND_ID_SIZE),
            command.getResponse ().end());

        serial = StringUtils::arrayHexToAsciiString(response);
    }

    logger->Debug("Result : " + serial);

    return serial;
}

bool PDBDevice::setSerialNumber(const std::string& serial)
{
    std::vector <unsigned char> data;
    StringUtils::asciiStringToArrayHex(serial, data);
    PDBSetSerialNumberCommand command(m_connection.get(), data);

    logger->Debug("Set serial number to PDB : " + serial);

    return command.execute ();
}

bool PDBDevice::turnAllLedsOn()
{
    PDBControlLEDSCommand command(m_connection.get(), LED_ALL, true);
    logger->Debug("turning ON all LEDs in the PDB");
    return command.execute ();
}

bool PDBDevice::turnAllLedsOff()
{
    PDBControlLEDSCommand command(m_connection.get(), LED_ALL, false);
    logger->Debug("turning OFF all LEDs in the PDB");
    return command.execute ();
}
