/**
 * @file   PDBDevice.hxx
 * @brief  Declaration of PDBDevice class.
 * @author manuel.arguelles
 * @date   Oct 31, 2017
 */

#ifndef PDB_DEVICE_HXX
#define PDB_DEVICE_HXX

#include <string>
#include <memory>
#include <Commands/PDBUsbPort.hxx>
#include <Configuration/PDBDeviceSchema.hxx>

using namespace Smartmatic::SAES::HW::Commands;
using namespace Smartmatic::SAES::HW::Configuration;

namespace Smartmatic
{
    namespace Log
    {
        class ISMTTLog;
    }

    namespace SAES
    {
        namespace HW
        {
            namespace Connections
            {
                class IConnection;
            }

            namespace Devices
            {
                /**
                 * @class  PDBDevice
                 * @brief  PDB Device controller.
                 * @author manuel.arguelles
                 * @date   Oct 31, 2017
                 */
                class PDBDevice
                {
                    public:

                        /**
                         * @brief  Create an instance of the class.
                         * @author manuel.arguelles
                         * @date   Oct 31, 2017
                         * @param  connection The connection to use.
                         * @remark Takes ownership of the IConnection pointer
                         */
                        explicit PDBDevice(HW::Connections::IConnection* connection);

                        /**
                         * @brief  Destroys an instance of the class.
                         * @author manuel.arguelles
                         * @date   Oct 31, 2017
                         */
                        virtual ~PDBDevice();

                    public:

                        /**
                         * @brief  Opens a connection with the device.
                         * @author manuel.arguelles
                         * @date   Oct 31, 2017
                         * @return True if the connection was established.
                         */
                        bool open();

                        /**
                         * @brief  Closes a connection with the device.
                         * @author manuel.arguelles
                         * @date   Nov  2, 2017
                         */
                        void close();

                        /**
                         * @brief  Disables the smartcard reader.
                         * @author manuel.arguelles
                         * @date   Nov  1, 2017
                         * @return True if the operation succeed, false if not.
                         */
                       bool disableSmartcardReader();

                        /**
                         * @brief  Enables the smartcard reader.
                         * @author manuel.arguelles
                         * @date   Nov  1, 2017
                         * @return True if the operation succeed, false if not.
                         */
                        bool enableSmartcardReader();

                        /**
                         * @brief  Converts the value derived from the configuration into an actual PDB usb port
                         * @author manuel.arguelles
                         * @date   Nov  1, 2017
                         * @return the actual PDB port
                        */
                        PDBUsbPort ConvertConfigurationToActualPort(PDBPortType PDBPortConfigurationType);

                        /**
                         * @brief  Gets the FW version from the PDB.
                         * @author manuel.arguelles
                         * @date   Nov  1, 2017
                         * @return The FW version is successful, empty string if not.
                         */
                        std::string getFWVersion();

                        /**
                         * @brief  Gets the serial number from the PDB's EEPROM.
                         * @author manuel.arguelles
                         * @date   Nov  1, 2017
                         * @return The serial number is successful, empty string if not.
                         */
                        std::string getSerialNumber();

                        /**
                         * @brief  Sets the serial number to the PDB's EEPROM.
                         * @author manuel.arguelles
                         * @date   Nov  1, 2017
                         * @param  serial The serial value to set.
                         * @return True if the operation succeed, false if not.
                        */
                        bool setSerialNumber(const std::string& serial);

                        /**
                         * @brief  turn on All LEDs in the PDB
                         * @author manuel.arguelles
                         * @date   Nov  1, 2017
                         * @return True is successful, false if not.
                         */
                        bool turnAllLedsOn();

                        /**
                         * @brief  turn off All LEDs in the PDB
                         * @author manuel.arguelles
                         * @date   Nov  1, 2017
                         * @param  serial The serial value to set.
                         * @return True is successful, false if not.
                        */
                        bool turnAllLedsOff();

                    private:

                        std::auto_ptr<HW::Connections::IConnection> m_connection;
                        static Smartmatic::Log::ISMTTLog * logger;  ///< The logger
                };

            } /* namespace Devices */
        } /* namespace HW */
    } /* namespace SAES */
} /* namespace Smartmatic */

#endif /* PDB_DEVICE_HXX */

