/**
 * @file	SimulatorAlarmBoxDevice.cxx
 * @brief	The simulator alarm device class.
 * @author	Yadickson Soto
 * @date	03/07/2015
 */

#include "SimulatorAlarmBoxDevice.hxx"
#include <Log/ISMTTLog.h>
#include <Log/SMTTLogManager.h>
#include <System/IO/File.h>
#include <System/Utils/StringUtils.hxx>
#include <System/Utils/SafeBufferUtils.hxx>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#define RED_LED 0
#define GREEN_LED 1
#define BUZZER 2

#define RED_LED_TEXT "RedLed:"
#define GREEN_LED_TEXT "GreenLed:"
#define BUZZER_TEXT "Buzzer:"

#define ON_TEXT "ON:"
#define OFF_TEXT "OFF"

using namespace Smartmatic::SAES::HW::Devices;
using namespace Smartmatic::System::IO;
using namespace Smartmatic::System::Utils;
using namespace Smartmatic::System;
using namespace Smartmatic::Log;

ISMTTLog * SimulatorAlarmBoxDevice::logger = SMTTLogManager::GetLogger ( "Smartmatic.SAES.HW.Devices.SimulatorAlarmBoxDevice", GETTEXT_PACKAGE );

SimulatorAlarmBoxDevice::SimulatorAlarmBoxDevice ( const std::string & pingFile, const std::string & buttonFile, const std::string & behaviorFile )
    : pingFile ( pingFile ), buttonFile ( buttonFile ), behaviorFile ( behaviorFile )
{
    resultList.push_back ( RED_LED_TEXT OFF_TEXT );
    resultList.push_back ( GREEN_LED_TEXT OFF_TEXT );
    resultList.push_back ( BUZZER_TEXT OFF_TEXT );
}

SimulatorAlarmBoxDevice::~SimulatorAlarmBoxDevice()
{

}

bool SimulatorAlarmBoxDevice::open ( const std::string & device )
{
    logger->Debug ( "Open simulator alarm box device" );
    return File::getInstance()->canOpenFile ( device );
}

bool SimulatorAlarmBoxDevice::close()
{
    logger->Debug ( "Close simulator alarm box device" );
    return true;
}

bool SimulatorAlarmBoxDevice::sendPing()
{
    bool status = false;
    SafeBuffer data;

    logger->Debug ( "Simulate send ping" );

    if ( File::getInstance()->openFileToBuffer ( pingFile, data ) && data.GetSize() > 0 )
    {
        std::string ping;

        SafeBufferUtils buffUtil ( data );
        std::vector < SafeBuffer > result = buffUtil.splitByEndLine();

        if ( ! result.empty() )
        {
            SafeBuffer item ( result.front() );
            ping.assign ( ( const char * ) item.GetData(), item.GetSize() );
            status = StringUtils::toUpper ( StringUtils::trim ( ping ) ) == "OK";
        }
    }
    else
    {
        logger->Debug ( "Error load file " + pingFile );
    }

    return status;
}

bool SimulatorAlarmBoxDevice::turnOnRedLed ( unsigned char period )
{
    resultList [ RED_LED ] = RED_LED_TEXT ON_TEXT + StringUtils::intToString ( period );
    return saveBehaviorFile();
}

bool SimulatorAlarmBoxDevice::turnOffRedLed()
{
    resultList [ RED_LED ] = RED_LED_TEXT OFF_TEXT;
    return saveBehaviorFile();
}

bool SimulatorAlarmBoxDevice::turnOnGreenLed ( unsigned char period )
{
    resultList [ GREEN_LED ] = GREEN_LED_TEXT ON_TEXT + StringUtils::intToString ( period );
    return saveBehaviorFile();
}

bool SimulatorAlarmBoxDevice::turnOffGreenLed()
{
    resultList [ GREEN_LED ] = GREEN_LED_TEXT OFF_TEXT;
    return saveBehaviorFile();
}

bool SimulatorAlarmBoxDevice::turnOnBuzzer ( unsigned char period )
{
    resultList [ BUZZER ] = BUZZER_TEXT ON_TEXT + StringUtils::intToString ( period );
    return saveBehaviorFile();
}

bool SimulatorAlarmBoxDevice::turnOffBuzzer()
{
    resultList [ BUZZER ] = BUZZER_TEXT OFF_TEXT;
    return saveBehaviorFile();
}

bool SimulatorAlarmBoxDevice::isButtonPressed ()
{
    bool status = false;
    SafeBuffer data;

    if ( File::getInstance()->openFileToBuffer ( buttonFile, data ) && data.GetSize() > 0 )
    {
        std::string event;

        SafeBufferUtils buffUtil ( data );
        std::vector < SafeBuffer > result = buffUtil.splitByEndLine();

        if ( ! result.empty() )
        {
            SafeBuffer item ( result.front() );
            event.assign ( ( const char * ) item.GetData(), item.GetSize() );
            status = StringUtils::toUpper ( StringUtils::trim ( event ) ) == "PUSH";
        }
    }
    else
    {
        logger->Debug ( "Error load file " + buttonFile );
    }

    return status;
}

bool SimulatorAlarmBoxDevice::saveBehaviorFile ()
{
    SafeBuffer data;
    std::string temp;

    std::vector < std::string >::iterator it;

    for ( it = resultList.begin();
            it != resultList.end();
            ++it )
    {
        temp.append ( *it );
        temp.append ( " - " );
    }

    temp = StringUtils::replaceLast ( temp, " - ", "" );

    data.AddData ( temp );
    data.AddData ( "\n" );

    return File::getInstance()->addBufferToFile ( data, behaviorFile );
}
