/**
 * @file	SimulatorAlarmBoxDevice.hxx
 * @brief	Declares the simulator alarm device class.
 * @author	Yadickson Soto
 * @date	03/07/2015
 */

#ifndef _SIMULATOR_ALARM_BOX_DEVICE_H_
#define _SIMULATOR_ALARM_BOX_DEVICE_H_

#include <Devices/AlarmBoxInterface.hxx>
#include <vector>
#include <string>

namespace Smartmatic
{
    namespace Log
    {
        class ISMTTLog;
    }

    namespace SAES
    {
        namespace HW
        {
            namespace Devices
            {
                /**
                 * @class	SimulatorAlarmBoxDevice
                 * @brief	Manage all the elements for configuring, activating and deactivating an alarm.
                 * @author	Yadickson Soto
                 * @date	03/07/2015
                 */
                class SimulatorAlarmBoxDevice: public AlarmBoxInterface
                {

                    public:

                        /**
                         * @fn	SimulatorAlarmBoxDevice( const std::string & pingFile, const std::string & buttonFile, const std::string & behaviorFile );
                         * @brief	Default constructor.
                         * @author	Yadickson Soto
                         * @date	03/07/2015
                         * @param   pingFile The ping file path to read ping status
                         * @param   buttonFile The button file path to read button status
                         * @param   behaviorFile The behavior file path to write status leds and buzzer
                         */
                        SimulatorAlarmBoxDevice ( const std::string & pingFile, const std::string & buttonFile, const std::string & behaviorFile );

                        /**
                         * @fn	virtual ~SimulatorAlarmBoxDevice();
                         * @brief	Class destroyer.
                         * @author	Yadickson Soto
                         * @date	03/07/2015
                         */
                        virtual ~SimulatorAlarmBoxDevice();

                        /**
                         * @fn	virtual bool open( const std::string & device );
                         * @brief	Opens the connections.
                         * @author	Yadickson Soto
                         * @date	03/07/2015
                         * @param 	device device path
                         * @return	true if successful.
                         */
                        virtual bool open ( const std::string & device );

                        /**
                         * @fn	virtual bool close();
                         * @brief	Closes the connections.
                         * @author	Yadickson Soto
                         * @date	03/07/2015
                         * @return	true if successful.
                         */
                        virtual bool close();

                        /**
                         * @fn	virtual bool sendPing();
                         * @brief	Sends a ping.
                         * @author	Yadickson Soto
                         * @date	03/07/2015
                         * @return	true if it succeeds, false if it fails.
                         */
                        virtual bool sendPing();

                        /**
                         * @fn	virtual bool turnOnRedLed(unsigned char period);
                         * @brief	Turn on the red led.
                         * @author	Yadickson Soto
                         * @date	03/07/2015
                         * @param	period	The period.
                         */
                        virtual bool turnOnRedLed ( unsigned char period );

                        /**
                         * @fn	virtual bool turnOffRedLed();
                         * @brief	Turn off the red led.
                         * @author	Yadickson Soto
                         * @date	03/07/2015
                         */
                        virtual bool turnOffRedLed();

                        /**
                         * @fn	virtual bool turnOnGreenLed(unsigned char period);
                         * @brief	Turn on the green led.
                         * @author	Yadickson Soto
                         * @date	03/07/2015
                         * @param	period	The period.
                         */
                        virtual bool turnOnGreenLed ( unsigned char period );

                        /**
                         * @fn	virtual bool turnOffGreenLed();
                         * @brief	Turn off the green led.
                         * @author	Yadickson Soto
                         * @date	03/07/2015
                         */
                        virtual bool turnOffGreenLed();

                        /**
                         * @fn	virtual bool turnOnBuzzer(unsigned char period);
                         * @brief	Turn on the buzzer.
                         * @author	Yadickson Soto
                         * @date	03/07/2015
                         * @param	period	The period.
                         */
                        virtual bool turnOnBuzzer ( unsigned char period );

                        /**
                         * @fn	virtual bool turnOffBuzzer();
                         * @brief	Turn off the buzzer.
                         * @author	Yadickson Soto
                         * @date	03/07/2015
                         */
                        virtual bool turnOffBuzzer();

                        /**
                         * @fn	virtual bool isButtonPressed ();
                         * @brief	Check if button is pressed.
                         * @author	Yadickson Soto
                         * @date	03/07/2015
                         * @return	true if button is pressed.
                         */
                        virtual bool isButtonPressed ();

                    protected:

                        /**
                         * @fn	bool saveBehaviorFile ();
                         * @brief	Save behaviour file
                         * @author	Yadickson Soto
                         * @date	03/07/2015
                         * @return true if safe file is success
                         */
                        bool saveBehaviorFile ();

                    private:

                        std::string pingFile; ///< The ping file path to read ping status
                        std::string buttonFile; ///< The button file path to read button status
                        std::string behaviorFile; ///< The behavior file path to write status leds and buzzer

                        std::vector < std::string > resultList; ///< Result behavior list
                        static Smartmatic::Log::ISMTTLog * logger;  ///< The logger

                };
            }
        }
    }
}

#endif // _SIMULATOR_ALARM_BOX_DEVICE_H_

