/**
 * @file	SimulatorP168ADevice.cxx
 * @brief	Body simulator P168A device class.
 * @author	Yadickson Soto
 * @date	07/06/2015
 */

#include "SimulatorP168ADevice.hxx"
#include <System/IO/File.h>
#include <System/Utils/StringUtils.hxx>
#include <System/Utils/SafeBufferUtils.hxx>
#include <Log/ISMTTLog.h>
#include <Log/SMTTLogManager.h>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

using namespace Smartmatic::SAES::HW::Devices;
using namespace Smartmatic::SAES::HW::Operation;
using namespace Smartmatic::System::IO;
using namespace Smartmatic::System::Utils;
using namespace Smartmatic::System;
using namespace Smartmatic::Log;

ISMTTLog * SimulatorP168ADevice::logger = SMTTLogManager::GetLogger ( "Smartmatic.SAES.HW.Devices.SimulatorP168ADevice", GETTEXT_PACKAGE );

SimulatorP168ADevice::SimulatorP168ADevice ( const std::string & serialFile, const std::string & buttonsFile )
    : serialFile ( serialFile ), buttonsFile ( buttonsFile )
{
    buttonStatus = new AbleDButtonState();
}

SimulatorP168ADevice::~SimulatorP168ADevice()
{
    if ( buttonStatus )
    {
        delete ( buttonStatus );
        buttonStatus = NULL;
    }
}

bool SimulatorP168ADevice::open ( const std::string & device )
{
    logger->Debug ( "Simulator Open p168a device" );
    return File::getInstance()->canOpenFile ( device );
}

bool SimulatorP168ADevice::close()
{
    logger->Debug ( "Simulator Close p168a device" );
    return true;
}

std::string SimulatorP168ADevice::getSerialMachine()
{
    logger->Debug ( "Get serial machine" );

    std::string serial;
    SafeBuffer data;

    if ( File::getInstance()->openFileToBuffer ( serialFile, data ) && data.GetSize() > 0 )
    {
        serial.assign ( ( const char * ) data.GetData(), data.GetSize() );
    }
    else
    {
        logger->Debug ( "Error load file " + serialFile );
    }

    logger->Debug ( "Result : " + serial );

    return serial;
}

bool SimulatorP168ADevice::setSerialMachine ( const std::string & serial )
{
    logger->Debug ( "Simulator Set serial machine : " + serial );

    SafeBuffer data ( serial );

    return File::getInstance()->saveBufferToFile ( data, serialFile );
}

bool SimulatorP168ADevice::enableVolumeControl()
{
    return true;
}

Smartmatic::SAES::HW::Operation::AbleDButtonState * SimulatorP168ADevice::getAbleDButtonState ()
{
    SafeBuffer data;

    if ( File::getInstance()->openFileToBuffer ( buttonsFile, data ) && data.GetSize() > 0 )
    {
        SafeBufferUtils buffUtil ( data );
        std::vector < SafeBuffer > result = buffUtil.splitByEndLine();
        std::vector < SafeBuffer >::iterator it;

        for ( it = result.begin();
                it != result.end();
                ++it )
        {
            std::string strType;
            std::string strStatus;

            SafeBufferUtils pair ( *it );

            std::vector < SafeBuffer > fields = pair.splitByBuffer ( SafeBuffer ( ":" ) );

            if ( fields.size() >= 2 )
            {
                strType.assign ( ( const char * ) ( fields[0] ).GetData(), ( fields[0] ).GetSize() );
                strStatus.assign ( ( const char * ) ( fields[1] ).GetData(), ( fields[1] ).GetSize() );

                AbleDButtonState::ButtonType type = buttonStatus->stringToButtonType ( strType );
                AbleDButtonState::ButtonState status = buttonStatus->stringToButtonState ( strStatus );

                logger->Debug ( "Button Event ( " + StringUtils::toHexString ( type, "", 2 ) + " - " + StringUtils::toHexString ( status, "", 2 ) + " )" );

                if ( type != AbleDButtonState::UNKNOW && status != AbleDButtonState::NONE )
                {
                    buttonStatus->updateStatus ( type, status );
                }
            }
        }
    }

    return buttonStatus;
}
