/**
 * @file   PDBDeviceTest.h
 * @brief  Implementation of tests for the PDB device class.
 * @author manuel.arguelles
 * @date   Oct 31, 2017
 */

#ifndef TEST_PDB_DEVICE_TEST_H
#define TEST_PDB_DEVICE_TEST_H

#ifdef ECUT_MAIN

#include <tests/MockConnection.hxx>
#include <cppunit/extensions/HelperMacros.h>
#include <Devices/PDBDevice.hxx>
#include <Connections/UsbConnection.hxx>
#include <tests/MockSAESHWConfigurationManager.hxx>
#include <tests/MockConfigurationPersistenceManager.hxx>

class PDBDeviceTest : public CPPUNIT_NS::TestFixture
{
    private:

        typedef Smartmatic::SAES::HW::Devices::PDBDevice PDBDevice;
        typedef Smartmatic::SAES::HW::Configuration::SAESHardwareConfigurationManager SAESHardwareConfigurationManager;
        typedef Smartmatic::Configuration::ConfigurationPersistenceManager ConfigurationPersistenceManager;

    public:

        PDBDeviceTest() {}

        void setUp() {}

        void tearDown() {}

        void testOpenConnection()
        {
            MockConnection* connection = new MockConnection();
            PDBDevice pdb(connection);

            MOCKPP_CHAINER_FOR_EXT(MockConnection, openConnection, ext) Open_mock(connection);
            MOCKPP_CHAINER_FOR_EXT(MockConnection, sendCommand, ext) Send_mock(connection);
            MOCKPP_CHAINER_FOR(MockConnection, readBuffer) Read_mock(connection);
            MOCKPP_CHAINER_FOR(MockConnection, closeConnection) Close_mock(connection);
            MOCKPP_CHAINER_FOR(MockConnection, getTimeOut) TimeOut_mock(connection);

            Open_mock.expects(MOCKPP_NS::once()).will(MOCKPP_NS::returnValue<bool>(true));
            Send_mock.expects(MOCKPP_NS::never());
            Read_mock.expects(MOCKPP_NS::never());
            Close_mock.expects(MOCKPP_NS::never());
            TimeOut_mock.expects(MOCKPP_NS::never());

            CPPUNIT_ASSERT(pdb.open() == true);
        }

        void testOpenConnectionFail()
        {
            MockConnection* connection = new MockConnection();
            PDBDevice pdb(connection);

            MOCKPP_CHAINER_FOR_EXT(MockConnection, openConnection, ext) Open_mock(connection);
            MOCKPP_CHAINER_FOR_EXT(MockConnection, sendCommand, ext) Send_mock(connection);
            MOCKPP_CHAINER_FOR(MockConnection, readBuffer) Read_mock(connection);
            MOCKPP_CHAINER_FOR(MockConnection, closeConnection) Close_mock(connection);
            MOCKPP_CHAINER_FOR(MockConnection, getTimeOut) TimeOut_mock(connection);

            Open_mock.expects(MOCKPP_NS::once()).will(MOCKPP_NS::returnValue<bool>(false));
            Send_mock.expects(MOCKPP_NS::never());
            Read_mock.expects(MOCKPP_NS::never());
            Close_mock.expects(MOCKPP_NS::never());
            TimeOut_mock.expects(MOCKPP_NS::never());

            CPPUNIT_ASSERT(pdb.open() == false);
        }

        void testCloseConnection()
        {
            MockConnection* connection = new MockConnection();
            PDBDevice pdb(connection);

            MOCKPP_CHAINER_FOR_EXT(MockConnection, openConnection, ext) Open_mock(connection);
            MOCKPP_CHAINER_FOR_EXT(MockConnection, sendCommand, ext) Send_mock(connection);
            MOCKPP_CHAINER_FOR(MockConnection, readBuffer) Read_mock(connection);
            MOCKPP_CHAINER_FOR(MockConnection, closeConnection) Close_mock(connection);
            MOCKPP_CHAINER_FOR(MockConnection, getTimeOut) TimeOut_mock(connection);

            Open_mock.expects(MOCKPP_NS::never());
            Send_mock.expects(MOCKPP_NS::never());
            Read_mock.expects(MOCKPP_NS::never());
            Close_mock.expects(MOCKPP_NS::once());
            TimeOut_mock.expects(MOCKPP_NS::never());

            pdb.close();
        }

        void testDisableSmartcardReader()
        {
            MockConfigurationPersistenceManager manager;
            MockSAESHWConfigurationManager saeshwManager;

            MOCKPP_CHAINER_FOR_EXT ( MockConfigurationPersistenceManager, Initialize, ext ) Initialize_mock ( &manager );

            Initialize_mock
            .expects ( MOCKPP_NS::once() );

            ConfigurationPersistenceManager::getInstance ( &manager );
            SAESHardwareConfigurationManager::getInstance ( &saeshwManager );


            MockConnection* connection = new MockConnection();
            PDBDevice pdb(connection);
            std::vector<unsigned char> cmd;
            Smartmatic::System::SafeVector<unsigned char> cmdReply;

            cmd.push_back(0xAA);
            cmd.push_back(0x55);
            cmd.push_back(0xAA);
            cmd.push_back(0x54);
            cmd.insert(cmd.end(), 16, 0x00);
            cmd.push_back(0x07);
            cmd.insert(cmd.end(), 3, 0x00);
            cmd.insert(cmd.end(), 2, 0xFF);
            cmd.push_back(0x1B);
            cmd.push_back(0x6C);
            cmd.push_back(0x08);
            cmd.push_back(0x00);
            cmd.push_back(0x00);

            cmdReply.push_back(0xAA);
            cmdReply.push_back(0x55);
            cmdReply.push_back(0xAA);
            cmdReply.push_back(0x56);
            cmdReply.insert(cmdReply.end(), 16, 0x00);
            cmdReply.push_back(0x05);
            cmdReply.insert(cmdReply.end(), 3, 0x00);
            cmdReply.insert(cmdReply.end(), 2, 0xFF);
            cmdReply.push_back(0x1B);
            cmdReply.push_back(0x6C);
            cmdReply.push_back(0x00);

            MOCKPP_CHAINER_FOR_EXT(MockConnection, openConnection, ext) Open_mock(connection);
            MOCKPP_CHAINER_FOR_EXT(MockConnection, sendCommand, ext) Send_mock(connection);
            MOCKPP_CHAINER_FOR(MockConnection, readBuffer) Read_mock(connection);
            MOCKPP_CHAINER_FOR(MockConnection, closeConnection) Close_mock(connection);
            MOCKPP_CHAINER_FOR(MockConnection, getTimeOut) TimeOut_mock(connection);

            Open_mock.expects(MOCKPP_NS::once())
                .will(MOCKPP_NS::returnValue<bool>(true));

            Close_mock.expects(MOCKPP_NS::once());
            TimeOut_mock.expects(MOCKPP_NS::atLeastOnce())
                .will(MOCKPP_NS::returnValue<unsigned int>(1000));

            Send_mock.expects(MOCKPP_NS::once())
                .with(MOCKPP_NS::eq(cmd))
                .will(MOCKPP_NS::returnValue<bool>(true));

            Read_mock.expects(MOCKPP_NS::once())
                .will(MOCKPP_NS::returnValue<Smartmatic::System::SafeVector<unsigned char> >(cmdReply));

            CPPUNIT_ASSERT(pdb.open() == true);
            CPPUNIT_ASSERT(pdb.disableSmartcardReader() == true);
            pdb.close();
            connection->verify();
        }

        void testEnableSmartcardReader()
        {
            MockConfigurationPersistenceManager manager;
            MockSAESHWConfigurationManager saeshwManager;

            MOCKPP_CHAINER_FOR_EXT ( MockConfigurationPersistenceManager, Initialize, ext ) Initialize_mock ( &manager );

            Initialize_mock
            .expects ( MOCKPP_NS::once() );

            ConfigurationPersistenceManager::getInstance ( &manager );
            SAESHardwareConfigurationManager::getInstance ( &saeshwManager );

            MockConnection* connection = new MockConnection();
            PDBDevice pdb(connection);
            std::vector<unsigned char> cmd;
            Smartmatic::System::SafeVector<unsigned char> cmdReply;

            cmd.push_back(0xAA);
            cmd.push_back(0x55);
            cmd.push_back(0xAA);
            cmd.push_back(0x54);
            cmd.insert(cmd.end(), 16, 0x00);
            cmd.push_back(0x07);
            cmd.insert(cmd.end(), 3, 0x00);
            cmd.insert(cmd.end(), 2, 0xFF);
            cmd.push_back(0x1B);
            cmd.push_back(0x6C);
            cmd.push_back(0x08);
            cmd.push_back(0x00);
            cmd.push_back(0x01);

            cmdReply.push_back(0xAA);
            cmdReply.push_back(0x55);
            cmdReply.push_back(0xAA);
            cmdReply.push_back(0x56);
            cmdReply.insert(cmdReply.end(), 16, 0x00);
            cmdReply.push_back(0x05);
            cmdReply.insert(cmdReply.end(), 3, 0x00);
            cmdReply.insert(cmdReply.end(), 2, 0xFF);
            cmdReply.push_back(0x1B);
            cmdReply.push_back(0x6C);
            cmdReply.push_back(0x00);

            MOCKPP_CHAINER_FOR_EXT(MockConnection, openConnection, ext) Open_mock(connection);
            MOCKPP_CHAINER_FOR_EXT(MockConnection, sendCommand, ext) Send_mock(connection);
            MOCKPP_CHAINER_FOR(MockConnection, readBuffer) Read_mock(connection);
            MOCKPP_CHAINER_FOR(MockConnection, closeConnection) Close_mock(connection);
            MOCKPP_CHAINER_FOR(MockConnection, getTimeOut) TimeOut_mock(connection);

            Open_mock.expects(MOCKPP_NS::once())
                .will(MOCKPP_NS::returnValue<bool>(true));

            Close_mock.expects(MOCKPP_NS::once());

            TimeOut_mock.expects(MOCKPP_NS::atLeastOnce())
                .will(MOCKPP_NS::returnValue<unsigned int>(1000));

            Send_mock.expects(MOCKPP_NS::once())
                .with(MOCKPP_NS::eq(cmd))
                .will(MOCKPP_NS::returnValue<bool>(true));

            Read_mock.expects(MOCKPP_NS::once())
                .will(MOCKPP_NS::returnValue<Smartmatic::System::SafeVector<unsigned char> >(cmdReply));

            CPPUNIT_ASSERT(pdb.open() == true);
            CPPUNIT_ASSERT(pdb.enableSmartcardReader() == true);
            pdb.close();
            connection->verify();
        }

        CPPUNIT_TEST_SUITE(PDBDeviceTest);

        CPPUNIT_TEST(testOpenConnection);
        CPPUNIT_TEST(testOpenConnectionFail);
        CPPUNIT_TEST(testCloseConnection);
        CPPUNIT_TEST(testDisableSmartcardReader);
        CPPUNIT_TEST(testEnableSmartcardReader);

        CPPUNIT_TEST_SUITE_END();
};

#endif /* ECUT_MAIN */

#endif /* TEST_PDB_DEVICE_TEST_H */
