#ifndef TEST_SIMULATOR_BARCODE_DEVICE_TEST
#define TEST_SIMULATOR_BARCODE_DEVICE_TEST

#ifdef ECUT_MAIN

#include <cppunit/extensions/HelperMacros.h>
#include <Devices/SimulatorBarcodeReaderDevice.hxx>
#include <System/Security/OpenSSLSecurity.hxx>
#include <tests/MockFile.hxx>
#include <tests/StoreMockFile.hxx>

class SimulatorBarcodeDeviceTest : public CPPUNIT_NS::TestFixture
{
    private:

        typedef Smartmatic::SAES::HW::Devices::SimulatorBarcodeReaderDevice SimulatorBarcodeReaderDevice;
        typedef Smartmatic::System::IO::File File;
        typedef Smartmatic::System::SafeBuffer SafeBuffer;

    public:

        SimulatorBarcodeDeviceTest ()
        {

        }

        void setUp()
        {
            Smartmatic::System::Security::OpenSSLSecurity::init();
        }

        void tearDown()
        {
            Smartmatic::System::Security::OpenSSLSecurity::clear();
        }

        void testOpenConnectionFail ()
        {
            MockFile mock;
            File::getInstance ( &mock );

            MOCKPP_CHAINER_FOR_EXT ( MockFile, canOpenFile, ext ) canOpenFile_mock ( &mock );

            canOpenFile_mock
            .expects ( MOCKPP_NS::once() )
            .will ( MOCKPP_NS::returnValue<bool> ( false ) );

            SimulatorBarcodeReaderDevice device ( "model", "serial", "commands", "buffer" );

            CPPUNIT_ASSERT ( false == device.open ( "" ) );

            mock.verify();
        }

        void testOpenConnectionSuccess ()
        {
            MockFile mock;
            File::getInstance ( &mock );

            MOCKPP_CHAINER_FOR_EXT ( MockFile, canOpenFile, ext ) canOpenFile_mock ( &mock );

            canOpenFile_mock
            .expects ( MOCKPP_NS::once() )
            .will ( MOCKPP_NS::returnValue<bool> ( true ) );

            SimulatorBarcodeReaderDevice device ( "model", "serial", "commands", "buffer" );

            CPPUNIT_ASSERT ( true == device.open ( "" ) );

            mock.verify();
        }

        void testCheckOpenFileConnection ()
        {
            StoreMockFile mock;
            File::getInstance ( &mock );

            mock.canOpenFileReturnValues.push_back ( true );

            SimulatorBarcodeReaderDevice device ( "model", "serial", "commands", "buffer" );

            CPPUNIT_ASSERT ( true == device.open ( "dummy" ) );
            CPPUNIT_ASSERT ( false == mock.canOpenFileList.empty() );
            CPPUNIT_ASSERT ( "dummy" == ( * mock.canOpenFileList.begin() ) );
        }

        void testCloseConnection ()
        {
            MockFile mock;
            File::getInstance ( &mock );

            MOCKPP_CHAINER_FOR_EXT ( MockFile, canOpenFile, ext ) canOpenFile_mock ( &mock );
            MOCKPP_CHAINER_FOR_EXT ( MockFile, openFileToBuffer, ext ) openFileToBuffer_mock ( &mock );
            MOCKPP_CHAINER_FOR_EXT ( MockFile, addBufferToFile, ext ) saveBufferToFile_mock ( &mock );

            canOpenFile_mock.expects ( MOCKPP_NS::never() );
            openFileToBuffer_mock.expects ( MOCKPP_NS::never() );
            saveBufferToFile_mock.expects ( MOCKPP_NS::never() );

            SimulatorBarcodeReaderDevice device ( "model", "serial", "commands", "buffer" );

            CPPUNIT_ASSERT ( true == device.close () );

            mock.verify();
        }

        void testGetDeviceModelFail()
        {
            StoreMockFile mock;
            File::getInstance ( &mock );

            mock.openFileValues.push_back ( StoreMockFile::OpenFileValues ( SafeBuffer (), false ) );

            SimulatorBarcodeReaderDevice device ( "model", "serial", "commands", "buffer" );

            CPPUNIT_ASSERT ( true == device.getDeviceModel().empty() );
            CPPUNIT_ASSERT ( false == mock.openFileListValues.empty() );
            CPPUNIT_ASSERT ( "model" == ( * mock.openFileListValues.begin() ) );
        }

        void testGetDeviceModelFailEmptyBuffer()
        {
            StoreMockFile mock;
            File::getInstance ( &mock );

            mock.openFileValues.push_back ( StoreMockFile::OpenFileValues ( SafeBuffer (), true ) );

            SimulatorBarcodeReaderDevice device ( "model", "serial", "commands", "buffer" );

            CPPUNIT_ASSERT ( true == device.getDeviceModel().empty() );
            CPPUNIT_ASSERT ( false == mock.openFileListValues.empty() );
            CPPUNIT_ASSERT ( "model" == ( * mock.openFileListValues.begin() ) );
        }

        void testGetDeviceModelSucess()
        {
            StoreMockFile mock;
            File::getInstance ( &mock );

            mock.openFileValues.push_back ( StoreMockFile::OpenFileValues ( SafeBuffer ( "hrs" ), true ) );

            SimulatorBarcodeReaderDevice device ( "model", "serial", "commands", "buffer" );

            CPPUNIT_ASSERT ( "hrs" == device.getDeviceModel() );
            CPPUNIT_ASSERT ( false == mock.openFileListValues.empty() );
            CPPUNIT_ASSERT ( "model" == ( * mock.openFileListValues.begin() ) );
        }

        void testSetDeviceSerialFail()
        {
            MockFile mock;
            File::getInstance ( &mock );

            MOCKPP_CHAINER_FOR_EXT ( MockFile, canOpenFile, ext ) canOpenFile_mock ( &mock );
            MOCKPP_CHAINER_FOR_EXT ( MockFile, saveBufferToFile, ext ) saveBufferToFile_mock ( &mock );

            canOpenFile_mock.expects ( MOCKPP_NS::never() );

            saveBufferToFile_mock
            .expects ( MOCKPP_NS::once() )
            .will ( MOCKPP_NS::returnValue<bool> ( false ) );

            SimulatorBarcodeReaderDevice device ( "model", "serial", "commands", "buffer" );

            CPPUNIT_ASSERT ( false == device.setDeviceSerial ( "SERIAL-DEVICE" ) );

            mock.verify();
        }

        void testSetDeviceSerialSuccess()
        {
            MockFile mock;
            File::getInstance ( &mock );

            MOCKPP_CHAINER_FOR_EXT ( MockFile, canOpenFile, ext ) canOpenFile_mock ( &mock );
            MOCKPP_CHAINER_FOR_EXT ( MockFile, saveBufferToFile, ext ) saveBufferToFile_mock ( &mock );

            canOpenFile_mock.expects ( MOCKPP_NS::never() );

            saveBufferToFile_mock
            .expects ( MOCKPP_NS::once() )
            .will ( MOCKPP_NS::returnValue<bool> ( true ) );

            SimulatorBarcodeReaderDevice device ( "model", "serial", "commands", "buffer" );

            CPPUNIT_ASSERT ( true == device.setDeviceSerial ( "SERIAL-DEVICE" ) );

            mock.verify();
        }

        void testSetDeviceSerialCheckSave()
        {
            StoreMockFile mock;
            File::getInstance ( &mock );

            SafeBuffer expected ( "SERIAL-DEVICE" );

            mock.saveReturnValues.push_back ( true );

            SimulatorBarcodeReaderDevice device ( "model", "serial", "commands", "buffer" );

            CPPUNIT_ASSERT ( true == device.setDeviceSerial ( "SERIAL-DEVICE" ) );
            CPPUNIT_ASSERT ( false == mock.saveBufferValues.empty() );
            CPPUNIT_ASSERT ( "serial" == ( * mock.saveBufferValues.begin() ).file );
            CPPUNIT_ASSERT ( expected == ( * mock.saveBufferValues.begin() ).buffer );
        }

        void testGetDeviceSerialFail()
        {
            StoreMockFile mock;
            File::getInstance ( &mock );

            mock.openFileValues.push_back ( StoreMockFile::OpenFileValues ( SafeBuffer (), false ) );

            SimulatorBarcodeReaderDevice device ( "model", "serial", "commands", "buffer" );

            CPPUNIT_ASSERT ( true == device.getDeviceSerial().empty() );
            CPPUNIT_ASSERT ( false == mock.openFileListValues.empty() );
            CPPUNIT_ASSERT ( "serial" == ( * mock.openFileListValues.begin() ) );
        }

        void testGetDeviceSerialFailEmptyBuffer()
        {
            StoreMockFile mock;
            File::getInstance ( &mock );

            mock.openFileValues.push_back ( StoreMockFile::OpenFileValues ( SafeBuffer (), true ) );

            SimulatorBarcodeReaderDevice device ( "model", "serial", "commands", "buffer" );

            CPPUNIT_ASSERT ( true == device.getDeviceSerial().empty() );
            CPPUNIT_ASSERT ( false == mock.openFileListValues.empty() );
            CPPUNIT_ASSERT ( "serial" == ( * mock.openFileListValues.begin() ) );
        }

        void testGetDeviceSerialSucess()
        {
            StoreMockFile mock;
            File::getInstance ( &mock );

            mock.openFileValues.push_back ( StoreMockFile::OpenFileValues ( SafeBuffer ( "PM1234" ), true ) );

            SimulatorBarcodeReaderDevice device ( "model", "serial", "commands", "buffer" );

            CPPUNIT_ASSERT ( "PM1234" == device.getDeviceSerial() );
            CPPUNIT_ASSERT ( false == mock.openFileListValues.empty() );
            CPPUNIT_ASSERT ( "serial" == ( * mock.openFileListValues.begin() ) );
        }

        void testSendCommandStatusFail_FileNotFound()
        {
            StoreMockFile mock;
            File::getInstance ( &mock );

            mock.openFileValues.push_back ( StoreMockFile::OpenFileValues ( SafeBuffer (), false ) );

            SimulatorBarcodeReaderDevice device ( "model", "serial", "commands", "buffer" );

            CPPUNIT_ASSERT ( false == device.sendReconfigureCommand ( "NLS", 0, 1, true ) );
            CPPUNIT_ASSERT ( false == mock.openFileListValues.empty() );
            CPPUNIT_ASSERT ( "commands" == ( * mock.openFileListValues.begin() ) );
        }

        void testSendCommandStatusFail_EmptyFile()
        {
            StoreMockFile mock;
            File::getInstance ( &mock );

            mock.openFileValues.push_back ( StoreMockFile::OpenFileValues ( SafeBuffer (), true ) );

            SimulatorBarcodeReaderDevice device ( "model", "serial", "commands", "buffer" );

            CPPUNIT_ASSERT ( false == device.sendReconfigureCommand ( "NLS", 0, 1, true ) );
            CPPUNIT_ASSERT ( false == mock.openFileListValues.empty() );
            CPPUNIT_ASSERT ( "commands" == ( * mock.openFileListValues.begin() ) );
        }

        void testSendCommandStatusFail_PairBufferFile()
        {
            StoreMockFile mock;
            File::getInstance ( &mock );

            mock.openFileValues.push_back ( StoreMockFile::OpenFileValues ( SafeBuffer ( "NLS,OK" ), true ) );

            SimulatorBarcodeReaderDevice device ( "model", "serial", "commands", "buffer" );

            CPPUNIT_ASSERT ( false == device.sendReconfigureCommand ( "NLS", 0, 1, true ) );
            CPPUNIT_ASSERT ( false == mock.openFileListValues.empty() );
            CPPUNIT_ASSERT ( "commands" == ( * mock.openFileListValues.begin() ) );
        }

        void testSendCommandStatusFail_PairBufferNotFoundFile()
        {
            StoreMockFile mock;
            File::getInstance ( &mock );

            mock.openFileValues.push_back ( StoreMockFile::OpenFileValues ( SafeBuffer ( "NLS3000:OK" ), true ) );

            SimulatorBarcodeReaderDevice device ( "model", "serial", "commands", "buffer" );

            CPPUNIT_ASSERT ( false == device.sendReconfigureCommand ( "NLS2000", 0, 1, true ) );
            CPPUNIT_ASSERT ( false == mock.openFileListValues.empty() );
            CPPUNIT_ASSERT ( "commands" == ( * mock.openFileListValues.begin() ) );
        }

        void testSendCommandStatusFail_MandatoryFalse_ResultSuccess()
        {
            StoreMockFile mock;
            File::getInstance ( &mock );

            mock.openFileValues.push_back ( StoreMockFile::OpenFileValues ( SafeBuffer ( "NLS3000:OK" ), true ) );

            SimulatorBarcodeReaderDevice device ( "model", "serial", "commands", "buffer" );

            CPPUNIT_ASSERT ( true == device.sendReconfigureCommand ( "NLS2000", 0, 1, false ) );
            CPPUNIT_ASSERT ( false == mock.openFileListValues.empty() );
            CPPUNIT_ASSERT ( "commands" == ( * mock.openFileListValues.begin() ) );
        }

        void testSendCommandStatusNOKFail()
        {
            StoreMockFile mock;
            File::getInstance ( &mock );

            SafeBuffer buffer;

            buffer.AddData ( "NLS2000:OK" );
            buffer.AddData ( "\n" );
            buffer.AddData ( "NLS3000:NOK" );
            buffer.AddData ( "\n" );
            buffer.AddData ( "NLS5000:OK" );

            mock.openFileValues.push_back ( StoreMockFile::OpenFileValues ( buffer, true ) );

            SimulatorBarcodeReaderDevice device ( "model", "serial", "commands", "buffer" );

            CPPUNIT_ASSERT ( false == device.sendReconfigureCommand ( "NLS3000", 0, 1, true ) );
            CPPUNIT_ASSERT ( false == mock.openFileListValues.empty() );
            CPPUNIT_ASSERT ( "commands" == ( * mock.openFileListValues.begin() ) );
        }

        void testSendCommandStatusSuccess()
        {
            StoreMockFile mock;
            File::getInstance ( &mock );

            SafeBuffer buffer;

            buffer.AddData ( "NLS2000:OK" );
            buffer.AddData ( "\n" );
            buffer.AddData ( "NLS3000:OK" );
            buffer.AddData ( "\n" );
            buffer.AddData ( "NLS5000:OK" );

            mock.openFileValues.push_back ( StoreMockFile::OpenFileValues ( buffer, true ) );

            SimulatorBarcodeReaderDevice device ( "model", "serial", "commands", "buffer" );

            CPPUNIT_ASSERT ( true == device.sendReconfigureCommand ( "NLS3000", 0, 1, true ) );
            CPPUNIT_ASSERT ( false == mock.openFileListValues.empty() );
            CPPUNIT_ASSERT ( "commands" == ( * mock.openFileListValues.begin() ) );
        }

        void testReadBufferFail()
        {
            StoreMockFile mock;
            File::getInstance ( &mock );

            mock.openFileValues.push_back ( StoreMockFile::OpenFileValues ( SafeBuffer (), false ) );

            SimulatorBarcodeReaderDevice device ( "model", "serial", "commands", "buffer" );

            CPPUNIT_ASSERT ( 0 == device.readBuffer().GetSize() );
            CPPUNIT_ASSERT ( false == mock.openFileListValues.empty() );
            CPPUNIT_ASSERT ( "buffer" == ( * mock.openFileListValues.begin() ) );
        }

        void testReadBufferFail_EmptyBuffer()
        {
            StoreMockFile mock;
            File::getInstance ( &mock );

            mock.openFileValues.push_back ( StoreMockFile::OpenFileValues ( SafeBuffer (), true ) );

            SimulatorBarcodeReaderDevice device ( "model", "serial", "commands", "buffer" );

            CPPUNIT_ASSERT ( 0 == device.readBuffer().GetSize() );
            CPPUNIT_ASSERT ( false == mock.openFileListValues.empty() );
            CPPUNIT_ASSERT ( "buffer" == ( * mock.openFileListValues.begin() ) );
        }

        void testReadBufferSuccess()
        {
            StoreMockFile mock;
            File::getInstance ( &mock );

            mock.openFileValues.push_back ( StoreMockFile::OpenFileValues ( SafeBuffer ( "BUFFER" ), true ) );
            mock.openFileValues.push_back ( StoreMockFile::OpenFileValues ( SafeBuffer ( "BUFFER" ), true ) );
            mock.openFileValues.push_back ( StoreMockFile::OpenFileValues ( SafeBuffer ( "OTHER-BUFFER" ), true ) );

            SimulatorBarcodeReaderDevice device ( "model", "serial", "commands", "buffer" );

            CPPUNIT_ASSERT ( SafeBuffer ( "BUFFER" ) == device.readBuffer() );
            CPPUNIT_ASSERT ( 0 == device.readBuffer().GetSize() );
            CPPUNIT_ASSERT ( SafeBuffer ( "OTHER-BUFFER" ) == device.readBuffer() );
        }

        void testReadBufferChangeSameBuffer()
        {
            StoreMockFile mock;
            File::getInstance ( &mock );

            mock.openFileValues.push_back ( StoreMockFile::OpenFileValues ( SafeBuffer ( "BUFFER" ), true ) );
            mock.openFileValues.push_back ( StoreMockFile::OpenFileValues ( SafeBuffer ( "BUFFER" ), false ) );
            mock.openFileValues.push_back ( StoreMockFile::OpenFileValues ( SafeBuffer ( "BUFFER" ), true ) );

            SimulatorBarcodeReaderDevice device ( "model", "serial", "commands", "buffer" );

            CPPUNIT_ASSERT ( SafeBuffer ( "BUFFER" ) == device.readBuffer() );
            CPPUNIT_ASSERT ( 0 == device.readBuffer().GetSize() );
            CPPUNIT_ASSERT ( SafeBuffer ( "BUFFER" ) == device.readBuffer() );
        }

        CPPUNIT_TEST_SUITE ( SimulatorBarcodeDeviceTest );

        CPPUNIT_TEST ( testOpenConnectionFail );
        CPPUNIT_TEST ( testOpenConnectionSuccess );
        CPPUNIT_TEST ( testCheckOpenFileConnection );

        CPPUNIT_TEST ( testCloseConnection );

        CPPUNIT_TEST ( testGetDeviceModelFail );
        CPPUNIT_TEST ( testGetDeviceModelFailEmptyBuffer );
        CPPUNIT_TEST ( testGetDeviceModelSucess );

        CPPUNIT_TEST ( testSetDeviceSerialFail );
        CPPUNIT_TEST ( testSetDeviceSerialSuccess );
        CPPUNIT_TEST ( testSetDeviceSerialCheckSave );

        CPPUNIT_TEST ( testGetDeviceSerialFail );
        CPPUNIT_TEST ( testGetDeviceSerialFailEmptyBuffer );
        CPPUNIT_TEST ( testGetDeviceSerialSucess );

        CPPUNIT_TEST ( testSendCommandStatusFail_FileNotFound );
        CPPUNIT_TEST ( testSendCommandStatusFail_EmptyFile );
        CPPUNIT_TEST ( testSendCommandStatusFail_PairBufferFile );
        CPPUNIT_TEST ( testSendCommandStatusFail_PairBufferNotFoundFile );

        CPPUNIT_TEST ( testSendCommandStatusFail_MandatoryFalse_ResultSuccess );
        CPPUNIT_TEST ( testSendCommandStatusNOKFail );
        CPPUNIT_TEST ( testSendCommandStatusSuccess );

        CPPUNIT_TEST ( testReadBufferFail );
        CPPUNIT_TEST ( testReadBufferFail_EmptyBuffer );
        CPPUNIT_TEST ( testReadBufferSuccess );
        CPPUNIT_TEST ( testReadBufferChangeSameBuffer );

        CPPUNIT_TEST_SUITE_END();
};

#endif /* ECUT_MAIN */

#endif /* TEST_SIMULATOR_BARCODE_DEVICE_TEST */
