/**
 * @file   Audio.hxx
 * @brief  Declaration of Audio class.
 * @author manuel.arguelles
 * @date   Sep 29, 2017
 */

#ifndef AUDIO_HXX
#define AUDIO_HXX

#include <vector>

namespace Smartmatic
{
    namespace System
    {
        namespace Audio
        {

            /**
             * @class  Audio
             * @brief  Storage class for audio samples.
             * @remark This class is trivially copyable.
             * @author manuel.arguelles
             * @date   Sep 29, 2017
             */
            class Audio
            {

                public:

                    /**
                     * @brief  Constructs an instance of the class.
                     * @author manuel.arguelles
                     * @date   Sep 29, 2017
                     */
                    Audio();

                    /**
                     * @brief  Destroys the instance of the class
                     * @author manuel.arguelles
                     * @date   Sep 29, 2017
                     */
                    ~Audio();

                public:

                    /**
                     * @brief  Gets the audio samples
                     * @author manuel.arguelles
                     * @date   Oct  5, 2017
                     * @return A vector with the samples.
                     */
                    std::vector<char> getSamples() const;

                    /**
                     * @brief  Adds audio samples.
                     * @author manuel.arguelles
                     * @date   Oct  5, 2017
                     * @param  samples The audio samples.
                     * @param  size    The amount of samples.
                     */
                    void setSamples(const char* samples, std::size_t size);

                    /**
                     * @brief  Gets the bits per sample.
                     * @author manuel.arguelles
                     * @date   Oct  5, 2017
                     * @return The bits per sample.
                     */
                    int getBitsPerSample() const;

                    /**
                     * @brief  Sets the amount of bits per sample.
                     * @author manuel.arguelles
                     * @date   Oct  5, 2017
                     * @param  bits The amount of bits per sample.
                     */
                    void setBitsPerSample(int bits);

                    /**
                     * @brief  Gets the amount of samples per second (rate) in a
                     * single channel.
                     * @author manuel.arguelles
                     * @date   Oct  5, 2017
                     * @return The amount of samples per second (rate).
                     */
                    int getSamplesPerSecond() const;

                    /**
                     * @brief  Sets the amount of samples per second in a single
                     * channel.
                     * @author manuel.arguelles
                     * @date   Oct  5, 2017
                     * @param  rate The amount of samples per second.
                     */
                    void setSamplesPerSecond(int rate);

                    /**
                     * @brief  Gets the number of channels.
                     * @author manuel.arguelles
                     * @date   Oct  5, 2017
                     * @return The number of channels.
                     */
                    int getChannels() const;

                    /**
                     * @brief  Sets the number of channels.
                     * @author manuel.arguelles
                     * @date   Oct  5, 2017
                     * @param  channels The number of channels.
                     */
                    void setChannels(int channels);

                private:

                    int               m_bitsPerSample;
                    int               m_channels;
                    int               m_samplesPerSecond;
                    std::vector<char> m_samples;
            };

        } /* namespace Audio */
    } /* namespace System */
} /* namespace Smartmatic */

#endif /* AUDIO_HXX */
