/**
 * @file   AudioPlayerImpl.cxx
 * @brief  Implementation of AudioPlayerImpl class.
 * @author manuel.arguelles
 * @date   Oct  5, 2017
 */

#include "AudioPlayerImpl.hxx"
#include <System/Exception/AudioException.hxx>
#include <cerrno>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#include <libintl.h>
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///< macro for gettext
#else
#define _(String) (String) ///< macro without gettext
#endif

#define N_(String) (String) ///< macro not translate

namespace
{
    class StaticInitializer
    {
        public:
            StaticInitializer();
            ~StaticInitializer();
    };
    StaticInitializer initializer;
}

StaticInitializer::StaticInitializer()
{
    ao_initialize();
}

StaticInitializer::~StaticInitializer()
{
    ao_shutdown();
}

static const std::string DRIVER_OPTION_OUTPUT_DEVICE_NAME = "dev";

/**
 * @brief  Creates a sample format.
 * @author manuel.arguelles
 * @date   Oct  5, 2017
 * @param  bits     The bits per sample.
 * @param  rate     The samples per second.
 * @param  channels The number of audio channels.
 * @return The sample format.
 */
static ao_sample_format createSampleFormat(const int bits, const int rate, const int channels)
{
    ao_sample_format sampleFormat = { 0, 0, 1, AO_FMT_NATIVE, NULL };

    sampleFormat.bits     = bits;
    sampleFormat.rate     = rate;
    sampleFormat.channels = channels;

    return sampleFormat;
}

static std::string errorToString(int errorNumber)
{
    std::string error;

    switch (errorNumber)
    {
        case AO_ENODRIVER:
            error = _("No driver found");
            break;
        case AO_ENOTLIVE:
            error = _("Not a valid driver");
            break;
        case AO_EBADOPTION:
            error = _("Invalid option");
            break;
        case AO_EOPENDEVICE:
            error = _("Cannot open the device");
            break;
        default:
        case AO_EFAIL:
            error = _("Unknown error");
            break;
    }

    return error;
}

using namespace Smartmatic::System::Audio;
using namespace Smartmatic::System::Exception;

AudioPlayerImpl::AudioPlayerImpl()
    : m_device(NULL)
    , m_options(NULL)
    , m_format(createSampleFormat(0, 0, 0))
{
}

AudioPlayerImpl::~AudioPlayerImpl()
{
    closeDevice();
}

void AudioPlayerImpl::closeDevice()
{
    if (m_device != NULL)
    {
        ao_close(m_device);
        m_device = NULL;
    }
}

void AudioPlayerImpl::openDevice()
{
    closeDevice();
    m_device = ao_open_live(ao_default_driver_id(), &m_format, m_options);

    if (m_device == NULL)
    {
        std::string message = errorToString(errno);
        throw AudioException(message, N_("Error opening device"));
    }
}

void AudioPlayerImpl::play(const Audio& audio)
{
    Glib::Mutex::Lock lock(m_playLock, Glib::TRY_LOCK);
    std::vector<char> audioSamples;

    if (!lock.locked())
    {
        throw AudioException(_("Device busy"), N_("Device busy"));
    }

    if (m_device == NULL)
    {
        m_format = createSampleFormat(
            audio.getBitsPerSample(),
            audio.getSamplesPerSecond(),
            audio.getChannels());
        openDevice();
    }

    audioSamples = audio.getSamples();

    if (ao_play(m_device, &audioSamples[0], audioSamples.size()) == 0)
    {
        closeDevice();
    }
}

void AudioPlayerImpl::setOutputDevice(const std::string& device)
{
    ao_free_options(m_options);
    ao_append_option(&m_options, DRIVER_OPTION_OUTPUT_DEVICE_NAME.c_str(), device.c_str());
}
