/**
 * @file   NuanceVocalizer.hxx
 * @brief  Declaration of NuanceVocalizer class.
 * @author manuel.arguelles
 * @date   Oct 10, 2017
 */

#ifndef NUAN_VOCALIZER_HXX
#define NUAN_VOCALIZER_HXX

#include "ITextToSpeech.hxx"
#include <memory>
#include <string>

namespace Smartmatic
{
    namespace System
    {
        namespace Audio
        {
            class IAudioPlayer;

            namespace TextToSpeech
            {

                class NuanceVocalizerImpl;

                /**
                 * @class  NuanceVocalizer
                 * @brief  Nuance Vocalizer Embedded class.
                 * @author manuel.arguelles
                 * @date   Oct 10, 2017
                 */
                class NuanceVocalizer : public ITextToSpeech
                {
                    public:

                        /**
                         * @brief  Creates an instance of the class.
                         * @author manuel.arguelles
                         * @date   Oct 10, 2017
                         *
                         * @remark Class takes ownership of audioPlayer.
                         *
                         * @throw  TextToSpeechException If the initialization fails.
                         *
                         * @param  dataPath    The path with the data/voices files.
                         * @param  audioPlayer The audio player to use.
                         */
                        NuanceVocalizer(const std::string& dataPath,
                                        System::Audio::IAudioPlayer* audioPlayer);

                        /**
                         * @brief  Destroys an instance of the class.
                         * @author manuel.arguelles
                         * @date   Oct 10, 2017
                         */
                        virtual ~NuanceVocalizer();

                    public:

                        /**
                         * @brief  Set the language to use.
                         * @author manuel.arguelles
                         * @date   Oct  9, 2017
                         * @param  language The language to use.
                         * @return True on success, false on error.
                         */
                        virtual bool setLanguage(const std::string& language);

                        /**
                         * @brief  Sets the speech rate.
                         * @author manuel.arguelles
                         * @date   Oct  9, 2017
                         * @param  rate The speed rate in percentage, 100 is
                         * normal. Valid values are 50-400.
                         */
                        virtual bool setSpeechRate(int rate);

                        /**
                         * @brief  Speak the given text.
                         * @author manuel.arguelles
                         * @date   Oct  9, 2017
                         * @param  text The text to speak.
                         */
                        virtual void speak(const std::string& text);

                        /**
                         * @brief  Stops the processing of text.
                         * @author manuel.arguelles
                         * @date   Oct  9, 2017
                         */
                        virtual void stop();

                        /**
                         * @brief  Tell if the engine is speaking.
                         * @author manuel.arguelles
                         * @date   Oct 10, 2017
                         * @return True if speaking, false if not.
                         */
                        virtual bool isSpeaking() const;

                        /**
                         * @brief  Wait while speaking.
                         * @author manuel.arguelles
                         * @date   Dec 18, 2017
                         */
                        virtual void waitWhileSpeaking() const;

                    private:

                        /**
                         * @brief  Disable assignment operator.
                         * @author manuel.arguelles
                         * @date   Oct  6, 2017
                         */
                        NuanceVocalizer& operator=(const NuanceVocalizer&);

                        /**
                         * @brief  Disable copy constructor.
                         * @author manuel.arguelles
                         * @date   Oct  6, 2017
                         */
                        NuanceVocalizer(const NuanceVocalizer&);


                    private:

                        std::auto_ptr<NuanceVocalizerImpl> m_impl;
                };

            } /* namespace TextToSpeech */
        } /* namespace Audio */
    } /* namespace System */
} /* namespace Smartmatic */

#endif /* NUAN_VOCALIZER_HXX */
