/**
 * @file   NuanceVocalizerImpl.cxx
 * @brief  Implementation of NuanceVocalizerImpl class.
 * @author manuel.arguelles
 * @date   Oct  6, 2017
 */

#include "NuanceVocalizerImpl.hxx"
#include "NuanceVocalizerUtility.hxx"

#include <System/Audio/Audio.hxx>
#include <System/Audio/IAudioPlayer.hxx>
#include <System/Exception/TextToSpeechException.hxx>
#include <System/Exception/AudioException.hxx>
#include <cstring>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#include <libintl.h>
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///< macro for gettext
#else
#define _(String) (String) ///< macro without gettext
#endif

#define N_(String) (String) ///< macro not translate

using namespace Smartmatic::System::Audio::TextToSpeech;
using namespace Smartmatic::System::Audio;
using namespace Smartmatic::System::Exception;

static const int         AUDIO_CHANNELS    = 1;
static const int         BITS_PER_SAMPLE   = 16;
static const int         SAMPLE_RATE       = 22000;
static const std::size_t AUDIO_BUFFER_SIZE = 1024;

struct NuanceVocalizerImpl::InternalData
{
    std::auto_ptr<IAudioPlayer> m_audioPlayer;
    bool                        m_speaking;
};

static NUAN_ERROR playAudio(VE_HINSTANCE tts, void* userData, VE_CALLBACKMSG* message)
{
    NuanceVocalizerImpl::InternalData* m_internal = static_cast<NuanceVocalizerImpl::InternalData*>(userData);

    if (message->eMessage == VE_MSG_OUTBUFREQ)
    {
        VE_OUTDATA* data = static_cast<VE_OUTDATA*>(message->pParam);

        data->pOutPcmBuf    = NuanceUtility::heapMalloc(NULL, AUDIO_BUFFER_SIZE);
        data->pMrkList      = static_cast<VE_MARKINFO*>(NuanceUtility::heapMalloc(NULL, AUDIO_BUFFER_SIZE));
        data->cntPcmBufLen  = AUDIO_BUFFER_SIZE;
        data->cntMrkListLen = AUDIO_BUFFER_SIZE;
    }
    else if (message->eMessage == VE_MSG_OUTBUFDONE)
    {
        VE_OUTDATA*   data    = static_cast<VE_OUTDATA*>(message->pParam);
        char*         rawData = static_cast<char*>(data->pOutPcmBuf);
        Audio samples;

        samples.setSamples(rawData, data->cntPcmBufLen);
        samples.setBitsPerSample(BITS_PER_SAMPLE);
        samples.setSamplesPerSecond(SAMPLE_RATE);
        samples.setChannels(AUDIO_CHANNELS);

        if (m_internal->m_audioPlayer.get() != NULL)
        {
            try
            {
                m_internal->m_audioPlayer->play(samples);
            }
            catch (const AudioException&)
            {
                m_internal->m_speaking = false;
            }
        }

        NuanceUtility::heapFree(NULL, data->pOutPcmBuf);
        NuanceUtility::heapFree(NULL, data->pMrkList);
    }
    else if (message->eMessage == VE_MSG_ENDPROCESS)
    {
        m_internal->m_speaking = false;
    }
    else if (message->eMessage == VE_MSG_BEGINPROCESS)
    {
        m_internal->m_speaking = true;
    }

    return NUAN_OK;

}

NuanceVocalizerImpl::NuanceVocalizerImpl(const std::string& dataPath, IAudioPlayer* audioPlayer)
    : m_internal(new InternalData())
    , m_dataPath(dataPath)
    , m_brokerInfo(NuanceUtility::getBrokerInfo(dataPath))
{
    VE_DATA_STREAM_INTERFACE     data;
    VE_HEAP_INTERFACE            heap;
    VE_INSTALL                   install;
    VE_OUTDEVINFO                outDevice;

    heap.pfMalloc  = NuanceUtility::heapMalloc;
    heap.pfCalloc  = NuanceUtility::heapCalloc;
    heap.pfRealloc = NuanceUtility::heapRealloc;
    heap.pfFree    = NuanceUtility::heapFree;

    data.pfOpen    = NuanceUtility::fileOpen;
    data.pfClose   = NuanceUtility::fileClose;
    data.pfRead    = NuanceUtility::fileRead;
    data.pfSeek    = NuanceUtility::fileSeek;
    data.pfGetSize = NuanceUtility::fileGetSize;
    data.pfError   = NuanceUtility::fileError;
    data.pfWrite   = NuanceUtility::fileWrite;

    std::memset(&install, 0, sizeof install);

    install.fmtVersion     = VE_CURRENT_VERSION;
    install.pBinBrokerInfo = m_brokerInfo.c_str();
    install.pIHeap         = &heap;
    install.pIDataStream   = &data;
    install.hDataClass     = &m_dataPath;

    m_internal->m_speaking = false;
    m_internal->m_audioPlayer.reset(audioPlayer);

    outDevice.pUserData   = m_internal.get();
    outDevice.pfOutNotify = playAudio;

    m_playbackThread.set_max_threads(1);

    if ((m_brokerInfo.empty()) ||
        (ve_ttsInitialize(&install, &m_speech) != NUAN_OK)  ||
        (ve_ttsOpen(m_speech, NULL, NULL, &m_tts) != NUAN_OK) ||
        (ve_ttsSetOutDevice(m_tts, &outDevice) != NUAN_OK))
    {
        throw TextToSpeechException(_("Initialization error"),
                                    N_("Initialization error"));
    }
}

NuanceVocalizerImpl::~NuanceVocalizerImpl()
{
    m_playbackThread.shutdown();

    ve_ttsClose(m_tts);
    ve_ttsUnInitialize(m_speech);
}

bool NuanceVocalizerImpl::setLanguage(const std::string& language)
{
    bool result = false;
    VE_PARAM_VALUE encoding, frequency, lang, voice, voice_point;

    std::strcpy(voice.szStringValue, "");
    std::strcpy(voice_point.szStringValue, "");
    std::strcpy(lang.szStringValue, language.c_str());

    encoding.usValue = VE_TYPE_OF_CHAR_UTF8;
    frequency.usValue = VE_FREQ_22KHZ;

    VE_PARAM parameters[] = {
        { VE_PARAM_LANGUAGE, lang },
        { VE_PARAM_VOICE, voice },
        { VE_PARAM_VOICE_OPERATING_POINT, voice_point },
        { VE_PARAM_TYPE_OF_CHAR, encoding },
        { VE_PARAM_FREQUENCY, frequency }
    };

    if (ve_ttsSetParamList(m_tts, parameters, 5) == NUAN_OK)
    {
        result = true;
    }

    return result;
}

bool NuanceVocalizerImpl::setSpeechRate(const int rate)
{
    bool result = false;
    VE_PARAM_VALUE speed;

    speed.usValue = rate;

    VE_PARAM parameters[] = {
        { VE_PARAM_SPEECHRATE, speed },
    };

    if (ve_ttsSetParamList(m_tts, parameters, 1) == NUAN_OK)
    {
        result = true;
    }

    return result;
}

void NuanceVocalizerImpl::speak(const std::string& text)
{
    m_internal->m_speaking = true;
    m_playbackThread.push(sigc::bind<1>(sigc::mem_fun(*this, &NuanceVocalizerImpl::playBack), text));
}

void NuanceVocalizerImpl::stop()
{
    ve_ttsStop(m_tts);
}

bool NuanceVocalizerImpl::isSpeaking() const
{
    return m_internal->m_speaking;
}

void NuanceVocalizerImpl::playBack(const std::string& text)
{
    VE_INTEXT inputText;

    inputText.eTextFormat   = VE_NORM_TEXT;
    inputText.cntTextLength = text.size();
    inputText.szInText      = const_cast<void*>(static_cast<const void*>(text.c_str()));
    ve_ttsProcessText2Speech(m_tts, &inputText);
}
