/**
 * @file   NuanceVocalizerImpl.hxx
 * @brief  Declaration of NuanceVocalizerImpl class.
 * @author manuel.arguelles
 * @date   Oct  6, 2017
 */

#ifndef NUAN_VOCALIZER_IMPL_HXX
#define NUAN_VOCALIZER_IMPL_HXX

#include <ve/ve_platform.h>
#include <glibmm.h>
#include <string>
#include <memory>

namespace Smartmatic
{
    namespace System
    {
        namespace Audio
        {
            class IAudioPlayer;

            namespace TextToSpeech
            {
                /**
                 * @class  NuanceVocalizerImpl
                 * @brief  Implementation of NuanceVocalizer class.
                 * @author manuel.arguelles
                 * @date   Oct  6, 2017
                 */
                class NuanceVocalizerImpl
                {
                    public:

                        /**
                         * @brief  Creates an instance of the class.
                         * @author manuel.arguelles
                         * @date   Oct  6, 2017
                         *
                         * @remark Class takes ownership of audioPlayer.
                         *
                         * @throw  TextToSpeechException If the initialization fails.
                         *
                         * @param  dataPath    The path with the data/voices files.
                         * @param  audioPlayer The audio player to use.
                         */
                        NuanceVocalizerImpl(const std::string& dataPath,
                                            System::Audio::IAudioPlayer* audioPlayer);

                        /**
                         * @brief  Destroys an instance of the class.
                         * @author manuel.arguelles
                         * @date   Oct  6, 2017
                         */
                        ~NuanceVocalizerImpl();

                    public:

                        /**
                         * @brief  Set the language to use.
                         * @author manuel.arguelles
                         * @date   Oct  9, 2017
                         * @param  language The language to use.
                         * @return True on success, false on error.
                         */
                        bool setLanguage(const std::string& language);

                        /**
                         * @brief  Sets the speech rate.
                         * @author manuel.arguelles
                         * @date   Oct  9, 2017
                         * @param  rate The speed rate in percentage, 100 is
                         * normal. Valid values are 50-400.
                         */
                        bool setSpeechRate(int rate);

                        /**
                         * @brief  Speak the given text.
                         * @author manuel.arguelles
                         * @date   Oct  9, 2017
                         * @param  text The text to speak.
                         */
                        void speak(const std::string& text);

                        /**
                         * @brief  Stops the processing of text.
                         * @author manuel.arguelles
                         * @date   Oct  9, 2017
                         */
                        void stop();

                        /**
                         * @brief  Tell if the engine is speaking.
                         * @author manuel.arguelles
                         * @date   Oct 10, 2017
                         * @return True if speaking, false if not.
                         */
                        bool isSpeaking() const;

                    private:

                        /**
                         * @brief  Disable assignment operator.
                         * @author manuel.arguelles
                         * @date   Oct  6, 2017
                         */
                        NuanceVocalizerImpl& operator=(const NuanceVocalizerImpl&);

                        /**
                         * @brief  Disable copy constructor.
                         * @author manuel.arguelles
                         * @date   Oct  6, 2017
                         */
                        NuanceVocalizerImpl(const NuanceVocalizerImpl&);

                        /**
                         * @brief  Starts playing the text.
                         * @author manuel.arguelles
                         * @date   Oct  9, 2017
                         * @param  text The text to play.
                         */
                        void playBack(const std::string& text);

                    public:

                        /**
                         * @brief  Opaque data.
                         * @author manuel.arguelles
                         * @date   Oct 10, 2017
                         */
                        struct InternalData;

                    private:

                        std::auto_ptr<InternalData>  m_internal;
                        VE_HINSTANCE                 m_tts;
                        VE_HSPEECH                   m_speech;
                        std::string                  m_dataPath;
                        std::string                  m_brokerInfo;
                        Glib::ThreadPool             m_playbackThread;
                };

            } /* namespace TextToSpeech */
        } /* namespace Audio */
    } /* namespace System */
} /* namespace Smartmatic */

#endif /* NUAN_VOCALIZER_IMPL_HXX */
