/**
 * @file   NuanceVocalizerUtility.hxx
 * @brief  Declaration of utility / helper functions required by
 * Nuance Vocalizer engine.
 * @author manuel.arguelles
 * @date   Oct  7, 2017
 */

#ifndef NUANCE_VOCALIZER_UTILITY_HXX
#define NUANCE_VOCALIZER_UTILITY_HXX

#include <ve/ve_platform.h>
#include <cstddef>
#include <string>

namespace Smartmatic
{
    namespace System
    {
        namespace Audio
        {
            namespace TextToSpeech
            {
                namespace NuanceUtility
                {

                    /**
                     * @brief  Allocates memory.
                     * @author manuel.arguelles
                     * @date   Oct  7, 2017
                     * @param  heap  The heap to use (currently not used).
                     * @param  bytes The amount of bytes to allocate.
                     * @return A pointer to the allocated memory or null.
                     */
                    void* heapMalloc(void* heap, std::size_t bytes);

                    /**
                     * @brief  Allocates memory initialized to 0.
                     * @author manuel.arguelles
                     * @date   Oct  7, 2017
                     * @param  heap         The heap to use (currently not used).
                     * @param  elements     The amount of elements to allocate.
                     * @param  elementBytes The amount of bytes per element.
                     * @return A pointer to the allocated memory or null.
                     */
                    void* heapCalloc(void* heap, std::size_t elements, std::size_t elementBytes);

                    /**
                     * @brief  Reallocates size of memory area.
                     * @author manuel.arguelles
                     * @date   Oct  7, 2017
                     * @param  heap  The heap to use (currently not used).
                     * @param  data  The pointer to existing memory area.
                     * @param  bytes The new size of the memory area.
                     * @return A pointer to the allocated memory or null.
                     */
                    void* heapRealloc(void* heap, void* data, std::size_t bytes);

                    /**
                     * @brief  Frees an allocated memory area.
                     * @author manuel.arguelles
                     * @date   Oct  7, 2017
                     * @param  heap The heap to use (currently not used).
                     * @param  data The pointer to the memory area to free.
                     */
                    void heapFree(void* heap, void* data);

                    /**
                     * @brief  Opens a file
                     * @author manuel.arguelles
                     * @date   Oct  7, 2017
                     * @param  data   Pointer to user data.
                     * @param  heap   The heap to use (currently not used).
                     * @param  name   The file name.
                     * @param  mode   The open mode.
                     * @param  stream The output file stream/descriptor.
                     * @return NUAN_OK on success, NUAN_E_COULDNOTOPENFILE on error.
                     */
                    NUAN_ERROR fileOpen(void* data, void* heap, const char* name,
                                        const char* mode, void** stream);

                    /**
                     * @brief  Closes a file.
                     * @author manuel.arguelles
                     * @date   Oct  7, 2017
                     * @param  stream The file stream/descriptor.
                     * @return NUAN_OK on success.
                     */
                    NUAN_ERROR fileClose(void* stream);

                    /**
                     * @brief  Reads data from file.
                     * @author manuel.arguelles
                     * @date   Oct  7, 2017
                     * @param  buffer       The output buffer where to copy the data.
                     * @param  elementBytes The size of each element.
                     * @param  elements     The amount of elements.
                     * @param  stream       The file stream/descriptor.
                     * @return The amount of read.
                     */
                    std::size_t fileRead(void* buffer, std::size_t elementBytes,
                                         std::size_t elements, void* stream);

                    /**
                     * @brief  Changes the position in the file.
                     * @author manuel.arguelles
                     * @date   Oct  7, 2017
                     * @param  stream    The file stream/descriptor.
                     * @param  offset    The offset from the origin to move.
                     * @param  origin    The starting position.
                     * @param  direction The direction to move.
                     * @return NUAN_OK on success, NUAN_E_FILESEEK on error.
                     */
                    NUAN_ERROR fileSeek(void* stream, std::size_t offset,
                                        VE_STREAM_ORIGIN origin,
                                        VE_STREAM_DIRECTION direction);

                    /**
                     * @brief  Gets the size of the file.
                     * @author manuel.arguelles
                     * @date   Oct  7, 2017
                     * @param  stream The file stream/descriptor.
                     * @return The size of the file.
                     */
                    std::size_t fileGetSize(void* stream);

                    /**
                     * @brief  Gets the status (error) of the file.
                     * @author manuel.arguelles
                     * @date   Oct  7, 2017
                     * @param  stream The file stream/descriptor.
                     * @return NUAN_OK if there are no error, error instead.
                     */
                    NUAN_ERROR fileError(void* stream);

                    /**
                     * @brief  Writes data to the file.
                     * @author manuel.arguelles
                     * @date   Oct  7, 2017
                     * @param  buffer       The data to write.
                     * @param  elementBytes The size of each element.
                     * @param  elements     The amount of elements to write.
                     * @param  stream       The file stream/descriptor.
                     * @return The amount of bytes written.
                     */
                    std::size_t fileWrite(const void* buffer, std::size_t elementBytes,
                                          std::size_t elements, void* stream);

                    /**
                     * @brief  Recursively walk basePath searching and loading
                     * .hdr files.
                     * @author manuel.arguelles
                     * @date   Oct  9, 2017
                     * @param  basePath The base directory.
                     * @return A string with the concatenated content of all the
                     * .hdr files found.
                     */
                    std::string getBrokerInfo(const std::string& basePath);

                } /* namespace NuanceUtility */
            } /* namespace TextToSpeech */
        } /* namespace Audio */
    } /* namespace System */
} /* namespace Smartmatic */

#endif /* NUANCE_VOCALIZER_UTILITY_HXX */
