/**
 * @file	Directory.h
 * @brief	Header file to the directory class.
 */

#ifndef _DIRECTORY_H_
#define _DIRECTORY_H_

#include "IDirectory.hxx"
#include <vector>
#include <string>

struct stat; ///< forward declaration
struct FTW; ///< forward declaration

namespace Smartmatic
{
    namespace Log
    {
        class ISMTTLog;
    }

    namespace System
    {
        namespace IO
        {
            /**
             * @class	Directory
             * @brief	Class utils for directory manager.
             * @author	Rafael.gasperi
             * @date	28/04/2011
             */
            class Directory : public IDirectory
            {
                protected:

                    /**
                     * @fn	Directory();
                     * @brief	Default Constructor.
                     * @author	Yadickson Soto
                     * @date	06-11-2013
                     */
                    Directory();

                public:

                    /**
                     * @fn	virtual ~Directory();
                     * @brief	Class destructor.
                     * @author	Yadickson Soto
                     * @date	06-11-2013
                     */
                    virtual ~Directory();

                    /**
                     * @fn	virtual bool Directory::Delete(const std::string & path);
                     * @brief	Deletes the Directory in the given pathname.
                     * @author	Yadickson Soto
                     * @date	13/03/2013
                     * @param	path		 	Full pathname of the file.
                     * @param	return 	true if delete is successfully
                     */
                    virtual bool Delete ( const std::string & path );

                    /**
                     * @fn	virtual bool Directory::Mkdir(const std::string & path);
                     * @brief	Make de Directory in the  the given pathname.
                     * @author	Rafael.gasperi
                     * @date	28/04/2011
                     * @param	path	Full pathname of the file.
                     * @param	return 	true if make directory is successfully
                     */
                    virtual bool Mkdir ( const std::string & path );

                    /**
                     * @fn	virtual bool Directory::DirectoryExists(const std::string & path);
                     * @brief	Queries if a given directory exists.
                     * @author	Rafael.gasperi
                     * @date	28/04/2011
                     * @param	path	Full pathname of the file.
                     * @return	true if it succeeds, false if it fails.
                     */
                    virtual bool DirectoryExists ( const std::string & path );

                    /**
                     * @fn	virtual bool Directory::FileExists(const std::string & path);
                     * @brief	Queries if a given file exists.
                     * @author	Rafael.gasperi
                     * @date	28/04/2011
                     * @param	path	Full pathname of the file.
                     * @return	true if it succeeds, false if it fails.
                     */
                    virtual bool FileExists ( const std::string & path );

                    /**
                     * @fn	virtual void Directory::GetFiles(const std::string & path, std::vector<std::string>& list);
                     * @brief	Gets the relative files.
                     * @author	Yadickson Soto
                     * @date	07/02/2014
                     * @param	path		 	Full pathname of the file.
                     * @param	list		 	The list of relative files.
                     */
                    virtual void GetFiles ( const std::string & path, std::vector<std::string> & list );

                    /**
                     * @fn	virtual void Directory::GetDirectories(const std::string & path, std::vector<std::string>& list);
                     * @brief	Gets the relative directories.
                     * @author	Manuel Arguelles
                     * @date	07/08/2017
                     * @param	path		 	Full pathname of the directory.
                     * @param	list		 	The list of relative directories.
                     */
                    virtual void GetDirectories ( const std::string & path, std::vector<std::string> & list );

                    /**
                     * @fn	virtual bool addAllPermission ( const std::string & folder );
                     * @brief	Method add all permission
                     * @author	Yadickson Soto
                     * @date	13/02/2013
                     * @param	folder folder path
                     * @param	return 	true if add all permission successfully
                     */
                    virtual bool addAllPermission ( const std::string & folder );

                    /**
                     * @fn	virtual std::string getCurrentDirectory ();
                     * @brief	Getter Method : Current Directory
                     * @author	Yadickson Soto
                     * @date	14/11/2013
                     * @return 	current directory
                     */
                    virtual std::string getCurrentDirectory ();

                    /**
                     * @fn	static Directory * getInstance ( Directory * externDirectory = NULL );
                     * @brief	Getter method instance of class.
                     * @author	Yadickson Soto
                     * @date	06/11/2013
                     * @param 	externDirectory external directory to unit test
                     * @return	instance of class.
                     */
                    static Directory * getInstance ( Directory * externDirectory = NULL );

                protected:

                    /**
                     * @fn virtual int executeRm ( const char *path, const struct stat *s, int flag, struct FTW *f );
                     * @brief Call unlink or rmdir on the path, as appropriate.
                     * @author Yadickson Soto
                     * @date 08/11/2013
                     * @param path path to remove
                     * @param s stat to recursive call
                     * @param flag flag type path
                     * @param f struct FTW type
                     * @return zero "0" if rmdir is successfully
                     */
                    static int executeRm ( const char * path, const struct stat * s, int flag, struct FTW * f );
                    /**
                     * @fn virtual int executeChmod ( const char *path, const struct stat *s, int flag, struct FTW *f );
                     * @brief Call chmod on the path.
                     * @author Yadickson Soto
                     * @date 12/11/2013
                     * @param path path to remove
                     * @param s stat to recursive call
                     * @param flag flag type path
                     * @param f struct FTW type
                     * @return zero "0" if chmod is successfully
                     */
                    static int executeChmod ( const char * path, const struct stat * s, int flag, struct FTW * f );

                    /**
                     * @fn virtual bool executeMkdir ( const std::string & dir );
                     * @brief Call mkdir to system.
                     * @author Yadickson Soto
                     * @date 12/11/2013
                     * @param dir path dir to remove
                     * @return true if success
                     */
                    virtual bool executeMkdir ( const std::string & dir );

                private:

                    static Directory * instance; ///< instance of class
                    static Smartmatic::Log::ISMTTLog * logger; ///< The logger
            };
        }
    }
}






#endif
