/**
 * @file   FatFileManager.hxx
 * @brief  Declaration of FatFileManager class.
 * @author manuel.arguelles
 * @date   Aug  3, 2017
 */

#ifndef FATFILEMANAGER_HXX_
#define FATFILEMANAGER_HXX_

#include <string>
#include <vector>

namespace Smartmatic
{
    namespace Log
    {
        class ISMTTLog;
    }
    namespace System
    {
        namespace Runtime
        {
            class IStorageDevice;
        }
        namespace IO
        {
            class IDirectory;
            class IFile;

            /**
             * @class  FatFileManager
             * @brief  Copy and verify files recursively with available vfat partitions.
             * @author manuel.arguelles
             * @date   Aug  3, 2017
             */
            class FatFileManager
            {
                public:

                    /**
                     * @brief  Class constructor.
                     * @author manuel.arguelles
                     * @date   Aug  3, 2017
                     */
                    FatFileManager();

                    /**
                     * @brief  Class constructor.
                     * @author manuel.arguelles
                     * @date   Aug  3, 2017
                     */
                    FatFileManager(System::IO::IDirectory& directoryInterface,
                                   System::IO::IFile& fileInterface);

                    /**
                     * @brief  Class destructor.
                     * @author manuel.arguelles
                     * @date   Aug  3, 2017
                     */
                    ~FatFileManager();

                public:

                    /**
                     * @brief  Add a device where to copy or verify.
                     * @author manuel.arguelles
                     * @date   Aug  8, 2017
                     * @param  device The device to add.
                     */
                    void addStorageDevice(System::Runtime::IStorageDevice* device);

                    /**
                     * @brief  Removes a storage device.
                     * @author manuel.arguelles
                     * @date   Aug  8, 2017
                     * @param  device The device to remove.
                     */
                    void removeStorageDevice(System::Runtime::IStorageDevice* device);

                    /**
                     * @brief  Adds one FAT storage per device.
                     * @author manuel.arguelles
                     * @date   Sep  6, 2017
                     */
                    void addFatDevices();

                    /**
                     * @brief  Gets the number of registered fat devices.
                     * @author manuel.arguelles
                     * @date   Sep  6, 2017
                     * @return The number of devices to use.
                     */
                    std::string::size_type getNumberOfDevices() const;

                    /**
                     * @brief  Copy all files recursively from a giving path to all the vfat partitions.
                     * @author manuel.arguelles
                     * @date   Aug  3, 2017
                     * @param  sourcePath The path with the files to copy.
                     * @param  stripFinalPath The path present in the sourcePath that will be removed from
                     * the final path in the destination.
                     * @return true if the operation succeed, false otherwise.
                     */
                    bool copyFiles(const std::string& sourcePath, const std::string& stripFinalPath = "");

                    /**
                     * @brief  Verify all files recursively from a giving path to all the vfat partitions.
                     * @author manuel.arguelles
                     * @date   Aug  3, 2017
                     * @param  sourcePath The path with the files to verify.
                     * @param  stripFinalPath The path present in the sourcePath that will be removed from
                     * the final path in the destination.
                     * @return true if the operation succeed, false otherwise.
                     */
                    bool verifyFiles(const std::string& sourcePath, const std::string& stripFinalPath = "");

                    /**
                     * @brief  Deletes given directory from all vfat partitions.
                     * @author eduardo.fernandez
                     * @date   Jan 3, 2019
                     * @param  directory Directory to delete from all vfat partitions.
                     * @return true if the operation succeeds, false otherwise.
                     */
                    bool deleteDirectory(const std::string& directory);

                private:

                    /**
                     * @brief  Mount all the fat devices.
                     * @author manuel.arguelles
                     * @date   Aug  7, 2017
                     * @return True if all the devices were mounted, false otherwise.
                     */
                    bool mountDevices();

                    /**
                     * @brief  Unmount all the fat devices.
                     * @author manuel.arguelles
                     * @date   Aug  7, 2017
                     * @return True if all the devices were unmounted, false otherwise.
                     */
                    bool unmountDevices();

                    /**
                     * @brief  Walks a directory recursively copying its content to the storage devices.
                     * @author manuel.arguelles
                     * @date   Aug  7, 2017
                     * @param  path The root path.
                     * @param  maxDepth The maximum levels to transverse.
                     * @return True on success, false otherwise.
                     */
                    bool copyWalker(const std::string& path, int maxDepth);

                    /**
                     * @brief  Walks a directory recursively verifying its content with the storage devices.
                     * @author manuel.arguelles
                     * @date   Aug  7, 2017
                     * @param  path The root path.
                     * @param  maxDepth The maximum levels to transverse.
                     * @return True on success, false otherwise.
                     */
                    bool verifyWalker(const std::string& path, int maxDepth);

                    /**
                     * @brief  Copy the source file (or directory) to the devices.
                     * @author manuel.arguelles
                     * @date   Aug  7, 2017
                     * @param  sourceFile The source file or directory to copy.
                     */
                    bool copyToDevice(const std::string& sourceFile);

                    /**
                     * @brief  Verify the source file (or directory) with the devices.
                     * @author manuel.arguelles
                     * @date   Aug  7, 2017
                     * @param  sourceFile The source file or directory to verify.
                     */
                    bool verifyWithDevice(const std::string& sourceFile);

                    /**
                     * @brief  Frees the allocated storage devices.
                     * @author manuel.arguelles
                     * @date   Sep  6, 2017
                     */
                    void freeStorageDevices();

                private:

                    std::string                                   m_pathToStrip;
                    std::vector<System::Runtime::IStorageDevice*> m_storageDevices;
                    System::IO::IDirectory&                       m_directoryInterface;
                    System::IO::IFile&                            m_fileInterface;
                    static Smartmatic::Log::ISMTTLog*             m_logger;
            };

        } /* namespace IO */
    } /* namespace System */
} /* namespace Smartmatic */

#endif /* FATFILEMANAGER_HXX_ */
