/**
 * @file File.h
 * @brief Class util for files
 */

#ifndef __FILE_H__
#define __FILE_H__

#include <string>
#include <System/SafeBuffer.h>
#include <System/IO/IFile.hxx>

namespace Smartmatic
{
    namespace Log
    {
        class ISMTTLog;
    }

    namespace System
    {
        namespace IO
        {
            /**
             * @class	File
             * @brief	Class for file utils.
             * @author	Yadickson Soto &lt;yadickson.soto@smartmatic.com&gt;
             * @date	21-02-2011
             */
            class File : public IFile
            {
                protected:

                    /**
                     * @fn	File();
                     * @brief	Default Constructor.
                     * @author	Yadickson Soto
                     * @date	14-11-2013
                     */
                    File();

                public:

                    /**
                     * @fn	virtual ~Directory();
                     * @brief	Class destructor.
                     * @author	Yadickson Soto
                     * @date	14-11-2013
                     */
                    virtual ~File();

                    /**
                    * @fn    virtual bool File::canOpenFile (const std::string & file);
                    * @brief	method to obtain is can open file.
                    * @author	Youssef.tourkmani
                    * @date    29/10/2012
                    * @param	file	file.
                    * @return	true is can open file.
                    */
                    virtual bool canOpenFile ( const std::string & file );

                    /**
                    * @fn    virtual bool File::readFile(const std::string & file, std::string & content);
                    * @brief    method to obtain the content of file.
                    * @author    Youssef.tourkmani
                    * @date    29/10/2012
                    * @param    file	 	file.
                    * @param    content	string of file.
                    * @return    true is can read file.
                    */
                    virtual bool readFile ( const std::string & file, std::string & content );

                    /**
                     * @fn	virtual bool File::DeleteFile(const std::string & fullPath);
                     * @brief	Deletes the file described by fullPath.
                     * @author	Yadickson Soto
                     * @date	28/04/2011
                     * @param	fullPath	Full pathname of the full file.
                     * @return	true if it succeeds, false if it fails.
                     */
                    virtual bool DeleteFile ( const std::string & fullPath );

                    /**
                     * @fn	virtual bool File::copyFile(const std::string & inputFile, const std::string & outputFile);
                     * @brief	Copies the file.
                     * @author	Yadickson Soto
                     * @date	28/04/2011
                     * @param	inputFile 	The input file.
                     * @param	outputFile	The output file.
                     * @return	true if it succeeds, false if it fails.
                     */
                    virtual bool copyFile ( const std::string & inputFile, const std::string & outputFile );

                    /**
                     * @fn	virtual bool File::compareFiles(const std::string & fileOne, const std::string & fileTwo);
                     * @brief	compare files.
                     * @author	Yadickson Soto
                     * @date	09/05/2011
                     * @param	fileOne	The first file.
                     * @param	fileTwo	The second file.
                     * @return	true if it succeeds, false if it fails.
                     */
                    virtual bool compareFiles ( const std::string & fileOne, const std::string & fileTwo );

                    /**
                     * @fn	virtual bool File::getHashFile(const std::string & file, Smartmatic::System::SafeBuffer & dataOut);
                     * @brief	Method getter hash file.
                     * @author	Yadickson Soto
                     * @date	04/06/2011
                     * @param	file	 	The file.
                     * @param	dataOut	data out buffer.
                     * @return true if successfully
                     */
                    virtual bool getHashFile ( const std::string & file, Smartmatic::System::SafeBuffer & dataOut );

                    /**
                     * @fn  virtual bool File::getHashFileByChunks(const std::string & file, Smartmatic::System::SafeBuffer & dataOut);
                     * @brief   Method getter hash file by chunks.
                     * @author  eduardo.fernandez
                     * @date    17/01/2019
                     * @param   file        The file.
                     * @param   chunkSize   The chunk size.
                     * @param   dataOut data out buffer.
                     * @return true if successfully
                     */
                    virtual bool getHashFileByChunks ( const std::string & file, int chunkSize, Smartmatic::System::SafeBuffer & dataOut );

                    /**
                     * @fn	virtual bool File::saveBufferToFile(const Smartmatic::System::SafeBuffer & buffer,
                     * 			const std::string & file);
                     * @brief	Method save buffer to file.
                     * @author	Yadickson Soto
                     * @date	08/06/2011
                     * @param	buffer	The buffer.
                     * @param	file		The file.
                     * @return	true is success.
                     */
                    virtual bool saveBufferToFile ( const Smartmatic::System::SafeBuffer & buffer, const std::string & file );

                    /**
                     * @fn	virtual bool File::addBufferToFile(const Smartmatic::System::SafeBuffer & buffer,
                     * 			const std::string & file);
                     * @brief	Method add buffer to file.
                     * @author	Yadickson Soto
                     * @date	12/06/2013
                     * @param	buffer	The buffer.
                     * @param	file		The file.
                     * @return	true is success.
                     */
                    virtual bool addBufferToFile ( const Smartmatic::System::SafeBuffer & buffer, const std::string & file );

                    /**
                     * @fn	virtual bool File::openFileToBuffer( const std::string & file,
                     * 			Smartmatic::System::SafeBuffer & dataOut);
                     * @brief	Method open file to buffer.
                     * @author	Yadickson Soto
                     * @date	04/06/2011
                     * @param	file	 	The file.
                     * @param	dataOut	data out buffer.
                     * @return	true is success.
                     */
                    virtual bool openFileToBuffer ( const std::string & file, Smartmatic::System::SafeBuffer & dataOut );

                    /**
                     * @fn	virtual bool File::renameFile( const std::string & oldName, const std::string & newName);
                     * @brief	Method rename file.
                     * @author	Yadickson Soto
                     * @date	04/06/2011
                     * @param	oldName	The old file name.
                     * @param	newName	The new file name.
                     * @return	true is success.
                     */
                    virtual bool renameFile ( const std::string & oldName, const std::string & newName );

                    /**
                     * @brief  Copy directory recursively.
                     * @author manuel.arguelles
                     * @date   sep 27, 2018
                     * @param  origin      The origin path.
                     * @param  destination The destination path.
                     * @return True if the operation succeed, false if not.
                     */
                    virtual bool copyDirectory ( const std::string & origin, const std::string & destination );

                    /**
                     * @fn	static File * getInstance ( File * externFile = NULL );
                     * @brief	Getter method instance of class.
                     * @author	Yadickson Soto
                     * @date	14/11/2013
                     * @param 	externFile external FIle to unit test
                     * @return	instance of class.
                     */
                    static File * getInstance ( File * externFile = NULL );

                private:

                    static File * instance; ///< instance of class
                    static Smartmatic::Log::ISMTTLog * logger; ///< The logger
            };
        }
    }
}

#endif /*__FILE_H__*/
