/**
 * @file ProcessLauncher.h
 * @brief Header process launcher class
 * @date 04/05/2011
 * @author Freddy Gomez
 */

#ifndef PROCESSLAUNCHER_H_
#define PROCESSLAUNCHER_H_

#include <string>
#include <list>
#include <System/IO/PipeFile.hxx>
#include <System/SafeList.hxx>

namespace Smartmatic
{
    namespace Log
    {
        class ISMTTLog;
    }

    namespace System
    {
        namespace Runtime
        {

            /**
            * @class	ProcessLauncher
            * @brief	Class definition file.
            * @author	Freddy Gomez &lt;fgomez@smartmatic.com&gt;
            * @date	04-05-2011
            */
            class ProcessLauncher
            {
                protected:

                    /**
                     * @fn	ProcessLauncher::ProcessLauncher();
                     * @brief	Default Constructor.
                     * @author	Freddy Gomez
                     * @date	04-05-2011
                     */
                    ProcessLauncher();

                public:

                    /**
                     * @fn	virtual ProcessLauncher::~ProcessLauncher();
                     * @brief	Class destructor.
                     * @author	Freddy Gomez
                     * @date	04/05/2011
                     */
                    virtual ~ProcessLauncher();

                    /**
                     * @fn	virtual Smartmatic::System::SafeList < std::string > RunProcessThroughPipe ( const std::string & commandLineWithArguments, Smartmatic::System::IO::PipeFile * pipeFile = NULL );
                     * @brief	This method run the process specified in the command line using the popen/pclose function.
                     * @author	Freddy Gomez, Yadickson Soto
                     * @date	04/05/2011
                     * @param [in]	commandLineWithArguments	process command line with arguments.
                     * @param	pipeFile	The pipe file to unint test
                     * @return	The data taken from the stdout of the process.
                     */
                    virtual Smartmatic::System::SafeList < std::string > RunProcessThroughPipe ( const std::string & commandLineWithArguments, Smartmatic::System::IO::PipeFile * pipeFile = NULL );

                    /**
                     * @fn	virtual int ProcessLauncher::RunProcessThroughSystem(const std::string & commandLineWithArguments, bool background = false);
                     * @brief	This method run the process specified in the command line using the system function.
                     * @author	Freddy Gomez
                     * @date	04/05/2011
                     * @param [in]	commandLineWithArguments	process command line with arguments.
                     * @param 	background run background script if true
                     * @return	The same return value of the "system" function.
                     */
                    virtual int RunProcessThroughSystem ( const std::string & commandLineWithArguments, bool background = false );

                    /**
                     * @fn	static ProcessLauncher * getInstance( ProcessLauncher * externLauncher = NULL );
                     * @brief	Getter method instance of class.
                     * @author	Yadickson Soto
                     * @date	31/10/2013
                     * @param 	externalLauncher external launcher to unit test
                     * @return	instance of class.
                     */
                    static ProcessLauncher * getInstance ( ProcessLauncher * externLauncher = NULL );

                protected:

                    /**
                     * @fn	virtual std::string getCommand( const std::string & command, bool background );
                     * @brief	This method make command line to background or not.
                     * @author	Yadickson Soto
                     * @date	13/11/2013
                     * @param [in]	command	process command line with arguments.
                     * @param 	background run background script if true
                     * @return	The final command line with end & to backgraound, else only command.
                     */
                    virtual std::string getCommand ( const std::string & command, bool background );

                    /**
                     * @fn	virtual int runCommand( const std::string & command );
                     * @brief	This method run command line to system.
                     * @author	Yadickson Soto
                     * @date	13/11/2013
                     * @param [in]	command	process command line with arguments.
                     * @return	The same return value of the "system" function.
                     */
                    virtual int runCommand ( const std::string & command );

                private:

                    static ProcessLauncher * instance; ///< instance of class
                    static Smartmatic::Log::ISMTTLog * logger; ///<logger
            };
        }
    }
}

#endif /* PROCESSLAUNCHER_H_ */
