/**
 * @file   StorageDevice.hxx
 * @brief  Declaration of the StorageDevice class.
 * @author manuel.arguelles
 * @date   Aug  2, 2017
 */

#ifndef STORAGEDEVICE_HXX_
#define STORAGEDEVICE_HXX_

#include "IStorageDevice.hxx"
#include <string>
#include <vector>
#include <glibmm.h>
#include <giomm.h>

namespace Smartmatic
{
    namespace System
    {
        namespace Runtime
        {
            /**
             * @class  StorageDevice
             * @brief  Handles mounting of storage devices.
             * @author manuel.arguelles
             * @date   Aug  2, 2017
             */
            class StorageDevice : public IStorageDevice
            {
                public:

                    /**
                     * @brief  Class constructor.
                     * @author manuel.arguelles
                     * @date   Aug  2, 2017
                     * @param  devicePath The path of the device (Usually in /dev/).
                     */
                    explicit StorageDevice(const std::string& devicePath);

                    /**
                     * @brief  Class destructor.
                     * @author manuel.arguelles
                     * @date   Aug  2, 2017
                     */
                    virtual ~StorageDevice();

                public:

                    /**
                     * @brief  Gets all the storage devices of the given file system type.
                     * @author manuel.arguelles
                     * @date   Aug  2, 2017
                     * @param  fileSystem The file system type.
                     * @return A vector of StorageDevices with the giving file system.
                     */
                    static std::vector<StorageDevice> getAllDevicesByFileSystem(const std::string& fileSystem);

                    /**
                     * @brief  Gets the first storage device found of the given file system type per physical device.
                     * @remark If more than one storage device with the given file system is present on the same
                     * physical device, only the first one detected (without guarantees of order, will be returned.
                     *
                     * @author manuel.arguelles
                     * @date   Sep 19, 2017
                     * @param  fileSystem The file system type.
                     * @return A vector of StorageDevices.
                     */
                    static std::vector<StorageDevice> getUniqueDevicesByFileSystem(const std::string& fileSystem);

                public:

                    /**
                     * @brief  Check if the storage device is mounted.
                     * @author manuel.arguelles
                     * @date   Aug  2, 2017
                     * @return true if mounted, false otherwise.
                     */
                    virtual bool isMounted() const;

                    /**
                     * @brief  Mount the storage device.
                     * @author manuel.arguelles
                     * @date   Aug  2, 2017
                     * @return true if the device was mounted, false otherwise.
                     */
                    virtual bool mount();

                    /**
                     * @brief  Unmount the storage device.
                     * @author manuel.arguelles
                     * @date   Aug  2, 2017
                     * @return true if the device was unmounted, false otherwise.
                     */
                    virtual bool unmount();

                    /**
                     * @brief  Gets the path where the device is mounted.
                     * @author manuel.arguelles
                     * @date   Aug  2, 2017
                     * @return The path where the device is mounted.
                     */
                    virtual std::string getMountPoint() const;

                    /**
                     * @brief  Gets the path of the device.
                     * @author manuel.arguelles
                     * @date   Aug  2, 2017
                     * @return The device path.
                     */
                    virtual std::string getDevicePath() const;

                    /**
                     * @brief  Checks if the given StorageDevice belongs to the same device.
                     *         An StorageDevice of /dev/sda1 should be the same device as an
                     *         StorageDevice of /dev/sda2.
                     * @author manuel.arguelles
                     * @date   Sep  6, 2017
                     * @param  other The StorageDevice to check.
                     * @return True if the StorageDevice belongs to the same device, false if not.
                     */
                    virtual bool isSameDevice(const IStorageDevice& other) const;

                private:

                    /**
                     * @brief  Called when the mount operation completes.
                     * @author manuel.arguelles
                     * @date   Sep 14, 2017
                     * @param  result The result of the operation.
                     */
                    void mountOperationReady(const Glib::RefPtr<Gio::AsyncResult>& result);

                    /**
                     * @brief  Called when the unmount operation completes.
                     * @author manuel.arguelles
                     * @date   Sep 14, 2017
                     * @param  result The result of the operation.
                     */
                    void unmountOperationReady(const Glib::RefPtr<Gio::AsyncResult>& result);

                    /**
                     * @brief  Waits until mount/unmount operation completes.
                     * @param  cancellable The cancellable object in case of timeout.
                     * @author manuel.arguelles
                     * @date   Sep 14, 2017
                     */
                    void waitUntilReady(Glib::RefPtr<Gio::Cancellable>& cancellable) const;

                private:

                    Glib::RefPtr<Gio::Mount>  m_mount;
                    Glib::RefPtr<Gio::Volume> m_volume;
                    bool                      m_isReady;
                    std::string               m_devicePath;
            };

        } /* namespace Runtime */
    } /* namespace System */
} /* namespace Smartmatic */

#endif /* STORAGEDEVICE_HXX_ */
