/**
 * @file SafeBuffer.cpp
 * @brief Body class safe buffer
 * @date 11/04/2011
 * @author Freddy Gomez
 */

#include "System/Encoding/Base64Encoder.h"
#include <cstring>
#include <stdlib.h>
#include <sstream>

using namespace Smartmatic::System;
using namespace Smartmatic::System::Encoding;

void operator<< ( std::ostream & out, const SafeBuffer & buffer )
{
    out << "SafeBuffer(" << buffer.GetSize() << ")";
}

void operator<< ( std::string & out, const SafeBuffer & buffer )
{
    std::stringstream ss;
    ss << "SafeBuffer(" << buffer.GetSize() << ")";
    out.append ( ss.str() );
}

SafeBuffer & SafeBuffer::operator= ( const SafeBuffer & dbuf )
{
    if ( this != &dbuf )
    {
        m_data = dbuf.m_data;
    }

    return *this;
}

bool SafeBuffer::operator== ( const SafeBuffer & rhs )
{
    return m_data == rhs.m_data;
}

bool SafeBuffer::operator!= ( const SafeBuffer & rhs )
{
    return m_data != rhs.m_data;
}

SafeBuffer::SafeBuffer ()
{
}

SafeBuffer::~SafeBuffer ()
{
    Clear ();
}

SafeBuffer::SafeBuffer ( const std::string & data )
{
    AddData ( data );
}

SafeBuffer::SafeBuffer ( unsigned long & data, int resultSize )
{
    AddData ( data, resultSize );
}

SafeBuffer::SafeBuffer ( unsigned char * buffer, const size_t & bufSize )
{
    AddData ( buffer, bufSize );
}


SafeBuffer::SafeBuffer ( const SafeBuffer & other )
{
    AddData ( other );
}

void SafeBuffer::AddData ( const unsigned char * moreData, const size_t & moreDataSize )
{
    if (moreData != NULL)
    {
        m_data.insert(m_data.end(), moreData, moreData + moreDataSize);
    }
}

void SafeBuffer::AddData ( unsigned long & data, int resultSize )
{
    std::vector<unsigned char> vector(resultSize);

    memcpy(&vector[0], &data, resultSize);

    m_data.insert(m_data.end(), vector.begin(), vector.end());
}

void SafeBuffer::AddData ( const SafeBuffer & moreData )
{
    m_data.insert(m_data.end(), moreData.m_data.begin(), moreData.m_data.end());
}

void SafeBuffer::AddData ( const std::string & moreData )
{
    AddData(reinterpret_cast<const unsigned char*>(moreData.c_str()), moreData.size());
}

void SafeBuffer::push_back ( unsigned char data )
{
    m_data.push_back(data);
}

std::string SafeBuffer::GetBase64Data () const
{
    std::string result;

    Base64Encoder::ConvertToBase64 ( *this, result );
    return result;
}

void SafeBuffer::Clear ()
{
    m_data.clear();
}

void SafeBuffer::insertData ( unsigned long index, const unsigned char * moreData, const size_t & moreDataSize )
{
    SafeBuffer newBuffer(const_cast<unsigned char*>(moreData), moreDataSize);

    insertData(index, newBuffer);
}

void SafeBuffer::insertData ( unsigned long index, const SafeBuffer & moreData )
{
    if (index > m_data.size())
    {
        index = 0;
    }

    m_data.insert(m_data.begin() + index, moreData.m_data.begin(), moreData.m_data.end());
}

void SafeBuffer::insertData ( unsigned long index, const std::string & moreData )
{
    if (index > m_data.size())
    {
        index = 0;
    }

    m_data.insert(m_data.begin() + index, moreData.begin(), moreData.end());
}

void SafeBuffer::removeData ( unsigned long index, const size_t & size )
{
    if (!m_data.empty() && index < m_data.size())
    {
        if ((index + size) < m_data.size())
        {
            m_data.erase(m_data.begin() + index, m_data.begin() + (index + size));
        }
        else
        {
            m_data.erase(m_data.begin() + index, m_data.end());
        }
    }
}

const unsigned char * SafeBuffer::GetData () const
{
    return (m_data.empty() ? NULL : &m_data[0]);
}

size_t SafeBuffer::GetSize () const
{
    return m_data.size();
}

std::vector<unsigned char>& SafeBuffer::GetVector() const
{
    return const_cast<std::vector<unsigned char>&>(m_data);
}
