/**
 * @file	SafeBuffer.h
 *
 * @brief	Header file to the safe buffer class.
 */

#ifndef SAFEBUFFER_H_
#define SAFEBUFFER_H_

#include <string>
#include <iostream>
#include <vector>

namespace Smartmatic
{
    namespace System
    {
        /**
         * @class	SafeBuffer
         * @brief	Buffer for safe class.
         * @author	freddyg
         * @date	28/04/2011
         */
        class SafeBuffer
        {

            public:

                /**
                 * @fn	SafeBuffer::SafeBuffer ();
                 * @brief	Default constructor.
                 * @author	freddyg
                 * @date	28/04/2011
                 */
                SafeBuffer ();

                /**
                 * @fn	SafeBuffer::SafeBuffer ( const std::string & data );
                 * @brief	Constructor.
                 * @author	Yadickson Soto yadickson.soto@smartmatic.com
                 * @date	02/06/2011
                 * @param	data	string to convert.
                 */
                SafeBuffer ( const std::string & data );

                /**
                 * @fn	SafeBuffer::SafeBuffer ( unsigned long & data, int resultSize );
                 * @brief	Constructor.
                 * @author	Yadickson Soto yadickson.soto@smartmatic.com
                 * @date	02/06/2011
                 * @param	data				long data to convert.
                 * @param	resultSize	result size to buffer.
                 */
                SafeBuffer ( unsigned long & data, int resultSize );

                /**
                 * @fn	SafeBuffer::SafeBuffer ( unsigned char * digestBuffer, const size_t & bufSize );
                 * @brief	Constructor.
                 * @author	freddyg
                 * @date	28/04/2011
                 * @param	digestBuffer	If non-null, buffer for digest data.
                 * @param	bufSize				Size of the buffer.
                 */
                SafeBuffer ( unsigned char * digestBuffer, const size_t & bufSize );

                /**
                 * @fn	SafeBuffer::SafeBuffer ( const SafeBuffer& buff );
                 * @brief	Copy constructor.
                 * @author	freddyg
                 * @date	28/04/2011
                 * @param	buff	The buffer.
                 */
                SafeBuffer ( const SafeBuffer & buff );

                /**
                 * @fn	virtual SafeBuffer::~SafeBuffer ();
                 * @brief	Finaliser.
                 * @author	freddyg
                 * @date	28/04/2011
                 */
                virtual ~SafeBuffer ();

                /**
                 * @fn	SafeBuffer& SafeBuffer::operator= ( const SafeBuffer& sk );
                 * @brief	Assignment operator.
                 * @author	freddyg
                 * @date	28/04/2011
                 * @param	sk	The sk.
                 * @return	A shallow copy of this object.
                 */
                SafeBuffer & operator= ( const SafeBuffer & sk );

                /**
                 * @fn	bool SafeBuffer::operator== ( const SafeBuffer& rhs );
                 * @brief	Method compare buffer.
                 * @author	Yadickson Soto
                 * @date	24/02/2012
                 * @param	rhs	buffer.
                 * @return	true is equals.
                 */
                bool operator== ( const SafeBuffer & rhs );

                /**
                 * @fn	bool SafeBuffer::operator!= ( const SafeBuffer& rhs );
                 * @brief	Method compare buffer.
                 * @author	Yadickson Soto
                 * @date	24/02/2012
                 * @param	rhs	buffer.
                 * @return	false is equals.
                 */
                bool operator!= ( const SafeBuffer & rhs );

                /**
                 * @fn	void SafeBuffer::AddData ( const unsigned char * moreData, const size_t & moreDataSize );
                 * @brief	Adds a data to 'moreDataSize'.
                 * @author	freddyg
                 * @date	28/04/2011
                 * @param	moreData			Data to add.
                 * @param	moreDataSize	Size of the data to add.
                 */
                void AddData ( const unsigned char * moreData, const size_t & moreDataSize );

                /**
                 * @fn	void AddData ( unsigned long & data, int resultSize );
                 * @brief	Adds a data to 'moreDataSize'.
                 * @author	Yadickson Soto
                 * @date	25/06/2015
                 * @param	data		long data to convert.
                 * @param	resultSize	result size to buffer.
                 */
                void AddData ( unsigned long & data, int resultSize );

                /**
                 * @fn	void SafeBuffer::AddData ( const SafeBuffer & moreData );
                 * @brief	Adds a buffer data.
                 * @author	Yadickson Soto yadickson.soto@smartmatic.com
                 * @date	02/06/2011
                 * @param	moreData	Data to add.
                 */
                void AddData ( const SafeBuffer & moreData );

                /**
                 * @fn	void SafeBuffer::AddData ( const std::string & moreData );
                 * @brief	Adds a buffer data.
                 * @author	Yadickson Soto yadickson.soto@smartmatic.com
                 * @date	24/02/2012
                 * @param	moreData	Data to add.
                 */
                void AddData ( const std::string & moreData );

                /**
                 * @fn	void SafeBuffer::push_back ( unsigned char data );
                 * @brief	Method insert back data in buffer.
                 * @author	Yadickson Soto
                 * @date	24/02/2012
                 * @param	data	new data.
                 */
                void push_back ( unsigned char data );

                /**
                 * @fn	const unsigned char * SafeBuffer::GetData () const
                 * @brief	Gets the data.
                 * @author	freddyg
                 * @date	28/04/2011
                 * @return	null if it fails, else the data.
                 */
                const unsigned char * GetData () const;

                /**
                 * @fn	size_t SafeBuffer::GetSize () const
                 * @brief	Gets the size.
                 * @author	freddyg
                 * @date	28/04/2011
                 * @return	The size of the Buffer.
                 */
                size_t GetSize () const;

                /**
                 * @fn	std::string SafeBuffer::GetBase64Data () const;
                 * @brief	Gets the base 64 data.
                 * @author	freddyg
                 * @date	28/04/2011
                 * @return	The date in format Base 64.
                 */
                std::string GetBase64Data () const;

                /**
                 * @fn	void SafeBuffer::Clear ();
                 * @brief	Method clear buffer.
                 * @author	Yadickson Soto
                 * @date	28/04/2011
                 */
                void Clear ();

                /**
                 * @fn	void SafeBuffer::insertData ( unsigned long index, const unsigned char * moreData,
                 * 			const size_t & moreDataSize );
                 * @brief	Method insert data in buffer.
                 * @author	Yadickson Soto
                 * @date	24/02/2012
                 * @param	index					index to insert data.
                 * @param	moreData			Data to insert.
                 * @param	moreDataSize	Size of the data to insert.
                 */
                void insertData ( unsigned long index, const unsigned char * moreData, const size_t & moreDataSize );

                /**
                 * @fn	void SafeBuffer::insertData ( unsigned long index, const SafeBuffer & moreData );
                 * @brief	Method insert data in buffer.
                 * @author	Yadickson Soto
                 * @date	24/02/2012
                 * @param	index			index to insert data.
                 * @param	moreData	Data to insert.
                 */
                void insertData ( unsigned long index, const SafeBuffer & moreData );

                /**
                 * @fn	void SafeBuffer::insertData ( unsigned long index, const std::string & moreData );
                 * @brief	Method insert data in buffer.
                 * @author	Yadickson Soto
                 * @date	24/02/2012
                 * @param	index			index to insert data.
                 * @param	moreData	Data to insert.
                 */
                void insertData ( unsigned long index, const std::string & moreData );

                /**
                 * @fn	void SafeBuffer::removeData ( unsigned long index, const size_t & size );
                 * @brief	Method remove data in buffer.
                 * @author	Yadickson Soto
                 * @date	25/02/2012
                 * @param	index	index to begin find data.
                 * @param	size 	size remove.
                 */
                void removeData ( unsigned long index, const size_t & size );

                /**
                 * @brief  Return a reference to the vector storing the data.
                 * @remark Used only in SafeBufferUtils for performance reasons.
                 * @author manuel.arguelles
                 * @date   Oct 31, 2017
                 */
                std::vector<unsigned char>& GetVector() const;

            private:

                std::vector<unsigned char> m_data;
        };
    }
}

/**
 * @fn	void operator<< ( std::ostream & out, const Smartmatic::System::SafeBuffer & buffer );
 * @brief	Overload operator << .
 * @author	Yadickson Soto
 * @date	21/04/2014
 * @param	out	output stream
 * @param	buffer to output (only buffer size will be output)
 */
void operator<< ( std::ostream & out, const Smartmatic::System::SafeBuffer & buffer );

/**
 * @fn	void operator<< ( std::string & out, const Smartmatic::System::SafeBuffer & buffer );
 * @brief	Overload operator << .
 * @author	Yadickson Soto
 * @date	21/04/2014
 * @param	out	output string
 * @param	buffer to output (only buffer size will be output)
 */
void operator<< ( std::string & out, const Smartmatic::System::SafeBuffer & buffer );

#endif /* SAFEBUFFER_H_ */
