/**
 * @file SafeThread.hxx
 * @brief Declaration of the SafeThread class.
 * @date 07/06/2015
 * @author Yadickson Soto
 */

#ifndef SAFE_THREAD_HXX_
#define SAFE_THREAD_HXX_

#include <glibmm.h>

namespace Smartmatic
{
    namespace System
    {
        /**
         * @class SafeThread
         * @brief Class safe thread.
         * @author Yadickson Soto &lt;yadickson.soto@smartmatic.com&gt;
         * @date 07/06/2015
         */
        class SafeThread
        {
            private:

                Glib::Mutex mutex;	///< The mutex
                Glib::Thread * m_thread; ///< The thread pointer to check press button
                unsigned int valueTime; ///< value time
                bool flagRun; ///< flag is runnnig poll
                bool flagThreadRun; ///< The flag to thread run

            public:

                /**
                 * @fn SafeThread::SafeThread();
                 * @brief Default constructor for the SafeThread class.
                 * @author Yadickson Soto &lt;yadickson.soto@smartmatic.com&gt;
                 * @date 07/06/2015
                 */
                SafeThread();

                /**
                 * @fn virtual SafeThread::~SafeThread();
                 * @brief Finalizes an instance of the SafeThread class.
                 * @author Yadickson Soto &lt;yadickson.soto@smartmatic.com&gt;
                 * @date 07/06/2015
                 */
                virtual ~SafeThread();

                /**
                 * @fn	    void start ( unsigned int valueTime = 1000 );
                 * @brief	Start thread to check press button.
                 * @author	Yadickson Soto
                 * @date	07/06/2015
                 * @param 	valueTime value time to wait in milliseconds.
                 */
                void start ( unsigned int valueTime = 1000 );

                /**
                 * @fn	    void stop ();
                 * @brief	Stop thread to check press button.
                 * @author	Yadickson Soto
                 * @date	07/06/2015
                 */
                void stop ();

                /**
                 * @fn bool isRunning();
                 * @brief Verifies if the thread is running.
                 * @author Yadickson Soto &lt;yadickson.soto@smartmatic.com&gt;
                 * @date 07/06/2015
                 * @return true if is running. Otherwise, false.
                 */
                bool isRunning();

            private:

                /**
                 * @fn	void onThread ();
                 * @brief	Thread for check device.
                 * @author	Yadickson Soto
                 * @date	07/06/2015
                 */
                void onThread ();

            protected:

                /**
                 * @fn	    virtual void preStart ();
                 * @brief	Callback from thread before start thread.
                 * @author	Yadickson Soto
                 * @date	07/06/2015
                 */
                virtual void preStart ();

                /**
                 * @fn	    virtual bool process () = 0;
                 * @brief	Callback from thread any time.
                 * @author	Yadickson Soto
                 * @date	07/06/2015
                 * @return true for continue thread
                 */
                virtual bool process () = 0;

                /**
                 * @fn	    virtual void postStop ();
                 * @brief	Callback from thread after stop thread.
                 * @author	Yadickson Soto
                 * @date	07/06/2015
                 */
                virtual void postStop ();
        };
    }
}

#endif /* SAFE_THREAD_HXX_ */
