/**
 * @file Crypto.hxx
 * @brief Header class for Crypto file
 */

#ifndef __CRYPTO__H__
#define __CRYPTO__H__

#include <string>
#include "System/Exception/CryptoException.hxx"
#include "System/SafeBuffer.h"

namespace Smartmatic
{
    namespace Log
    {
        class ISMTTLog;
    }

    namespace System
    {
        namespace Security
        {
            namespace Cryptography
            {
                /**
                 * @brief forward declaration
                 */
                class CryptoAES;
            }

            /**
             * @class	Crypto
             * @brief	Class utils for crypto data.
             * @author	Yadickson Soto
             * @date	31-03-2011
             */
            class Crypto
            {
                public:

                    /**
                     * @enum KeySize
                     */
                    typedef enum
                    {
                        KEY_AES_128 ///<key aes 128
                        , KEY_AES_192 ///<key aes 192
                        , KEY_AES_256 ///<key aes 256
                    } KeySize;

                    /**
                     * @enum Mode
                     */
                    typedef enum
                    {
                        MODE_AES_CBC /**<Cipher Block Chaining*/
                        , MODE_AES_CFB1 /**<Cipher FeedBack*/
                        , MODE_AES_CFB8 /**<Cipher FeedBack*/
                        , MODE_AES_CFB128 /**<Cipher FeedBack*/
                        , MODE_AES_ECB /**<Electronic CodeBook*/
                        , MODE_AES_OFB /**<Output FeedBack*/
                    } Mode;

                    /**
                     * @fn	Crypto();
                     * @brief	Class constructor.
                     * @author	Youssef.tourkmani
                     * @date	29/10/2012
                     */
                    Crypto();
                    /**
                     * @fn	~Crypto();
                     * @brief	Class destructor.
                     * @author	Youssef.tourkmani
                     * @date	29/10/2012
                     */
                    virtual ~Crypto();
                    /**
                     * @fn	void init(KeySize keySize, Mode mode,
                     * 			bool pad) throw (Smartmatic::System::Exception::CryptoException);
                     * @brief	method for init class.
                     * @author	Youssef.tourkmani
                     * @date	29/10/2012
                     * @exception	Smartmatic::System::Exception::CryptoException	Thrown when a crypto error condition occurs.
                     * @param	keySize	key size.
                     * @param	mode	 	encrypt mode.
                     * @param	pad		 	true padding
                     * @throw CryptoException.
                     */
                    void init ( KeySize keySize, Mode mode, bool pad )
                    throw ( Smartmatic::System::Exception::CryptoException );
                    /**
                     * @fn	void generateRandomIV(const Smartmatic::System::SafeBuffer & iv) throw (Smartmatic::System::Exception::CryptoException);
                     * @brief	method for generating a random IV.
                     * @author	marcel.mancini
                     * @date	06/07/2017
                     * @exception	Smartmatic::System::Exception::CryptoException	Thrown when a iv generation exception occurs.
                     * @param	iv	initialization vector buffer to fill
                     * @throw CryptoException.
                     */
                    void generateRandomIV(Smartmatic::System::SafeBuffer & iv)
                    throw ( Smartmatic::System::Exception::CryptoException );

                    void generateZeroIV(Smartmatic::System::SafeBuffer& iv);
                    /**
                     * @fn	void encryptBufferToFileAndCreateIV(const Smartmatic::System::SafeBuffer & dataIn, const std::string & file,
                     * 			const unsigned char * key) throw (Smartmatic::System::Exception::CryptoException);
                     * @brief	method to encrypt a buffer to an specific file.
                     * @author	marcel.mancini
                     * @date	06/07/2017
                     * @exception	Smartmatic::System::Exception::CryptoException	Thrown when a crypto error condition occurs.
                     * @param	dataIn	    data to encrypt.
                     * @param	file		file out.
                     * @param	key			private key.
                     * @throw CryptoException.
                     */
                    void encryptBufferToFileAndCreateIV ( const SafeBuffer & dataIn, const std::string & file, const unsigned char * key)
                    throw ( Smartmatic::System::Exception::CryptoException );
                    /**
                     * @fn	void encryptFileToBufferAndCreateIV(const std::string & fileIn, SafeBuffer & dataOut, const unsigned char * key)
                     * 			throw ( Smartmatic::System::Exception::CryptoException );
                     * @brief	method to encrypt a file and get a buffer as result.
                     * @author	marcel.mancini
                     * @date	06/07/2017
                     * @exception	Smartmatic::System::Exception::CryptoException	Thrown when a crypto error condition occurs.
                     * @param	fileIn	    file to encrypt.
                     * @param	dataOut		the result buffer.
                     * @param	key			private key.
                     * @throw CryptoException.
                     */
                    void encryptFileToBufferAndCreateIV ( const std::string & fileIn, SafeBuffer & dataOut, const unsigned char * key)
                    throw ( Smartmatic::System::Exception::CryptoException );
                    /**
                     * @fn	void encryptFileToFileAndCreateIV(const std::string & fileIn, const std::string & fileOut, const unsigned char * key)
                     * 			throw ( Smartmatic::System::Exception::CryptoException );
                     * @brief	method to encrypt a file and get the encrypted file result.
                     * @author	marcel.mancini
                     * @date	06/07/2017
                     * @exception	Smartmatic::System::Exception::CryptoException	Thrown when a crypto error condition occurs.
                     * @param	fileIn	file to encrypt.
                     * @param	fileOut		the result file.
                     * @param	key			private key.
                     * @throw CryptoException.
                     */
                    void encryptFileToFileAndCreateIV ( const std::string & fileIn, const std::string & fileOut, const unsigned char * key)
                    throw ( Smartmatic::System::Exception::CryptoException );
                    /**
                     * @fn	void encryptBufferToBufferAndCreateIV ( const SafeBuffer & dataIn, SafeBuffer & dataOut, const unsigned char * key)
                     * 			throw ( Smartmatic::System::Exception::CryptoException );
                     * @brief	method to encrypt a plain buffer and get the encrypted buffer as a result.
                     * @author	marcel.mancini
                     * @date	06/07/2017
                     * @exception	Smartmatic::System::Exception::CryptoException	Thrown when a crypto error condition occurs.
                     * @param	dataIn	buffer to encrypt.
                     * @param	dataOut		the result buffer.
                     * @param	key			private key.
                     * @throw CryptoException.
                     */
                    void encryptBufferToBufferAndCreateIV ( const SafeBuffer & dataIn, SafeBuffer & dataOut, const unsigned char * key, bool useCbcMac = false )
                    throw ( Smartmatic::System::Exception::CryptoException );
                    /**
                     * @fn	void decryptFileToBufferAndGetIV ( const std::string & file, SafeBuffer & dataOut, const unsigned char * key)
                     * 			throw ( Smartmatic::System::Exception::CryptoException );
                     * @brief	method to decrypt an encrypted file and get the plain buffer as a result.
                     * @author	marcel.mancini
                     * @date	06/07/2017
                     * @exception	Smartmatic::System::Exception::CryptoException	Thrown when a crypto error condition occurs.
                     * @param	file	file to decrypt.
                     * @param	dataOut		the result buffer.
                     * @param	key			private key.
                     * @throw CryptoException.
                     */
                    void decryptFileToBufferAndGetIV ( const std::string & file, SafeBuffer & dataOut, const unsigned char * key)
                    throw ( Smartmatic::System::Exception::CryptoException );
                    /**
                     * @fn	void decryptBufferToFileAndGetIV ( const SafeBuffer & dataIn, const std::string & file, const unsigned char * key)
                     * 			throw ( Smartmatic::System::Exception::CryptoException );
                     * @brief	method to decrypt an encrypted buffer and get a plain file as a result.
                     * @author	marcel.mancini
                     * @date	06/07/2017
                     * @exception	Smartmatic::System::Exception::CryptoException	Thrown when a crypto error condition occurs.
                     * @param	dataIn	buffer to decrypt.
                     * @param	file		the result file.
                     * @param	key			private key.
                     * @throw CryptoException.
                     */
                    void decryptBufferToFileAndGetIV ( const SafeBuffer & dataIn, const std::string & file, const unsigned char * key)
                    throw ( Smartmatic::System::Exception::CryptoException );
                    /**
                     * @fn	void decryptFileToFileAndGetIV ( const std::string & fileIn, const std::string & fileOut, const unsigned char * key)
                     * 			throw ( Smartmatic::System::Exception::CryptoException );
                     * @brief	method to decrypt an encrypted file and get a plain file as a result.
                     * @author	marcel.mancini
                     * @date	06/07/2017
                     * @exception	Smartmatic::System::Exception::CryptoException	Thrown when a crypto error condition occurs.
                     * @param	fileIn	file to decrypt.
                     * @param	fileOut		the result file.
                     * @param	key			private key.
                     * @throw CryptoException.
                     */
                    void decryptFileToFileAndGetIV ( const std::string & fileIn, const std::string & fileOut, const unsigned char * key)
                    throw ( Smartmatic::System::Exception::CryptoException );
                    /**
                     * @fn	void decryptBufferToBufferAndGetIV ( const SafeBuffer & dataIn, SafeBuffer & dataOut, const unsigned char * key)
                     * 			throw ( Smartmatic::System::Exception::CryptoException );
                     * @brief	method to decrypt an encrypted buffer and get a plain buffer as a result.
                     * @author	marcel.mancini
                     * @date	06/07/2017
                     * @exception	Smartmatic::System::Exception::CryptoException	Thrown when a crypto error condition occurs.
                     * @param	dataIn	    buffer to decrypt.
                     * @param	dataOut		the result buffer.
                     * @param	key			private key.
                     * @throw CryptoException.
                     */
                    void decryptBufferToBufferAndGetIV ( SafeBuffer  dataIn, SafeBuffer & dataOut, const unsigned char * key, const bool useCbcMac = false)
                    throw ( Smartmatic::System::Exception::CryptoException );

                private:

                    Smartmatic::System::Security::Cryptography::CryptoAES * cryptoAES; ///<object crypto aes
                    Mode mode; ///<encrypt mode
                    bool pad; ///< flag enabled pad
                    KeySize keySize; ///< key size
                    int ivSize; ///< size of the initialization vector
                    static Smartmatic::Log::ISMTTLog * logger; ///<logger

                    /**
                     * @fn	void Crypto::configCryptoAESKey128();
                     * @brief	method setter config crypto aes key 128.
                     * @author	Youssef.tourkmani
                     * @date	29/10/2012
                     */
                    void configCryptoAESKey128();
                    /**
                     * @fn	void Crypto::configCryptoAESKey192();
                     * @brief	method setter config crypto aes key 192.
                     * @author	Youssef.tourkmani
                     * @date	29/10/2012
                     */
                    void configCryptoAESKey192();
                    /**
                     * @fn	void Crypto::configCryptoAESKey256();
                     * @brief	method setter config crypto aes key 256.
                     * @author	Youssef.tourkmani
                     * @date	29/10/2012
                     */
                    void configCryptoAESKey256();
            };
        }
    }
}

#endif /*__crypt__H__*/
