/**
 * @file CMSSigner.hxx
 * @brief Header class CMSSigner
 * @date 02/03/2012
 * @author Marcel Mancini <yadickson.soto@smartmatic.com>
 */

#ifndef CMSSIGNER_HXX_
#define CMSSIGNER_HXX_

#include <System/Exception/SignedException.hxx>
#include <System/SafeBuffer.h>
#include <System/Security/Cryptography/CMSBase.hxx>

namespace Smartmatic
{
    namespace Log
    {
        class ISMTTLog;
    }

    namespace System
    {
        namespace Security
        {
            namespace Cryptography
            {
                /**
                 * @class	CMSSigner
                 * @brief	Class CMSSigner.
                 * @author	Marcel Mancini
                 * @date	02/03/2012
                 */
                class CMSSigner : public CMSBase
                {
                    public:

                        /**
                         * @fn	CMSSigner();
                         * @brief	Class constructor.
                         * @author	Marcel Mancini
                         * @date	02/03/2012
                         */
                        CMSSigner ();

                        /**
                         * @fn	virtual ~CMSSigner();
                         * @brief	Class destructor.
                         * @author	Marcel Mancini
                         * @date	02/03/2012
                         */
                        virtual ~CMSSigner();

                        /**
                         * @fn	void signBuffer(const Smartmatic::System::SafeBuffer & bufferIn,
                         * Smartmatic::System::SafeBuffer & bufferOut) throw (Smartmatic::System::Exception::SignedException);
                         * @brief	method to sign a buffer.
                         * @exception	Smartmatic::System::Exception::SignedException	Thrown when a signed error condition occurs.
                         * @author	Marcel Mancini
                         * @date	12/07/2017
                         * @param	bufferIn	buffer in.
                         * @param	bufferOut	buffer out.
                         */
                        void signBuffer ( const Smartmatic::System::SafeBuffer & bufferIn,
                                          Smartmatic::System::SafeBuffer & bufferOut)
                        throw ( Smartmatic::System::Exception::SignedException );

                        /**
                         * @fn	void verifyBuffer(const Smartmatic::System::SafeBuffer & bufferIn,
                         * 			Smartmatic::System::SafeBuffer & bufferSigned) throw (Smartmatic::System::Exception::SignedException);
                         * @brief	method validate signed buffer.
                         * @author	Marcel Mancini
                         * @date	12/07/2017
                         * @exception	Smartmatic::System::Exception::SignedException	Thrown when a signed error condition occurs.
                         * @param	bufferIn			buffer in.
                         * @param	bufferSigned	buffer signed.
                         */
                        void verifyBuffer ( const Smartmatic::System::SafeBuffer & bufferIn,
                                            Smartmatic::System::SafeBuffer & bufferSigned)
                        throw ( Smartmatic::System::Exception::SignedException );

                    protected:

                        /**
                         * @fn	virtual void * CMSProcess() = 0;
                         * @brief	Method to process CMS object
                         * @author	Marcel Mancini
                         * @date	12/07/2017
                         * @return	CMS pointer
                         */
                        virtual void * CMSProcess();

                        /**
                         * @fn	virtual int CMSProcess ( void * cms, BIO * result ) = 0;
                         * @brief	Method to process CMS object and getter result value
                         * @author	Marcel Mancini
                         * @date	12/07/2017
                         * @param	cms CMS pointer to process
                         * @param 	result result buffer
                         * @return	successfully if return value > 0
                         */
                        virtual int CMSProcess ( void * cms, BIO * result );

                    private:

                        static Smartmatic::Log::ISMTTLog * logger; /**<logger*/
                };
            }
        }
    }
}

#endif /* CMSSIGNER_HXX_ */
