/**
 * @file CertificateExtensionBase.cxx
 * @brief Body class certificate extension.
 * @date 18/12/2013
 * @author Beatriz Avila
 */

#include "CertificateExtensionBase.hxx"
#include <openssl/evp.h>
#include <openssl/x509.h>
#include <openssl/safestack.h>
#include <Log/ISMTTLog.h>
#include <Log/SMTTLogManager.h>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

using namespace Smartmatic::Log;
using namespace Smartmatic::System::Security::Cryptography;

ISMTTLog * CertificateExtensionBase::logger = SMTTLogManager::GetLogger ( "Smartmatic.System.Security.Cryptography.CertificateExtensionBase", GETTEXT_PACKAGE );

CertificateExtensionBase::CertificateExtensionBase ( const SafeBuffer & certificateBuffer, const std::string & certPassword )
{
    this->certificateBuffer = certificateBuffer;
    this->certPassword = certPassword;
    x509 = NULL;
    privateKey = NULL;
    publicKey = NULL;
    memoryBIO = NULL;
    certificateAuthorities = NULL;
}

CertificateExtensionBase::~CertificateExtensionBase ( )
{
}

X509 * CertificateExtensionBase::LoadCertifiedX509 ( )
{
    if ( x509 || !WriteSafeBufferToMemoryBIO () )
    {
        return x509;
    }

    releaseCertifiedX509();
    x509 = LoadCertifiedX509FromBIO ( );

    if ( !x509 )
    {
        logger->Debug ( "Unable to load certificate" );
    }

    releaseMemoryBIO();

    return x509;
}

EVP_PKEY * CertificateExtensionBase::LoadCertifiedPrivateKey ( )
{
    if ( privateKey || !WriteSafeBufferToMemoryBIO () )
    {
        return privateKey;
    }

    releaseCertifiedPrivateKey();
    privateKey = LoadCertifiedPrivateKeyFromBIO ( );

    if ( !privateKey )
    {
        logger->Debug ( "Error load private key" );
    }

    releaseMemoryBIO();

    return privateKey;
}

EVP_PKEY * CertificateExtensionBase::LoadCertifiedPublicKey ( )
{
    if ( publicKey != NULL )
    {
        return publicKey;
    }

    X509 * cert = LoadCertifiedX509();

    if ( cert == NULL )
    {
        return publicKey;
    }

    publicKey = X509_get_pubkey ( cert );

    return publicKey;
}

STACK_OF ( X509 ) * CertificateExtensionBase::getCertificateAuthorities ( )
{
    if ( certificateAuthorities || !WriteSafeBufferToMemoryBIO () )
    {
        return certificateAuthorities;
    }

    releaseCertificateAuthorities();
    certificateAuthorities = getCertificateAuthoritiesFromBIO ( );

    if ( !certificateAuthorities )
    {
        logger->Debug ( "Error load certificate authorities" );
    }

    releaseMemoryBIO();

    return certificateAuthorities;
}

bool CertificateExtensionBase::WriteSafeBufferToMemoryBIO ( )
{
    if ( certificateBuffer.GetSize() == 0 )
    {
        logger->Debug ( "Error empty buffer for BIO" );
        return false;
    }

    releaseMemoryBIO();

    memoryBIO = BIO_new ( BIO_s_mem() );

    BIO_write ( memoryBIO, certificateBuffer.GetData(), certificateBuffer.GetSize() );

    return true;
}

void CertificateExtensionBase::releaseMemoryBIO()
{
    if ( memoryBIO )
    {
        BIO_free ( memoryBIO );
        memoryBIO = NULL;
    }
}

void CertificateExtensionBase::releaseCertifiedX509()
{
    if ( x509 )
    {
        X509_free ( x509 );
        x509 = NULL;
    }
}

void CertificateExtensionBase::releaseCertifiedPrivateKey()
{
    if ( privateKey )
    {
        EVP_PKEY_free ( privateKey );
        privateKey = NULL;
    }
}

void CertificateExtensionBase::releaseCertifiedPublicKey()
{
    if ( publicKey )
    {
        EVP_PKEY_free ( publicKey );
        publicKey = NULL;
    }
}

void CertificateExtensionBase::releaseCertificateAuthorities()
{
    if ( certificateAuthorities )
    {
        sk_X509_pop_free ( certificateAuthorities, X509_free );
        certificateAuthorities = NULL;
    }
}

void CertificateExtensionBase::releaseAll()
{
    releaseCertifiedX509();
    releaseCertifiedPrivateKey();
    releaseCertifiedPublicKey();
    releaseCertificateAuthorities();
    releaseMemoryBIO();
}
