/**
 * @file CertificateValidator.h
 * @brief Header class certificate validator
 * @date 07/04/2011
 * @author Freddy Gomez
 */

#ifndef CERTIFICATEVALIDATOR_H_
#define CERTIFICATEVALIDATOR_H_

#include <string>
#include <openssl/x509.h>
#include <xsec/enc/OpenSSL/OpenSSLCryptoX509.hpp>
#include <Log/ISMTTLog.h>
#include "System/Security/Cryptography/CertUtil.h"
#include "System/Security/Cryptography/CertValidationResult.h"

#define DEFINE_STACK STACK_OF(X509) ///< macro stacks certificate

namespace Smartmatic
{
    namespace System
    {
        namespace Security
        {
            namespace Cryptography
            {
                /**
                 * @enum	CertificatePurpose
                 * @brief	Values that represent CertificatePurpose.
                 */
                enum CertificatePurpose
                {
                    DefaultPurpose = 0,
                    SSLClientCertificate = 1,
                    SSLServerCertificate = 2,
                    NSSSLServerCertificate = 3,
                    SMIMESignCertificate = 4,
                    SMIMEEncryptCertificate = 5,
                    CRLSIGNCertificate = 6,
                    AnyPurposeCertificate = 7
                };

                /**
                 * @enum	CertificateValidationFlags
                 * @brief	Values that represent CertificateValidationFlags.
                 */
                enum CertificateValidationFlags
                {
                    VERIFY_NO_FLAGS = 0,
                    VERIFY_DO_NOT_VALIDATE = 1, //this does nothing
                    VERIFY_IGNORE_CRITICAL = 2,
                    VERIFY_CB_ISSUER_CHECK = 4,
                    VERIFY_CRL_CHECK = 8,
                    VERIFY_CRL_CHECK_ALL = 16,
                    VERIFY_POLICY_CHECK = 32,
                    VERIFY_EXPLICIT_POLICY = 64,
                    VERIFY_INHIBIT_ANY = 128,
                    VERIFY_INHIBIT_MAP = 256,
                    VERIFY_X509_STRICT = 512,
                    VERIFY_EXTENDED_CRL_SUPPORT = 1024,
                    VERIFY_USE_DELTAS = 2048,
                    VERIFY_NOTIFY_POLICY = 4096,
                    VERIFY_CHECK_SS_SIGNATURE = 8192,
                    VERIFY_SET_CHECK_TIME = 16384,
                    VERIFY_ALLOW_PROXY_CERTS = 32768
                };

                /**
                 * @class	CertificateValidator
                 * @brief	Certificate validator.
                 * @author	freddyg
                 * @date	28/04/2011
                 */
                class CertificateValidator
                {
                    public:

                        /**
                         * @fn	CertificateValidator::CertificateValidator();
                         * @brief	Default constructor.
                         * @author	freddyg
                         * @date	28/04/2011
                         */
                        CertificateValidator();

                        /**
                         * @fn	virtual CertificateValidator::~CertificateValidator();
                         * @brief	Class destructor.
                         * @author	freddyg
                         * @date	28/04/2011
                         */
                        virtual ~CertificateValidator();

                        //Policy: indica cual poliza del archivo de configuracion usar. no parecen inyectarse al momento de hacer la validacion

                        /**
                         * @fn	void CertificateValidator::SetOptions(CertificateValidationFlags flags);
                         * @brief	Sets the options.
                         * @author	freddyg
                         * @date	28/04/2011
                         * @param	flags	The flags.
                         */
                        void SetOptions ( CertificateValidationFlags flags );

                        /**
                         * @fn	void CertificateValidator::ClearOptions(CertificateValidationFlags flags);
                         * @brief	Clears the options described by flags.
                         * @author	freddyg
                         * @date	28/04/2011
                         * @param	flags	The flags.
                         */
                        void ClearOptions ( CertificateValidationFlags flags );

                        /**
                         * @fn	void CertificateValidator::SetVerificationTime(time_t time);
                         * @brief	Sets a verification time.
                         * @author	freddyg
                         * @date	28/04/2011
                         * @param	time	The time.
                         */
                        void SetVerificationTime ( time_t time );

                        /**
                         * @fn	void CertificateValidator::ClearVerificationTime();
                         * @brief	Clears the verification time.
                         * @author	freddyg
                         * @date	28/04/2011
                         */
                        void ClearVerificationTime();

                        //This would be the Root CA

                        /**
                         * @fn	void CertificateValidator::SetCACertificate(const std::string & certPath,
                         * 			CertUtil::CertFileFormat format);
                         * @brief	Sets a ca certificate.
                         * @author	freddyg
                         * @date	28/04/2011
                         * @param	certPath	Full pathname of the cert file.
                         * @param	format		Describes the format to use.
                         */
                        void SetCACertificate ( const std::string & certPath, CertUtil::CertFileFormat format );

                        //These would be the certs in the chain, untrusted cert for openssl

                        /**
                         * @fn  void CertificateValidator::SetItermediateCACertificate(SafeBuffer & certBuffer,
                         *          CertUtil::CertFileFormat format);
                         * @brief   Sets an itermediate ca certificates.
                         * @author  eduardo.fernandez
                         * @date    06/11/2018
                         * @param   certBuffer  Certificate buffer.
                         * @param   format      Describes the format to use.
                         * @param   password    Password.
                         */
                        void SetIntermediateCACertificate ( SafeBuffer & certBuffer, CertUtil::CertFileFormat format, const std::string & password = "" );

                        /**
                         * @fn	CertValidationResult CertificateValidator::VerifyX509Certificate(const std::string & certPath,
                         * 			CertUtil::CertFileFormat format, CertificatePurpose purpose, const std::string & password );
                         * @brief	Verify x coordinate 509 certificate.
                         * @author	freddyg
                         * @date	28/04/2011
                         * @param	certPath	Full pathname of the cert file.
                         * @param	format		Describes the format to use.
                         * @param	purpose 	The purpose.
                         * @param	password	Password.
                         * @return	.
                         */
                        CertValidationResult VerifyX509Certificate ( const std::string & certPath, CertUtil::CertFileFormat format, CertificatePurpose purpose, const std::string & password = "" );

                        /**
                         * @fn	CertValidationResult CertificateValidator::VerifyX509Certificate(OpenSSLCryptoX509 * cert,
                         * 			CertificatePurpose purpose );
                         * @brief	Verify x coordinate 509 certificate.
                         * @author	freddyg
                         * @date	28/04/2011
                         * @param	cert	 	If non-null, the cert.
                         * @param	purpose	The purpose.
                         * @return	.
                         */
                        CertValidationResult VerifyX509Certificate ( OpenSSLCryptoX509 * cert, CertificatePurpose purpose, const bool loadIntermediateCertificate = false );

                        /**
                         * @fn  CertValidationResult CertificateValidator::VerifyCertificate(X509 * cert);
                         * @brief   Verify x coordinate 509 certificate.
                         * @author  eduardo.fernandez
                         * @date    06/02/2019
                         * @param   cert        If non-null, the cert.
                         * @return  .
                         */
                        CertValidationResult VerifyCertificate ( X509 * cert );

                        static Smartmatic::Log::ISMTTLog * logger;  ///< The logger

                    private:

                        /**
                         * @fn	CertValidationResult CertificateValidator::VerifyX509Certificate(X509 * cert,
                         * 			CertificatePurpose purpose );
                         * @brief	Verify x coordinate 509 certificate.
                         * @author	freddyg
                         * @date	28/04/2011
                         * @param	cert	 	If non-null, the cert.
                         * @param	purpose	The purpose.
                         * @return	.
                         */
                        CertValidationResult VerifyX509Certificate ( X509 * cert, CertificatePurpose purpose );

                        X509_STORE * m_cert_ctx;
                        DEFINE_STACK * m_untrusted;
                        std::string m_certAuthorityPath;  ///< Full pathname of the cert authority file
                        CertUtil::CertFileFormat m_certAuthorityFormat;  ///< The cert authority format
                        CertificateValidationFlags m_validationFlags;  ///< The validation flags
                        time_t m_checkTime;  ///< Time of the check

                };

            }
        }
    }
}
#endif /* CERTIFICATEVALIDATOR_H_ */
