/**
 * @file CryptoPKCS7.cxx
 * @brief Body class crypto PCKS7
 * @date 19/03/2012
 * @author Yadickson Soto
 */

#include "CryptoPKCS7.hxx"
#include <openssl/evp.h>
#include <openssl/pkcs7.h>
#include <Log/ISMTTLog.h>
#include <Log/SMTTLogManager.h>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#include <libintl.h>
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///< macro for gettext
#else
#define _(String) (String) ///< macro without gettext
#endif

#define N_(String) (String) ///< macro not translate

using namespace Smartmatic::System::Security::Cryptography;
using namespace Smartmatic::System::Exception;
using namespace Smartmatic::Log;
using namespace Smartmatic::System;

ISMTTLog * CryptoPKCS7::logger = SMTTLogManager::GetLogger ( "Smartmatic.System.Security.Cryptography.CryptoPKCS7", GETTEXT_PACKAGE );

CryptoPKCS7::CryptoPKCS7 ( const EVP_CIPHER * cipherMode )
    : PKCS7Base ( PKCS7_BINARY )
{
    if ( cipherMode == NULL )
    {
        this->cipherMode = EVP_aes_256_cbc();
    }
    else
    {
        this->cipherMode = cipherMode;
    }
}

CryptoPKCS7::~CryptoPKCS7()
{

}

void CryptoPKCS7::encryptBuffer ( const SafeBuffer & bufferIn, SafeBuffer & bufferOut, PKCS7BaseType type ) throw ( CryptoException )
{
    if ( ! processBuffer ( bufferIn, bufferOut, type ) )
    {
        throw CryptoException ( _ ( "Smartmatic.System.Security.Cryptography.CryptoPKCS7.ErrorCryptoData" ),
                                N_ ( "Smartmatic.System.Security.Cryptography.CryptoPKCS7.ErrorCryptoData" ), logger );
    }
}

void CryptoPKCS7::decryptBuffer ( const SafeBuffer & bufferIn, SafeBuffer & bufferOut, PKCS7BaseType type ) throw ( CryptoException )
{
    if ( ! reprocessBuffer ( bufferIn, bufferOut, type ) )
    {
        throw CryptoException ( _ ( "Smartmatic.System.Security.Cryptography.CryptoPKCS7.ErrorDecryptoData" ),
                                N_ ( "Smartmatic.System.Security.Cryptography.CryptoPKCS7.ErrorDecryptoData" ), logger );
    }
}

void * CryptoPKCS7::PKCS7Process()
{
    X509 * cert = certificate->LoadCertifiedX509();

    if ( cert == NULL )
    {
        return NULL;
    }

    STACK_OF ( X509 ) * ca = NULL;
    ca = sk_X509_new_null();
    sk_X509_push ( ca, cert );

    void * p7 = ( void * ) PKCS7_encrypt ( ca, memoryBIO, cipherMode, flags );

    sk_X509_free ( ca );

    return p7;
}

int CryptoPKCS7::PKCS7Process ( void * p7, BIO * result )
{
    EVP_PKEY * pkey = certificate->LoadCertifiedPrivateKey();

    if ( pkey == NULL )
    {
        return -1;
    }

    return PKCS7_decrypt ( ( PKCS7 * ) p7, pkey, certificate->LoadCertifiedX509(), result, flags );
}

