/**
 * @file ECDSASigner.hxx
 * @brief Header ECDSA signer class
 * @date 10/11/2011
 * @author Freddy Gomez
 * @author manuel.arguelles
 */

#ifndef ECDSASIGNER_H_
#define ECDSASIGNER_H_

#include <System/Exception/SignedException.hxx>
#include <System/SafeBuffer.h>
#include <System/Security/Cryptography/SignatureAlgorithm.h>

namespace Smartmatic
{
    namespace Log
    {
        class ISMTTLog;
    }
    namespace System
    {
        namespace Security
        {
            namespace Cryptography
            {
                /**
                 * @class ECDSASigner
                 * @brief Class ECDSA Signer.
                 * @author Yadickson Soto
                 * @date 29/10/2012
                 */
                class ECDSASigner : public SignatureAlgorithm
                {
                    public:

                        /**
                         * @brief  Default constructor.
                         * @author Youssef.tourkmani
                         * @date   24/10/2012
                         */
                        ECDSASigner();

                        /**
                         * @brief  Destructor.
                         * @author Youssef.tourkmani
                         * @date   24/10/2012
                         */
                        virtual ~ECDSASigner();

                        /**
                         * @brief         Signature calculation.
                         * @author        fgomez
                         * @date          Nov 10, 2011
                         * @throw         Smartmatic::System::Exception::SignedException Thrown when a ECDSA error condition occurs.
                         * @param[in,out] inSafeBuffer Reference to data.
                         * @param[in,out] outSafeBuffer Output buffer.
                         * @param         hashKind The hash algorithm to use.
                         */
                        virtual void SignData(const Smartmatic::System::SafeBuffer& inSafeBuffer,
                                              Smartmatic::System::SafeBuffer& outSafeBuffer,
                                              MsgDigestUtil::DigestAlgType hashKind)
                            throw(Smartmatic::System::Exception::SignedException);

                        /**
                         * @brief         Signature verification.
                         * @author        fgomez
                         * @date          Nov 10, 2011
                         * @throw         Smartmatic::System::Exception::SignedException Thrown when a ECDSA error condition occurs.
                         * @param[in,out] inSafeBuffer Reference to data.
                         * @param[in,out] signature Signature to verify.
                         * @param         hashKind The hash algorithm to use.
                         * @return        True if validation succeed. Otherwise, false.
                         */
                        virtual bool VerifySignature(const Smartmatic::System::SafeBuffer& inSafeBuffer,
                                                     const Smartmatic::System::SafeBuffer& signature,
                                                     MsgDigestUtil::DigestAlgType hashKind)
                            throw(Smartmatic::System::Exception::SignedException);

                    private:

                        static Smartmatic::Log::ISMTTLog* m_logger;
                };

            } /* namespace Cryptography */
        } /* namespace Security */
    } /* namespace System */
} /* namespace Smartmatic */
#endif /* ECDSASIGNER_H_ */
