/**
 * @file MsgDigestUtil.cpp
 * @brief Body class msg digest util
 * @date 01/04/2011
 * @author Marcel Cordovi
 */

#include "MsgDigestUtil.h"
#include <Log/SMTTLogManager.h>
#include <string.h>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#include <libintl.h>
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

#define N_(String) (String) ///<macro for gettext not translate

#define SHA224  "sha224" ///< digest sha224
#define SHA256  "sha256" ///< digest sha256
#define SHA384  "sha384" ///< digest sha384
#define SHA512  "sha512" ///< digest sha512

#define BUFFER_SIZE 256 ///< buffer size

#define _ERROR_SETTING_CHANNEL_ N_("Smartmatic.System.Security.Cryptography.MsgDigestUtil.ErrorSettingChannel") ///< error setting channel
#define _ERROR_READING_SECRET_KEY_ N_("Smartmatic.System.Security.Cryptography.MsgDigestUtil.ErrorReadingSecretKey") ///< erro reading secret key
#define _ERROR_READING_FILE_HASHING_ N_("Smartmatic.System.Security.Cryptography.MsgDigestUtil.ErrorReadingFileHashing") ///< error reading file hashing
#define _ERROR_UNKHOWN_MESSAGE_DIGEST_ N_("Smartmatic.System.Security.Cryptography.MsgDigestUtil.ErrorUnknownMessageDigest") ///< error unkhown digest

using namespace Smartmatic::System;
using namespace Smartmatic::System::Exception;
using namespace Smartmatic::System::Security::Cryptography;
using namespace Smartmatic::Log;

ISMTTLog * MsgDigestUtil::logger = SMTTLogManager::GetLogger ( "Smartmatic.System.Security.Cryptography.MsgDigestUtil.", GETTEXT_PACKAGE );

MsgDigestUtil::MsgDigestUtil(DigestAlgType alg)
{
    const char    *algName = mMapAlgName ( alg );
    const EVP_MD  *md = EVP_get_digestbyname ( algName );

    initializeHashing(&mdctx, md, algName);
}

MsgDigestUtil::MsgDigestUtil ( const Smartmatic::System::SafeBuffer & buffer )
    : buffer ( buffer )
{

}

MsgDigestUtil::~MsgDigestUtil()
{

}

void MsgDigestUtil::getDigest ( Smartmatic::System::SafeBuffer & result, DigestAlgType alg ) throw ( MessageDigestException )
{
    const char    *algName = mMapAlgName ( alg );
    const EVP_MD  *md = EVP_get_digestbyname ( algName );

    initializeHashing(&mdctx, md, algName);
    performHashing(&mdctx, buffer.GetData(), buffer.GetSize());
    obtainResult(result);
}

void MsgDigestUtil::updateDigest(const SafeBuffer &buffer) throw ( MessageDigestException )
{
    performHashing(&mdctx, buffer.GetData(), buffer.GetSize());
}

const char * MsgDigestUtil::mMapAlgName ( DigestAlgType alg )
{
    switch ( alg )
    {
        default:
        case DGSTUTIL_SHA224:
            return SHA224;

        case DGSTUTIL_SHA256:
            return SHA256;

        case DGSTUTIL_SHA384:
            return SHA384;

        case DGSTUTIL_SHA512:
            return SHA512;
    }
}

void MsgDigestUtil::initializeHashing(EVP_MD_CTX *mdctx, const EVP_MD * md, const char* algName)
{
    if ( !md )
    {
        throw MessageDigestException ( _ ( _ERROR_UNKHOWN_MESSAGE_DIGEST_ ), N_ ( _ERROR_UNKHOWN_MESSAGE_DIGEST_ ), logger );
    }

    EVP_MD_CTX_init ( mdctx );
    EVP_DigestInit_ex ( mdctx, md, NULL );
}

void MsgDigestUtil::performHashing(EVP_MD_CTX *mdctx, const unsigned char* buffer, const size_t bufferSize)
{
    EVP_DigestUpdate ( mdctx, buffer, bufferSize );
}

void MsgDigestUtil::cleanupHashing(EVP_MD_CTX *mdctx, unsigned char *md_value, unsigned int *md_len)
{
    EVP_DigestFinal_ex ( mdctx, md_value, md_len );
    EVP_MD_CTX_cleanup ( mdctx );
}

void MsgDigestUtil::obtainResult(SafeBuffer &result)
{
    unsigned int  md_len;
    unsigned char md_value[EVP_MAX_MD_SIZE];

    cleanupHashing(&mdctx, md_value, &md_len);

    unsigned char *res_buf = ( unsigned char * ) malloc ( md_len * sizeof ( char ) );
    memcpy ( res_buf, md_value, md_len );
    result.Clear();
    result.AddData ( res_buf, md_len );

    free(res_buf);
    res_buf = NULL;
}

MsgDigestUtil::DigestAlgType MsgDigestUtil::getDefaultFileDigest()
{
    return DGSTUTIL_SHA256;
}

MsgDigestUtil::DigestAlgType MsgDigestUtil::getDefaultFileNameDigest()
{
    return DGSTUTIL_SHA256;
}

MsgDigestUtil::DigestAlgType MsgDigestUtil::getDefaultCertificateDigest()
{
    return DGSTUTIL_SHA256;
}

MsgDigestUtil::DigestAlgType MsgDigestUtil::getDefaultOpeningCodeDigest()
{
    return DGSTUTIL_SHA256;
}

MsgDigestUtil::DigestAlgType MsgDigestUtil::getDefaultSecretKeyDigest()
{
    return DGSTUTIL_SHA256;
}

MsgDigestUtil::DigestAlgType MsgDigestUtil::getDefaultComposeKeyDigest()
{
    return DGSTUTIL_SHA256;
}

MsgDigestUtil::DigestAlgType MsgDigestUtil::getDefaultGenerateKeyDigest()
{
    return DGSTUTIL_SHA256;
}

MsgDigestUtil::DigestAlgType MsgDigestUtil::getDefaultVoteDigest()
{
    return DGSTUTIL_SHA256;
}
