/**
 * @file PKCS7Base.cxx
 * @brief Body PKCS7 extension base class to crypto and signed
 * @date 19/03/2012
 * @author Yadickson Soto
 */

#include "PKCS7Base.hxx"
#include <System/Security/Cryptography/CertificateExtensionBase.hxx>
#include <openssl/pkcs7.h>
#include <openssl/ssl.h>

using namespace Smartmatic::System::Security::Cryptography;
using namespace Smartmatic::System::Security;
using namespace Smartmatic::System;

PKCS7Base::PKCS7Base ( int flags )
    : SecurityBase()
{
    this->memoryBIO = NULL;
    this->flags = flags;
}

PKCS7Base::~PKCS7Base()
{

}

bool PKCS7Base::processBuffer ( const SafeBuffer & bufferIn, SafeBuffer & bufferOut, PKCS7BaseType type )
{
    bufferOut.Clear();

    if ( certificate == NULL )
    {
        return false;
    }

    memoryBIO = getMemoryBIO ( bufferIn );

    PKCS7 * p7 = static_cast < PKCS7 * > ( PKCS7Process() );

    if ( p7 == NULL )
    {
        return false;
    }

    BIO * message = BIO_new ( BIO_s_mem() );

    int resp = 0;

    switch ( type )
    {
        case PEM:
            resp = PEM_write_bio_PKCS7 ( message, p7 );
            break;

        case SMIME:
            resp = SMIME_write_PKCS7 ( message, p7, memoryBIO, flags );
            break;

        default:
        case DER:
            resp = i2d_PKCS7_bio ( message, p7 );
            break;
    }

    if ( resp > 0 )
    {
        safeToBuffer ( message, bufferOut );
    }

    PKCS7_free ( p7 );
    BIO_free ( memoryBIO );
    BIO_free ( message );

    memoryBIO = NULL;

    return resp > 0;
}

bool PKCS7Base::reprocessBuffer ( const SafeBuffer & bufferIn, SafeBuffer & bufferOut, PKCS7BaseType type )
{
    bufferOut.Clear();

    if ( certificate == NULL )
    {
        return false;
    }

    memoryBIO = getMemoryBIO ( bufferIn );

    PKCS7 * p7;

    switch ( type )
    {
        case PEM:
            p7 = PEM_read_bio_PKCS7 ( memoryBIO, NULL, NULL, NULL );
            break;

        case SMIME:
            p7 = SMIME_read_PKCS7 ( memoryBIO, &memoryBIO );
            break;

        default:
        case DER:
            p7 = d2i_PKCS7_bio ( memoryBIO, NULL );
            break;
    }

    if ( p7 == NULL )
    {
        BIO_free ( memoryBIO );
        return false;
    }

    BIO * message = BIO_new ( BIO_s_mem() );

    int resp = PKCS7Process ( p7, message );

    if ( resp > 0 )
    {
        safeToBuffer ( message, bufferOut );
    }

    PKCS7_free ( p7 );
    BIO_free ( memoryBIO );
    BIO_free ( message );

    return resp > 0;
}

BIO * PKCS7Base::getMemoryBIO ( const Smartmatic::System::SafeBuffer & bufferIn )
{
    return BIO_new_mem_buf ( ( void * ) bufferIn.GetData(), bufferIn.GetSize() );
}

void PKCS7Base::safeToBuffer ( BIO * message, Smartmatic::System::SafeBuffer & bufferOut )
{
    char * bioDataPtr = NULL;
    long   bioDataLen = 0;

    bioDataLen = BIO_get_mem_data ( message, &bioDataPtr );
    bufferOut.AddData ( ( unsigned char * ) bioDataPtr, bioDataLen );
}
