/**
 * @file PKCS7Signer.hxx
 * @brief Header class PKCS7Signer
 * @date 02/03/2012
 * @author Yadickson Soto <yadickson.soto@smartmatic.com>
 */

#ifndef PKCS7SIGNER_HXX_
#define PKCS7SIGNER_HXX_

#include <System/Exception/SignedException.hxx>
#include <System/SafeBuffer.h>
#include <System/Security/Cryptography/PKCS7Base.hxx>

namespace Smartmatic
{
    namespace Log
    {
        class ISMTTLog;
    }

    namespace System
    {
        namespace Security
        {
            namespace Cryptography
            {
                /**
                 * @class	PKCS7Signer
                 * @brief	Class PKCS7Signer.
                 * @author	Yadickson Soto
                 * @date	02/03/2012
                 */
                class PKCS7Signer : public PKCS7Base
                {
                    public:

                        /**
                         * @fn	PKCS7Signer();
                         * @brief	Class constructor.
                         * @author	Yadickson Soto
                         * @date	02/03/2012
                         */
                        PKCS7Signer ();

                        /**
                         * @fn	virtual ~PKCS7Signer();
                         * @brief	Class destructor.
                         * @author	Yadickson Soto
                         * @date	02/03/2012
                         */
                        virtual ~PKCS7Signer();

                        /**
                         * @fn	void signBuffer(const Smartmatic::System::SafeBuffer & bufferIn,
                         * Smartmatic::System::SafeBuffer & bufferOut,
                         *         	PKCS7BaseType type) throw (Smartmatic::System::Exception::SignedException);
                         * @brief	method to sign a buffer.
                         * @exception	Smartmatic::System::Exception::SignedException	Thrown when a signed error condition occurs.
                         * @author	Yadickson Soto
                         * @date	23/01/2014
                         * @param	bufferIn	buffer in.
                         * @param	bufferOut	buffer out.
                         * @param	type	signed type.
                         */
                        void signBuffer ( const Smartmatic::System::SafeBuffer & bufferIn,
                                          Smartmatic::System::SafeBuffer & bufferOut,
                                          PKCS7BaseType type )
                        throw ( Smartmatic::System::Exception::SignedException );

                        /**
                         * @fn	void verifyBuffer(const Smartmatic::System::SafeBuffer & bufferIn,
                         * 			Smartmatic::System::SafeBuffer & bufferSigned,
                         * 			PKCS7BaseType type) throw (Smartmatic::System::Exception::SignedException);
                         * @brief	method validate signed buffer.
                         * @author	Yadickson Soto
                         * @date	23/01/2014
                         * @exception	Smartmatic::System::Exception::SignedException	Thrown when a signed error condition occurs.
                         * @param	bufferIn			buffer in.
                         * @param	bufferSigned	buffer signed.
                         * @param	type					signed type.
                         */
                        void verifyBuffer ( const Smartmatic::System::SafeBuffer & bufferIn,
                                            Smartmatic::System::SafeBuffer & bufferSigned,
                                            PKCS7BaseType type )
                        throw ( Smartmatic::System::Exception::SignedException );

                    protected:

                        /**
                         * @fn	virtual void * PKCS7Process() = 0;
                         * @brief	Method to process pkcs7 object
                         * @author	Yadickson Soto
                         * @date	23/01/2014
                         * @return	PKCS7 pointer
                         */
                        virtual void * PKCS7Process();

                        /**
                         * @fn	virtual int PKCS7Process ( void * p7, BIO * result ) = 0;
                         * @brief	Method to process pkcs7 object and getter result value
                         * @author	Yadickson Soto
                         * @date	23/01/2014
                         * @param	p7 PKCS7 pointer to process
                         * @param 	result result buffer
                         * @return	successfully if return value > 0
                         */
                        virtual int PKCS7Process ( void * p7, BIO * result );

                    private:

                        static Smartmatic::Log::ISMTTLog * logger; /**<logger*/
                };
            }
        }
    }
}

#endif /* PKCS7SIGNER_HXX_ */
