/**
 * @file SignUtils.hxx
 * @brief Header class utility certificate signed manager
 * @date 20/01/2014
 * @author Yadickson Soto
 */

#ifndef SIGN_UTILS_H_
#define SIGN_UTILS_H_

#include <string>
#include <System/SafeBuffer.h>
#include <System/Security/Cryptography/MsgDigestUtil.h>
#include <System/Security/Cryptography/CertificateExtensionBase.hxx>

namespace Smartmatic
{
    namespace System
    {
        namespace Security
        {
            namespace Cryptography
            {
                class SignatureAlgorithm;

                /**
                 * @class	SignUtils
                 * @brief	Class utility certificate signed manager.
                 * @author	Yadickson Soto
                 * @date	21/01/2014
                 */
                class SignUtils
                {
                    public:

                        /**
                         * @fn	static bool signBuffer ( CertificateExtensionBase * certificate,
                         *		const Smartmatic::System::SafeBuffer & inputBuffer,
                         *		Smartmatic::System::SafeBuffer & outputSignature,
                         * 		MsgDigestUtil::DigestAlgType hash = MsgDigestUtil::DGSTUTIL_SHA256 );
                         * @brief	Method to sign a buffer with a certificate
                         * @author	Yadickson Soto
                         * @date	21/01/2014
                         * @param	certificate	certificate to signed buffer
                         * @param	inputBuffer	data input buffer.
                         * @param	outputSignature	data output signature buffer.
                         * @param	hash		signature hash to signature, default sha256
                         * @return	true if successfully
                         */
                        static bool signBuffer ( CertificateExtensionBase * certificate,
                                                 const Smartmatic::System::SafeBuffer & inputBuffer,
                                                 Smartmatic::System::SafeBuffer & outputSignature,
                                                 MsgDigestUtil::DigestAlgType hash );

                        /**
                         * @fn	static bool verifyBuffer ( CertificateExtensionBase * certificate,
                         *		const Smartmatic::System::SafeBuffer & inputBuffer,
                         *		const Smartmatic::System::SafeBuffer & signature,
                         *		MsgDigestUtil::DigestAlgType hash = MsgDigestUtil::DGSTUTIL_SHA256 );
                         * @brief	Method to verify the signature associated to a buffer with a certificate
                         * @author	Yadickson Soto
                         * @date	21/01/2014
                         * @param	certificate	certificate to signed buffer
                         * @param	inputBuffer	data input buffer.
                         * @param	signature	signature to verify
                         * @param	hash		signature hash to verify, default sha256
                         * @return	true if successfully
                         */
                        static bool verifyBuffer ( CertificateExtensionBase * certificate,
                                                   const Smartmatic::System::SafeBuffer & inputBuffer,
                                                   const Smartmatic::System::SafeBuffer & signature,
                                                   MsgDigestUtil::DigestAlgType hash );
                    private:

                        /**
                         * @fn	static SignatureAlgorithm* createSignatureAlgorithmByKeyType(EVP_PKEY* pkey);
                         * @brief	Factory method which Creates a SignatureAlgorithm manager based on the key type
                         * @author	Yadickson Soto
                         * @date	20/01/2014
                         * @param	pkey	Certificate key.
                         * @return	The new SignatureAlgorithm class manager by key type.
                         */
                        static SignatureAlgorithm * createSignatureAlgorithmByKeyType ( EVP_PKEY * pkey );

                };
            } /* namespace Cryptography */
        } /* namespace Security */
    } /* namespace System */
} /* namespace Smartmatic */
#endif /* SIGN_UTILS_H_ */
