/**
 * @file URILocalResolver.hxx
 * @brief class for resolving only local URIs.
 * @author 05/02/2014
 * @author Yadickson Soto
 */

#ifndef URI_LOCAL_RESOLVER_HXX_
#define URI_LOCAL_RESOLVER_HXX_

#include <string>
#include <xsec/framework/XSECURIResolver.hpp>

namespace Smartmatic
{
    namespace Log
    {
        class ISMTTLog;
    }

    namespace System
    {
        namespace Security
        {
            /**
             * @class URILocalResolver
             * @brief class for resolving only local URIs.
             * @date 05/02/2014
             * @author Yadickson Soto
             */
            class URILocalResolver : public XSECURIResolver
            {
                protected:

                    /**
                     * @fn URILocalResolver( const std::string & baseURI );
                     * @brief Class constructor.
                     * @author Yadickson Soto
                     * @date 05/02/2014
                     */
                    URILocalResolver();

                public:

                    /**
                     * @fn URILocalResolver( const std::string & baseURI );
                     * @brief Class constructor.
                     * @author Yadickson Soto
                     * @date 05/02/2014
                     * @param baseURI new base URI for relative URIs to resolved.
                     */
                    explicit URILocalResolver ( const std::string & baseURI );

                    /**
                     * @fn virtual ~URILocalResolver();
                     * @brief Class destructorer.
                     * @author Yadickson Soto
                     * @date 05/02/2014
                     */
                    virtual ~URILocalResolver();

                    /**
                     * @fn virtual XERCES_CPP_NAMESPACE_QUALIFIER BinInputStream * resolveURI ( const XMLCh * uri );
                     * @brief Create a BYTE_STREAM from a URI.
                     *
                     * The resolver is required to take the input URI and
                     * dereference it to an actual stream of octets.
                     *
                     * The octets are provided back to the library using
                     * the Xerces BinInputStream class.
                     *
                     * @note The returned stream is "owned" by the caller, which
                     * will delete it when processing is complete.
                     *
                     * @author Yadickson Soto
                     * @date 05/02/2014
                     *
                     * @param uri The string containing the URI to be de-referenced.
                     * @return The octet stream corresponding to the URI.
                     */
                    virtual XERCES_CPP_NAMESPACE_QUALIFIER BinInputStream * resolveURI ( const XMLCh * uri );

                    /**
                     * @fn virtual XSECURIResolver * clone ();
                     * @brief Clone the resolver to be installed in a new object.
                     *
                     * When URIResolvers are passed into signatures and other
                     * objects, they are cloned and control of the original object
                     * is left with the caller.
                     *
                     * @author Yadickson Soto
                     * @date 05/02/2014
                     *
                     * @return new clone object
                     */
                    virtual XSECURIResolver * clone ();

                private:

                    static Smartmatic::Log::ISMTTLog * logger; ///<logger
                    XMLCh * mp_baseURI; ///< base URI resolved
                    char * memory; ///< memory file to open

                    /**
                     * @fn void clearBaseURI();
                     * @brief Clear the base URI memory handle.
                     * @author Yadickson Soto
                     * @date 05/02/2014
                     */
                    void clearBaseURI();

                    /**
                     * @fn void clearMemory();
                     * @brief Clear the memory file handle.
                     * @author Yadickson Soto
                     * @date 05/02/2014
                     */
                    void clearMemory();
            };

        } /* namespace Security */
    } /* namespace System */
} /* namespace Smartmatic */

#endif /* URI_LOCAL_RESOLVER_HXX_ */
