/**
 * @file XmlSignatureAlgorithm.hxx
 * @brief Header class for XML signature algorithms.
 * @author	Eduardo Fernández
 * @date	09/10/2018
 */

#ifndef XML_SIGNATURE_ALGORITHM_HXX
#define XML_SIGNATURE_ALGORITHM_HXX

#include <System/SafeBuffer.h>
#include <System/Security/Cryptography/CertificateExtensionBase.hxx>
#include <System/Security/Signed.hxx>
#include <xsec/dsig/DSIGConstants.hpp>

namespace Smartmatic
{
    namespace Log
    {
        class ISMTTLog;
    }

    namespace System
    {
        namespace Security
        {
            /**
             * @class	XmlSignatureAlgorithm
             * @brief	Class for XML signature algorithms.
             * @author	Eduardo Fernández
             * @date	09/10/2018
             */
            class XmlSignatureAlgorithm
            {
                public:

                    /**
                     * @fn	virtual ~XmlSignatureAlgorithm();
                     * @brief	Class destructor.
                     * @author	Eduardo Fernández
                     * @date	09/10/2018
                     */
                    virtual ~XmlSignatureAlgorithm() {}

                    /**
                     * @fn virtual bool signBuffer(
                               const Smartmatic::System::SafeBuffer& bufferToSign,
                               Smartmatic::System::SafeBuffer& signature,
                               const Signed::SignedType& signatureType,
                               Smartmatic::System::Security::Cryptography::CertificateExtensionBase * const certificate,
                               const hashMethod& hash,
                               const std::string& basePath) = 0;
                     * @brief  method to sign a XML buffer with a certificate.
                     * @author Eduardo Fernández
                     * @date   09/10/2018
                     * @param  bufferToSign buffer to sign.
                     * @param  basePath base path of the file to sign.
                     * @param  signature signature buffer.
                     * @param  signatureType signature type.
                     * @param  certificate complete certificate information.
                     * @param  hash hash method.
                     * @return true if signing is successful.
                     */
                    virtual bool signBuffer(
                        const Smartmatic::System::SafeBuffer& bufferToSign,
                        Smartmatic::System::SafeBuffer& signature,
                        const Signed::SignedType& signatureType,
                        Smartmatic::System::Security::Cryptography::CertificateExtensionBase * const certificate,
                        const hashMethod& hash,
                        const std::string& basePath) = 0;

                    /**
                     * @fn virtual bool verifyBuffer(
                               SafeBuffer& bufferToVerify,
                               X509 * const certificate,
                               const std::string& basePath) = 0;
                     * @brief  method to verify a XML signature with a certificate.
                     * @author Eduardo Fernández
                     * @date   09/10/2018
                     * @param  bufferToVerify Buffer including XML signature.
                     * @param  certificate X509 certificate.
                     * @param  basePath reference information.
                     * @return true if verification is successful.
                     */
                    virtual bool verifyBuffer(
                        const Smartmatic::System::SafeBuffer& bufferToVerify,
                        X509 * const certificate,
                        const std::string& basePath) = 0;
            };
        }
    }
}

#endif /* XML_SIGNATURE_ALGORITHM_HXX */
