/**
 * @file StackTraceManager.cxx
 * @brief Body stack trace manager
 * @date 28/02/2013
 * @author Yadickson Soto
 */

#include "StackTraceManager.hxx"
#include "ZenityWrapper.hxx"
#include <System/Runtime/ExitProcess.hxx>

#include <sstream>
#include <execinfo.h>
#include <signal.h>
#include <stdlib.h>

#define __MAX__ 25 ///< max stack trace

using namespace Smartmatic::System;
using namespace Smartmatic::System::Runtime;

StackTraceManager * StackTraceManager::instance = NULL;

StackTraceManager::StackTraceManager()
{

}

StackTraceManager::~StackTraceManager()
{
    instance = NULL;
}

void StackTraceManager::start()
{
    ( void ) signal ( SIGABRT, StackTraceManager::signalHandler );
    ( void ) signal ( SIGSEGV, StackTraceManager::signalHandler );
}

void StackTraceManager::stop()
{
    ( void ) signal ( SIGABRT, SIG_DFL );
    ( void ) signal ( SIGSEGV, SIG_DFL );
}

void StackTraceManager::signalHandler ( int sig )
{
    switch ( sig )
    {
        case SIGSEGV:
            StackTraceManager::getInstance()->showSigSegvError();
            break;

        case SIGABRT:
            StackTraceManager::getInstance()->showSigAbortError();
            break;

        default:
            break;
    }
}

void StackTraceManager::getStack ( std::list < std::string > & list )
{
    void * array[__MAX__];
    size_t size;
    size = backtrace ( array, __MAX__ );
    char ** messages = backtrace_symbols ( array, size );

    std::stringstream ss;
    ss << "An error has occurred in the application." << std::endl;
    ss << "Please ask for technical assistance." << std::endl;
    list.push_back ( ss.str() );

    for ( size_t i = 4, c = 0; i < size and i < __MAX__; i++, c++ )
    {
        std::stringstream ss;
        ss << "[bt]: (" << c << ") " << messages [i];
        list.push_back ( ss.str() );
    }

    if ( messages )
    {
        free ( messages );
    }
}

std::string StackTraceManager::getStackString ()
{
    std::list < std::string > list;
    std::list < std::string >::iterator it;
    std::stringstream ss;

    getStack ( list );

    for ( it = list.begin();
            it != list.end();
            ++it )
    {
        ss << *it << std::endl;
    }

    return ss.str();
}

void StackTraceManager::showSigSegvError()
{
    Glib::Mutex::Lock lock ( StackTraceManager::mutexSignal );

    ZenityWrapper::showError ( getStackString(), "Signal ERROR SIGSEGV" );
    ExitProcess::getInstance()->exitProcess();
}

void StackTraceManager::showSigAbortError()
{
    Glib::Mutex::Lock lock ( StackTraceManager::mutexSignal );

    ZenityWrapper::showError ( getStackString(), "Signal ERROR SIGABRT" );
    ExitProcess::getInstance()->exitProcess();
}

StackTraceManager * StackTraceManager::getInstance ( StackTraceManager * externStackTrace )
{
    if ( instance == NULL )
    {
        instance = ( externStackTrace == NULL ? new StackTraceManager() : externStackTrace );
    }

    return instance;
}
