#ifndef TEST_FILE_TEST
#define TEST_FILE_TEST

#ifdef ECUT_MAIN

#include <cppunit/extensions/HelperMacros.h>
#include <System/IO/File.h>
#include <System/Security/OpenSSLSecurity.hxx>
#include <tests/StoreMockFile.hxx>
#include <tests/MockDirectory.hxx>
#include "MockBasicFile.hxx"

class FileTest : public CPPUNIT_NS::TestFixture
{
    private:

        typedef Smartmatic::System::IO::File File;
        typedef Smartmatic::System::IO::Directory Directory;
        typedef Smartmatic::System::SafeBuffer SafeBuffer;

    public:

        FileTest ()
        {

        }

        void setUp()
        {
            Smartmatic::System::Security::OpenSSLSecurity::init();
        }

        void tearDown()
        {
            Smartmatic::System::Security::OpenSSLSecurity::clear();
        }

        void testReadFileNotFound()
        {
            StoreMockFile file;
            File::getInstance ( &file );

            file.openFileValues.push_back ( StoreMockFile::OpenFileValues ( SafeBuffer(), false ) );

            std::string fileName = "fileName";
            std::string content = "data";

            bool result;
            bool expected = false;

            result = File::getInstance()->readFile ( fileName, content );

            CPPUNIT_ASSERT ( expected == result );
            CPPUNIT_ASSERT ( true == content.empty() );
        }

        void testReadFileFound()
        {
            StoreMockFile file;
            File::getInstance ( &file );

            file.openFileValues.push_back ( StoreMockFile::OpenFileValues ( SafeBuffer ( "line" ), true ) );

            std::string fileName = "fileName";
            std::string content;

            bool result;
            bool expected = true;

            std::string value = "line";

            result = File::getInstance()->readFile ( fileName, content );

            CPPUNIT_ASSERT ( expected == result );
            CPPUNIT_ASSERT ( false == content.empty() );
            CPPUNIT_ASSERT ( value == content );
        }

        void testGetHashToFileNotFound()
        {
            StoreMockFile file;
            File::getInstance ( &file );

            file.openFileValues.push_back ( StoreMockFile::OpenFileValues ( SafeBuffer (), false ) );

            std::string fileName = "fileName";
            SafeBuffer hash ( "hash" );

            bool result;
            bool expected = false;

            result = File::getInstance()->getHashFile ( fileName, hash );

            CPPUNIT_ASSERT ( expected == result );
            CPPUNIT_ASSERT ( 0 == hash.GetSize() );
        }

        void testGetHashToFileFound()
        {
            StoreMockFile file;
            File::getInstance ( &file );

            file.openFileValues.push_back ( StoreMockFile::OpenFileValues ( SafeBuffer ( "ABCDEFG" ), true ) );

            std::string fileName = "fileName";
            SafeBuffer hash ( "hash" );

            bool result;
            bool expected = true;

            // echo -n "ABCDEFG" | openssl dgst -binary -sha256  | openssl enc -base64
            std::string value = "6akqLtDVNzKsE7AxonsHGBQjHIYzyfQYRMy6iE1IKxY=";

            result = File::getInstance()->getHashFile ( fileName, hash );

            CPPUNIT_ASSERT ( expected == result );
            CPPUNIT_ASSERT ( value == hash.GetBase64Data() );
        }

        void testCompareTwoFilesNotFound()
        {
            StoreMockFile file;
            File::getInstance ( &file );

            file.openFileValues.push_back ( StoreMockFile::OpenFileValues ( SafeBuffer (), false ) );
            file.openFileValues.push_back ( StoreMockFile::OpenFileValues ( SafeBuffer (), false ) );

            std::string fileName1 = "fileName1";
            std::string fileName2 = "fileName2";

            bool result;
            bool expected = false;

            result = File::getInstance()->compareFiles ( fileName1, fileName2 );

            CPPUNIT_ASSERT ( expected == result );
        }

        void testCompareFiles_FirstFound_And_TwoNotFound()
        {
            StoreMockFile file;
            File::getInstance ( &file );

            file.openFileValues.push_back ( StoreMockFile::OpenFileValues ( SafeBuffer ( "ABCDEFG" ), true ) );
            file.openFileValues.push_back ( StoreMockFile::OpenFileValues ( SafeBuffer (), false ) );

            std::string fileName1 = "fileName1";
            std::string fileName2 = "fileName2";

            bool result;
            bool expected = false;

            result = File::getInstance()->compareFiles ( fileName1, fileName2 );

            CPPUNIT_ASSERT ( expected == result );
        }

        void testCompareFiles_FirstNotFound_And_TwoFound()
        {
            StoreMockFile file;
            File::getInstance ( &file );

            file.openFileValues.push_back ( StoreMockFile::OpenFileValues ( SafeBuffer (), false ) );
            file.openFileValues.push_back ( StoreMockFile::OpenFileValues ( SafeBuffer ( "ABCDEFG" ), true ) );

            std::string fileName1 = "fileName1";
            std::string fileName2 = "fileName2";

            bool result;
            bool expected = false;

            result = File::getInstance()->compareFiles ( fileName1, fileName2 );

            CPPUNIT_ASSERT ( expected == result );
        }

        void testCompareDiferentFiles()
        {
            StoreMockFile file;
            File::getInstance ( &file );

            file.openFileValues.push_back ( StoreMockFile::OpenFileValues ( SafeBuffer ( "1234567" ), true ) );
            file.openFileValues.push_back ( StoreMockFile::OpenFileValues ( SafeBuffer ( "ABCDEFG" ), true ) );

            std::string fileName1 = "fileName1";
            std::string fileName2 = "fileName2";

            bool result;
            bool expected = false;

            result = File::getInstance()->compareFiles ( fileName1, fileName2 );

            CPPUNIT_ASSERT ( expected == result );
        }

        void testCompareFiles()
        {
            StoreMockFile file;
            File::getInstance ( &file );

            file.openFileValues.push_back ( StoreMockFile::OpenFileValues ( SafeBuffer ( "ABCDEFG" ), true ) );
            file.openFileValues.push_back ( StoreMockFile::OpenFileValues ( SafeBuffer ( "ABCDEFG" ), true ) );

            std::string fileName1 = "fileName1";
            std::string fileName2 = "fileName2";

            bool result;
            bool expected = true;

            result = File::getInstance()->compareFiles ( fileName1, fileName2 );

            CPPUNIT_ASSERT ( expected == result );
        }

        void testCopyFileErrorOpenFile()
        {
            MockBasicFile file;
            File::getInstance ( &file );

            MockDirectory directory;
            Directory::getInstance ( &directory );

            MOCKPP_CHAINER_FOR_EXT ( MockBasicFile, openFileToBuffer, ext ) openFileToBuffer_mock ( &file );
            MOCKPP_CHAINER_FOR_EXT ( MockBasicFile, saveBufferToFile, ext ) saveBufferToFile_mock ( &file );
            MOCKPP_CHAINER_FOR_EXT ( MockDirectory, Mkdir, ext ) Mkdir_mock ( &directory );

            openFileToBuffer_mock
            .expects ( MOCKPP_NS::once() )
            .before ( "save" )
            .will ( MOCKPP_NS::returnValue<bool> ( false ) )
            .id ( "open" );

            saveBufferToFile_mock
            .expects ( MOCKPP_NS::never() )
            .after ( "open" )
            .will ( MOCKPP_NS::returnValue<bool> ( false ) )
            .id ( "save" );

            Mkdir_mock
            .expects ( MOCKPP_NS::once() )
            .will ( MOCKPP_NS::returnValue<bool> ( true ) );

            std::string fileName1 = "fileName1";
            std::string fileName2 = "fileName2";

            bool result;
            bool expected = false;

            result = File::getInstance()->copyFile ( fileName1, fileName2 );

            CPPUNIT_ASSERT ( expected == result );

            file.verify();
            directory.verify();
        }

        void testCopyFileErrorSaveFile()
        {
            MockBasicFile file;
            File::getInstance ( &file );

            MockDirectory directory;
            Directory::getInstance ( &directory );

            MOCKPP_CHAINER_FOR_EXT ( MockBasicFile, openFileToBuffer, ext ) openFileToBuffer_mock ( &file );
            MOCKPP_CHAINER_FOR_EXT ( MockBasicFile, saveBufferToFile, ext ) saveBufferToFile_mock ( &file );
            MOCKPP_CHAINER_FOR_EXT ( MockDirectory, Mkdir, ext ) Mkdir_mock ( &directory );

            openFileToBuffer_mock
            .expects ( MOCKPP_NS::once() )
            .before ( "save" )
            .will ( MOCKPP_NS::returnValue<bool> ( true ) )
            .id ( "open" );

            saveBufferToFile_mock
            .expects ( MOCKPP_NS::once() )
            .after ( "open" )
            .will ( MOCKPP_NS::returnValue<bool> ( false ) )
            .id ( "save" );

            Mkdir_mock
            .expects ( MOCKPP_NS::once() )
            .will ( MOCKPP_NS::returnValue<bool> ( true ) );

            std::string fileName1 = "fileName1";
            std::string fileName2 = "fileName2";

            bool result;
            bool expected = false;

            result = File::getInstance()->copyFile ( fileName1, fileName2 );

            CPPUNIT_ASSERT ( expected == result );

            file.verify();
            directory.verify();
        }

        void testCopyFile()
        {
            MockBasicFile file;
            File::getInstance ( &file );

            MockDirectory directory;
            Directory::getInstance ( &directory );

            MOCKPP_CHAINER_FOR_EXT ( MockBasicFile, openFileToBuffer, ext ) openFileToBuffer_mock ( &file );
            MOCKPP_CHAINER_FOR_EXT ( MockBasicFile, saveBufferToFile, ext ) saveBufferToFile_mock ( &file );
            MOCKPP_CHAINER_FOR_EXT ( MockDirectory, Mkdir, ext ) Mkdir_mock ( &directory );

            openFileToBuffer_mock
            .expects ( MOCKPP_NS::once() )
            .before ( "save" )
            .will ( MOCKPP_NS::returnValue<bool> ( true ) )
            .id ( "open" );

            saveBufferToFile_mock
            .expects ( MOCKPP_NS::once() )
            .after ( "open" )
            .will ( MOCKPP_NS::returnValue<bool> ( true ) )
            .id ( "save" );

            Mkdir_mock
            .expects ( MOCKPP_NS::once() )
            .will ( MOCKPP_NS::returnValue<bool> ( true ) );

            std::string fileName1 = "fileName1";
            std::string fileName2 = "fileName2";

            bool result;
            bool expected = true;

            result = File::getInstance()->copyFile ( fileName1, fileName2 );

            CPPUNIT_ASSERT ( expected == result );

            file.verify();
            directory.verify();
        }

        CPPUNIT_TEST_SUITE ( FileTest );

        CPPUNIT_TEST ( testReadFileNotFound );
        CPPUNIT_TEST ( testReadFileFound );

        CPPUNIT_TEST ( testGetHashToFileNotFound );
        CPPUNIT_TEST ( testGetHashToFileFound );

        CPPUNIT_TEST ( testCompareTwoFilesNotFound );
        CPPUNIT_TEST ( testCompareFiles_FirstFound_And_TwoNotFound );
        CPPUNIT_TEST ( testCompareFiles_FirstNotFound_And_TwoFound );
        CPPUNIT_TEST ( testCompareDiferentFiles );
        CPPUNIT_TEST ( testCompareFiles );

        CPPUNIT_TEST ( testCopyFileErrorOpenFile );
        CPPUNIT_TEST ( testCopyFileErrorSaveFile );
        CPPUNIT_TEST ( testCopyFile );

        CPPUNIT_TEST_SUITE_END();
};

#endif /* ECUT_MAIN */

#endif /* TEST_FILE_TEST */
