#ifndef TEST_CRYPTOCBCTEST
#define TEST_CRYPTOCBCTEST

#ifdef ECUT_MAIN

#include <cppunit/extensions/HelperMacros.h>
#include <System/Security/Cryptography/CryptoAES.hxx>
#include <System/Security/ComposeKey.hxx>
#include <System/Security/OpenSSLSecurity.hxx>
#include <openssl/evp.h>

class CryptoCBCTest : public CPPUNIT_NS::TestFixture
{
    private:

        Smartmatic::System::Security::Cryptography::CryptoAES * object;

    public:

        CryptoCBCTest()
        {
            object = NULL;
        }

        void setUp()
        {
            Smartmatic::System::Security::OpenSSLSecurity::init();
            object = new Smartmatic::System::Security::Cryptography::CryptoAES ( EVP_aes_256_cbc() );
        }

        void tearDown()
        {
            Smartmatic::System::Security::OpenSSLSecurity::clear();
            delete object;
        }

        void testCryptoKeyNull()
        {
            Smartmatic::System::SafeBuffer key;
            Smartmatic::System::SafeBuffer iv;

            Smartmatic::System::SafeBuffer dataIn ( "ABCDEFGH" );
            Smartmatic::System::SafeBuffer dataOut;

            CPPUNIT_ASSERT_THROW (
                object->encryptData ( dataIn, dataOut, key.GetData(), iv.GetData(), true ),
                Smartmatic::System::Exception::CryptoException );
        }

        void testCryptoChiperNull()
        {
            Smartmatic::System::SafeBuffer key;
            Smartmatic::System::SafeBuffer iv;

            Smartmatic::System::SafeBuffer dataIn ( "ABCDEFGH" );
            Smartmatic::System::SafeBuffer dataOut;

            Smartmatic::System::Security::Cryptography::CryptoAES withOutChiper ( NULL );

            CPPUNIT_ASSERT_THROW (
                withOutChiper.encryptData ( dataIn, dataOut, key.GetData(), iv.GetData(), true ),
                Smartmatic::System::Exception::CryptoException );
        }

        void testEncryptDataNull()
        {
            Smartmatic::System::SafeBuffer key ( "abcdefghijklmnopabcdefghijklmnop" );
            Smartmatic::System::SafeBuffer iv;

            Smartmatic::System::Security::ComposeKey compose ( key );
            compose.getCompose ( 16, iv );

            Smartmatic::System::SafeBuffer dataIn;
            Smartmatic::System::SafeBuffer dataOut;

            CPPUNIT_ASSERT_NO_THROW ( object->encryptData ( dataIn, dataOut, key.GetData(), iv.GetData(), true ) );
            CPPUNIT_ASSERT ( NULL != dataOut.GetData() );
            CPPUNIT_ASSERT ( 16 == dataOut.GetSize() );
        }

        void testEncryptData8()
        {
            Smartmatic::System::SafeBuffer key ( "abcdefghijklmnopabcdefghijklmnop" );
            Smartmatic::System::SafeBuffer iv;

            Smartmatic::System::Security::ComposeKey compose ( key );
            compose.getCompose ( 16, iv );

            Smartmatic::System::SafeBuffer dataIn ( "ABCDEFGH" );
            Smartmatic::System::SafeBuffer dataOut;

            object->encryptData ( dataIn, dataOut, key.GetData(), iv.GetData(), true );

            CPPUNIT_ASSERT ( NULL != dataOut.GetData() );
            CPPUNIT_ASSERT ( 16 == dataOut.GetSize() );
        }

        void testEncryptData16()
        {
            Smartmatic::System::SafeBuffer key ( "abcdefghijklmnopabcdefghijklmnop" );
            Smartmatic::System::SafeBuffer iv;

            Smartmatic::System::Security::ComposeKey compose ( key );
            compose.getCompose ( 16, iv );

            Smartmatic::System::SafeBuffer dataIn ( "ABCDEFGHIJKLMNOP" );
            Smartmatic::System::SafeBuffer dataOut;

            object->encryptData ( dataIn, dataOut, key.GetData(), iv.GetData(), true );

            CPPUNIT_ASSERT ( NULL != dataOut.GetData() );
            CPPUNIT_ASSERT ( 32 == dataOut.GetSize() );
        }

        void testEncryptData24()
        {
            Smartmatic::System::SafeBuffer key ( "abcdefghijklmnopabcdefghijklmnop" );
            Smartmatic::System::SafeBuffer iv;

            Smartmatic::System::Security::ComposeKey compose ( key );
            compose.getCompose ( 16, iv );

            Smartmatic::System::SafeBuffer dataIn ( "ABCDEFGHIJKMNOP12345678" );
            Smartmatic::System::SafeBuffer dataOut;

            object->encryptData ( dataIn, dataOut, key.GetData(), iv.GetData(), true );

            CPPUNIT_ASSERT ( NULL != dataOut.GetData() );
            CPPUNIT_ASSERT ( 32 == dataOut.GetSize() );
        }

        void testEncryptData32()
        {
            Smartmatic::System::SafeBuffer key ( "abcdefghijklmnopabcdefghijklmnop" );
            Smartmatic::System::SafeBuffer iv;

            Smartmatic::System::Security::ComposeKey compose ( key );
            compose.getCompose ( 16, iv );

            Smartmatic::System::SafeBuffer dataIn ( "ABCDEFGHIJKABCDEFGHIJKMNOP" );
            Smartmatic::System::SafeBuffer dataOut;

            object->encryptData ( dataIn, dataOut, key.GetData(), iv.GetData(), true );

            CPPUNIT_ASSERT ( NULL != dataOut.GetData() );
            CPPUNIT_ASSERT ( 32 == dataOut.GetSize() );
        }

        void testDecryptoKeyNull()
        {
            Smartmatic::System::SafeBuffer key;
            Smartmatic::System::SafeBuffer iv;

            Smartmatic::System::Security::ComposeKey compose ( key );
            compose.getCompose ( 16, iv );

            Smartmatic::System::SafeBuffer dataIn ( "ABCDEFGHIJKABCDEFGHIJKMNOP" );
            Smartmatic::System::SafeBuffer dataOut;

            CPPUNIT_ASSERT_THROW (
                object->decryptData ( dataIn, dataOut, key.GetData(), iv.GetData(), true ),
                Smartmatic::System::Exception::CryptoException );
        }

        void testDecryptDataNull()
        {
            Smartmatic::System::SafeBuffer key ( "abcdefghijklmnopabcdefghijklmnop" );
            Smartmatic::System::SafeBuffer iv;

            Smartmatic::System::Security::ComposeKey compose ( key );
            compose.getCompose ( 16, iv );

            Smartmatic::System::SafeBuffer dataIn;
            Smartmatic::System::SafeBuffer dataOut;

            CPPUNIT_ASSERT_THROW (
                object->decryptData ( dataIn, dataOut, key.GetData(), iv.GetData(), true ),
                Smartmatic::System::Exception::CryptoException );
        }

        void testDecryptDataError()
        {
            Smartmatic::System::SafeBuffer key ( "abcdefghijklmnopabcdefghijklmnop" );
            Smartmatic::System::SafeBuffer iv;

            Smartmatic::System::Security::ComposeKey compose ( key );
            compose.getCompose ( 16, iv );

            Smartmatic::System::SafeBuffer dataIn ( "ERROR" );
            Smartmatic::System::SafeBuffer dataOut;

            CPPUNIT_ASSERT_THROW (
                object->decryptData ( dataIn, dataOut, key.GetData(), iv.GetData(), true ),
                Smartmatic::System::Exception::CryptoException );
        }

        void testRegretion()
        {
            Smartmatic::System::SafeBuffer key ( "abcdefghijklmnopabcdefghijklmnop" );
            Smartmatic::System::SafeBuffer iv;

            Smartmatic::System::Security::ComposeKey compose ( key );
            compose.getCompose ( 16, iv );

            Smartmatic::System::SafeBuffer dataIn ( "ABCDEFGHIJKABCDEFGHIJKMNOP" );
            Smartmatic::System::SafeBuffer dataCrypto;
            Smartmatic::System::SafeBuffer dataOut;

            object->encryptData ( dataIn, dataCrypto, key.GetData(), iv.GetData(), true );
            object->decryptData ( dataCrypto, dataOut, key.GetData(), iv.GetData(), true );

            CPPUNIT_ASSERT ( NULL != dataOut.GetData() );
            CPPUNIT_ASSERT ( 26 == dataOut.GetSize() );

            std::string result;
            result.assign ( ( const char * ) dataOut.GetData(), dataOut.GetSize() );

            CPPUNIT_ASSERT ( "ABCDEFGHIJKABCDEFGHIJKMNOP" == result );
        }

        void testDecryptError()
        {
            Smartmatic::System::SafeBuffer key ( "abcdefghijklmnopabcdefghijklmnop" );
            Smartmatic::System::SafeBuffer iv;

            Smartmatic::System::Security::ComposeKey compose ( key );
            compose.getCompose ( 16, iv );

            Smartmatic::System::SafeBuffer dataIn ( "ABCDEFGHIJKABCDEFGHIJKMNOP" );
            Smartmatic::System::SafeBuffer dataCrypto;
            Smartmatic::System::SafeBuffer dataOut;

            object->encryptData ( dataIn, dataCrypto, key.GetData(), iv.GetData(), true );

            dataCrypto.removeData ( 0, 1 );

            CPPUNIT_ASSERT_THROW (
                object->decryptData ( dataCrypto, dataOut, key.GetData(), iv.GetData(), true ),
                Smartmatic::System::Exception::CryptoException );
        }

        void testDecryptDiferentKey()
        {
            Smartmatic::System::SafeBuffer key ( "abcdefghijklmnopabcdefghijklmnop" );
            Smartmatic::System::SafeBuffer iv;

            Smartmatic::System::SafeBuffer key2 ( "abcdefghijklmnopxabcdefghijklmnopx" );
            Smartmatic::System::SafeBuffer iv2;

            Smartmatic::System::Security::ComposeKey compose ( key );
            compose.getCompose ( 16, iv );

            Smartmatic::System::Security::ComposeKey compose2 ( key2 );
            compose.getCompose ( 16, iv2 );

            Smartmatic::System::SafeBuffer dataIn ( "ABCDEFGHIJKABCDEFGHIJKMNOP" );
            Smartmatic::System::SafeBuffer dataCrypto;
            Smartmatic::System::SafeBuffer dataOut;

            object->encryptData ( dataIn, dataCrypto, key.GetData(), iv.GetData(), true );

            CPPUNIT_ASSERT_THROW (
                object->decryptData ( dataCrypto, dataOut, key2.GetData(), iv2.GetData(), true ),
                Smartmatic::System::Exception::CryptoException );
        }

        CPPUNIT_TEST_SUITE ( CryptoCBCTest );
        CPPUNIT_TEST ( testCryptoKeyNull );
        CPPUNIT_TEST ( testCryptoChiperNull );
        CPPUNIT_TEST ( testEncryptDataNull );
        CPPUNIT_TEST ( testEncryptData8 );
        CPPUNIT_TEST ( testEncryptData16 );
        CPPUNIT_TEST ( testEncryptData24 );
        CPPUNIT_TEST ( testEncryptData32 );
        CPPUNIT_TEST ( testDecryptoKeyNull );
        CPPUNIT_TEST ( testDecryptDataNull );
        CPPUNIT_TEST ( testDecryptDataError );
        CPPUNIT_TEST ( testRegretion );
        CPPUNIT_TEST ( testDecryptError );
        CPPUNIT_TEST ( testDecryptDiferentKey );
        CPPUNIT_TEST_SUITE_END();
};

#endif /* ECUT_MAIN */

#endif /* TEST_CRYPTOCBCTEST */
