#ifndef TEST_CRYPTO_CLASS_TEST
#define TEST_CRYPTO_CLASS_TEST

#ifdef ECUT_MAIN

#include <cppunit/extensions/HelperMacros.h>
#include <System/Security/Crypto.hxx>
#include <System/Security/ComposeKey.hxx>
#include <System/Security/OpenSSLSecurity.hxx>
#include <tests/StoreMockFile.hxx>
#include <System/IO/File.h>

class CryptoClassTest : public CPPUNIT_NS::TestFixture
{
    private:

        typedef Smartmatic::System::Security::Crypto Crypto;
        typedef Smartmatic::System::SafeBuffer SafeBuffer;
        typedef Smartmatic::System::Exception::CryptoException CryptoException;
        typedef Smartmatic::System::Security::ComposeKey ComposeKey;
        typedef Smartmatic::System::IO::File File;

        Crypto * object;

    public:

        CryptoClassTest()
        {
            object = NULL;
        }

        void setUp()
        {
            Smartmatic::System::Security::OpenSSLSecurity::init();
            object = new Crypto();
        }

        void tearDown()
        {
            Smartmatic::System::Security::OpenSSLSecurity::clear();
            delete object;
        }

        void testEncryptBufferNotInitClass()
        {
            SafeBuffer key ( "abcdefghijklmnop" );

            SafeBuffer dataIn ( "ABCDEFGH" );
            SafeBuffer dataOut;

            CPPUNIT_ASSERT_THROW (
                object->encryptBufferToBufferAndCreateIV ( dataIn, dataOut, key.GetData()),
                CryptoException );
        }

        void testEncryptBufferInitClassKeyNull()
        {
            SafeBuffer key;

            SafeBuffer dataIn ( "ABCDEFGH" );
            SafeBuffer dataOut;

            object->init ( Crypto::KEY_AES_256, Crypto::MODE_AES_ECB, true );

            CPPUNIT_ASSERT_THROW (
                object->encryptBufferToBufferAndCreateIV ( dataIn, dataOut, key.GetData() ),
                CryptoException );
        }

        void testEncryptDataNull()
        {
            SafeBuffer key ( "abcdefghijklmnop" );

            SafeBuffer dataIn;
            SafeBuffer dataOut;

            object->init ( Crypto::KEY_AES_256, Crypto::MODE_AES_ECB, true );

            CPPUNIT_ASSERT_THROW (
                object->encryptBufferToBufferAndCreateIV ( dataIn, dataOut, key.GetData() ),
                CryptoException );
        }

        void testDecryptBufferNotInitClass()
        {
            SafeBuffer key ( "abcdefghijklmnop" );

            SafeBuffer dataIn ( "ABCDEFGH" );
            SafeBuffer dataOut;

            CPPUNIT_ASSERT_THROW (
                object->decryptBufferToBufferAndGetIV ( dataIn, dataOut, key.GetData() ),
                CryptoException );
        }

        void testDecryptBufferInitClassKeyNull()
        {
            SafeBuffer key;

            SafeBuffer dataIn ( "ABCDEFGH" );
            SafeBuffer dataOut;

            object->init ( Crypto::KEY_AES_256, Crypto::MODE_AES_ECB, true );

            CPPUNIT_ASSERT_THROW (
                object->encryptBufferToBufferAndCreateIV ( dataIn, dataOut, key.GetData() ),
                CryptoException );
        }

        void testDecryptDataNull()
        {
            SafeBuffer key ( "abcdefghijklmnopabcdefghijklmnop" );

            SafeBuffer dataIn;
            SafeBuffer dataOut;

            object->init ( Crypto::KEY_AES_256, Crypto::MODE_AES_ECB, true );

            CPPUNIT_ASSERT_THROW (
                object->decryptBufferToBufferAndGetIV ( dataIn, dataOut, key.GetData() ),
                CryptoException );
        }

        void testEncryptAndDecryptBufferError()
        {
            SafeBuffer key ( "abcdefghijklmnopabcdefghijklmnop" );

            SafeBuffer dataIn ( "Hello" );
            SafeBuffer encryptBuffer;
            SafeBuffer decryptBuffer;

            object->init ( Crypto::KEY_AES_256, Crypto::MODE_AES_ECB, true );
            object->encryptBufferToBufferAndCreateIV ( dataIn, encryptBuffer, key.GetData() );

            encryptBuffer.removeData ( 0, 1 );

            CPPUNIT_ASSERT_THROW (
                object->decryptBufferToBufferAndGetIV ( encryptBuffer, decryptBuffer, key.GetData()),
                CryptoException );
        }

        void testDecryptDiferentKey()
        {
            SafeBuffer key ( "abcdefghijklmnopabcdefghijklmnop" );

            SafeBuffer key2 ( "abcdefghijklmnopxabcdefghijklmnopx" );

            SafeBuffer dataIn ( "Hello" );
            SafeBuffer encryptBuffer;
            SafeBuffer decryptBuffer;

            object->init ( Crypto::KEY_AES_256, Crypto::MODE_AES_ECB, true );
            object->encryptBufferToBufferAndCreateIV ( dataIn, encryptBuffer, key.GetData() );

            CPPUNIT_ASSERT_THROW (
                object->decryptBufferToBufferAndGetIV( encryptBuffer, decryptBuffer, key2.GetData()),
                CryptoException );
        }

        void testRegretion_128_ECB()
        {
            SafeBuffer key ( "abcdefghijklmnop" );

            SafeBuffer dataIn ( "Hello" );
            SafeBuffer encryptBuffer;
            SafeBuffer decryptBuffer;

            object->init ( Crypto::KEY_AES_128, Crypto::MODE_AES_ECB, true );
            object->encryptBufferToBufferAndCreateIV( dataIn, encryptBuffer, key.GetData() );

            CPPUNIT_ASSERT ( 0 != encryptBuffer.GetSize() );
            CPPUNIT_ASSERT ( dataIn != encryptBuffer );

            object->decryptBufferToBufferAndGetIV ( encryptBuffer, decryptBuffer, key.GetData() );

            CPPUNIT_ASSERT ( dataIn == decryptBuffer );
        }

        void testRegretion_192_ECB()
        {
            SafeBuffer key ( "abcdefghijklmnopabcdefgh" );

            SafeBuffer dataIn ( "Hello" );
            SafeBuffer encryptBuffer;
            SafeBuffer decryptBuffer;

            object->init ( Crypto::KEY_AES_192, Crypto::MODE_AES_ECB, true );
            object->encryptBufferToBufferAndCreateIV( dataIn, encryptBuffer, key.GetData() );

            CPPUNIT_ASSERT ( 0 != encryptBuffer.GetSize() );
            CPPUNIT_ASSERT ( dataIn != encryptBuffer );

            object->decryptBufferToBufferAndGetIV ( encryptBuffer, decryptBuffer, key.GetData());

            CPPUNIT_ASSERT ( dataIn == decryptBuffer );
        }

        void testRegretion_256_ECB()
        {
            SafeBuffer key ( "abcdefghijklmnopabcdefghijklmnop" );

            SafeBuffer dataIn ( "Hello" );
            SafeBuffer encryptBuffer;
            SafeBuffer decryptBuffer;

            object->init ( Crypto::KEY_AES_256, Crypto::MODE_AES_ECB, true );
            object->encryptBufferToBufferAndCreateIV( dataIn, encryptBuffer, key.GetData() );

            CPPUNIT_ASSERT ( 0 != encryptBuffer.GetSize() );
            CPPUNIT_ASSERT ( dataIn != encryptBuffer );

            object->decryptBufferToBufferAndGetIV ( encryptBuffer, decryptBuffer, key.GetData());

            CPPUNIT_ASSERT ( dataIn == decryptBuffer );
        }

        void testRegretion_128_CBC()
        {
            SafeBuffer key ( "abcdefghijklmnop" );

            SafeBuffer dataIn ( "Hello" );
            SafeBuffer encryptBuffer;
            SafeBuffer decryptBuffer;

            object->init ( Crypto::KEY_AES_128, Crypto::MODE_AES_CBC, true );
            object->encryptBufferToBufferAndCreateIV ( dataIn, encryptBuffer, key.GetData());

            CPPUNIT_ASSERT ( 0 != encryptBuffer.GetSize() );
            CPPUNIT_ASSERT ( dataIn != encryptBuffer );

            object->decryptBufferToBufferAndGetIV ( encryptBuffer, decryptBuffer, key.GetData());

            CPPUNIT_ASSERT ( dataIn == decryptBuffer );
        }

        void testRegretion_192_CBC()
        {
            SafeBuffer key ( "abcdefghijklmnopabcdefgh" );

            SafeBuffer dataIn ( "Hello" );
            SafeBuffer encryptBuffer;
            SafeBuffer decryptBuffer;

            object->init ( Crypto::KEY_AES_192, Crypto::MODE_AES_CBC, true );
            object->encryptBufferToBufferAndCreateIV ( dataIn, encryptBuffer, key.GetData() );

            CPPUNIT_ASSERT ( 0 != encryptBuffer.GetSize() );
            CPPUNIT_ASSERT ( dataIn != encryptBuffer );

            object->decryptBufferToBufferAndGetIV ( encryptBuffer, decryptBuffer, key.GetData() );

            CPPUNIT_ASSERT ( dataIn == decryptBuffer );
        }

        void testRegretion_256_CBC()
        {
            SafeBuffer key ( "abcdefghijklmnopabcdefghijklmnop" );

            SafeBuffer dataIn ( "Hello" );
            SafeBuffer encryptBuffer;
            SafeBuffer decryptBuffer;

            object->init ( Crypto::KEY_AES_256, Crypto::MODE_AES_CBC, true );
            object->encryptBufferToBufferAndCreateIV ( dataIn, encryptBuffer, key.GetData() );

            CPPUNIT_ASSERT ( 0 != encryptBuffer.GetSize() );
            CPPUNIT_ASSERT ( dataIn != encryptBuffer );

            object->decryptBufferToBufferAndGetIV ( encryptBuffer, decryptBuffer, key.GetData() );

            CPPUNIT_ASSERT ( dataIn == decryptBuffer );
        }

        void testRegretion_128_CFB1()
        {
            SafeBuffer key ( "abcdefghijklmnop" );

            SafeBuffer dataIn ( "Hello" );
            SafeBuffer encryptBuffer;
            SafeBuffer decryptBuffer;

            object->init ( Crypto::KEY_AES_128, Crypto::MODE_AES_CFB1, true );
            object->encryptBufferToBufferAndCreateIV ( dataIn, encryptBuffer, key.GetData());

            CPPUNIT_ASSERT ( 0 != encryptBuffer.GetSize() );
            CPPUNIT_ASSERT ( dataIn != encryptBuffer );

            object->decryptBufferToBufferAndGetIV ( encryptBuffer, decryptBuffer, key.GetData() );

            CPPUNIT_ASSERT ( dataIn == decryptBuffer );
        }

        void testRegretion_192_CFB1()
        {
            SafeBuffer key ( "abcdefghijklmnopabcdefgh" );

            SafeBuffer dataIn ( "Hello" );
            SafeBuffer encryptBuffer;
            SafeBuffer decryptBuffer;

            object->init ( Crypto::KEY_AES_192, Crypto::MODE_AES_CFB1, true );
            object->encryptBufferToBufferAndCreateIV ( dataIn, encryptBuffer, key.GetData() );

            CPPUNIT_ASSERT ( 0 != encryptBuffer.GetSize() );
            CPPUNIT_ASSERT ( dataIn != encryptBuffer );

            object->decryptBufferToBufferAndGetIV ( encryptBuffer, decryptBuffer, key.GetData() );

            CPPUNIT_ASSERT ( dataIn == decryptBuffer );
        }

        void testRegretion_256_CFB1()
        {
            SafeBuffer key ( "abcdefghijklmnopabcdefghijklmnop" );

            SafeBuffer dataIn ( "Hello" );
            SafeBuffer encryptBuffer;
            SafeBuffer decryptBuffer;

            object->init ( Crypto::KEY_AES_256, Crypto::MODE_AES_CFB1, true );
            object->encryptBufferToBufferAndCreateIV ( dataIn, encryptBuffer, key.GetData() );

            CPPUNIT_ASSERT ( 0 != encryptBuffer.GetSize() );
            CPPUNIT_ASSERT ( dataIn != encryptBuffer );

            object->decryptBufferToBufferAndGetIV ( encryptBuffer, decryptBuffer, key.GetData() );

            CPPUNIT_ASSERT ( dataIn == decryptBuffer );
        }

        void testRegretion_128_CFB8()
        {
            SafeBuffer key ( "abcdefghijklmnop" );

            SafeBuffer dataIn ( "Hello" );
            SafeBuffer encryptBuffer;
            SafeBuffer decryptBuffer;

            object->init ( Crypto::KEY_AES_128, Crypto::MODE_AES_CFB8, true );
            object->encryptBufferToBufferAndCreateIV ( dataIn, encryptBuffer, key.GetData() );

            CPPUNIT_ASSERT ( 0 != encryptBuffer.GetSize() );
            CPPUNIT_ASSERT ( dataIn != encryptBuffer );

            object->decryptBufferToBufferAndGetIV ( encryptBuffer, decryptBuffer, key.GetData() );

            CPPUNIT_ASSERT ( dataIn == decryptBuffer );
        }

        void testRegretion_192_CFB8()
        {
            SafeBuffer key ( "abcdefghijklmnopabcdefgh" );

            SafeBuffer dataIn ( "Hello" );
            SafeBuffer encryptBuffer;
            SafeBuffer decryptBuffer;

            object->init ( Crypto::KEY_AES_192, Crypto::MODE_AES_CFB8, true );
            object->encryptBufferToBufferAndCreateIV ( dataIn, encryptBuffer, key.GetData() );

            CPPUNIT_ASSERT ( 0 != encryptBuffer.GetSize() );
            CPPUNIT_ASSERT ( dataIn != encryptBuffer );

            object->decryptBufferToBufferAndGetIV ( encryptBuffer, decryptBuffer, key.GetData() );

            CPPUNIT_ASSERT ( dataIn == decryptBuffer );
        }

        void testRegretion_256_CFB8()
        {
            SafeBuffer key ( "abcdefghijklmnopabcdefghijklmnop" );

            SafeBuffer dataIn ( "Hello" );
            SafeBuffer encryptBuffer;
            SafeBuffer decryptBuffer;

            object->init ( Crypto::KEY_AES_256, Crypto::MODE_AES_CFB8, true );
            object->encryptBufferToBufferAndCreateIV ( dataIn, encryptBuffer, key.GetData() );

            CPPUNIT_ASSERT ( 0 != encryptBuffer.GetSize() );
            CPPUNIT_ASSERT ( dataIn != encryptBuffer );

            object->decryptBufferToBufferAndGetIV ( encryptBuffer, decryptBuffer, key.GetData() );

            CPPUNIT_ASSERT ( dataIn == decryptBuffer );
        }

        void testRegretion_128_CFB128()
        {
            SafeBuffer key ( "abcdefghijklmnop" );

            SafeBuffer dataIn ( "Hello" );
            SafeBuffer encryptBuffer;
            SafeBuffer decryptBuffer;

            object->init ( Crypto::KEY_AES_128, Crypto::MODE_AES_CFB128, true );
            object->encryptBufferToBufferAndCreateIV ( dataIn, encryptBuffer, key.GetData() );

            CPPUNIT_ASSERT ( 0 != encryptBuffer.GetSize() );
            CPPUNIT_ASSERT ( dataIn != encryptBuffer );

            object->decryptBufferToBufferAndGetIV ( encryptBuffer, decryptBuffer, key.GetData() );

            CPPUNIT_ASSERT ( dataIn == decryptBuffer );
        }

        void testRegretion_192_CFB128()
        {
            SafeBuffer key ( "abcdefghijklmnopabcdefgh" );

            SafeBuffer dataIn ( "Hello" );
            SafeBuffer encryptBuffer;
            SafeBuffer decryptBuffer;

            object->init ( Crypto::KEY_AES_192, Crypto::MODE_AES_CFB128, true );
            object->encryptBufferToBufferAndCreateIV ( dataIn, encryptBuffer, key.GetData() );

            CPPUNIT_ASSERT ( 0 != encryptBuffer.GetSize() );
            CPPUNIT_ASSERT ( dataIn != encryptBuffer );

            object->decryptBufferToBufferAndGetIV ( encryptBuffer, decryptBuffer, key.GetData() );

            CPPUNIT_ASSERT ( dataIn == decryptBuffer );
        }

        void testRegretion_256_CFB128()
        {
            SafeBuffer key ( "abcdefghijklmnopabcdefghijklmnop" );

            SafeBuffer dataIn ( "Hello" );
            SafeBuffer encryptBuffer;
            SafeBuffer decryptBuffer;

            object->init ( Crypto::KEY_AES_256, Crypto::MODE_AES_CFB128, true );
            object->encryptBufferToBufferAndCreateIV ( dataIn, encryptBuffer, key.GetData() );

            CPPUNIT_ASSERT ( 0 != encryptBuffer.GetSize() );
            CPPUNIT_ASSERT ( dataIn != encryptBuffer );

            object->decryptBufferToBufferAndGetIV ( encryptBuffer, decryptBuffer, key.GetData() );

            CPPUNIT_ASSERT ( dataIn == decryptBuffer );
        }

        void testRegretion_128_OFB()
        {
            SafeBuffer key ( "abcdefghijklmnop" );

            SafeBuffer dataIn ( "Hello" );
            SafeBuffer encryptBuffer;
            SafeBuffer decryptBuffer;

            object->init ( Crypto::KEY_AES_128, Crypto::MODE_AES_OFB, true );
            object->encryptBufferToBufferAndCreateIV ( dataIn, encryptBuffer, key.GetData() );

            CPPUNIT_ASSERT ( 0 != encryptBuffer.GetSize() );
            CPPUNIT_ASSERT ( dataIn != encryptBuffer );

            object->decryptBufferToBufferAndGetIV ( encryptBuffer, decryptBuffer, key.GetData() );

            CPPUNIT_ASSERT ( dataIn == decryptBuffer );
        }

        void testRegretion_192_OFB()
        {
            SafeBuffer key ( "abcdefghijklmnopabcdefgh" );

            SafeBuffer dataIn ( "Hello" );
            SafeBuffer encryptBuffer;
            SafeBuffer decryptBuffer;

            object->init ( Crypto::KEY_AES_192, Crypto::MODE_AES_OFB, true );
            object->encryptBufferToBufferAndCreateIV ( dataIn, encryptBuffer, key.GetData() );

            CPPUNIT_ASSERT ( 0 != encryptBuffer.GetSize() );
            CPPUNIT_ASSERT ( dataIn != encryptBuffer );

            object->decryptBufferToBufferAndGetIV ( encryptBuffer, decryptBuffer, key.GetData() );

            CPPUNIT_ASSERT ( dataIn == decryptBuffer );
        }

        void testRegretion_256_OFB()
        {
            SafeBuffer key ( "abcdefghijklmnopabcdefghijklmnop" );

            SafeBuffer dataIn ( "Hello" );
            SafeBuffer encryptBuffer;
            SafeBuffer decryptBuffer;

            object->init ( Crypto::KEY_AES_256, Crypto::MODE_AES_OFB, true );
            object->encryptBufferToBufferAndCreateIV ( dataIn, encryptBuffer, key.GetData() );

            CPPUNIT_ASSERT ( 0 != encryptBuffer.GetSize() );
            CPPUNIT_ASSERT ( dataIn != encryptBuffer );

            object->decryptBufferToBufferAndGetIV ( encryptBuffer, decryptBuffer, key.GetData() );

            CPPUNIT_ASSERT ( dataIn == decryptBuffer );
        }

        void testRegretion_ChangeMode_Fail()
        {
            SafeBuffer key ( "abcdefghijklmnopabcdefghijklmnop" );

            SafeBuffer dataIn ( "Hello" );
            SafeBuffer encryptBuffer;
            SafeBuffer decryptBuffer;

            object->init ( Crypto::KEY_AES_256, Crypto::MODE_AES_OFB, true );
            object->encryptBufferToBufferAndCreateIV ( dataIn, encryptBuffer, key.GetData() );

            CPPUNIT_ASSERT ( 0 != encryptBuffer.GetSize() );
            CPPUNIT_ASSERT ( dataIn != encryptBuffer );

            object->init ( Crypto::KEY_AES_256, Crypto::MODE_AES_ECB, true );

            CPPUNIT_ASSERT_THROW (
                object->decryptBufferToBufferAndGetIV ( encryptBuffer, decryptBuffer, key.GetData() ),
                CryptoException );
        }

        void testRegretion_ChangeKeySize_Fail()
        {
            SafeBuffer key ( "abcdefghijklmnopabcdefghijklmnop" );

            SafeBuffer dataIn ( "Hello" );
            SafeBuffer encryptBuffer;
            SafeBuffer decryptBuffer;

            object->init ( Crypto::KEY_AES_128, Crypto::MODE_AES_CBC, true );
            object->encryptBufferToBufferAndCreateIV ( dataIn, encryptBuffer, key.GetData() );

            CPPUNIT_ASSERT ( 0 != encryptBuffer.GetSize() );
            CPPUNIT_ASSERT ( dataIn != encryptBuffer );

            object->init ( Crypto::KEY_AES_256, Crypto::MODE_AES_CBC, true );

            CPPUNIT_ASSERT_THROW (
                object->decryptBufferToBufferAndGetIV ( encryptBuffer, decryptBuffer, key.GetData() ),
                CryptoException );
        }

        void testEncryptOpenFileError()
        {
            SafeBuffer key ( "abcdefghijklmnopabcdefghijklmnop" );

            SafeBuffer dataIn ( "Hello" );
            SafeBuffer encryptBuffer;

            StoreMockFile file;
            File::getInstance ( &file );

            file.openFileValues.push_back ( StoreMockFile::OpenFileValues ( dataIn, false ) );

            object->init ( Crypto::KEY_AES_256, Crypto::MODE_AES_CBC, true );

            CPPUNIT_ASSERT_THROW (
                object->encryptFileToBufferAndCreateIV ( "file", encryptBuffer, key.GetData() ),
                CryptoException );
        }

        void testEncryptOpenFileOk()
        {
            SafeBuffer key ( "abcdefghijklmnopabcdefghijklmnop" );

            SafeBuffer dataIn ( "Hello" );
            SafeBuffer encryptBuffer;
            SafeBuffer decryptBuffer;

            StoreMockFile file;
            File::getInstance ( &file );

            file.openFileValues.push_back ( StoreMockFile::OpenFileValues ( dataIn, true ) );

            object->init ( Crypto::KEY_AES_256, Crypto::MODE_AES_CBC, true );
            object->encryptFileToBufferAndCreateIV ( "file", encryptBuffer, key.GetData() );

            CPPUNIT_ASSERT ( false == file.openFileListValues.empty() );
            CPPUNIT_ASSERT ( "file" == ( * ( file.openFileListValues.begin() ) ) );

            CPPUNIT_ASSERT ( dataIn != encryptBuffer );

            object->decryptBufferToBufferAndGetIV ( encryptBuffer, decryptBuffer, key.GetData() );

            CPPUNIT_ASSERT ( dataIn == decryptBuffer );
        }

        void testEncryptSaveFileError()
        {
            SafeBuffer key ( "abcdefghijklmnopabcdefghijklmnop" );

            SafeBuffer dataIn ( "Hello" );

            StoreMockFile file;
            File::getInstance ( &file );

            file.saveReturnValues.push_back ( false );

            object->init ( Crypto::KEY_AES_256, Crypto::MODE_AES_CBC, true );

            CPPUNIT_ASSERT_THROW (
                object->encryptBufferToFileAndCreateIV ( dataIn, "file", key.GetData() ),
                CryptoException );
        }

        void testEncryptSaveFileOk()
        {
            SafeBuffer key ( "abcdefghijklmnopabcdefghijklmnop" );

            SafeBuffer dataIn ( "Hello" );

            StoreMockFile file;
            File::getInstance ( &file );

            file.saveReturnValues.push_back ( true );

            object->init ( Crypto::KEY_AES_256, Crypto::MODE_AES_CBC, true );
            object->encryptBufferToFileAndCreateIV ( dataIn, "file", key.GetData() );

            CPPUNIT_ASSERT ( false == file.saveBufferValues.empty() );

            StoreMockFile::SafeBufferValues & value ( * ( file.saveBufferValues.begin() ) );

            SafeBuffer & encryptBuffer ( value.buffer );
            SafeBuffer decryptBuffer;

            CPPUNIT_ASSERT ( dataIn != encryptBuffer );
            CPPUNIT_ASSERT ( "file" == value.file );

            object->decryptBufferToBufferAndGetIV ( encryptBuffer, decryptBuffer, key.GetData() );

            CPPUNIT_ASSERT ( dataIn == decryptBuffer );
        }

        void testEncryptOpenErrorSaveFileError()
        {
            SafeBuffer key ( "abcdefghijklmnopabcdefghijklmnop" );

            SafeBuffer dataIn ( "Hello" );

            StoreMockFile file;
            File::getInstance ( &file );

            file.openFileValues.push_back ( StoreMockFile::OpenFileValues ( dataIn, false ) );

            object->init ( Crypto::KEY_AES_256, Crypto::MODE_AES_CBC, true );

            CPPUNIT_ASSERT_THROW (
                object->encryptFileToFileAndCreateIV ( "file1", "file2", key.GetData() ),
                CryptoException );
        }

        void testEncryptOpenOkSaveFileError()
        {
            SafeBuffer key ( "abcdefghijklmnopabcdefghijklmnop" );

            SafeBuffer dataIn ( "Hello" );

            StoreMockFile file;
            File::getInstance ( &file );

            file.openFileValues.push_back ( StoreMockFile::OpenFileValues ( dataIn, true ) );
            file.saveReturnValues.push_back ( false );

            object->init ( Crypto::KEY_AES_256, Crypto::MODE_AES_CBC, true );

            CPPUNIT_ASSERT_THROW (
                object->encryptFileToFileAndCreateIV ( "file1", "file2", key.GetData() ),
                CryptoException );
        }

        void testEncryptOpenOkSaveFileOk()
        {
            SafeBuffer key ( "abcdefghijklmnopabcdefghijklmnop" );

            SafeBuffer dataIn ( "Hello" );

            StoreMockFile file;
            File::getInstance ( &file );

            file.openFileValues.push_back ( StoreMockFile::OpenFileValues ( dataIn, true ) );
            file.saveReturnValues.push_back ( true );

            object->init ( Crypto::KEY_AES_256, Crypto::MODE_AES_CBC, true );
            object->encryptFileToFileAndCreateIV ( "file1", "file2", key.GetData() );

            CPPUNIT_ASSERT ( false == file.openFileListValues.empty() );
            CPPUNIT_ASSERT ( "file1" == ( * ( file.openFileListValues.begin() ) ) );

            CPPUNIT_ASSERT ( false == file.saveBufferValues.empty() );

            StoreMockFile::SafeBufferValues & value ( * ( file.saveBufferValues.begin() ) );

            SafeBuffer & encryptBuffer ( value.buffer );
            SafeBuffer decryptBuffer;

            CPPUNIT_ASSERT ( dataIn != encryptBuffer );
            CPPUNIT_ASSERT ( "file2" == value.file );

            object->decryptBufferToBufferAndGetIV ( encryptBuffer, decryptBuffer, key.GetData() );

            CPPUNIT_ASSERT ( dataIn == decryptBuffer );
        }

        void testDecryptOpenFileError()
        {
            SafeBuffer key ( "abcdefghijklmnopabcdefghijklmnop" );

            SafeBuffer dataIn ( "Hello" );
            SafeBuffer encryptBuffer;
            SafeBuffer decryptBuffer;

            object->init ( Crypto::KEY_AES_256, Crypto::MODE_AES_CBC, true );
            object->encryptBufferToBufferAndCreateIV ( dataIn, encryptBuffer, key.GetData() );

            CPPUNIT_ASSERT ( 0 != encryptBuffer.GetSize() );
            CPPUNIT_ASSERT ( dataIn != encryptBuffer );

            StoreMockFile file;
            File::getInstance ( &file );

            file.openFileValues.push_back ( StoreMockFile::OpenFileValues ( encryptBuffer, false ) );

            CPPUNIT_ASSERT_THROW (
                object->decryptFileToBufferAndGetIV ( "file", decryptBuffer, key.GetData() ),
                CryptoException );
        }

        void testDecryptOpenFileOk()
        {
            SafeBuffer key ( "abcdefghijklmnopabcdefghijklmnop" );

            SafeBuffer dataIn ( "Hello" );
            SafeBuffer encryptBuffer;
            SafeBuffer decryptBuffer;

            object->init ( Crypto::KEY_AES_256, Crypto::MODE_AES_CBC, true );
            object->encryptBufferToBufferAndCreateIV ( dataIn, encryptBuffer, key.GetData() );

            CPPUNIT_ASSERT ( 0 != encryptBuffer.GetSize() );
            CPPUNIT_ASSERT ( dataIn != encryptBuffer );

            StoreMockFile file;
            File::getInstance ( &file );

            file.openFileValues.push_back ( StoreMockFile::OpenFileValues ( encryptBuffer, true ) );

            object->decryptFileToBufferAndGetIV ( "file", decryptBuffer, key.GetData() );

            CPPUNIT_ASSERT ( false == file.openFileListValues.empty() );
            CPPUNIT_ASSERT ( "file" == ( * ( file.openFileListValues.begin() ) ) );

            CPPUNIT_ASSERT ( dataIn == decryptBuffer );
        }

        void testDecryptSaveFileError()
        {
            SafeBuffer key ( "abcdefghijklmnopabcdefghijklmnop" );

            SafeBuffer dataIn ( "Hello" );
            SafeBuffer encryptBuffer;
            SafeBuffer decryptBuffer;

            object->init ( Crypto::KEY_AES_256, Crypto::MODE_AES_CBC, true );
            object->encryptBufferToBufferAndCreateIV ( dataIn, encryptBuffer, key.GetData() );

            CPPUNIT_ASSERT ( 0 != encryptBuffer.GetSize() );
            CPPUNIT_ASSERT ( dataIn != encryptBuffer );

            StoreMockFile file;
            File::getInstance ( &file );

            file.saveReturnValues.push_back ( false );

            CPPUNIT_ASSERT_THROW (
                object->decryptBufferToFileAndGetIV ( encryptBuffer, "file", key.GetData() ),
                CryptoException );
        }

        void testDecryptSaveFileOk()
        {
            SafeBuffer key ( "abcdefghijklmnopabcdefghijklmnop" );

            SafeBuffer dataIn ( "Hello" );
            SafeBuffer encryptBuffer;

            object->init ( Crypto::KEY_AES_256, Crypto::MODE_AES_CBC, true );
            object->encryptBufferToBufferAndCreateIV ( dataIn, encryptBuffer, key.GetData() );

            CPPUNIT_ASSERT ( 0 != encryptBuffer.GetSize() );
            CPPUNIT_ASSERT ( dataIn != encryptBuffer );

            StoreMockFile file;
            File::getInstance ( &file );

            file.saveReturnValues.push_back ( true );

            object->decryptBufferToFileAndGetIV ( encryptBuffer, "file", key.GetData() ),

                   CPPUNIT_ASSERT ( false == file.saveBufferValues.empty() );

            StoreMockFile::SafeBufferValues & value ( * ( file.saveBufferValues.begin() ) );

            SafeBuffer & decryptBuffer ( value.buffer );

            CPPUNIT_ASSERT ( "file" == value.file );
            CPPUNIT_ASSERT ( dataIn == decryptBuffer );
        }

        void testDecryptOpenErrorSaveFileError()
        {
            SafeBuffer key ( "abcdefghijklmnopabcdefghijklmnop" );

            SafeBuffer dataIn ( "Hello" );

            object->init ( Crypto::KEY_AES_256, Crypto::MODE_AES_CBC, true );

            StoreMockFile file;
            File::getInstance ( &file );

            file.openFileValues.push_back ( StoreMockFile::OpenFileValues ( dataIn, false ) );

            CPPUNIT_ASSERT_THROW (
                object->decryptFileToFileAndGetIV ( "file1", "file2", key.GetData() ),
                CryptoException );
        }

        void testDecryptOpenOkSaveFileError()
        {
            SafeBuffer key ( "abcdefghijklmnopabcdefghijklmnop" );

            SafeBuffer dataIn ( "Hello" );
            SafeBuffer encryptBuffer;

            object->init ( Crypto::KEY_AES_256, Crypto::MODE_AES_CBC, true );
            object->encryptBufferToBufferAndCreateIV ( dataIn, encryptBuffer, key.GetData() );

            CPPUNIT_ASSERT ( 0 != encryptBuffer.GetSize() );
            CPPUNIT_ASSERT ( dataIn != encryptBuffer );

            StoreMockFile file;
            File::getInstance ( &file );

            file.openFileValues.push_back ( StoreMockFile::OpenFileValues ( encryptBuffer, true ) );
            file.saveReturnValues.push_back ( false );

            CPPUNIT_ASSERT_THROW (
                object->decryptFileToFileAndGetIV ( "file1", "file2", key.GetData() ),
                CryptoException );
        }

        void testDecryptOpenOkSaveFileOk()
        {
            SafeBuffer key ( "abcdefghijklmnopabcdefghijklmnop" );

            SafeBuffer dataIn ( "Hello" );
            SafeBuffer encryptBuffer;

            object->init ( Crypto::KEY_AES_256, Crypto::MODE_AES_CBC, true );
            object->encryptBufferToBufferAndCreateIV ( dataIn, encryptBuffer, key.GetData() );

            CPPUNIT_ASSERT ( 0 != encryptBuffer.GetSize() );
            CPPUNIT_ASSERT ( dataIn != encryptBuffer );

            StoreMockFile file;
            File::getInstance ( &file );

            file.openFileValues.push_back ( StoreMockFile::OpenFileValues ( encryptBuffer, true ) );
            file.saveReturnValues.push_back ( true );

            object->decryptFileToFileAndGetIV ( "file1", "file2", key.GetData() );

            CPPUNIT_ASSERT ( false == file.openFileListValues.empty() );
            CPPUNIT_ASSERT ( "file1" == ( * ( file.openFileListValues.begin() ) ) );

            CPPUNIT_ASSERT ( false == file.saveBufferValues.empty() );

            StoreMockFile::SafeBufferValues & value ( * ( file.saveBufferValues.begin() ) );

            SafeBuffer & decryptBuffer ( value.buffer );

            CPPUNIT_ASSERT ( "file2" == value.file );
            CPPUNIT_ASSERT ( dataIn == decryptBuffer );
        }

        CPPUNIT_TEST_SUITE ( CryptoClassTest );

        CPPUNIT_TEST ( testEncryptBufferNotInitClass );
        CPPUNIT_TEST ( testEncryptBufferInitClassKeyNull );
        CPPUNIT_TEST ( testEncryptDataNull );

        CPPUNIT_TEST ( testDecryptBufferNotInitClass );
        CPPUNIT_TEST ( testDecryptBufferInitClassKeyNull );
        CPPUNIT_TEST ( testDecryptDataNull );

        CPPUNIT_TEST ( testEncryptAndDecryptBufferError );
        CPPUNIT_TEST ( testDecryptDiferentKey );

        CPPUNIT_TEST ( testRegretion_128_CBC );
        CPPUNIT_TEST ( testRegretion_128_CFB1 );
        CPPUNIT_TEST ( testRegretion_128_CFB8 );
        CPPUNIT_TEST ( testRegretion_128_CFB128 );
        CPPUNIT_TEST ( testRegretion_128_ECB );
        CPPUNIT_TEST ( testRegretion_128_OFB );

        CPPUNIT_TEST ( testRegretion_192_CBC );
        CPPUNIT_TEST ( testRegretion_192_CFB1 );
        CPPUNIT_TEST ( testRegretion_192_CFB8 );
        CPPUNIT_TEST ( testRegretion_192_CFB128 );
        CPPUNIT_TEST ( testRegretion_192_ECB );
        CPPUNIT_TEST ( testRegretion_192_OFB );

        CPPUNIT_TEST ( testRegretion_256_CBC );
        CPPUNIT_TEST ( testRegretion_256_CFB1 );
        CPPUNIT_TEST ( testRegretion_256_CFB8 );
        CPPUNIT_TEST ( testRegretion_256_CFB128 );
        CPPUNIT_TEST ( testRegretion_256_ECB );
        CPPUNIT_TEST ( testRegretion_256_OFB );

        CPPUNIT_TEST ( testRegretion_ChangeMode_Fail );
        CPPUNIT_TEST ( testRegretion_ChangeKeySize_Fail );

        CPPUNIT_TEST ( testEncryptOpenFileError );
        CPPUNIT_TEST ( testEncryptOpenFileOk );
        CPPUNIT_TEST ( testEncryptSaveFileError );
        CPPUNIT_TEST ( testEncryptSaveFileOk );
        CPPUNIT_TEST ( testEncryptOpenErrorSaveFileError );
        CPPUNIT_TEST ( testEncryptOpenOkSaveFileError );
        CPPUNIT_TEST ( testEncryptOpenOkSaveFileOk );

        CPPUNIT_TEST ( testDecryptOpenFileError );
        CPPUNIT_TEST ( testDecryptOpenFileOk );
        CPPUNIT_TEST ( testDecryptSaveFileError );
        CPPUNIT_TEST ( testDecryptSaveFileOk );
        CPPUNIT_TEST ( testDecryptOpenErrorSaveFileError );
        CPPUNIT_TEST ( testDecryptOpenOkSaveFileError );
        CPPUNIT_TEST ( testDecryptOpenOkSaveFileOk );

        CPPUNIT_TEST_SUITE_END();
};

#endif /* ECUT_MAIN */

#endif /* TEST_CRYPTO_CLASS_TEST */
