/**
 * @file TableBoxSelectionWidget.hxx
 * @brief Header table box selection widget class
 * @date 14/11/2012
 * @author Yadickson Soto
 */

#ifndef TABLEBOXSELECTIONWIDGET_HXX_
#define TABLEBOXSELECTIONWIDGET_HXX_

#include <Widgets/TableCenterWidget.hxx>
#include <Widgets/BoxSelectionWidget.hxx>
#include <list>

namespace Smartmatic
{
    namespace GUI
    {
        namespace Widgets
        {

            /**
             * @class TableBoxSelectionWidget
             * @brief table box selection widget class
             * @date 14/11/2012
             * @author Yadickson Soto
             */
            class TableBoxSelectionWidget : public TableCenterWidget
            {
                public:

                    typedef sigc::signal<void, unsigned int, unsigned int> WarningMinSignal; ///< The warning minimum signal
                    typedef sigc::signal<void, unsigned int> WarningMaxSignal; ///< The warning maximum signal

                    /**
                     * @fn TableBoxSelectionWidget( bool multiple = false, bool homogeneous = true, bool verticalCenter = false );
                     * @brief Class constructor
                     * @date 14/11/2012
                     * @author Yadickson Soto
                     * @param  multiple multiple selected
                     * @param	homogeneous			column homogeneous.
                     * @param	verticalCenter	vertical center.
                     */
                    TableBoxSelectionWidget ( bool multiple = false, bool homogeneous = true, bool verticalCenter = false );
                    /**
                     * @fn virtual ~TableBoxSelectionWidget();
                     * @brief Class destructor
                     * @date 14/11/2012
                     * @author Yadickson Soto
                     */
                    virtual ~TableBoxSelectionWidget();
                    /**
                     * @fn	virtual void put( void * widget, int row, int column);
                     * @brief	method put new element in table.
                     * @author	Yadickson Soto
                     * @date	16/03/2012
                     * @param	widget	new widget.
                     * @param	row			row position.
                     * @param	column	column position.
                     */
                    virtual void put ( void * widget, int row, int column );
                    /**
                     * @fn void forceClick( unsigned int index );
                     * @brief Method force click widget
                     * @date 14/11/2012
                     * @author Yadickson Soto
                     * @param index index
                     */
                    void forceClick ( unsigned int index );
                    /**
                     * @fn int getCount();
                     * @brief Method getter count elements
                     * @date 14/11/2012
                     * @author Yadickson Soto
                     * @return count elements
                     */
                    int getCount();
                    /**
                     * @fn std::list <BoxSelectionWidget * > getSelected();
                     * @brief Gets selected elements
                     * @date 25/02/2014
                     * @author Yadickson Soto
                     * @return selected list elements
                     */
                    std::list <BoxSelectionWidget * > getSelected ();
                    /**
                     * @fn void clear ();
                     * @brief Method clear all selected elements
                     * @date 14/11/2012
                     * @author Yadickson Soto
                     */
                    void clear ();
                    /**
                     * @fn sigc::signal<void, std::list <BoxSelectionWidget * > & > getSignalClick();
                     * @brief Method getter signal click
                     * @date 14/11/2012
                     * @author Yadickson Soto
                     * @return signal
                     */
                    sigc::signal<void, std::list <BoxSelectionWidget * > & > getSignalClick();
                    /**
                     * @fn sigc::signal<void, BoxSelectionWidget * > getBoxSignalClick();
                     * @brief Gets box signal click
                     * @date 25/02/2014
                     * @author Yadickson Soto
                     * @return signal
                     */
                    sigc::signal<void, BoxSelectionWidget *> getBoxSignalClick();
                    /**
                     * @fn WarningMinSignal getMinSignal();
                     * @brief Gets minimum warning signal
                     * @date 25/02/2014
                     * @author Yadickson Soto
                     * @return signal
                     */
                    WarningMinSignal getMinSignal();
                    /**
                     * @fn WarningMaxSignal getMaxSignal();
                     * @brief Gets maximum warning signal
                     * @date 25/02/2014
                     * @author Yadickson Soto
                     * @return signal
                     */
                    WarningMaxSignal getMaxSignal();
                    /**
                     * @fn void setSelectionQuantitiesBoundaries( unsigned int minSelection , unsigned int maxSelection );
                     * @brief Sets min and max box selection widget
                     * @date 25/02/2014
                     * @author Yadickson Soto
                     * @param minSelection minimum box selection
                     * @param maxSelection maximum box selection
                     */
                    void setSelectionQuantitiesBoundaries ( unsigned int minSelection , unsigned int maxSelection );
                    /**
                     * @fn unsigned int getMin();
                     * @brief Gets the minimum number of selections allowed.
                     * @date 25/02/2014
                     * @author Yadickson Soto
                     * @return	The minimum number of selections allowed.
                     */
                    unsigned int getMin();
                    /**
                     * @fn unsigned int getMax();
                     * @brief Gets the maximum number of selections allowed.
                     * @date 25/02/2014
                     * @author Yadickson Soto
                     * @return	The maximum number of selections allowed
                     */
                    unsigned int getMax();

                protected:

                    /**
                     * @fn void onSignalClick ( BoxSelectionWidget * widget );
                     * @brief Method process signal
                     * @date 14/11/2012
                     * @author Yadickson Soto
                     */
                    void onSignalClick ( BoxSelectionWidget * widget );

                private:

                    bool multiple; ///< flag multiple selection
                    unsigned int minSelection; ///< minimum box selection
                    unsigned int maxSelection; ///< maximum box selection

                    sigc::signal<void, std::list <BoxSelectionWidget * > & > signalClick; ///< signal click
                    sigc::signal<void, BoxSelectionWidget * > boxSignalClick; ///< box signal click

                    WarningMinSignal minimumSignal; ///< warning minimum signal
                    WarningMaxSignal maximumSignal; ///< warning maximum signal

                    std::list < BoxSelectionWidget * > elementList; ///< element list
            };

        } /* namespace Widgets */
    } /* namespace GUI */
} /* namespace Smartmatic */
#endif /* TABLEBOXSELECTIONWIDGET_HXX_ */
