/**
 * @file	key-board-widget.h
 * @brief	Declares the key board widget class.
 */

/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */
/*
 * Key-Board
 * Copyright (C)  2010 <>
 *
 * Key-Board is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Key-Board is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef _KEY_BOARD_WIDGET_H_
#define _KEY_BOARD_WIDGET_H_

#include <gtkmm.h>
#include <string>
#include "key-board-widget-interface.h"
#include "smartmatic-button.h"
#include <Widgets/SmartmaticBasicTable.hxx>

namespace Smartmatic
{
    namespace Log
    {
        class ISMTTLog;
    }

    namespace GUI
    {
        namespace Widgets
        {
            enum SpecialKeys
            {
                enter = 47,
                shift = 48,
                upArrow = 49,
                backspace = 50,
                escape = 51,
                space = 52,
                leftArrow = 53,
                downArrow = 54,
                rightArrow = 55
            };

            /**
             * @class	Keyboard
             * @brief	Keyboard.
             * @author	Youssef.tourkmani
             * @date	25/10/2012
             */
            class Keyboard: public SmartmaticBasicTable
            {
                public:

                    /**
                     * @brief The detect signal.
                     */
                    typedef sigc::signal<void, std::string> DetectSignal;

                    /**
                     * @brief The delete signal.
                     */
                    typedef sigc::signal<void> DeleteSignal;

                    /**
                     * @brief The enter signal.
                     */
                    typedef sigc::signal<void> EnterSignal;

                    /**
                     * @brief The up signal.
                     */
                    typedef sigc::signal<void> UpSignal;

                    /**
                     * @brief The down signal.
                     */
                    typedef sigc::signal<void> DownSignal;

                    /**
                     * @brief The left signal.
                     */
                    typedef sigc::signal<void> LeftSignal;

                    /**
                     * @brief The right signal.
                     */
                    typedef sigc::signal<void> RightSignal;

                public:

                    /**
                     * @fn	Keyboard::Keyboard();
                     * @brief	Default constructor.
                     * @author	Luiggi.mendez
                     * @date	27/04/2011
                     * @param   mainKeys    Keyboard main layer.
                     * @param   secondKeys  Keyboard second layer.
                     */
                    explicit Keyboard(const std::vector<std::string>& mainKeys, const std::vector<std::string>& secondKeys);

                    /**
                     * @fn	virtual Keyboard::~Keyboard();
                     * @brief	Finaliser.
                     * @author	Luiggi.mendez
                     * @date	27/04/2011
                     */
                    virtual ~Keyboard();

                public:

                    /**
                     * @fn	void setSensitiveButton ( Smartmatic::GUI::Widgets::KeysKeyboard key, bool sensitive = false );
                     * @brief	set sensitive to button
                     * @author	Yadiockson Soto
                     * @date	06/09/2013
                     * @param	key	The key value from button
                     * @param   sensitive new value
                     */
                    void setSensitiveButton(const SpecialKeys& key, const bool sensitive = false);

                    /**
                     * @fn DetectSignal getDetectSignal();
                     * @brief Gets the detect signal.
                     * @author eduardo.fernandez
                     * @date   05/09/2018
                     * @return The detect signal.
                     */
                    DetectSignal getDetectSignal();

                    /**
                     * @fn DeleteSignal getDeleteSignal();
                     * @brief Gets the delete signal.
                     * @author eduardo.fernandez
                     * @date   05/09/2018
                     * @return The delete signal.
                     */
                    DeleteSignal getDeleteSignal();

                    /**
                     * @fn EnterSignal getEnterSignal();
                     * @brief Gets the enter signal.
                     * @author eduardo.fernandez
                     * @date   05/09/2018
                     * @return The enter signal.
                     */
                    EnterSignal getEnterSignal();

                    /**
                     * @fn UpSignal getUpSignal();
                     * @brief Gets the up signal.
                     * @author eduardo.fernandez
                     * @date   05/09/2018
                     * @return The up signal.
                     */
                    UpSignal getUpSignal();

                    /**
                     * @fn DownSignal getDownSignal();
                     * @brief Gets the down signal.
                     * @author eduardo.fernandez
                     * @date   05/09/2018
                     * @return The down signal.
                     */
                    DownSignal getDownSignal();

                    /**
                     * @fn LeftSignal getLeftSignal();
                     * @brief Gets the left signal.
                     * @author eduardo.fernandez
                     * @date   05/09/2018
                     * @return The left signal.
                     */
                    LeftSignal getLeftSignal();

                    /**
                     * @fn RightSignal getRightSignal();
                     * @brief Gets the right signal.
                     * @author eduardo.fernandez
                     * @date   05/09/2018
                     * @return The right signal.
                     */
                    RightSignal getRightSignal();

                private:

                    /**
                     * @fn void setKeysButtons(const std::vector<std::string> keys);
                     * @brief Sets the keys buttons.
                     * @author     eduardo.fernandez
                     * @date       05/09/2018
                     * @param keys A vector containing the buttons keys.
                     */
                    void setKeysButtons(const std::vector<std::string>& keys);

                    /**
                     * @fn void setSpecialKeysButtons();
                     * @brief Sets the special keys buttons.
                     * @author     eduardo.fernandez
                     * @date       05/09/2018
                     */
                    void setSpecialKeysButtons();

                    /**
                     * @fn void setSpecialKeysLabels();
                     * @brief Sets the special keys labels.
                     * @author     eduardo.fernandez
                     * @date       05/09/2018
                     */
                    void setSpecialKeysLabels();

                    /**
                     * @fn void setMainKeysActions();
                     * @brief Sets the main keys actions.
                     * @author     eduardo.fernandez
                     * @date       05/09/2018
                     */
                    void setMainKeysActions();

                    /**
                     * @fn void shandleSpecialKeyAction(unsigned int key);
                     * @brief Handles a special key action.
                     * @author     eduardo.fernandez
                     * @date       05/09/2018
                     * @param      key The special key to be handled.
                     */
                    void handleSpecialKeyAction(const size_t key);

                    /**
                     * @fn void onButtonClicked(unsigned int index);
                     * @brief Occurs when a key is pressed.
                     * @author     eduardo.fernandez
                     * @date       05/09/2018
                     * @param      identifier Identifier of the key pressed.
                     */
                    void onButtonClicked(const size_t identifier);

                    /**
                     * @fn void shiftKeys();
                     * @brief Shifts current keys.
                     * @author     eduardo.fernandez
                     * @date       05/09/2018
                     */
                    void shiftKeys();

                private:

                    static Smartmatic::Log::ISMTTLog * m_logger;
                    Glib::RefPtr<Gtk::Builder>         m_builder;
                    SmartmaticBasicTable *             m_keyBoardTable;
                    SmartmaticButton *                 m_buttonArray[56];
                    bool                               m_isShifted;
                    std::vector<std::string>           m_keysIdentifiers;
                    std::vector<std::string>           m_mainKeys;
                    std::vector<std::string>           m_secondKeys;
                    DetectSignal                       m_signalDetect;
                    DeleteSignal                       m_signalDelete;
                    EnterSignal                        m_signalEnter;
                    UpSignal                           m_signalUp;
                    DownSignal                         m_signalDown;
                    LeftSignal                         m_signalLeft;
                    RightSignal                        m_signalRight;
            };
        }
    }
}

#endif // _KEY_BOARD_WIDGET_H_
