/**
 * @file	keypad-widget.cc
 * @brief	keypad widget class.
 */

/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */
/*
 * MachineApplication
 * Copyright (C)  2010 <>
 *
 * MachineApplication is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * MachineApplication is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "keypad-widget.h"
#include "Configuration/MachineModelEnum.hxx"
#include "Configuration/s-m-t-t-g-u-i-conf-manager.h"
#include <Log/ISMTTLog.h>
#include <Log/SMTTLogManager.h>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#include <libintl.h>
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif
#define N_(String) (String) ///<macro for gettext not translate

#define BIG_IMAGES_SUFFIX "_60x60.png"
#define SMALL_IMAGES_SUFFIX "_30x30.png"

using namespace Smartmatic::Configuration;
using namespace Smartmatic::GUI::Configuration;
using namespace Smartmatic::Environment;
using namespace Smartmatic::GUI::Widgets;
using namespace Smartmatic::System::Sounds;
using namespace Smartmatic::Log;

ISMTTLog * KeypadWidget::logger = SMTTLogManager::GetLogger ( "Smartmatic.GUI.Widgets.KeypadWidget", GETTEXT_PACKAGE );

KeypadWidget::KeypadWidget ( KeypadEnum type, bool sound, int keypadTableMaxHeight, int keypadButtonsMaxWidth, int keypadButtonsMaxHeight, bool useSmallImages )
    : SmartmaticBasicTable ()
{
    keyPadTable = NULL;
    SmartmaticButton test;
    //Load the Glade file
    Glib::RefPtr<Gtk::Builder> builder;
    numericSound = SMTTGUIConfigurationManager::getInstance()->getNumericKeypadSound();
    cancelSound = SMTTGUIConfigurationManager::getInstance()->getCancelKeypadSound();
    acceptSound = SMTTGUIConfigurationManager::getInstance()->getAcceptKeypadSound();

    try
    {
        std::string path = ResourcePathProvider::getInstance()->GetResourcePath ( PACKAGE_NAME, "/ui/keypad.glade" );
        builder = Gtk::Builder::create_from_file ( path );
    }
    catch ( const Glib::FileError & ex )
    {
        logger->Debug ( ex.what().c_str() );
    }

    for ( int i = 0; i < numberKeys; i++ )
    {
        button[i] = NULL;
    }

    switch ( type )
    {
        default:
        case T9:
        case Numeric:
            builder->get_widget_derived ( "Numeric", keyPadTable );
            builder->get_widget_derived ( "buttonZeroNumeric", button[kZero] );
            builder->get_widget_derived ( "buttonOneNumeric", button[kOne] );
            builder->get_widget_derived ( "buttonTwoNumeric", button[kTwo] );
            builder->get_widget_derived ( "buttonThreeNumeric", button[kThree] );
            builder->get_widget_derived ( "buttonFourNumeric", button[kFour] );
            builder->get_widget_derived ( "buttonFiveNumeric", button[kFive] );
            builder->get_widget_derived ( "buttonSixNumeric", button[kSix] );
            builder->get_widget_derived ( "buttonSevenNumeric", button[kSeven] );
            builder->get_widget_derived ( "buttonEightNumeric", button[kEight] );
            builder->get_widget_derived ( "buttonNineNumeric", button[kNine] );

            if ( type == Numeric )
            {
                SetNormalImages ( true, useSmallImages );
                SubscribeNormalSignals ( true );
            }
            else if ( type == T9 )
            {
                SetNormalImages ( false, useSmallImages );
                SubscribeNormalSignals ( false );
            }

            AllButtonImages();
            break;

        case Complete:
            builder->get_widget_derived ( "Complete", keyPadTable );
            builder->get_widget_derived ( "buttonZeroComplete", button[kZero] );
            builder->get_widget_derived ( "buttonOneComplete", button[kOne] );
            builder->get_widget_derived ( "buttonTwoComplete", button[kTwo] );
            builder->get_widget_derived ( "buttonThreeComplete", button[kThree] );
            builder->get_widget_derived ( "buttonFourComplete", button[kFour] );
            builder->get_widget_derived ( "buttonFiveComplete", button[kFive] );
            builder->get_widget_derived ( "buttonSixComplete", button[kSix] );
            builder->get_widget_derived ( "buttonSevenComplete", button[kSeven] );
            builder->get_widget_derived ( "buttonEightComplete", button[kEight] );
            builder->get_widget_derived ( "buttonNineComplete", button[kNine] );
            builder->get_widget_derived ( "buttonBlankComplete", button[kBranco] );
            builder->get_widget_derived ( "buttonCorrectComplete", button[kCorrige] );
            builder->get_widget_derived ( "buttonConfirmComplete", button[kConfirma] );
            SetNormalImages ( true, useSmallImages );
            AllButtonImages();
            SubscribeNormalSignals ( true );
            break;

        case CorrigeConfirma:
            builder->get_widget_derived ( "CorrigeConfirma", keyPadTable );
            builder->get_widget_derived ( "buttonZeroCorrigeConfirma", button[kZero] );
            builder->get_widget_derived ( "buttonOneCorrigeConfirma", button[kOne] );
            builder->get_widget_derived ( "buttonTwoCorrigeConfirma", button[kTwo] );
            builder->get_widget_derived ( "buttonThreeCorrigeConfirma", button[kThree] );
            builder->get_widget_derived ( "buttonFourCorrigeConfirma", button[kFour] );
            builder->get_widget_derived ( "buttonFiveCorrigeConfirma", button[kFive] );
            builder->get_widget_derived ( "buttonSixCorrigeConfirma", button[kSix] );
            builder->get_widget_derived ( "buttonSevenCorrigeConfirma", button[kSeven] );
            builder->get_widget_derived ( "buttonEightCorrigeConfirma", button[kEight] );
            builder->get_widget_derived ( "buttonNineCorrigeConfirma", button[kNine] );
            builder->get_widget_derived ( "buttonCorrigeCorrigeConfirma", button[kCorrige] );
            builder->get_widget_derived ( "buttonConfirmaCorrigeConfirma", button[kConfirma] );
            SetNormalImages ( true, useSmallImages );
            AllButtonImages();
            SubscribeNormalSignals ( true );
            break;
    }

    int buttonWidth = -1;
    int buttonHeight = -1;

    if ( SMTTGUIConfigurationManager::getInstance()->getSmartmaticGUIMachineModelConfiguration().getId() == EnumToString( PM_VIU805 ) ||
        SMTTGUIConfigurationManager::getInstance()->getSmartmaticGUIMachineModelConfiguration().getId() == EnumToString( VM_SAES3370 ) )
    {
        keyPadTable->set_size_request( -1, keypadTableMaxHeight );
        buttonWidth = keypadButtonsMaxWidth;
        buttonHeight = keypadButtonsMaxHeight;
    }

    button[kZero]->set_name ( "key_pad_zero" );
    button[kOne]->set_name ( "key_pad_one" );
    button[kTwo]->set_name ( "key_pad_two" );
    button[kThree]->set_name ( "key_pad_three" );
    button[kFour]->set_name ( "key_pad_four" );
    button[kFive]->set_name ( "key_pad_five" );
    button[kSix]->set_name ( "key_pad_six" );
    button[kSeven]->set_name ( "key_pad_seven" );
    button[kEight]->set_name ( "key_pad_eight" );
    button[kNine]->set_name ( "key_pad_nine" );
    button[kCorrige]->set_name ( "key_pad_corrige" );
    button[kConfirma]->set_name ( "key_pad_confirma" );

    //Set buttons height
    button[kZero]->set_size_request( buttonWidth, buttonHeight );
    button[kOne]->set_size_request( buttonWidth, buttonHeight );
    button[kTwo]->set_size_request( buttonWidth, buttonHeight );
    button[kThree]->set_size_request( buttonWidth, buttonHeight );
    button[kFour]->set_size_request( buttonWidth, buttonHeight );;
    button[kFive]->set_size_request( buttonWidth, buttonHeight );
    button[kSix]->set_size_request( buttonWidth, buttonHeight );
    button[kSeven]->set_size_request( buttonWidth, buttonHeight );
    button[kEight]->set_size_request( buttonWidth, buttonHeight );
    button[kNine]->set_size_request( buttonWidth, buttonHeight );
    button[kCorrige]->set_size_request( buttonWidth, buttonHeight );
    button[kConfirma]->set_size_request( buttonWidth, buttonHeight );

    //Set special Sound type
    button[kZero]->SetSound ( numericSound );
    button[kOne]->SetSound ( numericSound );
    button[kTwo]->SetSound ( numericSound );
    button[kThree]->SetSound ( numericSound );
    button[kFour]->SetSound ( numericSound );
    button[kFive]->SetSound ( numericSound );
    button[kSix]->SetSound ( numericSound );
    button[kSeven]->SetSound ( numericSound );
    button[kEight]->SetSound ( numericSound );
    button[kNine]->SetSound ( numericSound );
    button[kCorrige]->SetSound ( cancelSound );
    button[kConfirma]->SetSound ( acceptSound );
    SetSound ( sound );
    //show childrens
    add ( *keyPadTable );
    show_all_children();
    //Initialization Function Pointer
    this->button_clicked = NULL;
    keypadinterface = NULL;
    //T9
    T9_strings[kOne] = ".,?1";
    T9_strings[kTwo] = "abc2";
    T9_strings[kThree] = "def3";
    T9_strings[kFour] = "ghi4";
    T9_strings[kFive] = "jkl5";
    T9_strings[kSix] = "mno6";
    T9_strings[kSeven] = "pqrs7";
    T9_strings[kEight] = "tuv8";
    T9_strings[kNine] = "wxyz9";
    T9_strings[kZero] = " 0";
    T9_strings[kBranco] = "*";
    T9_strings[kCorrige] = "#@";
    timesPressed = 0;
    keyLastPressed = kZero;
    timeElapsed = true;
    //Start TIMER
    slot = sigc::bind ( sigc::mem_fun ( *this, &KeypadWidget::OnTimerTimeout ), 0 );

    setAccessibleName ( "KeypadContainer" );
    keyPadTable->setAccessibleName ( "Keypad" );
    button[kZero]->setAccessibleName ( "Keypad0" );
    button[kOne]->setAccessibleName ( "Keypad1" );
    button[kTwo]->setAccessibleName ( "Keypad2" );
    button[kThree]->setAccessibleName ( "Keypad3" );
    button[kFour]->setAccessibleName ( "Keypad4" );
    button[kFive]->setAccessibleName ( "Keypad5" );
    button[kSix]->setAccessibleName ( "Keypad6" );
    button[kSeven]->setAccessibleName ( "Keypad7" );
    button[kEight]->setAccessibleName ( "Keypad8" );
    button[kNine]->setAccessibleName ( "Keypad9" );
    button[kCorrige]->setAccessibleName ( "KeypadClear" );
    button[kConfirma]->setAccessibleName ( "KeypadOk" );
}

KeypadWidget::~KeypadWidget()
{
    delete keyPadTable;
    delete button[kBranco];
    delete button[kConfirma];
    delete button[kCorrige];
    delete button[kEight];
    delete button[kFive];
    delete button[kFour];
    delete button[kNine];
    delete button[kOne];
    delete button[kSeven];
    delete button[kSix];
    delete button[kThree];
    delete button[kTwo];
    delete button[kZero];
}

//Signal Handlers
void KeypadWidget::on_button_numbered ( Keys data )
{
    if ( this->button_clicked != NULL )
    {
        this->button_clicked ( data );
    }

    if ( keypadinterface != NULL )
    {
        keypadinterface->OnKeyPress ( data );
    }
}

void KeypadWidget::on_button_numbered_T9 ( Keys data )
{
    if ( keypadinterface != NULL )
    {
        char c;

        if ( data == keyLastPressed && !timeElapsed )
        {
            keypadinterface->OnKeyPressT9 ( ( char ) 8 ); //backspace
            int size = T9_strings[data].size();

            if ( size > timesPressed )
            {
                c = T9_strings[data][timesPressed];
            }
            else
            {
                timesPressed = 0;
                c = T9_strings[data][0];
            }

            timesPressed ++;
            keypadinterface->OnKeyPressT9 ( c );
            timeElapsed = false;
        }
        else
        {
            c = T9_strings[data][0];
            keyLastPressed = data;
            timesPressed = 1;
            keypadinterface->OnKeyPressT9 ( c );
        }

        static int timeout = 1000;
        connector.disconnect();
        connector = Glib::signal_timeout().connect ( slot, timeout );
        timeElapsed = false;
    }
}

bool KeypadWidget::OnTimerTimeout ( int timer_number )
{
    timeElapsed = true;
    return false;
}

void KeypadWidget::AllButtonImages()
{
    for ( int i = 0; i < numberKeys; i++ )
        if ( button[i] != NULL )
        {
            button[i]->WidgetsPosition ( 0, 5, 0 );
        }
}

void KeypadWidget::SetNormalImages ( bool normal, bool useSmallImages )
{
    std::string imageFileNameSuffix = BIG_IMAGES_SUFFIX;

    if ( useSmallImages )
        imageFileNameSuffix = SMALL_IMAGES_SUFFIX;

    if ( button[kZero] != NULL )
    {
        button[kZero]->SetImage ( ResourcePathProvider::getInstance()->GetResourcePath ( PACKAGE_NAME, "ui/0" + imageFileNameSuffix ) );
    }

    if ( button[kOne] != NULL )
    {
        button[kOne]->SetImage ( ResourcePathProvider::getInstance()->GetResourcePath ( PACKAGE_NAME, "ui/1" + imageFileNameSuffix ) );
    }

    if ( button[kTwo] != NULL )
    {
        button[kTwo]->SetImage ( ResourcePathProvider::getInstance()->GetResourcePath ( PACKAGE_NAME, "ui/2" + imageFileNameSuffix ) );
    }

    if ( button[kThree] != NULL )
    {
        button[kThree]->SetImage ( ResourcePathProvider::getInstance()->GetResourcePath ( PACKAGE_NAME, "ui/3" + imageFileNameSuffix ) );
    }

    if ( button[kFour] != NULL )
    {
        button[kFour]->SetImage ( ResourcePathProvider::getInstance()->GetResourcePath ( PACKAGE_NAME, "ui/4" + imageFileNameSuffix ) );
    }

    if ( button[kFive] != NULL )
    {
        button[kFive]->SetImage ( ResourcePathProvider::getInstance()->GetResourcePath ( PACKAGE_NAME, "ui/5" + imageFileNameSuffix ) );
    }

    if ( button[kSix] != NULL )
    {
        button[kSix]->SetImage ( ResourcePathProvider::getInstance()->GetResourcePath ( PACKAGE_NAME, "ui/6" + imageFileNameSuffix ) );
    }

    if ( button[kSeven] != NULL )
    {
        button[kSeven]->SetImage ( ResourcePathProvider::getInstance()->GetResourcePath ( PACKAGE_NAME, "ui/7" + imageFileNameSuffix ) );
    }

    if ( button[kEight] != NULL )
    {
        button[kEight]->SetImage ( ResourcePathProvider::getInstance()->GetResourcePath ( PACKAGE_NAME, "ui/8" + imageFileNameSuffix ) );
    }

    if ( button[kNine] != NULL )
    {
        button[kNine]->SetImage ( ResourcePathProvider::getInstance()->GetResourcePath ( PACKAGE_NAME, "ui/9" + imageFileNameSuffix ) );
    }

    if ( button[kBranco] != NULL )
    {
        button[kBranco]->SetImage ( ResourcePathProvider::getInstance()->GetResourcePath ( PACKAGE_NAME, "ui/branco_02_60px.png" ) );
    }

    if ( button[kCorrige] != NULL )
    {
        button[kCorrige]->SetImage ( ResourcePathProvider::getInstance()->GetResourcePath ( PACKAGE_NAME, "ui/corrige_02_60px.png" ) );
    }

    if ( button[kConfirma] != NULL )
    {
        button[kConfirma]->SetImage ( ResourcePathProvider::getInstance()->GetResourcePath ( PACKAGE_NAME, "ui/confirma_60.png" ) );
    }
}

void KeypadWidget::SubscribeNormalSignals ( bool normal )
{
    if ( normal )
    {
        if ( button[kZero] != NULL )
        {
            button[kZero]->signal_clicked().connect ( sigc::bind<Keys> ( sigc::mem_fun ( *this, &KeypadWidget::on_button_numbered ), kZero ) );
        }

        if ( button[kOne] != NULL )
        {
            button[kOne]->signal_clicked().connect ( sigc::bind<Keys> ( sigc::mem_fun ( *this, &KeypadWidget::on_button_numbered ), kOne ) );
        }

        if ( button[kTwo] != NULL )
        {
            button[kTwo]->signal_clicked().connect ( sigc::bind<Keys> ( sigc::mem_fun ( *this, &KeypadWidget::on_button_numbered ), kTwo ) );
        }

        if ( button[kThree] != NULL )
        {
            button[kThree]->signal_clicked().connect ( sigc::bind<Keys> ( sigc::mem_fun ( *this, &KeypadWidget::on_button_numbered ), kThree ) );
        }

        if ( button[kFour] != NULL )
        {
            button[kFour]->signal_clicked().connect ( sigc::bind<Keys> ( sigc::mem_fun ( *this, &KeypadWidget::on_button_numbered ), kFour ) );
        }

        if ( button[kFive] != NULL )
        {
            button[kFive]->signal_clicked().connect ( sigc::bind<Keys> ( sigc::mem_fun ( *this, &KeypadWidget::on_button_numbered ), kFive ) );
        }

        if ( button[kSix] != NULL )
        {
            button[kSix]->signal_clicked().connect ( sigc::bind<Keys> ( sigc::mem_fun ( *this, &KeypadWidget::on_button_numbered ), kSix ) );
        }

        if ( button[kSeven] != NULL )
        {
            button[kSeven]->signal_clicked().connect ( sigc::bind<Keys> ( sigc::mem_fun ( *this, &KeypadWidget::on_button_numbered ), kSeven ) );
        }

        if ( button[kEight] != NULL )
        {
            button[kEight]->signal_clicked().connect ( sigc::bind<Keys> ( sigc::mem_fun ( *this, &KeypadWidget::on_button_numbered ), kEight ) );
        }

        if ( button[kNine] != NULL )
        {
            button[kNine]->signal_clicked().connect ( sigc::bind<Keys> ( sigc::mem_fun ( *this, &KeypadWidget::on_button_numbered ), kNine ) );
        }

        if ( button[kBranco] != NULL )
        {
            button[kBranco]->signal_clicked().connect ( sigc::bind<Keys> ( sigc::mem_fun ( *this, &KeypadWidget::on_button_numbered ), kBranco ) );
        }

        if ( button[kCorrige] != NULL )
        {
            button[kCorrige]->signal_clicked().connect ( sigc::bind<Keys> ( sigc::mem_fun ( *this, &KeypadWidget::on_button_numbered ), kCorrige ) );
        }

        if ( button[kConfirma] != NULL )
        {
            button[kConfirma]->signal_clicked().connect ( sigc::bind<Keys> ( sigc::mem_fun ( *this, &KeypadWidget::on_button_numbered ), kConfirma ) );
        }
    }
    else
    {
        if ( button[kZero] != NULL )
        {
            button[kZero]->signal_clicked().connect ( sigc::bind<Keys> ( sigc::mem_fun ( *this, &KeypadWidget::on_button_numbered_T9 ), kZero ) );
        }

        if ( button[kOne] != NULL )
        {
            button[kOne]->signal_clicked().connect ( sigc::bind<Keys> ( sigc::mem_fun ( *this, &KeypadWidget::on_button_numbered_T9 ), kOne ) );
        }

        if ( button[kTwo] != NULL )
        {
            button[kTwo]->signal_clicked().connect ( sigc::bind<Keys> ( sigc::mem_fun ( *this, &KeypadWidget::on_button_numbered_T9 ), kTwo ) );
        }

        if ( button[kThree] != NULL )
        {
            button[kThree]->signal_clicked().connect ( sigc::bind<Keys> ( sigc::mem_fun ( *this, &KeypadWidget::on_button_numbered_T9 ), kThree ) );
        }

        if ( button[kFour] != NULL )
        {
            button[kFour]->signal_clicked().connect ( sigc::bind<Keys> ( sigc::mem_fun ( *this, &KeypadWidget::on_button_numbered_T9 ), kFour ) );
        }

        if ( button[kFive] != NULL )
        {
            button[kFive]->signal_clicked().connect ( sigc::bind<Keys> ( sigc::mem_fun ( *this, &KeypadWidget::on_button_numbered_T9 ), kFive ) );
        }

        if ( button[kSix] != NULL )
        {
            button[kSix]->signal_clicked().connect ( sigc::bind<Keys> ( sigc::mem_fun ( *this, &KeypadWidget::on_button_numbered_T9 ), kSix ) );
        }

        if ( button[kSeven] != NULL )
        {
            button[kSeven]->signal_clicked().connect ( sigc::bind<Keys> ( sigc::mem_fun ( *this, &KeypadWidget::on_button_numbered_T9 ), kSeven ) );
        }

        if ( button[kEight] != NULL )
        {
            button[kEight]->signal_clicked().connect ( sigc::bind<Keys> ( sigc::mem_fun ( *this, &KeypadWidget::on_button_numbered_T9 ), kEight ) );
        }

        if ( button[kNine] != NULL )
        {
            button[kNine]->signal_clicked().connect ( sigc::bind<Keys> ( sigc::mem_fun ( *this, &KeypadWidget::on_button_numbered_T9 ), kNine ) );
        }

        if ( button[kBranco] != NULL )
        {
            button[kBranco]->signal_clicked().connect ( sigc::bind<Keys> ( sigc::mem_fun ( *this, &KeypadWidget::on_button_numbered_T9 ), kBranco ) );
        }

        if ( button[kCorrige] != NULL )
        {
            button[kCorrige]->signal_clicked().connect ( sigc::bind<Keys> ( sigc::mem_fun ( *this, &KeypadWidget::on_button_numbered_T9 ), kCorrige ) );
        }

        if ( button[kConfirma] != NULL )
        {
            button[kConfirma]->signal_clicked().connect ( sigc::bind<Keys> ( sigc::mem_fun ( *this, &KeypadWidget::on_button_numbered_T9 ), kConfirma ) );
        }
    }
}

void KeypadWidget::SetSound ( bool sound )
{
    if ( sound )
    {
        if ( button[kZero] != NULL )
        {
            button[kZero]->SetSoundOnClick();
        }

        if ( button[kOne] != NULL )
        {
            button[kOne]->SetSoundOnClick();
        }

        if ( button[kTwo] != NULL )
        {
            button[kTwo]->SetSoundOnClick();
        }

        if ( button[kThree] != NULL )
        {
            button[kThree]->SetSoundOnClick();
        }

        if ( button[kFour] != NULL )
        {
            button[kFour]->SetSoundOnClick();
        }

        if ( button[kFive] != NULL )
        {
            button[kFive]->SetSoundOnClick();
        }

        if ( button[kSix] != NULL )
        {
            button[kSix]->SetSoundOnClick();
        }

        if ( button[kSeven] != NULL )
        {
            button[kSeven]->SetSoundOnClick();
        }

        if ( button[kEight] != NULL )
        {
            button[kEight]->SetSoundOnClick();
        }

        if ( button[kNine] != NULL )
        {
            button[kNine]->SetSoundOnClick();
        }

        if ( button[kBranco] != NULL )
        {
            button[kBranco]->SetSoundOnClick();
        }

        if ( button[kCorrige] != NULL )
        {
            button[kCorrige]->SetSoundOnClick();
        }

        if ( button[kConfirma] != NULL )
        {
            button[kConfirma]->SetSoundOnClick();
        }
    }
}

void KeypadWidget::buttonSetNames ( const std::string& prefixName )
{
    std::stringstream out;

    for ( int i = 0; i < numberKeys; i++ )
    {
        out << i;
        button[i]->set_name ( prefixName + "_" + out.str() );
    }
}

void KeypadWidget::setSensitiveButton ( Smartmatic::GUI::Widgets::Keys key, bool sensitive )
{
    button [ key ]->set_sensitive ( sensitive );
}
