/**
 * @file	wizard-widget-manager.h
 * @brief	Declares the wizard widget manager class.
 */
/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */
/*
 * smartmaticgui
 * Copyright (C) Juan Delgado 2011 <juan.delgado@smartmatic.com>
 *
 * smartmaticgui is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * smartmaticgui is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef _WIZARD_WIDGET_MANAGER_H_
#define _WIZARD_WIDGET_MANAGER_H_

#include <gtkmm.h>
#include <Widgets/wizard-widget-base.h>

namespace Smartmatic
{
    namespace GUI
    {
        namespace Widgets
        {
            /**
             * @class	WizardWidgetManager
             * @brief	Manager for wizard widgets.
             * @author	Youssef.tourkmani
             * @date	25/10/2012
             */
            class WizardWidgetManager: public StepWidget
            {
                public:

                    /**
                     * @fn	WizardWidgetManager::WizardWidgetManager();
                     * @brief	Constructor.
                     * @author	Yadickson Soto
                     * @date	09/09/2013
                     */
                    WizardWidgetManager ();

                    /**
                     * @fn	WizardWidgetManager::WizardWidgetManager( const std::vector<WizardWidgetBase*>& widgetsVector );
                     * @brief	Constructor.
                     * @author	Luiggi.mendez
                     * @date	27/04/2011
                     * @param	widgetsVector	If non-null, the widgets vector.
                     */
                    explicit WizardWidgetManager ( const std::vector<WizardWidgetBase *>& widgetsVector );

                    /**
                     * @fn	virtual WizardWidgetManager::~WizardWidgetManager();
                     * @brief	Finaliser.
                     * @author	Luiggi.mendez
                     * @date	27/04/2011
                     */

                    virtual ~WizardWidgetManager();

                    /**
                     * @fn	void WizardWidgetManager::Start();
                     * @brief	Starts this object.
                     * @author	Youssef.tourkmani
                     * @date	25/10/2012
                     */
                    void Start();

                    /**
                     * @fn	void Start( const std::vector<WizardWidgetBase *>& widgetsVector );
                     * @brief	Starts this object.
                     * @author	Yadickson Soto
                     * @date	09/09/2013
                     * @param widgetsVector the widgets vector
                     */
                    void Start ( const std::vector<WizardWidgetBase *>& widgetsVector );

                    /**
                     * @brief  Stops the execution of the wizard.
                     * @author manuel.arguelles
                     * @date   20/6/2022
                     */
                    void Stop();

                    /**
                     * @typedef	sigc::signal<void> type_void_signal
                     * @brief	Defines an alias representing the type void signal. .
                     */
                    typedef sigc::signal<void> type_void_signal;

                    /**
                     * @fn	type_void_signal WizardWidgetManager::on_finish_signal_emit();
                     * @brief	Executes the finish signal emit action.
                     * @author	Luiggi.mendez
                     * @date	27/04/2011
                     * @return	.
                     */

                    type_void_signal on_finish_signal_emit();

                    /**
                     * @fn	type_void_signal WizardWidgetManager::on_transition_signal_emit();
                     * @brief	Executes the transition signal emit action.
                     * @author	Luiggi.mendez
                     * @date	27/04/2011
                     * @return	.
                     */

                    type_void_signal on_transition_signal_emit();

                    /**
                     * @fn     type_void_signal WizardWidgetManager::on_error_signal_emit();
                     * @brief  Returns the error signal handler.
                     * @author manuel.arguelles
                     * @data   22/06/2022
                     * @return error signal
                     */
                    type_void_signal on_error_signal_emit();

                    /**
                     * @fn	WizardWidgetBase * WizardWidgetManager::getCurrentWidget();
                     * @brief	Method getter current widget.
                     * @author	Yadickson Soto
                     * @date	22/02/2012
                     * @return	current widget.
                     */
                    WizardWidgetBase * getCurrentWidget();
                    /**
                     * @fn	virtual void refresh();
                     * @brief	refresh widget method.
                     * @author	Yadickson Soto
                     * @date	02/04/2013
                     */
                    virtual void refresh();

                protected:
                    type_void_signal m_finish_signal; ///< The finish signal
                    type_void_signal m_transition_signal; ///< The finish signal
                    type_void_signal m_error_signal; ///< signal error

                    sigc::connection nextConnect; ///< The widget connect
                    sigc::connection widgetConnect; ///< The widget connect
                    sigc::connection widgetErrorConnect; ///< The widget error connect

                    std::vector<WizardWidgetBase *> widgetsVector;  ///< The widgets vector
                    std::vector<WizardWidgetBase *>::iterator iter; ///< The iterator

                    sigc::signal<void> signalNextWidget; ///< signal next widget

                private:

                    /**
                     * @fn	void WizardWidgetManager::ShowWidget();
                     * @brief	Shows the widget.
                     * @author	Luiggi.mendez
                     * @date	27/04/2011
                     */

                    void ShowWidget();

                    /**
                     * @fn	void WizardWidgetManager::NextWidget();
                     * @brief	Next widget.
                     * @author	Luiggi.mendez
                     * @date	27/04/2011
                     */

                    void NextWidget();
                    /**
                     * @fn	void WizardWidgetManager::onNextWidgetInternal();
                     * @brief	Callback method with emit signal dispatcher.
                     * @author	Yadickson Soto
                     * @date	03/09/2013
                     */
                    void onNextWidgetInternal();
                    /**
                     * @fn     void WizardWidgetManager::onError();
                     * @brief  Callback method with emit signal dispatcher.
                     * @author manuel.arguelles
                     * @date   22/06/2022
                     */
                    void onError();
                    /**
                     * @fn     void WizardWidgetManager::onErrorInternal();
                     * @brief  Callback method with emit signal dispatcher.
                     * @author manuel.arguelles
                     * @date   22/06/2022
                     */
                    void onErrorInternal();
                    /**
                     * @fn	void WizardWidgetManager::NextWidgetInternal();
                     * @brief	Method process next widget.
                     * @author	Yadickson Soto
                     * @date	22/02/2012
                     */
                    void NextWidgetInternal();

            };
        }
    }
}

#endif // _WIZARD_WIDGET_MANAGER_H_
