/**
 * @file basic-window.cc
 * @brief Body class basic window
 * @date 26/03/2012
 * @author Yadickson Soto
 */

#include "basic-window.h"
#include "Configuration/s-m-t-t-g-u-i-conf-manager.h"
#include <Configuration/BasicWindowConfManager.hxx>
#include <Configuration/MachineModelEnum.hxx>
#include <Log/ISMTTLog.h>
#include <Log/SMTTLogManager.h>
#include <Widgets/HelpExitButtons.h>
#include <Widgets/StepWidget.hxx>
#include <Operation/WindowsManager.hxx>
#include <Operation/WindowsManager.hxx>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#include <libintl.h>
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

#define N_(String) (String) ///<macro for gettext not translate

#define ACCESSIBLE_TYPE "window" ///< accessible type

using namespace Smartmatic::GUI::Windows;
using namespace Smartmatic::GUI::Widgets;
using namespace Smartmatic::GUI::Configuration;
using namespace Smartmatic::GUI::Operation;
using namespace Smartmatic::Configuration;
using namespace Smartmatic::Environment;
using namespace Smartmatic::Log;

#define LOGGER_PROJECT "Smartmatic.GUI.Windows.BasicWindow" ///< logger project name
#define POP_UP_HORIZONTAL_DISPLACEMENT 25

#ifdef GETTEXT_PACKAGE
ISMTTLog * BasicWindow::logger = SMTTLogManager::GetLogger ( LOGGER_PROJECT, GETTEXT_PACKAGE ); ///< The logger
#else
ISMTTLog * BasicWindow::logger = SMTTLogManager::GetLogger ( LOGGER_PROJECT, "" ); ///< The logger
#endif

BasicWindow::BasicWindow ( bool useTitleFrame, Gtk::WindowType type )
    : Gtk::Window ( type ), Smartmatic::GUI::Operation::AccessibleNameWidget ( ACCESSIBLE_TYPE )
{
    Smartmatic::GUI::Operation::AccessibleNameWidget::accessibleWidget = this;

    this->useTitleFrame = useTitleFrame;

    blockState = false;
    mainVbox = NULL;
    titleeventbox = NULL;
    titleFrame = NULL;
    clientAreaEventBox = NULL;
    clientArea = NULL;
    keywordInterface = NULL;
    titleImage = NULL;
    titleLabel = NULL;
    titleHbox = NULL;
    titleHelpEventbox = NULL;
    helpExitButtons = NULL;

    helpExitButtons = new HelpExitButtons ( this, false, this, false );
    helpExitButtons->show();

    SMTTGUIConfigurationManager::getInstance()->getWindowSize ( formwidth, formheight );

    std::string windowTitle = BasicWindowConfManager::getInstance()->getWindowTitle();
    bool showBorderWindow = BasicWindowConfManager::getInstance()->isShowBorderWindow();

    set_skip_taskbar_hint ( !showBorderWindow );
    set_skip_pager_hint ( !showBorderWindow );
    set_decorated ( showBorderWindow );

    set_default_size ( formwidth, formheight );
    set_size_request ( formwidth, formheight );

    set_border_width ( 0 );
    set_position ( Gtk::WIN_POS_CENTER );

    set_can_focus ( true );
    set_resizable ( false );
    set_keep_above ( false );

    signal_key_press_event().connect ( sigc::mem_fun ( *this, &BasicWindow::key_pressed ) );

    currentWidget = NULL;
    initialize();
    resizeComponents();
    setTitleHelp ( helpExitButtons );
    showTitleHelp();

    if ( !useTitleFrame )
    {
        hideTitle();
    }

    helpExitButtons->setAccessibleName ( "ToolButtons" );
    setAccessibleName ( "Main" );

    remove();
    add ( *mainVbox );
    setBorderVisible ( true );

    set_title ( windowTitle );
}

BasicWindow::~BasicWindow ()
{
    dispose();
}

void BasicWindow::ShowDialog()
{
    logger->Debug ( "Show window" );

    WindowsManager::getInstance()->push ( this );
    show();
}

void BasicWindow::QuitDialog()
{
    logger->Debug ( "Quit window" );

    WindowsManager::getInstance()->pop();
    hide();
}

void BasicWindow::getWindowSize ( int & width, int & height )
{
    width = formwidth;
    height = formheight;
}

void BasicWindow::getContentSize ( int & width, int & height )
{
    clientArea->get_size_request ( width, height );
}

void BasicWindow::getTitleSize ( int & width, int & height )
{
    titleFrame->get_size_request ( width, height );
}

void BasicWindow::initialize()
{
    Glib::RefPtr<Gtk::Builder> builder;
    std::string gladeFile;
    gladeFile = ResourcePathProvider::getInstance()->GetResourcePath ( PACKAGE_NAME, "ui/BasicWindow.glade" );

    try
    {
        builder = Gtk::Builder::create_from_file ( gladeFile );
    }
    catch ( const Glib::FileError & ex )
    {
        logger->Debug ( "FATAL - BasicWindow Initialize" );
    }

    builder->get_widget_derived ( "main-vbox", mainVbox );
    builder->get_widget_derived ( "title-eventbox", titleeventbox );
    builder->get_widget_derived ( "title-frame", titleFrame );
    builder->get_widget_derived ( "center-eventbox", clientAreaEventBox );
    builder->get_widget_derived ( "center-frame", clientArea );
    builder->get_widget_derived ( "title-image", titleImage );
    builder->get_widget_derived ( "title-label", titleLabel );
    builder->get_widget_derived ( "title-hbox", titleHbox );
    builder->get_widget_derived ( "title-help-eventbox", titleHelpEventbox );

    mainVbox->setAccessibleName ( "MainVerticalContainer" );
    titleeventbox->setAccessibleName ( "TitleContainer" );
    titleFrame->setAccessibleName ( "TitleContainer" );
    clientAreaEventBox->setAccessibleName ( "MainCenterContainer" );
    clientArea->setAccessibleName ( "MainCenterContainer" );
    titleImage->setAccessibleName ( "Title" );
    titleLabel->setAccessibleName ( "Title" );
    titleHbox->setAccessibleName ( "TitleHorizontalContainer" );
    titleHelpEventbox->setAccessibleName ( "HelpContainer" );

    int panelsPaddingX, panelsPaddingY;

    SMTTGUIConfigurationManager::getInstance()->getPaddings ( panelsPaddingX, panelsPaddingY );

    mainVbox->set_spacing ( panelsPaddingY );
    mainVbox->set_border_width ( panelsPaddingX );

    titleLabel->set_no_show_all();
    titleImage->set_no_show_all();
    titleeventbox->set_no_show_all();
    titleFrame->set_no_show_all();
    titleHelpEventbox->set_no_show_all();
    titleHbox->set_no_show_all();
    titleImage->set_visible ( false );
    titleeventbox->set_visible ( false );
    titleHelpEventbox->set_visible ( false );
    titleeventbox->set_name ( "base_window_title_event_box" );
    titleLabel->set_text ( "" );
    mainVbox->show_all_children();
}

void BasicWindow::dispose()
{
    this->remove();

    if ( mainVbox )
    {
        delete mainVbox;
        mainVbox = NULL;
    }

    if ( titleFrame )
    {
        delete titleFrame;
        titleFrame = NULL;
    }

    if ( titleeventbox )
    {
        delete titleeventbox;
        titleeventbox = NULL;
    }

    if ( clientAreaEventBox )
    {
        delete clientAreaEventBox;
        clientAreaEventBox = NULL;
    }

    if ( clientArea )
    {
        delete clientArea;
        clientArea = NULL;
    }

    if ( titleLabel )
    {
        delete titleLabel;
        titleLabel = NULL;
    }

    if ( titleImage )
    {
        delete titleImage;
        titleImage = NULL;
    }

    if ( titleHbox )
    {
        delete titleHbox;
        titleHbox = NULL;
    }

    if ( titleHelpEventbox )
    {
        delete titleHelpEventbox;
        titleHelpEventbox = NULL;
    }

    if ( helpExitButtons )
    {
        delete helpExitButtons;
        helpExitButtons = NULL;
    }

    std::list < Smartmatic::GUI::Widgets::StepWidget * >::iterator it = stepWidgetList.begin();

    for (it = stepWidgetList.begin(); it != stepWidgetList.end(); ++it)
    {
        delete(*it);
    }

    stepWidgetList.clear();
}

bool BasicWindow::key_pressed ( GdkEventKey * event )
{
    try
    {
        if ( keywordInterface )
        {
            keyCode = KeysActionsCode ( event->keyval );
            keywordInterface->SetKey ( keyCode );
        }
    }
    catch ( std::bad_exception & )
    {
    }

    return false;
}

bool BasicWindow::on_delete_event ( GdkEventAny * G_GNUC_UNUSED )
{
    return true;
}

Gtk::Frame & BasicWindow::getTitleFrame()
{
    return * ( this->titleFrame );
}

Gtk::Frame & BasicWindow::getClientAreaFrame()
{
    return * ( this->clientArea );
}

void BasicWindow::showTitle()
{
    useTitleFrame = true;
    resizeComponents();
}

void BasicWindow::hideTitle()
{
    useTitleFrame = false;
    resizeComponents();
}

void BasicWindow::resizeComponents()
{
    int panelTitleWidth; ///< panel title width
    int panelTitleHeight; ///< panel title height

    int panelClientWidth; ///< panel client width
    int panelClientHeight; ///< panel client height

    SMTTGUIConfigurationManager::getInstance()->getHeaderSize ( panelTitleWidth, panelTitleHeight );

    if ( useTitleFrame )
    {
        SMTTGUIConfigurationManager::getInstance()->getCenterSizeWithHeader ( panelClientWidth, panelClientHeight );
    }
    else
    {
        SMTTGUIConfigurationManager::getInstance()->getCenterSize ( panelClientWidth, panelClientHeight );
    }

    this->titleFrame->set_size_request ( panelTitleWidth, panelTitleHeight );
    this->titleFrame->set_border_width ( 0 );
    this->titleeventbox->show_all();
    this->clientArea->set_size_request ( panelClientWidth, panelClientHeight );
    this->titleeventbox->set_visible ( useTitleFrame );
}

void BasicWindow::setPosition ( int x, int y )
{
    set_position ( Gtk::WIN_POS_NONE );
    move ( x, y );
}

void BasicWindow::setPosition ( Smartmatic::GUI::Configuration::PositionConfiguration & position )
{
    set_position ( Gtk::WIN_POS_NONE );

    int formwidth, formheight;

    get_size_request ( formwidth, formheight );

    switch ( position )
    {
        case PositionConfiguration::CONFIGURATION_TOP_RIGHT:
            move ( Gdk::Screen::get_default()->get_width() - formwidth, 0 );
            break;

        case PositionConfiguration::CONFIGURATION_TOP_LEFT:
            move ( 0, 0 );
            break;

        case PositionConfiguration::CONFIGURATION_TOP_CENTER:
            move ( ( Gdk::Screen::get_default()->get_width() - formwidth ) / 2, 0 );
            break;

        case PositionConfiguration::CONFIGURATION_CENTER_RIGHT:
            move ( Gdk::Screen::get_default()->get_width() - formwidth, ( Gdk::Screen::get_default()->get_height() - formheight ) / 2 );
            break;

        case PositionConfiguration::CONFIGURATION_CENTER_LEFT:
            move ( 0, ( Gdk::Screen::get_default()->get_height() - formheight ) / 2 );
            break;

        case PositionConfiguration::CONFIGURATION_CENTER_CENTER:
        {
            int x = ( Gdk::Screen::get_default()->get_width() - formwidth ) / 2;
            int y = ( Gdk::Screen::get_default()->get_height() - formheight ) / 2;

            if ( SMTTGUIConfigurationManager::getInstance()->getSmartmaticGUIMachineModelConfiguration().getId() == EnumToString( PM_VIU805 ) )
                x += POP_UP_HORIZONTAL_DISPLACEMENT;

            move ( x, y );
            break;
        }

        case PositionConfiguration::CONFIGURATION_BOTTOM_RIGHT:
            move ( Gdk::Screen::get_default()->get_width() - formwidth, Gdk::Screen::get_default()->get_height() - formheight );
            break;

        case PositionConfiguration::CONFIGURATION_BOTTOM_LEFT:
            move ( 0, Gdk::Screen::get_default()->get_height() - formheight );
            break;

        case PositionConfiguration::CONFIGURATION_BOTTOM_CENTER:
            move ( ( Gdk::Screen::get_default()->get_width() - formwidth ) / 2, Gdk::Screen::get_default()->get_height() - formheight );
            break;

        case PositionConfiguration::CONFIGURATION_FULL_SCREEN:
            move ( 0, 0 );
            break;

        case PositionConfiguration::CONFIGURATION_CUSTOM:
        default:
            break;
    }
}

void BasicWindow::setTitleLabelConfiguration ( LabelConfiguration lblConfig )
{
    LabelConfiguration::SetLabelConfiguration ( titleLabel, lblConfig );
}

void BasicWindow::setTitleText ( std::string title )
{
    titleLabel->set_text ( title );
}

void BasicWindow::setTitleColor ( Gdk::Color & color )
{
    titleLabel->modify_fg ( Gtk::STATE_NORMAL, color );
}

void BasicWindow::showTitleImage()
{
    titleImage->show();
    showTitle();
}

void BasicWindow::hideTitleImage()
{
    titleImage->hide();
}

void BasicWindow::setTitleImage ( std::string image )
{
    titleImage->set ( Gdk::PixbufAnimation::create_from_file ( image ) );
}
void BasicWindow::showTitleHelp()
{
    titleHelpEventbox->show();
    titleHelpEventbox->show_all_children();
    showTitle();
}

void BasicWindow::hideTitleHelp()
{
    titleHelpEventbox->hide();
}

void BasicWindow::setTitleHelp ( Gtk::Widget * widget )
{
    titleHelpEventbox->remove();
    titleHelpEventbox->add ( *widget );
}

void BasicWindow::setTitle ( Gtk::Widget * widget )
{
    titleFrame->remove();

    if ( widget )
    {
        titleFrame->add ( *widget );
        widget->show();
    }

    titleFrame->show_all_children();
}

void BasicWindow::setWidget ( Gtk::Widget * widget )
{
    clientArea->remove();

    if ( widget )
    {
        clientArea->add ( *widget );
        widget->show();
    }

    clientArea->show_all_children();
}

void BasicWindow::setExitButtonSensitive ( bool sensitive )
{
    helpExitButtons->ExitButton_SetSensitive ( sensitive );
}

void BasicWindow::setExitButtonVisible ( bool visible )
{
    helpExitButtons->ExitButton_SetVisible ( visible );
}

void BasicWindow::setHelpButtonSensitive ( bool sensitive )
{
    helpExitButtons->HelpButton_SetSensitive ( sensitive );
}

void BasicWindow::setHelpButtonVisible ( bool visible )
{
    helpExitButtons->HelpButton_SetVisible ( visible );
}

void BasicWindow::setBorderVisible ( bool visible )
{
    if ( visible )
    {
        titleFrame->set_shadow_type ( Gtk::SHADOW_IN );
        clientArea->set_shadow_type ( Gtk::SHADOW_IN );
    }
    else
    {
        titleFrame->set_shadow_type ( Gtk::SHADOW_NONE );
        clientArea->set_shadow_type ( Gtk::SHADOW_NONE );
    }
}

void BasicWindow::enableHelpButton()
{
    setHelpButtonSensitive ( true );
}

void BasicWindow::disableHelpButton()
{
    setHelpButtonSensitive ( false );
}

void BasicWindow::enableExitButton()
{
    setExitButtonSensitive ( true );
}

void BasicWindow::disableExitButton()
{
    setExitButtonSensitive ( false );
}

void BasicWindow::pushStepWidget ( StepWidget * widget, bool force )
{
    Glib::Mutex::Lock lock ( mutex );

    if ( !blockState || force )
    {
        logger->Debug ( "Push widget" );
        stepWidgetList.push_back ( widget );
        setStepWidget ( widget );
    }
}

StepWidget * BasicWindow::popPushStepWidget ( Smartmatic::GUI::Widgets::StepWidget * widget, bool force )
{
    Glib::Mutex::Lock lock ( mutex );
    StepWidget * lastWidget = NULL;

    if ( !blockState || force )
    {
        if ( stepWidgetList.size() > 1 )
        {
            logger->Debug ( "Remove widget" );
            lastWidget = stepWidgetList.back();
            stepWidgetList.pop_back();
        }

        logger->Debug ( "Push widget" );
        stepWidgetList.push_back ( widget );
        setStepWidget ( widget );
    }

    return lastWidget;
}

Smartmatic::GUI::Widgets::StepWidget * BasicWindow::popStepWidget ( bool force )
{
    Glib::Mutex::Lock lock ( mutex );
    StepWidget * widget = NULL;

    if ( !blockState || force )
    {
        if ( stepWidgetList.size() > 1 )
        {
            logger->Debug ( "Remove widget" );
            widget = stepWidgetList.back();
            stepWidgetList.pop_back();
            setStepWidget ( stepWidgetList.back() );
        }
        else
        {
            currentWidget = NULL;
            logger->Debug ( "Window clean" );
        }
    }

    return widget;
}

void BasicWindow::setStepWidget ( Smartmatic::GUI::Widgets::StepWidget * widget )
{
    currentWidget = widget;

    if ( widget )
    {
        refreshStepWidget();
        setWidget ( widget );

        if ( widget->isTitle() )
        {
            showTitle();
        }
        else
        {
            hideTitle();
        }
    }
    else
    {
        logger->Debug ( "ERROR set widget" );
    }
}

void BasicWindow::ShowHelp()
{
    StepWidget * helpWidget = currentWidget->getHelpWidget();

    if ( currentWidget &&  helpWidget)
    {
        logger->Debug ( "Show help" );
        pushStepWidget ( helpWidget );
        helpWidget->getSignalStart().emit();
    }
    else
    {
        logger->Debug ( "Not show help" );
    }
}

void BasicWindow::ShowExit()
{
    if ( currentWidget && currentWidget->getExitWidget() )
    {
        logger->Debug ( "Show exit" );
        pushStepWidget ( currentWidget->getExitWidget() );
    }
    else
    {
        logger->Debug ( "Not show exit" );
    }
}

void BasicWindow::Exit()
{
    logger->Debug ( "Exit" );
    popStepWidget();
}

void BasicWindow::CloseHelp()
{
    currentWidget->getSignalEnd().emit();
    BasicWindow::Exit();
}

void BasicWindow::refreshTitleText()
{
    if ( currentWidget )
    {
        setTitleText ( currentWidget->getTitleText() );
    }
    else
    {
        logger->Debug ( "Not refresh title" );
    }
}

void BasicWindow::refreshHelpButton()
{
    if ( currentWidget )
    {
        setHelpButtonSensitive ( currentWidget->isHelpSensitive() );
    }
    else
    {
        logger->Debug ( "Not refresh help button" );
    }
}

void BasicWindow::refreshExitButton()
{
    if ( currentWidget )
    {
        setExitButtonSensitive ( currentWidget->isExitSensitive() );
    }
    else
    {
        logger->Debug ( "Not refresh exit button" );
    }
}

void BasicWindow::refreshStepWidget()
{
    if ( currentWidget )
    {
        currentWidget->refresh();
    }

    refreshTitleText();
    refreshHelpButton();
    refreshExitButton();

    if ( currentWidget )
    {
        setHelpButtonVisible ( currentWidget->isHelp() );
        setExitButtonVisible ( currentWidget->isExit() );
        setBorderVisible ( currentWidget->isShowBorder() );
        set_name ( currentWidget->getName() );
        RefreshListener * object = currentWidget->getRefreshListener();

        if ( object )
        {
            object->refresh();
        }
    }
}

int BasicWindow::getCountStepWidget()
{
    return stepWidgetList.size();
}

StepWidget * BasicWindow::getCurrentWidget()
{
    return currentWidget;
}

void BasicWindow::setKeywordNavigation ( Smartmatic::GUI::Widgets::keywordNavigationInterface * keywordInterface )
{
    this->keywordInterface = keywordInterface;
}

void BasicWindow::setBlockPopPush ( bool blockState )
{
    Glib::Mutex::Lock lock ( mutex );
    this->blockState = blockState;
}
