/**
 * @file  ICryptoLayoutTransformer.hxx
 * @brief Implementation of the common interface that abstracts to the cryptographic functionalities of the logs.
 *
 * @author  Yadickson Soto
 * @date    15-07-2011
 *
 */

#ifndef CRYPTOINTERFACE_HXX_
#define CRYPTOINTERFACE_HXX_

#include <log4cxx/helpers/pool.h>
#include <log4cxx/helpers/mutex.h>
#include <Log/LogInterface.hxx>


namespace Smartmatic
{
    namespace Log
    {
        /**
         * @class   ICryptoLayoutTransformer
         * @brief   Interface that abstracts to the cryptographic functionalities and also contains the mutex that handles the concurrencies.
         * @author  Yadickson Soto
         * @date    15-07-2011
         *
         * @see LogInterface
         */
        class ICryptoLayoutTransformer : public LogInterface
        {
            public:

                /**
                 * @fn  ICryptoLayoutTransformer::ICryptoLayoutTransformer ( const std::string & code );
                 * @brief   Constructor initializes an instance of the ICryptoLayoutTransformer class.
                 * @author  Yadickson Soto
                 * @date    15-07-2011
                 * @param   code  The code that identifies to the instance.
                 */
                explicit ICryptoLayoutTransformer ( const std::string & code );

                /**
                 * @fn  virtual ICryptoLayoutTransformer::~ICryptoLayoutTransformer ();
                 * @brief   Finalizes the instance of the ICryptoLayoutTransformer class.
                 * @author  Yadickson Soto
                 * @date    15-07-2011
                 */
                virtual ~ICryptoLayoutTransformer ();

                /**
                 * @fn  virtual std::string ICryptoLayoutTransformer::encode ( std::string & line ) = 0;
                 * @brief Gets the resulting string from codification.
                 * This method encode the string using INIT_KEY and gives a base64 result.
                 * @author  Yadickson Soto
                 * @date    15-07-2011
                 * @param   line    The original string ( preferably one-line ) .
                 * @return  The encode string in base64 format.
                 */
                virtual std::string encode ( std::string & line ) = 0;

                /**
                 * @fn  virtual std::string ICryptoLayoutTransformer::decode ( std::string & line ) = 0;
                 * @brief   Gets the resulting string from decodification. This method decodes using INIT_KEY.
                 * @author  Yadickson Soto
                 * @date    15-07-2011
                 * @param   line    The base64 string.
                 * @return   A readable string.
                 */
                virtual std::string decode ( std::string & line ) = 0;

                /**
                 * @fn  log4cxx::helpers::Mutex & ICryptoLayoutTransformer::getMutex ();
                 * @brief   Gets a reference to mutex that handles the concurrencies.
                 * @author  Yadickson Soto
                 * @date    15-07-2011
                 * @return  The reference to mutex.
                 */
                log4cxx::helpers::Mutex & getMutex ();

            private:

                log4cxx::helpers::Pool pool; ///< Instance helper required by the mutex.
                log4cxx::helpers::Mutex mutex; ///< The mutex that handles the concurrencies.
        };
    }
}

#endif /* CRYPTOINTERFACE_HXX_ */
