/**
 * @file CryptoLayout.hxx
 * @brief Header class CryptoLayout
 * @date 15/07/2011
 * @author Yadickson Soto yadickson.soto@smartmatic.com
 */

#ifndef CRYPTOLAYOUT_HXX_
#define CRYPTOLAYOUT_HXX_

#include <log4cxx/patternlayout.h>
#include <Log/ICryptoLayoutTransformer.hxx>

namespace log4cxx
{
    /**
     * @class   CryptoLayout
     * @brief   This class allows the extension of the class log4cxx::PatternLayout.
     * @author  Yadickson Soto yadickson.soto@smartmatic.com
     * @date    15/07/2011
     */
    class CryptoLayout: public log4cxx::PatternLayout
    {
        public:
            DECLARE_LOG4CXX_OBJECT ( CryptoLayout )
            BEGIN_LOG4CXX_CAST_MAP ()
            LOG4CXX_CAST_ENTRY ( CryptoLayout )
            LOG4CXX_CAST_ENTRY_CHAIN ( PatternLayout )
            END_LOG4CXX_CAST_MAP ()

            /**
             * @fn  CryptoLayout::CryptoLayout ();
             * @brief   Class constructor.
             * @author  Yadickson Soto yadickson.soto@smartmatic.com
             * @date    15/07/2011
             */
            CryptoLayout ();

            /**
             * @fn  virtual CryptoLayout::~CryptoLayout ();
             * @brief   Class destructor.
             * @author  Yadickson Soto yadickson.soto@smartmatic.com
             * @date    15/07/2011
             */
            virtual ~CryptoLayout ();

            /**
             * @fn  virtual bool CryptoLayout::ignoresThrowable ();
             * @brief   virtual method, The PatternLayout does not handle the throwable contained within
             *              spi::LoggingEvent LoggingEvents.
             * @author  Yadickson Soto yadickson.soto@smartmatic.com
             * @date    15/07/2011
             * @return  Thus, it returns true.
             */
            virtual bool ignoresThrowable () const;

            /**
             * @fn  virtual void CryptoLayout::setOption ( const log4cxx::LogString& option,
             *          const log4cxx::LogString& value );
             * @brief   Virtual method set option config.
             * @author  Yadickson Soto yadickson.soto@smartmatic.com
             * @date    15/07/2011
             * @param   option  option name of config file logger.
             * @param   value   value of option.
             */
            virtual void setOption ( const log4cxx::LogString & option, const log4cxx::LogString & value );

            /**
             * @fn  virtual void CryptoLayout::format ( log4cxx::LogString& output,
             *          const spi::LoggingEventPtr& event, log4cxx::helpers::Pool& pool ) const;
             * @brief   virtual method, The format string.
             * @author  Yadickson Soto yadickson.soto@smartmatic.com
             * @date    15/07/2011
             * @param   output  output format line logger.
             * @param   event   object container data line.
             * @param   pool        object for synchronize.
             */
            virtual void format ( log4cxx::LogString & output,
                                  const spi::LoggingEventPtr & event,
                                  log4cxx::helpers::Pool & pool ) const;

            /**
             * @fn  static void CryptoLayout::setCryptoMap ( const std::map < std::string, Smartmatic::Log::ICryptoLayoutTransformer * >& iCryptoMap );
             * @brief   Method setter crypto map interface.
             * @author  Yadickson Soto yadickson.soto@smartmatic.com
             *                  @see Smartmatic::Log::ICryptoLayoutTransformer
             * @date    15/07/2011
             * @param   iCryptoMap  crypto map interface.
             */
            static void setCryptoMap ( const std::map < std::string, Smartmatic::Log::ICryptoLayoutTransformer * >& iCryptoMap );

            /**
             * @fn  static void CryptoLayout::putCryptoInterface ( Smartmatic::Log::ICryptoLayoutTransformer * iCrypto );
             * @brief   Method put new crypto interface.
             * @author  Yadickson Soto yadickson.soto@smartmatic.com
             *                  @see Smartmatic::Log::ICryptoLayoutTransformer
             * @date    15/07/2011
             * @param   iCrypto new crypto interface.
             */
            static void putCryptoInterface ( Smartmatic::Log::ICryptoLayoutTransformer * iCrypto );

        protected:

            log4cxx::LogString cryptoType; ///< crypto type
            static std::map <std::string, Smartmatic::Log::ICryptoLayoutTransformer * > iCryptoMap; ///< crypto map interface

            /**
             * @fn  static void CryptoLayout::format ( log4cxx::LogString& output, log4cxx::LogString cryptoType );
             * @brief   Method format output line logger.
             * @author  Yadickson Soto yadickson.soto@smartmatic.com
             * @date    15/07/2011
             * @param   output          output format line logger.
             * @param   cryptoType  crypto type.
             */
            static void format ( log4cxx::LogString & output, log4cxx::LogString cryptoType );
    };

    LOG4CXX_PTR_DEF ( CryptoLayout ); ///< create pointer of class
}

#endif /* CRYPTOLAYOUT_HXX_ */
